/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      4165 × 20 bytes =    83300 bytes
//   Child IDs:         3123 ×  4 bytes =    12492 bytes
//   Terminators:         55 ×  4 bytes =      220 bytes
//   Strings:           1917 strings      =    27626 bytes
//   Aux Data:         10289 ×  4 bytes =    41156 bytes
//   Aux Offsets:       4165 ×  4 bytes =    16660 bytes
//   Regex Patterns:       6 patterns     =     4073 bytes
//   Simple Hints:      1066 hints        =    17056 bytes
//   Hint Indices:      1664 ×  4 bytes =     6656 bytes
//   TOTAL:                                    ≈   209239 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [4] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [6] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5, child_count: 34, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [9] OneOf(34 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(ExportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 39, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 41, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 41, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 41, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 45, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 45, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 45, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 45, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(PrivilegeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 45, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(PrivilegeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 52, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Ref(GroupsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(KillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 60, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 60, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 60, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 62, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 65, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 65, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 65, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [51] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 65, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 65, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Ref(ImpersonateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 68, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 73, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 73, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 73, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [61] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 73, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [62] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 73, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [63] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [64] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [65] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [66] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [67] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [68] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [69] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [70] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [71] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 81, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [72] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [73] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [74] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [75] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 5, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [76] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 89, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [77] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [78] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [79] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [80] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [81] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [82] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [83] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 92, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [84] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 92, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [85] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 94, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [86] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 94, child_count: 7, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [87] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [88] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [89] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [90] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [91] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [92] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [93] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [94] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 101, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [95] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 104, child_count: 7, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [96] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [97] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [98] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [99] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [100] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [101] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [102] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [103] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [104] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [105] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 111, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [106] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 114, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [107] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 114, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [108] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 114, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [109] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 114, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [110] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 117, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [111] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 119, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [112] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 119, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [113] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 119, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [114] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 119, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [115] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 119, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [116] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [117] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 125, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [118] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 125, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [119] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 125, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [120] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 125, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [121] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 125, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [122] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 127, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [123] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 127, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [124] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 127, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [125] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 127, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [126] Ref(ExportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 127, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [127] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [128] Ref(ImpersonationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [129] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [130] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 130, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 132, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [132] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 132, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [133] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 132, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [134] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 134, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [135] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 134, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [136] Ref(RolesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 134, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [137] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [138] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [139] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [140] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [141] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 136, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [142] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 141, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [143] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 141, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [144] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 141, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [145] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 141, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [146] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 141, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [147] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [148] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [149] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [150] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [151] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [152] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [153] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [154] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [155] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 162, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 184, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 186, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 186, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 186, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 188, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 190, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 190, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 190, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 192, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 194, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 196, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 196, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] StringParser("ACOS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ADAPTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ADD_DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ADD_HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ADD_MINUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("ADD_MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("ADD_SECONDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] StringParser("ADD_WEEKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] StringParser("ADD_YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 205, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 205, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Meta("dedent")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] StringParser("ALIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Ref(ConnectionDefinition)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 221, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 223, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(ConsumerGroupParameterSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 223, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 223, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(Consumer_groupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 237, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(Raw_size_limitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 243, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 243, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Ref(ChangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 263, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 267, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 268, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 268, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Ref(SessionParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 274, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 276, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 276, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 276, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(SystemParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 284, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 288, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 288, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 297, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 301, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 312, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 314, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 322, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(AlterTableAddColumnSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] Ref(AlterTableDropColumnSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(AlterTableModifyColumnSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Ref(AlterTableRenameColumnSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Ref(AlterTableAlterColumnSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [369] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [370] Ref(TableOutOfLineConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [372] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 340, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [373] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [374] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [375] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [376] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [377] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [378] Ref(TableConstraintEnableDisableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [379] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [380] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [381] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [382] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 351, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [383] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 351, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [384] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 351, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [385] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 351, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [386] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 351, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [387] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [388] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [389] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [390] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [391] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [392] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [393] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [394] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [395] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [396] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [397] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [398] Ref(TableDistributionPartitionClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 362, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [399] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [400] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 365, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [401] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [402] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [403] Ref(DistributionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [404] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [405] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [406] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [407] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [408] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [409] Ref(DistributionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [410] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 373, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [411] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [412] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [413] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [414] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [415] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [416] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [417] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [418] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [419] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [420] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 382, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [422] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [423] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [424] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [425] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 387, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [426] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 387, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [427] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [428] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [429] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [430] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 394, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [431] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 394, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [432] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 397, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [433] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [434] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [435] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 404, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [436] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [437] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [438] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [439] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [440] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [441] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [442] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [443] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [444] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [445] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [446] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [448] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [449] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [450] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [451] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [452] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [453] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [454] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [455] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 421, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [456] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [457] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [458] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [459] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [460] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [461] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [462] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [463] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [464] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [465] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 428, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [466] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [467] Ref(AlterTableColumnSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [468] Ref(AlterTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [469] Ref(AlterTableDistributePartitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [470] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [471] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [472] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [473] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [474] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [476] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [477] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [478] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [479] Ref(UserPasswordAuthSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 448, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [480] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [481] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [482] Ref(PasswordLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [483] Ref(UserLDAPAuthSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [484] Ref(UserKerberosAuthSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [485] Ref(UserOpenIDAuthSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 450, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [486] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [487] Ref(Password_expiry_policyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [488] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [489] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [490] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [491] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [492] Ref(ExpireKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [493] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [494] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [495] Ref(FailedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [496] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [497] Ref(AttemptsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [498] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 463, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [499] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 463, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [500] Ref(Consumer_groupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 463, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [501] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 463, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [502] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [503] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [504] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 465, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [505] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [506] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [507] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [508] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [509] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [510] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 473, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [512] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 475, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [513] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [514] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [515] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [516] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [517] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [518] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 481, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [519] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 481, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [521] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [522] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [523] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [524] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [525] Ref(ChangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [526] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [527] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [528] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [529] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [530] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [531] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [532] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [533] StringParser("ANSI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [534] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [535] StringParser("APPEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [536] StringParser("APPROXIMATE_COUNT_DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [537] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [538] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [539] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [540] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [541] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [542] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [543] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [544] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [545] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [546] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [547] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [548] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [549] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [550] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [551] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [552] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [553] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [554] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [555] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [556] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [557] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 505, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [558] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [559] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 508, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [560] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [561] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [562] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [563] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [564] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [565] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [566] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [567] StringParser("ASCII")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [568] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [569] StringParser("ASIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [570] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [571] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [572] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [573] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [574] StringParser("ATAN2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [575] StringParser("ATAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [576] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [577] StringParser("ATTEMPTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [578] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [579] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [580] StringParser("AUTHENTICATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [581] StringParser("AUTHID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 511, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [582] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 511, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [583] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [584] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [585] StringParser("AUTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [586] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [587] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 512, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [588] MultiStringParser(20 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 512, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [589] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [590] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [591] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [592] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [593] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [594] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [595] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 519, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [597] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [598] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [599] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [600] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [601] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [602] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [603] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [604] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [605] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [606] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [607] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 521, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [608] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 521, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [609] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [610] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [611] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [612] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [613] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [614] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [615] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [616] StringParser("BIT_AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [617] StringParser("BIT_CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [618] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [619] StringParser("BIT_LROTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [620] StringParser("BIT_LSHIFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [621] StringParser("BIT_NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [622] StringParser("BIT_OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [623] StringParser("BIT_RROTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [624] StringParser("BIT_RSHIFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [625] StringParser("BIT_SET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [626] StringParser("BIT_TO_NUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [627] StringParser("BIT_XOR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [628] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 525, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [630] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [631] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [632] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 527, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [633] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [634] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [635] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [636] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [637] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [638] StringParser("BLOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 529, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [639] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [640] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 531, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [641] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 531, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [642] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 531, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [643] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 531, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [644] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [645] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [646] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [647] Ref(UnknownSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 534, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [648] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 534, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [649] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 538, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [650] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [651] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 540, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [652] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [653] Ref(BitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [654] Ref(ByteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [655] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [656] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 546, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [657] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [658] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 548, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [659] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [660] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [661] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [662] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [663] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [664] StringParser("BYTE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [665] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 554, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [666] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 556, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [667] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [668] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [669] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 561, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [670] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [671] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [672] Ref(RangeOperator)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [673] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 564, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [674] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [675] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [676] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [677] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 567, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [678] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [679] Ref(DelimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [680] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [681] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [682] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [683] Ref(NeverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [684] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [685] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [686] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [687] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [688] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [689] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 580, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [690] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 580, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [691] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [692] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [693] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [694] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [695] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [696] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [697] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [698] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [699] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [700] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [701] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 591, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [702] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [703] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [704] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [705] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [706] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [707] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [708] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 592, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [709] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [710] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [711] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 599, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [712] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [713] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [714] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [715] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [716] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [717] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [718] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [719] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [720] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [721] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [722] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [723] StringParser("CASESPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [724] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [725] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [726] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [727] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [728] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [729] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 600, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [730] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 600, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [731] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [732] Ref(Utf8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [733] Ref(AsciiKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [734] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [735] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [736] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [737] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [738] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [739] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [740] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [741] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [742] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [743] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [744] StringParser("CHR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [745] StringParser("CLEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [746] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [747] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [748] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [749] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [750] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [751] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [752] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [753] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [754] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [755] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [756] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [757] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [758] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 606, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [759] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [760] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [761] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [762] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [763] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [764] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [765] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [766] StringParser("COLOGNE_PHONETIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [767] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [768] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 608, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [769] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 608, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [770] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [771] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [772] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [773] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [774] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 612, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [775] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 615, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [776] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [777] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [778] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [779] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [780] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [781] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 623, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [783] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 623, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [784] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 625, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [785] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [786] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [787] Ref(TableInlineConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [788] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [789] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [790] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [791] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [792] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [793] Ref(ColumnDatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [794] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [795] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 632, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [796] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 632, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [797] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [798] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 634, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [799] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [800] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [801] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [802] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [803] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [804] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [805] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [806] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 636, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [807] StringParser(",")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 636, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [808] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [809] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [810] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [811] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [812] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [813] Token("comment")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [814] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [815] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [816] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 642, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [817] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [818] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [819] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 648, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [820] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 648, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [821] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 650, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [822] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 650, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [823] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 650, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [824] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [825] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [826] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [827] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [828] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [829] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 658, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [830] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [831] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [832] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [833] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [834] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [835] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [836] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [837] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [838] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [839] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [840] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [841] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [842] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [843] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [844] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [845] StringParser("COMMENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [846] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [847] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [848] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [849] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [850] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [851] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [852] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [853] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [854] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [855] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [856] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [857] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [858] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [859] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [860] StringParser("CONCAT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 680, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [861] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [862] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [863] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [864] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [865] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 682, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [866] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [867] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [868] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [869] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 685, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [870] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 687, child_count: 5, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [871] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [872] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [873] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [874] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [875] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [876] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [877] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [878] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 694, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [879] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [880] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [881] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [882] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 7, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [883] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [884] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [885] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [886] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [887] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [888] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [889] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [890] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [891] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [892] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [893] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [894] StringParser("CONNECT_BY_ISCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [895] StringParser("CONNECT_BY_ISLEAF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 706, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [896] StringParser("CONNECT_BY_ROOT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 706, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [897] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 708, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [898] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [899] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [900] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 710, child_count: 5, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [901] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [902] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [903] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [904] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [905] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [906] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [907] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [908] StringParser("CONSTANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [909] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [910] StringParser("CONSTRAINT_STATE_DEFAULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [911] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 715, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [912] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [913] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 718, child_count: 7, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [914] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [915] Ref(Cpu_weightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [916] Ref(PrecedenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [917] Ref(Group_temp_db_ram_limitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [918] Ref(User_temp_db_ram_limitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [919] Ref(Session_temp_db_ram_limitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [920] Ref(Query_timeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [921] Ref(Idle_timeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [922] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [923] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [924] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [925] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [926] StringParser("CONSUMER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [927] StringParser("CONSUMER_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [928] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [929] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [930] StringParser("CONTROL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [931] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [932] StringParser("CONVERT_TZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [933] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [934] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [935] StringParser("COS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [936] StringParser("COSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [937] StringParser("COT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [938] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [939] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [940] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [941] StringParser("CPU_WEIGHT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 10, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [942] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [943] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 737, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [944] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 737, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [945] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [946] Ref(JavaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [947] Ref(PythonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [948] Ref(LuaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [949] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [950] Ref(AdapterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [951] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [952] Ref(ScriptReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [953] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [954] Ref(ScriptContentSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 10, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [955] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [956] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [957] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 751, child_count: 5, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [958] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [959] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [960] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [961] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [962] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [963] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [964] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [965] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [966] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [967] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 762, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [969] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [970] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [971] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [972] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [973] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [974] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [975] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 765, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [976] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 767, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [977] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 767, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [978] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 767, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [979] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [980] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [981] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 6, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [982] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [983] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [984] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [985] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [986] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [987] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [988] Ref(ConnectionDefinition)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 6, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [989] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [990] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [991] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [992] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [993] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [994] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [995] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [996] Ref(ConsumerGroupParameterSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [997] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [998] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [999] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1000] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1001] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 787, child_count: 21, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1002] Sequence(21 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1003] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1004] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1005] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1006] Ref(FunctionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1007] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 811, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1008] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 813, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1009] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1010] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1011] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1012] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1013] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1014] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 816, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1015] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1016] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 818, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1017] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 818, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1018] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1019] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1020] Ref(VariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1021] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1022] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1023] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 822, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1024] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1025] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1026] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1027] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1028] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1029] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1030] Ref(FunctionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1031] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 9, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1032] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1033] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1034] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1035] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1036] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1037] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1038] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1039] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1040] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 832, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1041] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1042] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1043] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1044] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 838, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1045] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 838, child_count: 8, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1046] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1047] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1048] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1049] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1050] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1051] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 846, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1053] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1054] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 851, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1055] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 853, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1056] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1057] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1058] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1059] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1060] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 858, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1061] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1062] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1063] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1064] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1065] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1066] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1067] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1068] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1069] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1070] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1071] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1072] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1073] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1074] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 11, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1075] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1076] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1077] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1078] Ref(LuaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1079] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1080] Ref(ScriptReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1081] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 884, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1082] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 886, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1083] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 888, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1084] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 888, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1085] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 888, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1086] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1087] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 890, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1088] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1089] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1090] Ref(RowcountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1091] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1092] Ref(ScriptContentSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 7, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1093] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 899, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1094] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1095] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1096] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1097] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 902, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1098] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1099] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1100] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1101] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1102] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1103] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1104] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1106] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1107] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1108] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1109] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1110] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1111] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1112] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1113] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1114] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1115] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1116] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 915, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1117] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1118] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 6, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1119] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1120] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1121] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 922, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1122] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 925, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1123] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 927, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1124] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1125] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1126] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 929, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1128] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1129] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1130] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1131] Ref(DefaultsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 933, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1132] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1133] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1134] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1135] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1136] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 937, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1138] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1139] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1140] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1141] Ref(CommentsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 7, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1142] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1143] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1144] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1145] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1146] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1147] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 948, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1148] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1149] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 954, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1151] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 958, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1152] Ref(TableContentDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 958, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1154] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1155] Ref(TableDistributionPartitionClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 960, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1156] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1157] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1158] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1159] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1160] Ref(CreateTableLikeClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 963, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1161] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 963, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1162] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 972, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1163] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 972, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1164] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 972, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1165] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 972, child_count: 3, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1166] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1167] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1168] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1170] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1171] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [1172] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 979, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1173] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1174] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1175] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1176] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1177] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1178] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 2, _padding: 0 }, // [1179] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1180] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1181] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [1182] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1183] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [1184] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1185] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1186] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 987, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1187] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1188] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1189] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1190] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1191] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1192] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1193] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1194] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1195] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1196] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1197] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1198] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1199] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1200] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1201] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1202] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1203] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1204] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1007, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1206] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1011, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1209] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1210] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1212] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1213] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1214] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1018, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1215] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1018, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1216] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1018, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1218] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1219] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1020, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1220] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1221] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1022, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1222] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1025, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1223] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1025, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1224] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1225] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1226] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1227] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1228] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 12, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1229] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1230] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1231] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1041, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1232] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1233] Ref(JavaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1234] Ref(PythonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1235] Ref(LuaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1236] Ref(RKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1237] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1238] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1239] Ref(ScalarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1240] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1241] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1242] Ref(ScriptReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1243] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1051, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1244] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1245] Ref(UDFParameterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1246] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1247] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1055, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1249] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1250] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1251] Ref(EmitsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1252] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1253] Ref(ScriptContentSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1254] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1255] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1256] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1257] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1258] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1259] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1260] Ref(UserPasswordAuthSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1261] Ref(UserKerberosAuthSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1262] Ref(UserLDAPAuthSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1263] Ref(UserOpenIDAuthSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1066, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1264] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1265] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1266] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1267] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1268] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1269] Ref(ViewReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1270] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1271] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1080, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1272] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1273] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1274] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1275] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1082, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1276] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1084, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1277] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1278] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1279] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1280] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1281] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1282] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1283] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1284] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1285] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1286] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1287] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1096, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1288] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1096, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1289] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1097, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1290] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1291] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1292] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1293] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1294] StringParser("CREATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1295] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1296] StringParser("CS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1297] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1298] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1100, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1299] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1100, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1300] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1102, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1301] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1302] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1303] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1304] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1305] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1306] StringParser("CURDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1307] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1308] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1309] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1310] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1311] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1312] StringParser("CURRENT_SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1313] StringParser("CURRENT_STATEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1314] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1315] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1316] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1317] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1318] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1319] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1320] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1107, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1321] StringParser("DATALINK")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1107, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1322] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1323] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1109, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1324] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1325] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1110, child_count: 17, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1326] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1327] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1328] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1329] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1330] Ref(DecKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1331] Ref(NumberKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1332] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1333] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1334] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1336] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1337] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1338] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1339] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1340] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1341] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1342] Ref(ShortintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1343] Ref(TinyintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1344] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1135, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1345] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1346] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1347] Ref(BoolKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1137, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1348] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1349] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1350] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1351] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1141, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1352] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1353] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1354] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1355] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1356] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1357] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1358] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1359] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1360] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1361] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1362] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 6, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1363] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1364] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1365] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1366] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1367] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1368] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1156, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1369] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1370] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1371] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1158, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1372] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1158, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1160, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1374] Ref(HashtypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1160, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1375] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1160, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1376] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1377] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1378] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 6, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1379] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1380] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1381] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1382] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1383] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1384] Ref(Varchar2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1385] Ref(NcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1386] Ref(NvarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1387] Ref(Nvarchar2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1388] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1390] Ref(LongKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1391] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1178, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1392] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1393] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1180, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1394] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1182, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1395] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1396] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1397] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1398] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1399] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1400] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1402] Ref(ClobKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1403] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1404] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1405] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1406] MultiStringParser(7 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1407] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1188, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1408] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1189, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1409] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1410] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1194, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1411] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1412] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1413] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1414] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1415] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1416] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1417] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1418] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1419] StringParser("DATE_TRUNC")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1420] MultiStringParser(11 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1421] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1422] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1423] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1424] StringParser("DAYS_BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1425] StringParser("DBTIMEZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1426] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1427] StringParser("DEBUG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1428] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1429] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1430] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1431] StringParser("DECODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1432] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1433] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1434] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1435] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1436] StringParser("DEFAULT_CONSUMER_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1437] StringParser("DEFAULT_LIKE_ESCAPE_CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1438] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1439] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1440] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1441] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1442] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1443] StringParser("DEGREES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1444] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 6, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1445] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1446] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1447] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1448] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1449] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1450] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1451] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1452] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1453] Ref(PreferringClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1454] StringParser("DELIMIT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1455] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1456] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1457] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1458] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1459] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1460] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1461] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1462] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1463] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1464] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1465] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1466] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1467] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1468] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1469] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1470] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1471] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1472] StringParser("DISABLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1473] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1474] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1475] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1476] StringParser("DISTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1477] StringParser("DISTRIBUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1478] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1479] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1480] StringParser("DLURLCOMPLETE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1481] StringParser("DLURLPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1482] StringParser("DLURLPATHONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1483] StringParser("DLURLSCHEME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1484] StringParser("DLURLSERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1485] StringParser("DLVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1486] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1487] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1488] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1489] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1490] StringParser("DOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1213, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1491] StringParser("DRIVER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1213, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1492] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1493] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1494] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1495] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1496] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1497] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1498] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1499] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1500] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1501] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1502] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1503] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1504] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1505] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1506] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1507] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1508] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1509] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1511] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1512] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1513] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1514] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1515] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1516] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1517] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1518] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1519] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1520] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1521] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1522] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1243, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1523] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1524] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1243, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1525] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1526] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1527] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1528] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1529] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1530] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1531] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1532] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1533] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1534] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1535] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1536] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1537] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1538] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1539] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1540] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1541] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1542] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1257, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1543] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 7, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1544] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1545] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1546] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1547] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1548] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1549] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1550] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1264, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1551] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1264, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1552] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1553] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1554] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1270, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1555] Ref(AdapterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1270, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1556] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1270, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1557] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1270, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1558] Ref(ScriptReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1270, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1559] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1560] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1561] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1562] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 6, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1563] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1564] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1565] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1566] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1567] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1568] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1279, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1570] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1571] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1572] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1573] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1574] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1575] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1576] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1577] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1578] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1579] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1580] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1581] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1582] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1583] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1584] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1585] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1586] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1587] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1588] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1589] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1590] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1591] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1592] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1593] Ref(ViewReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1594] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1595] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1596] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1597] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1598] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1599] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1600] StringParser("EDIT_DISTANCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1601] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1602] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1603] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1604] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1605] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1606] StringParser("ELSIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1607] StringParser("EMITS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1608] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1609] Ref(EmitsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1610] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1611] Ref(UDFParameterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1612] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1613] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1311, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1614] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1615] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1616] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1617] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1618] StringParser("ENABLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1619] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1620] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1621] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1622] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1623] StringParser("ENDIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1624] StringParser("ENFORCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1625] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1626] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1627] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1628] StringParser("ERRORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1629] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1630] TypedParser("escaped_identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1631] StringParser("ESTIMATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1632] StringParser("EVALUATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1633] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1634] StringParser("EXA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1635] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1636] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1637] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1638] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1639] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1313, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1640] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1313, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1641] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1642] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1643] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1644] Ref(ScriptReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1318, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1645] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1646] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1323, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1647] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1648] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1649] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1325, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1650] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1651] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1652] Ref(OutputKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1653] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1654] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1655] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1656] StringParser("EXPERIMENTAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1657] StringParser("EXPIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1658] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1659] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1660] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1661] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1662] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1663] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1664] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1665] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1666] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1667] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1668] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1669] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1336, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1671] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1672] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1340, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1673] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1674] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1675] Ref(ImportFromExportIntoDbSrcSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1676] Ref(ImportFromExportIntoFileSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1677] Ref(RejectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1678] Ref(ImportFromExportIntoScriptSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1344, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1679] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1680] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1681] Ref(ExportKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1347, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1683] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1351, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1684] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1351, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1685] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1686] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1354, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1687] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1688] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1689] Ref(ExportIntoClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1690] StringParser("EXPRESSION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1691] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1692] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1693] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1359, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1694] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1360, child_count: 10, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1695] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1696] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1698] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1699] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1700] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1701] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1702] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1703] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1704] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1705] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1706] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1707] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1708] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1709] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1710] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1711] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1712] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1713] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1714] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1380, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1716] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1717] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1382, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1718] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1386, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1719] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1720] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1721] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1722] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1723] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1724] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1725] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1726] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1390, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1727] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1728] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1392, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1729] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1394, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1730] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1731] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1732] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1733] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1734] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1735] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1399, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1736] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1737] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1738] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1739] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1401, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [1740] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1404, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1742] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1743] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1744] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1411, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1746] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1747] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1748] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1413, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1749] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1750] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1751] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [1752] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1753] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1416, child_count: 9, min_times: 0, first_terminator_idx: 21, terminator_count: 1, _padding: 0 }, // [1754] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1755] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1756] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1757] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1758] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1759] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1760] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1761] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1762] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1434, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1763] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1764] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1765] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1766] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1767] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1768] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1769] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1770] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1771] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1441, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1773] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1774] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1775] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1446, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1776] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1777] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1450, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1779] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1450, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1780] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1781] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1782] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1783] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1784] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1785] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1786] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1787] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1788] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1789] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 7, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1790] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1791] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1792] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1793] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1794] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1795] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1796] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1797] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1798] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1799] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1800] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1801] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1802] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1467, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1803] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1804] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1473, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1805] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1806] Ref(SizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1807] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1808] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1809] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1475, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1810] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1478, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1811] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1812] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1813] Ref(PaddingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1814] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1815] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1816] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1817] Ref(AlignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1818] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1819] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1820] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1821] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1822] StringParser("FAILED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1823] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1824] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1825] StringParser("FBV")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 5, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1826] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1827] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1828] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1829] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1830] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1492, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1831] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1832] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1494, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1833] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1834] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1495, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1835] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1836] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1837] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1497, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1838] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1839] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1840] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1841] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1842] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1843] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1501, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1844] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1501, child_count: 7, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1845] AnyNumberOf(7 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 7, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1846] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1847] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1848] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1849] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1850] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1851] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1852] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1517, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1853] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1854] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1855] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1856] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1857] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1858] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1859] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1860] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1861] Ref(TrimKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1862] Ref(LtrimKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1863] Ref(RtrimKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1864] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1865] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1866] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1867] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1868] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1869] Ref(SizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1870] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1871] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1872] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1873] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1874] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1875] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1876] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1877] Ref(NamesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1878] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1879] Ref(DelimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1880] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1537, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1881] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1882] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1883] Ref(NeverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1884] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1540, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1885] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1542, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1886] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1887] Ref(FunctionScriptStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1888] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1544, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1889] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1890] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1891] Ref(FunctionScriptTerminatorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1546, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1892] StringParser("FILES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1546, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1894] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1895] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1897] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1898] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1899] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1900] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1901] StringParser("FIRST_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1902] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1903] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1904] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1906] Ref(FlushKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1907] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1908] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1909] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1910] StringParser("FORALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1911] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1555, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1913] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1914] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1915] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1916] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1917] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1918] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1919] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1920] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1921] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1922] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1560, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1924] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1925] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1926] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1927] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1928] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1929] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1930] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1931] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1932] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1933] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1934] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1935] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1936] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1937] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1938] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1939] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1940] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1941] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1580, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1942] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1943] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1944] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1945] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1946] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1947] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1948] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1949] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1586, child_count: 12, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1950] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1951] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1952] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1953] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1954] Ref(PreferringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1955] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1956] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1957] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1958] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1959] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1960] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1961] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1962] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1963] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1964] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1965] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1966] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1602, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1967] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1604, child_count: 7, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1968] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1969] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1971] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1972] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1616, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1973] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1616, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1974] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1975] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1976] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1977] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1978] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1621, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1979] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1621, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1981] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1982] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1983] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1984] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1985] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1986] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1987] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1629, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1988] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1989] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1990] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1632, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1991] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [1992] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1641, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 2, _padding: 0 }, // [1993] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1994] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1995] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1996] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1997] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1998] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [1999] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2000] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2001] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2002] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2003] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1651, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2004] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1651, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 2, _padding: 0 }, // [2005] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2006] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1653, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2007] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2008] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2009] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2010] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2011] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2012] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1657, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2013] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2014] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2015] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2016] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2017] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2018] StringParser("FROM_POSIX_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2019] StringParser("FS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1659, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2020] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1659, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2021] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2022] Ref(VariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2023] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2024] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2025] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2026] Ref(VariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2027] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2028] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2029] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2030] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2031] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2032] Ref(FunctionAssignmentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2033] Ref(FunctionIfBranchSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2034] Ref(FunctionForLoopSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2035] Ref(FunctionWhileLoopSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1672, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2036] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1672, child_count: 11, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2037] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2038] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2039] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2040] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2041] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2042] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2043] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2044] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1690, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2045] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2046] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2047] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2048] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1691, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2049] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2050] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2051] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2052] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2053] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2054] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2055] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2056] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2057] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2058] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2059] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2060] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2061] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2063] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2064] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2065] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1707, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2066] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2067] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2068] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2069] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2070] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2071] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2072] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2073] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2074] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2075] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2076] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2077] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1713, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2078] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2079] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2080] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2081] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2082] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2083] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1720, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2084] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2085] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2086] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2087] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2088] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2089] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 8, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2091] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2092] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2093] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2094] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2095] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2096] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1736, child_count: 1, min_times: 1, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2097] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2098] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2099] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2100] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1737, child_count: 8, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2101] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2102] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2103] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2104] Ref(RangeOperator)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2105] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2106] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1745, child_count: 1, min_times: 1, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2107] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2108] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2109] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2110] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2111] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1746, child_count: 11, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2112] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2113] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1757, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2114] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2115] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2116] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 1, min_times: 1, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2117] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1759, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2118] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1759, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2119] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2120] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2122] Ref(ElsifKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2123] Ref(ElseifKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2124] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2125] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 1, min_times: 1, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2126] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2127] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1769, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2128] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2129] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 1, min_times: 1, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2130] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2131] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2132] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2133] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2134] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2135] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2136] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1774, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2137] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1776, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [2138] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1777, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2140] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2141] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2142] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [2143] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2144] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2145] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2146] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2147] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2149] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2150] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1787, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2152] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2153] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2154] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2155] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1791, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2156] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2157] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2158] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2159] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2160] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1796, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2161] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2162] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1798, child_count: 4, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2163] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2164] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2165] Ref(CreateScriptingLuaScriptStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2166] Ref(CreateUDFScriptStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2167] Ref(CreateAdapterScriptStatementSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2168] TypedParser("function_script_terminator")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2169] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2170] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2171] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2172] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2173] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2175] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1811, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2176] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2177] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2178] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2179] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2180] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2181] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2182] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 7, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2183] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2184] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2185] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2186] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1822, child_count: 1, min_times: 1, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2187] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2188] Ref(FunctionBodySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2189] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2190] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2191] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2192] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2193] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2194] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2195] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2196] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2197] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2198] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2199] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2200] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 7, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2201] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2202] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2203] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1830, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2204] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1830, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2206] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2207] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1834, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2208] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2209] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2210] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2212] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2213] Ref(ObjectsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2214] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2215] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1838, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2216] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2217] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2218] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2219] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2220] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1841, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2221] StringParser("GRAPH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1841, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2222] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2223] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2224] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2225] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1843, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2226] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 5, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2227] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2228] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2229] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 5, _padding: 0 }, // [2230] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2231] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2232] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2233] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2234] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2235] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2236] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2237] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1858, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2238] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2239] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2240] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2241] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2242] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2243] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2244] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 6, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2245] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1866, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2246] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2247] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2248] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2249] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2250] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2251] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2252] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2253] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2254] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2255] StringParser("GROUP_CONCAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1868, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2256] StringParser("GROUP_TEMP_DB_RAM_LIMIT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1868, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2257] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 1, _padding: 0 }, // [2258] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 4, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2259] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2260] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2261] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2262] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 2, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2263] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 33, terminator_count: 0, _padding: 0 }, // [2264] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2265] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2266] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2267] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2268] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1880, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2269] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1883, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2270] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1885, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2272] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2273] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2274] StringParser("GROUPING_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2275] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2276] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2277] StringParser("HAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2278] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2279] StringParser("HASH_MD5")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2280] StringParser("HASH_SHA1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2281] StringParser("HASH_SHA256")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2282] StringParser("HASH_SHA512")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2283] StringParser("HASH_SHA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2284] StringParser("HASH_TIGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2285] StringParser("HASHTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2286] StringParser("HASHTYPE_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2287] StringParser("HASHTYPE_MD5")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2288] StringParser("HASHTYPE_SHA1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2289] StringParser("HASHTYPE_SHA256")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2290] StringParser("HASHTYPE_SHA512")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2291] StringParser("HASHTYPE_SHA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2292] StringParser("HASHTYPE_TIGER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2293] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2294] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1891, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2295] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2296] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2297] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2298] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2300] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2301] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2302] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2303] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2304] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2305] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2306] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2307] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2308] StringParser("HIGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2309] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2310] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2311] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2312] StringParser("HOURS_BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2313] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2314] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2315] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1903, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2316] StringParser("IDLE_TIMEOUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1903, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2317] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2318] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2319] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2320] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2321] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2322] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2323] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2324] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2325] StringParser("IFNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1908, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2326] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1908, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2327] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2328] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2329] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2330] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2331] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2332] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1912, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2333] StringParser("IMPERSONATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1912, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2335] Ref(ImpersonateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2336] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2337] StringParser("IMPERSONATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2338] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2339] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2340] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2341] Ref(ColumnDatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2342] Ref(CreateTableLikeClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1917, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2343] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1920, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2344] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2345] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2346] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2347] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2348] Ref(ConnectionDefinition)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2349] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2350] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2351] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2352] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2353] Ref(SecureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2354] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2355] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2356] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2357] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2358] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2359] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2360] Ref(ErrorsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2361] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1938, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2362] Ref(ImportErrorDestinationSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1938, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2363] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1941, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2364] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1941, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2365] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2366] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2367] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1943, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2368] Ref(RejectClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2369] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2370] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1945, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2371] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2372] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1949, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2373] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2374] Ref(ImportFromExportIntoDbSrcSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2375] Ref(ImportFromExportIntoFileSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2376] Ref(ImportErrorsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2377] Ref(ImportFromExportIntoScriptSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1951, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2378] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1954, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2379] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2380] Ref(ExaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2381] Ref(OraKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1957, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2382] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2383] Ref(JdbcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1959, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2384] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2385] Ref(DriverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2386] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2387] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2389] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2390] Ref(ConnectionDefinition)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1964, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2391] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2392] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2393] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2394] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1970, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2395] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2396] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1973, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2397] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1974, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2398] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1976, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2399] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2400] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2401] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2402] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2403] Ref(CreatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2404] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2405] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1981, child_count: 1, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2406] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1982, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2408] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2409] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2410] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2411] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2412] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2413] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2414] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2415] Ref(FbvKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 2, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2416] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1995, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1997, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2418] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1997, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2419] Ref(ConnectionDefinition)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1997, child_count: 2, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2420] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2421] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2422] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2423] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2424] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2003, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2425] Ref(SecureKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2003, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2426] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2427] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2428] Ref(FbvKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2005, child_count: 2, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2429] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2430] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2431] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2007, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2432] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2433] Ref(CSVColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2434] Ref(FBVColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2435] Ref(FileOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2436] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2437] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2013, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2438] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2013, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2440] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2441] Ref(ConnectionDefinition)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2015, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2442] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2443] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2017, child_count: 1, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2444] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2445] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2446] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2447] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2448] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2021, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2449] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2021, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2450] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2451] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2024, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2452] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2453] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2454] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2455] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2456] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2030, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2457] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2458] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2459] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2036, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2460] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2038, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2461] Ref(ImportColumnsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2038, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2462] Ref(ImportFromClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2038, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2463] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2038, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2464] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2041, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2465] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2041, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2466] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2041, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2467] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2468] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2469] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2048, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2470] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2471] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2472] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2473] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2050, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2474] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2050, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2475] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2052, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2476] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2052, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2477] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2478] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2479] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2480] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2481] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2482] StringParser("INITCAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2483] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2484] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2485] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2486] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2487] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2054, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2488] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2054, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2489] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2490] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2491] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2492] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2058, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2493] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2494] Ref(ValuesRangeClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2496] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2497] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2498] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2499] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2500] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2501] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2502] StringParser("INSTR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2503] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2504] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2505] StringParser("INTEGRITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2506] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2507] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2508] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2509] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2510] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2070, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2511] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2072, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2512] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2072, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2513] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2514] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2515] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2516] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2078, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2517] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2518] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2081, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2520] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2521] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2083, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2523] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2524] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2525] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2526] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2527] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2092, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2528] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2092, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2529] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2530] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2531] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2097, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2532] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2097, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2533] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2534] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2535] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2102, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2537] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2538] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2539] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2540] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2107, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2109, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2542] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2109, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2543] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2544] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2545] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2546] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2547] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2548] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2549] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2550] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2551] StringParser("INVALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2552] StringParser("INVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2553] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2115, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2554] StringParser("IPROC")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2555] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2556] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2557] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2558] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2559] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2560] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2120, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2561] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2562] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2563] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2564] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2565] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2566] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2567] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2568] StringParser("IS_BOOLEAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2569] StringParser("IS_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2570] StringParser("IS_DSINTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2571] StringParser("IS_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2572] StringParser("IS_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2573] StringParser("IS_YMINTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2574] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2575] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2576] StringParser("JAVA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2577] StringParser("JAVASCRIPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2578] StringParser("JDBC")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2579] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2580] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2581] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2582] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2134, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2583] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2134, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2584] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2135, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2585] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2135, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2586] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2587] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2588] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2589] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2590] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2591] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2141, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2592] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2141, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2593] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2594] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2595] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2596] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2147, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2597] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2598] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2599] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2600] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2601] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2602] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2603] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2604] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2605] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2156, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2606] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2156, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2607] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2156, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2608] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2158, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2609] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2161, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2610] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2161, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2611] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2161, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2612] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2613] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2163, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2614] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2615] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2616] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2617] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2618] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2168, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2619] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2168, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2620] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2621] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2172, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2622] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2623] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2624] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2625] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2626] StringParser("JSON_EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2627] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2628] StringParser("KEEP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2629] StringParser("KERBEROS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2630] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2631] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2632] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2633] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2634] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2635] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2179, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2637] Ref(KillKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2179, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2639] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2640] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2641] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2642] Ref(Current_sessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2643] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2644] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2645] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2646] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2647] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2648] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2649] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2191, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2650] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2651] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2652] Ref(MessageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2653] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2654] StringParser("LAG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2655] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2656] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2657] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2658] StringParser("LAST_VALUE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2659] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2660] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2661] StringParser("LCASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2662] StringParser("LDAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2663] StringParser("LEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2664] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2665] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2666] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2667] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2668] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2194, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2670] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2671] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2672] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2673] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2674] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2675] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2676] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2677] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2201, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2678] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2201, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2679] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2680] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2681] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2683] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2684] Ref(Regexp_likeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2685] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2686] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2687] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2688] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2689] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2690] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2691] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2692] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2693] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2212, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2694] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2214, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2695] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2214, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2697] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2698] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2699] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2700] StringParser("LINK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2701] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2216, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2702] StringParser("LISTAGG")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2216, child_count: 9, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2703] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2704] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2705] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2706] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2707] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2708] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2709] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2710] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2711] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2712] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2713] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2714] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2715] StringParser("LN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2716] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2717] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2718] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2719] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2720] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2721] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2722] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2723] StringParser("LOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2724] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2725] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2726] StringParser("LOG10")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2727] StringParser("LOG2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2728] StringParser("LOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2729] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2730] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2731] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2732] StringParser("LONGVARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2733] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2734] StringParser("LOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2735] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2736] StringParser("LPAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2737] StringParser("LTRIM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2738] StringParser("LUA")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2739] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2740] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2741] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2742] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2743] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2744] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2745] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2746] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2747] StringParser("MAXIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2748] StringParser("MEDIAN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2749] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2750] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2751] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2752] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2753] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2754] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2755] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2236, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2756] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2236, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2757] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2758] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2759] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2238, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2760] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2238, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2761] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2240, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2762] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2763] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2242, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2764] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2242, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2765] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2766] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2767] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2768] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2769] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2770] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2771] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2773] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2774] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2775] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2776] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2777] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2778] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2779] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2780] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 12, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2781] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2782] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2783] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2784] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2785] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2786] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2787] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2788] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2789] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2790] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2791] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2792] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2793] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2794] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2795] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2796] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2797] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2798] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2799] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2800] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2801] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2802] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2803] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2804] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2805] StringParser("MESSAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2806] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2807] StringParser("MID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2808] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2809] StringParser("MIN_SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2810] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2811] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2812] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2813] StringParser("MINUTES_BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2814] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2815] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2816] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2817] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2818] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2819] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2820] StringParser("MONTHS_BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2821] StringParser("MUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2822] StringParser("MULTIPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2823] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2824] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2825] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2826] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2827] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2828] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2829] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2830] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2831] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2832] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2833] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2834] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2835] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2836] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2837] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2838] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2839] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2840] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2841] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2842] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2843] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2844] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2845] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2846] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2847] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2848] StringParser("NEVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2849] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2850] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2851] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2852] StringParser("NICE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2853] StringParser("NLS_DATE_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2854] StringParser("NLS_DATE_LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2855] StringParser("NLS_FIRST_DAY_OF_WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2856] StringParser("NLS_NUMERIC_CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2857] StringParser("NLS_TIMESTAMP_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2858] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2859] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2299, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2860] StringParser("NOLOGGING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2299, child_count: 6, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2861] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2862] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2863] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2864] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2865] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2866] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2867] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2868] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2869] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2870] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2871] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2872] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2873] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2874] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2875] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2876] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2318, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2877] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2878] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2879] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2323, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2880] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2881] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2882] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2883] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2884] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2885] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2886] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2887] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2328, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2888] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2889] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2890] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2330, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2891] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2892] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2893] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2894] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2895] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2896] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2897] StringParser("NOTICE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2898] StringParser("NOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2899] StringParser("NPROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2900] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2901] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2902] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2903] StringParser("NULLIFZERO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2904] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2905] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2906] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2907] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2908] StringParser("NUMTODSINTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2909] StringParser("NUMTOYMINTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2910] StringParser("NVARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2911] StringParser("NVARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2912] StringParser("NVL2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2913] StringParser("NVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2914] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2915] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2916] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2917] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2918] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2919] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2338, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2920] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2921] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2340, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2922] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2340, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2923] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2924] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2342, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2925] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2344, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2926] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2344, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2927] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2344, child_count: 12, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2928] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2929] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2930] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2931] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2932] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2933] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2934] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2935] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2936] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2937] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2938] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2939] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2940] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2941] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2942] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2943] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2944] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2945] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2946] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2947] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2361, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2948] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2361, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2949] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2950] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2951] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2952] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2953] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2954] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2955] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2956] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2957] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2958] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2959] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2960] Ref(OpenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2961] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2962] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2963] StringParser("OPENID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2964] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2965] StringParser("OPTIMIZER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2966] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2967] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2968] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2969] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2970] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2971] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2972] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2973] StringParser("ORA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2974] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2975] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2976] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2374, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 2, _padding: 0 }, // [2977] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2376, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2978] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2979] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2980] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2981] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2383, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2982] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2383, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2983] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2984] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2985] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2385, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2986] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2987] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2988] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2989] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2990] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2991] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2992] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [2993] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2994] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2995] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2996] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2997] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2998] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2999] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3000] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3001] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3002] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3003] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3004] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3005] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3006] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3007] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3008] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3009] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3010] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2398, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3011] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2398, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3012] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3013] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3014] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2403, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3015] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3016] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2405, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3017] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3018] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3019] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3020] StringParser("OVERFLOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3021] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3022] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2410, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3023] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2412, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3024] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3025] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3026] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3027] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3028] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3029] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3030] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3031] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3032] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3033] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3034] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3035] StringParser("PADDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3036] StringParser("PARALLEL_ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3037] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3038] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3039] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3040] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3041] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3042] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2418, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3043] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2418, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3044] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3045] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3046] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3047] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2425, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3048] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2428, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3049] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3050] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3051] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3052] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3053] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3054] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3055] StringParser("PASSWORD_EXPIRY_POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3056] StringParser("PASSWORD_SECURITY_POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3057] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3058] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2432, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3060] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2434, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3061] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3062] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3063] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3064] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3065] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3066] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3067] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3068] StringParser("PERMISSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3069] StringParser("PI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3070] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3071] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3072] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3073] StringParser("PLUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3074] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3075] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3076] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2436, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3077] StringParser("POSIX_TIME")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2436, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3078] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3079] Ref(EmitsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3080] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2440, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3081] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2442, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3082] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3083] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3084] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3085] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3086] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3087] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3088] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3089] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3090] StringParser("PRECEDENCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3091] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3092] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3093] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3094] Ref(PreferringKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3095] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3096] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3097] Ref(PreferringPreferenceTermSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2452, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3098] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2452, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3099] StringParser("PREFERRING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3100] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3101] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2455, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3102] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3103] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3104] Ref(PlusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3106] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3107] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3108] Ref(PreferringPreferenceTermSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3110] Ref(InverseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3111] Ref(PreferringPreferenceTermSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2463, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3113] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3114] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2469, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3115] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2471, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3116] Ref(HighKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2471, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3117] Ref(LowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2471, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3118] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3119] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3120] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3121] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3122] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3123] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3124] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3125] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3126] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3127] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3128] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3129] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3130] Ref(PreferringPlusPriorTermSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2481, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3131] StringParser("PRELOAD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2481, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3132] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3133] Ref(PreloadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3134] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2488, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3135] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3136] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3137] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3138] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2491, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3140] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3141] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3142] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3143] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3144] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3145] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3146] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3147] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3148] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3149] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3150] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3151] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3152] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3154] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3155] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3156] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3157] StringParser("PRINCIPAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3158] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3159] StringParser("PRIVILEGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3160] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3161] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3162] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3163] StringParser("PYTHON")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2503, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3164] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3165] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3166] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3167] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2509, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3168] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2509, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3169] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2509, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3170] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2509, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3171] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2512, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3172] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2512, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3173] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3174] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3175] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3176] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3177] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3178] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3179] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3180] StringParser("QUERY_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3181] StringParser("QUERY_TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3182] StringParser("QUIET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3183] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3184] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3185] StringParser("R")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3186] StringParser("RADIANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3187] StringParser("RAND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3188] StringParser("RANDOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3189] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3190] TypedParser("range_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3191] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3192] StringParser("RATIO_TO_REPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3193] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3194] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3195] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3196] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3197] StringParser("RAW_SIZE_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3198] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3199] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3200] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3201] StringParser("RECOMPRESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3202] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2521, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3203] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3204] Ref(RecompressKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3205] Ref(ReorganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2523, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3206] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2528, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3207] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3208] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3209] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3210] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2531, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2533, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3212] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2533, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3213] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3214] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3215] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3216] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3217] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3218] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3219] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3220] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3221] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3222] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3223] Ref(EnforceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3224] StringParser("RECORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3225] StringParser("RECOVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3226] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3227] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3228] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3229] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3230] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3231] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3232] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2546, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3233] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3234] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3235] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3236] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3237] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3238] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3239] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3240] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3241] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3242] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3243] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3244] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3245] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3246] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3247] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2559, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3248] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2559, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3249] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3250] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3251] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3252] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3253] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2562, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3254] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3255] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3256] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3257] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2569, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3258] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2569, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3259] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2569, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3260] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3261] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3262] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3264] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3265] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3266] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3267] StringParser("REGEXP_INSTR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3268] StringParser("REGEXP_LIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3269] StringParser("REGEXP_REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3270] StringParser("REGEXP_SUBSTR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3271] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3272] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3273] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3274] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3275] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3276] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3277] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3278] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3279] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3280] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3281] Ref(RejectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3282] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3283] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3284] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3285] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3286] Ref(ErrorsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3287] StringParser("REJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3288] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3289] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3290] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3291] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3292] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2584, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3293] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3294] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3295] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3296] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3297] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3298] Ref(ScriptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3299] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3300] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3301] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3302] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3303] Ref(ConsumerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3304] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3305] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3306] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3307] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3308] StringParser("REORGANIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3309] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3310] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3311] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3312] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3313] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3314] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3315] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3316] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3317] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3318] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3319] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3320] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3321] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3322] StringParser("REVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3323] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3324] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3325] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3326] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3327] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2601, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3328] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3329] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3330] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3332] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3333] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2607, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3335] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3336] Ref(ObjectsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3337] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3338] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3339] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3340] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3341] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3342] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3343] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3344] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3345] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3346] StringParser("ROUND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3347] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3348] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3349] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3350] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3351] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3352] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3353] StringParser("ROWTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3354] StringParser("RPAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3355] StringParser("RTRIM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3356] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2613, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3357] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2613, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3358] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3359] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3360] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2615, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3361] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3362] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2618, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3364] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2621, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3365] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2621, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3366] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3367] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3368] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3369] StringParser("SCALAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3370] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3371] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3372] StringParser("SCHEME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3373] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3374] StringParser("SCOPE_USER")
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(2), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [3375] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3376] Ref(FunctionScriptTerminatorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3377] StringParser("SCRIPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3378] StringParser("SCRIPT_LANGUAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3379] StringParser("SCRIPT_OUTPUT_ADDRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3380] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3381] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3382] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3383] StringParser("SECONDS_BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3384] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3385] StringParser("SECURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2624, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3386] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3387] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3388] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3389] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2628, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3390] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2628, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3391] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2628, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3392] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3393] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3394] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 8, min_times: 0, first_terminator_idx: 37, terminator_count: 1, _padding: 0 }, // [3395] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2638, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3396] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2638, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3397] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 2638, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3398] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2640, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3399] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3401] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2646, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3402] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2646, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3403] Ref(InvalidKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2646, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3404] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3405] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3406] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3408] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3409] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3410] Ref(WithInvalidForeignKeySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3411] Ref(WithInvalidUniquePKSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3412] Ref(IntoTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [3413] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3414] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2657, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3415] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2658, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3416] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3417] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3419] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3420] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3421] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3422] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3423] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3424] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2660, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3425] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 11, min_times: 0, first_terminator_idx: 38, terminator_count: 3, _padding: 0 }, // [3426] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3427] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3428] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3429] Ref(ReferencingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3430] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3431] Ref(ConnectByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3432] Ref(PreferringClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3433] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3434] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3435] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3436] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3437] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3438] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3439] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3440] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3441] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3442] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3443] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3444] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2680, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2682, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3446] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3447] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3448] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2685, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3449] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3450] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3451] StringParser("SELECTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3452] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3453] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3454] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3455] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3456] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2688, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3458] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3459] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3460] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3461] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3462] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3463] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2694, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3464] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3465] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3466] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3467] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3469] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3470] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3471] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3472] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3473] MultiStringParser(22 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3474] StringParser("SESSION_PARAMETER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3475] StringParser("SESSION_TEMP_DB_RAM_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3476] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3477] StringParser("SESSIONTIMEZONE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2700, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3478] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2704, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3479] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2704, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 1, _padding: 0 }, // [3480] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2706, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3481] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2706, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [3482] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2706, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3483] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3484] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3485] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 7, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3486] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3487] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3488] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3489] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3490] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3491] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3492] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3493] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 5, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3494] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2721, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3495] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2721, child_count: 1, min_times: 1, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3496] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3497] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3498] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3499] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3500] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3501] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3502] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3503] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3504] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3506] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3507] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3508] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3509] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3510] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3511] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2731, child_count: 4, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3512] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3513] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3514] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3515] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3516] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3517] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3518] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2737, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3519] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3520] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3521] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 1, min_times: 1, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3522] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3523] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3524] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3525] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3526] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3527] StringParser("SHORTINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3528] StringParser("SHUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2743, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3529] StringParser("SIGN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2743, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3531] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3532] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3533] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3534] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3535] StringParser("SIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3536] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3537] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3538] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3539] Ref(EscapedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3540] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2750, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3541] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2750, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3542] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3543] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3544] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3545] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3546] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3547] StringParser("SINH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3548] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3549] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3550] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3551] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3552] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3553] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3554] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3555] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3556] StringParser("SNAPSHOT_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3557] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3558] StringParser("SOUNDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3559] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3560] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3561] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3562] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3563] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3564] StringParser("SQL_BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3565] StringParser("SQL_BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3566] StringParser("SQL_CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3567] StringParser("SQL_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3568] StringParser("SQL_DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3569] StringParser("SQL_DOUBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3570] StringParser("SQL_FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3571] StringParser("SQL_INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3572] StringParser("SQL_LONGVARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3573] StringParser("SQL_NUMERIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3574] StringParser("SQL_PREPROCESSOR_SCRIPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3575] StringParser("SQL_REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3576] StringParser("SQL_SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3577] StringParser("SQL_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3578] StringParser("SQL_TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3579] StringParser("SQL_TYPE_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3580] StringParser("SQL_TYPE_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3581] StringParser("SQL_VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3582] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3583] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3584] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3585] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3586] StringParser("ST_AREA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3587] StringParser("ST_BOUNDARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3588] StringParser("ST_BUFFER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3589] StringParser("ST_CENTROID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3590] StringParser("ST_CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3591] StringParser("ST_CONVEXHULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3592] StringParser("ST_CROSSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3593] StringParser("ST_DIFFERENCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3594] StringParser("ST_DIMENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3595] StringParser("ST_DISJOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3596] StringParser("ST_DISTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3597] StringParser("ST_ENDPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3598] StringParser("ST_ENVELOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3599] StringParser("ST_EQUALS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3600] StringParser("ST_EXTERIORRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3601] StringParser("ST_FORCE2D")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3602] StringParser("ST_GEOMETRYN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3603] StringParser("ST_GEOMETRYTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3604] StringParser("ST_INTERIORRINGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3605] StringParser("ST_INTERSECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3606] StringParser("ST_INTERSECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3607] StringParser("ST_ISCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3608] StringParser("ST_ISEMPTY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3609] StringParser("ST_ISRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3610] StringParser("ST_ISSIMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3611] StringParser("ST_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3612] StringParser("ST_MAX_DECIMAL_DIGITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3613] StringParser("ST_NUMGEOMETRIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3614] StringParser("ST_NUMINTERIORRINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3615] StringParser("ST_NUMPOINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3616] StringParser("ST_OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3617] StringParser("ST_POINTN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3618] StringParser("ST_SETSRID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3619] StringParser("ST_STARTPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3620] StringParser("ST_SYMDIFFERENCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3621] StringParser("ST_TOUCHES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3622] StringParser("ST_TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3623] StringParser("ST_UNION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3624] StringParser("ST_WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3625] StringParser("ST_X")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3626] StringParser("ST_Y")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3627] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3628] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3629] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2755, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3630] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2755, child_count: 48, min_times: 0, first_terminator_idx: 42, terminator_count: 1, _padding: 0 }, // [3631] OneOf(48 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3632] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3633] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3634] Ref(ExportStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3635] Ref(ImportStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3636] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3637] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3638] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3639] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3640] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3641] Ref(AlterSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3642] Ref(AlterVirtualSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3643] Ref(CommentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3644] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3645] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3646] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3647] Ref(CreateVirtualSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3648] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3649] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3650] Ref(DropScriptStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3651] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3652] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3653] Ref(RenameStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3654] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3655] Ref(AlterConnectionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3656] Ref(AlterUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3657] Ref(CreateConnectionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3658] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3659] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3660] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3661] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3662] Ref(DropConnectionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3663] Ref(CreateConsumerGroupSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3664] Ref(AlterConsumerGroupSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3665] Ref(DropConsumerGroupSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3666] Ref(AlterRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3667] Ref(AlterSessionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3668] Ref(AlterSystemSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3669] Ref(OpenSchemaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3670] Ref(CloseSchemaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3671] Ref(FlushStatisticsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3672] Ref(ImpersonateSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3673] Ref(RecompressReorganizeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3674] Ref(KillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3675] Ref(PreloadSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3676] Ref(TruncateAuditLogsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3677] Ref(ExplainVirtualSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3678] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3679] Ref(ExecuteScriptSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [3680] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3681] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3682] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3683] StringParser("STDDEV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3684] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3685] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3686] StringParser("STEP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3687] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3688] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2804, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3689] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2807, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3690] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2809, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3692] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3693] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3694] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3695] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3696] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3697] StringParser("SUBJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3698] StringParser("SUBSTR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3699] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3700] StringParser("SUBTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3701] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3702] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3703] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3704] StringParser("SYS_CONNECT_BY_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3705] StringParser("SYS_GUID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3706] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3707] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3708] MultiStringParser(26 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3709] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3710] StringParser("SYSTIMESTAMP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3711] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3712] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3713] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2815, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3716] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3717] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2817, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3718] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2820, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3719] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3720] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3721] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2822, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3722] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3723] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3724] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3725] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3726] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3727] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3728] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2827, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3729] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3730] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3731] Ref(TableOutOfLineConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3732] Ref(CreateTableLikeClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3733] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3734] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3735] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 2, _padding: 0 }, // [3736] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3737] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3738] Ref(TablePartitionByGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [3739] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3740] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3741] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3742] Ref(TableDistributeByGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2840, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3743] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2840, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3744] Ref(TablePartitionByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3745] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3746] Ref(TablePartitionByGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3747] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3748] Ref(TableDistributeByGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3749] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 8, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3750] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3751] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3752] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3753] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3754] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3755] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3756] Ref(ValuesRangeClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3757] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3758] Ref(ImportStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3759] Ref(ExplainVirtualSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3760] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2857, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3761] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2859, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3762] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2859, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3763] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3764] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3765] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3766] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3767] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3768] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3769] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2869, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3771] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2869, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3772] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2869, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3773] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2869, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3774] Ref(ForeignKeyReferencesClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2869, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3775] Ref(TableConstraintEnableDisableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2869, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3776] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2869, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3777] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2872, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2874, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3779] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2874, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3780] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3781] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3782] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3783] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3784] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3785] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3786] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2881, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3787] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3788] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3789] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2883, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3790] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2886, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3791] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2886, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3792] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2886, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3793] Ref(ForeignKeyReferencesClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2886, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3794] Ref(TableConstraintEnableDisableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2886, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3795] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3796] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3797] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 2, _padding: 0 }, // [3798] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3799] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3800] Ref(TableDistributeByGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [3801] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3802] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3803] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3804] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2893, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 1, _padding: 0 }, // [3805] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3806] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [3807] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3808] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2894, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3809] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2896, child_count: 1, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3810] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3811] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3812] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3813] StringParser("TAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3814] StringParser("TANH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3815] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3816] StringParser("TEMP_DB_RAM_LIMIT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2897, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3817] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3818] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3819] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3820] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3821] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3822] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3823] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3824] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3825] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3826] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3827] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3828] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3829] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3830] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3831] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3832] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3833] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3834] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2906, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3835] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2909, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3836] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3837] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3838] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3839] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2911, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3840] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2911, child_count: 1, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3841] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2912, child_count: 4, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3842] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3843] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3844] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3845] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3846] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3847] StringParser("TIME_ZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3848] StringParser("TIME_ZONE_BEHAVIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3849] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3850] StringParser("TIMESTAMP_ARITHMETIC_BEHAVIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3851] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3852] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3853] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3854] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3855] StringParser("TO_CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3856] StringParser("TO_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3857] StringParser("TO_DSINTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3858] StringParser("TO_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3859] StringParser("TO_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3860] StringParser("TO_YMINTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3861] StringParser("TRACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3862] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2916, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3863] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2916, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3864] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3865] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3866] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3867] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3868] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3869] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3870] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3871] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3872] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3873] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3874] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2920, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3875] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2920, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3876] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3877] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3878] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3879] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3880] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3881] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2923, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3882] StringParser("TRUNC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2923, child_count: 4, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3883] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3884] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3885] Ref(AuditKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3886] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2927, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3888] Ref(KeepKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2929, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3889] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2931, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3891] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3892] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3893] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3894] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3895] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3897] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3898] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2938, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3899] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2938, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3900] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3901] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3902] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3903] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3904] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2944, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3905] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2946, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3906] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2946, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3907] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2946, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3909] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3910] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3912] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3913] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3914] StringParser("TYPEOF")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3915] TypedParser("udf_param_dot_syntax")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3916] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3917] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3918] Ref(ColumnDatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3919] Ref(UDFParameterDotSyntaxSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3920] StringParser("UCASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3921] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3922] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3923] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 3, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3924] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3925] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3926] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3927] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3928] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3929] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3930] StringParser("UNICODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3931] StringParser("UNICODECHR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3932] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3933] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2959, child_count: 2, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3934] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3935] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3936] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3937] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 1, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3938] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3939] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3940] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3941] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3942] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3943] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3944] StringParser("UNLIMITED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3945] StringParser("UNLINK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3946] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 9, min_times: 0, first_terminator_idx: 48, terminator_count: 5, _padding: 0 }, // [3947] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3948] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3949] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3950] Ref(ReferencingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3951] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3952] Ref(ConnectByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3953] Ref(PreferringClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3954] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3955] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3956] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3957] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3958] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3959] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3960] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 0, min_times: 0, first_terminator_idx: 48, terminator_count: 0, _padding: 0 }, // [3961] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2971, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3962] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3963] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 1, min_times: 1, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3964] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2974, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3965] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3966] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3967] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3968] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3969] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 8, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3970] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2984, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3971] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2984, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3972] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3973] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3974] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3975] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3976] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3977] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3978] Ref(PreferringClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3979] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3980] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3981] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3982] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2988, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3983] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2988, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3984] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2988, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3985] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3986] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3987] Ref(KerberosKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3988] Ref(PrincipalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3989] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2992, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3990] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2992, child_count: 5, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3991] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3992] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3993] Ref(LdapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3994] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3995] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3996] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3997] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3998] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [3999] Ref(OpenidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4000] Ref(SubjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4001] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4002] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4003] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4004] Ref(PasswordLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4005] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4006] StringParser("USER_TEMP_DB_RAM_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4007] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4008] StringParser("UTF8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4009] StringParser("VALUE2PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3003, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4010] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3003, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4011] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4012] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4013] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3008, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4014] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3010, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4015] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3013, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4016] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3015, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4017] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4018] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4019] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4020] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3018, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4021] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3020, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4022] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4023] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4024] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4025] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4026] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 6, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4027] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4028] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4029] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3028, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4030] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3031, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4031] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3031, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4032] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3031, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4033] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3031, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4034] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3031, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4035] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4036] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4037] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4038] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3034, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4039] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4040] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4041] Ref(StepKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4042] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4043] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4044] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4045] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4046] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4047] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4048] StringParser("VARIANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4049] StringParser("VARRAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4050] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4051] StringParser("VERIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4052] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4053] StringParser("VIRTUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4054] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4055] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 8, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4056] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3045, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4057] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3045, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4058] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4059] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4060] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4061] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4062] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4063] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4064] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4065] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4066] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4067] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4068] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4069] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4070] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3054, child_count: 3, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4071] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4072] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 11, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4073] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4074] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4075] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4076] Ref(PreferringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4077] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3068, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4078] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4079] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4080] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3070, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4081] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4082] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4083] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4084] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4085] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4086] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4087] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4088] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4089] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4090] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4091] Token("whitespace")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4092] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3073, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4093] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3073, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4094] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3075, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4095] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3076, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4096] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3078, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4098] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4099] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3080, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4101] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4102] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4103] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3082, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4104] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3082, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4105] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 3086, child_count: 1, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4106] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3087, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4107] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3089, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4108] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3089, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4109] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3089, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4110] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3089, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4111] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3089, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4112] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3089, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4113] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4114] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4115] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4116] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 1, _padding: 0 }, // [4117] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4118] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [4119] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3097, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4120] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4121] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3097, child_count: 6, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4122] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4123] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4124] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3103, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4125] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3103, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 1, _padding: 0 }, // [4126] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3105, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4127] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3105, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [4128] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3105, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4129] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3105, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4130] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3105, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4131] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3108, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4132] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3108, child_count: 1, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4133] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4134] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4135] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4136] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4137] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3113, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4138] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3113, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4139] Ref(InvalidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3113, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4140] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3113, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4141] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3113, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4142] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4143] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4144] Ref(InvalidKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4145] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4146] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4147] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4148] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4149] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4150] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4151] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4152] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4153] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4154] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4155] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4156] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4157] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4158] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4159] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4160] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4161] StringParser("YEARS_BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4162] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4163] StringParser("ZEROIFNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3123, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [4164] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    4, 5, 6, 7, 8, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 22,
    27, 31, 36, 40, 44, 47, 53, 57, 63, 72, 76, 85, 95, 106, 110, 116,
    125, 126, 127, 131, 134, 137, 142, 20, 21, 23, 24, 25, 26, 28, 29, 30,
    32, 33, 34, 35, 37, 38, 39, 41, 42, 43, 45, 46, 48, 52, 49, 50,
    51, 54, 55, 56, 58, 61, 62, 59, 60, 64, 70, 71, 65, 66, 67, 68,
    69, 73, 74, 75, 77, 81, 82, 83, 84, 78, 79, 80, 86, 87, 88, 89,
    90, 91, 92, 93, 94, 96, 104, 105, 97, 98, 99, 100, 101, 102, 103, 107,
    108, 109, 111, 114, 115, 112, 113, 117, 121, 122, 118, 119, 120, 123, 124, 128,
    129, 130, 132, 133, 135, 136, 138, 139, 143, 144, 145, 146, 147, 148, 149, 152,
    150, 151, 154, 139, 157, 158, 159, 160, 161, 162, 163, 164, 165, 166, 167, 170,
    173, 176, 168, 169, 171, 172, 174, 175, 177, 178, 180, 181, 182, 183, 184, 185,
    186, 187, 188, 189, 190, 193, 196, 199, 191, 192, 194, 195, 197, 198, 200, 201,
    203, 204, 206, 139, 208, 224, 225, 226, 234, 227, 233, 228, 229, 230, 231, 232,
    236, 237, 242, 243, 244, 245, 246, 248, 249, 250, 251, 252, 253, 254, 139, 257,
    258, 259, 260, 261, 262, 263, 264, 265, 266, 268, 269, 270, 271, 272, 279, 273,
    274, 275, 276, 277, 278, 280, 281, 282, 284, 288, 289, 290, 295, 298, 285, 286,
    287, 291, 294, 292, 293, 296, 297, 300, 301, 302, 303, 304, 306, 307, 308, 309,
    310, 311, 312, 313, 315, 316, 317, 318, 319, 320, 321, 322, 324, 325, 326, 327,
    328, 329, 329, 231, 232, 331, 332, 333, 334, 335, 348, 336, 337, 338, 343, 339,
    340, 341, 342, 342, 231, 232, 344, 345, 346, 347, 349, 350, 351, 352, 354, 355,
    356, 357, 358, 359, 360, 361, 362, 364, 365, 366, 367, 368, 371, 379, 387, 369,
    370, 372, 373, 378, 374, 377, 375, 376, 380, 381, 382, 386, 383, 384, 385, 388,
    389, 390, 391, 392, 394, 395, 396, 397, 398, 399, 400, 401, 410, 402, 406, 403,
    404, 405, 407, 408, 409, 412, 413, 414, 415, 417, 418, 419, 420, 421, 422, 423,
    425, 426, 427, 428, 432, 429, 430, 431, 231, 232, 429, 430, 431, 434, 440, 453,
    454, 435, 436, 437, 438, 439, 441, 444, 445, 446, 442, 443, 447, 452, 448, 451,
    449, 450, 455, 456, 459, 457, 458, 461, 462, 463, 464, 465, 467, 468, 469, 471,
    472, 473, 474, 475, 486, 490, 493, 498, 476, 477, 478, 483, 484, 485, 479, 480,
    481, 482, 487, 488, 489, 491, 492, 494, 495, 496, 497, 499, 500, 501, 502, 503,
    504, 506, 507, 508, 509, 510, 511, 518, 524, 512, 513, 514, 515, 516, 517, 519,
    520, 521, 522, 523, 139, 525, 526, 527, 539, 540, 541, 542, 543, 544, 545, 546,
    547, 548, 550, 555, 556, 551, 554, 552, 553, 560, 555, 556, 561, 139, 564, 584,
    590, 591, 592, 593, 594, 595, 596, 597, 598, 610, 611, 612, 613, 630, 631, 634,
    635, 641, 642, 645, 646, 647, 650, 652, 231, 232, 651, 139, 653, 654, 655, 657,
    231, 232, 658, 139, 661, 231, 232, 666, 231, 232, 667, 139, 668, 674, 678, 669,
    670, 671, 672, 673, 675, 676, 677, 679, 680, 681, 682, 683, 684, 686, 231, 232,
    688, 689, 690, 691, 692, 231, 232, 699, 709, 700, 701, 702, 706, 234, 708, 703,
    710, 711, 701, 712, 716, 234, 718, 713, 732, 733, 749, 750, 758, 759, 760, 761,
    771, 772, 773, 774, 776, 787, 788, 777, 782, 778, 779, 780, 781, 783, 784, 785,
    786, 786, 231, 232, 790, 791, 793, 794, 798, 139, 800, 801, 809, 810, 811, 815,
    816, 817, 818, 830, 819, 820, 821, 824, 822, 823, 825, 231, 232, 826, 139, 827,
    828, 829, 831, 842, 843, 844, 832, 833, 834, 835, 836, 837, 838, 839, 840, 841,
    849, 850, 851, 852, 853, 854, 855, 856, 862, 863, 867, 868, 869, 871, 883, 872,
    873, 874, 875, 879, 876, 877, 880, 881, 882, 884, 885, 886, 887, 888, 889, 890,
    891, 892, 898, 901, 899, 900, 902, 903, 904, 905, 906, 914, 922, 923, 915, 916,
    917, 918, 919, 920, 921, 924, 925, 943, 944, 945, 950, 951, 952, 953, 224, 954,
    234, 946, 947, 948, 949, 956, 957, 958, 962, 963, 964, 974, 975, 976, 979, 959,
    960, 961, 231, 232, 965, 966, 967, 968, 969, 973, 970, 971, 972, 977, 978, 980,
    981, 983, 984, 985, 986, 987, 988, 990, 991, 992, 993, 994, 995, 996, 139, 998,
    999, 1000, 1001, 1003, 1004, 1005, 1006, 1007, 1013, 1014, 1015, 224, 1018, 234, 1023, 224,
    1024, 1026, 1027, 1028, 234, 1029, 1030, 1031, 1008, 231, 232, 1009, 139, 1010, 1011, 1012,
    1016, 1017, 1019, 1020, 1021, 1022, 1025, 1033, 1034, 1035, 1036, 1037, 1038, 1039, 1040, 1041,
    1042, 1043, 231, 232, 1044, 139, 1047, 1048, 1049, 1050, 1051, 1052, 1064, 1065, 1053, 1054,
    1055, 231, 232, 1056, 139, 1057, 1058, 1059, 1060, 1061, 1062, 555, 556, 1063, 139, 1067,
    1068, 1069, 1071, 1072, 1073, 1074, 1076, 1077, 1078, 1079, 1080, 1081, 1086, 1091, 224, 1092,
    234, 1082, 231, 232, 1083, 139, 1084, 1085, 1087, 1088, 1089, 1090, 1094, 1098, 1102, 1103,
    1104, 1109, 1112, 1095, 1096, 1097, 1099, 1100, 1101, 1105, 1108, 1106, 1107, 1110, 1111, 1114,
    1115, 1116, 1117, 1118, 1120, 1121, 1122, 1127, 1132, 1137, 1123, 231, 232, 1124, 139, 1125,
    1126, 1128, 1131, 1129, 1130, 1133, 1136, 1134, 1135, 1138, 1141, 1139, 1140, 1143, 1144, 1145,
    1146, 1147, 1148, 1161, 1149, 1156, 1160, 1150, 231, 232, 1151, 1153, 1152, 139, 1154, 1155,
    1157, 1158, 1159, 1163, 1164, 1165, 1166, 1172, 1185, 1186, 1187, 1224, 1167, 1168, 1169, 1170,
    1171, 1173, 1183, 1174, 1175, 1176, 1177, 1178, 1179, 1180, 139, 1188, 1198, 1201, 1214, 1220,
    1189, 1190, 1191, 1192, 1193, 1194, 1195, 1196, 1197, 1199, 1200, 1202, 1205, 1203, 1204, 1206,
    1207, 1208, 1211, 1209, 1210, 1212, 1213, 1215, 1216, 1217, 1218, 1219, 1221, 1222, 1223, 231,
    232, 1225, 1226, 1227, 1228, 1230, 1231, 1232, 1238, 1241, 1242, 1243, 1247, 1252, 224, 1253,
    234, 1233, 1234, 1235, 1236, 1237, 1239, 1240, 1244, 231, 232, 1245, 1246, 1248, 1251, 1249,
    1250, 1255, 1256, 1257, 1258, 1259, 1260, 1261, 1262, 1263, 1265, 1266, 1267, 1268, 1269, 1270,
    1275, 1276, 1279, 1271, 231, 232, 1272, 139, 1273, 1274, 1277, 1278, 1278, 231, 232, 1281,
    1282, 1283, 1284, 1285, 1286, 1287, 1288, 1289, 1290, 1291, 1292, 1293, 1301, 1304, 1302, 1303,
    1305, 231, 232, 1323, 1324, 1325, 1327, 1334, 1335, 1338, 1339, 1340, 1341, 1342, 1343, 1344,
    1345, 1348, 1357, 1363, 1370, 1373, 1376, 1328, 1333, 1329, 1330, 1331, 1332, 1336, 1337, 1346,
    1347, 1349, 1350, 1351, 1352, 1353, 1354, 1355, 1356, 1358, 1359, 1360, 1361, 1362, 1364, 1365,
    1366, 1367, 1368, 1369, 1371, 1372, 1374, 1375, 1377, 1404, 1378, 1389, 1392, 1401, 1379, 1388,
    1380, 1383, 1384, 1385, 1386, 1387, 1381, 1382, 1390, 1391, 1393, 1394, 1395, 1400, 1396, 1399,
    1397, 1398, 1402, 1403, 1409, 1410, 231, 232, 1411, 139, 1412, 1413, 1415, 1418, 1416, 1417,
    1434, 1435, 1446, 1447, 1448, 1449, 1452, 1453, 1450, 1451, 1464, 1465, 1466, 1493, 1494, 1496,
    1497, 1498, 1502, 1499, 1500, 1501, 231, 232, 1504, 1505, 1506, 1507, 1509, 1510, 1513, 1511,
    1512, 1515, 1516, 1517, 1518, 1519, 1521, 1522, 1523, 1524, 1525, 1527, 1528, 1529, 1530, 1531,
    1534, 1535, 1536, 1537, 1539, 1540, 1541, 1542, 1543, 1545, 1546, 1547, 1548, 1549, 1550, 1551,
    1553, 1554, 1557, 1558, 1555, 1556, 1560, 1561, 1562, 1564, 1565, 1566, 1567, 1568, 1569, 1570,
    1571, 1573, 1574, 1575, 1576, 1578, 1579, 1580, 1581, 1582, 1584, 1585, 1586, 1587, 1588, 1590,
    1591, 1592, 1593, 1594, 1602, 701, 1603, 234, 1609, 1610, 1611, 231, 232, 231, 232, 1615,
    1616, 1642, 1643, 1644, 1645, 1650, 1646, 231, 232, 1647, 139, 1648, 1649, 1651, 1652, 1660,
    1661, 1662, 1663, 1664, 1665, 1667, 1668, 1669, 1671, 1672, 1673, 1678, 1674, 1677, 1675, 1676,
    1681, 1682, 1689, 1683, 1687, 1684, 1685, 1686, 231, 232, 1688, 231, 232, 1693, 1694, 1695,
    1696, 1697, 1700, 1701, 1702, 1706, 1707, 1708, 1709, 1715, 1698, 1699, 1703, 1704, 1705, 1710,
    1711, 1712, 1713, 1714, 1716, 1717, 1719, 1722, 1723, 1724, 1720, 1721, 1726, 1727, 1728, 1729,
    1730, 1734, 1731, 1732, 1733, 1736, 1739, 1737, 1738, 1741, 1745, 1751, 1742, 1743, 1744, 231,
    232, 1746, 1749, 1747, 1748, 1750, 1754, 1789, 1755, 1756, 1757, 1769, 1770, 1771, 1778, 1786,
    1787, 1758, 231, 232, 1759, 1760, 1761, 1762, 1763, 139, 1764, 1765, 1766, 1767, 1768, 1772,
    1775, 1773, 1774, 1776, 231, 232, 1777, 139, 1779, 1780, 1781, 1782, 1783, 1784, 1785, 1791,
    1792, 1793, 1794, 1795, 1796, 1797, 1802, 231, 232, 1803, 139, 1804, 1810, 1816, 1805, 1808,
    1809, 1806, 1807, 1811, 1814, 1815, 1812, 1813, 1817, 1818, 1819, 1820, 1821, 1827, 1828, 1831,
    1835, 1838, 1829, 1830, 1832, 1833, 1834, 1836, 1837, 1839, 1840, 1841, 1842, 1846, 1860, 1864,
    1872, 1873, 1874, 1878, 1847, 1848, 1849, 1850, 1853, 1858, 1859, 1851, 1852, 1854, 1855, 1856,
    1857, 1861, 1862, 1863, 1865, 1870, 1871, 1866, 1867, 1868, 1869, 1875, 1876, 1877, 1879, 1880,
    1881, 1882, 1883, 1884, 1886, 1889, 1887, 1888, 1890, 1891, 1894, 1895, 1896, 231, 232, 1897,
    1898, 1906, 1907, 1913, 1914, 1916, 1917, 1918, 1924, 1925, 1926, 1939, 1927, 1930, 1928, 1929,
    1931, 1936, 1932, 1933, 1934, 1935, 1937, 1938, 1940, 1926, 1941, 1926, 1943, 1944, 1947, 1948,
    1949, 139, 1951, 1952, 1953, 1954, 1955, 1956, 1959, 1962, 1963, 1964, 1965, 1966, 1957, 1958,
    1960, 1961, 1968, 1986, 1969, 1970, 1973, 1974, 1980, 1984, 1985, 1971, 1972, 1972, 231, 232,
    1975, 1976, 1977, 1978, 1979, 1981, 1982, 1983, 1987, 231, 232, 1968, 1988, 1989, 1991, 1992,
    1992, 231, 232, 701, 1993, 234, 2004, 2005, 2016, 1994, 1995, 1996, 1997, 231, 232, 1999,
    2000, 2002, 2003, 2006, 2007, 2008, 2009, 2011, 2012, 2014, 2015, 2022, 2023, 2024, 2029, 2025,
    2026, 2027, 2028, 2031, 2032, 2033, 2034, 2035, 2038, 2039, 2043, 2049, 2055, 2061, 2062, 2065,
    2075, 2076, 2077, 2040, 2041, 2042, 2044, 2045, 2047, 2048, 2046, 2050, 2053, 2054, 2051, 2052,
    2056, 2057, 2058, 2059, 2060, 139, 2063, 2064, 2066, 2070, 2071, 2067, 2068, 2069, 2072, 2073,
    2074, 2079, 2080, 231, 232, 2082, 2083, 2084, 2085, 2086, 2088, 2089, 2090, 2111, 2091, 2101,
    2092, 2093, 2094, 2095, 2096, 2097, 2099, 2100, 2098, 2102, 2103, 2104, 2105, 2106, 2107, 2109,
    2110, 2108, 2113, 2114, 2116, 224, 2117, 234, 2119, 2128, 2132, 2133, 2134, 2115, 2118, 2120,
    2121, 2124, 2125, 224, 2126, 234, 2122, 2123, 2127, 2129, 224, 2130, 234, 2131, 2138, 2143,
    2139, 2140, 2141, 2144, 2145, 2148, 2155, 2149, 2150, 2151, 2154, 2152, 2153, 2156, 2159, 2157,
    2158, 2161, 231, 232, 2162, 139, 2164, 2165, 2166, 2167, 2170, 2173, 2174, 2171, 2172, 2175,
    2182, 2176, 2181, 2177, 2178, 2179, 2180, 2184, 2185, 2186, 2187, 2189, 2190, 2191, 2188, 2202,
    2203, 2204, 2205, 2214, 2215, 2216, 2206, 2207, 2208, 2211, 2209, 2210, 2212, 2213, 2217, 2218,
    2219, 2223, 2224, 2228, 2229, 224, 2230, 234, 2231, 139, 2232, 2233, 2234, 2235, 2236, 2237,
    231, 232, 2239, 2240, 2246, 2249, 2250, 2251, 2252, 2253, 2247, 2248, 2258, 2259, 139, 2260,
    2261, 2262, 2263, 231, 232, 2267, 2268, 2269, 2270, 231, 232, 2271, 139, 2272, 2273, 2294,
    701, 2295, 234, 2296, 2297, 2297, 231, 232, 2299, 2302, 2303, 2304, 2305, 2300, 2301, 2318,
    2319, 2322, 2323, 2324, 2328, 2331, 2329, 2330, 2335, 2336, 2340, 2341, 2342, 2344, 2351, 2357,
    2345, 2346, 2349, 2350, 2347, 2348, 2352, 2353, 2354, 2355, 2356, 2358, 2360, 2361, 2362, 2363,
    2365, 2368, 2364, 231, 232, 2366, 2367, 2370, 2371, 2372, 2377, 2373, 2376, 2374, 2375, 2379,
    2388, 2391, 2380, 2381, 2382, 2383, 2384, 2385, 2386, 2387, 2389, 2390, 2392, 2406, 2393, 2394,
    2395, 2397, 2396, 231, 232, 2398, 2399, 2402, 2400, 2401, 2403, 2404, 2405, 2407, 2408, 2409,
    2411, 2432, 2435, 2412, 2423, 2413, 2416, 2414, 2415, 2417, 2420, 2418, 2419, 2421, 2422, 2424,
    2425, 2426, 2429, 2427, 2428, 2430, 2431, 2433, 2434, 2437, 2438, 2439, 2442, 2440, 2441, 2443,
    2444, 2445, 2446, 2447, 2448, 2451, 2452, 2462, 2453, 2454, 2455, 2459, 2456, 2457, 2458, 231,
    232, 2460, 231, 232, 2461, 139, 2465, 2466, 2467, 2468, 2473, 2469, 231, 232, 2470, 2472,
    2471, 139, 2476, 2477, 2478, 2479, 2490, 2491, 2492, 2493, 2494, 2495, 2498, 2499, 2496, 2497,
    2508, 2509, 2510, 2511, 2515, 2522, 2512, 2513, 2514, 231, 232, 2516, 2517, 2519, 2518, 231,
    232, 2520, 2521, 2523, 2536, 2524, 2531, 2525, 2529, 2526, 2527, 2528, 2530, 231, 232, 2532,
    2533, 2534, 231, 232, 2535, 139, 2537, 2538, 2539, 2540, 2541, 2542, 2543, 2544, 231, 232,
    2548, 2549, 2550, 2556, 2557, 2558, 2559, 2560, 2562, 2563, 2564, 2565, 2580, 2593, 2598, 2581,
    2582, 224, 2583, 2584, 234, 2586, 2585, 2587, 2588, 2589, 2592, 2590, 2591, 2594, 2595, 224,
    2596, 2597, 234, 2599, 224, 2600, 234, 2603, 2606, 2607, 2608, 2611, 2609, 2610, 2610, 231,
    232, 2613, 2614, 2615, 2619, 2616, 2617, 2618, 2621, 224, 2622, 234, 2623, 231, 232, 2624,
    139, 2637, 2638, 2639, 2644, 2640, 2641, 2642, 2643, 2645, 2646, 2647, 2648, 2649, 2650, 2651,
    2652, 2653, 2670, 2671, 2675, 2678, 2679, 2676, 2677, 2680, 2681, 2683, 2684, 2688, 2689, 2690,
    2694, 2691, 2692, 2693, 2695, 2696, 2697, 2698, 2704, 2705, 2706, 2707, 2708, 2709, 2710, 2711,
    2712, 2717, 2718, 2719, 2750, 2751, 2753, 224, 2754, 234, 2755, 2756, 2758, 2759, 2762, 2765,
    2763, 2764, 2766, 2767, 2769, 2770, 2771, 2772, 2773, 2774, 2776, 2777, 2778, 2779, 2780, 2782,
    224, 2783, 234, 2786, 224, 2787, 234, 2797, 2798, 2799, 2800, 2784, 2785, 2788, 2789, 2790,
    2794, 2791, 2793, 2792, 231, 232, 2795, 2796, 2802, 2803, 2804, 2827, 2828, 2829, 2830, 2831,
    2832, 231, 232, 2834, 224, 2835, 234, 2836, 139, 2841, 2842, 2862, 2863, 2864, 2866, 2868,
    2870, 2865, 231, 232, 2867, 231, 232, 2869, 231, 232, 2873, 2874, 2875, 2876, 2878, 2879,
    2882, 2880, 2881, 2881, 231, 232, 2888, 2891, 2889, 2890, 2892, 2893, 2916, 2917, 2918, 2920,
    2922, 1621, 2921, 139, 2924, 2925, 2926, 2927, 2929, 2930, 2931, 2932, 2933, 2934, 2935, 2936,
    2937, 2938, 2939, 659, 2946, 2947, 2951, 2948, 2949, 2950, 2952, 2953, 2960, 2961, 2962, 2971,
    2972, 2975, 2976, 224, 2977, 234, 2978, 139, 2979, 2983, 2986, 2991, 2980, 2981, 2982, 2984,
    2985, 2987, 2988, 2989, 2990, 2995, 2996, 2997, 2998, 2999, 3000, 3001, 3004, 3005, 224, 3013,
    3014, 3015, 234, 3016, 3017, 3018, 231, 232, 3022, 3023, 3024, 3029, 3025, 3026, 3027, 3028,
    231, 232, 3045, 3046, 224, 3047, 234, 3048, 3049, 3049, 231, 232, 3050, 139, 3059, 3064,
    3060, 3061, 3062, 3063, 3079, 3080, 3081, 3086, 3082, 3085, 3083, 3084, 3094, 3095, 3098, 3096,
    3097, 3097, 231, 232, 3101, 3109, 3102, 3103, 3108, 3104, 3105, 3106, 3107, 3110, 3111, 3113,
    3130, 3114, 3124, 3115, 3118, 3116, 3117, 3119, 3120, 3121, 3122, 3123, 3125, 3126, 3127, 3128,
    3129, 3133, 3134, 3135, 3139, 3143, 3146, 3150, 3136, 3137, 3138, 3140, 3141, 3142, 139, 3144,
    3145, 3147, 3148, 3149, 139, 3154, 3155, 3165, 3166, 3168, 701, 3169, 234, 3172, 3173, 3177,
    3174, 3175, 3176, 3178, 231, 232, 3203, 3206, 3223, 3204, 3205, 3207, 3211, 3215, 3218, 3222,
    3208, 3209, 3210, 3212, 3213, 3214, 139, 3216, 3217, 3219, 3220, 3221, 139, 3229, 3230, 3231,
    3232, 3233, 3234, 3238, 3235, 3236, 3237, 3239, 3240, 3241, 3243, 3244, 3245, 3246, 3247, 3250,
    3251, 3252, 3255, 3256, 3257, 3260, 3263, 3258, 3259, 3261, 3262, 3264, 3265, 3281, 3282, 3283,
    3286, 3284, 3285, 3292, 3293, 3305, 3306, 3307, 3294, 3295, 3296, 3297, 3298, 3299, 3300, 3301,
    3302, 3303, 3304, 3325, 3326, 3327, 3328, 3337, 3338, 3329, 3330, 3331, 3334, 3332, 3333, 3335,
    3336, 3357, 3358, 3361, 3363, 3359, 3360, 3362, 231, 232, 3364, 3366, 3365, 3367, 231, 232,
    3388, 3389, 3390, 3391, 3393, 3394, 3396, 3397, 224, 3398, 3410, 3411, 3412, 234, 3399, 139,
    3400, 3401, 3407, 3402, 3403, 3404, 3405, 3406, 3408, 3409, 3415, 3417, 3418, 3421, 3422, 3423,
    3424, 3416, 3419, 3420, 3427, 3428, 3429, 3430, 3431, 3432, 3433, 3434, 3435, 3436, 3437, 3442,
    3445, 3448, 3449, 3443, 3444, 3444, 231, 232, 3446, 3447, 3447, 231, 232, 3450, 231, 232,
    3458, 3461, 3459, 3460, 3462, 3463, 3465, 3468, 3466, 3467, 3469, 3470, 3479, 224, 3480, 234,
    3481, 139, 3484, 3485, 3486, 3487, 3488, 3489, 3490, 3491, 3492, 3493, 3495, 3496, 3500, 3501,
    3502, 3497, 3498, 3499, 3505, 3508, 3509, 3506, 3507, 3510, 3511, 3513, 3514, 3515, 3516, 3519,
    3522, 3520, 3521, 3523, 3524, 3525, 3526, 3531, 3532, 3537, 3538, 3539, 3541, 139, 3543, 3544,
    3545, 3550, 3551, 3632, 3633, 3634, 3635, 3636, 3637, 3638, 3639, 3640, 3641, 3642, 3643, 3644,
    3645, 3646, 3647, 3648, 3649, 3650, 3651, 3652, 3653, 3654, 3655, 3656, 3657, 3658, 3659, 3660,
    3661, 3662, 3663, 3664, 3665, 3666, 3667, 3668, 3669, 3670, 3671, 3672, 3673, 3674, 3675, 3676,
    3677, 3678, 3679, 3688, 3690, 231, 232, 3691, 139, 3692, 3693, 3712, 3713, 3715, 3718, 3716,
    3717, 3719, 3722, 3725, 3720, 3721, 3723, 3724, 3726, 3727, 3728, 3730, 3731, 3732, 3734, 3735,
    3736, 3737, 139, 3741, 3745, 3742, 3743, 3744, 3746, 3747, 3748, 3751, 3752, 3753, 3754, 3756,
    3757, 3758, 3759, 3755, 231, 232, 3761, 3769, 3775, 3762, 3763, 3764, 3765, 3766, 3767, 3768,
    3770, 3773, 3774, 3771, 3772, 3778, 3786, 3794, 3779, 3780, 3781, 3782, 3783, 3784, 3785, 3787,
    3790, 3788, 3789, 3791, 3792, 3793, 3796, 3797, 3798, 3799, 139, 3805, 3808, 3806, 3810, 3812,
    3811, 3819, 3820, 3823, 3824, 3831, 3834, 3835, 3832, 3833, 3836, 3839, 3840, 3837, 3838, 3842,
    3843, 3844, 3845, 3846, 3865, 3868, 3866, 3867, 3877, 3878, 3879, 3884, 3885, 3886, 3887, 3888,
    3889, 3890, 3896, 3891, 3892, 3893, 3894, 3895, 3897, 3898, 3901, 3902, 3903, 3905, 231, 232,
    3906, 139, 3909, 3910, 3912, 3913, 3917, 3919, 3918, 139, 3925, 3926, 3927, 3933, 3934, 3935,
    3936, 3939, 3948, 3949, 3950, 3951, 3952, 3953, 3954, 3955, 3956, 3963, 3964, 3965, 3966, 3967,
    3971, 224, 3972, 234, 3975, 3976, 3977, 3978, 3973, 3974, 3983, 3984, 3986, 3987, 3988, 3989,
    3992, 3993, 3994, 3995, 3996, 3998, 3999, 4000, 4001, 4003, 4004, 4012, 4013, 4025, 4014, 139,
    4015, 4021, 4016, 231, 232, 4017, 139, 4018, 4019, 4020, 4022, 139, 4023, 4024, 4028, 4029,
    4030, 4034, 4035, 4039, 4031, 4032, 4033, 4036, 4037, 4038, 4040, 4041, 4042, 4057, 4058, 4060,
    4061, 4062, 4063, 4064, 4065, 701, 4059, 234, 4069, 701, 4070, 234, 4071, 4072, 4072, 231,
    232, 4074, 4075, 4076, 4077, 4078, 4081, 4084, 4085, 4086, 4087, 4088, 4079, 4080, 4082, 4083,
    4093, 4095, 4103, 4096, 4097, 4100, 4098, 4099, 4101, 4102, 4106, 4110, 4111, 4112, 4107, 4108,
    4109, 4114, 4115, 4116, 4117, 4120, 4121, 4118, 139, 4123, 4124, 4125, 4126, 4129, 4130, 4127,
    139, 4132, 4133, 4135, 4134, 4138, 4139, 4140, 4141, 4143, 4144, 4145, 4148, 4146, 4147, 4151,
    4152, 4153, 4154,
];

pub static TERMINATORS: &[u32] = &[
    140, 141, 155, 599, 600, 704, 705, 707, 714, 715, 717, 719, 720, 721, 762, 802,
    878, 1181, 1182, 1184, 1752, 1788, 1998, 2001, 2010, 2013, 2142, 2146, 2238, 2241, 2242, 2243,
    2244, 2264, 2992, 2993, 3376, 3413, 3438, 3439, 3440, 3482, 3680, 3738, 3739, 3800, 3801, 3807,
    3957, 3958, 3959, 3960, 3961, 4119, 4128,
];

pub static STRINGS: &[&str] = &[
    "ABS", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABSOLUTE", // [6]
    "ACCESS", // [7]
    "FUNCTION", // [8]
    "SCHEMA", // [9]
    "SCRIPT", // [10]
    "TABLE", // [11]
    "VIEW", // [12]
    "FunctionKeywordSegment", // [13]
    "SchemaKeywordSegment", // [14]
    "ScriptKeywordSegment", // [15]
    "TableKeywordSegment", // [16]
    "ViewKeywordSegment", // [17]
    "AlterKeywordSegment", // [18]
    "ALTER", // [19]
    "SelectKeywordSegment", // [20]
    "SELECT", // [21]
    "InsertKeywordSegment", // [22]
    "INSERT", // [23]
    "UpdateKeywordSegment", // [24]
    "UPDATE", // [25]
    "DeleteKeywordSegment", // [26]
    "DELETE", // [27]
    "ReferencesKeywordSegment", // [28]
    "REFERENCES", // [29]
    "ExecuteKeywordSegment", // [30]
    "EXECUTE", // [31]
    "ImportKeywordSegment", // [32]
    "IMPORT", // [33]
    "ExportKeywordSegment", // [34]
    "EXPORT", // [35]
    "ALL", // [36]
    "AllKeywordSegment", // [37]
    "PrivilegesKeywordSegment", // [38]
    "PRIVILEGES", // [39]
    "GRANT", // [40]
    "GrantKeywordSegment", // [41]
    "AnyKeywordSegment", // [42]
    "ANY", // [43]
    "ObjectKeywordSegment", // [44]
    "OBJECT", // [45]
    "PrivilegeKeywordSegment", // [46]
    "PRIVILEGE", // [47]
    "SET", // [48]
    "SetKeywordSegment", // [49]
    "ConsumerKeywordSegment", // [50]
    "CONSUMER", // [51]
    "GroupKeywordSegment", // [52]
    "GROUP", // [53]
    "MANAGE", // [54]
    "ManageKeywordSegment", // [55]
    "GroupsKeywordSegment", // [56]
    "GROUPS", // [57]
    "KILL", // [58]
    "KillKeywordSegment", // [59]
    "SessionKeywordSegment", // [60]
    "SESSION", // [61]
    "SystemKeywordSegment", // [62]
    "SYSTEM", // [63]
    "CREATE", // [64]
    "DROP", // [65]
    "CreateKeywordSegment", // [66]
    "DropKeywordSegment", // [67]
    "UserKeywordSegment", // [68]
    "USER", // [69]
    "IMPERSONATE", // [70]
    "ImpersonateKeywordSegment", // [71]
    "RoleKeywordSegment", // [72]
    "ROLE", // [73]
    "USE", // [74]
    "UseKeywordSegment", // [75]
    "AccessKeywordSegment", // [76]
    "ConnectionKeywordSegment", // [77]
    "CONNECTION", // [78]
    "VirtualKeywordSegment", // [79]
    "VIRTUAL", // [80]
    "RefreshKeywordSegment", // [81]
    "REFRESH", // [82]
    "DictionaryKeywordSegment", // [83]
    "DICTIONARY", // [84]
    "IMPERSONATION", // [85]
    "ImpersonationKeywordSegment", // [86]
    "OnKeywordSegment", // [87]
    "ON", // [88]
    "UserReferenceSegment", // [89]
    "user_reference", // [90]
    "object_reference", // [91]
    "ObjectReferenceSegment", // [92]
    "RolesKeywordSegment", // [93]
    "ROLES", // [94]
    "RoleReferenceSegment", // [95]
    "role_reference", // [96]
    "CommaSegment", // [97]
    ",", // [98]
    "ToKeywordSegment", // [99]
    "TO", // [100]
    "FromKeywordSegment", // [101]
    "FROM", // [102]
    "SingleIdentifierGrammar", // [103]
    "FOR", // [104]
    "ForKeywordSegment", // [105]
    "AccessPermissionSegment", // [106]
    "access_permission", // [107]
    "StageKeywordSegment", // [108]
    "ProcedureKeywordSegment", // [109]
    "PROCEDURE", // [110]
    "RoutineKeywordSegment", // [111]
    "ROUTINE", // [112]
    "SequenceKeywordSegment", // [113]
    "SEQUENCE", // [114]
    "StreamKeywordSegment", // [115]
    "TaskKeywordSegment", // [116]
    "MaskingKeywordSegment", // [117]
    "PolicyKeywordSegment", // [118]
    "MaterializedKeywordSegment", // [119]
    "EXTERNAL", // [120]
    "ExternalKeywordSegment", // [121]
    "FILE", // [122]
    "FileKeywordSegment", // [123]
    "FormatKeywordSegment", // [124]
    "FORMAT", // [125]
    "TablesKeywordSegment", // [126]
    "TABLES", // [127]
    "ViewsKeywordSegment", // [128]
    "StagesKeywordSegment", // [129]
    "FunctionsKeywordSegment", // [130]
    "ProceduresKeywordSegment", // [131]
    "RoutinesKeywordSegment", // [132]
    "SequencesKeywordSegment", // [133]
    "StreamsKeywordSegment", // [134]
    "TasksKeywordSegment", // [135]
    "TASKS", // [136]
    "SchemasKeywordSegment", // [137]
    "SCHEMAS", // [138]
    "PoliciesKeywordSegment", // [139]
    "FormatsKeywordSegment", // [140]
    "REVOKE", // [141]
    "GrantStatementSegment", // [142]
    "grant_statement", // [143]
    "RevokeStatementSegment", // [144]
    "revoke_statement", // [145]
    "[", // [146]
    "ArrayAccessorSegment", // [147]
    "array_accessor", // [148]
    "ACOS", // [149]
    "ACTION", // [150]
    "ADAPTER", // [151]
    "ADD", // [152]
    "ADD_DAYS", // [153]
    "ADD_HOURS", // [154]
    "ADD_MINUTES", // [155]
    "ADD_MONTHS", // [156]
    "ADD_SECONDS", // [157]
    "ADD_WEEKS", // [158]
    "ADD_YEARS", // [159]
    "ADMIN", // [160]
    "AFTER", // [161]
    "OrderByClauseSegment", // [162]
    "ORDER", // [163]
    "orderby_clause", // [164]
    "indent", // [165]
    "Indent", // [166]
    "meta", // [167]
    "AsAliasOperatorSegment", // [168]
    "AS", // [169]
    "alias_operator", // [170]
    "(", // [171]
    "SingleIdentifierListSegment", // [172]
    "identifier_list", // [173]
    "start_bracket", // [174]
    "SymbolSegment", // [175]
    "symbol", // [176]
    ")", // [177]
    "end_bracket", // [178]
    "SingleQuotedIdentifierSegment", // [179]
    "single_quote", // [180]
    "dedent", // [181]
    "Dedent", // [182]
    "TableReferenceSegment", // [183]
    "table_reference", // [184]
    "AliasExpressionSegment", // [185]
    "alias_expression", // [186]
    "ALIGN", // [187]
    "ALLOCATE", // [188]
    "NakedIdentifierSegment", // [189]
    "ConnectionDefinition", // [190]
    "connection_definition", // [191]
    "CPU_WEIGHT", // [192]
    "GROUP_TEMP_DB_RAM_LIMIT", // [193]
    "IDLE_TIMEOUT", // [194]
    "PRECEDENCE", // [195]
    "QUERY_TIMEOUT", // [196]
    "SESSION_TEMP_DB_RAM_LIMIT", // [197]
    "USER_TEMP_DB_RAM_LIMIT", // [198]
    "ConsumerGroupParameterSegment", // [199]
    "consumer_group_parameter", // [200]
    "CONSUMER_GROUP", // [201]
    "Consumer_groupKeywordSegment", // [202]
    "EqualsSegment", // [203]
    "=", // [204]
    "comparison_operator", // [205]
    "NullKeywordSegment", // [206]
    "NULL", // [207]
    "SchemaReferenceSegment", // [208]
    "schema_reference", // [209]
    "CHANGE", // [210]
    "Raw_size_limitKeywordSegment", // [211]
    "RAW_SIZE_LIMIT", // [212]
    "*", // [213]
    "numeric_literal", // [214]
    "NumericLiteralSegment", // [215]
    "StarSegment", // [216]
    "ChangeKeywordSegment", // [217]
    "OwnerKeywordSegment", // [218]
    "OWNER", // [219]
    "IncrementKeywordSegment", // [220]
    "ByKeywordSegment", // [221]
    "BY", // [222]
    "SequenceMinValueGrammar", // [223]
    "SequenceMaxValueGrammar", // [224]
    "CacheKeywordSegment", // [225]
    "NocacheKeywordSegment", // [226]
    "CYCLE", // [227]
    "NOCYCLE", // [228]
    "CycleKeywordSegment", // [229]
    "NocycleKeywordSegment", // [230]
    "OrderNoOrderGrammar", // [231]
    "SequenceReferenceSegment", // [232]
    "sequence_reference", // [233]
    "AlterSequenceOptionsSegment", // [234]
    "alter_sequence_options_segment", // [235]
    "SessionParameterSegment", // [236]
    "CONSTRAINT_STATE_DEFAULT", // [237]
    "DEFAULT_LIKE_ESCAPE_CHARACTER", // [238]
    "HASHTYPE_FORMAT", // [239]
    "NICE", // [240]
    "NLS_DATE_FORMAT", // [241]
    "NLS_DATE_LANGUAGE", // [242]
    "NLS_FIRST_DAY_OF_WEEK", // [243]
    "NLS_NUMERIC_CHARACTERS", // [244]
    "NLS_TIMESTAMP_FORMAT", // [245]
    "PROFILE", // [246]
    "QUERY_CACHE", // [247]
    "SCRIPT_LANGUAGES", // [248]
    "SCRIPT_OUTPUT_ADDRESS", // [249]
    "SNAPSHOT_MODE", // [250]
    "SQL_PREPROCESSOR_SCRIPT", // [251]
    "ST_MAX_DECIMAL_DIGITS", // [252]
    "TIMESTAMP_ARITHMETIC_BEHAVIOR", // [253]
    "TIME_ZONE", // [254]
    "TIME_ZONE_BEHAVIOR", // [255]
    "QuotedLiteralSegment", // [256]
    "SystemParameterSegment", // [257]
    "DEFAULT_CONSUMER_GROUP", // [258]
    "PASSWORD_EXPIRY_POLICY", // [259]
    "PASSWORD_SECURITY_POLICY", // [260]
    "TEMP_DB_RAM_LIMIT", // [261]
    "AddKeywordSegment", // [262]
    "ColumnKeywordSegment", // [263]
    "COLUMN", // [264]
    "IfNotExistsGrammar", // [265]
    "IF", // [266]
    "ColumnDefinitionSegment", // [267]
    "column_definition", // [268]
    "DEFAULT", // [269]
    "IDENTITY", // [270]
    "IdentityKeywordSegment", // [271]
    "DefaultKeywordSegment", // [272]
    "LiteralGrammar", // [273]
    "BareFunctionSegment", // [274]
    "CONNECT_BY_ISCYCLE", // [275]
    "CONNECT_BY_ISLEAF", // [276]
    "CONNECT_BY_ROOT", // [277]
    "CURDATE", // [278]
    "CURRENT_DATE", // [279]
    "CURRENT_SCHEMA", // [280]
    "CURRENT_SESSION", // [281]
    "CURRENT_STATEMENT", // [282]
    "CURRENT_TIMESTAMP", // [283]
    "CURRENT_USER", // [284]
    "DBTIMEZONE", // [285]
    "LEVEL", // [286]
    "LOCALTIMESTAMP", // [287]
    "NOW", // [288]
    "ROWID", // [289]
    "ROWNUM", // [290]
    "SESSIONTIMEZONE", // [291]
    "SYSDATE", // [292]
    "SYSTIMESTAMP", // [293]
    "MODIFY", // [294]
    "RENAME", // [295]
    "AlterTableAddColumnSegment", // [296]
    "alter_table_add_column", // [297]
    "AlterTableDropColumnSegment", // [298]
    "alter_table_drop_column", // [299]
    "AlterTableModifyColumnSegment", // [300]
    "alter_table_modify_column", // [301]
    "AlterTableRenameColumnSegment", // [302]
    "alter_table_rename_column", // [303]
    "AlterTableAlterColumnSegment", // [304]
    "alter_table_alter_column", // [305]
    "TableOutOfLineConstraintSegment", // [306]
    "CONSTRAINT", // [307]
    "FOREIGN", // [308]
    "PRIMARY", // [309]
    "table_constraint_definition", // [310]
    "ModifyKeywordSegment", // [311]
    "ConstraintKeywordSegment", // [312]
    "PrimaryKeyGrammar", // [313]
    "TableConstraintEnableDisableGrammar", // [314]
    "DISABLE", // [315]
    "ENABLE", // [316]
    "IfExistsGrammar", // [317]
    "RenameKeywordSegment", // [318]
    "DISTRIBUTE", // [319]
    "PARTITION", // [320]
    "TableDistributionPartitionClause", // [321]
    "table_distribution_partition_clause", // [322]
    "DISTRIBUTION", // [323]
    "DistributionKeywordSegment", // [324]
    "AndKeywordSegment", // [325]
    "AND", // [326]
    "PartitionKeywordSegment", // [327]
    "KeysKeywordSegment", // [328]
    "KEYS", // [329]
    "CASCADE", // [330]
    "CascadeKeywordSegment", // [331]
    "ConstraintsKeywordSegment", // [332]
    "CONSTRAINTS", // [333]
    "DatatypeSegment", // [334]
    "BIGINT", // [335]
    "BOOL", // [336]
    "BOOLEAN", // [337]
    "CHAR", // [338]
    "CHARACTER", // [339]
    "CLOB", // [340]
    "DATE", // [341]
    "DEC", // [342]
    "DECIMAL", // [343]
    "DOUBLE", // [344]
    "FLOAT", // [345]
    "GEOMETRY", // [346]
    "HASHTYPE", // [347]
    "INT", // [348]
    "INTEGER", // [349]
    "INTERVAL", // [350]
    "LONG", // [351]
    "NCHAR", // [352]
    "NUMBER", // [353]
    "NUMERIC", // [354]
    "NVARCHAR", // [355]
    "NVARCHAR2", // [356]
    "REAL", // [357]
    "SHORTINT", // [358]
    "SMALLINT", // [359]
    "TIMESTAMP", // [360]
    "TINYINT", // [361]
    "VARCHAR", // [362]
    "VARCHAR2", // [363]
    "data_type", // [364]
    "ColumnConstraintSegment", // [365]
    "COMMENT", // [366]
    "NOT", // [367]
    "column_constraint_segment", // [368]
    "ParameterNameSegment", // [369]
    "FIRST", // [370]
    "FirstKeywordSegment", // [371]
    "AfterKeywordSegment", // [372]
    "ColumnReferenceSegment", // [373]
    "column_reference", // [374]
    "BracketedColumnReferenceListGrammar", // [375]
    "AlterTableDropColumnGrammar", // [376]
    "AsKeywordSegment", // [377]
    "AlterTableColumnSegment", // [378]
    "alter_table_column_statement", // [379]
    "AlterTableConstraintSegment", // [380]
    "alter_table_constraint_statement", // [381]
    "AlterTableDistributePartitionSegment", // [382]
    "alter_table_distribute_partition_statement", // [383]
    "IDENTIFIED", // [384]
    "PASSWORD", // [385]
    "RESET", // [386]
    "IdentifiedKeywordSegment", // [387]
    "AT", // [388]
    "UserPasswordAuthSegment", // [389]
    "password_auth", // [390]
    "REPLACE", // [391]
    "ReplaceKeywordSegment", // [392]
    "PasswordLiteralSegment", // [393]
    "double_quote", // [394]
    "UserLDAPAuthSegment", // [395]
    "ldap_auth", // [396]
    "UserKerberosAuthSegment", // [397]
    "kerberos_auth", // [398]
    "UserOpenIDAuthSegment", // [399]
    "openid_auth", // [400]
    "Password_expiry_policyKeywordSegment", // [401]
    "PasswordKeywordSegment", // [402]
    "ExpireKeywordSegment", // [403]
    "EXPIRE", // [404]
    "ResetKeywordSegment", // [405]
    "FailedKeywordSegment", // [406]
    "FAILED", // [407]
    "LoginKeywordSegment", // [408]
    "LOGIN", // [409]
    "AttemptsKeywordSegment", // [410]
    "ATTEMPTS", // [411]
    "ALWAYS", // [412]
    "&", // [413]
    "ampersand", // [414]
    "ANALYZE", // [415]
    "binary_operator", // [416]
    "BinaryOperatorSegment", // [417]
    "ANSI", // [418]
    "APPEND", // [419]
    "APPROXIMATE_COUNT_DISTINCT", // [420]
    "ARE", // [421]
    "%", // [422]
    "+", // [423]
    "-", // [424]
    "/", // [425]
    "<", // [426]
    ">", // [427]
    "^", // [428]
    "|", // [429]
    "PlusSegment", // [430]
    "MinusSegment", // [431]
    "DivideSegment", // [432]
    "MultiplySegment", // [433]
    "ModuloSegment", // [434]
    "BitwiseAndSegment", // [435]
    "BitwiseOrSegment", // [436]
    "BitwiseXorSegment", // [437]
    "BitwiseLShiftSegment", // [438]
    "BitwiseRShiftSegment", // [439]
    "ExpressionSegment", // [440]
    "expression", // [441]
    "SliceSegment", // [442]
    ":", // [443]
    "start_square_bracket", // [444]
    "]", // [445]
    "end_square_bracket", // [446]
    "ARRAY", // [447]
    "BaseExpressionElementGrammar", // [448]
    "ASC", // [449]
    "ASCII", // [450]
    "ASENSITIVE", // [451]
    "ASIN", // [452]
    "ASSERTION", // [453]
    "ASSIGNMENT", // [454]
    "ASYMMETRIC", // [455]
    "ATAN2", // [456]
    "ATAN", // [457]
    "ATOMIC", // [458]
    "ATTRIBUTE", // [459]
    "AUDIT", // [460]
    "AUTHENTICATED", // [461]
    "AUTHID", // [462]
    "AUTHORIZATION", // [463]
    "Auto_incrementKeywordSegment", // [464]
    "AUTO", // [465]
    "AVG", // [466]
    "BACKUP", // [467]
    "bare_function", // [468]
    "CodeSegment", // [469]
    "IntervalExpressionSegment", // [470]
    "interval_expression", // [471]
    "FunctionSegment", // [472]
    "function", // [473]
    "file", // [474]
    "BaseFileSegment", // [475]
    "BaseSegment", // [476]
    "BEFORE", // [477]
    "BEGIN", // [478]
    "BERNOULLI", // [479]
    "BETWEEN", // [480]
    "BINARY", // [481]
    "!", // [482]
    "IS", // [483]
    "OR", // [484]
    "like_operator", // [485]
    "ArithmeticBinaryOperatorGrammar", // [486]
    "StringBinaryOperatorGrammar", // [487]
    "BooleanBinaryOperatorGrammar", // [488]
    "ComparisonOperatorGrammar", // [489]
    "BIT", // [490]
    "BIT_AND", // [491]
    "BIT_CHECK", // [492]
    "BIT_LENGTH", // [493]
    "BIT_LROTATE", // [494]
    "BIT_LSHIFT", // [495]
    "BIT_NOT", // [496]
    "BIT_OR", // [497]
    "BIT_RROTATE", // [498]
    "BIT_RSHIFT", // [499]
    "BIT_SET", // [500]
    "BIT_TO_NUM", // [501]
    "BIT_XOR", // [502]
    "AmpersandSegment", // [503]
    "RawLessThanSegment", // [504]
    "PipeSegment", // [505]
    "RawGreaterThanSegment", // [506]
    "BLOB", // [507]
    "BLOCKED", // [508]
    "AndOperatorGrammar", // [509]
    "OrOperatorGrammar", // [510]
    "FALSE", // [511]
    "TRUE", // [512]
    "UNKNOWN", // [513]
    "TrueSegment", // [514]
    "FalseSegment", // [515]
    "UnknownSegment", // [516]
    "BOTH", // [517]
    "BYTE", // [518]
    "BitKeywordSegment", // [519]
    "ByteKeywordSegment", // [520]
    "CharKeywordSegment", // [521]
    "bracketed", // [522]
    "BracketedSegment", // [523]
    "UnorderedSetExpressionSegment", // [524]
    "VALUES", // [525]
    "set_expression", // [526]
    "BREADTH", // [527]
    "RangeOperator", // [528]
    "range_operator", // [529]
    "DELIMIT", // [530]
    "DelimitKeywordSegment", // [531]
    "NEVER", // [532]
    "AlwaysKeywordSegment", // [533]
    "NeverKeywordSegment", // [534]
    "AutoKeywordSegment", // [535]
    "CTEColumnList", // [536]
    "cte_column_list", // [537]
    "SelectableGrammar", // [538]
    "WITH", // [539]
    "CALL", // [540]
    "CALLED", // [541]
    "CARDINALITY", // [542]
    "CASCADED", // [543]
    "CASE", // [544]
    "CaseKeywordSegment", // [545]
    "implicit_indent", // [546]
    "ImplicitIndent", // [547]
    "WHEN", // [548]
    "WhenClauseSegment", // [549]
    "when_clause", // [550]
    "ElseKeywordSegment", // [551]
    "ELSE", // [552]
    "EndKeywordSegment", // [553]
    "END", // [554]
    "ElseClauseSegment", // [555]
    "else_clause", // [556]
    "BinaryOperatorGrammar", // [557]
    "CASESPECIFIC", // [558]
    "CAST", // [559]
    "::", // [560]
    "casting_operator", // [561]
    "CATALOG", // [562]
    "CEIL", // [563]
    "CEILING", // [564]
    "CHAIN", // [565]
    "UTF8", // [566]
    "Utf8KeywordSegment", // [567]
    "AsciiKeywordSegment", // [568]
    "CHARACTER_LENGTH", // [569]
    "CHARACTER_SET_CATALOG", // [570]
    "CHARACTER_SET_NAME", // [571]
    "CHARACTER_SET_SCHEMA", // [572]
    "CHARACTERISTICS", // [573]
    "CHARACTERS", // [574]
    "CHECK", // [575]
    "CHECKED", // [576]
    "CHR", // [577]
    "CLEAR", // [578]
    "CLOSE", // [579]
    "CloseKeywordSegment", // [580]
    "COALESCE", // [581]
    "COBOL", // [582]
    "COLLATE", // [583]
    "COLLATION", // [584]
    "ObjectReferenceDelimiterGrammar", // [585]
    ".", // [586]
    "ObjectReferenceTerminatorGrammar", // [587]
    "COLLATION_CATALOG", // [588]
    "COLLATION_NAME", // [589]
    "COLLATION_SCHEMA", // [590]
    "COLOGNE_PHONETIC", // [591]
    "colon_delimiter", // [592]
    "colon_prefix", // [593]
    "colon", // [594]
    "ShorthandCastSegment", // [595]
    "cast_expression", // [596]
    "TableInlineConstraintSegment", // [597]
    "CommentClauseSegment", // [598]
    "comment_clause", // [599]
    "ColumnDatatypeSegment", // [600]
    "column_datatype_definition", // [601]
    "ColumnsExpressionNameGrammar", // [602]
    "comma", // [603]
    "CommentKeywordSegment", // [604]
    "IsKeywordSegment", // [605]
    "comment", // [606]
    "CommentSegment", // [607]
    "COMMENTS", // [608]
    "COMMIT", // [609]
    "COMMITTED", // [610]
    "GreaterThanSegment", // [611]
    "LessThanSegment", // [612]
    "GreaterThanOrEqualToSegment", // [613]
    "LessThanOrEqualToSegment", // [614]
    "NotEqualToSegment", // [615]
    "LikeOperatorSegment", // [616]
    "IsDistinctFromGrammar", // [617]
    "ComparisonOperatorSegment", // [618]
    "CompositeBinaryOperatorSegment", // [619]
    "CompositeComparisonOperatorSegment", // [620]
    "CONCAT", // [621]
    "CONDITION", // [622]
    "CrossKeywordSegment", // [623]
    "CROSS", // [624]
    "JoinTypeKeywordsGrammar", // [625]
    "FULL", // [626]
    "INNER", // [627]
    "LEFT", // [628]
    "RIGHT", // [629]
    "ConditionalCrossJoinKeywordsGrammar", // [630]
    "NonStandardJoinTypeKeywordsGrammar", // [631]
    "CONNECT", // [632]
    "START", // [633]
    "ConnectKeywordSegment", // [634]
    "StartKeywordSegment", // [635]
    "WithKeywordSegment", // [636]
    "CONSTANT", // [637]
    "CONSTRUCTOR", // [638]
    "Cpu_weightKeywordSegment", // [639]
    "PrecedenceKeywordSegment", // [640]
    "Group_temp_db_ram_limitKeywordSegment", // [641]
    "User_temp_db_ram_limitKeywordSegment", // [642]
    "Session_temp_db_ram_limitKeywordSegment", // [643]
    "Query_timeoutKeywordSegment", // [644]
    "Idle_timeoutKeywordSegment", // [645]
    "CONTAINS", // [646]
    "CONTINUE", // [647]
    "CONTROL", // [648]
    "CONVERT", // [649]
    "CONVERT_TZ", // [650]
    "CORR", // [651]
    "CORRESPONDING", // [652]
    "COS", // [653]
    "COSH", // [654]
    "COT", // [655]
    "COUNT", // [656]
    "COVAR_POP", // [657]
    "COVAR_SAMP", // [658]
    "OrReplaceGrammar", // [659]
    "JavaKeywordSegment", // [660]
    "JAVA", // [661]
    "PythonKeywordSegment", // [662]
    "PYTHON", // [663]
    "LuaKeywordSegment", // [664]
    "LUA", // [665]
    "AdapterKeywordSegment", // [666]
    "ScriptReferenceSegment", // [667]
    "script_reference", // [668]
    "ScriptContentSegment", // [669]
    "script_content", // [670]
    "CastKeywordSegment", // [671]
    "SpecificKeywordSegment", // [672]
    "SPECIFIC", // [673]
    "INSTANCE", // [674]
    "METHOD", // [675]
    "STATIC", // [676]
    "InstanceKeywordSegment", // [677]
    "StaticKeywordSegment", // [678]
    "ConstructorKeywordSegment", // [679]
    "MethodKeywordSegment", // [680]
    "FunctionNameSegment", // [681]
    "function_name", // [682]
    "FunctionParameterListGrammar", // [683]
    "function_parameter_list", // [684]
    "AssignmentKeywordSegment", // [685]
    "DatabaseKeywordSegment", // [686]
    "DATABASE", // [687]
    "DatabaseReferenceSegment", // [688]
    "database_reference", // [689]
    "FunctionReferenceSegment", // [690]
    "function_reference", // [691]
    "InKeywordSegment", // [692]
    "IN", // [693]
    "ReturnKeywordSegment", // [694]
    "RETURN", // [695]
    "VariableNameSegment", // [696]
    "DelimiterGrammar", // [697]
    ";", // [698]
    "BeginKeywordSegment", // [699]
    "FunctionBodySegment", // [700]
    "function_body", // [701]
    "FunctionContentsExpressionGrammar", // [702]
    "SemicolonSegment", // [703]
    "UniqueKeywordSegment", // [704]
    "UNIQUE", // [705]
    "IndexKeywordSegment", // [706]
    "INDEX", // [707]
    "IndexReferenceSegment", // [708]
    "index_reference", // [709]
    "IndexColumnDefinitionSegment", // [710]
    "index_column_definition", // [711]
    "ModelKeywordSegment", // [712]
    "OPTIONS", // [713]
    "OptionsKeywordSegment", // [714]
    "ArrayKeywordSegment", // [715]
    "RETURNS", // [716]
    "ReturnsKeywordSegment", // [717]
    "ROWCOUNT", // [718]
    "RowcountKeywordSegment", // [719]
    "CreateSequenceOptionsSegment", // [720]
    "create_sequence_options_segment", // [721]
    "LIKE", // [722]
    "LikeKeywordSegment", // [723]
    "EXCLUDING", // [724]
    "INCLUDING", // [725]
    "IncludingKeywordSegment", // [726]
    "ExcludingKeywordSegment", // [727]
    "DefaultsKeywordSegment", // [728]
    "DEFAULTS", // [729]
    "CommentsKeywordSegment", // [730]
    "TableContentDefinitionSegment", // [731]
    "table_content_definition", // [732]
    "WithDataClauseSegment", // [733]
    "with_data_clause", // [734]
    "CreateTableLikeClauseSegment", // [735]
    "table_like_clause", // [736]
    "TriggerKeywordSegment", // [737]
    "TRIGGER", // [738]
    "TriggerReferenceSegment", // [739]
    "trigger_reference", // [740]
    "BeforeKeywordSegment", // [741]
    "InsteadKeywordSegment", // [742]
    "OfKeywordSegment", // [743]
    "OF", // [744]
    "OrKeywordSegment", // [745]
    "DEFERRABLE", // [746]
    "INITIALLY", // [747]
    "REFERENCING", // [748]
    "ReferencingKeywordSegment", // [749]
    "OldKeywordSegment", // [750]
    "OLD", // [751]
    "RowKeywordSegment", // [752]
    "ROW", // [753]
    "NewKeywordSegment", // [754]
    "NEW", // [755]
    "NotKeywordSegment", // [756]
    "DeferrableKeywordSegment", // [757]
    "InitiallyKeywordSegment", // [758]
    "ImmediateKeywordSegment", // [759]
    "IMMEDIATE", // [760]
    "DeferredKeywordSegment", // [761]
    "DEFERRED", // [762]
    "EachKeywordSegment", // [763]
    "EACH", // [764]
    "STATEMENT", // [765]
    "StatementKeywordSegment", // [766]
    "WhenKeywordSegment", // [767]
    "FunctionNameIdentifierSegment", // [768]
    "FunctionContentsSegment", // [769]
    "function_contents", // [770]
    "RKeywordSegment", // [771]
    "R", // [772]
    "SCALAR", // [773]
    "ScalarKeywordSegment", // [774]
    "UDFParameterGrammar", // [775]
    "EMITS", // [776]
    "EmitsSegment", // [777]
    "emits_segment", // [778]
    "ForceKeywordSegment", // [779]
    "FORCE", // [780]
    "ViewReferenceSegment", // [781]
    "view_reference", // [782]
    "UsingKeywordSegment", // [783]
    "USING", // [784]
    "CREATED", // [785]
    "CS", // [786]
    "CSV", // [787]
    "CUBE", // [788]
    "function_name_identifier", // [789]
    "ROLLUP", // [790]
    "CubeFunctionNameSegment", // [791]
    "RollupFunctionNameSegment", // [792]
    "GroupingExpressionList", // [793]
    "grouping_expression_list", // [794]
    "CURRENT", // [795]
    "CURRENT_PATH", // [796]
    "CURRENT_ROLE", // [797]
    "CURRENT_TIME", // [798]
    "CURSOR", // [799]
    "DATA", // [800]
    "DATALINK", // [801]
    "data_type_identifier", // [802]
    "DecimalKeywordSegment", // [803]
    "DecKeywordSegment", // [804]
    "NumberKeywordSegment", // [805]
    "NumericKeywordSegment", // [806]
    "BracketedArguments", // [807]
    "bracketed_arguments", // [808]
    "BigintKeywordSegment", // [809]
    "DoubleKeywordSegment", // [810]
    "PrecisionKeywordSegment", // [811]
    "PRECISION", // [812]
    "FloatKeywordSegment", // [813]
    "IntKeywordSegment", // [814]
    "IntegerKeywordSegment", // [815]
    "RealKeywordSegment", // [816]
    "ShortintKeywordSegment", // [817]
    "TinyintKeywordSegment", // [818]
    "SmallintKeywordSegment", // [819]
    "BooleanKeywordSegment", // [820]
    "BoolKeywordSegment", // [821]
    "DateKeywordSegment", // [822]
    "TimestampKeywordSegment", // [823]
    "LocalKeywordSegment", // [824]
    "LOCAL", // [825]
    "TimeKeywordSegment", // [826]
    "TIME", // [827]
    "ZoneKeywordSegment", // [828]
    "ZONE", // [829]
    "IntervalKeywordSegment", // [830]
    "YearKeywordSegment", // [831]
    "YEAR", // [832]
    "MonthKeywordSegment", // [833]
    "MONTH", // [834]
    "DayKeywordSegment", // [835]
    "DAY", // [836]
    "SecondKeywordSegment", // [837]
    "SECOND", // [838]
    "GeometryKeywordSegment", // [839]
    "HashtypeKeywordSegment", // [840]
    "VaryingKeywordSegment", // [841]
    "VARYING", // [842]
    "VarcharKeywordSegment", // [843]
    "Varchar2KeywordSegment", // [844]
    "NcharKeywordSegment", // [845]
    "NvarcharKeywordSegment", // [846]
    "Nvarchar2KeywordSegment", // [847]
    "LongKeywordSegment", // [848]
    "CharacterKeywordSegment", // [849]
    "LARGE", // [850]
    "LargeKeywordSegment", // [851]
    "ClobKeywordSegment", // [852]
    "CharCharacterSetGrammar", // [853]
    "DatePartFunctionName", // [854]
    "DatetimeUnitSegment", // [855]
    "DAYOFYEAR", // [856]
    "HOUR", // [857]
    "MILLISECOND", // [858]
    "MINUTE", // [859]
    "QUARTER", // [860]
    "WEEK", // [861]
    "WEEKDAY", // [862]
    "FunctionContentsGrammar", // [863]
    "date_constructor_literal", // [864]
    "literal", // [865]
    "LiteralSegment", // [866]
    "DATE_TRUNC", // [867]
    "date_part", // [868]
    "DATETIME_INTERVAL_CODE", // [869]
    "DATETIME_INTERVAL_PRECISION", // [870]
    "DAYS_BETWEEN", // [871]
    "DEALLOCATE", // [872]
    "DEBUG", // [873]
    "DECLARE", // [874]
    "DECODE", // [875]
    "ValuesKeywordSegment", // [876]
    "DEFINED", // [877]
    "DEFINER", // [878]
    "DEGREES", // [879]
    "AliasedTableReferenceGrammar", // [880]
    "WhereClauseSegment", // [881]
    "WHERE", // [882]
    "where_clause", // [883]
    "PreferringClauseSegment", // [884]
    "PREFERRING", // [885]
    "preferring_clause", // [886]
    "DELIMITER", // [887]
    "DENSE_RANK", // [888]
    "DEPTH", // [889]
    "DEREF", // [890]
    "DERIVED", // [891]
    "DESC", // [892]
    "DESCRIBE", // [893]
    "DescribeKeywordSegment", // [894]
    "DESCRIPTOR", // [895]
    "DETERMINISTIC", // [896]
    "DIAGNOSTICS", // [897]
    "DISABLED", // [898]
    "DISCONNECT", // [899]
    "DISPATCH", // [900]
    "DISTINCT", // [901]
    "DIV", // [902]
    "DLURLCOMPLETE", // [903]
    "DLURLPATH", // [904]
    "DLURLPATHONLY", // [905]
    "DLURLSCHEME", // [906]
    "DLURLSERVER", // [907]
    "DLVALUE", // [908]
    "DO", // [909]
    "DOMAIN", // [910]
    "dot", // [911]
    "DOWN", // [912]
    "DRIVER", // [913]
    "RESTRICT", // [914]
    "RestrictKeywordSegment", // [915]
    "DropBehaviorGrammar", // [916]
    "TypeKeywordSegment", // [917]
    "TYPE", // [918]
    "DUMP", // [919]
    "DYNAMIC", // [920]
    "DYNAMIC_FUNCTION", // [921]
    "DYNAMIC_FUNCTION_CODE", // [922]
    "EDIT_DISTANCE", // [923]
    "ELSEIF", // [924]
    "ELSIF", // [925]
    "EmitsKeywordSegment", // [926]
    "EMPTY", // [927]
    "StructTypeSegment", // [928]
    "struct_type", // [929]
    "EmptyStructLiteralBracketsSegment", // [930]
    "struct_literal", // [931]
    "ENABLED", // [932]
    "ENCODING", // [933]
    "END-EXEC", // [934]
    "}", // [935]
    "end_curly_bracket", // [936]
    "ENDIF", // [937]
    "ENFORCE", // [938]
    "EQUALS", // [939]
    "RawEqualsSegment", // [940]
    "ERROR", // [941]
    "ERRORS", // [942]
    "ESCAPE", // [943]
    "escaped_identifier", // [944]
    "identifier", // [945]
    "ESTIMATE", // [946]
    "EVALUATE", // [947]
    "EVERY", // [948]
    "EXA", // [949]
    "EXCEPT", // [950]
    "EXCEPTION", // [951]
    "EXCLUDE", // [952]
    "EXEC", // [953]
    "OutputKeywordSegment", // [954]
    "OUTPUT", // [955]
    "EXISTS", // [956]
    "EXIT", // [957]
    "EXP", // [958]
    "EXPERIMENTAL", // [959]
    "EXPLAIN", // [960]
    "ExplainKeywordSegment", // [961]
    "InsertStatementSegment", // [962]
    "insert_statement", // [963]
    "UpdateStatementSegment", // [964]
    "update_statement", // [965]
    "DeleteStatementSegment", // [966]
    "delete_statement", // [967]
    "INTO", // [968]
    "IntoKeywordSegment", // [969]
    "FBV", // [970]
    "JDBC", // [971]
    "ORA", // [972]
    "ImportFromExportIntoDbSrcSegment", // [973]
    "import_export_dbsrc", // [974]
    "ImportFromExportIntoFileSegment", // [975]
    "import_file", // [976]
    "RejectClauseSegment", // [977]
    "REJECT", // [978]
    "reject_clause", // [979]
    "ImportFromExportIntoScriptSegment", // [980]
    "import_script", // [981]
    "ExportIntoClauseSegment", // [982]
    "export_into_clause", // [983]
    "EXPRESSION", // [984]
    "Expression_A_Grammar", // [985]
    "Tail_Recurse_Expression_A_Grammar", // [986]
    "LikeExpressionGrammar", // [987]
    "REGEXP_LIKE", // [988]
    "InOperatorGrammar", // [989]
    "QuantifiedComparisonOperatorGrammar", // [990]
    "IsClauseGrammar", // [991]
    "IsNullGrammar", // [992]
    "NotNullGrammar", // [993]
    "CollateGrammar", // [994]
    "BetweenKeywordSegment", // [995]
    "Expression_B_Grammar", // [996]
    "PatternMatchingGrammar", // [997]
    "PRIOR", // [998]
    "~", // [999]
    "SignedSegmentGrammar", // [1000]
    "QualifiedNumericLiteralSegment", // [1001]
    "TildeSegment", // [1002]
    "NotOperatorGrammar", // [1003]
    "PriorKeywordSegment", // [1004]
    "Tail_Recurse_Expression_B_Grammar", // [1005]
    "ExistsKeywordSegment", // [1006]
    "Expression_D_Grammar", // [1007]
    "CaseExpressionSegment", // [1008]
    "case_expression", // [1009]
    "TimeZoneGrammar", // [1010]
    "time_zone_grammar", // [1011]
    "LateralColumnAliasExpressionGrammar", // [1012]
    "LocalAliasSegment", // [1013]
    "local_alias_segment", // [1014]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1015]
    "SingleIdentifierWildcardSegment", // [1016]
    "wildcard_identifier", // [1017]
    "MapTypeSegment", // [1018]
    "map_type", // [1019]
    "BooleanLiteralGrammar", // [1020]
    "NullLiteralSegment", // [1021]
    "DateTimeLiteralGrammar", // [1022]
    "ListComprehensionGrammar", // [1023]
    "AccessorGrammar", // [1024]
    "SelectStatementSegment", // [1025]
    "select_statement", // [1026]
    "TypedStructLiteralSegment", // [1027]
    "typed_struct_literal", // [1028]
    "ArrayExpressionSegment", // [1029]
    "array_expression", // [1030]
    "OverlapsClauseSegment", // [1031]
    "OVERLAPS", // [1032]
    "overlaps_clause", // [1033]
    "EXTRACT", // [1034]
    "PADDING", // [1035]
    "SIZE", // [1036]
    "SizeKeywordSegment", // [1037]
    "PaddingKeywordSegment", // [1038]
    "AlignKeywordSegment", // [1039]
    "LeftKeywordSegment", // [1040]
    "RightKeywordSegment", // [1041]
    "boolean_literal", // [1042]
    "LiteralKeywordSegment", // [1043]
    "FETCH", // [1044]
    "FetchKeywordSegment", // [1045]
    "NEXT", // [1046]
    "NextKeywordSegment", // [1047]
    "ROWS", // [1048]
    "RowsKeywordSegment", // [1049]
    "ONLY", // [1050]
    "OnlyKeywordSegment", // [1051]
    "TiesKeywordSegment", // [1052]
    "TIES", // [1053]
    "LTRIM", // [1054]
    "RTRIM", // [1055]
    "SKIP", // [1056]
    "TRIM", // [1057]
    "TRUNCATE", // [1058]
    "EncodingKeywordSegment", // [1059]
    "SeparatorKeywordSegment", // [1060]
    "SEPARATOR", // [1061]
    "DelimiterKeywordSegment", // [1062]
    "TrimKeywordSegment", // [1063]
    "LtrimKeywordSegment", // [1064]
    "RtrimKeywordSegment", // [1065]
    "SkipKeywordSegment", // [1066]
    "TruncateKeywordSegment", // [1067]
    "NamesKeywordSegment", // [1068]
    "NAMES", // [1069]
    "FLUSH", // [1070]
    "MERGE", // [1071]
    "OPEN", // [1072]
    "PRELOAD", // [1073]
    "RECOMPRESS", // [1074]
    "REORGANIZE", // [1075]
    "ROLLBACK", // [1076]
    "FunctionScriptStatementSegment", // [1077]
    "statement", // [1078]
    "StatementSegment", // [1079]
    "function_script_terminator", // [1080]
    "FunctionScriptTerminatorSegment", // [1081]
    "FILES", // [1082]
    "FilterKeywordSegment", // [1083]
    "WhereKeywordSegment", // [1084]
    "FINAL", // [1085]
    "FIRST_VALUE", // [1086]
    "FLOOR", // [1087]
    "FlushKeywordSegment", // [1088]
    "StatisticsKeywordSegment", // [1089]
    "STATISTICS", // [1090]
    "FOLLOWING", // [1091]
    "FORALL", // [1092]
    "ForeignKeywordSegment", // [1093]
    "KeyKeywordSegment", // [1094]
    "KEY", // [1095]
    "FORTRAN", // [1096]
    "FOUND", // [1097]
    "RANGE", // [1098]
    "FrameClauseUnitGrammar", // [1099]
    "CurrentKeywordSegment", // [1100]
    "UnboundedKeywordSegment", // [1101]
    "UNBOUNDED", // [1102]
    "PRECEDING", // [1103]
    "PrecedingKeywordSegment", // [1104]
    "FollowingKeywordSegment", // [1105]
    "RangeKeywordSegment", // [1106]
    "FREE", // [1107]
    "FromExpressionSegment", // [1108]
    "from_expression", // [1109]
    "HAVING", // [1110]
    "INTERSECT", // [1111]
    "LIMIT", // [1112]
    "MINUS", // [1113]
    "QUALIFY", // [1114]
    "UNION", // [1115]
    "PreferringKeywordSegment", // [1116]
    "LimitKeywordSegment", // [1117]
    "OrderKeywordSegment", // [1118]
    "HavingKeywordSegment", // [1119]
    "QualifyKeywordSegment", // [1120]
    "SetOperatorSegment", // [1121]
    "set_operator", // [1122]
    "PreTableFunctionKeywordsGrammar", // [1123]
    "TableExpressionSegment", // [1124]
    "table_expression", // [1125]
    "TemporalQuerySegment", // [1126]
    "temporal_query", // [1127]
    "FromClauseTerminatorGrammar", // [1128]
    "SamplingExpressionSegment", // [1129]
    "TABLESAMPLE", // [1130]
    "sample_expression", // [1131]
    "JoinLikeClauseGrammar", // [1132]
    "JoinClauseSegment", // [1133]
    "join_clause", // [1134]
    "OffsetKeywordSegment", // [1135]
    "OFFSET", // [1136]
    "PostTableExpressionGrammar", // [1137]
    "MLTableExpressionSegment", // [1138]
    "ml_table_expression", // [1139]
    "FromExpressionElementSegment", // [1140]
    "from_expression_element", // [1141]
    "indented_joins", // [1142]
    "FROM_POSIX_TIME", // [1143]
    "FS", // [1144]
    "WalrusOperatorSegment", // [1145]
    ":=", // [1146]
    "FunctionAssignmentSegment", // [1147]
    "function_assignment", // [1148]
    "FunctionIfBranchSegment", // [1149]
    "function_if_branch", // [1150]
    "FunctionForLoopSegment", // [1151]
    "function_for_loop", // [1152]
    "FunctionWhileLoopSegment", // [1153]
    "WHILE", // [1154]
    "function_while_loop", // [1155]
    "LEADING", // [1156]
    "TRAILING", // [1157]
    "TrimParametersGrammar", // [1158]
    "DistinctKeywordSegment", // [1159]
    "AggregateOrderByClause", // [1160]
    "aggregate_order_by", // [1161]
    "IgnoreRespectNullsGrammar", // [1162]
    "IGNORE", // [1163]
    "RESPECT", // [1164]
    "EmptyStructLiteralSegment", // [1165]
    "LANGUAGE", // [1166]
    "LanguageKeywordSegment", // [1167]
    "DoKeywordSegment", // [1168]
    "LoopKeywordSegment", // [1169]
    "LOOP", // [1170]
    "IfKeywordSegment", // [1171]
    "ThenKeywordSegment", // [1172]
    "THEN", // [1173]
    "ElsifKeywordSegment", // [1174]
    "ElseifKeywordSegment", // [1175]
    "WordSegment", // [1176]
    "DotSegment", // [1177]
    "QuotedIdentifierSegment", // [1178]
    "FunctionParameterGrammar", // [1179]
    "CreateFunctionStatementSegment", // [1180]
    "create_function_statement", // [1181]
    "CreateScriptingLuaScriptStatementSegment", // [1182]
    "create_scripting_lua_script", // [1183]
    "CreateUDFScriptStatementSegment", // [1184]
    "create_udf_script", // [1185]
    "CreateAdapterScriptStatementSegment", // [1186]
    "create_adapter_script", // [1187]
    "DatePartFunctionNameSegment", // [1188]
    "DateTimeFunctionContentsSegment", // [1189]
    "ColumnsExpressionGrammar", // [1190]
    "ColumnsExpressionFunctionNameSegment", // [1191]
    "ValuesClauseSegment", // [1192]
    "values_clause", // [1193]
    "PostFunctionGrammar", // [1194]
    "WhileKeywordSegment", // [1195]
    "GENERAL", // [1196]
    "GENERATED", // [1197]
    "GET", // [1198]
    "glob_operator", // [1199]
    "GLOBAL", // [1200]
    "GO", // [1201]
    "GOTO", // [1202]
    "AccessPermissionsSegment", // [1203]
    "access_permissions", // [1204]
    "AccessObjectSegment", // [1205]
    "access_object", // [1206]
    "ObjectsKeywordSegment", // [1207]
    "OBJECTS", // [1208]
    "AccessTargetSegment", // [1209]
    "access_target", // [1210]
    "AdminKeywordSegment", // [1211]
    "OptionKeywordSegment", // [1212]
    "OPTION", // [1213]
    "GRANTED", // [1214]
    "GRAPH", // [1215]
    "GREATEST", // [1216]
    "CubeRollupClauseSegment", // [1217]
    "cube_rollup_clause", // [1218]
    "GroupingSetsClauseSegment", // [1219]
    "GROUPING", // [1220]
    "grouping_sets_clause", // [1221]
    "WINDOW", // [1222]
    "LimitClauseSegment", // [1223]
    "limit_clause", // [1224]
    "WindowKeywordSegment", // [1225]
    "GROUP_CONCAT", // [1226]
    "GroupByClauseTerminatorGrammar", // [1227]
    "GroupingKeywordSegment", // [1228]
    "SetsKeywordSegment", // [1229]
    "SETS", // [1230]
    "GROUPING_ID", // [1231]
    "HANDLER", // [1232]
    "HAS", // [1233]
    "HASH", // [1234]
    "HASH_MD5", // [1235]
    "HASH_SHA1", // [1236]
    "HASH_SHA256", // [1237]
    "HASH_SHA512", // [1238]
    "HASH_SHA", // [1239]
    "HASH_TIGER", // [1240]
    "HASHTYPE_MD5", // [1241]
    "HASHTYPE_SHA1", // [1242]
    "HASHTYPE_SHA256", // [1243]
    "HASHTYPE_SHA512", // [1244]
    "HASHTYPE_SHA", // [1245]
    "HASHTYPE_TIGER", // [1246]
    "HIERARCHY", // [1247]
    "HIGH", // [1248]
    "HOLD", // [1249]
    "HOURS_BETWEEN", // [1250]
    "IdentifierSegment", // [1251]
    "IFNULL", // [1252]
    "IgnoreKeywordSegment", // [1253]
    "RespectKeywordSegment", // [1254]
    "NullsKeywordSegment", // [1255]
    "NULLS", // [1256]
    "IMPLEMENTATION", // [1257]
    "CsvKeywordSegment", // [1258]
    "AtKeywordSegment", // [1259]
    "SecureKeywordSegment", // [1260]
    "SECURE", // [1261]
    "ErrorsKeywordSegment", // [1262]
    "ImportErrorDestinationSegment", // [1263]
    "import_error_destination", // [1264]
    "ImportErrorsClauseSegment", // [1265]
    "import_errors_clause", // [1266]
    "ExaKeywordSegment", // [1267]
    "OraKeywordSegment", // [1268]
    "JdbcKeywordSegment", // [1269]
    "DriverKeywordSegment", // [1270]
    "CreatedKeywordSegment", // [1271]
    "FbvKeywordSegment", // [1272]
    "CSVColumnDefinitionSegment", // [1273]
    "csv_cols", // [1274]
    "FBVColumnDefinitionSegment", // [1275]
    "fbv_cols", // [1276]
    "FileOptionSegment", // [1277]
    "file_opts", // [1278]
    "ImportColumnsSegment", // [1279]
    "import_columns", // [1280]
    "ImportFromClauseSegment", // [1281]
    "import_from_clause", // [1282]
    "AscKeywordSegment", // [1283]
    "DescKeywordSegment", // [1284]
    "INDICATOR", // [1285]
    "INITCAP", // [1286]
    "INOUT", // [1287]
    "INPUT", // [1288]
    "INSENSITIVE", // [1289]
    "ValuesRangeClauseSegment", // [1290]
    "values_range_clause", // [1291]
    "INSTANTIABLE", // [1292]
    "INSTR", // [1293]
    "INTEGRITY", // [1294]
    "HourKeywordSegment", // [1295]
    "MinuteKeywordSegment", // [1296]
    "INVALID", // [1297]
    "INVERSE", // [1298]
    "INVOKER", // [1299]
    "IPROC", // [1300]
    "NanLiteralSegment", // [1301]
    "UnknownLiteralSegment", // [1302]
    "NormalizedGrammar", // [1303]
    "IS_BOOLEAN", // [1304]
    "IS_DATE", // [1305]
    "IS_DSINTERVAL", // [1306]
    "IS_NUMBER", // [1307]
    "IS_TIMESTAMP", // [1308]
    "IS_YMINTERVAL", // [1309]
    "ISOLATION", // [1310]
    "ITERATE", // [1311]
    "JAVASCRIPT", // [1312]
    "ConditionalJoinKeywordsGrammar", // [1313]
    "JoinKeywordsGrammar", // [1314]
    "JOIN", // [1315]
    "NestedJoinGrammar", // [1316]
    "indented_using_on", // [1317]
    "MatchConditionSegment", // [1318]
    "match_condition", // [1319]
    "JoinOnConditionSegment", // [1320]
    "join_on_condition", // [1321]
    "JoinUsingConditionGrammar", // [1322]
    "UnconditionalJoinKeywordsGrammar", // [1323]
    "ExtendedNaturalJoinKeywordsGrammar", // [1324]
    "JoinKeywordSegment", // [1325]
    "indented_on_contents", // [1326]
    "InnerKeywordSegment", // [1327]
    "FullKeywordSegment", // [1328]
    "OuterKeywordSegment", // [1329]
    "OUTER", // [1330]
    "JSON", // [1331]
    "JSON_EXTRACT", // [1332]
    "JSON_VALUE", // [1333]
    "KEEP", // [1334]
    "KERBEROS", // [1335]
    "KEY_MEMBER", // [1336]
    "KEY_TYPE", // [1337]
    "Current_sessionKeywordSegment", // [1338]
    "MessageKeywordSegment", // [1339]
    "MESSAGE", // [1340]
    "LAG", // [1341]
    "LAST", // [1342]
    "LAST_VALUE", // [1343]
    "LATERAL", // [1344]
    "LCASE", // [1345]
    "LDAP", // [1346]
    "LEAD", // [1347]
    "LEAST", // [1348]
    "LEAVE", // [1349]
    "LENGTH", // [1350]
    "LikeGrammar", // [1351]
    "EscapeKeywordSegment", // [1352]
    "Regexp_likeKeywordSegment", // [1353]
    "LINK", // [1354]
    "LISTAGG", // [1355]
    "ArrayLiteralSegment", // [1356]
    "array_literal", // [1357]
    "TypedArrayLiteralSegment", // [1358]
    "typed_array_literal", // [1359]
    "ObjectLiteralSegment", // [1360]
    "{", // [1361]
    "object_literal", // [1362]
    "LN", // [1363]
    "LOCALTIME", // [1364]
    "LOCATE", // [1365]
    "LOCATOR", // [1366]
    "LOCK", // [1367]
    "LOG10", // [1368]
    "LOG2", // [1369]
    "LOG", // [1370]
    "LOGS", // [1371]
    "LONGVARCHAR", // [1372]
    "LOW", // [1373]
    "LOWER", // [1374]
    "LPAD", // [1375]
    "MAP", // [1376]
    "MATCH", // [1377]
    "MATCHED", // [1378]
    "MAX", // [1379]
    "MAXIMAL", // [1380]
    "MEDIAN", // [1381]
    "MergeKeywordSegment", // [1382]
    "MergeMatchedClauseSegment", // [1383]
    "merge_when_matched_clause", // [1384]
    "MergeNotMatchedClauseSegment", // [1385]
    "merge_when_not_matched_clause", // [1386]
    "MatchedKeywordSegment", // [1387]
    "MergeUpdateClauseSegment", // [1388]
    "merge_update_clause", // [1389]
    "MergeDeleteClauseSegment", // [1390]
    "merge_delete_clause", // [1391]
    "MergeInsertClauseSegment", // [1392]
    "merge_insert_clause", // [1393]
    "MergeIntoLiteralGrammar", // [1394]
    "MergeMatchSegment", // [1395]
    "merge_match", // [1396]
    "SetClauseListSegment", // [1397]
    "set_clause_list", // [1398]
    "MID", // [1399]
    "MIN", // [1400]
    "MIN_SCALE", // [1401]
    "MINUTES_BETWEEN", // [1402]
    "MOD", // [1403]
    "MODIFIES", // [1404]
    "MODULE", // [1405]
    "MONTHS_BETWEEN", // [1406]
    "MUL", // [1407]
    "MULTIPLE", // [1408]
    "MUMPS", // [1409]
    "naked_identifier", // [1410]
    "WindowSpecificationSegment", // [1411]
    "window_specification", // [1412]
    "NamedWindowExpressionSegment", // [1413]
    "named_window_expression", // [1414]
    "NATIONAL", // [1415]
    "NATURAL", // [1416]
    "NaturalKeywordSegment", // [1417]
    "NCLOB", // [1418]
    "sign_indicator", // [1419]
    "newline", // [1420]
    "NewlineSegment", // [1421]
    "NO", // [1422]
    "NOLOGGING", // [1423]
    "UnorderedSelectStatementSegment", // [1424]
    "WithCompoundStatementSegment", // [1425]
    "with_compound_statement", // [1426]
    "NonSetSelectableGrammar", // [1427]
    "BracketedSetExpressionGrammar", // [1428]
    "MergeStatementSegment", // [1429]
    "merge_statement", // [1430]
    "SetExpressionSegment", // [1431]
    "NONE", // [1432]
    "NORMALIZED", // [1433]
    "RawNotSegment", // [1434]
    "NOTICE", // [1435]
    "NPROC", // [1436]
    "null_literal", // [1437]
    "NULLIF", // [1438]
    "NULLIFZERO", // [1439]
    "NUMTODSINTERVAL", // [1440]
    "NUMTOYMINTERVAL", // [1441]
    "NVL2", // [1442]
    "NVL", // [1443]
    "ColonSegment", // [1444]
    "ObjectLiteralElementSegment", // [1445]
    "object_literal_element", // [1446]
    "start_curly_bracket", // [1447]
    "CastOperatorSegment", // [1448]
    "StartSquareBracketSegment", // [1449]
    "StartBracketSegment", // [1450]
    "OCTET_LENGTH", // [1451]
    "OCTETS", // [1452]
    "OFF", // [1453]
    "OpenKeywordSegment", // [1454]
    "OPENID", // [1455]
    "OPTIMIZE", // [1456]
    "OPTIMIZER", // [1457]
    "LastKeywordSegment", // [1458]
    "WithFillSegment", // [1459]
    "with_fill", // [1460]
    "NoorderKeywordSegment", // [1461]
    "ORDERING", // [1462]
    "ORDINALITY", // [1463]
    "OTHERS", // [1464]
    "OUT", // [1465]
    "OverKeywordSegment", // [1466]
    "OVER", // [1467]
    "OVERFLOW", // [1468]
    "OverlapsKeywordSegment", // [1469]
    "OVERLAY", // [1470]
    "OVERRIDING", // [1471]
    "PAD", // [1472]
    "PARALLEL_ENABLE", // [1473]
    "PARAMETER", // [1474]
    "parameter", // [1475]
    "?", // [1476]
    "PARAMETER_SPECIFIC_CATALOG", // [1477]
    "PARAMETER_SPECIFIC_NAME", // [1478]
    "PARAMETER_SPECIFIC_SCHEMA", // [1479]
    "PARTIAL", // [1480]
    "PASCAL", // [1481]
    "password_literal", // [1482]
    "PATH", // [1483]
    "SlashSegment", // [1484]
    "path_segment", // [1485]
    "PERCENTILE_CONT", // [1486]
    "PERCENTILE_DISC", // [1487]
    "PERMISSION", // [1488]
    "PI", // [1489]
    "pipe", // [1490]
    "PLACING", // [1491]
    "PLI", // [1492]
    "PLUS", // [1493]
    "POSITION", // [1494]
    "POSIX_TIME", // [1495]
    "OverClauseSegment", // [1496]
    "over_clause", // [1497]
    "POWER", // [1498]
    "PreferringPreferenceTermSegment", // [1499]
    "preference_term", // [1500]
    "PartitionClauseSegment", // [1501]
    "partitionby_clause", // [1502]
    "PlusKeywordSegment", // [1503]
    "InverseKeywordSegment", // [1504]
    "HighKeywordSegment", // [1505]
    "LowKeywordSegment", // [1506]
    "PreferringPlusPriorTermSegment", // [1507]
    "plus_prior_inverse", // [1508]
    "PreloadKeywordSegment", // [1509]
    "PREPARE", // [1510]
    "PRESERVE", // [1511]
    "PrimaryKeywordSegment", // [1512]
    "PRINCIPAL", // [1513]
    "SOME", // [1514]
    "SomeKeywordSegment", // [1515]
    "QUERY", // [1516]
    "QUIET", // [1517]
    "quoted_identifier", // [1518]
    "quoted_literal", // [1519]
    "RADIANS", // [1520]
    "RAND", // [1521]
    "RANDOM", // [1522]
    "RANK", // [1523]
    "RATIO_TO_REPORT", // [1524]
    "raw_comparison_operator", // [1525]
    "READ", // [1526]
    "READS", // [1527]
    "RecompressKeywordSegment", // [1528]
    "ReorganizeKeywordSegment", // [1529]
    "EnforceKeywordSegment", // [1530]
    "RECORD", // [1531]
    "RECOVERY", // [1532]
    "RECURSIVE", // [1533]
    "REF", // [1534]
    "ReferenceMatchGrammar", // [1535]
    "ReferentialActionGrammar", // [1536]
    "MatchKeywordSegment", // [1537]
    "SIMPLE", // [1538]
    "PartialKeywordSegment", // [1539]
    "SimpleKeywordSegment", // [1540]
    "NoKeywordSegment", // [1541]
    "ActionKeywordSegment", // [1542]
    "REGEXP_INSTR", // [1543]
    "REGEXP_REPLACE", // [1544]
    "REGEXP_SUBSTR", // [1545]
    "REGR_AVGX", // [1546]
    "REGR_AVGY", // [1547]
    "REGR_COUNT", // [1548]
    "REGR_INTERCEPT", // [1549]
    "REGR_R2", // [1550]
    "REGR_SLOPE", // [1551]
    "REGR_SXX", // [1552]
    "REGR_SXY", // [1553]
    "REGR_SYY", // [1554]
    "RejectKeywordSegment", // [1555]
    "UNLIMITED", // [1556]
    "UnlimitedKeywordSegment", // [1557]
    "RELATIVE", // [1558]
    "RELEASE", // [1559]
    "REPEAT", // [1560]
    "REPEATABLE", // [1561]
    "RESTORE", // [1562]
    "RESULT", // [1563]
    "RETURNED_LENGTH", // [1564]
    "RETURNED_OCTET_LENGTH", // [1565]
    "RETURNING", // [1566]
    "REVERSE", // [1567]
    "RevokeKeywordSegment", // [1568]
    "ROUND", // [1569]
    "ROW_NUMBER", // [1570]
    "ROWTYPE", // [1571]
    "RPAD", // [1572]
    "TablesampleKeywordSegment", // [1573]
    "BernoulliKeywordSegment", // [1574]
    "RepeatableKeywordSegment", // [1575]
    "SAVEPOINT", // [1576]
    "SCHEME", // [1577]
    "SCOPE", // [1578]
    "SCOPE_USER", // [1579]
    "SCROLL", // [1580]
    "SEARCH", // [1581]
    "SECONDS_BETWEEN", // [1582]
    "SECTION", // [1583]
    "SECURITY", // [1584]
    "WildcardExpressionSegment", // [1585]
    "wildcard_expression", // [1586]
    "SelectClauseModifierSegment", // [1587]
    "select_clause_modifier", // [1588]
    "SelectClauseElementSegment", // [1589]
    "InvalidKeywordSegment", // [1590]
    "select_clause_element", // [1591]
    "WithInvalidForeignKeySegment", // [1592]
    "with_invalid_foreign_key_clause", // [1593]
    "WithInvalidUniquePKSegment", // [1594]
    "with_invalid_unique_pk_clause", // [1595]
    "IntoTableSegment", // [1596]
    "into_table_clause", // [1597]
    "SelectClauseTerminatorGrammar", // [1598]
    "SelectClauseSegment", // [1599]
    "select_clause", // [1600]
    "FromClauseSegment", // [1601]
    "from_clause", // [1602]
    "ReferencingClauseSegment", // [1603]
    "referencing_clause", // [1604]
    "ConnectByClauseSegment", // [1605]
    "connect_by_clause", // [1606]
    "GroupByClauseSegment", // [1607]
    "groupby_clause", // [1608]
    "HavingClauseSegment", // [1609]
    "having_clause", // [1610]
    "QualifyClauseSegment", // [1611]
    "qualify_clause", // [1612]
    "WithCompoundNonSelectStatementSegment", // [1613]
    "NonWithSelectableGrammar", // [1614]
    "SELECTIVE", // [1615]
    "SELF", // [1616]
    "statement_terminator", // [1617]
    "SENSITIVE", // [1618]
    "MaxvalueKeywordSegment", // [1619]
    "MinvalueKeywordSegment", // [1620]
    "SERIALIZABLE", // [1621]
    "session_parameter", // [1622]
    "SESSION_PARAMETER", // [1623]
    "SESSION_USER", // [1624]
    "SetClauseSegment", // [1625]
    "set_clause", // [1626]
    "NamedWindowSegment", // [1627]
    "named_window", // [1628]
    "UnionKeywordSegment", // [1629]
    "IntersectKeywordSegment", // [1630]
    "MinusKeywordSegment", // [1631]
    "ExceptKeywordSegment", // [1632]
    "SHUT", // [1633]
    "SIGN", // [1634]
    "PositiveSegment", // [1635]
    "NegativeSegment", // [1636]
    "SIMILAR", // [1637]
    "SIN", // [1638]
    "EscapedIdentifierSegment", // [1639]
    "SINH", // [1640]
    "ArrayTypeSegment", // [1641]
    "array_type", // [1642]
    "slash", // [1643]
    "slice", // [1644]
    "SOUNDEX", // [1645]
    "SOURCE", // [1646]
    "SPACE", // [1647]
    "SPECIFICTYPE", // [1648]
    "SQL", // [1649]
    "SQL_BIGINT", // [1650]
    "SQL_BIT", // [1651]
    "SQL_CHAR", // [1652]
    "SQL_DATE", // [1653]
    "SQL_DECIMAL", // [1654]
    "SQL_DOUBLE", // [1655]
    "SQL_FLOAT", // [1656]
    "SQL_INTEGER", // [1657]
    "SQL_LONGVARCHAR", // [1658]
    "SQL_NUMERIC", // [1659]
    "SQL_REAL", // [1660]
    "SQL_SMALLINT", // [1661]
    "SQL_TIMESTAMP", // [1662]
    "SQL_TINYINT", // [1663]
    "SQL_TYPE_DATE", // [1664]
    "SQL_TYPE_TIMESTAMP", // [1665]
    "SQL_VARCHAR", // [1666]
    "SQLEXCEPTION", // [1667]
    "SQLSTATE", // [1668]
    "SQLWARNING", // [1669]
    "SQRT", // [1670]
    "ST_AREA", // [1671]
    "ST_BOUNDARY", // [1672]
    "ST_BUFFER", // [1673]
    "ST_CENTROID", // [1674]
    "ST_CONTAINS", // [1675]
    "ST_CONVEXHULL", // [1676]
    "ST_CROSSES", // [1677]
    "ST_DIFFERENCE", // [1678]
    "ST_DIMENSION", // [1679]
    "ST_DISJOINT", // [1680]
    "ST_DISTANCE", // [1681]
    "ST_ENDPOINT", // [1682]
    "ST_ENVELOPE", // [1683]
    "ST_EQUALS", // [1684]
    "ST_EXTERIORRING", // [1685]
    "ST_FORCE2D", // [1686]
    "ST_GEOMETRYN", // [1687]
    "ST_GEOMETRYTYPE", // [1688]
    "ST_INTERIORRINGN", // [1689]
    "ST_INTERSECTION", // [1690]
    "ST_INTERSECTS", // [1691]
    "ST_ISCLOSED", // [1692]
    "ST_ISEMPTY", // [1693]
    "ST_ISRING", // [1694]
    "ST_ISSIMPLE", // [1695]
    "ST_LENGTH", // [1696]
    "ST_NUMGEOMETRIES", // [1697]
    "ST_NUMINTERIORRINGS", // [1698]
    "ST_NUMPOINTS", // [1699]
    "ST_OVERLAPS", // [1700]
    "ST_POINTN", // [1701]
    "ST_SETSRID", // [1702]
    "ST_STARTPOINT", // [1703]
    "ST_SYMDIFFERENCE", // [1704]
    "ST_TOUCHES", // [1705]
    "ST_TRANSFORM", // [1706]
    "ST_UNION", // [1707]
    "ST_WITHIN", // [1708]
    "ST_X", // [1709]
    "ST_Y", // [1710]
    "star", // [1711]
    "STATE", // [1712]
    "ExportStatementSegment", // [1713]
    "export_statement", // [1714]
    "ImportStatementSegment", // [1715]
    "import_statement", // [1716]
    "TruncateStatementSegment", // [1717]
    "truncate_table", // [1718]
    "AlterTableStatementSegment", // [1719]
    "alter_table_statement", // [1720]
    "AlterSchemaStatementSegment", // [1721]
    "alter_schema_statement", // [1722]
    "AlterVirtualSchemaStatementSegment", // [1723]
    "alter_virtual_schema_statement", // [1724]
    "CommentStatementSegment", // [1725]
    "comment_statement", // [1726]
    "CreateSchemaStatementSegment", // [1727]
    "create_schema_statement", // [1728]
    "CreateTableStatementSegment", // [1729]
    "create_table_statement", // [1730]
    "CreateViewStatementSegment", // [1731]
    "create_view_statement", // [1732]
    "CreateVirtualSchemaStatementSegment", // [1733]
    "create_virtual_schema_statement", // [1734]
    "DropViewStatementSegment", // [1735]
    "drop_view_statement", // [1736]
    "DropFunctionStatementSegment", // [1737]
    "drop_function_statement", // [1738]
    "DropScriptStatementSegment", // [1739]
    "drop_script_statement", // [1740]
    "DropSchemaStatementSegment", // [1741]
    "drop_schema_statement", // [1742]
    "DropTableStatementSegment", // [1743]
    "drop_table_statement", // [1744]
    "RenameStatementSegment", // [1745]
    "rename_statement", // [1746]
    "AccessStatementSegment", // [1747]
    "access_statement", // [1748]
    "AlterConnectionSegment", // [1749]
    "alter_connection", // [1750]
    "AlterUserStatementSegment", // [1751]
    "alter_user_statement", // [1752]
    "CreateConnectionSegment", // [1753]
    "create_connection", // [1754]
    "CreateRoleStatementSegment", // [1755]
    "create_role_statement", // [1756]
    "CreateUserStatementSegment", // [1757]
    "create_user_statement", // [1758]
    "DropRoleStatementSegment", // [1759]
    "drop_role_statement", // [1760]
    "DropUserStatementSegment", // [1761]
    "drop_user_statement", // [1762]
    "DropConnectionStatementSegment", // [1763]
    "drop_connection_statement", // [1764]
    "CreateConsumerGroupSegment", // [1765]
    "create_consumer_group_statement", // [1766]
    "AlterConsumerGroupSegment", // [1767]
    "alter_consumer_group_statement", // [1768]
    "DropConsumerGroupSegment", // [1769]
    "drop_consumer_group_statement", // [1770]
    "AlterRoleStatementSegment", // [1771]
    "alter_role_statement", // [1772]
    "AlterSessionSegment", // [1773]
    "alter_session_statement", // [1774]
    "AlterSystemSegment", // [1775]
    "alter_system_statement", // [1776]
    "OpenSchemaSegment", // [1777]
    "open_schema_statement", // [1778]
    "CloseSchemaSegment", // [1779]
    "close_schema_statement", // [1780]
    "FlushStatisticsSegment", // [1781]
    "flush_statistics_statement", // [1782]
    "ImpersonateSegment", // [1783]
    "impersonate_statement", // [1784]
    "RecompressReorganizeSegment", // [1785]
    "recompress_reorganize_statement", // [1786]
    "KillSegment", // [1787]
    "kill_statement", // [1788]
    "PreloadSegment", // [1789]
    "preload_statement", // [1790]
    "TruncateAuditLogsSegment", // [1791]
    "truncate_audit_logs_statement", // [1792]
    "ExplainVirtualSegment", // [1793]
    "explain_virtual_statement", // [1794]
    "TransactionStatementSegment", // [1795]
    "transaction_statement", // [1796]
    "ExecuteScriptSegment", // [1797]
    "execute_script_statement", // [1798]
    "STDDEV", // [1799]
    "STDDEV_POP", // [1800]
    "STDDEV_SAMP", // [1801]
    "STEP", // [1802]
    "ConcatSegment", // [1803]
    "STRUCTURE", // [1804]
    "STYLE", // [1805]
    "SUBJECT", // [1806]
    "SUBSTR", // [1807]
    "SUBSTRING", // [1808]
    "SUBTYPE", // [1809]
    "SUM", // [1810]
    "SYMMETRIC", // [1811]
    "SYS_CONNECT_BY_PATH", // [1812]
    "SYS_GUID", // [1813]
    "system_parameter", // [1814]
    "SYSTEM_USER", // [1815]
    "EnableKeywordSegment", // [1816]
    "DisableKeywordSegment", // [1817]
    "ForeignKeyGrammar", // [1818]
    "ReferenceDefinitionGrammar", // [1819]
    "DistributeKeywordSegment", // [1820]
    "TablePartitionByGrammar", // [1821]
    "TableDistributeByGrammar", // [1822]
    "ForeignKeyReferencesClauseGrammar", // [1823]
    "Expression_A_Unary_Operator_Grammar", // [1824]
    "Expression_C_Grammar", // [1825]
    "Expression_B_Unary_Operator_Grammar", // [1826]
    "TAN", // [1827]
    "TANH", // [1828]
    "TempKeywordSegment", // [1829]
    "TemporaryKeywordSegment", // [1830]
    "TEMPORARY", // [1831]
    "TransientKeywordSegment", // [1832]
    "TemporaryGrammar", // [1833]
    "TEXT", // [1834]
    "tilde", // [1835]
    "WITHOUT", // [1836]
    "WithoutKeywordSegment", // [1837]
    "TIMEZONE_HOUR", // [1838]
    "TIMEZONE_MINUTE", // [1839]
    "TO_CHAR", // [1840]
    "TO_DATE", // [1841]
    "TO_DSINTERVAL", // [1842]
    "TO_NUMBER", // [1843]
    "TO_TIMESTAMP", // [1844]
    "TO_YMINTERVAL", // [1845]
    "TRACE", // [1846]
    "TRANSACTION", // [1847]
    "CommitKeywordSegment", // [1848]
    "RollbackKeywordSegment", // [1849]
    "WorkKeywordSegment", // [1850]
    "WORK", // [1851]
    "TRANSFORM", // [1852]
    "TRANSFORMS", // [1853]
    "TRANSLATE", // [1854]
    "TRANSLATION", // [1855]
    "TREAT", // [1856]
    "BothKeywordSegment", // [1857]
    "LeadingKeywordSegment", // [1858]
    "TrailingKeywordSegment", // [1859]
    "TRUNC", // [1860]
    "AuditKeywordSegment", // [1861]
    "LogsKeywordSegment", // [1862]
    "KeepKeywordSegment", // [1863]
    "StructLiteralSegment", // [1864]
    "TYPEOF", // [1865]
    "udf_param_dot_syntax", // [1866]
    "UDFParameterDotSyntaxSegment", // [1867]
    "UCASE", // [1868]
    "UNCOMMITTED", // [1869]
    "NaturalJoinKeywordsGrammar", // [1870]
    "UnconditionalCrossJoinKeywordsGrammar", // [1871]
    "HorizontalJoinKeywordsGrammar", // [1872]
    "UNDER", // [1873]
    "UNDO", // [1874]
    "UNICODE", // [1875]
    "UNICODECHR", // [1876]
    "UNLINK", // [1877]
    "UNNEST", // [1878]
    "UNTIL", // [1879]
    "UPPER", // [1880]
    "USAGE", // [1881]
    "KerberosKeywordSegment", // [1882]
    "PrincipalKeywordSegment", // [1883]
    "LdapKeywordSegment", // [1884]
    "OpenidKeywordSegment", // [1885]
    "SubjectKeywordSegment", // [1886]
    "VALUE2PROC", // [1887]
    "VALUE", // [1888]
    "StepKeywordSegment", // [1889]
    "VAR_POP", // [1890]
    "VAR_SAMP", // [1891]
    "variable", // [1892]
    "VARIANCE", // [1893]
    "VARRAY", // [1894]
    "VERIFY", // [1895]
    "assignment_operator", // [1896]
    "indented_then", // [1897]
    "indented_then_contents", // [1898]
    "WHENEVER", // [1899]
    "whitespace", // [1900]
    "WhitespaceSegment", // [1901]
    "WildcardIdentifierSegment", // [1902]
    "FrameClauseSegment", // [1903]
    "frame_clause", // [1904]
    "RecursiveKeywordSegment", // [1905]
    "indented_ctes", // [1906]
    "CTEDefinitionSegment", // [1907]
    "common_table_expression", // [1908]
    "NonWithNonSelectableGrammar", // [1909]
    "DataKeywordSegment", // [1910]
    "BindingKeywordSegment", // [1911]
    "WITHIN", // [1912]
    "WRITE", // [1913]
    "YEARS_BETWEEN", // [1914]
    "YES", // [1915]
    "ZEROIFNULL", // [1916]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0,
    0, 1, 0, 0, 4294967295, 4294967295, 0, 149, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 150, 1, 2, 1, 1, 4, 3, 1, 4, 5, 151, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 152, 1, 2, 1, 1, 4, 3, 1, 4, 5, 153,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 154, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 155, 1, 2, 1, 1, 4, 3, 1, 4, 5, 156, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 157, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 159, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 161,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 171, 174, 175, 1, 174, 3, 3,
    4, 176, 177, 178, 175, 1, 178, 3, 3, 4, 176, 1, 2, 1, 0, 187,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 36, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 188, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 19,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 2, 1, 0, 1, 2, 1, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 4, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 0, 0, 412,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 413, 414, 175, 1, 414, 3, 3,
    4, 176, 415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 326, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 326, 416, 417, 1, 416, 3, 3, 416, 4, 418,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 43, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 419, 1, 2, 1, 1, 4, 3, 1, 4, 5, 420, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 421, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 0, 146, 444, 175, 1, 444, 3, 3, 4, 176, 445, 446,
    175, 1, 446, 3, 3, 4, 176, 1, 2, 0, 447, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 0, 169, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 449, 1, 2, 1, 1, 4, 3, 1, 4, 5, 450, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 453, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 454, 1, 2, 1, 1, 4, 3, 1, 4, 5, 455,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 388, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 457, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 458, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 411, 1, 2, 1, 1, 4, 3, 1, 4, 5, 459, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 461,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 462, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 465, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 466, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 275, 276, 277, 278, 279,
    280, 281, 282, 283, 284, 285, 286, 287, 288, 289, 290, 291, 292, 293, 69, 651,
    20, 468, 469, 1, 468, 2, 3, 4, 0, 474, 475, 0, 2, 3, 474, 3,
    476, 0, 1, 3, 477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 478, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 479, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 480, 1, 2, 1, 1, 4, 3, 1, 4, 5, 335, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 481, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 416, 417, 9, 3, 3, 416, 4, 490, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 492, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 493, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 495, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 496, 1, 2, 1, 1, 4, 3, 1, 4, 5, 497, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 498, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 500, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 501, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    502, 1, 2, 1, 1, 4, 3, 1, 4, 5, 428, 416, 175, 1, 416, 3,
    3, 4, 176, 507, 1, 2, 1, 1, 4, 3, 1, 4, 5, 508, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 336, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 517, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 2, 3, 1, 1, 0, 1,
    2, 1, 522, 523, 0, 2, 3, 522, 1, 2, 1, 527, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 222, 1, 2, 1, 1, 4, 3, 1, 4, 5, 518,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 1, 2, 1,
    1, 2, 1, 1, 2, 1, 540, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 542, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 330, 1, 2, 1, 1, 4, 3, 1, 4, 5, 543, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0,
    0, 544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 558, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 560,
    561, 175, 1, 561, 3, 3, 4, 176, 562, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 564, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 565, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    210, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 338, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 339, 1, 2, 1, 1, 4, 3, 1, 4, 5, 569,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 570, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 571, 1, 2, 1, 1, 4, 3, 1, 4, 5, 572, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 573, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 575, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 577,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 578, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 579, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 581, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 469, 9, 2, 3,
    4, 583, 1, 2, 1, 1, 4, 3, 1, 4, 5, 584, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 0, 588, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 589, 1, 2, 1, 1, 4, 3, 1, 4, 5, 590, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 591, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    443, 592, 175, 1, 592, 3, 3, 4, 176, 443, 593, 175, 1, 593, 3, 3,
    4, 176, 443, 594, 175, 1, 594, 3, 3, 4, 176, 0, 0, 1, 2, 1,
    0, 0, 264, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 0,
    98, 603, 175, 1, 603, 3, 3, 4, 176, 366, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 606, 607, 10, 3, 3, 606, 4, 1, 0, 1, 2, 1, 0,
    0, 608, 1, 2, 1, 1, 4, 3, 1, 4, 5, 609, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 610, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    205, 618, 9, 3, 3, 205, 4, 416, 619, 0, 2, 3, 416, 205, 620, 0,
    2, 3, 205, 621, 1, 2, 1, 1, 4, 3, 1, 4, 5, 622, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 0, 0, 632, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 275, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 277, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 78, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    637, 1, 2, 1, 1, 4, 3, 1, 4, 5, 307, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 333, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 638, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 51, 1, 2, 1, 1, 4, 3, 1, 4, 5, 201, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 646, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 648, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 649, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 651, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 653, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 654, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 656, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 657, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 192, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 3, 4, 1, 0, 0, 1, 0, 1, 0, 1, 2,
    1, 0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 64,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 1, 0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0,
    1, 0, 0, 1, 0, 0, 0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0,
    0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 0,
    4294967295, 4294967295, 0, 785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 624, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 786, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 788, 789, 469, 1, 789,
    2, 3, 4, 788, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2,
    1, 278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 795, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 796,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 797, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 281, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 282, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 798, 1, 2, 1, 1, 4, 3, 1, 4, 5, 283, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 284, 1, 2, 1, 1, 4, 3, 1, 4, 5, 799,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 227, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 687, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 801, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 802, 469, 1, 802, 2, 3, 4, 0, 0, 0, 0, 0, 0,
    0, 0, 341, 1, 2, 1, 1, 4, 3, 1, 4, 5, 153, 154, 155, 156,
    157, 158, 159, 2172, 7, 789, 469, 1, 789, 2, 3, 4, 0, 1, 0, 1,
    2, 1, 0, 180, 864, 866, 3, 864, 865, 180, 3, 3, 865, 4, 867, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 836, 856, 857, 858, 859, 834, 860, 838,
    861, 862, 832, 2216, 11, 868, 469, 1, 868, 2, 3, 4, 869, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 870, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    836, 1, 2, 1, 1, 4, 3, 1, 4, 5, 871, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 872, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 873, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 342, 1, 2, 1, 1, 4, 3, 1, 4, 5, 343, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 874, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    875, 1, 2, 1, 1, 4, 3, 1, 4, 5, 269, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 238, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 729, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 762, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 877, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 879, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 27, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 530,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 887, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 888, 1, 2, 1, 1, 4, 3, 1, 4, 5, 889, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 890, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 891, 1, 2, 1, 1, 4, 3, 1, 4, 5, 892, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 893, 1, 2, 1, 1, 4, 3, 1, 4, 5, 895,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 896, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 84, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 315, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 898, 1, 2, 1, 1, 4, 3, 1, 4, 5, 899, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 900, 1, 2, 1, 1, 4, 3, 1, 4, 5, 901,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 319, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 902, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 425, 416, 175, 1, 416, 3, 3, 4, 176,
    903, 1, 2, 1, 1, 4, 3, 1, 4, 5, 904, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 905, 1, 2, 1, 1, 4, 3, 1, 4, 5, 906, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 907, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 908, 1, 2, 1, 1, 4, 3, 1, 4, 5, 909, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 910, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    586, 911, 175, 1, 911, 3, 3, 4, 176, 344, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 912, 1, 2, 1, 1, 4, 3, 1, 4, 5, 913, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 3, 4, 1, 65, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 919, 1, 2, 1, 1, 4, 3, 1, 4, 5, 920,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 921, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 922, 1, 2, 1, 1, 4, 3, 1, 4, 5, 764, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 923, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 552, 1, 2, 1, 1, 4, 3, 1, 4, 5, 924, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 776,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 927, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 1, 316, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 932, 1, 2, 1, 1, 4, 3, 1, 4, 5, 933, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 934, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 935, 936, 175, 1, 936, 3, 3, 4, 176, 554, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 937, 1, 2, 1, 1, 4, 3, 1, 4, 5, 938, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 939, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 941, 1, 2, 1, 1, 4, 3, 1, 4, 5, 942, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 943, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    944, 945, 175, 3, 945, 176, 944, 3, 3, 4, 176, 946, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 947, 1, 2, 1, 1, 4, 3, 1, 4, 5, 948,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 949, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 950, 1, 2, 1, 1, 4, 3, 1, 4, 5, 951, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 952, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 724, 1, 2, 1, 1, 4, 3, 1, 4, 5, 953, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 1, 2, 1, 956, 1, 2, 1, 1, 4, 3, 1, 4, 5, 957,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 958, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 959, 1, 2, 1, 1, 4, 3, 1, 4, 5, 404, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 960, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 35, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 1, 2, 1, 0, 984, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0,
    0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 1,
    2, 1, 0, 0, 0, 120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1034,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    1, 0, 1, 2, 1, 407, 1, 2, 1, 1, 4, 3, 1, 4, 5, 511,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 511, 1042, 1043, 1, 1042, 5, 3,
    1, 865, 4, 5, 970, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 1044, 1, 2, 1, 1, 4, 3, 1, 4, 5, 122, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    0, 1, 0, 1082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    1085, 1, 2, 1, 1, 4, 3, 1, 4, 5, 370, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4, 5, 345, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1087, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1070, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1091, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1092, 1, 2, 1, 1, 4, 3, 1, 4, 5, 780, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 308, 1, 2, 1, 1, 4, 3, 1, 4, 5, 125, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1096, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1097, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0,
    0, 1107, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2,
    1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 165,
    1142, 1, 0, 0, 4294967295, 4294967295, 0, 181, 1142, 1, 1, 2, 1, 0, 102, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1143, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1144, 1, 2, 1, 1, 4, 3, 1, 4, 5, 626, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 1, 2, 1, 1, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 0, 0, 4294967295,
    4294967295, 0, 1, 4294967295, 4294967295, 0, 0, 1, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1,
    4294967295, 4294967295, 0, 8, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 789, 1176,
    2, 789, 5, 3, 3, 4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 1080, 1080, 175, 3, 1080, 176, 1080, 3, 3, 4, 176,
    0, 0, 1, 4294967295, 4294967295, 0, 1196, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1197, 1, 2, 1, 1, 4, 3, 1, 4, 5, 346, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1198, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1199, 1199,
    618, 3, 1199, 205, 1199, 3, 3, 205, 4, 1200, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1201, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1202, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 40, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1215, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1216, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 1, 0, 1, 0, 0, 53, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 193, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1220, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1231, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 57, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1232,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1233, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1234, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1235, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1236, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1238, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1240,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 347, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 239, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1241, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1242, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1243, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1244, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1245, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1246,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 1110, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1247, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1248, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1249, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 857, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 384, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 945, 1251, 9, 3, 3, 945, 4, 270, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 194, 1, 2, 1, 1, 4, 3, 1, 4, 5, 266,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1252, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 760, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 70, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 85, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1257, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1, 2, 1, 0, 0, 0, 0, 1,
    2, 1, 0, 0, 2, 4294967295, 0, 1, 4294967295, 4294967295, 0, 0, 0, 1, 4294967295, 4294967295,
    0, 1, 4294967295, 4294967295, 0, 0, 1, 4294967295, 4294967295, 0, 0, 0, 1, 4294967295, 4294967295, 0,
    33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 0, 1,
    2, 1, 0, 693, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0,
    1, 2, 1, 0, 725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 707,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1285, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1286, 1, 2, 1, 1, 4, 3, 1, 4, 5, 747, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 627, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1288, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1289, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 674, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1292, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1293, 1, 2, 1, 1, 4, 3, 1, 4, 5, 348, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1294,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1111, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 1, 2, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 0, 350, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 968, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1297, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1298, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1299, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1300, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1304,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1305, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1307, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1308, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1310, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 661,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1312, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 971, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295,
    0, 165, 1317, 1, 0, 181, 1317, 1, 0, 1315, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 546, 1326, 1, 1, 2, 1, 0, 181, 1326, 1, 0, 0, 1,
    0, 1, 2, 1, 1331, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1332, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1333, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1334, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1335, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1095, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1336, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1337, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 329, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    9, 4, 3, 1, 4, 5, 58, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 1341, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1166, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 850, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1342, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1343, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1344, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1345, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1346, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1347, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1156, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1348, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 628, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 286, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 722, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 485, 485, 618, 3, 485, 205, 485, 3, 3, 205, 4, 0, 1112,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1354, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 865, 1043,
    9, 5, 3, 1, 865, 4, 5, 865, 866, 9, 3, 3, 865, 4, 1363, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 825, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1364, 1, 2, 1, 1, 4, 3, 1, 4, 5, 287, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1365, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1366, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1367, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1369, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1370, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 409, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1371, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 351, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1170, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1373, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1374, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1375, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1054, 1, 2, 1, 1, 4, 3, 1, 4, 5, 665, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 54, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1377, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1378, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1379, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1071, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 2, 1, 0, 165, 1317, 1,
    181, 1317, 1, 1340, 1, 2, 1, 1, 4, 3, 1, 4, 5, 675, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1399, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1401, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1113, 1, 2, 1, 1, 4, 3, 1, 4, 5, 424,
    416, 175, 1, 416, 3, 3, 4, 176, 859, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1402, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1403, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1404, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    294, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1405, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 422, 416, 175, 1, 416, 3, 3, 4, 176, 834, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1406, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1407, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1408, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 213, 416, 175, 1, 416, 3, 3, 4, 176, 1409, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2, 3, 1410, 1251, 1, 1410, 3, 3,
    945, 4, 1, 2, 1, 0, 1, 0, 1069, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1416, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 352, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1418, 1, 2, 1, 1, 4, 3, 1, 4, 5, 424, 1419, 175, 1, 1419, 3,
    3, 4, 176, 532, 1, 2, 1, 1, 4, 3, 1, 4, 5, 755, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1420, 1421, 12, 3, 3, 1420, 4, 1046, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 240, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 242, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 243, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    244, 1, 2, 1, 1, 4, 3, 1, 4, 5, 245, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1422, 1, 2, 1, 1, 4, 3, 1, 4, 5, 228, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1423, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 1, 2, 1, 1, 2, 1, 0, 0, 1, 2, 1,
    0, 0, 1432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1433, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 367, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 367, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1435, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1436,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 207, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 207, 1437, 1043, 1, 1437, 5, 3, 1, 865, 4, 5, 1438, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1439, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1256, 1, 2, 1, 1, 4, 3, 1, 4, 5, 353, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 354, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    214, 214, 866, 3, 214, 865, 214, 3, 3, 865, 4, 1440, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1441, 1, 2, 1, 1, 4, 3, 1, 4, 5, 356,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 355, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1442, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1443, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 45, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1361, 1447, 175, 1, 1447, 3, 3, 4, 176, 1, 2, 0, 0,
    0, 1208, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1451, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1452, 1, 2, 1, 1, 4, 3, 1, 4, 5, 744,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1453, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 751,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 88, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1050, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1072, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1455, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1457, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 484, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 484, 416, 417, 1, 416, 3, 3, 416, 4, 972, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 163, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1462, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1464, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1330,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 955, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 1467, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1468, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 1032,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1470, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1471, 1, 2, 1, 1, 4, 3, 1, 4, 5, 219, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1035, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1473, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1474, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4,
    4294967295, 1475, 469, 1, 1475, 2, 3, 4, 1476, 1475, 175, 1, 1475, 3, 3, 4,
    176, 1477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1478, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1480,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 320,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1481, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 385, 1, 2, 1, 1, 4, 3, 1, 4, 5, 394, 1482, 469,
    3, 1482, 4, 394, 2, 3, 4, 259, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1483, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 5, 1485, 1176, 2, 1485, 5, 3, 3, 4, 5, 1,
    0, 0, 1486, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1487, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1488, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1489, 1, 2, 1, 1, 4, 3, 1, 4, 5, 429, 1490, 175, 1, 1490, 3,
    3, 4, 176, 1491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1492, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1493, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 423, 416, 175, 1, 416, 3, 3, 4, 176, 1494, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 423, 1419, 175, 1, 1419, 3, 3, 4, 176, 1495, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 1498, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1103, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 812, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 0, 0, 0, 1073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 0, 0, 1510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1511, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 309, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1513, 1, 2, 1, 1, 4, 3, 1, 4, 5, 998, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 47, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    39, 1, 2, 1, 1, 4, 3, 1, 4, 5, 110, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 663, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1114, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 2, 1, 1516, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    247, 1, 2, 1, 1, 4, 3, 1, 4, 5, 196, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 394, 1518,
    1251, 3, 1518, 945, 394, 3, 3, 945, 4, 180, 1519, 866, 3, 1519, 865, 180,
    3, 3, 865, 4, 772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1520, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1521, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1522, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1098, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 529, 529, 175, 3, 529, 176, 529, 3, 3, 4,
    176, 1523, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1524, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 204, 1525, 175, 1, 1525, 3, 3, 4, 176, 427, 1525,
    175, 1, 1525, 3, 3, 4, 176, 426, 1525, 175, 1, 1525, 3, 3, 4, 176,
    482, 1525, 175, 1, 1525, 3, 3, 4, 176, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1526, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1527, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 357, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1074, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 0,
    0, 1531, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1532, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1533, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1534,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 29, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 748, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 82, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1543, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 988, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1544, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1545, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1547,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1548, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1550, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1551, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1552, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1553, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1558, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 295, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1075, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1560, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 391, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 386, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1562, 1, 2, 1, 1, 4, 3, 1, 4, 5, 914,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1563, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 695, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1564, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1565, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1566, 1, 2, 1, 1, 4, 3, 1, 4, 5, 716, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 141,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 629, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 73, 1, 2, 1, 1, 4, 3, 1, 4, 5, 94, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1076, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 790, 789, 469, 1, 789, 2, 3, 4, 790, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1569, 1, 2, 1, 1, 4, 3, 1, 4, 5, 112, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 753, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1570, 1, 2, 1, 1, 4, 3, 1, 4, 5, 718, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 289, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1048, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1571, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1055, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1,
    1576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 773, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 9, 1, 2, 1, 1, 4, 3, 1, 4, 5, 138, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1577, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1578, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1579, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    248, 1, 2, 1, 1, 4, 3, 1, 4, 5, 249, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1580, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1581, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 838, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1261, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0,
    2, 7845, 2, 7847, 3, 483, 901, 483, 367, 901, 0, 21, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1,
    0, 1615, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1616, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 698, 1617, 175, 1, 1617, 3, 3, 4, 176, 1618, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1061, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 114, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1621, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 61, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 237, 238, 239, 194, 240, 241, 242, 243, 244, 245, 246, 247, 196, 248,
    249, 197, 250, 251, 252, 253, 254, 255, 7954, 22, 1622, 469, 1, 1622, 2, 3,
    4, 1623, 1, 2, 1, 1, 4, 3, 1, 4, 5, 197, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1624, 1, 2, 1, 1, 4, 3, 1, 4, 5, 291,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 4294967295, 4294967295, 0,
    48, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1230, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 358, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1633, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1634,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1637, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1538, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1638, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 180, 1518, 1251, 3, 1518,
    945, 180, 3, 3, 945, 4, 1640, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1056, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 425, 1643, 175, 1, 1643, 3, 3, 4, 176, 443, 1644, 175,
    1, 1644, 3, 3, 4, 176, 359, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1514, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1645, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1646, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1647, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 673, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1648, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1649, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1651, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1653, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1654, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1656, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1657, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1659, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 251, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1661, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1662, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1663, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1664, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1666, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1667, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1668, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1669, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1670, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1671, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1672, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1673, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1674, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1676, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1677, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1678, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1679, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1680, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1682, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1683, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1684, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1685, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1686, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1687, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1688, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1690, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1691, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1692, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1693, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1694, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1695, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1696, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1697, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1698, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1699, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1700, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1701, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1702, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1703, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1704, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1705, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1707, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1708, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1710, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 213, 1711, 175, 1, 1711, 3, 3, 4, 176, 633,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1712, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 676, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1090, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1799, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1800, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1801, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1802, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1,
    1804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1805, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1807, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1808, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1809, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 176, 175, 9, 3, 3, 4, 176, 1811, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1812, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1813, 1, 2, 1, 1, 4, 3, 1, 4, 5, 292, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 63, 1, 2, 1, 1, 4, 3, 1, 4, 5, 237,
    258, 238, 239, 194, 241, 242, 243, 244, 245, 259, 260, 246, 247, 196, 248, 249,
    197, 250, 251, 252, 261, 253, 254, 255, 198, 9135, 26, 1814, 469, 1, 1814, 2,
    3, 4, 1815, 1, 2, 1, 1, 4, 3, 1, 4, 5, 293, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0,
    0, 0, 11, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0,
    127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1130, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1827, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1828, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 261, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1831, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1834, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1173, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1053, 1, 2, 1, 1, 4, 3, 1, 4, 5, 999, 1835,
    175, 1, 1835, 3, 3, 4, 176, 827, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 4294967295, 4294967295, 0, 254, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 255, 1, 2, 1, 1, 4, 3, 1, 4, 5, 360, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1838,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1839, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 361, 1, 2, 1, 1, 4, 3, 1, 4, 5, 100, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1840, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1842, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1843, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1844,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1845, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1846, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1157, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1847, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1853, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1854, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1856, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 738, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1057, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 512, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 512, 1042, 1043, 1, 1042, 5, 3, 1, 865, 4, 5, 1860, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1058, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 1, 918, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1866, 945, 175,
    3, 945, 176, 1866, 3, 3, 4, 176, 1, 0, 0, 1868, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1869,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1873, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1874, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1875, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1876, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 705, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 513, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 513, 1042, 1043, 1, 1042, 5, 3, 1, 865, 4, 5, 1556, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1877, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 1879, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 25, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1880, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1881, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 69, 1, 2, 1, 1, 4, 3, 1, 4, 5, 198, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 784, 1, 2, 1, 1, 4, 3, 1, 4, 5, 566,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1887, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1888, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0,
    1, 2, 1, 0, 1, 0, 0, 1, 0, 525, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 1890, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1891,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 363, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 362, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 4294967295, 1892,
    469, 1, 1892, 2, 3, 4, 1893, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 842, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 12, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 80, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1146, 1896, 175, 1, 1896, 3, 3, 4, 176, 861, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 165, 1897, 1, 546, 1898, 1, 181, 1898, 1, 181, 1897,
    1, 548, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1899, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 882, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1154, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1900, 1901,
    12, 3, 3, 4, 1900, 0, 0, 4294967295, 4294967295, 0, 1222, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 165, 1906, 1, 1, 0, 181, 1906, 1, 165, 1906, 1,
    1, 0, 181, 1906, 1, 0, 539, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1912, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1836, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 1176, 9, 3, 3, 4, 5, 1851, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 832,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1914, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1915, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1916, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 829, 1, 2, 1, 1, 4, 3, 1, 4,
    5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 13, 14, 15, 16, 17, 43, 18, 20, 22, 24, 26, 28,
    30, 32, 34, 0, 37, 38, 0, 41, 42, 44, 46, 0, 41, 42, 46, 0,
    49, 42, 50, 52, 0, 55, 50, 56, 0, 59, 42, 60, 0, 18, 62, 0,
    31, 66, 18, 67, 68, 0, 71, 42, 68, 0, 32, 67, 41, 42, 72, 0,
    33, 18, 67, 41, 75, 76, 42, 77, 0, 66, 79, 14, 0, 34, 18, 67,
    75, 42, 79, 14, 81, 0, 66, 35, 16, 17, 77, 72, 60, 13, 15, 0,
    36, 66, 18, 26, 67, 22, 20, 24, 42, 16, 0, 20, 42, 83, 0, 37,
    66, 67, 42, 17, 0, 38, 66, 67, 30, 42, 39, 15, 13, 32, 34, 0,
    86, 87, 89, 0, 77, 92, 0, 37, 93, 40, 95, 97, 99, 101, 0, 76,
    87, 77, 103, 0, 105, 42, 15, 14, 103, 44, 106, 87, 46, 16, 17, 108,
    13, 109, 111, 113, 115, 116, 14, 0, 117, 118, 0, 119, 17, 0, 121, 16,
    0, 123, 124, 47, 126, 128, 129, 130, 131, 132, 133, 134, 135, 137, 0, 117,
    139, 0, 119, 128, 0, 121, 126, 0, 123, 140, 48, 142, 144, 49, 92, 51,
    147, 55, 65, 75, 85, 95, 105, 115, 125, 135, 145, 155, 165, 175, 162, 0,
    165, 168, 206, 0, 103, 203, 172, 185, 194, 179, 181, 0, 183, 185, 207, 217,
    227, 0, 18, 77, 189, 99, 190, 0, 18, 50, 52, 103, 49, 237, 199, 239,
    0, 18, 72, 95, 49, 0, 202, 203, 249, 103, 206, 0, 18, 14, 208, 254,
    0, 49, 211, 203, 250, 215, 216, 0, 217, 218, 103, 257, 0, 220, 221, 215,
    223, 224, 255, 0, 225, 215, 226, 256, 229, 230, 231, 0, 18, 113, 232, 258,
    234, 0, 18, 60, 49, 236, 203, 262, 256, 215, 0, 18, 62, 49, 257, 203,
    263, 256, 215, 0, 262, 263, 265, 267, 264, 267, 0, 18, 263, 103, 275, 0,
    49, 273, 0, 271, 271, 268, 215, 0, 272, 272, 273, 274, 0, 67, 274, 271,
    272, 0, 18, 16, 183, 276, 296, 298, 300, 302, 304, 0, 18, 16, 183, 279,
    0, 262, 306, 0, 311, 277, 0, 312, 103, 313, 314, 0, 67, 278, 0, 312,
    317, 103, 313, 0, 318, 312, 103, 99, 103, 0, 18, 16, 183, 281, 321, 0,
    67, 280, 0, 324, 325, 327, 0, 327, 325, 324, 328, 0, 67, 263, 317, 103,
    0, 67, 263, 317, 103, 0, 331, 332, 0, 311, 263, 285, 282, 103, 334, 365,
    0, 291, 0, 369, 203, 286, 273, 189, 0, 287, 262, 311, 263, 267, 289, 0,
    288, 371, 372, 373, 375, 376, 0, 318, 290, 377, 99, 183, 0, 318, 263, 103,
    99, 103, 292, 378, 380, 382, 0, 18, 68, 95, 295, 0, 387, 293, 0, 389,
    0, 392, 393, 395, 397, 399, 0, 401, 203, 256, 0, 402, 403, 0, 405, 406,
    408, 410, 0, 49, 202, 203, 294, 103, 206, 0, 18, 79, 14, 208, 302, 0,
    49, 296, 0, 373, 203, 273, 0, 81, 0, 126, 300, 183, 0, 217, 218, 103,
    303, 313, 322, 332, 342, 351, 361, 371, 381, 391, 401, 430, 431, 432, 433, 434,
    435, 436, 437, 438, 439, 423, 403, 402, 215, 440, 442, 405, 414, 0, 426, 438,
    436, 448, 0, 0, 377, 441, 451, 461, 471, 481, 491, 501, 511, 521, 531, 541,
    551, 561, 571, 581, 591, 601, 611, 0, 464, 621, 631, 641, 671, 680, 273, 274,
    470, 472, 373, 440, 0, 334, 273, 97, 377, 681, 687, 692, 702, 712, 722, 732,
    742, 752, 486, 487, 488, 489, 753, 760, 770, 780, 790, 800, 810, 820, 830, 840,
    850, 860, 870, 880, 503, 0, 504, 504, 505, 0, 506, 506, 890, 899, 909, 919,
    929, 509, 510, 930, 940, 514, 515, 516, 941, 954, 951, 215, 953, 519, 520, 521,
    959, 957, 373, 962, 968, 524, 971, 981, 991, 1005, 1003, 0, 1001, 215, 0, 215,
    528, 215, 0, 124, 203, 256, 0, 531, 203, 1002, 533, 534, 535, 1008, 172, 0,
    103, 536, 377, 1011, 538, 1014, 1024, 1034, 1044, 1054, 1072, 0, 545, 546, 1064, 549,
    551, 553, 555, 553, 553, 0, 545, 440, 1068, 549, 551, 553, 555, 553, 553, 489,
    97, 557, 1073, 1083, 1093, 1103, 1112, 1122, 1132, 1142, 1152, 1162, 567, 568, 1163, 1173,
    1183, 1193, 1203, 1213, 1223, 1233, 1243, 1253, 1263, 1273, 1283, 1293, 0, 580, 14, 1303,
    1313, 1323, 0, 1329, 1339, 1351, 256, 1349, 103, 585, 587, 1352, 1362, 1372, 1382, 1392,
    1401, 1410, 1419, 595, 273, 472, 274, 0, 1425, 0, 272, 1420, 273, 274, 0, 271,
    1424, 1421, 215, 597, 598, 0, 103, 334, 0, 600, 365, 0, 1426, 1436, 373, 1438,
    103, 585, 587, 0, 602, 0, 0, 1440, 0, 604, 605, 256, 1449, 1459, 0, 604,
    87, 1472, 0, 16, 183, 0, 605, 256, 1468, 1466, 0, 103, 605, 256, 0, 1471,
    263, 14, 13, 15, 68, 72, 77, 0, 50, 52, 92, 605, 256, 1473, 1483, 1493,
    1503, 203, 611, 612, 613, 614, 615, 616, 617, 1504, 1511, 1517, 1523, 0, 505, 505,
    1533, 623, 1543, 625, 630, 631, 1548, 0, 634, 221, 230, 1544, 440, 325, 635, 0,
    635, 636, 440, 0, 635, 636, 440, 634, 221, 230, 1546, 440, 325, 1549, 1559, 1569,
    1579, 0, 1589, 103, 256, 0, 68, 256, 387, 221, 256, 1590, 1600, 1610, 1620, 1630,
    1640, 0, 1650, 639, 640, 641, 642, 643, 644, 645, 203, 1651, 256, 215, 1652, 1662,
    1672, 1682, 1692, 1702, 1712, 1722, 1732, 1742, 1752, 1762, 1772, 1782, 1792, 1802, 0, 66,
    659, 1812, 660, 662, 664, 103, 666, 15, 667, 377, 669, 0, 66, 671, 1813, 334,
    377, 334, 636, 672, 1817, 111, 13, 109, 0, 1816, 677, 678, 679, 680, 681, 683,
    0, 105, 92, 0, 377, 685, 0, 66, 659, 77, 189, 99, 190, 0, 66, 50,
    52, 103, 636, 1818, 199, 0, 66, 686, 265, 688, 0, 66, 659, 13, 690, 1822,
    1820, 0, 103, 692, 334, 694, 334, 1825, 605, 377, 1826, 0, 696, 334, 697, 699,
    1830, 700, 694, 702, 697, 553, 690, 703, 0, 66, 659, 704, 706, 265, 708, 87,
    183, 0, 1836, 1834, 710, 1839, 0, 66, 659, 712, 265, 92, 0, 714, 1857, 1855,
    0, 369, 203, 1854, 273, 1851, 1849, 256, 377, 538, 0, 66, 72, 95, 0, 66,
    14, 265, 208, 0, 66, 659, 664, 15, 667, 1862, 1860, 0, 715, 103, 0, 717,
    1865, 16, 719, 377, 669, 1868, 0, 220, 221, 215, 0, 635, 636, 215, 223, 224,
    1866, 0, 225, 215, 226, 1867, 229, 230, 231, 0, 66, 113, 232, 1869, 720, 0,
    723, 183, 1875, 1873, 0, 103, 185, 0, 1878, 726, 727, 728, 0, 1879, 726, 727,
    271, 0, 1880, 726, 727, 730, 0, 66, 659, 16, 265, 183, 1886, 1883, 0, 1881,
    731, 0, 97, 321, 0, 377, 538, 733, 735, 598, 0, 66, 737, 739, 1887, 741,
    372, 0, 742, 743, 1891, 1890, 22, 26, 0, 24, 743, 1888, 373, 745, 87, 745,
    87, 87, 183, 1899, 0, 749, 750, 752, 377, 369, 754, 752, 377, 369, 0, 101,
    183, 1894, 0, 756, 757, 0, 757, 1893, 0, 758, 759, 0, 758, 761, 0, 105,
    763, 1895, 752, 766, 0, 767, 1896, 440, 0, 30, 109, 768, 769, 0, 66, 659,
    1903, 660, 662, 664, 771, 103, 1904, 774, 49, 15, 667, 1905, 0, 775, 162, 1908,
    0, 717, 334, 777, 377, 669, 0, 66, 68, 95, 387, 1909, 389, 397, 395, 399,
    0, 66, 659, 779, 17, 781, 1912, 1910, 0, 373, 598, 377, 1918, 1915, 538, 598,
    0, 66, 79, 14, 265, 208, 783, 92, 636, 1919, 0, 369, 203, 273, 1923, 1933,
    1943, 1953, 1963, 1971, 0, 1981, 791, 792, 1982, 793, 1985, 1995, 2005, 2015, 2025, 2035,
    2045, 2055, 2065, 2075, 2085, 2095, 2105, 2115, 2125, 2135, 2154, 2145, 103, 189, 2161, 0,
    2155, 803, 804, 805, 806, 807, 809, 0, 810, 811, 813, 814, 815, 816, 817, 818,
    819, 2156, 820, 821, 2157, 822, 0, 823, 0, 636, 824, 826, 828, 0, 830, 831,
    807, 99, 833, 0, 830, 835, 807, 99, 837, 807, 0, 839, 807, 0, 840, 807,
    0, 2160, 0, 2158, 0, 521, 841, 843, 844, 845, 846, 847, 807, 0, 848, 843,
    0, 849, 0, 2159, 0, 851, 44, 841, 807, 0, 852, 807, 853, 2162, 2179, 854,
    0, 2191, 2189, 2188, 855, 863, 0, 2194, 822, 823, 2195, 2206, 2227, 2236, 2246, 2256,
    2266, 2276, 2286, 2296, 2306, 2316, 2326, 2336, 2346, 0, 272, 876, 2356, 2366, 2376, 2386,
    2396, 2406, 2416, 2426, 2436, 0, 26, 216, 101, 2446, 183, 880, 881, 884, 2447, 703,
    2457, 2467, 2477, 2487, 2497, 2507, 2517, 0, 894, 189, 92, 2527, 2537, 2547, 2557, 2567,
    2577, 2587, 2597, 2607, 2617, 2627, 2637, 2647, 2656, 2666, 2676, 2686, 2696, 2706, 2716, 2726,
    2736, 2745, 2755, 2765, 2775, 915, 331, 0, 67, 671, 2776, 334, 377, 334, 916, 0,
    67, 77, 317, 103, 0, 67, 0, 50, 52, 103, 0, 67, 686, 317, 688, 916,
    0, 67, 13, 317, 681, 916, 0, 67, 706, 317, 708, 916, 2779, 0, 67, 712,
    317, 92, 0, 67, 72, 317, 95, 331, 0, 67, 779, 79, 14, 317, 208, 916,
    0, 67, 0, 666, 15, 317, 667, 0, 67, 113, 232, 0, 67, 16, 317, 183,
    916, 0, 331, 332, 0, 67, 737, 317, 739, 0, 67, 917, 317, 92, 916, 0,
    67, 68, 317, 95, 331, 0, 67, 17, 317, 781, 916, 2789, 2799, 2809, 2819, 2829,
    2839, 0, 551, 440, 2849, 2859, 2869, 2879, 0, 926, 2889, 775, 2892, 2902, 0, 928,
    930, 2905, 2915, 2925, 2935, 2945, 2954, 2964, 2974, 2984, 940, 2994, 3004, 3014, 3024, 3035,
    3045, 3055, 3065, 3075, 3085, 3095, 3105, 3115, 3125, 0, 30, 15, 667, 3138, 3136, 3135,
    715, 440, 0, 636, 954, 3141, 3151, 3161, 3171, 3181, 3191, 0, 961, 3201, 538, 962,
    964, 966, 0, 961, 79, 538, 0, 969, 3203, 0, 3202, 973, 975, 977, 980, 3204,
    0, 34, 3220, 0, 183, 3214, 172, 3217, 538, 982, 3221, 985, 0, 986, 3232, 3231,
    987, 0, 557, 986, 989, 990, 0, 605, 756, 991, 992, 993, 994, 0, 756, 995,
    996, 325, 986, 0, 997, 985, 3236, 1000, 0, 1001, 1002, 1003, 1004, 0, 1005, 3239,
    3238, 0, 3237, 486, 487, 489, 1005, 3243, 1000, 0, 1001, 1002, 3252, 0, 1006, 3244,
    538, 0, 3247, 1007, 1008, 3248, 1010, 595, 97, 0, 3267, 274, 472, 3257, 3256, 1012,
    440, 538, 3254, 3253, 273, 373, 472, 1013, 440, 1015, 1016, 0, 3260, 928, 1018, 3263,
    3261, 440, 0, 334, 3266, 256, 215, 1020, 1021, 1022, 1013, 1023, 97, 1024, 3268, 1025,
    273, 470, 1027, 1029, 373, 1031, 0, 3269, 3279, 3298, 3296, 3292, 0, 3289, 1037, 635,
    203, 215, 0, 3290, 124, 1038, 203, 256, 0, 1039, 203, 3291, 1040, 1041, 3301, 3311,
    3321, 3332, 0, 1045, 3342, 371, 1047, 3343, 215, 440, 752, 3344, 752, 1049, 3345, 1051,
    0, 636, 1052, 3346, 3356, 3371, 3367, 1059, 206, 820, 0, 752, 1060, 0, 263, 3366,
    1060, 1062, 203, 256, 3368, 1063, 1064, 1065, 0, 3369, 1066, 0, 752, 1037, 203, 215,
    392, 1067, 0, 636, 263, 1068, 0, 531, 203, 3370, 533, 534, 535, 3377, 3375, 1077,
    1079, 3376, 697, 1081, 3379, 0, 1083, 3389, 0, 1084, 440, 3392, 3402, 3412, 3422, 3432,
    3442, 0, 1088, 1089, 3452, 3462, 3472, 3482, 0, 1093, 1094, 0, 28, 183, 375, 3492,
    3502, 3512, 3522, 0, 1099, 3535, 3534, 0, 1100, 752, 0, 3532, 215, 470, 1101, 373,
    3533, 1104, 1105, 0, 995, 325, 3536, 1049, 1106, 3537, 0, 101, 3547, 1108, 3549, 1084,
    634, 635, 1116, 1117, 0, 52, 221, 0, 1118, 221, 1119, 1120, 1121, 733, 598, 3562,
    0, 1123, 3553, 3550, 1124, 1126, 185, 3554, 1128, 1129, 1132, 1133, 0, 636, 1135, 185,
    1129, 1137, 3559, 0, 3555, 1133, 3581, 3578, 0, 3566, 1138, 1140, 3563, 1108, 0, 1118,
    221, 0, 52, 221, 3567, 3571, 0, 3570, 1133, 1132, 0, 1118, 221, 0, 52, 221,
    3575, 3582, 3592, 3602, 3612, 0, 696, 1145, 3622, 472, 696, 273, 440, 703, 0, 3623,
    1147, 1149, 1151, 1153, 440, 3630, 440, 0, 440, 377, 334, 0, 1158, 440, 101, 101,
    440, 0, 3624, 855, 440, 101, 440, 0, 1159, 3627, 216, 3625, 702, 1160, 0, 1060,
    273, 0, 3628, 256, 103, 373, 692, 3629, 256, 103, 373, 1162, 710, 1165, 0, 3634,
    863, 0, 377, 256, 0, 1167, 189, 0, 105, 189, 3645, 0, 1145, 440, 99, 440,
    1168, 3637, 700, 553, 105, 0, 692, 440, 528, 440, 1169, 3641, 700, 553, 1169, 703,
    0, 1171, 3646, 440, 1172, 3650, 700, 3659, 0, 3654, 1174, 1175, 440, 1172, 3655, 700,
    0, 551, 3663, 700, 553, 1171, 703, 3667, 3677, 0, 3687, 0, 103, 1177, 523, 3691,
    768, 1178, 523, 3694, 0, 369, 3692, 0, 42, 917, 334, 3693, 0, 42, 917, 334,
    3697, 3695, 1179, 3700, 1180, 1182, 1184, 1186, 3701, 3713, 0, 1188, 1189, 1190, 0, 0,
    681, 3712, 1188, 1191, 1192, 769, 1194, 0, 1195, 440, 1168, 3714, 700, 553, 1195, 703,
    3718, 3728, 3738, 3748, 3758, 3769, 3779, 3789, 3799, 0, 41, 1203, 1205, 0, 87, 3809,
    0, 1205, 92, 0, 37, 1207, 99, 1209, 0, 636, 1211, 1212, 3810, 3820, 0, 506,
    940, 506, 3830, 0, 52, 221, 3844, 3843, 373, 215, 1217, 1219, 440, 3840, 0, 1118,
    221, 1117, 1119, 1120, 1121, 3846, 0, 1118, 221, 1223, 1119, 1120, 1225, 1045, 3847, 3857,
    3867, 0, 3881, 3880, 373, 215, 440, 3877, 1227, 3883, 0, 1228, 1229, 3896, 3894, 3893,
    1217, 793, 3899, 3909, 3919, 3929, 3939, 3949, 3959, 3969, 3979, 3989, 3999, 4009, 4019, 4029,
    4039, 4049, 4059, 4069, 4079, 0, 1119, 4092, 4089, 440, 4093, 0, 1118, 221, 1223, 1120,
    1225, 1045, 4094, 4104, 4114, 4124, 0, 4134, 4144, 4154, 4164, 4171, 4181, 0, 1171, 1006,
    4191, 0, 1171, 756, 1006, 4201, 4211, 0, 4221, 1253, 1254, 1255, 4222, 4232, 0, 71,
    103, 4242, 4252, 0, 4262, 600, 735, 4263, 0, 1258, 0, 1259, 190, 123, 256, 0,
    824, 1260, 1258, 123, 256, 0, 183, 0, 1262, 969, 1263, 4264, 440, 4267, 392, 1067,
    977, 0, 101, 4269, 0, 4268, 973, 975, 1265, 980, 0, 4270, 1267, 1268, 0, 1269,
    0, 1270, 203, 256, 0, 1259, 190, 4283, 0, 16, 183, 4271, 172, 0, 4275, 4274,
    392, 1067, 0, 1271, 221, 256, 4279, 0, 766, 256, 0, 4298, 0, 4284, 1258, 1272,
    4289, 0, 1259, 190, 4285, 123, 256, 0, 824, 1260, 4293, 1258, 1272, 4294, 123, 256,
    4299, 1273, 1275, 1277, 0, 15, 92, 0, 1259, 190, 0, 636, 4300, 0, 369, 203,
    273, 4304, 0, 32, 0, 969, 4322, 0, 183, 4314, 172, 4319, 4317, 1279, 1281, 4323,
    0, 756, 692, 4339, 4336, 4335, 4333, 985, 538, 472, 4340, 0, 103, 4350, 1283, 1284,
    4351, 4361, 4371, 4381, 4391, 4401, 4411, 4421, 4431, 0, 22, 969, 183, 4441, 1290, 0,
    272, 876, 538, 375, 4445, 4455, 4465, 4475, 4485, 4495, 4505, 0, 830, 256, 4535, 0,
    833, 4515, 215, 0, 831, 4518, 215, 0, 99, 833, 0, 4530, 0, 4521, 835, 1295,
    1296, 4522, 215, 0, 837, 4527, 4525, 215, 0, 99, 4534, 1295, 1296, 0, 837, 4531,
    215, 4536, 4546, 0, 969, 16, 183, 4556, 4566, 4576, 4586, 4596, 1021, 1301, 1302, 1020,
    1303, 0, 605, 756, 1159, 101, 4597, 0, 4607, 4617, 4627, 4637, 4647, 4657, 4667, 4677,
    4687, 4697, 4707, 4728, 0, 1313, 1314, 1140, 4717, 1316, 0, 4721, 1318, 4724, 1320, 1322,
    4725, 0, 1323, 1314, 1140, 1318, 0, 1324, 1140, 4729, 0, 1325, 0, 0, 87, 4739,
    4745, 4742, 440, 4746, 4750, 1327, 0, 4749, 1328, 1040, 1041, 1329, 0, 783, 4753, 4751,
    103, 4756, 4766, 4776, 4786, 4796, 4806, 4816, 4826, 4836, 4846, 4854, 0, 59, 4865, 0,
    60, 4864, 1338, 215, 0, 766, 215, 692, 60, 215, 0, 636, 1339, 256, 4866, 4876,
    4886, 4896, 4906, 0, 4916, 4926, 4936, 4946, 4956, 4966, 4976, 4986, 4996, 0, 504, 940,
    504, 5006, 0, 0, 756, 1351, 985, 0, 1352, 986, 5016, 723, 1353, 5017, 5027, 0,
    1117, 5038, 0, 215, 97, 215, 0, 215, 0, 1135, 215, 5039, 5049, 0, 5059, 5069,
    256, 215, 1020, 1001, 1021, 1022, 1356, 1358, 1360, 5070, 5079, 5086, 0, 824, 1177, 103,
    5096, 5106, 5116, 5126, 5136, 5146, 5156, 5166, 5176, 5186, 5196, 5206, 5216, 5226, 5236, 5246,
    5256, 5266, 5276, 0, 5286, 5296, 0, 0, 5306, 5316, 5326, 5336, 5346, 0, 26, 881,
    0, 22, 375, 1192, 881, 0, 1382, 969, 5356, 5366, 0, 1383, 1385, 0, 1385, 1383,
    0, 767, 1387, 1172, 5367, 1388, 1390, 0, 767, 756, 1387, 1172, 1392, 0, 1394, 5368,
    183, 880, 783, 5372, 183, 880, 0, 5369, 538, 185, 0, 472, 185, 5373, 1320, 5376,
    1395, 0, 24, 1397, 881, 5379, 5389, 5399, 5409, 5419, 5429, 5439, 5448, 5458, 5468, 5478,
    5488, 5498, 5508, 5517, 5527, 5537, 5547, 5557, 5566, 5576, 0, 103, 377, 5589, 103, 5586,
    1411, 0, 1225, 5590, 1413, 5592, 0, 5602, 0, 1417, 625, 5612, 5622, 5632, 5642, 0,
    5651, 5661, 5671, 5678, 5688, 5698, 5708, 5718, 5728, 5738, 5748, 5758, 5768, 5787, 1192, 1424,
    5778, 1025, 5781, 1425, 5784, 1427, 1428, 0, 5788, 964, 962, 966, 1429, 5793, 1431, 5792,
    5789, 1025, 1427, 5794, 0, 5804, 0, 5814, 0, 1434, 940, 0, 504, 506, 5815, 0,
    5825, 5835, 5845, 5855, 5865, 5875, 5886, 5896, 5906, 5916, 5926, 5936, 5947, 5957, 5967, 5977,
    5987, 5997, 6007, 0, 256, 1444, 448, 6028, 6017, 1445, 6019, 6031, 1177, 0, 1177, 1177,
    6032, 87, 377, 783, 97, 1448, 1449, 1450, 557, 1444, 697, 1132, 6033, 6043, 6053, 6063,
    6073, 0, 1135, 6083, 215, 440, 752, 6084, 752, 1049, 6085, 6095, 6105, 6115, 6125, 0,
    1454, 14, 208, 6135, 6145, 6155, 6165, 6175, 6185, 6195, 0, 745, 392, 6204, 0, 1118,
    221, 6217, 0, 6214, 373, 215, 440, 6215, 1283, 1284, 0, 1255, 6216, 371, 1458, 1459,
    1223, 1099, 6219, 1223, 1119, 1120, 1225, 1099, 1060, 1045, 6220, 6230, 1118, 1461, 6231, 6241,
    6251, 6261, 6271, 6281, 0, 1162, 1466, 6294, 103, 6291, 1411, 6295, 6305, 0, 1469, 6318,
    0, 6315, 1022, 97, 1022, 373, 6319, 6329, 6339, 6349, 6359, 6369, 6379, 6389, 6399, 6408,
    6417, 6427, 6437, 6447, 0, 327, 221, 6462, 6459, 6457, 440, 6463, 6473, 6483, 6493, 6503,
    6513, 6523, 6545, 0, 1484, 6543, 6533, 1484, 256, 0, 6546, 6556, 6566, 6576, 6586, 6595,
    6605, 6615, 6625, 6634, 6644, 6653, 6664, 777, 0, 0, 6663, 1253, 1254, 1255, 1496, 0,
    6665, 16, 6675, 6685, 6695, 0, 1116, 6708, 6705, 1499, 1501, 6709, 6720, 0, 0, 6719,
    1503, 0, 1004, 99, 1499, 0, 1504, 1499, 0, 6724, 0, 6721, 1505, 1506, 6722, 273,
    274, 472, 373, 1013, 6723, 273, 274, 472, 373, 1013, 1507, 6725, 0, 1509, 6739, 0,
    16, 183, 375, 0, 126, 6735, 183, 0, 14, 208, 0, 137, 6737, 208, 686, 6740,
    6750, 0, 1512, 1094, 6760, 6770, 6780, 6790, 6800, 6810, 6820, 6830, 0, 1000, 215, 0,
    1120, 440, 6840, 0, 489, 6850, 37, 42, 1515, 6851, 538, 6854, 6864, 6874, 6884, 6894,
    6905, 6916, 6926, 6936, 6946, 6956, 6966, 6977, 6987, 6997, 7006, 7015, 7024, 7033, 7043, 7053,
    7063, 7073, 0, 7083, 1528, 1529, 7088, 0, 16, 183, 375, 0, 126, 7084, 183, 0,
    14, 208, 0, 137, 7086, 208, 686, 1530, 7089, 7099, 7109, 7119, 0, 28, 183, 375,
    1535, 7129, 0, 87, 26, 1536, 0, 87, 24, 1536, 0, 1537, 7133, 1328, 1539, 1540,
    7134, 0, 749, 183, 375, 7144, 7154, 915, 331, 0, 49, 206, 0, 1541, 1542, 0,
    49, 272, 7155, 7165, 7175, 7185, 7195, 7205, 7215, 7225, 7235, 7245, 7255, 7265, 7275, 7285,
    0, 1555, 1117, 7295, 215, 1557, 1262, 7296, 7306, 7316, 7326, 0, 318, 7336, 14, 16,
    17, 13, 15, 68, 72, 77, 0, 50, 52, 92, 99, 92, 7337, 7347, 7357, 7367,
    7377, 7387, 7397, 7407, 7417, 7427, 7437, 7447, 7457, 7467, 7477, 7487, 0, 1568, 1203, 1205,
    0, 87, 7497, 0, 1205, 92, 0, 37, 1207, 101, 1209, 7498, 7508, 103, 7518, 7528,
    7538, 7546, 7556, 7566, 7576, 7586, 7596, 7606, 7616, 7626, 7636, 7646, 0, 1573, 7656, 1574,
    62, 7657, 215, 0, 7660, 1575, 7661, 215, 7664, 7674, 7684, 7694, 7704, 7714, 7724, 0,
    1081, 7734, 7744, 7754, 7764, 7774, 7784, 7794, 7804, 7814, 7824, 7834, 1585, 0, 448, 185,
    7835, 1159, 37, 0, 20, 1587, 7838, 1589, 7837, 0, 636, 1590, 7836, 1093, 1512, 0,
    969, 16, 1592, 1594, 1596, 1598, 7850, 101, 7840, 1084, 0, 1118, 221, 1117, 1121, 733,
    598, 7851, 0, 1599, 1601, 1603, 881, 1605, 884, 1607, 1609, 1611, 162, 1223, 1121, 733,
    598, 7872, 7864, 7861, 1425, 7868, 7865, 1613, 1614, 7869, 538, 7873, 7883, 7893, 7902, 7912,
    7922, 7932, 0, 1619, 215, 0, 1541, 1619, 7933, 0, 1620, 215, 0, 1541, 1620, 7934,
    7944, 7976, 7985, 7995, 8005, 8015, 0, 49, 8025, 1625, 101, 0, 373, 203, 8027, 440,
    273, 274, 472, 373, 206, 272, 0, 1427, 8028, 0, 1121, 1427, 162, 1223, 1627, 8032,
    8043, 0, 1629, 37, 1630, 8042, 1631, 1632, 0, 49, 14, 265, 208, 8044, 0, 8054,
    1007, 1008, 8055, 0, 1448, 334, 1010, 8059, 8069, 8079, 8089, 1635, 1636, 8090, 8100, 8110,
    8120, 189, 1178, 1639, 8121, 103, 0, 103, 585, 216, 8123, 8134, 8144, 0, 1641, 147,
    8154, 8164, 8173, 8182, 8192, 8202, 8212, 8222, 8232, 8242, 8252, 8262, 8272, 8282, 8292, 8302,
    8312, 8322, 8332, 8342, 8352, 8362, 8372, 8382, 8392, 8402, 8412, 8422, 8432, 8442, 8452, 8462,
    8472, 8482, 8492, 8502, 8512, 8522, 8532, 8542, 8552, 8562, 8572, 8582, 8592, 8602, 8612, 8622,
    8632, 8642, 8652, 8662, 8672, 8682, 8692, 8702, 8712, 8722, 8732, 8742, 8752, 8762, 8772, 8782,
    8792, 8802, 8812, 8822, 8832, 8842, 8852, 8862, 8872, 8882, 8892, 8902, 8911, 8921, 8931, 8941,
    538, 966, 1713, 1715, 962, 1429, 1717, 964, 1719, 1721, 1723, 1725, 1727, 1729, 1731, 1733,
    1735, 1737, 1739, 1741, 1743, 1745, 1747, 1749, 1751, 1753, 1755, 1757, 1759, 1761, 1763, 1765,
    1767, 1769, 1771, 1773, 1775, 1777, 1779, 1781, 1783, 1785, 1787, 1789, 1791, 1793, 1795, 1797,
    697, 8942, 8952, 8962, 8972, 8982, 8992, 9002, 1803, 9005, 9003, 0, 448, 185, 0, 9008,
    9018, 9028, 9038, 9048, 9058, 9068, 9078, 9085, 9095, 9105, 9115, 9125, 9161, 9170, 9180, 9190,
    1816, 1817, 0, 0, 312, 92, 9191, 0, 704, 375, 0, 313, 375, 0, 1818, 375,
    1819, 9192, 267, 306, 735, 0, 1820, 221, 9193, 373, 1821, 697, 9195, 0, 1822, 97,
    1821, 0, 1821, 97, 1822, 0, 9199, 274, 472, 183, 9196, 538, 1290, 1192, 1715, 1793,
    0, 0, 312, 103, 9200, 756, 206, 1512, 1093, 9201, 0, 756, 206, 313, 1823, 314,
    9202, 0, 0, 312, 103, 9212, 756, 206, 1512, 1093, 9213, 0, 313, 375, 0, 1818,
    375, 1823, 314, 0, 327, 221, 9214, 373, 1822, 697, 9216, 9226, 0, 9236, 1824, 557,
    1825, 0, 9240, 1826, 1825, 9244, 9254, 9264, 9274, 0, 9284, 1829, 1830, 9285, 9295, 1832,
    1833, 9296, 9306, 9316, 9326, 9335, 0, 9345, 826, 823, 807, 0, 9346, 636, 1837, 826,
    828, 9347, 0, 1259, 826, 828, 440, 9351, 9361, 9371, 9381, 9391, 9401, 9411, 9421, 9431,
    9441, 9451, 9461, 9471, 9481, 9491, 9501, 9511, 0, 9521, 1848, 1849, 1850, 9522, 9532, 9542,
    9552, 9562, 9572, 9582, 9592, 1857, 1858, 1859, 9593, 9603, 9614, 0, 1067, 1861, 1862, 0,
    1863, 9625, 0, 1458, 9624, 835, 833, 831, 0, 101, 256, 9626, 0, 1067, 16, 183,
    9638, 9636, 448, 9641, 0, 1641, 1356, 0, 928, 1864, 9651, 9661, 9674, 9672, 600, 1867,
    9675, 9685, 9695, 0, 9705, 1870, 1871, 1872, 9706, 9716, 9726, 9736, 0, 1629, 9746, 1159,
    37, 9747, 0, 704, 9757, 9767, 0, 9777, 9788, 9798, 9808, 0, 1599, 1601, 1603, 881,
    1605, 884, 1607, 1609, 1611, 1121, 733, 598, 162, 1223, 0, 1427, 9818, 0, 1121, 1427,
    9822, 9832, 0, 24, 9842, 183, 880, 1397, 1601, 881, 884, 9843, 9853, 9863, 0, 75,
    688, 0, 221, 1882, 1883, 256, 9873, 0, 1259, 1884, 377, 256, 779, 0, 221, 1885,
    1886, 256, 0, 221, 393, 103, 9883, 9893, 9903, 9913, 9923, 0, 876, 9943, 9942, 9936,
    9934, 9933, 272, 273, 440, 9940, 9939, 272, 440, 185, 9945, 0, 876, 995, 9955, 215,
    274, 472, 325, 9956, 215, 274, 472, 0, 636, 1889, 215, 9957, 9967, 9977, 9987, 9997,
    10006, 10016, 10026, 10036, 10046, 10056, 10066, 10075, 0, 767, 0, 440, 10085, 1172, 10088, 440,
    10091, 10094, 10097, 10107, 0, 1084, 10120, 10117, 440, 10121, 634, 635, 1116, 1117, 0, 52,
    221, 0, 1118, 221, 1119, 1120, 1121, 733, 598, 10122, 10132, 10142, 0, 1902, 0, 10150,
    10149, 0, 103, 585, 0, 216, 1177, 216, 10154, 0, 103, 10164, 327, 1118, 1501, 162,
    1903, 0, 636, 1905, 10165, 10168, 1907, 20, 10170, 1909, 0, 636, 1905, 10173, 10176, 1907,
    20, 10178, 1614, 0, 636, 0, 1541, 1910, 0, 0, 636, 1590, 1818, 375, 0, 636,
    1590, 10181, 704, 313, 375, 10182, 0, 636, 1541, 14, 1911, 10192, 10202, 10212, 10219, 10229,
    10239, 10249, 10259, 10269, 10279,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 90, 4294967295, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 107, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 143, 145, 4294967295, 91, 4294967295,
    148, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 164, 4294967295,
    165, 170, 4294967295, 4294967295, 4294967295, 4294967295, 173, 4294967295, 4294967295, 4294967295, 181, 4294967295, 184, 186, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 209, 4294967295,
    4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 233, 4294967295,
    235, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 297, 299, 301, 303, 305, 4294967295, 4294967295, 4294967295, 184, 4294967295,
    4294967295, 4294967295, 310, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 322, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 364, 368,
    4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 379, 381, 383, 4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 390,
    4294967295, 4294967295, 4294967295, 396, 398, 400, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 209, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 374, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    416, 416, 4294967295, 416, 416, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    471, 473, 374, 441, 4294967295, 364, 4294967295, 4294967295, 4294967295, 474, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 374, 522, 4294967295, 526, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 173, 4294967295,
    4294967295, 537, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 550,
    4294967295, 4294967295, 556, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 550, 4294967295, 4294967295, 556, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 596, 4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 310, 599, 4294967295, 4294967295, 364, 4294967295, 601, 368, 4294967295, 4294967295, 4294967295, 374, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 606, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 205, 205, 205, 205, 205, 205, 4294967295, 4294967295, 205, 416, 205, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 668, 4294967295, 670, 4294967295, 4294967295, 4294967295, 4294967295, 364,
    4294967295, 364, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 682, 684,
    4294967295, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 689, 4294967295, 4294967295, 4294967295, 4294967295, 691, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 364, 4294967295, 364, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 364, 4294967295, 4294967295,
    4294967295, 701, 4294967295, 4294967295, 4294967295, 4294967295, 691, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 709, 4294967295,
    184, 4294967295, 4294967295, 4294967295, 711, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295,
    4294967295, 4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 668, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 670, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 233, 4294967295, 721, 4294967295,
    4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295,
    732, 4294967295, 4294967295, 322, 4294967295, 4294967295, 4294967295, 734, 736, 599, 4294967295, 4294967295, 4294967295, 740, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 770, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 668, 4294967295, 4294967295, 4294967295, 164, 4294967295,
    4294967295, 4294967295, 364, 778, 4294967295, 670, 4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295, 390, 398, 396, 400,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 782, 4294967295, 4294967295, 4294967295, 374, 599, 4294967295, 4294967295, 4294967295, 4294967295, 599,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 209, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 682, 682, 4294967295, 794, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 808, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    808, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 808, 4294967295, 4294967295, 808, 4294967295, 4294967295, 808, 4294967295, 4294967295, 808,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 808, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 808, 4294967295, 4294967295, 808, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 883, 886, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 364, 4294967295, 364, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 689, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 682, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 709, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 209, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 668, 4294967295, 4294967295, 4294967295, 233, 4294967295, 4294967295, 4294967295, 4294967295, 184,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 4294967295, 4294967295, 4294967295, 4294967295, 91, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 96, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 782, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 929,
    931, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 668, 4294967295, 4294967295, 4294967295,
    4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 963,
    965, 967, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 974, 976, 979, 981, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 173, 4294967295, 4294967295, 983, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 214, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 214, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1009, 4294967295, 1011, 596, 4294967295, 4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295,
    441, 4294967295, 4294967295, 4294967295, 4294967295, 374, 473, 1014, 441, 4294967295, 1017, 4294967295, 4294967295, 929, 1019, 4294967295,
    4294967295, 441, 4294967295, 364, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1014, 4294967295, 4294967295, 4294967295, 4294967295, 1026,
    4294967295, 471, 1028, 1030, 374, 1033, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1078,
    1078, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 471, 4294967295, 374,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1109, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1122, 734, 599, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1125, 1127, 186, 4294967295, 4294967295, 1131, 4294967295, 1134, 4294967295, 4294967295, 4294967295, 186,
    1131, 4294967295, 4294967295, 4294967295, 4294967295, 1134, 4294967295, 4294967295, 4294967295, 4294967295, 1139, 1141, 4294967295, 1109, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 1134, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    181, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295,
    1148, 1150, 1152, 1155, 441, 4294967295, 441, 4294967295, 441, 4294967295, 364, 4294967295, 4294967295, 441, 4294967295, 4294967295,
    441, 4294967295, 4294967295, 4294967295, 441, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1161, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 711, 1028, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 441,
    4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 441, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 701,
    4294967295, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 522, 4294967295,
    4294967295, 4294967295, 522, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 364, 4294967295, 4294967295, 4294967295, 4294967295, 364,
    4294967295, 4294967295, 4294967295, 4294967295, 1181, 1183, 1185, 1187, 4294967295, 4294967295, 4294967295, 682, 770, 4294967295, 4294967295, 4294967295,
    682, 4294967295, 682, 682, 1193, 770, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1204, 1206, 4294967295, 4294967295, 4294967295,
    4294967295, 1206, 91, 4294967295, 4294967295, 4294967295, 4294967295, 1210, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 1218, 1221, 441, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1122, 4294967295, 4294967295, 4294967295, 4294967295, 1224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1218, 794, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 1224, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 945, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 601, 736, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 1264, 4294967295, 441, 4294967295, 4294967295, 4294967295,
    979, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 974, 976, 1266, 981, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 191, 4294967295, 4294967295, 4294967295, 184, 4294967295, 173, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 191, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1274, 1276, 1278, 4294967295, 4294967295, 91, 4294967295, 4294967295, 191, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 173, 4294967295, 4294967295, 1280, 1282, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 1291, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1141, 4294967295, 4294967295, 4294967295, 165, 1319, 4294967295, 1321, 4294967295,
    181, 4294967295, 4294967295, 4294967295, 1141, 1319, 4294967295, 4294967295, 1141, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165,
    4294967295, 4294967295, 441, 181, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 214, 4294967295, 4294967295, 1357, 1359, 1362, 865, 865, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 883,
    4294967295, 4294967295, 4294967295, 1193, 883, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1384, 1386, 4294967295, 1386, 1384,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1389, 1391, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1393, 4294967295, 4294967295, 4294967295,
    184, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 473, 186, 165, 1321, 181,
    1396, 4294967295, 4294967295, 1398, 883, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1412, 4294967295, 4294967295, 4294967295, 1414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1193, 1026,
    4294967295, 1026, 4294967295, 1426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 965, 963, 967, 1430, 4294967295, 526, 4294967295,
    4294967295, 1026, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1446, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1460,
    1224, 4294967295, 4294967295, 1224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1412, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1497, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1500, 1502, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1500, 4294967295, 4294967295, 1500, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 473, 374, 1014, 4294967295, 4294967295, 4294967295, 473, 374, 1014, 1508, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 209, 4294967295, 4294967295, 4294967295, 209, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295,
    4294967295, 209, 4294967295, 4294967295, 4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 184, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 91, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1204, 1206,
    4294967295, 4294967295, 4294967295, 4294967295, 1206, 91, 4294967295, 4294967295, 4294967295, 4294967295, 1210, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1586, 4294967295, 4294967295, 186,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1588, 4294967295, 1591, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1593, 1595, 1597, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1122, 734,
    599, 4294967295, 4294967295, 1600, 1602, 1604, 883, 1606, 886, 1608, 1610, 1612, 164, 1224, 1122, 734,
    599, 4294967295, 4294967295, 4294967295, 1426, 4294967295, 4294967295, 1426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1626, 4294967295, 4294967295, 374, 205, 4294967295, 441,
    4294967295, 4294967295, 473, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1122, 4294967295, 164, 1224, 1628, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 209, 4294967295, 4294967295, 4294967295,
    4294967295, 1009, 4294967295, 4294967295, 4294967295, 364, 1011, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1642, 148,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 967, 1714, 1716, 963, 1430, 1718, 965, 1720, 1722, 1724, 1726, 1728, 1730, 1732, 1734,
    1736, 1738, 1740, 1742, 1744, 1746, 1748, 1750, 1752, 1754, 1756, 1758, 1760, 1762, 1764, 1766,
    1768, 1770, 1772, 1774, 1776, 1778, 1780, 1782, 1784, 1786, 1788, 1790, 1792, 1794, 1796, 1798,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 416, 4294967295, 4294967295, 4294967295, 4294967295, 186, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 176, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 91, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 268, 310, 736, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 473, 184, 4294967295, 4294967295, 1291, 1193, 1716, 1794,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 374, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 808, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1642, 1357, 4294967295, 929, 931, 4294967295, 4294967295, 4294967295, 4294967295, 601, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1600, 1602, 1604, 883,
    1606, 886, 1608, 1610, 1612, 1122, 734, 599, 164, 1224, 4294967295, 4294967295, 4294967295, 4294967295, 1122, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 184, 4294967295, 1398, 1602, 883, 886, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    689, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 441, 186, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 473, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 165, 4294967295, 165, 441,
    181, 181, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1122, 734, 599, 4294967295, 4294967295, 1900, 4294967295, 1017, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1502, 164,
    1904, 4294967295, 4294967295, 4294967295, 165, 4294967295, 1908, 4294967295, 181, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 1908,
    4294967295, 181, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 5, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 89, 4294967295, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 106, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 142, 144, 4294967295, 92, 4294967295,
    147, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295,
    166, 168, 4294967295, 4294967295, 4294967295, 4294967295, 172, 4294967295, 4294967295, 4294967295, 182, 4294967295, 183, 185, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 190, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295,
    4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295,
    234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 296, 298, 300, 302, 304, 4294967295, 4294967295, 4294967295, 183, 4294967295,
    4294967295, 4294967295, 306, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 321, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 334, 365,
    4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 378, 380, 382, 4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389,
    4294967295, 4294967295, 4294967295, 395, 397, 399, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 373, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    435, 436, 4294967295, 438, 439, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    470, 472, 373, 440, 4294967295, 334, 4294967295, 4294967295, 4294967295, 475, 476, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 417, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 373, 523, 4294967295, 524, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 172, 4294967295,
    4294967295, 536, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547, 4294967295, 549,
    4294967295, 4294967295, 555, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 549, 4294967295, 4294967295, 555, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 469, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 595, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 597, 598, 4294967295, 4294967295, 334, 4294967295, 600, 365, 4294967295, 4294967295, 4294967295, 373, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 607, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 203, 611, 612, 613, 614, 615, 4294967295, 4294967295, 618, 619, 620, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 667, 4294967295, 669, 4294967295, 4294967295, 4294967295, 4294967295, 334,
    4294967295, 334, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 681, 683,
    4294967295, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 190, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 688, 4294967295, 4294967295, 4294967295, 4294967295, 690, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 334, 4294967295, 334, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 334, 4294967295, 4294967295,
    4294967295, 700, 4294967295, 4294967295, 4294967295, 4294967295, 690, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 708, 4294967295,
    183, 4294967295, 4294967295, 4294967295, 710, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295,
    4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 667, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 669, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 720, 4294967295,
    4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 185, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295,
    731, 4294967295, 4294967295, 321, 4294967295, 4294967295, 4294967295, 733, 735, 598, 4294967295, 4294967295, 4294967295, 739, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 769, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 667, 4294967295, 4294967295, 4294967295, 162, 4294967295,
    4294967295, 4294967295, 334, 777, 4294967295, 669, 4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295, 389, 397, 395, 399,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 781, 4294967295, 4294967295, 4294967295, 373, 598, 4294967295, 4294967295, 4294967295, 4294967295, 598,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 791, 792, 4294967295, 793, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 807, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    807, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 807, 4294967295, 4294967295, 807, 4294967295, 4294967295, 807, 4294967295, 4294967295, 807,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 807, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 807, 4294967295, 4294967295, 807, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 881, 884, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 334, 4294967295, 334, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 688, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 681, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 708, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 667, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 183,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 739, 4294967295, 4294967295, 4294967295, 4294967295, 92, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 95, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 781, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 928,
    930, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 667, 4294967295, 4294967295, 4294967295,
    4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 962,
    964, 966, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 973, 975, 977, 980, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 172, 4294967295, 4294967295, 982, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1001, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1001, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1008, 4294967295, 1010, 595, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295,
    440, 4294967295, 4294967295, 4294967295, 4294967295, 373, 472, 1013, 440, 4294967295, 1016, 4294967295, 4294967295, 928, 1018, 4294967295,
    4294967295, 440, 4294967295, 334, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1013, 4294967295, 4294967295, 4294967295, 4294967295, 1025,
    4294967295, 470, 1027, 1029, 373, 1031, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1077,
    1079, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 470, 4294967295, 373,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1108, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 733, 598, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1124, 1126, 185, 4294967295, 4294967295, 1129, 4294967295, 1133, 4294967295, 4294967295, 4294967295, 185,
    1129, 4294967295, 4294967295, 4294967295, 4294967295, 1133, 4294967295, 4294967295, 4294967295, 4294967295, 1138, 1140, 4294967295, 1108, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 166, 4294967295, 4294967295, 4294967295, 1133, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    182, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295,
    1147, 1149, 1151, 1153, 440, 4294967295, 440, 4294967295, 440, 4294967295, 334, 4294967295, 4294967295, 440, 4294967295, 4294967295,
    440, 4294967295, 4294967295, 4294967295, 440, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1160, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 710, 1165, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 440,
    4294967295, 4294967295, 700, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 440, 4294967295, 4294967295, 700, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 700, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 700,
    4294967295, 4294967295, 4294967295, 700, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 523, 4294967295,
    4294967295, 4294967295, 523, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 334, 4294967295, 4294967295, 4294967295, 4294967295, 334,
    4294967295, 4294967295, 4294967295, 4294967295, 1180, 1182, 1184, 1186, 4294967295, 4294967295, 4294967295, 1188, 1189, 4294967295, 4294967295, 4294967295,
    681, 4294967295, 1188, 1191, 1192, 769, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 700, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1203, 1205, 4294967295, 4294967295, 4294967295,
    4294967295, 1205, 92, 4294967295, 4294967295, 4294967295, 4294967295, 1209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 1217, 1219, 440, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1121, 4294967295, 4294967295, 4294967295, 4294967295, 1223, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1217, 793, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 1223, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 600, 735, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 190, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 1263, 4294967295, 440, 4294967295, 4294967295, 4294967295,
    977, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 973, 975, 1265, 980, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 190, 4294967295, 4294967295, 4294967295, 183, 4294967295, 172, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 190, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1273, 1275, 1277, 4294967295, 4294967295, 92, 4294967295, 4294967295, 190, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 172, 4294967295, 4294967295, 1279, 1281, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 1290, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1140, 4294967295, 4294967295, 4294967295, 166, 1318, 4294967295, 1320, 4294967295,
    182, 4294967295, 4294967295, 4294967295, 1140, 1318, 4294967295, 4294967295, 1140, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547,
    4294967295, 4294967295, 440, 182, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1001, 4294967295, 4294967295, 1356, 1358, 1360, 1043, 866, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 881,
    4294967295, 4294967295, 4294967295, 1192, 881, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1383, 1385, 4294967295, 1385, 1383,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1388, 1390, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1392, 4294967295, 4294967295, 4294967295,
    183, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 185, 4294967295, 472, 185, 166, 1320, 182,
    1395, 4294967295, 4294967295, 1397, 881, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1411, 4294967295, 4294967295, 4294967295, 1413, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1421, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1192, 1424,
    4294967295, 1025, 4294967295, 1425, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 964, 962, 966, 1429, 4294967295, 1431, 4294967295,
    4294967295, 1025, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1445, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1459,
    1223, 4294967295, 4294967295, 1223, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1411, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1496, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1499, 1501, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1499, 4294967295, 4294967295, 1499, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 472, 373, 1013, 4294967295, 4294967295, 4294967295, 472, 373, 1013, 1507, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 183, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 92, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1203, 1205,
    4294967295, 4294967295, 4294967295, 4294967295, 1205, 92, 4294967295, 4294967295, 4294967295, 4294967295, 1209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1585, 4294967295, 4294967295, 185,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1587, 4294967295, 1589, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1592, 1594, 1596, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 733,
    598, 4294967295, 4294967295, 1599, 1601, 1603, 881, 1605, 884, 1607, 1609, 1611, 162, 1223, 1121, 733,
    598, 4294967295, 4294967295, 4294967295, 1425, 4294967295, 4294967295, 1613, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1625, 4294967295, 4294967295, 373, 203, 4294967295, 440,
    4294967295, 4294967295, 472, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 4294967295, 162, 1223, 1627, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295,
    4294967295, 1008, 4294967295, 4294967295, 4294967295, 334, 1010, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1641, 147,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 966, 1713, 1715, 962, 1429, 1717, 964, 1719, 1721, 1723, 1725, 1727, 1729, 1731, 1733,
    1735, 1737, 1739, 1741, 1743, 1745, 1747, 1749, 1751, 1753, 1755, 1757, 1759, 1761, 1763, 1765,
    1767, 1769, 1771, 1773, 1775, 1777, 1779, 1781, 1783, 1785, 1787, 1789, 1791, 1793, 1795, 1797,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1803, 4294967295, 4294967295, 4294967295, 4294967295, 185, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 175, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 92, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 306, 735, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 472, 183, 4294967295, 4294967295, 1290, 1192, 1715, 1793,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 807, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1641, 1356, 4294967295, 928, 1864, 4294967295, 4294967295, 4294967295, 4294967295, 600, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1599, 1601, 1603, 881,
    1605, 884, 1607, 1609, 1611, 1121, 733, 598, 162, 1223, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 183, 4294967295, 1397, 1601, 881, 884, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    688, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 440, 185, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 472, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 166, 4294967295, 547, 440,
    182, 182, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1121, 733, 598, 4294967295, 4294967295, 1901, 4294967295, 1902, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1501, 162,
    1903, 4294967295, 4294967295, 4294967295, 166, 4294967295, 1907, 4294967295, 182, 4294967295, 4294967295, 4294967295, 4294967295, 166, 4294967295, 1907,
    4294967295, 182, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1176, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (2825, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (130, 0, 3),
    (133, 3, 2),
    (138, 5, 3),
    (154, 8, 2),
    (203, 10, 2),
    (204, 12, 2),
    (206, 14, 2),
    (208, 16, 2),
    (222, 18, 2),
    (224, 20, 4),
    (225, 24, 2),
    (230, 26, 2),
    (234, 28, 5),
    (236, 33, 3),
    (237, 36, 2),
    (246, 38, 2),
    (254, 40, 2),
    (259, 42, 3),
    (263, 45, 2),
    (270, 47, 3),
    (275, 50, 2),
    (302, 52, 3),
    (304, 55, 2),
    (310, 57, 2),
    (319, 59, 2),
    (329, 61, 2),
    (356, 63, 3),
    (358, 66, 2),
    (359, 68, 2),
    (360, 70, 2),
    (361, 72, 2),
    (362, 74, 2),
    (366, 76, 3),
    (370, 79, 2),
    (396, 81, 3),
    (398, 84, 2),
    (430, 86, 2),
    (431, 88, 2),
    (436, 90, 2),
    (445, 92, 2),
    (451, 94, 3),
    (459, 97, 3),
    (467, 100, 2),
    (468, 102, 2),
    (469, 104, 2),
    (473, 106, 3),
    (479, 109, 2),
    (483, 111, 2),
    (484, 113, 2),
    (485, 115, 2),
    (488, 117, 2),
    (501, 119, 2),
    (509, 121, 3),
    (515, 124, 3),
    (516, 127, 2),
    (523, 129, 3),
    (544, 132, 2),
    (545, 134, 2),
    (547, 136, 2),
    (548, 138, 2),
    (553, 140, 2),
    (592, 142, 2),
    (593, 144, 2),
    (594, 146, 3),
    (595, 149, 2),
    (597, 151, 2),
    (601, 153, 2),
    (602, 155, 1),
    (614, 156, 3),
    (658, 159, 3),
    (659, 162, 2),
    (661, 164, 2),
    (676, 166, 2),
    (680, 168, 2),
    (686, 170, 2),
    (689, 172, 2),
    (701, 174, 4),
    (703, 178, 2),
    (706, 180, 2),
    (711, 182, 2),
    (713, 184, 2),
    (716, 186, 2),
    (753, 188, 2),
    (771, 190, 2),
    (773, 192, 2),
    (787, 194, 2),
    (788, 196, 2),
    (791, 198, 2),
    (793, 200, 2),
    (794, 202, 2),
    (798, 204, 3),
    (813, 207, 3),
    (820, 210, 3),
    (842, 213, 2),
    (849, 215, 2),
    (850, 217, 2),
    (851, 219, 2),
    (852, 221, 2),
    (853, 223, 2),
    (854, 225, 2),
    (857, 227, 3),
    (858, 230, 2),
    (859, 232, 2),
    (876, 234, 2),
    (882, 236, 2),
    (886, 238, 2),
    (891, 240, 2),
    (922, 242, 2),
    (952, 244, 3),
    (954, 247, 2),
    (959, 249, 2),
    (961, 251, 2),
    (974, 253, 2),
    (975, 255, 2),
    (978, 257, 2),
    (988, 259, 2),
    (996, 261, 2),
    (1001, 263, 3),
    (1006, 266, 3),
    (1012, 269, 2),
    (1014, 271, 2),
    (1021, 273, 2),
    (1025, 275, 2),
    (1030, 277, 3),
    (1038, 280, 3),
    (1040, 283, 3),
    (1044, 286, 2),
    (1051, 288, 2),
    (1058, 290, 2),
    (1069, 292, 3),
    (1074, 295, 3),
    (1080, 298, 3),
    (1092, 301, 2),
    (1116, 303, 3),
    (1118, 306, 2),
    (1121, 308, 3),
    (1126, 311, 2),
    (1147, 313, 3),
    (1152, 316, 2),
    (1155, 318, 2),
    (1159, 320, 2),
    (1160, 322, 2),
    (1161, 324, 2),
    (1165, 326, 3),
    (1180, 329, 3),
    (1186, 332, 3),
    (1200, 335, 3),
    (1223, 338, 2),
    (1228, 340, 2),
    (1242, 342, 3),
    (1246, 345, 2),
    (1250, 347, 2),
    (1251, 349, 2),
    (1253, 351, 2),
    (1257, 353, 3),
    (1260, 356, 2),
    (1261, 358, 2),
    (1262, 360, 2),
    (1263, 362, 2),
    (1269, 364, 3),
    (1273, 367, 3),
    (1274, 370, 2),
    (1279, 372, 2),
    (1285, 374, 3),
    (1287, 377, 2),
    (1292, 379, 2),
    (1302, 381, 2),
    (1303, 383, 2),
    (1305, 385, 2),
    (1333, 387, 2),
    (1360, 389, 2),
    (1366, 391, 2),
    (1369, 393, 2),
    (1372, 395, 2),
    (1375, 397, 2),
    (1388, 399, 2),
    (1400, 401, 2),
    (1403, 403, 2),
    (1450, 405, 3),
    (1452, 408, 2),
    (1453, 410, 2),
    (1466, 412, 2),
    (1499, 414, 2),
    (1501, 416, 2),
    (1518, 418, 3),
    (1524, 421, 2),
    (1530, 423, 3),
    (1537, 426, 2),
    (1542, 428, 3),
    (1550, 431, 3),
    (1558, 434, 3),
    (1562, 437, 3),
    (1567, 440, 3),
    (1576, 443, 3),
    (1581, 446, 2),
    (1587, 448, 3),
    (1593, 451, 3),
    (1603, 454, 2),
    (1615, 456, 2),
    (1616, 458, 2),
    (1644, 460, 3),
    (1649, 463, 2),
    (1663, 465, 2),
    (1664, 467, 2),
    (1665, 469, 2),
    (1675, 471, 2),
    (1676, 473, 2),
    (1677, 475, 2),
    (1678, 477, 2),
    (1684, 479, 3),
    (1686, 482, 2),
    (1689, 484, 2),
    (1721, 486, 2),
    (1738, 488, 2),
    (1748, 490, 2),
    (1750, 492, 2),
    (1751, 494, 2),
    (1756, 496, 2),
    (1760, 498, 2),
    (1765, 500, 3),
    (1766, 503, 2),
    (1767, 505, 2),
    (1768, 507, 2),
    (1770, 509, 3),
    (1773, 512, 2),
    (1774, 514, 2),
    (1777, 516, 2),
    (1779, 518, 2),
    (1786, 520, 2),
    (1791, 522, 2),
    (1793, 524, 2),
    (1794, 526, 2),
    (1795, 528, 2),
    (1796, 530, 3),
    (1797, 533, 2),
    (1808, 535, 2),
    (1814, 537, 2),
    (1818, 539, 2),
    (1833, 541, 2),
    (1858, 543, 2),
    (1870, 545, 2),
    (1880, 547, 2),
    (1887, 549, 2),
    (1888, 551, 2),
    (1898, 553, 2),
    (1917, 555, 3),
    (1933, 558, 2),
    (1935, 560, 3),
    (1949, 563, 2),
    (1964, 565, 2),
    (1965, 567, 2),
    (1966, 569, 2),
    (1972, 571, 2),
    (1973, 573, 2),
    (1974, 579, 2),
    (1977, 575, 2),
    (1979, 577, 2),
    (1983, 581, 2),
    (1984, 583, 2),
    (1989, 585, 2),
    (1994, 587, 2),
    (1995, 589, 2),
    (1997, 591, 2),
    (2008, 593, 2),
    (2025, 595, 2),
    (2028, 597, 2),
    (2032, 599, 2),
    (2033, 601, 2),
    (2034, 603, 2),
    (2035, 605, 2),
    (2036, 607, 2),
    (2038, 609, 2),
    (2040, 611, 2),
    (2042, 613, 2),
    (2045, 615, 2),
    (2048, 617, 2),
    (2052, 619, 2),
    (2054, 621, 2),
    (2061, 623, 2),
    (2069, 625, 3),
    (2074, 628, 3),
    (2076, 631, 2),
    (2077, 633, 2),
    (2093, 635, 2),
    (2095, 637, 2),
    (2098, 639, 2),
    (2103, 641, 2),
    (2105, 643, 2),
    (2108, 645, 2),
    (2115, 647, 2),
    (2118, 649, 2),
    (2124, 651, 2),
    (2127, 653, 2),
    (2131, 655, 2),
    (2142, 657, 2),
    (2146, 659, 2),
    (2154, 661, 2),
    (2159, 663, 2),
    (2164, 665, 2),
    (2165, 667, 2),
    (2166, 669, 2),
    (2167, 671, 2),
    (2171, 673, 2),
    (2172, 675, 2),
    (2176, 683, 2),
    (2178, 677, 2),
    (2179, 679, 2),
    (2180, 681, 2),
    (2181, 685, 2),
    (2185, 687, 2),
    (2188, 689, 2),
    (2203, 691, 2),
    (2204, 693, 2),
    (2209, 695, 2),
    (2210, 697, 2),
    (2215, 699, 2),
    (2232, 701, 3),
    (2234, 704, 2),
    (2235, 706, 2),
    (2236, 708, 2),
    (2244, 710, 2),
    (2249, 712, 2),
    (2260, 714, 3),
    (2262, 717, 2),
    (2272, 719, 2),
    (2273, 721, 2),
    (2297, 723, 2),
    (2302, 725, 2),
    (2314, 727, 3),
    (2341, 730, 2),
    (2342, 732, 2),
    (2348, 734, 2),
    (2358, 736, 3),
    (2362, 739, 2),
    (2364, 741, 2),
    (2368, 743, 2),
    (2374, 745, 2),
    (2375, 747, 2),
    (2376, 749, 2),
    (2377, 751, 2),
    (2386, 753, 2),
    (2390, 755, 2),
    (2394, 757, 3),
    (2396, 760, 2),
    (2419, 762, 2),
    (2433, 764, 2),
    (2434, 766, 2),
    (2435, 768, 2),
    (2438, 770, 2),
    (2441, 772, 2),
    (2447, 774, 2),
    (2456, 776, 3),
    (2458, 779, 2),
    (2461, 781, 2),
    (2462, 783, 2),
    (2473, 785, 2),
    (2492, 787, 3),
    (2494, 790, 2),
    (2550, 792, 3),
    (2583, 795, 2),
    (2588, 797, 2),
    (2590, 799, 2),
    (2596, 801, 2),
    (2597, 803, 2),
    (2600, 805, 2),
    (2610, 807, 2),
    (2634, 809, 4),
    (2707, 813, 2),
    (2710, 815, 2),
    (2711, 817, 2),
    (2712, 819, 2),
    (2713, 821, 5),
    (2714, 826, 3),
    (2751, 829, 2),
    (2755, 831, 2),
    (2756, 833, 2),
    (2763, 835, 2),
    (2764, 837, 2),
    (2766, 839, 2),
    (2767, 841, 2),
    (2773, 843, 2),
    (2774, 845, 2),
    (2780, 847, 2),
    (2784, 849, 3),
    (2788, 852, 3),
    (2793, 855, 2),
    (2795, 857, 2),
    (2796, 859, 2),
    (2798, 861, 2),
    (2800, 863, 2),
    (2803, 865, 2),
    (2804, 867, 2),
    (2832, 869, 2),
    (2836, 871, 2),
    (2850, 873, 3),
    (2862, 876, 2),
    (2863, 878, 2),
    (2865, 880, 2),
    (2867, 882, 2),
    (2873, 884, 2),
    (2874, 886, 2),
    (2875, 888, 2),
    (2876, 890, 2),
    (2878, 892, 2),
    (2881, 894, 2),
    (2921, 896, 2),
    (2949, 898, 2),
    (2962, 900, 3),
    (2980, 903, 3),
    (2982, 906, 2),
    (2991, 908, 2),
    (2992, 910, 2),
    (2995, 912, 2),
    (3018, 914, 2),
    (3029, 916, 3),
    (3050, 919, 2),
    (3079, 921, 2),
    (3086, 923, 2),
    (3097, 925, 2),
    (3098, 927, 2),
    (3108, 929, 2),
    (3111, 931, 2),
    (3121, 933, 2),
    (3122, 935, 3),
    (3123, 938, 2),
    (3127, 940, 2),
    (3128, 942, 3),
    (3129, 945, 2),
    (3130, 947, 2),
    (3137, 949, 3),
    (3142, 952, 3),
    (3145, 955, 3),
    (3149, 958, 3),
    (3169, 961, 2),
    (3209, 963, 3),
    (3214, 966, 3),
    (3217, 969, 3),
    (3221, 972, 3),
    (3230, 975, 3),
    (3251, 978, 3),
    (3305, 981, 2),
    (3307, 983, 2),
    (3326, 985, 2),
    (3327, 987, 2),
    (3332, 989, 2),
    (3333, 991, 2),
    (3338, 993, 2),
    (3388, 995, 2),
    (3391, 997, 2),
    (3397, 999, 2),
    (3399, 1001, 2),
    (3410, 1003, 2),
    (3411, 1005, 2),
    (3412, 1007, 2),
    (3422, 1009, 2),
    (3423, 1011, 2),
    (3424, 1013, 2),
    (3427, 1015, 2),
    (3428, 1017, 2),
    (3429, 1019, 2),
    (3430, 1021, 2),
    (3431, 1023, 2),
    (3432, 1025, 2),
    (3433, 1027, 2),
    (3434, 1029, 2),
    (3435, 1031, 2),
    (3436, 1033, 2),
    (3437, 1035, 2),
    (3438, 1037, 2),
    (3439, 1039, 2),
    (3440, 1041, 2),
    (3444, 1043, 2),
    (3447, 1045, 2),
    (3481, 1047, 2),
    (3484, 1049, 3),
    (3485, 1052, 2),
    (3487, 1054, 2),
    (3490, 1056, 2),
    (3491, 1058, 3),
    (3498, 1061, 2),
    (3500, 1063, 2),
    (3501, 1065, 2),
    (3502, 1067, 2),
    (3516, 1069, 3),
    (3521, 1072, 2),
    (3525, 1074, 2),
    (3526, 1076, 2),
    (3550, 1078, 2),
    (3551, 1080, 2),
    (3633, 1082, 2),
    (3634, 1084, 2),
    (3635, 1086, 2),
    (3636, 1088, 2),
    (3637, 1090, 2),
    (3638, 1092, 2),
    (3639, 1094, 2),
    (3640, 1096, 2),
    (3641, 1098, 2),
    (3642, 1100, 2),
    (3643, 1102, 2),
    (3644, 1104, 2),
    (3645, 1106, 2),
    (3646, 1108, 2),
    (3647, 1110, 2),
    (3648, 1112, 2),
    (3649, 1114, 2),
    (3650, 1116, 2),
    (3651, 1118, 2),
    (3652, 1120, 2),
    (3653, 1122, 2),
    (3654, 1124, 2),
    (3655, 1126, 2),
    (3656, 1128, 2),
    (3657, 1130, 2),
    (3658, 1132, 2),
    (3659, 1134, 2),
    (3660, 1136, 2),
    (3661, 1138, 2),
    (3662, 1140, 2),
    (3663, 1142, 2),
    (3664, 1144, 2),
    (3665, 1146, 2),
    (3666, 1148, 2),
    (3667, 1150, 2),
    (3668, 1152, 2),
    (3669, 1154, 2),
    (3670, 1156, 2),
    (3671, 1158, 2),
    (3672, 1160, 2),
    (3673, 1162, 2),
    (3674, 1164, 2),
    (3675, 1166, 2),
    (3676, 1168, 2),
    (3677, 1170, 2),
    (3678, 1172, 2),
    (3679, 1174, 2),
    (3688, 1176, 2),
    (3693, 1178, 2),
    (3702, 1180, 3),
    (3717, 1183, 2),
    (3730, 1185, 2),
    (3731, 1187, 2),
    (3732, 1189, 2),
    (3737, 1191, 3),
    (3752, 1194, 2),
    (3753, 1196, 3),
    (3756, 1199, 2),
    (3757, 1201, 2),
    (3758, 1203, 2),
    (3759, 1205, 2),
    (3799, 1207, 3),
    (3834, 1210, 2),
    (3846, 1212, 2),
    (3903, 1214, 3),
    (3909, 1217, 2),
    (3910, 1219, 2),
    (3912, 1221, 2),
    (3913, 1223, 2),
    (3918, 1225, 2),
    (3948, 1227, 2),
    (3949, 1229, 2),
    (3950, 1231, 2),
    (3951, 1233, 2),
    (3952, 1235, 2),
    (3953, 1237, 2),
    (3954, 1239, 2),
    (3955, 1241, 2),
    (3956, 1243, 2),
    (3957, 1245, 2),
    (3958, 1247, 2),
    (3959, 1249, 2),
    (3960, 1251, 2),
    (3961, 1253, 2),
    (3966, 1255, 2),
    (3973, 1257, 3),
    (3975, 1260, 2),
    (3976, 1262, 2),
    (3977, 1264, 2),
    (3978, 1266, 2),
    (3984, 1268, 3),
    (4020, 1271, 2),
    (4024, 1273, 2),
    (4025, 1275, 2),
    (4033, 1277, 2),
    (4038, 1279, 2),
    (4059, 1281, 2),
    (4063, 1283, 2),
    (4072, 1285, 2),
    (4086, 1287, 2),
    (4087, 1289, 2),
    (4088, 1291, 2),
    (4091, 1293, 3),
    (4093, 1296, 3),
    (4110, 1299, 2),
    (4111, 1301, 2),
    (4112, 1303, 2),
    (4118, 1305, 2),
    (4127, 1307, 2),
    (4157, 1309, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    3, 91, 90, 3, 91, 3, 91, 96, 107, 3, 3, 143, 3, 145, 3, 91,
    148, 3, 3, 164, 3, 165, 167, 4, 170, 3, 3, 173, 3, 181, 165, 167,
    4, 3, 91, 184, 186, 3, 3, 191, 3, 200, 3, 91, 96, 3, 205, 3,
    91, 209, 3, 205, 3, 91, 233, 235, 3, 3, 205, 3, 205, 3, 268, 3,
    91, 184, 297, 3, 299, 3, 301, 3, 303, 3, 305, 3, 3, 91, 184, 3,
    310, 3, 91, 184, 3, 322, 3, 364, 3, 368, 3, 205, 3, 268, 3, 374,
    91, 3, 91, 184, 379, 3, 381, 3, 383, 3, 3, 91, 96, 3, 390, 3,
    396, 3, 398, 3, 400, 3, 205, 3, 205, 3, 91, 209, 3, 374, 91, 3,
    205, 3, 91, 184, 3, 416, 3, 416, 3, 416, 3, 416, 3, 441, 3, 471,
    3, 473, 3, 374, 91, 3, 441, 3, 364, 3, 474, 3, 3, 416, 4, 3,
    374, 91, 3, 522, 3, 526, 3, 205, 3, 205, 3, 173, 3, 537, 3, 165,
    167, 4, 3, 550, 3, 556, 3, 441, 3, 550, 3, 556, 3, 4, 3, 596,
    3, 473, 3, 310, 3, 599, 3, 364, 3, 601, 3, 368, 3, 374, 91, 3,
    606, 4, 3, 91, 184, 3, 91, 3, 205, 3, 205, 3, 205, 3, 205, 3,
    205, 3, 205, 3, 205, 4, 3, 416, 3, 205, 3, 441, 3, 441, 3, 441,
    3, 441, 3, 205, 3, 91, 668, 3, 670, 3, 364, 3, 364, 3, 682, 3,
    684, 3, 91, 3, 191, 3, 200, 3, 689, 91, 3, 691, 91, 3, 364, 3,
    364, 3, 364, 3, 701, 3, 691, 91, 3, 709, 91, 3, 91, 184, 3, 711,
    3, 91, 3, 205, 3, 91, 96, 3, 91, 209, 3, 91, 668, 3, 670, 3,
    91, 233, 3, 721, 3, 91, 184, 186, 3, 3, 91, 184, 3, 732, 3, 322,
    3, 734, 3, 736, 3, 599, 3, 91, 740, 3, 374, 91, 3, 91, 184, 3,
    91, 184, 3, 441, 3, 770, 3, 91, 668, 3, 164, 3, 364, 3, 778, 3,
    670, 3, 91, 96, 3, 390, 3, 398, 3, 396, 3, 400, 3, 91, 782, 3,
    374, 91, 3, 599, 3, 599, 3, 91, 209, 3, 91, 3, 205, 3, 682, 3,
    682, 3, 794, 3, 808, 3, 808, 3, 808, 3, 808, 3, 808, 3, 808, 3,
    808, 3, 808, 3, 808, 3, 91, 184, 3, 883, 3, 886, 3, 91, 3, 364,
    3, 364, 3, 689, 91, 3, 682, 3, 709, 91, 3, 91, 3, 91, 96, 3,
    91, 209, 3, 91, 668, 3, 91, 233, 3, 91, 184, 3, 91, 740, 3, 91,
    3, 91, 96, 3, 91, 782, 3, 441, 3, 929, 3, 931, 3, 91, 668, 3,
    441, 3, 963, 3, 965, 3, 967, 3, 974, 3, 976, 3, 979, 3, 981, 3,
    91, 184, 3, 173, 3, 983, 3, 214, 3, 214, 3, 1009, 3, 1011, 3, 596,
    3, 473, 3, 441, 3, 374, 91, 3, 473, 3, 1014, 3, 441, 3, 91, 1017,
    3, 929, 3, 1019, 3, 441, 3, 364, 3, 1014, 3, 1026, 3, 471, 3, 1028,
    1030, 3, 3, 374, 91, 3, 1033, 3, 205, 3, 205, 3, 205, 3, 441, 3,
    205, 3, 205, 3, 205, 3, 1078, 3, 1078, 3, 441, 3, 91, 184, 3, 471,
    3, 374, 91, 3, 1109, 3, 1122, 3, 734, 3, 599, 3, 1125, 3, 1127, 3,
    1131, 3, 1134, 186, 3, 186, 3, 3, 1131, 3, 1134, 3, 1139, 3, 1141, 3,
    1109, 3, 1134, 3, 473, 3, 441, 3, 1148, 3, 1150, 3, 1152, 3, 1155, 3,
    441, 3, 441, 3, 441, 3, 364, 3, 441, 3, 441, 3, 441, 3, 441, 1161,
    3, 3, 374, 91, 3, 374, 91, 3, 711, 3, 1028, 3, 441, 3, 441, 3,
    701, 3, 441, 3, 441, 3, 701, 3, 441, 3, 701, 3, 441, 3, 701, 3,
    701, 3, 522, 3, 522, 3, 364, 3, 364, 3, 1181, 3, 1183, 3, 1185, 3,
    1187, 3, 682, 3, 770, 3, 682, 3, 682, 3, 1193, 3, 682, 3, 770, 3,
    441, 3, 701, 1204, 3, 1206, 3, 1206, 3, 3, 91, 1210, 3, 3, 374, 91,
    3, 1218, 3, 1221, 3, 441, 3, 1122, 3, 1224, 3, 374, 91, 3, 441, 3,
    1218, 3, 794, 3, 441, 3, 1224, 3, 945, 4, 3, 601, 3, 736, 3, 191,
    3, 91, 184, 3, 1264, 3, 441, 3, 979, 3, 974, 3, 976, 3, 1266, 3,
    981, 3, 205, 3, 191, 3, 91, 184, 3, 173, 3, 191, 3, 1274, 3, 1276,
    3, 1278, 3, 91, 3, 191, 3, 205, 3, 91, 184, 3, 173, 3, 1280, 3,
    1282, 3, 473, 3, 91, 184, 3, 1291, 3, 91, 184, 3, 1141, 3, 1319, 3,
    1321, 3, 1141, 3, 1319, 3, 1141, 3, 441, 3, 1, 4, 5, 3, 214, 1357,
    3, 3, 1359, 3, 1362, 3, 1, 865, 4, 5, 3, 865, 4, 3, 883, 3,
    1193, 3, 883, 3, 1384, 3, 1386, 3, 1386, 3, 1384, 3, 1389, 3, 1391, 3,
    1393, 3, 91, 184, 3, 91, 184, 186, 3, 3, 473, 186, 3, 3, 1321, 3,
    1396, 3, 1398, 3, 883, 3, 1412, 3, 1414, 3, 1420, 4, 3, 1193, 3, 1026,
    3, 1026, 3, 1426, 3, 965, 3, 963, 3, 967, 3, 1430, 3, 526, 3, 1026,
    3, 1446, 3, 441, 3, 91, 209, 3, 374, 91, 3, 441, 3, 1460, 3, 1224,
    3, 1224, 3, 1412, 3, 374, 91, 3, 441, 3, 778, 3, 1497, 3, 1500, 3,
    1502, 3, 1500, 3, 1500, 3, 473, 3, 374, 91, 3, 1014, 3, 473, 3, 374,
    91, 3, 1014, 3, 1508, 3, 91, 184, 3, 91, 184, 3, 91, 209, 3, 91,
    209, 3, 441, 3, 91, 184, 3, 91, 184, 3, 91, 209, 3, 91, 209, 3,
    91, 184, 3, 91, 184, 3, 91, 3, 91, 1204, 3, 1206, 3, 1206, 3, 3,
    91, 1210, 3, 3, 1586, 186, 3, 3, 1588, 3, 1591, 3, 1593, 3, 1595, 3,
    1597, 3, 1122, 3, 734, 3, 599, 3, 1600, 3, 1602, 3, 1604, 3, 883, 3,
    1606, 3, 886, 3, 1608, 3, 1610, 3, 1612, 3, 164, 3, 1224, 3, 1122, 3,
    734, 3, 599, 3, 1426, 3, 1426, 3, 1626, 3, 374, 91, 3, 205, 3, 441,
    3, 473, 3, 374, 91, 3, 1122, 3, 164, 3, 1224, 3, 1628, 3, 91, 209,
    3, 1009, 3, 364, 3, 1011, 1642, 3, 148, 3, 3, 967, 3, 1714, 3, 1716,
    3, 963, 3, 1430, 3, 1718, 3, 965, 1720, 3, 1722, 3, 1724, 3, 3, 1726,
    3, 1728, 3, 1730, 3, 1732, 3, 1734, 3, 1736, 3, 1738, 3, 1740, 3, 1742,
    3, 1744, 3, 1746, 1748, 3, 1750, 3, 1752, 3, 3, 1754, 3, 1756, 3, 1758,
    3, 1760, 3, 1762, 3, 1764, 3, 1766, 1768, 3, 3, 1770, 1772, 3, 1774, 3,
    1776, 3, 3, 1778, 3, 1780, 3, 1782, 3, 1784, 3, 1786, 3, 1788, 3, 1790,
    3, 1792, 3, 1794, 3, 1796, 3, 1798, 3, 416, 186, 3, 3, 4, 176, 3,
    91, 3, 268, 3, 310, 3, 736, 3, 374, 91, 3, 473, 3, 91, 184, 3,
    1291, 3, 1193, 3, 1716, 3, 1794, 3, 374, 91, 3, 808, 3, 441, 3, 91,
    184, 1642, 3, 1357, 3, 3, 929, 3, 931, 3, 601, 3, 1600, 3, 1602, 3,
    1604, 3, 883, 3, 1606, 3, 886, 3, 1608, 3, 1610, 3, 1612, 3, 1122, 3,
    734, 3, 599, 3, 164, 3, 1224, 3, 1122, 3, 91, 184, 3, 1398, 3, 1602,
    3, 883, 3, 886, 3, 689, 91, 3, 441, 3, 441, 186, 3, 3, 473, 3,
    473, 3, 441, 3, 441, 3, 441, 3, 1122, 3, 734, 3, 599, 3, 4, 1900,
    3, 91, 1017, 3, 1502, 3, 164, 3, 1904, 3, 1908, 3, 1908, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_]*[A-Z][A-Z0-9_]*"#, // [2]
    r#"^(ABSOLUTE|ACTION|ADD|AFTER|ALL|ALLOCATE|ALTER|AND|ANY|APPEND|ARE|ARRAY|AS|ASC|ASENSITIVE|ASSERTION|AT|ATTRIBUTE|AUTHID|AUTHORIZATION|BEFORE|BEGIN|BETWEEN|BIGINT|BINARY|BIT|BLOB|BLOCKED|BOOL|BOOLEAN|BOTH|BY|BYTE|CALL|CALLED|CARDINALITY|CASCADE|CASCADED|CASE|CASESPECIFIC|CAST|CATALOG|CHAIN|CHAR|CHARACTER|CHARACTERISTICS|CHARACTER_SET_CATALOG|CHARACTER_SET_NAME|CHARACTER_SET_SCHEMA|CHECK|CHECKED|CLOB|CLOSE|COALESCE|COLLATE|COLLATION|COLLATION_CATALOG|COLLATION_NAME|COLLATION_SCHEMA|COLUMN|COMMIT|CONDITION|CONNECTION|CONNECT_BY_ISCYCLE|CONNECT_BY_ISLEAF|CONNECT_BY_ROOT|CONSTANT|CONSTRAINT|CONSTRAINTS|CONSTRAINT_STATE_DEFAULT|CONSTRUCTOR|CONTAINS|CONTINUE|CONTROL|CONVERT|CORRESPONDING|CREATE|CROSS|CS|CSV|CUBE|CURRENT|CURRENT_DATE|CURRENT_PATH|CURRENT_ROLE|CURRENT_SCHEMA|CURRENT_SESSION|CURRENT_STATEMENT|CURRENT_TIME|CURRENT_TIMESTAMP|CURRENT_USER|CURSOR|CYCLE|DATA|DATALINK|DATE|DATETIME_INTERVAL_CODE|DATETIME_INTERVAL_PRECISION|DAY|DBTIMEZONE|DEALLOCATE|DEC|DECIMAL|DECLARE|DEFAULT|DEFAULT_LIKE_ESCAPE_CHARACTER|DEFERRABLE|DEFERRED|DEFINED|DEFINER|DELETE|DEREF|DERIVED|DESC|DESCRIBE|DESCRIPTOR|DETERMINISTIC|DISABLE|DISABLED|DISCONNECT|DISPATCH|DISTINCT|DLURLCOMPLETE|DLURLPATH|DLURLPATHONLY|DLURLSCHEME|DLURLSERVER|DLVALUE|DO|DOMAIN|DOUBLE|DROP|DYNAMIC|DYNAMIC_FUNCTION|DYNAMIC_FUNCTION_CODE|EACH|ELSE|ELSEIF|ELSIF|EMITS|ENABLE|ENABLED|END|END-EXEC|ENDIF|ENFORCE|EQUALS|ERRORS|ESCAPE|EXCEPT|EXCEPTION|EXEC|EXECUTE|EXISTS|EXIT|EXPORT|EXTERNAL|EXTRACT|FALSE|FBV|FETCH|FILE|FINAL|FIRST|FLOAT|FOLLOWING|FOR|FORALL|FORCE|FORMAT|FOUND|FREE|FROM|FS|FULL|FUNCTION|GENERAL|GENERATED|GEOMETRY|GET|GLOBAL|GO|GOTO|GRANT|GRANTED|GROUP|GROUPING|GROUPS|GROUP_CONCAT|HASHTYPE|HASHTYPE_FORMAT|HAVING|HIGH|HOLD|HOUR|IDENTITY|IF|IFNULL|IMMEDIATE|IMPERSONATE|IMPLEMENTATION|IMPORT|IN|INDEX|INDICATOR|INNER|INOUT|INPUT|INSENSITIVE|INSERT|INSTANCE|INSTANTIABLE|INT|INTEGER|INTEGRITY|INTERSECT|INTERVAL|INTO|INVERSE|INVOKER|IS|ITERATE|JOIN|KEY_MEMBER|KEY_TYPE|LARGE|LAST|LATERAL|LDAP|LEADING|LEAVE|LEFT|LEVEL|LIKE|LIMIT|LISTAGG|LOCAL|LOCALTIME|LOCALTIMESTAMP|LOCATOR|LOG|LONGVARCHAR|LOOP|LOW|MAP|MATCH|MATCHED|MERGE|METHOD|MINUS|MINUTE|MOD|MODIFIES|MODIFY|MODULE|MONTH|NAMES|NATIONAL|NATURAL|NCHAR|NCLOB|NEW|NEXT|NLS_DATE_FORMAT|NLS_DATE_LANGUAGE|NLS_FIRST_DAY_OF_WEEK|NLS_NUMERIC_CHARACTERS|NLS_TIMESTAMP_FORMAT|NO|NOCYCLE|NOLOGGING|NONE|NOT|NULL|NULLIF|NUMBER|NUMERIC|NVARCHAR|NVARCHAR2|OBJECT|OF|OFF|OLD|ON|ONLY|OPEN|OPTION|OPTIONS|OR|ORDER|ORDERING|ORDINALITY|OTHERS|OUT|OUTER|OUTPUT|OVER|OVERLAPS|OVERLAY|OVERRIDING|PAD|PARALLEL_ENABLE|PARAMETER|PARAMETER_SPECIFIC_CATALOG|PARAMETER_SPECIFIC_NAME|PARAMETER_SPECIFIC_SCHEMA|PARTIAL|PARTITION|PATH|PERMISSION|PLACING|PLUS|POSITION|PRECEDING|PREFERRING|PREPARE|PRESERVE|PRIOR|PRIVILEGES|PROCEDURE|PROFILE|QUALIFY|RANDOM|RANGE|READ|READS|REAL|RECOVERY|RECURSIVE|REF|REFERENCES|REFERENCING|REFRESH|REGEXP_LIKE|RELATIVE|RELEASE|RENAME|REPEAT|REPLACE|RESTORE|RESTRICT|RESULT|RETURN|RETURNED_LENGTH|RETURNED_OCTET_LENGTH|RETURNS|REVOKE|RIGHT|ROLLBACK|ROLLUP|ROUTINE|ROW|ROWS|ROWTYPE|SAVEPOINT|SCHEMA|SCOPE|SCOPE_USER|SCRIPT|SCROLL|SEARCH|SECOND|SECTION|SECURITY|SELECT|SELECTIVE|SELF|SENSITIVE|SEPARATOR|SEQUENCE|SESSION|SESSIONTIMEZONE|SESSION_USER|SET|SETS|SHORTINT|SIMILAR|SMALLINT|SOME|SOURCE|SPACE|SPECIFIC|SPECIFICTYPE|SQL|SQLEXCEPTION|SQLSTATE|SQLWARNING|SQL_BIGINT|SQL_BIT|SQL_CHAR|SQL_DATE|SQL_DECIMAL|SQL_DOUBLE|SQL_FLOAT|SQL_INTEGER|SQL_LONGVARCHAR|SQL_NUMERIC|SQL_PREPROCESSOR_SCRIPT|SQL_REAL|SQL_SMALLINT|SQL_TIMESTAMP|SQL_TINYINT|SQL_TYPE_DATE|SQL_TYPE_TIMESTAMP|SQL_VARCHAR|START|STATE|STATEMENT|STATIC|STRUCTURE|STYLE|SUBSTRING|SUBTYPE|SYSDATE|SYSTEM|SYSTEM_USER|SYSTIMESTAMP|TABLE|TEMPORARY|TEXT|THEN|TIME|TIMESTAMP|TIMEZONE_HOUR|TIMEZONE_MINUTE|TINYINT|TO|TRAILING|TRANSACTION|TRANSFORM|TRANSFORMS|TRANSLATION|TREAT|TRIGGER|TRIM|TRUE|TRUNCATE|UNDER|UNION|UNIQUE|UNKNOWN|UNLINK|UNNEST|UNTIL|UPDATE|USAGE|USER|USING|VALUE|VALUES|VARCHAR|VARCHAR2|VARRAY|VERIFY|VIEW|WHEN|WHENEVER|WHERE|WHILE|WINDOW|WITH|WITHIN|WITHOUT|WORK|YEAR|YES|ZONE)$"#, // [3]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [4]
    r#"[A-Z][A-Z0-9_]*"#, // [5]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 9, 10, 11, 12, 8, 9,
    10, 11, 12, 19, 21, 23, 25, 27, 29, 31,
    33, 35, 36, 39, 40, 43, 45, 47, 48, 51,
    53, 54, 57, 58, 61, 63, 19, 64, 65, 64,
    65, 69, 70, 65, 40, 73, 7, 19, 65, 40,
    74, 74, 78, 80, 19, 65, 74, 82, 78, 8,
    73, 10, 61, 11, 12, 19, 64, 27, 65, 23,
    21, 25, 84, 64, 65, 64, 65, 31, 8, 10,
    85, 88, 94, 98, 100, 102, 104, 9, 10, 110,
    112, 114, 120, 122, 125, 127, 136, 138, 40, 141,
    141, 146, 149, 150, 151, 152, 153, 154, 155, 156,
    157, 158, 159, 160, 161, 163, 169, 171, 177, 180,
    187, 188, 192, 193, 194, 195, 196, 197, 198, 201,
    204, 207, 210, 48, 212, 213, 214, 214, 213, 210,
    219, 222, 227, 228, 227, 228, 237, 238, 239, 194,
    240, 241, 242, 243, 244, 245, 246, 247, 196, 248,
    249, 197, 250, 251, 252, 253, 254, 255, 214, 180,
    237, 258, 238, 239, 194, 241, 242, 243, 244, 245,
    259, 260, 246, 247, 196, 248, 249, 197, 250, 251,
    252, 261, 253, 254, 255, 198, 264, 266, 65, 48,
    269, 270, 270, 171, 214, 269, 275, 276, 277, 278,
    279, 280, 281, 282, 283, 284, 285, 286, 287, 288,
    289, 290, 291, 292, 293, 69, 152, 19, 65, 294,
    295, 294, 295, 152, 65, 294, 295, 307, 308, 309,
    307, 309, 307, 309, 315, 316, 319, 65, 320, 319,
    320, 323, 320, 323, 326, 320, 329, 330, 333, 335,
    336, 337, 338, 339, 340, 341, 342, 343, 344, 345,
    346, 347, 348, 349, 350, 351, 352, 353, 354, 355,
    356, 357, 358, 359, 360, 361, 362, 363, 366, 307,
    269, 270, 367, 207, 309, 29, 152, 294, 171, 161,
    370, 161, 370, 370, 169, 100, 384, 385, 259, 386,
    48, 384, 388, 222, 391, 394, 388, 259, 385, 404,
    386, 407, 409, 411, 210, 82, 48, 412, 413, 415,
    418, 419, 420, 421, 422, 413, 213, 423, 424, 425,
    426, 427, 428, 429, 423, 424, 425, 422, 429, 428,
    426, 427, 443, 445, 447, 449, 450, 451, 452, 453,
    454, 455, 456, 457, 458, 459, 460, 461, 462, 463,
    465, 466, 467, 350, 477, 478, 479, 480, 335, 481,
    482, 422, 413, 213, 423, 424, 425, 426, 204, 427,
    326, 483, 484, 428, 429, 485, 326, 484, 482, 426,
    204, 427, 483, 485, 490, 491, 492, 493, 494, 495,
    496, 497, 498, 499, 500, 501, 502, 507, 508, 336,
    484, 337, 511, 512, 513, 512, 511, 513, 517, 490,
    518, 338, 518, 338, 171, 21, 525, 527, 529, 530,
    412, 465, 532, 532, 171, 21, 525, 539, 540, 541,
    542, 543, 544, 548, 552, 554, 558, 559, 560, 562,
    563, 564, 565, 450, 566, 566, 339, 569, 570, 571,
    572, 573, 574, 575, 576, 577, 578, 340, 579, 581,
    582, 583, 584, 586, 588, 589, 590, 591, 307, 367,
    207, 309, 29, 366, 483, 264, 78, 51, 8, 73,
    9, 10, 69, 608, 609, 610, 482, 426, 485, 621,
    622, 624, 626, 627, 628, 629, 632, 633, 632, 633,
    539, 275, 276, 277, 637, 237, 638, 192, 195, 193,
    198, 197, 196, 194, 646, 647, 648, 649, 650, 651,
    652, 653, 654, 655, 656, 657, 658, 661, 663, 665,
    673, 638, 8, 674, 675, 110, 112, 676, 638, 674,
    675, 676, 638, 674, 676, 674, 676, 675, 687, 693,
    695, 169, 483, 698, 705, 707, 713, 716, 718, 11,
    718, 722, 724, 725, 725, 724, 729, 171, 169, 722,
    738, 744, 27, 23, 25, 746, 104, 102, 747, 367,
    748, 548, 748, 751, 753, 755, 746, 747, 367, 367,
    746, 746, 747, 747, 760, 762, 764, 753, 765, 765,
    5, 772, 773, 48, 773, 776, 716, 776, 780, 784,
    785, 786, 787, 788, 788, 790, 790, 278, 795, 279,
    796, 797, 280, 281, 282, 798, 283, 284, 799, 800,
    801, 342, 343, 353, 354, 343, 342, 353, 354, 344,
    812, 345, 348, 349, 357, 358, 361, 359, 336, 337,
    341, 360, 341, 360, 825, 827, 829, 832, 834, 836,
    838, 346, 347, 338, 339, 340, 351, 352, 355, 356,
    362, 363, 338, 352, 355, 356, 362, 363, 842, 362,
    363, 352, 355, 356, 351, 171, 850, 842, 850, 842,
    850, 153, 154, 155, 156, 157, 158, 159, 836, 856,
    857, 858, 859, 834, 860, 838, 861, 862, 832, 867,
    869, 870, 871, 285, 872, 873, 874, 875, 525, 258,
    238, 877, 878, 879, 882, 885, 887, 888, 889, 890,
    891, 892, 893, 895, 896, 897, 315, 898, 899, 900,
    901, 319, 902, 903, 904, 905, 906, 907, 908, 909,
    910, 912, 913, 330, 914, 914, 151, 10, 918, 919,
    920, 921, 922, 923, 924, 925, 927, 316, 932, 933,
    934, 935, 937, 938, 939, 941, 942, 943, 944, 946,
    947, 948, 949, 950, 951, 952, 953, 955, 956, 957,
    958, 959, 960, 171, 27, 23, 21, 25, 525, 539,
    968, 787, 949, 970, 971, 825, 972, 10, 787, 949,
    970, 971, 825, 972, 949, 971, 972, 787, 970, 825,
    978, 984, 722, 367, 988, 693, 367, 480, 367, 423,
    424, 367, 998, 999, 423, 424, 999, 998, 482, 422,
    413, 213, 423, 424, 425, 426, 204, 427, 483, 428,
    429, 485, 423, 424, 999, 341, 511, 207, 360, 512,
    513, 214, 180, 1032, 1034, 187, 125, 1035, 1036, 633,
    1036, 633, 1036, 125, 1035, 1035, 628, 629, 628, 629,
    970, 1044, 370, 1046, 1046, 753, 1048, 1048, 1050, 539,
    1050, 1053, 204, 337, 264, 530, 933, 1054, 207, 391,
    753, 1055, 1056, 1057, 1058, 539, 180, 204, 337, 264,
    933, 207, 753, 180, 1061, 887, 1061, 1054, 1055, 1057,
    1057, 1054, 1055, 753, 1056, 1056, 1058, 1069, 171, 19,
    579, 366, 609, 64, 27, 65, 31, 960, 35, 1070,
    40, 70, 33, 23, 58, 1071, 1072, 1073, 1074, 295,
    1075, 141, 1076, 21, 1058, 25, 525, 539, 698, 1080,
    1080, 1082, 1085, 1086, 1087, 1070, 1090, 1091, 1092, 308,
    1095, 1096, 1097, 1098, 1048, 1102, 1091, 1103, 1103, 1098,
    1107, 366, 632, 950, 53, 1110, 1111, 1112, 1113, 163,
    885, 1114, 633, 1115, 882, 539, 1112, 1110, 1114, 950,
    1111, 1113, 1115, 1130, 1136, 1143, 1144, 626, 1146, 1154,
    517, 1156, 1157, 1163, 1164, 1166, 1146, 693, 1170, 1173,
    924, 925, 394, 5, 43, 335, 336, 337, 338, 339,
    340, 341, 342, 343, 344, 345, 346, 347, 348, 349,
    350, 351, 352, 353, 354, 355, 356, 357, 358, 359,
    360, 361, 362, 363, 1196, 1197, 1198, 1199, 1200, 1201,
    1202, 1208, 1213, 1214, 1215, 1216, 1220, 1044, 1110, 1112,
    163, 1114, 1222, 1222, 1226, 1230, 1231, 1232, 1233, 1234,
    1235, 1236, 1237, 1238, 1239, 1240, 239, 1241, 1242, 1243,
    1244, 1245, 1246, 1044, 1112, 163, 1114, 1222, 1247, 1248,
    1249, 857, 1250, 1252, 1163, 1164, 1256, 1257, 1261, 391,
    1058, 972, 971, 765, 11, 785, 391, 1058, 787, 970,
    388, 122, 180, 122, 180, 449, 892, 1285, 1286, 627,
    1287, 1288, 1289, 171, 269, 21, 525, 539, 1292, 1293,
    1294, 1111, 836, 857, 859, 834, 838, 832, 836, 857,
    859, 838, 836, 857, 859, 859, 857, 859, 838, 1297,
    1298, 1299, 1300, 1304, 1305, 1306, 1307, 1308, 1309, 1310,
    1311, 1312, 1315, 88, 784, 626, 628, 629, 1330, 1331,
    1332, 1333, 1334, 1335, 1336, 1337, 61, 765, 281, 214,
    1340, 1341, 1342, 1343, 1344, 1345, 1346, 1347, 1156, 1348,
    1349, 1350, 286, 722, 988, 988, 1354, 1355, 1361, 1363,
    1364, 287, 1365, 1366, 1367, 1368, 1369, 1370, 1371, 1372,
    1373, 1374, 1375, 1376, 1377, 1378, 1379, 1380, 1381, 1071,
    27, 25, 1399, 1400, 1401, 1113, 1402, 1403, 1404, 1405,
    1406, 1407, 1408, 1409, 1415, 1416, 1418, 240, 241, 242,
    243, 244, 245, 1422, 1423, 27, 23, 1071, 25, 171,
    21, 1432, 1433, 482, 1435, 288, 1436, 1438, 1439, 1440,
    1441, 1442, 1443, 1451, 1452, 1453, 1072, 1455, 1456, 1457,
    370, 1342, 1044, 1110, 1112, 1114, 1098, 1048, 1061, 1222,
    1462, 1463, 1464, 1465, 1467, 1468, 1470, 1471, 1472, 1473,
    1474, 1476, 1477, 1478, 1479, 1480, 1481, 260, 1483, 425,
    180, 1486, 1487, 1488, 1489, 1491, 1492, 1493, 1494, 1495,
    1498, 1298, 1493, 998, 1493, 998, 1248, 1373, 1073, 687,
    9, 138, 11, 127, 1510, 1511, 1513, 246, 36, 43,
    1514, 1514, 1516, 247, 1517, 1520, 1521, 1522, 1523, 1524,
    1526, 1527, 1074, 1074, 1075, 1075, 1531, 1532, 1533, 1534,
    330, 1422, 914, 48, 626, 1480, 1538, 1538, 1543, 1544,
    1545, 1546, 1547, 1548, 1549, 1550, 1551, 1552, 1553, 1554,
    1556, 214, 1556, 1558, 1559, 78, 51, 8, 73, 9,
    10, 11, 69, 12, 1560, 1561, 1562, 1563, 1564, 1565,
    1566, 1567, 1076, 1569, 1570, 289, 1571, 1572, 479, 63,
    1576, 1577, 1578, 1579, 248, 249, 1580, 1581, 1582, 1583,
    1584, 36, 901, 968, 1297, 539, 1297, 539, 308, 309,
    366, 950, 102, 1111, 1112, 1113, 163, 1115, 882, 539,
    171, 539, 1615, 1616, 1618, 1621, 1623, 1624, 291, 1115,
    950, 1113, 1633, 1634, 1637, 1638, 1640, 250, 1645, 1646,
    1647, 1648, 1649, 1650, 1651, 1652, 1653, 1654, 1655, 1656,
    1657, 1658, 1659, 251, 1660, 1661, 1662, 1663, 1664, 1665,
    1666, 1667, 1668, 1669, 1670, 1671, 1672, 1673, 1674, 1675,
    1676, 1677, 1678, 1679, 1680, 1681, 1682, 1683, 1684, 1685,
    1686, 1687, 1688, 1689, 1690, 1691, 1692, 1693, 1694, 1695,
    1696, 252, 1697, 1698, 1699, 1700, 1701, 1702, 1703, 1704,
    1705, 1706, 1707, 1708, 1709, 1710, 1712, 609, 1076, 1799,
    1800, 1801, 1802, 1804, 1805, 1806, 1807, 1808, 1809, 1810,
    1811, 1812, 1813, 292, 1815, 293, 307, 308, 309, 705,
    308, 309, 705, 308, 367, 207, 309, 367, 207, 309,
    29, 367, 207, 1827, 1828, 261, 1831, 1834, 827, 360,
    539, 1836, 1836, 254, 255, 253, 1838, 1839, 1840, 1841,
    1842, 1843, 1844, 1845, 1846, 1157, 1847, 1851, 1852, 1853,
    1854, 1855, 1856, 1860, 102, 1342, 836, 834, 832, 1865,
    1866, 1868, 1869, 1873, 1874, 1875, 1876, 1877, 1878, 1879,
    1880, 1881, 1887, 1888, 1890, 1891, 1893, 1894, 1895, 861,
    1899, 366, 632, 950, 53, 1110, 1111, 1112, 1113, 163,
    885, 1114, 633, 1115, 539, 163, 320, 309, 705, 1912,
    1913, 1914, 1915, 1916,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 5, token_types_start: 8, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 8, raw_values_count: 1, token_types_start: 9, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 9, raw_values_count: 1, token_types_start: 10, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 10, raw_values_count: 1, token_types_start: 11, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 11, raw_values_count: 1, token_types_start: 12, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 12, raw_values_count: 1, token_types_start: 13, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 13, raw_values_count: 1, token_types_start: 14, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 14, raw_values_count: 1, token_types_start: 15, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 15, raw_values_count: 1, token_types_start: 16, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 16, raw_values_count: 1, token_types_start: 17, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 17, raw_values_count: 1, token_types_start: 18, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 18, raw_values_count: 1, token_types_start: 19, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 19, raw_values_count: 1, token_types_start: 20, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 20, raw_values_count: 1, token_types_start: 21, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 21, raw_values_count: 1, token_types_start: 22, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 22, raw_values_count: 1, token_types_start: 23, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 23, raw_values_count: 1, token_types_start: 24, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 24, raw_values_count: 1, token_types_start: 25, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 25, raw_values_count: 1, token_types_start: 26, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 26, raw_values_count: 1, token_types_start: 27, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 27, raw_values_count: 1, token_types_start: 28, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 28, raw_values_count: 1, token_types_start: 29, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 29, raw_values_count: 1, token_types_start: 30, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 30, raw_values_count: 1, token_types_start: 31, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 31, raw_values_count: 1, token_types_start: 32, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 36, raw_values_count: 3, token_types_start: 39, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [37]
    SimpleHintData { raw_values_start: 43, raw_values_count: 2, token_types_start: 45, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 45, raw_values_count: 1, token_types_start: 46, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 46, raw_values_count: 5, token_types_start: 51, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 54, raw_values_count: 3, token_types_start: 57, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 57, raw_values_count: 1, token_types_start: 58, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 58, raw_values_count: 7, token_types_start: 65, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 65, raw_values_count: 7, token_types_start: 72, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 72, raw_values_count: 1, token_types_start: 73, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 73, raw_values_count: 2, token_types_start: 75, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 75, raw_values_count: 3, token_types_start: 78, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 78, raw_values_count: 2, token_types_start: 80, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 80, raw_values_count: 1, token_types_start: 81, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 81, raw_values_count: 1, token_types_start: 82, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 82, raw_values_count: 1, token_types_start: 83, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 83, raw_values_count: 1, token_types_start: 84, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 87, raw_values_count: 2, token_types_start: 89, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 90, raw_values_count: 1, token_types_start: 91, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 91, raw_values_count: 1, token_types_start: 92, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 92, raw_values_count: 1, token_types_start: 93, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 93, raw_values_count: 1, token_types_start: 94, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 94, raw_values_count: 1, token_types_start: 95, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 95, raw_values_count: 1, token_types_start: 96, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 96, raw_values_count: 1, token_types_start: 97, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 97, raw_values_count: 1, token_types_start: 98, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 98, raw_values_count: 2, token_types_start: 100, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 100, raw_values_count: 1, token_types_start: 101, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 101, raw_values_count: 1, token_types_start: 102, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 102, raw_values_count: 1, token_types_start: 103, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 103, raw_values_count: 1, token_types_start: 104, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 104, raw_values_count: 1, token_types_start: 105, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 105, raw_values_count: 1, token_types_start: 106, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 106, raw_values_count: 1, token_types_start: 107, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 107, raw_values_count: 1, token_types_start: 108, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 108, raw_values_count: 1, token_types_start: 109, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 109, raw_values_count: 1, token_types_start: 110, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 110, raw_values_count: 1, token_types_start: 111, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 111, raw_values_count: 1, token_types_start: 112, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 112, raw_values_count: 1, token_types_start: 113, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 113, raw_values_count: 1, token_types_start: 114, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 114, raw_values_count: 1, token_types_start: 115, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 115, raw_values_count: 1, token_types_start: 116, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 116, raw_values_count: 1, token_types_start: 117, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 118, raw_values_count: 1, token_types_start: 119, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 119, raw_values_count: 0, token_types_start: 119, token_types_count: 1 }, // [89]
    SimpleHintData { raw_values_start: 120, raw_values_count: 1, token_types_start: 121, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 121, raw_values_count: 1, token_types_start: 122, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 122, raw_values_count: 7, token_types_start: 129, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 131, raw_values_count: 1, token_types_start: 132, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 132, raw_values_count: 2, token_types_start: 134, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 135, raw_values_count: 1, token_types_start: 136, token_types_count: 1 }, // [98]
    SimpleHintData { raw_values_start: 137, raw_values_count: 0, token_types_start: 137, token_types_count: 1 }, // [99]
    SimpleHintData { raw_values_start: 138, raw_values_count: 1, token_types_start: 139, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 139, raw_values_count: 1, token_types_start: 140, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 140, raw_values_count: 1, token_types_start: 141, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 141, raw_values_count: 1, token_types_start: 142, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 142, raw_values_count: 2, token_types_start: 144, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 144, raw_values_count: 1, token_types_start: 145, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 145, raw_values_count: 1, token_types_start: 146, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 146, raw_values_count: 22, token_types_start: 168, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 168, raw_values_count: 0, token_types_start: 168, token_types_count: 2 }, // [108]
    SimpleHintData { raw_values_start: 170, raw_values_count: 26, token_types_start: 196, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 198, raw_values_count: 2, token_types_start: 200, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 200, raw_values_count: 2, token_types_start: 202, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 1 }, // [115]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 206, raw_values_count: 20, token_types_start: 226, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 226, raw_values_count: 5, token_types_start: 231, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 233, raw_values_count: 4, token_types_start: 237, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 237, raw_values_count: 3, token_types_start: 240, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 240, raw_values_count: 2, token_types_start: 242, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 244, raw_values_count: 2, token_types_start: 246, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 246, raw_values_count: 3, token_types_start: 249, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 249, raw_values_count: 2, token_types_start: 251, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 251, raw_values_count: 2, token_types_start: 253, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 253, raw_values_count: 1, token_types_start: 254, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 257, raw_values_count: 1, token_types_start: 258, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 258, raw_values_count: 1, token_types_start: 259, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 259, raw_values_count: 29, token_types_start: 288, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 288, raw_values_count: 8, token_types_start: 296, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 296, raw_values_count: 2, token_types_start: 298, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 298, raw_values_count: 3, token_types_start: 301, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 301, raw_values_count: 2, token_types_start: 303, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 304, raw_values_count: 2, token_types_start: 306, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 306, raw_values_count: 5, token_types_start: 311, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 312, raw_values_count: 2, token_types_start: 314, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 315, raw_values_count: 0, token_types_start: 315, token_types_count: 1 }, // [147]
    SimpleHintData { raw_values_start: 316, raw_values_count: 1, token_types_start: 317, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 317, raw_values_count: 1, token_types_start: 318, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 323, raw_values_count: 1, token_types_start: 324, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 324, raw_values_count: 3, token_types_start: 327, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 327, raw_values_count: 1, token_types_start: 328, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 330, raw_values_count: 1, token_types_start: 331, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 331, raw_values_count: 1, token_types_start: 332, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 332, raw_values_count: 1, token_types_start: 333, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 333, raw_values_count: 1, token_types_start: 334, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 334, raw_values_count: 10, token_types_start: 344, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 344, raw_values_count: 1, token_types_start: 345, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 345, raw_values_count: 1, token_types_start: 346, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 346, raw_values_count: 1, token_types_start: 347, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 347, raw_values_count: 1, token_types_start: 348, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 348, raw_values_count: 1, token_types_start: 349, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 349, raw_values_count: 1, token_types_start: 350, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 350, raw_values_count: 1, token_types_start: 351, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 351, raw_values_count: 1, token_types_start: 352, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 352, raw_values_count: 1, token_types_start: 353, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 353, raw_values_count: 1, token_types_start: 354, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 354, raw_values_count: 1, token_types_start: 355, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 355, raw_values_count: 1, token_types_start: 356, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 356, raw_values_count: 1, token_types_start: 357, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 357, raw_values_count: 1, token_types_start: 358, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 358, raw_values_count: 1, token_types_start: 359, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 359, raw_values_count: 1, token_types_start: 360, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 360, raw_values_count: 1, token_types_start: 361, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 361, raw_values_count: 1, token_types_start: 362, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 362, raw_values_count: 1, token_types_start: 363, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 363, raw_values_count: 1, token_types_start: 364, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 364, raw_values_count: 1, token_types_start: 365, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 365, raw_values_count: 1, token_types_start: 366, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 366, raw_values_count: 1, token_types_start: 367, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 367, raw_values_count: 1, token_types_start: 368, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 368, raw_values_count: 1, token_types_start: 369, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 369, raw_values_count: 1, token_types_start: 370, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 370, raw_values_count: 1, token_types_start: 371, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 371, raw_values_count: 1, token_types_start: 372, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 372, raw_values_count: 1, token_types_start: 373, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 373, raw_values_count: 1, token_types_start: 374, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 374, raw_values_count: 1, token_types_start: 375, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 375, raw_values_count: 1, token_types_start: 376, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 376, raw_values_count: 1, token_types_start: 377, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 377, raw_values_count: 1, token_types_start: 378, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 378, raw_values_count: 1, token_types_start: 379, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 379, raw_values_count: 1, token_types_start: 380, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 380, raw_values_count: 15, token_types_start: 395, token_types_count: 1 }, // [201]
    SimpleHintData { raw_values_start: 396, raw_values_count: 2, token_types_start: 398, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 398, raw_values_count: 5, token_types_start: 403, token_types_count: 1 }, // [203]
    SimpleHintData { raw_values_start: 404, raw_values_count: 1, token_types_start: 405, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 405, raw_values_count: 1, token_types_start: 406, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 406, raw_values_count: 1, token_types_start: 407, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 407, raw_values_count: 1, token_types_start: 408, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 408, raw_values_count: 1, token_types_start: 409, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 409, raw_values_count: 1, token_types_start: 410, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 410, raw_values_count: 1, token_types_start: 411, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 412, raw_values_count: 1, token_types_start: 413, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 413, raw_values_count: 1, token_types_start: 414, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 415, raw_values_count: 1, token_types_start: 416, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 416, raw_values_count: 1, token_types_start: 417, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 417, raw_values_count: 1, token_types_start: 418, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 419, raw_values_count: 1, token_types_start: 420, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 420, raw_values_count: 1, token_types_start: 421, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 421, raw_values_count: 1, token_types_start: 422, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 422, raw_values_count: 3, token_types_start: 425, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 429, raw_values_count: 3, token_types_start: 432, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 434, raw_values_count: 3, token_types_start: 437, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 438, raw_values_count: 0, token_types_start: 438, token_types_count: 1 }, // [232]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 440, raw_values_count: 3, token_types_start: 443, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 443, raw_values_count: 1, token_types_start: 444, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 444, raw_values_count: 4, token_types_start: 448, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 448, raw_values_count: 1, token_types_start: 449, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 449, raw_values_count: 1, token_types_start: 450, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 450, raw_values_count: 1, token_types_start: 451, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 451, raw_values_count: 1, token_types_start: 452, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 452, raw_values_count: 1, token_types_start: 453, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 455, raw_values_count: 1, token_types_start: 456, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 460, raw_values_count: 1, token_types_start: 461, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 461, raw_values_count: 1, token_types_start: 462, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 463, raw_values_count: 2, token_types_start: 465, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 465, raw_values_count: 1, token_types_start: 466, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 466, raw_values_count: 1, token_types_start: 467, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 467, raw_values_count: 1, token_types_start: 468, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 468, raw_values_count: 1, token_types_start: 469, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 469, raw_values_count: 1, token_types_start: 470, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 470, raw_values_count: 1, token_types_start: 471, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 472, raw_values_count: 1, token_types_start: 473, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 473, raw_values_count: 1, token_types_start: 474, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 475, raw_values_count: 1, token_types_start: 476, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 476, raw_values_count: 1, token_types_start: 477, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 477, raw_values_count: 1, token_types_start: 478, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 478, raw_values_count: 1, token_types_start: 479, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 479, raw_values_count: 1, token_types_start: 480, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 480, raw_values_count: 1, token_types_start: 481, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 481, raw_values_count: 1, token_types_start: 482, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 482, raw_values_count: 1, token_types_start: 483, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 483, raw_values_count: 1, token_types_start: 484, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 484, raw_values_count: 1, token_types_start: 485, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 488, raw_values_count: 5, token_types_start: 493, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 493, raw_values_count: 1, token_types_start: 494, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 494, raw_values_count: 1, token_types_start: 495, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 495, raw_values_count: 8, token_types_start: 503, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 503, raw_values_count: 1, token_types_start: 504, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 506, raw_values_count: 2, token_types_start: 508, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 508, raw_values_count: 0, token_types_start: 508, token_types_count: 1 }, // [284]
    SimpleHintData { raw_values_start: 509, raw_values_count: 1, token_types_start: 510, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 510, raw_values_count: 1, token_types_start: 511, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 511, raw_values_count: 1, token_types_start: 512, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 512, raw_values_count: 4, token_types_start: 516, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 516, raw_values_count: 2, token_types_start: 518, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 518, raw_values_count: 1, token_types_start: 519, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 519, raw_values_count: 1, token_types_start: 520, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 529, raw_values_count: 1, token_types_start: 530, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 533, raw_values_count: 1, token_types_start: 534, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 543, raw_values_count: 1, token_types_start: 544, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 544, raw_values_count: 1, token_types_start: 545, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 551, raw_values_count: 7, token_types_start: 558, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 558, raw_values_count: 4, token_types_start: 562, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 562, raw_values_count: 3, token_types_start: 565, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 565, raw_values_count: 1, token_types_start: 566, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 566, raw_values_count: 1, token_types_start: 567, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 568, raw_values_count: 1, token_types_start: 569, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 569, raw_values_count: 1, token_types_start: 570, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 571, raw_values_count: 2, token_types_start: 573, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 576, raw_values_count: 1, token_types_start: 577, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 578, raw_values_count: 2, token_types_start: 580, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 582, raw_values_count: 2, token_types_start: 584, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 586, raw_values_count: 1, token_types_start: 587, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 587, raw_values_count: 3, token_types_start: 590, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 590, raw_values_count: 1, token_types_start: 591, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 592, raw_values_count: 3, token_types_start: 595, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 595, raw_values_count: 7, token_types_start: 602, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 602, raw_values_count: 1, token_types_start: 603, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 603, raw_values_count: 1, token_types_start: 604, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 604, raw_values_count: 1, token_types_start: 605, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 605, raw_values_count: 1, token_types_start: 606, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 606, raw_values_count: 3, token_types_start: 609, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 609, raw_values_count: 1, token_types_start: 610, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 610, raw_values_count: 1, token_types_start: 611, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 611, raw_values_count: 2, token_types_start: 613, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 617, raw_values_count: 2, token_types_start: 619, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 620, raw_values_count: 0, token_types_start: 620, token_types_count: 1 }, // [364]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 622, raw_values_count: 2, token_types_start: 624, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 625, raw_values_count: 2, token_types_start: 627, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 628, raw_values_count: 1, token_types_start: 629, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 630, raw_values_count: 1, token_types_start: 631, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 634, raw_values_count: 2, token_types_start: 636, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 637, raw_values_count: 1, token_types_start: 638, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 640, raw_values_count: 1, token_types_start: 641, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 641, raw_values_count: 1, token_types_start: 642, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 642, raw_values_count: 1, token_types_start: 643, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 646, raw_values_count: 1, token_types_start: 647, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 647, raw_values_count: 1, token_types_start: 648, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 648, raw_values_count: 1, token_types_start: 649, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 651, raw_values_count: 4, token_types_start: 655, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 655, raw_values_count: 1, token_types_start: 656, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 656, raw_values_count: 1, token_types_start: 657, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 657, raw_values_count: 1, token_types_start: 658, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 658, raw_values_count: 1, token_types_start: 659, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 661, raw_values_count: 1, token_types_start: 662, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 662, raw_values_count: 1, token_types_start: 663, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 663, raw_values_count: 1, token_types_start: 664, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 664, raw_values_count: 1, token_types_start: 665, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 665, raw_values_count: 1, token_types_start: 666, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 667, raw_values_count: 1, token_types_start: 668, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 668, raw_values_count: 2, token_types_start: 670, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 670, raw_values_count: 2, token_types_start: 672, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 672, raw_values_count: 1, token_types_start: 673, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 673, raw_values_count: 1, token_types_start: 674, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 674, raw_values_count: 1, token_types_start: 675, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 675, raw_values_count: 1, token_types_start: 676, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 676, raw_values_count: 1, token_types_start: 677, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 681, raw_values_count: 1, token_types_start: 682, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 682, raw_values_count: 1, token_types_start: 683, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 683, raw_values_count: 9, token_types_start: 692, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 692, raw_values_count: 6, token_types_start: 698, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 703, raw_values_count: 1, token_types_start: 704, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 704, raw_values_count: 1, token_types_start: 705, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 705, raw_values_count: 3, token_types_start: 708, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 708, raw_values_count: 2, token_types_start: 710, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 711, raw_values_count: 7, token_types_start: 718, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 718, raw_values_count: 11, token_types_start: 729, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 731, raw_values_count: 1, token_types_start: 732, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 732, raw_values_count: 1, token_types_start: 733, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 733, raw_values_count: 1, token_types_start: 734, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 734, raw_values_count: 1, token_types_start: 735, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 757, raw_values_count: 1, token_types_start: 758, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 760, raw_values_count: 1, token_types_start: 761, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 761, raw_values_count: 1, token_types_start: 762, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 762, raw_values_count: 1, token_types_start: 763, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 763, raw_values_count: 1, token_types_start: 764, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 766, raw_values_count: 1, token_types_start: 767, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 773, raw_values_count: 2, token_types_start: 775, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 776, raw_values_count: 2, token_types_start: 778, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 782, raw_values_count: 1, token_types_start: 783, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 783, raw_values_count: 1, token_types_start: 784, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 784, raw_values_count: 1, token_types_start: 785, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 786, raw_values_count: 1, token_types_start: 787, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 789, raw_values_count: 1, token_types_start: 790, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 790, raw_values_count: 1, token_types_start: 791, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 791, raw_values_count: 1, token_types_start: 792, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 792, raw_values_count: 1, token_types_start: 793, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 798, raw_values_count: 0, token_types_start: 798, token_types_count: 1 }, // [500]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 805, raw_values_count: 1, token_types_start: 806, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 806, raw_values_count: 1, token_types_start: 807, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 807, raw_values_count: 1, token_types_start: 808, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 808, raw_values_count: 1, token_types_start: 809, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 809, raw_values_count: 1, token_types_start: 810, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 811, raw_values_count: 1, token_types_start: 812, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 812, raw_values_count: 1, token_types_start: 813, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 813, raw_values_count: 7, token_types_start: 820, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 821, raw_values_count: 7, token_types_start: 828, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 828, raw_values_count: 6, token_types_start: 834, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 834, raw_values_count: 3, token_types_start: 837, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 837, raw_values_count: 3, token_types_start: 840, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 840, raw_values_count: 1, token_types_start: 841, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 841, raw_values_count: 1, token_types_start: 842, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 842, raw_values_count: 3, token_types_start: 845, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 845, raw_values_count: 2, token_types_start: 847, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 847, raw_values_count: 2, token_types_start: 849, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 849, raw_values_count: 5, token_types_start: 854, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 854, raw_values_count: 2, token_types_start: 856, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 856, raw_values_count: 1, token_types_start: 857, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 857, raw_values_count: 1, token_types_start: 858, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 858, raw_values_count: 13, token_types_start: 871, token_types_count: 1 }, // [530]
    SimpleHintData { raw_values_start: 872, raw_values_count: 3, token_types_start: 875, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 875, raw_values_count: 6, token_types_start: 881, token_types_count: 2 }, // [532]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 884, raw_values_count: 1, token_types_start: 885, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 885, raw_values_count: 5, token_types_start: 890, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 890, raw_values_count: 2, token_types_start: 892, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 893, raw_values_count: 2, token_types_start: 895, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 895, raw_values_count: 1, token_types_start: 896, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 896, raw_values_count: 2, token_types_start: 898, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 898, raw_values_count: 1, token_types_start: 899, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 899, raw_values_count: 1, token_types_start: 900, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 900, raw_values_count: 1, token_types_start: 901, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 901, raw_values_count: 1, token_types_start: 902, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 902, raw_values_count: 2, token_types_start: 904, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 904, raw_values_count: 1, token_types_start: 905, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 905, raw_values_count: 2, token_types_start: 907, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 907, raw_values_count: 1, token_types_start: 908, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 908, raw_values_count: 2, token_types_start: 910, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 910, raw_values_count: 1, token_types_start: 911, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 911, raw_values_count: 1, token_types_start: 912, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 912, raw_values_count: 14, token_types_start: 926, token_types_count: 1 }, // [552]
    SimpleHintData { raw_values_start: 927, raw_values_count: 6, token_types_start: 933, token_types_count: 1 }, // [553]
    SimpleHintData { raw_values_start: 934, raw_values_count: 1, token_types_start: 935, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 935, raw_values_count: 2, token_types_start: 937, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 937, raw_values_count: 3, token_types_start: 940, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 940, raw_values_count: 1, token_types_start: 941, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 941, raw_values_count: 1, token_types_start: 942, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 942, raw_values_count: 1, token_types_start: 943, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 943, raw_values_count: 2, token_types_start: 945, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 945, raw_values_count: 1, token_types_start: 946, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 946, raw_values_count: 1, token_types_start: 947, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 947, raw_values_count: 1, token_types_start: 948, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 948, raw_values_count: 30, token_types_start: 978, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 978, raw_values_count: 1, token_types_start: 979, token_types_count: 1 }, // [565]
    SimpleHintData { raw_values_start: 980, raw_values_count: 0, token_types_start: 980, token_types_count: 1 }, // [566]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 986, raw_values_count: 1, token_types_start: 987, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 990, raw_values_count: 1, token_types_start: 991, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 993, raw_values_count: 2, token_types_start: 995, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 996, raw_values_count: 2, token_types_start: 998, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 998, raw_values_count: 1, token_types_start: 999, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 999, raw_values_count: 1, token_types_start: 1000, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 1, token_types_start: 1001, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 15, token_types_start: 1016, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 4, token_types_start: 1023, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 1, token_types_start: 1024, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 1, token_types_start: 1026, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 1, token_types_start: 1027, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 1, token_types_start: 1028, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 3, token_types_start: 1033, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 2, token_types_start: 1035, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 1, token_types_start: 1036, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 2, token_types_start: 1038, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1038, raw_values_count: 1, token_types_start: 1039, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 2, token_types_start: 1042, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 0, token_types_start: 1042, token_types_count: 2 }, // [604]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 30, token_types_start: 1074, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 0, token_types_start: 1077, token_types_count: 1 }, // [609]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 6, token_types_start: 1093, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 1, token_types_start: 1095, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 1, token_types_start: 1099, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1099, raw_values_count: 1, token_types_start: 1100, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 1, token_types_start: 1103, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1103, raw_values_count: 1, token_types_start: 1104, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 1, token_types_start: 1108, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1108, raw_values_count: 1, token_types_start: 1109, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 1, token_types_start: 1110, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1110, raw_values_count: 1, token_types_start: 1111, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 1, token_types_start: 1113, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1113, raw_values_count: 5, token_types_start: 1118, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 2, token_types_start: 1131, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 2, token_types_start: 1135, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 3, token_types_start: 1138, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 2, token_types_start: 1140, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 2, token_types_start: 1142, token_types_count: 1 }, // [658]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 1 }, // [659]
    SimpleHintData { raw_values_start: 1145, raw_values_count: 2, token_types_start: 1147, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 5, token_types_start: 1158, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 6, token_types_start: 1168, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 4, token_types_start: 1172, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 3, token_types_start: 1175, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 3, token_types_start: 1179, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 1, token_types_start: 1181, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 2, token_types_start: 1195, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 3, token_types_start: 1198, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 1, token_types_start: 1200, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 1, token_types_start: 1201, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 1, token_types_start: 1202, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 1, token_types_start: 1204, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 1, token_types_start: 1205, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 1, token_types_start: 1206, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 2, token_types_start: 1208, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1208, raw_values_count: 1, token_types_start: 1209, token_types_count: 1 }, // [702]
    SimpleHintData { raw_values_start: 1210, raw_values_count: 1, token_types_start: 1211, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1211, raw_values_count: 1, token_types_start: 1212, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1212, raw_values_count: 1, token_types_start: 1213, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1213, raw_values_count: 1, token_types_start: 1214, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1214, raw_values_count: 1, token_types_start: 1215, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1215, raw_values_count: 1, token_types_start: 1216, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1216, raw_values_count: 1, token_types_start: 1217, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 2, token_types_start: 1225, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 1, token_types_start: 1234, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1234, raw_values_count: 1, token_types_start: 1235, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 1, token_types_start: 1241, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1241, raw_values_count: 1, token_types_start: 1242, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 1, token_types_start: 1245, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 2, token_types_start: 1252, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 1, token_types_start: 1261, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 1, token_types_start: 1263, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 1, token_types_start: 1264, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 1, token_types_start: 1270, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1270, raw_values_count: 1, token_types_start: 1271, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 1, token_types_start: 1272, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1272, raw_values_count: 1, token_types_start: 1273, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1273, raw_values_count: 1, token_types_start: 1274, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1274, raw_values_count: 1, token_types_start: 1275, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1275, raw_values_count: 4, token_types_start: 1279, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 2, token_types_start: 1281, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 1, token_types_start: 1283, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1283, raw_values_count: 1, token_types_start: 1284, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1284, raw_values_count: 1, token_types_start: 1285, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1285, raw_values_count: 1, token_types_start: 1286, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1286, raw_values_count: 1, token_types_start: 1287, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1287, raw_values_count: 1, token_types_start: 1288, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1288, raw_values_count: 1, token_types_start: 1289, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1289, raw_values_count: 1, token_types_start: 1290, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1290, raw_values_count: 1, token_types_start: 1291, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1291, raw_values_count: 1, token_types_start: 1292, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1292, raw_values_count: 1, token_types_start: 1293, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1293, raw_values_count: 1, token_types_start: 1294, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1294, raw_values_count: 1, token_types_start: 1295, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1295, raw_values_count: 1, token_types_start: 1296, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1296, raw_values_count: 1, token_types_start: 1297, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1297, raw_values_count: 1, token_types_start: 1298, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 1, token_types_start: 1299, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1299, raw_values_count: 1, token_types_start: 1300, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1300, raw_values_count: 2, token_types_start: 1302, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 8, token_types_start: 1310, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 1, token_types_start: 1321, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1321, raw_values_count: 1, token_types_start: 1322, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 1, token_types_start: 1323, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 1, token_types_start: 1324, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 1, token_types_start: 1325, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1325, raw_values_count: 1, token_types_start: 1326, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1326, raw_values_count: 1, token_types_start: 1327, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 1 }, // [808]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 1, token_types_start: 1332, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 1, token_types_start: 1333, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 1, token_types_start: 1339, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 1, token_types_start: 1340, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 3, token_types_start: 1344, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1344, raw_values_count: 2, token_types_start: 1346, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1346, raw_values_count: 2, token_types_start: 1348, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1348, raw_values_count: 1, token_types_start: 1349, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1349, raw_values_count: 5, token_types_start: 1354, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 1, token_types_start: 1358, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1358, raw_values_count: 3, token_types_start: 1361, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 1, token_types_start: 1366, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 1, token_types_start: 1372, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 2, token_types_start: 1375, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 1, token_types_start: 1376, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1376, raw_values_count: 1, token_types_start: 1377, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1377, raw_values_count: 1, token_types_start: 1378, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 1, token_types_start: 1379, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1379, raw_values_count: 1, token_types_start: 1380, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 4, token_types_start: 1384, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1384, raw_values_count: 3, token_types_start: 1387, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1387, raw_values_count: 1, token_types_start: 1388, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1388, raw_values_count: 1, token_types_start: 1389, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1389, raw_values_count: 1, token_types_start: 1390, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1390, raw_values_count: 1, token_types_start: 1391, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1391, raw_values_count: 1, token_types_start: 1392, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1392, raw_values_count: 1, token_types_start: 1393, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1393, raw_values_count: 1, token_types_start: 1394, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 1, token_types_start: 1396, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 1, token_types_start: 1397, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 1, token_types_start: 1398, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1398, raw_values_count: 1, token_types_start: 1399, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1399, raw_values_count: 1, token_types_start: 1400, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 1 }, // [862]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 1, token_types_start: 1403, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1403, raw_values_count: 1, token_types_start: 1404, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1404, raw_values_count: 1, token_types_start: 1405, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 9, token_types_start: 1414, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 1, token_types_start: 1420, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 1, token_types_start: 1421, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 1, token_types_start: 1422, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1422, raw_values_count: 1, token_types_start: 1423, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 1, token_types_start: 1424, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1424, raw_values_count: 1, token_types_start: 1425, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1425, raw_values_count: 1, token_types_start: 1426, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 2, token_types_start: 1430, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 1, token_types_start: 1433, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 1, token_types_start: 1434, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 2, token_types_start: 1443, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 3, token_types_start: 1446, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 2, token_types_start: 1448, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1448, raw_values_count: 2, token_types_start: 1450, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 10, token_types_start: 1460, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 2, token_types_start: 1462, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 1, token_types_start: 1463, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 2, token_types_start: 1472, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 1, token_types_start: 1473, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 1, token_types_start: 1479, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 1, token_types_start: 1482, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1482, raw_values_count: 1, token_types_start: 1483, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 1, token_types_start: 1501, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 1, token_types_start: 1515, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 1, token_types_start: 1519, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 1, token_types_start: 1520, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 1, token_types_start: 1522, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 1, token_types_start: 1523, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 1, token_types_start: 1524, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1524, raw_values_count: 1, token_types_start: 1525, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 1, token_types_start: 1527, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1527, raw_values_count: 1, token_types_start: 1528, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1528, raw_values_count: 1, token_types_start: 1529, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1529, raw_values_count: 1, token_types_start: 1530, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 1, token_types_start: 1545, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1545, raw_values_count: 1, token_types_start: 1546, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1546, raw_values_count: 1, token_types_start: 1547, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1547, raw_values_count: 2, token_types_start: 1549, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 1, token_types_start: 1551, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 1, token_types_start: 1552, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 1, token_types_start: 1556, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 1, token_types_start: 1561, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 1, token_types_start: 1562, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 1, token_types_start: 1564, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 1, token_types_start: 1565, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 1, token_types_start: 1566, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 4, token_types_start: 1570, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 3, token_types_start: 1573, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 4, token_types_start: 1577, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 4, token_types_start: 1581, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 2, token_types_start: 1583, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 1, token_types_start: 1585, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 1, token_types_start: 1588, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 2, token_types_start: 1590, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 2, token_types_start: 1592, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 1, token_types_start: 1593, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1593, raw_values_count: 1, token_types_start: 1594, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1594, raw_values_count: 1, token_types_start: 1595, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1595, raw_values_count: 1, token_types_start: 1596, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 1, token_types_start: 1605, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 1, token_types_start: 1606, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 1, token_types_start: 1607, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1607, raw_values_count: 1, token_types_start: 1608, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1608, raw_values_count: 1, token_types_start: 1609, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 1, token_types_start: 1610, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1610, raw_values_count: 1, token_types_start: 1611, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1611, raw_values_count: 1, token_types_start: 1612, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1612, raw_values_count: 1, token_types_start: 1613, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1613, raw_values_count: 1, token_types_start: 1614, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 2, token_types_start: 1616, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1616, raw_values_count: 3, token_types_start: 1619, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 1, token_types_start: 1620, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1620, raw_values_count: 0, token_types_start: 1620, token_types_count: 1 }, // [1038]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 1, token_types_start: 1627, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1627, raw_values_count: 1, token_types_start: 1628, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 1, token_types_start: 1635, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1635, raw_values_count: 1, token_types_start: 1636, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 14, token_types_start: 1655, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 2, token_types_start: 1657, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 2, token_types_start: 1659, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [1066]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 5, 6, 7, 8, 9, 0, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 19, 20, 21, 21, 22, 23, 24, 21, 21, 22, 24, 25,
    25, 22, 26, 27, 28, 28, 26, 29, 30, 30, 22, 31, 10, 10, 32, 33,
    33, 34, 10, 35, 36, 37, 37, 22, 36, 38, 38, 35, 21, 22, 39, 40,
    40, 10, 35, 21, 41, 3, 22, 42, 34, 34, 43, 6, 44, 44, 10, 35,
    41, 22, 43, 6, 45, 34, 34, 46, 8, 9, 42, 39, 31, 5, 7, 47,
    47, 34, 10, 14, 35, 12, 11, 13, 22, 8, 11, 11, 22, 48, 49, 49,
    34, 35, 22, 9, 50, 50, 34, 35, 16, 22, 51, 7, 5, 17, 18, 52,
    52, 53, 0, 42, 42, 0, 19, 19, 54, 0, 0, 55, 56, 57, 3, 3,
    53, 42, 0, 58, 58, 59, 7, 6, 0, 0, 0, 53, 0, 8, 9, 0,
    5, 60, 61, 62, 0, 0, 6, 0, 0, 0, 0, 0, 9, 63, 63, 8,
    64, 64, 65, 0, 66, 0, 0, 0, 0, 0, 0, 0, 67, 68, 0, 0,
    0, 0, 0, 0, 63, 63, 66, 64, 64, 0, 69, 21, 70, 0, 0, 71,
    71, 72, 73, 74, 75, 76, 77, 78, 79, 80, 81, 82, 83, 84, 85, 0,
    0, 86, 0, 0, 0, 87, 0, 87, 88, 89, 0, 0, 0, 0, 90, 19,
    91, 10, 10, 42, 0, 56, 0, 10, 10, 26, 27, 0, 25, 92, 92, 10,
    10, 10, 39, 0, 25, 93, 93, 94, 0, 0, 95, 10, 10, 6, 0, 96,
    25, 25, 97, 94, 98, 99, 100, 101, 101, 102, 0, 0, 0, 0, 103, 99,
    0, 0, 0, 0, 0, 99, 0, 104, 105, 106, 0, 10, 10, 62, 0, 0,
    0, 10, 10, 31, 25, 107, 94, 108, 89, 99, 10, 10, 32, 25, 109, 94,
    108, 89, 99, 75, 75, 110, 111, 0, 87, 0, 10, 10, 110, 0, 112, 25,
    25, 113, 114, 114, 115, 87, 99, 116, 116, 0, 0, 117, 35, 35, 113, 114,
    116, 10, 10, 8, 0, 118, 75, 35, 119, 120, 10, 10, 10, 8, 0, 121,
    75, 75, 122, 119, 119, 123, 124, 124, 0, 125, 126, 35, 35, 123, 124, 124,
    111, 0, 125, 120, 120, 124, 0, 56, 0, 10, 10, 8, 0, 127, 128, 35,
    35, 129, 130, 130, 131, 132, 132, 132, 131, 130, 133, 35, 35, 110, 111, 0,
    35, 35, 110, 111, 0, 134, 134, 135, 119, 119, 110, 0, 87, 0, 136, 137,
    0, 0, 0, 0, 94, 0, 0, 0, 138, 138, 75, 119, 110, 0, 139, 140,
    140, 141, 84, 0, 87, 35, 120, 120, 142, 86, 56, 0, 120, 120, 110, 0,
    56, 0, 10, 10, 10, 10, 10, 10, 36, 0, 143, 144, 144, 145, 103, 103,
    146, 146, 147, 148, 103, 103, 149, 149, 94, 89, 150, 150, 151, 152, 152, 153,
    154, 155, 25, 25, 93, 94, 0, 0, 95, 10, 10, 43, 6, 0, 156, 25,
    25, 0, 0, 0, 94, 0, 45, 45, 66, 66, 0, 0, 101, 101, 102, 0,
    157, 158, 159, 131, 131, 160, 22, 161, 162, 163, 164, 165, 166, 167, 100, 168,
    158, 169, 170, 171, 172, 71, 0, 0, 99, 0, 173, 71, 174, 0, 175, 71,
    0, 0, 0, 86, 86, 86, 176, 177, 178, 179, 180, 181, 182, 148, 183, 184,
    185, 155, 186, 187, 188, 189, 190, 0, 0, 191, 192, 193, 117, 0, 0, 117,
    194, 0, 0, 0, 136, 136, 0, 55, 86, 0, 0, 195, 196, 197, 198, 199,
    200, 201, 164, 169, 202, 203, 0, 204, 205, 206, 207, 208, 209, 210, 211, 212,
    213, 214, 215, 216, 158, 171, 171, 171, 169, 172, 172, 172, 170, 217, 218, 219,
    202, 131, 220, 221, 222, 223, 224, 225, 226, 87, 99, 99, 227, 204, 228, 229,
    87, 0, 0, 0, 87, 230, 231, 103, 228, 87, 99, 99, 99, 99, 99, 99,
    232, 99, 65, 65, 94, 89, 233, 233, 94, 234, 157, 235, 191, 87, 0, 0,
    0, 87, 86, 87, 236, 237, 238, 239, 134, 240, 241, 241, 241, 0, 242, 242,
    243, 244, 243, 244, 244, 241, 241, 0, 242, 242, 243, 244, 243, 244, 244, 203,
    55, 201, 241, 245, 246, 247, 248, 249, 250, 251, 101, 252, 253, 177, 229, 254,
    255, 256, 257, 258, 259, 260, 261, 262, 263, 264, 265, 266, 266, 266, 6, 267,
    268, 0, 0, 269, 270, 0, 89, 0, 0, 271, 0, 272, 273, 274, 275, 173,
    173, 173, 0, 0, 0, 0, 117, 137, 113, 116, 116, 0, 0, 117, 114, 114,
    115, 87, 99, 276, 277, 0, 0, 136, 0, 0, 137, 0, 110, 0, 0, 0,
    0, 271, 0, 0, 0, 0, 0, 55, 277, 277, 278, 89, 277, 0, 277, 277,
    53, 0, 0, 8, 0, 278, 278, 89, 87, 0, 0, 0, 278, 89, 279, 279,
    110, 6, 5, 7, 36, 39, 42, 26, 26, 27, 0, 278, 89, 280, 281, 282,
    203, 94, 172, 171, 172, 171, 283, 284, 278, 0, 0, 0, 285, 169, 169, 169,
    286, 287, 0, 288, 287, 0, 289, 290, 290, 103, 106, 0, 0, 131, 291, 291,
    291, 292, 0, 291, 291, 292, 0, 290, 103, 106, 0, 0, 131, 290, 293, 294,
    295, 0, 0, 0, 89, 36, 36, 89, 144, 103, 89, 42, 296, 124, 297, 135,
    298, 92, 92, 299, 300, 301, 302, 303, 304, 305, 94, 108, 89, 99, 26, 93,
    306, 307, 308, 309, 310, 311, 312, 313, 314, 315, 316, 317, 318, 299, 34, 34,
    220, 0, 319, 320, 321, 0, 74, 7, 0, 86, 0, 34, 34, 246, 87, 136,
    86, 136, 292, 322, 323, 61, 5, 60, 324, 325, 326, 327, 298, 328, 0, 87,
    58, 58, 0, 86, 86, 181, 34, 34, 220, 42, 0, 56, 0, 34, 34, 26,
    27, 0, 292, 92, 92, 34, 34, 329, 111, 0, 34, 34, 220, 5, 0, 87,
    0, 0, 0, 330, 136, 331, 136, 332, 278, 86, 0, 0, 0, 136, 333, 196,
    0, 0, 331, 0, 333, 244, 0, 333, 34, 34, 220, 334, 335, 111, 0, 53,
    0, 87, 87, 0, 0, 34, 34, 34, 220, 0, 111, 0, 336, 336, 87, 0,
    0, 0, 94, 0, 0, 71, 89, 89, 86, 236, 34, 34, 39, 0, 34, 34,
    6, 111, 0, 34, 34, 220, 321, 7, 0, 87, 0, 0, 175, 0, 337, 337,
    338, 8, 339, 86, 0, 0, 0, 0, 103, 99, 291, 291, 292, 99, 0, 0,
    0, 0, 0, 99, 0, 104, 105, 106, 0, 34, 34, 62, 0, 0, 0, 340,
    340, 0, 87, 0, 0, 0, 0, 341, 341, 342, 343, 344, 341, 341, 342, 343,
    114, 341, 341, 342, 343, 280, 34, 34, 220, 8, 111, 0, 345, 87, 0, 0,
    0, 55, 55, 128, 86, 86, 236, 292, 340, 277, 34, 34, 346, 0, 0, 195,
    84, 0, 0, 347, 348, 348, 12, 14, 13, 13, 347, 0, 0, 220, 53, 220,
    53, 53, 0, 349, 350, 350, 351, 352, 86, 0, 353, 352, 86, 0, 57, 57,
    0, 354, 355, 355, 356, 357, 356, 358, 358, 358, 359, 358, 358, 360, 58, 58,
    361, 362, 352, 363, 242, 242, 87, 0, 16, 16, 60, 364, 87, 34, 34, 220,
    0, 319, 320, 321, 365, 0, 366, 367, 25, 7, 0, 87, 0, 0, 85, 368,
    337, 337, 136, 369, 86, 0, 34, 34, 36, 0, 144, 145, 103, 103, 148, 103,
    34, 34, 220, 370, 9, 0, 87, 0, 0, 0, 277, 86, 236, 87, 236, 277,
    34, 34, 43, 6, 111, 0, 371, 0, 292, 0, 0, 0, 94, 0, 372, 287,
    373, 374, 375, 375, 376, 376, 375, 377, 87, 0, 378, 379, 380, 381, 382, 383,
    384, 385, 386, 387, 388, 389, 105, 390, 329, 391, 0, 0, 0, 0, 136, 392,
    392, 393, 394, 395, 396, 87, 199, 397, 397, 398, 399, 400, 401, 402, 403, 404,
    405, 406, 221, 219, 407, 408, 409, 409, 292, 292, 410, 411, 412, 194, 194, 413,
    87, 56, 414, 194, 194, 415, 87, 56, 416, 87, 417, 417, 87, 418, 418, 87,
    419, 419, 420, 420, 229, 229, 421, 422, 423, 424, 425, 426, 87, 427, 427, 422,
    254, 254, 428, 429, 430, 430, 23, 421, 87, 265, 265, 87, 252, 408, 431, 431,
    87, 87, 0, 0, 432, 0, 407, 407, 408, 409, 89, 433, 432, 434, 435, 415,
    436, 437, 438, 439, 394, 393, 440, 441, 116, 116, 116, 442, 443, 444, 344, 356,
    360, 445, 446, 447, 14, 14, 14, 100, 57, 0, 0, 0, 448, 449, 233, 333,
    450, 451, 452, 453, 454, 455, 456, 456, 456, 0, 0, 457, 458, 459, 48, 460,
    461, 462, 463, 464, 465, 130, 466, 167, 467, 468, 469, 470, 471, 472, 473, 474,
    271, 397, 475, 476, 477, 478, 134, 35, 35, 246, 87, 136, 86, 136, 477, 35,
    35, 42, 111, 0, 35, 35, 26, 26, 27, 0, 35, 35, 329, 111, 0, 477,
    35, 35, 5, 111, 0, 477, 35, 35, 335, 111, 0, 477, 35, 35, 35, 0,
    111, 0, 35, 35, 39, 111, 0, 134, 35, 35, 370, 43, 6, 111, 0, 477,
    35, 35, 479, 74, 7, 111, 0, 35, 35, 62, 0, 35, 35, 8, 111, 0,
    477, 134, 134, 135, 35, 35, 346, 111, 0, 35, 35, 480, 111, 0, 477, 35,
    35, 36, 111, 0, 134, 35, 35, 9, 111, 0, 477, 481, 482, 483, 484, 361,
    485, 243, 243, 0, 243, 486, 487, 369, 369, 369, 87, 0, 488, 87, 0, 0,
    87, 489, 490, 491, 492, 493, 244, 494, 495, 496, 94, 497, 498, 499, 500, 501,
    502, 503, 504, 505, 506, 507, 343, 508, 16, 16, 16, 7, 0, 87, 0, 0,
    175, 0, 292, 292, 509, 510, 511, 512, 513, 151, 514, 514, 514, 515, 236, 12,
    13, 14, 514, 514, 43, 236, 516, 516, 517, 518, 518, 519, 520, 521, 7, 18,
    18, 18, 0, 0, 0, 87, 0, 87, 236, 516, 522, 0, 0, 0, 0, 0,
    523, 201, 201, 0, 524, 203, 278, 278, 355, 0, 0, 0, 0, 525, 355, 198,
    0, 131, 0, 0, 0, 0, 526, 527, 527, 527, 528, 355, 529, 0, 0, 530,
    530, 530, 530, 164, 169, 203, 0, 531, 527, 527, 527, 528, 0, 510, 510, 87,
    236, 0, 0, 0, 241, 148, 148, 0, 55, 0, 0, 117, 0, 87, 0, 0,
    0, 236, 0, 0, 0, 0, 0, 410, 0, 0, 0, 0, 0, 0, 0, 87,
    0, 0, 136, 136, 532, 89, 99, 222, 95, 407, 410, 0, 55, 71, 0, 11,
    0, 194, 0, 0, 0, 533, 0, 63, 534, 87, 535, 535, 536, 536, 537, 291,
    94, 99, 538, 538, 65, 539, 94, 89, 90, 90, 94, 540, 541, 542, 153, 224,
    224, 543, 544, 544, 545, 141, 546, 0, 99, 0, 352, 547, 352, 548, 549, 550,
    292, 292, 551, 544, 64, 552, 553, 491, 95, 221, 352, 352, 554, 110, 110, 555,
    554, 450, 94, 89, 556, 557, 558, 559, 560, 560, 561, 352, 352, 537, 94, 99,
    146, 562, 292, 292, 110, 563, 233, 233, 94, 234, 157, 235, 191, 564, 564, 34,
    564, 565, 333, 566, 567, 0, 0, 87, 448, 448, 0, 568, 141, 569, 399, 570,
    571, 571, 571, 572, 573, 58, 574, 370, 575, 575, 576, 15, 15, 0, 87, 575,
    65, 577, 578, 579, 579, 0, 0, 379, 379, 352, 0, 0, 99, 194, 580, 0,
    581, 582, 573, 198, 198, 131, 579, 548, 583, 584, 57, 57, 0, 0, 585, 448,
    290, 291, 449, 586, 27, 27, 103, 85, 85, 103, 587, 588, 589, 292, 277, 0,
    0, 8, 0, 87, 0, 0, 0, 0, 585, 590, 0, 0, 292, 292, 591, 0,
    590, 0, 87, 0, 0, 0, 0, 87, 0, 0, 0, 0, 87, 0, 85, 85,
    103, 27, 27, 103, 0, 0, 0, 0, 0, 0, 85, 85, 103, 27, 27, 103,
    0, 57, 592, 593, 594, 0, 0, 595, 0, 0, 0, 0, 0, 333, 0, 0,
    0, 111, 58, 596, 0, 0, 0, 0, 0, 86, 136, 597, 597, 0, 57, 57,
    0, 0, 0, 432, 0, 57, 0, 0, 464, 0, 100, 0, 0, 85, 554, 554,
    0, 0, 0, 89, 0, 0, 330, 0, 89, 0, 0, 598, 0, 0, 87, 87,
    0, 86, 86, 89, 599, 599, 0, 58, 58, 0, 600, 595, 595, 0, 56, 0,
    473, 0, 0, 244, 58, 330, 330, 0, 232, 0, 601, 0, 0, 244, 601, 333,
    111, 111, 0, 0, 602, 0, 0, 603, 603, 603, 487, 486, 0, 602, 0, 0,
    243, 243, 0, 0, 244, 111, 333, 5, 364, 0, 0, 0, 0, 271, 87, 604,
    364, 147, 87, 0, 0, 0, 605, 22, 22, 480, 136, 605, 22, 22, 480, 136,
    87, 0, 0, 34, 34, 34, 34, 34, 566, 0, 431, 431, 87, 0, 0, 0,
    0, 0, 431, 0, 442, 87, 0, 596, 596, 0, 473, 0, 0, 244, 596, 333,
    606, 607, 417, 608, 609, 610, 611, 612, 21, 21, 21, 0, 4, 53, 53, 0,
    0, 4, 0, 19, 19, 613, 56, 0, 292, 292, 83, 614, 615, 616, 172, 172,
    94, 172, 617, 27, 27, 103, 0, 0, 0, 99, 376, 618, 0, 87, 85, 85,
    103, 586, 587, 588, 589, 619, 85, 85, 103, 586, 587, 588, 620, 544, 27, 621,
    301, 0, 0, 0, 0, 99, 0, 87, 619, 618, 618, 618, 622, 87, 0, 0,
    376, 0, 623, 29, 624, 625, 626, 627, 628, 629, 630, 631, 632, 418, 633, 634,
    635, 636, 637, 638, 639, 587, 587, 0, 87, 0, 640, 85, 85, 103, 586, 588,
    620, 544, 587, 641, 642, 643, 0, 644, 645, 144, 0, 114, 305, 111, 111, 510,
    111, 111, 111, 355, 510, 646, 647, 598, 598, 647, 648, 649, 359, 37, 37, 37,
    0, 52, 650, 0, 0, 0, 340, 0, 374, 374, 148, 148, 0, 64, 89, 410,
    410, 651, 374, 64, 89, 0, 0, 498, 498, 516, 0, 87, 0, 652, 146, 562,
    521, 57, 57, 517, 518, 518, 519, 520, 498, 7, 519, 519, 504, 653, 654, 654,
    476, 476, 94, 89, 148, 148, 0, 655, 8, 8, 0, 87, 0, 656, 656, 652,
    146, 562, 372, 372, 103, 89, 363, 363, 363, 89, 520, 520, 657, 657, 374, 543,
    658, 148, 148, 0, 659, 64, 89, 410, 410, 651, 657, 374, 543, 659, 64, 89,
    87, 87, 87, 552, 7, 7, 0, 148, 148, 0, 292, 292, 0, 0, 0, 94,
    0, 17, 17, 17, 516, 516, 0, 0, 0, 87, 0, 87, 0, 0, 57, 330,
    524, 355, 330, 0, 87, 0, 0, 0, 236, 0, 342, 0, 0, 660, 176, 455,
    335, 661, 662, 358, 663, 664, 665, 666, 12, 12, 12, 516, 0, 667, 442, 116,
    116, 442, 236, 87, 326, 668, 669, 400, 401, 670, 671, 194, 194, 89, 672, 414,
    414, 87, 99, 413, 413, 87, 99, 56, 56, 414, 673, 673, 674, 674, 415, 644,
    675, 87, 99, 416, 416, 87, 99, 99, 56, 56, 676, 644, 675, 416, 416, 87,
    99, 194, 516, 516, 516, 8, 0, 677, 678, 679, 680, 0, 95, 0, 0, 222,
    0, 278, 278, 355, 464, 57, 278, 0, 681, 682, 683, 684, 685, 686, 687, 688,
    319, 689, 654, 0, 0, 0, 690, 0, 0, 0, 0, 0, 0, 691, 53, 371,
    0, 0, 0, 690, 0, 0, 0, 0, 0, 690, 690, 690, 0, 53, 53, 0,
    0, 87, 0, 0, 288, 663, 692, 692, 594, 541, 542, 693, 371, 371, 87, 0,
    0, 694, 695, 696, 697, 698, 576, 699, 700, 133, 0, 30, 30, 30, 701, 31,
    31, 702, 384, 99, 363, 363, 99, 330, 31, 99, 292, 292, 703, 89, 704, 599,
    430, 705, 706, 0, 707, 708, 709, 710, 711, 712, 713, 541, 714, 171, 171, 94,
    171, 715, 523, 523, 355, 716, 0, 499, 499, 0, 716, 340, 717, 340, 284, 586,
    586, 99, 99, 99, 55, 99, 99, 99, 591, 591, 99, 586, 718, 0, 719, 0,
    89, 99, 222, 527, 95, 407, 71, 0, 720, 0, 0, 721, 410, 410, 271, 0,
    410, 722, 723, 724, 725, 726, 727, 728, 729, 154, 730, 427, 731, 601, 732, 733,
    734, 558, 321, 0, 28, 735, 0, 0, 736, 737, 738, 739, 740, 14, 14, 448,
    12, 12, 87, 442, 448, 741, 741, 516, 741, 242, 242, 242, 242, 242, 242, 242,
    242, 242, 737, 602, 742, 13, 14, 242, 242, 355, 737, 602, 12, 741, 741, 0,
    0, 0, 371, 0, 0, 0, 87, 87, 236, 0, 0, 0, 0, 0, 53, 0,
    242, 13, 13, 25, 448, 703, 328, 743, 744, 745, 746, 166, 675, 747, 748, 749,
    119, 750, 168, 414, 751, 752, 753, 100, 754, 0, 0, 0, 86, 0, 0, 87,
    0, 620, 620, 0, 0, 563, 0, 755, 756, 756, 288, 756, 424, 757, 166, 0,
    235, 353, 0, 546, 758, 759, 760, 761, 762, 763, 764, 106, 765, 230, 442, 11,
    87, 11, 87, 292, 87, 230, 87, 0, 766, 13, 12, 14, 741, 230, 230, 767,
    87, 11, 230, 768, 0, 769, 0, 283, 770, 770, 94, 171, 171, 172, 355, 0,
    355, 771, 772, 773, 95, 95, 774, 775, 649, 395, 396, 99, 776, 777, 426, 425,
    778, 779, 23, 89, 89, 173, 0, 720, 89, 89, 720, 271, 271, 271, 271, 271,
    0, 53, 86, 371, 55, 247, 71, 87, 201, 173, 333, 0, 613, 780, 781, 347,
    782, 591, 591, 0, 99, 0, 352, 547, 352, 548, 591, 351, 53, 550, 783, 783,
    783, 6, 0, 784, 785, 786, 614, 336, 220, 220, 220, 220, 146, 653, 85, 85,
    103, 0, 0, 0, 0, 99, 0, 660, 176, 455, 649, 649, 787, 141, 705, 0,
    586, 579, 788, 586, 587, 588, 620, 579, 554, 544, 85, 0, 85, 0, 789, 790,
    791, 792, 693, 509, 0, 598, 793, 0, 0, 87, 0, 793, 794, 533, 533, 0,
    87, 87, 407, 55, 407, 0, 533, 795, 796, 102, 797, 539, 798, 799, 0, 800,
    801, 802, 803, 804, 132, 132, 103, 0, 87, 0, 0, 132, 805, 150, 147, 149,
    806, 807, 808, 167, 167, 364, 364, 167, 89, 0, 809, 810, 811, 812, 169, 813,
    814, 815, 165, 816, 165, 817, 0, 369, 0, 598, 598, 647, 648, 649, 0, 0,
    818, 8, 300, 582, 398, 449, 449, 0, 87, 0, 132, 449, 819, 820, 820, 820,
    815, 529, 529, 56, 0, 678, 678, 0, 0, 0, 821, 821, 642, 732, 0, 0,
    117, 0, 0, 410, 0, 0, 117, 0, 0, 410, 819, 822, 822, 822, 823, 8,
    8, 0, 87, 66, 66, 0, 0, 6, 6, 0, 68, 68, 0, 0, 329, 824,
    825, 125, 125, 576, 125, 826, 529, 24, 20, 60, 827, 320, 527, 527, 99, 588,
    588, 0, 588, 203, 203, 828, 19, 22, 829, 87, 236, 830, 831, 304, 832, 147,
    89, 365, 833, 834, 835, 583, 232, 836, 837, 94, 172, 171, 770, 97, 838, 839,
    402, 840, 841, 841, 840, 842, 823, 8, 8, 0, 87, 66, 66, 0, 0, 6,
    6, 0, 68, 68, 0, 0, 329, 495, 843, 844, 845, 846, 15, 15, 0, 87,
    736, 53, 53, 53, 14, 847, 53, 53, 13, 847, 736, 736, 848, 594, 804, 849,
    15, 350, 350, 0, 87, 350, 847, 478, 134, 25, 25, 95, 764, 764, 73, 25,
    25, 116, 45, 850, 717, 851, 852, 853, 854, 855, 856, 857, 858, 859, 860, 861,
    521, 521, 586, 862, 99, 863, 498, 521, 864, 865, 120, 120, 120, 866, 6, 8,
    9, 5, 7, 36, 39, 42, 26, 26, 27, 0, 56, 0, 842, 867, 868, 146,
    152, 648, 869, 478, 870, 331, 871, 872, 873, 337, 874, 70, 70, 70, 0, 4,
    53, 53, 0, 0, 4, 0, 19, 19, 613, 57, 0, 542, 39, 0, 54, 875,
    377, 377, 876, 61, 352, 877, 339, 878, 548, 879, 880, 559, 590, 590, 881, 197,
    32, 87, 99, 868, 868, 868, 87, 99, 882, 367, 6, 68, 883, 884, 885, 0,
    566, 7, 886, 887, 888, 889, 416, 890, 891, 651, 892, 0, 0, 0, 0, 0,
    893, 464, 19, 11, 11, 893, 0, 0, 894, 895, 292, 677, 896, 575, 125, 516,
    516, 8, 895, 895, 516, 897, 897, 57, 0, 448, 85, 85, 103, 586, 589, 292,
    277, 11, 11, 11, 57, 350, 448, 289, 449, 27, 587, 588, 85, 586, 589, 292,
    277, 236, 898, 87, 292, 898, 87, 292, 230, 87, 236, 899, 900, 333, 901, 554,
    62, 0, 0, 0, 99, 764, 764, 0, 0, 0, 0, 99, 764, 764, 0, 902,
    31, 107, 903, 303, 904, 905, 25, 25, 0, 0, 57, 0, 0, 94, 0, 0,
    0, 117, 0, 0, 95, 116, 230, 230, 589, 589, 589, 230, 85, 586, 620, 25,
    589, 906, 906, 19, 671, 907, 746, 505, 25, 25, 6, 111, 0, 622, 0, 0,
    0, 241, 247, 247, 247, 136, 148, 403, 908, 909, 527, 165, 166, 910, 849, 911,
    0, 0, 147, 500, 0, 0, 0, 0, 271, 100, 89, 912, 537, 0, 0, 71,
    561, 167, 173, 405, 913, 829, 914, 915, 916, 322, 917, 918, 919, 920, 921, 922,
    923, 924, 925, 926, 927, 928, 929, 930, 931, 932, 933, 934, 935, 936, 937, 938,
    939, 940, 941, 942, 943, 944, 945, 946, 947, 948, 949, 950, 951, 952, 953, 954,
    955, 956, 957, 958, 959, 960, 961, 962, 963, 964, 965, 966, 967, 968, 969, 970,
    971, 972, 973, 974, 975, 976, 977, 978, 979, 980, 981, 100, 291, 982, 363, 564,
    236, 14, 18, 17, 12, 741, 562, 13, 10, 10, 10, 277, 34, 34, 34, 34,
    35, 35, 35, 35, 35, 120, 69, 10, 10, 34, 34, 34, 35, 35, 35, 34,
    10, 35, 10, 10, 10, 783, 266, 571, 37, 841, 30, 822, 562, 514, 983, 16,
    333, 327, 572, 984, 985, 986, 987, 169, 169, 87, 0, 0, 0, 0, 0, 988,
    989, 990, 991, 992, 993, 994, 0, 995, 996, 997, 998, 32, 109, 999, 1000, 126,
    489, 460, 1001, 124, 124, 0, 1002, 334, 334, 87, 125, 125, 87, 575, 575, 87,
    15, 0, 0, 122, 340, 465, 465, 103, 0, 0, 132, 333, 128, 465, 465, 55,
    132, 132, 132, 55, 465, 0, 0, 117, 0, 0, 87, 236, 442, 442, 17, 514,
    276, 124, 124, 0, 1003, 355, 95, 125, 575, 1004, 1005, 355, 95, 125, 15, 126,
    8, 122, 124, 124, 0, 1003, 355, 95, 125, 575, 896, 125, 125, 87, 575, 575,
    87, 15, 126, 132, 132, 103, 0, 0, 465, 333, 66, 590, 0, 526, 526, 201,
    0, 0, 531, 531, 0, 1006, 1007, 67, 1008, 0, 0, 0, 1009, 1009, 0, 0,
    0, 1010, 602, 551, 528, 411, 1011, 1011, 411, 409, 87, 1012, 1012, 292, 1013, 411,
    412, 148, 148, 148, 411, 412, 0, 1014, 1015, 409, 1016, 1017, 1018, 404, 56, 1019,
    1020, 1021, 1022, 1023, 1024, 1025, 1026, 1027, 983, 983, 281, 875, 1028, 1029, 1030, 1031,
    1032, 1033, 346, 557, 597, 226, 711, 1026, 223, 223, 1034, 562, 562, 187, 730, 697,
    697, 1035, 705, 705, 1036, 415, 414, 413, 57, 57, 89, 562, 562, 562, 8, 0,
    87, 0, 0, 480, 0, 0, 71, 0, 0, 87, 1037, 1038, 0, 0, 0, 1038,
    1039, 580, 1040, 0, 0, 756, 0, 0, 1041, 1042, 1043, 1044, 906, 906, 893, 464,
    19, 906, 334, 334, 334, 225, 0, 225, 863, 1045, 1046, 11, 11, 57, 350, 448,
    289, 449, 27, 587, 588, 589, 292, 277, 85, 586, 230, 230, 589, 589, 589, 230,
    1047, 13, 13, 13, 0, 0, 0, 25, 57, 448, 449, 1048, 1049, 41, 41, 41,
    0, 103, 103, 698, 826, 89, 36, 148, 148, 709, 86, 89, 370, 103, 103, 784,
    990, 89, 103, 103, 147, 0, 302, 371, 253, 1050, 1051, 442, 442, 0, 0, 87,
    0, 0, 116, 0, 0, 0, 0, 116, 0, 0, 442, 442, 442, 198, 0, 99,
    117, 0, 131, 0, 99, 117, 0, 292, 292, 987, 99, 1052, 1053, 423, 422, 0,
    1054, 1055, 421, 1056, 9, 43, 595, 1057, 242, 242, 0, 0, 0, 602, 0, 0,
    0, 0, 242, 1058, 448, 448, 0, 87, 0, 1059, 290, 291, 449, 586, 27, 27,
    103, 85, 85, 103, 587, 588, 589, 292, 277, 448, 596, 0, 0, 0, 0, 0,
    0, 0, 0, 271, 100, 100, 271, 100, 620, 0, 0, 1060, 132, 85, 132, 85,
    579, 292, 292, 845, 0, 0, 0, 11, 0, 766, 292, 292, 845, 0, 0, 0,
    11, 0, 230, 292, 292, 764, 764, 390, 0, 895, 292, 677, 575, 87, 895, 292,
    677, 1061, 334, 125, 87, 292, 292, 292, 764, 6, 0, 1062, 1013, 0, 1028, 1063,
    413, 1064, 1065, 1066, 412,
];

pub fn get_exasol_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &EXASOL_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &EXASOL_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &EXASOL_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &EXASOL_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(9), tables: &EXASOL_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(153), tables: &EXASOL_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(156), tables: &EXASOL_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(179), tables: &EXASOL_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(202), tables: &EXASOL_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(205), tables: &EXASOL_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(207), tables: &EXASOL_TABLES }),
        "AcosKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(209), tables: &EXASOL_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(210), tables: &EXASOL_TABLES }),
        "AdapterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &EXASOL_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &EXASOL_TABLES }),
        "Add_daysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &EXASOL_TABLES }),
        "Add_hoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &EXASOL_TABLES }),
        "Add_minutesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &EXASOL_TABLES }),
        "Add_monthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &EXASOL_TABLES }),
        "Add_secondsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &EXASOL_TABLES }),
        "Add_weeksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &EXASOL_TABLES }),
        "Add_yearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &EXASOL_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &EXASOL_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &EXASOL_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &EXASOL_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &EXASOL_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(235), tables: &EXASOL_TABLES }),
        "AlignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(238), tables: &EXASOL_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &EXASOL_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &EXASOL_TABLES }),
        "AlterConnectionSegment" => Some(RootGrammar { grammar_id: GrammarId(241), tables: &EXASOL_TABLES }),
        "AlterConsumerGroupSegment" => Some(RootGrammar { grammar_id: GrammarId(247), tables: &EXASOL_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(255), tables: &EXASOL_TABLES }),
        "AlterRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(256), tables: &EXASOL_TABLES }),
        "AlterSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(267), tables: &EXASOL_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(283), tables: &EXASOL_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(299), tables: &EXASOL_TABLES }),
        "AlterSessionSegment" => Some(RootGrammar { grammar_id: GrammarId(305), tables: &EXASOL_TABLES }),
        "AlterSystemSegment" => Some(RootGrammar { grammar_id: GrammarId(314), tables: &EXASOL_TABLES }),
        "AlterTableAddColumnSegment" => Some(RootGrammar { grammar_id: GrammarId(323), tables: &EXASOL_TABLES }),
        "AlterTableAlterColumnSegment" => Some(RootGrammar { grammar_id: GrammarId(330), tables: &EXASOL_TABLES }),
        "AlterTableColumnSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &EXASOL_TABLES }),
        "AlterTableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(363), tables: &EXASOL_TABLES }),
        "AlterTableDistributePartitionSegment" => Some(RootGrammar { grammar_id: GrammarId(393), tables: &EXASOL_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(411), tables: &EXASOL_TABLES }),
        "AlterTableDropColumnSegment" => Some(RootGrammar { grammar_id: GrammarId(416), tables: &EXASOL_TABLES }),
        "AlterTableModifyColumnSegment" => Some(RootGrammar { grammar_id: GrammarId(424), tables: &EXASOL_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(433), tables: &EXASOL_TABLES }),
        "AlterTableRenameColumnSegment" => Some(RootGrammar { grammar_id: GrammarId(460), tables: &EXASOL_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(466), tables: &EXASOL_TABLES }),
        "AlterUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(470), tables: &EXASOL_TABLES }),
        "AlterVirtualSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(505), tables: &EXASOL_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(528), tables: &EXASOL_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(529), tables: &EXASOL_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(530), tables: &EXASOL_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(531), tables: &EXASOL_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(532), tables: &EXASOL_TABLES }),
        "AnsiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(533), tables: &EXASOL_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(534), tables: &EXASOL_TABLES }),
        "AppendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(535), tables: &EXASOL_TABLES }),
        "Approximate_count_distinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(536), tables: &EXASOL_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(537), tables: &EXASOL_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(538), tables: &EXASOL_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(549), tables: &EXASOL_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(557), tables: &EXASOL_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(558), tables: &EXASOL_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(559), tables: &EXASOL_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(562), tables: &EXASOL_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(563), tables: &EXASOL_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(565), tables: &EXASOL_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(566), tables: &EXASOL_TABLES }),
        "AsciiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(567), tables: &EXASOL_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(568), tables: &EXASOL_TABLES }),
        "AsinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(569), tables: &EXASOL_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(570), tables: &EXASOL_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(571), tables: &EXASOL_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(572), tables: &EXASOL_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(573), tables: &EXASOL_TABLES }),
        "Atan2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(574), tables: &EXASOL_TABLES }),
        "AtanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(575), tables: &EXASOL_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(576), tables: &EXASOL_TABLES }),
        "AttemptsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(577), tables: &EXASOL_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(578), tables: &EXASOL_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(579), tables: &EXASOL_TABLES }),
        "AuthenticatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(580), tables: &EXASOL_TABLES }),
        "AuthidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(581), tables: &EXASOL_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(582), tables: &EXASOL_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(583), tables: &EXASOL_TABLES }),
        "AutoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(585), tables: &EXASOL_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(586), tables: &EXASOL_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(587), tables: &EXASOL_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(588), tables: &EXASOL_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(589), tables: &EXASOL_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(601), tables: &EXASOL_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(602), tables: &EXASOL_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(603), tables: &EXASOL_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(604), tables: &EXASOL_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(605), tables: &EXASOL_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(606), tables: &EXASOL_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(607), tables: &EXASOL_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(608), tables: &EXASOL_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(609), tables: &EXASOL_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(614), tables: &EXASOL_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(615), tables: &EXASOL_TABLES }),
        "Bit_andKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(616), tables: &EXASOL_TABLES }),
        "Bit_checkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(617), tables: &EXASOL_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(618), tables: &EXASOL_TABLES }),
        "Bit_lrotateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(619), tables: &EXASOL_TABLES }),
        "Bit_lshiftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(620), tables: &EXASOL_TABLES }),
        "Bit_notKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(621), tables: &EXASOL_TABLES }),
        "Bit_orKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(622), tables: &EXASOL_TABLES }),
        "Bit_rrotateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(623), tables: &EXASOL_TABLES }),
        "Bit_rshiftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(624), tables: &EXASOL_TABLES }),
        "Bit_setKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(625), tables: &EXASOL_TABLES }),
        "Bit_to_numKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(626), tables: &EXASOL_TABLES }),
        "Bit_xorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(627), tables: &EXASOL_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(628), tables: &EXASOL_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(629), tables: &EXASOL_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &EXASOL_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &EXASOL_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(636), tables: &EXASOL_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(637), tables: &EXASOL_TABLES }),
        "BlockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(638), tables: &EXASOL_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(639), tables: &EXASOL_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(640), tables: &EXASOL_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(643), tables: &EXASOL_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(644), tables: &EXASOL_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(648), tables: &EXASOL_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(649), tables: &EXASOL_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(656), tables: &EXASOL_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(659), tables: &EXASOL_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(660), tables: &EXASOL_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(662), tables: &EXASOL_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(663), tables: &EXASOL_TABLES }),
        "ByteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(664), tables: &EXASOL_TABLES }),
        "CSVColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &EXASOL_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(685), tables: &EXASOL_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(687), tables: &EXASOL_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(693), tables: &EXASOL_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(694), tables: &EXASOL_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(695), tables: &EXASOL_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(696), tables: &EXASOL_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(697), tables: &EXASOL_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(698), tables: &EXASOL_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(722), tables: &EXASOL_TABLES }),
        "CasespecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(723), tables: &EXASOL_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(724), tables: &EXASOL_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(725), tables: &EXASOL_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(726), tables: &EXASOL_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(727), tables: &EXASOL_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(728), tables: &EXASOL_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(729), tables: &EXASOL_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(730), tables: &EXASOL_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(731), tables: &EXASOL_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(734), tables: &EXASOL_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(735), tables: &EXASOL_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(736), tables: &EXASOL_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(737), tables: &EXASOL_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(738), tables: &EXASOL_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(739), tables: &EXASOL_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(740), tables: &EXASOL_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(741), tables: &EXASOL_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(742), tables: &EXASOL_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(743), tables: &EXASOL_TABLES }),
        "ChrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(744), tables: &EXASOL_TABLES }),
        "ClearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(745), tables: &EXASOL_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(746), tables: &EXASOL_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(747), tables: &EXASOL_TABLES }),
        "CloseSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(748), tables: &EXASOL_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(751), tables: &EXASOL_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(752), tables: &EXASOL_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(753), tables: &EXASOL_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(754), tables: &EXASOL_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(755), tables: &EXASOL_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(756), tables: &EXASOL_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(757), tables: &EXASOL_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(763), tables: &EXASOL_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(764), tables: &EXASOL_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(765), tables: &EXASOL_TABLES }),
        "Cologne_phoneticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(766), tables: &EXASOL_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(767), tables: &EXASOL_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(768), tables: &EXASOL_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(769), tables: &EXASOL_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(770), tables: &EXASOL_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(775), tables: &EXASOL_TABLES }),
        "ColumnDatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(789), tables: &EXASOL_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(792), tables: &EXASOL_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(795), tables: &EXASOL_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(796), tables: &EXASOL_TABLES }),
        "ColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(797), tables: &EXASOL_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(803), tables: &EXASOL_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(804), tables: &EXASOL_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(805), tables: &EXASOL_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(806), tables: &EXASOL_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(807), tables: &EXASOL_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(808), tables: &EXASOL_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(812), tables: &EXASOL_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(813), tables: &EXASOL_TABLES }),
        "CommentStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(814), tables: &EXASOL_TABLES }),
        "CommentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(845), tables: &EXASOL_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(846), tables: &EXASOL_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(847), tables: &EXASOL_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(848), tables: &EXASOL_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(857), tables: &EXASOL_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(858), tables: &EXASOL_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(859), tables: &EXASOL_TABLES }),
        "ConcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(860), tables: &EXASOL_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &EXASOL_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(864), tables: &EXASOL_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(865), tables: &EXASOL_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(866), tables: &EXASOL_TABLES }),
        "ConnectByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(870), tables: &EXASOL_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(893), tables: &EXASOL_TABLES }),
        "Connect_by_iscycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(894), tables: &EXASOL_TABLES }),
        "Connect_by_isleafKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(895), tables: &EXASOL_TABLES }),
        "Connect_by_rootKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(896), tables: &EXASOL_TABLES }),
        "ConnectionDefinition" => Some(RootGrammar { grammar_id: GrammarId(897), tables: &EXASOL_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(907), tables: &EXASOL_TABLES }),
        "ConstantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(908), tables: &EXASOL_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(909), tables: &EXASOL_TABLES }),
        "Constraint_state_defaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(910), tables: &EXASOL_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(911), tables: &EXASOL_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(912), tables: &EXASOL_TABLES }),
        "ConsumerGroupParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(913), tables: &EXASOL_TABLES }),
        "ConsumerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(926), tables: &EXASOL_TABLES }),
        "Consumer_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(927), tables: &EXASOL_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(928), tables: &EXASOL_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(929), tables: &EXASOL_TABLES }),
        "ControlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(930), tables: &EXASOL_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(931), tables: &EXASOL_TABLES }),
        "Convert_tzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(932), tables: &EXASOL_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(933), tables: &EXASOL_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(934), tables: &EXASOL_TABLES }),
        "CosKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(935), tables: &EXASOL_TABLES }),
        "CoshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &EXASOL_TABLES }),
        "CotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &EXASOL_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &EXASOL_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &EXASOL_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &EXASOL_TABLES }),
        "Cpu_weightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(941), tables: &EXASOL_TABLES }),
        "CreateAdapterScriptStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(942), tables: &EXASOL_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(955), tables: &EXASOL_TABLES }),
        "CreateConnectionSegment" => Some(RootGrammar { grammar_id: GrammarId(982), tables: &EXASOL_TABLES }),
        "CreateConsumerGroupSegment" => Some(RootGrammar { grammar_id: GrammarId(989), tables: &EXASOL_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(997), tables: &EXASOL_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1002), tables: &EXASOL_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1032), tables: &EXASOL_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1045), tables: &EXASOL_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1046), tables: &EXASOL_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1066), tables: &EXASOL_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1070), tables: &EXASOL_TABLES }),
        "CreateScriptingLuaScriptStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1075), tables: &EXASOL_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1093), tables: &EXASOL_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1113), tables: &EXASOL_TABLES }),
        "CreateTableLikeClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1119), tables: &EXASOL_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1142), tables: &EXASOL_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1162), tables: &EXASOL_TABLES }),
        "CreateUDFScriptStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1229), tables: &EXASOL_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1254), tables: &EXASOL_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1264), tables: &EXASOL_TABLES }),
        "CreateVirtualSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1280), tables: &EXASOL_TABLES }),
        "CreatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1294), tables: &EXASOL_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1295), tables: &EXASOL_TABLES }),
        "CsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1296), tables: &EXASOL_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1297), tables: &EXASOL_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1298), tables: &EXASOL_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1299), tables: &EXASOL_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1300), tables: &EXASOL_TABLES }),
        "CurdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1306), tables: &EXASOL_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1307), tables: &EXASOL_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1308), tables: &EXASOL_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1309), tables: &EXASOL_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1310), tables: &EXASOL_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1311), tables: &EXASOL_TABLES }),
        "Current_sessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1312), tables: &EXASOL_TABLES }),
        "Current_statementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1313), tables: &EXASOL_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1314), tables: &EXASOL_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1315), tables: &EXASOL_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1316), tables: &EXASOL_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1317), tables: &EXASOL_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1318), tables: &EXASOL_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1319), tables: &EXASOL_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1320), tables: &EXASOL_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "DatalinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1321), tables: &EXASOL_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1322), tables: &EXASOL_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1326), tables: &EXASOL_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1405), tables: &EXASOL_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1406), tables: &EXASOL_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1407), tables: &EXASOL_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1408), tables: &EXASOL_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1414), tables: &EXASOL_TABLES }),
        "Date_truncKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1419), tables: &EXASOL_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1420), tables: &EXASOL_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1421), tables: &EXASOL_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1422), tables: &EXASOL_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1423), tables: &EXASOL_TABLES }),
        "Days_betweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1424), tables: &EXASOL_TABLES }),
        "DbtimezoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1425), tables: &EXASOL_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1426), tables: &EXASOL_TABLES }),
        "DebugKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1427), tables: &EXASOL_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1428), tables: &EXASOL_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1429), tables: &EXASOL_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1430), tables: &EXASOL_TABLES }),
        "DecodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1431), tables: &EXASOL_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(234), tables: &EXASOL_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1432), tables: &EXASOL_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1433), tables: &EXASOL_TABLES }),
        "Default_consumer_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1436), tables: &EXASOL_TABLES }),
        "Default_like_escape_characterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1437), tables: &EXASOL_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1438), tables: &EXASOL_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1439), tables: &EXASOL_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1440), tables: &EXASOL_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1441), tables: &EXASOL_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1442), tables: &EXASOL_TABLES }),
        "DegreesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1443), tables: &EXASOL_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1444), tables: &EXASOL_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1445), tables: &EXASOL_TABLES }),
        "DelimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1454), tables: &EXASOL_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1455), tables: &EXASOL_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1456), tables: &EXASOL_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1457), tables: &EXASOL_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1458), tables: &EXASOL_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1459), tables: &EXASOL_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1460), tables: &EXASOL_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1461), tables: &EXASOL_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1462), tables: &EXASOL_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1463), tables: &EXASOL_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1467), tables: &EXASOL_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1468), tables: &EXASOL_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1469), tables: &EXASOL_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1470), tables: &EXASOL_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1471), tables: &EXASOL_TABLES }),
        "DisabledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1472), tables: &EXASOL_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1473), tables: &EXASOL_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1474), tables: &EXASOL_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1475), tables: &EXASOL_TABLES }),
        "DistributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1476), tables: &EXASOL_TABLES }),
        "DistributionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1477), tables: &EXASOL_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1478), tables: &EXASOL_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1479), tables: &EXASOL_TABLES }),
        "DlurlcompleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1480), tables: &EXASOL_TABLES }),
        "DlurlpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1481), tables: &EXASOL_TABLES }),
        "DlurlpathonlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1482), tables: &EXASOL_TABLES }),
        "DlurlschemeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1483), tables: &EXASOL_TABLES }),
        "DlurlserverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1484), tables: &EXASOL_TABLES }),
        "DlvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1485), tables: &EXASOL_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1486), tables: &EXASOL_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1487), tables: &EXASOL_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1488), tables: &EXASOL_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1489), tables: &EXASOL_TABLES }),
        "DownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1490), tables: &EXASOL_TABLES }),
        "DriverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1491), tables: &EXASOL_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1492), tables: &EXASOL_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1495), tables: &EXASOL_TABLES }),
        "DropConnectionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1503), tables: &EXASOL_TABLES }),
        "DropConsumerGroupSegment" => Some(RootGrammar { grammar_id: GrammarId(1508), tables: &EXASOL_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1514), tables: &EXASOL_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1520), tables: &EXASOL_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1526), tables: &EXASOL_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1532), tables: &EXASOL_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1533), tables: &EXASOL_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1538), tables: &EXASOL_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1544), tables: &EXASOL_TABLES }),
        "DropScriptStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1552), tables: &EXASOL_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1559), tables: &EXASOL_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1563), tables: &EXASOL_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1572), tables: &EXASOL_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1577), tables: &EXASOL_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1583), tables: &EXASOL_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1589), tables: &EXASOL_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1595), tables: &EXASOL_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1596), tables: &EXASOL_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1597), tables: &EXASOL_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1598), tables: &EXASOL_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1599), tables: &EXASOL_TABLES }),
        "Edit_distanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1600), tables: &EXASOL_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1601), tables: &EXASOL_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1604), tables: &EXASOL_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1605), tables: &EXASOL_TABLES }),
        "ElsifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1606), tables: &EXASOL_TABLES }),
        "EmitsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1607), tables: &EXASOL_TABLES }),
        "EmitsSegment" => Some(RootGrammar { grammar_id: GrammarId(1608), tables: &EXASOL_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1612), tables: &EXASOL_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1613), tables: &EXASOL_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1614), tables: &EXASOL_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1617), tables: &EXASOL_TABLES }),
        "EnabledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1618), tables: &EXASOL_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1619), tables: &EXASOL_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1620), tables: &EXASOL_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &EXASOL_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1621), tables: &EXASOL_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1622), tables: &EXASOL_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(556), tables: &EXASOL_TABLES }),
        "EndifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1623), tables: &EXASOL_TABLES }),
        "EnforceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1624), tables: &EXASOL_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1625), tables: &EXASOL_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1626), tables: &EXASOL_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1627), tables: &EXASOL_TABLES }),
        "ErrorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1628), tables: &EXASOL_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1629), tables: &EXASOL_TABLES }),
        "EscapedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1630), tables: &EXASOL_TABLES }),
        "EstimateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1631), tables: &EXASOL_TABLES }),
        "EvaluateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1632), tables: &EXASOL_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1633), tables: &EXASOL_TABLES }),
        "ExaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1634), tables: &EXASOL_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1635), tables: &EXASOL_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1636), tables: &EXASOL_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1637), tables: &EXASOL_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1638), tables: &EXASOL_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1639), tables: &EXASOL_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1640), tables: &EXASOL_TABLES }),
        "ExecuteScriptSegment" => Some(RootGrammar { grammar_id: GrammarId(1641), tables: &EXASOL_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1653), tables: &EXASOL_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1654), tables: &EXASOL_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1655), tables: &EXASOL_TABLES }),
        "ExperimentalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1656), tables: &EXASOL_TABLES }),
        "ExpireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1657), tables: &EXASOL_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1658), tables: &EXASOL_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1659), tables: &EXASOL_TABLES }),
        "ExplainVirtualSegment" => Some(RootGrammar { grammar_id: GrammarId(1666), tables: &EXASOL_TABLES }),
        "ExportIntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1670), tables: &EXASOL_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1679), tables: &EXASOL_TABLES }),
        "ExportStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1680), tables: &EXASOL_TABLES }),
        "ExpressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1690), tables: &EXASOL_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1691), tables: &EXASOL_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1692), tables: &EXASOL_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1718), tables: &EXASOL_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1725), tables: &EXASOL_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1735), tables: &EXASOL_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1740), tables: &EXASOL_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1753), tables: &EXASOL_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1790), tables: &EXASOL_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1798), tables: &EXASOL_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1799), tables: &EXASOL_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1800), tables: &EXASOL_TABLES }),
        "FBVColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1801), tables: &EXASOL_TABLES }),
        "FailedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1822), tables: &EXASOL_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1823), tables: &EXASOL_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1824), tables: &EXASOL_TABLES }),
        "FbvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1825), tables: &EXASOL_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1826), tables: &EXASOL_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1843), tables: &EXASOL_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1844), tables: &EXASOL_TABLES }),
        "FileOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(1845), tables: &EXASOL_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1885), tables: &EXASOL_TABLES }),
        "FilesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1892), tables: &EXASOL_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1893), tables: &EXASOL_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1899), tables: &EXASOL_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1900), tables: &EXASOL_TABLES }),
        "First_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1901), tables: &EXASOL_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1902), tables: &EXASOL_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1903), tables: &EXASOL_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1904), tables: &EXASOL_TABLES }),
        "FlushStatisticsSegment" => Some(RootGrammar { grammar_id: GrammarId(1905), tables: &EXASOL_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1908), tables: &EXASOL_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1909), tables: &EXASOL_TABLES }),
        "ForallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1910), tables: &EXASOL_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1911), tables: &EXASOL_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1912), tables: &EXASOL_TABLES }),
        "ForeignKeyReferencesClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1915), tables: &EXASOL_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1919), tables: &EXASOL_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1920), tables: &EXASOL_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1921), tables: &EXASOL_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1922), tables: &EXASOL_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1923), tables: &EXASOL_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1942), tables: &EXASOL_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1945), tables: &EXASOL_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1946), tables: &EXASOL_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1950), tables: &EXASOL_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1967), tables: &EXASOL_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1990), tables: &EXASOL_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2017), tables: &EXASOL_TABLES }),
        "From_posix_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2018), tables: &EXASOL_TABLES }),
        "FsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2019), tables: &EXASOL_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &EXASOL_TABLES }),
        "FunctionAssignmentSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &EXASOL_TABLES }),
        "FunctionBodySegment" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &EXASOL_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2036), tables: &EXASOL_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2037), tables: &EXASOL_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2078), tables: &EXASOL_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &EXASOL_TABLES }),
        "FunctionForLoopSegment" => Some(RootGrammar { grammar_id: GrammarId(2087), tables: &EXASOL_TABLES }),
        "FunctionIfBranchSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &EXASOL_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &EXASOL_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2136), tables: &EXASOL_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2137), tables: &EXASOL_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2147), tables: &EXASOL_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &EXASOL_TABLES }),
        "FunctionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "FunctionScriptStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &EXASOL_TABLES }),
        "FunctionScriptTerminatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2168), tables: &EXASOL_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2169), tables: &EXASOL_TABLES }),
        "FunctionWhileLoopSegment" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &EXASOL_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2192), tables: &EXASOL_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2193), tables: &EXASOL_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2194), tables: &EXASOL_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2195), tables: &EXASOL_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2196), tables: &EXASOL_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2197), tables: &EXASOL_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &EXASOL_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2199), tables: &EXASOL_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2200), tables: &EXASOL_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2201), tables: &EXASOL_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &EXASOL_TABLES }),
        "GraphKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &EXASOL_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &EXASOL_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &EXASOL_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &EXASOL_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &EXASOL_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &EXASOL_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &EXASOL_TABLES }),
        "Group_concatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &EXASOL_TABLES }),
        "Group_temp_db_ram_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &EXASOL_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(2257), tables: &EXASOL_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2265), tables: &EXASOL_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2266), tables: &EXASOL_TABLES }),
        "Grouping_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &EXASOL_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &EXASOL_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2276), tables: &EXASOL_TABLES }),
        "HasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2277), tables: &EXASOL_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &EXASOL_TABLES }),
        "Hash_md5KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2279), tables: &EXASOL_TABLES }),
        "Hash_sha1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2280), tables: &EXASOL_TABLES }),
        "Hash_sha256KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2281), tables: &EXASOL_TABLES }),
        "Hash_sha512KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2282), tables: &EXASOL_TABLES }),
        "Hash_shaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2283), tables: &EXASOL_TABLES }),
        "Hash_tigerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &EXASOL_TABLES }),
        "HashtypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2285), tables: &EXASOL_TABLES }),
        "Hashtype_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &EXASOL_TABLES }),
        "Hashtype_md5KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &EXASOL_TABLES }),
        "Hashtype_sha1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &EXASOL_TABLES }),
        "Hashtype_sha256KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &EXASOL_TABLES }),
        "Hashtype_sha512KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &EXASOL_TABLES }),
        "Hashtype_shaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &EXASOL_TABLES }),
        "Hashtype_tigerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &EXASOL_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &EXASOL_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &EXASOL_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &EXASOL_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &EXASOL_TABLES }),
        "HighKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2308), tables: &EXASOL_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2309), tables: &EXASOL_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2310), tables: &EXASOL_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2311), tables: &EXASOL_TABLES }),
        "Hours_betweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2312), tables: &EXASOL_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2313), tables: &EXASOL_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2314), tables: &EXASOL_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2315), tables: &EXASOL_TABLES }),
        "Idle_timeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2316), tables: &EXASOL_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2317), tables: &EXASOL_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2320), tables: &EXASOL_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2321), tables: &EXASOL_TABLES }),
        "IfnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2325), tables: &EXASOL_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2326), tables: &EXASOL_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2327), tables: &EXASOL_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2332), tables: &EXASOL_TABLES }),
        "ImpersonateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2333), tables: &EXASOL_TABLES }),
        "ImpersonateSegment" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &EXASOL_TABLES }),
        "ImpersonationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &EXASOL_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &EXASOL_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(701), tables: &EXASOL_TABLES }),
        "ImportColumnsSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &EXASOL_TABLES }),
        "ImportErrorDestinationSegment" => Some(RootGrammar { grammar_id: GrammarId(2343), tables: &EXASOL_TABLES }),
        "ImportErrorsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2359), tables: &EXASOL_TABLES }),
        "ImportFromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2369), tables: &EXASOL_TABLES }),
        "ImportFromExportIntoDbSrcSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &EXASOL_TABLES }),
        "ImportFromExportIntoFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &EXASOL_TABLES }),
        "ImportFromExportIntoScriptSegment" => Some(RootGrammar { grammar_id: GrammarId(2436), tables: &EXASOL_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &EXASOL_TABLES }),
        "ImportStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &EXASOL_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2463), tables: &EXASOL_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2464), tables: &EXASOL_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2474), tables: &EXASOL_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &EXASOL_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2475), tables: &EXASOL_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2480), tables: &EXASOL_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2481), tables: &EXASOL_TABLES }),
        "InitcapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2482), tables: &EXASOL_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2483), tables: &EXASOL_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2484), tables: &EXASOL_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2485), tables: &EXASOL_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2486), tables: &EXASOL_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2487), tables: &EXASOL_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2488), tables: &EXASOL_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2489), tables: &EXASOL_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2500), tables: &EXASOL_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2501), tables: &EXASOL_TABLES }),
        "InstrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &EXASOL_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &EXASOL_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &EXASOL_TABLES }),
        "IntegrityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &EXASOL_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2506), tables: &EXASOL_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2507), tables: &EXASOL_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2545), tables: &EXASOL_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2546), tables: &EXASOL_TABLES }),
        "IntoTableSegment" => Some(RootGrammar { grammar_id: GrammarId(2547), tables: &EXASOL_TABLES }),
        "InvalidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2551), tables: &EXASOL_TABLES }),
        "InverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2552), tables: &EXASOL_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2553), tables: &EXASOL_TABLES }),
        "IprocKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &EXASOL_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &EXASOL_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(2561), tables: &EXASOL_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2566), tables: &EXASOL_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2567), tables: &EXASOL_TABLES }),
        "Is_booleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2568), tables: &EXASOL_TABLES }),
        "Is_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &EXASOL_TABLES }),
        "Is_dsintervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &EXASOL_TABLES }),
        "Is_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &EXASOL_TABLES }),
        "Is_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &EXASOL_TABLES }),
        "Is_ymintervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &EXASOL_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2574), tables: &EXASOL_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2575), tables: &EXASOL_TABLES }),
        "JavaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2576), tables: &EXASOL_TABLES }),
        "JavascriptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2577), tables: &EXASOL_TABLES }),
        "JdbcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2578), tables: &EXASOL_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2579), tables: &EXASOL_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &EXASOL_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2602), tables: &EXASOL_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2604), tables: &EXASOL_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2605), tables: &EXASOL_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2612), tables: &EXASOL_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2620), tables: &EXASOL_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2625), tables: &EXASOL_TABLES }),
        "Json_extractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2626), tables: &EXASOL_TABLES }),
        "Json_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2627), tables: &EXASOL_TABLES }),
        "KeepKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2628), tables: &EXASOL_TABLES }),
        "KerberosKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2629), tables: &EXASOL_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2630), tables: &EXASOL_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2631), tables: &EXASOL_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &EXASOL_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2633), tables: &EXASOL_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2634), tables: &EXASOL_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2635), tables: &EXASOL_TABLES }),
        "KillSegment" => Some(RootGrammar { grammar_id: GrammarId(2636), tables: &EXASOL_TABLES }),
        "LagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2654), tables: &EXASOL_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2655), tables: &EXASOL_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2656), tables: &EXASOL_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2657), tables: &EXASOL_TABLES }),
        "Last_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2658), tables: &EXASOL_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2659), tables: &EXASOL_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &EXASOL_TABLES }),
        "LcaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &EXASOL_TABLES }),
        "LdapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &EXASOL_TABLES }),
        "LeadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2663), tables: &EXASOL_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2664), tables: &EXASOL_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2665), tables: &EXASOL_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &EXASOL_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &EXASOL_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2668), tables: &EXASOL_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2669), tables: &EXASOL_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2672), tables: &EXASOL_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2673), tables: &EXASOL_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2674), tables: &EXASOL_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2682), tables: &EXASOL_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2685), tables: &EXASOL_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2686), tables: &EXASOL_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2687), tables: &EXASOL_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2699), tables: &EXASOL_TABLES }),
        "LinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2700), tables: &EXASOL_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2701), tables: &EXASOL_TABLES }),
        "ListaggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2702), tables: &EXASOL_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2703), tables: &EXASOL_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2713), tables: &EXASOL_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2714), tables: &EXASOL_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2715), tables: &EXASOL_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(2716), tables: &EXASOL_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2720), tables: &EXASOL_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2721), tables: &EXASOL_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2722), tables: &EXASOL_TABLES }),
        "LocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2723), tables: &EXASOL_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2724), tables: &EXASOL_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2725), tables: &EXASOL_TABLES }),
        "Log10KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &EXASOL_TABLES }),
        "Log2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2727), tables: &EXASOL_TABLES }),
        "LogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2728), tables: &EXASOL_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2729), tables: &EXASOL_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2730), tables: &EXASOL_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2731), tables: &EXASOL_TABLES }),
        "LongvarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2732), tables: &EXASOL_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2733), tables: &EXASOL_TABLES }),
        "LowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2734), tables: &EXASOL_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2735), tables: &EXASOL_TABLES }),
        "LpadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2736), tables: &EXASOL_TABLES }),
        "LtrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2737), tables: &EXASOL_TABLES }),
        "LuaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2738), tables: &EXASOL_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2739), tables: &EXASOL_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2740), tables: &EXASOL_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2741), tables: &EXASOL_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2742), tables: &EXASOL_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2743), tables: &EXASOL_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2744), tables: &EXASOL_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2745), tables: &EXASOL_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2746), tables: &EXASOL_TABLES }),
        "MaximalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2747), tables: &EXASOL_TABLES }),
        "MedianKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2748), tables: &EXASOL_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2749), tables: &EXASOL_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2752), tables: &EXASOL_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2757), tables: &EXASOL_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2760), tables: &EXASOL_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(2761), tables: &EXASOL_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2768), tables: &EXASOL_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2775), tables: &EXASOL_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2781), tables: &EXASOL_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &EXASOL_TABLES }),
        "MessageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &EXASOL_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &EXASOL_TABLES }),
        "MidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2807), tables: &EXASOL_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2808), tables: &EXASOL_TABLES }),
        "Min_scaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &EXASOL_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &EXASOL_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(2811), tables: &EXASOL_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2812), tables: &EXASOL_TABLES }),
        "Minutes_betweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2813), tables: &EXASOL_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2814), tables: &EXASOL_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2815), tables: &EXASOL_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2816), tables: &EXASOL_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2817), tables: &EXASOL_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(2818), tables: &EXASOL_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2819), tables: &EXASOL_TABLES }),
        "Months_betweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2820), tables: &EXASOL_TABLES }),
        "MulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2821), tables: &EXASOL_TABLES }),
        "MultipleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2822), tables: &EXASOL_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(2823), tables: &EXASOL_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2824), tables: &EXASOL_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2825), tables: &EXASOL_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2826), tables: &EXASOL_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2833), tables: &EXASOL_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2837), tables: &EXASOL_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2838), tables: &EXASOL_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2839), tables: &EXASOL_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2840), tables: &EXASOL_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2843), tables: &EXASOL_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2844), tables: &EXASOL_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &EXASOL_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &EXASOL_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &EXASOL_TABLES }),
        "NeverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &EXASOL_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &EXASOL_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &EXASOL_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2851), tables: &EXASOL_TABLES }),
        "NiceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &EXASOL_TABLES }),
        "Nls_date_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2853), tables: &EXASOL_TABLES }),
        "Nls_date_languageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2854), tables: &EXASOL_TABLES }),
        "Nls_first_day_of_weekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2855), tables: &EXASOL_TABLES }),
        "Nls_numeric_charactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2856), tables: &EXASOL_TABLES }),
        "Nls_timestamp_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2857), tables: &EXASOL_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &EXASOL_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2859), tables: &EXASOL_TABLES }),
        "NologgingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2860), tables: &EXASOL_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2861), tables: &EXASOL_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2871), tables: &EXASOL_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2872), tables: &EXASOL_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2877), tables: &EXASOL_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2883), tables: &EXASOL_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2884), tables: &EXASOL_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2885), tables: &EXASOL_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2886), tables: &EXASOL_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2887), tables: &EXASOL_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2894), tables: &EXASOL_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2895), tables: &EXASOL_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &EXASOL_TABLES }),
        "NoticeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2897), tables: &EXASOL_TABLES }),
        "NowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2898), tables: &EXASOL_TABLES }),
        "NprocKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2899), tables: &EXASOL_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2900), tables: &EXASOL_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2901), tables: &EXASOL_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2902), tables: &EXASOL_TABLES }),
        "NullifzeroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2903), tables: &EXASOL_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &EXASOL_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2905), tables: &EXASOL_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2906), tables: &EXASOL_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2907), tables: &EXASOL_TABLES }),
        "NumtodsintervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2908), tables: &EXASOL_TABLES }),
        "NumtoymintervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2909), tables: &EXASOL_TABLES }),
        "Nvarchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2910), tables: &EXASOL_TABLES }),
        "NvarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2911), tables: &EXASOL_TABLES }),
        "Nvl2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2912), tables: &EXASOL_TABLES }),
        "NvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2913), tables: &EXASOL_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2914), tables: &EXASOL_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2915), tables: &EXASOL_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2919), tables: &EXASOL_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2923), tables: &EXASOL_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &EXASOL_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2940), tables: &EXASOL_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &EXASOL_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2942), tables: &EXASOL_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2943), tables: &EXASOL_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2944), tables: &EXASOL_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2945), tables: &EXASOL_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2954), tables: &EXASOL_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2955), tables: &EXASOL_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2956), tables: &EXASOL_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &EXASOL_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &EXASOL_TABLES }),
        "OpenSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(2959), tables: &EXASOL_TABLES }),
        "OpenidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &EXASOL_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2964), tables: &EXASOL_TABLES }),
        "OptimizerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &EXASOL_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2966), tables: &EXASOL_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &EXASOL_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &EXASOL_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2969), tables: &EXASOL_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2970), tables: &EXASOL_TABLES }),
        "OraKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2973), tables: &EXASOL_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2974), tables: &EXASOL_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(2994), tables: &EXASOL_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3002), tables: &EXASOL_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(3003), tables: &EXASOL_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3006), tables: &EXASOL_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3007), tables: &EXASOL_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &EXASOL_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &EXASOL_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3010), tables: &EXASOL_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3011), tables: &EXASOL_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3012), tables: &EXASOL_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3019), tables: &EXASOL_TABLES }),
        "OverflowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3020), tables: &EXASOL_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3021), tables: &EXASOL_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3030), tables: &EXASOL_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3031), tables: &EXASOL_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3032), tables: &EXASOL_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3033), tables: &EXASOL_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3034), tables: &EXASOL_TABLES }),
        "PaddingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3035), tables: &EXASOL_TABLES }),
        "Parallel_enableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3036), tables: &EXASOL_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3037), tables: &EXASOL_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3038), tables: &EXASOL_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3039), tables: &EXASOL_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3040), tables: &EXASOL_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3041), tables: &EXASOL_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3042), tables: &EXASOL_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3043), tables: &EXASOL_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3044), tables: &EXASOL_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3051), tables: &EXASOL_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3052), tables: &EXASOL_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3053), tables: &EXASOL_TABLES }),
        "PasswordLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3054), tables: &EXASOL_TABLES }),
        "Password_expiry_policyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3055), tables: &EXASOL_TABLES }),
        "Password_security_policyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3056), tables: &EXASOL_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3057), tables: &EXASOL_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(3058), tables: &EXASOL_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3065), tables: &EXASOL_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3066), tables: &EXASOL_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3067), tables: &EXASOL_TABLES }),
        "PermissionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3068), tables: &EXASOL_TABLES }),
        "PiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3069), tables: &EXASOL_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(3070), tables: &EXASOL_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3071), tables: &EXASOL_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3072), tables: &EXASOL_TABLES }),
        "PlusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3073), tables: &EXASOL_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(3074), tables: &EXASOL_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3075), tables: &EXASOL_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(3076), tables: &EXASOL_TABLES }),
        "Posix_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3077), tables: &EXASOL_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3078), tables: &EXASOL_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3087), tables: &EXASOL_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3088), tables: &EXASOL_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3089), tables: &EXASOL_TABLES }),
        "PrecedenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3090), tables: &EXASOL_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3091), tables: &EXASOL_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3092), tables: &EXASOL_TABLES }),
        "PreferringClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3093), tables: &EXASOL_TABLES }),
        "PreferringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3099), tables: &EXASOL_TABLES }),
        "PreferringPlusPriorTermSegment" => Some(RootGrammar { grammar_id: GrammarId(3100), tables: &EXASOL_TABLES }),
        "PreferringPreferenceTermSegment" => Some(RootGrammar { grammar_id: GrammarId(3112), tables: &EXASOL_TABLES }),
        "PreloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3131), tables: &EXASOL_TABLES }),
        "PreloadSegment" => Some(RootGrammar { grammar_id: GrammarId(3132), tables: &EXASOL_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3151), tables: &EXASOL_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3152), tables: &EXASOL_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3153), tables: &EXASOL_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3156), tables: &EXASOL_TABLES }),
        "PrincipalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3157), tables: &EXASOL_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3158), tables: &EXASOL_TABLES }),
        "PrivilegeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3159), tables: &EXASOL_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3160), tables: &EXASOL_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3161), tables: &EXASOL_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3162), tables: &EXASOL_TABLES }),
        "PythonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3163), tables: &EXASOL_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3164), tables: &EXASOL_TABLES }),
        "QualifyClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3167), tables: &EXASOL_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3170), tables: &EXASOL_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3171), tables: &EXASOL_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3179), tables: &EXASOL_TABLES }),
        "Query_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3180), tables: &EXASOL_TABLES }),
        "Query_timeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3181), tables: &EXASOL_TABLES }),
        "QuietKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3182), tables: &EXASOL_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3183), tables: &EXASOL_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3184), tables: &EXASOL_TABLES }),
        "RKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3185), tables: &EXASOL_TABLES }),
        "RadiansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3186), tables: &EXASOL_TABLES }),
        "RandKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3187), tables: &EXASOL_TABLES }),
        "RandomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3188), tables: &EXASOL_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3189), tables: &EXASOL_TABLES }),
        "RangeOperator" => Some(RootGrammar { grammar_id: GrammarId(3190), tables: &EXASOL_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3191), tables: &EXASOL_TABLES }),
        "Ratio_to_reportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3192), tables: &EXASOL_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(3193), tables: &EXASOL_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3194), tables: &EXASOL_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3195), tables: &EXASOL_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(3196), tables: &EXASOL_TABLES }),
        "Raw_size_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3197), tables: &EXASOL_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3198), tables: &EXASOL_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3199), tables: &EXASOL_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3200), tables: &EXASOL_TABLES }),
        "RecompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3201), tables: &EXASOL_TABLES }),
        "RecompressReorganizeSegment" => Some(RootGrammar { grammar_id: GrammarId(3202), tables: &EXASOL_TABLES }),
        "RecordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3224), tables: &EXASOL_TABLES }),
        "RecoveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3225), tables: &EXASOL_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3226), tables: &EXASOL_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3227), tables: &EXASOL_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3228), tables: &EXASOL_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(3242), tables: &EXASOL_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3248), tables: &EXASOL_TABLES }),
        "ReferencingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &EXASOL_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3253), tables: &EXASOL_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3254), tables: &EXASOL_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3266), tables: &EXASOL_TABLES }),
        "Regexp_instrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3267), tables: &EXASOL_TABLES }),
        "Regexp_likeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3268), tables: &EXASOL_TABLES }),
        "Regexp_replaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3269), tables: &EXASOL_TABLES }),
        "Regexp_substrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3270), tables: &EXASOL_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3271), tables: &EXASOL_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3272), tables: &EXASOL_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3273), tables: &EXASOL_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3274), tables: &EXASOL_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3275), tables: &EXASOL_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3276), tables: &EXASOL_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3277), tables: &EXASOL_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3278), tables: &EXASOL_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3279), tables: &EXASOL_TABLES }),
        "RejectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3280), tables: &EXASOL_TABLES }),
        "RejectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3287), tables: &EXASOL_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3288), tables: &EXASOL_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3289), tables: &EXASOL_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3290), tables: &EXASOL_TABLES }),
        "RenameStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3291), tables: &EXASOL_TABLES }),
        "ReorganizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3308), tables: &EXASOL_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3309), tables: &EXASOL_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3310), tables: &EXASOL_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3311), tables: &EXASOL_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3312), tables: &EXASOL_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3313), tables: &EXASOL_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3314), tables: &EXASOL_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3315), tables: &EXASOL_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3316), tables: &EXASOL_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3317), tables: &EXASOL_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3318), tables: &EXASOL_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3319), tables: &EXASOL_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3320), tables: &EXASOL_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3321), tables: &EXASOL_TABLES }),
        "ReverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3322), tables: &EXASOL_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3323), tables: &EXASOL_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3324), tables: &EXASOL_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3339), tables: &EXASOL_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3340), tables: &EXASOL_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3341), tables: &EXASOL_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &EXASOL_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3343), tables: &EXASOL_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3344), tables: &EXASOL_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3345), tables: &EXASOL_TABLES }),
        "RoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3346), tables: &EXASOL_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3347), tables: &EXASOL_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3348), tables: &EXASOL_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3349), tables: &EXASOL_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3350), tables: &EXASOL_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3351), tables: &EXASOL_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3352), tables: &EXASOL_TABLES }),
        "RowtypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3353), tables: &EXASOL_TABLES }),
        "RpadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3354), tables: &EXASOL_TABLES }),
        "RtrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3355), tables: &EXASOL_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3356), tables: &EXASOL_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3368), tables: &EXASOL_TABLES }),
        "ScalarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3369), tables: &EXASOL_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3370), tables: &EXASOL_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3371), tables: &EXASOL_TABLES }),
        "SchemeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &EXASOL_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3373), tables: &EXASOL_TABLES }),
        "Scope_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3374), tables: &EXASOL_TABLES }),
        "ScriptContentSegment" => Some(RootGrammar { grammar_id: GrammarId(3375), tables: &EXASOL_TABLES }),
        "ScriptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &EXASOL_TABLES }),
        "ScriptReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "Script_languagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3378), tables: &EXASOL_TABLES }),
        "Script_output_addressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3379), tables: &EXASOL_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3380), tables: &EXASOL_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &EXASOL_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &EXASOL_TABLES }),
        "Seconds_betweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &EXASOL_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &EXASOL_TABLES }),
        "SecureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &EXASOL_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &EXASOL_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3387), tables: &EXASOL_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &EXASOL_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &EXASOL_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3414), tables: &EXASOL_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3425), tables: &EXASOL_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3426), tables: &EXASOL_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3441), tables: &EXASOL_TABLES }),
        "SelectiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3451), tables: &EXASOL_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3452), tables: &EXASOL_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(3453), tables: &EXASOL_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3454), tables: &EXASOL_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3455), tables: &EXASOL_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3456), tables: &EXASOL_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3457), tables: &EXASOL_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3464), tables: &EXASOL_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3471), tables: &EXASOL_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3472), tables: &EXASOL_TABLES }),
        "SessionParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3473), tables: &EXASOL_TABLES }),
        "Session_parameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3474), tables: &EXASOL_TABLES }),
        "Session_temp_db_ram_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3475), tables: &EXASOL_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &EXASOL_TABLES }),
        "SessiontimezoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &EXASOL_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &EXASOL_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &EXASOL_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3494), tables: &EXASOL_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3503), tables: &EXASOL_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3504), tables: &EXASOL_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3512), tables: &EXASOL_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3517), tables: &EXASOL_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(3518), tables: &EXASOL_TABLES }),
        "ShortintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3527), tables: &EXASOL_TABLES }),
        "ShutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3528), tables: &EXASOL_TABLES }),
        "SignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3529), tables: &EXASOL_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(3530), tables: &EXASOL_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3533), tables: &EXASOL_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3534), tables: &EXASOL_TABLES }),
        "SinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3535), tables: &EXASOL_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(3536), tables: &EXASOL_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(3540), tables: &EXASOL_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(3542), tables: &EXASOL_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3546), tables: &EXASOL_TABLES }),
        "SinhKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3547), tables: &EXASOL_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3548), tables: &EXASOL_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3549), tables: &EXASOL_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3552), tables: &EXASOL_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(3553), tables: &EXASOL_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(3554), tables: &EXASOL_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3555), tables: &EXASOL_TABLES }),
        "Snapshot_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3556), tables: &EXASOL_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3557), tables: &EXASOL_TABLES }),
        "SoundexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3558), tables: &EXASOL_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3559), tables: &EXASOL_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3560), tables: &EXASOL_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3561), tables: &EXASOL_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3562), tables: &EXASOL_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3563), tables: &EXASOL_TABLES }),
        "Sql_bigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3564), tables: &EXASOL_TABLES }),
        "Sql_bitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3565), tables: &EXASOL_TABLES }),
        "Sql_charKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3566), tables: &EXASOL_TABLES }),
        "Sql_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3567), tables: &EXASOL_TABLES }),
        "Sql_decimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3568), tables: &EXASOL_TABLES }),
        "Sql_doubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3569), tables: &EXASOL_TABLES }),
        "Sql_floatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3570), tables: &EXASOL_TABLES }),
        "Sql_integerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3571), tables: &EXASOL_TABLES }),
        "Sql_longvarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3572), tables: &EXASOL_TABLES }),
        "Sql_numericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3573), tables: &EXASOL_TABLES }),
        "Sql_preprocessor_scriptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3574), tables: &EXASOL_TABLES }),
        "Sql_realKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3575), tables: &EXASOL_TABLES }),
        "Sql_smallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3576), tables: &EXASOL_TABLES }),
        "Sql_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &EXASOL_TABLES }),
        "Sql_tinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3578), tables: &EXASOL_TABLES }),
        "Sql_type_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3579), tables: &EXASOL_TABLES }),
        "Sql_type_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3580), tables: &EXASOL_TABLES }),
        "Sql_varcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3581), tables: &EXASOL_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3582), tables: &EXASOL_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3583), tables: &EXASOL_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3584), tables: &EXASOL_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3585), tables: &EXASOL_TABLES }),
        "St_areaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3586), tables: &EXASOL_TABLES }),
        "St_boundaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3587), tables: &EXASOL_TABLES }),
        "St_bufferKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3588), tables: &EXASOL_TABLES }),
        "St_centroidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3589), tables: &EXASOL_TABLES }),
        "St_containsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3590), tables: &EXASOL_TABLES }),
        "St_convexhullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3591), tables: &EXASOL_TABLES }),
        "St_crossesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3592), tables: &EXASOL_TABLES }),
        "St_differenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3593), tables: &EXASOL_TABLES }),
        "St_dimensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3594), tables: &EXASOL_TABLES }),
        "St_disjointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3595), tables: &EXASOL_TABLES }),
        "St_distanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3596), tables: &EXASOL_TABLES }),
        "St_endpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &EXASOL_TABLES }),
        "St_envelopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3598), tables: &EXASOL_TABLES }),
        "St_equalsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3599), tables: &EXASOL_TABLES }),
        "St_exteriorringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3600), tables: &EXASOL_TABLES }),
        "St_force2dKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3601), tables: &EXASOL_TABLES }),
        "St_geometrynKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3602), tables: &EXASOL_TABLES }),
        "St_geometrytypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3603), tables: &EXASOL_TABLES }),
        "St_interiorringnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3604), tables: &EXASOL_TABLES }),
        "St_intersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3605), tables: &EXASOL_TABLES }),
        "St_intersectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3606), tables: &EXASOL_TABLES }),
        "St_isclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3607), tables: &EXASOL_TABLES }),
        "St_isemptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3608), tables: &EXASOL_TABLES }),
        "St_isringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3609), tables: &EXASOL_TABLES }),
        "St_issimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3610), tables: &EXASOL_TABLES }),
        "St_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3611), tables: &EXASOL_TABLES }),
        "St_max_decimal_digitsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3612), tables: &EXASOL_TABLES }),
        "St_numgeometriesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3613), tables: &EXASOL_TABLES }),
        "St_numinteriorringsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3614), tables: &EXASOL_TABLES }),
        "St_numpointsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3615), tables: &EXASOL_TABLES }),
        "St_overlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3616), tables: &EXASOL_TABLES }),
        "St_pointnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3617), tables: &EXASOL_TABLES }),
        "St_setsridKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3618), tables: &EXASOL_TABLES }),
        "St_startpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3619), tables: &EXASOL_TABLES }),
        "St_symdifferenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3620), tables: &EXASOL_TABLES }),
        "St_touchesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3621), tables: &EXASOL_TABLES }),
        "St_transformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3622), tables: &EXASOL_TABLES }),
        "St_unionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3623), tables: &EXASOL_TABLES }),
        "St_withinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &EXASOL_TABLES }),
        "St_xKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3625), tables: &EXASOL_TABLES }),
        "St_yKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3626), tables: &EXASOL_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(3627), tables: &EXASOL_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &EXASOL_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2922), tables: &EXASOL_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3628), tables: &EXASOL_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(555), tables: &EXASOL_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3629), tables: &EXASOL_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3630), tables: &EXASOL_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3631), tables: &EXASOL_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3681), tables: &EXASOL_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3682), tables: &EXASOL_TABLES }),
        "StddevKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3683), tables: &EXASOL_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3684), tables: &EXASOL_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3685), tables: &EXASOL_TABLES }),
        "StepKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3686), tables: &EXASOL_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3687), tables: &EXASOL_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3689), tables: &EXASOL_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3694), tables: &EXASOL_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3695), tables: &EXASOL_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3696), tables: &EXASOL_TABLES }),
        "SubjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3697), tables: &EXASOL_TABLES }),
        "SubstrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3698), tables: &EXASOL_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3699), tables: &EXASOL_TABLES }),
        "SubtypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3700), tables: &EXASOL_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3701), tables: &EXASOL_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(3702), tables: &EXASOL_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3703), tables: &EXASOL_TABLES }),
        "Sys_connect_by_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3704), tables: &EXASOL_TABLES }),
        "Sys_guidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3705), tables: &EXASOL_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3706), tables: &EXASOL_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3707), tables: &EXASOL_TABLES }),
        "SystemParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3708), tables: &EXASOL_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3709), tables: &EXASOL_TABLES }),
        "SystimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3710), tables: &EXASOL_TABLES }),
        "TableConstraintEnableDisableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3711), tables: &EXASOL_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3714), tables: &EXASOL_TABLES }),
        "TableContentDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(3729), tables: &EXASOL_TABLES }),
        "TableDistributeByGrammar" => Some(RootGrammar { grammar_id: GrammarId(3733), tables: &EXASOL_TABLES }),
        "TableDistributionPartitionClause" => Some(RootGrammar { grammar_id: GrammarId(3740), tables: &EXASOL_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3749), tables: &EXASOL_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3750), tables: &EXASOL_TABLES }),
        "TableInlineConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3760), tables: &EXASOL_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3776), tables: &EXASOL_TABLES }),
        "TableOutOfLineConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(3777), tables: &EXASOL_TABLES }),
        "TablePartitionByGrammar" => Some(RootGrammar { grammar_id: GrammarId(3795), tables: &EXASOL_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3802), tables: &EXASOL_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3803), tables: &EXASOL_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3804), tables: &EXASOL_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(3809), tables: &EXASOL_TABLES }),
        "TanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3813), tables: &EXASOL_TABLES }),
        "TanhKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3814), tables: &EXASOL_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3815), tables: &EXASOL_TABLES }),
        "Temp_db_ram_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3816), tables: &EXASOL_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(3817), tables: &EXASOL_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(3818), tables: &EXASOL_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3821), tables: &EXASOL_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(3822), tables: &EXASOL_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3825), tables: &EXASOL_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3826), tables: &EXASOL_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3827), tables: &EXASOL_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(3828), tables: &EXASOL_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3829), tables: &EXASOL_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(3830), tables: &EXASOL_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(3841), tables: &EXASOL_TABLES }),
        "Time_zoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3847), tables: &EXASOL_TABLES }),
        "Time_zone_behaviorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3848), tables: &EXASOL_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3849), tables: &EXASOL_TABLES }),
        "Timestamp_arithmetic_behaviorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3850), tables: &EXASOL_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3851), tables: &EXASOL_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3852), tables: &EXASOL_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3853), tables: &EXASOL_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3854), tables: &EXASOL_TABLES }),
        "To_charKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3855), tables: &EXASOL_TABLES }),
        "To_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3856), tables: &EXASOL_TABLES }),
        "To_dsintervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3857), tables: &EXASOL_TABLES }),
        "To_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3858), tables: &EXASOL_TABLES }),
        "To_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3859), tables: &EXASOL_TABLES }),
        "To_ymintervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3860), tables: &EXASOL_TABLES }),
        "TraceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3861), tables: &EXASOL_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3862), tables: &EXASOL_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3863), tables: &EXASOL_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3864), tables: &EXASOL_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3869), tables: &EXASOL_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3870), tables: &EXASOL_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3871), tables: &EXASOL_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3872), tables: &EXASOL_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3873), tables: &EXASOL_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3874), tables: &EXASOL_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3875), tables: &EXASOL_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(3876), tables: &EXASOL_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3880), tables: &EXASOL_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(3881), tables: &EXASOL_TABLES }),
        "TruncKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3882), tables: &EXASOL_TABLES }),
        "TruncateAuditLogsSegment" => Some(RootGrammar { grammar_id: GrammarId(3883), tables: &EXASOL_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3899), tables: &EXASOL_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3900), tables: &EXASOL_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(3904), tables: &EXASOL_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3907), tables: &EXASOL_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3908), tables: &EXASOL_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3911), tables: &EXASOL_TABLES }),
        "TypeofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3914), tables: &EXASOL_TABLES }),
        "UDFParameterDotSyntaxSegment" => Some(RootGrammar { grammar_id: GrammarId(3915), tables: &EXASOL_TABLES }),
        "UDFParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3916), tables: &EXASOL_TABLES }),
        "UcaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3920), tables: &EXASOL_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3921), tables: &EXASOL_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3922), tables: &EXASOL_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3923), tables: &EXASOL_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3924), tables: &EXASOL_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3928), tables: &EXASOL_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3929), tables: &EXASOL_TABLES }),
        "UnicodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3930), tables: &EXASOL_TABLES }),
        "UnicodechrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3931), tables: &EXASOL_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3932), tables: &EXASOL_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3937), tables: &EXASOL_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3938), tables: &EXASOL_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3940), tables: &EXASOL_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3941), tables: &EXASOL_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3942), tables: &EXASOL_TABLES }),
        "UnknownSegment" => Some(RootGrammar { grammar_id: GrammarId(3943), tables: &EXASOL_TABLES }),
        "UnlimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3944), tables: &EXASOL_TABLES }),
        "UnlinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3945), tables: &EXASOL_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3946), tables: &EXASOL_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3947), tables: &EXASOL_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3962), tables: &EXASOL_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3968), tables: &EXASOL_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3969), tables: &EXASOL_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3970), tables: &EXASOL_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3979), tables: &EXASOL_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3980), tables: &EXASOL_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3981), tables: &EXASOL_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3982), tables: &EXASOL_TABLES }),
        "UserKerberosAuthSegment" => Some(RootGrammar { grammar_id: GrammarId(3985), tables: &EXASOL_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3990), tables: &EXASOL_TABLES }),
        "UserLDAPAuthSegment" => Some(RootGrammar { grammar_id: GrammarId(3991), tables: &EXASOL_TABLES }),
        "UserOpenIDAuthSegment" => Some(RootGrammar { grammar_id: GrammarId(3997), tables: &EXASOL_TABLES }),
        "UserPasswordAuthSegment" => Some(RootGrammar { grammar_id: GrammarId(4002), tables: &EXASOL_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4005), tables: &EXASOL_TABLES }),
        "User_temp_db_ram_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4006), tables: &EXASOL_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4007), tables: &EXASOL_TABLES }),
        "Utf8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4008), tables: &EXASOL_TABLES }),
        "Value2procKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4009), tables: &EXASOL_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4010), tables: &EXASOL_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4011), tables: &EXASOL_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4026), tables: &EXASOL_TABLES }),
        "ValuesRangeClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4027), tables: &EXASOL_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4043), tables: &EXASOL_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4044), tables: &EXASOL_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4045), tables: &EXASOL_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4046), tables: &EXASOL_TABLES }),
        "VariableNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4047), tables: &EXASOL_TABLES }),
        "VarianceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4048), tables: &EXASOL_TABLES }),
        "VarrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4049), tables: &EXASOL_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4050), tables: &EXASOL_TABLES }),
        "VerifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4051), tables: &EXASOL_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4052), tables: &EXASOL_TABLES }),
        "ViewReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &EXASOL_TABLES }),
        "VirtualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4053), tables: &EXASOL_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4054), tables: &EXASOL_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4055), tables: &EXASOL_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4056), tables: &EXASOL_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4066), tables: &EXASOL_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4067), tables: &EXASOL_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4068), tables: &EXASOL_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4073), tables: &EXASOL_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &EXASOL_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4090), tables: &EXASOL_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(4091), tables: &EXASOL_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &EXASOL_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4094), tables: &EXASOL_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4104), tables: &EXASOL_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(4105), tables: &EXASOL_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4113), tables: &EXASOL_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4122), tables: &EXASOL_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4131), tables: &EXASOL_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(4136), tables: &EXASOL_TABLES }),
        "WithInvalidForeignKeySegment" => Some(RootGrammar { grammar_id: GrammarId(4137), tables: &EXASOL_TABLES }),
        "WithInvalidUniquePKSegment" => Some(RootGrammar { grammar_id: GrammarId(4142), tables: &EXASOL_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4149), tables: &EXASOL_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4150), tables: &EXASOL_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4155), tables: &EXASOL_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4156), tables: &EXASOL_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(4157), tables: &EXASOL_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4158), tables: &EXASOL_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4159), tables: &EXASOL_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4160), tables: &EXASOL_TABLES }),
        "Years_betweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4161), tables: &EXASOL_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4162), tables: &EXASOL_TABLES }),
        "ZeroifnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4163), tables: &EXASOL_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4164), tables: &EXASOL_TABLES }),
        _ => None,
    }
}

pub fn get_exasol_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterConnectionSegment" => Some("alter_connection"),
        "AlterConsumerGroupSegment" => Some("alter_consumer_group_statement"),
        "AlterRoleStatementSegment" => Some("alter_role_statement"),
        "AlterSchemaStatementSegment" => Some("alter_schema_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterSessionSegment" => Some("alter_session_statement"),
        "AlterSystemSegment" => Some("alter_system_statement"),
        "AlterTableAddColumnSegment" => Some("alter_table_add_column"),
        "AlterTableAlterColumnSegment" => Some("alter_table_alter_column"),
        "AlterTableColumnSegment" => Some("alter_table_column_statement"),
        "AlterTableConstraintSegment" => Some("alter_table_constraint_statement"),
        "AlterTableDistributePartitionSegment" => Some("alter_table_distribute_partition_statement"),
        "AlterTableDropColumnSegment" => Some("alter_table_drop_column"),
        "AlterTableModifyColumnSegment" => Some("alter_table_modify_column"),
        "AlterTableRenameColumnSegment" => Some("alter_table_rename_column"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterUserStatementSegment" => Some("alter_user_statement"),
        "AlterVirtualSchemaStatementSegment" => Some("alter_virtual_schema_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CSVColumnDefinitionSegment" => Some("csv_cols"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CloseSchemaSegment" => Some("close_schema_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDatatypeSegment" => Some("column_datatype_definition"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "CommentStatementSegment" => Some("comment_statement"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "ConnectByClauseSegment" => Some("connect_by_clause"),
        "ConnectionDefinition" => Some("connection_definition"),
        "ConsumerGroupParameterSegment" => Some("consumer_group_parameter"),
        "CreateAdapterScriptStatementSegment" => Some("create_adapter_script"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateConnectionSegment" => Some("create_connection"),
        "CreateConsumerGroupSegment" => Some("create_consumer_group_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateScriptingLuaScriptStatementSegment" => Some("create_scripting_lua_script"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableLikeClauseSegment" => Some("table_like_clause"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUDFScriptStatementSegment" => Some("create_udf_script"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CreateVirtualSchemaStatementSegment" => Some("create_virtual_schema_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropConnectionStatementSegment" => Some("drop_connection_statement"),
        "DropConsumerGroupSegment" => Some("drop_consumer_group_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropScriptStatementSegment" => Some("drop_script_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmitsSegment" => Some("emits_segment"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExecuteScriptSegment" => Some("execute_script_statement"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExplainVirtualSegment" => Some("explain_virtual_statement"),
        "ExportIntoClauseSegment" => Some("export_into_clause"),
        "ExportStatementSegment" => Some("export_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FBVColumnDefinitionSegment" => Some("fbv_cols"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileOptionSegment" => Some("file_opts"),
        "FileSegment" => Some("file"),
        "FlushStatisticsSegment" => Some("flush_statistics_statement"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionAssignmentSegment" => Some("function_assignment"),
        "FunctionBodySegment" => Some("function_body"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionForLoopSegment" => Some("function_for_loop"),
        "FunctionIfBranchSegment" => Some("function_if_branch"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionReferenceSegment" => Some("function_reference"),
        "FunctionScriptStatementSegment" => Some("statement"),
        "FunctionSegment" => Some("function"),
        "FunctionWhileLoopSegment" => Some("function_while_loop"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImpersonateSegment" => Some("impersonate_statement"),
        "ImplicitIndent" => Some("indent"),
        "ImportColumnsSegment" => Some("import_columns"),
        "ImportErrorDestinationSegment" => Some("import_error_destination"),
        "ImportErrorsClauseSegment" => Some("import_errors_clause"),
        "ImportFromClauseSegment" => Some("import_from_clause"),
        "ImportFromExportIntoDbSrcSegment" => Some("import_export_dbsrc"),
        "ImportFromExportIntoFileSegment" => Some("import_file"),
        "ImportFromExportIntoScriptSegment" => Some("import_script"),
        "ImportStatementSegment" => Some("import_statement"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoTableSegment" => Some("into_table_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "KillSegment" => Some("kill_statement"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OpenSchemaSegment" => Some("open_schema_statement"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PreferringClauseSegment" => Some("preferring_clause"),
        "PreferringPlusPriorTermSegment" => Some("plus_prior_inverse"),
        "PreferringPreferenceTermSegment" => Some("preference_term"),
        "PreloadSegment" => Some("preload_statement"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifyClauseSegment" => Some("qualify_clause"),
        "RecompressReorganizeSegment" => Some("recompress_reorganize_statement"),
        "ReferencingClauseSegment" => Some("referencing_clause"),
        "RejectClauseSegment" => Some("reject_clause"),
        "RenameStatementSegment" => Some("rename_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "ScriptContentSegment" => Some("script_content"),
        "ScriptReferenceSegment" => Some("script_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableContentDefinitionSegment" => Some("table_content_definition"),
        "TableDistributionPartitionClause" => Some("table_distribution_partition_clause"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableInlineConstraintSegment" => Some("table_constraint_definition"),
        "TableOutOfLineConstraintSegment" => Some("table_constraint_definition"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateAuditLogsSegment" => Some("truncate_audit_logs_statement"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserKerberosAuthSegment" => Some("kerberos_auth"),
        "UserLDAPAuthSegment" => Some("ldap_auth"),
        "UserOpenIDAuthSegment" => Some("openid_auth"),
        "UserPasswordAuthSegment" => Some("password_auth"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "ValuesRangeClauseSegment" => Some("values_range_clause"),
        "ViewReferenceSegment" => Some("view_reference"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithInvalidForeignKeySegment" => Some("with_invalid_foreign_key_clause"),
        "WithInvalidUniquePKSegment" => Some("with_invalid_unique_pk_clause"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static EXASOL_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_exasol_root_grammar_id() -> GrammarId {
    GrammarId(1885)
}
pub fn get_exasol_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_exasol_root_grammar_id(),
        tables: &EXASOL_TABLES,
    }
}
pub fn get_exasol_root_grammar() -> RootGrammar {
    get_exasol_root_grammar_table()
}
