/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      5646 × 20 bytes =   112920 bytes
//   Child IDs:         4234 ×  4 bytes =    16936 bytes
//   Terminators:         58 ×  4 bytes =      232 bytes
//   Strings:           2519 strings      =    36030 bytes
//   Aux Data:         14760 ×  4 bytes =    59040 bytes
//   Aux Offsets:       5646 ×  4 bytes =    22584 bytes
//   Regex Patterns:       8 patterns     =     2304 bytes
//   Simple Hints:      1539 hints        =    24624 bytes
//   Hint Indices:      2357 ×  4 bytes =     9428 bytes
//   TOTAL:                                    ≈   284098 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] StringParser("ACCESSIBLE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 202, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 202, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 205, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Ref(VersioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] StringParser("AGAINST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 207, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser("ALGORITHM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 207, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(InplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] Ref(NocopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(InstantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 215, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 219, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 224, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(DoubleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 227, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 227, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 229, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 229, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 235, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 235, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Ref(AlterOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 246, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 246, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(ScheduleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(EveryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 254, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 254, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 255, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 256, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 258, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(StartsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(EndsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 260, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 260, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(CompletionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 264, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 264, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 267, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 270, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 270, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 270, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Ref(SlaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 275, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 278, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 282, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 286, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 286, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(DoubleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 294, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Ref(EncryptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 302, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 302, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 304, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 310, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 313, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 315, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 317, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 317, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 317, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 317, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 319, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 323, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 324, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 327, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 329, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 331, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 331, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 342, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [369] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [370] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [371] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [372] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [373] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 344, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [374] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [375] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [376] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [377] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [378] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [379] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 349, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [380] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 349, child_count: 8, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [381] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [382] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [383] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [384] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [385] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [386] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [387] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [388] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 357, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [389] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 358, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [390] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 361, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [391] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 361, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [392] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [393] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [394] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 363, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [395] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [396] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [397] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 368, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [399] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [400] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [402] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [403] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [404] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [405] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [406] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [407] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [408] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 377, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [409] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [410] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 380, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [411] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [412] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [413] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [414] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [415] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [416] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 390, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [417] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [418] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [419] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [420] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [421] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [422] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [423] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [424] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [425] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [426] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 400, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [427] Ref(FulltextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 400, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [428] Ref(SpatialKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 400, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [429] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [430] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [431] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [432] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [433] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [434] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 402, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [435] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 402, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [436] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [437] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [438] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [439] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [440] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [441] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 408, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [442] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 411, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [443] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 411, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [444] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 411, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [445] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [446] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [447] Ref(InplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [448] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [449] Ref(InstantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 415, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [450] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [451] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [452] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [453] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [454] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [455] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [456] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [457] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 422, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [458] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [459] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [460] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 429, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [461] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 429, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [462] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [463] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [464] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 431, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [465] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [466] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [467] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [468] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [469] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 437, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [470] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 437, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [471] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 439, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [472] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 441, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [473] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [474] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [475] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [476] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [477] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [478] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [479] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [480] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 446, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [481] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [482] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [483] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [484] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [485] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [486] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [487] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [488] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 453, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [489] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [490] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [491] Ref(TableOptionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [492] Ref(AlterTableOnlineDDLOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [493] Ref(AddDropSystemVersioningGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 469, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [494] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [495] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [496] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [497] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 474, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [498] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 474, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [499] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [500] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [502] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [503] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [504] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [505] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [506] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [507] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [508] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 483, child_count: 2, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [509] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [510] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [511] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [512] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [513] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [514] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [515] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [516] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [518] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [519] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [521] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [522] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [523] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [524] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 498, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [525] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [526] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [527] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [528] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [529] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 503, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [530] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [531] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 506, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [532] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [533] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [534] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [535] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [537] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [538] Ref(AlterTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [539] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [540] Ref(ChangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [541] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [542] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [543] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 516, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [544] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 516, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [545] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 517, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [546] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [547] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [548] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [549] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [550] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [551] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [553] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [554] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 529, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [555] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [556] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [557] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [558] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [559] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 535, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [560] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [561] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [562] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [563] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [564] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [565] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [566] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [567] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [568] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [569] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 540, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [570] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [571] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [572] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [573] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [574] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [575] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [576] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [577] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 549, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [578] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [579] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [580] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [581] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 551, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [582] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [583] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [584] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [585] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [586] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [587] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [588] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 560, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [589] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [590] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [591] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [592] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [593] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [594] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [595] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [596] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [597] Ref(IgnoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 566, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [598] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [599] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [600] Ref(IgnoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [601] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [602] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 570, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [603] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 573, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 575, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [606] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [607] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [608] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 577, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [609] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 581, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [610] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [611] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [612] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [613] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [614] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [615] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [616] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [617] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [618] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [619] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [620] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [621] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [622] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [623] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [624] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [625] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [626] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [627] Ref(ConvertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [628] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 594, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [629] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 595, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [630] Ref(AlterOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 595, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [631] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [632] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [633] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [634] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [635] Ref(DiscardKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [636] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [637] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [638] Ref(CoalesceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [639] Ref(ReorganizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [640] Ref(ExchangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [641] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [642] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [643] Ref(OptimizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [644] Ref(RebuildKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [645] Ref(RepairKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [646] Ref(RemoveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [647] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [648] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [649] Ref(PartitioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [650] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [651] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [652] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [653] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 621, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [654] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [655] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [656] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [657] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 626, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [658] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [659] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [660] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [661] Ref(TableReference)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [663] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [664] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [665] Ref(ValidationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 633, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [667] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [668] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 638, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [669] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [670] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 640, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [671] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 650, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [672] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 650, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [673] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [674] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [675] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 653, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [676] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [677] Ref(UndefinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [678] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [679] Ref(TemptableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 656, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [680] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 656, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [681] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [682] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [683] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [684] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [685] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [686] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [687] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [688] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [689] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [690] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [691] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [692] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 666, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [693] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 666, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [694] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 666, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [695] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 666, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [696] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 666, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [697] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 666, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [698] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 666, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [699] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [700] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 670, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [701] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [702] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [703] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [704] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [705] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 675, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [706] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [707] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [708] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [709] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [710] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [711] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [712] Ref(HistogramKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [713] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 684, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [714] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [715] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 686, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [716] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [717] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [718] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [719] Ref(BucketsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 689, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [720] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [721] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [722] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [723] Ref(HistogramKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [724] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [725] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [726] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 696, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [727] StringParser("AND")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 696, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [728] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [729] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [730] StringParser("&&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [731] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [732] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 698, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [733] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 698, child_count: 12, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [734] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [735] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [736] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [737] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [738] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [739] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [740] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [741] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [742] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [743] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [744] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [745] Ref(DivOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [746] Ref(ModOperatorSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 710, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [747] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [748] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [749] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [750] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [751] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [752] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [753] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [754] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [755] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [756] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [757] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 720, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [758] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [759] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [760] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [761] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [762] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [763] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [764] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [765] StringParser("ASCII")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [766] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [767] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [768] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [769] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [770] StringParser("AT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [771] TypedParser("at_sign_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [772] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [773] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [774] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [775] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [776] StringParser("AUTHENTICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [777] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 723, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [778] StringParser("AUTHORS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 723, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [779] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [780] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [781] StringParser("AUTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [782] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [783] StringParser("AUTOEXTEND_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [784] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [785] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [786] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [787] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 724, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [788] MultiStringParser(6 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 724, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [789] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [790] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [791] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [792] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [793] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [794] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [795] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [796] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [797] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [798] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [799] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [800] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [801] Ref(VariableAssignmentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [802] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [803] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [804] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [805] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [806] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [807] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [808] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [809] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [810] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [811] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [812] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [813] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [814] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [815] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [816] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [817] Ref(ColumnPathOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [818] Ref(InlinePathOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [819] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [820] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [821] StringParser("BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [822] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [823] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [824] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 742, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [825] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 742, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [826] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [827] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [828] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [829] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 744, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [830] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [831] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [832] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [833] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [834] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [835] StringParser("BLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [836] StringParser("BODY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 746, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [837] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 746, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [838] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [839] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [840] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [841] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [842] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [843] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [844] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [845] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [846] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [847] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [848] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [849] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [850] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [851] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [852] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [853] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 759, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [854] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 761, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [855] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 761, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [856] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 764, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [857] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [858] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 766, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [859] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 769, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [860] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 771, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [861] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [862] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [863] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [864] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [865] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [866] Ref(IndexColumnPrefixLengthSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 778, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [867] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [868] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 781, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [869] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [870] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [871] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 783, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [872] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 783, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [873] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [874] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [875] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [876] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [877] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [878] StringParser("BTREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [879] StringParser("BUCKETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [880] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [881] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [882] StringParser("BYTE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [883] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [884] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 789, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [885] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [886] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [887] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 793, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [888] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 793, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [889] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 796, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [890] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 796, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [891] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 796, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [892] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 796, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [894] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [895] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [896] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [897] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [898] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [899] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [900] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 800, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [901] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [902] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 806, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [903] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 806, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [904] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [905] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [906] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [907] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [908] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [909] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [910] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 807, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [911] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [912] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [913] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 814, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [914] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [915] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [916] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [917] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [918] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [919] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [920] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [921] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [922] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [923] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [924] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [925] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [926] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [927] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [928] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [929] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [930] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [931] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [932] StringParser("CHALLENGE_RESPONSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [933] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [934] StringParser("CHANGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [935] StringParser("CHANGED_PAGE_BITMAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [936] StringParser("CHANNEL")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [937] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [938] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [939] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [940] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [941] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [942] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [943] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 815, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [944] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 815, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [945] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [946] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [947] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [948] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [949] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 823, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [950] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 823, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [951] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [952] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 825, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [953] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 825, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [954] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 829, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [955] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [956] Ref(ContainsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [957] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 831, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [958] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [959] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [960] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 833, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [961] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [962] Ref(ReadsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [963] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [964] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 836, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [965] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [966] Ref(ModifiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [967] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 839, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [968] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 839, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [969] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [970] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [971] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [972] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [973] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [974] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [975] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [976] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [977] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [978] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [979] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [980] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [981] Ref(CharsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [982] StringParser("CHARSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [983] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [984] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [985] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [986] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 852, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [987] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [988] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 854, child_count: 6, min_times: 1, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [989] AnyNumberOf(6 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 860, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [991] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [992] Ref(UpgradeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [993] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [994] Ref(FastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [995] Ref(MediumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [996] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [997] Ref(ChangedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [998] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [999] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 862, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1000] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 862, child_count: 4, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1001] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1002] Ref(ChecksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1003] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1004] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1005] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1006] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1007] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1008] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1009] StringParser("CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1010] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1011] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1012] StringParser("CLIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1013] StringParser("CLIENT_STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1014] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1015] StringParser("CLONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1016] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1017] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1018] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1019] StringParser("CLUSTERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1020] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1021] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1022] StringParser("CODE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 870, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1023] Token("raw")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 870, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1024] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1025] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1026] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1027] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 872, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1028] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 872, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1029] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 874, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1030] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 874, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 1, _padding: 0 }, // [1031] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1032] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1033] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [1034] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1035] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1036] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1037] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1038] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1039] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1040] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1041] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1042] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1043] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1044] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1045] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1046] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1047] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1048] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1049] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 6, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1050] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 893, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1051] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 895, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1052] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1053] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1054] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 897, child_count: 10, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1055] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1056] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1057] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1058] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 909, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1060] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1061] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 914, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1062] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 914, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1064] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1065] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1067] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1068] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1069] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1070] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 918, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1072] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1073] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1074] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1075] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1076] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1077] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1078] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 922, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1079] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1080] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 924, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1081] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1082] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1083] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1084] Ref(DoubleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1085] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1086] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 928, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1087] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 932, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1088] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 934, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1089] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 934, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1090] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 934, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1091] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 934, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1092] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 937, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1093] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 937, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1094] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1095] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1096] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 939, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1097] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1098] Ref(SridKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1099] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 941, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1100] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1101] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1102] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 943, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1103] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 947, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1105] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1106] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1107] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1108] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 952, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1109] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 952, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1110] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 955, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1111] Ref(PersistentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 955, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1112] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 955, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1113] Ref(VirtualKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 955, child_count: 4, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1114] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1115] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1116] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 961, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1117] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1118] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1119] Ref(DatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1120] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1121] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 967, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1122] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1123] Ref(DatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1124] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1125] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 969, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1126] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 972, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1127] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 974, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1128] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1129] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 975, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1130] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 975, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1131] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1132] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 5, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1133] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1134] Ref(CurrentTimestampLikeFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1135] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1136] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1137] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1138] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 982, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1139] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1140] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1141] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1142] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1143] Ref(CurrentTimestampLikeFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1144] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 987, child_count: 3, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1145] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 990, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1146] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 990, child_count: 1, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1147] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1148] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1149] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1150] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1151] StringParser("->")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 991, child_count: 2, min_times: 0, first_terminator_idx: 15, terminator_count: 1, _padding: 0 }, // [1152] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1153] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1154] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 15, terminator_count: 0, _padding: 0 }, // [1155] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1156] StringParser("COLUMN_ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1157] StringParser("COLUMN_CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1158] StringParser("COLUMN_CREATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1159] StringParser("COLUMN_DELETE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1160] StringParser("COLUMN_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1161] StringParser("COLUMN_GET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1162] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1163] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1164] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1165] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1166] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1167] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1168] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1169] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 993, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1170] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 993, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1172] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1173] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1174] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1175] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1176] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1177] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 995, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1178] StringParser("COMPACT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 995, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1179] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1180] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1181] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1182] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1183] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1184] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1185] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1186] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1187] Ref(IsDistinctFromGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1188] Ref(NullSafeEqualsSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1189] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1190] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1191] StringParser("COMPONENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1192] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1193] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1194] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1195] StringParser("COMPRESSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1196] StringParser("COMPRESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1197] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1004, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1198] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1199] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1200] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1201] StringParser("CONCURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1202] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1203] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1204] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1006, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1205] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1206] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1207] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1208] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1209] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1210] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1211] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1212] StringParser("CONSISTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1213] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1214] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1215] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1216] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1217] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1218] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1219] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1220] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1221] StringParser("CONTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1222] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1223] StringParser("CONTRIBUTORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1224] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1225] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1226] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1013, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1227] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1228] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1229] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1230] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1231] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1232] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1233] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1234] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1235] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1236] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1237] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1238] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1239] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1240] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1241] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1242] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1243] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1244] StringParser("CPU")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1245] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1246] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1247] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1248] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1249] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1250] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1251] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1252] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1036, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1253] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1036, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1254] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1255] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1256] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1257] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1258] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1042, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1259] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1045, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1260] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1045, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1261] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1045, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1262] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1045, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1263] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1045, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1264] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1045, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1265] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1045, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1266] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1267] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1268] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1047, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1269] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1270] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1271] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1049, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1272] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1273] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1054, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1274] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1275] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1276] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1277] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1278] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1056, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1279] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1280] Ref(CreateOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1057, child_count: 16, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1281] Sequence(16 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1282] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1283] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1284] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1285] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1286] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1287] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1288] Ref(ScheduleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1073, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1290] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1291] Ref(EveryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1075, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1292] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1075, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1293] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1294] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1076, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1295] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1296] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1298] Ref(StartsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1299] Ref(EndsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1300] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1081, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1301] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1302] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1303] Ref(CompletionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1304] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1305] Ref(PreserveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1085, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1306] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1307] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1308] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1088, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1309] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1310] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1311] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1091, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1312] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1313] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1314] Ref(SlaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1315] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1316] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1317] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1318] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1319] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1320] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1321] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1322] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1323] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1324] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1325] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1326] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1327] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1328] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1329] Ref(CharacteristicStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1330] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1105, child_count: 13, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1331] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1332] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1118, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1333] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1118, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1334] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1335] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1336] Ref(FulltextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1337] Ref(SpatialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1338] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1339] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1340] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1341] Ref(IndexTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1342] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1343] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1344] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1345] Ref(WaitOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1346] Ref(IndexOptionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1347] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1123, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1348] Ref(AlgorithmOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1123, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1349] Ref(LockOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1123, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1350] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1123, child_count: 8, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1351] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1352] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1353] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1354] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1355] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1356] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1131, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1358] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1359] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1360] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1138, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1361] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1362] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1363] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1364] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1143, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1365] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1143, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1366] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1146, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1367] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1368] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1369] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1370] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1371] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1150, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1372] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1150, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1373] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1374] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1375] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1376] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1377] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1378] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1379] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1380] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1381] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1382] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1383] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1384] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1385] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1386] Ref(EncryptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1387] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1388] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 10, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1389] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1390] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1391] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1392] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1393] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1394] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1395] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1396] Ref(ProcedureParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1397] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1398] Ref(CharacteristicStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1399] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1175, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1400] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1401] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1402] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1403] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1404] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1405] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1406] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1407] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1408] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1409] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1183, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1410] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1411] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1412] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1413] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1414] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1193, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1415] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1416] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1417] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1418] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1419] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1420] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1421] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1198, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1422] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1423] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1424] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1425] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1426] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1427] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1428] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1429] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1202, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1430] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1431] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1432] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1433] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1206, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1434] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1435] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1207, child_count: 9, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1436] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1437] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1438] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1439] Ref(TemporaryTransientGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1440] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1441] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1442] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1216, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1443] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1219, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1444] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1222, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1445] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1225, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1446] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1227, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1447] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1448] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1449] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1450] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1229, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1231, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1452] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1231, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1453] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1454] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1455] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1236, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1456] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1457] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1458] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1240, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1459] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1460] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1243, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1461] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1462] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1463] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1245, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1464] Ref(TableEndClauseSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1245, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1465] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1247, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1466] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1251, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1467] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1468] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1469] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1470] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1471] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1257, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1472] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1473] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1474] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1475] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1259, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1261, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1477] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1261, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1478] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1261, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1479] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1261, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1480] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1481] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1482] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1483] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1484] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1485] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1486] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1268, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1487] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1271, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1488] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1489] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1490] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1491] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1492] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1279, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1493] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1281, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1494] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1495] Ref(LinearKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1496] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1497] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1287, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1498] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1287, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1499] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1287, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1500] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1501] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1290, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1502] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1503] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1504] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1505] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1506] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1507] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1508] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1509] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1510] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1511] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1512] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1513] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1514] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1515] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1306, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1516] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1306, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1518] Ref(PartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1308, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1519] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1308, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1520] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1521] Ref(SubpartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1522] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1523] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1524] Ref(LinearKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1525] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1527] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1528] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1318, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1529] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1321, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1530] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1321, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1531] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1532] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1533] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1534] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1324, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1535] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1327, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1536] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1327, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1537] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1538] Ref(SubpartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1329, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1539] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1329, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1540] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1541] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1542] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1543] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1544] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1338, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1545] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1338, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1546] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1548] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1343, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1549] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1345, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1550] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1551] Ref(LessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1552] Ref(ThanKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1553] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1554] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1555] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1556] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1557] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1558] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1559] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1560] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1357, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1561] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1562] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1563] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1564] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1364, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1565] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1566] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1567] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1366, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1568] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1369, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1569] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1570] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1571] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1373, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1572] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1573] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1574] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1575] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1576] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1578] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1579] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1580] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1581] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1582] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1583] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1584] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1585] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1586] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1587] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1588] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1389, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1589] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1590] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1591] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1592] Ref(SubpartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1394, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1593] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1394, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1594] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1396, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1595] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1596] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1597] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1400, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1598] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1599] Ref(LessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1600] Ref(ThanKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1403, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1601] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1602] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1603] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1604] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1606] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1414, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1607] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1608] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1609] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1610] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1611] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1612] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1613] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1614] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1615] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1616] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1617] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1619] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1620] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1430, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1621] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 7, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1622] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1623] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1624] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1625] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1626] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1627] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1628] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1629] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1440, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1630] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1631] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 13, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1632] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1633] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1634] Ref(TriggerOrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1635] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1636] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1637] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1638] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1455, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1639] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1640] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1641] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1642] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1643] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1644] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1645] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1646] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1647] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1460, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1648] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1649] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1650] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1651] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1463, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1652] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1653] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1654] Ref(FollowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1655] Ref(PrecedesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1656] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1467, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1657] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1658] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1659] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1660] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1661] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1662] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 11, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1663] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1664] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1665] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1666] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1483, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1667] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1668] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1670] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1487, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1671] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1488, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1672] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1673] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1674] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1492, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1675] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1677] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1678] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1679] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1680] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1681] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1682] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1683] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1684] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1503, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1685] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1503, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1686] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1687] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1507, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1688] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1689] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1509, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1690] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1511, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1692] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1693] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1694] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1513, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1695] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1696] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1697] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1698] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1699] Ref(InitialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1700] Ref(AuthenticationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1701] Ref(IdentifiedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1703] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1704] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1705] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1525, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1707] Ref(RandomKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1708] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1709] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1527, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1710] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1711] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1712] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1713] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1714] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1531, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1715] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1531, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1716] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1717] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1718] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1719] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1720] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1536, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1721] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1722] Ref(RequireKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1723] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1724] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1725] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1542, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1726] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1727] Ref(SslKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1728] Ref(X509KeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1547, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1729] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1730] Ref(CipherKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1731] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1549, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1732] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1733] Ref(IssuerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1734] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1551, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1735] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1736] Ref(SubjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1737] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1553, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1738] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1553, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1555, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1740] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1555, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1741] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1556, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1742] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1558, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1743] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1744] Ref(Max_queries_per_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1745] Ref(Max_updates_per_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1746] Ref(Max_connections_per_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1747] Ref(Max_user_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1748] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1562, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1749] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1563, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1750] AnyNumberOf(6 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1569, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1751] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1752] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1753] Ref(ExpireKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1572, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1754] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1573, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1755] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1756] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1757] Ref(NeverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1758] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1759] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1760] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1761] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1762] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1763] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1764] Ref(HistoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1765] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1766] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1767] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1584, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1768] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1769] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1770] Ref(ReuseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1771] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1588, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1773] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1590, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1774] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1775] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1776] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1777] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1596, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1778] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1596, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1779] Ref(RequireKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1596, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1780] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1596, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1781] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1597, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1782] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1783] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1784] Ref(OptionalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1786] Ref(Failed_login_attemptsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1787] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1788] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1789] Ref(Password_lock_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1790] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1791] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1792] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1793] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1794] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1795] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1796] Ref(UnlockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1797] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1609, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1798] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1799] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1800] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1801] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1802] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 11, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1803] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1804] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1624, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1805] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1624, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1806] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1807] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1808] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1627, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1809] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1810] Ref(UndefinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1811] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1812] Ref(TemptableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1813] Ref(DefinerSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1630, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1814] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1815] Ref(SqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1816] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1817] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1818] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1819] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1820] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1821] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1822] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1823] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1824] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1825] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1826] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1827] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1828] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1829] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1830] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1831] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1832] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1833] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1834] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1835] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1642, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1836] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1837] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1838] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1839] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1840] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1841] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1647, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1842] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1647, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1843] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1844] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1845] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1846] Ref(Current_timestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1847] Ref(NowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1848] Ref(LocaltimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1849] Ref(LocaltimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1654, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1850] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1851] Ref(CurrentTimestampLikeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1852] Ref(CurrentTimestampLikeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1853] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1854] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1855] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1856] StringParser("CURRENT_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1857] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1858] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1859] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1860] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1861] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1862] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1661, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1863] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1661, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1864] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1865] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1866] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1867] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1868] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1869] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1665, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1870] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1871] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1872] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1667, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1873] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1667, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1875] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1876] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1877] Ref(OpenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1878] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1879] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1880] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1881] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1882] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1883] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1884] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1885] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1886] StringParser("DATAFILE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1887] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1675, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1888] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1675, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1889] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1890] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1676, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1891] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1892] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1894] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1895] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1683, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1897] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1898] Ref(SignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1899] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1687, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1900] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1687, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1901] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1690, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1902] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1692, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1903] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1694, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1904] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1905] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1906] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1907] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1908] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1909] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1910] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1700, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1911] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1700, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1702, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1913] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1914] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1915] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1916] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1917] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1704, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1918] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1919] Ref(CharCharacterSetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1920] Ref(SignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1921] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1922] Ref(ZerofillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1710, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1924] Ref(ZerofillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1925] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1712, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1926] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1927] Ref(UnsignedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1928] Ref(ZerofillKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1929] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1930] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1931] MultiStringParser(3 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1932] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1933] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1934] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1935] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1720, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1936] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1937] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1722, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1938] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1722, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1939] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1724, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1940] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1941] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1942] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1943] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1944] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1945] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1946] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1947] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1948] MultiStringParser(20 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1949] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1950] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1951] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1952] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1953] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1954] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1955] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1956] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1957] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1958] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1959] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1960] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1729, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1961] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1729, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1962] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1731, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1963] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1964] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1965] Ref(DeallocateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1966] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1967] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1968] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1969] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1970] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1971] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1972] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1739, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1973] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1974] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1975] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1976] Ref(CursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1977] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1978] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 6, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1979] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1980] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1750, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1981] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1982] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1983] Ref(ExitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1984] Ref(UndoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1985] Ref(HandlerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1986] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1753, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1987] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1988] Ref(SqlexceptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1989] Ref(SqlwarningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1758, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1991] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1992] Ref(FoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1760, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1993] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1994] Ref(SqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1995] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1996] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1997] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1998] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [1999] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2000] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1766, child_count: 1, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2001] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2002] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1767, child_count: 5, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2003] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2004] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2005] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2006] Ref(ConditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2007] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1772, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2008] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2009] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2010] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1774, child_count: 4, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2011] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2012] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2013] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1778, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2014] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1778, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2015] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2016] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2017] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2018] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2019] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2020] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1783, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2021] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2023] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2024] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2025] StringParser("DEFAULT_AUTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2026] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2027] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2028] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2029] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1785, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2030] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1785, child_count: 3, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2031] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2032] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2033] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2034] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2035] StringParser("DEFINITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2036] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2037] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2038] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1788, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2039] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1788, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2040] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2041] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2042] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2043] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1793, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2044] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2045] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1796, child_count: 4, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2046] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2047] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1800, child_count: 2, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [2048] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2049] Ref(DeleteTargetTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2050] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2051] Ref(DeleteUsingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1802, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2052] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1802, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2053] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 2, min_times: 0, first_terminator_idx: 17, terminator_count: 1, _padding: 0 }, // [2054] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2055] Ref(DeleteTargetTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2056] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2057] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1807, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2058] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1807, child_count: 6, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2059] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1813, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2060] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1813, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2061] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1813, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2062] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1813, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2063] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1813, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2064] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1813, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2065] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1813, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1815, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2067] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1815, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2068] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2069] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2070] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1817, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2072] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2073] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2074] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1821, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2075] StringParser("DELETE_DOMAIN_ID")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2076] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2077] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2078] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2079] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2080] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2081] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2082] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2083] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2084] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2085] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2086] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2087] StringParser("DES_KEY_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2088] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1823, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2089] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1823, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2090] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2091] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2092] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2093] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2094] StringParser("DESCRIPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2095] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2096] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2097] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2098] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2099] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2100] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2101] StringParser("DIRECTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2102] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2103] StringParser("DISCARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2104] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2105] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2106] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2107] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2108] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2109] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2110] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2111] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2112] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2113] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2114] StringParser("DO_DOMAIN_IDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2115] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2116] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2117] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2118] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2119] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2120] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1826, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2122] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2123] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1828, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2124] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2125] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2126] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1832, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2127] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2128] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2129] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2130] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1837, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2131] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1837, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2132] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2133] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2134] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2135] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2136] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1842, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2137] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1842, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2138] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2139] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2140] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2141] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2142] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1846, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2143] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2144] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2145] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2146] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2147] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1850, child_count: 6, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2148] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2149] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2150] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2151] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2152] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1856, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2153] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1856, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2154] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1858, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2155] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1861, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2156] Ref(AlgorithmKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1861, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2157] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1861, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2158] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2159] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2160] Ref(InplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2161] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1864, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2162] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2163] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1867, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2164] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1867, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2165] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2166] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2167] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2168] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2169] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1871, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2170] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2171] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2172] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2173] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2174] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2175] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2176] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2177] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2178] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2179] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2180] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2181] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2182] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1881, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2183] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2184] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2185] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2186] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2187] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2188] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2189] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2190] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2191] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2192] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1890, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2193] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1890, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2194] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2195] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2196] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2197] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1893, child_count: 6, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2198] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2199] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2200] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2201] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1899, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2202] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2204] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1901, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2205] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1901, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2206] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2207] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2208] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2209] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2210] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2211] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2212] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2213] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2214] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2215] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1910, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2216] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1910, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2217] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2218] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2219] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2220] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2221] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2222] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2223] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2224] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2225] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2226] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2227] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2228] StringParser("DUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2229] StringParser("DUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2230] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2231] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2232] StringParser("DUMPFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2233] StringParser("DUPLICATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2234] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2235] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2236] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2237] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1919, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2238] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1919, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2239] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2240] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2241] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2242] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2243] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2244] StringParser("ELSIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2245] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2246] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2247] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2248] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2249] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2250] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2251] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2252] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2253] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2254] StringParser("ENCRYPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2255] StringParser("ENCRYPTION_KEY_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2256] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2257] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2258] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2259] StringParser("ENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2260] StringParser("ENFORCED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2261] StringParser("ENGINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2262] StringParser("ENGINE_ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2263] StringParser("ENGINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2264] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2265] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2266] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2267] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2268] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2269] StringParser("ERRORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2270] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2271] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2272] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2273] StringParser("EVENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2274] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2275] StringParser("EXAMINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2276] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2277] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2278] StringParser("EXCHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2279] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2280] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2281] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2282] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1927, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2283] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1927, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2285] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2286] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1930, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2287] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2288] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1932, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2289] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2290] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2291] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2292] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2293] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2294] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2295] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2296] StringParser("EXPANSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2297] StringParser("EXPIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2298] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2300] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2301] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2302] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2303] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2304] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2305] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2306] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2307] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1940, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2308] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2309] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1942, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2310] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1943, child_count: 10, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2311] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2312] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2313] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2314] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2315] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2316] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2317] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2318] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2319] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2320] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2321] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2322] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2323] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2324] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 5, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2325] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2326] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2327] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2328] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2329] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2330] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2332] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2333] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1965, child_count: 4, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2334] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1969, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2335] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1970, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2336] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2337] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2338] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2339] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2340] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1971, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2341] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2342] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1973, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2343] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2344] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2345] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1977, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2346] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2347] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2348] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2349] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2350] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2351] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1982, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2352] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2353] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2354] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2355] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1984, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2356] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1986, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2358] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2359] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 1, _padding: 0 }, // [2360] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1991, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2361] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2362] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1993, child_count: 3, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2363] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2364] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2365] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1998, child_count: 2, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2367] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2000, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2368] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2000, child_count: 1, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2369] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2370] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2371] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 18, terminator_count: 0, _padding: 0 }, // [2372] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2373] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2003, child_count: 9, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [2374] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2375] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2376] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2377] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2015, child_count: 4, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2378] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2379] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2380] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2381] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2019, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2382] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2021, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2383] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2384] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2385] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2386] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2387] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2388] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2389] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2390] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2392] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2393] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2394] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2395] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2396] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2397] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2399] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2400] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2401] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2402] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2403] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2404] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2405] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2406] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2407] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2408] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2042, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2409] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 9, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2410] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2411] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2412] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2413] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2414] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2415] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2416] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2417] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2418] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2419] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2420] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2421] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2422] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2423] StringParser("EXTENT_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2424] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2425] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2426] StringParser("FACTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2427] StringParser("FAILED_LOGIN_ATTEMPTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2428] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2429] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2430] StringParser("FAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2431] StringParser("FAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2432] StringParser("FEDERATED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 5, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2433] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2434] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2435] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2436] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2437] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2058, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2438] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2439] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2060, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2440] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2441] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2442] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2443] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2444] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2063, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2065, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2446] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2065, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2447] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2448] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2449] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2450] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2451] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2452] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2453] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2070, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2454] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2071, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2455] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 2071, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2456] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2457] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 1, min_times: 1, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2458] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2074, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2459] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2074, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2460] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2461] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2462] StringParser("FILE_BLOCK_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2075, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2463] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2075, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2465] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2466] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2080, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2467] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2468] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2469] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2470] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2471] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2472] StringParser("FINISH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2473] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2474] StringParser("FIRST_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2475] StringParser("FIXED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2476] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2477] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2478] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2479] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2082, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2480] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2082, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2481] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2085, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2482] Ref(FlushKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2085, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2483] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2484] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2485] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2486] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2090, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2487] Delimited(23 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2092, child_count: 23, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2488] OneOf(23 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2115, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2490] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2491] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2492] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2493] Ref(EngineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2494] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2119, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2496] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2497] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2121, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2498] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2499] Ref(GeneralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2500] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2123, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2502] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2503] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2125, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2504] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2505] Ref(SlowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2506] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2127, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2507] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2508] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2509] Ref(MasterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2129, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2131, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2511] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2512] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2513] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2514] Ref(StatusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2133, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2515] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2516] Ref(RelayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2517] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2137, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2518] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2519] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2520] Ref(ChannelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2521] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2522] Ref(HostsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2523] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2524] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2525] Ref(Changed_page_bitmapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2526] Ref(Client_statisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2527] Ref(Des_key_fileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2528] Ref(Index_statisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2529] Ref(Query_response_timeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2530] Ref(SlaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2531] Ref(SslKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2532] Ref(Table_statisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2533] Ref(User_statisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2534] Ref(User_variablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2535] Ref(User_resourcesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2139, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2536] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2537] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2142, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2538] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2143, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [2539] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2540] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2145, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2541] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2145, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2542] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2543] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2544] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2545] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2149, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2546] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2547] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2548] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2549] Ref(CheckpointKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2152, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2550] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2551] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2552] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2156, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 1, _padding: 0 }, // [2553] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2158, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2554] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2158, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [2555] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2158, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2557] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2558] Ref(ExportKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2559] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2160, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2560] StringParser("FOLLOWS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2160, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2561] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2562] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2165, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2563] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2564] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2565] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2566] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2567] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2169, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2568] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2569] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2171, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2570] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2571] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2173, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2572] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2573] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2575] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2576] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2577] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2578] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2579] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2580] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2581] Ref(ModeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2582] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2583] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2584] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2585] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2586] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2587] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2588] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2589] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2590] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2183, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2591] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2183, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2592] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2593] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2185, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2594] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2187, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2595] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2596] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2597] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2598] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2191, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2599] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2193, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2600] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2601] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2602] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2603] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2604] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2197, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2606] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2607] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2608] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2609] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2610] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2203, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2611] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2612] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2613] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2614] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2615] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2616] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2205, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2617] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2205, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2619] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2207, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2620] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2621] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 17, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2622] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2623] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2624] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2226, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2626] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2627] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2628] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2629] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2630] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2631] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2632] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2633] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2634] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2635] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2636] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2637] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2638] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2639] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2640] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2641] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2642] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2643] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2644] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 7, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2645] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2646] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2647] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2648] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2649] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2244, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2650] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2244, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2651] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2245, child_count: 4, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2652] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2653] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2654] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2655] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2249, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2656] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2249, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2657] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2658] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2659] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2660] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2661] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2252, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2662] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2252, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2663] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2664] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2257, child_count: 1, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2665] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2666] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2258, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2667] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2260, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2668] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 6, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2669] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 2, _padding: 0 }, // [2670] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2671] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2672] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2272, child_count: 3, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2673] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2674] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2675] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2676] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2677] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 2, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2678] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2679] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 22, terminator_count: 0, _padding: 0 }, // [2680] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2279, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2681] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2279, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 2, _padding: 0 }, // [2682] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2683] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2281, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2684] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2685] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2686] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2283, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2687] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2688] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2689] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2691] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2692] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2693] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2694] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2695] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2696] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2697] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2287, child_count: 11, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2698] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2699] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2298, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2700] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2701] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2702] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2703] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2301, child_count: 4, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2704] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2705] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 2305, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2706] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2707] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2708] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2709] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2710] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2309, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2711] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2712] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2713] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2714] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2715] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2311, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2716] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2313, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2717] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2313, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2718] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2315, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2719] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2315, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2720] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2721] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2722] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2723] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2724] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2725] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2726] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2322, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2727] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2728] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2729] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2730] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2731] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2732] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2733] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2734] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2735] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2736] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2737] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2738] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2739] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2329, child_count: 3, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2740] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2741] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2742] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2743] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2744] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2332, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2745] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2334, child_count: 1, min_times: 0, first_terminator_idx: 26, terminator_count: 1, _padding: 0 }, // [2746] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 2, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2747] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2748] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2749] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 0, min_times: 0, first_terminator_idx: 26, terminator_count: 0, _padding: 0 }, // [2750] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 2, min_times: 0, first_terminator_idx: 27, terminator_count: 1, _padding: 0 }, // [2751] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2752] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2753] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 27, terminator_count: 0, _padding: 0 }, // [2754] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2755] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2756] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2757] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2758] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2345, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2759] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2760] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2761] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2762] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2763] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2349, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2764] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2765] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2766] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2767] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2351, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2768] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2354, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2769] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2770] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 6, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2771] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2773] Ref(JsonTableFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2774] Ref(JsonTableFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2364, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2776] Ref(JsonValueFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2777] Ref(JsonValueFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2366, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2779] Ref(ConvertFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2780] Ref(ConvertFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2781] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2782] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2783] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2784] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2372, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2786] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2374, child_count: 1, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2787] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2788] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2789] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2790] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2791] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2792] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2793] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2794] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2795] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2796] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2797] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2798] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2799] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2800] StringParser("GEOMCOLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2801] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2378, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2802] StringParser("GEOMETRYCOLLECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2378, child_count: 7, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2803] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2804] Ref(GetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2385, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2805] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2806] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2807] Ref(StackedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2387, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2808] Ref(DiagnosticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2387, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2809] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2810] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2811] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2812] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2813] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2814] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2816] Ref(NumberKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2817] Ref(Row_countKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2818] Ref(ConditionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2819] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2820] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2821] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2822] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2399, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2823] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2824] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2825] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2826] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2827] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2828] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 13, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2829] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2830] Ref(Class_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2831] Ref(Subclass_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2832] Ref(Returned_sqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2833] Ref(Message_textKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2834] Ref(Mysql_errnoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2835] Ref(Constraint_catalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2836] Ref(Constraint_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2837] Ref(Constraint_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2838] Ref(Catalog_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2839] Ref(Schema_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2840] Ref(Table_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2841] Ref(Column_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2842] Ref(Cursor_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2843] StringParser("GET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2844] StringParser("GET_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2845] StringParser("GET_MASTER_PUBLIC_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2846] StringParser("GET_SOURCE_PUBLIC_KEY")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2847] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2848] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2849] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2850] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2419, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2851] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 5, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2852] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2853] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2854] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2855] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2856] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2857] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2858] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2859] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2860] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2861] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2862] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2433, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2863] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2433, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2864] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2435, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2865] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2866] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2867] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2868] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2438, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2869] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2870] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2871] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2872] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2873] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2874] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2441, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2875] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2876] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2877] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2878] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2879] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2880] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2881] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2882] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 1, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2883] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2450, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 1, _padding: 0 }, // [2884] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2885] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2454, child_count: 3, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2886] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2887] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2888] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2889] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2457, child_count: 2, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2890] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2891] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2892] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 28, terminator_count: 0, _padding: 0 }, // [2893] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2459, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2894] Ref(WithRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 6, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2895] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2465, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2896] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2897] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2898] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2899] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2900] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2901] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2902] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2903] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2904] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2467, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2905] StringParser("GROUP_REPLICATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2467, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2906] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 1, _padding: 0 }, // [2907] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 4, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2908] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2909] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2910] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2911] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2474, child_count: 2, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2912] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 29, terminator_count: 0, _padding: 0 }, // [2913] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2914] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2915] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2916] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2917] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2479, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2918] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2482, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2919] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2484, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2920] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2921] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2922] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2923] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2924] StringParser("GTID_ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2925] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2926] StringParser("HARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2486, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2927] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2486, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2928] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2929] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2930] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2931] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2932] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2495, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2933] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2500, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2934] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2935] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2936] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2937] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2938] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2939] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2940] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2941] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2942] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2943] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2944] StringParser("HELP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2945] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2946] Ref(HelpKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2947] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2948] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2949] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2950] StringParser("HISTOGRAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2951] StringParser("HISTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2952] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2953] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2954] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2955] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2956] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2957] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2958] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2959] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2960] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2961] StringParser("ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2962] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2963] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2964] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2965] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2966] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2504, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2967] StringParser("IETF_QUOTES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2969] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2506, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2970] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2506, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2971] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2510, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2972] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2973] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2974] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2975] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2976] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2514, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2977] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2978] Ref(ElseifKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2979] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2980] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2518, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2981] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2518, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2982] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2983] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2984] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2985] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2522, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2986] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2522, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2987] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2522, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2988] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2522, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2989] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2525, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2990] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2525, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2991] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2525, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2992] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2525, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2993] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2525, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2994] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2527, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2995] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2996] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2997] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2998] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [2999] StringParser("IGNORE_DOMAIN_IDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3000] StringParser("IGNORE_SERVER_IDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3001] StringParser("IGNORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3002] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3003] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3004] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3005] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3006] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3007] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3008] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2529, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3009] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2529, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3010] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3011] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3012] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3013] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2534, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3014] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2537, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3015] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3016] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3017] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3018] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3019] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3020] StringParser("INACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3021] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3022] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3023] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3024] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2543, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3025] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2543, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3026] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3027] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3028] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3029] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3030] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3031] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3032] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3033] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3034] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3035] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3036] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3037] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3038] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2558, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3039] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2560, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3040] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2560, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3041] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3042] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3043] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2565, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3044] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2565, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3045] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2565, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3046] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3047] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3048] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2567, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3049] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2570, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3050] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2570, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3051] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2570, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3052] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2570, child_count: 6, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3053] AnyNumberOf(6 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2576, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3054] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3055] Ref(Key_block_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3056] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3057] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3058] Ref(IndexTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3059] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3060] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3061] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3062] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3064] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3065] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3066] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2587, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3067] Ref(ClusteringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2587, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3068] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2587, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3069] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3070] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3071] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3072] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3073] Ref(IgnoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3074] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3075] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3076] Ref(IgnoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2593, child_count: 7, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3077] AnySetOf(7 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3078] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3079] Ref(Key_block_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3080] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3081] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3082] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3083] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3084] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3085] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3086] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3087] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3088] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3089] Ref(VisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3090] Ref(InvisibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2608, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3091] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3092] Ref(Engine_attributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3093] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3094] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3095] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3096] Ref(Secondary_engine_attributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3097] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3098] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3099] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3100] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2616, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3101] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3102] Ref(BtreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3103] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2618, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3105] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3106] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3107] Ref(BtreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3108] Ref(HashKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3109] Ref(RtreeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3110] StringParser("INDEX_STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3111] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3112] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3113] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3114] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3115] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3116] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3117] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3118] StringParser("INITIAL_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3119] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3120] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3121] StringParser("INITIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3122] StringParser("->>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3123] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3124] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3125] StringParser("INPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3126] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3127] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3128] StringParser("IN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3129] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3130] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3131] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3132] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2626, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3133] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2626, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3134] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3135] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 11, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3136] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2640, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3137] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2640, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3138] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3139] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3140] Ref(DelayedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3141] Ref(High_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3142] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3143] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2643, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3144] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2643, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3145] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2645, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3146] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2645, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3147] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3148] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3149] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2648, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3150] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3151] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3152] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [3153] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [3154] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3155] Ref(InsertRowAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3156] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3157] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3158] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3159] StringParser("INSERT_METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3160] StringParser("INSTALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3161] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3162] StringParser("INSTANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3163] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3164] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3165] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3166] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3167] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3168] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3169] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3170] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3171] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3172] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3173] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3174] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2651, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3175] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2651, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3176] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3177] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3178] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2655, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3179] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2655, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2657, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3181] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2657, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3182] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2657, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3183] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2657, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 1, _padding: 0 }, // [3184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2659, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3185] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2659, child_count: 1, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3186] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2660, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3187] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2662, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3188] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2665, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3189] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3190] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3191] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2669, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3193] Ref(DumpfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2669, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3194] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2669, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3195] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3196] Ref(OutfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2674, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3197] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2674, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3198] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3199] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3200] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3201] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2677, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3202] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2681, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3203] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2683, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3204] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2683, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3205] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2683, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3206] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3207] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3208] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3209] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2686, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3210] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3211] Ref(OptionallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3212] Ref(EnclosedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3213] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3214] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2690, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3215] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2693, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3216] Ref(EscapedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2693, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3217] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2693, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3218] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2693, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3219] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3220] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2696, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3221] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3222] Ref(StartingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3223] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2699, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3224] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2699, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3225] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3226] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3227] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3228] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [3229] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3230] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3231] StringParser("INVISIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3232] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3233] StringParser("IO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3234] StringParser("IO_THREAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3235] StringParser("IPC")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3236] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3237] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3238] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3239] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3240] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3241] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2707, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3242] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3243] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3244] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3245] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3246] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3247] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3248] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3249] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3250] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3251] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3252] StringParser("ISOPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3253] StringParser("ISSUER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2711, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3254] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2711, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3255] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3256] Ref(IterateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3257] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2713, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3258] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3259] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3260] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3261] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3262] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2723, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3263] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3264] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3265] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2728, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3266] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2728, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3267] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3269] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3270] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3271] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3272] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3273] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3274] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3275] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2736, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3276] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2736, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3277] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3278] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3279] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2740, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3280] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2740, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3281] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3282] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2741, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3283] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2741, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3284] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3285] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3286] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3287] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2747, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3288] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2750, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3289] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2750, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3290] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2750, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3291] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3292] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2752, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3293] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2754, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3294] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3295] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3296] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3297] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2757, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3298] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2757, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3299] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2761, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3300] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2761, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3301] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2764, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3302] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2766, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3303] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2766, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3304] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2766, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3306] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3307] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2771, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3308] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3309] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2776, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3310] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3311] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3312] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3313] Ref(OrdinalityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3314] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3315] Ref(NestedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2783, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3316] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3317] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3318] Ref(JsonTableColumnsClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2783, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3319] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3320] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3321] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3322] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3323] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3324] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3325] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3326] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2791, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3327] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3328] Ref(PathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3329] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2794, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3330] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2795, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3331] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2798, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3332] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2801, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3333] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2801, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3334] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2801, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3336] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3337] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3338] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3339] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3340] Ref(EmptyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3341] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3342] Bracketed(4 elements -> 4 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3343] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3344] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3345] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3346] Ref(JsonTableColumnsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2811, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3347] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2811, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3348] Bracketed(5 elements -> 5 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3349] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3350] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2818, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3351] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2818, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3352] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2821, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3353] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2821, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3354] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2821, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3355] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3356] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3357] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3358] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2824, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3359] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2825, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3360] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3361] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3362] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3363] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2831, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3364] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3365] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3366] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3367] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3368] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3369] Ref(EmptyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3370] Ref(ErrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3371] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3372] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3373] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3374] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3375] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3376] StringParser("KEY_BLOCK_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3377] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3378] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3379] StringParser("KEYRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3380] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3381] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3382] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3383] StringParser("LAG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3384] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3385] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3386] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3387] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3388] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3389] StringParser("LAST_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3390] StringParser("LASTVAL")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3391] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3392] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3393] StringParser("LEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3394] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3395] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3396] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3397] StringParser("LEAVES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3398] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3399] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3400] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2835, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3401] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3402] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3403] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3404] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3405] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3406] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2840, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3408] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3409] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2842, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3410] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2842, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3411] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3412] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3413] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2844, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3414] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3415] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3416] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3417] Ref(RegexpKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3418] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3419] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3420] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3421] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3422] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3423] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3424] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3425] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3426] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2860, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3427] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2860, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3428] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2862, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3429] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3430] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2864, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3431] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3432] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3433] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3435] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3436] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3437] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3438] StringParser("LINEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3439] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3440] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3441] StringParser("LINESTRING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3442] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3443] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2868, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3444] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2868, child_count: 11, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3445] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3446] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3447] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3448] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3449] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3450] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3451] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3452] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3453] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3454] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3455] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3456] Ref(SystemVariableSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3457] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3458] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2879, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3459] StringParser("LN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2879, child_count: 17, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3460] Sequence(17 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3461] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3462] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2896, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3463] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2898, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3464] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2898, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3465] Ref(ConcurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2898, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3466] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3467] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3468] Ref(InfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3469] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2899, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3470] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3471] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3472] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3473] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3474] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3475] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3476] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2901, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3477] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3478] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3479] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3480] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2904, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3481] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3482] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3483] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3484] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2910, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3485] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3486] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3487] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3488] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2913, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3489] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2917, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3490] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3491] Ref(OptionallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3492] Ref(EnclosedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3493] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3494] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2918, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3495] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3496] Ref(EscapedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3497] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2921, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3498] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2921, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3499] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3500] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2924, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3501] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3502] Ref(StartingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3503] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2927, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3504] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2927, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3505] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3506] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3507] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3508] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2930, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3509] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3510] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3511] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2933, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3512] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2935, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3513] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2935, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3514] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2935, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3515] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3516] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2939, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3517] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3518] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2941, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3519] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3520] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3521] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3522] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3523] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3524] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3525] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3526] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3527] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3528] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3529] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3530] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3531] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2946, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3532] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2946, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3533] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2946, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3534] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3535] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3536] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3537] Ref(SharedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3538] Ref(ExclusiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3539] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3540] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3541] StringParser("LOGFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3542] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3543] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3544] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3545] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3546] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3547] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3548] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3549] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2955, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3550] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3551] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3552] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3553] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3554] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3555] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3556] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3557] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3558] Ref(LoopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3559] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3560] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3561] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2962, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3562] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2962, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3563] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3564] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3565] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3566] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3567] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2970, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3568] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2972, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3569] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2972, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3570] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2972, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2974, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3572] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2974, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3573] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3574] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3575] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3576] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3577] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3578] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3579] StringParser("MASTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3580] StringParser("MASTER_AUTO_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3581] StringParser("MASTER_COMPRESSION_ALGORITHMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3582] StringParser("MASTER_CONNECT_RETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3583] StringParser("MASTER_DELAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3584] StringParser("MASTER_DEMOTE_TO_SLAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3585] StringParser("MASTER_GTID_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3586] StringParser("MASTER_HEARTBEAT_PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3587] StringParser("MASTER_HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3588] StringParser("MASTER_LOG_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3589] StringParser("MASTER_LOG_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3590] StringParser("MASTER_PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3591] StringParser("MASTER_PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3592] StringParser("MASTER_PUBLIC_KEY_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3593] StringParser("MASTER_RETRY_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3594] StringParser("MASTER_SERVER_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3595] StringParser("MASTER_SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3596] StringParser("MASTER_SSL_CA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3597] StringParser("MASTER_SSL_CAPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3598] StringParser("MASTER_SSL_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3599] StringParser("MASTER_SSL_CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3600] StringParser("MASTER_SSL_CRL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3601] StringParser("MASTER_SSL_CRLPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3602] StringParser("MASTER_SSL_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3603] StringParser("MASTER_SSL_VERIFY_SERVER_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3604] StringParser("MASTER_TLS_CIPHERSUITES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3605] StringParser("MASTER_TLS_VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3606] StringParser("MASTER_USE_GTID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3607] StringParser("MASTER_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3608] StringParser("MASTER_ZSTD_COMPRESSION_LEVEL")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3609] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3610] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3611] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3612] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3613] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3614] StringParser("MAX_CONNECTIONS_PER_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3615] StringParser("MAX_QUERIES_PER_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3616] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3617] StringParser("MAX_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3618] StringParser("MAX_STATEMENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3619] StringParser("MAX_UPDATES_PER_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3620] StringParser("MAX_USER_CONNECTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3621] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3622] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3623] StringParser("MEDIUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3624] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3625] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3626] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3627] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3628] StringParser("MEMORY")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3629] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2977, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3630] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3631] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3632] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3633] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3634] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2984, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3635] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2984, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3636] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2984, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3637] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2984, child_count: 2, min_times: 1, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3638] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3639] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3640] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2986, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3641] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2993, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3642] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2993, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3643] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2993, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3644] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3645] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3646] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3647] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3648] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3649] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3650] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 8, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3651] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3005, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3652] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3005, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3653] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3005, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3654] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3005, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3655] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3656] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3657] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3658] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3659] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 12, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3660] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3661] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3663] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3664] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3021, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3665] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3021, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3666] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3667] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3668] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3669] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3670] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3671] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3030, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3672] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3030, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3673] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3674] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3675] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3676] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3677] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3678] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3679] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3680] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3681] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3682] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3683] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3684] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3685] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3686] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3687] StringParser("MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3688] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3689] StringParser("MIGRATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3690] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3691] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3692] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3693] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3694] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3695] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3696] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3697] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3698] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3699] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3700] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3701] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3702] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3703] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3704] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3705] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3706] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3707] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3708] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3709] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3710] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3711] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3712] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3713] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3714] StringParser("MULTILINESTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3715] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3716] StringParser("MULTIPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3717] StringParser("MULTIPOLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3718] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3719] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3720] StringParser("MUTEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3721] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3722] StringParser("MYSQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3723] StringParser("MYSQL_ERRNO")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3724] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3725] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3726] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3727] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3728] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3729] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3730] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3731] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3732] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3733] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3734] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3048, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3735] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3736] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3737] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3738] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3739] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3740] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3742] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3743] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3744] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3745] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3746] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3747] StringParser("NDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3748] StringParser("NDBCLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3749] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3750] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3751] StringParser("NESTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3752] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3753] StringParser("NETWORK_NAMESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3754] StringParser("NEVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3755] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3756] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3757] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3758] StringParser("NEXTVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3759] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3760] StringParser("NO_WAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3761] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3762] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3763] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3764] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3765] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3766] StringParser("NOCOPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3767] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3768] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3769] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3770] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3771] StringParser("NODEGROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3772] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3773] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3774] StringParser("NOMAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3052, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3775] StringParser("NOMINVALUE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3052, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3776] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3777] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3778] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3058, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3779] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3780] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3781] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3064, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3782] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3064, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3783] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3784] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3785] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3067, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3786] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3787] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3071, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3788] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3071, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3789] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3071, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3790] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3071, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3791] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3071, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3792] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3074, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3793] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3074, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3076, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3795] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3796] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3797] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3798] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3799] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3800] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3801] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3802] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3803] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3804] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3079, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3805] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3079, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3806] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3081, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3808] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3809] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3083, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3810] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3085, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3811] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3085, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3812] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3085, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3813] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3085, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3814] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3085, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3816] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3817] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3818] StringParser("NOTFOUND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3819] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3820] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3821] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3822] StringParser("NOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3823] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3824] StringParser("NTH_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3825] StringParser("NTILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3826] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3827] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3087, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3828] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3829] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3830] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3831] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3832] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3833] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3834] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3835] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3836] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3837] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3838] StringParser("NVARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3090, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3839] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3090, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3840] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3841] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3842] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3843] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3093, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3844] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3096, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3845] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3098, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3846] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3098, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3847] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3098, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3848] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3100, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3849] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3100, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3850] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3851] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3852] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 12, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3853] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3854] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3855] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3856] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3857] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3858] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3859] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3860] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3861] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3862] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3863] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3864] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3865] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3866] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3867] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3868] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3869] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3114, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3870] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3114, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3871] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3872] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3873] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3874] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3119, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3875] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3120, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3876] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3120, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3877] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3878] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3879] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3880] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3881] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3882] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3883] StringParser("OJ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3884] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3885] StringParser("OLD_PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3886] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3887] StringParser("ONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3888] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3889] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3890] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3891] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3892] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3893] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3894] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3895] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3896] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3897] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3898] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3899] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3126, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3900] Ref(OptimizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3126, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3901] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3902] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3903] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3904] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3128, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3905] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3906] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3907] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3908] StringParser("OPTIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3909] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3910] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3911] StringParser("OR")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3912] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3913] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3914] StringParser("||")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3915] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3916] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3917] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3918] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3919] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3140, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3920] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3140, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3921] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3140, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 2, _padding: 0 }, // [3922] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3142, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3923] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3924] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3925] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3926] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3149, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3927] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3149, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3928] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3151, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3929] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3151, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3930] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3151, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3931] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3932] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3153, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3933] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3934] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3935] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3936] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3937] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [3938] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3155, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3939] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3940] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3941] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3942] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3943] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3944] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3945] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3946] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3162, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3947] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3162, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3948] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3949] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3950] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3951] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3952] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3953] StringParser("ORGANIZATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3954] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3955] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3956] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3957] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3958] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3164, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3959] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3164, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3960] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3169, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3961] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3169, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3962] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3169, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3963] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3964] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3965] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3174, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3966] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3174, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3967] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3174, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3176, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3969] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3176, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3178, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3971] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3972] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3973] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3974] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3975] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3976] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3977] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3978] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3979] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3980] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3981] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3982] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3983] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3984] StringParser("PACKAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3985] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3986] StringParser("PAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3987] StringParser("PAGE_CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3988] StringParser("PAGE_COMPRESSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3989] StringParser("PAGE_COMPRESSION_LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3990] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3991] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3992] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3993] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3994] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3995] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3996] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3997] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3998] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [3999] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4000] StringParser("PARSE_GCOL_EXPR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4001] StringParser("PARSE_VCOL_EXPR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4002] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3184, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4003] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3184, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4004] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4005] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4006] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4007] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3191, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4008] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3194, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4009] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4010] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4011] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4012] StringParser("PARTITIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4013] StringParser("PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4014] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4015] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4016] StringParser("PASSWORD_LOCK_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3196, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4017] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3196, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4018] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3198, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3200, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4020] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3200, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4021] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4022] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4023] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4024] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4025] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4026] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4027] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4028] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4029] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4030] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4031] StringParser("PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4032] StringParser("PERSIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4033] StringParser("PERSIST_ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4034] StringParser("PERSISTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4035] StringParser("PHASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4036] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4037] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4038] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4039] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4040] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4041] StringParser("PLUGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4042] StringParser("PLUGIN_DIR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4043] StringParser("PLUGINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4044] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4045] StringParser("POINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4046] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4047] StringParser("POLYGON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4048] StringParser("PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4049] StringParser("PORTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4050] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4051] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3202, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4052] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4053] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4054] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4055] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4056] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4057] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4058] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4059] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4060] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4061] StringParser("PRECEDES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4062] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4063] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4064] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4065] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4066] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4067] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4068] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4069] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4070] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4071] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4072] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4073] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4074] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4075] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4076] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4077] StringParser("PREV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3213, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4078] StringParser("PREVIOUS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3213, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4079] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4080] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4081] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4082] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4083] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4084] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4085] StringParser("PRIVILEGE_CHECKS_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4086] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4087] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4088] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3215, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4089] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3215, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3217, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4091] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3220, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4092] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4093] Ref(OutputParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4094] Ref(InputParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4095] Ref(InputOutputParameterSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4096] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4097] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4098] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4099] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3226, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4100] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4101] Ref(ProcedureParameterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4102] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4103] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4104] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4105] StringParser("PROFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4106] StringParser("PROFILES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4107] StringParser("PROXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4108] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4109] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4110] Ref(PurgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4111] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4112] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4113] Ref(MasterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4114] Ref(LogsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4115] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3236, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4116] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4117] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4118] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3238, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4120] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3240, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4121] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3241, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4122] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3241, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4123] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3241, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4124] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4125] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4126] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3243, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4127] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3243, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4128] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3246, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4129] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3246, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4130] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4132] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4133] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3249, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4134] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4135] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4136] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4137] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4138] StringParser("QUERY_RESPONSE_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4139] StringParser("QUICK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4140] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4141] TypedParser("back_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 2, min_times: 1, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4142] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4143] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4144] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4145] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4146] StringParser("RAISE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4147] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4148] StringParser("RANDOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4149] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4150] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4151] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4152] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4153] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4154] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4155] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4156] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4157] StringParser("READ_ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4158] StringParser("READ_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4159] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4160] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4161] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4162] StringParser("REBUILD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4163] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4164] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4165] StringParser("RECOVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4166] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4167] StringParser("REDO_BUFFER_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4168] StringParser("REDOFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4169] StringParser("REDUNDANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4170] StringParser("REF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4171] StringParser("REF_SYSTEM_ID")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4172] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4173] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4174] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4175] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4176] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3259, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4177] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3261, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4178] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4179] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4180] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4181] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3264, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4182] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4183] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4184] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4185] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3267, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4186] StringParser("REFERENCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4188] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3269, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4189] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3272, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4190] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3272, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4191] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3272, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4192] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3272, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4193] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3272, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4194] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3272, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4195] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3272, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4196] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4197] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4198] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4200] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4201] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4202] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4203] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4204] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3281, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4206] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4207] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4208] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4209] StringParser("REGISTRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4210] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4211] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4212] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4213] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4214] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4215] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4216] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4217] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4218] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4219] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4220] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4221] StringParser("RELAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4222] StringParser("RELAY_LOG_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4223] StringParser("RELAY_LOG_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4224] StringParser("RELAY_THREAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4225] StringParser("RELAYLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4226] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4227] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4228] StringParser("REMOTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4229] StringParser("REMOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3283, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4230] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3283, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4231] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3286, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4232] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3286, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4233] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3286, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4234] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3288, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4235] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4236] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4237] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4238] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4239] StringParser("REORGANIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4240] StringParser("REPAIR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 5, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4241] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3296, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4242] Ref(RepairKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3296, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4243] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4244] Ref(No_write_to_binlogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4245] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4246] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4247] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3300, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4248] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3300, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4249] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4250] Ref(QuickKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4251] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4252] Ref(Use_frmKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3303, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4253] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3303, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4254] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4255] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3308, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4257] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4258] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4259] Ref(RepeatKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3310, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4260] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4261] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4262] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3314, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4263] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3314, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4264] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3314, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4265] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4266] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4267] Ref(RepeatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4268] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4269] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3317, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4270] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3317, child_count: 7, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4271] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4272] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3324, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4273] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4274] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4275] Ref(DelayedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3326, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4276] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4277] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4278] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4279] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4280] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4282] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4283] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4284] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4285] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4286] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4287] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4288] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4289] StringParser("REPLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4290] StringParser("REPLICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4291] StringParser("REPLICA_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4292] StringParser("REPLICAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4293] StringParser("REPLICATE_DO_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4294] StringParser("REPLICATE_DO_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4295] StringParser("REPLICATE_IGNORE_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4296] StringParser("REPLICATE_IGNORE_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4297] StringParser("REPLICATE_REWRITE_DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4298] StringParser("REPLICATE_WILD_DO_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4299] StringParser("REPLICATE_WILD_IGNORE_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4300] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4301] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4302] StringParser("REQUIRE_ROW_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4303] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4304] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4305] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4306] Ref(MasterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3337, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4307] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4308] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4309] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3339, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4310] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3339, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4311] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4312] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4313] Ref(SignalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4314] Ref(ResignalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3344, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4315] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3346, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4316] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4317] Ref(SqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4318] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4319] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4320] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3349, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4321] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3351, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4322] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3351, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4323] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3353, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4324] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 13, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4325] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4326] Ref(Class_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4327] Ref(Subclass_originKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4328] Ref(Returned_sqlstateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4329] Ref(Message_textKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4330] Ref(Mysql_errnoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4331] Ref(Constraint_catalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4332] Ref(Constraint_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4333] Ref(Constraint_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4334] Ref(Catalog_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4335] Ref(Schema_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4336] Ref(Table_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4337] Ref(Column_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4338] Ref(Cursor_nameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4339] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 3, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4340] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4341] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4342] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4343] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4344] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4345] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4346] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4347] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4348] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4349] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4350] StringParser("RESUME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4351] StringParser("RETAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3372, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4352] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3372, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4353] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4354] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4355] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4356] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4357] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4358] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3374, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4359] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3374, child_count: 4, min_times: 0, first_terminator_idx: 34, terminator_count: 1, _padding: 0 }, // [4360] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4361] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 3378, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4362] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4363] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [4364] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4365] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4366] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4367] StringParser("REUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4368] StringParser("REVERSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4369] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 6, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4370] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3386, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4371] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3386, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4372] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4373] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4374] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4375] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4376] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3393, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4377] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4378] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4379] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4380] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4381] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4382] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4383] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4384] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4385] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4386] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4387] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4388] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4389] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4390] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4391] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4392] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4393] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4394] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3402, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4395] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3402, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4396] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4397] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4398] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4399] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4400] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4401] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4402] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 3410, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4403] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4404] Ref(AtSignLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4405] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4406] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4407] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4408] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4409] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4410] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4411] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4412] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4413] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4414] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4415] StringParser("ROTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4416] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4417] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4418] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4419] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4420] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4421] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4422] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4423] StringParser("ROW_FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4424] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4425] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4426] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4427] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4428] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4429] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4430] StringParser("ROWTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4431] StringParser("RTREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4432] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 4, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4433] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4434] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3420, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4435] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4436] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4437] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4438] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4439] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3425, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4440] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3427, child_count: 1, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4441] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4442] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4443] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4444] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4445] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4446] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4447] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4448] StringParser("SCHEDULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4449] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4450] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4451] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4452] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4453] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4454] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4455] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4456] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4457] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4458] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4459] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4460] StringParser("SECONDARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4461] StringParser("SECONDARY_ENGINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4462] StringParser("SECONDARY_ENGINE_ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4463] StringParser("SECONDARY_LOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4464] StringParser("SECONDARY_UNLOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4465] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3431, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4466] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3431, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4467] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4468] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4469] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4470] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4471] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3435, child_count: 9, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4472] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3444, child_count: 3, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4473] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4474] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4475] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4476] Ref(DistinctrowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4477] Ref(High_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4478] Ref(Straight_joinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4479] Ref(Sql_small_resultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4480] Ref(Sql_big_resultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4481] Ref(Sql_buffer_resultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4482] Ref(Sql_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4483] Ref(Sql_no_cacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4484] Ref(Sql_calc_found_rowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3447, child_count: 5, min_times: 0, first_terminator_idx: 35, terminator_count: 2, _padding: 0 }, // [4485] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3452, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4486] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3452, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4487] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 3452, child_count: 2, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4488] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4489] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4490] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 0, min_times: 0, first_terminator_idx: 35, terminator_count: 0, _padding: 0 }, // [4491] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 7, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4492] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3461, child_count: 1, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4493] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3462, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4494] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3462, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4495] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3462, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4497] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4498] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4499] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4500] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4501] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4502] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3464, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4503] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3464, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4504] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4505] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4506] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3469, child_count: 2, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4507] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3471, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4508] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3471, child_count: 15, min_times: 0, first_terminator_idx: 37, terminator_count: 4, _padding: 0 }, // [4509] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4510] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4511] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4512] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4513] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4514] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4515] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4516] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4517] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4518] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4519] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4520] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4521] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4522] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4523] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4524] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4525] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4526] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4527] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [4528] Ref(ReturningClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4529] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3492, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4531] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4532] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4533] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4534] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3500, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4535] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3500, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4536] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3500, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4537] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4538] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4539] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4540] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4541] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4542] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4543] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4544] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4545] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4546] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4547] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3507, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4548] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4549] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4550] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3509, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4551] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4553] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4554] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4555] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4556] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4557] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4558] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4559] StringParser("SERIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4560] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4561] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4562] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4563] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4564] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4565] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4566] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4567] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4568] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3519, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4569] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3524, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4570] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3526, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4571] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3528, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4572] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3528, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4573] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3528, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4574] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3528, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4575] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4576] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4577] Ref(PersistKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4578] Ref(Persist_onlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4579] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4580] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4581] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3536, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4582] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3536, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4583] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3536, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4584] Ref(SystemVariableSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3536, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4585] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3538, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4586] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3538, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4587] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3538, child_count: 10, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4588] AnyNumberOf(10 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4589] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4590] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4591] Ref(DoubleQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4592] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4593] Ref(SystemVariableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4594] Ref(BooleanDynamicSystemVariablesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4595] Ref(LocalVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4596] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4597] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4598] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4599] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3552, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4600] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3552, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4601] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3554, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4602] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3554, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4603] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3557, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4604] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3557, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4605] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3557, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4606] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4607] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4608] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4609] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4610] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4611] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4612] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4613] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3564, child_count: 6, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4614] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4615] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4616] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4617] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3570, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4618] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4619] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4620] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4621] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4622] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4623] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4624] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4625] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3577, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4626] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3582, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4627] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3582, child_count: 1, min_times: 1, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4628] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3583, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4630] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4631] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4632] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4633] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4634] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4635] StringParser("SET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3585, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4636] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4637] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4638] Ref(NamesKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4639] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3592, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4640] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3592, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4641] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3592, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4642] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3592, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4643] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4644] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4645] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4646] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3597, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4647] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3597, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4648] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3599, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4649] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4650] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4651] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4652] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4653] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4654] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4655] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4656] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3605, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4657] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4658] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3605, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4659] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3609, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4660] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3609, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4661] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3611, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4662] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3611, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4663] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3611, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4664] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3611, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4665] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3613, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4666] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3615, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4667] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4668] Ref(IsolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4669] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4670] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3621, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4671] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3623, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4672] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3623, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4673] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3625, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4674] Ref(CommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3625, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4675] Ref(UncommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3625, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4676] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4677] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4678] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4679] Ref(SerializableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3627, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4681] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3629, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4683] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4684] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4685] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4686] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4687] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4688] StringParser("SETVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4689] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4690] StringParser("SHARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4691] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3631, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4692] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3633, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4693] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3635, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4694] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3635, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4695] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3635, child_count: 1, min_times: 1, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4696] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4697] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4698] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4699] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4700] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4701] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4702] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4703] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3639, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4704] StringParser("SIGNED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3639, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4705] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4706] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4707] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4708] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3641, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4709] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3641, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 1, _padding: 0 }, // [4710] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4711] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4712] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4713] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4714] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3644, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4715] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4716] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 3, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4717] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4718] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4719] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4720] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4721] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4722] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3649, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4723] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3649, child_count: 2, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4724] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4725] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4726] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4727] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4728] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4729] StringParser("SLAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4730] StringParser("SLAVE_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4731] StringParser("SLAVES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4732] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4733] StringParser("SLOW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4734] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4735] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4736] StringParser("SOCKET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4737] StringParser("SOFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4738] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4739] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4740] StringParser("SOUNDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4741] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4742] StringParser("SOURCE_AUTO_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4743] StringParser("SOURCE_BIND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4744] StringParser("SOURCE_COMPRESSION_ALGORITHMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4745] StringParser("SOURCE_CONNECT_RETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4746] StringParser("SOURCE_DELAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4747] StringParser("SOURCE_HEARTBEAT_PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4748] StringParser("SOURCE_HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4749] StringParser("SOURCE_LOG_FILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4750] StringParser("SOURCE_LOG_POS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4751] StringParser("SOURCE_PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4752] StringParser("SOURCE_PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4753] StringParser("SOURCE_PUBLIC_KEY_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4754] StringParser("SOURCE_RETRY_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4755] StringParser("SOURCE_SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4756] StringParser("SOURCE_SSL_CA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4757] StringParser("SOURCE_SSL_CAPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4758] StringParser("SOURCE_SSL_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4759] StringParser("SOURCE_SSL_CIPHER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4760] StringParser("SOURCE_SSL_CRL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4761] StringParser("SOURCE_SSL_CRLPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4762] StringParser("SOURCE_SSL_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4763] StringParser("SOURCE_SSL_VERIFY_SERVER_CERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4764] StringParser("SOURCE_TLS_CIPHERSUITES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4765] StringParser("SOURCE_TLS_VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4766] StringParser("SOURCE_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4767] StringParser("SOURCE_ZSTD_COMPRESSION_LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4768] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4769] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4770] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4771] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4772] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4773] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4774] StringParser("SQL_AFTER_GTIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4775] StringParser("SQL_AFTER_MTS_GAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4776] StringParser("SQL_BEFORE_GTIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4777] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4778] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4779] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4780] StringParser("SQL_BUFFER_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4781] StringParser("SQL_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4782] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4783] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4784] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4785] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4786] StringParser("SQL_NO_CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4787] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4788] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4789] StringParser("SQL_THREAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4790] StringParser("SQL_TSI_DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4791] StringParser("SQL_TSI_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4792] StringParser("SQL_TSI_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4793] StringParser("SQL_TSI_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4794] StringParser("SQL_TSI_QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4795] StringParser("SQL_TSI_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4796] StringParser("SQL_TSI_WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4797] StringParser("SQL_TSI_YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4798] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4799] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4800] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4801] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4802] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4803] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4804] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4805] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4806] StringParser("SRID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4807] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4808] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4809] StringParser("STACKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4810] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4811] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4812] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4813] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4814] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4815] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4816] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3651, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4817] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 81, min_times: 0, first_terminator_idx: 42, terminator_count: 1, _padding: 0 }, // [4818] OneOf(81 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4819] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4820] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4821] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4822] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4823] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4824] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4825] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4826] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4827] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4828] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4829] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4830] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4831] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4832] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4833] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4834] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4835] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4836] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4837] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4838] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4839] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4840] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4841] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4842] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4843] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4844] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4845] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4846] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4847] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4848] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4849] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4850] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4851] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4852] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4853] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4854] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4855] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4856] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4857] Ref(DelimiterStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4858] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4859] Ref(DeclareStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4860] Ref(SetTransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4861] Ref(SetAssignmentStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4862] Ref(IfExpressionStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4863] Ref(WhileStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4864] Ref(IterateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4865] Ref(RepeatStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4866] Ref(LoopStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4867] Ref(CallStoredProcedureSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4868] Ref(PrepareSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4869] Ref(ExecuteSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4870] Ref(DeallocateSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4871] Ref(GetDiagnosticsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4872] Ref(ResignalSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4873] Ref(CursorOpenCloseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4874] Ref(CursorFetchSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4875] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4876] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4877] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4878] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4879] Ref(RenameTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4880] Ref(ResetMasterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4881] Ref(PurgeBinaryLogsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4882] Ref(HelpStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4883] Ref(CheckTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4884] Ref(ChecksumTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4885] Ref(AnalyzeTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4886] Ref(RepairTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4887] Ref(OptimizeTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4888] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4889] Ref(InsertRowAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4890] Ref(FlushStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4891] Ref(LoadDataSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4892] Ref(ReplaceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4893] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4894] Ref(ReturnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4895] Ref(SetNamesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4896] Ref(CreateEventStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4897] Ref(AlterEventStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4898] Ref(DropEventStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4899] Ref(SetDefaultRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 42, terminator_count: 0, _padding: 0 }, // [4900] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4901] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4902] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4903] StringParser("STATS_AUTO_RECALC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4904] StringParser("STATS_PERSISTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4905] StringParser("STATS_SAMPLE_PAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4906] StringParser("STATUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4907] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4908] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4909] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4910] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4911] StringParser("STOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4912] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4913] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4914] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4915] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4916] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3732, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4917] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3732, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4918] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4919] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3733, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4920] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3733, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4921] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3736, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4922] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3738, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4924] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4925] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4926] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4927] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4928] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4929] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4930] StringParser("SUBJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4931] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4932] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4933] StringParser("SUBPARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4934] StringParser("SUBPARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4935] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4936] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4937] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4938] StringParser("SUPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4939] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4940] StringParser("SUSPEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4941] StringParser("SWAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4942] StringParser("SWITCHES")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4943] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4944] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4945] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4946] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4947] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4948] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4949] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4950] StringParser("SYSTEM_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3740, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4951] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4952] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3743, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4953] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3745, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4955] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3747, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4956] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3747, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4957] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3751, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4958] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3757, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4959] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3757, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4961] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4962] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4963] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4964] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4965] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4966] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4967] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3763, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4968] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3763, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4969] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3763, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4970] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3763, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4971] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3763, child_count: 7, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4972] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4973] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4974] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4975] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4976] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4977] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4978] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3770, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4979] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3771, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3774, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4981] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3774, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4982] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4983] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4984] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4985] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3781, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4986] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3781, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4987] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3781, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4988] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4989] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4990] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4991] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4992] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4993] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4994] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4995] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4996] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4997] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3790, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4998] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3790, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [4999] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5000] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3793, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5001] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5002] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5003] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5004] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5005] Ref(EnforcedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3797, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5006] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3802, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5007] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5008] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5009] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5010] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5011] Ref(IndexTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5012] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3804, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5013] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3804, child_count: 5, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5014] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5015] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5016] Ref(FulltextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3811, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5017] Ref(SpatialKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3811, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5018] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5019] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5020] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5021] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5022] Ref(BracketedKeyPartListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5023] Ref(IndexOptionsSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3813, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5024] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3813, child_count: 6, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5025] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5026] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5027] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5028] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5029] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5030] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5031] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5032] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3825, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5033] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3825, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5034] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(20), first_child_idx: 3825, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5035] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3827, child_count: 32, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5036] OneOf(32 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3859, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5037] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5038] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5039] Ref(EngineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3863, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5040] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3863, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5041] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3865, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5042] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3865, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5043] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3865, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5044] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5045] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3868, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5046] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5047] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3868, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5048] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3871, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5049] Ref(Avg_row_lengthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3871, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5050] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3871, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5051] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3871, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5052] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3875, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5053] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3875, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5054] Ref(CharsetGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3875, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5055] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3875, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5056] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5057] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5058] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5059] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3878, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5060] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5061] Ref(ChecksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5062] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5063] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5064] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3885, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5065] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3885, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5066] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3885, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5067] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3885, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5068] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5069] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5070] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5071] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3888, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5072] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3891, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5073] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3891, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5074] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3891, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5075] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3891, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5076] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3894, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5077] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3894, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5078] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3894, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5079] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3894, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5080] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5081] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5082] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5083] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5084] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5085] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5086] Ref(Delay_key_writeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5087] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5088] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5089] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3904, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5090] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3904, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5091] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3904, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5092] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5093] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5094] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3906, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5095] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3909, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5096] Ref(Encryption_key_idKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3909, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5097] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3909, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5098] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3909, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5099] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5100] Ref(Ietf_quotesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5101] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5103] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5104] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3914, child_count: 4, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5105] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5106] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5107] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5108] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5109] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3918, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5110] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5111] Ref(Insert_methodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3921, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5112] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5113] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5114] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5115] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5116] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5117] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5118] Ref(Key_block_sizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5119] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5120] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3927, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5121] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5122] Ref(Max_rowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5123] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3930, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5124] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3930, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5125] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5126] Ref(Min_rowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5127] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5128] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5129] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5130] Ref(Pack_keysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3936, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5131] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3936, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3938, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5133] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3938, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5134] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3938, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5135] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5136] Ref(Page_checksumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5137] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5138] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3941, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5139] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3944, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5140] Ref(Page_compressedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3944, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5141] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3944, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5142] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3944, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5143] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3947, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5144] Ref(Page_compression_levelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3947, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5145] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3947, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5146] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3947, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5147] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3950, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5148] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3950, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5149] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3950, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5150] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3950, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5151] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3953, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5152] Ref(Row_formatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3953, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5153] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3953, child_count: 7, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5154] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5155] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5156] Ref(DynamicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5157] Ref(FixedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5158] Ref(CompressedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5159] Ref(RedundantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5160] Ref(CompactKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5161] Ref(PageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3960, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5162] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3963, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5163] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3963, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5164] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3963, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5165] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3963, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5166] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5167] Ref(Stats_auto_recalcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5168] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5169] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5170] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5171] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5172] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3971, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5173] Ref(Stats_persistentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3971, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5174] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3971, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5175] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3973, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5176] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3973, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5177] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3973, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5178] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5179] Ref(Stats_sample_pagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3976, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5180] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3976, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5181] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5182] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5183] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3978, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5185] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5186] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3980, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5187] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5188] Ref(TransactionalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5189] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5190] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5191] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5192] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3986, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5193] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3986, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5194] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3989, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5195] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5196] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3991, child_count: 3, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5197] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3994, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5198] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3994, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5199] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3994, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5200] Ref(VersioningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3994, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5201] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5202] Ref(DualIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3996, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 1, _padding: 0 }, // [5203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5204] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5205] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5206] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5207] StringParser("TABLE_CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5208] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5209] StringParser("TABLE_STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5210] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5211] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3998, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5212] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3998, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5213] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4000, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 1, _padding: 0 }, // [5214] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4001, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5215] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4001, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [5216] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4001, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5217] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4001, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5218] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4003, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5219] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5220] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5221] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5222] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5223] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5224] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5225] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4004, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5226] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4004, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5227] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4006, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5228] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4006, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5229] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4006, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5230] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4006, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5231] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5232] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5233] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5234] StringParser("TEMPTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5235] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5236] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5237] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5238] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5239] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5240] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5241] StringParser("THREAD_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5242] StringParser("THREADS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5243] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5244] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4008, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5245] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4008, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5246] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4011, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5247] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4013, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5248] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4013, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5249] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4013, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5250] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4013, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5251] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4016, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5252] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5253] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5254] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5255] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4018, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5256] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4018, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5257] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5258] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5259] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5260] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5261] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5262] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5263] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5264] StringParser("TIMESTAMPADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5265] StringParser("TIMESTAMPDIFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5266] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5267] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5268] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5269] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5270] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5271] StringParser("TLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5272] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5273] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5274] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5275] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5276] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5277] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5278] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5279] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4029, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5280] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4031, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5281] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4031, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5282] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4031, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5283] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4033, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5284] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4035, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5285] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4035, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5286] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4035, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5287] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5288] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5289] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4038, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5290] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4038, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5291] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4040, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5292] Ref(LeaveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4040, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5293] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4040, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5294] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4043, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5295] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4043, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5296] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4043, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5297] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4046, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5298] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4046, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5299] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4046, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5300] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4046, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4048, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5302] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4048, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5303] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4048, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5304] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5305] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5306] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5307] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5308] StringParser("TRANSACTIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5309] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5310] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5311] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5312] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5313] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5314] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5315] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5316] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5317] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5318] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4050, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5319] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5320] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5321] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5322] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5323] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5324] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5325] StringParser("TRIGGERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4052, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5326] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4052, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5327] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5328] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5329] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5330] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5331] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5332] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4055, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5333] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4055, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5334] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4058, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5335] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4058, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5336] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4058, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5337] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4058, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5338] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4058, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5339] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4058, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5340] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4061, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5341] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4063, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5342] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4063, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5343] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4063, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5344] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5345] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5346] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4065, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5347] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5348] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5349] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5350] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5351] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5352] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5353] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5354] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4067, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5355] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4067, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5356] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5357] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5358] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5359] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5360] StringParser("UNDEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5361] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5362] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5363] StringParser("UNDO_BUFFER_SIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5364] StringParser("UNDOFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5365] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5366] StringParser("UNICODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5367] StringParser("UNINSTALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5368] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4072, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5369] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4072, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5370] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4074, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5371] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4074, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5372] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4074, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5373] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4074, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5374] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5375] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5376] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5377] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5378] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5379] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5380] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5381] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5382] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5383] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 11, min_times: 0, first_terminator_idx: 45, terminator_count: 11, _padding: 0 }, // [5384] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5385] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5386] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5387] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5388] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5389] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5390] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5391] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5392] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5393] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5394] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5395] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5396] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5397] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5398] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5399] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5400] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5401] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5402] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5403] Ref(IndexHintClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5404] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5405] Ref(SelectPartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5406] Ref(UpsertClauseListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4087, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4089, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5408] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4089, child_count: 1, min_times: 1, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5409] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5410] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5411] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5412] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5413] StringParser("UNREGISTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5414] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5415] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5416] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 10, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5417] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5418] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5419] Ref(Low_priorityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4102, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5420] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4102, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5421] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4104, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5422] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5423] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5424] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5425] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5426] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5427] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5428] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5429] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5430] StringParser("UPGRADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5431] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5432] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5433] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5434] Ref(DuplicateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5435] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5436] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4111, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5437] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5438] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5439] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5440] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5441] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5442] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5443] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5444] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5445] StringParser("USE_FRM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5446] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5447] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5448] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5449] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5450] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5451] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5452] StringParser("USER_RESOURCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5453] StringParser("USER_STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5454] StringParser("USER_VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5455] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5456] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5457] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5458] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5459] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5460] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5461] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5462] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5463] StringParser("VALIDATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5464] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4115, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5465] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4115, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5466] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4117, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5467] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5468] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5469] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4119, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5470] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4121, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5471] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4123, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5472] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4123, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5473] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4126, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5474] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4128, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5475] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5476] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5477] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5478] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5479] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5480] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5481] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5482] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5483] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5484] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4131, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5485] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4131, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5486] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5487] Ref(SessionVariableNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5488] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5489] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5490] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5491] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5492] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5493] StringParser("VCPU")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5494] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5495] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5496] StringParser("VERSIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5497] StringParser("VIA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5498] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5499] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5500] StringParser("VIRTUAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5501] StringParser("VISIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5502] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4134, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5503] StringParser("WAIT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4134, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5504] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5506] Ref(WaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5507] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5508] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5509] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5510] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5511] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5512] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5513] StringParser("WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5514] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5515] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4138, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5516] StringParser("WEIGHT_STRING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4138, child_count: 8, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5517] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4146, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5518] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4146, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5519] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5520] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5521] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5522] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5523] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5524] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5525] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5526] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5527] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4149, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5528] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4149, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5529] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4153, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5530] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4153, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5531] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4155, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5532] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5533] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4158, child_count: 9, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5534] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5535] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4167, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5536] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4169, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5537] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4169, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5538] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4169, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5539] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5540] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5541] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5542] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5543] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5544] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5545] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5546] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5547] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5548] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5549] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5550] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4173, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4175, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5553] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5554] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4177, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5555] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5556] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5557] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5558] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4181, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5559] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5560] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4182, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5561] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5562] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5563] Ref(WhileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5564] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5565] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5566] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5567] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4186, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5568] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4186, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4188, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5570] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4189, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5571] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4191, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5572] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5573] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5574] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4193, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5575] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4195, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5576] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4195, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5577] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4195, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5578] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4195, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5579] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4195, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5580] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4199, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5581] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4200, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5582] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5583] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5584] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5585] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5586] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4202, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5587] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4202, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5588] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4206, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5589] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4206, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5591] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5592] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5593] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5594] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4208, child_count: 6, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5595] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4214, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5596] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4214, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5597] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4214, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5598] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4214, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 1, _padding: 0 }, // [5599] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5600] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [5601] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5602] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5603] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5604] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4222, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5605] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4222, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5606] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4222, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5607] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4222, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 1, _padding: 0 }, // [5608] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5609] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [5610] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5611] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5612] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5613] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4227, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5614] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4227, child_count: 1, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5615] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4228, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5616] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4228, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5617] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4228, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5618] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4228, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5619] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4228, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5620] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5621] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5622] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5623] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5624] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4232, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5626] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5627] Ref(RollupKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5628] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5629] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5630] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5631] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5632] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5633] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5634] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5635] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5636] StringParser("XA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5637] StringParser("XID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5638] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5639] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5640] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5641] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5642] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5643] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5644] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4234, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [5645] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 210, 217, 220, 221, 218, 219, 230,
    231, 232, 233, 234, 235, 236, 237, 239, 240, 241, 250, 242, 248, 249, 243, 244,
    245, 246, 247, 253, 254, 259, 260, 263, 264, 261, 262, 265, 267, 268, 269, 270,
    271, 286, 291, 295, 304, 305, 272, 273, 274, 277, 278, 280, 275, 276, 279, 281,
    282, 285, 283, 284, 287, 288, 289, 290, 292, 293, 294, 296, 297, 298, 299, 300,
    301, 302, 303, 306, 307, 310, 311, 320, 325, 330, 312, 313, 314, 315, 316, 317,
    318, 319, 321, 322, 323, 324, 326, 327, 328, 329, 331, 332, 333, 334, 335, 336,
    338, 342, 343, 344, 349, 352, 339, 340, 341, 345, 348, 346, 347, 350, 351, 354,
    355, 356, 357, 358, 360, 416, 425, 361, 364, 362, 363, 365, 375, 381, 407, 366,
    367, 370, 371, 372, 373, 374, 368, 369, 376, 377, 378, 379, 380, 382, 383, 384,
    385, 386, 387, 388, 389, 390, 391, 392, 395, 393, 394, 396, 397, 398, 401, 404,
    399, 400, 402, 403, 405, 406, 408, 409, 411, 410, 246, 247, 412, 413, 414, 415,
    417, 420, 421, 422, 423, 424, 418, 419, 426, 429, 432, 433, 434, 435, 427, 428,
    430, 431, 437, 438, 439, 440, 442, 450, 443, 444, 445, 446, 447, 448, 449, 451,
    452, 453, 454, 455, 456, 457, 459, 465, 478, 479, 460, 461, 462, 463, 464, 466,
    469, 470, 471, 467, 468, 472, 477, 473, 476, 474, 475, 480, 481, 484, 482, 483,
    486, 487, 488, 489, 631, 490, 58, 491, 492, 493, 494, 505, 525, 536, 539, 552,
    576, 587, 601, 621, 626, 495, 496, 497, 498, 499, 500, 501, 504, 502, 503, 506,
    507, 508, 509, 510, 520, 511, 517, 512, 513, 514, 515, 516, 518, 519, 521, 522,
    523, 524, 526, 527, 528, 529, 530, 531, 532, 535, 533, 534, 537, 538, 540, 541,
    542, 543, 544, 545, 546, 547, 548, 549, 550, 551, 553, 554, 555, 559, 565, 566,
    570, 556, 557, 558, 560, 563, 564, 561, 562, 567, 568, 569, 571, 574, 575, 572,
    573, 577, 578, 581, 582, 579, 580, 583, 584, 585, 586, 588, 589, 592, 593, 594,
    590, 591, 595, 596, 597, 598, 599, 600, 602, 603, 604, 609, 616, 605, 608, 606,
    607, 610, 613, 614, 615, 611, 612, 617, 618, 619, 620, 622, 625, 623, 624, 627,
    628, 629, 630, 632, 647, 650, 657, 658, 666, 633, 634, 635, 636, 637, 638, 639,
    640, 641, 642, 643, 644, 645, 646, 648, 649, 651, 652, 653, 654, 655, 246, 247,
    656, 58, 659, 660, 661, 662, 665, 663, 664, 667, 668, 669, 246, 247, 670, 58,
    672, 673, 680, 681, 687, 688, 689, 690, 691, 694, 674, 675, 676, 677, 678, 679,
    682, 683, 684, 685, 686, 692, 693, 693, 246, 247, 700, 701, 704, 705, 702, 703,
    706, 709, 720, 707, 708, 58, 710, 711, 712, 713, 714, 716, 715, 58, 717, 718,
    719, 721, 722, 723, 724, 725, 726, 58, 729, 730, 735, 736, 737, 738, 739, 740,
    741, 742, 743, 744, 745, 746, 748, 753, 754, 749, 752, 750, 751, 758, 753, 754,
    759, 58, 762, 780, 790, 791, 792, 793, 794, 795, 796, 799, 800, 801, 797, 798,
    813, 814, 815, 816, 817, 818, 827, 828, 831, 832, 839, 840, 842, 845, 843, 844,
    846, 847, 850, 851, 854, 246, 247, 855, 58, 857, 246, 247, 858, 58, 860, 246,
    247, 861, 58, 862, 869, 863, 864, 867, 865, 866, 868, 246, 247, 870, 871, 874,
    246, 247, 884, 246, 247, 886, 887, 888, 889, 890, 246, 247, 894, 895, 901, 911,
    902, 903, 904, 908, 250, 910, 905, 912, 913, 903, 914, 918, 250, 920, 915, 946,
    951, 954, 969, 947, 948, 949, 950, 952, 953, 955, 958, 961, 965, 956, 957, 959,
    960, 962, 963, 964, 966, 967, 968, 970, 971, 972, 973, 974, 978, 981, 979, 980,
    985, 986, 987, 989, 988, 58, 990, 993, 994, 995, 996, 997, 991, 992, 1002, 1003,
    1004, 1006, 1005, 58, 1007, 1008, 1025, 1026, 1030, 1031, 1032, 1033, 1043, 1044, 1044, 246,
    247, 1045, 1046, 1047, 1048, 1050, 1103, 1051, 1079, 1086, 1087, 1097, 1100, 1052, 1055, 1053,
    1054, 1056, 1059, 1063, 1066, 1069, 1070, 1071, 1074, 1075, 1078, 1057, 1058, 1060, 1061, 1062,
    246, 247, 1064, 1065, 1067, 1068, 1072, 1073, 1076, 1077, 1080, 1081, 1082, 1083, 1084, 1085,
    1088, 1091, 1092, 1094, 1089, 1090, 1093, 246, 247, 1095, 1096, 1098, 1099, 1101, 1102, 1104,
    1107, 1108, 1110, 1105, 1106, 1109, 246, 247, 1111, 1112, 1113, 1115, 1116, 1145, 1147, 1117,
    1121, 1118, 1119, 1120, 1122, 1125, 1126, 1123, 1124, 1127, 1131, 1139, 1128, 1130, 1129, 1132,
    1133, 1134, 1135, 1136, 1137, 1138, 1140, 1141, 1142, 1143, 1144, 1146, 246, 247, 1148, 1153,
    1154, 1172, 1173, 1180, 1181, 1182, 1183, 1184, 1185, 1186, 1187, 1188, 1199, 1200, 1206, 1207,
    1208, 1226, 1227, 246, 247, 1228, 1231, 1229, 1230, 1232, 1233, 1234, 1235, 1246, 1247, 1248,
    1252, 1253, 1254, 1264, 1265, 1266, 1269, 1249, 1250, 1251, 246, 247, 1255, 1256, 1257, 1258,
    1259, 1263, 1260, 1261, 1262, 1267, 1268, 1270, 1271, 1273, 1274, 1277, 1278, 1279, 1275, 1276,
    1280, 1282, 1283, 1284, 1285, 1286, 1287, 1288, 1289, 1292, 1293, 1295, 1301, 1306, 1315, 1316,
    1317, 1290, 1291, 1294, 1296, 1297, 1300, 1298, 1299, 1302, 1303, 1304, 1305, 1307, 1308, 1309,
    1310, 1311, 1312, 1313, 1314, 1319, 1320, 1321, 1322, 1323, 1324, 1325, 1328, 1329, 1330, 1326,
    1327, 1332, 1333, 1334, 1338, 1339, 1340, 1341, 1342, 1343, 1344, 1345, 1346, 1347, 1335, 1336,
    1337, 1348, 1349, 1352, 1353, 1354, 1355, 1356, 1357, 1369, 1370, 1358, 1359, 1360, 246, 247,
    1361, 58, 1362, 1363, 1364, 1365, 1366, 1367, 753, 754, 1368, 58, 1372, 1373, 1374, 1381,
    1385, 1375, 1376, 1377, 1378, 1379, 1380, 1382, 1383, 1384, 1386, 1387, 1388, 1390, 1391, 1392,
    1393, 1394, 1395, 1396, 1397, 1398, 1399, 1401, 1402, 1403, 1404, 1406, 1407, 1408, 1409, 1411,
    1415, 1419, 1420, 1421, 1426, 1429, 1412, 1413, 1414, 1416, 1417, 1418, 1422, 1425, 1423, 1424,
    1427, 1428, 1431, 1432, 1433, 1434, 1435, 1437, 1438, 1439, 1440, 1441, 1442, 1443, 1464, 1465,
    1444, 1456, 1461, 1445, 1450, 1451, 1446, 246, 247, 1447, 58, 1448, 1449, 1452, 1453, 1454,
    1455, 1455, 246, 247, 1457, 1458, 1459, 1460, 1460, 246, 247, 1462, 1463, 1466, 1490, 1467,
    1468, 1479, 1480, 1469, 1470, 1471, 1476, 1472, 1475, 1473, 1474, 1477, 1478, 1481, 1482, 1483,
    1484, 1485, 1486, 1487, 1488, 246, 247, 1489, 58, 1491, 1492, 1493, 1517, 1520, 1540, 1494,
    1508, 1495, 1496, 1497, 1500, 1498, 1499, 1501, 1502, 1506, 1503, 1504, 1505, 1507, 58, 1509,
    1512, 1510, 1511, 1513, 1514, 1515, 246, 247, 1516, 58, 1518, 1519, 1521, 1522, 1523, 1537,
    1524, 1525, 1526, 1529, 1527, 1528, 1530, 1531, 1535, 1532, 1533, 1534, 1536, 246, 247, 1538,
    1539, 1541, 1542, 246, 247, 1543, 58, 1544, 1545, 1546, 1547, 1568, 1591, 1548, 1549, 1550,
    1561, 1551, 1552, 1553, 1554, 1555, 1556, 246, 247, 1557, 1558, 1559, 1560, 1562, 1563, 1564,
    246, 247, 1565, 58, 1566, 1567, 1569, 1580, 1581, 1570, 1571, 1572, 1577, 1573, 1576, 1574,
    1575, 1578, 1579, 1582, 1583, 1584, 1585, 1586, 1587, 1588, 1589, 246, 247, 1590, 58, 1592,
    1593, 1594, 1595, 1609, 1596, 1597, 1598, 1605, 1599, 1600, 1601, 1602, 1603, 1604, 1557, 246,
    247, 1558, 246, 247, 1606, 1607, 1608, 246, 247, 1610, 1621, 1622, 1611, 1612, 1613, 1618,
    1614, 1617, 1615, 1616, 1619, 1620, 1623, 1624, 1625, 1626, 1627, 1628, 1629, 1630, 246, 247,
    1631, 58, 1633, 1634, 1635, 1636, 1637, 1638, 1639, 1642, 1646, 1647, 1648, 1652, 1657, 1640,
    1641, 1643, 1644, 1645, 1649, 1650, 1651, 1653, 1656, 1654, 1655, 1658, 1659, 1660, 1661, 1662,
    1664, 1665, 1666, 1667, 1668, 1716, 1721, 1739, 1749, 1793, 1798, 1669, 58, 1670, 1671, 1672,
    1673, 1715, 1674, 1675, 1676, 1683, 1677, 1678, 1679, 1682, 1680, 1681, 1684, 1685, 1686, 1687,
    1688, 1695, 1698, 1689, 1690, 1691, 1694, 1692, 1693, 1696, 1697, 1699, 1700, 1701, 1702, 1703,
    1710, 1704, 1705, 1706, 1709, 1707, 1708, 1711, 1712, 1713, 1714, 1717, 1718, 1719, 1720, 58,
    1722, 1723, 1724, 1725, 1726, 1738, 1727, 1728, 1729, 1732, 1735, 1730, 1731, 1733, 1734, 1736,
    1737, 1740, 1741, 1742, 1743, 1748, 1744, 1745, 1746, 1747, 1750, 1751, 1762, 1768, 1777, 1785,
    1788, 1752, 1753, 1754, 1755, 1756, 1757, 1758, 1759, 1760, 1761, 1763, 1764, 1765, 1766, 1767,
    1769, 1770, 1771, 1772, 1773, 1774, 1775, 1776, 1778, 1779, 1780, 1781, 1782, 1783, 1784, 1786,
    1787, 1789, 1790, 1791, 1792, 1794, 1795, 1796, 1797, 1799, 1802, 1800, 1801, 1804, 1805, 1806,
    1813, 1814, 1820, 1821, 1822, 1823, 1824, 1827, 1807, 1808, 1809, 1810, 1811, 1812, 1815, 1816,
    1817, 1818, 1819, 1825, 1826, 1826, 246, 247, 1836, 1839, 1837, 1838, 1840, 246, 247, 1844,
    246, 247, 1846, 1847, 1848, 1849, 1851, 1852, 1863, 1864, 1867, 1868, 1869, 1865, 1866, 1870,
    58, 1871, 1872, 1875, 1878, 1876, 1877, 1879, 1880, 1888, 1889, 1890, 1892, 1893, 1896, 1901,
    1929, 1894, 1895, 1897, 1900, 1898, 1899, 1902, 1917, 1918, 1903, 1912, 1904, 1907, 1905, 1906,
    1908, 1909, 1910, 1911, 1913, 1916, 1914, 1915, 1919, 1920, 1921, 1922, 1923, 1926, 1924, 1925,
    1927, 1928, 1934, 1935, 246, 247, 1936, 58, 1937, 1938, 1940, 1944, 1941, 1942, 1943, 1945,
    1946, 1963, 1968, 1964, 1967, 1965, 1966, 1973, 1979, 2003, 2011, 1974, 1975, 1976, 1977, 1978,
    1980, 1981, 1985, 1986, 1987, 2001, 1982, 1983, 1984, 1988, 1989, 1990, 1993, 1997, 1991, 1992,
    1994, 1995, 1996, 1998, 1999, 2000, 2002, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2012, 2013,
    2014, 2015, 2016, 2017, 2018, 2019, 2020, 2023, 2024, 2032, 2033, 2034, 2041, 2042, 2043, 2044,
    2045, 2046, 2053, 2059, 2047, 2048, 2051, 2052, 2049, 58, 2054, 2057, 2058, 2055, 58, 2060,
    2061, 2062, 2063, 2064, 2065, 2067, 2068, 2069, 2070, 2072, 2073, 2074, 58, 2077, 2078, 2091,
    2092, 2093, 2122, 2123, 2125, 2126, 2127, 2131, 2128, 2129, 2130, 246, 247, 2133, 2134, 2135,
    2136, 2137, 2139, 2140, 2141, 2142, 2144, 2145, 2146, 2147, 2149, 2150, 2151, 2152, 2153, 2154,
    2155, 2162, 2156, 2157, 2158, 2159, 2160, 2161, 2163, 2164, 2165, 2166, 2167, 2168, 2169, 2172,
    2173, 2174, 2175, 2177, 2178, 2181, 2182, 2179, 2180, 2184, 2185, 2186, 2187, 2189, 2190, 2191,
    2192, 2193, 2195, 2196, 2197, 2199, 2200, 2201, 2202, 2203, 2205, 2204, 58, 2207, 2208, 2209,
    2210, 2212, 2213, 2214, 2215, 2216, 2218, 2219, 2220, 2221, 2223, 2224, 2225, 2226, 2227, 2240,
    903, 2241, 250, 246, 247, 2248, 2249, 2285, 2286, 2287, 2288, 2289, 2290, 58, 2300, 2301,
    2302, 2303, 2304, 2305, 2309, 2310, 2311, 2312, 2313, 2316, 2317, 2318, 2322, 2323, 2324, 2325,
    2331, 2314, 2315, 2319, 2320, 2321, 2326, 2327, 2328, 2329, 2330, 2332, 2333, 2335, 2338, 2339,
    2340, 2336, 2337, 2342, 2343, 2344, 2345, 2346, 2350, 2347, 2348, 2349, 2352, 2355, 2353, 2354,
    2357, 2360, 2358, 2359, 2361, 2365, 2371, 2362, 2363, 2364, 246, 247, 2366, 2369, 2367, 2368,
    2370, 2374, 2409, 2375, 2376, 2377, 2389, 2390, 2391, 2398, 2406, 2407, 2378, 246, 247, 2379,
    2380, 2381, 2382, 2383, 58, 2384, 2385, 2386, 2387, 2388, 2392, 2395, 2393, 2394, 2396, 246,
    247, 2397, 58, 2399, 2400, 2401, 2402, 2403, 2404, 2405, 2411, 2412, 2413, 2414, 2415, 2416,
    2417, 2418, 2419, 2434, 2435, 2438, 2442, 2445, 2436, 2437, 2439, 2440, 2441, 2443, 2444, 2446,
    2447, 2448, 2449, 2454, 2456, 2460, 2455, 2457, 2458, 2459, 2461, 2465, 2466, 2467, 246, 247,
    2468, 2469, 2482, 2483, 2486, 2484, 2485, 2487, 2536, 2550, 2488, 58, 2489, 2492, 2495, 2498,
    2501, 2504, 2507, 2510, 2515, 2522, 2523, 2524, 2525, 2526, 2527, 2528, 2529, 2530, 2531, 2532,
    2533, 2534, 2535, 2490, 2491, 2493, 2494, 2496, 2497, 2499, 2500, 2502, 2503, 2505, 2506, 2508,
    2509, 2511, 2514, 2512, 2513, 2516, 2517, 2518, 2521, 2519, 2520, 2537, 2538, 2542, 2539, 2540,
    58, 2543, 2544, 2545, 2546, 2547, 2548, 2549, 2551, 2552, 2556, 2553, 2554, 58, 2557, 2558,
    2562, 2577, 2563, 2568, 2572, 2564, 2565, 2566, 2567, 2569, 2570, 2571, 58, 2573, 2574, 2575,
    2576, 2578, 2579, 2580, 2581, 2585, 2586, 2593, 2594, 2595, 2608, 2596, 2599, 2597, 2598, 2600,
    2605, 2601, 2602, 2603, 2604, 2606, 2607, 2609, 2595, 2610, 2595, 2612, 2613, 2619, 2620, 2621,
    58, 2623, 2624, 2625, 2628, 2631, 2632, 2633, 2634, 2635, 2636, 2637, 2638, 2639, 2640, 2641,
    2642, 2643, 2626, 2627, 2629, 2630, 2645, 2663, 2646, 2647, 2650, 2651, 2657, 2661, 2662, 2648,
    2649, 2649, 246, 247, 2652, 2653, 2654, 2655, 2656, 2658, 2659, 2660, 2664, 246, 247, 2645,
    2665, 2666, 2668, 2669, 2669, 246, 247, 903, 2670, 250, 2681, 2682, 2693, 2671, 2672, 2673,
    2674, 246, 247, 2676, 2677, 2679, 2680, 2683, 2684, 2685, 2686, 2688, 2689, 2691, 2692, 2699,
    2700, 2704, 2710, 2716, 2722, 2723, 2726, 2736, 2737, 2738, 2701, 2702, 2703, 2705, 2706, 2708,
    2709, 2707, 2711, 2714, 2715, 2712, 2713, 2717, 2718, 2719, 2720, 2721, 58, 2724, 2725, 2727,
    2731, 2732, 2728, 2729, 2730, 2733, 2734, 2735, 2740, 2741, 246, 247, 2746, 2751, 2747, 2748,
    2749, 2752, 2753, 2756, 2763, 2757, 2758, 2759, 2762, 2760, 2761, 2764, 2767, 2765, 2766, 2769,
    246, 247, 2770, 58, 2772, 2775, 2778, 2781, 2784, 2785, 2773, 2774, 2776, 2777, 2779, 2780,
    2782, 2783, 2786, 2793, 2787, 2792, 2788, 2789, 2790, 2791, 2804, 2805, 2808, 2809, 2818, 2819,
    2823, 2806, 2807, 2810, 58, 2811, 2814, 2815, 2812, 2813, 2816, 2817, 2820, 2821, 2822, 2824,
    58, 2825, 2828, 2829, 2826, 2827, 2830, 2831, 2832, 2833, 2834, 2835, 2836, 2837, 2838, 2839,
    2840, 2841, 2842, 2853, 2854, 2861, 2862, 2864, 2855, 2859, 2856, 2857, 2858, 2860, 58, 2863,
    58, 2865, 2869, 2866, 2867, 2868, 2870, 2871, 2872, 2876, 2877, 2881, 2882, 239, 2883, 2894,
    250, 2884, 2885, 58, 2886, 2890, 2887, 2888, 2889, 2891, 2892, 2896, 2899, 2900, 2901, 2902,
    2903, 2897, 2898, 2907, 2908, 58, 2909, 2910, 2911, 2912, 246, 247, 2916, 2917, 2918, 2919,
    246, 247, 2920, 58, 2921, 2922, 2929, 903, 2930, 250, 2931, 2932, 2932, 246, 247, 2934,
    2937, 2938, 2939, 2940, 2935, 2936, 2946, 2947, 2969, 2970, 2972, 2977, 2982, 2985, 2973, 2974,
    2975, 2976, 2978, 2979, 2980, 2981, 2983, 2984, 2986, 2987, 2990, 2991, 2992, 2995, 2998, 2996,
    2997, 3011, 3012, 3013, 3014, 3019, 3015, 246, 247, 3016, 3018, 3017, 58, 3025, 3026, 3027,
    3028, 3030, 246, 247, 3032, 3036, 3039, 3049, 3051, 3033, 3034, 3035, 3037, 3038, 3040, 3041,
    3042, 3043, 3046, 3044, 3045, 3047, 3048, 3050, 246, 247, 3054, 3058, 3059, 3063, 3066, 3072,
    3055, 3056, 3057, 3060, 3061, 3062, 3064, 3065, 3067, 3068, 3069, 3070, 3071, 3073, 3074, 3075,
    3076, 3078, 3082, 3083, 3087, 3088, 3091, 3095, 3079, 3080, 3081, 3084, 3085, 3086, 3089, 3090,
    3092, 3093, 3094, 3096, 3097, 3098, 3100, 3101, 3102, 3103, 3105, 3106, 3107, 3108, 3109, 3132,
    3133, 3134, 3135, 246, 247, 3137, 3138, 3142, 3143, 3144, 3145, 3149, 3150, 3155, 3156, 3157,
    3139, 3140, 3141, 3146, 3147, 3148, 246, 247, 3151, 3152, 3153, 3177, 3178, 3179, 3180, 3181,
    3182, 3185, 3186, 3187, 3188, 58, 3189, 3192, 3195, 3190, 3191, 3193, 3194, 3196, 3197, 3198,
    3202, 3219, 3199, 3200, 3201, 3203, 3206, 3210, 3215, 3204, 3205, 3207, 3208, 3209, 3211, 3212,
    3213, 3214, 3216, 3217, 3218, 3220, 3221, 3225, 3222, 3223, 3224, 3226, 3227, 3228, 3237, 3238,
    3239, 3240, 3241, 3243, 3244, 3245, 3246, 3256, 3257, 3259, 3272, 3277, 3260, 3261, 239, 3262,
    3263, 250, 3265, 3264, 3266, 3267, 3268, 3271, 3269, 3270, 3273, 3274, 239, 3275, 3276, 250,
    3278, 239, 3279, 250, 3282, 3285, 3286, 3287, 3290, 3288, 3289, 3289, 246, 247, 3292, 3293,
    3294, 3298, 3295, 3296, 3297, 3300, 239, 3301, 250, 3302, 246, 247, 3303, 58, 3306, 3307,
    3308, 246, 247, 3309, 58, 3310, 3314, 3319, 3311, 3312, 3313, 3315, 3316, 3317, 3318, 3320,
    3321, 3322, 3323, 3327, 3324, 3325, 3326, 3328, 3329, 3330, 3331, 3332, 3338, 3339, 3333, 3334,
    3335, 3336, 3337, 3340, 3341, 3343, 3344, 3345, 3346, 246, 247, 3349, 3350, 3351, 3352, 3359,
    246, 247, 3353, 3354, 3355, 3356, 3357, 3358, 3360, 3361, 3367, 3368, 3362, 3363, 3364, 3365,
    3366, 3369, 3370, 3402, 3403, 3407, 3410, 3411, 3408, 3409, 3412, 3413, 3415, 3416, 3417, 3421,
    239, 3422, 3428, 250, 3423, 3424, 3424, 246, 247, 3425, 3426, 3427, 3429, 3434, 3430, 3431,
    3432, 3433, 3435, 3436, 3446, 3447, 3448, 3449, 3450, 3451, 3452, 3453, 3454, 3455, 3456, 3461,
    3462, 3463, 3466, 3468, 3469, 3470, 3473, 3474, 3475, 3476, 3477, 3481, 3499, 3509, 3515, 3519,
    3464, 3465, 3467, 3471, 3472, 3478, 3479, 3480, 3482, 3485, 3489, 3495, 3483, 3484, 3486, 3487,
    3488, 3490, 3492, 3493, 3494, 3491, 3496, 3497, 3498, 3500, 3501, 3505, 3502, 3503, 3504, 3506,
    3507, 3508, 3510, 3511, 3512, 3513, 3514, 3516, 3517, 246, 247, 3518, 58, 3520, 3521, 3532,
    3533, 3534, 3535, 3536, 3537, 3538, 3549, 3556, 3550, 3553, 3554, 3551, 3552, 3555, 58, 3557,
    3558, 3559, 3564, 3565, 3566, 3567, 3568, 3571, 246, 247, 3569, 3570, 3572, 3573, 3574, 246,
    247, 3631, 239, 3632, 250, 3633, 3635, 3636, 3639, 3640, 3642, 3643, 3644, 3647, 239, 3648,
    250, 3645, 3646, 3649, 3650, 3652, 3653, 3654, 3655, 3658, 239, 3659, 250, 3656, 3657, 3661,
    239, 3662, 250, 3665, 239, 3666, 250, 3676, 3677, 3678, 3679, 3663, 3664, 3667, 3668, 3669,
    3673, 3670, 3672, 3671, 246, 247, 3674, 3675, 3681, 239, 3682, 250, 3727, 3728, 3729, 3730,
    3731, 3732, 246, 247, 3734, 239, 3735, 250, 3736, 58, 3742, 3743, 3777, 3778, 3779, 3781,
    3783, 3785, 3780, 246, 247, 3782, 246, 247, 3784, 246, 247, 3788, 3789, 3790, 3791, 3793,
    3794, 3797, 3795, 3796, 3796, 246, 247, 3807, 3810, 3808, 3809, 3811, 3812, 3816, 3817, 3829,
    3830, 3831, 3841, 3842, 3843, 3845, 3847, 2257, 3846, 58, 3849, 3850, 3851, 3852, 3854, 3855,
    3856, 3857, 3858, 3859, 3860, 3861, 3862, 3863, 3864, 872, 3872, 3873, 3877, 3874, 3875, 3876,
    3878, 3879, 3900, 3901, 3904, 3905, 3902, 3903, 3906, 58, 3913, 3914, 3915, 3917, 3918, 3920,
    3921, 239, 3922, 250, 3923, 58, 3924, 3928, 3931, 3936, 3925, 3926, 3927, 3929, 3930, 3932,
    3933, 3934, 3935, 3940, 3941, 3942, 3943, 3944, 3945, 3946, 3949, 3950, 239, 3961, 3962, 3963,
    250, 3964, 3965, 3966, 246, 247, 3969, 3970, 3971, 3976, 3972, 3973, 3974, 3975, 246, 247,
    4005, 4006, 239, 4007, 250, 4008, 4009, 4009, 246, 247, 4010, 58, 4019, 4024, 4020, 4021,
    4022, 4023, 4053, 4054, 4056, 4057, 4068, 4069, 4070, 4071, 4072, 4073, 4074, 4080, 4081, 4091,
    4098, 4092, 4096, 4097, 4093, 4094, 4095, 4100, 246, 247, 4101, 58, 4110, 4111, 4114, 4115,
    4112, 4113, 4116, 4119, 4117, 4118, 4120, 4121, 4122, 4125, 4126, 4129, 4130, 4134, 4131, 4132,
    4133, 4135, 246, 247, 4143, 4144, 4173, 4174, 4175, 4176, 4177, 4178, 4182, 4179, 4180, 4181,
    4183, 4184, 4185, 4188, 4189, 4190, 4191, 4192, 4197, 4198, 4199, 4202, 4205, 4200, 4201, 4203,
    4204, 4206, 4207, 4232, 4233, 4234, 4235, 58, 4236, 4237, 4238, 4242, 4243, 4246, 4247, 4249,
    4244, 4245, 4248, 58, 4250, 4251, 4252, 4255, 4262, 4256, 4259, 4260, 4257, 4258, 4261, 4263,
    4264, 4265, 4266, 4267, 4268, 4272, 4273, 4276, 4278, 4279, 4280, 4288, 4274, 4275, 4277, 4281,
    4284, 4285, 4282, 4283, 4286, 4287, 4305, 4306, 4307, 4308, 4309, 4312, 4315, 4321, 4313, 4314,
    4316, 4320, 4317, 4318, 4319, 4322, 4323, 4324, 58, 4325, 4339, 4340, 4326, 4327, 4328, 4329,
    4330, 4331, 4332, 4333, 4334, 4335, 4336, 4337, 4338, 4341, 4342, 4343, 4354, 4355, 4361, 239,
    4362, 250, 4363, 58, 4371, 4372, 4376, 4390, 4391, 4392, 4373, 4374, 4375, 4377, 4381, 4384,
    4389, 4378, 4379, 4380, 4382, 4383, 4385, 4386, 4387, 4388, 4397, 4410, 4398, 4403, 4399, 4400,
    4401, 4402, 4404, 4405, 4406, 4407, 4408, 4409, 4434, 4435, 4438, 4440, 4436, 4437, 4439, 246,
    247, 4441, 4443, 4442, 4444, 246, 247, 4468, 4469, 4470, 4471, 4473, 4477, 4478, 4479, 4480,
    4481, 4482, 4483, 4484, 4474, 4475, 4476, 4486, 4487, 239, 4488, 250, 4489, 58, 4493, 4495,
    4496, 4499, 4500, 4501, 4502, 4494, 4497, 4498, 4505, 4506, 4507, 246, 247, 4508, 58, 4510,
    4511, 4512, 4513, 4514, 4515, 4516, 4517, 4518, 4519, 4520, 4521, 4522, 4523, 4524, 4530, 4533,
    4536, 4537, 4531, 4532, 4532, 246, 247, 4534, 4535, 4535, 246, 247, 4538, 246, 247, 4545,
    4548, 4546, 4547, 4549, 4550, 4552, 4555, 4553, 4554, 4556, 4557, 4567, 4568, 4569, 58, 4570,
    4575, 4581, 4585, 4588, 4571, 4574, 4572, 4573, 4576, 4577, 4578, 4579, 4580, 4582, 4583, 4584,
    4586, 4587, 4589, 4590, 4591, 4592, 4593, 4594, 4595, 4596, 4597, 4598, 4600, 239, 4601, 250,
    4602, 58, 4604, 4605, 4606, 4607, 4608, 4609, 4610, 4611, 4612, 4613, 4615, 4616, 4617, 4618,
    4623, 4624, 4619, 4620, 4621, 4622, 58, 4625, 58, 4627, 4628, 4632, 4633, 4634, 4629, 4630,
    4631, 4637, 4638, 4639, 4643, 4640, 4641, 4642, 4644, 4645, 4647, 4648, 4653, 4649, 4652, 4650,
    4651, 4655, 4656, 4657, 4658, 4660, 4661, 4664, 4665, 4662, 4663, 4666, 58, 4667, 4680, 4668,
    4669, 4670, 4671, 4676, 4679, 4672, 4673, 4674, 4675, 4677, 4678, 4681, 4682, 4683, 4684, 4693,
    4696, 4694, 4695, 4697, 4698, 4699, 4700, 4706, 4707, 4711, 4712, 4713, 4716, 58, 4718, 4719,
    4720, 4725, 4726, 4819, 4820, 4821, 4822, 4823, 4824, 4825, 4826, 4827, 4828, 4829, 4830, 4831,
    4832, 4833, 4834, 4835, 4836, 4837, 4838, 4839, 4840, 4841, 4842, 4843, 4844, 4845, 4846, 4847,
    4848, 4849, 4850, 4851, 4852, 4853, 4854, 4855, 4856, 4857, 4858, 4859, 4860, 4861, 4862, 4863,
    4864, 4865, 4866, 4867, 4868, 4869, 4870, 4871, 4872, 4873, 4874, 4875, 4876, 4877, 4878, 4879,
    4880, 4881, 4882, 4883, 4884, 4885, 4886, 4887, 4888, 4889, 4890, 4891, 4892, 4893, 4894, 4895,
    4896, 4897, 4898, 4899, 4919, 4922, 246, 247, 4923, 58, 4924, 4925, 4953, 5006, 5014, 4954,
    4957, 4955, 4956, 4958, 4967, 4972, 4997, 4959, 4960, 4963, 4964, 4965, 4966, 4961, 4962, 4968,
    4969, 4970, 4971, 4973, 4974, 4975, 4976, 4977, 4978, 4979, 4980, 4981, 4982, 4985, 4983, 4984,
    4986, 4987, 4988, 4991, 4994, 4989, 4990, 4992, 4993, 4995, 4996, 4998, 4999, 5001, 5000, 246,
    247, 5002, 5003, 5004, 5005, 5007, 5010, 5011, 5012, 5013, 5008, 5009, 5015, 5018, 5021, 5022,
    5023, 5016, 5017, 5019, 5020, 5026, 5027, 5028, 5029, 5030, 5032, 5031, 246, 247, 5033, 246,
    247, 5036, 58, 5037, 5044, 5048, 5052, 5060, 5064, 5072, 5076, 5080, 5085, 5089, 5095, 5099,
    5105, 5110, 5117, 5121, 5125, 5129, 5135, 5139, 5143, 5147, 5151, 5162, 5166, 5172, 5178, 5184,
    5187, 5191, 5197, 5038, 5039, 5040, 5041, 5042, 5043, 5045, 5046, 5047, 5049, 5050, 5051, 5053,
    5054, 5055, 5056, 5057, 5058, 5059, 5061, 5062, 5063, 5065, 5066, 5067, 5068, 5069, 5070, 5071,
    5073, 5074, 5075, 5077, 5078, 5079, 5081, 5082, 5083, 5084, 5086, 5087, 5088, 5090, 5091, 5092,
    5093, 5094, 5096, 5097, 5098, 5100, 5101, 5102, 5103, 5104, 5106, 5107, 5108, 5109, 5111, 5112,
    5113, 5114, 5115, 5116, 5118, 5119, 5120, 5122, 5123, 5124, 5126, 5127, 5128, 5130, 5131, 5132,
    5133, 5134, 5136, 5137, 5138, 5140, 5141, 5142, 5144, 5145, 5146, 5148, 5149, 5150, 5152, 5153,
    5154, 5155, 5156, 5157, 5158, 5159, 5160, 5161, 5163, 5164, 5165, 5167, 5168, 5169, 5170, 5171,
    5173, 5174, 5175, 5176, 5177, 5179, 5180, 5181, 5182, 5183, 5185, 5186, 5188, 5189, 5190, 5192,
    5193, 5194, 5195, 246, 247, 5196, 58, 5198, 5199, 5200, 5202, 5203, 5204, 5205, 5214, 5217,
    5215, 5219, 5221, 5220, 5228, 5229, 5232, 5233, 5247, 5250, 5251, 5248, 5249, 5252, 5255, 5256,
    5253, 5254, 5258, 5259, 5260, 5261, 5262, 5280, 5283, 5291, 5294, 5301, 5304, 5281, 5282, 5284,
    5287, 5285, 5286, 5288, 5289, 5290, 5292, 5293, 5295, 5296, 5297, 5298, 5299, 5300, 5302, 5303,
    5305, 5306, 5320, 5321, 5328, 5329, 5330, 5335, 5336, 5337, 5341, 246, 247, 5342, 58, 5345,
    5346, 5348, 5349, 5357, 5358, 5359, 5369, 5370, 5371, 5372, 5375, 5376, 5385, 5386, 5387, 5388,
    5389, 5390, 5391, 5392, 5393, 5394, 5395, 5408, 5409, 5410, 5411, 5412, 5418, 5419, 5420, 239,
    5421, 250, 5425, 5426, 5427, 5428, 5422, 58, 5423, 5424, 5433, 5434, 5435, 5436, 5437, 5438,
    58, 5442, 5443, 5467, 5470, 5468, 5469, 5471, 58, 5472, 5473, 5474, 246, 247, 5475, 58,
    5476, 5477, 5478, 5487, 5488, 5489, 5505, 5508, 5506, 5507, 5518, 5519, 5521, 5522, 5523, 5524,
    5525, 5526, 903, 5520, 250, 5530, 903, 5531, 250, 5532, 5533, 5533, 246, 247, 5535, 5536,
    5539, 5542, 5543, 5544, 5545, 5546, 5547, 5537, 5538, 5540, 5541, 5551, 5561, 5552, 5555, 5553,
    5554, 5556, 5557, 5558, 5559, 5560, 5562, 5563, 5564, 5568, 5570, 5578, 5571, 5572, 5575, 5573,
    5574, 5576, 5577, 5581, 5585, 5586, 5587, 5582, 5583, 5584, 5589, 5590, 5593, 5594, 5591, 5592,
    5596, 5597, 5598, 5599, 5602, 5603, 5600, 58, 5605, 5606, 5607, 5608, 5611, 5612, 5609, 58,
    5614, 5615, 5617, 5616, 5621, 5622, 5623, 5624, 5626, 5627,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 802, 803, 906, 907, 909, 916, 917, 919, 921, 922, 923, 1034, 1155,
    2050, 2056, 2372, 2408, 2541, 2555, 2675, 2678, 2687, 2690, 2750, 2754, 2893, 2913, 3154, 3229,
    3937, 3938, 4364, 4490, 4491, 4525, 4526, 4527, 4528, 4714, 4900, 5206, 5216, 5396, 5397, 5398,
    5399, 5400, 5401, 5402, 5403, 5404, 5405, 5406, 5601, 5610,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FOREIGN", // [17]
    "FUNCTION", // [18]
    "FUTURE", // [19]
    "INTEGRATION", // [20]
    "LANGUAGE", // [21]
    "MASKING", // [22]
    "MATERIALIZED", // [23]
    "PROCEDURE", // [24]
    "RESOURCE", // [25]
    "ROLE", // [26]
    "ROUTINE", // [27]
    "SCHEMA", // [28]
    "SEQUENCE", // [29]
    "STAGE", // [30]
    "STREAM", // [31]
    "TABLE", // [32]
    "TABLESPACE", // [33]
    "TASK", // [34]
    "TYPE", // [35]
    "VIEW", // [36]
    "WAREHOUSE", // [37]
    "ResourceKeywordSegment", // [38]
    "MonitorKeywordSegment", // [39]
    "MONITOR", // [40]
    "WarehouseKeywordSegment", // [41]
    "DatabaseKeywordSegment", // [42]
    "DomainKeywordSegment", // [43]
    "IntegrationKeywordSegment", // [44]
    "LanguageKeywordSegment", // [45]
    "SchemaKeywordSegment", // [46]
    "CatalogKeywordSegment", // [47]
    "RoleKeywordSegment", // [48]
    "TablespaceKeywordSegment", // [49]
    "TypeKeywordSegment", // [50]
    "ForeignKeywordSegment", // [51]
    "DATA", // [52]
    "SERVER", // [53]
    "ServerKeywordSegment", // [54]
    "DataKeywordSegment", // [55]
    "WrapperKeywordSegment", // [56]
    "WRAPPER", // [57]
    "AllKeywordSegment", // [58]
    "SchemasKeywordSegment", // [59]
    "SCHEMAS", // [60]
    "InKeywordSegment", // [61]
    "IN", // [62]
    "FutureKeywordSegment", // [63]
    "AccessSchemaObjectSegment", // [64]
    "access_schema_object", // [65]
    "AccessSchemaPluralObjectSegment", // [66]
    "FUNCTIONS", // [67]
    "PROCEDURES", // [68]
    "ROUTINES", // [69]
    "SEQUENCES", // [70]
    "STAGES", // [71]
    "STREAMS", // [72]
    "TABLES", // [73]
    "TASKS", // [74]
    "VIEWS", // [75]
    "access_schema_plural_object", // [76]
    "ObjectReferenceSegment", // [77]
    "object_reference", // [78]
    "FunctionNameSegment", // [79]
    "function_name", // [80]
    "FunctionParameterListGrammar", // [81]
    "(", // [82]
    "function_parameter_list", // [83]
    "WildcardIdentifierSegment", // [84]
    "wildcard_identifier", // [85]
    "CommaSegment", // [86]
    ",", // [87]
    "ToKeywordSegment", // [88]
    "TO", // [89]
    "FromKeywordSegment", // [90]
    "FROM", // [91]
    "LARGE", // [92]
    "LargeKeywordSegment", // [93]
    "ObjectKeywordSegment", // [94]
    "OBJECT", // [95]
    "NumericLiteralSegment", // [96]
    "numeric_literal", // [97]
    "APPLY", // [98]
    "CONNECT", // [99]
    "CREATE", // [100]
    "DELETE", // [101]
    "EXECUTE", // [102]
    "IMPORTED", // [103]
    "INSERT", // [104]
    "MANAGE", // [105]
    "MODIFY", // [106]
    "OPERATE", // [107]
    "OWNERSHIP", // [108]
    "READ", // [109]
    "REFERENCES", // [110]
    "REFERENCE_USAGE", // [111]
    "SELECT", // [112]
    "TEMP", // [113]
    "TEMPORARY", // [114]
    "TRIGGER", // [115]
    "TRUNCATE", // [116]
    "UPDATE", // [117]
    "USAGE", // [118]
    "USE", // [119]
    "USE_ANY_ROLE", // [120]
    "WRITE", // [121]
    "CreateKeywordSegment", // [122]
    "USER", // [123]
    "UserKeywordSegment", // [124]
    "ApplyKeywordSegment", // [125]
    "MaskingKeywordSegment", // [126]
    "PolicyKeywordSegment", // [127]
    "POLICY", // [128]
    "ExecuteKeywordSegment", // [129]
    "TaskKeywordSegment", // [130]
    "ManageKeywordSegment", // [131]
    "GrantsKeywordSegment", // [132]
    "GRANTS", // [133]
    "EXECUTION", // [134]
    "ExecutionKeywordSegment", // [135]
    "UsageKeywordSegment", // [136]
    "PIPE", // [137]
    "PipeKeywordSegment", // [138]
    "UseKeywordSegment", // [139]
    "ImportedKeywordSegment", // [140]
    "PrivilegesKeywordSegment", // [141]
    "PRIVILEGES", // [142]
    "ConnectKeywordSegment", // [143]
    "DeleteKeywordSegment", // [144]
    "InsertKeywordSegment", // [145]
    "ModifyKeywordSegment", // [146]
    "OperateKeywordSegment", // [147]
    "OwnershipKeywordSegment", // [148]
    "ReadKeywordSegment", // [149]
    "Reference_usageKeywordSegment", // [150]
    "ReferencesKeywordSegment", // [151]
    "SelectKeywordSegment", // [152]
    "TempKeywordSegment", // [153]
    "TemporaryKeywordSegment", // [154]
    "TriggerKeywordSegment", // [155]
    "TruncateKeywordSegment", // [156]
    "UpdateKeywordSegment", // [157]
    "Use_any_roleKeywordSegment", // [158]
    "WriteKeywordSegment", // [159]
    "BracketedColumnReferenceListGrammar", // [160]
    "AccessPermissionSegment", // [161]
    "access_permission", // [162]
    "OnKeywordSegment", // [163]
    "ON", // [164]
    "TableKeywordSegment", // [165]
    "ViewKeywordSegment", // [166]
    "StageKeywordSegment", // [167]
    "FunctionKeywordSegment", // [168]
    "ProcedureKeywordSegment", // [169]
    "RoutineKeywordSegment", // [170]
    "SequenceKeywordSegment", // [171]
    "StreamKeywordSegment", // [172]
    "MaterializedKeywordSegment", // [173]
    "ExternalKeywordSegment", // [174]
    "FileKeywordSegment", // [175]
    "FormatKeywordSegment", // [176]
    "FORMAT", // [177]
    "TablesKeywordSegment", // [178]
    "ViewsKeywordSegment", // [179]
    "StagesKeywordSegment", // [180]
    "FunctionsKeywordSegment", // [181]
    "ProceduresKeywordSegment", // [182]
    "RoutinesKeywordSegment", // [183]
    "SequencesKeywordSegment", // [184]
    "StreamsKeywordSegment", // [185]
    "TasksKeywordSegment", // [186]
    "PoliciesKeywordSegment", // [187]
    "FormatsKeywordSegment", // [188]
    "GRANT", // [189]
    "REVOKE", // [190]
    "GrantStatementSegment", // [191]
    "grant_statement", // [192]
    "RevokeStatementSegment", // [193]
    "revoke_statement", // [194]
    "GROUP", // [195]
    "GroupKeywordSegment", // [196]
    "UserReferenceSegment", // [197]
    "user_reference", // [198]
    "RoleReferenceSegment", // [199]
    "role_reference", // [200]
    "SHARE", // [201]
    "ShareKeywordSegment", // [202]
    "FunctionSegment", // [203]
    "function", // [204]
    "PublicKeywordSegment", // [205]
    "PUBLIC", // [206]
    "ACCESSIBLE", // [207]
    "[", // [208]
    "ArrayAccessorSegment", // [209]
    "array_accessor", // [210]
    "ACCOUNTS", // [211]
    "ACTION", // [212]
    "ACTIVE", // [213]
    "ADA", // [214]
    "ADD", // [215]
    "DROP", // [216]
    "AddKeywordSegment", // [217]
    "DropKeywordSegment", // [218]
    "SystemKeywordSegment", // [219]
    "SYSTEM", // [220]
    "VersioningKeywordSegment", // [221]
    "VERSIONING", // [222]
    "ADMIN", // [223]
    "AFTER", // [224]
    "AGAINST", // [225]
    "AGGREGATE", // [226]
    "OrderByClauseSegment", // [227]
    "ORDER", // [228]
    "orderby_clause", // [229]
    "ALGORITHM", // [230]
    "AlgorithmKeywordSegment", // [231]
    "EqualsSegment", // [232]
    "=", // [233]
    "comparison_operator", // [234]
    "COPY", // [235]
    "DEFAULT", // [236]
    "INPLACE", // [237]
    "INSTANT", // [238]
    "NOCOPY", // [239]
    "DefaultKeywordSegment", // [240]
    "InplaceKeywordSegment", // [241]
    "CopyKeywordSegment", // [242]
    "NocopyKeywordSegment", // [243]
    "InstantKeywordSegment", // [244]
    "indent", // [245]
    "Indent", // [246]
    "meta", // [247]
    "AsAliasOperatorSegment", // [248]
    "AS", // [249]
    "alias_operator", // [250]
    "SingleIdentifierGrammar", // [251]
    "SingleIdentifierListSegment", // [252]
    "identifier_list", // [253]
    "start_bracket", // [254]
    "SymbolSegment", // [255]
    "symbol", // [256]
    ")", // [257]
    "end_bracket", // [258]
    "SingleQuotedIdentifierSegment", // [259]
    "single_quote", // [260]
    "DoubleQuotedIdentifierSegment", // [261]
    "double_quote", // [262]
    "dedent", // [263]
    "Dedent", // [264]
    "ALIAS", // [265]
    "TableReferenceSegment", // [266]
    "table_reference", // [267]
    "AliasExpressionSegment", // [268]
    "alias_expression", // [269]
    "ALLOCATE", // [270]
    "ALSO", // [271]
    "ALTER", // [272]
    "AlterKeywordSegment", // [273]
    "DatabaseReferenceSegment", // [274]
    "database_reference", // [275]
    "CHARACTER", // [276]
    "CHARSET", // [277]
    "COLLATE", // [278]
    "ENCRYPTION", // [279]
    "AlterOptionSegment", // [280]
    "alter_option_segment", // [281]
    "DefinerSegment", // [282]
    "DEFINER", // [283]
    "definer_segment", // [284]
    "EventKeywordSegment", // [285]
    "EVENT", // [286]
    "ScheduleKeywordSegment", // [287]
    "SCHEDULE", // [288]
    "AT", // [289]
    "EVERY", // [290]
    "AtKeywordSegment", // [291]
    "EveryKeywordSegment", // [292]
    "ExpressionSegment", // [293]
    "expression", // [294]
    "DAY", // [295]
    "DAY_HOUR", // [296]
    "DAY_MICROSECOND", // [297]
    "DAY_MINUTE", // [298]
    "DAY_SECOND", // [299]
    "HOUR", // [300]
    "HOUR_MICROSECOND", // [301]
    "HOUR_MINUTE", // [302]
    "HOUR_SECOND", // [303]
    "MICROSECOND", // [304]
    "MINUTE", // [305]
    "MINUTE_MICROSECOND", // [306]
    "MINUTE_SECOND", // [307]
    "MONTH", // [308]
    "QUARTER", // [309]
    "SECOND", // [310]
    "SECOND_MICROSECOND", // [311]
    "WEEK", // [312]
    "YEAR", // [313]
    "YEAR_MONTH", // [314]
    "DatetimeUnitSegment", // [315]
    "ENDS", // [316]
    "STARTS", // [317]
    "StartsKeywordSegment", // [318]
    "EndsKeywordSegment", // [319]
    "CompletionKeywordSegment", // [320]
    "COMPLETION", // [321]
    "NotKeywordSegment", // [322]
    "NOT", // [323]
    "PreserveKeywordSegment", // [324]
    "PRESERVE", // [325]
    "RENAME", // [326]
    "RenameKeywordSegment", // [327]
    "DISABLE", // [328]
    "ENABLE", // [329]
    "EnableKeywordSegment", // [330]
    "DisableKeywordSegment", // [331]
    "REPLICA", // [332]
    "SLAVE", // [333]
    "ReplicaKeywordSegment", // [334]
    "SlaveKeywordSegment", // [335]
    "CommentClauseSegment", // [336]
    "COMMENT", // [337]
    "comment_clause", // [338]
    "DO", // [339]
    "DoKeywordSegment", // [340]
    "StatementSegment", // [341]
    "statement", // [342]
    "CharsetGrammar", // [343]
    "CollateKeywordSegment", // [344]
    "CollationReferenceSegment", // [345]
    "collation_reference", // [346]
    "EncryptionKeywordSegment", // [347]
    "QuotedLiteralSegment", // [348]
    "OnlyKeywordSegment", // [349]
    "ONLY", // [350]
    "CACHE", // [351]
    "CYCLE", // [352]
    "INCREMENT", // [353]
    "MAXVALUE", // [354]
    "MINVALUE", // [355]
    "NO", // [356]
    "NOCACHE", // [357]
    "NOCYCLE", // [358]
    "NOORDER", // [359]
    "IncrementKeywordSegment", // [360]
    "ByKeywordSegment", // [361]
    "BY", // [362]
    "SequenceMinValueGrammar", // [363]
    "SequenceMaxValueGrammar", // [364]
    "CacheKeywordSegment", // [365]
    "NocacheKeywordSegment", // [366]
    "CycleKeywordSegment", // [367]
    "NocycleKeywordSegment", // [368]
    "OrderNoOrderGrammar", // [369]
    "SequenceReferenceSegment", // [370]
    "sequence_reference", // [371]
    "AlterSequenceOptionsSegment", // [372]
    "alter_sequence_options_segment", // [373]
    "CHECK", // [374]
    "CONSTRAINT", // [375]
    "FULLTEXT", // [376]
    "INDEX", // [377]
    "KEY", // [378]
    "PRIMARY", // [379]
    "SPATIAL", // [380]
    "UNIQUE", // [381]
    "ConstraintKeywordSegment", // [382]
    "UniqueKeywordSegment", // [383]
    "IndexKeywordSegment", // [384]
    "KeyKeywordSegment", // [385]
    "IfNotExistsGrammar", // [386]
    "IF", // [387]
    "IndexReferenceSegment", // [388]
    "index_reference", // [389]
    "IndexTypeGrammar", // [390]
    "USING", // [391]
    "index_type", // [392]
    "BracketedKeyPartListGrammar", // [393]
    "IndexOptionsSegment", // [394]
    "ENGINE_ATTRIBUTE", // [395]
    "INVISIBLE", // [396]
    "KEY_BLOCK_SIZE", // [397]
    "SECONDARY_ENGINE_ATTRIBUTE", // [398]
    "VISIBLE", // [399]
    "WITH", // [400]
    "index_option", // [401]
    "PrimaryKeyGrammar", // [402]
    "ForeignKeyGrammar", // [403]
    "ColumnReferenceSegment", // [404]
    "column_reference", // [405]
    "CASCADE", // [406]
    "RESTRICT", // [407]
    "SET", // [408]
    "RestrictKeywordSegment", // [409]
    "CascadeKeywordSegment", // [410]
    "SetKeywordSegment", // [411]
    "NullKeywordSegment", // [412]
    "NULL", // [413]
    "NoKeywordSegment", // [414]
    "ActionKeywordSegment", // [415]
    "CheckKeywordSegment", // [416]
    "ENFORCED", // [417]
    "EnforcedKeywordSegment", // [418]
    "FulltextKeywordSegment", // [419]
    "SpatialKeywordSegment", // [420]
    "ColumnKeywordSegment", // [421]
    "COLUMN", // [422]
    "IfExistsGrammar", // [423]
    "LOCK", // [424]
    "LockKeywordSegment", // [425]
    "EXCLUSIVE", // [426]
    "NONE", // [427]
    "SHARED", // [428]
    "NoneKeywordSegment", // [429]
    "SharedKeywordSegment", // [430]
    "ExclusiveKeywordSegment", // [431]
    "ParameterNameSegment", // [432]
    "LiteralGrammar", // [433]
    "NakedIdentifierSegment", // [434]
    "ColumnDefinitionSegment", // [435]
    "column_definition", // [436]
    "FIRST", // [437]
    "FirstKeywordSegment", // [438]
    "AfterKeywordSegment", // [439]
    "AlterTableDropColumnGrammar", // [440]
    "AsKeywordSegment", // [441]
    "AUTO_INCREMENT", // [442]
    "AVG_ROW_LENGTH", // [443]
    "CHANGE", // [444]
    "CHECKSUM", // [445]
    "CONNECTION", // [446]
    "CONVERT", // [447]
    "DELAY_KEY_WRITE", // [448]
    "ENCRYPTED", // [449]
    "ENCRYPTION_KEY_ID", // [450]
    "ENGINE", // [451]
    "IETF_QUOTES", // [452]
    "INSERT_METHOD", // [453]
    "MAX_ROWS", // [454]
    "MIN_ROWS", // [455]
    "PACK_KEYS", // [456]
    "PAGE_CHECKSUM", // [457]
    "PAGE_COMPRESSED", // [458]
    "PAGE_COMPRESSION_LEVEL", // [459]
    "PASSWORD", // [460]
    "ROW_FORMAT", // [461]
    "STATS_AUTO_RECALC", // [462]
    "STATS_PERSISTENT", // [463]
    "STATS_SAMPLE_PAGES", // [464]
    "STORAGE", // [465]
    "TRANSACTIONAL", // [466]
    "UNION", // [467]
    "TableOptionsSegment", // [468]
    "table_options", // [469]
    "AlterTableOnlineDDLOptionSegment", // [470]
    "alter_table_online_ddl_option", // [471]
    "AddDropSystemVersioningGrammar", // [472]
    "InvisibleKeywordSegment", // [473]
    "VisibleKeywordSegment", // [474]
    "AlterTableConstraintSegment", // [475]
    "table_constraint", // [476]
    "ChangeKeywordSegment", // [477]
    "IGNORED", // [478]
    "IgnoredKeywordSegment", // [479]
    "KeysKeywordSegment", // [480]
    "KEYS", // [481]
    "ConvertKeywordSegment", // [482]
    "ANALYZE", // [483]
    "COALESCE", // [484]
    "DISCARD", // [485]
    "EXCHANGE", // [486]
    "IMPORT", // [487]
    "OPTIMIZE", // [488]
    "REBUILD", // [489]
    "REMOVE", // [490]
    "REORGANIZE", // [491]
    "REPAIR", // [492]
    "DiscardKeywordSegment", // [493]
    "ImportKeywordSegment", // [494]
    "CoalesceKeywordSegment", // [495]
    "ReorganizeKeywordSegment", // [496]
    "ExchangeKeywordSegment", // [497]
    "AnalyzeKeywordSegment", // [498]
    "OptimizeKeywordSegment", // [499]
    "RebuildKeywordSegment", // [500]
    "RepairKeywordSegment", // [501]
    "RemoveKeywordSegment", // [502]
    "PARTITION", // [503]
    "PARTITIONING", // [504]
    "PartitionKeywordSegment", // [505]
    "PartitioningKeywordSegment", // [506]
    "WithKeywordSegment", // [507]
    "TableReference", // [508]
    "WITHOUT", // [509]
    "WithoutKeywordSegment", // [510]
    "ValidationKeywordSegment", // [511]
    "VALIDATION", // [512]
    "INTO", // [513]
    "IntoKeywordSegment", // [514]
    "MERGE", // [515]
    "TEMPTABLE", // [516]
    "UNDEFINED", // [517]
    "UndefinedKeywordSegment", // [518]
    "MergeKeywordSegment", // [519]
    "TemptableKeywordSegment", // [520]
    "SQL", // [521]
    "SqlKeywordSegment", // [522]
    "SecurityKeywordSegment", // [523]
    "SECURITY", // [524]
    "INVOKER", // [525]
    "DefinerKeywordSegment", // [526]
    "InvokerKeywordSegment", // [527]
    "VALUE", // [528]
    "VALUES", // [529]
    "SelectableGrammar", // [530]
    "WithCheckOptionSegment", // [531]
    "with_check_options", // [532]
    "ALWAYS", // [533]
    "&", // [534]
    "ampersand", // [535]
    "ANALYSE", // [536]
    "LOCAL", // [537]
    "NO_WRITE_TO_BINLOG", // [538]
    "No_write_to_binlogKeywordSegment", // [539]
    "LocalKeywordSegment", // [540]
    "HistogramKeywordSegment", // [541]
    "HISTOGRAM", // [542]
    "BucketsKeywordSegment", // [543]
    "BUCKETS", // [544]
    "AND", // [545]
    "&&", // [546]
    "binary_operator", // [547]
    "BinaryOperatorSegment", // [548]
    "ANY", // [549]
    "ARE", // [550]
    "%", // [551]
    "*", // [552]
    "+", // [553]
    "-", // [554]
    "/", // [555]
    "<", // [556]
    ">", // [557]
    "DIV", // [558]
    "MOD", // [559]
    "^", // [560]
    "|", // [561]
    "PlusSegment", // [562]
    "MinusSegment", // [563]
    "DivideSegment", // [564]
    "MultiplySegment", // [565]
    "ModuloSegment", // [566]
    "BitwiseAndSegment", // [567]
    "BitwiseOrSegment", // [568]
    "BitwiseXorSegment", // [569]
    "BitwiseLShiftSegment", // [570]
    "BitwiseRShiftSegment", // [571]
    "DivOperatorSegment", // [572]
    "ModOperatorSegment", // [573]
    "SliceSegment", // [574]
    ":", // [575]
    "start_square_bracket", // [576]
    "]", // [577]
    "end_square_bracket", // [578]
    "ARRAY", // [579]
    "BaseExpressionElementGrammar", // [580]
    "ASC", // [581]
    "ASCII", // [582]
    "ASENSITIVE", // [583]
    "ASSERTION", // [584]
    "ASSIGNMENT", // [585]
    "ASYMMETRIC", // [586]
    "at_sign_literal", // [587]
    "literal", // [588]
    "LiteralSegment", // [589]
    "ATOMIC", // [590]
    "ATTRIBUTE", // [591]
    "ATTRIBUTES", // [592]
    "AUDIT", // [593]
    "AUTHENTICATION", // [594]
    "AUTHORIZATION", // [595]
    "AUTHORS", // [596]
    "Auto_incrementKeywordSegment", // [597]
    "AUTO", // [598]
    "AUTOEXTEND_SIZE", // [599]
    "AVG", // [600]
    "BACKUP", // [601]
    "BACKWARD", // [602]
    "CURRENT_DATE", // [603]
    "CURRENT_TIME", // [604]
    "CURRENT_TIMESTAMP", // [605]
    "LOCALTIME", // [606]
    "LOCALTIMESTAMP", // [607]
    "NOW", // [608]
    "bare_function", // [609]
    "CodeSegment", // [610]
    "BareFunctionSegment", // [611]
    "IntervalExpressionSegment", // [612]
    "INTERVAL", // [613]
    "interval_expression", // [614]
    "DatatypeSegment", // [615]
    "data_type", // [616]
    "SessionVariableNameSegment", // [617]
    "LocalVariableNameSegment", // [618]
    "VariableAssignmentSegment", // [619]
    "file", // [620]
    "BaseFileSegment", // [621]
    "BaseSegment", // [622]
    "BEFORE", // [623]
    "BEGIN", // [624]
    "BERNOULLI", // [625]
    "BETWEEN", // [626]
    "BIGINT", // [627]
    "BINARY", // [628]
    "!", // [629]
    "->", // [630]
    "->>", // [631]
    "IS", // [632]
    "OR", // [633]
    "XOR", // [634]
    "||", // [635]
    "like_operator", // [636]
    "ArithmeticBinaryOperatorGrammar", // [637]
    "StringBinaryOperatorGrammar", // [638]
    "BooleanBinaryOperatorGrammar", // [639]
    "ComparisonOperatorGrammar", // [640]
    "ColumnPathOperatorSegment", // [641]
    "InlinePathOperatorSegment", // [642]
    "BINDING", // [643]
    "BINLOG", // [644]
    "BIT", // [645]
    "BIT_LENGTH", // [646]
    "BITVAR", // [647]
    "AmpersandSegment", // [648]
    "RawLessThanSegment", // [649]
    "PipeSegment", // [650]
    "RawGreaterThanSegment", // [651]
    "BLOB", // [652]
    "BLOCK", // [653]
    "BODY", // [654]
    "BOOL", // [655]
    "AndOperatorGrammar", // [656]
    "OrOperatorGrammar", // [657]
    "FALSE", // [658]
    "OFF", // [659]
    "TRUE", // [660]
    "OffKeywordSegment", // [661]
    "TrueKeywordSegment", // [662]
    "FalseKeywordSegment", // [663]
    "BOOLEAN", // [664]
    "TrueSegment", // [665]
    "FalseSegment", // [666]
    "BOTH", // [667]
    "IndexColumnPrefixLengthSegment", // [668]
    "bracketed_arguments", // [669]
    "DESC", // [670]
    "AscKeywordSegment", // [671]
    "DescKeywordSegment", // [672]
    "bracketed", // [673]
    "BracketedSegment", // [674]
    "UnorderedSetExpressionSegment", // [675]
    "set_expression", // [676]
    "BREADTH", // [677]
    "BREAK", // [678]
    "BROWSE", // [679]
    "BTREE", // [680]
    "BULK", // [681]
    "BYTE", // [682]
    "CTEColumnList", // [683]
    "cte_column_list", // [684]
    "CALL", // [685]
    "CallKeywordSegment", // [686]
    "CALLED", // [687]
    "CARDINALITY", // [688]
    "CASCADED", // [689]
    "CASE", // [690]
    "CaseKeywordSegment", // [691]
    "implicit_indent", // [692]
    "ImplicitIndent", // [693]
    "WHEN", // [694]
    "WhenClauseSegment", // [695]
    "when_clause", // [696]
    "ElseKeywordSegment", // [697]
    "ELSE", // [698]
    "EndKeywordSegment", // [699]
    "END", // [700]
    "ElseClauseSegment", // [701]
    "else_clause", // [702]
    "BinaryOperatorGrammar", // [703]
    "CAST", // [704]
    "::", // [705]
    "casting_operator", // [706]
    "CATALOG_NAME", // [707]
    "CEIL", // [708]
    "CEILING", // [709]
    "CHAIN", // [710]
    "CHALLENGE_RESPONSE", // [711]
    "CHANGED", // [712]
    "CHANGED_PAGE_BITMAPS", // [713]
    "CHANNEL", // [714]
    "BinaryKeywordSegment", // [715]
    "CHAR", // [716]
    "CHAR_LENGTH", // [717]
    "CHARACTER_LENGTH", // [718]
    "CHARACTER_SET_CATALOG", // [719]
    "CHARACTER_SET_NAME", // [720]
    "CHARACTER_SET_SCHEMA", // [721]
    "DETERMINISTIC", // [722]
    "DeterministicKeywordSegment", // [723]
    "CONTAINS", // [724]
    "MODIFIES", // [725]
    "READS", // [726]
    "ContainsKeywordSegment", // [727]
    "ReadsKeywordSegment", // [728]
    "ModifiesKeywordSegment", // [729]
    "CHARACTERISTICS", // [730]
    "CHARACTERS", // [731]
    "CharacterKeywordSegment", // [732]
    "CharsetKeywordSegment", // [733]
    "EXTENDED", // [734]
    "FAST", // [735]
    "FOR", // [736]
    "MEDIUM", // [737]
    "QUICK", // [738]
    "ForKeywordSegment", // [739]
    "UpgradeKeywordSegment", // [740]
    "UPGRADE", // [741]
    "QuickKeywordSegment", // [742]
    "FastKeywordSegment", // [743]
    "MediumKeywordSegment", // [744]
    "ExtendedKeywordSegment", // [745]
    "ChangedKeywordSegment", // [746]
    "CHECKED", // [747]
    "CHECKPOINT", // [748]
    "ChecksumKeywordSegment", // [749]
    "CIPHER", // [750]
    "CLASS", // [751]
    "CLASS_ORIGIN", // [752]
    "CLIENT", // [753]
    "CLIENT_STATISTICS", // [754]
    "CLOB", // [755]
    "CLONE", // [756]
    "CLOSE", // [757]
    "CLUSTER", // [758]
    "CLUSTERED", // [759]
    "CLUSTERING", // [760]
    "COBOL", // [761]
    "CODE", // [762]
    "COLLATION", // [763]
    "ObjectReferenceDelimiterGrammar", // [764]
    ".", // [765]
    "ObjectReferenceTerminatorGrammar", // [766]
    "COLLATION_CATALOG", // [767]
    "COLLATION_NAME", // [768]
    "COLLATION_SCHEMA", // [769]
    "COLLECT", // [770]
    "colon_delimiter", // [771]
    "colon_prefix", // [772]
    "colon", // [773]
    "ShorthandCastSegment", // [774]
    "cast_expression", // [775]
    "ColumnConstraintDefaultGrammar", // [776]
    "NotEnforcedGrammar", // [777]
    "UniqueKeyGrammar", // [778]
    "AutoIncrementGrammar", // [779]
    "ReferenceDefinitionGrammar", // [780]
    "ColumnGeneratedGrammar", // [781]
    "CollateGrammar", // [782]
    "GENERATED", // [783]
    "GeneratedKeywordSegment", // [784]
    "AlwaysKeywordSegment", // [785]
    "STORED", // [786]
    "VIRTUAL", // [787]
    "StoredKeywordSegment", // [788]
    "VirtualKeywordSegment", // [789]
    "SRID", // [790]
    "SridKeywordSegment", // [791]
    "PERSISTENT", // [792]
    "PersistentKeywordSegment", // [793]
    "DATETIME", // [794]
    "TIMESTAMP", // [795]
    "DatetimeKeywordSegment", // [796]
    "TimestampKeywordSegment", // [797]
    "BracketedArguments", // [798]
    "CurrentTimestampLikeFunctionSegment", // [799]
    "ColumnConstraintSegment", // [800]
    "column_constraint_segment", // [801]
    "column_path_operator", // [802]
    "COLUMN_ADD", // [803]
    "COLUMN_CHECK", // [804]
    "COLUMN_CREATE", // [805]
    "COLUMN_DELETE", // [806]
    "COLUMN_FORMAT", // [807]
    "COLUMN_GET", // [808]
    "COLUMN_NAME", // [809]
    "ColumnsExpressionNameGrammar", // [810]
    "COLUMNS", // [811]
    "comma", // [812]
    "COMMAND_FUNCTION", // [813]
    "COMMAND_FUNCTION_CODE", // [814]
    "CommentKeywordSegment", // [815]
    "comment", // [816]
    "CommentSegment", // [817]
    "COMMIT", // [818]
    "COMMITTED", // [819]
    "COMPACT", // [820]
    "GreaterThanSegment", // [821]
    "LessThanSegment", // [822]
    "GreaterThanOrEqualToSegment", // [823]
    "LessThanOrEqualToSegment", // [824]
    "NotEqualToSegment", // [825]
    "LikeOperatorSegment", // [826]
    "IsDistinctFromGrammar", // [827]
    "NullSafeEqualsSegment", // [828]
    "ComparisonOperatorSegment", // [829]
    "COMPONENT", // [830]
    "CompositeBinaryOperatorSegment", // [831]
    "CompositeComparisonOperatorSegment", // [832]
    "COMPRESS", // [833]
    "COMPRESSED", // [834]
    "COMPRESSION", // [835]
    "COMPUTE", // [836]
    "CONCURRENT", // [837]
    "CONDITION", // [838]
    "CONDITION_NUMBER", // [839]
    "CrossKeywordSegment", // [840]
    "CROSS", // [841]
    "JoinTypeKeywordsGrammar", // [842]
    "FULL", // [843]
    "INNER", // [844]
    "LEFT", // [845]
    "RIGHT", // [846]
    "ConditionalCrossJoinKeywordsGrammar", // [847]
    "NonStandardJoinTypeKeywordsGrammar", // [848]
    "CONNECTION_NAME", // [849]
    "CONSISTENT", // [850]
    "CONSTRAINT_CATALOG", // [851]
    "CONSTRAINT_NAME", // [852]
    "CONSTRAINT_SCHEMA", // [853]
    "CONSTRAINTS", // [854]
    "CONSTRUCTOR", // [855]
    "CONTAINSTABLE", // [856]
    "CONTEXT", // [857]
    "CONTINUE", // [858]
    "CONTRIBUTORS", // [859]
    "CONVERSION", // [860]
    "UsingKeywordSegment", // [861]
    "function_name_identifier", // [862]
    "CORR", // [863]
    "CORRESPONDING", // [864]
    "COUNT", // [865]
    "COVAR_POP", // [866]
    "COVAR_SAMP", // [867]
    "CPU", // [868]
    "CastKeywordSegment", // [869]
    "SpecificKeywordSegment", // [870]
    "SPECIFIC", // [871]
    "INSTANCE", // [872]
    "METHOD", // [873]
    "STATIC", // [874]
    "InstanceKeywordSegment", // [875]
    "StaticKeywordSegment", // [876]
    "ConstructorKeywordSegment", // [877]
    "MethodKeywordSegment", // [878]
    "AssignmentKeywordSegment", // [879]
    "CreateOptionSegment", // [880]
    "create_option_segment", // [881]
    "OrReplaceGrammar", // [882]
    "RETURNS", // [883]
    "ReturnsKeywordSegment", // [884]
    "CharacteristicStatement", // [885]
    "characteristic_statement", // [886]
    "FunctionDefinitionGrammar", // [887]
    "function_definition", // [888]
    "IndexTypeSegment", // [889]
    "WaitOptionSegment", // [890]
    "NOWAIT", // [891]
    "WAIT", // [892]
    "wait_option", // [893]
    "IndexOptionSegment", // [894]
    "AlgorithmOptionSegment", // [895]
    "algorithm_option", // [896]
    "LockOptionSegment", // [897]
    "lock_option", // [898]
    "ModelKeywordSegment", // [899]
    "MODEL", // [900]
    "OPTIONS", // [901]
    "OptionsKeywordSegment", // [902]
    "ProcedureParameterListGrammar", // [903]
    "procedure_parameter_list", // [904]
    "SchemaReferenceSegment", // [905]
    "schema_reference", // [906]
    "START", // [907]
    "StartKeywordSegment", // [908]
    "CreateSequenceOptionsSegment", // [909]
    "create_sequence_options_segment", // [910]
    "TemporaryTransientGrammar", // [911]
    "TRANSIENT", // [912]
    "LIKE", // [913]
    "TableConstraintSegment", // [914]
    "LikeKeywordSegment", // [915]
    "TableEndClauseSegment", // [916]
    "table_end_clause_segment", // [917]
    "DirectoryKeywordSegment", // [918]
    "DIRECTORY", // [919]
    "HASH", // [920]
    "LINEAR", // [921]
    "LIST", // [922]
    "RANGE", // [923]
    "LinearKeywordSegment", // [924]
    "HashKeywordSegment", // [925]
    "RangeKeywordSegment", // [926]
    "ListKeywordSegment", // [927]
    "PARTITIONS", // [928]
    "PartitionsKeywordSegment", // [929]
    "SUBPARTITION", // [930]
    "SubpartitionKeywordSegment", // [931]
    "SUBPARTITIONS", // [932]
    "SubpartitionsKeywordSegment", // [933]
    "ValuesKeywordSegment", // [934]
    "LESS", // [935]
    "LessKeywordSegment", // [936]
    "ThanKeywordSegment", // [937]
    "THAN", // [938]
    "MaxvalueKeywordSegment", // [939]
    "TriggerOrReplaceGrammar", // [940]
    "TriggerReferenceSegment", // [941]
    "trigger_reference", // [942]
    "BeforeKeywordSegment", // [943]
    "EachKeywordSegment", // [944]
    "EACH", // [945]
    "RowKeywordSegment", // [946]
    "ROW", // [947]
    "FOLLOWS", // [948]
    "PRECEDES", // [949]
    "FollowsKeywordSegment", // [950]
    "PrecedesKeywordSegment", // [951]
    "BeginKeywordSegment", // [952]
    "IDENTIFIED", // [953]
    "IdentifiedKeywordSegment", // [954]
    "RANDOM", // [955]
    "RandomKeywordSegment", // [956]
    "PasswordKeywordSegment", // [957]
    "INITIAL", // [958]
    "InitialKeywordSegment", // [959]
    "AuthenticationKeywordSegment", // [960]
    "AndKeywordSegment", // [961]
    "REQUIRE", // [962]
    "RequireKeywordSegment", // [963]
    "ISSUER", // [964]
    "SSL", // [965]
    "SUBJECT", // [966]
    "X509", // [967]
    "SslKeywordSegment", // [968]
    "X509KeywordSegment", // [969]
    "CipherKeywordSegment", // [970]
    "IssuerKeywordSegment", // [971]
    "SubjectKeywordSegment", // [972]
    "MAX_CONNECTIONS_PER_HOUR", // [973]
    "MAX_QUERIES_PER_HOUR", // [974]
    "MAX_UPDATES_PER_HOUR", // [975]
    "MAX_USER_CONNECTIONS", // [976]
    "Max_queries_per_hourKeywordSegment", // [977]
    "Max_updates_per_hourKeywordSegment", // [978]
    "Max_connections_per_hourKeywordSegment", // [979]
    "Max_user_connectionsKeywordSegment", // [980]
    "FAILED_LOGIN_ATTEMPTS", // [981]
    "PASSWORD_LOCK_TIME", // [982]
    "ExpireKeywordSegment", // [983]
    "EXPIRE", // [984]
    "NEVER", // [985]
    "NeverKeywordSegment", // [986]
    "IntervalKeywordSegment", // [987]
    "DayKeywordSegment", // [988]
    "HistoryKeywordSegment", // [989]
    "HISTORY", // [990]
    "ReuseKeywordSegment", // [991]
    "REUSE", // [992]
    "CurrentKeywordSegment", // [993]
    "CURRENT", // [994]
    "OPTIONAL", // [995]
    "OptionalKeywordSegment", // [996]
    "Failed_login_attemptsKeywordSegment", // [997]
    "Password_lock_timeKeywordSegment", // [998]
    "UNBOUNDED", // [999]
    "UnboundedKeywordSegment", // [1000]
    "UNLOCK", // [1001]
    "UnlockKeywordSegment", // [1002]
    "AttributeKeywordSegment", // [1003]
    "CREATEDB", // [1004]
    "CREATEROLE", // [1005]
    "CREATEUSER", // [1006]
    "CSV", // [1007]
    "CUBE", // [1008]
    "ROLLUP", // [1009]
    "CubeFunctionNameSegment", // [1010]
    "RollupFunctionNameSegment", // [1011]
    "GroupingExpressionList", // [1012]
    "grouping_expression_list", // [1013]
    "CUME_DIST", // [1014]
    "Current_timestampKeywordSegment", // [1015]
    "NowKeywordSegment", // [1016]
    "LocaltimeKeywordSegment", // [1017]
    "LocaltimestampKeywordSegment", // [1018]
    "CurrentTimestampLikeFunctionNameSegment", // [1019]
    "CurrentTimestampLikeFunctionContentsSegment", // [1020]
    "function_contents", // [1021]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [1022]
    "CURRENT_PATH", // [1023]
    "CURRENT_POS", // [1024]
    "CURRENT_ROLE", // [1025]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [1026]
    "CURRENT_USER", // [1027]
    "FETCH", // [1028]
    "FetchKeywordSegment", // [1029]
    "NEXT", // [1030]
    "NextKeywordSegment", // [1031]
    "CURSOR", // [1032]
    "OPEN", // [1033]
    "CloseKeywordSegment", // [1034]
    "OpenKeywordSegment", // [1035]
    "QuotedIdentifierSegment", // [1036]
    "back_quote", // [1037]
    "CURSOR_NAME", // [1038]
    "DATABASES", // [1039]
    "DATAFILE", // [1040]
    "data_type_identifier", // [1041]
    "TimeWithTZGrammar", // [1042]
    "TIME", // [1043]
    "DOUBLE", // [1044]
    "DoubleKeywordSegment", // [1045]
    "PrecisionKeywordSegment", // [1046]
    "PRECISION", // [1047]
    "SIGNED", // [1048]
    "UNSIGNED", // [1049]
    "SignedKeywordSegment", // [1050]
    "UnsignedKeywordSegment", // [1051]
    "IntegerKeywordSegment", // [1052]
    "INTEGER", // [1053]
    "VARYING", // [1054]
    "VaryingKeywordSegment", // [1055]
    "DotSegment", // [1056]
    "DatatypeIdentifierSegment", // [1057]
    "ZEROFILL", // [1058]
    "CharCharacterSetGrammar", // [1059]
    "ZerofillKeywordSegment", // [1060]
    "ArrayTypeSegment", // [1061]
    "array_type", // [1062]
    "DATE", // [1063]
    "EXTRACT", // [1064]
    "TIMESTAMPADD", // [1065]
    "TIMESTAMPDIFF", // [1066]
    "DatePartFunctionName", // [1067]
    "FunctionContentsGrammar", // [1068]
    "DateKeywordSegment", // [1069]
    "TimeKeywordSegment", // [1070]
    "date_constructor_literal", // [1071]
    "date_part", // [1072]
    "DATETIME_INTERVAL_CODE", // [1073]
    "DATETIME_INTERVAL_PRECISION", // [1074]
    "DAYOFMONTH", // [1075]
    "DAYOFWEEK", // [1076]
    "DAYOFYEAR", // [1077]
    "DAYS", // [1078]
    "DBCC", // [1079]
    "DEALLOCATE", // [1080]
    "DeallocateKeywordSegment", // [1081]
    "PrepareKeywordSegment", // [1082]
    "PREPARE", // [1083]
    "DEC", // [1084]
    "DECIMAL", // [1085]
    "DECLARE", // [1086]
    "DeclareKeywordSegment", // [1087]
    "CursorKeywordSegment", // [1088]
    "EXIT", // [1089]
    "UNDO", // [1090]
    "ContinueKeywordSegment", // [1091]
    "ExitKeywordSegment", // [1092]
    "UndoKeywordSegment", // [1093]
    "HandlerKeywordSegment", // [1094]
    "HANDLER", // [1095]
    "SqlexceptionKeywordSegment", // [1096]
    "SQLEXCEPTION", // [1097]
    "SqlwarningKeywordSegment", // [1098]
    "SQLWARNING", // [1099]
    "FoundKeywordSegment", // [1100]
    "FOUND", // [1101]
    "SQLSTATE", // [1102]
    "SqlstateKeywordSegment", // [1103]
    "ValueKeywordSegment", // [1104]
    "ConditionKeywordSegment", // [1105]
    "DEFAULT_AUTH", // [1106]
    "DEFAULTS", // [1107]
    "DEFERRABLE", // [1108]
    "DEFERRED", // [1109]
    "DEFINED", // [1110]
    "DEFINITION", // [1111]
    "DEGREE", // [1112]
    "DELAYED", // [1113]
    "Low_priorityKeywordSegment", // [1114]
    "LOW_PRIORITY", // [1115]
    "IgnoreKeywordSegment", // [1116]
    "IGNORE", // [1117]
    "DeleteTargetTableSegment", // [1118]
    "delete_target_table", // [1119]
    "DeleteUsingClauseSegment", // [1120]
    "using_clause", // [1121]
    "WhereClauseSegment", // [1122]
    "WHERE", // [1123]
    "where_clause", // [1124]
    "FromClauseSegment", // [1125]
    "from_clause", // [1126]
    "SelectPartitionClauseSegment", // [1127]
    "partition_clause", // [1128]
    "LimitClauseSegment", // [1129]
    "LIMIT", // [1130]
    "limit_clause", // [1131]
    "ReturningClauseSegment", // [1132]
    "RETURNING", // [1133]
    "returning_clause", // [1134]
    "StarSegment", // [1135]
    "FromExpressionSegment", // [1136]
    "from_expression", // [1137]
    "DELETE_DOMAIN_ID", // [1138]
    ";", // [1139]
    "~", // [1140]
    "SemicolonSegment", // [1141]
    "TildeSegment", // [1142]
    "DELIMITER", // [1143]
    "DelimiterKeywordSegment", // [1144]
    "DELIMITERS", // [1145]
    "DENSE_RANK", // [1146]
    "DENY", // [1147]
    "DEPTH", // [1148]
    "DEREF", // [1149]
    "DERIVED", // [1150]
    "DES_KEY_FILE", // [1151]
    "DESCRIBE", // [1152]
    "DescribeKeywordSegment", // [1153]
    "DESCRIPTION", // [1154]
    "DESCRIPTOR", // [1155]
    "DESTROY", // [1156]
    "DESTRUCTOR", // [1157]
    "DIAGNOSTICS", // [1158]
    "DICTIONARY", // [1159]
    "DISCONNECT", // [1160]
    "DISK", // [1161]
    "DISPATCH", // [1162]
    "DISTINCT", // [1163]
    "DISTINCTROW", // [1164]
    "DISTRIBUTED", // [1165]
    "DO_DOMAIN_IDS", // [1166]
    "dot", // [1167]
    "quoted_identifier", // [1168]
    "identifier", // [1169]
    "IdentifierSegment", // [1170]
    "json_path", // [1171]
    "\"", // [1172]
    "quoted_literal", // [1173]
    "DropBehaviorGrammar", // [1174]
    "TemporaryGrammar", // [1175]
    "DUAL", // [1176]
    "naked_identifier", // [1177]
    "DUMMY", // [1178]
    "DUMP", // [1179]
    "DUMPFILE", // [1180]
    "DUPLICATE", // [1181]
    "DYNAMIC", // [1182]
    "DYNAMIC_FUNCTION", // [1183]
    "DYNAMIC_FUNCTION_CODE", // [1184]
    "ELEMENT", // [1185]
    "ELSEIF", // [1186]
    "ELSIF", // [1187]
    "EMPTY", // [1188]
    "StructTypeSegment", // [1189]
    "struct_type", // [1190]
    "EmptyStructLiteralBracketsSegment", // [1191]
    "struct_literal", // [1192]
    "ENCLOSED", // [1193]
    "ENCODING", // [1194]
    "END-EXEC", // [1195]
    "}", // [1196]
    "end_curly_bracket", // [1197]
    "ENGINES", // [1198]
    "ENUM", // [1199]
    "EQUALS", // [1200]
    "RawEqualsSegment", // [1201]
    "ERRLVL", // [1202]
    "ERROR", // [1203]
    "ERRORS", // [1204]
    "ESCAPE", // [1205]
    "ESCAPED", // [1206]
    "EVENTS", // [1207]
    "EXAMINED", // [1208]
    "EXCEPT", // [1209]
    "EXCEPTION", // [1210]
    "EXCLUDE", // [1211]
    "EXCLUDING", // [1212]
    "EXEC", // [1213]
    "EXISTING", // [1214]
    "EXISTS", // [1215]
    "EXP", // [1216]
    "EXPANSION", // [1217]
    "EXPLAIN", // [1218]
    "ExplainKeywordSegment", // [1219]
    "InsertStatementSegment", // [1220]
    "insert_statement", // [1221]
    "UpdateStatementSegment", // [1222]
    "update_statement", // [1223]
    "DeleteStatementSegment", // [1224]
    "delete_statement", // [1225]
    "EXPORT", // [1226]
    "Expression_A_Grammar", // [1227]
    "Tail_Recurse_Expression_A_Grammar", // [1228]
    "LikeExpressionGrammar", // [1229]
    "REGEXP", // [1230]
    "RLIKE", // [1231]
    "InOperatorGrammar", // [1232]
    "QuantifiedComparisonOperatorGrammar", // [1233]
    "IsKeywordSegment", // [1234]
    "IsClauseGrammar", // [1235]
    "IsNullGrammar", // [1236]
    "NotNullGrammar", // [1237]
    "BetweenKeywordSegment", // [1238]
    "Expression_B_Grammar", // [1239]
    "PatternMatchingGrammar", // [1240]
    "PRIOR", // [1241]
    "SignedSegmentGrammar", // [1242]
    "QualifiedNumericLiteralSegment", // [1243]
    "NotOperatorGrammar", // [1244]
    "PriorKeywordSegment", // [1245]
    "Tail_Recurse_Expression_B_Grammar", // [1246]
    "WalrusOperatorSegment", // [1247]
    ":=", // [1248]
    "ExistsKeywordSegment", // [1249]
    "Expression_D_Grammar", // [1250]
    "CaseExpressionSegment", // [1251]
    "case_expression", // [1252]
    "TimeZoneGrammar", // [1253]
    "time_zone_grammar", // [1254]
    "LateralColumnAliasExpressionGrammar", // [1255]
    "LocalAliasSegment", // [1256]
    "local_alias_segment", // [1257]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1258]
    "SingleIdentifierWildcardSegment", // [1259]
    "MapTypeSegment", // [1260]
    "map_type", // [1261]
    "BooleanLiteralGrammar", // [1262]
    "NullLiteralSegment", // [1263]
    "DateTimeLiteralGrammar", // [1264]
    "ListComprehensionGrammar", // [1265]
    "AccessorGrammar", // [1266]
    "ValuesClauseSegment", // [1267]
    "values_clause", // [1268]
    "SelectStatementSegment", // [1269]
    "select_statement", // [1270]
    "TypedStructLiteralSegment", // [1271]
    "typed_struct_literal", // [1272]
    "ArrayExpressionSegment", // [1273]
    "array_expression", // [1274]
    "OverlapsClauseSegment", // [1275]
    "OVERLAPS", // [1276]
    "overlaps_clause", // [1277]
    "EXTENSION", // [1278]
    "EXTENT_SIZE", // [1279]
    "FACTOR", // [1280]
    "boolean_literal", // [1281]
    "LiteralKeywordSegment", // [1282]
    "FAULTS", // [1283]
    "FEDERATED", // [1284]
    "ROWS", // [1285]
    "RowsKeywordSegment", // [1286]
    "TiesKeywordSegment", // [1287]
    "TIES", // [1288]
    "FIELDS", // [1289]
    "DelimiterGrammar", // [1290]
    "FILE_BLOCK_SIZE", // [1291]
    "FILLFACTOR", // [1292]
    "FILTER", // [1293]
    "FilterKeywordSegment", // [1294]
    "WhereKeywordSegment", // [1295]
    "FINAL", // [1296]
    "FINISH", // [1297]
    "FIRST_VALUE", // [1298]
    "FIXED", // [1299]
    "FLOAT4", // [1300]
    "FLOAT8", // [1301]
    "FLOAT", // [1302]
    "FLOOR", // [1303]
    "FLUSH", // [1304]
    "FlushKeywordSegment", // [1305]
    "GENERAL", // [1306]
    "GLOBAL", // [1307]
    "HOSTS", // [1308]
    "INDEX_STATISTICS", // [1309]
    "LOGS", // [1310]
    "MASTER", // [1311]
    "QUERY", // [1312]
    "QUERY_RESPONSE_TIME", // [1313]
    "RELAY", // [1314]
    "RESET", // [1315]
    "SESSION", // [1316]
    "SLOW", // [1317]
    "STATUS", // [1318]
    "TABLE_STATISTICS", // [1319]
    "USER_RESOURCES", // [1320]
    "USER_STATISTICS", // [1321]
    "USER_VARIABLES", // [1322]
    "LogsKeywordSegment", // [1323]
    "EngineKeywordSegment", // [1324]
    "ErrorKeywordSegment", // [1325]
    "GeneralKeywordSegment", // [1326]
    "QueryKeywordSegment", // [1327]
    "SlowKeywordSegment", // [1328]
    "ResetKeywordSegment", // [1329]
    "MasterKeywordSegment", // [1330]
    "GlobalKeywordSegment", // [1331]
    "SessionKeywordSegment", // [1332]
    "StatusKeywordSegment", // [1333]
    "RelayKeywordSegment", // [1334]
    "ChannelKeywordSegment", // [1335]
    "HostsKeywordSegment", // [1336]
    "Changed_page_bitmapsKeywordSegment", // [1337]
    "Client_statisticsKeywordSegment", // [1338]
    "Des_key_fileKeywordSegment", // [1339]
    "Index_statisticsKeywordSegment", // [1340]
    "Query_response_timeKeywordSegment", // [1341]
    "Table_statisticsKeywordSegment", // [1342]
    "User_statisticsKeywordSegment", // [1343]
    "User_variablesKeywordSegment", // [1344]
    "User_resourcesKeywordSegment", // [1345]
    "CheckpointKeywordSegment", // [1346]
    "ExportKeywordSegment", // [1347]
    "FOLLOWING", // [1348]
    "OF", // [1349]
    "OfKeywordSegment", // [1350]
    "SKIP", // [1351]
    "NowaitKeywordSegment", // [1352]
    "SkipKeywordSegment", // [1353]
    "LockedKeywordSegment", // [1354]
    "LOCKED", // [1355]
    "ModeKeywordSegment", // [1356]
    "MODE", // [1357]
    "FORCE", // [1358]
    "FORTRAN", // [1359]
    "FORWARD", // [1360]
    "FrameClauseUnitGrammar", // [1361]
    "PRECEDING", // [1362]
    "PrecedingKeywordSegment", // [1363]
    "FollowingKeywordSegment", // [1364]
    "FREE", // [1365]
    "FREETEXT", // [1366]
    "FREETEXTTABLE", // [1367]
    "FREEZE", // [1368]
    "HAVING", // [1369]
    "INTERSECT", // [1370]
    "MINUS", // [1371]
    "OFFSET", // [1372]
    "QUALIFY", // [1373]
    "WINDOW", // [1374]
    "OrderKeywordSegment", // [1375]
    "HavingKeywordSegment", // [1376]
    "QualifyKeywordSegment", // [1377]
    "WindowKeywordSegment", // [1378]
    "SetOperatorSegment", // [1379]
    "set_operator", // [1380]
    "WithNoSchemaBindingClauseSegment", // [1381]
    "with_no_schema_binding_clause", // [1382]
    "WithDataClauseSegment", // [1383]
    "with_data_clause", // [1384]
    "OffsetKeywordSegment", // [1385]
    "ForClauseSegment", // [1386]
    "for_clause", // [1387]
    "IntoClauseSegment", // [1388]
    "into_clause", // [1389]
    "PreTableFunctionKeywordsGrammar", // [1390]
    "TableExpressionSegment", // [1391]
    "table_expression", // [1392]
    "TemporalQuerySegment", // [1393]
    "temporal_query", // [1394]
    "FromClauseTerminatorGrammar", // [1395]
    "SamplingExpressionSegment", // [1396]
    "TABLESAMPLE", // [1397]
    "sample_expression", // [1398]
    "JoinLikeClauseGrammar", // [1399]
    "JoinClauseSegment", // [1400]
    "join_clause", // [1401]
    "PostTableExpressionGrammar", // [1402]
    "MLTableExpressionSegment", // [1403]
    "ML", // [1404]
    "ml_table_expression", // [1405]
    "FromExpressionElementSegment", // [1406]
    "from_expression_element", // [1407]
    "indented_joins", // [1408]
    "LEADING", // [1409]
    "TRAILING", // [1410]
    "TrimParametersGrammar", // [1411]
    "DistinctKeywordSegment", // [1412]
    "FunctionContentsExpressionGrammar", // [1413]
    "AggregateOrderByClause", // [1414]
    "aggregate_order_by", // [1415]
    "SEPARATOR", // [1416]
    "SeparatorKeywordSegment", // [1417]
    "IgnoreRespectNullsGrammar", // [1418]
    "RESPECT", // [1419]
    "IndexColumnDefinitionSegment", // [1420]
    "index_column_definition", // [1421]
    "EmptyStructLiteralSegment", // [1422]
    "TransactionStatementSegment", // [1423]
    "transaction_statement", // [1424]
    "WordSegment", // [1425]
    "FunctionNameIdentifierSegment", // [1426]
    "AnyKeywordSegment", // [1427]
    "FunctionParameterGrammar", // [1428]
    "JSON_TABLE", // [1429]
    "JsonTableFunctionNameSegment", // [1430]
    "JsonTableFunctionContentsSegment", // [1431]
    "JSON_VALUE", // [1432]
    "JsonValueFunctionNameSegment", // [1433]
    "JsonValueFunctionContentsSegment", // [1434]
    "ConvertFunctionNameSegment", // [1435]
    "ConvertFunctionContentsSegment", // [1436]
    "DatePartFunctionNameSegment", // [1437]
    "DateTimeFunctionContentsSegment", // [1438]
    "ColumnsExpressionGrammar", // [1439]
    "ColumnsExpressionFunctionNameSegment", // [1440]
    "FunctionContentsSegment", // [1441]
    "PostFunctionGrammar", // [1442]
    "FUSION", // [1443]
    "G", // [1444]
    "GEOMCOLLECTION", // [1445]
    "GEOMETRY", // [1446]
    "GEOMETRYCOLLECTION", // [1447]
    "GET", // [1448]
    "GetKeywordSegment", // [1449]
    "StackedKeywordSegment", // [1450]
    "STACKED", // [1451]
    "DiagnosticsKeywordSegment", // [1452]
    "NUMBER", // [1453]
    "ROW_COUNT", // [1454]
    "NumberKeywordSegment", // [1455]
    "Row_countKeywordSegment", // [1456]
    "MESSAGE_TEXT", // [1457]
    "MYSQL_ERRNO", // [1458]
    "RETURNED_SQLSTATE", // [1459]
    "SCHEMA_NAME", // [1460]
    "SUBCLASS_ORIGIN", // [1461]
    "TABLE_NAME", // [1462]
    "Class_originKeywordSegment", // [1463]
    "Subclass_originKeywordSegment", // [1464]
    "Returned_sqlstateKeywordSegment", // [1465]
    "Message_textKeywordSegment", // [1466]
    "Mysql_errnoKeywordSegment", // [1467]
    "Constraint_catalogKeywordSegment", // [1468]
    "Constraint_schemaKeywordSegment", // [1469]
    "Constraint_nameKeywordSegment", // [1470]
    "Catalog_nameKeywordSegment", // [1471]
    "Schema_nameKeywordSegment", // [1472]
    "Table_nameKeywordSegment", // [1473]
    "Column_nameKeywordSegment", // [1474]
    "Cursor_nameKeywordSegment", // [1475]
    "GET_FORMAT", // [1476]
    "GET_MASTER_PUBLIC_KEY", // [1477]
    "GET_SOURCE_PUBLIC_KEY", // [1478]
    "glob_operator", // [1479]
    "GO", // [1480]
    "GOTO", // [1481]
    "GrantKeywordSegment", // [1482]
    "AccessPermissionsSegment", // [1483]
    "access_permissions", // [1484]
    "AccessObjectSegment", // [1485]
    "access_object", // [1486]
    "OptionKeywordSegment", // [1487]
    "OPTION", // [1488]
    "AdminKeywordSegment", // [1489]
    "GRANTED", // [1490]
    "GREATEST", // [1491]
    "GroupByClauseTerminatorGrammar", // [1492]
    "WithRollupClauseSegment", // [1493]
    "with_rollup_clause", // [1494]
    "GROUP_REPLICATION", // [1495]
    "GROUPING", // [1496]
    "GroupingKeywordSegment", // [1497]
    "SetsKeywordSegment", // [1498]
    "SETS", // [1499]
    "CubeRollupClauseSegment", // [1500]
    "cube_rollup_clause", // [1501]
    "GROUPS", // [1502]
    "GTID_ONLY", // [1503]
    "HARD", // [1504]
    "HEADER", // [1505]
    "HEAP", // [1506]
    "HELP", // [1507]
    "HelpKeywordSegment", // [1508]
    "HIERARCHY", // [1509]
    "HIGH_PRIORITY", // [1510]
    "HOLD", // [1511]
    "HOLDLOCK", // [1512]
    "HOST", // [1513]
    "ID", // [1514]
    "IDENTITY", // [1515]
    "IDENTITY_INSERT", // [1516]
    "IDENTITYCOL", // [1517]
    "IfKeywordSegment", // [1518]
    "ThenKeywordSegment", // [1519]
    "THEN", // [1520]
    "ElseifKeywordSegment", // [1521]
    "RespectKeywordSegment", // [1522]
    "NullsKeywordSegment", // [1523]
    "NULLS", // [1524]
    "IGNORE_DOMAIN_IDS", // [1525]
    "IGNORE_SERVER_IDS", // [1526]
    "ILIKE", // [1527]
    "IMMEDIATE", // [1528]
    "IMMUTABLE", // [1529]
    "IMPLEMENTATION", // [1530]
    "IMPLICIT", // [1531]
    "INACTIVE", // [1532]
    "INCLUDE", // [1533]
    "INCLUDING", // [1534]
    "ForceKeywordSegment", // [1535]
    "JOIN", // [1536]
    "JoinKeywordSegment", // [1537]
    "JoinOnConditionSegment", // [1538]
    "join_on_condition", // [1539]
    "Key_block_sizeKeywordSegment", // [1540]
    "ParserKeywordSegment", // [1541]
    "PARSER", // [1542]
    "ClusteringKeywordSegment", // [1543]
    "YES", // [1544]
    "YesKeywordSegment", // [1545]
    "Engine_attributeKeywordSegment", // [1546]
    "Secondary_engine_attributeKeywordSegment", // [1547]
    "BtreeKeywordSegment", // [1548]
    "RTREE", // [1549]
    "RtreeKeywordSegment", // [1550]
    "INDEXES", // [1551]
    "INDICATOR", // [1552]
    "INFILE", // [1553]
    "INFIX", // [1554]
    "INHERIT", // [1555]
    "INHERITS", // [1556]
    "INITIAL_SIZE", // [1557]
    "INITIALIZE", // [1558]
    "INITIALLY", // [1559]
    "INITIATE", // [1560]
    "INOUT", // [1561]
    "INPUT", // [1562]
    "parameter_direction", // [1563]
    "INSENSITIVE", // [1564]
    "DelayedKeywordSegment", // [1565]
    "High_priorityKeywordSegment", // [1566]
    "SetClauseListSegment", // [1567]
    "set_clause_list", // [1568]
    "InsertRowAliasSegment", // [1569]
    "insert_row_alias", // [1570]
    "UpsertClauseListSegment", // [1571]
    "upsert_clause_list", // [1572]
    "INSERT_ID", // [1573]
    "INSTALL", // [1574]
    "INSTANTIABLE", // [1575]
    "INSTEAD", // [1576]
    "INT1", // [1577]
    "INT2", // [1578]
    "INT3", // [1579]
    "INT4", // [1580]
    "INT8", // [1581]
    "INT", // [1582]
    "INTEGRATIONS", // [1583]
    "INTERSECTION", // [1584]
    "DumpfileKeywordSegment", // [1585]
    "OUTFILE", // [1586]
    "OutfileKeywordSegment", // [1587]
    "FieldsKeywordSegment", // [1588]
    "ColumnsKeywordSegment", // [1589]
    "TERMINATED", // [1590]
    "TerminatedKeywordSegment", // [1591]
    "OPTIONALLY", // [1592]
    "OptionallyKeywordSegment", // [1593]
    "EnclosedKeywordSegment", // [1594]
    "EscapedKeywordSegment", // [1595]
    "LINES", // [1596]
    "LinesKeywordSegment", // [1597]
    "STARTING", // [1598]
    "StartingKeywordSegment", // [1599]
    "SelectClauseTerminatorGrammar", // [1600]
    "IO", // [1601]
    "IO_THREAD", // [1602]
    "IPC", // [1603]
    "NanLiteralSegment", // [1604]
    "NAN", // [1605]
    "UnknownLiteralSegment", // [1606]
    "NormalizedGrammar", // [1607]
    "ISAM", // [1608]
    "ISNULL", // [1609]
    "ISOLATION", // [1610]
    "ISOPEN", // [1611]
    "ITERATE", // [1612]
    "IterateKeywordSegment", // [1613]
    "ConditionalJoinKeywordsGrammar", // [1614]
    "JoinKeywordsGrammar", // [1615]
    "NestedJoinGrammar", // [1616]
    "indented_using_on", // [1617]
    "MatchConditionSegment", // [1618]
    "match_condition", // [1619]
    "JoinUsingConditionGrammar", // [1620]
    "UnconditionalJoinKeywordsGrammar", // [1621]
    "ExtendedNaturalJoinKeywordsGrammar", // [1622]
    "indented_on_contents", // [1623]
    "InnerKeywordSegment", // [1624]
    "FullKeywordSegment", // [1625]
    "LeftKeywordSegment", // [1626]
    "RightKeywordSegment", // [1627]
    "OuterKeywordSegment", // [1628]
    "OUTER", // [1629]
    "JSON", // [1630]
    "OrdinalityKeywordSegment", // [1631]
    "ORDINALITY", // [1632]
    "NESTED", // [1633]
    "NestedKeywordSegment", // [1634]
    "PathKeywordSegment", // [1635]
    "PATH", // [1636]
    "JsonTableColumnsClauseSegment", // [1637]
    "json_table_columns_clause", // [1638]
    "EmptyKeywordSegment", // [1639]
    "ReturningKeywordSegment", // [1640]
    "K", // [1641]
    "KEY_MEMBER", // [1642]
    "KEY_TYPE", // [1643]
    "KEYRING", // [1644]
    "KILL", // [1645]
    "LAG", // [1646]
    "LANCOMPILER", // [1647]
    "LAST", // [1648]
    "LAST_INSERT_ID", // [1649]
    "LAST_VALUE", // [1650]
    "LASTVAL", // [1651]
    "LATERAL", // [1652]
    "LEAD", // [1653]
    "LEAST", // [1654]
    "LEAVE", // [1655]
    "LEAVES", // [1656]
    "LENGTH", // [1657]
    "LEVEL", // [1658]
    "LikeGrammar", // [1659]
    "EscapeKeywordSegment", // [1660]
    "RlikeKeywordSegment", // [1661]
    "RegexpKeywordSegment", // [1662]
    "LimitKeywordSegment", // [1663]
    "LINENO", // [1664]
    "LINESTRING", // [1665]
    "LISTEN", // [1666]
    "ArrayLiteralSegment", // [1667]
    "array_literal", // [1668]
    "TypedArrayLiteralSegment", // [1669]
    "typed_array_literal", // [1670]
    "ObjectLiteralSegment", // [1671]
    "{", // [1672]
    "object_literal", // [1673]
    "DoubleQuotedLiteralSegment", // [1674]
    "SystemVariableSegment", // [1675]
    "LN", // [1676]
    "LOAD", // [1677]
    "LoadKeywordSegment", // [1678]
    "ConcurrentKeywordSegment", // [1679]
    "InfileKeywordSegment", // [1680]
    "REPLACE", // [1681]
    "ReplaceKeywordSegment", // [1682]
    "variable", // [1683]
    "LOCATION", // [1684]
    "LOCATOR", // [1685]
    "LOCKS", // [1686]
    "LOGFILE", // [1687]
    "LOGIN", // [1688]
    "LONG", // [1689]
    "LONGBLOB", // [1690]
    "LONGTEXT", // [1691]
    "LOOP", // [1692]
    "ColonSegment", // [1693]
    "LoopKeywordSegment", // [1694]
    "LOWER", // [1695]
    "M", // [1696]
    "MlKeywordSegment", // [1697]
    "MAP", // [1698]
    "MASTER_AUTO_POSITION", // [1699]
    "MASTER_COMPRESSION_ALGORITHMS", // [1700]
    "MASTER_CONNECT_RETRY", // [1701]
    "MASTER_DELAY", // [1702]
    "MASTER_DEMOTE_TO_SLAVE", // [1703]
    "MASTER_GTID_POS", // [1704]
    "MASTER_HEARTBEAT_PERIOD", // [1705]
    "MASTER_HOST", // [1706]
    "MASTER_LOG_FILE", // [1707]
    "MASTER_LOG_POS", // [1708]
    "MASTER_PASSWORD", // [1709]
    "MASTER_PORT", // [1710]
    "MASTER_PUBLIC_KEY_PATH", // [1711]
    "MASTER_RETRY_COUNT", // [1712]
    "MASTER_SERVER_ID", // [1713]
    "MASTER_SSL", // [1714]
    "MASTER_SSL_CA", // [1715]
    "MASTER_SSL_CAPATH", // [1716]
    "MASTER_SSL_CERT", // [1717]
    "MASTER_SSL_CIPHER", // [1718]
    "MASTER_SSL_CRL", // [1719]
    "MASTER_SSL_CRLPATH", // [1720]
    "MASTER_SSL_KEY", // [1721]
    "MASTER_SSL_VERIFY_SERVER_CERT", // [1722]
    "MASTER_TLS_CIPHERSUITES", // [1723]
    "MASTER_TLS_VERSION", // [1724]
    "MASTER_USE_GTID", // [1725]
    "MASTER_USER", // [1726]
    "MASTER_ZSTD_COMPRESSION_LEVEL", // [1727]
    "MATCH", // [1728]
    "MATCHED", // [1729]
    "MAX", // [1730]
    "MAX_SIZE", // [1731]
    "MAX_STATEMENT_TIME", // [1732]
    "MAXEXTENTS", // [1733]
    "MEDIUMBLOB", // [1734]
    "MEDIUMINT", // [1735]
    "MEDIUMTEXT", // [1736]
    "MEMBER", // [1737]
    "MEMORY", // [1738]
    "MergeMatchedClauseSegment", // [1739]
    "merge_when_matched_clause", // [1740]
    "MergeNotMatchedClauseSegment", // [1741]
    "merge_when_not_matched_clause", // [1742]
    "WhenKeywordSegment", // [1743]
    "MatchedKeywordSegment", // [1744]
    "MergeUpdateClauseSegment", // [1745]
    "merge_update_clause", // [1746]
    "MergeDeleteClauseSegment", // [1747]
    "merge_delete_clause", // [1748]
    "MergeInsertClauseSegment", // [1749]
    "merge_insert_clause", // [1750]
    "MergeIntoLiteralGrammar", // [1751]
    "AliasedTableReferenceGrammar", // [1752]
    "MergeMatchSegment", // [1753]
    "merge_match", // [1754]
    "MESSAGE_LENGTH", // [1755]
    "MESSAGE_OCTET_LENGTH", // [1756]
    "MIDDLEINT", // [1757]
    "MIGRATE", // [1758]
    "MILLISECOND", // [1759]
    "MIN", // [1760]
    "MLSLABEL", // [1761]
    "MODULE", // [1762]
    "MONTHNAME", // [1763]
    "MORE", // [1764]
    "MOVE", // [1765]
    "MULTILINESTRING", // [1766]
    "MULTIPOINT", // [1767]
    "MULTIPOLYGON", // [1768]
    "MULTISET", // [1769]
    "MUMPS", // [1770]
    "MUTEX", // [1771]
    "MYISAM", // [1772]
    "MYSQL", // [1773]
    "NAME", // [1774]
    "WindowSpecificationSegment", // [1775]
    "window_specification", // [1776]
    "NamedWindowExpressionSegment", // [1777]
    "named_window_expression", // [1778]
    "NAMES", // [1779]
    "null_literal", // [1780]
    "NATIONAL", // [1781]
    "NATURAL", // [1782]
    "NaturalKeywordSegment", // [1783]
    "NCHAR", // [1784]
    "NCLOB", // [1785]
    "NDB", // [1786]
    "NDBCLUSTER", // [1787]
    "sign_indicator", // [1788]
    "NESTING", // [1789]
    "NETWORK_NAMESPACE", // [1790]
    "NEW", // [1791]
    "newline", // [1792]
    "NewlineSegment", // [1793]
    "NEXTVAL", // [1794]
    "NO_WAIT", // [1795]
    "NOAUDIT", // [1796]
    "NOCHECK", // [1797]
    "NOCOMPRESS", // [1798]
    "NOCREATEDB", // [1799]
    "NOCREATEROLE", // [1800]
    "NOCREATEUSER", // [1801]
    "NODEGROUP", // [1802]
    "NOINHERIT", // [1803]
    "NOLOGIN", // [1804]
    "NOMAXVALUE", // [1805]
    "NOMINVALUE", // [1806]
    "UnorderedSelectStatementSegment", // [1807]
    "WithCompoundStatementSegment", // [1808]
    "with_compound_statement", // [1809]
    "NonSetSelectableGrammar", // [1810]
    "BracketedSetExpressionGrammar", // [1811]
    "MergeStatementSegment", // [1812]
    "merge_statement", // [1813]
    "SetExpressionSegment", // [1814]
    "NONCLUSTERED", // [1815]
    "NORMALIZE", // [1816]
    "NORMALIZED", // [1817]
    "NOSUPERUSER", // [1818]
    "RawNotSegment", // [1819]
    "not_operator", // [1820]
    "NOTFOUND", // [1821]
    "NOTHING", // [1822]
    "NOTIFY", // [1823]
    "NOTNULL", // [1824]
    "NTH_VALUE", // [1825]
    "NTILE", // [1826]
    "NULLABLE", // [1827]
    "NULLIF", // [1828]
    "NUMERIC", // [1829]
    "NVARCHAR", // [1830]
    "ObjectLiteralElementSegment", // [1831]
    "object_literal_element", // [1832]
    "start_curly_bracket", // [1833]
    "CastOperatorSegment", // [1834]
    "StartSquareBracketSegment", // [1835]
    "StartBracketSegment", // [1836]
    "OBJECTS", // [1837]
    "OCTET_LENGTH", // [1838]
    "OCTETS", // [1839]
    "OFFLINE", // [1840]
    "OFFSETS", // [1841]
    "OIDS", // [1842]
    "OJ", // [1843]
    "OLD", // [1844]
    "OLD_PASSWORD", // [1845]
    "ONE", // [1846]
    "ONLINE", // [1847]
    "OPENDATASOURCE", // [1848]
    "OPENQUERY", // [1849]
    "OPENROWSET", // [1850]
    "OPENXML", // [1851]
    "OPERATION", // [1852]
    "OPERATOR", // [1853]
    "OrKeywordSegment", // [1854]
    "LastKeywordSegment", // [1855]
    "WithFillSegment", // [1856]
    "with_fill", // [1857]
    "NoorderKeywordSegment", // [1858]
    "ORDERING", // [1859]
    "ORGANIZATION", // [1860]
    "OTHERS", // [1861]
    "OUT", // [1862]
    "OUTPUT", // [1863]
    "OverKeywordSegment", // [1864]
    "OVER", // [1865]
    "OverlapsKeywordSegment", // [1866]
    "OVERLAY", // [1867]
    "OVERRIDING", // [1868]
    "OVERWRITE", // [1869]
    "OWNER", // [1870]
    "PACKAGE", // [1871]
    "PAD", // [1872]
    "PAGE", // [1873]
    "PARAMETER", // [1874]
    "parameter", // [1875]
    "?", // [1876]
    "PARAMETER_MODE", // [1877]
    "PARAMETER_NAME", // [1878]
    "PARAMETER_ORDINAL_POSITION", // [1879]
    "PARAMETER_SPECIFIC_CATALOG", // [1880]
    "PARAMETER_SPECIFIC_NAME", // [1881]
    "PARAMETER_SPECIFIC_SCHEMA", // [1882]
    "PARAMETERS", // [1883]
    "PARSE_GCOL_EXPR", // [1884]
    "PARSE_VCOL_EXPR", // [1885]
    "PARTIAL", // [1886]
    "PASCAL", // [1887]
    "SlashSegment", // [1888]
    "path_segment", // [1889]
    "PCTFREE", // [1890]
    "PERCENT", // [1891]
    "PERCENT_RANK", // [1892]
    "PERCENTILE_CONT", // [1893]
    "PERCENTILE_DISC", // [1894]
    "PERIOD", // [1895]
    "PERSIST", // [1896]
    "PERSIST_ONLY", // [1897]
    "PHASE", // [1898]
    "pipe", // [1899]
    "PLACING", // [1900]
    "PLAN", // [1901]
    "PLI", // [1902]
    "PLUGIN", // [1903]
    "PLUGIN_DIR", // [1904]
    "PLUGINS", // [1905]
    "POINT", // [1906]
    "POLYGON", // [1907]
    "PORT", // [1908]
    "PORTION", // [1909]
    "POSITION", // [1910]
    "OverClauseSegment", // [1911]
    "over_clause", // [1912]
    "FilterClauseGrammar", // [1913]
    "IndexHintClauseSegment", // [1914]
    "index_hint_clause", // [1915]
    "POSTFIX", // [1916]
    "POWER", // [1917]
    "PREFIX", // [1918]
    "PREORDER", // [1919]
    "PREPARED", // [1920]
    "PREV", // [1921]
    "PREVIOUS", // [1922]
    "PrimaryKeywordSegment", // [1923]
    "PRINT", // [1924]
    "PRIVILEGE_CHECKS_USER", // [1925]
    "PROC", // [1926]
    "PROCEDURAL", // [1927]
    "OutputParameterSegment", // [1928]
    "InputParameterSegment", // [1929]
    "InputOutputParameterSegment", // [1930]
    "ProcedureParameterGrammar", // [1931]
    "PROCESS", // [1932]
    "PROCESSLIST", // [1933]
    "PROFILE", // [1934]
    "PROFILES", // [1935]
    "PROXY", // [1936]
    "PURGE", // [1937]
    "PurgeKeywordSegment", // [1938]
    "SOME", // [1939]
    "SomeKeywordSegment", // [1940]
    "QUOTE", // [1941]
    "`", // [1942]
    "RAID0", // [1943]
    "RAISE", // [1944]
    "RAISERROR", // [1945]
    "RANK", // [1946]
    "raw_comparison_operator", // [1947]
    "RAW", // [1948]
    "READ_ONLY", // [1949]
    "READ_WRITE", // [1950]
    "READTEXT", // [1951]
    "REAL", // [1952]
    "RECHECK", // [1953]
    "RECONFIGURE", // [1954]
    "RECOVER", // [1955]
    "RECURSIVE", // [1956]
    "REDO_BUFFER_SIZE", // [1957]
    "REDOFILE", // [1958]
    "REDUNDANT", // [1959]
    "REF", // [1960]
    "REF_SYSTEM_ID", // [1961]
    "ReferenceMatchGrammar", // [1962]
    "ReferentialActionGrammar", // [1963]
    "REFERENCE", // [1964]
    "MatchKeywordSegment", // [1965]
    "SIMPLE", // [1966]
    "PartialKeywordSegment", // [1967]
    "SimpleKeywordSegment", // [1968]
    "REFERENCING", // [1969]
    "REGISTRATION", // [1970]
    "REGR_AVGX", // [1971]
    "REGR_AVGY", // [1972]
    "REGR_COUNT", // [1973]
    "REGR_INTERCEPT", // [1974]
    "REGR_R2", // [1975]
    "REGR_SLOPE", // [1976]
    "REGR_SXX", // [1977]
    "REGR_SXY", // [1978]
    "REGR_SYY", // [1979]
    "REINDEX", // [1980]
    "RELATIVE", // [1981]
    "RELAY_LOG_FILE", // [1982]
    "RELAY_LOG_POS", // [1983]
    "RELAY_THREAD", // [1984]
    "RELAYLOG", // [1985]
    "RELEASE", // [1986]
    "RELOAD", // [1987]
    "REMOTE", // [1988]
    "USE_FRM", // [1989]
    "Use_frmKeywordSegment", // [1990]
    "REPEAT", // [1991]
    "RepeatKeywordSegment", // [1992]
    "UNTIL", // [1993]
    "UntilKeywordSegment", // [1994]
    "REPEATABLE", // [1995]
    "REPLAY", // [1996]
    "REPLICA_POS", // [1997]
    "REPLICAS", // [1998]
    "REPLICATE_DO_DB", // [1999]
    "REPLICATE_DO_TABLE", // [2000]
    "REPLICATE_IGNORE_DB", // [2001]
    "REPLICATE_IGNORE_TABLE", // [2002]
    "REPLICATE_REWRITE_DB", // [2003]
    "REPLICATE_WILD_DO_TABLE", // [2004]
    "REPLICATE_WILD_IGNORE_TABLE", // [2005]
    "REPLICATION", // [2006]
    "REQUIRE_ROW_FORMAT", // [2007]
    "RESIGNAL", // [2008]
    "SIGNAL", // [2009]
    "SignalKeywordSegment", // [2010]
    "ResignalKeywordSegment", // [2011]
    "RESTART", // [2012]
    "RESTORE", // [2013]
    "RESULT", // [2014]
    "RESUME", // [2015]
    "RETAIN", // [2016]
    "RETURN", // [2017]
    "ReturnKeywordSegment", // [2018]
    "RETURNED_CARDINALITY", // [2019]
    "RETURNED_LENGTH", // [2020]
    "RETURNED_OCTET_LENGTH", // [2021]
    "SelectClauseElementSegment", // [2022]
    "select_clause_element", // [2023]
    "REVERSE", // [2024]
    "RevokeKeywordSegment", // [2025]
    "AccessTargetSegment", // [2026]
    "access_target", // [2027]
    "AtSignLiteralSegment", // [2028]
    "Current_userKeywordSegment", // [2029]
    "ROLES", // [2030]
    "ROLLBACK", // [2031]
    "ROTATE", // [2032]
    "ROUTINE_CATALOG", // [2033]
    "ROUTINE_NAME", // [2034]
    "ROUTINE_SCHEMA", // [2035]
    "ROW_NUMBER", // [2036]
    "ROWCOUNT", // [2037]
    "ROWGUIDCOL", // [2038]
    "ROWID", // [2039]
    "ROWNUM", // [2040]
    "ROWTYPE", // [2041]
    "RULE", // [2042]
    "TablesampleKeywordSegment", // [2043]
    "BernoulliKeywordSegment", // [2044]
    "RepeatableKeywordSegment", // [2045]
    "SAVE", // [2046]
    "SAVEPOINT", // [2047]
    "SCALE", // [2048]
    "SCOPE", // [2049]
    "SCOPE_CATALOG", // [2050]
    "SCOPE_NAME", // [2051]
    "SCOPE_SCHEMA", // [2052]
    "SCROLL", // [2053]
    "SEARCH", // [2054]
    "SECONDARY", // [2055]
    "SECONDARY_ENGINE", // [2056]
    "SECONDARY_LOAD", // [2057]
    "SECONDARY_UNLOAD", // [2058]
    "SECTION", // [2059]
    "WildcardExpressionSegment", // [2060]
    "wildcard_expression", // [2061]
    "SQL_BIG_RESULT", // [2062]
    "SQL_BUFFER_RESULT", // [2063]
    "SQL_CACHE", // [2064]
    "SQL_CALC_FOUND_ROWS", // [2065]
    "SQL_NO_CACHE", // [2066]
    "SQL_SMALL_RESULT", // [2067]
    "STRAIGHT_JOIN", // [2068]
    "DistinctrowKeywordSegment", // [2069]
    "Straight_joinKeywordSegment", // [2070]
    "Sql_small_resultKeywordSegment", // [2071]
    "Sql_big_resultKeywordSegment", // [2072]
    "Sql_buffer_resultKeywordSegment", // [2073]
    "Sql_cacheKeywordSegment", // [2074]
    "Sql_no_cacheKeywordSegment", // [2075]
    "Sql_calc_found_rowsKeywordSegment", // [2076]
    "SelectClauseModifierSegment", // [2077]
    "select_clause_modifier", // [2078]
    "SelectClauseSegment", // [2079]
    "select_clause", // [2080]
    "GroupByClauseSegment", // [2081]
    "groupby_clause", // [2082]
    "HavingClauseSegment", // [2083]
    "having_clause", // [2084]
    "NamedWindowSegment", // [2085]
    "named_window", // [2086]
    "WithCompoundNonSelectStatementSegment", // [2087]
    "NonWithSelectableGrammar", // [2088]
    "SELF", // [2089]
    "statement_terminator", // [2090]
    "SENSITIVE", // [2091]
    "MinvalueKeywordSegment", // [2092]
    "SERIAL", // [2093]
    "SERIALIZABLE", // [2094]
    "SERVER_NAME", // [2095]
    "SESSION_USER", // [2096]
    "NewKeywordSegment", // [2097]
    "OldKeywordSegment", // [2098]
    "PersistKeywordSegment", // [2099]
    "Persist_onlyKeywordSegment", // [2100]
    "BooleanDynamicSystemVariablesGrammar", // [2101]
    "SetClauseSegment", // [2102]
    "set_clause", // [2103]
    "NamesKeywordSegment", // [2104]
    "UnionGrammar", // [2105]
    "IntersectKeywordSegment", // [2106]
    "ExceptKeywordSegment", // [2107]
    "MinusKeywordSegment", // [2108]
    "TransactionKeywordSegment", // [2109]
    "TRANSACTION", // [2110]
    "IsolationKeywordSegment", // [2111]
    "LevelKeywordSegment", // [2112]
    "UNCOMMITTED", // [2113]
    "CommittedKeywordSegment", // [2114]
    "UncommittedKeywordSegment", // [2115]
    "SerializableKeywordSegment", // [2116]
    "SETOF", // [2117]
    "SETUSER", // [2118]
    "SETVAL", // [2119]
    "SHARES", // [2120]
    "SHOW", // [2121]
    "SHUTDOWN", // [2122]
    "PositiveSegment", // [2123]
    "NegativeSegment", // [2124]
    "SIMILAR", // [2125]
    "'", // [2126]
    "SIZE", // [2127]
    "slash", // [2128]
    "SLAVE_POS", // [2129]
    "SLAVES", // [2130]
    "slice", // [2131]
    "SMALLINT", // [2132]
    "SNAPSHOT", // [2133]
    "SOCKET", // [2134]
    "SOFT", // [2135]
    "SONAME", // [2136]
    "SOUNDS", // [2137]
    "SOURCE", // [2138]
    "SOURCE_AUTO_POSITION", // [2139]
    "SOURCE_BIND", // [2140]
    "SOURCE_COMPRESSION_ALGORITHMS", // [2141]
    "SOURCE_CONNECT_RETRY", // [2142]
    "SOURCE_DELAY", // [2143]
    "SOURCE_HEARTBEAT_PERIOD", // [2144]
    "SOURCE_HOST", // [2145]
    "SOURCE_LOG_FILE", // [2146]
    "SOURCE_LOG_POS", // [2147]
    "SOURCE_PASSWORD", // [2148]
    "SOURCE_PORT", // [2149]
    "SOURCE_PUBLIC_KEY_PATH", // [2150]
    "SOURCE_RETRY_COUNT", // [2151]
    "SOURCE_SSL", // [2152]
    "SOURCE_SSL_CA", // [2153]
    "SOURCE_SSL_CAPATH", // [2154]
    "SOURCE_SSL_CERT", // [2155]
    "SOURCE_SSL_CIPHER", // [2156]
    "SOURCE_SSL_CRL", // [2157]
    "SOURCE_SSL_CRLPATH", // [2158]
    "SOURCE_SSL_KEY", // [2159]
    "SOURCE_SSL_VERIFY_SERVER_CERT", // [2160]
    "SOURCE_TLS_CIPHERSUITES", // [2161]
    "SOURCE_TLS_VERSION", // [2162]
    "SOURCE_USER", // [2163]
    "SOURCE_ZSTD_COMPRESSION_LEVEL", // [2164]
    "SPACE", // [2165]
    "SPECIFIC_NAME", // [2166]
    "SPECIFICTYPE", // [2167]
    "SQL_AFTER_GTIDS", // [2168]
    "SQL_AFTER_MTS_GAPS", // [2169]
    "SQL_BEFORE_GTIDS", // [2170]
    "SQL_BIG_SELECTS", // [2171]
    "SQL_BIG_TABLES", // [2172]
    "SQL_LOG_OFF", // [2173]
    "SQL_LOG_UPDATE", // [2174]
    "SQL_LOW_PRIORITY_UPDATES", // [2175]
    "SQL_SELECT_LIMIT", // [2176]
    "SQL_THREAD", // [2177]
    "SQL_TSI_DAY", // [2178]
    "SQL_TSI_HOUR", // [2179]
    "SQL_TSI_MINUTE", // [2180]
    "SQL_TSI_MONTH", // [2181]
    "SQL_TSI_QUARTER", // [2182]
    "SQL_TSI_SECOND", // [2183]
    "SQL_TSI_WEEK", // [2184]
    "SQL_TSI_YEAR", // [2185]
    "SQL_WARNINGS", // [2186]
    "SQLCA", // [2187]
    "SQLCODE", // [2188]
    "SQLERROR", // [2189]
    "SQRT", // [2190]
    "STABLE", // [2191]
    "star", // [2192]
    "STATE", // [2193]
    "STATEMENT", // [2194]
    "DropTableStatementSegment", // [2195]
    "drop_table_statement", // [2196]
    "DropViewStatementSegment", // [2197]
    "drop_view_statement", // [2198]
    "CreateUserStatementSegment", // [2199]
    "create_user_statement", // [2200]
    "DropUserStatementSegment", // [2201]
    "drop_user_statement", // [2202]
    "TruncateStatementSegment", // [2203]
    "truncate_table", // [2204]
    "AccessStatementSegment", // [2205]
    "access_statement", // [2206]
    "CreateTableStatementSegment", // [2207]
    "create_table_statement", // [2208]
    "CreateRoleStatementSegment", // [2209]
    "create_role_statement", // [2210]
    "DropRoleStatementSegment", // [2211]
    "drop_role_statement", // [2212]
    "AlterTableStatementSegment", // [2213]
    "alter_table_statement", // [2214]
    "SetSchemaStatementSegment", // [2215]
    "set_schema_statement", // [2216]
    "DropSchemaStatementSegment", // [2217]
    "drop_schema_statement", // [2218]
    "DropTypeStatementSegment", // [2219]
    "drop_type_statement", // [2220]
    "CreateDatabaseStatementSegment", // [2221]
    "create_database_statement", // [2222]
    "DropDatabaseStatementSegment", // [2223]
    "drop_database_statement", // [2224]
    "CreateIndexStatementSegment", // [2225]
    "create_index_statement", // [2226]
    "DropIndexStatementSegment", // [2227]
    "drop_index_statement", // [2228]
    "CreateViewStatementSegment", // [2229]
    "create_view_statement", // [2230]
    "CreateCastStatementSegment", // [2231]
    "create_cast_statement", // [2232]
    "DropCastStatementSegment", // [2233]
    "drop_cast_statement", // [2234]
    "CreateFunctionStatementSegment", // [2235]
    "create_function_statement", // [2236]
    "DropFunctionStatementSegment", // [2237]
    "drop_function_statement", // [2238]
    "CreateModelStatementSegment", // [2239]
    "create_model_statement", // [2240]
    "DropModelStatementSegment", // [2241]
    "drop_MODELstatement", // [2242]
    "DescribeStatementSegment", // [2243]
    "describe_statement", // [2244]
    "UseStatementSegment", // [2245]
    "use_statement", // [2246]
    "ExplainStatementSegment", // [2247]
    "explain_statement", // [2248]
    "CreateSequenceStatementSegment", // [2249]
    "create_sequence_statement", // [2250]
    "AlterSequenceStatementSegment", // [2251]
    "alter_sequence_statement", // [2252]
    "DropSequenceStatementSegment", // [2253]
    "drop_sequence_statement", // [2254]
    "CreateTriggerStatementSegment", // [2255]
    "create_trigger", // [2256]
    "DropTriggerStatementSegment", // [2257]
    "drop_trigger", // [2258]
    "DelimiterStatement", // [2259]
    "delimiter_statement", // [2260]
    "CreateProcedureStatementSegment", // [2261]
    "create_procedure_statement", // [2262]
    "DeclareStatement", // [2263]
    "declare_statement", // [2264]
    "SetTransactionStatementSegment", // [2265]
    "set_transaction_statement", // [2266]
    "SetAssignmentStatementSegment", // [2267]
    "set_statement", // [2268]
    "IfExpressionStatement", // [2269]
    "if_then_statement", // [2270]
    "WhileStatementSegment", // [2271]
    "while_statement", // [2272]
    "IterateStatementSegment", // [2273]
    "iterate_statement", // [2274]
    "RepeatStatementSegment", // [2275]
    "repeat_statement", // [2276]
    "LoopStatementSegment", // [2277]
    "loop_statement", // [2278]
    "CallStoredProcedureSegment", // [2279]
    "call_statement", // [2280]
    "PrepareSegment", // [2281]
    "prepare_segment", // [2282]
    "ExecuteSegment", // [2283]
    "execute_segment", // [2284]
    "DeallocateSegment", // [2285]
    "deallocate_segment", // [2286]
    "GetDiagnosticsSegment", // [2287]
    "get_diagnostics_segment", // [2288]
    "ResignalSegment", // [2289]
    "resignal_segment", // [2290]
    "CursorOpenCloseSegment", // [2291]
    "cursor_open_close_segment", // [2292]
    "CursorFetchSegment", // [2293]
    "cursor_fetch_segment", // [2294]
    "DropProcedureStatementSegment", // [2295]
    "drop_procedure_statement", // [2296]
    "AlterViewStatementSegment", // [2297]
    "alter_view_statement", // [2298]
    "RenameTableStatementSegment", // [2299]
    "rename_table_statement", // [2300]
    "ResetMasterStatementSegment", // [2301]
    "reset_master_statement", // [2302]
    "PurgeBinaryLogsStatementSegment", // [2303]
    "purge_binary_logs_statement", // [2304]
    "HelpStatementSegment", // [2305]
    "help_statement", // [2306]
    "CheckTableStatementSegment", // [2307]
    "check_table_statement", // [2308]
    "ChecksumTableStatementSegment", // [2309]
    "checksum_table_statement", // [2310]
    "AnalyzeTableStatementSegment", // [2311]
    "analyze_table_statement", // [2312]
    "RepairTableStatementSegment", // [2313]
    "repair_table_statement", // [2314]
    "OptimizeTableStatementSegment", // [2315]
    "optimize_table_statement", // [2316]
    "FlushStatementSegment", // [2317]
    "flush_statement", // [2318]
    "LoadDataSegment", // [2319]
    "load_data_statement", // [2320]
    "ReplaceSegment", // [2321]
    "replace_statement", // [2322]
    "AlterDatabaseStatementSegment", // [2323]
    "alter_database_statement", // [2324]
    "ReturnStatementSegment", // [2325]
    "return_statement", // [2326]
    "SetNamesStatementSegment", // [2327]
    "set_names_statement", // [2328]
    "CreateEventStatementSegment", // [2329]
    "create_event_statement", // [2330]
    "AlterEventStatementSegment", // [2331]
    "alter_event_statement", // [2332]
    "DropEventStatementSegment", // [2333]
    "drop_event_statement", // [2334]
    "SetDefaultRoleStatementSegment", // [2335]
    "set_default_role_statement", // [2336]
    "STATISTICS", // [2337]
    "STDDEV_POP", // [2338]
    "STDDEV_SAMP", // [2339]
    "STDIN", // [2340]
    "STDOUT", // [2341]
    "STOP", // [2342]
    "STRICT", // [2343]
    "ConcatSegment", // [2344]
    "STRING", // [2345]
    "STRUCTURE", // [2346]
    "STYLE", // [2347]
    "SUBLIST", // [2348]
    "SUBMULTISET", // [2349]
    "SUBSTRING", // [2350]
    "SUCCESSFUL", // [2351]
    "SUM", // [2352]
    "SUPER", // [2353]
    "SUPERUSER", // [2354]
    "SUSPEND", // [2355]
    "SWAPS", // [2356]
    "SWITCHES", // [2357]
    "SYMMETRIC", // [2358]
    "SYNONYM", // [2359]
    "SYSDATE", // [2360]
    "SYSID", // [2361]
    "system_variable", // [2362]
    "SYSTEM_TIME", // [2363]
    "SYSTEM_USER", // [2364]
    "StorageKeywordSegment", // [2365]
    "Avg_row_lengthKeywordSegment", // [2366]
    "ConnectionKeywordSegment", // [2367]
    "Delay_key_writeKeywordSegment", // [2368]
    "EncryptedKeywordSegment", // [2369]
    "Encryption_key_idKeywordSegment", // [2370]
    "Ietf_quotesKeywordSegment", // [2371]
    "Insert_methodKeywordSegment", // [2372]
    "Max_rowsKeywordSegment", // [2373]
    "Min_rowsKeywordSegment", // [2374]
    "Pack_keysKeywordSegment", // [2375]
    "Page_checksumKeywordSegment", // [2376]
    "Page_compressedKeywordSegment", // [2377]
    "Page_compression_levelKeywordSegment", // [2378]
    "Row_formatKeywordSegment", // [2379]
    "DynamicKeywordSegment", // [2380]
    "FixedKeywordSegment", // [2381]
    "CompressedKeywordSegment", // [2382]
    "RedundantKeywordSegment", // [2383]
    "CompactKeywordSegment", // [2384]
    "PageKeywordSegment", // [2385]
    "Stats_auto_recalcKeywordSegment", // [2386]
    "Stats_persistentKeywordSegment", // [2387]
    "Stats_sample_pagesKeywordSegment", // [2388]
    "TransactionalKeywordSegment", // [2389]
    "UnionKeywordSegment", // [2390]
    "DualIdentifierSegment", // [2391]
    "TABLE_CHECKSUM", // [2392]
    "Expression_A_Unary_Operator_Grammar", // [2393]
    "Expression_C_Grammar", // [2394]
    "Expression_B_Unary_Operator_Grammar", // [2395]
    "TEMPLATE", // [2396]
    "TransientKeywordSegment", // [2397]
    "TERMINATE", // [2398]
    "TEXT", // [2399]
    "TEXTSIZE", // [2400]
    "THREAD_PRIORITY", // [2401]
    "THREADS", // [2402]
    "ZoneKeywordSegment", // [2403]
    "ZONE", // [2404]
    "TIMEZONE_HOUR", // [2405]
    "TIMEZONE_MINUTE", // [2406]
    "TINYBLOB", // [2407]
    "TINYINT", // [2408]
    "TINYTEXT", // [2409]
    "TLS", // [2410]
    "TOAST", // [2411]
    "TOP", // [2412]
    "TOP_LEVEL_COUNT", // [2413]
    "TRAN", // [2414]
    "WorkKeywordSegment", // [2415]
    "WORK", // [2416]
    "LeaveKeywordSegment", // [2417]
    "CommitKeywordSegment", // [2418]
    "ChainKeywordSegment", // [2419]
    "RollbackKeywordSegment", // [2420]
    "TRANSACTION_ACTIVE", // [2421]
    "TRANSACTIONS", // [2422]
    "TRANSACTIONS_COMMITTED", // [2423]
    "TRANSACTIONS_ROLLED_BACK", // [2424]
    "TRANSFORM", // [2425]
    "TRANSFORMS", // [2426]
    "TRANSLATE", // [2427]
    "TRANSLATION", // [2428]
    "TREAT", // [2429]
    "TRIGGER_CATALOG", // [2430]
    "TRIGGER_NAME", // [2431]
    "TRIGGER_SCHEMA", // [2432]
    "TRIGGERS", // [2433]
    "TRIM", // [2434]
    "BothKeywordSegment", // [2435]
    "LeadingKeywordSegment", // [2436]
    "TrailingKeywordSegment", // [2437]
    "TRUSTED", // [2438]
    "TSEQUAL", // [2439]
    "StructLiteralSegment", // [2440]
    "TYPES", // [2441]
    "UESCAPE", // [2442]
    "UID", // [2443]
    "NaturalJoinKeywordsGrammar", // [2444]
    "UnconditionalCrossJoinKeywordsGrammar", // [2445]
    "HorizontalJoinKeywordsGrammar", // [2446]
    "UNDER", // [2447]
    "UNDO_BUFFER_SIZE", // [2448]
    "UNDOFILE", // [2449]
    "UNENCRYPTED", // [2450]
    "UNICODE", // [2451]
    "UNINSTALL", // [2452]
    "UNKNOWN", // [2453]
    "UNLISTEN", // [2454]
    "UNNAMED", // [2455]
    "UNNEST", // [2456]
    "UNREGISTER", // [2457]
    "UPDATETEXT", // [2458]
    "UPPER", // [2459]
    "DuplicateKeywordSegment", // [2460]
    "USER_DEFINED_TYPE_CATALOG", // [2461]
    "USER_DEFINED_TYPE_CODE", // [2462]
    "USER_DEFINED_TYPE_NAME", // [2463]
    "USER_DEFINED_TYPE_SCHEMA", // [2464]
    "USERS", // [2465]
    "UTC_DATE", // [2466]
    "UTC_TIME", // [2467]
    "UTC_TIMESTAMP", // [2468]
    "VACUUM", // [2469]
    "VALID", // [2470]
    "VALIDATE", // [2471]
    "VALIDATOR", // [2472]
    "VAR_POP", // [2473]
    "VAR_SAMP", // [2474]
    "VARBINARY", // [2475]
    "VARCHAR2", // [2476]
    "VARCHAR", // [2477]
    "VARCHARACTER", // [2478]
    "VARIABLE", // [2479]
    "VARIABLES", // [2480]
    "VCPU", // [2481]
    "VERBOSE", // [2482]
    "VERSION", // [2483]
    "VIA", // [2484]
    "VOLATILE", // [2485]
    "WaitKeywordSegment", // [2486]
    "WAITFOR", // [2487]
    "assignment_operator", // [2488]
    "WAREHOUSES", // [2489]
    "WARNINGS", // [2490]
    "WEEKDAY", // [2491]
    "WEIGHT_STRING", // [2492]
    "indented_then", // [2493]
    "indented_then_contents", // [2494]
    "WHENEVER", // [2495]
    "WHILE", // [2496]
    "WhileKeywordSegment", // [2497]
    "whitespace", // [2498]
    "WhitespaceSegment", // [2499]
    "WIDTH_BUCKET", // [2500]
    "PartitionClauseSegment", // [2501]
    "partitionby_clause", // [2502]
    "FrameClauseSegment", // [2503]
    "frame_clause", // [2504]
    "CascadedKeywordSegment", // [2505]
    "RecursiveKeywordSegment", // [2506]
    "indented_ctes", // [2507]
    "CTEDefinitionSegment", // [2508]
    "common_table_expression", // [2509]
    "NonWithNonSelectableGrammar", // [2510]
    "BindingKeywordSegment", // [2511]
    "RollupKeywordSegment", // [2512]
    "WITHIN", // [2513]
    "WRITETEXT", // [2514]
    "XA", // [2515]
    "XID", // [2516]
    "XML", // [2517]
    "YAML", // [2518]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 207, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 212, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 214, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 215, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 223, 1, 2, 1, 1, 4, 3, 1, 4, 5, 224, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 225, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 230, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 82, 254, 255, 1, 254, 3, 3, 4, 256, 257, 258,
    255, 1, 258, 3, 3, 4, 256, 1, 2, 1, 0, 265, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 11, 1, 2, 1, 1, 4, 3, 1, 4, 5, 270,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 271, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    0, 272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0,
    0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 1, 4294967295, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 0, 0, 1, 0, 1,
    2, 1, 0, 0, 1, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 533,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 534, 535, 255, 1, 535, 3, 3,
    4, 256, 536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 483, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 0, 1, 0, 0, 545, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 545, 547, 548, 1, 547, 3, 3, 547,
    4, 546, 547, 548, 1, 547, 3, 3, 547, 4, 0, 549, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 98, 1, 2, 1, 1, 4, 3, 1, 4, 5, 550,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 208, 576, 255,
    1, 576, 3, 3, 4, 256, 577, 578, 255, 1, 578, 3, 3, 4, 256, 1,
    2, 0, 579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2,
    0, 249, 1, 2, 1, 1, 4, 3, 1, 4, 5, 581, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 584, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 585, 1, 2, 1, 1, 4, 3, 1, 4, 5, 586, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 289, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 587, 587, 589, 3, 587, 588, 587, 3, 3, 588, 4, 590, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 591, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    592, 1, 2, 1, 1, 4, 3, 1, 4, 5, 593, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 595, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 598, 1, 2, 1, 1, 4, 3, 1, 4, 5, 442, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 599, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    600, 1, 2, 1, 1, 4, 3, 1, 4, 5, 443, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 601, 1, 2, 1, 1, 4, 3, 1, 4, 5, 602, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 603, 604, 605, 606, 607, 608, 744, 6,
    609, 610, 1, 609, 2, 3, 4, 0, 620, 621, 0, 2, 3, 620, 3, 622,
    0, 1, 3, 623, 1, 2, 1, 1, 4, 3, 1, 4, 5, 624, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 625, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 626, 1, 2, 1, 1, 4, 3, 1, 4, 5, 627, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 628, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    547, 548, 9, 3, 3, 547, 4, 643, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 644, 1, 2, 1, 1, 4, 3, 1, 4, 5, 645, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 646, 1, 2, 1, 1, 4, 3, 1, 4, 5, 647,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 560, 547, 255, 1, 547, 3, 3,
    4, 256, 652, 1, 2, 1, 1, 4, 3, 1, 4, 5, 653, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 654, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 664, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 667, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 2, 1,
    0, 0, 1, 0, 1, 2, 1, 673, 674, 0, 2, 3, 673, 1, 2, 1,
    677, 1, 2, 1, 1, 4, 3, 1, 4, 5, 678, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 679, 1, 2, 1, 1, 4, 3, 1, 4, 5, 680, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 544, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 362, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 682, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 1, 2, 1, 351, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 687, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 406, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 689, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 690, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4, 3, 1, 4, 5, 705,
    706, 255, 1, 706, 3, 3, 4, 256, 12, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 708, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 709, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    710, 1, 2, 1, 1, 4, 3, 1, 4, 5, 711, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 444, 1, 2, 1, 1, 4, 3, 1, 4, 5, 712, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 713, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 716, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 717, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 718, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 719, 1, 2, 1, 1, 4, 3, 1, 4, 5, 720, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 721, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 730, 1, 2, 1, 1, 4, 3, 1, 4, 5, 731,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 277, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 4294967295, 4294967295, 0, 747, 1, 2, 1, 1, 4, 3, 1, 4, 5, 748, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 445, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 0, 750, 1, 2, 1, 1, 4, 3, 1, 4, 5, 751,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 752, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 753, 1, 2, 1, 1, 4, 3, 1, 4, 5, 754, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 755, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 756, 1, 2, 1, 1, 4, 3, 1, 4, 5, 757, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 759,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 760, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 484, 1, 2, 1, 1, 4, 3, 1, 4, 5, 761, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 762, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 4, 610, 9, 2, 3, 4, 278, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 763, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 767, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 768, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 770, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 575, 771, 255, 1, 771, 3, 3, 4, 256, 575,
    772, 255, 1, 772, 3, 3, 4, 256, 575, 773, 255, 1, 773, 3, 3, 4,
    256, 0, 1, 2, 1, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0,
    0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1,
    2, 1, 0, 4294967295, 4294967295, 0, 422, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    630, 802, 255, 1, 802, 3, 3, 4, 256, 1, 0, 803, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 805,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 806, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 807, 1, 2, 1, 1, 4, 3, 1, 4, 5, 808, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 809, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 87, 812, 255, 1, 812,
    3, 3, 4, 256, 813, 1, 2, 1, 1, 4, 3, 1, 4, 5, 814, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 337, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 816, 817, 10, 3, 3, 816, 4, 818, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 819, 1, 2, 1, 1, 4, 3, 1, 4, 5, 820, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 234, 829, 9, 3, 3, 234, 4, 321,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 830, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 547, 831, 0, 2, 3, 547, 234, 832, 0, 2, 3, 234, 833,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 834, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 835, 1, 2, 1, 1, 4, 3, 1, 4, 5, 836, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 837, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 838, 1, 2, 1, 1, 4, 3, 1, 4, 5, 839, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    446, 1, 2, 1, 1, 4, 3, 1, 4, 5, 849, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 850, 1, 2, 1, 1, 4, 3, 1, 4, 5, 375, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 851, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 853, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 854, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 724, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 857, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 858, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 859, 1, 2, 1, 1, 4, 3, 1, 4, 5, 860, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 2, 3, 1, 447, 862, 2, 1, 862,
    4, 3, 1, 4, 5, 447, 1, 2, 1, 1, 4, 3, 1, 4, 5, 235,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 863, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 865, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 866, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 868, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 3, 4, 1, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 1, 0, 100, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0, 1, 0, 1,
    2, 1, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 1, 2,
    1, 1, 2, 1, 0, 1, 2, 1, 0, 0, 0, 0, 1, 0, 1, 2,
    1, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 1, 2, 1,
    0, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1, 2, 1, 0,
    0, 0, 1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0,
    1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    0, 0, 0, 1, 2, 1, 0, 1004, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1005, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1006, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 841, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1007,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1008, 862, 610, 1, 862, 2, 3,
    4, 1008, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 1014,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 994, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 603, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1022, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1023, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1024, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1025,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 604, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 605, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1026, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1027, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 0, 1032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 352, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 52, 1, 2, 1, 1, 4, 3, 1, 4, 5, 13, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1039, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1040, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1041, 610,
    1, 1041, 2, 3, 4, 0, 0, 0, 0, 0, 0, 0, 1063, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1064, 1065, 1066, 2726, 3, 862, 610, 1, 862, 2,
    3, 4, 0, 1, 0, 1, 2, 1, 0, 260, 1071, 589, 3, 1071, 588, 260,
    3, 3, 588, 4, 0, 794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 295,
    296, 297, 298, 299, 300, 301, 302, 303, 304, 305, 306, 307, 308, 309, 310, 311,
    312, 313, 314, 2767, 20, 1072, 610, 1, 1072, 2, 3, 4, 1073, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1074, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    295, 1, 2, 1, 1, 4, 3, 1, 4, 5, 296, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 297, 1, 2, 1, 1, 4, 3, 1, 4, 5, 298, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 299, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1075, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1076, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1077, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1079, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1080, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1084,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1085, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    0, 0, 0, 236, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1106, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1107, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1109, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 283,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1111, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 448, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1113, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 0, 0,
    1, 0, 1138, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1143, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1145, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1146, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1147, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1148, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1149,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1150, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1151, 1, 2, 1, 1, 4, 3, 1, 4, 5, 670, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1152, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1154, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1155, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1156, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1157,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 722, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1159, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 919, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 328, 1, 2, 1, 1, 4, 3, 1, 4, 5, 485, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1161,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1162, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1164, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1165, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 558, 1, 2, 1, 1, 4, 3, 1, 4, 5, 558, 547, 2, 1, 547,
    4, 3, 1, 4, 5, 555, 547, 255, 1, 547, 3, 3, 4, 256, 339, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1166, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4, 5, 765, 1167, 255, 1,
    1167, 3, 3, 4, 256, 1044, 1, 2, 1, 1, 4, 3, 1, 4, 5, 262,
    1168, 1170, 3, 1168, 1169, 262, 3, 3, 1169, 4, 262, 1171, 610, 3, 1171, 4,
    262, 2, 3, 4, 262, 1173, 589, 3, 1173, 588, 262, 3, 3, 588, 4, 0,
    3, 4, 1, 0, 0, 0, 216, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 0, 1176, 1177, 1170, 1, 1177, 3, 3, 1169, 4, 1176, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1178, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1179, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1180, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1181, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1182, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1183, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1184, 1, 2, 1, 1, 4, 3, 1, 4, 5, 945, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1185, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    698, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1186, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1187, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1188, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 329, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1193, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1194,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 449, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 450, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1195, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1196, 1197, 255, 1, 1197, 3, 3, 4, 256, 700, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 316, 1, 2, 1, 1, 4, 3, 1, 4, 5, 417, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 451, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1198, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1199, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1200, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1202, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1204, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1205, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 286, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1207, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    290, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1208, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1210, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 486, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1211, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1212, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 426, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 134, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1215, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1089, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1216, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1217, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 984, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1218, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1226, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2,
    1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 734, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1278, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1279, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1064,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1280, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 981, 1, 2, 1, 1, 4, 3, 1, 4, 5, 658, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 658, 1281, 1282, 1, 1281, 5, 3, 1, 588,
    4, 5, 735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1283, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1284, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 0, 1028, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 16, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295, 4294967295, 0,
    1291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1292, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 1, 1293, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1296, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1297, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 437, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1298,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1299, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1300, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1301, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1302, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1303, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1304, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1348,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 948, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 0, 0, 736, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 17, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 177, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1359, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1360, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 0, 1365, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1366, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1367, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2,
    1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 245,
    1408, 1, 0, 0, 4294967295, 4294967295, 0, 263, 1408, 1, 1, 2, 1, 0, 91, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 843, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 18, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 5, 862, 1425, 2, 862, 5, 3, 3, 4, 5, 0, 4294967295, 4294967295,
    0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 67, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1443, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    19, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1444, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 783, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1445, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1446, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1447, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1448,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1476, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1477, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1478, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1479, 1479, 829, 3, 1479, 234, 1479, 3, 3,
    234, 4, 1307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1480, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1481, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 0, 0,
    1490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 133, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    1, 0, 0, 195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1495, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1496, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1502, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1503, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1504, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 920, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 0, 0, 1369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1505, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1506, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1507, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1509, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1510, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 990, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1511, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1512, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1513, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1308, 1, 2, 1, 1, 4, 3, 1, 4, 5, 300, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 301, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    302, 1, 2, 1, 1, 4, 3, 1, 4, 5, 303, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1514, 1, 2, 1, 1, 4, 3, 1, 4, 5, 953, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1169, 1170, 9, 3, 3, 1169, 4, 1515,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1516, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 452, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 387, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1526, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 478, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1527, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1528, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1530, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1531, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 103, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 62, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    0, 1, 2, 1, 0, 1532, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1533,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1534, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2,
    1, 0, 0, 0, 1, 2, 1, 377, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 4294967295, 1, 0, 0, 0, 1309, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1551, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1552, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1553, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1554, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1556, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 958, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1557, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1558, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1559, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1560, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 631, 802, 255, 1, 802, 3, 3, 4, 256, 844,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1562, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1561, 1563, 255, 1, 1563, 3, 3, 4, 256,
    62, 1563, 255, 1, 1563, 3, 3, 4, 256, 1564, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 1, 2, 1, 0, 1573, 1, 2, 1, 1, 4, 3, 1, 4, 5, 453,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1574, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 872, 1, 2, 1, 1, 4, 3, 1, 4, 5, 238, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1575, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1577, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1578, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1579,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1580, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1581, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1582, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1053, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 20, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1584,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 613, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 513, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 396, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1601, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1603,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 632, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1608, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1609, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1610, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1611, 1, 2, 1, 1, 4, 3, 1, 4, 5, 964, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1612, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 245, 1617, 1, 0, 263, 1617, 1, 0, 1536, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 692, 1623, 1, 1, 2, 1, 0, 263, 1623, 1,
    0, 0, 1, 0, 1, 2, 1, 1630, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 1, 2, 1, 4, 5,
    1, 1429, 862, 1425, 1, 862, 3, 3, 4, 5, 0, 0, 0, 2, 4294967295, 0,
    5, 6, 1, 1432, 862, 2, 1, 862, 4, 3, 1, 4, 5, 1429, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1432, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1641, 1, 2, 1, 1, 4, 3, 1, 4, 5, 378, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 397, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1642,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1643, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1644, 1, 2, 1, 1, 4, 3, 1, 4, 5, 481, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 9, 4, 3, 1, 4, 5, 1645,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1646, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 21, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 92, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1648, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1649, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1651,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1652, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1653, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1409, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1654, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1655, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1656, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 845, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1657,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 935, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1658, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 913, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 636, 636, 829, 3, 636, 234, 636, 3,
    3, 234, 4, 0, 1, 2, 1, 0, 0, 0, 1130, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 921, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1664, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 922, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1666, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 588, 1282, 9, 5, 3, 1, 588, 4, 5, 588, 589, 9, 3, 3, 588,
    4, 1676, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 1677, 1, 2, 1, 1, 4, 3, 1, 4, 5, 537, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2, 4294967295, 1683, 610, 1, 1683, 2, 3,
    4, 606, 1, 2, 1, 1, 4, 3, 1, 4, 5, 607, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1684, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1685,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 424, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1686, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1687, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1310, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1689, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1690, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1691, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1692, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    0, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1695, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1696, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 2, 3, 1, 105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1698,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1311, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1699, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1700, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1701, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1702, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1703, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1704,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1705, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1707, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1708, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1710, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1711, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1712,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1713, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1714, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1715, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1716, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1717, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1718, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1719, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1720,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1721, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1722, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1723, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1724, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1726, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1727, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1728,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1729, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 23, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 973, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 974, 1, 2, 1, 1, 4, 3, 1, 4, 5, 454, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1731, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1732,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 975, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 976, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1733, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 354, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 737, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1734, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1736,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1737, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1738, 1, 2, 1, 1, 4, 3, 1, 4, 5, 515, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 1, 2, 1,
    0, 245, 1617, 1, 263, 1617, 1, 1755, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1756, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1457, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 873, 1, 2, 1, 1, 4, 3, 1, 4, 5, 304,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1757, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1759, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1760, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1371, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 554, 547, 255, 1, 547, 3, 3, 4, 256, 305, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 306, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 307, 1, 2, 1, 1, 4, 3, 1, 4, 5, 355, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1404, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 559, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 559, 547, 2, 1, 547, 4, 3, 1, 4, 5, 1357, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 900, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 106, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1762, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    551, 547, 255, 1, 547, 3, 3, 4, 256, 40, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 308, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1763, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1764, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1765, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1766, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 552, 547, 255, 1, 547, 3, 3, 4, 256, 1767, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1768, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1770, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1771, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1773, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4,
    1177, 1170, 1, 1177, 3, 3, 1169, 4, 1774, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 0, 1, 0, 1779, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1605, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1605, 1780, 1282, 1,
    1780, 5, 3, 1, 588, 4, 5, 1781, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1784, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1786,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1787, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 554, 1788, 255, 1, 1788, 3, 3, 4, 256, 1633, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1789, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 985, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1791, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1792, 1793,
    12, 3, 3, 1792, 4, 1030, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1794,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 356, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1795, 1, 2, 1, 1, 4, 3, 1, 4, 5, 538, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1796, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 357, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1797, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1798, 1, 2, 1, 1, 4, 3, 1, 4, 5, 239,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1799, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1800, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1801, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 358, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1802, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1803, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1805,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1806, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 1, 2, 1, 1, 2, 1, 0, 0, 1, 2,
    1, 0, 0, 1815, 1, 2, 1, 1, 4, 3, 1, 4, 5, 427, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 359, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1817, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1818, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 323, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 629, 1820, 610, 1, 1820, 2, 3, 4, 0, 1821, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1822, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1823, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1824, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 608, 1, 2, 1, 1, 4, 3, 1, 4, 5, 891,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1825, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1826, 1, 2, 1, 1, 4, 3, 1, 4, 5, 413, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 413, 1780, 1282, 1, 1780, 5, 3, 1, 588,
    4, 5, 1827, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1828, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1524, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1453, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1829, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 97, 97, 589, 3, 97, 588, 97, 3, 3, 588, 4, 1830,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 95, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1672, 1833, 255, 1, 1833, 3, 3, 4, 256, 1, 2,
    0, 0, 0, 1837, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1838, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1839, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 659, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1840, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 1372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1841, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1842, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1843,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1844, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1845, 1, 2, 1, 1, 4, 3, 1, 4, 5, 164, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1846, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1847, 1, 2, 1, 1, 4, 3, 1, 4, 5, 350, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1033, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1848,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1849, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1850, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1851, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 107, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1853, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 1488, 1, 2, 1, 1, 4, 3, 1, 4, 5, 995, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1592, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    901, 1, 2, 1, 1, 4, 3, 1, 4, 5, 633, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 633, 547, 548, 1, 547, 3, 3, 547, 4, 635, 547, 548,
    1, 547, 3, 3, 547, 4, 634, 547, 548, 1, 547, 3, 3, 547, 4, 0,
    0, 0, 0, 1, 0, 0, 228, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1859, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1632, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1861,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1862, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1629, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1586, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1863, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1862, 1563, 255, 1, 1563, 3, 3, 4, 256, 1, 2, 1, 0, 1865, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 0, 1276, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1867, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1868, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1869, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1870, 1, 2, 1, 1, 4, 3, 1, 4, 5, 108, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 456, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1871, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1872, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1873, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 458, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 459, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1874, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 5, 4294967295, 1875, 610, 1, 1875, 2, 3,
    4, 1876, 1875, 255, 1, 1875, 3, 3, 4, 256, 1877, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1878, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1879, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1880, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1881, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1882, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1883, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1884, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1885, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1886, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 503, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 504, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 928, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1887, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 460, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    982, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1636, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 5, 1889, 1425, 2, 1889, 5, 3, 3, 4, 5, 1, 0,
    0, 1890, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1891, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1892, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1893,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1894, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1896, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1897, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1898, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 561,
    1899, 255, 1, 1899, 3, 3, 4, 256, 1900, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1901, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1902, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1903, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1904, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1905, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 553, 547, 255, 1, 547, 3, 3, 4, 256, 1906, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 128, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1907, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1908, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1909, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1910,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 553, 1788, 255, 1, 1788, 3, 3,
    4, 256, 0, 0, 1916, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1917, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 949, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1362, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1047, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1918, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1919, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1083, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1920, 1, 2, 1, 1, 4, 3, 1, 4, 5, 325,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1921, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1922, 1, 2, 1, 1, 4, 3, 1, 4, 5, 379, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1924, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1241, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1925, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1926,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1927, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1,
    0, 1, 2, 1, 68, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1932, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1933, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1934, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1935, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1936, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 1937, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1373, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1, 2, 1, 309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1312,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1313, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 738, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1941, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1037, 1168, 1170, 3, 1168, 1169, 1037, 3, 3,
    1169, 4, 260, 1173, 589, 3, 1173, 588, 260, 3, 3, 588, 4, 1, 4294967295, 4294967295,
    0, 1943, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1944, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1945, 1, 2, 1, 1, 4, 3, 1, 4, 5, 955,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 923, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1946, 1, 2, 1, 1, 4, 3, 1, 4, 5, 233, 1947, 255,
    1, 1947, 3, 3, 4, 256, 557, 1947, 255, 1, 1947, 3, 3, 4, 256, 1948,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 556, 1947, 255, 1, 1947, 3, 3,
    4, 256, 629, 1947, 255, 1, 1947, 3, 3, 4, 256, 109, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1949, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1950,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 726, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1951, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1952, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 489, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1953, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1954, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1955, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1956,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1957, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1958, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1959, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1960, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1961, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 1964,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 111, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1969, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1230, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1970, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1971, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1972, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1973, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1974, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1975, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1976,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1977, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1979, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1980, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1981, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1314, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1982, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1983,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1984, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1985, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1986, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1987, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1988, 1, 2, 1, 1, 4, 3, 1, 4, 5, 490, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 326, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 491, 1, 2, 1, 1, 4, 3, 1, 4, 5, 492, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1991, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 1995, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 1996, 1, 2, 1, 1, 4, 3, 1, 4, 5, 332, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1998,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1999, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2000, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2001, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2002, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2003, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2004, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2005, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2006,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 962, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2007, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1315, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2008, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 0, 1, 0, 25, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1419, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2012, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2013, 1, 2, 1, 1, 4, 3, 1, 4, 5, 407,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2014, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2015, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2016, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2017, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2019, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2020, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1459,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1133, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 883, 1, 2, 1, 1, 4, 3, 1, 4, 5, 992,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2024, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 846, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1231, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 26, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 2030,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2031, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1009, 862, 610, 1, 862, 2, 3, 4, 1009, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 27,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2033, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2034, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2035, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 69, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 947, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1454, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2036,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2037, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2038, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2039, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2040, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2041, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2042,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2,
    1, 2046, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2047, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2048, 1, 2, 1, 1, 4, 3, 1, 4, 5, 288,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1460, 1, 2, 1, 1, 4, 3, 1, 4, 5, 60, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2049, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2050, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2051, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2052, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2053,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2054, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 310, 1, 2, 1, 1, 4, 3, 1, 4, 5, 311, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2055, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2056, 1, 2, 1, 1, 4, 3, 1, 4, 5, 398, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2057, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2058,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2059, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1,
    0, 2, 11238, 2, 11240, 3, 632, 1163, 632, 323, 1163, 0, 112, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 2089, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1139, 2090, 255, 1, 2090, 3, 3, 4, 256, 2091, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 29, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 70, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2093, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2094, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 53, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2095, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1316, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 6, 4294967295, 1683, 610, 1, 1683, 2, 3, 4, 2096,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 1, 0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 4294967295, 4294967295, 0, 408,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 0,
    0, 1, 0, 2117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1499, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2118, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2119, 1, 2, 1, 1, 4, 3, 1, 4, 5, 201, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2120,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 2121, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2122, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 2125, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1966, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 260, 1168,
    1170, 3, 1168, 1169, 260, 3, 3, 1169, 4, 260, 1171, 610, 3, 1171, 4, 260,
    2, 3, 4, 2127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1351, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 555, 2128, 255, 1, 2128, 3, 3, 4, 256,
    333, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2129, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 575, 2131,
    255, 1, 2131, 3, 3, 4, 256, 1317, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2132, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2133, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2135,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1939, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2137, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2138, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2139, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2140, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2141, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2142,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2143, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2144, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2145, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2146, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2148, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2150,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2151, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2152, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2153, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2154, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2155, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2156, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2157, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2158,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2159, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2161, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2162, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2163, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2164, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 380,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 871, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2167, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 521, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2168, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2169, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2170, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2062,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2171, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2172, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2063, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2064, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2065, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2173, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2174, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2175,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2066, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2176, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2067, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2177, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2178, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2179, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2180, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2181,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2182, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2183, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2184, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2185, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2186, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2187, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2188, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2189,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1097, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1099, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2190, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 965, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2191, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1451,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 71, 1, 2, 1, 1, 4, 3, 1, 4, 5, 552, 2192, 255,
    1, 2192, 3, 3, 4, 256, 907, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1598, 1, 2, 1, 1, 4, 3, 1, 4, 5, 317, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2193, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2194, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 874, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 462, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 463, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 464, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1318, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2339,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2340, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2341, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2342, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 465, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 786, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2068, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 72,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2343, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 2345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 2, 1, 2346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2347, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1461, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 966, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2348, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 930,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 932, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2350, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2351, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2352, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2353, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2354, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2355, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2356,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2357, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 256, 255, 9, 3, 3, 4, 256, 2358, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2359, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2360, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2361, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 220, 1, 2, 1, 1, 4, 3, 1, 4, 5, 7, 4294967295, 2362, 610,
    1, 2362, 2, 3, 4, 2363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2364,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 4294967295, 4294967295, 0,
    1, 2, 1, 0, 0, 0, 0, 0, 0, 1, 2, 1, 1, 2, 1, 0,
    32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 0, 0,
    2392, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1462, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1319, 1, 2, 1, 1, 4, 3, 1, 4, 5, 73, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1397, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0,
    0, 4294967295, 4294967295, 0, 34, 1, 2, 1, 1, 4, 3, 1, 4, 5, 74, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 113, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 114, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 516, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1590, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2399, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 938, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2401, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2402, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1140, 2090, 255, 1,
    2090, 3, 3, 4, 256, 1043, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    0, 0, 4294967295, 4294967295, 0, 795, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1065,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1066, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2405, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2406, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2407, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2408, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2409, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 89,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2411, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2412, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2413, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1410, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2414, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2110, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2421, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    466, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2422, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2423, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2424, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2425, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2426, 1, 2, 1, 1, 4, 3, 1, 4, 5, 912, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2427, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2429, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2430, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2431, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2433, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2434, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 660, 1281, 1282, 1, 1281,
    5, 3, 1, 588, 4, 5, 116, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2438, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2439, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 1, 2, 1, 35, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2441, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2442, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2443, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 999, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2113, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 517, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2447, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1090, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2448, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2449, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2450, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2451, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2452, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 467, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2453, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2454, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1001,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2455, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295,
    0, 2457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1049, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1993, 1, 2, 1, 1, 4, 3, 1, 4, 5, 117,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 2458, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 741, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2459, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 118, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1989, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2461, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2462, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2463, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2464, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1320, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1321, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1322, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2465, 1, 2, 1, 1, 4, 3, 1, 4, 5, 391, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2466, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2467, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2468, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2469, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2470, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2471, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 512, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2472, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 528, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1, 0, 1, 2, 1, 1, 0, 529, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2474,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2475, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2476, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2477, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2478, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2479, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2480, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1054, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2481,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2482, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2483, 1, 2, 1, 1, 4, 3, 1, 4, 5, 222, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2484, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 399,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2485, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 892, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2487, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1248, 2488, 255, 1, 2488, 3, 3, 4,
    256, 37, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2489, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2490, 1, 2, 1, 1, 4, 3, 1, 4, 5, 312,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2491, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 245, 2493, 1,
    692, 2494, 1, 263, 2494, 1, 263, 2493, 1, 694, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 0, 1123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2496, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 2498, 2499, 12, 3, 3,
    4, 2498, 2500, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295,
    0, 1374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 245, 2507, 1,
    1, 0, 263, 2507, 1, 245, 2507, 1, 1, 0, 263, 2507, 1, 400, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2513, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1425, 9, 3, 3,
    4, 5, 2416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 57, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 121, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2514, 1, 2, 1, 1, 4, 3, 1, 4, 5, 967, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2515, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2516, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2517, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 634, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2518, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 313, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    314, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1544, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1058, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2404, 1,
    2, 1, 1, 4, 3, 1, 4, 5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 38, 39, 41, 42, 43, 44, 45,
    46, 47, 48, 49, 50, 0, 51, 40, 54, 0, 55, 56, 0, 58, 59, 61,
    42, 0, 63, 59, 61, 42, 64, 0, 58, 66, 61, 46, 0, 63, 66, 61,
    41, 42, 46, 44, 43, 77, 0, 79, 81, 84, 86, 88, 90, 0, 93, 94,
    96, 52, 0, 122, 47, 48, 124, 41, 42, 44, 0, 125, 126, 127, 0, 129,
    130, 0, 131, 132, 0, 39, 48, 135, 136, 0, 51, 0, 122, 49, 46, 0,
    126, 127, 138, 64, 0, 139, 50, 46, 47, 0, 140, 141, 125, 143, 122, 144,
    129, 145, 146, 39, 147, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 136,
    158, 159, 0, 58, 141, 160, 53, 161, 163, 55, 165, 166, 167, 168, 169, 170,
    171, 172, 130, 46, 0, 126, 127, 0, 173, 166, 0, 174, 165, 0, 175, 176,
    56, 178, 179, 180, 181, 182, 183, 184, 185, 186, 59, 0, 126, 187, 0, 173,
    179, 0, 174, 178, 0, 175, 188, 57, 191, 193, 70, 0, 196, 58, 77, 0,
    124, 60, 197, 0, 48, 62, 199, 0, 202, 64, 77, 66, 199, 68, 203, 205,
    71, 81, 209, 85, 95, 105, 115, 125, 0, 135, 217, 218, 219, 221, 136, 146,
    156, 166, 176, 227, 186, 0, 231, 232, 196, 240, 241, 242, 243, 244, 0, 245,
    248, 218, 0, 251, 215, 252, 197, 206, 259, 261, 263, 219, 0, 266, 268, 229,
    239, 249, 0, 273, 259, 42, 46, 274, 260, 280, 0, 273, 282, 285, 77, 0,
    163, 287, 264, 291, 292, 293, 265, 315, 267, 0, 266, 318, 319, 293, 0, 163,
    320, 322, 324, 0, 327, 88, 77, 272, 330, 331, 0, 331, 163, 271, 334, 335,
    336, 0, 340, 341, 273, 0, 285, 0, 240, 343, 232, 283, 251, 259, 261, 240,
    0, 240, 344, 232, 345, 0, 240, 347, 232, 348, 0, 149, 349, 232, 284, 240,
    96, 288, 0, 360, 361, 96, 363, 364, 286, 0, 365, 96, 366, 287, 367, 368,
    369, 0, 273, 171, 370, 289, 372, 308, 0, 0, 382, 77, 304, 0, 383, 293,
    384, 385, 386, 388, 390, 393, 394, 0, 402, 386, 390, 393, 394, 0, 403, 386,
    388, 160, 151, 404, 160, 296, 0, 163, 294, 144, 157, 295, 409, 410, 0, 411,
    412, 0, 414, 415, 0, 411, 240, 0, 416, 300, 293, 303, 418, 0, 322, 418,
    0, 305, 384, 385, 386, 388, 390, 393, 394, 0, 306, 419, 420, 307, 384, 385,
    386, 388, 393, 394, 0, 218, 421, 423, 251, 311, 0, 231, 232, 309, 240, 241,
    242, 244, 0, 425, 232, 310, 240, 429, 430, 431, 317, 0, 432, 232, 312, 433,
    434, 0, 313, 217, 146, 421, 435, 315, 0, 314, 438, 439, 404, 160, 440, 0,
    327, 316, 441, 88, 266, 0, 273, 165, 266, 345, 344, 468, 470, 472, 0, 217,
    421, 386, 435, 318, 438, 0, 439, 404, 160, 0, 273, 421, 251, 322, 320, 0,
    411, 240, 319, 433, 293, 0, 218, 240, 0, 411, 321, 473, 474, 0, 146, 421,
    423, 435, 326, 438, 0, 439, 404, 160, 0, 217, 475, 0, 477, 421, 423, 404,
    435, 328, 0, 327, 438, 0, 439, 404, 0, 218, 331, 0, 421, 423, 404, 0,
    329, 384, 385, 423, 388, 402, 0, 403, 423, 77, 0, 330, 416, 382, 423, 77,
    0, 273, 332, 416, 382, 77, 333, 418, 0, 322, 418, 0, 273, 334, 384, 385,
    423, 388, 335, 474, 473, 479, 0, 322, 479, 0, 327, 338, 0, 336, 441, 88,
    266, 0, 337, 384, 385, 388, 88, 388, 0, 421, 404, 88, 404, 0, 339, 331,
    330, 480, 0, 482, 88, 340, 280, 0, 347, 217, 218, 493, 494, 156, 495, 496,
    497, 498, 416, 499, 500, 501, 502, 348, 505, 506, 354, 251, 96, 58, 351, 349,
    77, 49, 0, 507, 165, 508, 355, 507, 510, 511, 0, 514, 358, 356, 77, 0,
    273, 0, 231, 232, 361, 518, 519, 520, 282, 0, 522, 523, 362, 526, 527, 166,
    266, 160, 441, 366, 363, 530, 531, 367, 377, 386, 396, 0, 498, 406, 539, 540,
    165, 413, 0, 407, 266, 0, 266, 157, 541, 163, 409, 404, 0, 507, 96, 543,
    0, 266, 218, 541, 163, 411, 404, 414, 442, 424, 433, 443, 453, 463, 473, 562,
    563, 564, 565, 566, 567, 568, 569, 570, 571, 572, 573, 495, 475, 474, 96, 293,
    574, 477, 486, 0, 498, 510, 508, 580, 0, 0, 441, 513, 523, 533, 543, 553,
    563, 573, 583, 593, 604, 614, 624, 634, 644, 654, 664, 0, 597, 674, 684, 694,
    704, 714, 724, 734, 750, 759, 433, 611, 612, 203, 404, 293, 0, 615, 433, 617,
    618, 619, 86, 441, 760, 766, 771, 781, 791, 801, 811, 821, 831, 637, 638, 639,
    640, 641, 642, 832, 839, 849, 859, 869, 879, 648, 0, 649, 649, 650, 0, 651,
    651, 889, 898, 908, 918, 928, 938, 656, 657, 941, 939, 163, 661, 940, 662, 663,
    942, 952, 665, 666, 953, 965, 963, 433, 970, 968, 404, 980, 978, 0, 976, 404,
    0, 404, 668, 973, 293, 977, 671, 672, 983, 989, 675, 992, 1002, 1012, 1022, 1032,
    1042, 1052, 1062, 1072, 252, 0, 251, 683, 441, 1075, 530, 1078, 1088, 0, 686, 203,
    1098, 1108, 1118, 1128, 1146, 0, 691, 692, 1138, 695, 697, 699, 701, 699, 699, 0,
    691, 293, 1142, 695, 697, 699, 701, 699, 699, 640, 86, 703, 1147, 1157, 1167, 1176,
    1186, 1196, 1206, 1216, 1226, 1236, 1246, 1256, 1266, 715, 1276, 1286, 1296, 1306, 1316, 1326,
    1336, 0, 1346, 723, 0, 322, 723, 0, 45, 522, 1347, 0, 727, 522, 0, 414,
    522, 0, 728, 522, 55, 0, 729, 522, 55, 0, 522, 523, 1348, 526, 527, 1349,
    1359, 1369, 0, 732, 411, 733, 1370, 1380, 0, 416, 165, 1390, 266, 1392, 0, 739,
    740, 742, 743, 744, 745, 746, 1396, 1406, 1416, 0, 749, 165, 1426, 266, 1428, 742,
    745, 1429, 1439, 1449, 1459, 1469, 1479, 1489, 1499, 1509, 1519, 1529, 1539, 1549, 1559, 1569,
    0, 344, 345, 1575, 1585, 1597, 348, 1595, 251, 764, 766, 1598, 1608, 1618, 1628, 1638,
    1647, 1656, 1669, 1666, 1665, 774, 433, 203, 611, 1685, 1680, 0, 0, 382, 77, 1673,
    0, 322, 412, 0, 416, 1670, 293, 0, 240, 776, 0, 402, 777, 778, 779, 0,
    780, 777, 336, 0, 344, 345, 781, 0, 343, 1674, 251, 259, 261, 240, 782, 0,
    0, 784, 785, 441, 1675, 293, 1678, 788, 789, 0, 791, 96, 1679, 473, 474, 0,
    0, 784, 785, 441, 1681, 293, 1684, 793, 788, 789, 0, 251, 1694, 615, 1686, 796,
    797, 0, 1687, 796, 797, 798, 1690, 0, 0, 322, 412, 0, 240, 1688, 799, 611,
    96, 348, 412, 0, 163, 157, 1689, 799, 611, 1695, 0, 1698, 800, 0, 1702, 1712,
    1721, 251, 764, 766, 1723, 1733, 1743, 1753, 1763, 1773, 1783, 0, 810, 0, 0, 1793,
    1803, 1812, 1822, 0, 815, 348, 1832, 1842, 1849, 1859, 1869, 1879, 232, 821, 822, 823,
    824, 825, 826, 827, 828, 1880, 1887, 1897, 1907, 1913, 1919, 1929, 1939, 1949, 0, 650,
    650, 1959, 1969, 1979, 840, 1989, 842, 847, 848, 1990, 2000, 2010, 2020, 2030, 2040, 2050,
    2060, 2070, 2080, 2090, 2100, 2110, 2120, 2130, 2140, 2152, 293, 2151, 0, 86, 615, 0,
    861, 2150, 715, 434, 2155, 2165, 2175, 2185, 2195, 2205, 2215, 2225, 2235, 0, 122, 869,
    2245, 615, 441, 615, 507, 870, 2249, 170, 168, 169, 0, 2248, 875, 876, 877, 878,
    79, 81, 0, 739, 77, 0, 441, 879, 0, 122, 2250, 42, 46, 386, 274, 2251,
    880, 0, 122, 282, 285, 386, 77, 163, 287, 2255, 291, 292, 293, 2256, 315, 2258,
    0, 2257, 318, 319, 293, 0, 163, 320, 322, 324, 2263, 330, 331, 0, 331, 163,
    2262, 334, 335, 336, 340, 341, 0, 122, 282, 882, 168, 79, 81, 0, 884, 615,
    336, 885, 887, 0, 122, 882, 2264, 383, 419, 420, 384, 386, 388, 889, 163, 266,
    393, 890, 894, 2265, 895, 897, 2269, 0, 122, 882, 899, 386, 77, 0, 902, 2287,
    2285, 0, 432, 232, 2284, 433, 2281, 2279, 348, 441, 530, 0, 240, 2291, 0, 732,
    411, 232, 2290, 434, 348, 0, 344, 232, 345, 0, 347, 232, 348, 0, 122, 282,
    882, 169, 386, 79, 903, 336, 885, 887, 0, 122, 48, 386, 199, 0, 122, 46,
    386, 905, 2294, 0, 360, 361, 96, 0, 908, 507, 96, 363, 364, 2292, 0, 365,
    96, 366, 2293, 367, 368, 369, 0, 122, 171, 370, 2295, 909, 0, 122, 882, 911,
    165, 386, 266, 2313, 0, 2302, 2300, 2299, 914, 435, 336, 0, 441, 2308, 2305, 530,
    0, 441, 2312, 2309, 530, 0, 915, 266, 916, 2393, 0, 240, 2315, 432, 343, 0,
    2314, 55, 384, 918, 0, 507, 219, 232, 2321, 433, 432, 348, 259, 96, 240, 2318,
    2316, 266, 0, 505, 361, 2332, 0, 924, 2324, 0, 925, 293, 0, 385, 0, 231,
    232, 96, 2322, 404, 0, 2325, 926, 927, 2331, 293, 2328, 2326, 404, 0, 929, 96,
    0, 931, 361, 0, 924, 2336, 0, 925, 293, 0, 385, 0, 231, 232, 96, 2333,
    404, 0, 933, 96, 2389, 2386, 2384, 0, 505, 404, 2380, 0, 934, 2348, 0, 936,
    937, 2341, 939, 2338, 2337, 293, 404, 96, 433, 0, 61, 2345, 2343, 2342, 433, 77,
    0, 2350, 432, 343, 0, 2349, 55, 384, 918, 0, 507, 219, 232, 2356, 433, 432,
    348, 259, 96, 240, 2353, 2351, 266, 0, 931, 433, 2376, 0, 934, 2367, 0, 936,
    937, 2363, 939, 2357, 2360, 0, 61, 2364, 77, 0, 2369, 432, 343, 0, 2368, 55,
    384, 918, 0, 507, 219, 232, 2375, 433, 432, 348, 259, 96, 240, 2372, 2370, 266,
    0, 122, 940, 282, 155, 386, 941, 2397, 943, 439, 2398, 145, 157, 144, 163, 266,
    0, 739, 944, 946, 0, 2399, 950, 951, 251, 2400, 341, 0, 952, 341, 699, 0,
    122, 882, 124, 386, 2409, 0, 199, 0, 2407, 0, 954, 2406, 0, 361, 2401, 0,
    956, 957, 348, 0, 507, 77, 0, 2405, 0, 361, 2402, 0, 956, 957, 348, 0,
    441, 348, 0, 959, 960, 954, 2404, 0, 361, 2403, 0, 956, 957, 348, 0, 507,
    77, 441, 348, 961, 0, 240, 48, 2411, 199, 0, 963, 2416, 429, 2414, 2413, 968,
    969, 0, 970, 348, 0, 971, 348, 0, 972, 348, 961, 0, 507, 2418, 0, 2417,
    977, 978, 979, 980, 96, 0, 2427, 0, 957, 983, 0, 2422, 240, 986, 0, 987,
    96, 988, 0, 957, 989, 2423, 240, 96, 0, 957, 991, 987, 2424, 240, 0, 96,
    988, 0, 957, 963, 993, 0, 2425, 240, 996, 0, 997, 96, 0, 998, 2426, 96,
    1000, 0, 9, 2431, 1002, 425, 0, 2432, 815, 1003, 348, 0, 122, 882, 0, 231,
    232, 2433, 518, 519, 520, 282, 0, 522, 523, 2434, 526, 527, 166, 266, 160, 441,
    2438, 2435, 530, 531, 2439, 2449, 2459, 2469, 2479, 2489, 2497, 0, 2507, 1010, 1011, 2508,
    1012, 2511, 2521, 2531, 96, 2534, 1015, 1016, 1017, 1018, 0, 1019, 1020, 2535, 2545, 2555,
    2565, 2575, 2585, 2595, 2605, 2615, 0, 1029, 0, 1031, 90, 434, 514, 2626, 2625, 617,
    618, 2628, 0, 2638, 1034, 1035, 2639, 251, 1036, 2640, 2650, 2660, 2670, 2680, 2690, 2709,
    2700, 251, 434, 2715, 1042, 0, 1045, 1046, 0, 2710, 1050, 1051, 1052, 0, 2713, 0,
    2711, 732, 715, 2712, 1055, 0, 93, 94, 0, 0, 251, 1056, 1057, 798, 2714, 1059,
    1050, 1051, 1060, 0, 1060, 1051, 0, 1051, 1060, 1061, 2716, 2729, 1067, 0, 2741, 2739,
    2738, 315, 1068, 0, 2744, 1069, 1070, 797, 2756, 2745, 96, 2757, 2787, 2796, 2806, 2816,
    2826, 2836, 2846, 2856, 2866, 2876, 2886, 2896, 2906, 2916, 0, 0, 2926, 1081, 218, 1082,
    434, 2927, 2937, 2947, 2962, 0, 1087, 434, 1088, 739, 341, 0, 1087, 2957, 1091, 1092,
    1093, 1094, 739, 2959, 1096, 1098, 0, 322, 1100, 0, 1103, 1104, 348, 2958, 348, 96,
    434, 0, 341, 0, 1087, 434, 1105, 739, 2960, 348, 96, 0, 1087, 618, 615, 0,
    240, 2961, 348, 96, 203, 2963, 0, 240, 934, 2973, 2983, 2993, 3003, 3013, 3023, 0,
    526, 232, 199, 3033, 3043, 3053, 3063, 3073, 0, 144, 1114, 742, 1116, 3087, 0, 90,
    3083, 1118, 861, 1120, 1122, 0, 3085, 1118, 90, 1125, 1122, 0, 1125, 1127, 1122, 227,
    1129, 1132, 0, 266, 0, 1056, 1135, 0, 861, 3088, 1136, 3090, 3100, 1141, 1142, 3101,
    1144, 3111, 3121, 3131, 3141, 3151, 3161, 3171, 3181, 3191, 0, 1153, 434, 77, 3201, 3211,
    3221, 3231, 3241, 3251, 3261, 3271, 3281, 3291, 3301, 3311, 3321, 3331, 3341, 3351, 3361, 3371,
    3381, 3390, 3400, 3410, 3420, 3429, 3439, 3450, 3460, 3471, 409, 410, 0, 218, 869, 3472,
    615, 441, 615, 1174, 0, 218, 42, 423, 274, 1174, 0, 218, 285, 423, 77, 0,
    218, 168, 423, 79, 0, 218, 384, 388, 163, 266, 3477, 0, 231, 232, 3475, 240,
    241, 242, 0, 425, 232, 3476, 240, 429, 430, 431, 3478, 0, 218, 899, 423, 77,
    0, 218, 3488, 169, 168, 423, 77, 0, 218, 48, 423, 251, 0, 218, 46, 423,
    905, 1174, 0, 218, 171, 370, 0, 218, 1175, 165, 423, 3489, 266, 1174, 0, 218,
    155, 423, 941, 0, 218, 50, 423, 77, 1174, 0, 218, 124, 423, 199, 0, 218,
    166, 423, 266, 1174, 3491, 3500, 3510, 3520, 3530, 3540, 3550, 3560, 3570, 3580, 3590, 0,
    697, 293, 3600, 3610, 3620, 3630, 3640, 0, 1189, 1191, 3643, 3653, 3663, 3673, 3683, 3693,
    3703, 3713, 3722, 3732, 3742, 3752, 3762, 3772, 3782, 3792, 1201, 3802, 3812, 3822, 3832, 3842,
    3852, 3862, 3872, 3882, 3892, 3902, 3912, 3922, 3932, 3942, 3952, 3962, 0, 129, 434, 0,
    861, 3972, 617, 3974, 3984, 3994, 4004, 4014, 4024, 4034, 4044, 0, 1219, 4054, 530, 1220,
    1222, 1224, 4055, 1227, 0, 1228, 4066, 4065, 1229, 0, 703, 1228, 1232, 1233, 0, 1234,
    322, 1235, 1236, 1237, 782, 0, 322, 1238, 1239, 961, 1228, 0, 1240, 1227, 4070, 1242,
    0, 1243, 1142, 1244, 1245, 0, 1246, 4073, 4072, 0, 4071, 637, 638, 640, 1246, 4077,
    1242, 0, 1243, 1142, 0, 0, 617, 1247, 4086, 0, 1249, 4078, 530, 0, 4081, 1250,
    1251, 4082, 1253, 774, 86, 0, 4101, 611, 203, 4091, 4090, 1255, 293, 530, 4088, 4087,
    433, 404, 203, 1256, 293, 1258, 1259, 0, 4094, 1189, 1260, 4097, 4095, 293, 0, 615,
    4100, 348, 96, 1262, 1263, 1264, 1256, 1265, 86, 1266, 4102, 617, 1267, 1269, 433, 612,
    1271, 1273, 404, 1275, 4103, 0, 4113, 4123, 4133, 4143, 4153, 4163, 4173, 4183, 4194, 4204,
    4214, 0, 1029, 4224, 438, 1031, 4225, 96, 293, 946, 4226, 946, 1286, 4227, 349, 0,
    507, 1287, 4228, 4238, 4248, 0, 4258, 1290, 4266, 341, 4262, 1290, 4268, 1290, 4272, 4282,
    0, 1294, 4292, 0, 1295, 293, 4295, 4305, 4315, 4325, 4335, 4345, 4355, 4365, 4375, 4385,
    4395, 0, 1305, 4405, 539, 540, 4414, 4408, 4407, 0, 715, 1323, 0, 1324, 1323, 0,
    1325, 1323, 0, 1326, 1323, 0, 1327, 365, 0, 1328, 1323, 0, 1329, 1330, 0, 4406,
    1331, 1332, 1333, 0, 1334, 1323, 0, 739, 1335, 77, 1336, 1323, 141, 1337, 1338, 1339,
    1340, 1341, 335, 968, 1342, 1343, 1344, 1345, 0, 178, 0, 4410, 266, 507, 0, 507,
    149, 425, 0, 961, 331, 1346, 0, 178, 0, 4412, 266, 739, 0, 739, 1347, 4415,
    4425, 4439, 0, 0, 739, 4435, 157, 202, 0, 1350, 4436, 434, 4438, 1352, 0, 1353,
    1354, 0, 425, 61, 202, 1356, 4440, 4450, 0, 51, 385, 4460, 4470, 4480, 4490, 4500,
    0, 1361, 4513, 4512, 0, 993, 946, 0, 4510, 96, 612, 1000, 404, 4511, 1363, 1364,
    0, 1238, 961, 4514, 1286, 926, 4515, 4525, 4535, 4545, 0, 90, 4555, 1136, 4557, 1295,
    1129, 0, 196, 361, 0, 1375, 361, 1376, 1377, 1378, 1379, 1381, 1383, 1029, 1385, 1386,
    1379, 1381, 531, 1388, 4570, 0, 1390, 4561, 4558, 1391, 1393, 268, 4562, 1395, 1396, 1399,
    1400, 0, 507, 1385, 268, 1396, 1402, 4567, 0, 4563, 1400, 4589, 4586, 0, 4574, 1403,
    1406, 4571, 1136, 0, 1375, 361, 0, 196, 361, 4575, 4579, 0, 4578, 1400, 1399, 0,
    1375, 361, 0, 196, 361, 4583, 4590, 4600, 4610, 293, 4626, 293, 0, 293, 441, 615,
    0, 1411, 293, 90, 90, 293, 0, 4620, 315, 293, 90, 293, 0, 1412, 4623, 1135,
    4621, 1413, 1414, 0, 1417, 433, 0, 4624, 348, 251, 404, 61, 4625, 348, 251, 404,
    1418, 1420, 1422, 0, 4630, 1068, 1423, 4633, 4643, 0, 4653, 0, 251, 1056, 674, 4657,
    1426, 1036, 674, 4660, 0, 432, 4658, 0, 1427, 50, 615, 4659, 0, 1427, 50, 615,
    4663, 4661, 1428, 4667, 0, 1430, 1431, 0, 1433, 1434, 0, 1435, 1436, 0, 1437, 1438,
    1439, 0, 0, 79, 4666, 1437, 1440, 1267, 1441, 1442, 4668, 4678, 4688, 4698, 4708, 4718,
    4728, 4738, 4748, 0, 1449, 0, 993, 1450, 1452, 4760, 0, 4758, 617, 618, 232, 4759,
    1455, 1456, 1105, 4762, 617, 618, 96, 4765, 0, 4763, 617, 618, 232, 4764, 1463, 1464,
    1465, 1466, 1467, 1468, 1469, 1470, 1471, 1472, 1473, 1474, 1475, 4767, 4777, 4787, 4797, 4807,
    4818, 4828, 4838, 4848, 0, 1482, 4860, 0, 1483, 163, 1485, 4858, 199, 88, 4861, 199,
    4863, 0, 507, 1482, 1487, 0, 507, 1489, 1487, 4864, 4874, 0, 651, 1201, 651, 4884,
    0, 196, 361, 0, 4896, 0, 4894, 404, 96, 293, 4895, 671, 672, 1492, 1493, 4898,
    0, 1375, 361, 1129, 1376, 1377, 1378, 1029, 4899, 4909, 0, 4923, 4922, 404, 96, 293,
    4919, 1492, 4925, 0, 1497, 1498, 4938, 4936, 4935, 1500, 1012, 4941, 4951, 4961, 4971, 4981,
    0, 1376, 4994, 4991, 293, 4995, 0, 1375, 361, 1129, 1377, 1378, 1029, 4996, 5006, 5016,
    5026, 0, 1508, 348, 5036, 5046, 5056, 5066, 5076, 5086, 0, 5096, 5106, 5116, 5126, 5136,
    5146, 5156, 5166, 5176, 5183, 5193, 5203, 5213, 0, 1518, 1249, 5223, 0, 1518, 293, 1519,
    341, 0, 1521, 293, 1519, 341, 0, 697, 341, 0, 699, 1518, 5227, 0, 1518, 322,
    1249, 5237, 0, 5247, 1116, 1522, 1523, 5248, 5258, 5268, 5278, 5288, 5298, 5308, 5318, 5328,
    5338, 5348, 0, 322, 61, 5364, 5361, 5360, 5358, 1227, 530, 203, 5365, 5375, 5385, 5395,
    0, 251, 5405, 671, 672, 5406, 96, 0, 5409, 139, 1116, 1535, 5410, 384, 385, 0,
    739, 5411, 1537, 0, 1375, 361, 0, 196, 361, 5412, 77, 1538, 5415, 5427, 0, 1540,
    232, 96, 889, 0, 507, 1541, 77, 0, 815, 348, 0, 1543, 232, 5425, 1545, 414,
    5426, 479, 0, 322, 479, 5432, 0, 1540, 232, 96, 390, 0, 507, 1541, 77, 336,
    5431, 474, 473, 0, 1546, 232, 348, 0, 1547, 232, 348, 0, 861, 5436, 1548, 925,
    0, 861, 5437, 1548, 925, 1550, 5438, 5448, 5458, 5468, 5478, 5488, 5498, 5508, 5518, 5528,
    5538, 5548, 5558, 5567, 5577, 5587, 5597, 5607, 5616, 5625, 5635, 0, 441, 251, 5645, 252,
    0, 145, 5648, 1114, 1565, 1566, 1116, 514, 266, 0, 505, 5649, 252, 160, 5652, 1267,
    1567, 530, 1132, 1569, 1571, 1132, 5653, 5663, 5673, 5683, 5693, 5703, 5713, 5723, 5733, 5743,
    5753, 5763, 5773, 5783, 5793, 5803, 5813, 5823, 0, 987, 5833, 315, 0, 293, 315, 5834,
    0, 514, 5852, 5850, 5849, 5844, 617, 618, 0, 1585, 348, 0, 1587, 348, 0, 732,
    411, 434, 0, 5848, 1588, 1589, 0, 1591, 361, 348, 0, 1593, 1594, 361, 348, 0,
    1595, 361, 348, 0, 1597, 0, 1599, 361, 348, 0, 1591, 361, 348, 1600, 5853, 5863,
    5873, 5883, 5893, 5903, 5913, 1263, 1604, 1606, 1262, 1607, 0, 1234, 322, 1412, 90, 5914,
    0, 5924, 5934, 5944, 5954, 5964, 5974, 0, 1613, 251, 5995, 0, 1614, 1615, 1406, 5984,
    1616, 0, 5988, 1618, 5991, 1538, 1620, 5992, 0, 1621, 1615, 1406, 1618, 0, 1622, 1406,
    5996, 0, 1537, 0, 0, 163, 6006, 6012, 6009, 293, 6013, 6017, 1624, 0, 6016, 1625,
    1626, 1627, 1628, 0, 861, 6020, 6018, 251, 6023, 0, 1589, 6043, 6041, 6040, 0, 251,
    739, 1631, 0, 1634, 1635, 348, 1637, 0, 251, 615, 6039, 0, 1249, 1635, 348, 0,
    1635, 348, 6035, 0, 6033, 412, 1325, 0, 240, 348, 163, 6034, 1639, 1325, 6046, 293,
    86, 348, 1637, 6049, 6064, 293, 86, 293, 0, 1640, 615, 0, 732, 411, 434, 6060,
    0, 6058, 412, 1325, 0, 240, 293, 163, 6059, 1639, 1325, 6067, 6077, 6087, 6097, 6107,
    6117, 6127, 6137, 6147, 6157, 6167, 6175, 6185, 6195, 6205, 6215, 6225, 6235, 6245, 6255, 0,
    6265, 6275, 6285, 6295, 6305, 6315, 6325, 6335, 6345, 0, 649, 1201, 649, 6355, 0, 0,
    322, 1659, 1227, 0, 1660, 1228, 6365, 915, 1661, 1662, 6366, 6376, 0, 1663, 6391, 6388,
    6387, 96, 293, 58, 6393, 0, 1385, 6392, 96, 293, 0, 86, 96, 6394, 6404, 6414,
    6424, 6434, 0, 6444, 6454, 6464, 348, 96, 1262, 1243, 1263, 1264, 1667, 1669, 1671, 1674,
    1675, 6465, 6474, 6481, 0, 1678, 55, 6491, 1114, 1679, 0, 540, 1680, 348, 6492, 1682,
    1116, 514, 165, 266, 1127, 0, 732, 411, 434, 0, 6493, 1588, 1589, 0, 1591, 361,
    348, 0, 0, 1593, 1594, 361, 348, 0, 1595, 361, 348, 0, 1597, 0, 1599, 361,
    348, 0, 1591, 361, 348, 0, 1116, 96, 6494, 1597, 1286, 0, 6497, 6495, 404, 0,
    411, 1239, 6500, 0, 6510, 6520, 6529, 6539, 6549, 6559, 6569, 0, 425, 232, 6579, 240,
    429, 430, 431, 6580, 6590, 6600, 6610, 6620, 6630, 6640, 6650, 6660, 6672, 0, 0, 251,
    1693, 1694, 6670, 341, 0, 699, 1694, 251, 6673, 6683, 6693, 0, 1697, 1056, 251, 6706,
    0, 899, 77, 0, 86, 6703, 530, 6709, 6719, 0, 6729, 6739, 6749, 6759, 6769, 6779,
    6789, 6799, 6809, 6819, 6829, 6839, 6849, 6859, 6869, 6879, 6889, 6899, 6909, 6919, 6929, 6939,
    6949, 6959, 6969, 6979, 6989, 6999, 7009, 7019, 7029, 0, 7039, 7049, 7059, 7069, 7079, 7089,
    7099, 7109, 7119, 7129, 7139, 7149, 7159, 7169, 7179, 7189, 7199, 7209, 7219, 144, 0, 145,
    160, 1267, 0, 519, 514, 7229, 7239, 1739, 1741, 0, 1743, 1744, 0, 961, 293, 1519,
    7243, 1745, 1747, 0, 1743, 322, 1744, 0, 961, 293, 1519, 1749, 0, 1751, 7244, 266,
    1752, 861, 7248, 266, 1752, 0, 7245, 530, 268, 0, 203, 268, 7249, 1538, 7252, 1753,
    0, 157, 1567, 7255, 7265, 7275, 7285, 7295, 7305, 7315, 7325, 7335, 7345, 7355, 7365, 7374,
    7384, 7394, 7404, 7414, 7424, 7434, 7444, 7454, 7464, 7474, 7484, 7494, 7504, 7513, 7523, 7533,
    7543, 7553, 7563, 7573, 7582, 7592, 7602, 7612, 7622, 7632, 7642, 7652, 7662, 7672, 0, 251,
    441, 7685, 251, 7682, 1775, 0, 1378, 7686, 1777, 7688, 7698, 7708, 7719, 0, 1783, 842,
    7729, 7739, 7749, 7759, 7769, 7779, 0, 7788, 7798, 7808, 7818, 7828, 7838, 7845, 7855, 7865,
    7875, 7885, 7895, 7905, 7915, 7925, 7935, 7945, 7955, 7965, 7975, 7985, 7995, 8005, 8015, 8025,
    8044, 1267, 1807, 8035, 1269, 8038, 1808, 8041, 1810, 1811, 0, 8045, 1222, 1220, 1224, 1812,
    8050, 1814, 8049, 8046, 1269, 1810, 8051, 8061, 8071, 8081, 0, 8091, 8101, 0, 8111, 0,
    1819, 1201, 0, 649, 651, 8112, 0, 8140, 8122, 8132, 8141, 8151, 8161, 8171, 8181, 8191,
    8201, 8211, 8221, 8231, 0, 649, 1201, 651, 8242, 8252, 8262, 8272, 8282, 8292, 8303, 8313,
    0, 348, 1693, 580, 8334, 8323, 1831, 8325, 8337, 1056, 0, 1056, 1056, 8338, 163, 441,
    861, 86, 1834, 1835, 1836, 703, 1693, 1290, 1399, 8339, 8349, 8359, 8369, 8379, 8389, 0,
    1385, 8399, 96, 293, 946, 8400, 946, 1286, 8401, 8411, 8421, 8431, 8441, 8451, 8461, 8471,
    8481, 8491, 8501, 8511, 8521, 8531, 8541, 8551, 8561, 8571, 8581, 0, 499, 8591, 539, 540,
    165, 8592, 266, 8594, 8604, 8614, 8624, 8634, 8671, 8644, 8653, 8662, 0, 1854, 1682, 0,
    1375, 361, 8675, 0, 8672, 404, 96, 293, 8673, 671, 672, 0, 1523, 8674, 438, 1855,
    1856, 1129, 1361, 8677, 1129, 1376, 1377, 1378, 1361, 1417, 1029, 8678, 8688, 1375, 1858, 8689,
    8699, 8709, 8719, 8729, 8739, 8749, 8759, 8769, 0, 1418, 1864, 8781, 251, 8778, 1775, 8782,
    0, 1866, 8795, 0, 8792, 1264, 86, 1264, 404, 8796, 8806, 8816, 8826, 8836, 8846, 8856,
    8866, 8876, 8886, 8896, 8906, 8916, 8926, 8936, 8945, 8954, 8964, 8974, 8984, 8994, 9004, 9014,
    9024, 9034, 9044, 9054, 0, 505, 361, 9069, 9066, 9064, 293, 9070, 9080, 9090, 9100, 9110,
    9120, 9130, 9152, 0, 1888, 9150, 9140, 1888, 348, 0, 9153, 9163, 9173, 9183, 9193, 9203,
    9213, 9223, 9233, 9243, 9253, 9263, 9272, 9282, 9292, 9302, 9312, 9322, 9332, 9341, 9351, 9361,
    9371, 9381, 9391, 9401, 9410, 1911, 1913, 9411, 1914, 1127, 9412, 9422, 0, 9432, 9442, 9452,
    9462, 9472, 9482, 0, 1082, 434, 90, 9492, 348, 617, 618, 9493, 9503, 9513, 9523, 0,
    1923, 385, 9533, 9543, 9553, 9563, 9573, 9583, 9593, 9603, 9614, 0, 9613, 1928, 1929, 1930,
    432, 615, 615, 9617, 9615, 1931, 9620, 9630, 9640, 9650, 9660, 9670, 9680, 0, 1938, 9690,
    715, 1330, 1323, 9692, 0, 88, 348, 0, 943, 9691, 293, 9693, 0, 1242, 96, 9703,
    0, 640, 9713, 58, 1427, 1940, 9714, 530, 9717, 9727, 9737, 9747, 9757, 9767, 9789, 9778,
    1674, 9793, 9803, 9813, 9823, 9833, 9843, 9853, 9862, 9871, 9881, 9890, 9899, 9909, 9919, 9929,
    9939, 9949, 9959, 9969, 9979, 9989, 9999, 10009, 10019, 10029, 10039, 10049, 0, 151, 266, 160,
    1962, 10059, 0, 163, 144, 1963, 0, 163, 157, 1963, 10063, 0, 1965, 10073, 1625, 1967,
    1968, 10074, 10084, 10094, 10104, 409, 410, 0, 411, 412, 0, 414, 415, 0, 411, 240,
    10105, 10115, 10125, 10135, 10145, 10155, 10165, 10175, 10185, 10195, 10205, 10215, 10225, 10235, 10245, 10255,
    10265, 10275, 10285, 10295, 10305, 10315, 10325, 0, 327, 165, 10335, 0, 266, 88, 266, 10337,
    10347, 0, 501, 10357, 539, 540, 165, 10358, 266, 10360, 742, 745, 1990, 10364, 10378, 0,
    0, 251, 1693, 1992, 10374, 341, 0, 1994, 293, 0, 699, 1992, 251, 10379, 10389, 0,
    1682, 10399, 1114, 1565, 0, 514, 266, 1127, 10400, 0, 160, 1267, 1567, 0, 160, 1269,
    1132, 10401, 10411, 10421, 10431, 10441, 10451, 10461, 10471, 10481, 10491, 10501, 10511, 10521, 10531, 10541,
    0, 1329, 1330, 0, 88, 96, 10551, 0, 10561, 2010, 2011, 10562, 0, 1103, 1104, 348,
    434, 0, 411, 10565, 0, 10563, 1463, 1464, 1465, 1466, 1467, 1468, 1469, 1470, 1471, 1472,
    1473, 1474, 1475, 232, 10564, 617, 618, 348, 10567, 10577, 10587, 10597, 10607, 10617, 10627, 10637,
    10647, 0, 2018, 293, 10657, 10667, 10677, 10687, 0, 1640, 10697, 2022, 1600, 10699, 10709, 10719,
    10729, 10739, 0, 2025, 0, 1482, 1487, 739, 10749, 0, 1483, 163, 1485, 0, 48, 199,
    0, 148, 163, 124, 197, 197, 90, 2026, 1174, 10750, 10760, 10770, 10782, 0, 10780, 434,
    1036, 259, 1674, 0, 2028, 10781, 434, 1036, 259, 1674, 2029, 10783, 10793, 10803, 10811, 10821,
    10831, 10841, 10851, 10861, 10871, 10881, 10891, 10901, 10911, 10921, 10931, 10941, 10951, 10961, 10971, 10981,
    10991, 0, 2043, 11001, 2044, 219, 11002, 96, 0, 11005, 2045, 11006, 96, 11009, 11019, 11029,
    11039, 11049, 11059, 11069, 11079, 11089, 11099, 11109, 11119, 11129, 11139, 11149, 11159, 11169, 11179, 11189,
    11199, 11209, 11219, 11229, 2060, 0, 580, 268, 0, 11230, 1412, 58, 2069, 1566, 2070, 2071,
    2072, 2073, 2074, 2075, 2076, 0, 152, 2077, 11231, 2022, 1600, 514, 11243, 90, 11233, 1295,
    0, 1375, 361, 1129, 1866, 1379, 1029, 11244, 0, 505, 11256, 11254, 77, 0, 2079, 1388,
    1125, 1914, 1127, 1122, 2081, 2083, 1275, 2085, 1386, 227, 1129, 2085, 1388, 1379, 1571, 531,
    1132, 11270, 11262, 11259, 1808, 11266, 11263, 2087, 2088, 11267, 530, 11271, 11281, 11290, 11300, 11310,
    11320, 0, 939, 96, 0, 414, 939, 11321, 0, 2092, 96, 0, 414, 2092, 11322, 11332,
    11342, 11352, 11362, 11372, 11382, 11391, 0, 411, 11409, 0, 0, 11401, 2097, 2098, 1056, 11402,
    1331, 2099, 2100, 1332, 540, 11403, 617, 618, 1675, 11404, 232, 1247, 11405, 96, 348, 1674,
    617, 1675, 2101, 618, 203, 637, 293, 0, 411, 11411, 2102, 0, 404, 232, 11413, 433,
    611, 203, 404, 293, 1267, 240, 0, 411, 240, 48, 11416, 58, 429, 11414, 199, 88,
    11417, 199, 0, 1810, 11419, 0, 1379, 1810, 227, 1129, 2085, 11423, 0, 411, 2104, 11433,
    240, 348, 434, 0, 344, 345, 11435, 2105, 0, 11434, 2106, 2107, 58, 2108, 0, 411,
    46, 386, 905, 0, 411, 11436, 1331, 1332, 2109, 11441, 11440, 0, 2111, 2112, 11438, 0,
    149, 11437, 2114, 2115, 0, 2045, 149, 2116, 0, 149, 11439, 159, 349, 11443, 11453, 11463,
    11473, 11483, 11493, 11503, 0, 11513, 1250, 1251, 11514, 0, 1834, 615, 1253, 11518, 11528, 11538,
    11548, 11558, 2123, 2124, 11559, 11569, 11579, 434, 1036, 617, 1056, 11580, 251, 0, 251, 764,
    1135, 11582, 11593, 11603, 0, 1061, 209, 11613, 11623, 11632, 11642, 11652, 11662, 11671, 11681, 11691,
    11701, 11711, 11721, 11731, 11741, 11751, 11761, 11771, 11781, 11791, 11801, 11811, 11821, 11831, 11841, 11851,
    11861, 11871, 11881, 11891, 11901, 11911, 11921, 11931, 11941, 11951, 11961, 11971, 11981, 11991, 12001, 12011,
    12021, 12031, 12041, 12051, 12061, 12071, 12081, 12091, 12101, 12111, 12121, 12131, 12141, 12151, 12161, 12171,
    12181, 12191, 12201, 12211, 12221, 12231, 12241, 12251, 12261, 12271, 12281, 12291, 12301, 12311, 12321, 12331,
    12341, 12351, 12361, 12371, 12381, 12391, 12401, 12411, 12421, 12431, 12441, 12451, 12461, 12470, 12480, 12490,
    12500, 12510, 12520, 530, 1812, 1220, 1423, 2195, 2197, 2199, 2201, 2203, 2205, 2207, 2209, 2211,
    2213, 2215, 2217, 2219, 2221, 2223, 2225, 2227, 2229, 1224, 1222, 2231, 2233, 2235, 2237, 2239,
    2241, 2243, 2245, 2247, 2249, 2251, 2253, 2255, 2257, 2259, 2261, 2263, 2265, 2267, 2269, 2271,
    2273, 2275, 2277, 2279, 2281, 2283, 2285, 2287, 2289, 2291, 2293, 2295, 2213, 2297, 2229, 2299,
    2301, 2303, 2305, 2307, 2309, 2311, 2313, 2315, 1571, 1569, 2317, 2319, 2321, 2323, 2325, 2327,
    2329, 2331, 2333, 2335, 1290, 12521, 12531, 12541, 12551, 12561, 12571, 12581, 12591, 12601, 12611, 12621,
    12631, 12641, 12651, 12661, 12671, 12681, 12691, 2344, 12692, 12704, 12702, 0, 580, 268, 0, 12707,
    12717, 12727, 12737, 12747, 12757, 12767, 12777, 12787, 12797, 12807, 12817, 12827, 12837, 12847, 12857, 12867,
    12874, 12884, 12894, 12904, 12914, 12924, 12933, 12943, 12968, 0, 0, 382, 77, 12964, 0, 383,
    12953, 384, 385, 388, 390, 393, 394, 0, 402, 390, 393, 394, 0, 403, 388, 160,
    151, 404, 160, 12956, 0, 163, 12954, 144, 157, 12955, 409, 410, 0, 411, 412, 0,
    414, 415, 0, 411, 240, 0, 416, 12960, 293, 12963, 418, 0, 322, 418, 0, 12965,
    384, 385, 388, 390, 393, 394, 0, 12966, 419, 420, 12967, 384, 385, 388, 393, 394,
    0, 12975, 1267, 611, 203, 266, 12969, 530, 12972, 1812, 12976, 13003, 13002, 0, 2365, 1324,
    232, 12986, 348, 434, 0, 597, 232, 96, 0, 2366, 232, 96, 0, 240, 343, 232,
    12987, 348, 434, 240, 0, 749, 232, 96, 0, 240, 344, 232, 12988, 348, 434, 240,
    0, 815, 232, 348, 0, 2367, 232, 348, 0, 55, 918, 232, 348, 0, 2368, 232,
    96, 0, 2369, 232, 12989, 1545, 414, 0, 2370, 232, 96, 0, 2371, 232, 12990, 1545,
    414, 0, 384, 918, 232, 348, 0, 2372, 232, 12991, 414, 438, 1855, 0, 1540, 232,
    96, 0, 2373, 232, 96, 0, 2374, 232, 96, 0, 2375, 232, 12992, 96, 240, 0,
    2376, 232, 96, 0, 2377, 232, 96, 0, 2378, 232, 96, 0, 957, 232, 348, 0,
    2379, 232, 12993, 240, 2380, 2381, 2382, 2383, 2384, 2385, 0, 171, 232, 96, 0, 2386,
    232, 12994, 240, 96, 0, 2387, 232, 12995, 240, 96, 0, 2388, 232, 12996, 240, 96,
    0, 49, 434, 0, 2389, 232, 96, 0, 2390, 232, 12999, 12997, 266, 0, 507, 219,
    221, 13007, 2391, 13005, 251, 764, 766, 13008, 13018, 13028, 13038, 13048, 13058, 0, 13068, 2393,
    703, 2394, 0, 13072, 2395, 2394, 13076, 13086, 13096, 13106, 0, 13116, 153, 154, 13117, 13127,
    2397, 1175, 13128, 13138, 13148, 13158, 13168, 13178, 13188, 13198, 13208, 13218, 13228, 13237, 0, 13247,
    1070, 797, 798, 0, 13248, 507, 510, 1070, 2403, 13249, 0, 291, 1070, 2403, 293, 13253,
    13263, 13273, 13283, 13293, 13303, 13313, 13323, 13333, 13343, 13353, 13363, 13373, 13383, 13393, 13403, 13413,
    0, 908, 2109, 0, 0, 251, 1693, 0, 952, 2415, 341, 0, 2417, 251, 0, 2418,
    2415, 0, 961, 414, 2419, 0, 2420, 2415, 0, 699, 251, 13414, 13424, 13434, 13444, 13454,
    13464, 13474, 13484, 13494, 13504, 13514, 13524, 0, 1854, 1682, 13534, 13544, 13554, 13564, 13574, 13584,
    2435, 2436, 2437, 13585, 13595, 13606, 0, 156, 165, 266, 13616, 13626, 13638, 13636, 580, 13641,
    0, 1061, 1667, 0, 1189, 2440, 13651, 13661, 13671, 13681, 13691, 0, 13701, 2444, 2445, 2446,
    13702, 13712, 13722, 13732, 13742, 13752, 13762, 13772, 0, 2390, 13782, 1412, 58, 13783, 0, 383,
    385, 13793, 13803, 0, 13813, 13823, 13833, 13843, 0, 2079, 1388, 1125, 1914, 1127, 1122, 2081,
    2083, 1275, 2085, 1386, 1379, 1381, 1383, 227, 1129, 1388, 1386, 1914, 531, 1127, 1571, 0,
    1810, 13853, 0, 1379, 1810, 13857, 13867, 13877, 13887, 0, 157, 1114, 1116, 13898, 13897, 266,
    1136, 1567, 1122, 227, 1129, 13900, 13910, 13920, 0, 163, 2460, 385, 157, 13930, 2102, 13932,
    13942, 0, 139, 274, 13952, 13962, 13972, 251, 13982, 13992, 14002, 14012, 14022, 14032, 14042, 14052,
    14062, 14072, 14082, 14092, 14102, 14112, 14122, 14132, 14142, 14152, 0, 14162, 1104, 934, 14169, 0,
    946, 14166, 14164, 14163, 240, 433, 293, 14171, 14181, 14191, 14201, 14211, 14221, 14231, 0, 617,
    1247, 580, 14241, 14251, 14261, 14271, 14281, 14291, 14301, 14311, 14321, 14331, 14341, 14351, 14361, 14371,
    14381, 0, 2486, 96, 1352, 14382, 14392, 14401, 14411, 14421, 14431, 14441, 14451, 0, 1743, 0,
    293, 14461, 1519, 14464, 293, 14467, 14470, 14473, 14483, 0, 1295, 14496, 14493, 293, 14497, 1129,
    0, 196, 361, 0, 1375, 361, 1376, 1377, 1378, 1866, 1029, 1388, 14498, 14508, 14522, 0,
    0, 251, 1693, 0, 2497, 293, 340, 14518, 341, 0, 699, 2497, 251, 14523, 14530, 0,
    84, 0, 14541, 14540, 0, 251, 764, 0, 1135, 1056, 1135, 14545, 0, 251, 14555, 505,
    1375, 2501, 227, 2503, 0, 507, 14556, 2505, 540, 416, 1487, 0, 507, 2506, 14557, 14560,
    2508, 152, 14562, 2510, 0, 507, 2506, 14565, 14568, 2508, 152, 14570, 2088, 0, 507, 0,
    414, 55, 0, 14573, 0, 507, 414, 46, 2511, 0, 507, 2512, 14583, 14593, 14603, 14610,
    14620, 14630, 14640, 14650, 14660, 14670, 14680, 14690, 14700, 14710, 14720, 14730, 14740, 14750,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 76, 4294967295, 4294967295, 4294967295, 4294967295, 76, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 210, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 229, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245,
    250, 4294967295, 4294967295, 4294967295, 4294967295, 253, 4294967295, 4294967295, 4294967295, 4294967295, 263, 4294967295, 4294967295, 267, 269, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 275, 4294967295, 281, 4294967295, 4294967295, 284, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    338, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 234, 346, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 371, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 389, 392, 4294967295, 401, 4294967295, 4294967295, 4294967295, 392, 4294967295, 401, 4294967295, 4294967295, 4294967295,
    389, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 392, 4294967295, 401, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 389, 4294967295, 401, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 469, 471, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 476, 4294967295, 4294967295, 4294967295, 4294967295, 405,
    436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 389, 4294967295, 4294967295, 405, 4294967295, 405, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 532, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 547, 547, 4294967295, 547, 547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 614, 204, 405, 294, 4294967295, 616, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 620, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 405,
    4294967295, 405, 669, 4294967295, 294, 4294967295, 4294967295, 4294967295, 673, 4294967295, 676, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 253, 4294967295, 4294967295, 684, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 696, 4294967295, 4294967295, 702, 4294967295, 4294967295, 4294967295,
    4294967295, 294, 4294967295, 696, 4294967295, 4294967295, 702, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4,
    4294967295, 4294967295, 346, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 775, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 338, 4294967295, 4294967295, 346, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 669, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 801, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 816, 4294967295, 4294967295, 4294967295, 4294967295, 234, 234, 234, 234,
    234, 234, 4294967295, 4294967295, 234, 234, 4294967295, 4294967295, 547, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 616, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 616, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    80, 83, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 275, 4294967295,
    881, 4294967295, 4294967295, 284, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 338, 4294967295, 342, 4294967295, 4294967295, 284, 4294967295, 4294967295, 80, 83, 4294967295, 4294967295, 616,
    338, 886, 888, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 392, 4294967295, 267,
    4294967295, 893, 401, 4294967295, 896, 898, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 346, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 284,
    4294967295, 4294967295, 4294967295, 80, 904, 338, 886, 888, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295,
    4294967295, 906, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 371, 4294967295, 910, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 476, 436, 338, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 917, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295,
    234, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 942, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 342, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    234, 4294967295, 4294967295, 4294967295, 4294967295, 284, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 532, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 80, 4294967295,
    1013, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 1021, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 669, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 234, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1119, 4294967295, 1121, 1124, 4294967295, 4294967295, 1119, 4294967295, 1126, 1124, 4294967295, 1126, 1128, 1124, 229,
    1131, 1134, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1137, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    616, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 389, 4294967295, 267, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    906, 4294967295, 4294967295, 4294967295, 4294967295, 371, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 942, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1190, 1192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1221,
    1223, 1225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 97, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1252, 4294967295, 1254, 775, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295,
    4294967295, 405, 204, 1257, 294, 4294967295, 85, 4294967295, 4294967295, 1190, 1261, 4294967295, 4294967295, 294, 4294967295, 616,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1268, 1270, 4294967295, 614,
    1272, 1274, 405, 1277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 614, 4294967295, 405, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1137, 4294967295, 4294967295,
    1131, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1380, 1382, 1384, 4294967295, 4294967295, 1387,
    1380, 1382, 532, 1389, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1392, 1394, 269, 4294967295, 4294967295, 1398, 4294967295,
    1401, 4294967295, 4294967295, 4294967295, 269, 1398, 4294967295, 4294967295, 4294967295, 4294967295, 1401, 4294967295, 4294967295, 4294967295, 4294967295, 1405,
    1407, 4294967295, 1137, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 1401, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 263, 4294967295, 4294967295, 4294967295, 294, 4294967295, 294, 4294967295, 294, 4294967295, 616,
    4294967295, 4294967295, 294, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 294, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 405,
    4294967295, 1421, 1272, 4294967295, 4294967295, 4294967295, 1424, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 673, 4294967295,
    4294967295, 4294967295, 673, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 616,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 1021, 4294967295, 80, 1021, 4294967295, 80, 1021, 4294967295, 80, 1021,
    4294967295, 4294967295, 4294967295, 80, 4294967295, 80, 80, 1268, 1021, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1484, 4294967295, 1486, 4294967295, 200, 4294967295, 4294967295, 200,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 1494, 4294967295,
    4294967295, 4294967295, 4294967295, 1131, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 294,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1501, 1013, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 1131, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1169, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295,
    342, 4294967295, 4294967295, 294, 4294967295, 342, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 1539, 4294967295, 4294967295, 4294967295, 4294967295,
    234, 4294967295, 392, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 392, 4294967295, 4294967295, 4294967295, 78, 338,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 253,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 253, 4294967295, 4294967295, 1268,
    1568, 4294967295, 1134, 1570, 1572, 1134, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1407, 4294967295,
    4294967295, 4294967295, 245, 1619, 4294967295, 1539, 4294967295, 263, 4294967295, 4294967295, 4294967295, 1407, 1619, 4294967295, 4294967295, 1407,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 294, 263, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1638, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294,
    4294967295, 4294967295, 1638, 4294967295, 4294967295, 294, 4294967295, 294, 4294967295, 4294967295, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 97, 4294967295, 4294967295, 1668, 1670, 1673, 4294967295,
    4294967295, 588, 588, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 267, 1128, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1740, 1742, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295,
    4294967295, 1746, 1748, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 1750, 4294967295, 4294967295, 4294967295, 267,
    4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 204, 269, 245, 1539, 263, 1754,
    4294967295, 4294967295, 1568, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1776, 4294967295, 4294967295, 4294967295, 1778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1792, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1268, 1270, 4294967295, 1270, 4294967295, 1809, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1223, 1221, 1225, 1813,
    4294967295, 676, 4294967295, 4294967295, 1270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1832, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1857, 1131, 4294967295, 4294967295, 1131, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1776, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1912, 4294967295, 4294967295, 1915, 1128, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 616, 616, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 267, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 1128, 4294967295, 4294967295, 4294967295, 1268, 1568, 4294967295, 4294967295, 1270,
    1134, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2023, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1484, 4294967295, 1486, 4294967295, 4294967295, 200,
    4294967295, 4294967295, 4294967295, 4294967295, 198, 198, 4294967295, 2027, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 2061, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2078, 4294967295, 2023, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1131, 4294967295, 1380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 2080, 1389,
    1126, 1915, 1128, 1124, 2082, 2084, 1277, 2086, 1387, 229, 1131, 2086, 1389, 1380, 1572, 532,
    1134, 4294967295, 4294967295, 4294967295, 1809, 4294967295, 4294967295, 1809, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 294, 4294967295, 4294967295, 4294967295, 2103, 4294967295, 405, 234, 4294967295, 4294967295,
    4294967295, 204, 405, 294, 1268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295,
    4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 1380, 4294967295, 229, 1131, 2086, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 346, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 906, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1252, 4294967295, 4294967295, 4294967295, 616, 1254, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1062, 210, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1813, 1221, 1424, 2196, 2198, 2200, 2202, 2204, 2206, 2208, 2210, 2212,
    2214, 2216, 2218, 2220, 2222, 2224, 2226, 2228, 2230, 1225, 1223, 2232, 2234, 2236, 2238, 2240,
    2242, 2244, 2246, 2248, 2250, 2252, 2254, 2256, 2258, 2260, 2262, 2264, 2266, 2268, 2270, 2272,
    2274, 2276, 2278, 2280, 2282, 2284, 2286, 2288, 2290, 2292, 2294, 2296, 2214, 2298, 2230, 2300,
    2302, 2304, 2306, 2308, 2310, 2312, 2314, 2316, 1572, 1570, 2318, 2320, 2322, 2324, 2326, 2328,
    2330, 2332, 2334, 2336, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 547, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 389, 392, 4294967295, 401, 4294967295, 4294967295, 392, 4294967295, 401, 4294967295, 4294967295, 389, 4294967295,
    4294967295, 405, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 389, 392, 4294967295, 401, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 389, 4294967295, 401,
    4294967295, 4294967295, 1268, 4294967295, 204, 267, 4294967295, 4294967295, 4294967295, 1813, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 234,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234,
    4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234,
    4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295,
    4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295,
    234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 4294967295, 234, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 669, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1062, 1668, 4294967295, 1190, 1192, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2080, 1389, 1126, 1915, 1128, 1124, 2082,
    2084, 1277, 2086, 1387, 1380, 1382, 1384, 229, 1131, 1389, 1387, 1915, 532, 1128, 1572, 4294967295,
    4294967295, 4294967295, 4294967295, 1380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 267,
    1137, 1568, 1124, 229, 1131, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2103, 4294967295,
    4294967295, 4294967295, 4294967295, 275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    294, 245, 4294967295, 245, 294, 263, 263, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 1131,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1389, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 2498, 4294967295, 4294967295,
    85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2502, 229, 2504, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295,
    2509, 4294967295, 263, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 2509, 4294967295, 263, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 5, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 79, 81, 84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 64, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 161, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 191, 193, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 197, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 77, 4294967295, 199, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 227, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246,
    248, 4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 266, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 274, 4294967295, 280, 4294967295, 4294967295, 282, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    336, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 232, 345, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 370, 4294967295, 372, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 388, 390, 4294967295, 394, 4294967295, 4294967295, 4294967295, 390, 4294967295, 394, 4294967295, 4294967295, 4294967295,
    388, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 388, 390, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 388, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 468, 470, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 475, 4294967295, 4294967295, 4294967295, 4294967295, 404,
    435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 388, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 388, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    266, 4294967295, 4294967295, 4294967295, 4294967295, 388, 4294967295, 388, 4294967295, 4294967295, 404, 4294967295, 404, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 531, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 567, 568, 4294967295, 570, 571, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 612, 203, 404, 293, 4294967295, 615, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 621, 622, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 548, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 404,
    4294967295, 404, 668, 4294967295, 293, 4294967295, 4294967295, 4294967295, 674, 4294967295, 675, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 683, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 693, 4294967295, 695, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295,
    4294967295, 293, 4294967295, 695, 4294967295, 4294967295, 701, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 610,
    4294967295, 4294967295, 345, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 774, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 336, 4294967295, 4294967295, 345, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 615, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 798, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 799, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 799, 4294967295, 4294967295, 4294967295, 4294967295, 800, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 817, 4294967295, 4294967295, 4294967295, 4294967295, 232, 821, 822, 823,
    824, 825, 4294967295, 4294967295, 828, 829, 4294967295, 4294967295, 831, 832, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 615, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 615, 4294967295, 615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    79, 81, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 274, 4294967295,
    880, 4294967295, 4294967295, 282, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 336, 4294967295, 341, 4294967295, 4294967295, 282, 4294967295, 4294967295, 79, 81, 4294967295, 4294967295, 615,
    336, 885, 887, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 388, 889, 4294967295, 266,
    4294967295, 890, 894, 4294967295, 895, 897, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 345, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 282,
    4294967295, 4294967295, 4294967295, 79, 903, 336, 885, 887, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295,
    4294967295, 905, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 370, 4294967295, 909, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 914, 435, 336, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 916, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295,
    232, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266,
    4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295, 941, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 341, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    232, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 531, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1010, 1011, 4294967295,
    1012, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1019, 1020, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 798, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1061, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 615, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 232, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1118, 4294967295, 1120, 1122, 4294967295, 4294967295, 1118, 4294967295, 1125, 1122, 4294967295, 1125, 1127, 1122, 227,
    1129, 1132, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1136, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    615, 4294967295, 615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 274, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 388, 4294967295, 266, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    905, 4294967295, 4294967295, 4294967295, 4294967295, 370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 941, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295,
    4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1189, 1191, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1220,
    1222, 1224, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1251, 4294967295, 1253, 774, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295,
    4294967295, 404, 203, 1256, 293, 4294967295, 1259, 4294967295, 4294967295, 1189, 1260, 4294967295, 4294967295, 293, 4294967295, 615,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1267, 1269, 4294967295, 612,
    1271, 1273, 404, 1275, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 612, 4294967295, 404, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1136, 4294967295, 4294967295,
    1129, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1379, 1381, 1383, 4294967295, 4294967295, 1386,
    1379, 1381, 531, 1388, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1391, 1393, 268, 4294967295, 4294967295, 1396, 4294967295,
    1400, 4294967295, 4294967295, 4294967295, 268, 1396, 4294967295, 4294967295, 4294967295, 4294967295, 1400, 4294967295, 4294967295, 4294967295, 4294967295, 1403,
    1406, 4294967295, 1136, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 1400, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 264, 4294967295, 4294967295, 4294967295, 293, 4294967295, 293, 4294967295, 293, 4294967295, 615,
    4294967295, 4294967295, 293, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 293, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 404,
    4294967295, 1420, 1422, 4294967295, 4294967295, 4294967295, 1423, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 674, 4294967295,
    4294967295, 4294967295, 674, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 615, 4294967295, 4294967295, 4294967295, 4294967295, 615,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1430, 1431, 4294967295, 1433, 1434, 4294967295, 1435, 1436, 4294967295, 1437, 1438,
    4294967295, 4294967295, 4294967295, 79, 4294967295, 1437, 1440, 1267, 1441, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1483, 4294967295, 1485, 4294967295, 199, 4294967295, 4294967295, 199,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 1493, 4294967295,
    4294967295, 4294967295, 4294967295, 1129, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 293,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1500, 1012, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 1129, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1170, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295,
    341, 4294967295, 4294967295, 293, 4294967295, 341, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 1538, 4294967295, 4294967295, 4294967295, 4294967295,
    232, 4294967295, 889, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 390, 4294967295, 4294967295, 4294967295, 77, 336,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 1267,
    1567, 4294967295, 1132, 1569, 1571, 1132, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1406, 4294967295,
    4294967295, 4294967295, 246, 1618, 4294967295, 1538, 4294967295, 264, 4294967295, 4294967295, 4294967295, 1406, 1618, 4294967295, 4294967295, 1406,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 693, 4294967295, 4294967295, 293, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1637, 4294967295, 4294967295, 615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293,
    4294967295, 4294967295, 1637, 4294967295, 4294967295, 293, 4294967295, 293, 4294967295, 4294967295, 615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1243, 4294967295, 4294967295, 1667, 1669, 1671, 4294967295,
    4294967295, 1282, 589, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 266, 1127, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1739, 1741, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295,
    4294967295, 1745, 1747, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 1749, 4294967295, 4294967295, 4294967295, 266,
    4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 203, 268, 246, 1538, 264, 1753,
    4294967295, 4294967295, 1567, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1775, 4294967295, 4294967295, 4294967295, 1777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1793, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1267, 1807, 4294967295, 1269, 4294967295, 1808, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1222, 1220, 1224, 1812,
    4294967295, 1814, 4294967295, 4294967295, 1269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1831, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1856, 1129, 4294967295, 4294967295, 1129, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1775, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1911, 4294967295, 4294967295, 1914, 1127, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 615, 615, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 266, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 1127, 4294967295, 4294967295, 4294967295, 1267, 1567, 4294967295, 4294967295, 1269,
    1132, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2022, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1483, 4294967295, 1485, 4294967295, 4294967295, 199,
    4294967295, 4294967295, 4294967295, 4294967295, 197, 197, 4294967295, 2026, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 2060, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2077, 4294967295, 2022, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1129, 4294967295, 1379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 2079, 1388,
    1125, 1914, 1127, 1122, 2081, 2083, 1275, 2085, 1386, 227, 1129, 2085, 1388, 1379, 1571, 531,
    1132, 4294967295, 4294967295, 4294967295, 1808, 4294967295, 4294967295, 2087, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 293, 4294967295, 4294967295, 4294967295, 2102, 4294967295, 404, 232, 4294967295, 4294967295,
    4294967295, 203, 404, 293, 1267, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295,
    4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 1379, 4294967295, 227, 1129, 2085, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 345, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 905, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1251, 4294967295, 4294967295, 4294967295, 615, 1253, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1061, 209, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1812, 1220, 1423, 2195, 2197, 2199, 2201, 2203, 2205, 2207, 2209, 2211,
    2213, 2215, 2217, 2219, 2221, 2223, 2225, 2227, 2229, 1224, 1222, 2231, 2233, 2235, 2237, 2239,
    2241, 2243, 2245, 2247, 2249, 2251, 2253, 2255, 2257, 2259, 2261, 2263, 2265, 2267, 2269, 2271,
    2273, 2275, 2277, 2279, 2281, 2283, 2285, 2287, 2289, 2291, 2293, 2295, 2213, 2297, 2229, 2299,
    2301, 2303, 2305, 2307, 2309, 2311, 2313, 2315, 1571, 1569, 2317, 2319, 2321, 2323, 2325, 2327,
    2329, 2331, 2333, 2335, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2344, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 388, 390, 4294967295, 394, 4294967295, 4294967295, 390, 4294967295, 394, 4294967295, 4294967295, 388, 4294967295,
    4294967295, 404, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 388, 390, 4294967295, 394, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 388, 4294967295, 394,
    4294967295, 4294967295, 1267, 4294967295, 203, 266, 4294967295, 4294967295, 4294967295, 1812, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 232,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232,
    4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232,
    4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295,
    4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295,
    232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 4294967295, 232, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 798, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1061, 1667, 4294967295, 1189, 2440, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2079, 1388, 1125, 1914, 1127, 1122, 2081,
    2083, 1275, 2085, 1386, 1379, 1381, 1383, 227, 1129, 1388, 1386, 1914, 531, 1127, 1571, 4294967295,
    4294967295, 4294967295, 4294967295, 1379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266,
    1136, 1567, 1122, 227, 1129, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2102, 4294967295,
    4294967295, 4294967295, 4294967295, 274, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    293, 246, 4294967295, 693, 293, 264, 264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 1129,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1388, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 293, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 2499, 4294967295, 4294967295,
    84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2501, 227, 2503, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295,
    2508, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 2508, 4294967295, 264, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1425, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
    (2119, 0, 1),
    (2120, 1, 1),
    (4141, 2, 1),
    (4722, 3, 1),
];

pub static TRIM_CHARS_DATA: &[u32] = &[
    1172, 1172, 1942, 2126,
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (210, 38, 2),
    (227, 40, 2),
    (231, 42, 2),
    (239, 44, 4),
    (240, 48, 2),
    (245, 50, 2),
    (250, 52, 5),
    (253, 57, 3),
    (254, 60, 2),
    (263, 62, 3),
    (265, 65, 2),
    (268, 67, 2),
    (270, 69, 2),
    (277, 71, 2),
    (285, 73, 2),
    (294, 75, 2),
    (304, 77, 2),
    (307, 79, 2),
    (314, 81, 2),
    (323, 83, 2),
    (324, 85, 3),
    (328, 88, 2),
    (333, 90, 2),
    (356, 92, 3),
    (358, 95, 2),
    (363, 97, 2),
    (371, 99, 3),
    (372, 102, 2),
    (374, 104, 2),
    (378, 106, 2),
    (380, 108, 2),
    (384, 110, 3),
    (387, 113, 3),
    (410, 116, 2),
    (421, 118, 3),
    (422, 121, 2),
    (424, 123, 2),
    (433, 125, 3),
    (435, 128, 2),
    (444, 130, 2),
    (452, 132, 2),
    (461, 134, 2),
    (470, 136, 2),
    (476, 138, 3),
    (484, 141, 3),
    (488, 144, 3),
    (491, 147, 2),
    (492, 149, 2),
    (498, 151, 2),
    (503, 153, 3),
    (516, 156, 2),
    (529, 158, 2),
    (534, 160, 3),
    (538, 163, 2),
    (543, 165, 3),
    (544, 168, 2),
    (551, 170, 3),
    (558, 173, 3),
    (564, 176, 3),
    (569, 179, 2),
    (575, 181, 2),
    (581, 183, 2),
    (593, 185, 3),
    (608, 188, 3),
    (613, 191, 3),
    (615, 194, 3),
    (618, 197, 3),
    (620, 200, 3),
    (630, 203, 2),
    (656, 205, 2),
    (670, 207, 2),
    (675, 209, 2),
    (680, 211, 2),
    (688, 213, 3),
    (694, 216, 2),
    (708, 218, 3),
    (710, 221, 3),
    (715, 224, 3),
    (721, 227, 3),
    (726, 230, 3),
    (740, 233, 2),
    (741, 235, 2),
    (743, 237, 2),
    (744, 239, 2),
    (751, 241, 2),
    (792, 243, 2),
    (793, 245, 2),
    (794, 247, 3),
    (795, 250, 2),
    (797, 252, 2),
    (804, 254, 2),
    (805, 256, 1),
    (819, 257, 3),
    (858, 260, 3),
    (863, 263, 3),
    (865, 266, 3),
    (866, 269, 2),
    (868, 271, 2),
    (872, 273, 2),
    (874, 275, 2),
    (884, 277, 2),
    (887, 279, 2),
    (895, 281, 2),
    (903, 283, 4),
    (905, 287, 2),
    (908, 289, 2),
    (913, 291, 2),
    (915, 293, 2),
    (918, 295, 2),
    (988, 297, 3),
    (1005, 300, 3),
    (1023, 303, 2),
    (1026, 305, 3),
    (1045, 308, 2),
    (1047, 310, 2),
    (1054, 312, 2),
    (1062, 314, 2),
    (1074, 316, 2),
    (1077, 318, 3),
    (1093, 321, 2),
    (1109, 323, 2),
    (1117, 325, 2),
    (1125, 327, 2),
    (1134, 329, 2),
    (1143, 331, 2),
    (1148, 333, 2),
    (1175, 335, 3),
    (1180, 338, 2),
    (1181, 340, 2),
    (1182, 342, 2),
    (1183, 344, 2),
    (1184, 346, 2),
    (1185, 348, 2),
    (1188, 350, 2),
    (1189, 352, 3),
    (1192, 355, 2),
    (1193, 357, 2),
    (1226, 359, 2),
    (1230, 361, 2),
    (1249, 363, 2),
    (1251, 365, 2),
    (1264, 367, 2),
    (1265, 369, 2),
    (1268, 371, 2),
    (1278, 373, 3),
    (1280, 376, 2),
    (1283, 378, 2),
    (1286, 380, 2),
    (1292, 382, 2),
    (1300, 384, 2),
    (1315, 386, 2),
    (1317, 388, 2),
    (1320, 390, 2),
    (1323, 392, 2),
    (1324, 394, 2),
    (1327, 396, 2),
    (1328, 398, 2),
    (1329, 400, 2),
    (1330, 402, 2),
    (1340, 404, 3),
    (1341, 407, 2),
    (1343, 409, 3),
    (1345, 412, 2),
    (1346, 414, 2),
    (1348, 416, 2),
    (1349, 418, 2),
    (1356, 420, 2),
    (1363, 422, 2),
    (1377, 424, 2),
    (1383, 426, 2),
    (1384, 428, 3),
    (1387, 431, 2),
    (1391, 433, 2),
    (1395, 435, 2),
    (1396, 437, 2),
    (1397, 439, 2),
    (1398, 441, 2),
    (1399, 443, 2),
    (1404, 445, 3),
    (1409, 448, 3),
    (1433, 451, 3),
    (1435, 454, 2),
    (1442, 456, 3),
    (1448, 459, 2),
    (1449, 461, 2),
    (1450, 463, 2),
    (1463, 465, 3),
    (1464, 468, 2),
    (1479, 470, 2),
    (1489, 472, 3),
    (1499, 475, 2),
    (1504, 477, 2),
    (1507, 479, 3),
    (1513, 482, 2),
    (1516, 484, 3),
    (1528, 487, 2),
    (1533, 489, 2),
    (1536, 491, 3),
    (1545, 494, 3),
    (1557, 497, 2),
    (1558, 499, 3),
    (1567, 502, 2),
    (1580, 504, 2),
    (1590, 506, 3),
    (1608, 509, 2),
    (1621, 511, 2),
    (1631, 513, 3),
    (1635, 516, 2),
    (1638, 518, 3),
    (1647, 521, 3),
    (1658, 524, 2),
    (1661, 526, 2),
    (1670, 528, 3),
    (1685, 531, 2),
    (1712, 533, 2),
    (1720, 535, 3),
    (1808, 538, 2),
    (1813, 540, 2),
    (1821, 542, 3),
    (1827, 545, 2),
    (1837, 547, 2),
    (1838, 549, 2),
    (1840, 551, 2),
    (1851, 553, 2),
    (1852, 555, 2),
    (1917, 557, 2),
    (1929, 559, 2),
    (1978, 561, 2),
    (2002, 563, 2),
    (2014, 565, 2),
    (2020, 567, 2),
    (2033, 569, 2),
    (2034, 571, 3),
    (2049, 574, 2),
    (2051, 576, 2),
    (2052, 578, 2),
    (2055, 580, 2),
    (2057, 582, 2),
    (2058, 584, 2),
    (2060, 586, 2),
    (2061, 588, 2),
    (2062, 590, 2),
    (2063, 592, 2),
    (2064, 594, 2),
    (2065, 596, 2),
    (2067, 598, 3),
    (2074, 601, 2),
    (2093, 603, 2),
    (2128, 605, 2),
    (2130, 607, 2),
    (2136, 609, 3),
    (2142, 612, 2),
    (2147, 614, 2),
    (2151, 616, 3),
    (2153, 619, 3),
    (2157, 622, 2),
    (2164, 624, 2),
    (2175, 626, 2),
    (2182, 628, 2),
    (2192, 630, 3),
    (2197, 633, 3),
    (2204, 636, 3),
    (2210, 639, 3),
    (2215, 642, 2),
    (2221, 644, 3),
    (2226, 647, 3),
    (2241, 650, 2),
    (2248, 652, 2),
    (2249, 654, 2),
    (2303, 656, 2),
    (2304, 658, 2),
    (2305, 660, 2),
    (2337, 662, 2),
    (2354, 664, 2),
    (2368, 666, 2),
    (2370, 668, 2),
    (2371, 670, 2),
    (2376, 672, 2),
    (2380, 674, 2),
    (2385, 676, 3),
    (2386, 679, 2),
    (2387, 681, 2),
    (2388, 683, 2),
    (2390, 685, 3),
    (2393, 688, 2),
    (2394, 690, 2),
    (2397, 692, 2),
    (2399, 694, 2),
    (2406, 696, 2),
    (2412, 698, 2),
    (2413, 700, 2),
    (2415, 702, 2),
    (2416, 704, 2),
    (2417, 706, 2),
    (2418, 708, 3),
    (2419, 711, 2),
    (2440, 713, 2),
    (2457, 715, 2),
    (2469, 717, 2),
    (2521, 719, 2),
    (2540, 721, 3),
    (2554, 724, 3),
    (2602, 727, 2),
    (2604, 729, 3),
    (2621, 732, 2),
    (2624, 734, 2),
    (2634, 736, 2),
    (2635, 738, 2),
    (2636, 740, 2),
    (2639, 742, 2),
    (2640, 744, 2),
    (2641, 746, 2),
    (2642, 748, 2),
    (2643, 750, 2),
    (2649, 752, 2),
    (2650, 754, 2),
    (2651, 760, 2),
    (2654, 756, 2),
    (2656, 758, 2),
    (2660, 762, 2),
    (2661, 764, 2),
    (2666, 766, 2),
    (2671, 768, 2),
    (2672, 770, 2),
    (2674, 772, 2),
    (2685, 774, 2),
    (2697, 776, 2),
    (2699, 778, 2),
    (2701, 780, 2),
    (2703, 782, 2),
    (2706, 784, 2),
    (2709, 786, 2),
    (2713, 788, 2),
    (2715, 790, 2),
    (2722, 792, 2),
    (2730, 794, 3),
    (2735, 797, 3),
    (2737, 800, 2),
    (2738, 802, 2),
    (2742, 804, 2),
    (2750, 806, 2),
    (2754, 808, 2),
    (2762, 810, 2),
    (2767, 812, 2),
    (2773, 814, 2),
    (2774, 816, 2),
    (2776, 818, 2),
    (2777, 820, 2),
    (2779, 822, 2),
    (2780, 824, 2),
    (2782, 826, 2),
    (2783, 828, 2),
    (2787, 836, 2),
    (2789, 830, 2),
    (2790, 832, 2),
    (2791, 834, 2),
    (2792, 838, 2),
    (2814, 840, 2),
    (2828, 842, 2),
    (2856, 844, 2),
    (2858, 846, 2),
    (2860, 848, 3),
    (2863, 851, 3),
    (2887, 854, 3),
    (2889, 857, 2),
    (2894, 859, 2),
    (2899, 861, 2),
    (2909, 863, 3),
    (2911, 866, 2),
    (2921, 868, 2),
    (2922, 870, 2),
    (2932, 872, 2),
    (2937, 874, 2),
    (2963, 876, 3),
    (2974, 879, 2),
    (2976, 881, 2),
    (2979, 883, 2),
    (2981, 885, 2),
    (2984, 887, 2),
    (3019, 889, 2),
    (3050, 891, 2),
    (3051, 893, 2),
    (3056, 895, 2),
    (3058, 897, 2),
    (3062, 899, 2),
    (3068, 901, 2),
    (3080, 903, 2),
    (3082, 905, 2),
    (3086, 907, 2),
    (3087, 909, 2),
    (3093, 911, 2),
    (3097, 913, 2),
    (3135, 915, 2),
    (3144, 917, 3),
    (3148, 920, 2),
    (3151, 922, 2),
    (3152, 924, 2),
    (3154, 926, 2),
    (3155, 928, 2),
    (3156, 930, 2),
    (3157, 932, 2),
    (3181, 934, 2),
    (3262, 936, 2),
    (3267, 938, 2),
    (3269, 940, 2),
    (3275, 942, 2),
    (3276, 944, 2),
    (3279, 946, 2),
    (3289, 948, 2),
    (3318, 950, 2),
    (3321, 952, 2),
    (3343, 954, 2),
    (3346, 956, 2),
    (3349, 958, 2),
    (3351, 960, 2),
    (3354, 962, 2),
    (3366, 964, 2),
    (3381, 966, 4),
    (3426, 970, 2),
    (3433, 972, 2),
    (3449, 974, 2),
    (3452, 976, 2),
    (3453, 978, 2),
    (3454, 980, 2),
    (3457, 982, 5),
    (3458, 987, 3),
    (3475, 990, 3),
    (3476, 993, 2),
    (3518, 995, 3),
    (3533, 998, 2),
    (3555, 1000, 2),
    (3570, 1002, 2),
    (3633, 1004, 2),
    (3639, 1006, 2),
    (3640, 1008, 2),
    (3646, 1010, 2),
    (3649, 1012, 2),
    (3650, 1014, 2),
    (3657, 1016, 2),
    (3659, 1018, 2),
    (3663, 1020, 3),
    (3667, 1023, 3),
    (3672, 1026, 2),
    (3674, 1028, 2),
    (3675, 1030, 2),
    (3677, 1032, 2),
    (3679, 1034, 2),
    (3682, 1036, 2),
    (3732, 1038, 2),
    (3736, 1040, 2),
    (3756, 1042, 3),
    (3777, 1045, 2),
    (3778, 1047, 2),
    (3780, 1049, 2),
    (3782, 1051, 2),
    (3788, 1053, 2),
    (3789, 1055, 2),
    (3790, 1057, 2),
    (3791, 1059, 2),
    (3793, 1061, 2),
    (3796, 1063, 2),
    (3846, 1065, 2),
    (3875, 1067, 2),
    (3906, 1069, 3),
    (3925, 1072, 3),
    (3927, 1075, 2),
    (3936, 1077, 2),
    (3937, 1079, 2),
    (3940, 1081, 2),
    (3966, 1083, 2),
    (3976, 1085, 3),
    (4010, 1088, 2),
    (4053, 1090, 2),
    (4056, 1092, 2),
    (4057, 1094, 2),
    (4097, 1096, 2),
    (4098, 1098, 2),
    (4122, 1100, 2),
    (4174, 1102, 3),
    (4236, 1105, 3),
    (4238, 1108, 3),
    (4248, 1111, 3),
    (4261, 1114, 2),
    (4264, 1116, 2),
    (4278, 1118, 3),
    (4279, 1121, 2),
    (4283, 1123, 2),
    (4284, 1125, 2),
    (4287, 1127, 2),
    (4288, 1129, 2),
    (4339, 1131, 2),
    (4355, 1133, 2),
    (4363, 1135, 2),
    (4378, 1137, 2),
    (4380, 1139, 2),
    (4383, 1141, 3),
    (4388, 1144, 3),
    (4389, 1147, 3),
    (4391, 1150, 2),
    (4468, 1152, 2),
    (4471, 1154, 2),
    (4487, 1156, 2),
    (4489, 1158, 2),
    (4499, 1160, 2),
    (4501, 1162, 2),
    (4508, 1164, 2),
    (4510, 1166, 2),
    (4511, 1168, 2),
    (4512, 1170, 2),
    (4513, 1172, 2),
    (4514, 1174, 2),
    (4515, 1176, 2),
    (4516, 1178, 2),
    (4517, 1180, 2),
    (4518, 1182, 2),
    (4519, 1184, 2),
    (4520, 1186, 2),
    (4521, 1188, 2),
    (4522, 1190, 2),
    (4523, 1192, 2),
    (4524, 1194, 2),
    (4525, 1196, 2),
    (4526, 1198, 2),
    (4527, 1200, 2),
    (4528, 1202, 2),
    (4532, 1204, 2),
    (4535, 1206, 2),
    (4586, 1208, 2),
    (4596, 1210, 2),
    (4598, 1212, 2),
    (4602, 1214, 2),
    (4604, 1216, 3),
    (4605, 1219, 2),
    (4609, 1221, 2),
    (4610, 1223, 3),
    (4611, 1226, 2),
    (4612, 1228, 2),
    (4622, 1230, 3),
    (4625, 1233, 3),
    (4630, 1236, 2),
    (4632, 1238, 2),
    (4633, 1240, 2),
    (4634, 1242, 2),
    (4645, 1244, 3),
    (4658, 1247, 3),
    (4695, 1250, 2),
    (4699, 1252, 2),
    (4700, 1254, 2),
    (4725, 1256, 2),
    (4726, 1258, 2),
    (4820, 1260, 2),
    (4821, 1262, 2),
    (4822, 1264, 2),
    (4823, 1266, 2),
    (4824, 1268, 2),
    (4825, 1270, 2),
    (4826, 1272, 2),
    (4827, 1274, 2),
    (4828, 1276, 2),
    (4829, 1278, 2),
    (4830, 1280, 2),
    (4831, 1282, 2),
    (4832, 1284, 2),
    (4833, 1286, 2),
    (4834, 1288, 2),
    (4835, 1290, 2),
    (4836, 1292, 2),
    (4837, 1294, 2),
    (4838, 1296, 2),
    (4839, 1298, 2),
    (4840, 1300, 2),
    (4841, 1302, 2),
    (4842, 1304, 2),
    (4843, 1306, 2),
    (4844, 1308, 2),
    (4845, 1310, 2),
    (4846, 1312, 2),
    (4847, 1314, 2),
    (4848, 1316, 2),
    (4849, 1318, 2),
    (4850, 1320, 2),
    (4851, 1322, 2),
    (4852, 1324, 2),
    (4853, 1326, 2),
    (4854, 1328, 2),
    (4855, 1330, 2),
    (4856, 1332, 2),
    (4857, 1334, 2),
    (4858, 1336, 2),
    (4859, 1338, 2),
    (4860, 1340, 2),
    (4861, 1342, 2),
    (4862, 1344, 2),
    (4863, 1346, 2),
    (4864, 1348, 2),
    (4865, 1350, 2),
    (4866, 1352, 2),
    (4867, 1354, 2),
    (4868, 1356, 2),
    (4869, 1358, 2),
    (4870, 1360, 2),
    (4871, 1362, 2),
    (4872, 1364, 2),
    (4873, 1366, 2),
    (4874, 1368, 2),
    (4875, 1370, 2),
    (4876, 1372, 2),
    (4877, 1374, 2),
    (4878, 1376, 2),
    (4879, 1378, 2),
    (4880, 1380, 2),
    (4881, 1382, 2),
    (4882, 1384, 2),
    (4883, 1386, 2),
    (4884, 1388, 2),
    (4885, 1390, 2),
    (4886, 1392, 2),
    (4887, 1394, 2),
    (4888, 1396, 2),
    (4889, 1398, 2),
    (4890, 1400, 2),
    (4891, 1402, 2),
    (4892, 1404, 2),
    (4893, 1406, 2),
    (4894, 1408, 2),
    (4895, 1410, 2),
    (4896, 1412, 2),
    (4897, 1414, 2),
    (4898, 1416, 2),
    (4899, 1418, 2),
    (4919, 1420, 2),
    (4925, 1422, 2),
    (4943, 1424, 3),
    (4956, 1427, 2),
    (4963, 1429, 3),
    (4964, 1432, 2),
    (4966, 1434, 2),
    (4969, 1436, 2),
    (4971, 1438, 2),
    (4974, 1440, 3),
    (4977, 1443, 3),
    (5000, 1446, 2),
    (5010, 1448, 3),
    (5011, 1451, 2),
    (5013, 1453, 2),
    (5021, 1455, 3),
    (5023, 1458, 2),
    (5026, 1460, 2),
    (5028, 1462, 2),
    (5029, 1464, 3),
    (5033, 1467, 2),
    (5040, 1469, 2),
    (5046, 1471, 2),
    (5050, 1473, 2),
    (5055, 1475, 2),
    (5062, 1477, 2),
    (5067, 1479, 2),
    (5074, 1481, 2),
    (5078, 1483, 2),
    (5083, 1485, 2),
    (5087, 1487, 2),
    (5091, 1489, 2),
    (5097, 1491, 2),
    (5101, 1493, 2),
    (5108, 1495, 2),
    (5112, 1497, 2),
    (5119, 1499, 2),
    (5123, 1501, 2),
    (5127, 1503, 2),
    (5131, 1505, 2),
    (5137, 1507, 2),
    (5141, 1509, 2),
    (5145, 1511, 2),
    (5149, 1513, 2),
    (5153, 1515, 2),
    (5164, 1517, 2),
    (5168, 1519, 2),
    (5174, 1521, 2),
    (5180, 1523, 2),
    (5189, 1525, 2),
    (5193, 1527, 2),
    (5196, 1529, 3),
    (5250, 1532, 2),
    (5262, 1534, 2),
    (5290, 1536, 2),
    (5337, 1538, 3),
    (5345, 1541, 2),
    (5346, 1543, 2),
    (5348, 1545, 2),
    (5349, 1547, 2),
    (5385, 1549, 2),
    (5386, 1551, 2),
    (5387, 1553, 2),
    (5388, 1555, 2),
    (5389, 1557, 2),
    (5390, 1559, 2),
    (5391, 1561, 2),
    (5392, 1563, 2),
    (5393, 1565, 2),
    (5394, 1567, 2),
    (5395, 1569, 2),
    (5396, 1571, 2),
    (5397, 1573, 2),
    (5398, 1575, 2),
    (5399, 1577, 2),
    (5400, 1579, 2),
    (5401, 1581, 2),
    (5402, 1583, 2),
    (5403, 1585, 2),
    (5404, 1587, 2),
    (5405, 1589, 2),
    (5406, 1591, 2),
    (5411, 1593, 2),
    (5423, 1595, 3),
    (5424, 1598, 2),
    (5425, 1600, 2),
    (5426, 1602, 2),
    (5427, 1604, 2),
    (5428, 1606, 2),
    (5438, 1608, 2),
    (5443, 1610, 3),
    (5478, 1613, 2),
    (5520, 1615, 2),
    (5524, 1617, 2),
    (5533, 1619, 2),
    (5535, 1621, 2),
    (5547, 1623, 2),
    (5557, 1625, 2),
    (5560, 1627, 2),
    (5565, 1629, 3),
    (5568, 1632, 3),
    (5585, 1635, 2),
    (5586, 1637, 2),
    (5587, 1639, 2),
    (5600, 1641, 2),
    (5609, 1643, 2),
    (5630, 1645, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    65, 3, 76, 3, 76, 3, 3, 78, 3, 80, 3, 83, 3, 78, 85, 65,
    3, 162, 3, 3, 192, 3, 194, 3, 78, 3, 78, 198, 3, 78, 200, 3,
    78, 3, 78, 200, 3, 204, 210, 3, 3, 229, 3, 234, 3, 245, 247, 4,
    250, 3, 3, 253, 3, 263, 245, 247, 4, 3, 78, 267, 269, 3, 3, 275,
    78, 281, 3, 3, 284, 3, 78, 3, 294, 3, 294, 3, 78, 3, 338, 3,
    342, 3, 234, 3, 234, 3, 346, 78, 3, 234, 3, 234, 3, 78, 371, 373,
    3, 3, 78, 3, 389, 78, 3, 392, 3, 401, 3, 392, 3, 401, 3, 389,
    78, 3, 405, 78, 3, 294, 3, 389, 78, 3, 392, 3, 401, 3, 389, 78,
    3, 401, 3, 234, 3, 234, 3, 234, 3, 436, 3, 405, 78, 3, 78, 267,
    3, 78, 267, 3, 469, 471, 3, 3, 436, 3, 405, 78, 3, 294, 3, 436,
    3, 405, 78, 3, 476, 3, 405, 78, 3, 436, 3, 405, 78, 3, 405, 78,
    3, 389, 78, 3, 78, 3, 78, 3, 78, 3, 389, 78, 3, 78, 267, 3,
    389, 78, 3, 389, 78, 3, 405, 78, 3, 405, 78, 281, 3, 3, 78, 3,
    78, 3, 234, 3, 284, 3, 78, 267, 3, 532, 3, 78, 267, 3, 78, 267,
    3, 405, 78, 3, 78, 267, 3, 405, 78, 3, 547, 3, 547, 3, 547, 3,
    547, 3, 294, 3, 614, 3, 204, 3, 405, 78, 3, 294, 3, 616, 3, 620,
    3, 3, 547, 4, 3, 405, 78, 3, 405, 78, 3, 405, 78, 3, 669, 3,
    294, 3, 673, 3, 676, 3, 253, 3, 684, 3, 204, 3, 245, 247, 4, 3,
    696, 3, 702, 3, 294, 3, 696, 3, 702, 3, 78, 267, 3, 78, 267, 3,
    4, 3, 346, 78, 3, 775, 3, 204, 3, 78, 3, 294, 3, 338, 3, 346,
    78, 3, 294, 3, 294, 3, 616, 3, 669, 3, 204, 3, 204, 3, 801, 3,
    816, 4, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234,
    3, 234, 4, 3, 547, 3, 234, 3, 294, 3, 616, 3, 616, 3, 616, 3,
    80, 3, 83, 3, 78, 3, 275, 78, 3, 881, 3, 284, 3, 78, 3, 294,
    3, 294, 3, 338, 3, 342, 3, 284, 3, 80, 3, 83, 3, 616, 3, 338,
    3, 886, 3, 888, 3, 389, 78, 3, 392, 3, 78, 267, 3, 893, 3, 401,
    896, 3, 3, 898, 3, 78, 3, 234, 3, 234, 3, 234, 3, 346, 78, 3,
    234, 3, 284, 3, 80, 3, 904, 3, 338, 3, 886, 3, 888, 3, 78, 200,
    3, 78, 906, 3, 78, 371, 3, 910, 3, 78, 267, 3, 476, 3, 436, 3,
    338, 3, 78, 267, 3, 917, 3, 234, 3, 78, 267, 3, 294, 3, 234, 3,
    405, 78, 3, 294, 3, 405, 78, 3, 294, 3, 234, 3, 405, 78, 3, 405,
    78, 3, 294, 3, 405, 78, 3, 78, 3, 234, 3, 78, 267, 3, 78, 3,
    234, 3, 78, 267, 3, 284, 3, 78, 942, 3, 78, 267, 3, 342, 3, 342,
    3, 78, 200, 3, 78, 3, 78, 3, 78, 200, 3, 234, 3, 284, 3, 78,
    267, 3, 532, 3, 80, 3, 80, 3, 1013, 3, 80, 3, 1021, 3, 669, 1062,
    3, 3, 342, 3, 342, 3, 616, 3, 204, 3, 234, 3, 78, 200, 3, 1119,
    3, 1121, 3, 1124, 3, 1119, 3, 1126, 3, 1124, 3, 1126, 3, 1128, 3, 1124,
    3, 229, 3, 1131, 3, 1134, 3, 78, 267, 3, 1137, 3, 78, 3, 616, 3,
    616, 3, 275, 78, 3, 78, 3, 80, 3, 389, 78, 3, 78, 267, 3, 234,
    3, 234, 3, 78, 3, 78, 3, 78, 906, 3, 78, 371, 3, 78, 267, 3,
    78, 942, 3, 78, 3, 78, 200, 3, 78, 267, 3, 294, 3, 1190, 3, 1192,
    3, 1221, 3, 1223, 3, 1225, 3, 97, 3, 97, 3, 1252, 3, 1254, 3, 775,
    3, 204, 3, 294, 3, 405, 78, 3, 204, 3, 1257, 3, 294, 3, 78, 85,
    3, 1190, 3, 1261, 3, 294, 3, 616, 3, 1257, 3, 1268, 3, 1270, 3, 614,
    3, 1272, 1274, 3, 3, 405, 78, 3, 1277, 3, 294, 3, 342, 3, 294, 3,
    78, 3, 78, 267, 3, 78, 267, 3, 614, 3, 405, 78, 3, 1137, 3, 1131,
    3, 1380, 3, 1382, 3, 1384, 3, 1387, 3, 1380, 3, 1382, 3, 532, 3, 1389,
    3, 1392, 3, 1394, 3, 1398, 3, 1401, 269, 3, 269, 3, 3, 1398, 3, 1401,
    3, 1405, 3, 1407, 3, 1137, 3, 1401, 3, 294, 3, 294, 3, 294, 3, 616,
    3, 294, 3, 294, 3, 294, 3, 294, 1415, 3, 3, 405, 78, 3, 405, 78,
    3, 1421, 3, 1272, 3, 1424, 3, 673, 3, 673, 3, 616, 3, 616, 3, 80,
    3, 1021, 3, 80, 3, 1021, 3, 80, 3, 1021, 3, 80, 3, 1021, 3, 80,
    3, 80, 3, 1268, 3, 80, 3, 1021, 3, 234, 3, 234, 1484, 3, 1486, 3,
    3, 78, 200, 3, 78, 200, 3, 405, 78, 3, 294, 3, 1494, 3, 1131, 3,
    405, 78, 3, 294, 3, 1501, 3, 1013, 3, 294, 3, 1131, 3, 1169, 4, 3,
    294, 3, 342, 3, 294, 3, 342, 3, 342, 3, 204, 3, 78, 3, 1539, 3,
    234, 3, 392, 3, 78, 3, 234, 3, 234, 3, 392, 3, 78, 3, 338, 3,
    234, 3, 234, 3, 253, 3, 78, 267, 3, 253, 3, 1268, 3, 1568, 3, 1134,
    3, 1570, 3, 1572, 3, 1134, 3, 294, 3, 1407, 3, 1619, 3, 1539, 3, 1407,
    3, 1619, 3, 1407, 3, 294, 3, 1638, 3, 616, 3, 294, 3, 1638, 3, 294,
    3, 294, 3, 616, 3, 294, 3, 1, 4, 5, 3, 294, 3, 294, 3, 97,
    1668, 3, 3, 1670, 3, 1673, 3, 1, 588, 4, 5, 3, 588, 4, 3, 78,
    267, 3, 1128, 3, 405, 78, 3, 234, 3, 342, 3, 78, 3, 1268, 3, 1740,
    3, 1742, 3, 294, 3, 1746, 3, 1748, 3, 294, 3, 1750, 3, 78, 267, 3,
    78, 267, 269, 3, 3, 204, 269, 3, 3, 1539, 3, 1754, 3, 1568, 3, 1776,
    3, 1778, 3, 1792, 4, 3, 1268, 3, 1270, 3, 1270, 3, 1809, 3, 1223, 3,
    1221, 3, 1225, 3, 1813, 3, 676, 3, 1270, 3, 1832, 3, 294, 3, 78, 267,
    3, 405, 78, 3, 294, 3, 1857, 3, 1131, 3, 1131, 3, 1776, 3, 405, 78,
    3, 294, 3, 1912, 3, 1915, 3, 1128, 3, 616, 3, 616, 3, 294, 3, 78,
    267, 3, 78, 267, 3, 78, 267, 3, 78, 267, 3, 342, 3, 294, 3, 78,
    267, 3, 1128, 3, 1268, 3, 1568, 3, 1270, 3, 1134, 3, 234, 3, 294, 3,
    2023, 1484, 3, 1486, 3, 3, 78, 200, 3, 78, 198, 3, 78, 198, 2027, 3,
    3, 2061, 269, 3, 3, 2078, 3, 2023, 3, 1131, 3, 1380, 3, 78, 3, 2080,
    3, 1389, 3, 1126, 3, 1915, 3, 1128, 3, 1124, 3, 2082, 3, 2084, 3, 1277,
    3, 2086, 3, 1387, 3, 229, 3, 1131, 3, 2086, 3, 1389, 3, 1380, 3, 1572,
    3, 532, 3, 1134, 3, 1809, 3, 1809, 3, 234, 3, 204, 3, 294, 3, 2103,
    3, 405, 78, 3, 234, 3, 204, 3, 405, 78, 3, 294, 3, 1268, 3, 78,
    200, 3, 78, 200, 3, 1380, 3, 229, 3, 1131, 3, 2086, 3, 346, 78, 3,
    78, 906, 3, 1252, 3, 616, 3, 1254, 1062, 3, 210, 3, 3, 1813, 3, 1221,
    3, 1424, 3, 2196, 3, 2198, 3, 2200, 3, 2202, 3, 2204, 2206, 3, 3, 2208,
    3, 2210, 3, 2212, 2214, 3, 3, 2216, 3, 2218, 3, 2220, 3, 2222, 3, 2224,
    3, 2226, 3, 2228, 3, 2230, 3, 1225, 3, 1223, 3, 2232, 3, 2234, 3, 2236,
    3, 2238, 3, 2240, 3, 2242, 3, 2244, 3, 2246, 3, 2248, 3, 2250, 2252, 3,
    3, 2254, 3, 2256, 3, 2258, 3, 2260, 3, 2262, 3, 2264, 3, 2266, 3, 2268,
    3, 2270, 3, 2272, 3, 2274, 3, 2276, 3, 2278, 3, 2280, 3, 2282, 3, 2284,
    3, 2286, 3, 2288, 3, 2290, 3, 2292, 3, 2294, 3, 2296, 2214, 3, 2298, 3,
    3, 2230, 3, 2300, 3, 2302, 3, 2304, 3, 2306, 3, 2308, 3, 2310, 2312, 3,
    3, 2314, 3, 2316, 3, 1572, 3, 1570, 3, 2318, 3, 2320, 3, 2322, 2324, 3,
    3, 2326, 3, 2328, 3, 2330, 2332, 3, 3, 2334, 3, 2336, 3, 547, 269, 3,
    3, 4, 256, 3, 78, 3, 389, 78, 3, 392, 3, 401, 3, 392, 3, 401,
    3, 389, 78, 3, 405, 78, 3, 294, 3, 389, 78, 3, 392, 3, 401, 3,
    389, 78, 3, 401, 3, 1268, 3, 204, 3, 78, 267, 3, 1813, 3, 234, 3,
    234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3,
    234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3,
    234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 234, 3,
    234, 3, 234, 3, 234, 3, 234, 3, 234, 3, 78, 267, 3, 669, 3, 294,
    3, 342, 3, 78, 267, 1062, 3, 1668, 3, 3, 1190, 3, 1192, 3, 2080, 3,
    1389, 3, 1126, 3, 1915, 3, 1128, 3, 1124, 3, 2082, 3, 2084, 3, 1277, 3,
    2086, 3, 1387, 3, 1380, 3, 1382, 3, 1384, 3, 229, 3, 1131, 3, 1389, 3,
    1387, 3, 1915, 3, 532, 3, 1128, 3, 1572, 3, 1380, 3, 78, 267, 3, 1137,
    3, 1568, 3, 1124, 3, 229, 3, 1131, 3, 2103, 3, 275, 78, 3, 294, 3,
    294, 3, 294, 3, 294, 3, 1131, 3, 1389, 3, 294, 3, 342, 3, 4, 2498,
    3, 78, 85, 3, 2502, 3, 229, 3, 2504, 3, 2509, 3, 2509, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"`?[a-zA-Z0-9_$]*`?"#, // [2]
    r#"([A-Z0-9_]*[A-Z][A-Z0-9_]*)|_"#, // [3]
    r#"^(ACCESSIBLE|ADD|ALL|ALTER|ANALYZE|AND|AS|ASC|ASENSITIVE|BEFORE|BETWEEN|BIGINT|BINARY|BLOB|BOTH|BY|CALL|CASCADE|CASE|CHANGE|CHANGED_PAGE_BITMAPS|CHAR|CHARACTER|CHECK|CLIENT_STATISTICS|COLLATE|COLUMN|CONDITION|CONSTRAINT|CONTINUE|CONVERT|CREATE|CROSS|CURRENT_DATE|CURRENT_ROLE|CURRENT_TIME|CURRENT_TIMESTAMP|CURRENT_USER|CURSOR|DATABASE|DATABASES|DAY_HOUR|DAY_MICROSECOND|DAY_MINUTE|DAY_SECOND|DEC|DECIMAL|DECLARE|DEFAULT|DELAYED|DELETE|DELETE_DOMAIN_ID|DESC|DESCRIBE|DETERMINISTIC|DISTINCT|DISTINCTROW|DIV|DOUBLE|DO_DOMAIN_IDS|DROP|DUAL|EACH|ELSE|ELSEIF|ENCLOSED|ESCAPED|EXCEPT|EXISTS|EXIT|EXPLAIN|FALSE|FETCH|FLOAT|FLOAT4|FLOAT8|FOR|FORCE|FOREIGN|FROM|FULLTEXT|GENERAL|GRANT|GROUP|HAVING|HIGH_PRIORITY|HOUR_MICROSECOND|HOUR_MINUTE|HOUR_SECOND|IF|IGNORE|IGNORE_DOMAIN_IDS|IGNORE_SERVER_IDS|IN|INDEX|INDEX_STATISTICS|INFILE|INNER|INOUT|INSENSITIVE|INSERT|INT|INT1|INT2|INT3|INT4|INT8|INTEGER|INTERSECT|INTERVAL|INTO|IS|ITERATE|JOIN|KEY|KEYS|KILL|LEADING|LEAVE|LEFT|LIKE|LIMIT|LINEAR|LINES|LOAD|LOCALTIME|LOCALTIMESTAMP|LOCK|LONG|LONGBLOB|LONGTEXT|LOOP|LOW_PRIORITY|MASTER_HEARTBEAT_PERIOD|MASTER_SSL_VERIFY_SERVER_CERT|MATCH|MAXVALUE|MEDIUMBLOB|MEDIUMINT|MEDIUMTEXT|MIDDLEINT|MINUTE_MICROSECOND|MINUTE_SECOND|MOD|MODIFIES|NATURAL|NOT|NO_WRITE_TO_BINLOG|NULL|NUMERIC|OFFSET|ON|OPTIMIZE|OPTION|OPTIONALLY|OR|ORDER|OUT|OUTER|OUTFILE|OVER|PAGE_CHECKSUM|PARSE_VCOL_EXPR|PARTITION|PRECISION|PRIMARY|PROCEDURE|PURGE|RANGE|READ|READS|READ_WRITE|REAL|RECURSIVE|REFERENCES|REF_SYSTEM_ID|REGEXP|RELEASE|RENAME|REPEAT|REPLACE|REQUIRE|RESIGNAL|RESTRICT|RETURN|RETURNING|REVOKE|RIGHT|RLIKE|ROWS|ROW_NUMBER|SCHEMA|SCHEMAS|SECOND_MICROSECOND|SELECT|SENSITIVE|SEPARATOR|SET|SHOW|SIGNAL|SLOW|SMALLINT|SPATIAL|SPECIFIC|SQL|SQLEXCEPTION|SQLSTATE|SQLWARNING|SQL_BIG_RESULT|SQL_CALC_FOUND_ROWS|SQL_SMALL_RESULT|SSL|STARTING|STATS_AUTO_RECALC|STATS_PERSISTENT|STATS_SAMPLE_PAGES|STRAIGHT_JOIN|TABLE|TERMINATED|THEN|TINYBLOB|TINYINT|TINYTEXT|TO|TRAILING|TRIGGER|TRUE|UNDO|UNION|UNIQUE|UNLOCK|UNSIGNED|UPDATE|USAGE|USE|USING|UTC_DATE|UTC_TIME|UTC_TIMESTAMP|VALUES|VARBINARY|VARCHAR|VARCHARACTER|VARYING|WHEN|WHERE|WHILE|WINDOW|WITH|WRITE|XOR|YEAR_MONTH|ZEROFILL)$"#, // [4]
    r#"`?[A-Za-z0-9_]*`?"#, // [5]
    r#"[@][a-zA-Z0-9_$]*"#, // [6]
    r#"@@((session|global|local|persist|persist_only)\.)?[A-Za-z0-9_]+"#, // [7]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 25, 40, 37, 13, 14, 20, 21, 28,
    12, 26, 33, 35, 17, 52, 53, 53, 52, 57,
    11, 60, 62, 19, 15, 16, 18, 22, 23, 24,
    27, 28, 29, 30, 31, 32, 34, 36, 15, 16,
    67, 22, 23, 68, 69, 60, 70, 71, 72, 73,
    74, 75, 13, 28, 82, 87, 89, 91, 92, 95,
    97, 11, 98, 99, 100, 101, 102, 103, 104, 105,
    106, 40, 107, 108, 109, 110, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 100, 13, 20,
    26, 123, 37, 123, 98, 22, 128, 102, 34, 105,
    133, 134, 118, 134, 118, 11, 98, 99, 100, 101,
    102, 103, 104, 106, 40, 107, 108, 109, 110, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    15, 16, 18, 22, 23, 137, 24, 27, 28, 29,
    30, 31, 32, 34, 36, 137, 119, 12, 28, 103,
    142, 99, 101, 104, 106, 107, 108, 109, 111, 110,
    112, 113, 114, 115, 116, 117, 120, 121, 164, 32,
    36, 30, 18, 24, 27, 29, 31, 23, 15, 16,
    177, 73, 75, 71, 67, 68, 69, 70, 72, 74,
    189, 190, 189, 190, 195, 201, 206, 207, 208, 211,
    212, 213, 214, 215, 216, 215, 216, 220, 222, 223,
    224, 225, 226, 228, 230, 233, 235, 236, 237, 238,
    239, 236, 237, 235, 239, 238, 249, 257, 260, 262,
    265, 270, 271, 272, 276, 277, 278, 236, 279, 109,
    283, 286, 288, 289, 290, 289, 290, 295, 296, 297,
    298, 299, 300, 301, 302, 303, 304, 305, 306, 307,
    308, 309, 310, 311, 312, 313, 314, 316, 317, 317,
    316, 321, 323, 325, 326, 328, 329, 329, 328, 332,
    333, 332, 333, 337, 339, 276, 277, 236, 276, 277,
    278, 236, 278, 236, 279, 279, 262, 260, 350, 236,
    97, 351, 352, 353, 354, 355, 356, 357, 358, 359,
    228, 353, 362, 355, 356, 354, 356, 351, 357, 351,
    357, 352, 358, 352, 358, 359, 228, 374, 375, 17,
    376, 377, 378, 379, 380, 381, 374, 375, 17, 379,
    381, 375, 374, 17, 379, 381, 381, 377, 378, 377,
    378, 387, 391, 337, 395, 396, 397, 398, 391, 399,
    400, 379, 101, 117, 406, 356, 407, 408, 407, 406,
    408, 413, 356, 374, 417, 323, 417, 376, 380, 376,
    380, 422, 230, 424, 235, 236, 237, 238, 424, 236,
    426, 427, 428, 427, 428, 426, 215, 106, 82, 224,
    437, 224, 437, 437, 249, 89, 215, 230, 272, 442,
    443, 444, 276, 277, 445, 278, 337, 446, 447, 52,
    236, 448, 328, 216, 329, 449, 450, 451, 452, 377,
    453, 397, 424, 454, 455, 106, 456, 457, 458, 459,
    460, 326, 461, 29, 462, 463, 464, 465, 33, 466,
    467, 400, 442, 443, 276, 277, 445, 278, 337, 446,
    52, 236, 448, 449, 450, 451, 452, 377, 453, 397,
    454, 455, 456, 457, 458, 459, 460, 461, 29, 462,
    463, 464, 465, 33, 466, 467, 400, 216, 408, 396,
    399, 396, 399, 444, 374, 375, 478, 396, 323, 399,
    478, 481, 447, 215, 483, 374, 484, 485, 216, 486,
    487, 488, 489, 490, 491, 492, 116, 485, 487, 484,
    491, 486, 483, 488, 489, 492, 490, 503, 504, 503,
    504, 400, 400, 509, 509, 512, 513, 515, 516, 517,
    517, 515, 516, 521, 524, 283, 525, 525, 82, 112,
    528, 529, 400, 533, 534, 536, 537, 538, 538, 537,
    542, 544, 545, 546, 545, 546, 549, 550, 551, 534,
    552, 553, 554, 555, 556, 557, 558, 559, 560, 561,
    553, 554, 555, 552, 551, 561, 560, 556, 557, 558,
    559, 575, 577, 579, 581, 582, 583, 584, 585, 586,
    587, 590, 591, 592, 593, 594, 595, 596, 442, 598,
    599, 600, 443, 601, 602, 603, 604, 605, 606, 607,
    608, 613, 623, 624, 625, 626, 627, 628, 629, 551,
    534, 546, 552, 553, 554, 630, 631, 555, 556, 233,
    557, 545, 558, 632, 559, 633, 634, 560, 561, 635,
    636, 546, 545, 633, 634, 635, 629, 556, 233, 557,
    632, 636, 630, 631, 643, 644, 645, 646, 647, 652,
    653, 654, 655, 633, 634, 635, 658, 659, 164, 660,
    659, 164, 659, 658, 660, 660, 658, 664, 667, 581,
    670, 670, 82, 112, 528, 529, 677, 678, 679, 680,
    681, 682, 685, 687, 688, 689, 690, 694, 698, 700,
    704, 705, 707, 708, 709, 710, 711, 712, 713, 714,
    716, 717, 276, 718, 719, 720, 721, 722, 323, 722,
    724, 725, 356, 726, 724, 726, 725, 730, 731, 277,
    712, 734, 735, 736, 737, 738, 736, 741, 738, 735,
    737, 734, 747, 748, 445, 734, 738, 750, 751, 752,
    753, 754, 755, 756, 757, 758, 759, 760, 761, 762,
    763, 765, 767, 768, 769, 770, 323, 413, 249, 783,
    783, 786, 787, 786, 787, 790, 792, 786, 787, 792,
    794, 795, 794, 795, 236, 323, 413, 164, 603, 604,
    605, 606, 607, 608, 413, 262, 97, 260, 605, 606,
    607, 608, 803, 804, 805, 806, 807, 808, 809, 811,
    813, 814, 818, 819, 820, 629, 556, 636, 632, 830,
    833, 834, 835, 836, 837, 838, 839, 841, 843, 844,
    845, 846, 446, 849, 850, 851, 852, 853, 854, 855,
    856, 857, 858, 859, 860, 87, 391, 863, 864, 865,
    866, 867, 868, 871, 855, 18, 872, 873, 24, 27,
    874, 855, 872, 873, 874, 855, 872, 874, 872, 874,
    873, 276, 278, 236, 279, 633, 883, 376, 380, 381,
    891, 892, 760, 337, 478, 397, 323, 391, 400, 900,
    901, 276, 278, 279, 351, 352, 353, 354, 355, 356,
    357, 358, 359, 228, 907, 907, 113, 114, 912, 82,
    249, 913, 112, 528, 529, 400, 82, 249, 112, 528,
    529, 400, 913, 52, 377, 919, 920, 378, 921, 922,
    923, 920, 378, 921, 921, 920, 378, 920, 922, 923,
    923, 922, 928, 930, 932, 529, 62, 935, 935, 938,
    82, 354, 354, 224, 623, 101, 104, 117, 945, 947,
    948, 949, 948, 949, 953, 362, 400, 955, 262, 260,
    955, 460, 249, 362, 958, 958, 962, 750, 964, 427,
    965, 966, 967, 750, 964, 965, 966, 967, 965, 967,
    964, 966, 973, 974, 975, 976, 974, 975, 973, 976,
    981, 460, 982, 984, 236, 613, 985, 985, 295, 990,
    992, 994, 236, 995, 995, 981, 982, 999, 97, 999,
    424, 1001, 1001, 591, 337, 1004, 1005, 1006, 1007, 1008,
    1008, 1009, 1009, 1014, 605, 608, 606, 607, 603, 1022,
    1023, 1024, 1025, 604, 1026, 1027, 1028, 91, 1030, 1030,
    1032, 757, 1033, 1033, 1037, 1038, 1039, 1040, 1043, 795,
    1044, 1047, 1048, 1049, 1048, 1049, 1053, 628, 276, 92,
    1054, 1054, 628, 1048, 1049, 1058, 1058, 1063, 1064, 1065,
    1066, 1063, 1043, 795, 97, 260, 1063, 1043, 795, 1043,
    97, 260, 1073, 1074, 296, 297, 298, 299, 1075, 1076,
    1077, 1078, 1079, 1080, 1080, 216, 1083, 1084, 1085, 1086,
    858, 1089, 1090, 1089, 1090, 1095, 1097, 1099, 1101, 1102,
    528, 262, 97, 260, 1106, 1107, 1108, 1109, 1110, 1111,
    1112, 448, 1113, 1115, 1117, 1123, 1130, 1133, 1138, 1139,
    1140, 1139, 1140, 1143, 1145, 1146, 1147, 1148, 1149, 1150,
    1151, 1152, 1154, 1155, 1156, 1157, 1158, 1159, 1160, 1161,
    1162, 1163, 1164, 1165, 1166, 406, 407, 235, 236, 237,
    18, 24, 113, 114, 1176, 1178, 1179, 1180, 1181, 1182,
    1183, 1184, 1185, 1186, 1187, 1188, 1193, 1194, 449, 450,
    1195, 1196, 451, 395, 1198, 1199, 1200, 1202, 1203, 1204,
    1205, 1206, 1207, 1208, 1209, 1210, 1211, 1212, 1213, 1214,
    1215, 1216, 1217, 1218, 82, 101, 104, 112, 117, 528,
    529, 400, 1226, 913, 323, 1230, 1231, 62, 323, 626,
    323, 629, 553, 554, 323, 1241, 1140, 553, 554, 629,
    323, 1241, 629, 551, 534, 552, 553, 554, 555, 556,
    233, 557, 558, 632, 559, 560, 561, 636, 553, 554,
    1140, 1248, 1063, 658, 413, 1043, 795, 660, 262, 97,
    260, 528, 529, 1276, 1278, 1279, 1064, 1280, 1283, 1284,
    437, 1030, 947, 1285, 1285, 350, 400, 1288, 1289, 1291,
    1292, 1293, 1296, 1297, 1298, 1299, 1300, 1301, 1302, 1303,
    1304, 628, 713, 754, 1151, 451, 1203, 1306, 1307, 1308,
    1309, 1310, 1311, 142, 1312, 1313, 1314, 1315, 1316, 333,
    1317, 965, 1318, 73, 1319, 1320, 1321, 1322, 628, 713,
    754, 1151, 451, 1203, 1306, 1307, 1308, 1309, 1310, 1311,
    142, 1312, 1313, 1314, 1315, 1316, 333, 1317, 965, 1318,
    1319, 1320, 1321, 1322, 1310, 1306, 1312, 1317, 1311, 1315,
    1315, 1311, 1307, 1316, 1318, 1307, 1316, 1307, 1316, 1318,
    1314, 1308, 1309, 1313, 1319, 1321, 1322, 1320, 1348, 736,
    424, 201, 117, 1349, 891, 1351, 891, 1351, 1355, 1357,
    1358, 1359, 1360, 923, 1285, 1348, 1362, 1362, 1365, 1366,
    1367, 1368, 1209, 1028, 736, 195, 1369, 1370, 513, 1130,
    424, 1371, 1372, 228, 1373, 467, 1123, 1374, 400, 1369,
    1373, 1374, 1209, 1370, 1371, 467, 1372, 1397, 1358, 1117,
    503, 119, 1404, 843, 667, 1409, 1410, 1416, 1117, 1419,
    5, 1037, 5, 1429, 1432, 1443, 1444, 1445, 1446, 1447,
    1448, 1451, 1453, 1454, 1453, 1454, 707, 752, 809, 851,
    852, 853, 1038, 1457, 1458, 1459, 1460, 1461, 1462, 1461,
    1459, 1457, 1458, 1460, 1462, 1476, 1477, 1478, 1479, 1480,
    1481, 1488, 1490, 1491, 1028, 1369, 1130, 228, 1373, 1374,
    1495, 1496, 1499, 1502, 1503, 1504, 1028, 1130, 228, 1373,
    1374, 1505, 1506, 1507, 1509, 1510, 1511, 1512, 1513, 300,
    301, 302, 303, 1514, 1515, 1516, 1517, 452, 698, 1186,
    700, 387, 1520, 1419, 1524, 1525, 1526, 1527, 1528, 1529,
    1530, 1531, 1532, 1533, 1534, 1358, 1117, 119, 195, 1536,
    228, 1536, 397, 1542, 356, 1544, 1544, 478, 323, 398,
    680, 920, 680, 920, 1549, 1549, 1551, 1552, 1553, 1554,
    1555, 1556, 1557, 1558, 1559, 1560, 844, 1561, 1562, 1564,
    1113, 1510, 1115, 82, 112, 408, 528, 529, 400, 1573,
    453, 1574, 1575, 1576, 1577, 1578, 1579, 1580, 1581, 1582,
    1583, 1370, 1584, 1586, 811, 1289, 1590, 1193, 1592, 1592,
    1596, 1598, 1209, 1028, 91, 1370, 1130, 1371, 228, 1276,
    467, 1123, 1601, 1602, 1603, 1605, 1608, 1609, 1610, 1611,
    1612, 164, 391, 843, 845, 846, 845, 846, 1629, 1630,
    1632, 1633, 1636, 1215, 1636, 236, 1203, 413, 1188, 1203,
    1641, 1642, 1643, 1644, 1645, 1646, 1647, 1648, 1649, 1650,
    1651, 1652, 1653, 1409, 1654, 1655, 1656, 1657, 1658, 913,
    1230, 1231, 1231, 1230, 87, 1372, 1664, 1665, 1666, 1672,
    1676, 1677, 837, 1115, 1117, 1681, 1681, 1596, 1285, 1684,
    1685, 1686, 1687, 1688, 1689, 1690, 1691, 1692, 1695, 1696,
    1698, 1699, 1700, 1701, 1702, 1703, 1704, 1705, 1706, 1707,
    1708, 1709, 1710, 1711, 1712, 1713, 1714, 1715, 1716, 1717,
    1718, 1719, 1720, 1721, 1722, 1723, 1724, 1725, 1726, 1727,
    1728, 1729, 1730, 454, 1731, 1732, 1733, 1734, 1735, 1736,
    1737, 1738, 1755, 1756, 304, 1757, 1758, 1759, 1760, 455,
    1371, 305, 306, 307, 355, 1761, 1762, 308, 1763, 1764,
    1765, 1766, 1767, 1768, 1769, 1770, 1771, 1772, 1773, 1774,
    1779, 1781, 1782, 1784, 1785, 1786, 1787, 1789, 1790, 1791,
    1794, 1795, 1796, 1797, 1798, 1799, 1800, 1801, 1802, 1803,
    1804, 1805, 1806, 101, 104, 515, 117, 82, 112, 1815,
    359, 1816, 1817, 1818, 629, 1821, 1822, 1823, 1824, 1825,
    1826, 1827, 1828, 1829, 1830, 1837, 1838, 1839, 1840, 1841,
    1842, 1843, 1844, 1845, 1846, 1847, 1848, 1849, 1850, 1851,
    1852, 1853, 635, 634, 437, 1648, 1028, 1369, 1130, 1373,
    923, 1285, 1416, 1374, 1859, 1860, 1861, 1862, 1863, 1865,
    1867, 1868, 1869, 1870, 456, 1871, 1872, 1873, 457, 458,
    459, 1874, 1876, 1877, 1878, 1879, 1880, 1881, 1882, 1883,
    1884, 1885, 1886, 1887, 555, 262, 260, 1890, 1891, 1892,
    1893, 1894, 1895, 1896, 1897, 1898, 1900, 1901, 1902, 1903,
    1904, 1905, 1906, 1907, 1908, 1909, 1910, 1916, 1917, 1918,
    1919, 1920, 1921, 1922, 1924, 1925, 1926, 1927, 62, 1561,
    1862, 1932, 1933, 1934, 1935, 1936, 1937, 628, 1311, 623,
    89, 11, 549, 1939, 1939, 309, 1941, 1943, 1944, 1945,
    1946, 1948, 1949, 1950, 1951, 1952, 1953, 1954, 1955, 1956,
    1957, 1958, 1959, 1960, 1961, 1964, 843, 1886, 1966, 1966,
    1969, 1970, 1971, 1972, 1973, 1974, 1975, 1976, 1977, 1978,
    1979, 1980, 1981, 1982, 1983, 1984, 1985, 1986, 1987, 1988,
    734, 738, 1989, 1989, 1991, 1993, 1995, 1113, 1115, 82,
    112, 408, 528, 529, 82, 528, 529, 1996, 1997, 1998,
    1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
    2008, 2009, 2009, 2012, 2013, 2014, 2015, 2016, 2017, 2019,
    2020, 2021, 2024, 2030, 2031, 2032, 2033, 2034, 2035, 461,
    2036, 2037, 2038, 2039, 2040, 2041, 2042, 625, 220, 2046,
    2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 310, 311,
    2055, 2056, 2057, 2058, 2059, 11, 1163, 1164, 1510, 2062,
    2063, 2064, 2065, 2066, 2067, 2068, 11, 1163, 1164, 2068,
    2067, 2062, 2063, 2064, 2066, 2065, 82, 400, 2089, 2091,
    2093, 2094, 2095, 2096, 1791, 1844, 1307, 537, 1896, 1897,
    1316, 1248, 233, 467, 1209, 1370, 2110, 1610, 109, 109,
    1995, 2094, 819, 2113, 2113, 350, 121, 2117, 2118, 2119,
    2120, 2121, 2122, 2125, 2127, 2129, 2130, 2132, 2133, 2134,
    2135, 2136, 2137, 2138, 2139, 2140, 2141, 2142, 2143, 2144,
    2145, 2146, 2147, 2148, 2149, 2150, 2151, 2152, 2153, 2154,
    2155, 2156, 2157, 2158, 2159, 2160, 2161, 2162, 2163, 2164,
    2165, 2166, 2167, 2168, 2169, 2170, 2171, 2172, 2173, 2174,
    2175, 2176, 2177, 2178, 2179, 2180, 2181, 2182, 2183, 2184,
    2185, 2186, 2187, 2188, 2189, 2190, 2191, 2193, 2194, 2337,
    462, 463, 464, 2338, 2339, 2340, 2341, 2342, 465, 2343,
    2345, 2346, 2347, 2348, 2349, 2350, 2351, 2352, 2353, 2354,
    2355, 2356, 2357, 2358, 2359, 2360, 2361, 2363, 2364, 451,
    465, 437, 1648, 356, 820, 834, 236, 1182, 1299, 1873,
    1959, 466, 2392, 2396, 912, 2398, 2399, 2400, 2401, 2402,
    2404, 1065, 1066, 2405, 2406, 2407, 2408, 2409, 2410, 2411,
    2412, 2413, 1410, 2414, 2416, 2421, 2422, 2423, 2424, 2425,
    2426, 2427, 2428, 2429, 2430, 2431, 2432, 2433, 2434, 2438,
    2439, 2441, 2442, 2443, 2447, 2448, 2449, 2450, 2451, 2452,
    11, 1163, 2453, 2454, 2455, 2456, 2457, 2458, 2459, 2461,
    2462, 2463, 2464, 2465, 2466, 2467, 2468, 2469, 2470, 2471,
    2472, 82, 947, 2473, 2474, 2475, 2476, 2477, 2478, 2479,
    2480, 2481, 2482, 2483, 2484, 2485, 892, 2487, 2489, 2490,
    312, 2491, 2492, 2495, 1028, 195, 1369, 513, 1130, 228,
    1276, 1373, 1374, 2496, 2500, 228, 503, 689, 537, 2513,
    2514, 2515, 2516, 2517, 2518, 313, 314,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 27, token_types_start: 32, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 32, raw_values_count: 1, token_types_start: 33, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 2, token_types_start: 47, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 14, token_types_start: 68, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 68, raw_values_count: 14, token_types_start: 82, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 82, raw_values_count: 2, token_types_start: 84, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 84, raw_values_count: 1, token_types_start: 85, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 85, raw_values_count: 1, token_types_start: 86, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 86, raw_values_count: 1, token_types_start: 87, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 0, token_types_start: 90, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 26, token_types_start: 117, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 117, raw_values_count: 1, token_types_start: 118, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 118, raw_values_count: 5, token_types_start: 123, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 123, raw_values_count: 1, token_types_start: 124, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 124, raw_values_count: 1, token_types_start: 125, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 125, raw_values_count: 1, token_types_start: 126, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 2, token_types_start: 133, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 135, raw_values_count: 25, token_types_start: 160, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 160, raw_values_count: 15, token_types_start: 175, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 175, raw_values_count: 1, token_types_start: 176, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 176, raw_values_count: 1, token_types_start: 177, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 177, raw_values_count: 2, token_types_start: 179, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 181, raw_values_count: 1, token_types_start: 182, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 182, raw_values_count: 1, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 2, token_types_start: 222, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 225, raw_values_count: 1, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 1, token_types_start: 233, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 233, raw_values_count: 2, token_types_start: 235, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 245, raw_values_count: 1, token_types_start: 246, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 246, raw_values_count: 5, token_types_start: 251, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 251, raw_values_count: 1, token_types_start: 252, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 252, raw_values_count: 1, token_types_start: 253, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 253, raw_values_count: 1, token_types_start: 254, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 257, raw_values_count: 1, token_types_start: 258, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 258, raw_values_count: 0, token_types_start: 258, token_types_count: 1 }, // [130]
    SimpleHintData { raw_values_start: 259, raw_values_count: 0, token_types_start: 259, token_types_count: 1 }, // [131]
    SimpleHintData { raw_values_start: 260, raw_values_count: 1, token_types_start: 261, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 261, raw_values_count: 1, token_types_start: 262, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 262, raw_values_count: 1, token_types_start: 263, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 264, raw_values_count: 6, token_types_start: 270, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 271, raw_values_count: 1, token_types_start: 272, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 273, raw_values_count: 2, token_types_start: 275, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 275, raw_values_count: 1, token_types_start: 276, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 276, raw_values_count: 1, token_types_start: 277, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 277, raw_values_count: 20, token_types_start: 297, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 297, raw_values_count: 2, token_types_start: 299, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 299, raw_values_count: 1, token_types_start: 300, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 300, raw_values_count: 1, token_types_start: 301, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 301, raw_values_count: 1, token_types_start: 302, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 302, raw_values_count: 1, token_types_start: 303, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 304, raw_values_count: 1, token_types_start: 305, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 305, raw_values_count: 2, token_types_start: 307, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 307, raw_values_count: 1, token_types_start: 308, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 308, raw_values_count: 1, token_types_start: 309, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 309, raw_values_count: 2, token_types_start: 311, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 312, raw_values_count: 1, token_types_start: 313, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 313, raw_values_count: 1, token_types_start: 314, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 315, raw_values_count: 3, token_types_start: 318, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 318, raw_values_count: 2, token_types_start: 320, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 320, raw_values_count: 2, token_types_start: 322, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 322, raw_values_count: 1, token_types_start: 323, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 323, raw_values_count: 2, token_types_start: 325, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 325, raw_values_count: 1, token_types_start: 326, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 326, raw_values_count: 0, token_types_start: 326, token_types_count: 2 }, // [165]
    SimpleHintData { raw_values_start: 328, raw_values_count: 1, token_types_start: 329, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 329, raw_values_count: 1, token_types_start: 330, token_types_count: 1 }, // [167]
    SimpleHintData { raw_values_start: 331, raw_values_count: 10, token_types_start: 341, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 341, raw_values_count: 1, token_types_start: 342, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 342, raw_values_count: 1, token_types_start: 343, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 343, raw_values_count: 2, token_types_start: 345, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 345, raw_values_count: 2, token_types_start: 347, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 347, raw_values_count: 2, token_types_start: 349, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 349, raw_values_count: 1, token_types_start: 350, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 350, raw_values_count: 1, token_types_start: 351, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 351, raw_values_count: 2, token_types_start: 353, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 353, raw_values_count: 1, token_types_start: 354, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 354, raw_values_count: 1, token_types_start: 355, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 355, raw_values_count: 2, token_types_start: 357, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 357, raw_values_count: 9, token_types_start: 366, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 366, raw_values_count: 5, token_types_start: 371, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 371, raw_values_count: 1, token_types_start: 372, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 372, raw_values_count: 4, token_types_start: 376, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 376, raw_values_count: 1, token_types_start: 377, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 377, raw_values_count: 2, token_types_start: 379, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 379, raw_values_count: 1, token_types_start: 380, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 380, raw_values_count: 1, token_types_start: 381, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 381, raw_values_count: 1, token_types_start: 382, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 382, raw_values_count: 1, token_types_start: 383, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 383, raw_values_count: 8, token_types_start: 391, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 391, raw_values_count: 1, token_types_start: 392, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 392, raw_values_count: 2, token_types_start: 394, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 394, raw_values_count: 4, token_types_start: 398, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 398, raw_values_count: 1, token_types_start: 399, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 399, raw_values_count: 1, token_types_start: 400, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 400, raw_values_count: 1, token_types_start: 401, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 401, raw_values_count: 1, token_types_start: 402, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 402, raw_values_count: 1, token_types_start: 403, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 403, raw_values_count: 1, token_types_start: 404, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 404, raw_values_count: 2, token_types_start: 406, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 406, raw_values_count: 1, token_types_start: 407, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 407, raw_values_count: 2, token_types_start: 409, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 409, raw_values_count: 1, token_types_start: 410, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 410, raw_values_count: 1, token_types_start: 411, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 412, raw_values_count: 2, token_types_start: 414, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 414, raw_values_count: 4, token_types_start: 418, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 418, raw_values_count: 1, token_types_start: 419, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 419, raw_values_count: 4, token_types_start: 423, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 423, raw_values_count: 1, token_types_start: 424, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 426, raw_values_count: 2, token_types_start: 428, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 428, raw_values_count: 3, token_types_start: 431, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 431, raw_values_count: 2, token_types_start: 433, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 434, raw_values_count: 2, token_types_start: 436, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 436, raw_values_count: 46, token_types_start: 482, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 482, raw_values_count: 35, token_types_start: 517, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 517, raw_values_count: 2, token_types_start: 519, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 519, raw_values_count: 2, token_types_start: 521, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 524, raw_values_count: 2, token_types_start: 526, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 526, raw_values_count: 4, token_types_start: 530, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 532, raw_values_count: 1, token_types_start: 533, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 533, raw_values_count: 14, token_types_start: 547, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 547, raw_values_count: 1, token_types_start: 548, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 548, raw_values_count: 1, token_types_start: 549, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 550, raw_values_count: 1, token_types_start: 551, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 551, raw_values_count: 1, token_types_start: 552, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 554, raw_values_count: 1, token_types_start: 555, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 555, raw_values_count: 1, token_types_start: 556, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 557, raw_values_count: 2, token_types_start: 559, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 559, raw_values_count: 1, token_types_start: 560, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 560, raw_values_count: 1, token_types_start: 561, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 561, raw_values_count: 1, token_types_start: 562, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 562, raw_values_count: 2, token_types_start: 564, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 565, raw_values_count: 1, token_types_start: 566, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 566, raw_values_count: 1, token_types_start: 567, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 567, raw_values_count: 3, token_types_start: 570, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 570, raw_values_count: 1, token_types_start: 571, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 571, raw_values_count: 1, token_types_start: 572, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 575, raw_values_count: 2, token_types_start: 577, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 578, raw_values_count: 5, token_types_start: 583, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 583, raw_values_count: 1, token_types_start: 584, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 586, raw_values_count: 2, token_types_start: 588, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 588, raw_values_count: 1, token_types_start: 589, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 589, raw_values_count: 1, token_types_start: 590, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 590, raw_values_count: 1, token_types_start: 591, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 592, raw_values_count: 1, token_types_start: 593, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 593, raw_values_count: 2, token_types_start: 595, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 598, raw_values_count: 12, token_types_start: 610, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 610, raw_values_count: 1, token_types_start: 611, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 611, raw_values_count: 1, token_types_start: 612, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 612, raw_values_count: 1, token_types_start: 613, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 613, raw_values_count: 1, token_types_start: 614, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 617, raw_values_count: 1, token_types_start: 618, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 618, raw_values_count: 1, token_types_start: 619, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 623, raw_values_count: 1, token_types_start: 624, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 625, raw_values_count: 1, token_types_start: 626, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 628, raw_values_count: 1, token_types_start: 629, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 630, raw_values_count: 0, token_types_start: 630, token_types_count: 1 }, // [292]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 637, raw_values_count: 1, token_types_start: 638, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 640, raw_values_count: 1, token_types_start: 641, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 641, raw_values_count: 1, token_types_start: 642, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 642, raw_values_count: 1, token_types_start: 643, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 645, raw_values_count: 6, token_types_start: 651, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 651, raw_values_count: 1, token_types_start: 652, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 652, raw_values_count: 1, token_types_start: 653, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 654, raw_values_count: 1, token_types_start: 655, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 655, raw_values_count: 1, token_types_start: 656, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 656, raw_values_count: 1, token_types_start: 657, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 657, raw_values_count: 1, token_types_start: 658, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 658, raw_values_count: 22, token_types_start: 680, token_types_count: 1 }, // [315]
    SimpleHintData { raw_values_start: 681, raw_values_count: 5, token_types_start: 686, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 686, raw_values_count: 5, token_types_start: 691, token_types_count: 1 }, // [317]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 695, raw_values_count: 1, token_types_start: 696, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 696, raw_values_count: 1, token_types_start: 697, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 697, raw_values_count: 1, token_types_start: 698, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 703, raw_values_count: 3, token_types_start: 706, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 706, raw_values_count: 4, token_types_start: 710, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 710, raw_values_count: 2, token_types_start: 712, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 712, raw_values_count: 1, token_types_start: 713, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 713, raw_values_count: 2, token_types_start: 715, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 715, raw_values_count: 1, token_types_start: 716, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 716, raw_values_count: 1, token_types_start: 717, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 717, raw_values_count: 1, token_types_start: 718, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 718, raw_values_count: 1, token_types_start: 719, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 719, raw_values_count: 2, token_types_start: 721, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 722, raw_values_count: 4, token_types_start: 726, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 726, raw_values_count: 1, token_types_start: 727, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 731, raw_values_count: 1, token_types_start: 732, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 732, raw_values_count: 1, token_types_start: 733, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 733, raw_values_count: 1, token_types_start: 734, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 734, raw_values_count: 1, token_types_start: 735, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 738, raw_values_count: 1, token_types_start: 739, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 741, raw_values_count: 1, token_types_start: 742, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 742, raw_values_count: 1, token_types_start: 743, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 745, raw_values_count: 1, token_types_start: 746, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 746, raw_values_count: 1, token_types_start: 747, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 756, raw_values_count: 1, token_types_start: 757, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 757, raw_values_count: 2, token_types_start: 759, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 760, raw_values_count: 4, token_types_start: 764, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 765, raw_values_count: 1, token_types_start: 766, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 766, raw_values_count: 1, token_types_start: 767, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 767, raw_values_count: 1, token_types_start: 768, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 768, raw_values_count: 1, token_types_start: 769, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 769, raw_values_count: 1, token_types_start: 770, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 770, raw_values_count: 6, token_types_start: 776, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 780, raw_values_count: 1, token_types_start: 781, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 782, raw_values_count: 1, token_types_start: 783, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 783, raw_values_count: 1, token_types_start: 784, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 784, raw_values_count: 1, token_types_start: 785, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 785, raw_values_count: 2, token_types_start: 787, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 788, raw_values_count: 1, token_types_start: 789, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 789, raw_values_count: 1, token_types_start: 790, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 790, raw_values_count: 1, token_types_start: 791, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 791, raw_values_count: 1, token_types_start: 792, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 792, raw_values_count: 1, token_types_start: 793, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 793, raw_values_count: 1, token_types_start: 794, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 794, raw_values_count: 1, token_types_start: 795, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 795, raw_values_count: 1, token_types_start: 796, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 796, raw_values_count: 1, token_types_start: 797, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 797, raw_values_count: 1, token_types_start: 798, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 798, raw_values_count: 1, token_types_start: 799, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 799, raw_values_count: 1, token_types_start: 800, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 800, raw_values_count: 1, token_types_start: 801, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 801, raw_values_count: 1, token_types_start: 802, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 802, raw_values_count: 1, token_types_start: 803, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 805, raw_values_count: 1, token_types_start: 806, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 806, raw_values_count: 2, token_types_start: 808, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 808, raw_values_count: 2, token_types_start: 810, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 810, raw_values_count: 1, token_types_start: 811, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 811, raw_values_count: 2, token_types_start: 813, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 813, raw_values_count: 1, token_types_start: 814, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 814, raw_values_count: 1, token_types_start: 815, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 815, raw_values_count: 1, token_types_start: 816, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 816, raw_values_count: 3, token_types_start: 819, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 820, raw_values_count: 2, token_types_start: 822, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 822, raw_values_count: 1, token_types_start: 823, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 823, raw_values_count: 1, token_types_start: 824, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 824, raw_values_count: 4, token_types_start: 828, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 828, raw_values_count: 7, token_types_start: 835, token_types_count: 3 }, // [424]
    SimpleHintData { raw_values_start: 838, raw_values_count: 4, token_types_start: 842, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 842, raw_values_count: 1, token_types_start: 843, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 843, raw_values_count: 1, token_types_start: 844, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 844, raw_values_count: 1, token_types_start: 845, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 845, raw_values_count: 1, token_types_start: 846, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 846, raw_values_count: 1, token_types_start: 847, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 847, raw_values_count: 1, token_types_start: 848, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 848, raw_values_count: 1, token_types_start: 849, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 849, raw_values_count: 1, token_types_start: 850, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 850, raw_values_count: 1, token_types_start: 851, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 852, raw_values_count: 1, token_types_start: 853, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 853, raw_values_count: 1, token_types_start: 854, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 854, raw_values_count: 1, token_types_start: 855, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 855, raw_values_count: 2, token_types_start: 857, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 857, raw_values_count: 0, token_types_start: 857, token_types_count: 1 }, // [440]
    SimpleHintData { raw_values_start: 858, raw_values_count: 1, token_types_start: 859, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 859, raw_values_count: 1, token_types_start: 860, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 860, raw_values_count: 1, token_types_start: 861, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 861, raw_values_count: 1, token_types_start: 862, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 862, raw_values_count: 1, token_types_start: 863, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 863, raw_values_count: 1, token_types_start: 864, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 864, raw_values_count: 1, token_types_start: 865, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 865, raw_values_count: 1, token_types_start: 866, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 866, raw_values_count: 1, token_types_start: 867, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 867, raw_values_count: 1, token_types_start: 868, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 868, raw_values_count: 4, token_types_start: 872, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 872, raw_values_count: 1, token_types_start: 873, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 873, raw_values_count: 1, token_types_start: 874, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 874, raw_values_count: 1, token_types_start: 875, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 875, raw_values_count: 1, token_types_start: 876, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 876, raw_values_count: 1, token_types_start: 877, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 877, raw_values_count: 1, token_types_start: 878, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 878, raw_values_count: 1, token_types_start: 879, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 879, raw_values_count: 1, token_types_start: 880, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 880, raw_values_count: 1, token_types_start: 881, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 881, raw_values_count: 1, token_types_start: 882, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 882, raw_values_count: 1, token_types_start: 883, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 884, raw_values_count: 1, token_types_start: 885, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 885, raw_values_count: 2, token_types_start: 887, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 887, raw_values_count: 1, token_types_start: 888, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 888, raw_values_count: 1, token_types_start: 889, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 889, raw_values_count: 1, token_types_start: 890, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 890, raw_values_count: 1, token_types_start: 891, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 891, raw_values_count: 1, token_types_start: 892, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 894, raw_values_count: 7, token_types_start: 901, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 901, raw_values_count: 4, token_types_start: 905, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 905, raw_values_count: 3, token_types_start: 908, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 908, raw_values_count: 1, token_types_start: 909, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 909, raw_values_count: 1, token_types_start: 910, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 910, raw_values_count: 1, token_types_start: 911, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 911, raw_values_count: 4, token_types_start: 915, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 915, raw_values_count: 1, token_types_start: 916, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 916, raw_values_count: 1, token_types_start: 917, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 917, raw_values_count: 3, token_types_start: 920, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 920, raw_values_count: 2, token_types_start: 922, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 922, raw_values_count: 7, token_types_start: 929, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 929, raw_values_count: 1, token_types_start: 930, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 930, raw_values_count: 1, token_types_start: 931, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 931, raw_values_count: 3, token_types_start: 934, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 934, raw_values_count: 11, token_types_start: 945, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 945, raw_values_count: 1, token_types_start: 946, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 946, raw_values_count: 3, token_types_start: 949, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 949, raw_values_count: 7, token_types_start: 956, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 956, raw_values_count: 6, token_types_start: 962, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 962, raw_values_count: 1, token_types_start: 963, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 963, raw_values_count: 2, token_types_start: 965, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 965, raw_values_count: 1, token_types_start: 966, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 966, raw_values_count: 5, token_types_start: 971, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 971, raw_values_count: 3, token_types_start: 974, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 975, raw_values_count: 2, token_types_start: 977, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 977, raw_values_count: 1, token_types_start: 978, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 978, raw_values_count: 2, token_types_start: 980, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 986, raw_values_count: 2, token_types_start: 988, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 990, raw_values_count: 2, token_types_start: 992, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 993, raw_values_count: 2, token_types_start: 995, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 995, raw_values_count: 3, token_types_start: 998, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 998, raw_values_count: 1, token_types_start: 999, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 999, raw_values_count: 1, token_types_start: 1000, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 2, token_types_start: 1002, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 1, token_types_start: 1003, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1003, raw_values_count: 1, token_types_start: 1004, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1004, raw_values_count: 1, token_types_start: 1005, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1005, raw_values_count: 2, token_types_start: 1007, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 1007, raw_values_count: 1, token_types_start: 1008, token_types_count: 2 }, // [522]
    SimpleHintData { raw_values_start: 1010, raw_values_count: 1, token_types_start: 1011, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1011, raw_values_count: 1, token_types_start: 1012, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 3, token_types_start: 1015, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 6, token_types_start: 1023, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 5, token_types_start: 1028, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 1, token_types_start: 1031, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 1, token_types_start: 1032, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 4, token_types_start: 1036, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 1, token_types_start: 1037, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1037, raw_values_count: 1, token_types_start: 1038, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1038, raw_values_count: 1, token_types_start: 1039, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1039, raw_values_count: 1, token_types_start: 1040, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 3, token_types_start: 1043, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1043, raw_values_count: 1, token_types_start: 1044, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1044, raw_values_count: 3, token_types_start: 1047, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1048, raw_values_count: 1, token_types_start: 1049, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1049, raw_values_count: 1, token_types_start: 1050, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1050, raw_values_count: 1, token_types_start: 1051, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1051, raw_values_count: 1, token_types_start: 1052, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1052, raw_values_count: 2, token_types_start: 1054, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 1, token_types_start: 1055, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1055, raw_values_count: 1, token_types_start: 1056, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 1 }, // [551]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 1, token_types_start: 1060, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 2, token_types_start: 1062, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 1, token_types_start: 1063, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 2, token_types_start: 1065, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 2, token_types_start: 1072, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 1, token_types_start: 1077, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1077, raw_values_count: 1, token_types_start: 1078, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1078, raw_values_count: 1, token_types_start: 1079, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1079, raw_values_count: 1, token_types_start: 1080, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1080, raw_values_count: 1, token_types_start: 1081, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1081, raw_values_count: 1, token_types_start: 1082, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 1, token_types_start: 1083, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1083, raw_values_count: 1, token_types_start: 1084, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1084, raw_values_count: 1, token_types_start: 1085, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1085, raw_values_count: 1, token_types_start: 1086, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1086, raw_values_count: 1, token_types_start: 1087, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1087, raw_values_count: 2, token_types_start: 1089, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1089, raw_values_count: 1, token_types_start: 1090, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1090, raw_values_count: 1, token_types_start: 1091, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1091, raw_values_count: 2, token_types_start: 1093, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1093, raw_values_count: 1, token_types_start: 1094, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1094, raw_values_count: 0, token_types_start: 1094, token_types_count: 1 }, // [582]
    SimpleHintData { raw_values_start: 1095, raw_values_count: 1, token_types_start: 1096, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1096, raw_values_count: 1, token_types_start: 1097, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1097, raw_values_count: 1, token_types_start: 1098, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 2, token_types_start: 1100, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 1, token_types_start: 1101, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1101, raw_values_count: 1, token_types_start: 1102, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 2, token_types_start: 1104, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1104, raw_values_count: 1, token_types_start: 1105, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1105, raw_values_count: 1, token_types_start: 1106, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1106, raw_values_count: 1, token_types_start: 1107, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1107, raw_values_count: 2, token_types_start: 1109, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1109, raw_values_count: 2, token_types_start: 1111, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1111, raw_values_count: 1, token_types_start: 1112, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1112, raw_values_count: 4, token_types_start: 1116, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 1, token_types_start: 1117, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1117, raw_values_count: 1, token_types_start: 1118, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 3, token_types_start: 1121, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 3, token_types_start: 1124, token_types_count: 2 }, // [600]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 3, token_types_start: 1129, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 0, token_types_start: 1130, token_types_count: 2 }, // [603]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [606]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 2, token_types_start: 1146, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 3, token_types_start: 1153, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 1, token_types_start: 1155, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 1, token_types_start: 1156, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 0, token_types_start: 1161, token_types_count: 3 }, // [630]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 2, token_types_start: 1181, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 1, token_types_start: 1189, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1189, raw_values_count: 1, token_types_start: 1190, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1190, raw_values_count: 1, token_types_start: 1191, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 1, token_types_start: 1192, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 1, token_types_start: 1194, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1194, raw_values_count: 1, token_types_start: 1195, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 1, token_types_start: 1196, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1196, raw_values_count: 1, token_types_start: 1197, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1197, raw_values_count: 1, token_types_start: 1198, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1198, raw_values_count: 1, token_types_start: 1199, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1199, raw_values_count: 1, token_types_start: 1200, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1200, raw_values_count: 1, token_types_start: 1201, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 1, token_types_start: 1202, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1202, raw_values_count: 1, token_types_start: 1203, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 1, token_types_start: 1204, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 1, token_types_start: 1205, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1205, raw_values_count: 2, token_types_start: 1207, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 3, token_types_start: 1210, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1210, raw_values_count: 2, token_types_start: 1212, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1212, raw_values_count: 2, token_types_start: 1214, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1214, raw_values_count: 1, token_types_start: 1215, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1215, raw_values_count: 1, token_types_start: 1216, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1216, raw_values_count: 1, token_types_start: 1217, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1217, raw_values_count: 1, token_types_start: 1218, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1218, raw_values_count: 1, token_types_start: 1219, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1219, raw_values_count: 1, token_types_start: 1220, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1220, raw_values_count: 1, token_types_start: 1221, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1221, raw_values_count: 1, token_types_start: 1222, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1222, raw_values_count: 1, token_types_start: 1223, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1223, raw_values_count: 1, token_types_start: 1224, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 1, token_types_start: 1225, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1225, raw_values_count: 1, token_types_start: 1226, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 1, token_types_start: 1232, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1232, raw_values_count: 1, token_types_start: 1233, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1233, raw_values_count: 1, token_types_start: 1234, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1234, raw_values_count: 1, token_types_start: 1235, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1235, raw_values_count: 1, token_types_start: 1236, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1236, raw_values_count: 1, token_types_start: 1237, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1237, raw_values_count: 1, token_types_start: 1238, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 1, token_types_start: 1239, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 1, token_types_start: 1241, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1241, raw_values_count: 1, token_types_start: 1242, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 1, token_types_start: 1245, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 1, token_types_start: 1247, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1247, raw_values_count: 1, token_types_start: 1248, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 1, token_types_start: 1250, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 8, token_types_start: 1262, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 1, token_types_start: 1263, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 4, token_types_start: 1267, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 2, token_types_start: 1269, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 2, token_types_start: 1271, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1271, raw_values_count: 6, token_types_start: 1277, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1277, raw_values_count: 2, token_types_start: 1279, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1279, raw_values_count: 2, token_types_start: 1281, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1281, raw_values_count: 1, token_types_start: 1282, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1282, raw_values_count: 15, token_types_start: 1297, token_types_count: 1 }, // [724]
    SimpleHintData { raw_values_start: 1298, raw_values_count: 3, token_types_start: 1301, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1301, raw_values_count: 1, token_types_start: 1302, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 6, token_types_start: 1308, token_types_count: 3 }, // [727]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 2, token_types_start: 1313, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 1, token_types_start: 1319, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1319, raw_values_count: 1, token_types_start: 1320, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1320, raw_values_count: 2, token_types_start: 1322, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1322, raw_values_count: 2, token_types_start: 1324, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 1, token_types_start: 1325, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1325, raw_values_count: 2, token_types_start: 1327, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 1, token_types_start: 1332, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1332, raw_values_count: 1, token_types_start: 1333, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 1, token_types_start: 1339, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 1, token_types_start: 1340, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1340, raw_values_count: 1, token_types_start: 1341, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1341, raw_values_count: 27, token_types_start: 1368, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 26, token_types_start: 1394, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1394, raw_values_count: 1, token_types_start: 1395, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1395, raw_values_count: 1, token_types_start: 1396, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1396, raw_values_count: 1, token_types_start: 1397, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1397, raw_values_count: 1, token_types_start: 1398, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1398, raw_values_count: 2, token_types_start: 1400, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1400, raw_values_count: 1, token_types_start: 1401, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1401, raw_values_count: 1, token_types_start: 1402, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1402, raw_values_count: 3, token_types_start: 1405, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1405, raw_values_count: 2, token_types_start: 1407, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1407, raw_values_count: 1, token_types_start: 1408, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1408, raw_values_count: 1, token_types_start: 1409, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1409, raw_values_count: 1, token_types_start: 1410, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1410, raw_values_count: 1, token_types_start: 1411, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 1, token_types_start: 1412, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1412, raw_values_count: 1, token_types_start: 1413, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 1, token_types_start: 1414, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 2, token_types_start: 1421, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 2, token_types_start: 1423, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 1, token_types_start: 1424, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1424, raw_values_count: 2, token_types_start: 1426, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 1, token_types_start: 1431, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1431, raw_values_count: 1, token_types_start: 1432, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 1, token_types_start: 1433, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1433, raw_values_count: 2, token_types_start: 1435, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 2, token_types_start: 1437, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 1, token_types_start: 1439, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 1, token_types_start: 1440, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1440, raw_values_count: 1, token_types_start: 1441, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1441, raw_values_count: 1, token_types_start: 1442, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1442, raw_values_count: 17, token_types_start: 1459, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 1, token_types_start: 1462, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 4, token_types_start: 1466, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 4, token_types_start: 1472, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 1, token_types_start: 1473, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 3, token_types_start: 1477, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 2, token_types_start: 1480, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 0, token_types_start: 1480, token_types_count: 1 }, // [808]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 0, token_types_start: 1481, token_types_count: 2 }, // [809]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 2, token_types_start: 1494, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 13, token_types_start: 1509, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 1, token_types_start: 1512, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1512, raw_values_count: 1, token_types_start: 1513, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 1, token_types_start: 1515, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 0, token_types_start: 1518, token_types_count: 1 }, // [832]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 1, token_types_start: 1520, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 1, token_types_start: 1522, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 1, token_types_start: 1523, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 1, token_types_start: 1524, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1524, raw_values_count: 6, token_types_start: 1530, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1530, raw_values_count: 1, token_types_start: 1531, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 1, token_types_start: 1532, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1532, raw_values_count: 1, token_types_start: 1533, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1533, raw_values_count: 1, token_types_start: 1534, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 5, token_types_start: 1541, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 1, token_types_start: 1544, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1544, raw_values_count: 1, token_types_start: 1545, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1545, raw_values_count: 1, token_types_start: 1546, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1546, raw_values_count: 1, token_types_start: 1547, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1547, raw_values_count: 1, token_types_start: 1548, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1548, raw_values_count: 1, token_types_start: 1549, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 1, token_types_start: 1551, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1551, raw_values_count: 1, token_types_start: 1552, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 1, token_types_start: 1556, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 4, token_types_start: 1562, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 1, token_types_start: 1564, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 1, token_types_start: 1565, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 1, token_types_start: 1566, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 1, token_types_start: 1567, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1567, raw_values_count: 1, token_types_start: 1568, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1568, raw_values_count: 1, token_types_start: 1569, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 3, token_types_start: 1578, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 3, token_types_start: 1581, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1581, raw_values_count: 1, token_types_start: 1582, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1582, raw_values_count: 1, token_types_start: 1583, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 1, token_types_start: 1584, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1584, raw_values_count: 2, token_types_start: 1586, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 2, token_types_start: 1589, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 2, token_types_start: 1592, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 3, token_types_start: 1595, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1595, raw_values_count: 1, token_types_start: 1596, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 1, token_types_start: 1605, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 1, token_types_start: 1606, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 1, token_types_start: 1607, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1607, raw_values_count: 1, token_types_start: 1608, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1608, raw_values_count: 1, token_types_start: 1609, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 1, token_types_start: 1610, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1610, raw_values_count: 3, token_types_start: 1613, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1613, raw_values_count: 6, token_types_start: 1619, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 1, token_types_start: 1620, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1620, raw_values_count: 1, token_types_start: 1621, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 1, token_types_start: 1627, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1627, raw_values_count: 1, token_types_start: 1628, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 2, token_types_start: 1636, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 2, token_types_start: 1639, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 10, token_types_start: 1652, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 1, token_types_start: 1654, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1654, raw_values_count: 1, token_types_start: 1655, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1655, raw_values_count: 1, token_types_start: 1656, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 2, token_types_start: 1663, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 3, token_types_start: 1666, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 2, token_types_start: 1675, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 3, token_types_start: 1678, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 2, token_types_start: 1680, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 3, token_types_start: 1702, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 2, token_types_start: 1706, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 2, token_types_start: 1714, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 2, token_types_start: 1716, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1716, raw_values_count: 1, token_types_start: 1717, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1717, raw_values_count: 2, token_types_start: 1719, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1719, raw_values_count: 1, token_types_start: 1720, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1720, raw_values_count: 1, token_types_start: 1721, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1721, raw_values_count: 1, token_types_start: 1722, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 1, token_types_start: 1725, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 1, token_types_start: 1726, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 1, token_types_start: 1737, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 1, token_types_start: 1750, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1750, raw_values_count: 1, token_types_start: 1751, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 1, token_types_start: 1761, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 1, token_types_start: 1763, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 1, token_types_start: 1764, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1764, raw_values_count: 1, token_types_start: 1765, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1765, raw_values_count: 1, token_types_start: 1766, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1766, raw_values_count: 1, token_types_start: 1767, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1767, raw_values_count: 1, token_types_start: 1768, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1768, raw_values_count: 1, token_types_start: 1769, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1769, raw_values_count: 1, token_types_start: 1770, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1770, raw_values_count: 1, token_types_start: 1771, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1771, raw_values_count: 1, token_types_start: 1772, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1772, raw_values_count: 1, token_types_start: 1773, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1773, raw_values_count: 1, token_types_start: 1774, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1774, raw_values_count: 1, token_types_start: 1775, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 1, token_types_start: 1776, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 1, token_types_start: 1777, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 1, token_types_start: 1778, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1778, raw_values_count: 1, token_types_start: 1779, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 1, token_types_start: 1780, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 1, token_types_start: 1792, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1792, raw_values_count: 1, token_types_start: 1793, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 1, token_types_start: 1795, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 1, token_types_start: 1796, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 1, token_types_start: 1799, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 1, token_types_start: 1801, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1801, raw_values_count: 1, token_types_start: 1802, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1802, raw_values_count: 1, token_types_start: 1803, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1803, raw_values_count: 1, token_types_start: 1804, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 1, token_types_start: 1805, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1805, raw_values_count: 1, token_types_start: 1806, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1806, raw_values_count: 1, token_types_start: 1807, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1807, raw_values_count: 1, token_types_start: 1808, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 1808, raw_values_count: 1, token_types_start: 1809, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1809, raw_values_count: 1, token_types_start: 1810, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1810, raw_values_count: 1, token_types_start: 1811, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1811, raw_values_count: 1, token_types_start: 1812, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1812, raw_values_count: 1, token_types_start: 1813, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1813, raw_values_count: 1, token_types_start: 1814, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1814, raw_values_count: 1, token_types_start: 1815, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1815, raw_values_count: 1, token_types_start: 1816, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1816, raw_values_count: 1, token_types_start: 1817, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 1, token_types_start: 1818, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1818, raw_values_count: 1, token_types_start: 1819, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1819, raw_values_count: 1, token_types_start: 1820, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1820, raw_values_count: 1, token_types_start: 1821, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 1, token_types_start: 1823, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1823, raw_values_count: 4, token_types_start: 1827, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 2, token_types_start: 1829, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 1, token_types_start: 1836, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 1836, raw_values_count: 1, token_types_start: 1837, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 1837, raw_values_count: 1, token_types_start: 1838, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 1838, raw_values_count: 1, token_types_start: 1839, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 1839, raw_values_count: 1, token_types_start: 1840, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 1840, raw_values_count: 1, token_types_start: 1841, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 1, token_types_start: 1842, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 1842, raw_values_count: 1, token_types_start: 1843, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 1, token_types_start: 1844, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 1844, raw_values_count: 1, token_types_start: 1845, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 1, token_types_start: 1846, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 1846, raw_values_count: 1, token_types_start: 1847, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 1847, raw_values_count: 1, token_types_start: 1848, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 1848, raw_values_count: 1, token_types_start: 1849, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 1849, raw_values_count: 1, token_types_start: 1850, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 1850, raw_values_count: 1, token_types_start: 1851, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 1851, raw_values_count: 1, token_types_start: 1852, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 1852, raw_values_count: 1, token_types_start: 1853, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 1, token_types_start: 1855, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 1858, raw_values_count: 1, token_types_start: 1859, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 1, token_types_start: 1860, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 1860, raw_values_count: 1, token_types_start: 1861, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 1861, raw_values_count: 1, token_types_start: 1862, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 1862, raw_values_count: 1, token_types_start: 1863, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 1, token_types_start: 1864, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 1864, raw_values_count: 2, token_types_start: 1866, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 8, token_types_start: 1874, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 1, token_types_start: 1875, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 1875, raw_values_count: 1, token_types_start: 1876, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 1876, raw_values_count: 1, token_types_start: 1877, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 1, token_types_start: 1879, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 1879, raw_values_count: 1, token_types_start: 1880, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 1880, raw_values_count: 1, token_types_start: 1881, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 1, token_types_start: 1882, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 1882, raw_values_count: 1, token_types_start: 1883, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 1883, raw_values_count: 1, token_types_start: 1884, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 1884, raw_values_count: 1, token_types_start: 1885, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 1885, raw_values_count: 1, token_types_start: 1886, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 1886, raw_values_count: 1, token_types_start: 1887, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 1887, raw_values_count: 1, token_types_start: 1888, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 1888, raw_values_count: 1, token_types_start: 1889, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 1, token_types_start: 1891, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 1891, raw_values_count: 1, token_types_start: 1892, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 1, token_types_start: 1893, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 1893, raw_values_count: 1, token_types_start: 1894, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 1894, raw_values_count: 1, token_types_start: 1895, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 1895, raw_values_count: 1, token_types_start: 1896, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 1896, raw_values_count: 1, token_types_start: 1897, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 1897, raw_values_count: 1, token_types_start: 1898, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 1, token_types_start: 1900, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 1900, raw_values_count: 1, token_types_start: 1901, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 1901, raw_values_count: 1, token_types_start: 1902, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 1902, raw_values_count: 1, token_types_start: 1903, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 1903, raw_values_count: 1, token_types_start: 1904, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 1904, raw_values_count: 1, token_types_start: 1905, token_types_count: 2 }, // [1154]
    SimpleHintData { raw_values_start: 1907, raw_values_count: 1, token_types_start: 1908, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 1908, raw_values_count: 1, token_types_start: 1909, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 1909, raw_values_count: 1, token_types_start: 1910, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 1910, raw_values_count: 1, token_types_start: 1911, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 1911, raw_values_count: 1, token_types_start: 1912, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 1912, raw_values_count: 1, token_types_start: 1913, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 1913, raw_values_count: 1, token_types_start: 1914, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 1914, raw_values_count: 1, token_types_start: 1915, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 1, token_types_start: 1916, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 1916, raw_values_count: 1, token_types_start: 1917, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 1, token_types_start: 1918, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 1918, raw_values_count: 1, token_types_start: 1919, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 1919, raw_values_count: 1, token_types_start: 1920, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 1920, raw_values_count: 1, token_types_start: 1921, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 1921, raw_values_count: 1, token_types_start: 1922, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 1, token_types_start: 1924, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 1, token_types_start: 1925, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 1, token_types_start: 1926, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 1926, raw_values_count: 1, token_types_start: 1927, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 1, token_types_start: 1928, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 1928, raw_values_count: 1, token_types_start: 1929, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 1929, raw_values_count: 1, token_types_start: 1930, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 1930, raw_values_count: 1, token_types_start: 1931, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 1931, raw_values_count: 1, token_types_start: 1932, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 1932, raw_values_count: 1, token_types_start: 1933, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 1933, raw_values_count: 1, token_types_start: 1934, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 1934, raw_values_count: 1, token_types_start: 1935, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 1935, raw_values_count: 1, token_types_start: 1936, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 1936, raw_values_count: 1, token_types_start: 1937, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 1937, raw_values_count: 1, token_types_start: 1938, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 1938, raw_values_count: 3, token_types_start: 1941, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 1941, raw_values_count: 1, token_types_start: 1942, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 1942, raw_values_count: 1, token_types_start: 1943, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 1943, raw_values_count: 1, token_types_start: 1944, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 1944, raw_values_count: 1, token_types_start: 1945, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 1945, raw_values_count: 1, token_types_start: 1946, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 1946, raw_values_count: 1, token_types_start: 1947, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 1947, raw_values_count: 2, token_types_start: 1949, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 1949, raw_values_count: 2, token_types_start: 1951, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 1951, raw_values_count: 3, token_types_start: 1954, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 1954, raw_values_count: 1, token_types_start: 1955, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 1955, raw_values_count: 1, token_types_start: 1956, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 1, token_types_start: 1957, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 1, token_types_start: 1958, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 1958, raw_values_count: 1, token_types_start: 1959, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 1959, raw_values_count: 1, token_types_start: 1960, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 1960, raw_values_count: 1, token_types_start: 1961, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 1961, raw_values_count: 1, token_types_start: 1962, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 1962, raw_values_count: 1, token_types_start: 1963, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 1963, raw_values_count: 1, token_types_start: 1964, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 1964, raw_values_count: 1, token_types_start: 1965, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 1965, raw_values_count: 1, token_types_start: 1966, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 1966, raw_values_count: 1, token_types_start: 1967, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 1967, raw_values_count: 1, token_types_start: 1968, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 1968, raw_values_count: 1, token_types_start: 1969, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 1969, raw_values_count: 1, token_types_start: 1970, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 1970, raw_values_count: 1, token_types_start: 1971, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 1971, raw_values_count: 1, token_types_start: 1972, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 1972, raw_values_count: 1, token_types_start: 1973, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 1973, raw_values_count: 1, token_types_start: 1974, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 1974, raw_values_count: 1, token_types_start: 1975, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 1975, raw_values_count: 1, token_types_start: 1976, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 1976, raw_values_count: 3, token_types_start: 1979, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 1979, raw_values_count: 1, token_types_start: 1980, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 1, token_types_start: 1981, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 1, token_types_start: 1982, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 1982, raw_values_count: 1, token_types_start: 1983, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 1983, raw_values_count: 1, token_types_start: 1984, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 1984, raw_values_count: 1, token_types_start: 1985, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 1985, raw_values_count: 1, token_types_start: 1986, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 1986, raw_values_count: 1, token_types_start: 1987, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 1987, raw_values_count: 1, token_types_start: 1988, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 1988, raw_values_count: 1, token_types_start: 1989, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 1989, raw_values_count: 1, token_types_start: 1990, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 1990, raw_values_count: 1, token_types_start: 1991, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 1991, raw_values_count: 1, token_types_start: 1992, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 1992, raw_values_count: 1, token_types_start: 1993, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 1993, raw_values_count: 1, token_types_start: 1994, token_types_count: 0 }, // [1233]
    SimpleHintData { raw_values_start: 1994, raw_values_count: 1, token_types_start: 1995, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 1995, raw_values_count: 1, token_types_start: 1996, token_types_count: 0 }, // [1235]
    SimpleHintData { raw_values_start: 1996, raw_values_count: 1, token_types_start: 1997, token_types_count: 0 }, // [1236]
    SimpleHintData { raw_values_start: 1997, raw_values_count: 1, token_types_start: 1998, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 1998, raw_values_count: 1, token_types_start: 1999, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 1999, raw_values_count: 1, token_types_start: 2000, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 2000, raw_values_count: 3, token_types_start: 2003, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 2003, raw_values_count: 1, token_types_start: 2004, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 2004, raw_values_count: 1, token_types_start: 2005, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 2005, raw_values_count: 1, token_types_start: 2006, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 2006, raw_values_count: 1, token_types_start: 2007, token_types_count: 0 }, // [1244]
    SimpleHintData { raw_values_start: 2007, raw_values_count: 2, token_types_start: 2009, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 2009, raw_values_count: 5, token_types_start: 2014, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 2014, raw_values_count: 3, token_types_start: 2017, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 2017, raw_values_count: 1, token_types_start: 2018, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 2018, raw_values_count: 1, token_types_start: 2019, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 2019, raw_values_count: 1, token_types_start: 2020, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 2020, raw_values_count: 1, token_types_start: 2021, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 2021, raw_values_count: 1, token_types_start: 2022, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 2022, raw_values_count: 1, token_types_start: 2023, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 2023, raw_values_count: 1, token_types_start: 2024, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [1255]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 1, token_types_start: 2026, token_types_count: 0 }, // [1256]
    SimpleHintData { raw_values_start: 2026, raw_values_count: 1, token_types_start: 2027, token_types_count: 0 }, // [1257]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 1, token_types_start: 2028, token_types_count: 0 }, // [1258]
    SimpleHintData { raw_values_start: 2028, raw_values_count: 1, token_types_start: 2029, token_types_count: 0 }, // [1259]
    SimpleHintData { raw_values_start: 2029, raw_values_count: 1, token_types_start: 2030, token_types_count: 0 }, // [1260]
    SimpleHintData { raw_values_start: 2030, raw_values_count: 2, token_types_start: 2032, token_types_count: 0 }, // [1261]
    SimpleHintData { raw_values_start: 2032, raw_values_count: 1, token_types_start: 2033, token_types_count: 0 }, // [1262]
    SimpleHintData { raw_values_start: 2033, raw_values_count: 1, token_types_start: 2034, token_types_count: 0 }, // [1263]
    SimpleHintData { raw_values_start: 2034, raw_values_count: 1, token_types_start: 2035, token_types_count: 0 }, // [1264]
    SimpleHintData { raw_values_start: 2035, raw_values_count: 1, token_types_start: 2036, token_types_count: 0 }, // [1265]
    SimpleHintData { raw_values_start: 2036, raw_values_count: 1, token_types_start: 2037, token_types_count: 0 }, // [1266]
    SimpleHintData { raw_values_start: 2037, raw_values_count: 1, token_types_start: 2038, token_types_count: 0 }, // [1267]
    SimpleHintData { raw_values_start: 2038, raw_values_count: 1, token_types_start: 2039, token_types_count: 0 }, // [1268]
    SimpleHintData { raw_values_start: 2039, raw_values_count: 1, token_types_start: 2040, token_types_count: 0 }, // [1269]
    SimpleHintData { raw_values_start: 2040, raw_values_count: 1, token_types_start: 2041, token_types_count: 0 }, // [1270]
    SimpleHintData { raw_values_start: 2041, raw_values_count: 1, token_types_start: 2042, token_types_count: 0 }, // [1271]
    SimpleHintData { raw_values_start: 2042, raw_values_count: 1, token_types_start: 2043, token_types_count: 0 }, // [1272]
    SimpleHintData { raw_values_start: 2043, raw_values_count: 1, token_types_start: 2044, token_types_count: 0 }, // [1273]
    SimpleHintData { raw_values_start: 2044, raw_values_count: 1, token_types_start: 2045, token_types_count: 0 }, // [1274]
    SimpleHintData { raw_values_start: 2045, raw_values_count: 1, token_types_start: 2046, token_types_count: 0 }, // [1275]
    SimpleHintData { raw_values_start: 2046, raw_values_count: 1, token_types_start: 2047, token_types_count: 0 }, // [1276]
    SimpleHintData { raw_values_start: 2047, raw_values_count: 1, token_types_start: 2048, token_types_count: 0 }, // [1277]
    SimpleHintData { raw_values_start: 2048, raw_values_count: 1, token_types_start: 2049, token_types_count: 0 }, // [1278]
    SimpleHintData { raw_values_start: 2049, raw_values_count: 1, token_types_start: 2050, token_types_count: 0 }, // [1279]
    SimpleHintData { raw_values_start: 2050, raw_values_count: 1, token_types_start: 2051, token_types_count: 0 }, // [1280]
    SimpleHintData { raw_values_start: 2051, raw_values_count: 1, token_types_start: 2052, token_types_count: 0 }, // [1281]
    SimpleHintData { raw_values_start: 2052, raw_values_count: 1, token_types_start: 2053, token_types_count: 0 }, // [1282]
    SimpleHintData { raw_values_start: 2053, raw_values_count: 1, token_types_start: 2054, token_types_count: 0 }, // [1283]
    SimpleHintData { raw_values_start: 2054, raw_values_count: 1, token_types_start: 2055, token_types_count: 0 }, // [1284]
    SimpleHintData { raw_values_start: 2055, raw_values_count: 1, token_types_start: 2056, token_types_count: 0 }, // [1285]
    SimpleHintData { raw_values_start: 2056, raw_values_count: 1, token_types_start: 2057, token_types_count: 0 }, // [1286]
    SimpleHintData { raw_values_start: 2057, raw_values_count: 2, token_types_start: 2059, token_types_count: 0 }, // [1287]
    SimpleHintData { raw_values_start: 2059, raw_values_count: 1, token_types_start: 2060, token_types_count: 0 }, // [1288]
    SimpleHintData { raw_values_start: 2060, raw_values_count: 1, token_types_start: 2061, token_types_count: 0 }, // [1289]
    SimpleHintData { raw_values_start: 2061, raw_values_count: 1, token_types_start: 2062, token_types_count: 0 }, // [1290]
    SimpleHintData { raw_values_start: 2062, raw_values_count: 1, token_types_start: 2063, token_types_count: 0 }, // [1291]
    SimpleHintData { raw_values_start: 2063, raw_values_count: 1, token_types_start: 2064, token_types_count: 0 }, // [1292]
    SimpleHintData { raw_values_start: 2064, raw_values_count: 1, token_types_start: 2065, token_types_count: 0 }, // [1293]
    SimpleHintData { raw_values_start: 2065, raw_values_count: 1, token_types_start: 2066, token_types_count: 0 }, // [1294]
    SimpleHintData { raw_values_start: 2066, raw_values_count: 1, token_types_start: 2067, token_types_count: 0 }, // [1295]
    SimpleHintData { raw_values_start: 2067, raw_values_count: 1, token_types_start: 2068, token_types_count: 0 }, // [1296]
    SimpleHintData { raw_values_start: 2068, raw_values_count: 1, token_types_start: 2069, token_types_count: 0 }, // [1297]
    SimpleHintData { raw_values_start: 2069, raw_values_count: 1, token_types_start: 2070, token_types_count: 0 }, // [1298]
    SimpleHintData { raw_values_start: 2070, raw_values_count: 1, token_types_start: 2071, token_types_count: 0 }, // [1299]
    SimpleHintData { raw_values_start: 2071, raw_values_count: 1, token_types_start: 2072, token_types_count: 0 }, // [1300]
    SimpleHintData { raw_values_start: 2072, raw_values_count: 1, token_types_start: 2073, token_types_count: 0 }, // [1301]
    SimpleHintData { raw_values_start: 2073, raw_values_count: 1, token_types_start: 2074, token_types_count: 0 }, // [1302]
    SimpleHintData { raw_values_start: 2074, raw_values_count: 1, token_types_start: 2075, token_types_count: 0 }, // [1303]
    SimpleHintData { raw_values_start: 2075, raw_values_count: 11, token_types_start: 2086, token_types_count: 0 }, // [1304]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 3, token_types_start: 2089, token_types_count: 0 }, // [1305]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 1, token_types_start: 2090, token_types_count: 0 }, // [1306]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 1, token_types_start: 2091, token_types_count: 0 }, // [1307]
    SimpleHintData { raw_values_start: 2091, raw_values_count: 1, token_types_start: 2092, token_types_count: 0 }, // [1308]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 1, token_types_start: 2093, token_types_count: 0 }, // [1309]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 1, token_types_start: 2094, token_types_count: 0 }, // [1310]
    SimpleHintData { raw_values_start: 2094, raw_values_count: 1, token_types_start: 2095, token_types_count: 0 }, // [1311]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [1312]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 2, token_types_start: 2098, token_types_count: 0 }, // [1313]
    SimpleHintData { raw_values_start: 2098, raw_values_count: 1, token_types_start: 2099, token_types_count: 0 }, // [1314]
    SimpleHintData { raw_values_start: 2099, raw_values_count: 1, token_types_start: 2100, token_types_count: 0 }, // [1315]
    SimpleHintData { raw_values_start: 2100, raw_values_count: 1, token_types_start: 2101, token_types_count: 0 }, // [1316]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 1, token_types_start: 2102, token_types_count: 0 }, // [1317]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 1, token_types_start: 2103, token_types_count: 0 }, // [1318]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [1319]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 2, token_types_start: 2106, token_types_count: 0 }, // [1320]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 5, token_types_start: 2111, token_types_count: 0 }, // [1321]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 2, token_types_start: 2113, token_types_count: 0 }, // [1322]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 1, token_types_start: 2114, token_types_count: 0 }, // [1323]
    SimpleHintData { raw_values_start: 2114, raw_values_count: 2, token_types_start: 2116, token_types_count: 0 }, // [1324]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [1325]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 2, token_types_start: 2119, token_types_count: 0 }, // [1326]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 3, token_types_start: 2122, token_types_count: 0 }, // [1327]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 2, token_types_start: 2124, token_types_count: 0 }, // [1328]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [1329]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 2, token_types_start: 2127, token_types_count: 0 }, // [1330]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [1331]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 1, token_types_start: 2129, token_types_count: 0 }, // [1332]
    SimpleHintData { raw_values_start: 2129, raw_values_count: 1, token_types_start: 2130, token_types_count: 0 }, // [1333]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [1334]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [1335]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [1336]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [1337]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [1338]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 1, token_types_start: 2136, token_types_count: 0 }, // [1339]
    SimpleHintData { raw_values_start: 2136, raw_values_count: 1, token_types_start: 2137, token_types_count: 0 }, // [1340]
    SimpleHintData { raw_values_start: 2137, raw_values_count: 1, token_types_start: 2138, token_types_count: 0 }, // [1341]
    SimpleHintData { raw_values_start: 2138, raw_values_count: 1, token_types_start: 2139, token_types_count: 0 }, // [1342]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 1, token_types_start: 2140, token_types_count: 0 }, // [1343]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [1344]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [1345]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 1, token_types_start: 2143, token_types_count: 0 }, // [1346]
    SimpleHintData { raw_values_start: 2143, raw_values_count: 1, token_types_start: 2144, token_types_count: 0 }, // [1347]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [1348]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 1, token_types_start: 2146, token_types_count: 0 }, // [1349]
    SimpleHintData { raw_values_start: 2146, raw_values_count: 1, token_types_start: 2147, token_types_count: 0 }, // [1350]
    SimpleHintData { raw_values_start: 2147, raw_values_count: 1, token_types_start: 2148, token_types_count: 0 }, // [1351]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 1, token_types_start: 2149, token_types_count: 0 }, // [1352]
    SimpleHintData { raw_values_start: 2149, raw_values_count: 1, token_types_start: 2150, token_types_count: 0 }, // [1353]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [1354]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 1, token_types_start: 2152, token_types_count: 0 }, // [1355]
    SimpleHintData { raw_values_start: 2152, raw_values_count: 1, token_types_start: 2153, token_types_count: 0 }, // [1356]
    SimpleHintData { raw_values_start: 2153, raw_values_count: 1, token_types_start: 2154, token_types_count: 0 }, // [1357]
    SimpleHintData { raw_values_start: 2154, raw_values_count: 1, token_types_start: 2155, token_types_count: 0 }, // [1358]
    SimpleHintData { raw_values_start: 2155, raw_values_count: 1, token_types_start: 2156, token_types_count: 0 }, // [1359]
    SimpleHintData { raw_values_start: 2156, raw_values_count: 1, token_types_start: 2157, token_types_count: 0 }, // [1360]
    SimpleHintData { raw_values_start: 2157, raw_values_count: 1, token_types_start: 2158, token_types_count: 0 }, // [1361]
    SimpleHintData { raw_values_start: 2158, raw_values_count: 1, token_types_start: 2159, token_types_count: 0 }, // [1362]
    SimpleHintData { raw_values_start: 2159, raw_values_count: 1, token_types_start: 2160, token_types_count: 0 }, // [1363]
    SimpleHintData { raw_values_start: 2160, raw_values_count: 1, token_types_start: 2161, token_types_count: 0 }, // [1364]
    SimpleHintData { raw_values_start: 2161, raw_values_count: 1, token_types_start: 2162, token_types_count: 0 }, // [1365]
    SimpleHintData { raw_values_start: 2162, raw_values_count: 1, token_types_start: 2163, token_types_count: 0 }, // [1366]
    SimpleHintData { raw_values_start: 2163, raw_values_count: 1, token_types_start: 2164, token_types_count: 0 }, // [1367]
    SimpleHintData { raw_values_start: 2164, raw_values_count: 1, token_types_start: 2165, token_types_count: 0 }, // [1368]
    SimpleHintData { raw_values_start: 2165, raw_values_count: 1, token_types_start: 2166, token_types_count: 0 }, // [1369]
    SimpleHintData { raw_values_start: 2166, raw_values_count: 1, token_types_start: 2167, token_types_count: 0 }, // [1370]
    SimpleHintData { raw_values_start: 2167, raw_values_count: 1, token_types_start: 2168, token_types_count: 0 }, // [1371]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 1, token_types_start: 2169, token_types_count: 0 }, // [1372]
    SimpleHintData { raw_values_start: 2169, raw_values_count: 1, token_types_start: 2170, token_types_count: 0 }, // [1373]
    SimpleHintData { raw_values_start: 2170, raw_values_count: 1, token_types_start: 2171, token_types_count: 0 }, // [1374]
    SimpleHintData { raw_values_start: 2171, raw_values_count: 1, token_types_start: 2172, token_types_count: 0 }, // [1375]
    SimpleHintData { raw_values_start: 2172, raw_values_count: 1, token_types_start: 2173, token_types_count: 0 }, // [1376]
    SimpleHintData { raw_values_start: 2173, raw_values_count: 1, token_types_start: 2174, token_types_count: 0 }, // [1377]
    SimpleHintData { raw_values_start: 2174, raw_values_count: 1, token_types_start: 2175, token_types_count: 0 }, // [1378]
    SimpleHintData { raw_values_start: 2175, raw_values_count: 1, token_types_start: 2176, token_types_count: 0 }, // [1379]
    SimpleHintData { raw_values_start: 2176, raw_values_count: 1, token_types_start: 2177, token_types_count: 0 }, // [1380]
    SimpleHintData { raw_values_start: 2177, raw_values_count: 1, token_types_start: 2178, token_types_count: 0 }, // [1381]
    SimpleHintData { raw_values_start: 2178, raw_values_count: 1, token_types_start: 2179, token_types_count: 0 }, // [1382]
    SimpleHintData { raw_values_start: 2179, raw_values_count: 1, token_types_start: 2180, token_types_count: 0 }, // [1383]
    SimpleHintData { raw_values_start: 2180, raw_values_count: 1, token_types_start: 2181, token_types_count: 0 }, // [1384]
    SimpleHintData { raw_values_start: 2181, raw_values_count: 1, token_types_start: 2182, token_types_count: 0 }, // [1385]
    SimpleHintData { raw_values_start: 2182, raw_values_count: 1, token_types_start: 2183, token_types_count: 0 }, // [1386]
    SimpleHintData { raw_values_start: 2183, raw_values_count: 1, token_types_start: 2184, token_types_count: 0 }, // [1387]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 1, token_types_start: 2185, token_types_count: 0 }, // [1388]
    SimpleHintData { raw_values_start: 2185, raw_values_count: 1, token_types_start: 2186, token_types_count: 0 }, // [1389]
    SimpleHintData { raw_values_start: 2186, raw_values_count: 1, token_types_start: 2187, token_types_count: 0 }, // [1390]
    SimpleHintData { raw_values_start: 2187, raw_values_count: 1, token_types_start: 2188, token_types_count: 0 }, // [1391]
    SimpleHintData { raw_values_start: 2188, raw_values_count: 1, token_types_start: 2189, token_types_count: 0 }, // [1392]
    SimpleHintData { raw_values_start: 2189, raw_values_count: 1, token_types_start: 2190, token_types_count: 0 }, // [1393]
    SimpleHintData { raw_values_start: 2190, raw_values_count: 1, token_types_start: 2191, token_types_count: 0 }, // [1394]
    SimpleHintData { raw_values_start: 2191, raw_values_count: 1, token_types_start: 2192, token_types_count: 0 }, // [1395]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 1, token_types_start: 2193, token_types_count: 0 }, // [1396]
    SimpleHintData { raw_values_start: 2193, raw_values_count: 1, token_types_start: 2194, token_types_count: 0 }, // [1397]
    SimpleHintData { raw_values_start: 2194, raw_values_count: 1, token_types_start: 2195, token_types_count: 0 }, // [1398]
    SimpleHintData { raw_values_start: 2195, raw_values_count: 1, token_types_start: 2196, token_types_count: 0 }, // [1399]
    SimpleHintData { raw_values_start: 2196, raw_values_count: 1, token_types_start: 2197, token_types_count: 0 }, // [1400]
    SimpleHintData { raw_values_start: 2197, raw_values_count: 1, token_types_start: 2198, token_types_count: 0 }, // [1401]
    SimpleHintData { raw_values_start: 2198, raw_values_count: 1, token_types_start: 2199, token_types_count: 0 }, // [1402]
    SimpleHintData { raw_values_start: 2199, raw_values_count: 1, token_types_start: 2200, token_types_count: 0 }, // [1403]
    SimpleHintData { raw_values_start: 2200, raw_values_count: 1, token_types_start: 2201, token_types_count: 0 }, // [1404]
    SimpleHintData { raw_values_start: 2201, raw_values_count: 1, token_types_start: 2202, token_types_count: 0 }, // [1405]
    SimpleHintData { raw_values_start: 2202, raw_values_count: 1, token_types_start: 2203, token_types_count: 0 }, // [1406]
    SimpleHintData { raw_values_start: 2203, raw_values_count: 1, token_types_start: 2204, token_types_count: 0 }, // [1407]
    SimpleHintData { raw_values_start: 2204, raw_values_count: 1, token_types_start: 2205, token_types_count: 0 }, // [1408]
    SimpleHintData { raw_values_start: 2205, raw_values_count: 1, token_types_start: 2206, token_types_count: 0 }, // [1409]
    SimpleHintData { raw_values_start: 2206, raw_values_count: 1, token_types_start: 2207, token_types_count: 0 }, // [1410]
    SimpleHintData { raw_values_start: 2207, raw_values_count: 1, token_types_start: 2208, token_types_count: 0 }, // [1411]
    SimpleHintData { raw_values_start: 2208, raw_values_count: 1, token_types_start: 2209, token_types_count: 0 }, // [1412]
    SimpleHintData { raw_values_start: 2209, raw_values_count: 1, token_types_start: 2210, token_types_count: 0 }, // [1413]
    SimpleHintData { raw_values_start: 2210, raw_values_count: 1, token_types_start: 2211, token_types_count: 0 }, // [1414]
    SimpleHintData { raw_values_start: 2211, raw_values_count: 1, token_types_start: 2212, token_types_count: 0 }, // [1415]
    SimpleHintData { raw_values_start: 2212, raw_values_count: 1, token_types_start: 2213, token_types_count: 0 }, // [1416]
    SimpleHintData { raw_values_start: 2213, raw_values_count: 1, token_types_start: 2214, token_types_count: 0 }, // [1417]
    SimpleHintData { raw_values_start: 2214, raw_values_count: 1, token_types_start: 2215, token_types_count: 0 }, // [1418]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 1, token_types_start: 2216, token_types_count: 0 }, // [1419]
    SimpleHintData { raw_values_start: 2216, raw_values_count: 1, token_types_start: 2217, token_types_count: 0 }, // [1420]
    SimpleHintData { raw_values_start: 2217, raw_values_count: 1, token_types_start: 2218, token_types_count: 0 }, // [1421]
    SimpleHintData { raw_values_start: 2218, raw_values_count: 1, token_types_start: 2219, token_types_count: 0 }, // [1422]
    SimpleHintData { raw_values_start: 2219, raw_values_count: 1, token_types_start: 2220, token_types_count: 0 }, // [1423]
    SimpleHintData { raw_values_start: 2220, raw_values_count: 1, token_types_start: 2221, token_types_count: 0 }, // [1424]
    SimpleHintData { raw_values_start: 2221, raw_values_count: 1, token_types_start: 2222, token_types_count: 0 }, // [1425]
    SimpleHintData { raw_values_start: 2222, raw_values_count: 1, token_types_start: 2223, token_types_count: 0 }, // [1426]
    SimpleHintData { raw_values_start: 2223, raw_values_count: 1, token_types_start: 2224, token_types_count: 0 }, // [1427]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 1, token_types_start: 2225, token_types_count: 0 }, // [1428]
    SimpleHintData { raw_values_start: 2225, raw_values_count: 1, token_types_start: 2226, token_types_count: 0 }, // [1429]
    SimpleHintData { raw_values_start: 2226, raw_values_count: 1, token_types_start: 2227, token_types_count: 0 }, // [1430]
    SimpleHintData { raw_values_start: 2227, raw_values_count: 1, token_types_start: 2228, token_types_count: 0 }, // [1431]
    SimpleHintData { raw_values_start: 2228, raw_values_count: 1, token_types_start: 2229, token_types_count: 0 }, // [1432]
    SimpleHintData { raw_values_start: 2229, raw_values_count: 2, token_types_start: 2231, token_types_count: 0 }, // [1433]
    SimpleHintData { raw_values_start: 2231, raw_values_count: 3, token_types_start: 2234, token_types_count: 0 }, // [1434]
    SimpleHintData { raw_values_start: 2234, raw_values_count: 7, token_types_start: 2241, token_types_count: 0 }, // [1435]
    SimpleHintData { raw_values_start: 2241, raw_values_count: 1, token_types_start: 2242, token_types_count: 0 }, // [1436]
    SimpleHintData { raw_values_start: 2242, raw_values_count: 1, token_types_start: 2243, token_types_count: 0 }, // [1437]
    SimpleHintData { raw_values_start: 2243, raw_values_count: 1, token_types_start: 2244, token_types_count: 0 }, // [1438]
    SimpleHintData { raw_values_start: 2244, raw_values_count: 1, token_types_start: 2245, token_types_count: 0 }, // [1439]
    SimpleHintData { raw_values_start: 2245, raw_values_count: 1, token_types_start: 2246, token_types_count: 0 }, // [1440]
    SimpleHintData { raw_values_start: 2246, raw_values_count: 1, token_types_start: 2247, token_types_count: 0 }, // [1441]
    SimpleHintData { raw_values_start: 2247, raw_values_count: 1, token_types_start: 2248, token_types_count: 0 }, // [1442]
    SimpleHintData { raw_values_start: 2248, raw_values_count: 1, token_types_start: 2249, token_types_count: 0 }, // [1443]
    SimpleHintData { raw_values_start: 2249, raw_values_count: 1, token_types_start: 2250, token_types_count: 0 }, // [1444]
    SimpleHintData { raw_values_start: 2250, raw_values_count: 1, token_types_start: 2251, token_types_count: 0 }, // [1445]
    SimpleHintData { raw_values_start: 2251, raw_values_count: 1, token_types_start: 2252, token_types_count: 0 }, // [1446]
    SimpleHintData { raw_values_start: 2252, raw_values_count: 1, token_types_start: 2253, token_types_count: 0 }, // [1447]
    SimpleHintData { raw_values_start: 2253, raw_values_count: 1, token_types_start: 2254, token_types_count: 0 }, // [1448]
    SimpleHintData { raw_values_start: 2254, raw_values_count: 1, token_types_start: 2255, token_types_count: 0 }, // [1449]
    SimpleHintData { raw_values_start: 2255, raw_values_count: 1, token_types_start: 2256, token_types_count: 0 }, // [1450]
    SimpleHintData { raw_values_start: 2256, raw_values_count: 1, token_types_start: 2257, token_types_count: 0 }, // [1451]
    SimpleHintData { raw_values_start: 2257, raw_values_count: 1, token_types_start: 2258, token_types_count: 0 }, // [1452]
    SimpleHintData { raw_values_start: 2258, raw_values_count: 1, token_types_start: 2259, token_types_count: 0 }, // [1453]
    SimpleHintData { raw_values_start: 2259, raw_values_count: 1, token_types_start: 2260, token_types_count: 0 }, // [1454]
    SimpleHintData { raw_values_start: 2260, raw_values_count: 1, token_types_start: 2261, token_types_count: 0 }, // [1455]
    SimpleHintData { raw_values_start: 2261, raw_values_count: 1, token_types_start: 2262, token_types_count: 0 }, // [1456]
    SimpleHintData { raw_values_start: 2262, raw_values_count: 1, token_types_start: 2263, token_types_count: 0 }, // [1457]
    SimpleHintData { raw_values_start: 2263, raw_values_count: 1, token_types_start: 2264, token_types_count: 0 }, // [1458]
    SimpleHintData { raw_values_start: 2264, raw_values_count: 1, token_types_start: 2265, token_types_count: 0 }, // [1459]
    SimpleHintData { raw_values_start: 2265, raw_values_count: 1, token_types_start: 2266, token_types_count: 0 }, // [1460]
    SimpleHintData { raw_values_start: 2266, raw_values_count: 1, token_types_start: 2267, token_types_count: 0 }, // [1461]
    SimpleHintData { raw_values_start: 2267, raw_values_count: 1, token_types_start: 2268, token_types_count: 0 }, // [1462]
    SimpleHintData { raw_values_start: 2268, raw_values_count: 1, token_types_start: 2269, token_types_count: 0 }, // [1463]
    SimpleHintData { raw_values_start: 2269, raw_values_count: 1, token_types_start: 2270, token_types_count: 0 }, // [1464]
    SimpleHintData { raw_values_start: 2270, raw_values_count: 1, token_types_start: 2271, token_types_count: 0 }, // [1465]
    SimpleHintData { raw_values_start: 2271, raw_values_count: 1, token_types_start: 2272, token_types_count: 0 }, // [1466]
    SimpleHintData { raw_values_start: 2272, raw_values_count: 1, token_types_start: 2273, token_types_count: 0 }, // [1467]
    SimpleHintData { raw_values_start: 2273, raw_values_count: 1, token_types_start: 2274, token_types_count: 0 }, // [1468]
    SimpleHintData { raw_values_start: 2274, raw_values_count: 1, token_types_start: 2275, token_types_count: 0 }, // [1469]
    SimpleHintData { raw_values_start: 2275, raw_values_count: 1, token_types_start: 2276, token_types_count: 0 }, // [1470]
    SimpleHintData { raw_values_start: 2276, raw_values_count: 1, token_types_start: 2277, token_types_count: 0 }, // [1471]
    SimpleHintData { raw_values_start: 2277, raw_values_count: 1, token_types_start: 2278, token_types_count: 0 }, // [1472]
    SimpleHintData { raw_values_start: 2278, raw_values_count: 1, token_types_start: 2279, token_types_count: 0 }, // [1473]
    SimpleHintData { raw_values_start: 2279, raw_values_count: 1, token_types_start: 2280, token_types_count: 0 }, // [1474]
    SimpleHintData { raw_values_start: 2280, raw_values_count: 1, token_types_start: 2281, token_types_count: 0 }, // [1475]
    SimpleHintData { raw_values_start: 2281, raw_values_count: 1, token_types_start: 2282, token_types_count: 0 }, // [1476]
    SimpleHintData { raw_values_start: 2282, raw_values_count: 1, token_types_start: 2283, token_types_count: 0 }, // [1477]
    SimpleHintData { raw_values_start: 2283, raw_values_count: 1, token_types_start: 2284, token_types_count: 0 }, // [1478]
    SimpleHintData { raw_values_start: 2284, raw_values_count: 1, token_types_start: 2285, token_types_count: 0 }, // [1479]
    SimpleHintData { raw_values_start: 2285, raw_values_count: 1, token_types_start: 2286, token_types_count: 0 }, // [1480]
    SimpleHintData { raw_values_start: 2286, raw_values_count: 1, token_types_start: 2287, token_types_count: 0 }, // [1481]
    SimpleHintData { raw_values_start: 2287, raw_values_count: 1, token_types_start: 2288, token_types_count: 0 }, // [1482]
    SimpleHintData { raw_values_start: 2288, raw_values_count: 1, token_types_start: 2289, token_types_count: 0 }, // [1483]
    SimpleHintData { raw_values_start: 2289, raw_values_count: 1, token_types_start: 2290, token_types_count: 0 }, // [1484]
    SimpleHintData { raw_values_start: 2290, raw_values_count: 2, token_types_start: 2292, token_types_count: 0 }, // [1485]
    SimpleHintData { raw_values_start: 2292, raw_values_count: 1, token_types_start: 2293, token_types_count: 0 }, // [1486]
    SimpleHintData { raw_values_start: 2293, raw_values_count: 1, token_types_start: 2294, token_types_count: 0 }, // [1487]
    SimpleHintData { raw_values_start: 2294, raw_values_count: 1, token_types_start: 2295, token_types_count: 0 }, // [1488]
    SimpleHintData { raw_values_start: 2295, raw_values_count: 1, token_types_start: 2296, token_types_count: 0 }, // [1489]
    SimpleHintData { raw_values_start: 2296, raw_values_count: 1, token_types_start: 2297, token_types_count: 0 }, // [1490]
    SimpleHintData { raw_values_start: 2297, raw_values_count: 1, token_types_start: 2298, token_types_count: 0 }, // [1491]
    SimpleHintData { raw_values_start: 2298, raw_values_count: 1, token_types_start: 2299, token_types_count: 0 }, // [1492]
    SimpleHintData { raw_values_start: 2299, raw_values_count: 1, token_types_start: 2300, token_types_count: 0 }, // [1493]
    SimpleHintData { raw_values_start: 2300, raw_values_count: 1, token_types_start: 2301, token_types_count: 0 }, // [1494]
    SimpleHintData { raw_values_start: 2301, raw_values_count: 1, token_types_start: 2302, token_types_count: 0 }, // [1495]
    SimpleHintData { raw_values_start: 2302, raw_values_count: 1, token_types_start: 2303, token_types_count: 0 }, // [1496]
    SimpleHintData { raw_values_start: 2303, raw_values_count: 1, token_types_start: 2304, token_types_count: 0 }, // [1497]
    SimpleHintData { raw_values_start: 2304, raw_values_count: 1, token_types_start: 2305, token_types_count: 0 }, // [1498]
    SimpleHintData { raw_values_start: 2305, raw_values_count: 1, token_types_start: 2306, token_types_count: 0 }, // [1499]
    SimpleHintData { raw_values_start: 2306, raw_values_count: 1, token_types_start: 2307, token_types_count: 0 }, // [1500]
    SimpleHintData { raw_values_start: 2307, raw_values_count: 1, token_types_start: 2308, token_types_count: 0 }, // [1501]
    SimpleHintData { raw_values_start: 2308, raw_values_count: 1, token_types_start: 2309, token_types_count: 0 }, // [1502]
    SimpleHintData { raw_values_start: 2309, raw_values_count: 1, token_types_start: 2310, token_types_count: 0 }, // [1503]
    SimpleHintData { raw_values_start: 2310, raw_values_count: 1, token_types_start: 2311, token_types_count: 0 }, // [1504]
    SimpleHintData { raw_values_start: 2311, raw_values_count: 2, token_types_start: 2313, token_types_count: 0 }, // [1505]
    SimpleHintData { raw_values_start: 2313, raw_values_count: 1, token_types_start: 2314, token_types_count: 0 }, // [1506]
    SimpleHintData { raw_values_start: 2314, raw_values_count: 1, token_types_start: 2315, token_types_count: 0 }, // [1507]
    SimpleHintData { raw_values_start: 2315, raw_values_count: 1, token_types_start: 2316, token_types_count: 0 }, // [1508]
    SimpleHintData { raw_values_start: 2316, raw_values_count: 1, token_types_start: 2317, token_types_count: 0 }, // [1509]
    SimpleHintData { raw_values_start: 2317, raw_values_count: 1, token_types_start: 2318, token_types_count: 0 }, // [1510]
    SimpleHintData { raw_values_start: 2318, raw_values_count: 1, token_types_start: 2319, token_types_count: 0 }, // [1511]
    SimpleHintData { raw_values_start: 2319, raw_values_count: 1, token_types_start: 2320, token_types_count: 0 }, // [1512]
    SimpleHintData { raw_values_start: 2320, raw_values_count: 1, token_types_start: 2321, token_types_count: 0 }, // [1513]
    SimpleHintData { raw_values_start: 2321, raw_values_count: 1, token_types_start: 2322, token_types_count: 0 }, // [1514]
    SimpleHintData { raw_values_start: 2322, raw_values_count: 1, token_types_start: 2323, token_types_count: 0 }, // [1515]
    SimpleHintData { raw_values_start: 2323, raw_values_count: 1, token_types_start: 2324, token_types_count: 0 }, // [1516]
    SimpleHintData { raw_values_start: 2324, raw_values_count: 1, token_types_start: 2325, token_types_count: 0 }, // [1517]
    SimpleHintData { raw_values_start: 2325, raw_values_count: 1, token_types_start: 2326, token_types_count: 0 }, // [1518]
    SimpleHintData { raw_values_start: 2326, raw_values_count: 1, token_types_start: 2327, token_types_count: 0 }, // [1519]
    SimpleHintData { raw_values_start: 2327, raw_values_count: 1, token_types_start: 2328, token_types_count: 0 }, // [1520]
    SimpleHintData { raw_values_start: 2328, raw_values_count: 1, token_types_start: 2329, token_types_count: 0 }, // [1521]
    SimpleHintData { raw_values_start: 2329, raw_values_count: 1, token_types_start: 2330, token_types_count: 0 }, // [1522]
    SimpleHintData { raw_values_start: 2330, raw_values_count: 1, token_types_start: 2331, token_types_count: 0 }, // [1523]
    SimpleHintData { raw_values_start: 2331, raw_values_count: 1, token_types_start: 2332, token_types_count: 0 }, // [1524]
    SimpleHintData { raw_values_start: 2332, raw_values_count: 1, token_types_start: 2333, token_types_count: 0 }, // [1525]
    SimpleHintData { raw_values_start: 2333, raw_values_count: 1, token_types_start: 2334, token_types_count: 0 }, // [1526]
    SimpleHintData { raw_values_start: 2334, raw_values_count: 9, token_types_start: 2343, token_types_count: 0 }, // [1527]
    SimpleHintData { raw_values_start: 2343, raw_values_count: 1, token_types_start: 2344, token_types_count: 0 }, // [1528]
    SimpleHintData { raw_values_start: 2344, raw_values_count: 1, token_types_start: 2345, token_types_count: 0 }, // [1529]
    SimpleHintData { raw_values_start: 2345, raw_values_count: 2, token_types_start: 2347, token_types_count: 0 }, // [1530]
    SimpleHintData { raw_values_start: 2347, raw_values_count: 2, token_types_start: 2349, token_types_count: 0 }, // [1531]
    SimpleHintData { raw_values_start: 2349, raw_values_count: 1, token_types_start: 2350, token_types_count: 0 }, // [1532]
    SimpleHintData { raw_values_start: 2350, raw_values_count: 1, token_types_start: 2351, token_types_count: 0 }, // [1533]
    SimpleHintData { raw_values_start: 2351, raw_values_count: 1, token_types_start: 2352, token_types_count: 0 }, // [1534]
    SimpleHintData { raw_values_start: 2352, raw_values_count: 1, token_types_start: 2353, token_types_count: 0 }, // [1535]
    SimpleHintData { raw_values_start: 2353, raw_values_count: 1, token_types_start: 2354, token_types_count: 0 }, // [1536]
    SimpleHintData { raw_values_start: 2354, raw_values_count: 1, token_types_start: 2355, token_types_count: 0 }, // [1537]
    SimpleHintData { raw_values_start: 2355, raw_values_count: 1, token_types_start: 2356, token_types_count: 0 }, // [1538]
    SimpleHintData { raw_values_start: 2356, raw_values_count: 1, token_types_start: 2357, token_types_count: 0 }, // [1539]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 105, 105, 5, 106, 107, 108, 109, 110, 110, 111, 112, 113, 114, 111, 115,
    116, 117, 118, 119, 120, 120, 120, 121, 122, 123, 124, 125, 126, 127, 0, 0,
    128, 0, 0, 0, 31, 0, 31, 129, 130, 131, 0, 132, 0, 0, 0, 24,
    133, 134, 135, 135, 30, 10, 14, 0, 136, 136, 135, 135, 137, 138, 0, 76,
    76, 139, 140, 141, 142, 0, 143, 143, 144, 144, 144, 145, 146, 0, 76, 76,
    147, 148, 149, 150, 150, 33, 0, 151, 152, 153, 153, 153, 76, 154, 155, 156,
    157, 158, 158, 0, 135, 136, 136, 159, 123, 160, 121, 0, 0, 130, 131, 123,
    161, 123, 162, 121, 0, 163, 123, 164, 121, 165, 65, 65, 166, 121, 167, 123,
    37, 168, 169, 169, 170, 37, 171, 172, 173, 174, 174, 37, 175, 176, 177, 178,
    179, 135, 135, 83, 0, 168, 168, 180, 181, 182, 182, 0, 183, 184, 184, 185,
    186, 187, 188, 0, 189, 31, 190, 191, 191, 188, 189, 31, 190, 19, 19, 188,
    0, 31, 67, 0, 31, 76, 76, 76, 192, 60, 73, 193, 194, 195, 196, 196,
    197, 198, 198, 107, 196, 196, 123, 199, 199, 31, 0, 200, 201, 148, 148, 201,
    185, 185, 186, 187, 188, 0, 189, 31, 190, 202, 202, 203, 204, 185, 186, 187,
    188, 0, 31, 190, 112, 112, 205, 188, 0, 206, 120, 120, 121, 207, 123, 124,
    125, 127, 208, 208, 121, 209, 123, 210, 211, 212, 0, 0, 0, 121, 0, 0,
    0, 213, 213, 111, 62, 205, 0, 214, 215, 215, 216, 116, 0, 31, 112, 150,
    150, 217, 128, 33, 0, 135, 135, 77, 0, 218, 218, 219, 206, 110, 111, 111,
    205, 188, 0, 214, 216, 116, 116, 0, 31, 135, 135, 205, 0, 220, 220, 196,
    196, 123, 0, 0, 0, 112, 112, 123, 196, 196, 221, 222, 223, 62, 62, 205,
    188, 0, 214, 216, 116, 116, 0, 31, 111, 111, 180, 224, 224, 205, 188, 0,
    0, 215, 215, 215, 216, 116, 116, 0, 112, 112, 0, 0, 205, 188, 0, 0,
    185, 186, 187, 188, 0, 191, 19, 19, 188, 0, 225, 225, 199, 182, 188, 0,
    135, 135, 225, 199, 182, 0, 200, 201, 148, 148, 201, 135, 135, 185, 186, 187,
    188, 0, 226, 223, 222, 227, 148, 148, 227, 150, 150, 0, 0, 217, 128, 33,
    0, 185, 185, 186, 187, 0, 33, 0, 205, 205, 0, 33, 0, 151, 151, 153,
    152, 228, 229, 229, 33, 136, 136, 230, 230, 111, 112, 231, 232, 72, 233, 234,
    235, 236, 199, 237, 238, 239, 240, 241, 242, 243, 0, 0, 37, 24, 31, 0,
    0, 17, 244, 244, 77, 0, 245, 244, 246, 247, 248, 248, 31, 0, 0, 135,
    135, 120, 120, 121, 249, 250, 251, 252, 137, 253, 253, 254, 255, 137, 256, 78,
    0, 31, 128, 257, 31, 257, 244, 258, 259, 260, 236, 236, 236, 261, 262, 263,
    77, 0, 0, 0, 0, 0, 0, 73, 264, 76, 0, 0, 244, 244, 37, 265,
    0, 0, 112, 264, 76, 0, 0, 266, 267, 266, 268, 269, 42, 270, 271, 272,
    273, 274, 275, 276, 259, 277, 278, 279, 280, 281, 282, 105, 0, 0, 37, 0,
    283, 105, 284, 0, 285, 105, 0, 0, 0, 128, 128, 128, 286, 287, 288, 289,
    290, 291, 141, 292, 293, 294, 295, 296, 297, 298, 299, 300, 300, 301, 300, 302,
    303, 304, 305, 306, 307, 0, 0, 307, 308, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 32, 128, 0, 0, 309, 310, 311, 312, 313, 314, 315, 271, 277, 316,
    317, 318, 319, 0, 320, 321, 322, 323, 324, 259, 279, 279, 279, 277, 280, 280,
    280, 278, 325, 326, 327, 328, 316, 267, 329, 330, 331, 76, 332, 333, 334, 335,
    336, 333, 334, 335, 337, 31, 0, 0, 31, 0, 0, 31, 0, 0, 0, 0,
    0, 0, 31, 31, 0, 338, 286, 339, 0, 31, 340, 341, 342, 343, 344, 265,
    345, 170, 346, 31, 0, 0, 0, 31, 128, 31, 257, 174, 347, 347, 347, 0,
    348, 349, 195, 350, 351, 351, 351, 0, 352, 352, 353, 354, 353, 354, 354, 351,
    351, 0, 352, 352, 353, 354, 353, 354, 354, 317, 32, 315, 351, 355, 356, 15,
    357, 358, 359, 360, 361, 224, 362, 363, 364, 314, 365, 366, 367, 368, 369, 370,
    371, 372, 372, 373, 148, 148, 373, 13, 13, 253, 374, 375, 375, 253, 198, 198,
    253, 376, 376, 253, 22, 377, 377, 253, 22, 253, 253, 254, 255, 137, 256, 378,
    379, 160, 367, 367, 196, 380, 380, 199, 199, 199, 77, 0, 0, 381, 382, 382,
    383, 384, 385, 386, 387, 362, 388, 389, 390, 390, 390, 77, 0, 0, 391, 384,
    387, 392, 393, 394, 395, 396, 397, 398, 399, 400, 401, 402, 233, 403, 404, 0,
    162, 162, 0, 162, 405, 0, 165, 0, 0, 406, 0, 407, 408, 409, 410, 283,
    283, 283, 0, 31, 0, 0, 0, 0, 307, 0, 0, 0, 182, 182, 0, 0,
    411, 148, 197, 199, 199, 31, 0, 123, 123, 0, 191, 191, 0, 184, 300, 67,
    67, 0, 157, 162, 162, 0, 0, 160, 160, 0, 0, 130, 131, 123, 162, 412,
    413, 413, 258, 128, 31, 0, 414, 415, 416, 417, 417, 37, 221, 222, 223, 412,
    413, 413, 258, 128, 31, 0, 418, 419, 415, 416, 0, 0, 0, 0, 420, 421,
    422, 420, 420, 421, 422, 31, 423, 411, 148, 148, 197, 123, 123, 424, 425, 307,
    37, 165, 197, 76, 76, 73, 307, 425, 307, 31, 0, 0, 0, 0, 205, 318,
    0, 0, 406, 0, 426, 427, 428, 429, 430, 431, 432, 0, 0, 0, 0, 433,
    32, 434, 435, 157, 157, 165, 157, 0, 436, 437, 438, 317, 121, 280, 279, 280,
    279, 439, 440, 441, 279, 0, 147, 442, 0, 0, 443, 444, 445, 446, 277, 277,
    277, 447, 448, 449, 450, 0, 451, 450, 0, 59, 452, 453, 454, 182, 455, 456,
    457, 458, 459, 375, 460, 461, 462, 463, 464, 31, 0, 465, 32, 32, 0, 189,
    189, 0, 314, 0, 229, 229, 125, 466, 467, 468, 469, 470, 471, 39, 39, 355,
    31, 0, 128, 0, 244, 472, 473, 82, 80, 81, 474, 475, 476, 477, 459, 478,
    0, 31, 382, 382, 0, 128, 128, 290, 39, 39, 30, 10, 14, 188, 0, 479,
    479, 39, 39, 137, 138, 188, 0, 76, 139, 140, 141, 142, 0, 143, 143, 144,
    144, 144, 145, 146, 0, 76, 76, 147, 148, 149, 151, 152, 153, 153, 153, 76,
    154, 155, 156, 157, 158, 0, 39, 39, 137, 480, 80, 0, 31, 481, 481, 0,
    157, 372, 0, 39, 39, 480, 482, 184, 203, 204, 186, 188, 0, 189, 76, 0,
    31, 483, 484, 206, 120, 208, 39, 39, 39, 480, 485, 188, 0, 486, 486, 31,
    0, 0, 0, 121, 0, 0, 105, 165, 165, 128, 257, 479, 123, 487, 367, 367,
    196, 121, 0, 0, 165, 162, 162, 121, 0, 164, 164, 121, 165, 39, 39, 137,
    480, 81, 188, 0, 31, 157, 372, 0, 39, 39, 16, 188, 0, 39, 39, 14,
    188, 0, 488, 169, 169, 170, 37, 489, 489, 244, 37, 171, 172, 173, 174, 174,
    37, 175, 176, 177, 178, 179, 39, 39, 83, 0, 488, 488, 39, 39, 480, 490,
    77, 188, 0, 491, 31, 31, 0, 0, 180, 0, 157, 492, 128, 257, 31, 257,
    492, 128, 257, 31, 257, 493, 493, 0, 0, 0, 0, 123, 0, 0, 160, 494,
    494, 22, 186, 495, 244, 244, 113, 121, 0, 0, 0, 165, 130, 37, 123, 31,
    0, 0, 242, 242, 170, 496, 497, 498, 499, 500, 500, 0, 187, 187, 120, 120,
    121, 37, 0, 0, 501, 501, 502, 503, 0, 0, 31, 0, 0, 504, 504, 37,
    505, 505, 170, 497, 498, 499, 500, 500, 0, 187, 187, 120, 120, 121, 37, 31,
    0, 506, 506, 37, 31, 31, 242, 242, 242, 0, 0, 507, 507, 508, 509, 509,
    510, 511, 512, 31, 0, 0, 0, 37, 0, 26, 26, 31, 0, 0, 0, 0,
    0, 0, 0, 160, 494, 494, 22, 186, 495, 244, 244, 113, 121, 0, 0, 0,
    165, 130, 37, 123, 31, 0, 0, 0, 505, 0, 0, 507, 507, 508, 509, 509,
    510, 511, 512, 31, 31, 26, 26, 31, 0, 0, 0, 0, 160, 494, 494, 22,
    186, 495, 244, 244, 113, 121, 0, 0, 0, 165, 130, 37, 123, 31, 0, 0,
    39, 39, 480, 137, 71, 188, 0, 513, 309, 116, 514, 61, 73, 60, 76, 0,
    382, 382, 515, 516, 517, 517, 518, 519, 0, 0, 0, 310, 310, 0, 354, 39,
    39, 480, 41, 188, 0, 0, 0, 520, 520, 520, 520, 521, 170, 170, 522, 523,
    523, 524, 165, 244, 244, 0, 525, 525, 170, 170, 522, 523, 523, 524, 165, 128,
    128, 165, 526, 526, 297, 520, 521, 170, 170, 522, 523, 523, 524, 165, 244, 244,
    0, 128, 165, 266, 123, 123, 16, 0, 0, 527, 527, 528, 210, 529, 529, 530,
    531, 392, 392, 165, 532, 532, 165, 533, 533, 165, 266, 244, 244, 534, 534, 534,
    535, 536, 537, 538, 37, 539, 539, 524, 524, 540, 541, 541, 123, 542, 308, 308,
    37, 543, 524, 524, 544, 167, 123, 37, 524, 524, 545, 308, 167, 123, 37, 37,
    543, 524, 524, 527, 546, 547, 547, 123, 548, 549, 549, 37, 550, 550, 551, 37,
    552, 5, 5, 553, 554, 208, 555, 555, 157, 294, 165, 39, 39, 480, 120, 120,
    121, 249, 250, 251, 252, 137, 253, 253, 254, 255, 137, 256, 78, 0, 31, 128,
    257, 31, 257, 244, 556, 557, 558, 450, 559, 560, 560, 561, 561, 560, 562, 31,
    0, 563, 546, 31, 37, 425, 564, 565, 566, 567, 425, 425, 31, 568, 569, 570,
    571, 572, 573, 564, 574, 575, 576, 576, 577, 578, 34, 0, 248, 0, 0, 0,
    0, 579, 580, 580, 399, 581, 0, 0, 582, 583, 177, 22, 10, 584, 585, 0,
    0, 0, 0, 0, 586, 587, 587, 588, 589, 589, 590, 591, 592, 0, 0, 593,
    593, 367, 314, 594, 595, 35, 35, 36, 0, 0, 0, 406, 0, 31, 596, 314,
    590, 591, 597, 597, 597, 591, 591, 591, 597, 0, 598, 599, 599, 31, 31, 0,
    0, 143, 0, 600, 601, 598, 602, 422, 603, 130, 37, 421, 143, 604, 605, 543,
    606, 607, 608, 609, 610, 611, 612, 613, 614, 615, 616, 616, 616, 615, 112, 617,
    0, 618, 619, 620, 620, 620, 620, 0, 579, 382, 0, 620, 620, 621, 462, 622,
    623, 624, 382, 0, 625, 626, 148, 148, 627, 628, 628, 629, 165, 0, 165, 37,
    0, 0, 0, 620, 620, 0, 448, 382, 630, 165, 37, 620, 620, 0, 0, 123,
    123, 0, 165, 37, 0, 123, 123, 123, 507, 631, 632, 633, 634, 635, 137, 137,
    137, 121, 0, 636, 637, 638, 639, 60, 60, 60, 640, 384, 641, 0, 34, 34,
    0, 0, 189, 189, 642, 0, 0, 0, 34, 34, 642, 34, 34, 242, 642, 119,
    643, 644, 0, 0, 406, 406, 275, 189, 189, 0, 0, 645, 646, 647, 648, 649,
    649, 650, 651, 652, 653, 654, 655, 656, 339, 657, 657, 657, 0, 0, 658, 659,
    660, 661, 373, 662, 663, 495, 153, 231, 664, 665, 666, 667, 668, 669, 281, 281,
    274, 158, 670, 11, 406, 587, 131, 131, 131, 671, 194, 195, 112, 112, 355, 31,
    0, 128, 0, 671, 112, 112, 10, 188, 0, 671, 112, 112, 138, 188, 0, 112,
    112, 80, 188, 0, 112, 112, 186, 0, 76, 0, 206, 120, 120, 121, 672, 123,
    124, 125, 208, 208, 121, 209, 123, 210, 211, 212, 112, 112, 112, 485, 188, 0,
    112, 112, 673, 81, 80, 188, 0, 112, 112, 16, 188, 0, 112, 112, 14, 188,
    0, 671, 112, 112, 83, 0, 112, 112, 674, 77, 188, 0, 0, 671, 112, 112,
    71, 188, 0, 112, 112, 18, 188, 0, 671, 112, 112, 41, 188, 0, 112, 112,
    78, 188, 0, 671, 675, 675, 676, 677, 678, 679, 680, 681, 682, 515, 683, 353,
    353, 0, 353, 684, 685, 686, 31, 0, 0, 31, 152, 687, 688, 689, 164, 690,
    691, 692, 354, 146, 201, 693, 694, 695, 696, 697, 121, 698, 699, 700, 701, 702,
    138, 703, 142, 704, 705, 706, 235, 707, 708, 212, 709, 45, 45, 45, 0, 189,
    189, 0, 0, 50, 710, 711, 622, 712, 713, 540, 714, 714, 714, 715, 257, 61,
    73, 60, 716, 0, 0, 0, 0, 0, 717, 315, 315, 0, 718, 317, 441, 441,
    148, 0, 0, 0, 162, 719, 148, 312, 0, 266, 0, 0, 0, 0, 720, 721,
    721, 721, 648, 722, 723, 0, 0, 724, 724, 724, 724, 271, 277, 317, 0, 725,
    721, 721, 721, 648, 0, 0, 0, 726, 0, 711, 711, 31, 257, 0, 0, 0,
    351, 141, 141, 0, 32, 0, 0, 307, 0, 31, 0, 0, 0, 257, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 31, 0, 0, 0, 0,
    727, 165, 37, 333, 197, 600, 0, 0, 32, 105, 0, 0, 728, 68, 0, 308,
    0, 0, 0, 729, 387, 0, 730, 731, 86, 732, 733, 549, 335, 335, 385, 734,
    735, 576, 576, 736, 216, 578, 0, 37, 0, 516, 737, 516, 738, 739, 166, 244,
    244, 740, 576, 741, 87, 0, 646, 646, 0, 0, 646, 646, 646, 646, 742, 743,
    744, 744, 31, 642, 642, 0, 744, 745, 746, 216, 747, 748, 749, 750, 751, 752,
    753, 753, 753, 261, 262, 263, 754, 755, 755, 314, 314, 756, 693, 693, 756, 699,
    699, 756, 757, 757, 756, 758, 758, 174, 759, 759, 756, 760, 761, 762, 763, 764,
    765, 766, 767, 768, 768, 756, 382, 382, 364, 0, 769, 756, 58, 363, 396, 656,
    770, 771, 156, 530, 772, 773, 774, 775, 89, 89, 0, 0, 0, 244, 244, 244,
    65, 208, 266, 266, 153, 389, 89, 89, 0, 0, 0, 382, 382, 382, 716, 776,
    518, 777, 382, 382, 382, 778, 73, 102, 779, 779, 0, 0, 780, 781, 782, 782,
    783, 208, 208, 26, 102, 784, 382, 785, 19, 19, 187, 19, 88, 786, 787, 627,
    788, 788, 0, 0, 546, 546, 516, 0, 0, 37, 308, 552, 0, 789, 790, 776,
    312, 312, 266, 788, 738, 502, 791, 792, 793, 794, 34, 34, 0, 0, 795, 642,
    643, 101, 101, 170, 119, 119, 170, 796, 797, 798, 799, 244, 244, 576, 800, 777,
    799, 244, 244, 248, 0, 0, 0, 0, 31, 0, 0, 0, 0, 795, 801, 0,
    0, 244, 244, 800, 0, 801, 802, 31, 0, 0, 0, 0, 31, 0, 0, 803,
    0, 31, 0, 119, 119, 170, 101, 101, 170, 0, 0, 0, 0, 0, 0, 119,
    119, 170, 101, 101, 170, 0, 34, 804, 203, 0, 0, 0, 0, 0, 128, 0,
    805, 805, 0, 34, 34, 0, 0, 0, 143, 0, 34, 0, 0, 667, 0, 275,
    0, 0, 119, 806, 806, 0, 0, 0, 165, 0, 0, 26, 0, 165, 0, 0,
    807, 0, 0, 31, 31, 0, 0, 80, 808, 0, 0, 0, 0, 406, 31, 809,
    808, 582, 31, 0, 0, 0, 0, 269, 269, 18, 0, 0, 269, 269, 18, 0,
    31, 0, 0, 0, 810, 810, 31, 811, 811, 31, 229, 229, 31, 599, 599, 31,
    0, 0, 0, 0, 0, 599, 0, 728, 31, 0, 92, 812, 27, 813, 757, 413,
    814, 815, 816, 817, 817, 546, 546, 818, 662, 0, 0, 0, 0, 0, 121, 819,
    820, 821, 448, 0, 0, 0, 37, 0, 0, 0, 0, 0, 121, 822, 394, 823,
    824, 825, 826, 455, 457, 456, 357, 827, 828, 432, 583, 817, 829, 830, 831, 832,
    765, 833, 834, 99, 99, 99, 0, 38, 38, 76, 0, 0, 0, 33, 0, 0,
    244, 244, 244, 99, 835, 244, 244, 115, 835, 836, 48, 280, 280, 121, 280, 837,
    101, 101, 170, 0, 0, 0, 0, 0, 37, 0, 338, 286, 339, 838, 244, 838,
    119, 119, 170, 643, 796, 797, 798, 576, 101, 839, 0, 0, 0, 0, 37, 0,
    31, 838, 840, 840, 840, 841, 31, 0, 0, 561, 0, 842, 843, 624, 844, 500,
    796, 796, 0, 31, 0, 845, 119, 119, 170, 643, 797, 798, 576, 796, 846, 847,
    848, 848, 848, 165, 849, 850, 264, 544, 851, 852, 0, 853, 769, 854, 855, 856,
    857, 858, 520, 0, 859, 860, 861, 862, 188, 188, 711, 863, 188, 188, 0, 864,
    0, 684, 684, 0, 864, 0, 353, 353, 0, 354, 354, 188, 188, 188, 188, 148,
    711, 641, 807, 807, 641, 865, 866, 867, 868, 227, 869, 870, 871, 872, 873, 232,
    57, 26, 718, 148, 26, 0, 31, 0, 0, 0, 257, 0, 874, 875, 876, 169,
    0, 0, 338, 286, 339, 31, 37, 877, 877, 55, 641, 785, 185, 186, 187, 382,
    382, 878, 879, 119, 119, 170, 101, 101, 170, 31, 0, 76, 186, 484, 880, 880,
    121, 37, 189, 244, 244, 881, 0, 157, 157, 165, 402, 402, 121, 882, 883, 198,
    884, 227, 148, 148, 227, 190, 880, 880, 121, 37, 189, 244, 244, 881, 0, 157,
    221, 223, 222, 694, 694, 121, 165, 885, 885, 121, 165, 189, 189, 886, 344, 500,
    189, 189, 887, 344, 500, 888, 770, 889, 890, 891, 892, 893, 894, 526, 895, 896,
    897, 898, 319, 899, 900, 124, 901, 900, 26, 902, 61, 128, 128, 0, 31, 0,
    61, 61, 903, 640, 639, 850, 641, 248, 0, 242, 242, 31, 0, 31, 904, 728,
    196, 257, 644, 128, 76, 644, 905, 906, 907, 476, 127, 908, 909, 910, 911, 912,
    913, 914, 915, 592, 12, 916, 917, 918, 308, 308, 0, 143, 0, 0, 143, 308,
    248, 248, 0, 0, 0, 0, 0, 0, 678, 678, 165, 919, 919, 165, 367, 367,
    196, 0, 920, 920, 741, 433, 921, 921, 170, 165, 922, 923, 687, 170, 165, 702,
    702, 170, 165, 924, 924, 925, 925, 170, 165, 921, 921, 170, 165, 926, 248, 222,
    256, 927, 928, 929, 0, 197, 930, 0, 333, 0, 441, 441, 148, 667, 34, 441,
    0, 931, 932, 933, 934, 532, 935, 935, 935, 0, 0, 0, 0, 879, 0, 0,
    0, 0, 0, 0, 936, 76, 189, 0, 0, 0, 879, 0, 0, 0, 0, 0,
    879, 879, 879, 0, 76, 76, 0, 0, 31, 0, 0, 451, 899, 937, 937, 804,
    938, 939, 940, 189, 189, 31, 0, 0, 941, 433, 433, 31, 0, 0, 0, 0,
    382, 942, 943, 943, 944, 165, 433, 0, 0, 0, 945, 711, 711, 944, 165, 944,
    944, 165, 946, 946, 946, 197, 699, 123, 123, 165, 76, 947, 686, 699, 31, 0,
    32, 165, 433, 810, 31, 0, 32, 0, 644, 644, 0, 367, 367, 196, 0, 946,
    946, 946, 197, 699, 123, 123, 0, 76, 947, 686, 699, 811, 810, 811, 948, 187,
    880, 949, 950, 951, 228, 0, 952, 953, 954, 13, 35, 955, 956, 957, 958, 0,
    959, 960, 961, 962, 963, 964, 938, 965, 509, 279, 279, 121, 279, 966, 717, 717,
    148, 967, 0, 701, 701, 0, 967, 493, 968, 969, 493, 440, 643, 643, 0, 31,
    0, 37, 0, 24, 970, 800, 800, 0, 37, 0, 32, 32, 37, 643, 498, 971,
    924, 972, 0, 503, 973, 0, 165, 37, 333, 721, 197, 600, 105, 0, 974, 131,
    0, 0, 0, 975, 976, 976, 22, 977, 640, 447, 263, 263, 891, 165, 978, 979,
    641, 248, 77, 0, 242, 367, 367, 196, 0, 920, 920, 741, 433, 921, 921, 170,
    165, 922, 923, 923, 687, 170, 165, 702, 702, 170, 165, 924, 924, 925, 925, 170,
    165, 921, 921, 170, 165, 641, 641, 37, 980, 924, 738, 31, 31, 0, 0, 196,
    196, 0, 976, 0, 263, 0, 566, 567, 981, 982, 208, 208, 208, 121, 209, 123,
    210, 211, 212, 783, 983, 984, 985, 756, 986, 987, 988, 989, 0, 0, 0, 0,
    283, 989, 0, 0, 354, 354, 989, 0, 640, 990, 991, 803, 803, 406, 0, 31,
    485, 485, 0, 32, 32, 31, 257, 47, 992, 0, 43, 762, 993, 994, 995, 996,
    997, 998, 999, 1000, 1001, 1002, 1003, 1004, 1005, 1006, 1007, 1008, 1009, 1010, 1011, 1012,
    1013, 1014, 1015, 1016, 1017, 1018, 1019, 1020, 1021, 0, 1022, 1023, 85, 1024, 537, 535,
    1025, 1026, 1027, 536, 538, 1028, 512, 386, 1029, 1030, 1031, 1032, 1033, 60, 61, 61,
    31, 728, 251, 251, 248, 251, 352, 352, 352, 352, 352, 1023, 266, 266, 0, 864,
    192, 73, 60, 352, 352, 148, 1023, 266, 266, 0, 864, 61, 251, 251, 0, 0,
    0, 189, 0, 0, 0, 31, 31, 257, 0, 0, 0, 0, 0, 76, 0, 352,
    73, 73, 196, 1034, 1035, 825, 478, 1036, 1037, 1038, 1039, 1040, 1041, 1042, 273, 1043,
    1044, 1045, 1046, 803, 1047, 282, 282, 784, 485, 377, 62, 1048, 276, 8, 1049, 1050,
    1051, 1052, 1053, 275, 1054, 1055, 1056, 1057, 1058, 1059, 1060, 826, 0, 1061, 0, 0,
    128, 0, 0, 31, 0, 798, 798, 0, 0, 1062, 930, 930, 1063, 1064, 1064, 451,
    1064, 1065, 1066, 1067, 1068, 273, 0, 943, 1069, 1070, 542, 1071, 0, 578, 1072, 198,
    1073, 262, 1074, 175, 1075, 1076, 126, 1077, 1078, 1079, 178, 1080, 1081, 1082, 1083, 1084,
    340, 728, 68, 31, 68, 31, 244, 31, 340, 31, 0, 1085, 73, 61, 60, 251,
    340, 340, 1086, 31, 68, 340, 1087, 210, 1088, 1089, 0, 1090, 1091, 0, 439, 1092,
    1092, 121, 279, 279, 280, 148, 0, 722, 148, 1092, 1093, 1094, 1095, 1096, 565, 781,
    1097, 1098, 197, 197, 279, 279, 121, 280, 1099, 1100, 866, 820, 1101, 37, 1102, 36,
    165, 165, 283, 0, 974, 165, 165, 974, 406, 406, 406, 406, 406, 0, 76, 128,
    189, 32, 356, 105, 31, 315, 283, 646, 0, 1103, 1104, 1105, 779, 332, 1106, 800,
    800, 0, 37, 0, 516, 737, 516, 738, 800, 1107, 1108, 1109, 1110, 1111, 76, 1112,
    1113, 166, 581, 1114, 1115, 1116, 1117, 63, 1118, 1119, 237, 237, 237, 261, 262, 263,
    77, 0, 0, 835, 548, 923, 486, 480, 329, 480, 1120, 1121, 480, 480, 979, 119,
    119, 170, 0, 0, 0, 0, 37, 0, 338, 286, 339, 866, 866, 1122, 216, 955,
    0, 643, 788, 1123, 643, 796, 797, 798, 788, 806, 576, 119, 179, 119, 1088, 1124,
    942, 1125, 1126, 1127, 940, 919, 1128, 1127, 0, 807, 1129, 0, 0, 31, 0, 1129,
    729, 729, 0, 31, 31, 600, 32, 600, 0, 729, 1130, 1131, 1132, 1133, 64, 1134,
    1135, 1136, 1137, 1138, 1139, 1140, 1141, 0, 1142, 1143, 1144, 1145, 1146, 1147, 1148, 1149,
    1150, 1151, 881, 1152, 242, 242, 170, 0, 31, 0, 0, 242, 243, 504, 1153, 524,
    550, 944, 1154, 274, 274, 808, 808, 274, 165, 0, 1155, 1156, 1157, 1158, 1159, 1160,
    1161, 1162, 419, 1163, 54, 277, 1164, 1165, 1166, 1167, 1168, 1169, 272, 1170, 44, 1171,
    1172, 1173, 1174, 272, 0, 0, 744, 802, 877, 242, 1175, 1176, 0, 519, 790, 588,
    1177, 1178, 617, 617, 617, 0, 34, 0, 165, 0, 0, 1179, 149, 1180, 1181, 191,
    191, 187, 191, 1182, 723, 1183, 58, 1184, 1185, 81, 0, 0, 1186, 1127, 26, 900,
    0, 0, 0, 31, 0, 0, 93, 1187, 1188, 1189, 1190, 1191, 103, 1192, 1192, 1193,
    314, 762, 756, 1194, 33, 33, 165, 309, 309, 0, 0, 1192, 721, 721, 37, 797,
    317, 317, 1195, 24, 269, 1196, 31, 257, 1197, 758, 771, 384, 1198, 582, 165, 130,
    131, 1199, 1200, 1201, 523, 502, 1202, 121, 280, 1203, 279, 1092, 65, 1204, 1205, 376,
    1206, 1207, 238, 1208, 1209, 1210, 1211, 1212, 1213, 1214, 1215, 1216, 67, 67, 0, 31,
    1022, 76, 76, 76, 60, 193, 76, 76, 73, 193, 1217, 1022, 1022, 1218, 804, 1152,
    1219, 66, 67, 1220, 193, 194, 195, 196, 196, 197, 198, 198, 107, 196, 196, 123,
    969, 1221, 1222, 1223, 1224, 1225, 1226, 1227, 1228, 1229, 1230, 1231, 1232, 768, 1233, 1234,
    1235, 1236, 1237, 1238, 1239, 240, 150, 150, 150, 77, 0, 0, 0, 33, 0, 234,
    239, 239, 239, 261, 262, 263, 77, 0, 0, 1240, 384, 387, 1241, 1242, 0, 0,
    0, 0, 283, 1242, 0, 0, 1243, 1243, 0, 354, 354, 1242, 0, 1244, 979, 979,
    979, 1245, 640, 639, 248, 248, 0, 242, 1246, 1247, 31, 728, 196, 1086, 31, 68,
    644, 1248, 155, 1249, 1250, 1251, 1252, 1253, 1254, 1255, 1256, 1257, 1258, 527, 1259, 761,
    761, 761, 762, 33, 33, 37, 1260, 1261, 1261, 1262, 1260, 0, 628, 628, 629, 165,
    0, 196, 196, 822, 822, 822, 394, 823, 824, 825, 826, 455, 457, 456, 357, 827,
    828, 432, 583, 121, 0, 0, 0, 165, 7, 865, 1263, 1264, 194, 1265, 1266, 1267,
    1268, 1268, 1268, 0, 1269, 1270, 1271, 824, 644, 644, 0, 0, 926, 644, 481, 545,
    1272, 100, 100, 100, 99, 99, 835, 382, 0, 38, 38, 76, 0, 16, 16, 0,
    64, 64, 76, 41, 0, 0, 34, 0, 671, 939, 968, 16, 0, 0, 0, 0,
    582, 130, 131, 292, 292, 0, 0, 582, 130, 131, 575, 1273, 1274, 562, 562, 1275,
    82, 1276, 1277, 1278, 94, 516, 821, 1279, 1280, 1281, 1282, 1283, 1284, 738, 1285, 888,
    1286, 801, 801, 1287, 311, 113, 31, 37, 1244, 1244, 1244, 31, 37, 1288, 1289, 1290,
    139, 14, 827, 25, 1291, 1292, 1293, 1294, 1295, 1296, 1297, 1298, 1299, 1300, 885, 1301,
    1302, 1303, 254, 0, 0, 0, 0, 0, 1304, 1305, 667, 24, 668, 850, 1306, 1307,
    1308, 1309, 1310, 1311, 1312, 68, 68, 1304, 0, 0, 926, 248, 926, 34, 0, 642,
    119, 119, 170, 643, 729, 799, 576, 68, 242, 242, 31, 0, 0, 68, 68, 248,
    34, 877, 242, 642, 101, 796, 729, 798, 777, 119, 643, 798, 248, 799, 76, 244,
    644, 257, 1313, 31, 244, 1313, 31, 244, 340, 31, 257, 1314, 647, 1315, 806, 83,
    172, 512, 512, 37, 198, 198, 512, 171, 1046, 1046, 37, 198, 198, 1046, 95, 1316,
    1317, 21, 1318, 766, 0, 1319, 196, 196, 0, 0, 1320, 1320, 1071, 1110, 406, 1321,
    765, 1161, 1162, 766, 263, 0, 0, 0, 0, 1322, 121, 726, 0, 37, 165, 131,
    0, 0, 330, 0, 0, 271, 0, 196, 196, 0, 0, 0, 0, 121, 0, 0,
    307, 0, 0, 0, 728, 123, 196, 196, 123, 16, 0, 24, 210, 0, 0, 33,
    0, 0, 340, 340, 799, 799, 799, 340, 119, 643, 798, 196, 196, 196, 1062, 0,
    123, 165, 0, 162, 162, 0, 799, 1323, 1324, 1324, 917, 705, 24, 1042, 196, 196,
    14, 188, 0, 196, 196, 764, 765, 766, 1325, 1326, 1326, 933, 933, 966, 1327, 65,
    65, 1328, 437, 1329, 1244, 1244, 65, 1317, 65, 65, 1330, 75, 166, 1331, 841, 1332,
    1333, 102, 211, 1334, 0, 0, 0, 351, 356, 356, 356, 0, 141, 1335, 1336, 1262,
    590, 721, 272, 273, 1337, 1219, 0, 0, 582, 0, 406, 0, 0, 0, 0, 406,
    275, 130, 130, 1338, 0, 0, 105, 782, 274, 156, 1339, 1340, 283, 759, 1341, 1342,
    1343, 1344, 1196, 1345, 1346, 1347, 1348, 1349, 1350, 1351, 1352, 1353, 1354, 1355, 1356, 1357,
    1358, 1359, 1360, 1361, 1362, 1363, 1364, 1365, 1366, 1367, 1368, 1369, 1370, 1371, 1372, 1373,
    1374, 204, 472, 1375, 1376, 253, 1377, 1378, 1379, 1308, 1380, 1381, 1309, 1310, 1312, 1382,
    1383, 1384, 1311, 1385, 1307, 1386, 1387, 1388, 1389, 1390, 1391, 1392, 1393, 1394, 1395, 1396,
    1397, 1398, 625, 628, 626, 1399, 417, 530, 1400, 818, 79, 91, 275, 489, 925, 145,
    1401, 1402, 0, 257, 251, 61, 0, 112, 112, 39, 112, 72, 98, 39, 39, 112,
    135, 196, 112, 112, 39, 112, 39, 112, 39, 60, 73, 39, 112, 39, 112, 39,
    112, 657, 55, 714, 39, 135, 112, 39, 112, 649, 39, 620, 196, 196, 863, 0,
    935, 0, 0, 347, 617, 45, 616, 817, 1261, 580, 576, 112, 135, 135, 39, 150,
    761, 1192, 848, 199, 390, 236, 239, 237, 76, 128, 753, 976, 979, 135, 1268, 196,
    39, 135, 112, 196, 646, 477, 1403, 1404, 1405, 1406, 767, 1407, 1408, 1409, 1410, 1411,
    1412, 415, 1306, 84, 96, 1413, 277, 277, 1414, 31, 0, 0, 0, 0, 0, 1415,
    1416, 823, 533, 1417, 1418, 505, 506, 1419, 1420, 1421, 1422, 1423, 1424, 1425, 1426, 0,
    1427, 1428, 1429, 1430, 113, 0, 1431, 1432, 180, 181, 182, 182, 0, 183, 184, 184,
    185, 186, 187, 0, 189, 31, 190, 191, 191, 189, 31, 190, 19, 19, 0, 31,
    67, 0, 31, 76, 76, 76, 192, 60, 73, 193, 194, 195, 196, 196, 197, 198,
    198, 107, 196, 196, 123, 199, 199, 31, 0, 200, 201, 148, 148, 201, 185, 185,
    186, 187, 0, 189, 31, 190, 202, 202, 203, 204, 185, 186, 187, 0, 31, 190,
    0, 0, 728, 307, 0, 0, 31, 257, 31, 251, 77, 219, 219, 1433, 1412, 693,
    121, 0, 165, 0, 300, 300, 121, 37, 304, 304, 121, 37, 159, 123, 160, 121,
    0, 165, 0, 123, 390, 390, 121, 37, 161, 123, 162, 121, 0, 165, 0, 123,
    157, 157, 121, 165, 452, 452, 121, 165, 22, 22, 495, 121, 165, 638, 638, 121,
    37, 689, 689, 121, 882, 883, 198, 690, 690, 121, 37, 862, 862, 121, 882, 883,
    198, 186, 186, 495, 121, 165, 906, 906, 121, 1434, 198, 216, 955, 880, 880, 121,
    37, 1025, 1025, 121, 37, 1041, 1041, 121, 37, 1134, 1134, 121, 167, 37, 123, 1138,
    1138, 121, 37, 1139, 1139, 121, 37, 1140, 1140, 121, 37, 524, 524, 121, 165, 1279,
    1279, 121, 1435, 123, 680, 748, 444, 1214, 438, 1137, 83, 83, 121, 37, 1404, 1404,
    121, 167, 123, 37, 1405, 1405, 121, 167, 123, 37, 1406, 1406, 121, 167, 123, 37,
    17, 17, 0, 1436, 1436, 121, 37, 1323, 1323, 121, 31, 0, 0, 244, 244, 113,
    114, 0, 675, 0, 0, 406, 0, 1437, 828, 772, 89, 801, 17, 0, 720, 720,
    315, 0, 0, 725, 725, 0, 46, 97, 69, 1438, 0, 674, 69, 70, 70, 490,
    1439, 674, 252, 1440, 921, 1441, 1442, 510, 864, 1443, 1444, 740, 648, 602, 586, 586,
    602, 422, 31, 245, 245, 244, 246, 602, 1445, 141, 141, 141, 602, 1445, 0, 422,
    1446, 1447, 1448, 1449, 1450, 1451, 1452, 1453, 33, 1454, 1455, 1456, 1457, 1458, 1325, 0,
    489, 489, 1325, 0, 0, 0, 283, 310, 310, 1459, 0, 963, 963, 0, 436, 436,
    1459, 266, 266, 198, 360, 1274, 1274, 1459, 354, 354, 0, 1460, 1436, 1461, 1462, 1463,
    1464, 1465, 1439, 1466, 1467, 1468, 71, 480, 480, 979, 1469, 1470, 1471, 1472, 1473, 805,
    337, 961, 1457, 334, 334, 72, 72, 72, 77, 0, 1474, 1475, 31, 0, 0, 18,
    0, 0, 105, 0, 0, 31, 1476, 1477, 1478, 552, 1329, 0, 0, 1064, 0, 0,
    250, 1479, 623, 1480, 1481, 1482, 1483, 1484, 1323, 1323, 1485, 667, 24, 1323, 184, 184,
    187, 184, 1486, 0, 1487, 554, 1488, 1489, 68, 68, 248, 34, 877, 242, 642, 101,
    796, 729, 798, 777, 799, 244, 244, 119, 643, 248, 777, 877, 244, 242, 76, 340,
    340, 799, 799, 799, 340, 1490, 591, 1243, 73, 73, 73, 640, 641, 0, 0, 0,
    0, 196, 642, 119, 643, 1491, 383, 1492, 76, 76, 679, 187, 73, 0, 0, 51,
    55, 55, 55, 0, 74, 1241, 41, 0, 1493, 1494, 1495, 1496, 775, 773, 774, 1497,
    189, 1498, 1499, 1500, 1501, 1502, 1503, 247, 1504, 629, 728, 728, 629, 507, 1505, 1505,
    516, 31, 0, 0, 123, 0, 0, 507, 1506, 1507, 1508, 1509, 1510, 1511, 0, 0,
    726, 0, 1512, 1513, 595, 1514, 1515, 1516, 114, 1517, 78, 90, 416, 223, 1518, 1519,
    483, 1519, 1519, 37, 781, 1520, 726, 9, 1521, 1522, 1523, 1524, 1525, 352, 352, 0,
    0, 0, 864, 0, 0, 0, 0, 352, 1526, 642, 642, 0, 31, 0, 1527, 643,
    101, 101, 170, 119, 119, 170, 796, 797, 798, 729, 576, 248, 642, 1528, 0, 0,
    0, 0, 283, 1528, 1528, 0, 158, 0, 0, 354, 354, 1528, 0, 0, 1529, 0,
    0, 0, 0, 0, 0, 0, 406, 275, 275, 406, 275, 798, 0, 0, 1530, 242,
    119, 242, 119, 788, 244, 244, 1531, 350, 263, 199, 835, 244, 244, 1211, 0, 0,
    0, 68, 0, 1085, 244, 244, 1211, 0, 0, 0, 68, 0, 340, 244, 244, 198,
    198, 22, 0, 244, 244, 244, 198, 14, 320, 244, 244, 562, 1532, 246, 0, 1459,
    23, 75, 1533, 531, 1534, 1535, 1536, 1121, 1537, 1538, 1539, 883, 597, 1445,
];

pub fn get_mariadb_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &MARIADB_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &MARIADB_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &MARIADB_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &MARIADB_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &MARIADB_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &MARIADB_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &MARIADB_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &MARIADB_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &MARIADB_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &MARIADB_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &MARIADB_TABLES }),
        "AccessibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &MARIADB_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(209), tables: &MARIADB_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &MARIADB_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &MARIADB_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &MARIADB_TABLES }),
        "ActiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &MARIADB_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &MARIADB_TABLES }),
        "AddDropSystemVersioningGrammar" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &MARIADB_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &MARIADB_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &MARIADB_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(224), tables: &MARIADB_TABLES }),
        "AgainstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(225), tables: &MARIADB_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(226), tables: &MARIADB_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &MARIADB_TABLES }),
        "AlgorithmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &MARIADB_TABLES }),
        "AlgorithmOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(229), tables: &MARIADB_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(238), tables: &MARIADB_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(251), tables: &MARIADB_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(252), tables: &MARIADB_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(255), tables: &MARIADB_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(256), tables: &MARIADB_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(257), tables: &MARIADB_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(258), tables: &MARIADB_TABLES }),
        "AlterEventStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(266), tables: &MARIADB_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(308), tables: &MARIADB_TABLES }),
        "AlterOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(309), tables: &MARIADB_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(337), tables: &MARIADB_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(353), tables: &MARIADB_TABLES }),
        "AlterTableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(359), tables: &MARIADB_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(436), tables: &MARIADB_TABLES }),
        "AlterTableOnlineDDLOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(441), tables: &MARIADB_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(458), tables: &MARIADB_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(485), tables: &MARIADB_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(671), tables: &MARIADB_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(695), tables: &MARIADB_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(696), tables: &MARIADB_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(697), tables: &MARIADB_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(698), tables: &MARIADB_TABLES }),
        "AnalyzeTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(699), tables: &MARIADB_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(727), tables: &MARIADB_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(728), tables: &MARIADB_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(731), tables: &MARIADB_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(732), tables: &MARIADB_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(733), tables: &MARIADB_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(734), tables: &MARIADB_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(747), tables: &MARIADB_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(755), tables: &MARIADB_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(756), tables: &MARIADB_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(757), tables: &MARIADB_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(760), tables: &MARIADB_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(761), tables: &MARIADB_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(763), tables: &MARIADB_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(764), tables: &MARIADB_TABLES }),
        "AsciiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(765), tables: &MARIADB_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(766), tables: &MARIADB_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(767), tables: &MARIADB_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(768), tables: &MARIADB_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(769), tables: &MARIADB_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(770), tables: &MARIADB_TABLES }),
        "AtSignLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(771), tables: &MARIADB_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(772), tables: &MARIADB_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(773), tables: &MARIADB_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(774), tables: &MARIADB_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(775), tables: &MARIADB_TABLES }),
        "AuthenticationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(776), tables: &MARIADB_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(777), tables: &MARIADB_TABLES }),
        "AuthorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(778), tables: &MARIADB_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(779), tables: &MARIADB_TABLES }),
        "AutoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(781), tables: &MARIADB_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(782), tables: &MARIADB_TABLES }),
        "Autoextend_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(783), tables: &MARIADB_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(784), tables: &MARIADB_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(785), tables: &MARIADB_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(786), tables: &MARIADB_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(787), tables: &MARIADB_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(788), tables: &MARIADB_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(789), tables: &MARIADB_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(804), tables: &MARIADB_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(805), tables: &MARIADB_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(806), tables: &MARIADB_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(807), tables: &MARIADB_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(808), tables: &MARIADB_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(809), tables: &MARIADB_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(810), tables: &MARIADB_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(811), tables: &MARIADB_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(812), tables: &MARIADB_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(819), tables: &MARIADB_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(820), tables: &MARIADB_TABLES }),
        "BinlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(821), tables: &MARIADB_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(822), tables: &MARIADB_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(823), tables: &MARIADB_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(824), tables: &MARIADB_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(825), tables: &MARIADB_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(826), tables: &MARIADB_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(829), tables: &MARIADB_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(830), tables: &MARIADB_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(833), tables: &MARIADB_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(834), tables: &MARIADB_TABLES }),
        "BlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(835), tables: &MARIADB_TABLES }),
        "BodyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(836), tables: &MARIADB_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(837), tables: &MARIADB_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(838), tables: &MARIADB_TABLES }),
        "BooleanDynamicSystemVariablesGrammar" => Some(RootGrammar { grammar_id: GrammarId(841), tables: &MARIADB_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(848), tables: &MARIADB_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(849), tables: &MARIADB_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(852), tables: &MARIADB_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(853), tables: &MARIADB_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(856), tables: &MARIADB_TABLES }),
        "BracketedKeyPartListGrammar" => Some(RootGrammar { grammar_id: GrammarId(859), tables: &MARIADB_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(872), tables: &MARIADB_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(873), tables: &MARIADB_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(875), tables: &MARIADB_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(876), tables: &MARIADB_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(877), tables: &MARIADB_TABLES }),
        "BtreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(878), tables: &MARIADB_TABLES }),
        "BucketsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(879), tables: &MARIADB_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(880), tables: &MARIADB_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(881), tables: &MARIADB_TABLES }),
        "ByteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(882), tables: &MARIADB_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(883), tables: &MARIADB_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(885), tables: &MARIADB_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(891), tables: &MARIADB_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(892), tables: &MARIADB_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(893), tables: &MARIADB_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(896), tables: &MARIADB_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(897), tables: &MARIADB_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(898), tables: &MARIADB_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(899), tables: &MARIADB_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(900), tables: &MARIADB_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(924), tables: &MARIADB_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(925), tables: &MARIADB_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(926), tables: &MARIADB_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(927), tables: &MARIADB_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(928), tables: &MARIADB_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(929), tables: &MARIADB_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(930), tables: &MARIADB_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(931), tables: &MARIADB_TABLES }),
        "Challenge_responseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(932), tables: &MARIADB_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(933), tables: &MARIADB_TABLES }),
        "ChangedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(934), tables: &MARIADB_TABLES }),
        "Changed_page_bitmapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(935), tables: &MARIADB_TABLES }),
        "ChannelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(936), tables: &MARIADB_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(937), tables: &MARIADB_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &MARIADB_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(939), tables: &MARIADB_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(940), tables: &MARIADB_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(941), tables: &MARIADB_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(942), tables: &MARIADB_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(943), tables: &MARIADB_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(944), tables: &MARIADB_TABLES }),
        "CharacteristicStatement" => Some(RootGrammar { grammar_id: GrammarId(945), tables: &MARIADB_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(975), tables: &MARIADB_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(976), tables: &MARIADB_TABLES }),
        "CharsetGrammar" => Some(RootGrammar { grammar_id: GrammarId(977), tables: &MARIADB_TABLES }),
        "CharsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(982), tables: &MARIADB_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(983), tables: &MARIADB_TABLES }),
        "CheckTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(984), tables: &MARIADB_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(998), tables: &MARIADB_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(999), tables: &MARIADB_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1000), tables: &MARIADB_TABLES }),
        "ChecksumTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1001), tables: &MARIADB_TABLES }),
        "CipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1009), tables: &MARIADB_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1010), tables: &MARIADB_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1011), tables: &MARIADB_TABLES }),
        "ClientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1012), tables: &MARIADB_TABLES }),
        "Client_statisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1013), tables: &MARIADB_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1014), tables: &MARIADB_TABLES }),
        "CloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1015), tables: &MARIADB_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1016), tables: &MARIADB_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1017), tables: &MARIADB_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1018), tables: &MARIADB_TABLES }),
        "ClusteringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1019), tables: &MARIADB_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1020), tables: &MARIADB_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1021), tables: &MARIADB_TABLES }),
        "CodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1022), tables: &MARIADB_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(1023), tables: &MARIADB_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(1024), tables: &MARIADB_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1027), tables: &MARIADB_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1028), tables: &MARIADB_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1029), tables: &MARIADB_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1035), tables: &MARIADB_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1036), tables: &MARIADB_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1037), tables: &MARIADB_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1038), tables: &MARIADB_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(1039), tables: &MARIADB_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(1040), tables: &MARIADB_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(1041), tables: &MARIADB_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(1042), tables: &MARIADB_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(1049), tables: &MARIADB_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(1114), tables: &MARIADB_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(1149), tables: &MARIADB_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1150), tables: &MARIADB_TABLES }),
        "ColumnPathOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1151), tables: &MARIADB_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "Column_addKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1156), tables: &MARIADB_TABLES }),
        "Column_checkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1157), tables: &MARIADB_TABLES }),
        "Column_createKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1158), tables: &MARIADB_TABLES }),
        "Column_deleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1159), tables: &MARIADB_TABLES }),
        "Column_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1160), tables: &MARIADB_TABLES }),
        "Column_getKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1161), tables: &MARIADB_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1162), tables: &MARIADB_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &MARIADB_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1164), tables: &MARIADB_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1165), tables: &MARIADB_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(1166), tables: &MARIADB_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1167), tables: &MARIADB_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(1168), tables: &MARIADB_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1169), tables: &MARIADB_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1170), tables: &MARIADB_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1171), tables: &MARIADB_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1174), tables: &MARIADB_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(1175), tables: &MARIADB_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1176), tables: &MARIADB_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1177), tables: &MARIADB_TABLES }),
        "CompactKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1178), tables: &MARIADB_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1179), tables: &MARIADB_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1189), tables: &MARIADB_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1190), tables: &MARIADB_TABLES }),
        "ComponentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1191), tables: &MARIADB_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1192), tables: &MARIADB_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1193), tables: &MARIADB_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1194), tables: &MARIADB_TABLES }),
        "CompressedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1195), tables: &MARIADB_TABLES }),
        "CompressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1196), tables: &MARIADB_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1197), tables: &MARIADB_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(1198), tables: &MARIADB_TABLES }),
        "ConcurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1201), tables: &MARIADB_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1202), tables: &MARIADB_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1203), tables: &MARIADB_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1204), tables: &MARIADB_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1205), tables: &MARIADB_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1209), tables: &MARIADB_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1210), tables: &MARIADB_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1211), tables: &MARIADB_TABLES }),
        "ConsistentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1212), tables: &MARIADB_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1213), tables: &MARIADB_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1214), tables: &MARIADB_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1215), tables: &MARIADB_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1216), tables: &MARIADB_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1217), tables: &MARIADB_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1218), tables: &MARIADB_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1219), tables: &MARIADB_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1220), tables: &MARIADB_TABLES }),
        "ContextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1221), tables: &MARIADB_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1222), tables: &MARIADB_TABLES }),
        "ContributorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1223), tables: &MARIADB_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1224), tables: &MARIADB_TABLES }),
        "ConvertFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1225), tables: &MARIADB_TABLES }),
        "ConvertFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1236), tables: &MARIADB_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1237), tables: &MARIADB_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1238), tables: &MARIADB_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1239), tables: &MARIADB_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1240), tables: &MARIADB_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1241), tables: &MARIADB_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1242), tables: &MARIADB_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1243), tables: &MARIADB_TABLES }),
        "CpuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1244), tables: &MARIADB_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1245), tables: &MARIADB_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1272), tables: &MARIADB_TABLES }),
        "CreateEventStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1281), tables: &MARIADB_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1318), tables: &MARIADB_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1331), tables: &MARIADB_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1350), tables: &MARIADB_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1351), tables: &MARIADB_TABLES }),
        "CreateOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(1371), tables: &MARIADB_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1389), tables: &MARIADB_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1400), tables: &MARIADB_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1405), tables: &MARIADB_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1410), tables: &MARIADB_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1430), tables: &MARIADB_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1436), tables: &MARIADB_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1632), tables: &MARIADB_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1663), tables: &MARIADB_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1803), tables: &MARIADB_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1828), tables: &MARIADB_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1829), tables: &MARIADB_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &MARIADB_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &MARIADB_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1832), tables: &MARIADB_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1833), tables: &MARIADB_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1834), tables: &MARIADB_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1835), tables: &MARIADB_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1841), tables: &MARIADB_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1842), tables: &MARIADB_TABLES }),
        "CurrentTimestampLikeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1843), tables: &MARIADB_TABLES }),
        "CurrentTimestampLikeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1845), tables: &MARIADB_TABLES }),
        "CurrentTimestampLikeFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(1850), tables: &MARIADB_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1853), tables: &MARIADB_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1854), tables: &MARIADB_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1855), tables: &MARIADB_TABLES }),
        "Current_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1856), tables: &MARIADB_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1857), tables: &MARIADB_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1858), tables: &MARIADB_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &MARIADB_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1860), tables: &MARIADB_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1861), tables: &MARIADB_TABLES }),
        "CursorFetchSegment" => Some(RootGrammar { grammar_id: GrammarId(1862), tables: &MARIADB_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1873), tables: &MARIADB_TABLES }),
        "CursorOpenCloseSegment" => Some(RootGrammar { grammar_id: GrammarId(1874), tables: &MARIADB_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1881), tables: &MARIADB_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1882), tables: &MARIADB_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1883), tables: &MARIADB_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1884), tables: &MARIADB_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1885), tables: &MARIADB_TABLES }),
        "DatafileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1886), tables: &MARIADB_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1887), tables: &MARIADB_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1891), tables: &MARIADB_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1930), tables: &MARIADB_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1931), tables: &MARIADB_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1932), tables: &MARIADB_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1933), tables: &MARIADB_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1939), tables: &MARIADB_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1947), tables: &MARIADB_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1948), tables: &MARIADB_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1949), tables: &MARIADB_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1950), tables: &MARIADB_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1951), tables: &MARIADB_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1952), tables: &MARIADB_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1953), tables: &MARIADB_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1954), tables: &MARIADB_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1955), tables: &MARIADB_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1956), tables: &MARIADB_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1957), tables: &MARIADB_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1958), tables: &MARIADB_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1959), tables: &MARIADB_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1960), tables: &MARIADB_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1961), tables: &MARIADB_TABLES }),
        "DeallocateSegment" => Some(RootGrammar { grammar_id: GrammarId(1962), tables: &MARIADB_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1969), tables: &MARIADB_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1970), tables: &MARIADB_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1971), tables: &MARIADB_TABLES }),
        "DeclareStatement" => Some(RootGrammar { grammar_id: GrammarId(1972), tables: &MARIADB_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(250), tables: &MARIADB_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &MARIADB_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &MARIADB_TABLES }),
        "Default_authKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2025), tables: &MARIADB_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2026), tables: &MARIADB_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2027), tables: &MARIADB_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2028), tables: &MARIADB_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2029), tables: &MARIADB_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2030), tables: &MARIADB_TABLES }),
        "DefinerSegment" => Some(RootGrammar { grammar_id: GrammarId(2031), tables: &MARIADB_TABLES }),
        "DefinitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2035), tables: &MARIADB_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2036), tables: &MARIADB_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2037), tables: &MARIADB_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2038), tables: &MARIADB_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2039), tables: &MARIADB_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2040), tables: &MARIADB_TABLES }),
        "DeleteTargetTableSegment" => Some(RootGrammar { grammar_id: GrammarId(2066), tables: &MARIADB_TABLES }),
        "DeleteUsingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2071), tables: &MARIADB_TABLES }),
        "Delete_domain_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2075), tables: &MARIADB_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2076), tables: &MARIADB_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2079), tables: &MARIADB_TABLES }),
        "DelimiterStatement" => Some(RootGrammar { grammar_id: GrammarId(2080), tables: &MARIADB_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2081), tables: &MARIADB_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2082), tables: &MARIADB_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2083), tables: &MARIADB_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2084), tables: &MARIADB_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2085), tables: &MARIADB_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2086), tables: &MARIADB_TABLES }),
        "Des_key_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2087), tables: &MARIADB_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &MARIADB_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &MARIADB_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &MARIADB_TABLES }),
        "DescriptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &MARIADB_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &MARIADB_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2096), tables: &MARIADB_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2097), tables: &MARIADB_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2098), tables: &MARIADB_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2099), tables: &MARIADB_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2100), tables: &MARIADB_TABLES }),
        "DirectoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2101), tables: &MARIADB_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2102), tables: &MARIADB_TABLES }),
        "DiscardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2103), tables: &MARIADB_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2104), tables: &MARIADB_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2105), tables: &MARIADB_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2106), tables: &MARIADB_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2107), tables: &MARIADB_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2108), tables: &MARIADB_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2109), tables: &MARIADB_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &MARIADB_TABLES }),
        "DivOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2111), tables: &MARIADB_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &MARIADB_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &MARIADB_TABLES }),
        "Do_domain_idsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &MARIADB_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &MARIADB_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(2116), tables: &MARIADB_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2117), tables: &MARIADB_TABLES }),
        "DoubleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2118), tables: &MARIADB_TABLES }),
        "DoubleQuotedJSONPath" => Some(RootGrammar { grammar_id: GrammarId(2119), tables: &MARIADB_TABLES }),
        "DoubleQuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &MARIADB_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2121), tables: &MARIADB_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2124), tables: &MARIADB_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2132), tables: &MARIADB_TABLES }),
        "DropEventStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2138), tables: &MARIADB_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2143), tables: &MARIADB_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &MARIADB_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2170), tables: &MARIADB_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &MARIADB_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &MARIADB_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &MARIADB_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2188), tables: &MARIADB_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2194), tables: &MARIADB_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2198), tables: &MARIADB_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &MARIADB_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2211), tables: &MARIADB_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2217), tables: &MARIADB_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &MARIADB_TABLES }),
        "DualIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &MARIADB_TABLES }),
        "DualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &MARIADB_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &MARIADB_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2231), tables: &MARIADB_TABLES }),
        "DumpfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2232), tables: &MARIADB_TABLES }),
        "DuplicateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2233), tables: &MARIADB_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2234), tables: &MARIADB_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2235), tables: &MARIADB_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2236), tables: &MARIADB_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2237), tables: &MARIADB_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2238), tables: &MARIADB_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2239), tables: &MARIADB_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2242), tables: &MARIADB_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2243), tables: &MARIADB_TABLES }),
        "ElsifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2244), tables: &MARIADB_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &MARIADB_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &MARIADB_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2247), tables: &MARIADB_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2250), tables: &MARIADB_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2251), tables: &MARIADB_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &MARIADB_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &MARIADB_TABLES }),
        "EncryptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &MARIADB_TABLES }),
        "Encryption_key_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &MARIADB_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &MARIADB_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(247), tables: &MARIADB_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2257), tables: &MARIADB_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2258), tables: &MARIADB_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(754), tables: &MARIADB_TABLES }),
        "EndsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2259), tables: &MARIADB_TABLES }),
        "EnforcedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2260), tables: &MARIADB_TABLES }),
        "EngineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2261), tables: &MARIADB_TABLES }),
        "Engine_attributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2262), tables: &MARIADB_TABLES }),
        "EnginesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2263), tables: &MARIADB_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2264), tables: &MARIADB_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2265), tables: &MARIADB_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(2266), tables: &MARIADB_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &MARIADB_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2268), tables: &MARIADB_TABLES }),
        "ErrorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2269), tables: &MARIADB_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &MARIADB_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2271), tables: &MARIADB_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &MARIADB_TABLES }),
        "EventsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &MARIADB_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &MARIADB_TABLES }),
        "ExaminedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &MARIADB_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2276), tables: &MARIADB_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2277), tables: &MARIADB_TABLES }),
        "ExchangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &MARIADB_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2279), tables: &MARIADB_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2280), tables: &MARIADB_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2281), tables: &MARIADB_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2282), tables: &MARIADB_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2283), tables: &MARIADB_TABLES }),
        "ExecuteSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &MARIADB_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2291), tables: &MARIADB_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &MARIADB_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &MARIADB_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &MARIADB_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &MARIADB_TABLES }),
        "ExpansionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &MARIADB_TABLES }),
        "ExpireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &MARIADB_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &MARIADB_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &MARIADB_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &MARIADB_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &MARIADB_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2308), tables: &MARIADB_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &MARIADB_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2341), tables: &MARIADB_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2351), tables: &MARIADB_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2356), tables: &MARIADB_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(2373), tables: &MARIADB_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &MARIADB_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &MARIADB_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &MARIADB_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &MARIADB_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "Extent_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &MARIADB_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &MARIADB_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &MARIADB_TABLES }),
        "FactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2426), tables: &MARIADB_TABLES }),
        "Failed_login_attemptsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &MARIADB_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &MARIADB_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(2429), tables: &MARIADB_TABLES }),
        "FastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2430), tables: &MARIADB_TABLES }),
        "FaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2431), tables: &MARIADB_TABLES }),
        "FederatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2432), tables: &MARIADB_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2433), tables: &MARIADB_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &MARIADB_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &MARIADB_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2452), tables: &MARIADB_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(2453), tables: &MARIADB_TABLES }),
        "File_block_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2462), tables: &MARIADB_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2463), tables: &MARIADB_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2464), tables: &MARIADB_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2470), tables: &MARIADB_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2471), tables: &MARIADB_TABLES }),
        "FinishKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2472), tables: &MARIADB_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2473), tables: &MARIADB_TABLES }),
        "First_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2474), tables: &MARIADB_TABLES }),
        "FixedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2475), tables: &MARIADB_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2476), tables: &MARIADB_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2477), tables: &MARIADB_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2478), tables: &MARIADB_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2479), tables: &MARIADB_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2480), tables: &MARIADB_TABLES }),
        "FlushStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2481), tables: &MARIADB_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2559), tables: &MARIADB_TABLES }),
        "FollowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2560), tables: &MARIADB_TABLES }),
        "ForClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2561), tables: &MARIADB_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2582), tables: &MARIADB_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2583), tables: &MARIADB_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(2584), tables: &MARIADB_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2587), tables: &MARIADB_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2588), tables: &MARIADB_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2589), tables: &MARIADB_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2590), tables: &MARIADB_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2591), tables: &MARIADB_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2592), tables: &MARIADB_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(2611), tables: &MARIADB_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &MARIADB_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2615), tables: &MARIADB_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2616), tables: &MARIADB_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2617), tables: &MARIADB_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2618), tables: &MARIADB_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2622), tables: &MARIADB_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2644), tables: &MARIADB_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &MARIADB_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2694), tables: &MARIADB_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2695), tables: &MARIADB_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2696), tables: &MARIADB_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2697), tables: &MARIADB_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2698), tables: &MARIADB_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2739), tables: &MARIADB_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2742), tables: &MARIADB_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2743), tables: &MARIADB_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2744), tables: &MARIADB_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2745), tables: &MARIADB_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2755), tables: &MARIADB_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2768), tables: &MARIADB_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2771), tables: &MARIADB_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &MARIADB_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2795), tables: &MARIADB_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2796), tables: &MARIADB_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2797), tables: &MARIADB_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2798), tables: &MARIADB_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2799), tables: &MARIADB_TABLES }),
        "GeomcollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &MARIADB_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &MARIADB_TABLES }),
        "GeometrycollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &MARIADB_TABLES }),
        "GetDiagnosticsSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &MARIADB_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2843), tables: &MARIADB_TABLES }),
        "Get_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2844), tables: &MARIADB_TABLES }),
        "Get_master_public_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &MARIADB_TABLES }),
        "Get_source_public_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &MARIADB_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &MARIADB_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &MARIADB_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2849), tables: &MARIADB_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2850), tables: &MARIADB_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2851), tables: &MARIADB_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2852), tables: &MARIADB_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2873), tables: &MARIADB_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2874), tables: &MARIADB_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2875), tables: &MARIADB_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2878), tables: &MARIADB_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2879), tables: &MARIADB_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2880), tables: &MARIADB_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2895), tables: &MARIADB_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &MARIADB_TABLES }),
        "Group_replicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2905), tables: &MARIADB_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(2906), tables: &MARIADB_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2914), tables: &MARIADB_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2915), tables: &MARIADB_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2923), tables: &MARIADB_TABLES }),
        "Gtid_onlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2924), tables: &MARIADB_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2925), tables: &MARIADB_TABLES }),
        "HardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2926), tables: &MARIADB_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2927), tables: &MARIADB_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &MARIADB_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2933), tables: &MARIADB_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2941), tables: &MARIADB_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2942), tables: &MARIADB_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2943), tables: &MARIADB_TABLES }),
        "HelpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2944), tables: &MARIADB_TABLES }),
        "HelpStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2945), tables: &MARIADB_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2948), tables: &MARIADB_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2949), tables: &MARIADB_TABLES }),
        "HistogramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2950), tables: &MARIADB_TABLES }),
        "HistoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2951), tables: &MARIADB_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &MARIADB_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2953), tables: &MARIADB_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2954), tables: &MARIADB_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2955), tables: &MARIADB_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2956), tables: &MARIADB_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &MARIADB_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &MARIADB_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2959), tables: &MARIADB_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2960), tables: &MARIADB_TABLES }),
        "IdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2961), tables: &MARIADB_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2962), tables: &MARIADB_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &MARIADB_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2964), tables: &MARIADB_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &MARIADB_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2966), tables: &MARIADB_TABLES }),
        "Ietf_quotesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &MARIADB_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &MARIADB_TABLES }),
        "IfExpressionStatement" => Some(RootGrammar { grammar_id: GrammarId(2971), tables: &MARIADB_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2988), tables: &MARIADB_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2989), tables: &MARIADB_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2993), tables: &MARIADB_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2994), tables: &MARIADB_TABLES }),
        "Ignore_domain_idsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2999), tables: &MARIADB_TABLES }),
        "Ignore_server_idsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3000), tables: &MARIADB_TABLES }),
        "IgnoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3001), tables: &MARIADB_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3002), tables: &MARIADB_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3003), tables: &MARIADB_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3004), tables: &MARIADB_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3005), tables: &MARIADB_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(903), tables: &MARIADB_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3006), tables: &MARIADB_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3007), tables: &MARIADB_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &MARIADB_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &MARIADB_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3010), tables: &MARIADB_TABLES }),
        "InactiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3020), tables: &MARIADB_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3021), tables: &MARIADB_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3022), tables: &MARIADB_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3023), tables: &MARIADB_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(239), tables: &MARIADB_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(3024), tables: &MARIADB_TABLES }),
        "IndexColumnPrefixLengthSegment" => Some(RootGrammar { grammar_id: GrammarId(3029), tables: &MARIADB_TABLES }),
        "IndexHintClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3031), tables: &MARIADB_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3052), tables: &MARIADB_TABLES }),
        "IndexOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(3053), tables: &MARIADB_TABLES }),
        "IndexOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(3077), tables: &MARIADB_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "IndexTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(3099), tables: &MARIADB_TABLES }),
        "IndexTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3104), tables: &MARIADB_TABLES }),
        "Index_statisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3110), tables: &MARIADB_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3111), tables: &MARIADB_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3112), tables: &MARIADB_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3113), tables: &MARIADB_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3114), tables: &MARIADB_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3115), tables: &MARIADB_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3116), tables: &MARIADB_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3117), tables: &MARIADB_TABLES }),
        "Initial_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3118), tables: &MARIADB_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3119), tables: &MARIADB_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3120), tables: &MARIADB_TABLES }),
        "InitiateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3121), tables: &MARIADB_TABLES }),
        "InlinePathOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3122), tables: &MARIADB_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3123), tables: &MARIADB_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3124), tables: &MARIADB_TABLES }),
        "InplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3125), tables: &MARIADB_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3126), tables: &MARIADB_TABLES }),
        "InputOutputParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3127), tables: &MARIADB_TABLES }),
        "InputParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3128), tables: &MARIADB_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3129), tables: &MARIADB_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3130), tables: &MARIADB_TABLES }),
        "InsertRowAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(3131), tables: &MARIADB_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3136), tables: &MARIADB_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3158), tables: &MARIADB_TABLES }),
        "Insert_methodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3159), tables: &MARIADB_TABLES }),
        "InstallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3160), tables: &MARIADB_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3161), tables: &MARIADB_TABLES }),
        "InstantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3162), tables: &MARIADB_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3163), tables: &MARIADB_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3164), tables: &MARIADB_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3165), tables: &MARIADB_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3166), tables: &MARIADB_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3167), tables: &MARIADB_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3168), tables: &MARIADB_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3169), tables: &MARIADB_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3170), tables: &MARIADB_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3171), tables: &MARIADB_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3172), tables: &MARIADB_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3173), tables: &MARIADB_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3174), tables: &MARIADB_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3175), tables: &MARIADB_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3176), tables: &MARIADB_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3183), tables: &MARIADB_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3184), tables: &MARIADB_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3230), tables: &MARIADB_TABLES }),
        "InvisibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3231), tables: &MARIADB_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3232), tables: &MARIADB_TABLES }),
        "IoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3233), tables: &MARIADB_TABLES }),
        "Io_threadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3234), tables: &MARIADB_TABLES }),
        "IpcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3235), tables: &MARIADB_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3236), tables: &MARIADB_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(3242), tables: &MARIADB_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3247), tables: &MARIADB_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(3248), tables: &MARIADB_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &MARIADB_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3250), tables: &MARIADB_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3251), tables: &MARIADB_TABLES }),
        "IsopenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3252), tables: &MARIADB_TABLES }),
        "IssuerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3253), tables: &MARIADB_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3254), tables: &MARIADB_TABLES }),
        "IterateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3255), tables: &MARIADB_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3258), tables: &MARIADB_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3280), tables: &MARIADB_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3281), tables: &MARIADB_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(3283), tables: &MARIADB_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &MARIADB_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3291), tables: &MARIADB_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3299), tables: &MARIADB_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3304), tables: &MARIADB_TABLES }),
        "JsonTableColumnsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3305), tables: &MARIADB_TABLES }),
        "JsonTableFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &MARIADB_TABLES }),
        "JsonTableFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3347), tables: &MARIADB_TABLES }),
        "JsonValueFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(3348), tables: &MARIADB_TABLES }),
        "JsonValueFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3371), tables: &MARIADB_TABLES }),
        "Json_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &MARIADB_TABLES }),
        "Json_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3373), tables: &MARIADB_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3374), tables: &MARIADB_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3375), tables: &MARIADB_TABLES }),
        "Key_block_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3376), tables: &MARIADB_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &MARIADB_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3378), tables: &MARIADB_TABLES }),
        "KeyringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3379), tables: &MARIADB_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3380), tables: &MARIADB_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3381), tables: &MARIADB_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &MARIADB_TABLES }),
        "LagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &MARIADB_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &MARIADB_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &MARIADB_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3386), tables: &MARIADB_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3387), tables: &MARIADB_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3388), tables: &MARIADB_TABLES }),
        "Last_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &MARIADB_TABLES }),
        "LastvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3390), tables: &MARIADB_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3391), tables: &MARIADB_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3392), tables: &MARIADB_TABLES }),
        "LeadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3393), tables: &MARIADB_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3394), tables: &MARIADB_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3395), tables: &MARIADB_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3396), tables: &MARIADB_TABLES }),
        "LeavesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3397), tables: &MARIADB_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3398), tables: &MARIADB_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3399), tables: &MARIADB_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3400), tables: &MARIADB_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(3401), tables: &MARIADB_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3404), tables: &MARIADB_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3405), tables: &MARIADB_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3406), tables: &MARIADB_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(3414), tables: &MARIADB_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3418), tables: &MARIADB_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3419), tables: &MARIADB_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3420), tables: &MARIADB_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3437), tables: &MARIADB_TABLES }),
        "LinearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3438), tables: &MARIADB_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3439), tables: &MARIADB_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3440), tables: &MARIADB_TABLES }),
        "LinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3441), tables: &MARIADB_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3442), tables: &MARIADB_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3443), tables: &MARIADB_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3444), tables: &MARIADB_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3445), tables: &MARIADB_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3457), tables: &MARIADB_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3458), tables: &MARIADB_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3459), tables: &MARIADB_TABLES }),
        "LoadDataSegment" => Some(RootGrammar { grammar_id: GrammarId(3460), tables: &MARIADB_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3522), tables: &MARIADB_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(3523), tables: &MARIADB_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3524), tables: &MARIADB_TABLES }),
        "LocalVariableNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3525), tables: &MARIADB_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3526), tables: &MARIADB_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3527), tables: &MARIADB_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3528), tables: &MARIADB_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3529), tables: &MARIADB_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3530), tables: &MARIADB_TABLES }),
        "LockOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(3531), tables: &MARIADB_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3539), tables: &MARIADB_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3540), tables: &MARIADB_TABLES }),
        "LogfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3541), tables: &MARIADB_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3542), tables: &MARIADB_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3543), tables: &MARIADB_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3544), tables: &MARIADB_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3545), tables: &MARIADB_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3546), tables: &MARIADB_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3547), tables: &MARIADB_TABLES }),
        "LoopStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3548), tables: &MARIADB_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3560), tables: &MARIADB_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3561), tables: &MARIADB_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3562), tables: &MARIADB_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3563), tables: &MARIADB_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3575), tables: &MARIADB_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3576), tables: &MARIADB_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3577), tables: &MARIADB_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3578), tables: &MARIADB_TABLES }),
        "MasterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3579), tables: &MARIADB_TABLES }),
        "Master_auto_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3580), tables: &MARIADB_TABLES }),
        "Master_compression_algorithmsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3581), tables: &MARIADB_TABLES }),
        "Master_connect_retryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3582), tables: &MARIADB_TABLES }),
        "Master_delayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3583), tables: &MARIADB_TABLES }),
        "Master_demote_to_slaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3584), tables: &MARIADB_TABLES }),
        "Master_gtid_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3585), tables: &MARIADB_TABLES }),
        "Master_heartbeat_periodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3586), tables: &MARIADB_TABLES }),
        "Master_hostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3587), tables: &MARIADB_TABLES }),
        "Master_log_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3588), tables: &MARIADB_TABLES }),
        "Master_log_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3589), tables: &MARIADB_TABLES }),
        "Master_passwordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3590), tables: &MARIADB_TABLES }),
        "Master_portKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3591), tables: &MARIADB_TABLES }),
        "Master_public_key_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3592), tables: &MARIADB_TABLES }),
        "Master_retry_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3593), tables: &MARIADB_TABLES }),
        "Master_server_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3594), tables: &MARIADB_TABLES }),
        "Master_sslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3595), tables: &MARIADB_TABLES }),
        "Master_ssl_caKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3596), tables: &MARIADB_TABLES }),
        "Master_ssl_capathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3597), tables: &MARIADB_TABLES }),
        "Master_ssl_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3598), tables: &MARIADB_TABLES }),
        "Master_ssl_cipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3599), tables: &MARIADB_TABLES }),
        "Master_ssl_crlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3600), tables: &MARIADB_TABLES }),
        "Master_ssl_crlpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3601), tables: &MARIADB_TABLES }),
        "Master_ssl_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3602), tables: &MARIADB_TABLES }),
        "Master_ssl_verify_server_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3603), tables: &MARIADB_TABLES }),
        "Master_tls_ciphersuitesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3604), tables: &MARIADB_TABLES }),
        "Master_tls_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3605), tables: &MARIADB_TABLES }),
        "Master_use_gtidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3606), tables: &MARIADB_TABLES }),
        "Master_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3607), tables: &MARIADB_TABLES }),
        "Master_zstd_compression_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3608), tables: &MARIADB_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(3609), tables: &MARIADB_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3610), tables: &MARIADB_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3611), tables: &MARIADB_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3612), tables: &MARIADB_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3613), tables: &MARIADB_TABLES }),
        "Max_connections_per_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3614), tables: &MARIADB_TABLES }),
        "Max_queries_per_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3615), tables: &MARIADB_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3616), tables: &MARIADB_TABLES }),
        "Max_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3617), tables: &MARIADB_TABLES }),
        "Max_statement_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3618), tables: &MARIADB_TABLES }),
        "Max_updates_per_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3619), tables: &MARIADB_TABLES }),
        "Max_user_connectionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3620), tables: &MARIADB_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3621), tables: &MARIADB_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3622), tables: &MARIADB_TABLES }),
        "MediumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3623), tables: &MARIADB_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &MARIADB_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3625), tables: &MARIADB_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3626), tables: &MARIADB_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3627), tables: &MARIADB_TABLES }),
        "MemoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3628), tables: &MARIADB_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3629), tables: &MARIADB_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3630), tables: &MARIADB_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(3634), tables: &MARIADB_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3637), tables: &MARIADB_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(3638), tables: &MARIADB_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3641), tables: &MARIADB_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3651), tables: &MARIADB_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3660), tables: &MARIADB_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3680), tables: &MARIADB_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3683), tables: &MARIADB_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3684), tables: &MARIADB_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3685), tables: &MARIADB_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3686), tables: &MARIADB_TABLES }),
        "MicrosecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3687), tables: &MARIADB_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3688), tables: &MARIADB_TABLES }),
        "MigrateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3689), tables: &MARIADB_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3690), tables: &MARIADB_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3691), tables: &MARIADB_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3692), tables: &MARIADB_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3693), tables: &MARIADB_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(3694), tables: &MARIADB_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3695), tables: &MARIADB_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3696), tables: &MARIADB_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3697), tables: &MARIADB_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3698), tables: &MARIADB_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3699), tables: &MARIADB_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3700), tables: &MARIADB_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3701), tables: &MARIADB_TABLES }),
        "ModOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3702), tables: &MARIADB_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3703), tables: &MARIADB_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3704), tables: &MARIADB_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3705), tables: &MARIADB_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3706), tables: &MARIADB_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3707), tables: &MARIADB_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(3708), tables: &MARIADB_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3709), tables: &MARIADB_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3710), tables: &MARIADB_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3711), tables: &MARIADB_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3712), tables: &MARIADB_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3713), tables: &MARIADB_TABLES }),
        "MultilinestringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3714), tables: &MARIADB_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(3715), tables: &MARIADB_TABLES }),
        "MultipointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3716), tables: &MARIADB_TABLES }),
        "MultipolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3717), tables: &MARIADB_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3718), tables: &MARIADB_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3719), tables: &MARIADB_TABLES }),
        "MutexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3720), tables: &MARIADB_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3721), tables: &MARIADB_TABLES }),
        "MysqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3722), tables: &MARIADB_TABLES }),
        "Mysql_errnoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3723), tables: &MARIADB_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3724), tables: &MARIADB_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3725), tables: &MARIADB_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3726), tables: &MARIADB_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(3733), tables: &MARIADB_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3737), tables: &MARIADB_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3738), tables: &MARIADB_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3739), tables: &MARIADB_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3740), tables: &MARIADB_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3741), tables: &MARIADB_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3744), tables: &MARIADB_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3745), tables: &MARIADB_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3746), tables: &MARIADB_TABLES }),
        "NdbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3747), tables: &MARIADB_TABLES }),
        "NdbclusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3748), tables: &MARIADB_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(3749), tables: &MARIADB_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(3750), tables: &MARIADB_TABLES }),
        "NestedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3751), tables: &MARIADB_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3752), tables: &MARIADB_TABLES }),
        "Network_namespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3753), tables: &MARIADB_TABLES }),
        "NeverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3754), tables: &MARIADB_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3755), tables: &MARIADB_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(3756), tables: &MARIADB_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3757), tables: &MARIADB_TABLES }),
        "NextvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3758), tables: &MARIADB_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3759), tables: &MARIADB_TABLES }),
        "No_waitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3760), tables: &MARIADB_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3761), tables: &MARIADB_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3762), tables: &MARIADB_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3763), tables: &MARIADB_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3764), tables: &MARIADB_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3765), tables: &MARIADB_TABLES }),
        "NocopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3766), tables: &MARIADB_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3767), tables: &MARIADB_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3768), tables: &MARIADB_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3769), tables: &MARIADB_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3770), tables: &MARIADB_TABLES }),
        "NodegroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3771), tables: &MARIADB_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3772), tables: &MARIADB_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3773), tables: &MARIADB_TABLES }),
        "NomaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3774), tables: &MARIADB_TABLES }),
        "NominvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3775), tables: &MARIADB_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3776), tables: &MARIADB_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3786), tables: &MARIADB_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3787), tables: &MARIADB_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3792), tables: &MARIADB_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3798), tables: &MARIADB_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3799), tables: &MARIADB_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3800), tables: &MARIADB_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3801), tables: &MARIADB_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(3802), tables: &MARIADB_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3803), tables: &MARIADB_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3804), tables: &MARIADB_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(3805), tables: &MARIADB_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(3806), tables: &MARIADB_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3813), tables: &MARIADB_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(3814), tables: &MARIADB_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3815), tables: &MARIADB_TABLES }),
        "NotfoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3818), tables: &MARIADB_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3819), tables: &MARIADB_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3820), tables: &MARIADB_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3821), tables: &MARIADB_TABLES }),
        "NowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3822), tables: &MARIADB_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3823), tables: &MARIADB_TABLES }),
        "Nth_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3824), tables: &MARIADB_TABLES }),
        "NtileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3825), tables: &MARIADB_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3826), tables: &MARIADB_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3827), tables: &MARIADB_TABLES }),
        "NullSafeEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(3828), tables: &MARIADB_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3832), tables: &MARIADB_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3833), tables: &MARIADB_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3834), tables: &MARIADB_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3835), tables: &MARIADB_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3836), tables: &MARIADB_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3837), tables: &MARIADB_TABLES }),
        "NvarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3838), tables: &MARIADB_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3839), tables: &MARIADB_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3840), tables: &MARIADB_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3844), tables: &MARIADB_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(3848), tables: &MARIADB_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3853), tables: &MARIADB_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3865), tables: &MARIADB_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3866), tables: &MARIADB_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3867), tables: &MARIADB_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3868), tables: &MARIADB_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3869), tables: &MARIADB_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3870), tables: &MARIADB_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3871), tables: &MARIADB_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3880), tables: &MARIADB_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3881), tables: &MARIADB_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3882), tables: &MARIADB_TABLES }),
        "OjKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3883), tables: &MARIADB_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3884), tables: &MARIADB_TABLES }),
        "Old_passwordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3885), tables: &MARIADB_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3886), tables: &MARIADB_TABLES }),
        "OneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3887), tables: &MARIADB_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3888), tables: &MARIADB_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3889), tables: &MARIADB_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3890), tables: &MARIADB_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3891), tables: &MARIADB_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3892), tables: &MARIADB_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3893), tables: &MARIADB_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3894), tables: &MARIADB_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3895), tables: &MARIADB_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3896), tables: &MARIADB_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3897), tables: &MARIADB_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3898), tables: &MARIADB_TABLES }),
        "OptimizeTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3899), tables: &MARIADB_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3907), tables: &MARIADB_TABLES }),
        "OptionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3908), tables: &MARIADB_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3909), tables: &MARIADB_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3910), tables: &MARIADB_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3911), tables: &MARIADB_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3912), tables: &MARIADB_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(3916), tables: &MARIADB_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3919), tables: &MARIADB_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(3939), tables: &MARIADB_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3947), tables: &MARIADB_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(3948), tables: &MARIADB_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3951), tables: &MARIADB_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3952), tables: &MARIADB_TABLES }),
        "OrganizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3953), tables: &MARIADB_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3954), tables: &MARIADB_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3955), tables: &MARIADB_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3956), tables: &MARIADB_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3957), tables: &MARIADB_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3958), tables: &MARIADB_TABLES }),
        "OutputParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3959), tables: &MARIADB_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3960), tables: &MARIADB_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3967), tables: &MARIADB_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3968), tables: &MARIADB_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3977), tables: &MARIADB_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3978), tables: &MARIADB_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3979), tables: &MARIADB_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3980), tables: &MARIADB_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3981), tables: &MARIADB_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3982), tables: &MARIADB_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3983), tables: &MARIADB_TABLES }),
        "PackageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3984), tables: &MARIADB_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3985), tables: &MARIADB_TABLES }),
        "PageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3986), tables: &MARIADB_TABLES }),
        "Page_checksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3987), tables: &MARIADB_TABLES }),
        "Page_compressedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3988), tables: &MARIADB_TABLES }),
        "Page_compression_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3989), tables: &MARIADB_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3990), tables: &MARIADB_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3991), tables: &MARIADB_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3992), tables: &MARIADB_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3993), tables: &MARIADB_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3994), tables: &MARIADB_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3995), tables: &MARIADB_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3996), tables: &MARIADB_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3997), tables: &MARIADB_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3998), tables: &MARIADB_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3999), tables: &MARIADB_TABLES }),
        "Parse_gcol_exprKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4000), tables: &MARIADB_TABLES }),
        "Parse_vcol_exprKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4001), tables: &MARIADB_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4002), tables: &MARIADB_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4003), tables: &MARIADB_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4004), tables: &MARIADB_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4011), tables: &MARIADB_TABLES }),
        "PartitioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4012), tables: &MARIADB_TABLES }),
        "PartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4013), tables: &MARIADB_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4014), tables: &MARIADB_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4015), tables: &MARIADB_TABLES }),
        "Password_lock_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4016), tables: &MARIADB_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4017), tables: &MARIADB_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(4018), tables: &MARIADB_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(4025), tables: &MARIADB_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4026), tables: &MARIADB_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4027), tables: &MARIADB_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4028), tables: &MARIADB_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4029), tables: &MARIADB_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4030), tables: &MARIADB_TABLES }),
        "PeriodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4031), tables: &MARIADB_TABLES }),
        "PersistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4032), tables: &MARIADB_TABLES }),
        "Persist_onlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &MARIADB_TABLES }),
        "PersistentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4034), tables: &MARIADB_TABLES }),
        "PhaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4035), tables: &MARIADB_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &MARIADB_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(4037), tables: &MARIADB_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4038), tables: &MARIADB_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4039), tables: &MARIADB_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4040), tables: &MARIADB_TABLES }),
        "PluginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4041), tables: &MARIADB_TABLES }),
        "Plugin_dirKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4042), tables: &MARIADB_TABLES }),
        "PluginsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4043), tables: &MARIADB_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(4044), tables: &MARIADB_TABLES }),
        "PointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4045), tables: &MARIADB_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4046), tables: &MARIADB_TABLES }),
        "PolygonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4047), tables: &MARIADB_TABLES }),
        "PortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4048), tables: &MARIADB_TABLES }),
        "PortionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4049), tables: &MARIADB_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4050), tables: &MARIADB_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(4051), tables: &MARIADB_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4052), tables: &MARIADB_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4055), tables: &MARIADB_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4058), tables: &MARIADB_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4059), tables: &MARIADB_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4060), tables: &MARIADB_TABLES }),
        "PrecedesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4061), tables: &MARIADB_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4062), tables: &MARIADB_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4063), tables: &MARIADB_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4064), tables: &MARIADB_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4065), tables: &MARIADB_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4066), tables: &MARIADB_TABLES }),
        "PrepareSegment" => Some(RootGrammar { grammar_id: GrammarId(4067), tables: &MARIADB_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4075), tables: &MARIADB_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4076), tables: &MARIADB_TABLES }),
        "PrevKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4077), tables: &MARIADB_TABLES }),
        "PreviousKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4078), tables: &MARIADB_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4079), tables: &MARIADB_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4082), tables: &MARIADB_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4083), tables: &MARIADB_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4084), tables: &MARIADB_TABLES }),
        "Privilege_checks_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4085), tables: &MARIADB_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4086), tables: &MARIADB_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4087), tables: &MARIADB_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4088), tables: &MARIADB_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &MARIADB_TABLES }),
        "ProcedureParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(4090), tables: &MARIADB_TABLES }),
        "ProcedureParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(4099), tables: &MARIADB_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4102), tables: &MARIADB_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4103), tables: &MARIADB_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4104), tables: &MARIADB_TABLES }),
        "ProfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4105), tables: &MARIADB_TABLES }),
        "ProfilesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4106), tables: &MARIADB_TABLES }),
        "ProxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4107), tables: &MARIADB_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4108), tables: &MARIADB_TABLES }),
        "PurgeBinaryLogsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4109), tables: &MARIADB_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4123), tables: &MARIADB_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4124), tables: &MARIADB_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4127), tables: &MARIADB_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4128), tables: &MARIADB_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4136), tables: &MARIADB_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4137), tables: &MARIADB_TABLES }),
        "Query_response_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4138), tables: &MARIADB_TABLES }),
        "QuickKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4139), tables: &MARIADB_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4140), tables: &MARIADB_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4141), tables: &MARIADB_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4142), tables: &MARIADB_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4145), tables: &MARIADB_TABLES }),
        "RaiseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4146), tables: &MARIADB_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4147), tables: &MARIADB_TABLES }),
        "RandomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4148), tables: &MARIADB_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4149), tables: &MARIADB_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4150), tables: &MARIADB_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4151), tables: &MARIADB_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4152), tables: &MARIADB_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4153), tables: &MARIADB_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(4154), tables: &MARIADB_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(4155), tables: &MARIADB_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4156), tables: &MARIADB_TABLES }),
        "Read_onlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4157), tables: &MARIADB_TABLES }),
        "Read_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4158), tables: &MARIADB_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4159), tables: &MARIADB_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4160), tables: &MARIADB_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4161), tables: &MARIADB_TABLES }),
        "RebuildKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4162), tables: &MARIADB_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4163), tables: &MARIADB_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4164), tables: &MARIADB_TABLES }),
        "RecoverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4165), tables: &MARIADB_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4166), tables: &MARIADB_TABLES }),
        "Redo_buffer_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4167), tables: &MARIADB_TABLES }),
        "RedofileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4168), tables: &MARIADB_TABLES }),
        "RedundantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4169), tables: &MARIADB_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4170), tables: &MARIADB_TABLES }),
        "Ref_system_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4171), tables: &MARIADB_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4172), tables: &MARIADB_TABLES }),
        "ReferenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4186), tables: &MARIADB_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(4187), tables: &MARIADB_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4193), tables: &MARIADB_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4194), tables: &MARIADB_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4195), tables: &MARIADB_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4196), tables: &MARIADB_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4208), tables: &MARIADB_TABLES }),
        "RegistrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4209), tables: &MARIADB_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4210), tables: &MARIADB_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4211), tables: &MARIADB_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4212), tables: &MARIADB_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4213), tables: &MARIADB_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4214), tables: &MARIADB_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4215), tables: &MARIADB_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4216), tables: &MARIADB_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4217), tables: &MARIADB_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4218), tables: &MARIADB_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4219), tables: &MARIADB_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4220), tables: &MARIADB_TABLES }),
        "RelayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4221), tables: &MARIADB_TABLES }),
        "Relay_log_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4222), tables: &MARIADB_TABLES }),
        "Relay_log_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4223), tables: &MARIADB_TABLES }),
        "Relay_threadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4224), tables: &MARIADB_TABLES }),
        "RelaylogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4225), tables: &MARIADB_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4226), tables: &MARIADB_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4227), tables: &MARIADB_TABLES }),
        "RemoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4228), tables: &MARIADB_TABLES }),
        "RemoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4229), tables: &MARIADB_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4230), tables: &MARIADB_TABLES }),
        "RenameTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4231), tables: &MARIADB_TABLES }),
        "ReorganizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4239), tables: &MARIADB_TABLES }),
        "RepairKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4240), tables: &MARIADB_TABLES }),
        "RepairTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4241), tables: &MARIADB_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4253), tables: &MARIADB_TABLES }),
        "RepeatStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4254), tables: &MARIADB_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4269), tables: &MARIADB_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4270), tables: &MARIADB_TABLES }),
        "ReplaceSegment" => Some(RootGrammar { grammar_id: GrammarId(4271), tables: &MARIADB_TABLES }),
        "ReplayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4289), tables: &MARIADB_TABLES }),
        "ReplicaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4290), tables: &MARIADB_TABLES }),
        "Replica_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4291), tables: &MARIADB_TABLES }),
        "ReplicasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4292), tables: &MARIADB_TABLES }),
        "Replicate_do_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4293), tables: &MARIADB_TABLES }),
        "Replicate_do_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4294), tables: &MARIADB_TABLES }),
        "Replicate_ignore_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4295), tables: &MARIADB_TABLES }),
        "Replicate_ignore_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4296), tables: &MARIADB_TABLES }),
        "Replicate_rewrite_dbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4297), tables: &MARIADB_TABLES }),
        "Replicate_wild_do_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4298), tables: &MARIADB_TABLES }),
        "Replicate_wild_ignore_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4299), tables: &MARIADB_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4300), tables: &MARIADB_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4301), tables: &MARIADB_TABLES }),
        "Require_row_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4302), tables: &MARIADB_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4303), tables: &MARIADB_TABLES }),
        "ResetMasterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4304), tables: &MARIADB_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4310), tables: &MARIADB_TABLES }),
        "ResignalSegment" => Some(RootGrammar { grammar_id: GrammarId(4311), tables: &MARIADB_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4344), tables: &MARIADB_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4345), tables: &MARIADB_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4346), tables: &MARIADB_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4347), tables: &MARIADB_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4348), tables: &MARIADB_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4349), tables: &MARIADB_TABLES }),
        "ResumeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4350), tables: &MARIADB_TABLES }),
        "RetainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4351), tables: &MARIADB_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4352), tables: &MARIADB_TABLES }),
        "ReturnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4353), tables: &MARIADB_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4356), tables: &MARIADB_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4357), tables: &MARIADB_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4358), tables: &MARIADB_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4359), tables: &MARIADB_TABLES }),
        "ReturningClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4360), tables: &MARIADB_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4365), tables: &MARIADB_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4366), tables: &MARIADB_TABLES }),
        "ReuseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4367), tables: &MARIADB_TABLES }),
        "ReverseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4368), tables: &MARIADB_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4369), tables: &MARIADB_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4370), tables: &MARIADB_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4393), tables: &MARIADB_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4394), tables: &MARIADB_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4395), tables: &MARIADB_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4396), tables: &MARIADB_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4411), tables: &MARIADB_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4412), tables: &MARIADB_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4413), tables: &MARIADB_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4414), tables: &MARIADB_TABLES }),
        "RotateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4415), tables: &MARIADB_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4416), tables: &MARIADB_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4417), tables: &MARIADB_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4418), tables: &MARIADB_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4419), tables: &MARIADB_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4420), tables: &MARIADB_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4421), tables: &MARIADB_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4422), tables: &MARIADB_TABLES }),
        "Row_formatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4423), tables: &MARIADB_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4424), tables: &MARIADB_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4425), tables: &MARIADB_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4426), tables: &MARIADB_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4427), tables: &MARIADB_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4428), tables: &MARIADB_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4429), tables: &MARIADB_TABLES }),
        "RowtypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4430), tables: &MARIADB_TABLES }),
        "RtreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4431), tables: &MARIADB_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4432), tables: &MARIADB_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4433), tables: &MARIADB_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4445), tables: &MARIADB_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4446), tables: &MARIADB_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4447), tables: &MARIADB_TABLES }),
        "ScheduleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4448), tables: &MARIADB_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4449), tables: &MARIADB_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4450), tables: &MARIADB_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4451), tables: &MARIADB_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4452), tables: &MARIADB_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4453), tables: &MARIADB_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4454), tables: &MARIADB_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4455), tables: &MARIADB_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4456), tables: &MARIADB_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4457), tables: &MARIADB_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4458), tables: &MARIADB_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4459), tables: &MARIADB_TABLES }),
        "SecondaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4460), tables: &MARIADB_TABLES }),
        "Secondary_engineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4461), tables: &MARIADB_TABLES }),
        "Secondary_engine_attributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4462), tables: &MARIADB_TABLES }),
        "Secondary_loadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4463), tables: &MARIADB_TABLES }),
        "Secondary_unloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4464), tables: &MARIADB_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4465), tables: &MARIADB_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4466), tables: &MARIADB_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4467), tables: &MARIADB_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4472), tables: &MARIADB_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4485), tables: &MARIADB_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4492), tables: &MARIADB_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4503), tables: &MARIADB_TABLES }),
        "SelectPartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4504), tables: &MARIADB_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4509), tables: &MARIADB_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(4529), tables: &MARIADB_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4539), tables: &MARIADB_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(4540), tables: &MARIADB_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4541), tables: &MARIADB_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4542), tables: &MARIADB_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4543), tables: &MARIADB_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4544), tables: &MARIADB_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4551), tables: &MARIADB_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4558), tables: &MARIADB_TABLES }),
        "SerialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4559), tables: &MARIADB_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4560), tables: &MARIADB_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4561), tables: &MARIADB_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4562), tables: &MARIADB_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4563), tables: &MARIADB_TABLES }),
        "SessionVariableNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4564), tables: &MARIADB_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4565), tables: &MARIADB_TABLES }),
        "SetAssignmentStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4566), tables: &MARIADB_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(4599), tables: &MARIADB_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4603), tables: &MARIADB_TABLES }),
        "SetDefaultRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4614), tables: &MARIADB_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4626), tables: &MARIADB_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4635), tables: &MARIADB_TABLES }),
        "SetNamesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4636), tables: &MARIADB_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(4646), tables: &MARIADB_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4654), tables: &MARIADB_TABLES }),
        "SetTransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4659), tables: &MARIADB_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4685), tables: &MARIADB_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4686), tables: &MARIADB_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4687), tables: &MARIADB_TABLES }),
        "SetvalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4688), tables: &MARIADB_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4689), tables: &MARIADB_TABLES }),
        "SharedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4690), tables: &MARIADB_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4691), tables: &MARIADB_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(4692), tables: &MARIADB_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4701), tables: &MARIADB_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4702), tables: &MARIADB_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4703), tables: &MARIADB_TABLES }),
        "SignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4704), tables: &MARIADB_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(4705), tables: &MARIADB_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4708), tables: &MARIADB_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4709), tables: &MARIADB_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(4710), tables: &MARIADB_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(4715), tables: &MARIADB_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(4717), tables: &MARIADB_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4721), tables: &MARIADB_TABLES }),
        "SingleQuotedJSONPath" => Some(RootGrammar { grammar_id: GrammarId(4722), tables: &MARIADB_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4723), tables: &MARIADB_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4724), tables: &MARIADB_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4727), tables: &MARIADB_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(4728), tables: &MARIADB_TABLES }),
        "SlaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4729), tables: &MARIADB_TABLES }),
        "Slave_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4730), tables: &MARIADB_TABLES }),
        "SlavesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4731), tables: &MARIADB_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(4732), tables: &MARIADB_TABLES }),
        "SlowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4733), tables: &MARIADB_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4734), tables: &MARIADB_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4735), tables: &MARIADB_TABLES }),
        "SocketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4736), tables: &MARIADB_TABLES }),
        "SoftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4737), tables: &MARIADB_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4738), tables: &MARIADB_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4739), tables: &MARIADB_TABLES }),
        "SoundsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4740), tables: &MARIADB_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4741), tables: &MARIADB_TABLES }),
        "Source_auto_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4742), tables: &MARIADB_TABLES }),
        "Source_bindKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4743), tables: &MARIADB_TABLES }),
        "Source_compression_algorithmsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4744), tables: &MARIADB_TABLES }),
        "Source_connect_retryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4745), tables: &MARIADB_TABLES }),
        "Source_delayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4746), tables: &MARIADB_TABLES }),
        "Source_heartbeat_periodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4747), tables: &MARIADB_TABLES }),
        "Source_hostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4748), tables: &MARIADB_TABLES }),
        "Source_log_fileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4749), tables: &MARIADB_TABLES }),
        "Source_log_posKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4750), tables: &MARIADB_TABLES }),
        "Source_passwordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4751), tables: &MARIADB_TABLES }),
        "Source_portKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4752), tables: &MARIADB_TABLES }),
        "Source_public_key_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4753), tables: &MARIADB_TABLES }),
        "Source_retry_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4754), tables: &MARIADB_TABLES }),
        "Source_sslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4755), tables: &MARIADB_TABLES }),
        "Source_ssl_caKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4756), tables: &MARIADB_TABLES }),
        "Source_ssl_capathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4757), tables: &MARIADB_TABLES }),
        "Source_ssl_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4758), tables: &MARIADB_TABLES }),
        "Source_ssl_cipherKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4759), tables: &MARIADB_TABLES }),
        "Source_ssl_crlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4760), tables: &MARIADB_TABLES }),
        "Source_ssl_crlpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4761), tables: &MARIADB_TABLES }),
        "Source_ssl_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4762), tables: &MARIADB_TABLES }),
        "Source_ssl_verify_server_certKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4763), tables: &MARIADB_TABLES }),
        "Source_tls_ciphersuitesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4764), tables: &MARIADB_TABLES }),
        "Source_tls_versionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4765), tables: &MARIADB_TABLES }),
        "Source_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4766), tables: &MARIADB_TABLES }),
        "Source_zstd_compression_levelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4767), tables: &MARIADB_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4768), tables: &MARIADB_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4769), tables: &MARIADB_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4770), tables: &MARIADB_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4771), tables: &MARIADB_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4772), tables: &MARIADB_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4773), tables: &MARIADB_TABLES }),
        "Sql_after_gtidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4774), tables: &MARIADB_TABLES }),
        "Sql_after_mts_gapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4775), tables: &MARIADB_TABLES }),
        "Sql_before_gtidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4776), tables: &MARIADB_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4777), tables: &MARIADB_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4778), tables: &MARIADB_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4779), tables: &MARIADB_TABLES }),
        "Sql_buffer_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4780), tables: &MARIADB_TABLES }),
        "Sql_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4781), tables: &MARIADB_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4782), tables: &MARIADB_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4783), tables: &MARIADB_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4784), tables: &MARIADB_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4785), tables: &MARIADB_TABLES }),
        "Sql_no_cacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4786), tables: &MARIADB_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4787), tables: &MARIADB_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4788), tables: &MARIADB_TABLES }),
        "Sql_threadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4789), tables: &MARIADB_TABLES }),
        "Sql_tsi_dayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4790), tables: &MARIADB_TABLES }),
        "Sql_tsi_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4791), tables: &MARIADB_TABLES }),
        "Sql_tsi_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4792), tables: &MARIADB_TABLES }),
        "Sql_tsi_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4793), tables: &MARIADB_TABLES }),
        "Sql_tsi_quarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4794), tables: &MARIADB_TABLES }),
        "Sql_tsi_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4795), tables: &MARIADB_TABLES }),
        "Sql_tsi_weekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4796), tables: &MARIADB_TABLES }),
        "Sql_tsi_yearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4797), tables: &MARIADB_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4798), tables: &MARIADB_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4799), tables: &MARIADB_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4800), tables: &MARIADB_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4801), tables: &MARIADB_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4802), tables: &MARIADB_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4803), tables: &MARIADB_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4804), tables: &MARIADB_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4805), tables: &MARIADB_TABLES }),
        "SridKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4806), tables: &MARIADB_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4807), tables: &MARIADB_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4808), tables: &MARIADB_TABLES }),
        "StackedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4809), tables: &MARIADB_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4810), tables: &MARIADB_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4811), tables: &MARIADB_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(4812), tables: &MARIADB_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(246), tables: &MARIADB_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(3847), tables: &MARIADB_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4813), tables: &MARIADB_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(753), tables: &MARIADB_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4814), tables: &MARIADB_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4815), tables: &MARIADB_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4816), tables: &MARIADB_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4817), tables: &MARIADB_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4818), tables: &MARIADB_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4901), tables: &MARIADB_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4902), tables: &MARIADB_TABLES }),
        "Stats_auto_recalcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4903), tables: &MARIADB_TABLES }),
        "Stats_persistentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4904), tables: &MARIADB_TABLES }),
        "Stats_sample_pagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4905), tables: &MARIADB_TABLES }),
        "StatusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4906), tables: &MARIADB_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4907), tables: &MARIADB_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4908), tables: &MARIADB_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4909), tables: &MARIADB_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4910), tables: &MARIADB_TABLES }),
        "StopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4911), tables: &MARIADB_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4912), tables: &MARIADB_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4913), tables: &MARIADB_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4914), tables: &MARIADB_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4915), tables: &MARIADB_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4916), tables: &MARIADB_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4917), tables: &MARIADB_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4918), tables: &MARIADB_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4920), tables: &MARIADB_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4921), tables: &MARIADB_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4926), tables: &MARIADB_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4927), tables: &MARIADB_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4928), tables: &MARIADB_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4929), tables: &MARIADB_TABLES }),
        "SubjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4930), tables: &MARIADB_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4931), tables: &MARIADB_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4932), tables: &MARIADB_TABLES }),
        "SubpartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4933), tables: &MARIADB_TABLES }),
        "SubpartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4934), tables: &MARIADB_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4935), tables: &MARIADB_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4936), tables: &MARIADB_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4937), tables: &MARIADB_TABLES }),
        "SuperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4938), tables: &MARIADB_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4939), tables: &MARIADB_TABLES }),
        "SuspendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4940), tables: &MARIADB_TABLES }),
        "SwapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4941), tables: &MARIADB_TABLES }),
        "SwitchesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4942), tables: &MARIADB_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(4943), tables: &MARIADB_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4944), tables: &MARIADB_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4945), tables: &MARIADB_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4946), tables: &MARIADB_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4947), tables: &MARIADB_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4948), tables: &MARIADB_TABLES }),
        "SystemVariableSegment" => Some(RootGrammar { grammar_id: GrammarId(4949), tables: &MARIADB_TABLES }),
        "System_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4950), tables: &MARIADB_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4951), tables: &MARIADB_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4952), tables: &MARIADB_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5024), tables: &MARIADB_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5025), tables: &MARIADB_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5034), tables: &MARIADB_TABLES }),
        "TableOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(5035), tables: &MARIADB_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5201), tables: &MARIADB_TABLES }),
        "Table_checksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5207), tables: &MARIADB_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5208), tables: &MARIADB_TABLES }),
        "Table_statisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5209), tables: &MARIADB_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5210), tables: &MARIADB_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5211), tables: &MARIADB_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5212), tables: &MARIADB_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(5213), tables: &MARIADB_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(5218), tables: &MARIADB_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5222), tables: &MARIADB_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5223), tables: &MARIADB_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5224), tables: &MARIADB_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5225), tables: &MARIADB_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(5226), tables: &MARIADB_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(5227), tables: &MARIADB_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5230), tables: &MARIADB_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(5231), tables: &MARIADB_TABLES }),
        "TemptableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5234), tables: &MARIADB_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5235), tables: &MARIADB_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5236), tables: &MARIADB_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5237), tables: &MARIADB_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5238), tables: &MARIADB_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5239), tables: &MARIADB_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5240), tables: &MARIADB_TABLES }),
        "Thread_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5241), tables: &MARIADB_TABLES }),
        "ThreadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5242), tables: &MARIADB_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5243), tables: &MARIADB_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(5244), tables: &MARIADB_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5245), tables: &MARIADB_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(5246), tables: &MARIADB_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(5257), tables: &MARIADB_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5263), tables: &MARIADB_TABLES }),
        "TimestampaddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5264), tables: &MARIADB_TABLES }),
        "TimestampdiffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5265), tables: &MARIADB_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5266), tables: &MARIADB_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5267), tables: &MARIADB_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5268), tables: &MARIADB_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5269), tables: &MARIADB_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5270), tables: &MARIADB_TABLES }),
        "TlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5271), tables: &MARIADB_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5272), tables: &MARIADB_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5273), tables: &MARIADB_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5274), tables: &MARIADB_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5275), tables: &MARIADB_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5276), tables: &MARIADB_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5277), tables: &MARIADB_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5278), tables: &MARIADB_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5279), tables: &MARIADB_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5307), tables: &MARIADB_TABLES }),
        "TransactionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5308), tables: &MARIADB_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5309), tables: &MARIADB_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5310), tables: &MARIADB_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5311), tables: &MARIADB_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5312), tables: &MARIADB_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5313), tables: &MARIADB_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5314), tables: &MARIADB_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5315), tables: &MARIADB_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5316), tables: &MARIADB_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5317), tables: &MARIADB_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5318), tables: &MARIADB_TABLES }),
        "TriggerOrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(5319), tables: &MARIADB_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1152), tables: &MARIADB_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5322), tables: &MARIADB_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5323), tables: &MARIADB_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5324), tables: &MARIADB_TABLES }),
        "TriggersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5325), tables: &MARIADB_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5326), tables: &MARIADB_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(5327), tables: &MARIADB_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5331), tables: &MARIADB_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(5332), tables: &MARIADB_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5333), tables: &MARIADB_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5334), tables: &MARIADB_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5338), tables: &MARIADB_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5339), tables: &MARIADB_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(5340), tables: &MARIADB_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5343), tables: &MARIADB_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5344), tables: &MARIADB_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5347), tables: &MARIADB_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5350), tables: &MARIADB_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5351), tables: &MARIADB_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5352), tables: &MARIADB_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5353), tables: &MARIADB_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5354), tables: &MARIADB_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5355), tables: &MARIADB_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5356), tables: &MARIADB_TABLES }),
        "UndefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5360), tables: &MARIADB_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5361), tables: &MARIADB_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5362), tables: &MARIADB_TABLES }),
        "Undo_buffer_sizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5363), tables: &MARIADB_TABLES }),
        "UndofileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5364), tables: &MARIADB_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5365), tables: &MARIADB_TABLES }),
        "UnicodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5366), tables: &MARIADB_TABLES }),
        "UninstallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5367), tables: &MARIADB_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5368), tables: &MARIADB_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5373), tables: &MARIADB_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(5374), tables: &MARIADB_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5377), tables: &MARIADB_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5378), tables: &MARIADB_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5379), tables: &MARIADB_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5380), tables: &MARIADB_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5381), tables: &MARIADB_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5382), tables: &MARIADB_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5383), tables: &MARIADB_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5384), tables: &MARIADB_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5407), tables: &MARIADB_TABLES }),
        "UnregisterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5413), tables: &MARIADB_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5414), tables: &MARIADB_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5415), tables: &MARIADB_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5416), tables: &MARIADB_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5417), tables: &MARIADB_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5429), tables: &MARIADB_TABLES }),
        "UpgradeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5430), tables: &MARIADB_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5431), tables: &MARIADB_TABLES }),
        "UpsertClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(5432), tables: &MARIADB_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5439), tables: &MARIADB_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5440), tables: &MARIADB_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5441), tables: &MARIADB_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5444), tables: &MARIADB_TABLES }),
        "Use_frmKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5445), tables: &MARIADB_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5446), tables: &MARIADB_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(5447), tables: &MARIADB_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5448), tables: &MARIADB_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5449), tables: &MARIADB_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5450), tables: &MARIADB_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5451), tables: &MARIADB_TABLES }),
        "User_resourcesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5452), tables: &MARIADB_TABLES }),
        "User_statisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5453), tables: &MARIADB_TABLES }),
        "User_variablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5454), tables: &MARIADB_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5455), tables: &MARIADB_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5456), tables: &MARIADB_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5457), tables: &MARIADB_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5458), tables: &MARIADB_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5459), tables: &MARIADB_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5460), tables: &MARIADB_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5461), tables: &MARIADB_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5462), tables: &MARIADB_TABLES }),
        "ValidationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5463), tables: &MARIADB_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5464), tables: &MARIADB_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5465), tables: &MARIADB_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5466), tables: &MARIADB_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5479), tables: &MARIADB_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5480), tables: &MARIADB_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5481), tables: &MARIADB_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5482), tables: &MARIADB_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5483), tables: &MARIADB_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5484), tables: &MARIADB_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5485), tables: &MARIADB_TABLES }),
        "VariableAssignmentSegment" => Some(RootGrammar { grammar_id: GrammarId(5486), tables: &MARIADB_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5490), tables: &MARIADB_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5491), tables: &MARIADB_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5492), tables: &MARIADB_TABLES }),
        "VcpuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5493), tables: &MARIADB_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5494), tables: &MARIADB_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5495), tables: &MARIADB_TABLES }),
        "VersioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5496), tables: &MARIADB_TABLES }),
        "ViaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5497), tables: &MARIADB_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5498), tables: &MARIADB_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5499), tables: &MARIADB_TABLES }),
        "VirtualKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5500), tables: &MARIADB_TABLES }),
        "VisibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5501), tables: &MARIADB_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5502), tables: &MARIADB_TABLES }),
        "WaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5503), tables: &MARIADB_TABLES }),
        "WaitOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5504), tables: &MARIADB_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5509), tables: &MARIADB_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5510), tables: &MARIADB_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5511), tables: &MARIADB_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5512), tables: &MARIADB_TABLES }),
        "WarningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5513), tables: &MARIADB_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5514), tables: &MARIADB_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5515), tables: &MARIADB_TABLES }),
        "Weight_stringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5516), tables: &MARIADB_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5517), tables: &MARIADB_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5527), tables: &MARIADB_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5528), tables: &MARIADB_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5529), tables: &MARIADB_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5534), tables: &MARIADB_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5548), tables: &MARIADB_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5549), tables: &MARIADB_TABLES }),
        "WhileStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5550), tables: &MARIADB_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(5565), tables: &MARIADB_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5566), tables: &MARIADB_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5567), tables: &MARIADB_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5569), tables: &MARIADB_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5579), tables: &MARIADB_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(5580), tables: &MARIADB_TABLES }),
        "WithCheckOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5588), tables: &MARIADB_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5595), tables: &MARIADB_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5604), tables: &MARIADB_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5613), tables: &MARIADB_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(5618), tables: &MARIADB_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5619), tables: &MARIADB_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5620), tables: &MARIADB_TABLES }),
        "WithRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5625), tables: &MARIADB_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5628), tables: &MARIADB_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5629), tables: &MARIADB_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(5630), tables: &MARIADB_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5631), tables: &MARIADB_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5632), tables: &MARIADB_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5633), tables: &MARIADB_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5634), tables: &MARIADB_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5635), tables: &MARIADB_TABLES }),
        "XaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5636), tables: &MARIADB_TABLES }),
        "XidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5637), tables: &MARIADB_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5638), tables: &MARIADB_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5639), tables: &MARIADB_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5640), tables: &MARIADB_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5641), tables: &MARIADB_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5642), tables: &MARIADB_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5643), tables: &MARIADB_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5644), tables: &MARIADB_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5645), tables: &MARIADB_TABLES }),
        _ => None,
    }
}

pub fn get_mariadb_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AlgorithmOptionSegment" => Some("algorithm_option"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterEventStatementSegment" => Some("alter_event_statement"),
        "AlterOptionSegment" => Some("alter_option_segment"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableConstraintSegment" => Some("table_constraint"),
        "AlterTableOnlineDDLOptionSegment" => Some("alter_table_online_ddl_option"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AnalyzeTableStatementSegment" => Some("analyze_table_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStoredProcedureSegment" => Some("call_statement"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CharacteristicStatement" => Some("characteristic_statement"),
        "CheckTableStatementSegment" => Some("check_table_statement"),
        "ChecksumTableStatementSegment" => Some("checksum_table_statement"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "ConvertFunctionContentsSegment" => Some("function_contents"),
        "ConvertFunctionNameSegment" => Some("function_name"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateEventStatementSegment" => Some("create_event_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateOptionSegment" => Some("create_option_segment"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "CurrentTimestampLikeFunctionContentsSegment" => Some("function_contents"),
        "CurrentTimestampLikeFunctionNameSegment" => Some("function_name"),
        "CurrentTimestampLikeFunctionSegment" => Some("function"),
        "CursorFetchSegment" => Some("cursor_fetch_segment"),
        "CursorOpenCloseSegment" => Some("cursor_open_close_segment"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DeallocateSegment" => Some("deallocate_segment"),
        "DeclareStatement" => Some("declare_statement"),
        "Dedent" => Some("dedent"),
        "DefinerSegment" => Some("definer_segment"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DeleteTargetTableSegment" => Some("delete_target_table"),
        "DeleteUsingClauseSegment" => Some("using_clause"),
        "DelimiterStatement" => Some("delimiter_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropEventStatementSegment" => Some("drop_event_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExecuteSegment" => Some("execute_segment"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileSegment" => Some("file"),
        "FlushStatementSegment" => Some("flush_statement"),
        "ForClauseSegment" => Some("for_clause"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GetDiagnosticsSegment" => Some("get_diagnostics_segment"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "HelpStatementSegment" => Some("help_statement"),
        "IdentifierSegment" => Some("identifier"),
        "IfExpressionStatement" => Some("if_then_statement"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexColumnPrefixLengthSegment" => Some("bracketed_arguments"),
        "IndexHintClauseSegment" => Some("index_hint_clause"),
        "IndexOptionSegment" => Some("index_option"),
        "IndexOptionsSegment" => Some("index_option"),
        "IndexReferenceSegment" => Some("index_reference"),
        "IndexTypeGrammar" => Some("index_type"),
        "IndexTypeSegment" => Some("index_type"),
        "InsertRowAliasSegment" => Some("insert_row_alias"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "IterateStatementSegment" => Some("iterate_statement"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "JsonTableColumnsClauseSegment" => Some("json_table_columns_clause"),
        "JsonTableFunctionContentsSegment" => Some("function_contents"),
        "JsonTableFunctionNameSegment" => Some("function_name"),
        "JsonValueFunctionContentsSegment" => Some("function_contents"),
        "JsonValueFunctionNameSegment" => Some("function_name"),
        "KeywordSegment" => Some("keyword"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadDataSegment" => Some("load_data_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockOptionSegment" => Some("lock_option"),
        "LoopStatementSegment" => Some("loop_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NullSafeEqualsSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OptimizeTableStatementSegment" => Some("optimize_table_statement"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PrepareSegment" => Some("prepare_segment"),
        "ProcedureParameterListGrammar" => Some("procedure_parameter_list"),
        "PurgeBinaryLogsStatementSegment" => Some("purge_binary_logs_statement"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "RenameTableStatementSegment" => Some("rename_table_statement"),
        "RepairTableStatementSegment" => Some("repair_table_statement"),
        "RepeatStatementSegment" => Some("repeat_statement"),
        "ReplaceSegment" => Some("replace_statement"),
        "ResetMasterStatementSegment" => Some("reset_master_statement"),
        "ResignalSegment" => Some("resignal_segment"),
        "ReturnStatementSegment" => Some("return_statement"),
        "ReturningClauseSegment" => Some("returning_clause"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectPartitionClauseSegment" => Some("partition_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetAssignmentStatementSegment" => Some("set_statement"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetDefaultRoleStatementSegment" => Some("set_default_role_statement"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetNamesStatementSegment" => Some("set_names_statement"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetTransactionStatementSegment" => Some("set_transaction_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableOptionsSegment" => Some("table_options"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UpsertClauseListSegment" => Some("upsert_clause_list"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WaitOptionSegment" => Some("wait_option"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhileStatementSegment" => Some("while_statement"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCheckOptionSegment" => Some("with_check_options"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithRollupClauseSegment" => Some("with_rollup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static MARIADB_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_mariadb_root_grammar_id() -> GrammarId {
    GrammarId(2453)
}
pub fn get_mariadb_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_mariadb_root_grammar_id(),
        tables: &MARIADB_TABLES,
    }
}
pub fn get_mariadb_root_grammar() -> RootGrammar {
    get_mariadb_root_grammar_table()
}
