/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      4643 × 20 bytes =    92860 bytes
//   Child IDs:         3525 ×  4 bytes =    14100 bytes
//   Terminators:         70 ×  4 bytes =      280 bytes
//   Strings:           2163 strings      =    30067 bytes
//   Aux Data:         12033 ×  4 bytes =    48132 bytes
//   Aux Offsets:       4643 ×  4 bytes =    18572 bytes
//   Regex Patterns:       8 patterns     =      553 bytes
//   Simple Hints:      1231 hints        =    19696 bytes
//   Hint Indices:      2187 ×  4 bytes =     8748 bytes
//   TOTAL:                                    ≈   233008 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [4] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [6] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [8] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [10] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [51] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [55] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [61] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [62] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [63] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [64] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [134] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [137] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [138] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [139] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [140] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] Ref(SemiStructuredAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACCOUNTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 206, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] Ref(FileLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 208, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 211, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 211, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Ref(JarKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 211, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Ref(FileLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 213, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 217, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 220, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 222, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] StringParser(")")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 225, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Ref(JoinTypeKeywords)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Ref(PivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Meta("dedent")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 231, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] StringParser("ALIAS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 231, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 238, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 240, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(DatabasePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 242, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 251, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 253, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 257, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 261, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 261, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 262, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 262, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 266, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 270, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 273, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 273, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 279, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 281, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 281, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 285, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 290, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 292, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 298, child_count: 20, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] OneOf(20 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [339] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 330, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 335, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [345] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [348] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 339, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [349] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 342, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [350] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 344, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [351] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [352] Ref(ColumnFieldDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 346, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [353] Ref(FirstOrAfterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 346, child_count: 10, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [354] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 356, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [355] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [356] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [357] Ref(ChangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [358] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 358, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [359] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 359, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [360] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 360, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [361] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [362] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [363] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [364] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [365] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [366] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [367] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [368] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [369] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [370] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [371] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [372] Ref(FirstOrAfterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 367, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [373] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 370, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [374] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [375] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [376] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [377] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [378] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 372, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [379] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [380] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [381] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 375, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [382] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [383] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [384] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [385] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [386] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [387] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [388] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 384, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [389] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [390] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [391] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [392] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [393] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 389, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [395] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 392, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [396] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 392, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [397] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 395, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [398] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 397, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [399] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [400] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 398, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [401] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [402] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [403] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 2, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [404] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [405] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [406] Ref(PartitionFieldGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 403, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [407] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [408] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 407, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [409] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 407, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [410] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [411] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 409, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [412] Ref(PartitionFieldGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 409, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [413] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [414] Ref(PurgeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [416] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [417] Ref(PartitionFieldGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [419] Ref(RecoverKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [420] Ref(PartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [421] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [422] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [423] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [424] Ref(UnsetTablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [425] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 419, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [426] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 419, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [427] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 419, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [428] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 421, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [429] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [430] Ref(SerdepropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [431] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 423, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [432] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [433] Ref(SerdeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [434] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 426, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [435] Ref(SerdePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 426, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [436] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [437] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [438] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [439] Ref(FileformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [440] Ref(DataSourceFormatSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 430, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [441] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [442] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [443] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [444] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 433, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [445] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 440, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [446] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [447] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [448] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [449] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 442, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [450] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [451] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 443, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [452] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 443, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [453] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [454] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 446, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [455] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [456] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 2, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [457] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 450, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [458] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [459] Ref(DistributedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [460] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 453, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [461] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 453, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [462] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [463] Ref(LocallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [464] Ref(OrderedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [465] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 459, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [466] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 461, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [467] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [468] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [469] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [470] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [471] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 466, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [472] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [473] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [474] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [475] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [476] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 470, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [477] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [478] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [479] Ref(IdentifierKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [480] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 476, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [481] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 478, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [482] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [483] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 479, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [484] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [485] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [486] Ref(IdentifierKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [487] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 485, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [488] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 487, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [489] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [490] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 488, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [491] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [492] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [493] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [494] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [495] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 496, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [496] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [497] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [498] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [499] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [501] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [502] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [503] Ref(UnsetTablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [504] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [505] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 503, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [506] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 505, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [507] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [508] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [509] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [510] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [511] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [512] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [513] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [514] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [515] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 512, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [516] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [517] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [518] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [519] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [520] Ref(ComputeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [521] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 518, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [522] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [523] Ref(NoscanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 520, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [524] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [525] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [526] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 523, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [527] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 525, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [528] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 528, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [529] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [530] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [531] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 535, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [532] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 535, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [534] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [535] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [536] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [537] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [538] Ref(ComputeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [539] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [540] Ref(NoscanKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [541] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [542] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [543] StringParser("ANTI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [544] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [545] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [546] StringParser("ARCHIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [547] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 11, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [548] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [549] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [550] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [551] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [552] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [553] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [554] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [555] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [556] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [557] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [558] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [559] Ref(DivBinaryOperatorSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 550, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [560] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [561] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 555, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [562] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [563] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [564] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [565] Ref(SliceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [566] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [567] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [568] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 557, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [569] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [570] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 560, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [571] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [572] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 562, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [573] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 564, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [574] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 564, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [575] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [576] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [577] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 567, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [578] StringParser(">")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 567, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [579] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [580] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [581] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [582] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [583] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [584] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [585] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [586] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [587] StringParser("AT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [588] TypedParser("at_sign_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [589] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [590] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [591] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [592] StringParser("AUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 568, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [593] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 568, child_count: 1, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [594] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [595] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [596] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [597] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [598] StringParser("AVG_ROW_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [599] StringParser("AVRO")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [600] TypedParser("back_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [601] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [602] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 569, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [603] MultiStringParser(3 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 569, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 2, _padding: 0 }, // [604] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [605] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [606] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [607] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [608] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [609] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [610] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 576, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [611] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [612] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [613] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [614] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [615] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [616] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [617] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [618] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [619] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [620] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [621] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [622] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 578, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [623] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 578, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [624] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [625] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [626] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [627] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [628] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [629] Ref(RightArrowOperator)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [630] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [631] StringParser("BINARYFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [632] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [633] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [634] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [635] StringParser("BITVAR")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 583, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [636] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 583, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [637] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [638] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [639] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 585, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [640] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 585, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [642] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [643] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [644] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [645] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 587, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [646] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [647] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [648] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [649] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 589, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [650] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 589, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [651] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [652] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [653] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 591, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [654] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 591, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [655] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 594, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [656] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 596, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [657] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 596, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [658] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 599, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [659] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [660] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 601, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [661] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [662] Ref(PropertyListGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 604, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [663] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [664] Ref(PropertyNameListGrammar)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 607, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [665] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 607, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [666] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [667] Ref(UnorderedSetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [668] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [669] StringParser("BREAK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [670] StringParser("BROWSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 610, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [671] StringParser("BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 610, child_count: 5, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [672] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [673] Ref(ClusteredBySpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [674] Ref(SortedBySpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [675] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [676] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [677] Ref(BucketsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [678] StringParser("BUCKETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [679] StringParser("BULK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 615, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [680] StringParser("BY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 615, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [681] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [682] TypedParser("bytes_single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 617, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [683] TypedParser("bytes_double_quote")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 617, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [684] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [685] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 620, child_count: 4, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [686] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [687] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [688] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 624, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [689] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 624, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [690] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 627, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [691] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 627, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [692] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 627, child_count: 6, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [693] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [694] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [695] Ref(LazyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [696] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [697] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 633, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [698] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 633, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [699] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [700] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [701] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [702] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [703] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [704] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [705] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 635, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [706] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 2, min_times: 0, first_terminator_idx: 11, terminator_count: 3, _padding: 0 }, // [707] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 637, child_count: 6, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [708] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 643, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [709] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 643, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [710] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 643, child_count: 1, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [711] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [712] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [713] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [714] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 1, _padding: 0 }, // [715] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [716] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [717] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 644, child_count: 7, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [718] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [719] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 651, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [720] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 651, child_count: 1, min_times: 0, first_terminator_idx: 8, terminator_count: 2, _padding: 0 }, // [721] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [722] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [723] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 8, terminator_count: 0, _padding: 0 }, // [724] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 1, _padding: 0 }, // [725] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [726] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [727] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [728] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [729] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 11, terminator_count: 0, _padding: 0 }, // [730] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [731] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [732] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [733] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [734] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [735] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [736] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [737] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [738] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [739] StringParser("CHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [740] StringParser("CHANGES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [741] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [742] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [743] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [744] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [745] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [746] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [747] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [748] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [749] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [750] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [751] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [752] StringParser("CHECKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [753] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [754] StringParser("CHECKSUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [755] StringParser("CHOICES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [756] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 652, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [757] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 652, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [758] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [759] Ref(ClearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [760] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [761] StringParser("CLEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [762] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 654, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [763] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 654, child_count: 5, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [764] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [765] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [766] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 5, _padding: 0 }, // [767] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 661, child_count: 1, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [768] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 662, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [769] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [770] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [771] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [772] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [773] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [774] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [775] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [776] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [777] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 665, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [778] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 665, child_count: 3, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [779] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [780] Ref(ClusteredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [781] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [782] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [783] StringParser("CLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [784] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [785] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [786] Token("raw")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [787] StringParser("CODEGEN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [788] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [789] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 668, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [790] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 668, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [791] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [792] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 670, child_count: 2, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [793] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [794] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [795] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [796] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [797] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [798] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [799] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [800] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [801] StringParser("COLLECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [802] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [803] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [804] StringParser(":")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [805] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [806] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [807] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [808] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [809] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [810] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 678, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [812] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [813] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 10, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [814] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [815] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [816] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [817] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 692, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [819] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 694, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [820] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [821] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [822] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [823] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [824] Ref(ColumnConstraintDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 699, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [825] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 701, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [826] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 701, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [827] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 701, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [828] Ref(UniqueKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 701, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [829] Ref(AutoIncrementGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 701, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [830] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [831] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [832] Ref(NotEnforcedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [833] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 703, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [834] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [835] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [836] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [837] Ref(ColumnGeneratedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 705, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [838] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [839] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [840] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 709, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [841] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 712, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [842] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 712, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [843] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [844] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [845] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [846] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [847] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 717, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [848] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 720, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [849] Anything
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 720, child_count: 1, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [850] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 721, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [851] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 721, child_count: 4, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [852] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [853] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [854] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [855] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 725, child_count: 3, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [856] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 728, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [857] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [858] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [859] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 730, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [860] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 730, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 1, _padding: 0 }, // [861] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [862] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [863] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [864] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [865] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [866] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [867] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [868] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [869] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [870] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [871] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [872] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [873] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [875] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [876] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 734, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [877] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [878] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [879] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [880] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [881] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [882] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [883] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [884] StringParser("COMPACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 736, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [885] StringParser("COMPACTIONS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 736, child_count: 11, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [886] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [887] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [888] Ref(EqualsSegment_a)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [889] Ref(EqualsSegment_b)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [890] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [891] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [892] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [893] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [894] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [895] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 747, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [896] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [897] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [898] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [899] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [900] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [901] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [902] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [903] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [904] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [905] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [906] StringParser("COMPENSATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [907] StringParser("COMPLETION")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [908] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [909] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [910] StringParser("COMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [911] StringParser("COMPUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 754, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [912] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [913] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [914] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [915] StringParser("CONCATENATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [916] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [917] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [918] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [919] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [920] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [921] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [922] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [923] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [924] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [925] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 759, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [926] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 759, child_count: 6, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [927] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [928] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [929] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [930] Ref(ExpectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [931] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [932] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 768, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [933] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [934] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [935] Ref(ViolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 770, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [936] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 772, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [937] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [938] Ref(FailKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [939] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 774, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [940] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [941] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [942] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [943] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [944] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [945] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [946] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [947] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [948] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [949] StringParser("CONTAINSTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [950] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [951] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 776, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [952] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 776, child_count: 6, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [953] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [954] Ref(ConvertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [955] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [956] Ref(DeltaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 782, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [957] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 782, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [958] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [959] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [960] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [961] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [962] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [963] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [964] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [965] StringParser("COST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [966] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [967] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [968] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 10, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [969] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [970] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [971] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 794, child_count: 5, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [972] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [973] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [974] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [975] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [976] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 799, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [977] Ref(SpecificKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 799, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [978] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [979] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [980] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [981] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 803, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [982] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 805, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [983] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [984] Ref(InstanceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [985] Ref(StaticKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [986] Ref(ConstructorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [987] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [988] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 808, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [989] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 808, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [991] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 810, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [992] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 810, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [993] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [994] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [995] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 812, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [996] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [997] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 819, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [998] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [999] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1000] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1001] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1002] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1003] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 821, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1004] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 821, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1005] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1006] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1007] Ref(DbpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1008] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 824, child_count: 9, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1009] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1010] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1011] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1012] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1013] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1014] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1015] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1016] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1017] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 833, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1018] Ref(ResourceLocationGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 833, child_count: 9, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1019] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1020] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1021] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1022] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1023] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1024] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1025] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1026] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1027] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 842, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1028] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 843, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1029] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1030] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1031] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 848, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1032] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 848, child_count: 8, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1033] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1034] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1035] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1036] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1037] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1038] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 856, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1039] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1040] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 858, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1041] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 861, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1042] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1043] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1044] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1045] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 866, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1046] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1047] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 868, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1048] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1049] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1050] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1051] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1052] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1053] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1054] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1055] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1056] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1057] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 877, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1058] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1059] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1060] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1061] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1062] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 881, child_count: 7, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1063] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 888, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1064] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1065] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1066] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1067] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 891, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1068] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1069] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1070] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1071] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1072] Ref(SequenceMinValueGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1073] Ref(SequenceMaxValueGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1074] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1075] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1076] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1077] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1078] Ref(NocacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1079] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1080] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1081] Ref(NocycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1082] Ref(OrderNoOrderGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 900, child_count: 4, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1083] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1084] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1085] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 904, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1086] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 904, child_count: 1, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1087] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1088] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1089] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1090] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1091] Ref(TableDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 9, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1092] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1093] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1094] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 916, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1095] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 916, child_count: 3, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1096] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1097] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1098] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 919, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1099] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1100] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1101] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 921, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 1, _padding: 0 }, // [1102] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 923, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1103] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1104] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1105] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1106] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1107] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1108] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 2, min_times: 0, first_terminator_idx: 21, terminator_count: 2, _padding: 0 }, // [1109] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1110] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1111] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 21, terminator_count: 0, _padding: 0 }, // [1112] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1113] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [1114] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1115] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 931, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1116] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 931, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1117] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 936, child_count: 9, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1118] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1119] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1120] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1121] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1122] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1123] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1124] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1125] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1126] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1127] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 945, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1129] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1130] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 947, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1131] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1132] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1133] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1134] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1135] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1136] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1137] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 955, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1138] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1139] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1140] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 957, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1141] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1142] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1143] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1144] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1145] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 962, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1146] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 962, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1147] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1148] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1149] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1151] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 966, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1152] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 969, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1153] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 969, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1154] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1155] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1156] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1157] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1158] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 973, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1159] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1160] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1161] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1162] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 976, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1163] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 979, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1164] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 979, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1165] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 979, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1166] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1167] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1168] Ref(CompensationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 982, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1170] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1171] Ref(EvolutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 984, child_count: 18, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1172] Sequence(18 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1002, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1173] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1002, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1174] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1175] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1176] Ref(OrRefreshGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1177] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1178] Ref(StreamingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1179] Ref(LiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1180] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1181] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1182] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1004, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1183] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1004, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1184] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1005, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1185] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1008, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1186] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1010, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1187] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1012, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1188] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1189] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1190] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1191] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1015, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1192] Ref(ConstraintStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1015, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1193] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1194] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1195] Ref(DataSourceFormatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1017, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1196] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1017, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1197] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1198] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1199] Ref(TableClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1200] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1201] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1202] Ref(CreateViewClausesGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1019, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1203] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1204] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1205] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1206] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1207] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1026, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1208] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1026, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1209] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1210] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1211] Ref(WidgetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1212] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1031, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1213] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1214] Ref(DropdownKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1215] Ref(WidgetNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1216] Ref(WidgetDefaultGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1035, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1218] Ref(ChoicesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1219] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1037, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1220] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1221] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1222] Ref(WidgetNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1223] Ref(WidgetDefaultGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1224] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1225] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1226] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1227] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1228] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1229] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1040, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1230] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1040, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1231] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1042, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1232] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1233] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1234] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1044, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1235] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1236] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1237] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1238] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1239] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1240] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1241] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1242] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1243] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1244] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1245] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1246] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1247] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1248] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1249] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1047, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1250] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1047, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1251] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1252] Ref(FileFormatGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1253] Ref(JdbcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1254] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1050, child_count: 10, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1255] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1256] Ref(AvroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1257] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1258] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1259] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1260] Ref(OrcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1261] Ref(DeltaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1262] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1263] Ref(IcebergKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1264] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1265] Ref(BinaryfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1060, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1266] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1060, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1267] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1268] Ref(DbpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1269] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1062, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1270] StringParser("DATABASES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1062, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1271] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1064, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1272] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1064, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1273] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1274] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1275] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1276] Ref(PrimitiveTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1277] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1069, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1278] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1279] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1280] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1281] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1282] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1283] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1284] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1285] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1286] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1287] MultiStringParser(6 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1288] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1077, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1289] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1078, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1290] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1291] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1083, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1293] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1294] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1295] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 6, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1296] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1297] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1298] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1299] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1300] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1301] Ref(Timestamp_ltzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1302] Ref(Timestamp_ntzKeywordSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1303] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1304] StringParser("DATE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1305] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1306] MultiStringParser(26 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1307] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1308] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1309] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1310] StringParser("DAY_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1311] StringParser("DAY_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1312] StringParser("DAY_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1313] StringParser("DAY_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1314] StringParser("DAYOFMONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1315] StringParser("DAYOFWEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1316] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1317] StringParser("DAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1318] StringParser("DBCC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1319] StringParser("DBPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1320] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1321] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1322] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1323] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1093, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1324] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1093, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1325] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1326] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1327] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1328] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1329] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1330] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1331] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1332] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1333] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1334] StringParser("DELAY_KEY_WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1335] StringParser("DELAYED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1336] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1337] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1338] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1339] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1340] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1341] StringParser("DELETES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1342] StringParser("DELIMITED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1343] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1344] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1345] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1346] StringParser("DELTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1347] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1348] StringParser("DENY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1349] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1350] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1351] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1098, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1352] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1098, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1353] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1101, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1354] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1101, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1355] Ref(DetailKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1101, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1356] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1357] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1358] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1359] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1104, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1360] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1361] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1362] Ref(HistoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1108, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1363] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1364] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1365] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1366] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1367] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1111, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1368] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1111, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1369] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1116, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1370] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1119, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1371] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1372] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1373] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1374] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1375] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1376] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1377] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1378] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1379] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 5, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1380] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1381] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1382] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1383] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1129, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1384] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1129, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1131, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1386] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 1131, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1387] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1132, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1389] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1390] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1134, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1136, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1392] Ref(QueryKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1136, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1393] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1139, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1394] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1395] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1396] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1141, child_count: 8, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1397] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1398] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1399] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1400] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1149, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1401] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1402] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1403] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1404] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1405] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1406] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1407] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1151, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1408] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1409] Ref(HistoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1410] Ref(DetailKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1153, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1411] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1155, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1412] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1413] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1414] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1415] Ref(DescribeObjectGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1416] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1417] StringParser("DESTROY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1418] StringParser("DESTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1419] StringParser("DETAIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1420] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1421] StringParser("DFS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1422] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1423] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1424] StringParser("DIRECTORIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1425] StringParser("DIRECTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1426] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1427] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1428] StringParser("DISK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1429] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1430] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1157, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1431] StringParser("DISTINCTROW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1157, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1432] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1433] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1434] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 6, _padding: 0 }, // [1435] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 1, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1436] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1165, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1437] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1438] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1439] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1440] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1441] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1442] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1443] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1444] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1445] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [1446] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1447] StringParser("DISTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1448] StringParser("DISTRIBUTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1449] Ref(DivKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1450] StringParser("DIV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1451] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1452] StringParser("DO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1453] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1454] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1168, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1455] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1168, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1456] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1457] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1458] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1170, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1459] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1460] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1461] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1174, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1462] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1463] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1464] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1465] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1466] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1467] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1468] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1469] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1470] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1471] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1472] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1473] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1186, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1474] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1186, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1475] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1476] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1477] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1478] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1479] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1480] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1191, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1481] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1482] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1483] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1484] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1485] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1486] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1196, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1487] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1196, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1488] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1489] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1490] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1491] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1492] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1200, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1493] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1494] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1495] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1496] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1497] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1498] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1499] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1500] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1501] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1502] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1209, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1503] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1209, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1504] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1505] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1506] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1507] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1212, child_count: 6, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1508] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1509] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1510] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1511] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1218, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1512] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1218, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1513] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1514] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1515] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1516] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1517] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1518] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1519] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1520] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1224, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1521] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1522] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1523] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1524] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1525] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1229, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1526] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1229, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1527] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1528] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1529] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1530] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1531] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1233, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1532] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1533] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1534] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1535] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1536] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1537] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1538] StringParser("DROPDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1539] StringParser("DRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1540] StringParser("DUMMY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1541] StringParser("DUMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1542] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1543] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1544] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1545] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1238, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1546] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1547] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1548] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1549] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1550] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1551] StringParser("ELSEIF")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1552] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1554] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1555] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1556] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1557] StringParser("ENCLOSED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1558] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1559] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1560] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1561] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1562] StringParser("*/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1563] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1564] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1565] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1566] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1567] StringParser("==")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1568] StringParser("<=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1569] StringParser("ERRLVL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1570] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1571] StringParser("ESCAPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1572] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1573] StringParser("EVOLUTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1575] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1576] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1251, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1577] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1578] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1579] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1580] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1581] StringParser("EXCHANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1582] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1583] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1584] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1585] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1586] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1587] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1588] StringParser("EXISTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1589] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1590] StringParser("EXIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1591] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1592] StringParser("EXPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1253, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1593] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1253, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1594] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1256, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1595] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1256, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1596] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1597] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1598] Ref(CodegenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1599] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1600] Ref(FormattedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1601] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1602] StringParser("EXPORT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1603] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1260, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1605] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1262, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1606] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1263, child_count: 10, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1607] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1608] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1610] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1611] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1612] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1613] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1275, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1614] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1615] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1616] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1617] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1618] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1619] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1620] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1278, child_count: 5, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1621] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1622] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1623] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1624] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1625] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1626] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1283, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1627] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1628] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1629] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1285, child_count: 4, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1630] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1289, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1631] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1290, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1632] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1633] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1634] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1635] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1636] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1637] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1638] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1293, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1639] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1294, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1640] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1642] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1643] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1644] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1645] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1646] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1300, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1647] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1302, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1648] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1303, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1649] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1650] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1651] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 1, _padding: 0 }, // [1652] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1653] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1654] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 3, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1655] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1656] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1657] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 2, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1658] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1659] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1316, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1660] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1316, child_count: 1, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1661] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1662] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1663] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 30, terminator_count: 0, _padding: 0 }, // [1664] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1665] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1319, child_count: 9, min_times: 0, first_terminator_idx: 31, terminator_count: 1, _padding: 0 }, // [1666] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1667] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1668] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1669] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1331, child_count: 4, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1670] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1671] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1672] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1673] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1674] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1337, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1675] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1676] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1677] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1678] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1679] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1680] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1681] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1682] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1342, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1683] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1344, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1684] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1685] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1686] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 3, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1687] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1349, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1688] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1351, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1689] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1351, child_count: 2, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1691] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1353, child_count: 5, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1692] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1693] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1694] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1695] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1696] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1697] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1698] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1699] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 31, terminator_count: 0, _padding: 0 }, // [1700] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1358, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1701] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1358, child_count: 7, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1702] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1703] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1704] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1705] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1706] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1707] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1708] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1709] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1710] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1711] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1712] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1713] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1714] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1715] StringParser("FAIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1716] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1717] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1718] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1719] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1370, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1720] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1721] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1372, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1722] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1372, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1723] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1374, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1724] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1374, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1725] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1726] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1728] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1729] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1730] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1731] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1379, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1732] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1733] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1734] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1735] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1736] StringParser("FIELD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1737] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 6, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1738] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1739] Ref(DataSourcesV2FileTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1740] Ref(SequencefileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1741] Ref(TextfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1742] Ref(RcfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1743] Ref(JsonfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1387, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1744] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1745] Ref(InputformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1746] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1747] Ref(OutputformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1748] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1391, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1749] MultiStringParser(2 templates)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1391, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1750] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1751] TypedParser("file_literal")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1752] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1753] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1395, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1754] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1397, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1755] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1756] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1757] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1758] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1399, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1759] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1760] Ref(DataSourcesV2FileTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1761] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1762] Ref(BackQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1763] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1405, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1764] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1765] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1766] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1767] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 1, min_times: 1, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1768] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1409, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1769] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1409, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1770] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1771] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1772] StringParser("FILEFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1773] StringParser("FILLFACTOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1774] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1775] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1412, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1776] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1777] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1778] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1779] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1780] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1781] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1417, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1782] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1417, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1783] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1418, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1784] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1785] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1786] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1787] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1788] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1789] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1790] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1791] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1792] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1793] StringParser("FLUSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1794] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1795] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1422, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1796] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1422, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1798] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1799] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1800] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1801] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1802] StringParser("FORMATTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1803] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1804] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1424, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1805] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1424, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1806] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1807] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1426, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1808] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1809] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1430, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1810] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1811] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1812] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1432, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1813] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1814] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1815] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1816] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1817] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1818] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1819] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1820] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1439, child_count: 4, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1821] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1822] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1823] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1443, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1824] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1825] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1826] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1827] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1828] StringParser("FREETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1829] StringParser("FREETEXTTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1445, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1830] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1831] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1832] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1447, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1833] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1834] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1449, child_count: 13, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1835] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1836] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1837] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1462, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1838] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1839] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1840] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1464, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1841] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1842] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1843] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1466, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1844] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1845] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1846] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1468, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1847] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1848] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1849] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1470, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1850] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1851] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1852] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1853] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1854] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1855] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1856] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1857] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1858] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1472, child_count: 5, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1859] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1477, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1860] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1477, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1861] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1479, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1862] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1863] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1482, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1864] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1482, child_count: 1, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1865] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1483, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1866] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1867] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1868] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1485, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1869] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1485, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1870] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1871] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1490, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1872] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 2, _padding: 0 }, // [1873] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1874] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1875] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1499, child_count: 3, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1876] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1877] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1502, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1878] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1879] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1880] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1504, child_count: 2, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1506, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1882] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1506, child_count: 0, min_times: 0, first_terminator_idx: 32, terminator_count: 0, _padding: 0 }, // [1883] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1506, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1884] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1506, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 2, _padding: 0 }, // [1885] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1507, child_count: 1, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1886] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1887] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1888] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1889] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1510, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1890] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1891] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1892] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 2, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1894] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 34, terminator_count: 0, _padding: 0 }, // [1895] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1514, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1896] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1514, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1897] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1898] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1519, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1899] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1900] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1901] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1902] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1903] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1521, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1904] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1523, child_count: 3, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1905] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1906] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1907] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 1, _padding: 0 }, // [1908] Ref(InsertSourceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [1909] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1910] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1911] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1912] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 1, _padding: 0 }, // [1913] Ref(InsertSourceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 0, min_times: 0, first_terminator_idx: 37, terminator_count: 0, _padding: 0 }, // [1914] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1529, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1915] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1916] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1917] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1918] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1919] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1539, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1920] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1539, child_count: 1, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1921] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1540, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1922] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1923] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1924] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1925] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1543, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1926] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1543, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1927] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1545, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1928] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1548, child_count: 6, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1929] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 3, _padding: 0 }, // [1930] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1931] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1932] Ref(FromInsertFromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1933] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1560, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1934] Ref(FromInsertFromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1560, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1935] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1936] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1937] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1938] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1939] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1940] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [1941] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1564, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1942] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1564, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 3, _padding: 0 }, // [1943] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1565, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1944] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1945] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1946] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1947] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1568, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1948] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1949] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1950] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1951] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1952] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1953] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [1954] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1955] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1956] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1957] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1574, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1958] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1959] Ref(FromInsertFromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1960] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1961] Ref(FromInsertSourceClauseSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 1, min_times: 1, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1962] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1963] Ref(FromInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1964] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1965] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1579, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1966] StringParser("FULLTEXT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1579, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1967] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1968] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1581, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1969] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1581, child_count: 11, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1970] AnyNumberOf(11 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1971] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1592, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1972] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1973] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1974] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1975] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 4, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1976] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1599, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1977] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1599, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1978] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1979] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1980] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1981] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1600, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1982] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1603, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1983] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1984] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1985] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1986] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1987] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1988] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1607, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1989] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1607, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1990] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1991] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1609, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1992] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1993] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1994] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1611, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1995] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1996] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1997] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1613, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1998] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1616, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [1999] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2000] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2001] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2002] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2003] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1619, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2004] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2005] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2006] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2007] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2008] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2009] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2010] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2011] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1623, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2012] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1626, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2013] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1626, child_count: 3, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2014] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2015] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1629, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2016] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1629, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2017] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2018] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2019] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2020] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2021] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1631, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1633, child_count: 1, min_times: 0, first_terminator_idx: 44, terminator_count: 1, _padding: 0 }, // [2023] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1634, child_count: 2, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2024] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2025] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2026] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 0, min_times: 0, first_terminator_idx: 44, terminator_count: 0, _padding: 0 }, // [2027] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1636, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 1, _padding: 0 }, // [2028] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2029] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2030] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [2031] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1638, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2032] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2033] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1642, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2034] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1642, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2035] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2036] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2037] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2038] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2039] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1646, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2040] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1648, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2041] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2042] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2043] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2044] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1650, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2045] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1653, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2046] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2047] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1655, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2048] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1658, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2049] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2050] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1662, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2051] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1662, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2052] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1664, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2053] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2054] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1666, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2055] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2056] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2057] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1668, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2058] Ref(CommentClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1668, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2059] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2060] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2061] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2062] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2063] Ref(ColumnsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2064] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1675, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2065] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 1677, child_count: 1, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2066] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1678, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2067] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2068] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2069] Ref(ColumnsExpressionFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2070] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2071] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2072] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2073] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2074] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2075] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2076] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2077] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1681, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2078] StringParser("GENERATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1681, child_count: 5, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2079] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2080] Ref(GenerateKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2081] StringParser("SYMLINK_FORMAT_MANIFEST")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2082] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2083] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1686, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2084] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2085] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2086] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2087] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2088] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2089] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2090] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2091] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2092] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2093] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1689, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2094] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1689, child_count: 6, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2095] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2096] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1695, child_count: 4, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2097] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1699, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2098] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2099] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2100] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2101] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1702, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2102] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2103] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2104] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1704, child_count: 4, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2105] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2106] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2107] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2108] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2109] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2110] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2111] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2112] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1708, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2113] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2114] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2115] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2116] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2117] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1714, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2118] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2119] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2120] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2121] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1717, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2122] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2123] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2124] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1720, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2125] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1720, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2126] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2127] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2128] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1723, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2129] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2130] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2131] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2132] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2133] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1726, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2134] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1726, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2135] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2136] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2137] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2138] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1728, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2139] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1728, child_count: 5, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2140] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2141] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2142] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1733, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2143] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2144] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1737, child_count: 5, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2145] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2146] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2147] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2148] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2149] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2150] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1742, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2152] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1746, child_count: 3, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2153] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2154] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2155] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2156] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2157] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2158] Ref(WithCubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2159] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1751, child_count: 4, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2160] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2161] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1757, child_count: 4, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2162] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2163] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2164] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2165] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2166] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2167] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2168] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2169] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2170] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1761, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2171] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1761, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2172] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1762, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 1, _padding: 0 }, // [2173] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1764, child_count: 4, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2174] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2175] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2176] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2177] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1768, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2178] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [2179] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1770, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2180] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1770, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2181] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2182] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2183] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2184] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1776, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2185] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1778, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2186] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2187] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2188] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1780, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2189] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1780, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2190] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2191] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1784, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2192] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1786, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2193] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2194] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2195] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1793, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1795, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2197] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2198] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2199] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2200] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2201] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2202] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2203] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2204] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2205] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2206] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2207] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2208] StringParser("HEAP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2209] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2210] StringParser("HIGH_PRIORITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2212] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2213] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2214] StringParser("HISTORY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2215] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2216] StringParser("HOLDLOCK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2217] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2218] StringParser("HOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2219] StringParser("HOSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2220] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2221] StringParser("HOUR_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2222] StringParser("HOUR_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2223] StringParser("HOUR_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2224] StringParser("HOURS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1801, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2225] StringParser("ICEBERG")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1801, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2226] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1803, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2227] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1805, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2228] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2229] Ref(YearsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2230] Ref(MonthsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2231] Ref(DaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2232] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2233] Ref(HoursKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2234] Ref(Date_hourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1811, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2235] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2236] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1814, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2237] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2238] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2239] Ref(BucketKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2240] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1818, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2241] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1821, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2242] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2243] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2244] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2245] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2246] StringParser("IDENTIFIED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2247] StringParser("IDENTIFIER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2248] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2249] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2250] StringParser("IDENTITY_INSERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1824, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2251] StringParser("IDENTITYCOL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1824, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2253] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2254] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1826, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2255] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1826, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2256] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2257] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2258] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2259] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1829, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2260] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1829, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2261] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1831, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2262] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2263] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2264] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2265] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2266] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2267] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2268] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2269] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2270] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2271] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2272] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1833, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2273] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1833, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2274] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2275] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2276] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1836, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2277] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1838, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2278] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1841, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2279] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1843, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2280] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2281] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2282] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2283] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2284] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2285] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1845, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2286] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2287] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1847, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2288] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1847, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2290] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2291] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2292] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2293] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2294] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2295] StringParser("INFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2296] StringParser("INFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2297] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2298] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2299] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2300] StringParser("INITIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2301] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2302] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2303] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2304] StringParser("INPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2305] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2306] StringParser("INPUTFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2307] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1849, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2308] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1849, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2309] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2310] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2311] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2312] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2313] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2314] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2315] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2316] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2317] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1859, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2318] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2319] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2320] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 9, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2321] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2322] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2323] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2324] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2325] Ref(DirectoryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2326] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2327] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2328] Ref(DataSourceFormatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1869, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2329] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2330] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1871, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2331] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2332] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1872, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2333] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1872, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2334] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1876, child_count: 1, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2335] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2336] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2337] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2339] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2340] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1879, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2341] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2342] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2343] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2344] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2345] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2346] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2347] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2348] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2349] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2350] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2351] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2352] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2353] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2354] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2355] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2356] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2357] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1896, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2358] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1899, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2359] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1902, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2360] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1902, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2361] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2362] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1904, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2364] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2365] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2366] Ref(InsertSourceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1906, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2367] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2368] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2369] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2370] Ref(InsertSourceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2371] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2372] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2373] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2374] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2375] Ref(InsertSourceGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2376] StringParser("INSERT_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2377] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2378] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2379] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2380] StringParser("INT1")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2381] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2382] StringParser("INT3")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2383] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2384] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2385] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2386] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2387] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2388] StringParser("INTEGRATIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2389] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1913, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2390] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1913, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2391] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2392] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1915, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2393] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1917, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2394] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2395] Ref(IntervalLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2396] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1918, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2397] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1918, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2398] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1923, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2399] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1923, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2400] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2401] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2402] Ref(SignedQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2403] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2404] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2405] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2406] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1925, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2407] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1925, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2408] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2409] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2410] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2411] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2412] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2413] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1930, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2414] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2415] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2416] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2417] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2418] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2419] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2420] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2421] StringParser("ISAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2422] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2423] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2424] StringParser("ITEMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2425] StringParser("ITERATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2426] MultiStringParser(2 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2427] StringParser("JDBC")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2428] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2429] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2430] Ref(JoinTypeKeywords)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2431] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2432] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1944, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2433] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1944, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2434] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2435] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1946, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2436] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2437] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1950, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2438] Meta(conditional: indented_using_on=False -> indent)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2439] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1953, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2440] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2441] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2442] Meta(conditional: indented_using_on=False -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1955, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2443] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1955, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2444] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2445] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2446] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2447] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2448] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2449] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1961, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2450] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1961, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1963, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2452] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2453] Ref(PivotClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2454] Ref(UnpivotClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1966, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2455] Ref(LateralViewClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 1966, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2456] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2457] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2458] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2459] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1969, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2460] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2461] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1973, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2462] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1973, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2463] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1975, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2464] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2465] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2466] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2467] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2468] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2469] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1985, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2471] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2472] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2473] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2474] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1988, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2475] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1988, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2477] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2478] Ref(SemiKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1990, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2479] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2480] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2481] Ref(AntiKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2482] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2483] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1994, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2485] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2486] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2487] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2488] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1999, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2489] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1999, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2490] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2003, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2491] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2003, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2492] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2006, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2493] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2494] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2495] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2496] StringParser("JSONFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2497] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2498] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2499] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2500] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2501] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2502] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2503] StringParser("KILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2504] StringParser("LANCOMPILER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2505] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2506] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2507] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2008, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2508] StringParser("LAST_INSERT_ID")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2008, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2510] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2511] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2010, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2512] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2010, child_count: 8, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2513] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2514] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2515] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2516] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2517] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2518] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2519] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2020, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2520] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2022, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2521] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2022, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2024, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2523] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2024, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2524] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2525] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2026, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2028, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2527] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2028, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2528] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2529] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2530] StringParser("LAZY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2531] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2532] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2533] StringParser("LEAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2534] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2535] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2536] StringParser("LESS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2030, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2537] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2538] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2539] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2540] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2032, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2541] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2032, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2542] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2037, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2544] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2545] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2037, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2546] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2037, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2547] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2548] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2549] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2550] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2042, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2551] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2044, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2552] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2553] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2046, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2554] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2046, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2555] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2556] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2557] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2558] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2559] Ref(RlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2560] Ref(RegexpKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2561] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2562] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2563] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2564] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2053, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2565] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2566] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2567] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2568] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2569] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2570] StringParser("LINENO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2571] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2572] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2573] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2574] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2059, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2575] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2059, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2576] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2577] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2578] Ref(FileLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2061, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2579] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2580] Ref(ListKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2581] Ref(JarKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2064, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2582] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2583] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2584] Ref(FileLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2585] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2066, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2586] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2066, child_count: 11, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2587] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2588] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2589] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2590] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2591] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2592] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2593] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2594] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2595] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2596] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2597] Ref(RawQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2598] Ref(BytesQuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2599] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2600] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2601] StringParser("LIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2077, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2602] StringParser("LN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2077, child_count: 10, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2603] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2604] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2605] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2606] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2607] Ref(InpathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2608] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2609] Ref(OverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2610] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2611] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2612] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2613] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2614] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2615] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2616] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2617] StringParser("LOCALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2618] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2087, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2619] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2087, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2620] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2621] Ref(LocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2622] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2623] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2624] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2625] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2626] StringParser("LOCKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2627] StringParser("LOGICAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2628] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2629] StringParser("LOGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2630] StringParser("LONG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2631] StringParser("LONGBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2632] StringParser("LONGTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2633] StringParser("LOOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2634] StringParser("LOW_PRIORITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2635] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2636] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2089, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2637] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2638] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2639] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2640] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2641] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2097, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2643] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2099, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2644] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2099, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2645] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2646] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2101, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2647] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2648] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2649] StringParser("MACRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2650] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2651] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2652] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2653] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2654] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2655] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2656] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2657] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2658] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2659] StringParser("MAX_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2660] StringParser("MAXEXTENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2661] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2662] StringParser("MEDIUMBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2663] StringParser("MEDIUMINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2664] StringParser("MEDIUMTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2665] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2666] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2667] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2668] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2669] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2670] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2671] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2672] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2673] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2112, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2674] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2675] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2676] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2677] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2114, child_count: 3, min_times: 1, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2678] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2679] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2680] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2681] Ref(MergeNotMatchedBySourceClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2117, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2682] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2683] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2124, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2684] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2124, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2685] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2686] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2687] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2688] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2689] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2128, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2690] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2128, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2691] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2128, child_count: 10, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2692] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2693] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2694] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2695] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2696] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2138, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2697] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2138, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2698] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2699] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2700] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2701] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2140, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2703] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2704] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 9, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2705] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2706] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2707] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2151, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2708] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2151, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2709] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2710] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2153, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2711] Ref(TargetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2153, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2713] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2714] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2715] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2716] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2155, child_count: 12, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2717] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2718] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2719] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2720] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2721] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2722] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2169, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2723] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2724] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2725] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2726] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2727] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2728] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2178, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2729] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2178, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2730] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2731] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2732] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2733] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2734] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2735] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2736] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2180, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2737] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2738] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2182, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2739] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2184, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2740] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2741] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2742] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2743] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2744] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2745] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2746] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2747] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2748] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2749] StringParser("MIDDLEINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2750] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2751] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2752] StringParser("MIN_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2753] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2754] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2755] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2756] StringParser("MINUTE_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2757] StringParser("MINUTE_SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2758] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2759] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2760] StringParser("MLSLABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2761] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2762] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2763] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2764] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2765] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2766] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2767] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2768] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2769] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2770] StringParser("MONTHNAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2771] StringParser("MONTHS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2772] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2773] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2189, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2774] StringParser("MSCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2189, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2775] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2776] Ref(MsckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2777] Ref(RepairKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2778] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2194, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2779] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2194, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2780] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2196, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2781] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2782] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2783] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2784] Ref(SyncKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2785] Ref(PartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2199, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2786] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2206, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2787] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2788] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2789] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2790] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2791] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2792] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2793] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2794] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2795] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2219, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2796] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2223, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2797] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2798] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2799] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2800] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2801] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2802] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2803] StringParser("MYISAM")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2804] RegexParser
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2805] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2225, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2806] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2225, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2807] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2808] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2809] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2228, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2810] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2811] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2230, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2812] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2813] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2233, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2814] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2815] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2237, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2816] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2817] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2818] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2819] StringParser("NAMESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2820] StringParser("NAMESPACES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2821] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2822] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2239, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2823] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2239, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2824] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2825] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2826] Ref(JoinTypeKeywords)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2827] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2828] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2829] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2830] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2831] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2832] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2833] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2834] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2835] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2836] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2837] StringParser("NO_WRITE_TO_BINLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2838] StringParser("NOAUDIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2839] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2840] StringParser("NOCHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2841] StringParser("NOCOMPRESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2842] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2843] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2844] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2845] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2846] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2847] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 6, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2848] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2849] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2850] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2247, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2851] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2852] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2250, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2853] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2253, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2854] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2253, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2855] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2856] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2857] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2256, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2858] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2256, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2859] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2860] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2861] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2862] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2863] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2864] Ref(InsertOverwriteDirectorySegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2261, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2865] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2264, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2866] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2264, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2867] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2266, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2868] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2869] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2870] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2871] StringParser("NONCLUSTERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2872] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2873] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2874] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2875] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2876] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2877] StringParser("NOSCAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2878] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2879] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2880] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2271, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2882] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2273, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2883] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2273, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2884] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2885] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2886] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2887] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2275, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2888] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2275, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2889] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2890] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2891] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2892] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2893] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2894] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2895] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2896] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2897] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2898] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2899] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2900] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2901] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2902] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2903] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2904] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2905] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2906] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2907] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2908] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2280, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2909] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2283, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2910] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2911] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2912] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2913] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2914] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2287, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2915] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2916] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2917] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2289, child_count: 11, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2918] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2919] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2920] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2921] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2922] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2923] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2924] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2925] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2926] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2927] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2928] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2929] StringParser("OBJECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2930] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2931] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2932] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2933] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2934] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2935] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2936] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2303, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2937] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2305, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2938] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2305, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2939] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2940] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2941] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2942] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2943] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2944] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2945] StringParser("OFFSETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2946] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2947] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2948] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2949] StringParser("ONLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2950] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2951] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2952] StringParser("OPENDATASOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2953] StringParser("OPENQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2954] StringParser("OPENROWSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2955] StringParser("OPENXML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2956] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2957] StringParser("OPERATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2958] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2959] StringParser("OPTIMIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2960] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2961] StringParser("OPTIONALLY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2962] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2963] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2964] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2965] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2966] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2967] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2968] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2969] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2970] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2312, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2971] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2972] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2973] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2314, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2974] StringParser("ORC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2314, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2975] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2976] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2977] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 2, _padding: 0 }, // [2978] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2979] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2325, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2980] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2981] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2982] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2328, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2983] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2328, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2984] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2985] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2330, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2986] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2330, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2987] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2988] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2989] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2990] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2991] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2992] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2993] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [2994] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2334, child_count: 7, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2995] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2996] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2997] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2998] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [2999] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3000] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3001] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3002] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3003] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3004] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3005] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3006] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3007] StringParser("ORDERED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3008] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3009] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3010] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3011] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3012] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3013] StringParser("OUTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3014] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3015] StringParser("OUTPUTFORMAT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3016] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3017] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3018] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2348, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3019] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3020] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3021] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3022] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2353, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3023] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2353, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3024] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3025] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2355, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3026] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2357, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3027] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2358, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3028] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3029] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3030] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3031] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3032] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3033] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3034] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3035] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3036] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3037] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3038] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3039] StringParser("PACK_KEYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3040] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3041] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3042] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3043] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3044] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3045] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3046] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3047] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3048] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3049] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3050] StringParser("PARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3051] StringParser("PARQUET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2363, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3052] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2363, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3053] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3054] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3055] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2368, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3056] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2370, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3057] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2373, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3058] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3059] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2375, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3060] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3061] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3062] Ref(FieldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2380, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3063] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3064] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3065] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2385, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3066] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3067] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3068] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3069] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3070] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3071] Ref(IcebergTransformationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2389, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2391, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3073] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2391, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3074] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2393, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3075] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3076] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3077] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2400, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3078] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2400, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3079] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2400, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3080] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2400, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3081] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2400, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3082] Ref(IcebergTransformationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2400, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3083] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3084] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3085] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2402, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3086] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2402, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3088] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3089] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3090] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3091] Ref(PartitionedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3092] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2408, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3093] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2411, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3094] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2413, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3095] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2416, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3096] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2416, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3097] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3098] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3099] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3100] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3101] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3102] Ref(IcebergTransformationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3103] StringParser("PARTITIONED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3104] StringParser("PARTITIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3105] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3106] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2420, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3107] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2420, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3108] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2422, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2424, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3110] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2424, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3111] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3112] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3113] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3114] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3115] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3116] StringParser("PCTFREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3117] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3118] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3119] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3120] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3121] StringParser("PERCENTLIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3122] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2426, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3123] StringParser("|")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3124] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3125] Ref(PivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2430, child_count: 9, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3126] Bracketed(7 elements -> 7 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3127] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3128] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3129] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3130] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3131] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2443, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3133] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2448, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3134] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2450, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3135] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2450, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3136] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3137] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3138] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3139] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2455, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3140] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2457, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3141] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2459, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3142] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2461, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3143] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2464, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3144] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2466, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3145] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2466, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3146] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3147] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3148] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3149] StringParser("PIVOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3150] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3151] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3152] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3153] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3154] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3155] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2468, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3156] StringParser("+")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2468, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3157] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3158] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3159] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3160] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3161] StringParser("POSTFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3162] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3163] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3164] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3165] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3166] StringParser("PREFIX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3167] StringParser("PREORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3168] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3169] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2470, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3170] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2470, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3171] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3172] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3173] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2472, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3174] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2472, child_count: 20, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3175] OneOf(20 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3176] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3177] Ref(TinyintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3178] Ref(LongKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3179] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3180] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3181] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3182] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3183] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3184] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3185] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3186] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3187] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3188] Ref(Timestamp_ltzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3189] Ref(Timestamp_ntzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3190] Ref(StringKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2492, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2494, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3192] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3193] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3194] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3195] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3196] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2497, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3197] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3198] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3199] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3200] Ref(DecKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3201] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3202] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3203] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3204] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3205] Ref(VariantKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3206] StringParser("PRINCIPALS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3207] StringParser("PRINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3208] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3209] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3210] StringParser("PROC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3211] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3212] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3213] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3214] StringParser("PROCESS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3215] StringParser("PROCESSLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3216] StringParser("PROPERTIES")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2502, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3217] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2502, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3218] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3219] Ref(PropertyNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2505, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3220] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2505, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3221] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3222] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2508, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3223] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3224] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3225] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3226] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3227] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3228] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3229] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2517, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3230] Ref(PropertyGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2517, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3231] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3232] Ref(PropertyNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2519, child_count: 1, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3233] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2520, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3234] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2522, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3235] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3236] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3237] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3238] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3239] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2524, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3240] StringParser("PURGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2524, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3241] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3242] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3243] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2526, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3244] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2530, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3245] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2530, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3246] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3247] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3248] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2535, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3249] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3250] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3251] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2538, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3252] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3253] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3254] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3255] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2541, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3256] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3257] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3258] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3259] StringParser("QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3260] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2544, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3261] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2544, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3262] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3263] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3264] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3265] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3266] StringParser("RAID0")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3267] StringParser("RAISERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3268] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3269] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3270] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3271] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3272] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3273] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2546, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3274] StringParser("!")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2546, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3275] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3276] TypedParser("raw_single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3277] TypedParser("raw_double_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3278] StringParser("RCFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3279] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3280] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3281] StringParser("READTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3282] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3283] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3284] StringParser("RECONFIGURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3285] StringParser("RECORDREADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3286] StringParser("RECORDWRITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3287] StringParser("RECOVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3288] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3289] StringParser("REDUCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2548, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3290] StringParser("REF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2548, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3291] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3292] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2553, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3293] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2553, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3294] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2553, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3295] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2553, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3296] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2555, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3297] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3298] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3299] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3300] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3301] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3302] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3303] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3304] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2561, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3305] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3306] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2563, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3307] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3308] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3309] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3310] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3311] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3312] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3313] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3314] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3315] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3316] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3317] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3318] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3319] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2573, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3320] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2575, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3321] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2575, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3322] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2575, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3323] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3324] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3325] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2577, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3326] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2577, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3327] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3328] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2579, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3329] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3330] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2582, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3332] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3333] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2584, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3335] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3336] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3337] StringParser("REGEXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3338] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3339] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3340] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3341] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3342] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3343] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3344] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3345] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3346] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3347] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3348] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3349] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3350] StringParser("RELOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2586, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3351] StringParser("REMOVE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2586, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3352] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3353] Ref(RemoveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3354] Ref(WidgetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3355] Ref(WidgetNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3356] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3357] StringParser("REPAIR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3358] StringParser("REPEAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3359] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2589, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3360] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2589, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3361] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3362] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3363] Ref(TableDefinitionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3364] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3365] StringParser("REQUIRE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2591, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3366] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2591, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3367] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2593, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3368] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2593, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3369] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3370] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3371] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2595, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3372] StringParser("RESIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3373] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3374] Ref(JarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3375] Ref(WhlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3376] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2598, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3377] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3378] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3379] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3380] Ref(ResourceFileGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3381] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3382] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3383] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2601, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3384] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2601, child_count: 5, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3385] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3386] Ref(RestoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3387] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2606, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3388] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3389] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3390] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3391] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3392] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3393] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3394] Ref(TimestampAsOfGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3395] Ref(VersionAsOfGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3396] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3397] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3398] StringParser("RETAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3399] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3400] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3401] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3402] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3403] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3404] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2611, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3405] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2611, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3406] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2617, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3407] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2617, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3408] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3409] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3410] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3411] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3412] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2624, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3413] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3414] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3415] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3416] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3418] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3419] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2629, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3420] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3421] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3422] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3423] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3424] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3425] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3426] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3427] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3428] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3429] StringParser("->")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3430] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3431] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3432] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3433] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3434] StringParser("ROLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3435] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3436] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3437] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3438] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3439] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3440] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3441] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3442] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3443] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3444] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3445] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3446] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2638, child_count: 6, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3447] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2644, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3448] Ref(DelimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2644, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3449] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2647, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3450] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2647, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3451] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2647, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3452] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3453] Ref(EscapedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3454] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2650, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3455] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2650, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3456] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3457] Ref(CollectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3458] Ref(ItemsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2653, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3459] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2653, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3460] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3461] Ref(MapKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3462] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2656, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3463] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2656, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3465] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2658, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3466] Ref(TerminatedByGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2658, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3467] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3468] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3469] Ref(DefinedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3470] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3471] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2662, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3472] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3473] Ref(SerdeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3474] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3475] Ref(SerdePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3476] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3477] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3478] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3479] StringParser("ROWCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3480] StringParser("ROWGUIDCOL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3481] StringParser("ROWID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3482] StringParser("ROWNUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3483] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3484] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3485] StringParser("RUN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2665, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3486] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2667, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3487] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2667, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3488] StringParser("V")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2667, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2669, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3490] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2669, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3491] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2671, child_count: 4, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3492] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3493] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2675, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3494] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3495] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3496] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2677, child_count: 7, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3497] Bracketed(5 elements -> 5 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3498] Ref(BucketKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3499] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3500] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3501] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3502] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3503] StringParser("SAVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3504] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3505] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3506] StringParser("SCD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3507] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3508] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3509] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3510] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3511] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3512] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3513] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3514] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3515] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3516] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3517] StringParser("SECOND_MICROSECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3518] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2684, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3519] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2684, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3520] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3521] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3522] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3523] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2688, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3524] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2688, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3525] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2690, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3526] Ref(SelectHintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2690, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3527] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3528] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3529] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2692, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 1, _padding: 0 }, // [3530] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3531] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2695, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3532] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2697, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3533] Ref(TransformClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2697, child_count: 3, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3534] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2700, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3535] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 2700, child_count: 2, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3536] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3537] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 0, min_times: 0, first_terminator_idx: 49, terminator_count: 0, _padding: 0 }, // [3538] Ref(SelectClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2702, child_count: 11, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3539] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2713, child_count: 1, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3540] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3541] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3542] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2714, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3543] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3544] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3545] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3546] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3547] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3548] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3549] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2716, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3550] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2718, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3551] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2718, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3552] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2718, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3553] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3554] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3555] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3556] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3557] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3558] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3559] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2722, child_count: 1, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3560] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2723, child_count: 3, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3561] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3562] Ref(StartHintSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2726, child_count: 2, min_times: 0, first_terminator_idx: 50, terminator_count: 1, _padding: 0 }, // [3563] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2728, child_count: 1, min_times: 1, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3564] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3565] Ref(HintFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 50, terminator_count: 0, _padding: 0 }, // [3566] Ref(EndHintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3567] Ref(EndHintSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2729, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3568] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 2729, child_count: 16, min_times: 0, first_terminator_idx: 51, terminator_count: 3, _padding: 0 }, // [3569] Sequence(16 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3570] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3571] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3572] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3573] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3574] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3575] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3576] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3577] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3578] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3579] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3580] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3581] Ref(ClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3582] Ref(DistributeByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3583] Ref(SortByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3584] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3585] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3586] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3587] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 51, terminator_count: 0, _padding: 0 }, // [3588] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3589] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2749, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2751, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3591] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2754, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3592] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2754, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3593] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2756, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3594] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2759, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3595] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2759, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3596] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2759, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3597] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3598] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3599] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2762, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3600] StringParser("SEMI")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2762, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3601] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2766, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3602] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3603] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3604] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2768, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3605] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2770, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3606] Ref(NakedSemiStructuredElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2770, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3607] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3608] Ref(QuotedSemiStructuredElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3609] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2773, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3610] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2775, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3611] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2777, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3612] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3613] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3614] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2779, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3615] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3616] Ref(NakedSemiStructuredElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2781, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3617] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3618] Ref(QuotedSemiStructuredElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3619] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3620] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3621] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3622] StringParser("SEPARATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3623] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2784, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3624] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2784, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3625] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2786, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3627] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3628] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2788, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2790, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3630] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2790, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3631] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2790, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3632] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2792, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3633] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3634] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3635] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2794, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3637] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3638] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3639] StringParser("SEQUENCEFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3640] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3641] StringParser("SERDE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3642] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3643] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3644] Ref(SerdepropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3645] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3646] StringParser("SERDEPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3647] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3648] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3649] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3650] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3651] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3652] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3653] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2803, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3654] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3655] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3656] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3657] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3658] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2808, child_count: 7, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3659] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3660] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3661] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3662] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3663] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3664] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3665] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3666] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 6, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3667] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2821, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3668] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2821, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3669] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3670] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3671] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3672] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3673] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 2, min_times: 1, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3674] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3675] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3676] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(2), first_child_idx: 2828, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 1, _padding: 0 }, // [3677] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [3678] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2828, child_count: 5, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3679] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3680] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 1, min_times: 1, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3681] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2834, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3682] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3683] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3684] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3685] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3686] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3687] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2836, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3688] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 2836, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3689] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2839, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3691] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3692] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3693] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3694] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3695] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2845, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3696] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3697] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2847, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3698] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2847, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3699] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2849, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3700] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2849, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3701] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2849, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3702] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3703] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3704] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3705] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3706] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2858, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3707] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2858, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3708] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2858, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3709] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2858, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3710] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2858, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3711] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3712] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3713] Ref(SQLConfPropertiesSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2861, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3714] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2863, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3715] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3716] Ref(PropertyNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3717] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3718] Ref(SetConfigValueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3719] Ref(PropertyNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2866, child_count: 5, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3720] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2871, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3721] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2871, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3722] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2873, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3723] Ref(VarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2873, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3724] Ref(VariableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2873, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3725] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2875, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3726] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3727] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3728] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3729] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3730] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3731] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3732] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2884, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3733] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3734] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3735] StringParser("SETMINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3736] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3737] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3738] StringParser("SETUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3739] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2887, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3740] StringParser("SHARES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2887, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2889, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3742] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3743] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3744] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2891, child_count: 1, min_times: 1, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3745] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2892, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3746] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3747] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3748] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2895, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3749] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2895, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3750] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2897, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3751] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3752] Ref(DatabasesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2899, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3753] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2899, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3754] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3755] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3756] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3757] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2904, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3758] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3759] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3760] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3761] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3762] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2907, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3763] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 2910, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3764] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3765] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3766] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3767] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2913, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3768] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2913, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3769] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2915, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3770] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2915, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3771] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2915, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3772] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2915, child_count: 9, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3773] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2924, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3774] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3775] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3776] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3777] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2928, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3778] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3779] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3780] Ref(SerdeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3781] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3782] Ref(ColumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3783] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2934, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3784] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2934, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3786] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3787] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2936, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3788] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2939, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3789] Ref(PartitionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2939, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3790] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2939, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3791] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2939, child_count: 6, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3792] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3793] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3794] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2945, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3795] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2947, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3796] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2949, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3797] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2949, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3798] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2949, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3799] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2949, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3800] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2949, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3801] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2949, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3802] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2949, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3803] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3804] Ref(TblpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3805] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3806] Ref(BracketedPropertyNameListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3807] Ref(ShowDatabasesSchemasGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3808] Ref(ShowFunctionsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3809] Ref(ShowTablesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3810] Ref(ShowViewsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3811] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3812] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3813] Ref(ShowObjectGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3814] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2957, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3815] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2957, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3816] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3817] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3818] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3819] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3820] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2961, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3821] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3822] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3823] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3824] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2966, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3825] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2966, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3826] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2968, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3827] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2970, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3828] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2970, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3829] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2970, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3830] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2970, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3831] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2972, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3832] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2972, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3833] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2972, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3834] StringParser("SHUTDOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2972, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3835] StringParser("SIGNAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2972, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3836] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2974, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3837] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2974, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3838] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2974, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3839] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3840] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3841] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3842] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2976, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3843] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2976, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3844] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3845] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3846] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3847] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3848] Ref(BackQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2980, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3849] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3850] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2982, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3851] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3852] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3853] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3854] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2985, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3855] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2985, child_count: 6, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3856] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3857] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3858] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3859] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3860] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 4, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3861] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2995, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3862] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2997, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3864] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2999, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3865] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2999, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3866] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3867] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3868] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3869] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 6, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3870] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3871] Ref(SkewedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3872] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3873] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3874] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3007, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3875] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3010, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3876] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3012, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3877] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3014, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3878] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3014, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3879] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3017, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3880] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3019, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3881] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3019, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3882] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3883] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3884] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3885] Ref(DirectoriesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3886] StringParser("SKEWED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3887] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3888] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3889] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3890] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3891] StringParser("SONAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 5, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3892] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3893] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3894] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 6, _padding: 0 }, // [3895] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3896] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 3, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3897] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3898] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3899] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3035, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3900] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3035, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3901] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3902] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3037, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3903] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3037, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3904] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3905] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 2, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3906] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3907] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3908] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3909] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3910] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3911] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3912] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3913] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 55, terminator_count: 0, _padding: 0 }, // [3914] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3041, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3915] StringParser("SORT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3041, child_count: 3, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3916] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3917] Ref(SortedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3918] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3044, child_count: 3, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3919] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 2, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3920] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3049, child_count: 2, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3921] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3051, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3922] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3051, child_count: 2, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3923] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3924] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3925] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3926] StringParser("SORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3927] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3928] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3929] StringParser("SPATIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3930] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3931] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3932] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3933] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3934] StringParser("SQL_BIG_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3935] StringParser("SQL_BIG_SELECTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3936] StringParser("SQL_BIG_TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3937] StringParser("SQL_CALC_FOUND_ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3938] StringParser("SQL_LOG_OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3939] StringParser("SQL_LOG_UPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3940] StringParser("SQL_LOW_PRIORITY_UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3941] StringParser("SQL_SELECT_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3942] StringParser("SQL_SMALL_RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3943] StringParser("SQL_WARNINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3944] StringParser("SQLCA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3945] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3946] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3947] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3948] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3949] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3950] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3951] StringParser("SSL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3952] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3953] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3954] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3955] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3956] StringParser("/*+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3957] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3958] StringParser("STARTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3959] StringParser("STARTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3960] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3053, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3961] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3053, child_count: 69, min_times: 0, first_terminator_idx: 61, terminator_count: 1, _padding: 0 }, // [3962] OneOf(69 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3963] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3964] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3965] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3966] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3967] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3968] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3969] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3970] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3971] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3972] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3973] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3974] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3975] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3976] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3977] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3978] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3979] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3980] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3981] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3982] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3983] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3984] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3985] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3986] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3987] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3988] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3989] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3990] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3991] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3992] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3993] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3994] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3995] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3996] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3997] Ref(FromInsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3998] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [3999] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4000] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4001] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4002] Ref(MsckRepairTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4003] Ref(UseDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4004] Ref(AddFileSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4005] Ref(AddJarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4006] Ref(AnalyzeTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4007] Ref(CacheTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4008] Ref(ClearCacheSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4009] Ref(ListFileSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4010] Ref(ListJarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4011] Ref(RefreshStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4012] Ref(ResetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4013] Ref(SetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4014] Ref(ShowStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4015] Ref(UncacheTableSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4016] Ref(InsertOverwriteDirectorySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4017] Ref(InsertOverwriteDirectoryHiveFmtSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4018] Ref(LoadDataSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4019] Ref(ClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4020] Ref(DistributeByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4021] Ref(VacuumStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4022] Ref(DescribeHistoryStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4023] Ref(DescribeDetailStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4024] Ref(GenerateManifestFileStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4025] Ref(ConvertToDeltaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4026] Ref(RestoreTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4027] Ref(ConstraintStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4028] Ref(CreateWidgetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4029] Ref(RemoveWidgetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4030] Ref(ReplaceTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4031] Ref(SetVariableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 61, terminator_count: 0, _padding: 0 }, // [4032] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4033] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4034] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4035] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4036] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4037] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3122, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4038] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3122, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3124, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4040] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3126, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4041] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3126, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4042] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3126, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4043] Ref(StoredByGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3126, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4044] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3126, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4045] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3129, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4046] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3129, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4047] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3129, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4048] Ref(FileFormatGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3129, child_count: 4, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4049] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4050] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4051] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4052] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4053] Ref(SerdePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4054] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4055] StringParser("STRAIGHT_JOIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4056] StringParser("STRATIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4057] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4058] StringParser("STREAMING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4059] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3133, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4060] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 1, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4061] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3134, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4062] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3134, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4063] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3134, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4064] StringParser("STRUCT")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3134, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4065] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4066] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3139, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4068] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3141, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4069] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3141, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4070] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3144, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4071] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3146, child_count: 4, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4072] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4073] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4074] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4075] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4076] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4077] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4078] Ref(StructKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4079] Ref(StructTypeSchemaSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4080] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4081] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4082] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4083] StringParser("SUBLIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4084] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4085] StringParser("SUBSTR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4086] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4087] StringParser("SUCCESSFUL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4088] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4089] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4090] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4091] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4092] StringParser("SYNC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4093] StringParser("SYNONYM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4094] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4095] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4096] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3152, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4097] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3152, child_count: 5, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4098] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3157, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4099] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3157, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4100] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3157, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4101] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4102] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4103] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3161, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4105] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3163, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4106] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3163, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4107] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3163, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4108] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4110] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4111] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3168, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4112] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3170, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4113] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3170, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4114] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3170, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4115] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4116] Ref(ForeignKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4117] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4118] Ref(ReferenceDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 12, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4119] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3185, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4120] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4121] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4122] Ref(OrRefreshGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4123] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4124] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4125] Ref(StreamingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4126] Ref(LiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4127] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3187, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4128] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3187, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4129] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4130] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3189, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4131] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3189, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4132] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3191, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4133] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3194, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4134] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3196, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4135] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3198, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4136] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3200, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4137] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4138] Ref(ColumnFieldDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4139] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4140] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3202, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4141] Ref(ConstraintStatementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3202, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4142] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4143] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3204, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4144] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4145] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4146] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3206, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4147] Ref(UsingClauseSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3206, child_count: 10, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4148] AnySetOf(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4149] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4150] Ref(StoredAsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4151] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4152] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4153] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4154] Ref(BucketSpecGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4155] Ref(LocationGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4156] Ref(CommentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4157] Ref(TablePropertiesGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3216, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4158] Ref(TableClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3216, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4159] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3218, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4160] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3218, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4161] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3220, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4162] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4163] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3223, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4164] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3223, child_count: 5, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4165] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4166] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4167] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4168] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4169] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3230, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4170] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4171] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3232, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4172] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3232, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4173] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4174] Ref(AtSignLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3234, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4175] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4177] Ref(TimestampAsOfGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4178] Ref(VersionAsOfGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 3, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4179] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4180] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4181] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 2, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4182] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3244, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4183] Ref(TblpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3244, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4184] Ref(BracketedPropertyListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3244, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4185] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3244, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4186] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3244, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4187] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3244, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4188] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3244, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4189] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3246, child_count: 1, min_times: 0, first_terminator_idx: 62, terminator_count: 1, _padding: 0 }, // [4190] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4191] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 62, terminator_count: 0, _padding: 0 }, // [4192] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4193] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3247, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4194] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3249, child_count: 1, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4195] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4196] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4197] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4198] StringParser("TARGET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4199] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4200] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4201] StringParser("TBLPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4202] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4203] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3250, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4204] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3250, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3252, child_count: 1, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4206] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3253, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4207] Ref(GlobalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3253, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4208] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4209] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4210] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3255, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4211] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3255, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4212] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4213] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4214] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3257, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4215] StringParser("TERMINATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3257, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4216] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4217] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4218] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4219] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4220] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4221] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4222] StringParser("TEXTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4223] StringParser("TEXTSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4224] StringParser("THAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4225] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4226] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4227] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3260, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4228] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3260, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4229] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3263, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4230] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4231] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4232] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3265, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4233] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3265, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4234] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3268, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4235] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4236] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4237] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4238] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4239] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3270, child_count: 1, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4240] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3271, child_count: 4, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4241] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4242] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4243] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4244] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4245] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 4, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4246] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4247] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4248] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4249] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4250] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4251] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4252] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4253] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4254] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4255] StringParser("TIMESTAMP_LTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4256] StringParser("TIMESTAMP_NTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4257] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4258] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4259] StringParser("TINYBLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4260] StringParser("TINYINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4261] StringParser("TINYTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4262] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4263] StringParser("TOAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4264] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4265] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4266] StringParser("TOUCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4267] StringParser("TRACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4268] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4269] StringParser("TRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3282, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4270] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 4, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4271] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3286, child_count: 5, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4272] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4273] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4274] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4275] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4276] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3291, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4277] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3291, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4278] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3293, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4279] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3293, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4280] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3293, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4281] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4282] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3295, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4283] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3295, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4284] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4285] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4286] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4287] Ref(ChainKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4288] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4289] StringParser("TRANSACTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4290] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3298, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4291] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3298, child_count: 8, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4292] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3306, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4293] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3306, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4294] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3309, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4295] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4296] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4297] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4298] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3311, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4299] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3311, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4300] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4301] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4302] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3316, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4303] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3318, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4304] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4305] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4306] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4307] Ref(RowFormatClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4308] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4309] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4310] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4311] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4312] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4313] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4314] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4315] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4316] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4317] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4318] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4319] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4320] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4321] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4322] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4323] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4324] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3323, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4325] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3323, child_count: 4, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4326] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4327] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4328] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4329] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4330] Ref(PartitionSpecGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4331] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4332] StringParser("TRY_CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3327, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4333] StringParser("TSEQUAL")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3327, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4334] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4335] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4336] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4337] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4338] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4339] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4340] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3334, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4341] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4342] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4343] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4344] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4345] StringParser("UID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4346] StringParser("UNARCHIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4347] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3336, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4348] StringParser("UNCACHE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3336, child_count: 4, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4349] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4350] Ref(UncacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4351] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4352] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4353] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4354] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4355] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 3, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4356] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4357] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4358] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4359] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4360] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4361] StringParser("UNDO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3343, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4362] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3343, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4363] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3345, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4364] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3345, child_count: 2, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4365] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3347, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4366] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3347, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4367] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3347, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4368] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3347, child_count: 1, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4369] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4370] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4371] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4372] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4373] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4374] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4375] StringParser("UNLOCK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4376] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3348, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4377] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 3348, child_count: 10, min_times: 0, first_terminator_idx: 63, terminator_count: 5, _padding: 0 }, // [4378] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4379] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4380] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4381] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4382] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4383] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4384] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4385] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4386] Ref(ClusterByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4387] Ref(DistributeByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4388] Ref(SortByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4389] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4390] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4391] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4392] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 0, min_times: 0, first_terminator_idx: 63, terminator_count: 0, _padding: 0 }, // [4393] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3358, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4394] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3360, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4395] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3360, child_count: 1, min_times: 1, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4396] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3361, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4397] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4398] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4399] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3363, child_count: 6, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4400] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4401] Ref(UnpivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3369, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4402] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4403] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4404] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4405] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4406] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4407] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3376, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4408] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4409] Ref(SingleValueColumnUnpivotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4410] Ref(MultiValueColumnUnpivotSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4411] StringParser("UNPIVOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3378, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4412] StringParser("UNSET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3378, child_count: 4, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4413] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4414] Ref(UnsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4415] Ref(TblpropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4416] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4417] Ref(BracketedPropertyNameListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4418] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4419] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3382, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4420] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3382, child_count: 7, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4421] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4422] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3389, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4423] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4424] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3391, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4425] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 3391, child_count: 1, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4426] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4427] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4428] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4429] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4430] StringParser("UPDATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4431] StringParser("UPDATETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4432] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3392, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4433] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3392, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4435] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4436] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3394, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4437] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4438] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4439] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4440] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4441] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4442] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4443] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4444] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4445] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4446] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4447] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4448] StringParser("USERS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4450] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4451] Ref(DataSourceFormatSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4452] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4453] StringParser("UTC_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4454] StringParser("UTC_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4455] StringParser("UTC_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3398, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4456] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3398, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4457] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3401, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4458] Ref(VacuumKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3401, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4459] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4460] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4461] Ref(FileReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4462] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3404, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4463] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3406, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4464] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4465] Ref(RetainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4466] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4467] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3409, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4469] Ref(DryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4470] Ref(RunKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4471] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4472] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4473] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3411, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4474] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3411, child_count: 5, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4475] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4476] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3416, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4477] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3418, child_count: 4, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4478] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4479] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3425, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4480] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3427, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4481] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3429, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4482] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3429, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4483] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3429, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4484] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3429, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4485] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3429, child_count: 1, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4486] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3430, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4487] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 3430, child_count: 1, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4488] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3431, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4489] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4490] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4491] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4492] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4493] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4494] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4495] StringParser("VAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4496] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4497] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4498] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4499] StringParser("VARCHAR2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4500] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4501] StringParser("VARCHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4502] StringParser("VARIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4503] StringParser("VARIABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4504] StringParser("VARIANT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4505] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4506] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 4, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4507] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4508] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4509] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4510] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4511] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4512] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4513] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4514] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4515] StringParser("VIOLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4516] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4517] StringParser("WAITFOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4518] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4519] StringParser("WAREHOUSES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4520] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3437, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4521] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3437, child_count: 8, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4522] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3445, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4523] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3445, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4524] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4525] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4526] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4527] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4528] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4529] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4530] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4531] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4532] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3448, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4533] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3448, child_count: 4, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4534] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3452, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4535] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3452, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4536] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3454, child_count: 3, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4537] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3457, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4538] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3457, child_count: 9, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4539] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4540] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3466, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4541] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3468, child_count: 5, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4542] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4543] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4544] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4545] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4546] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4547] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4548] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3473, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4549] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3475, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4550] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3475, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4551] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3475, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4552] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4553] Ref(DistributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4554] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4555] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4556] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4557] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4558] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4559] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4560] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4561] StringParser("WHILE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4562] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3477, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4563] StringParser("WHL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3477, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4564] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4565] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4566] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4567] StringParser("WIDGET")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4568] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3479, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4569] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3479, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4570] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3481, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4571] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3481, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4572] Ref(ExceptClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3481, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4573] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3483, child_count: 1, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4574] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3484, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4575] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3486, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4576] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4577] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4578] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4579] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4580] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4581] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4582] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3490, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4583] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3490, child_count: 4, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4584] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 3494, child_count: 1, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4585] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3495, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4586] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4587] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4588] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4589] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4590] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3497, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4591] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3497, child_count: 6, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4592] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4593] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4594] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4595] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 2, min_times: 0, first_terminator_idx: 68, terminator_count: 1, _padding: 0 }, // [4596] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4597] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 68, terminator_count: 0, _padding: 0 }, // [4598] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4599] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4600] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 6, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4601] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4602] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4603] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3511, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4604] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3511, child_count: 2, min_times: 0, first_terminator_idx: 69, terminator_count: 1, _padding: 0 }, // [4605] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4606] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 69, terminator_count: 0, _padding: 0 }, // [4607] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4608] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4609] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3513, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4610] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4611] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3515, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4612] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4613] Ref(CubeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4614] Ref(RollupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3517, child_count: 3, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4615] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3520, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4616] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3520, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4617] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3521, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4618] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3521, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4619] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3521, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4620] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3521, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4621] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3521, child_count: 4, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4622] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4623] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4624] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4625] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4626] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4627] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4628] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4629] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4630] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4631] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4632] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4633] StringParser("WRITETEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4634] StringParser("X509")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4635] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4636] StringParser("XOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4637] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4638] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4639] StringParser("YEAR_MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4640] StringParser("YEARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4641] StringParser("ZEROFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [4642] StringParser("ZONE")
];

pub static CHILD_IDS: &[u32] = &[
    5, 6, 61, 7, 51, 8, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20,
    21, 28, 33, 38, 39, 44, 9, 10, 22, 23, 24, 25, 26, 27, 29, 30,
    31, 32, 34, 35, 36, 37, 40, 41, 42, 43, 45, 46, 47, 48, 49, 50,
    52, 58, 53, 54, 57, 55, 56, 62, 63, 64, 66, 74, 78, 81, 84, 89,
    67, 68, 69, 70, 71, 72, 73, 75, 76, 77, 79, 80, 82, 83, 85, 86,
    87, 88, 90, 133, 91, 100, 105, 108, 109, 110, 111, 112, 113, 114, 115, 116,
    117, 118, 119, 120, 121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 92, 93,
    94, 95, 98, 99, 96, 97, 101, 102, 103, 104, 106, 107, 131, 132, 135, 58,
    138, 139, 140, 141, 142, 143, 144, 145, 146, 147, 148, 151, 154, 157, 149, 150,
    152, 153, 155, 156, 158, 159, 161, 162, 163, 164, 165, 166, 167, 168, 169, 170,
    171, 174, 177, 180, 172, 173, 175, 176, 178, 179, 181, 182, 184, 185, 187, 191,
    195, 199, 203, 205, 207, 188, 189, 190, 58, 192, 193, 194, 58, 196, 197, 198,
    58, 200, 201, 202, 58, 204, 58, 206, 58, 209, 210, 216, 217, 218, 219, 220,
    222, 223, 224, 225, 226, 233, 234, 235, 250, 236, 242, 243, 237, 238, 239, 240,
    241, 244, 245, 246, 247, 248, 249, 253, 254, 259, 260, 263, 264, 265, 261, 262,
    266, 267, 270, 274, 275, 276, 281, 284, 271, 272, 273, 277, 280, 278, 279, 282,
    283, 286, 287, 288, 289, 290, 292, 293, 294, 295, 297, 303, 316, 317, 298, 299,
    300, 301, 302, 304, 307, 308, 309, 305, 306, 310, 315, 311, 314, 312, 313, 318,
    319, 322, 320, 321, 324, 325, 326, 233, 327, 250, 328, 332, 337, 343, 354, 379,
    387, 401, 407, 415, 418, 421, 424, 425, 436, 441, 445, 455, 477, 484, 329, 330,
    331, 333, 334, 335, 336, 338, 339, 340, 341, 342, 344, 345, 233, 348, 250, 346,
    347, 349, 350, 350, 240, 241, 351, 58, 352, 353, 355, 358, 233, 359, 369, 370,
    371, 372, 373, 250, 356, 357, 360, 361, 362, 363, 364, 365, 366, 367, 368, 374,
    377, 378, 375, 376, 380, 381, 382, 383, 240, 241, 384, 58, 385, 386, 388, 389,
    390, 394, 391, 392, 393, 395, 396, 397, 398, 240, 241, 399, 58, 400, 402, 403,
    404, 405, 406, 408, 409, 410, 413, 411, 412, 414, 416, 417, 419, 420, 422, 423,
    426, 427, 428, 429, 432, 430, 431, 433, 434, 435, 437, 438, 439, 440, 442, 443,
    444, 233, 446, 449, 450, 452, 453, 250, 447, 448, 451, 454, 240, 241, 456, 457,
    458, 462, 459, 460, 461, 463, 464, 465, 233, 466, 250, 467, 58, 468, 469, 472,
    470, 471, 473, 474, 475, 476, 478, 479, 480, 233, 481, 250, 482, 58, 483, 485,
    486, 487, 233, 488, 250, 489, 58, 490, 492, 493, 494, 495, 496, 500, 503, 504,
    497, 498, 499, 501, 502, 505, 506, 507, 508, 508, 240, 241, 514, 515, 516, 531,
    517, 518, 519, 520, 521, 522, 523, 524, 525, 526, 527, 528, 529, 529, 240, 241,
    530, 58, 532, 533, 538, 539, 540, 534, 537, 535, 536, 549, 550, 551, 552, 553,
    554, 555, 556, 557, 558, 559, 561, 566, 567, 562, 565, 563, 564, 571, 566, 567,
    572, 58, 574, 575, 576, 577, 578, 580, 595, 605, 606, 607, 608, 609, 610, 611,
    612, 613, 625, 626, 627, 628, 629, 638, 639, 642, 643, 648, 649, 652, 653, 656,
    240, 241, 657, 58, 659, 240, 241, 660, 58, 662, 240, 241, 664, 240, 241, 667,
    240, 241, 673, 674, 675, 676, 677, 682, 683, 685, 240, 241, 687, 688, 689, 690,
    691, 240, 241, 694, 695, 696, 697, 698, 699, 700, 701, 708, 718, 709, 710, 711,
    715, 250, 717, 712, 719, 720, 710, 721, 725, 250, 727, 722, 759, 760, 765, 766,
    233, 767, 250, 768, 58, 769, 770, 771, 772, 780, 781, 782, 792, 793, 794, 795,
    806, 807, 808, 809, 811, 814, 812, 813, 815, 818, 822, 825, 828, 829, 830, 833,
    834, 837, 816, 817, 819, 820, 821, 240, 241, 823, 824, 826, 827, 831, 832, 835,
    836, 839, 840, 841, 843, 842, 240, 241, 844, 846, 847, 848, 850, 849, 240, 241,
    851, 853, 854, 855, 856, 857, 240, 241, 858, 859, 862, 863, 875, 876, 878, 879,
    887, 888, 889, 890, 891, 892, 893, 894, 895, 896, 900, 897, 898, 899, 901, 902,
    903, 904, 913, 914, 920, 921, 922, 928, 929, 930, 931, 933, 936, 932, 240, 241,
    934, 935, 937, 940, 938, 939, 941, 942, 954, 955, 956, 957, 958, 961, 959, 960,
    970, 971, 972, 976, 977, 978, 988, 989, 990, 993, 973, 974, 975, 240, 241, 979,
    980, 981, 982, 983, 987, 984, 985, 986, 991, 992, 994, 995, 997, 998, 1001, 1002,
    1003, 1004, 1005, 999, 1000, 1006, 1007, 1008, 1010, 1011, 1012, 1013, 1014, 1015, 1016, 1017,
    1018, 1020, 1021, 1022, 1023, 1024, 1025, 1026, 1027, 1028, 1029, 1030, 240, 241, 1031, 58,
    1034, 1035, 1036, 1037, 1038, 1039, 1051, 1052, 1040, 1041, 1042, 240, 241, 1043, 58, 1044,
    1045, 1046, 1047, 1048, 1049, 566, 567, 1050, 58, 1054, 1055, 1056, 1057, 1059, 1060, 1061,
    1062, 1064, 1068, 1072, 1073, 1074, 1079, 1082, 1065, 1066, 1067, 1069, 1070, 1071, 1075, 1078,
    1076, 1077, 1080, 1081, 1084, 1085, 1086, 1087, 1088, 1090, 1091, 1093, 1094, 1095, 1096, 1102,
    1115, 1116, 1117, 1154, 1097, 1098, 1099, 1100, 1101, 1103, 1113, 1104, 1105, 1106, 1107, 1108,
    1109, 1110, 58, 1118, 1128, 1131, 1144, 1150, 1119, 1120, 1121, 1122, 1123, 1124, 1125, 1126,
    1127, 1129, 1130, 1132, 1135, 1133, 1134, 1136, 1137, 1138, 1141, 1139, 1140, 1142, 1143, 1145,
    1146, 1147, 1148, 1149, 1151, 1152, 1153, 240, 241, 1155, 1156, 1157, 1158, 1160, 1161, 1162,
    1164, 1165, 1166, 1167, 1168, 1169, 1170, 1171, 1173, 1174, 1177, 1178, 1179, 1180, 1181, 1182,
    1183, 1184, 1193, 1196, 1197, 1200, 1201, 1202, 1203, 1208, 1175, 1176, 1185, 1186, 240, 241,
    1187, 58, 1188, 1192, 1189, 1190, 1191, 1194, 1195, 1198, 1199, 1204, 1205, 1206, 1207, 1207,
    240, 241, 1210, 1211, 1212, 1213, 1220, 1214, 1215, 1216, 1217, 1218, 1219, 1221, 1222, 1223,
    1232, 1235, 1233, 1234, 1236, 240, 241, 1252, 1253, 1254, 1256, 1257, 1258, 1259, 1260, 1261,
    1262, 1263, 1264, 1265, 1268, 1269, 1272, 1273, 1274, 1276, 1277, 1278, 1285, 1279, 1280, 1281,
    577, 578, 1282, 1283, 1284, 1290, 1291, 240, 241, 1292, 58, 1293, 1294, 1296, 1303, 1297,
    1298, 1299, 1300, 1301, 1302, 1326, 1327, 1338, 1339, 1340, 1354, 1355, 1356, 1357, 1358, 1359,
    1361, 1362, 1363, 1367, 1364, 1365, 1366, 1370, 1376, 1380, 1391, 1408, 1371, 1374, 1375, 1372,
    1373, 1377, 1378, 1379, 1381, 1382, 1383, 1384, 1385, 1386, 1387, 1388, 1389, 1390, 1392, 1393,
    1394, 1397, 1407, 1395, 1396, 1398, 1399, 1400, 1401, 1403, 1404, 1405, 1406, 1402, 58, 1409,
    1410, 1412, 1415, 1413, 1414, 1433, 1434, 233, 1435, 250, 1436, 58, 1437, 1438, 1439, 1440,
    1457, 1458, 1460, 1461, 1462, 1466, 1463, 1464, 1465, 240, 241, 1468, 1469, 1472, 1473, 1474,
    1470, 1471, 1476, 1477, 1478, 1479, 1480, 1482, 1483, 1484, 1485, 1486, 1489, 1490, 1491, 1492,
    1494, 1495, 1496, 1497, 1499, 1500, 1501, 1502, 1503, 1505, 1506, 1507, 1509, 1510, 1511, 1512,
    1513, 1515, 1514, 58, 1517, 1518, 1519, 1520, 1522, 1523, 1524, 1525, 1526, 1528, 1529, 1530,
    1531, 1533, 1534, 1535, 1536, 1537, 1548, 710, 1549, 250, 240, 241, 1554, 1555, 1575, 1576,
    1577, 240, 241, 1578, 58, 1595, 1596, 1601, 1597, 1598, 1599, 1600, 1605, 1606, 1607, 1608,
    1609, 1612, 1613, 1614, 1618, 1619, 1620, 1621, 1627, 1610, 1611, 1615, 1616, 1617, 1622, 1623,
    1624, 1625, 1626, 1628, 1629, 1631, 1634, 1635, 1636, 1632, 1633, 1638, 1639, 1640, 1641, 1642,
    1646, 1643, 1644, 1645, 1648, 1651, 1649, 1650, 1653, 1657, 1663, 1654, 1655, 1656, 240, 241,
    1658, 1661, 1659, 1660, 1662, 1666, 1701, 1667, 1668, 1669, 1681, 1682, 1683, 1690, 1698, 1699,
    1670, 240, 241, 1671, 1672, 1673, 1674, 1675, 58, 1676, 1677, 1678, 1679, 1680, 1684, 1687,
    1685, 1686, 1688, 240, 241, 1689, 58, 1691, 1692, 1693, 1694, 1695, 1696, 1697, 1703, 1704,
    1705, 1706, 1707, 1708, 1709, 1719, 1720, 1723, 1727, 1730, 1721, 1722, 1724, 1725, 1726, 1728,
    1729, 1731, 1732, 1733, 1734, 1739, 1740, 1741, 1742, 1743, 1744, 1745, 1746, 1747, 1748, 1751,
    1752, 1753, 1754, 1755, 1758, 1756, 1757, 1760, 1761, 1762, 1764, 1766, 1770, 1765, 1767, 1768,
    1769, 1771, 1775, 1776, 1777, 240, 241, 1778, 1779, 1784, 1785, 1786, 1787, 1788, 1798, 1799,
    1807, 1808, 1809, 1821, 1810, 1813, 1811, 1812, 1814, 1818, 1815, 1816, 1817, 1819, 1820, 1822,
    1809, 1823, 1809, 1825, 1826, 1832, 1833, 1834, 58, 1836, 1837, 1838, 1841, 1844, 1847, 1850,
    1853, 1854, 1855, 1856, 1857, 1858, 1839, 1840, 1842, 1843, 1845, 1846, 1848, 1849, 1851, 1852,
    1860, 1861, 1864, 1865, 1869, 1862, 1863, 1863, 240, 241, 1866, 1867, 1868, 1871, 1872, 1872,
    240, 241, 710, 1873, 250, 1884, 1885, 1896, 1874, 1875, 1876, 1877, 240, 241, 1879, 1880,
    1882, 1883, 1886, 1887, 1888, 1889, 1891, 1892, 1894, 1895, 1898, 1899, 1902, 1903, 1904, 1900,
    1901, 1905, 1910, 1906, 1907, 1908, 1911, 1912, 1913, 1916, 1917, 1920, 1921, 1926, 1918, 1919,
    1919, 240, 241, 1922, 1923, 1924, 1925, 1928, 1929, 1929, 240, 241, 233, 1930, 250, 1942,
    1943, 1955, 1931, 1932, 1933, 1934, 240, 241, 1936, 1937, 1939, 1940, 1944, 1945, 1946, 1947,
    1949, 1950, 1952, 1953, 1957, 1958, 1959, 58, 1961, 1962, 1963, 1968, 1969, 1971, 1972, 1976,
    1982, 1988, 1994, 1995, 1998, 2008, 2009, 2010, 1973, 1974, 1975, 1977, 1978, 1980, 1981, 1979,
    1983, 1986, 1987, 1984, 1985, 1989, 1990, 1991, 1992, 1993, 58, 1996, 1997, 1999, 2003, 2004,
    2000, 2001, 2002, 2005, 2006, 2007, 2012, 2013, 240, 241, 2015, 2016, 2017, 2018, 2019, 2023,
    2028, 2024, 2025, 2026, 2029, 2030, 2033, 2040, 2034, 2035, 2036, 2039, 2037, 2038, 2041, 2044,
    2042, 2043, 2046, 240, 241, 2047, 58, 2049, 240, 241, 2050, 58, 2051, 2052, 2053, 2058,
    2054, 2055, 2056, 2057, 2060, 2063, 2064, 2061, 2062, 2065, 2072, 2066, 2071, 2067, 2068, 2069,
    2070, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2096, 2097, 2111, 2112, 2113, 2126, 2098,
    2102, 2105, 2110, 2099, 2100, 2101, 2103, 2104, 2106, 2107, 2108, 2109, 2114, 2118, 2122, 2115,
    2116, 2117, 2119, 2120, 2121, 2123, 2124, 2125, 2127, 2128, 2129, 2130, 2131, 2132, 2136, 2137,
    2141, 2142, 233, 2143, 250, 2144, 2151, 2145, 58, 2146, 2147, 2148, 2149, 2150, 2152, 2157,
    2153, 58, 2154, 2155, 2156, 2158, 2159, 2161, 2168, 2169, 2170, 2162, 2167, 2163, 2164, 2165,
    2166, 2173, 2174, 58, 2175, 2176, 2177, 2178, 240, 241, 2182, 2183, 2184, 2185, 240, 241,
    2186, 58, 2187, 2188, 2191, 710, 2192, 250, 2193, 2194, 2194, 240, 241, 2196, 2203, 2204,
    2205, 2197, 2202, 2198, 2199, 2200, 2201, 2212, 2213, 2227, 2237, 2228, 2235, 2229, 2230, 2231,
    2232, 2233, 2234, 2236, 240, 241, 2238, 2241, 2239, 2240, 2242, 240, 241, 2243, 2244, 2245,
    2253, 2254, 2257, 2258, 2259, 2262, 2265, 2263, 2264, 2275, 2276, 2277, 2278, 2283, 2279, 240,
    241, 2280, 2282, 2281, 58, 2288, 2289, 2290, 2291, 2310, 2311, 2312, 2313, 2314, 2315, 2316,
    2317, 2318, 2320, 2319, 2322, 2323, 2324, 2325, 2326, 2327, 2328, 2329, 2330, 2331, 2333, 2332,
    2335, 2337, 2338, 2341, 2336, 2339, 2340, 2342, 2343, 2344, 2345, 2347, 2348, 2349, 2350, 2346,
    58, 2352, 2353, 2356, 2357, 2358, 2354, 2355, 2359, 2367, 2371, 2360, 2361, 2366, 2362, 2363,
    2364, 2365, 2368, 2369, 2370, 2372, 2373, 2374, 2375, 2392, 2393, 2394, 2396, 2395, 2399, 2400,
    2403, 2404, 2405, 2401, 2402, 2409, 2410, 2411, 2412, 2413, 2415, 2416, 2417, 2418, 2429, 2444,
    2430, 2431, 233, 2432, 250, 2433, 2434, 2443, 2435, 2436, 2437, 2438, 2439, 2442, 2440, 240,
    241, 2441, 58, 2445, 2446, 233, 2447, 250, 2450, 2452, 2456, 2453, 2454, 2455, 2457, 2458,
    2459, 2461, 2462, 2463, 2466, 2464, 2465, 2465, 240, 241, 2468, 2469, 2470, 2476, 2479, 2471,
    2475, 2472, 2473, 2474, 2477, 2478, 2480, 2481, 2483, 2484, 2485, 2489, 2486, 2487, 2488, 2491,
    233, 2492, 250, 2493, 240, 241, 2494, 58, 2510, 2511, 2514, 233, 2515, 2516, 2517, 2518,
    2519, 250, 2520, 2526, 2521, 2522, 2523, 2524, 2525, 58, 2527, 2528, 2529, 58, 2538, 2539,
    2543, 2546, 2547, 2544, 2545, 2548, 2549, 2551, 2559, 2560, 2552, 2555, 2553, 2554, 2556, 2557,
    2558, 2564, 233, 2565, 250, 2566, 2567, 2568, 2574, 2575, 2576, 2577, 2578, 2580, 2581, 2582,
    2583, 2584, 2588, 2589, 2590, 2591, 2592, 2593, 2594, 2595, 2596, 2597, 2598, 2604, 2605, 2606,
    2607, 2608, 2609, 2610, 2611, 2612, 2613, 2621, 2622, 2638, 2639, 2640, 2641, 2642, 2645, 240,
    241, 2643, 2644, 2646, 2647, 2648, 240, 241, 2668, 2669, 2670, 2671, 233, 2672, 250, 2673,
    2675, 2676, 2679, 2680, 2681, 2683, 2684, 2685, 2688, 233, 2689, 250, 2686, 2687, 2690, 2691,
    2693, 2694, 2695, 2696, 2697, 2698, 2701, 233, 2702, 250, 2699, 2700, 2703, 2704, 2706, 2707,
    2708, 2709, 2712, 2715, 233, 2716, 250, 2710, 2711, 2713, 2714, 2718, 233, 2719, 250, 2722,
    233, 2723, 250, 2733, 2734, 2735, 2736, 2720, 2721, 2724, 2725, 2726, 2730, 2727, 2729, 2728,
    240, 241, 2731, 2732, 2738, 2739, 2740, 2743, 2741, 2742, 233, 2744, 250, 2776, 2777, 2778,
    2779, 2780, 2781, 2785, 2782, 2783, 2784, 2787, 233, 2790, 2791, 2792, 2793, 250, 2788, 240,
    241, 2789, 58, 233, 2794, 240, 241, 2795, 58, 2796, 2799, 233, 2797, 240, 241, 2798,
    58, 2808, 2809, 2810, 2811, 2812, 2813, 240, 241, 2815, 233, 2816, 250, 2817, 58, 2825,
    2826, 2849, 2850, 2851, 2853, 2855, 2857, 2852, 240, 241, 2854, 240, 241, 2856, 240, 241,
    2860, 2861, 2862, 2863, 2864, 2866, 2867, 2870, 2868, 2869, 2869, 240, 241, 2881, 2884, 2882,
    2883, 2885, 2886, 2890, 2891, 2906, 2907, 2908, 2910, 2912, 1561, 2911, 58, 2914, 2915, 2916,
    2917, 2919, 2920, 2921, 2922, 2923, 2924, 2925, 2926, 2927, 2928, 665, 2936, 2937, 2941, 2938,
    2939, 2940, 2942, 2943, 2963, 2964, 2969, 2970, 2972, 2973, 2976, 2977, 233, 2978, 250, 2979,
    58, 2980, 2984, 2987, 2992, 2981, 2982, 2983, 2985, 2986, 2988, 2989, 2990, 2991, 2996, 2997,
    2998, 2999, 3000, 3001, 3002, 3005, 3006, 233, 3017, 3018, 3019, 250, 3020, 3021, 3022, 240,
    241, 3025, 3026, 3027, 3032, 3028, 3029, 3030, 3031, 240, 241, 3054, 3055, 233, 3056, 250,
    3057, 3058, 3058, 240, 241, 3059, 58, 3061, 3062, 3063, 3072, 3083, 3064, 58, 3065, 3066,
    3071, 3067, 3068, 3069, 3070, 3073, 3074, 3075, 58, 3076, 3077, 3082, 3078, 3079, 3080, 3081,
    3084, 3085, 3088, 3093, 3089, 3090, 3091, 3092, 3094, 240, 241, 3095, 58, 3096, 3097, 3102,
    3098, 3099, 3100, 3101, 3109, 3114, 3110, 3111, 3112, 3113, 233, 3125, 3126, 250, 233, 3127,
    3131, 3132, 3138, 3139, 250, 240, 241, 3128, 58, 3129, 3130, 3133, 3134, 3134, 240, 241,
    3135, 3136, 3137, 58, 3140, 240, 241, 3141, 58, 3142, 3148, 3143, 3146, 3144, 240, 241,
    3145, 58, 3147, 58, 3158, 3159, 3172, 3173, 3176, 3177, 3178, 3179, 3180, 3181, 3182, 3183,
    3184, 3185, 3186, 3187, 3188, 3189, 3190, 3191, 3197, 3203, 3204, 3205, 3192, 3196, 3193, 3194,
    3195, 3198, 3202, 3199, 3200, 3201, 3219, 3220, 3221, 3222, 3223, 3226, 3224, 240, 241, 3225,
    58, 3227, 3228, 3230, 58, 3232, 58, 3234, 3235, 3238, 3236, 3237, 3242, 3243, 3245, 710,
    3246, 250, 3247, 3248, 3248, 240, 241, 3251, 3252, 3256, 3253, 3254, 3255, 3257, 240, 241,
    3263, 3264, 3276, 3277, 3292, 3293, 3294, 3295, 3296, 3297, 3301, 3298, 3299, 3300, 3302, 3303,
    3304, 3306, 3307, 3308, 3309, 3310, 3315, 3316, 3317, 3320, 3323, 3318, 3319, 3321, 3322, 3324,
    3325, 3328, 3329, 3330, 3331, 3334, 3332, 3333, 3335, 3336, 3353, 3354, 3355, 3362, 3363, 3368,
    3369, 3370, 3371, 3374, 3375, 3376, 3379, 3380, 3381, 3386, 3387, 3388, 3392, 3393, 3389, 3390,
    3391, 3394, 3395, 3407, 3408, 3412, 3426, 3427, 3428, 3409, 3410, 3411, 3413, 3417, 3420, 3425,
    3414, 3415, 3416, 3418, 3419, 3421, 3422, 3423, 3424, 3444, 3445, 3446, 3447, 3472, 3448, 3449,
    3456, 3460, 3464, 3467, 3450, 3451, 3452, 3453, 3454, 3455, 3457, 3458, 3459, 3461, 3462, 3463,
    3465, 3466, 3468, 3469, 3470, 3471, 3473, 3474, 3475, 3487, 3488, 3490, 3491, 3492, 3497, 3493,
    3494, 240, 241, 3495, 3496, 3498, 3499, 3500, 3501, 3502, 240, 241, 3521, 3522, 3523, 3524,
    3526, 3527, 3528, 3529, 3531, 3532, 250, 3533, 3534, 3535, 233, 3536, 3537, 58, 3540, 3542,
    3543, 3546, 3547, 3548, 3549, 3550, 3553, 3556, 3559, 3541, 3544, 3545, 3551, 3552, 3554, 3555,
    3557, 3558, 3561, 3562, 3563, 3567, 3564, 58, 3565, 3570, 3571, 3572, 3573, 3574, 3575, 3576,
    3577, 3578, 3579, 3580, 3581, 3582, 3583, 3584, 3585, 3590, 3593, 3596, 3597, 3591, 3592, 3592,
    240, 241, 3594, 3595, 3595, 240, 241, 3598, 240, 241, 3602, 3605, 3609, 3610, 3603, 3604,
    3606, 3607, 3608, 566, 567, 3611, 3619, 3612, 3615, 3613, 3614, 3616, 3617, 3618, 566, 567,
    3626, 3629, 3627, 3628, 3630, 3631, 3633, 3636, 3634, 3635, 3637, 3638, 3643, 3644, 3645, 3653,
    233, 3654, 250, 3655, 58, 3657, 3658, 3659, 3660, 3661, 3662, 3663, 3664, 3665, 3666, 3668,
    3669, 3672, 3673, 3674, 3677, 3670, 240, 241, 3671, 58, 3675, 3676, 3680, 3681, 3685, 3686,
    3687, 3682, 3683, 3684, 3690, 3695, 3702, 3691, 3694, 3692, 3693, 3696, 3699, 3697, 3698, 3700,
    3701, 3703, 3704, 3705, 240, 241, 3707, 3708, 3709, 3710, 3712, 3713, 3714, 3715, 3719, 3716,
    3717, 3718, 3721, 3722, 3725, 3729, 3730, 3723, 3724, 3726, 3727, 3727, 240, 241, 3728, 58,
    3731, 3732, 3733, 3734, 3734, 240, 241, 3742, 3745, 3743, 3744, 3746, 3747, 3748, 3749, 3751,
    3754, 3752, 3753, 3755, 3756, 3758, 3762, 3763, 3759, 3760, 3761, 3764, 3768, 3769, 3765, 3766,
    3767, 3770, 3771, 3774, 3781, 3788, 3792, 3803, 3807, 3808, 3809, 3810, 3775, 3776, 3777, 3778,
    3779, 3780, 3782, 3783, 3784, 3785, 3786, 3787, 3789, 3790, 3791, 3793, 3794, 3795, 3800, 3801,
    3802, 3796, 3799, 3797, 3798, 3804, 3805, 3806, 3812, 3813, 3815, 3816, 3821, 3817, 3820, 3818,
    3819, 3822, 3823, 3825, 3826, 3831, 3827, 3830, 3828, 3829, 3832, 3833, 3837, 3838, 3840, 3841,
    3845, 3846, 3847, 3848, 3850, 58, 3852, 3853, 3854, 3857, 3858, 3859, 3860, 3861, 250, 233,
    3862, 240, 241, 3863, 58, 3864, 3865, 3868, 3869, 3871, 3872, 3873, 3874, 3875, 3882, 3876,
    240, 241, 3877, 58, 3878, 3879, 3880, 240, 241, 3881, 58, 3883, 3884, 3885, 3893, 3894,
    233, 3895, 250, 3896, 58, 3897, 3901, 3904, 3898, 3899, 3900, 3902, 3903, 3905, 3906, 3907,
    3908, 3917, 3918, 3919, 3920, 240, 241, 3921, 58, 3922, 3923, 3924, 3925, 3963, 3964, 3965,
    3966, 3967, 3968, 3969, 3970, 3971, 3972, 3973, 3974, 3975, 3976, 3977, 3978, 3979, 3980, 3981,
    3982, 3983, 3984, 3985, 3986, 3987, 3988, 3989, 3990, 3991, 3992, 3993, 3994, 3995, 3996, 3997,
    3998, 3999, 4000, 4001, 4002, 4003, 4004, 4005, 4006, 4007, 4008, 4009, 4010, 4011, 4012, 4013,
    4014, 4015, 4016, 4017, 4018, 4019, 4020, 4021, 4022, 4023, 4024, 4025, 4026, 4027, 4028, 4029,
    4030, 4031, 4040, 4043, 4041, 4042, 4046, 4047, 4048, 4050, 4051, 4052, 4053, 4062, 4066, 240,
    241, 4067, 58, 4068, 4069, 4071, 577, 578, 4072, 58, 4073, 4074, 4075, 4076, 4078, 4079,
    4099, 4100, 233, 4101, 250, 4102, 4103, 4105, 4108, 4106, 4107, 4109, 4112, 4115, 4110, 4111,
    4113, 4114, 4116, 4117, 4118, 4120, 4123, 4124, 4125, 4126, 4127, 4128, 4129, 4132, 4147, 4148,
    4159, 4121, 4122, 4130, 4131, 4133, 4142, 4134, 240, 241, 4135, 58, 4136, 4141, 4137, 4140,
    4138, 4139, 4143, 4144, 4145, 4146, 4149, 4150, 4151, 4152, 4153, 4154, 4155, 4156, 4157, 4158,
    4160, 4161, 4162, 4163, 4163, 240, 241, 4166, 4167, 4168, 4169, 4179, 4170, 4173, 4171, 4172,
    4174, 4175, 233, 4176, 250, 4177, 4178, 4180, 240, 241, 4183, 4184, 4190, 4193, 4191, 4195,
    4197, 4196, 4206, 4208, 4207, 4209, 4210, 4213, 4214, 4217, 4218, 4219, 4230, 4233, 4234, 4231,
    4232, 4235, 4238, 4239, 4236, 4237, 4241, 4242, 4243, 4244, 4245, 4247, 4248, 4249, 4250, 4251,
    4252, 4253, 4272, 4278, 4281, 4284, 4273, 4274, 4275, 4276, 4277, 4279, 4280, 4282, 4283, 4285,
    4286, 4287, 4293, 4294, 233, 4297, 4298, 4299, 4300, 4307, 4295, 240, 241, 4296, 58, 4301,
    4302, 4303, 240, 241, 4304, 58, 4305, 4306, 4320, 4321, 4322, 4327, 4328, 4329, 4330, 4335,
    240, 241, 4336, 58, 4339, 4340, 4342, 4343, 4350, 4351, 4352, 4353, 4357, 4358, 4359, 4364,
    4365, 4366, 4367, 4370, 4379, 4380, 4381, 4382, 4383, 4384, 4385, 4386, 4387, 4388, 4395, 4396,
    4397, 4398, 4399, 233, 4401, 4402, 233, 4407, 250, 4403, 4406, 4404, 4405, 4408, 240, 241,
    4409, 4410, 4414, 4415, 4416, 4417, 4422, 233, 4423, 4426, 250, 4428, 4429, 4424, 4425, 4427,
    4435, 4436, 4439, 4440, 4450, 4451, 4458, 4459, 4463, 4460, 4461, 4462, 4464, 4468, 4465, 4466,
    4467, 4469, 4470, 4476, 4477, 4488, 4492, 4493, 4478, 58, 4479, 4484, 4485, 4486, 4480, 240,
    241, 4481, 58, 4482, 4483, 4487, 4489, 4490, 4491, 4508, 4509, 4510, 4511, 4523, 4524, 4526,
    4527, 4528, 4529, 4530, 4531, 710, 4525, 250, 4535, 710, 4536, 250, 4537, 4538, 4538, 240,
    241, 4540, 4541, 4549, 4552, 4555, 4556, 4557, 4558, 4559, 4542, 4548, 4543, 4544, 4545, 4546,
    4547, 4550, 4551, 4553, 4554, 4565, 4566, 4571, 4572, 4574, 4582, 4575, 4576, 4579, 4577, 4578,
    4580, 4581, 4585, 4589, 4590, 4591, 4586, 4587, 4588, 4593, 4594, 4595, 4596, 4599, 4600, 4597,
    58, 4602, 4603, 4604, 4605, 4608, 4609, 4606, 58, 4611, 4612, 4613, 4614, 4616, 4617, 4619,
    4618, 4623, 4624, 4625, 4626,
];

pub static TERMINATORS: &[u32] = &[
    59, 60, 136, 614, 615, 713, 714, 716, 723, 724, 726, 728, 729, 730, 773, 774,
    775, 776, 777, 796, 864, 1111, 1112, 1114, 1441, 1442, 1443, 1444, 1445, 1446, 1664, 1700,
    1878, 1881, 1890, 1893, 1909, 1914, 1935, 1938, 1941, 1948, 1951, 1954, 2027, 2031, 2179, 2993,
    2994, 3538, 3566, 3586, 3587, 3588, 3678, 3909, 3910, 3911, 3912, 3913, 3914, 4032, 4192, 4389,
    4390, 4391, 4392, 4393, 4598, 4607,
];

pub static STRINGS: &[&str] = &[
    "ABORT", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABS", // [6]
    "ABSOLUTE", // [7]
    "ACCESS", // [8]
    "AccountKeywordSegment", // [9]
    "ACCOUNT", // [10]
    "ALL", // [11]
    "CATALOG", // [12]
    "DATABASE", // [13]
    "DOMAIN", // [14]
    "EXTERNAL", // [15]
    "FILE", // [16]
    "FILES", // [17]
    "FOREIGN", // [18]
    "FUNCTION", // [19]
    "FUTURE", // [20]
    "INTEGRATION", // [21]
    "LANGUAGE", // [22]
    "MASKING", // [23]
    "MATERIALIZED", // [24]
    "PROCEDURE", // [25]
    "RESOURCE", // [26]
    "ROLE", // [27]
    "ROUTINE", // [28]
    "SCHEMA", // [29]
    "SEQUENCE", // [30]
    "STAGE", // [31]
    "STREAM", // [32]
    "TABLE", // [33]
    "TABLESPACE", // [34]
    "TASK", // [35]
    "TYPE", // [36]
    "VIEW", // [37]
    "WAREHOUSE", // [38]
    "ResourceKeywordSegment", // [39]
    "MonitorKeywordSegment", // [40]
    "MONITOR", // [41]
    "WarehouseKeywordSegment", // [42]
    "DatabaseKeywordSegment", // [43]
    "DomainKeywordSegment", // [44]
    "IntegrationKeywordSegment", // [45]
    "LanguageKeywordSegment", // [46]
    "SchemaKeywordSegment", // [47]
    "CatalogKeywordSegment", // [48]
    "RoleKeywordSegment", // [49]
    "TablespaceKeywordSegment", // [50]
    "TypeKeywordSegment", // [51]
    "ForeignKeywordSegment", // [52]
    "DATA", // [53]
    "SERVER", // [54]
    "ServerKeywordSegment", // [55]
    "DataKeywordSegment", // [56]
    "WrapperKeywordSegment", // [57]
    "WRAPPER", // [58]
    "AllKeywordSegment", // [59]
    "SchemasKeywordSegment", // [60]
    "SCHEMAS", // [61]
    "InKeywordSegment", // [62]
    "IN", // [63]
    "FutureKeywordSegment", // [64]
    "AccessSchemaObjectSegment", // [65]
    "access_schema_object", // [66]
    "AccessSchemaPluralObjectSegment", // [67]
    "FUNCTIONS", // [68]
    "PROCEDURES", // [69]
    "ROUTINES", // [70]
    "SEQUENCES", // [71]
    "STAGES", // [72]
    "STREAMS", // [73]
    "TABLES", // [74]
    "TASKS", // [75]
    "VIEWS", // [76]
    "access_schema_plural_object", // [77]
    "ObjectReferenceSegment", // [78]
    "object_reference", // [79]
    "FunctionNameSegment", // [80]
    "function_name", // [81]
    "FunctionParameterListGrammar", // [82]
    "(", // [83]
    "function_parameter_list", // [84]
    "WildcardIdentifierSegment", // [85]
    "wildcard_identifier", // [86]
    "CommaSegment", // [87]
    ",", // [88]
    "ToKeywordSegment", // [89]
    "TO", // [90]
    "FromKeywordSegment", // [91]
    "FROM", // [92]
    "LARGE", // [93]
    "LargeKeywordSegment", // [94]
    "ObjectKeywordSegment", // [95]
    "OBJECT", // [96]
    "NumericLiteralSegment", // [97]
    "numeric_literal", // [98]
    "APPLY", // [99]
    "CONNECT", // [100]
    "CREATE", // [101]
    "DELETE", // [102]
    "EXECUTE", // [103]
    "IMPORTED", // [104]
    "INSERT", // [105]
    "MANAGE", // [106]
    "MODIFY", // [107]
    "OPERATE", // [108]
    "OWNERSHIP", // [109]
    "READ", // [110]
    "REFERENCES", // [111]
    "REFERENCE_USAGE", // [112]
    "SELECT", // [113]
    "TEMP", // [114]
    "TEMPORARY", // [115]
    "TRIGGER", // [116]
    "TRUNCATE", // [117]
    "UPDATE", // [118]
    "USAGE", // [119]
    "USE", // [120]
    "USE_ANY_ROLE", // [121]
    "WRITE", // [122]
    "CreateKeywordSegment", // [123]
    "USER", // [124]
    "UserKeywordSegment", // [125]
    "ApplyKeywordSegment", // [126]
    "MaskingKeywordSegment", // [127]
    "PolicyKeywordSegment", // [128]
    "POLICY", // [129]
    "ExecuteKeywordSegment", // [130]
    "TaskKeywordSegment", // [131]
    "ManageKeywordSegment", // [132]
    "GrantsKeywordSegment", // [133]
    "GRANTS", // [134]
    "EXECUTION", // [135]
    "ExecutionKeywordSegment", // [136]
    "UsageKeywordSegment", // [137]
    "PIPE", // [138]
    "PipeKeywordSegment", // [139]
    "UseKeywordSegment", // [140]
    "ImportedKeywordSegment", // [141]
    "PrivilegesKeywordSegment", // [142]
    "PRIVILEGES", // [143]
    "ConnectKeywordSegment", // [144]
    "DeleteKeywordSegment", // [145]
    "InsertKeywordSegment", // [146]
    "ModifyKeywordSegment", // [147]
    "OperateKeywordSegment", // [148]
    "OwnershipKeywordSegment", // [149]
    "ReadKeywordSegment", // [150]
    "Reference_usageKeywordSegment", // [151]
    "ReferencesKeywordSegment", // [152]
    "SelectKeywordSegment", // [153]
    "TempKeywordSegment", // [154]
    "TemporaryKeywordSegment", // [155]
    "TriggerKeywordSegment", // [156]
    "TruncateKeywordSegment", // [157]
    "UpdateKeywordSegment", // [158]
    "Use_any_roleKeywordSegment", // [159]
    "WriteKeywordSegment", // [160]
    "BracketedColumnReferenceListGrammar", // [161]
    "AccessPermissionSegment", // [162]
    "access_permission", // [163]
    "OnKeywordSegment", // [164]
    "ON", // [165]
    "TableKeywordSegment", // [166]
    "ViewKeywordSegment", // [167]
    "StageKeywordSegment", // [168]
    "FunctionKeywordSegment", // [169]
    "ProcedureKeywordSegment", // [170]
    "RoutineKeywordSegment", // [171]
    "SequenceKeywordSegment", // [172]
    "StreamKeywordSegment", // [173]
    "MaterializedKeywordSegment", // [174]
    "ExternalKeywordSegment", // [175]
    "FileKeywordSegment", // [176]
    "FormatKeywordSegment", // [177]
    "FORMAT", // [178]
    "TablesKeywordSegment", // [179]
    "ViewsKeywordSegment", // [180]
    "StagesKeywordSegment", // [181]
    "FunctionsKeywordSegment", // [182]
    "ProceduresKeywordSegment", // [183]
    "RoutinesKeywordSegment", // [184]
    "SequencesKeywordSegment", // [185]
    "StreamsKeywordSegment", // [186]
    "TasksKeywordSegment", // [187]
    "PoliciesKeywordSegment", // [188]
    "FormatsKeywordSegment", // [189]
    "GRANT", // [190]
    "REVOKE", // [191]
    "GrantStatementSegment", // [192]
    "grant_statement", // [193]
    "RevokeStatementSegment", // [194]
    "revoke_statement", // [195]
    "GROUP", // [196]
    "GroupKeywordSegment", // [197]
    "UserReferenceSegment", // [198]
    "user_reference", // [199]
    "RoleReferenceSegment", // [200]
    "role_reference", // [201]
    "SHARE", // [202]
    "ShareKeywordSegment", // [203]
    "FunctionSegment", // [204]
    "function", // [205]
    "PublicKeywordSegment", // [206]
    "PUBLIC", // [207]
    ".", // [208]
    ":", // [209]
    "[", // [210]
    "ArrayAccessorSegment", // [211]
    "array_accessor", // [212]
    "SemiStructuredAccessorSegment", // [213]
    "semi_structured_expression", // [214]
    "ACCOUNTS", // [215]
    "ACTION", // [216]
    "ADA", // [217]
    "ADD", // [218]
    "AddKeywordSegment", // [219]
    "/", // [220]
    "double_quote", // [221]
    "file_literal", // [222]
    "single_quote", // [223]
    "QuotedLiteralSegment", // [224]
    "FileLiteralSegment", // [225]
    "JarKeywordSegment", // [226]
    "JAR", // [227]
    "JARS", // [228]
    "ADMIN", // [229]
    "AFTER", // [230]
    "AGGREGATE", // [231]
    "OrderByClauseSegment", // [232]
    "ORDER", // [233]
    "orderby_clause", // [234]
    "indent", // [235]
    "Indent", // [236]
    "meta", // [237]
    "AsAliasOperatorSegment", // [238]
    "AS", // [239]
    "alias_operator", // [240]
    "SingleIdentifierGrammar", // [241]
    "SingleIdentifierListSegment", // [242]
    "identifier_list", // [243]
    "start_bracket", // [244]
    "SymbolSegment", // [245]
    "symbol", // [246]
    ")", // [247]
    "end_bracket", // [248]
    "ANTI", // [249]
    "CROSS", // [250]
    "FULL", // [251]
    "INNER", // [252]
    "KEYS", // [253]
    "LATERAL", // [254]
    "LEFT", // [255]
    "PIVOT", // [256]
    "RIGHT", // [257]
    "SEMI", // [258]
    "WINDOW", // [259]
    "LateralKeywordSegment", // [260]
    "JoinTypeKeywords", // [261]
    "WindowKeywordSegment", // [262]
    "PivotKeywordSegment", // [263]
    "KeysKeywordSegment", // [264]
    "dedent", // [265]
    "Dedent", // [266]
    "ALIAS", // [267]
    "TableReferenceSegment", // [268]
    "table_reference", // [269]
    "AliasExpressionSegment", // [270]
    "alias_expression", // [271]
    "ALLOCATE", // [272]
    "ALSO", // [273]
    "ALTER", // [274]
    "AlterKeywordSegment", // [275]
    "DatabaseReferenceSegment", // [276]
    "database_reference", // [277]
    "SetKeywordSegment", // [278]
    "SET", // [279]
    "DBPROPERTIES", // [280]
    "LOCATION", // [281]
    "DatabasePropertiesGrammar", // [282]
    "LocationGrammar", // [283]
    "CACHE", // [284]
    "CYCLE", // [285]
    "INCREMENT", // [286]
    "MAXVALUE", // [287]
    "MINVALUE", // [288]
    "NO", // [289]
    "NOCACHE", // [290]
    "NOCYCLE", // [291]
    "NOORDER", // [292]
    "IncrementKeywordSegment", // [293]
    "ByKeywordSegment", // [294]
    "BY", // [295]
    "SequenceMinValueGrammar", // [296]
    "SequenceMaxValueGrammar", // [297]
    "CacheKeywordSegment", // [298]
    "NocacheKeywordSegment", // [299]
    "CycleKeywordSegment", // [300]
    "NocycleKeywordSegment", // [301]
    "OrderNoOrderGrammar", // [302]
    "SequenceReferenceSegment", // [303]
    "sequence_reference", // [304]
    "AlterSequenceOptionsSegment", // [305]
    "alter_sequence_options_segment", // [306]
    "DROP", // [307]
    "DropKeywordSegment", // [308]
    "ColumnKeywordSegment", // [309]
    "COLUMN", // [310]
    "IfExistsGrammar", // [311]
    "IF", // [312]
    "ParameterNameSegment", // [313]
    "EqualsSegment", // [314]
    "=", // [315]
    "comparison_operator", // [316]
    "LiteralGrammar", // [317]
    "+", // [318]
    "-", // [319]
    "ARRAY", // [320]
    "DATE", // [321]
    "FALSE", // [322]
    "INTERVAL", // [323]
    "NULL", // [324]
    "TIME", // [325]
    "TIMESTAMP", // [326]
    "TIMESTAMP_LTZ", // [327]
    "TIMESTAMP_NTZ", // [328]
    "TRUE", // [329]
    "{", // [330]
    "bytes_double_quote", // [331]
    "bytes_single_quote", // [332]
    "raw_double_quote", // [333]
    "raw_single_quote", // [334]
    "NakedIdentifierSegment", // [335]
    "ColumnDefinitionSegment", // [336]
    "column_definition", // [337]
    "FIRST", // [338]
    "FirstKeywordSegment", // [339]
    "AfterKeywordSegment", // [340]
    "ColumnReferenceSegment", // [341]
    "column_reference", // [342]
    "AlterTableDropColumnGrammar", // [343]
    "RENAME", // [344]
    "RenameKeywordSegment", // [345]
    "AsKeywordSegment", // [346]
    "PARTITION", // [347]
    "PARTITIONED", // [348]
    "PartitionSpecGrammar", // [349]
    "COLUMNS", // [350]
    "ColumnsKeywordSegment", // [351]
    "ColumnFieldDefinitionSegment", // [352]
    "FirstOrAfterGrammar", // [353]
    "CHANGE", // [354]
    "ChangeKeywordSegment", // [355]
    "BIGINT", // [356]
    "BINARY", // [357]
    "BOOLEAN", // [358]
    "CHAR", // [359]
    "CHARACTER", // [360]
    "COMMENT", // [361]
    "DEC", // [362]
    "DECIMAL", // [363]
    "DOUBLE", // [364]
    "FLOAT", // [365]
    "INT", // [366]
    "INTEGER", // [367]
    "LONG", // [368]
    "MAP", // [369]
    "NUMERIC", // [370]
    "REAL", // [371]
    "SMALLINT", // [372]
    "STRING", // [373]
    "STRUCT", // [374]
    "TINYINT", // [375]
    "VARCHAR", // [376]
    "VARIANT", // [377]
    "CommentKeywordSegment", // [378]
    "DatatypeSegment", // [379]
    "data_type", // [380]
    "CommentGrammar", // [381]
    "NotKeywordSegment", // [382]
    "NOT", // [383]
    "NullKeywordSegment", // [384]
    "REPLACE", // [385]
    "ReplaceKeywordSegment", // [386]
    "IfNotExistsGrammar", // [387]
    "PartitionFieldGrammar", // [388]
    "PURGE", // [389]
    "PurgeKeywordSegment", // [390]
    "RECOVER", // [391]
    "RecoverKeywordSegment", // [392]
    "PartitionsKeywordSegment", // [393]
    "PARTITIONS", // [394]
    "TablePropertiesGrammar", // [395]
    "TBLPROPERTIES", // [396]
    "UnsetTablePropertiesGrammar", // [397]
    "UNSET", // [398]
    "SERDE", // [399]
    "SERDEPROPERTIES", // [400]
    "SerdepropertiesKeywordSegment", // [401]
    "BracketedPropertyListGrammar", // [402]
    "SerdeKeywordSegment", // [403]
    "SerdePropertiesGrammar", // [404]
    "WITH", // [405]
    "FileformatKeywordSegment", // [406]
    "FILEFORMAT", // [407]
    "DataSourceFormatSegment", // [408]
    "data_source_format", // [409]
    "ConstraintKeywordSegment", // [410]
    "CONSTRAINT", // [411]
    "CheckKeywordSegment", // [412]
    "CHECK", // [413]
    "ExpressionSegment", // [414]
    "expression", // [415]
    "DISTRIBUTED", // [416]
    "LOCALLY", // [417]
    "ORDERED", // [418]
    "DistributedKeywordSegment", // [419]
    "PartitionKeywordSegment", // [420]
    "LocallyKeywordSegment", // [421]
    "OrderedKeywordSegment", // [422]
    "ASC", // [423]
    "DESC", // [424]
    "AscKeywordSegment", // [425]
    "DescKeywordSegment", // [426]
    "NULLS", // [427]
    "NullsKeywordSegment", // [428]
    "LAST", // [429]
    "LastKeywordSegment", // [430]
    "IdentifierKeywordSegment", // [431]
    "IDENTIFIER", // [432]
    "FieldsKeywordSegment", // [433]
    "FIELDS", // [434]
    "SelectStatementSegment", // [435]
    "select_statement", // [436]
    "ALWAYS", // [437]
    "&", // [438]
    "ampersand", // [439]
    "ANALYSE", // [440]
    "ANALYZE", // [441]
    "AnalyzeKeywordSegment", // [442]
    "ComputeKeywordSegment", // [443]
    "COMPUTE", // [444]
    "StatisticsKeywordSegment", // [445]
    "STATISTICS", // [446]
    "FOR", // [447]
    "NOSCAN", // [448]
    "NoscanKeywordSegment", // [449]
    "ForKeywordSegment", // [450]
    "AND", // [451]
    "binary_operator", // [452]
    "BinaryOperatorSegment", // [453]
    "ANY", // [454]
    "ARCHIVE", // [455]
    "ARE", // [456]
    "%", // [457]
    "*", // [458]
    "<", // [459]
    ">", // [460]
    "DIV", // [461]
    "^", // [462]
    "|", // [463]
    "PlusSegment", // [464]
    "MinusSegment", // [465]
    "DivideSegment", // [466]
    "MultiplySegment", // [467]
    "ModuloSegment", // [468]
    "BitwiseAndSegment", // [469]
    "BitwiseOrSegment", // [470]
    "BitwiseXorSegment", // [471]
    "BitwiseLShiftSegment", // [472]
    "BitwiseRShiftSegment", // [473]
    "DivBinaryOperatorSegment", // [474]
    "SliceSegment", // [475]
    "start_square_bracket", // [476]
    "]", // [477]
    "end_square_bracket", // [478]
    "BaseExpressionElementGrammar", // [479]
    "ArrayKeywordSegment", // [480]
    "start_angle_bracket", // [481]
    "end_angle_bracket", // [482]
    "ASENSITIVE", // [483]
    "ASSERTION", // [484]
    "ASSIGNMENT", // [485]
    "ASYMMETRIC", // [486]
    "AT", // [487]
    "at_sign_literal", // [488]
    "literal", // [489]
    "LiteralSegment", // [490]
    "@", // [491]
    "ATOMIC", // [492]
    "ATTRIBUTE", // [493]
    "ATTRIBUTES", // [494]
    "AUDIT", // [495]
    "AUTHORIZATION", // [496]
    "AUTO_INCREMENT", // [497]
    "Auto_incrementKeywordSegment", // [498]
    "AVG", // [499]
    "AVG_ROW_LENGTH", // [500]
    "AVRO", // [501]
    "back_quote", // [502]
    "quoted_identifier", // [503]
    "identifier", // [504]
    "IdentifierSegment", // [505]
    "`", // [506]
    "BACKUP", // [507]
    "BACKWARD", // [508]
    "CURRENT_DATE", // [509]
    "CURRENT_TIMESTAMP", // [510]
    "CURRENT_USER", // [511]
    "bare_function", // [512]
    "CodeSegment", // [513]
    "BareFunctionSegment", // [514]
    "IntervalExpressionSegment", // [515]
    "interval_expression", // [516]
    "file", // [517]
    "BaseFileSegment", // [518]
    "BaseSegment", // [519]
    "BEFORE", // [520]
    "BEGIN", // [521]
    "BERNOULLI", // [522]
    "BETWEEN", // [523]
    "!", // [524]
    "->", // [525]
    "<=>", // [526]
    "==", // [527]
    "IS", // [528]
    "OR", // [529]
    "like_operator", // [530]
    "ArithmeticBinaryOperatorGrammar", // [531]
    "StringBinaryOperatorGrammar", // [532]
    "BooleanBinaryOperatorGrammar", // [533]
    "ComparisonOperatorGrammar", // [534]
    "RightArrowOperator", // [535]
    "BINARYFILE", // [536]
    "file_format", // [537]
    "BINDING", // [538]
    "BIT", // [539]
    "BIT_LENGTH", // [540]
    "BITVAR", // [541]
    "AmpersandSegment", // [542]
    "RawLessThanSegment", // [543]
    "PipeSegment", // [544]
    "RawGreaterThanSegment", // [545]
    "BLOB", // [546]
    "BOOL", // [547]
    "AndOperatorGrammar", // [548]
    "OrOperatorGrammar", // [549]
    "TrueSegment", // [550]
    "FalseSegment", // [551]
    "BOTH", // [552]
    "PropertyListGrammar", // [553]
    "PropertyNameListGrammar", // [554]
    "bracketed", // [555]
    "BracketedSegment", // [556]
    "UnorderedSetExpressionSegment", // [557]
    "VALUES", // [558]
    "set_expression", // [559]
    "BREADTH", // [560]
    "BREAK", // [561]
    "BROWSE", // [562]
    "BUCKET", // [563]
    "CLUSTERED", // [564]
    "ClusteredBySpecGrammar", // [565]
    "SortedBySpecGrammar", // [566]
    "SORTED", // [567]
    "IntoKeywordSegment", // [568]
    "INTO", // [569]
    "BucketsKeywordSegment", // [570]
    "BUCKETS", // [571]
    "BULK", // [572]
    "bytes_quoted_literal", // [573]
    "CTEColumnList", // [574]
    "cte_column_list", // [575]
    "SelectableGrammar", // [576]
    "LazyKeywordSegment", // [577]
    "LAZY", // [578]
    "OptionsGrammar", // [579]
    "OPTIONS", // [580]
    "CALL", // [581]
    "CALLED", // [582]
    "CARDINALITY", // [583]
    "CASCADE", // [584]
    "CASCADED", // [585]
    "CASE", // [586]
    "CaseKeywordSegment", // [587]
    "implicit_indent", // [588]
    "ImplicitIndent", // [589]
    "WHEN", // [590]
    "WhenClauseSegment", // [591]
    "when_clause", // [592]
    "ElseKeywordSegment", // [593]
    "ELSE", // [594]
    "EndKeywordSegment", // [595]
    "END", // [596]
    "ElseClauseSegment", // [597]
    "else_clause", // [598]
    "BinaryOperatorGrammar", // [599]
    "CAST", // [600]
    "::", // [601]
    "casting_operator", // [602]
    "CATALOG_NAME", // [603]
    "CEIL", // [604]
    "CEILING", // [605]
    "CHAIN", // [606]
    "CHANGES", // [607]
    "CHAR_LENGTH", // [608]
    "CHARACTER_LENGTH", // [609]
    "CHARACTER_SET_CATALOG", // [610]
    "CHARACTER_SET_NAME", // [611]
    "CHARACTER_SET_SCHEMA", // [612]
    "CHARACTERISTICS", // [613]
    "CHARACTERS", // [614]
    "CHECKED", // [615]
    "CHECKPOINT", // [616]
    "CHECKSUM", // [617]
    "CHOICES", // [618]
    "CLASS", // [619]
    "CLASS_ORIGIN", // [620]
    "CLEAR", // [621]
    "ClearKeywordSegment", // [622]
    "CLOB", // [623]
    "CLOSE", // [624]
    "CLUSTER", // [625]
    "ClusterKeywordSegment", // [626]
    "LimitKeywordSegment", // [627]
    "LIMIT", // [628]
    "HavingKeywordSegment", // [629]
    "HAVING", // [630]
    "FrameClauseUnitGrammar", // [631]
    "RANGE", // [632]
    "ROWS", // [633]
    "SeparatorKeywordSegment", // [634]
    "SEPARATOR", // [635]
    "ClusteredKeywordSegment", // [636]
    "COALESCE", // [637]
    "COBOL", // [638]
    "CODEGEN", // [639]
    "COLLATE", // [640]
    "COLLATION", // [641]
    "ObjectReferenceDelimiterGrammar", // [642]
    "ObjectReferenceTerminatorGrammar", // [643]
    "COLLATION_CATALOG", // [644]
    "COLLATION_NAME", // [645]
    "COLLATION_SCHEMA", // [646]
    "COLLECT", // [647]
    "COLLECTION", // [648]
    "colon_delimiter", // [649]
    "colon_prefix", // [650]
    "colon", // [651]
    "ShorthandCastSegment", // [652]
    "cast_expression", // [653]
    "DEFAULT", // [654]
    "GENERATED", // [655]
    "PRIMARY", // [656]
    "UNIQUE", // [657]
    "DefaultKeywordSegment", // [658]
    "ColumnConstraintDefaultGrammar", // [659]
    "PrimaryKeyGrammar", // [660]
    "NotEnforcedGrammar", // [661]
    "UniqueKeyGrammar", // [662]
    "AutoIncrementGrammar", // [663]
    "ReferenceDefinitionGrammar", // [664]
    "CommentClauseSegment", // [665]
    "comment_clause", // [666]
    "CollateKeywordSegment", // [667]
    "CollationReferenceSegment", // [668]
    "collation_reference", // [669]
    "ColumnGeneratedGrammar", // [670]
    "ColumnConstraintSegment", // [671]
    "column_constraint_segment", // [672]
    "GeneratedKeywordSegment", // [673]
    "AlwaysKeywordSegment", // [674]
    "COLUMN_NAME", // [675]
    "ColumnsExpressionNameGrammar", // [676]
    "comma", // [677]
    "COMMAND_FUNCTION", // [678]
    "COMMAND_FUNCTION_CODE", // [679]
    "comment", // [680]
    "CommentSegment", // [681]
    "COMMIT", // [682]
    "COMMITTED", // [683]
    "COMPACT", // [684]
    "COMPACTIONS", // [685]
    "EqualsSegment_a", // [686]
    "EqualsSegment_b", // [687]
    "GreaterThanSegment", // [688]
    "LessThanSegment", // [689]
    "GreaterThanOrEqualToSegment", // [690]
    "LessThanOrEqualToSegment", // [691]
    "NotEqualToSegment", // [692]
    "LikeOperatorSegment", // [693]
    "IsKeywordSegment", // [694]
    "DistinctKeywordSegment", // [695]
    "DISTINCT", // [696]
    "ComparisonOperatorSegment", // [697]
    "COMPENSATION", // [698]
    "COMPLETION", // [699]
    "CompositeBinaryOperatorSegment", // [700]
    "CompositeComparisonOperatorSegment", // [701]
    "COMPRESS", // [702]
    "CONCATENATE", // [703]
    "CONDITION", // [704]
    "CONDITION_NUMBER", // [705]
    "CrossKeywordSegment", // [706]
    "JoinTypeKeywordsGrammar", // [707]
    "ConditionalCrossJoinKeywordsGrammar", // [708]
    "NonStandardJoinTypeKeywordsGrammar", // [709]
    "CONNECTION", // [710]
    "CONNECTION_NAME", // [711]
    "ExpectKeywordSegment", // [712]
    "EXPECT", // [713]
    "ViolationKeywordSegment", // [714]
    "VIOLATION", // [715]
    "FAIL", // [716]
    "FailKeywordSegment", // [717]
    "RowKeywordSegment", // [718]
    "ROW", // [719]
    "CONSTRAINT_CATALOG", // [720]
    "CONSTRAINT_NAME", // [721]
    "CONSTRAINT_SCHEMA", // [722]
    "CONSTRAINTS", // [723]
    "CONSTRUCTOR", // [724]
    "CONTAINS", // [725]
    "CONTAINSTABLE", // [726]
    "CONTINUE", // [727]
    "CONVERSION", // [728]
    "CONVERT", // [729]
    "ConvertKeywordSegment", // [730]
    "DeltaKeywordSegment", // [731]
    "DELTA", // [732]
    "FileReferenceSegment", // [733]
    "CSV", // [734]
    "ICEBERG", // [735]
    "JSON", // [736]
    "ORC", // [737]
    "PARQUET", // [738]
    "TEXT", // [739]
    "file_reference", // [740]
    "NoKeywordSegment", // [741]
    "COPY", // [742]
    "CORR", // [743]
    "CORRESPONDING", // [744]
    "COST", // [745]
    "COUNT", // [746]
    "COVAR_POP", // [747]
    "COVAR_SAMP", // [748]
    "CastKeywordSegment", // [749]
    "WithKeywordSegment", // [750]
    "SpecificKeywordSegment", // [751]
    "SPECIFIC", // [752]
    "INSTANCE", // [753]
    "METHOD", // [754]
    "STATIC", // [755]
    "InstanceKeywordSegment", // [756]
    "StaticKeywordSegment", // [757]
    "ConstructorKeywordSegment", // [758]
    "MethodKeywordSegment", // [759]
    "AssignmentKeywordSegment", // [760]
    "DbpropertiesKeywordSegment", // [761]
    "OrReplaceGrammar", // [762]
    "TemporaryGrammar", // [763]
    "GLOBAL", // [764]
    "FunctionNameIdentifierSegment", // [765]
    "ResourceLocationGrammar", // [766]
    "USING", // [767]
    "UniqueKeywordSegment", // [768]
    "IndexKeywordSegment", // [769]
    "INDEX", // [770]
    "IndexReferenceSegment", // [771]
    "index_reference", // [772]
    "IndexColumnDefinitionSegment", // [773]
    "index_column_definition", // [774]
    "ModelKeywordSegment", // [775]
    "MODEL", // [776]
    "OptionsKeywordSegment", // [777]
    "SchemaReferenceSegment", // [778]
    "schema_reference", // [779]
    "START", // [780]
    "StartKeywordSegment", // [781]
    "CreateSequenceOptionsSegment", // [782]
    "create_sequence_options_segment", // [783]
    "TableDefinitionSegment", // [784]
    "LIVE", // [785]
    "STREAMING", // [786]
    "TriggerReferenceSegment", // [787]
    "trigger_reference", // [788]
    "INSTEAD", // [789]
    "BeforeKeywordSegment", // [790]
    "InsteadKeywordSegment", // [791]
    "OfKeywordSegment", // [792]
    "OF", // [793]
    "OrKeywordSegment", // [794]
    "DEFERRABLE", // [795]
    "INITIALLY", // [796]
    "REFERENCING", // [797]
    "ReferencingKeywordSegment", // [798]
    "OldKeywordSegment", // [799]
    "OLD", // [800]
    "NewKeywordSegment", // [801]
    "NEW", // [802]
    "DeferrableKeywordSegment", // [803]
    "InitiallyKeywordSegment", // [804]
    "ImmediateKeywordSegment", // [805]
    "IMMEDIATE", // [806]
    "DeferredKeywordSegment", // [807]
    "DEFERRED", // [808]
    "EachKeywordSegment", // [809]
    "EACH", // [810]
    "STATEMENT", // [811]
    "StatementKeywordSegment", // [812]
    "WhenKeywordSegment", // [813]
    "FunctionContentsSegment", // [814]
    "function_contents", // [815]
    "EVOLUTION", // [816]
    "BindingKeywordSegment", // [817]
    "CompensationKeywordSegment", // [818]
    "EvolutionKeywordSegment", // [819]
    "OrRefreshGrammar", // [820]
    "StreamingKeywordSegment", // [821]
    "LiveKeywordSegment", // [822]
    "ConstraintStatementSegment", // [823]
    "constraint_statement", // [824]
    "UsingKeywordSegment", // [825]
    "TableClusterByClauseSegment", // [826]
    "table_cluster_by_clause", // [827]
    "CreateViewClausesGrammar", // [828]
    "WithNoSchemaBindingClauseSegment", // [829]
    "with_no_schema_binding_clause", // [830]
    "WidgetKeywordSegment", // [831]
    "WIDGET", // [832]
    "DROPDOWN", // [833]
    "DropdownKeywordSegment", // [834]
    "WidgetNameIdentifierSegment", // [835]
    "WidgetDefaultGrammar", // [836]
    "ChoicesKeywordSegment", // [837]
    "TextKeywordSegment", // [838]
    "CREATEDB", // [839]
    "CREATEROLE", // [840]
    "CREATEUSER", // [841]
    "CUBE", // [842]
    "function_name_identifier", // [843]
    "ROLLUP", // [844]
    "CubeFunctionNameSegment", // [845]
    "RollupFunctionNameSegment", // [846]
    "GroupingExpressionList", // [847]
    "grouping_expression_list", // [848]
    "CUME_DIST", // [849]
    "CURRENT", // [850]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [851]
    "CURRENT_PATH", // [852]
    "CURRENT_ROLE", // [853]
    "CURRENT_TIME", // [854]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [855]
    "CURSOR", // [856]
    "CURSOR_NAME", // [857]
    "FileFormatGrammar", // [858]
    "INPUTFORMAT", // [859]
    "JSONFILE", // [860]
    "RCFILE", // [861]
    "SEQUENCEFILE", // [862]
    "TEXTFILE", // [863]
    "JdbcKeywordSegment", // [864]
    "JDBC", // [865]
    "AvroKeywordSegment", // [866]
    "CsvKeywordSegment", // [867]
    "JsonKeywordSegment", // [868]
    "ParquetKeywordSegment", // [869]
    "OrcKeywordSegment", // [870]
    "IcebergKeywordSegment", // [871]
    "BinaryfileKeywordSegment", // [872]
    "DATABASES", // [873]
    "data_type_identifier", // [874]
    "PrimitiveTypeSegment", // [875]
    "primitive_type", // [876]
    "ArrayTypeSegment", // [877]
    "array_type", // [878]
    "MapKeywordSegment", // [879]
    "StructTypeSegment", // [880]
    "struct_type", // [881]
    "DATEADD", // [882]
    "DATEDIFF", // [883]
    "DATE_ADD", // [884]
    "DATE_DIFF", // [885]
    "TIMESTAMPADD", // [886]
    "TIMESTAMPDIFF", // [887]
    "DatePartFunctionName", // [888]
    "DatetimeUnitSegment", // [889]
    "DAY", // [890]
    "DAYOFYEAR", // [891]
    "DAYS", // [892]
    "DD", // [893]
    "HOUR", // [894]
    "HOURS", // [895]
    "MICROSECOND", // [896]
    "MICROSECONDS", // [897]
    "MILLISECOND", // [898]
    "MILLISECONDS", // [899]
    "MINUTE", // [900]
    "MINUTES", // [901]
    "MM", // [902]
    "MON", // [903]
    "MONTH", // [904]
    "MONTHS", // [905]
    "QUARTER", // [906]
    "QUARTERS", // [907]
    "SECOND", // [908]
    "SECONDS", // [909]
    "WEEK", // [910]
    "WEEKS", // [911]
    "YEAR", // [912]
    "YEARS", // [913]
    "YY", // [914]
    "YYYY", // [915]
    "FunctionContentsGrammar", // [916]
    "DateKeywordSegment", // [917]
    "TimeKeywordSegment", // [918]
    "TimestampKeywordSegment", // [919]
    "IntervalKeywordSegment", // [920]
    "Timestamp_ltzKeywordSegment", // [921]
    "Timestamp_ntzKeywordSegment", // [922]
    "date_constructor_literal", // [923]
    "DATE_HOUR", // [924]
    "DATETIME", // [925]
    "date_part", // [926]
    "DATETIME_INTERVAL_CODE", // [927]
    "DATETIME_INTERVAL_PRECISION", // [928]
    "DAY_HOUR", // [929]
    "DAY_MICROSECOND", // [930]
    "DAY_MINUTE", // [931]
    "DAY_SECOND", // [932]
    "DAYOFMONTH", // [933]
    "DAYOFWEEK", // [934]
    "DBCC", // [935]
    "DEALLOCATE", // [936]
    "DECLARE", // [937]
    "ValuesKeywordSegment", // [938]
    "DEFAULTS", // [939]
    "DEFINED", // [940]
    "DEFINER", // [941]
    "DEGREE", // [942]
    "DELAY_KEY_WRITE", // [943]
    "DELAYED", // [944]
    "FromClauseSegment", // [945]
    "from_clause", // [946]
    "WhereClauseSegment", // [947]
    "WHERE", // [948]
    "where_clause", // [949]
    "DELETES", // [950]
    "DELIMITED", // [951]
    "SemicolonSegment", // [952]
    ";", // [953]
    "DELIMITER", // [954]
    "DELIMITERS", // [955]
    "DENSE_RANK", // [956]
    "DENY", // [957]
    "DEPTH", // [958]
    "DEREF", // [959]
    "DERIVED", // [960]
    "DESCRIBE", // [961]
    "DescribeKeywordSegment", // [962]
    "DetailKeywordSegment", // [963]
    "DETAIL", // [964]
    "HistoryKeywordSegment", // [965]
    "HISTORY", // [966]
    "LimitClauseSegment", // [967]
    "limit_clause", // [968]
    "ExtendedKeywordSegment", // [969]
    "EXTENDED", // [970]
    "DotSegment", // [971]
    "DISTRIBUTE", // [972]
    "EXPLAIN", // [973]
    "GENERATE", // [974]
    "LIST", // [975]
    "LOAD", // [976]
    "MERGE", // [977]
    "MSCK", // [978]
    "QUERY", // [979]
    "REFRESH", // [980]
    "REMOVE", // [981]
    "RESET", // [982]
    "RESTORE", // [983]
    "SHOW", // [984]
    "UNCACHE", // [985]
    "VACUUM", // [986]
    "QueryKeywordSegment", // [987]
    "GroupByClauseSegment", // [988]
    "groupby_clause", // [989]
    "StatementSegment", // [990]
    "statement", // [991]
    "DescribeObjectGrammar", // [992]
    "DESCRIPTOR", // [993]
    "DESTROY", // [994]
    "DESTRUCTOR", // [995]
    "DETERMINISTIC", // [996]
    "DFS", // [997]
    "DIAGNOSTICS", // [998]
    "DICTIONARY", // [999]
    "DIRECTORIES", // [1000]
    "DIRECTORY", // [1001]
    "DISABLE", // [1002]
    "DISCONNECT", // [1003]
    "DISK", // [1004]
    "DISPATCH", // [1005]
    "DISTINCTROW", // [1006]
    "DistributeKeywordSegment", // [1007]
    "SortKeywordSegment", // [1008]
    "SORT", // [1009]
    "DivKeywordSegment", // [1010]
    "DO", // [1011]
    "dot", // [1012]
    "RESTRICT", // [1013]
    "RestrictKeywordSegment", // [1014]
    "CascadeKeywordSegment", // [1015]
    "DropBehaviorGrammar", // [1016]
    "DRY", // [1017]
    "DUMMY", // [1018]
    "DUMP", // [1019]
    "DYNAMIC", // [1020]
    "DYNAMIC_FUNCTION", // [1021]
    "DYNAMIC_FUNCTION_CODE", // [1022]
    "ELEMENT", // [1023]
    "ELSEIF", // [1024]
    "EmptyStructLiteralBracketsSegment", // [1025]
    "struct_literal", // [1026]
    "ENABLE", // [1027]
    "ENCLOSED", // [1028]
    "ENCODING", // [1029]
    "ENCRYPTED", // [1030]
    "END-EXEC", // [1031]
    "}", // [1032]
    "end_curly_bracket", // [1033]
    "*/", // [1034]
    "end_hint", // [1035]
    "ENUM", // [1036]
    "EQUALS", // [1037]
    "RawEqualsSegment", // [1038]
    "ERRLVL", // [1039]
    "ESCAPE", // [1040]
    "ESCAPED", // [1041]
    "EVERY", // [1042]
    "EXCEPT", // [1043]
    "ExceptKeywordSegment", // [1044]
    "EXCEPTION", // [1045]
    "EXCHANGE", // [1046]
    "EXCLUDE", // [1047]
    "EXCLUDING", // [1048]
    "EXCLUSIVE", // [1049]
    "EXEC", // [1050]
    "EXISTING", // [1051]
    "EXISTS", // [1052]
    "EXIT", // [1053]
    "EXP", // [1054]
    "ExplainKeywordSegment", // [1055]
    "FORMATTED", // [1056]
    "CodegenKeywordSegment", // [1057]
    "CostKeywordSegment", // [1058]
    "FormattedKeywordSegment", // [1059]
    "EXPORT", // [1060]
    "Expression_A_Grammar", // [1061]
    "Tail_Recurse_Expression_A_Grammar", // [1062]
    "LikeExpressionGrammar", // [1063]
    "ILIKE", // [1064]
    "LIKE", // [1065]
    "REGEXP", // [1066]
    "RLIKE", // [1067]
    "InOperatorGrammar", // [1068]
    "QuantifiedComparisonOperatorGrammar", // [1069]
    "IsClauseGrammar", // [1070]
    "IsNullGrammar", // [1071]
    "NotNullGrammar", // [1072]
    "CollateGrammar", // [1073]
    "BetweenKeywordSegment", // [1074]
    "Expression_B_Grammar", // [1075]
    "AndKeywordSegment", // [1076]
    "PatternMatchingGrammar", // [1077]
    "PRIOR", // [1078]
    "~", // [1079]
    "SignedSegmentGrammar", // [1080]
    "QualifiedNumericLiteralSegment", // [1081]
    "TildeSegment", // [1082]
    "NotOperatorGrammar", // [1083]
    "PriorKeywordSegment", // [1084]
    "Tail_Recurse_Expression_B_Grammar", // [1085]
    "ExistsKeywordSegment", // [1086]
    "Expression_D_Grammar", // [1087]
    "CaseExpressionSegment", // [1088]
    "case_expression", // [1089]
    "TimeZoneGrammar", // [1090]
    "time_zone_grammar", // [1091]
    "LateralColumnAliasExpressionGrammar", // [1092]
    "LocalAliasSegment", // [1093]
    "local_alias_segment", // [1094]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1095]
    "SingleIdentifierWildcardSegment", // [1096]
    "MapTypeSegment", // [1097]
    "map_type", // [1098]
    "BooleanLiteralGrammar", // [1099]
    "NullLiteralSegment", // [1100]
    "DateTimeLiteralGrammar", // [1101]
    "ListComprehensionGrammar", // [1102]
    "AccessorGrammar", // [1103]
    "TypedStructLiteralSegment", // [1104]
    "typed_struct_literal", // [1105]
    "ArrayExpressionSegment", // [1106]
    "array_expression", // [1107]
    "OverlapsClauseSegment", // [1108]
    "OVERLAPS", // [1109]
    "overlaps_clause", // [1110]
    "EXTENSION", // [1111]
    "EXTRACT", // [1112]
    "boolean_literal", // [1113]
    "LiteralKeywordSegment", // [1114]
    "FETCH", // [1115]
    "FetchKeywordSegment", // [1116]
    "NEXT", // [1117]
    "NextKeywordSegment", // [1118]
    "RowsKeywordSegment", // [1119]
    "ONLY", // [1120]
    "OnlyKeywordSegment", // [1121]
    "TiesKeywordSegment", // [1122]
    "TIES", // [1123]
    "FIELD", // [1124]
    "DataSourcesV2FileTypeGrammar", // [1125]
    "SequencefileKeywordSegment", // [1126]
    "TextfileKeywordSegment", // [1127]
    "RcfileKeywordSegment", // [1128]
    "JsonfileKeywordSegment", // [1129]
    "InputformatKeywordSegment", // [1130]
    "OutputformatKeywordSegment", // [1131]
    "OUTPUTFORMAT", // [1132]
    "file_keyword", // [1133]
    "SlashSegment", // [1134]
    "path_segment", // [1135]
    "WordSegment", // [1136]
    "BackQuotedIdentifierSegment", // [1137]
    "DelimiterGrammar", // [1138]
    "FILLFACTOR", // [1139]
    "FILTER", // [1140]
    "FilterKeywordSegment", // [1141]
    "WhereKeywordSegment", // [1142]
    "FINAL", // [1143]
    "FLOAT4", // [1144]
    "FLOAT8", // [1145]
    "FLOOR", // [1146]
    "FLUSH", // [1147]
    "FOLLOWING", // [1148]
    "FORCE", // [1149]
    "KeyKeywordSegment", // [1150]
    "KEY", // [1151]
    "FORTRAN", // [1152]
    "FORWARD", // [1153]
    "FOUND", // [1154]
    "UNBOUNDED", // [1155]
    "CurrentKeywordSegment", // [1156]
    "UnboundedKeywordSegment", // [1157]
    "PRECEDING", // [1158]
    "PrecedingKeywordSegment", // [1159]
    "FollowingKeywordSegment", // [1160]
    "RangeKeywordSegment", // [1161]
    "FREE", // [1162]
    "FREETEXT", // [1163]
    "FREETEXTTABLE", // [1164]
    "FREEZE", // [1165]
    "FromExpressionSegment", // [1166]
    "from_expression", // [1167]
    "INTERSECT", // [1168]
    "MINUS", // [1169]
    "QUALIFY", // [1170]
    "UNION", // [1171]
    "OrderKeywordSegment", // [1172]
    "QualifyKeywordSegment", // [1173]
    "SetOperatorSegment", // [1174]
    "set_operator", // [1175]
    "WithDataClauseSegment", // [1176]
    "with_data_clause", // [1177]
    "PreTableFunctionKeywordsGrammar", // [1178]
    "TableExpressionSegment", // [1179]
    "table_expression", // [1180]
    "SamplingExpressionSegment", // [1181]
    "TABLESAMPLE", // [1182]
    "sample_expression", // [1183]
    "FromClauseTerminatorGrammar", // [1184]
    "JoinLikeClauseGrammar", // [1185]
    "PostTableExpressionGrammar", // [1186]
    "MLTableExpressionSegment", // [1187]
    "ML", // [1188]
    "ml_table_expression", // [1189]
    "FromExpressionElementSegment", // [1190]
    "from_expression_element", // [1191]
    "indented_joins", // [1192]
    "JoinClauseSegment", // [1193]
    "JOIN", // [1194]
    "NATURAL", // [1195]
    "join_clause", // [1196]
    "OVERWRITE", // [1197]
    "OverwriteKeywordSegment", // [1198]
    "InsertSourceGrammar", // [1199]
    "FromInsertFromExpressionElementSegment", // [1200]
    "FromInsertFromExpressionSegment", // [1201]
    "FromInsertSourceClauseSegment", // [1202]
    "from_insert_source_clause", // [1203]
    "FromInsertClauseSegment", // [1204]
    "from_insert_clause", // [1205]
    "FULLTEXT", // [1206]
    "StarSegment", // [1207]
    "LEADING", // [1208]
    "TRAILING", // [1209]
    "TrimParametersGrammar", // [1210]
    "FunctionContentsExpressionGrammar", // [1211]
    "AggregateOrderByClause", // [1212]
    "aggregate_order_by", // [1213]
    "IgnoreRespectNullsGrammar", // [1214]
    "IGNORE", // [1215]
    "RESPECT", // [1216]
    "EmptyStructLiteralSegment", // [1217]
    "QuotedIdentifierSegment", // [1218]
    "AnyKeywordSegment", // [1219]
    "FunctionParameterGrammar", // [1220]
    "DatePartFunctionNameSegment", // [1221]
    "DateTimeFunctionContentsSegment", // [1222]
    "ColumnsExpressionGrammar", // [1223]
    "ColumnsExpressionFunctionNameSegment", // [1224]
    "ValuesClauseSegment", // [1225]
    "values_clause", // [1226]
    "PostFunctionGrammar", // [1227]
    "FUSION", // [1228]
    "G", // [1229]
    "GENERAL", // [1230]
    "GenerateKeywordSegment", // [1231]
    "SYMLINK_FORMAT_MANIFEST", // [1232]
    "symlink_format_manifest", // [1233]
    "GET", // [1234]
    "glob_operator", // [1235]
    "GO", // [1236]
    "GOTO", // [1237]
    "GrantKeywordSegment", // [1238]
    "AccessPermissionsSegment", // [1239]
    "access_permissions", // [1240]
    "AccessObjectSegment", // [1241]
    "access_object", // [1242]
    "AccessTargetSegment", // [1243]
    "access_target", // [1244]
    "OptionKeywordSegment", // [1245]
    "OPTION", // [1246]
    "AdminKeywordSegment", // [1247]
    "CopyKeywordSegment", // [1248]
    "GRANTED", // [1249]
    "GrantedKeywordSegment", // [1250]
    "Current_userKeywordSegment", // [1251]
    "Session_userKeywordSegment", // [1252]
    "SESSION_USER", // [1253]
    "GREATEST", // [1254]
    "CubeRollupClauseSegment", // [1255]
    "cube_rollup_clause", // [1256]
    "GroupingSetsClauseSegment", // [1257]
    "GROUPING", // [1258]
    "grouping_sets_clause", // [1259]
    "WithCubeRollupClauseSegment", // [1260]
    "with_cube_rollup_clause", // [1261]
    "GroupByClauseTerminatorGrammar", // [1262]
    "GroupingKeywordSegment", // [1263]
    "SetsKeywordSegment", // [1264]
    "SETS", // [1265]
    "HANDLER", // [1266]
    "HEADER", // [1267]
    "HEAP", // [1268]
    "HIERARCHY", // [1269]
    "HIGH_PRIORITY", // [1270]
    "HOLD", // [1271]
    "HOLDLOCK", // [1272]
    "HOST", // [1273]
    "HOSTS", // [1274]
    "HOUR_MICROSECOND", // [1275]
    "HOUR_MINUTE", // [1276]
    "HOUR_SECOND", // [1277]
    "YearsKeywordSegment", // [1278]
    "MonthsKeywordSegment", // [1279]
    "DaysKeywordSegment", // [1280]
    "HoursKeywordSegment", // [1281]
    "Date_hourKeywordSegment", // [1282]
    "BucketKeywordSegment", // [1283]
    "IDENTIFIED", // [1284]
    "IDENTITY", // [1285]
    "IDENTITY_INSERT", // [1286]
    "IDENTITYCOL", // [1287]
    "IfKeywordSegment", // [1288]
    "IgnoreKeywordSegment", // [1289]
    "RespectKeywordSegment", // [1290]
    "IMMUTABLE", // [1291]
    "IMPLEMENTATION", // [1292]
    "IMPLICIT", // [1293]
    "IMPORT", // [1294]
    "INCLUDE", // [1295]
    "INCLUDING", // [1296]
    "INDEXES", // [1297]
    "INDICATOR", // [1298]
    "INFILE", // [1299]
    "INFIX", // [1300]
    "INHERIT", // [1301]
    "INHERITS", // [1302]
    "INITIAL", // [1303]
    "INITIALIZE", // [1304]
    "INOUT", // [1305]
    "INPATH", // [1306]
    "INPUT", // [1307]
    "INSENSITIVE", // [1308]
    "LocalKeywordSegment", // [1309]
    "LOCAL", // [1310]
    "DirectoryKeywordSegment", // [1311]
    "RowFormatClauseSegment", // [1312]
    "row_format_clause", // [1313]
    "StoredAsGrammar", // [1314]
    "STORED", // [1315]
    "NameKeywordSegment", // [1316]
    "NAME", // [1317]
    "INSERT_ID", // [1318]
    "INSTANTIABLE", // [1319]
    "INT1", // [1320]
    "INT2", // [1321]
    "INT3", // [1322]
    "INT4", // [1323]
    "INT8", // [1324]
    "INTEGRATIONS", // [1325]
    "INTERSECTION", // [1326]
    "IntervalLiteralSegment", // [1327]
    "interval_literal", // [1328]
    "SignedQuotedLiteralSegment", // [1329]
    "INVOKER", // [1330]
    "NanLiteralSegment", // [1331]
    "NAN", // [1332]
    "UnknownLiteralSegment", // [1333]
    "NormalizedGrammar", // [1334]
    "ISAM", // [1335]
    "ISNULL", // [1336]
    "ISOLATION", // [1337]
    "ITEMS", // [1338]
    "ITERATE", // [1339]
    "JoinKeywordsGrammar", // [1340]
    "indented_using_on", // [1341]
    "JoinOnConditionSegment", // [1342]
    "join_on_condition", // [1343]
    "NaturalJoinKeywordsGrammar", // [1344]
    "JoinKeywordSegment", // [1345]
    "PivotClauseSegment", // [1346]
    "pivot_clause", // [1347]
    "UnpivotClauseSegment", // [1348]
    "unpivot_clause", // [1349]
    "LateralViewClauseSegment", // [1350]
    "lateral_view_clause", // [1351]
    "indented_on_contents", // [1352]
    "InnerKeywordSegment", // [1353]
    "FullKeywordSegment", // [1354]
    "LeftKeywordSegment", // [1355]
    "RightKeywordSegment", // [1356]
    "OuterKeywordSegment", // [1357]
    "OUTER", // [1358]
    "SemiKeywordSegment", // [1359]
    "AntiKeywordSegment", // [1360]
    "K", // [1361]
    "KEY_MEMBER", // [1362]
    "KEY_TYPE", // [1363]
    "KILL", // [1364]
    "LANCOMPILER", // [1365]
    "LAST_INSERT_ID", // [1366]
    "LEAST", // [1367]
    "LEAVE", // [1368]
    "LENGTH", // [1369]
    "LESS", // [1370]
    "LEVEL", // [1371]
    "LikeGrammar", // [1372]
    "EscapeKeywordSegment", // [1373]
    "LikeKeywordSegment", // [1374]
    "IlikeKeywordSegment", // [1375]
    "SOME", // [1376]
    "SomeKeywordSegment", // [1377]
    "RlikeKeywordSegment", // [1378]
    "RegexpKeywordSegment", // [1379]
    "LINENO", // [1380]
    "LINES", // [1381]
    "ListKeywordSegment", // [1382]
    "LISTEN", // [1383]
    "ArrayLiteralSegment", // [1384]
    "array_literal", // [1385]
    "TypedArrayLiteralSegment", // [1386]
    "typed_array_literal", // [1387]
    "ObjectLiteralSegment", // [1388]
    "object_literal", // [1389]
    "RawQuotedLiteralSegment", // [1390]
    "BytesQuotedLiteralSegment", // [1391]
    "LN", // [1392]
    "LoadKeywordSegment", // [1393]
    "InpathKeywordSegment", // [1394]
    "LOCALTIME", // [1395]
    "LOCALTIMESTAMP", // [1396]
    "LocationKeywordSegment", // [1397]
    "LOCATOR", // [1398]
    "LOCK", // [1399]
    "LOCKS", // [1400]
    "LOGICAL", // [1401]
    "LOGIN", // [1402]
    "LOGS", // [1403]
    "LONGBLOB", // [1404]
    "LONGTEXT", // [1405]
    "LOOP", // [1406]
    "LOW_PRIORITY", // [1407]
    "LOWER", // [1408]
    "M", // [1409]
    "MlKeywordSegment", // [1410]
    "MACRO", // [1411]
    "MATCH", // [1412]
    "MATCHED", // [1413]
    "MAX", // [1414]
    "MAX_ROWS", // [1415]
    "MAXEXTENTS", // [1416]
    "MEDIUMBLOB", // [1417]
    "MEDIUMINT", // [1418]
    "MEDIUMTEXT", // [1419]
    "MEMBER", // [1420]
    "WildcardExpressionSegment", // [1421]
    "wildcard_expression", // [1422]
    "MergeKeywordSegment", // [1423]
    "MergeMatchedClauseSegment", // [1424]
    "merge_when_matched_clause", // [1425]
    "MergeNotMatchedClauseSegment", // [1426]
    "merge_when_not_matched_clause", // [1427]
    "MergeNotMatchedBySourceClauseSegment", // [1428]
    "merge_when_not_matched_by_source_clause", // [1429]
    "MatchedKeywordSegment", // [1430]
    "ThenKeywordSegment", // [1431]
    "THEN", // [1432]
    "MergeUpdateClauseSegment", // [1433]
    "merge_update_clause", // [1434]
    "MergeDeleteClauseSegment", // [1435]
    "merge_delete_clause", // [1436]
    "SourceKeywordSegment", // [1437]
    "SOURCE", // [1438]
    "TargetKeywordSegment", // [1439]
    "TARGET", // [1440]
    "MergeInsertClauseSegment", // [1441]
    "merge_insert_clause", // [1442]
    "MergeIntoLiteralGrammar", // [1443]
    "AliasedTableReferenceGrammar", // [1444]
    "MergeMatchSegment", // [1445]
    "merge_match", // [1446]
    "SetClauseListSegment", // [1447]
    "set_clause_list", // [1448]
    "MESSAGE_LENGTH", // [1449]
    "MESSAGE_OCTET_LENGTH", // [1450]
    "MESSAGE_TEXT", // [1451]
    "MIDDLEINT", // [1452]
    "MIN", // [1453]
    "MIN_ROWS", // [1454]
    "MINUTE_MICROSECOND", // [1455]
    "MINUTE_SECOND", // [1456]
    "MLSLABEL", // [1457]
    "MOD", // [1458]
    "MODE", // [1459]
    "MODIFIES", // [1460]
    "MODULE", // [1461]
    "MONTHNAME", // [1462]
    "MORE", // [1463]
    "MOVE", // [1464]
    "MsckKeywordSegment", // [1465]
    "RepairKeywordSegment", // [1466]
    "REPAIR", // [1467]
    "SYNC", // [1468]
    "SyncKeywordSegment", // [1469]
    "MULTISET", // [1470]
    "MUMPS", // [1471]
    "MYISAM", // [1472]
    "naked_identifier", // [1473]
    "semi_structured_element", // [1474]
    "WindowSpecificationSegment", // [1475]
    "window_specification", // [1476]
    "NamedWindowExpressionSegment", // [1477]
    "named_window_expression", // [1478]
    "NAMES", // [1479]
    "NAMESPACE", // [1480]
    "NAMESPACES", // [1481]
    "null_literal", // [1482]
    "NATIONAL", // [1483]
    "NaturalKeywordSegment", // [1484]
    "NCHAR", // [1485]
    "NCLOB", // [1486]
    "sign_indicator", // [1487]
    "NESTING", // [1488]
    "newline", // [1489]
    "NewlineSegment", // [1490]
    "NO_WRITE_TO_BINLOG", // [1491]
    "NOAUDIT", // [1492]
    "NOCHECK", // [1493]
    "NOCOMPRESS", // [1494]
    "NOCREATEDB", // [1495]
    "NOCREATEROLE", // [1496]
    "NOCREATEUSER", // [1497]
    "NOINHERIT", // [1498]
    "NOLOGIN", // [1499]
    "UnorderedSelectStatementSegment", // [1500]
    "WithCompoundStatementSegment", // [1501]
    "with_compound_statement", // [1502]
    "NonSetSelectableGrammar", // [1503]
    "BracketedSetExpressionGrammar", // [1504]
    "UpdateStatementSegment", // [1505]
    "update_statement", // [1506]
    "InsertStatementSegment", // [1507]
    "insert_statement", // [1508]
    "DeleteStatementSegment", // [1509]
    "delete_statement", // [1510]
    "MergeStatementSegment", // [1511]
    "merge_statement", // [1512]
    "InsertOverwriteDirectorySegment", // [1513]
    "insert_overwrite_directory_statement", // [1514]
    "SetExpressionSegment", // [1515]
    "NONCLUSTERED", // [1516]
    "NONE", // [1517]
    "NORMALIZE", // [1518]
    "NORMALIZED", // [1519]
    "NOSUPERUSER", // [1520]
    "RawNotSegment", // [1521]
    "not_operator", // [1522]
    "NOTHING", // [1523]
    "NOTIFY", // [1524]
    "NOTNULL", // [1525]
    "NOWAIT", // [1526]
    "NULLABLE", // [1527]
    "NULLIF", // [1528]
    "NUMBER", // [1529]
    "ColonSegment", // [1530]
    "ObjectLiteralElementSegment", // [1531]
    "object_literal_element", // [1532]
    "start_curly_bracket", // [1533]
    "CastOperatorSegment", // [1534]
    "StartSquareBracketSegment", // [1535]
    "StartBracketSegment", // [1536]
    "OBJECTS", // [1537]
    "OCTET_LENGTH", // [1538]
    "OCTETS", // [1539]
    "OFF", // [1540]
    "OFFLINE", // [1541]
    "OFFSET", // [1542]
    "OffsetKeywordSegment", // [1543]
    "OFFSETS", // [1544]
    "OIDS", // [1545]
    "ONLINE", // [1546]
    "OPEN", // [1547]
    "OPENDATASOURCE", // [1548]
    "OPENQUERY", // [1549]
    "OPENROWSET", // [1550]
    "OPENXML", // [1551]
    "OPERATION", // [1552]
    "OPERATOR", // [1553]
    "OPTIMIZE", // [1554]
    "OPTIONALLY", // [1555]
    "RefreshKeywordSegment", // [1556]
    "WithFillSegment", // [1557]
    "with_fill", // [1558]
    "NoorderKeywordSegment", // [1559]
    "ORDERING", // [1560]
    "ORDINALITY", // [1561]
    "OTHERS", // [1562]
    "OUT", // [1563]
    "OUTFILE", // [1564]
    "OUTPUT", // [1565]
    "OverKeywordSegment", // [1566]
    "OVER", // [1567]
    "OverlapsKeywordSegment", // [1568]
    "OVERLAY", // [1569]
    "OVERRIDING", // [1570]
    "OWNER", // [1571]
    "PACK_KEYS", // [1572]
    "PAD", // [1573]
    "PARAMETER", // [1574]
    "parameter", // [1575]
    "?", // [1576]
    "PARAMETER_MODE", // [1577]
    "PARAMETER_NAME", // [1578]
    "PARAMETER_ORDINAL_POSITION", // [1579]
    "PARAMETER_SPECIFIC_CATALOG", // [1580]
    "PARAMETER_SPECIFIC_NAME", // [1581]
    "PARAMETER_SPECIFIC_SCHEMA", // [1582]
    "PARAMETERS", // [1583]
    "PARTIAL", // [1584]
    "FieldKeywordSegment", // [1585]
    "IcebergTransformationSegment", // [1586]
    "iceberg_transformation", // [1587]
    "PartitionedKeywordSegment", // [1588]
    "PASCAL", // [1589]
    "PASSWORD", // [1590]
    "PATH", // [1591]
    "PCTFREE", // [1592]
    "PERCENT", // [1593]
    "PERCENT_RANK", // [1594]
    "PERCENTILE_CONT", // [1595]
    "PERCENTILE_DISC", // [1596]
    "PERCENTLIT", // [1597]
    "pipe", // [1598]
    "PLACING", // [1599]
    "PLAN", // [1600]
    "PLI", // [1601]
    "POSITION", // [1602]
    "OverClauseSegment", // [1603]
    "over_clause", // [1604]
    "FilterClauseGrammar", // [1605]
    "POSTFIX", // [1606]
    "POWER", // [1607]
    "PRECISION", // [1608]
    "PREFIX", // [1609]
    "PREORDER", // [1610]
    "PREPARE", // [1611]
    "PREPARED", // [1612]
    "PRESERVE", // [1613]
    "PrimaryKeywordSegment", // [1614]
    "BooleanKeywordSegment", // [1615]
    "TinyintKeywordSegment", // [1616]
    "LongKeywordSegment", // [1617]
    "SmallintKeywordSegment", // [1618]
    "IntKeywordSegment", // [1619]
    "IntegerKeywordSegment", // [1620]
    "BigintKeywordSegment", // [1621]
    "FloatKeywordSegment", // [1622]
    "RealKeywordSegment", // [1623]
    "DoubleKeywordSegment", // [1624]
    "StringKeywordSegment", // [1625]
    "CharKeywordSegment", // [1626]
    "CharacterKeywordSegment", // [1627]
    "VarcharKeywordSegment", // [1628]
    "BracketedArguments", // [1629]
    "bracketed_arguments", // [1630]
    "DecimalKeywordSegment", // [1631]
    "DecKeywordSegment", // [1632]
    "NumericKeywordSegment", // [1633]
    "BinaryKeywordSegment", // [1634]
    "VariantKeywordSegment", // [1635]
    "PRINCIPALS", // [1636]
    "PRINT", // [1637]
    "PROC", // [1638]
    "PROCEDURAL", // [1639]
    "PROCESS", // [1640]
    "PROCESSLIST", // [1641]
    "PROPERTIES", // [1642]
    "properties_naked_identifier", // [1643]
    "PropertyNameSegment", // [1644]
    "property_name_identifier", // [1645]
    "PropertiesNakedIdentifierSegment", // [1646]
    "PropertyGrammar", // [1647]
    "QUOTE", // [1648]
    "quoted_literal", // [1649]
    "RAID0", // [1650]
    "RAISERROR", // [1651]
    "RANK", // [1652]
    "raw_comparison_operator", // [1653]
    "RAW", // [1654]
    "raw_quoted_literal", // [1655]
    "READS", // [1656]
    "READTEXT", // [1657]
    "RECHECK", // [1658]
    "RECONFIGURE", // [1659]
    "RECORDREADER", // [1660]
    "RECORDWRITER", // [1661]
    "RECURSIVE", // [1662]
    "REDUCE", // [1663]
    "REF", // [1664]
    "ReferenceMatchGrammar", // [1665]
    "ReferentialActionGrammar", // [1666]
    "MatchKeywordSegment", // [1667]
    "SIMPLE", // [1668]
    "PartialKeywordSegment", // [1669]
    "SimpleKeywordSegment", // [1670]
    "ActionKeywordSegment", // [1671]
    "REGR_AVGX", // [1672]
    "REGR_AVGY", // [1673]
    "REGR_COUNT", // [1674]
    "REGR_INTERCEPT", // [1675]
    "REGR_R2", // [1676]
    "REGR_SLOPE", // [1677]
    "REGR_SXX", // [1678]
    "REGR_SXY", // [1679]
    "REGR_SYY", // [1680]
    "REINDEX", // [1681]
    "RELATIVE", // [1682]
    "RELEASE", // [1683]
    "RELOAD", // [1684]
    "RemoveKeywordSegment", // [1685]
    "REPEAT", // [1686]
    "REPEATABLE", // [1687]
    "REPLICATION", // [1688]
    "REQUIRE", // [1689]
    "ResetKeywordSegment", // [1690]
    "RESIGNAL", // [1691]
    "WHL", // [1692]
    "WhlKeywordSegment", // [1693]
    "ResourceFileGrammar", // [1694]
    "RESTART", // [1695]
    "RestoreKeywordSegment", // [1696]
    "VERSION", // [1697]
    "TimestampAsOfGrammar", // [1698]
    "VersionAsOfGrammar", // [1699]
    "RESULT", // [1700]
    "RETAIN", // [1701]
    "RETURN", // [1702]
    "RETURNED_CARDINALITY", // [1703]
    "RETURNED_LENGTH", // [1704]
    "RETURNED_OCTET_LENGTH", // [1705]
    "RETURNED_SQLSTATE", // [1706]
    "RETURNS", // [1707]
    "RevokeKeywordSegment", // [1708]
    "ROLES", // [1709]
    "ROLLBACK", // [1710]
    "ROUTINE_CATALOG", // [1711]
    "ROUTINE_NAME", // [1712]
    "ROUTINE_SCHEMA", // [1713]
    "DelimitedKeywordSegment", // [1714]
    "TerminatedByGrammar", // [1715]
    "TERMINATED", // [1716]
    "EscapedKeywordSegment", // [1717]
    "CollectionKeywordSegment", // [1718]
    "ItemsKeywordSegment", // [1719]
    "LinesKeywordSegment", // [1720]
    "DefinedKeywordSegment", // [1721]
    "ROW_COUNT", // [1722]
    "ROW_NUMBER", // [1723]
    "ROWCOUNT", // [1724]
    "ROWGUIDCOL", // [1725]
    "ROWID", // [1726]
    "ROWNUM", // [1727]
    "RULE", // [1728]
    "RUN", // [1729]
    "dash", // [1730]
    "V", // [1731]
    "sql_conf_option", // [1732]
    "TablesampleKeywordSegment", // [1733]
    "PercentKeywordSegment", // [1734]
    "OutKeywordSegment", // [1735]
    "SAVE", // [1736]
    "SAVEPOINT", // [1737]
    "SCALE", // [1738]
    "SCD", // [1739]
    "SCHEMA_NAME", // [1740]
    "SCOPE", // [1741]
    "SCOPE_CATALOG", // [1742]
    "SCOPE_NAME", // [1743]
    "SCOPE_SCHEMA", // [1744]
    "SCROLL", // [1745]
    "SEARCH", // [1746]
    "SECOND_MICROSECOND", // [1747]
    "SECTION", // [1748]
    "SECURITY", // [1749]
    "/*+", // [1750]
    "SelectHintSegment", // [1751]
    "select_hint", // [1752]
    "TransformClauseSegment", // [1753]
    "TRANSFORM", // [1754]
    "transform_clause", // [1755]
    "SelectClauseModifierSegment", // [1756]
    "select_clause_modifier", // [1757]
    "SelectClauseElementSegment", // [1758]
    "select_clause_element", // [1759]
    "SelectClauseTerminatorGrammar", // [1760]
    "StartHintSegment", // [1761]
    "HintFunctionSegment", // [1762]
    "hint_function", // [1763]
    "EndHintSegment", // [1764]
    "SelectClauseSegment", // [1765]
    "select_clause", // [1766]
    "HavingClauseSegment", // [1767]
    "having_clause", // [1768]
    "NamedWindowSegment", // [1769]
    "named_window", // [1770]
    "QualifyClauseSegment", // [1771]
    "qualify_clause", // [1772]
    "OffsetClauseSegment", // [1773]
    "offset_clause", // [1774]
    "FetchClauseSegment", // [1775]
    "fetch_clause", // [1776]
    "ClusterByClauseSegment", // [1777]
    "cluster_by_clause", // [1778]
    "DistributeByClauseSegment", // [1779]
    "distribute_by_clause", // [1780]
    "SortByClauseSegment", // [1781]
    "sort_by_clause", // [1782]
    "WithCompoundNonSelectStatementSegment", // [1783]
    "NonWithSelectableGrammar", // [1784]
    "SELF", // [1785]
    "NakedSemiStructuredElementSegment", // [1786]
    "QuotedSemiStructuredElementSegment", // [1787]
    "statement_terminator", // [1788]
    "SENSITIVE", // [1789]
    "SEPARATED", // [1790]
    "MaxvalueKeywordSegment", // [1791]
    "MinvalueKeywordSegment", // [1792]
    "SERIALIZABLE", // [1793]
    "SERVER_NAME", // [1794]
    "SESSION", // [1795]
    "SetClauseSegment", // [1796]
    "set_clause", // [1797]
    "MinusKeywordSegment", // [1798]
    "UnionKeywordSegment", // [1799]
    "IntersectKeywordSegment", // [1800]
    "SQLConfPropertiesSegment", // [1801]
    "SetConfigValueSegment", // [1802]
    "set_config_value", // [1803]
    "VAR", // [1804]
    "VARIABLE", // [1805]
    "VarKeywordSegment", // [1806]
    "VariableKeywordSegment", // [1807]
    "SETMINUS", // [1808]
    "SETOF", // [1809]
    "SETUSER", // [1810]
    "SHARES", // [1811]
    "DatabasesKeywordSegment", // [1812]
    "SYSTEM", // [1813]
    "SystemKeywordSegment", // [1814]
    "TblpropertiesKeywordSegment", // [1815]
    "BracketedPropertyNameListGrammar", // [1816]
    "ShowDatabasesSchemasGrammar", // [1817]
    "ShowFunctionsGrammar", // [1818]
    "ShowTablesGrammar", // [1819]
    "ShowViewsGrammar", // [1820]
    "ShowKeywordSegment", // [1821]
    "ShowObjectGrammar", // [1822]
    "SHUTDOWN", // [1823]
    "SIGNAL", // [1824]
    "signed_quoted_literal", // [1825]
    "PositiveSegment", // [1826]
    "NegativeSegment", // [1827]
    "SIMILAR", // [1828]
    "SingleQuotedIdentifierSegment", // [1829]
    "SIZE", // [1830]
    "SKEWED", // [1831]
    "SkewedKeywordSegment", // [1832]
    "StoredKeywordSegment", // [1833]
    "DirectoriesKeywordSegment", // [1834]
    "slash", // [1835]
    "slice", // [1836]
    "SONAME", // [1837]
    "SortedKeywordSegment", // [1838]
    "SPACE", // [1839]
    "SPATIAL", // [1840]
    "SPECIFIC_NAME", // [1841]
    "SPECIFICTYPE", // [1842]
    "SQL", // [1843]
    "SQL_BIG_RESULT", // [1844]
    "SQL_BIG_SELECTS", // [1845]
    "SQL_BIG_TABLES", // [1846]
    "SQL_CALC_FOUND_ROWS", // [1847]
    "SQL_LOG_OFF", // [1848]
    "SQL_LOG_UPDATE", // [1849]
    "SQL_LOW_PRIORITY_UPDATES", // [1850]
    "SQL_SELECT_LIMIT", // [1851]
    "SQL_SMALL_RESULT", // [1852]
    "SQL_WARNINGS", // [1853]
    "SQLCA", // [1854]
    "SQLCODE", // [1855]
    "SQLERROR", // [1856]
    "SQLEXCEPTION", // [1857]
    "SQLSTATE", // [1858]
    "SQLWARNING", // [1859]
    "SQRT", // [1860]
    "SSL", // [1861]
    "STABLE", // [1862]
    "star", // [1863]
    "start_hint", // [1864]
    "STARTING", // [1865]
    "STARTS", // [1866]
    "STATE", // [1867]
    "DropTableStatementSegment", // [1868]
    "drop_table_statement", // [1869]
    "DropViewStatementSegment", // [1870]
    "drop_view_statement", // [1871]
    "CreateUserStatementSegment", // [1872]
    "create_user_statement", // [1873]
    "DropUserStatementSegment", // [1874]
    "drop_user_statement", // [1875]
    "TruncateStatementSegment", // [1876]
    "truncate_table", // [1877]
    "AccessStatementSegment", // [1878]
    "access_statement", // [1879]
    "CreateTableStatementSegment", // [1880]
    "create_table_statement", // [1881]
    "CreateRoleStatementSegment", // [1882]
    "create_role_statement", // [1883]
    "DropRoleStatementSegment", // [1884]
    "drop_role_statement", // [1885]
    "AlterTableStatementSegment", // [1886]
    "alter_table_statement", // [1887]
    "DropSchemaStatementSegment", // [1888]
    "drop_schema_statement", // [1889]
    "DropTypeStatementSegment", // [1890]
    "drop_type_statement", // [1891]
    "CreateDatabaseStatementSegment", // [1892]
    "create_database_statement", // [1893]
    "DropDatabaseStatementSegment", // [1894]
    "drop_database_statement", // [1895]
    "CreateIndexStatementSegment", // [1896]
    "create_index_statement", // [1897]
    "DropIndexStatementSegment", // [1898]
    "drop_index_statement", // [1899]
    "CreateViewStatementSegment", // [1900]
    "create_view_statement", // [1901]
    "CreateCastStatementSegment", // [1902]
    "create_cast_statement", // [1903]
    "DropCastStatementSegment", // [1904]
    "drop_cast_statement", // [1905]
    "CreateFunctionStatementSegment", // [1906]
    "create_function_statement", // [1907]
    "DropFunctionStatementSegment", // [1908]
    "drop_function_statement", // [1909]
    "DescribeStatementSegment", // [1910]
    "describe_statement", // [1911]
    "UseStatementSegment", // [1912]
    "use_statement", // [1913]
    "ExplainStatementSegment", // [1914]
    "explain_statement", // [1915]
    "CreateSequenceStatementSegment", // [1916]
    "create_sequence_statement", // [1917]
    "AlterSequenceStatementSegment", // [1918]
    "alter_sequence_statement", // [1919]
    "DropSequenceStatementSegment", // [1920]
    "drop_sequence_statement", // [1921]
    "CreateTriggerStatementSegment", // [1922]
    "create_trigger", // [1923]
    "DropTriggerStatementSegment", // [1924]
    "drop_trigger", // [1925]
    "FromInsertStatementSegment", // [1926]
    "from_insert_statement", // [1927]
    "AlterDatabaseStatementSegment", // [1928]
    "alter_database_statement", // [1929]
    "AlterViewStatementSegment", // [1930]
    "alter_view_statement", // [1931]
    "MsckRepairTableStatementSegment", // [1932]
    "msck_repair_table_statement", // [1933]
    "UseDatabaseStatementSegment", // [1934]
    "use_database_statement", // [1935]
    "AddFileSegment", // [1936]
    "add_file_statement", // [1937]
    "AddJarSegment", // [1938]
    "add_jar_statement", // [1939]
    "AnalyzeTableSegment", // [1940]
    "analyze_table_statement", // [1941]
    "CacheTableSegment", // [1942]
    "cache_table", // [1943]
    "ClearCacheSegment", // [1944]
    "clear_cache", // [1945]
    "ListFileSegment", // [1946]
    "list_file_statement", // [1947]
    "ListJarSegment", // [1948]
    "list_jar_statement", // [1949]
    "RefreshStatementSegment", // [1950]
    "refresh_statement", // [1951]
    "ResetStatementSegment", // [1952]
    "reset_statement", // [1953]
    "SetStatementSegment", // [1954]
    "set_statement", // [1955]
    "ShowStatement", // [1956]
    "show_statement", // [1957]
    "UncacheTableSegment", // [1958]
    "uncache_table", // [1959]
    "InsertOverwriteDirectoryHiveFmtSegment", // [1960]
    "insert_overwrite_directory_hive_fmt_statement", // [1961]
    "LoadDataSegment", // [1962]
    "load_data_statement", // [1963]
    "VacuumStatementSegment", // [1964]
    "vacuum_statement", // [1965]
    "DescribeHistoryStatementSegment", // [1966]
    "describe_history_statement", // [1967]
    "DescribeDetailStatementSegment", // [1968]
    "describe_detail_statement", // [1969]
    "GenerateManifestFileStatementSegment", // [1970]
    "generate_manifest_file_statement", // [1971]
    "ConvertToDeltaStatementSegment", // [1972]
    "convert_to_delta_statement", // [1973]
    "RestoreTableStatementSegment", // [1974]
    "restore_table_statement", // [1975]
    "CreateWidgetStatementSegment", // [1976]
    "create_widget_statement", // [1977]
    "RemoveWidgetStatementSegment", // [1978]
    "remove_widget_statement", // [1979]
    "ReplaceTableStatementSegment", // [1980]
    "replace_table_statement", // [1981]
    "SetVariableStatementSegment", // [1982]
    "set_variable_statement", // [1983]
    "STDDEV_POP", // [1984]
    "STDDEV_SAMP", // [1985]
    "STDIN", // [1986]
    "STDOUT", // [1987]
    "StoredByGrammar", // [1988]
    "STORAGE", // [1989]
    "STRAIGHT_JOIN", // [1990]
    "STRATIFY", // [1991]
    "STRICT", // [1992]
    "ConcatSegment", // [1993]
    "StructKeywordSegment", // [1994]
    "StructTypeSchemaSegment", // [1995]
    "struct_type_schema", // [1996]
    "STRUCTURE", // [1997]
    "STYLE", // [1998]
    "SUBCLASS_ORIGIN", // [1999]
    "SUBLIST", // [2000]
    "SUBMULTISET", // [2001]
    "SUBSTR", // [2002]
    "SUBSTRING", // [2003]
    "SUCCESSFUL", // [2004]
    "SUM", // [2005]
    "SUPERUSER", // [2006]
    "SYMMETRIC", // [2007]
    "SYNONYM", // [2008]
    "SYSDATE", // [2009]
    "SYSID", // [2010]
    "SYSTEM_USER", // [2011]
    "NoneKeywordSegment", // [2012]
    "ForeignKeyGrammar", // [2013]
    "TableConstraintSegment", // [2014]
    "table_constraint", // [2015]
    "UsingClauseSegment", // [2016]
    "using_clause", // [2017]
    "BucketSpecGrammar", // [2018]
    "AtSignLiteralSegment", // [2019]
    "TABLE_NAME", // [2020]
    "Expression_A_Unary_Operator_Grammar", // [2021]
    "Expression_C_Grammar", // [2022]
    "Expression_B_Unary_Operator_Grammar", // [2023]
    "TEMPLATE", // [2024]
    "GlobalKeywordSegment", // [2025]
    "TRANSIENT", // [2026]
    "TransientKeywordSegment", // [2027]
    "TERMINATE", // [2028]
    "TerminatedKeywordSegment", // [2029]
    "TEXTSIZE", // [2030]
    "THAN", // [2031]
    "tilde", // [2032]
    "WITHOUT", // [2033]
    "WithoutKeywordSegment", // [2034]
    "ZoneKeywordSegment", // [2035]
    "ZONE", // [2036]
    "AtKeywordSegment", // [2037]
    "TIMEZONE_HOUR", // [2038]
    "TIMEZONE_MINUTE", // [2039]
    "TINYBLOB", // [2040]
    "TINYTEXT", // [2041]
    "TOAST", // [2042]
    "TOP", // [2043]
    "TOP_LEVEL_COUNT", // [2044]
    "TOUCH", // [2045]
    "TRACK", // [2046]
    "TRAN", // [2047]
    "TRANSACTION", // [2048]
    "BeginKeywordSegment", // [2049]
    "CommitKeywordSegment", // [2050]
    "RollbackKeywordSegment", // [2051]
    "WORK", // [2052]
    "TransactionKeywordSegment", // [2053]
    "WorkKeywordSegment", // [2054]
    "ChainKeywordSegment", // [2055]
    "TRANSACTION_ACTIVE", // [2056]
    "TRANSACTIONS", // [2057]
    "TRANSACTIONS_COMMITTED", // [2058]
    "TRANSACTIONS_ROLLED_BACK", // [2059]
    "TransformKeywordSegment", // [2060]
    "TRANSFORMS", // [2061]
    "TRANSLATE", // [2062]
    "TRANSLATION", // [2063]
    "TREAT", // [2064]
    "TRIGGER_CATALOG", // [2065]
    "TRIGGER_NAME", // [2066]
    "TRIGGER_SCHEMA", // [2067]
    "TRIM", // [2068]
    "BothKeywordSegment", // [2069]
    "LeadingKeywordSegment", // [2070]
    "TrailingKeywordSegment", // [2071]
    "TRUSTED", // [2072]
    "TRY_CAST", // [2073]
    "TSEQUAL", // [2074]
    "StructLiteralSegment", // [2075]
    "UESCAPE", // [2076]
    "UID", // [2077]
    "UNARCHIVE", // [2078]
    "UncacheKeywordSegment", // [2079]
    "UNCOMMITTED", // [2080]
    "UnconditionalCrossJoinKeywordsGrammar", // [2081]
    "HorizontalJoinKeywordsGrammar", // [2082]
    "UNDER", // [2083]
    "UNDO", // [2084]
    "UNENCRYPTED", // [2085]
    "UNKNOWN", // [2086]
    "UNLISTEN", // [2087]
    "UNLOCK", // [2088]
    "UNNAMED", // [2089]
    "UNNEST", // [2090]
    "UnpivotKeywordSegment", // [2091]
    "UNPIVOT", // [2092]
    "IncludeKeywordSegment", // [2093]
    "ExcludeKeywordSegment", // [2094]
    "SingleValueColumnUnpivotSegment", // [2095]
    "unpivot_single_column", // [2096]
    "MultiValueColumnUnpivotSegment", // [2097]
    "unpivot_multi_column", // [2098]
    "UnsetKeywordSegment", // [2099]
    "UNSIGNED", // [2100]
    "UNTIL", // [2101]
    "UPDATES", // [2102]
    "UPDATETEXT", // [2103]
    "UPPER", // [2104]
    "USER_DEFINED_TYPE_CATALOG", // [2105]
    "USER_DEFINED_TYPE_CODE", // [2106]
    "USER_DEFINED_TYPE_NAME", // [2107]
    "USER_DEFINED_TYPE_SCHEMA", // [2108]
    "USERS", // [2109]
    "UTC_DATE", // [2110]
    "UTC_TIME", // [2111]
    "UTC_TIMESTAMP", // [2112]
    "VacuumKeywordSegment", // [2113]
    "RetainKeywordSegment", // [2114]
    "DryKeywordSegment", // [2115]
    "RunKeywordSegment", // [2116]
    "VALID", // [2117]
    "VALIDATE", // [2118]
    "VALIDATOR", // [2119]
    "VALUE", // [2120]
    "VAR_POP", // [2121]
    "VAR_SAMP", // [2122]
    "VARBINARY", // [2123]
    "VARCHAR2", // [2124]
    "VARCHARACTER", // [2125]
    "VARIABLES", // [2126]
    "VARYING", // [2127]
    "VERBOSE", // [2128]
    "VersionKeywordSegment", // [2129]
    "VOLATILE", // [2130]
    "WAITFOR", // [2131]
    "WAREHOUSES", // [2132]
    "WEEKDAY", // [2133]
    "indented_then", // [2134]
    "indented_then_contents", // [2135]
    "WHENEVER", // [2136]
    "WHILE", // [2137]
    "whitespace", // [2138]
    "WhitespaceSegment", // [2139]
    "widget_name_identifier", // [2140]
    "WIDTH_BUCKET", // [2141]
    "ExceptClauseSegment", // [2142]
    "select_except_clause", // [2143]
    "PartitionClauseSegment", // [2144]
    "partitionby_clause", // [2145]
    "FrameClauseSegment", // [2146]
    "frame_clause", // [2147]
    "RecursiveKeywordSegment", // [2148]
    "indented_ctes", // [2149]
    "CTEDefinitionSegment", // [2150]
    "common_table_expression", // [2151]
    "NonWithNonSelectableGrammar", // [2152]
    "CubeKeywordSegment", // [2153]
    "RollupKeywordSegment", // [2154]
    "WITHIN", // [2155]
    "WRITETEXT", // [2156]
    "X509", // [2157]
    "XML", // [2158]
    "XOR", // [2159]
    "YAML", // [2160]
    "YEAR_MONTH", // [2161]
    "ZEROFILL", // [2162]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0,
    1, 0, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 10, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 215, 1, 2, 1, 1, 4, 3, 1, 4, 5, 216,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 217, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 218, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 229, 1, 2, 1, 1, 4, 3, 1, 4, 5, 230,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 231, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 83, 244, 245, 1, 244, 3, 3, 4, 246, 247, 248, 245, 1,
    248, 3, 3, 4, 246, 1, 2, 1, 0, 1, 267, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 11, 1, 2, 1, 1, 4, 3, 1, 4, 5, 272, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 273, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 274, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0,
    0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1, 2,
    1, 0, 0, 0, 0, 2, 4294967295, 0, 0, 1, 0, 1, 2, 1, 0, 4294967295,
    4294967295, 0, 1, 0, 1, 2, 1, 0, 1, 4294967295, 4294967295, 0, 0, 0, 0, 1,
    2, 1, 0, 0, 1, 0, 1, 4294967295, 1, 0, 1, 0, 1, 0, 0, 1,
    2, 1, 0, 0, 437, 1, 2, 1, 1, 4, 3, 1, 4, 5, 438, 439,
    245, 1, 439, 3, 3, 4, 246, 440, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 441, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    0, 0, 0, 0, 451, 1, 2, 1, 1, 4, 3, 1, 4, 5, 451, 452,
    453, 1, 452, 3, 3, 452, 4, 249, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 454, 1, 2, 1, 1, 4, 3, 1, 4, 5, 99, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 455, 1, 2, 1, 1, 4, 3, 1, 4, 5, 456,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 210, 476, 245,
    1, 476, 3, 3, 4, 246, 477, 478, 245, 1, 478, 3, 3, 4, 246, 1,
    2, 0, 320, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2,
    0, 459, 481, 245, 1, 481, 3, 3, 4, 246, 460, 482, 245, 1, 482, 3,
    3, 4, 246, 1, 2, 0, 239, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    423, 1, 2, 1, 1, 4, 3, 1, 4, 5, 483, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 484, 1, 2, 1, 1, 4, 3, 1, 4, 5, 485, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 486, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 488, 488, 490, 3,
    488, 489, 488, 3, 3, 489, 4, 492, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 493, 1, 2, 1, 1, 4, 3, 1, 4, 5, 494, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 495, 1, 2, 1, 1, 4, 3, 1, 4, 5, 496,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 497, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 500, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 501, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 502, 503, 505, 3, 503, 504, 502, 3, 3, 504, 4, 507, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 508, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    509, 510, 511, 688, 3, 512, 513, 1, 512, 2, 3, 4, 0, 517, 518, 0,
    2, 3, 517, 3, 519, 0, 1, 3, 520, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 521, 1, 2, 1, 1, 4, 3, 1, 4, 5, 522, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 523, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    356, 1, 2, 1, 1, 4, 3, 1, 4, 5, 357, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 452, 453, 9, 3, 3, 452, 4, 536, 537, 2, 1,
    537, 4, 3, 1, 4, 5, 538, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    539, 1, 2, 1, 1, 4, 3, 1, 4, 5, 540, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 462, 452,
    245, 1, 452, 3, 3, 4, 246, 546, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 547, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 358, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 552, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 2, 1, 1, 2,
    1, 555, 556, 0, 2, 3, 555, 1, 2, 1, 560, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 561, 1, 2, 1, 1, 4, 3, 1, 4, 5, 562, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 563, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 571, 1, 2, 1, 1, 4, 3, 1, 4, 5, 572, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 295, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    332, 573, 490, 3, 573, 489, 332, 3, 3, 489, 4, 331, 573, 490, 3, 573,
    489, 331, 3, 3, 489, 4, 0, 1, 2, 1, 1, 2, 1, 284, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 581, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 583, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 585,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295,
    0, 0, 586, 1, 2, 1, 1, 4, 3, 1, 4, 5, 600, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 601, 602, 245, 1, 602, 3, 3, 4, 246, 12,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 603, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 604, 1, 2, 1, 1, 4, 3, 1, 4, 5, 605, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 606, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 354, 1, 2, 1, 1, 4, 3, 1, 4, 5, 607, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 359, 1, 2, 1, 1, 4, 3, 1, 4, 5, 608,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 360, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 609, 1, 2, 1, 1, 4, 3, 1, 4, 5, 610, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 611, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 612, 1, 2, 1, 1, 4, 3, 1, 4, 5, 613, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 614, 1, 2, 1, 1, 4, 3, 1, 4, 5, 413,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 615, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 616, 1, 2, 1, 1, 4, 3, 1, 4, 5, 617, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 618, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 619, 1, 2, 1, 1, 4, 3, 1, 4, 5, 620, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 621, 1, 2, 1, 1, 4, 3, 1, 4, 5, 623,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 624, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 625, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    564, 1, 2, 1, 1, 4, 3, 1, 4, 5, 637, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 513,
    9, 2, 3, 4, 639, 1, 2, 1, 1, 4, 3, 1, 4, 5, 640, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 641, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 0, 644, 1, 2, 1, 1, 4, 3, 1, 4, 5, 645,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 646, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 647, 1, 2, 1, 1, 4, 3, 1, 4, 5, 648, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 209, 649, 245, 1, 649, 3, 3, 4, 246,
    209, 650, 245, 1, 650, 3, 3, 4, 246, 209, 651, 245, 1, 651, 3, 3,
    4, 246, 0, 1, 2, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 1, 2,
    1, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 310, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 350,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 88, 677, 245, 1, 677, 3, 3,
    4, 246, 678, 1, 2, 1, 1, 4, 3, 1, 4, 5, 679, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 361, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    680, 681, 10, 3, 3, 680, 4, 682, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 683, 1, 2, 1, 1, 4, 3, 1, 4, 5, 684, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    316, 697, 9, 3, 3, 316, 4, 698, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 699, 1, 2, 1, 1, 4, 3, 1, 4, 5, 452, 700, 0, 2, 3,
    452, 316, 701, 0, 2, 3, 316, 702, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 444, 1, 2, 1, 1, 4, 3, 1, 4, 5, 703, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 704, 1, 2, 1, 1, 4, 3, 1, 4, 5, 705,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 100, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 710, 1, 2, 1, 1, 4, 3, 1, 4, 5, 711, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 411, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 0, 720, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    721, 1, 2, 1, 1, 4, 3, 1, 4, 5, 722, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 723, 1, 2, 1, 1, 4, 3, 1, 4, 5, 724, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 725, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 727, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 728, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    729, 1, 2, 1, 1, 4, 3, 1, 4, 5, 742, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 743, 1, 2, 1, 1, 4, 3, 1, 4, 5, 744, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 745, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 747, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 748, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    3, 4, 1, 0, 0, 0, 1, 0, 1, 2, 1, 101, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 0, 1, 0, 1, 2, 1,
    0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0,
    1, 2, 1, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0,
    1, 2, 1, 0, 0, 839, 1, 2, 1, 1, 4, 3, 1, 4, 5, 840,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 841, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 250, 1, 2, 1, 1, 4, 3, 1, 4, 5, 734, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 842, 843, 513, 1, 843, 2, 3, 4, 842,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 849, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 850, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 851, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 853,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 854, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 510, 1, 2, 1, 1, 4, 3, 1, 4, 5, 855, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 511, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 856, 1, 2, 1, 1, 4, 3, 1, 4, 5, 857, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 53,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 13, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 873, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 874, 513, 1, 874, 2, 3, 4, 0, 1, 2, 0, 0, 321, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 882, 883, 884, 885, 886, 887, 2295, 6, 843,
    513, 1, 843, 2, 3, 4, 0, 1, 0, 1, 2, 1, 0, 223, 923, 490,
    3, 923, 489, 223, 3, 3, 489, 4, 924, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 925, 1, 2, 1, 1, 4, 3, 1, 4, 5, 890, 891, 892, 893,
    894, 895, 896, 897, 898, 899, 900, 901, 902, 903, 904, 905, 906, 907, 908, 909,
    910, 911, 912, 913, 914, 915, 2348, 26, 926, 513, 1, 926, 2, 3, 4, 927,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 928, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 890, 1, 2, 1, 1, 4, 3, 1, 4, 5, 929, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 930, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 931, 1, 2, 1, 1, 4, 3, 1, 4, 5, 932, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 933, 1, 2, 1, 1, 4, 3, 1, 4, 5, 934,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 891, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 892, 1, 2, 1, 1, 4, 3, 1, 4, 5, 935, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 280, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 936, 1, 2, 1, 1, 4, 3, 1, 4, 5, 362, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 937,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 654, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 939, 1, 2, 1, 1, 4, 3, 1, 4, 5, 795, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 808, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 940, 1, 2, 1, 1, 4, 3, 1, 4, 5, 941, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 942, 1, 2, 1, 1, 4, 3, 1, 4, 5, 943,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 944, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 102, 1, 2, 1, 1, 4, 3, 1, 4, 5, 950, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 951, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 954, 1, 2, 1, 1, 4, 3, 1, 4, 5, 955, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 732, 1, 2, 1, 1, 4, 3, 1, 4, 5, 956,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 957, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 958, 1, 2, 1, 1, 4, 3, 1, 4, 5, 959, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 960, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 424, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 961, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 2, 4294967295, 0, 1, 0, 0, 0,
    1, 0, 993, 1, 2, 1, 1, 4, 3, 1, 4, 5, 994, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 995, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    964, 1, 2, 1, 1, 4, 3, 1, 4, 5, 996, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 997, 1, 2, 1, 1, 4, 3, 1, 4, 5, 998, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 999, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1000, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1001, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1002, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1003, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1004, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1005, 1, 2, 1, 1, 4, 3, 1, 4, 5, 696, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1006, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 972, 1, 2, 1, 1, 4, 3, 1, 4, 5, 416,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 461, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 220, 452, 245, 1, 452, 3, 3, 4, 246, 1011, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    208, 1012, 245, 1, 1012, 3, 3, 4, 246, 364, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 3, 4, 1, 0, 307, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 0, 833, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1017, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1018, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1019, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1020, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1021, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1022, 1, 2, 1, 1, 4, 3, 1, 4, 5, 810, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1023, 1, 2, 1, 1, 4, 3, 1, 4, 5, 594, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1024, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 1, 1027, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1028,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1029, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1030, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1031, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1032, 1033, 245, 1, 1033, 3, 3, 4, 246,
    1034, 1035, 245, 1, 1035, 3, 3, 4, 246, 596, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1037, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 527, 316, 697, 1, 316, 3, 3, 316, 4,
    526, 316, 697, 1, 316, 3, 3, 316, 4, 1039, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1040, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1041, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1042, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    1043, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1045, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1046, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1047, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1048, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1049, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1050, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 103, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    135, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1051, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1052, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1053, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1054, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 713, 1, 2, 1, 1, 4, 3, 1, 4, 5, 973, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1060, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2,
    1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 970, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1111, 1, 2, 1, 1, 4, 3, 1, 4, 5, 15, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 716,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 322, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 322, 1113, 1114, 1, 1113, 5, 3, 1, 489, 4, 5, 0, 0,
    0, 0, 1115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1124, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 434, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 16, 17, 3633, 2, 1133, 2, 1, 1133, 4, 3, 1, 4, 5, 222, 489,
    490, 2, 489, 222, 3, 3, 489, 4, 5, 1135, 1136, 2, 1135, 5, 3, 3,
    4, 5, 1, 0, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 4294967295, 4294967295, 0, 1,
    0, 0, 4294967295, 4294967295, 0, 407, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1139,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1140, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1143, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    338, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1144, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1145, 1, 2, 1, 1, 4, 3, 1, 4, 5, 365,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1146, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1148, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 447, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 18, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 178, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1056,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1152, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1153, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1154, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 1162, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1163, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1164, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1165, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0,
    235, 1192, 1, 0, 0, 4294967295, 4294967295, 0, 265, 1192, 1, 1, 2, 1, 0, 0,
    0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 235, 1192, 1, 0, 0, 4294967295,
    4294967295, 0, 265, 1192, 1, 1, 2, 1, 0, 1, 0, 1, 4294967295, 4294967295, 0, 92,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 251, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1206, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1,
    0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1, 19, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 5, 843, 1136, 2, 843, 5, 3, 3, 4, 5, 0,
    4294967295, 4294967295, 0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295,
    0, 1, 0, 1, 2, 1, 0, 0, 68, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 20, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1229, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1230, 1, 2, 1, 1, 4, 3, 1, 4, 5, 974, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1232, 1233, 513, 1, 1233, 2, 3, 4, 0, 655, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1234, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1235, 1235, 697, 3, 1235, 316, 1235, 3, 3, 316, 4, 764, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1236, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 190, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 1249, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1254, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 0, 1, 0, 0, 0, 0, 0, 196,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1258,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1266,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 0, 0, 630,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1267, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1268, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1269, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1270, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 966, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1271, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1272, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1273,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1274, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 894, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1275, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1276, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1277, 1, 2, 1, 1, 4, 3, 1, 4, 5, 895, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 2, 1, 0, 1, 2, 1, 0, 1284, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 504, 505, 9, 3,
    3, 504, 4, 1285, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1286, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1287, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1215, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1064, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1291, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1292, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1293, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1294, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 63, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 2, 1, 0, 1295, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1296, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 286, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 770, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1297, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1298, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1299, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1300, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1301, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1302, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1303, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1304, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 796, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1305, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1306, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1307, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 859, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1308, 1, 2, 1, 1, 4, 3, 1, 4, 5, 105, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 1, 4294967295, 4294967295, 0, 0,
    1, 4294967295, 4294967295, 0, 1, 0, 0, 0, 0, 0, 1318, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 753, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1319, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 789, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1320, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1321, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1322, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1324, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 366, 1, 2, 1, 1, 4, 3, 1, 4, 5, 367, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 21, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1168, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1326, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 4294967295, 0, 0, 323, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    569, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1330, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1335,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1336, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1338, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1339, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 227, 228, 5105, 2, 1133, 2, 1, 1133, 4, 3, 1, 4, 5, 865, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 235, 1341, 1, 235, 1341, 0, 1, 0,
    1, 2, 1, 265, 1341, 0, 0, 265, 1341, 1, 0, 1194, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 588, 1352, 1, 1, 2, 1, 0, 265, 1352,
    1, 0, 0, 0, 0, 1, 0, 1, 2, 1, 736, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 860, 537, 2, 1, 537, 4, 3, 1, 4, 5, 1361, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1151, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1362, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1363, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 253, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 9, 4, 3, 1, 4, 5, 1364, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1365, 1, 2, 1, 1, 4, 3, 1, 4, 5, 22, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 93, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    429, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1366, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 254, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 0, 0, 578, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1208, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1367, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 255, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1370,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1371, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 0, 1065, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    530, 530, 697, 3, 530, 316, 530, 3, 3, 316, 4, 0, 628, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1381, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295,
    4294967295, 0, 975, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1383, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 489, 1114, 9, 5, 3, 1, 489, 4, 5,
    489, 490, 9, 3, 3, 489, 4, 785, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1392, 1, 2, 1, 1, 4, 3, 1, 4, 5, 976, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1310, 1, 2, 1, 1, 4, 3, 1, 4, 5, 417,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1395, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1396, 1, 2, 1, 1, 4, 3, 1, 4, 5, 281, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1398, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1399, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1400, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1401, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1402,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1403, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 368, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1404, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1405, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1406, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1407, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1408, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1409,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2, 3, 1, 1411,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 106, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 369, 1, 2, 1, 1, 4, 3, 1, 4, 5, 23, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1412, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1413, 1, 2, 1, 1, 4, 3, 1, 4, 5, 24, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1414, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1415,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1416, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1417, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1418, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1419, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1420, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 977, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 4294967295, 4294967295, 0, 0, 0, 0, 1, 2, 1, 0, 235, 1341, 1, 265, 1341,
    1, 0, 1449, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1450, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1451, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1452, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 898, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1453, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1454, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 319, 452, 245, 1,
    452, 3, 3, 4, 246, 900, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1455,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1456, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 288, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1188, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1457, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1459, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 776, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1460,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 107, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 457, 452, 245,
    1, 452, 3, 3, 4, 246, 41, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    904, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1462, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 905, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1463, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1464, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1,
    2, 1, 1, 0, 2, 3, 1, 1, 0, 2, 3, 1, 458, 452, 245, 1,
    452, 3, 3, 4, 246, 1470, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1471,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1472, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2, 3, 1473, 505, 1, 1473, 3, 3, 504, 4, 4, 4294967295, 1474,
    513, 1, 1474, 2, 3, 4, 1317, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 1, 0, 1479, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1480, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1481, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1332, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1332, 1482,
    1114, 1, 1482, 5, 3, 1, 489, 4, 5, 1483, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1195, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1485, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1486, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 319, 1487, 245, 1, 1487, 3, 3, 4, 246, 1488, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 802, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1489, 1490,
    12, 3, 3, 1489, 4, 1117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 289,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1491, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1492, 1, 2, 1, 1, 4, 3, 1, 4, 5, 290, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1493, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1495, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1496, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1497,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 291, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1498, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1499, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 1, 2, 1,
    0, 0, 1, 2, 1, 0, 0, 1516, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 292, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1518, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1519,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 448, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 383, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 383, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 524, 1522, 513, 1, 1522, 2, 3, 4, 0, 1523, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1524, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1525,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1526, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 324, 1, 2, 1, 1, 4, 3, 1, 4, 5, 324, 1482, 1114,
    1, 1482, 5, 3, 1, 489, 4, 5, 1527, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1528, 1, 2, 1, 1, 4, 3, 1, 4, 5, 427, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1529, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    370, 1, 2, 1, 1, 4, 3, 1, 4, 5, 98, 98, 490, 3, 98, 489,
    98, 3, 3, 489, 4, 96, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 330, 1533, 245, 1, 1533, 3, 3, 4, 246, 1, 2, 0, 0, 0, 1537,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1538, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1539, 1, 2, 1, 1, 4, 3, 1, 4, 5, 793, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1540, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1541, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1542, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1544, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1545, 1, 2, 1, 1, 4, 3, 1, 4, 5, 800, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 165, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1546,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1120, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1547, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1548, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1549, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1550, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1551, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1552,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1553, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1246, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1555, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 580, 1, 2, 1, 1, 4, 3, 1, 4, 5, 529, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 529, 452, 453, 1, 452, 3, 3, 452, 4, 737, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 233, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 418, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1560, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1561, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1562, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1358, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1564, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1565,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1132, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 1567, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 3, 4, 1, 0, 1109, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1569,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1570, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1197, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1571, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 109, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1573, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1574, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5,
    4294967295, 1575, 513, 1, 1575, 2, 3, 4, 1576, 1575, 245, 1, 1575, 3, 3, 4,
    246, 1577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1578, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1580,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1581, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1582, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 738, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1584, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    0, 0, 1, 0, 0, 1, 0, 347, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 1, 0, 1, 2, 1, 348, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 394, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1589, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1590, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1591, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 1135, 1136, 2, 1135, 5,
    3, 3, 4, 5, 1, 0, 0, 1592, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1593, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1594, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1595, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1596,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1597, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 138, 1, 2, 1, 1, 4, 3, 1, 4, 5, 463, 1598, 245,
    1, 1598, 3, 3, 4, 246, 1, 0, 1, 0, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 1, 0, 0, 1, 0, 1, 2, 1, 7, 8, 1, 256,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1599, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1600, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1601, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 318, 452, 245, 1, 452, 3, 3, 4, 246,
    129, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1602, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 318, 1487, 245, 1, 1487, 3, 3, 4, 246, 0, 1606, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1607, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1608, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1609, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1610, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1611, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1612, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1613, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 656, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 1636, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1637,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1078, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 143, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1638, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1639, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 25, 1, 2, 1, 1, 4, 3, 1, 4, 5, 69, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1640, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1641,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1642, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 6, 4294967295, 1643, 505, 1, 1643, 3, 3, 504, 4, 1, 0, 1,
    2, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 207, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 389, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 2, 1, 0, 1170, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    2, 1, 906, 1, 2, 1, 1, 4, 3, 1, 4, 5, 979, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1648, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    221, 503, 505, 3, 503, 504, 221, 3, 3, 504, 4, 223, 1649, 490, 3, 1649,
    489, 223, 3, 3, 489, 4, 221, 1649, 490, 3, 1649, 489, 221, 3, 3, 489,
    4, 0, 223, 1474, 513, 3, 1474, 4, 223, 2, 3, 4, 1650, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1651, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    632, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1652, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 315, 1653, 245, 1, 1653, 3, 3, 4, 246, 460, 1653, 245,
    1, 1653, 3, 3, 4, 246, 1654, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    459, 1653, 245, 1, 1653, 3, 3, 4, 246, 524, 1653, 245, 1, 1653, 3, 3,
    4, 246, 334, 1655, 490, 3, 1655, 489, 334, 3, 3, 489, 4, 333, 1655, 490,
    3, 1655, 489, 333, 3, 3, 489, 4, 0, 861, 537, 2, 1, 537, 4, 3,
    1, 4, 5, 110, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1656, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1657, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1658, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1661, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 391, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1662, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1663, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1664, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0,
    112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 111, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 797, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 980,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1066, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1673, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1674, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1676, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1677, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1678, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1679, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1680, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1681, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1682, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1683, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1684, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 981, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    344, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1467, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1686, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1687, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 385, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1688, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1689, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 982, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1691, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 26, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1216, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1695, 1, 2, 1, 1, 4, 3, 1, 4, 5, 983, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1013, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1700, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1701, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1702, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1703,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1704, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1705, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1706, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1707, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 191, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 525, 452, 245, 1,
    452, 3, 3, 4, 246, 257, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1067,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 27, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1709, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1710, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 844, 843, 513, 1, 843, 2, 3, 4, 844,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1711, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1712, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1713, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 70, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 719, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1722, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1723, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1724, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1725, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1726, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1727, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 633, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1728, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1729, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    319, 1730, 245, 1, 1730, 3, 3, 4, 246, 1731, 1732, 245, 1, 1732, 3, 3,
    4, 246, 0, 2, 3, 1, 5, 6, 1, 0, 1736, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1737, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1738, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1739, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1740, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 61, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1741, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1742, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1743, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1744, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1745, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1746, 1, 2, 1, 1, 4, 3, 1, 4, 5, 908, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1747, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1748, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1749, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 1, 0, 0, 2, 9150, 2, 9152, 3, 528, 696,
    528, 383, 696, 0, 1, 4294967295, 4294967295, 0, 1, 0, 113, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0,
    1785, 1, 2, 1, 1, 4, 3, 1, 4, 5, 258, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 2, 0, 0, 0, 1, 2, 0, 0, 0, 4294967295,
    4294967295, 0, 953, 1788, 245, 1, 1788, 3, 3, 4, 246, 1789, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 635,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 0, 862, 537, 2, 1, 537, 4, 3, 1, 4, 5, 71,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 399, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 400, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1793, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 54, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1794, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1795, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 0, 1, 0, 1, 2, 1, 1, 1, 0, 1, 4294967295, 4294967295, 0, 279, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 2, 1, 1, 0,
    0, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 0, 1808, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1809, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1265, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1810, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 202, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1811, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0, 0, 0, 0,
    984, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1823, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1824, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 223, 1825, 490, 3, 1825, 489, 223, 3, 3, 489, 4, 221, 1825, 490,
    3, 1825, 489, 221, 3, 3, 489, 4, 0, 0, 1828, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1668, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 223, 503, 505, 3, 503, 504, 223, 3, 3, 504, 4, 1, 0, 2, 3,
    1, 1830, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    0, 1, 0, 1, 2, 1, 1831, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    220, 1835, 245, 1, 1835, 3, 3, 4, 246, 209, 1836, 245, 1, 1836, 3, 3,
    4, 246, 372, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1376, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1837, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 1, 0, 1009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 0, 1, 2, 1, 567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1438,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1839, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1840, 1, 2, 1, 1, 4, 3, 1, 4, 5, 752, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1841, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1842, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1843, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1844, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1845,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1846, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1847, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1848, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1849, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1850, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1851, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1852, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1853,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1854, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1855, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1856, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1857, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1858, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1859, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1860, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1861,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1862, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 31, 1, 2, 1, 1, 4, 3, 1, 4, 5, 72, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 458, 1863, 245, 1, 1863, 3, 3, 4, 246,
    1750, 1864, 245, 1, 1864, 3, 3, 4, 246, 780, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1865, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1866, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1867, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 755, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 446, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1984, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1985, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1986, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1987, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1989, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1315, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1990, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1991, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 32, 1, 2, 1, 1, 4, 3, 1, 4, 5, 786, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 73, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1992,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 373, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 0, 1997, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1998, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1999, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2000, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2001, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2002, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2003, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2004, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2005, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2006, 1, 2, 1, 1, 4, 3, 1, 4, 5, 246, 245, 9, 3,
    3, 4, 246, 2007, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1468, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2008, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2009, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2010, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1813, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2011,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 4294967295, 1, 0, 1, 2, 1, 0, 0, 0,
    0, 1, 2, 1, 0, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2020,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1182, 1, 2, 1, 1, 4, 3, 1, 4, 5, 34, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 1440,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 35, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 75, 1, 2, 1, 1, 4, 3, 1, 4, 5, 396, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 114, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2024, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 115, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 2028, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1716, 1, 2, 1, 1, 4, 3, 1, 4, 5, 739, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 863, 537, 2, 1, 537, 4, 3, 1, 4, 5, 2030,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2031, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1432, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1123, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1079, 2032, 245, 1, 2032, 3, 3, 4, 246,
    325, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0,
    0, 326, 1, 2, 1, 1, 4, 3, 1, 4, 5, 327, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 328, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2038,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2039, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2040, 1, 2, 1, 1, 4, 3, 1, 4, 5, 375, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2041, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 90, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2042, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2043, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2044,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2045, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2046, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1209, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2047, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2048, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 2056, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2057, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2058, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2059, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0, 1, 0,
    1, 2, 1, 1754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2061, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2026, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2062, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2063, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2064, 1, 2, 1, 1, 4, 3, 1, 4, 5, 116,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2065, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2066, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2067, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2068, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 329, 1, 2, 1, 1, 4, 3, 1, 4, 5, 329, 1113, 1114, 1,
    1113, 5, 3, 1, 489, 4, 5, 117, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2073, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2074, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 36, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2076, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2077, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2078, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1155, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 985, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2080, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2083, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2084, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2085,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1171, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 657, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2086, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2087, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2088, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2089, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2090, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 4294967295, 4294967295, 0, 0, 0, 1, 2, 1, 2092, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 398, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2100, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2101, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2102, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2103, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2104, 1, 2, 1, 1, 4, 3, 1, 4, 5, 119, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 121, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 124, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2106, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2107, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2109, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2110, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2111, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2112, 1, 2, 1, 1, 4, 3, 1, 4, 5, 986, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 2117, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2118, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2119, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2120, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1, 0, 1, 2, 1, 0, 1, 1, 0, 0, 558, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1804, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2121,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2122, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2124, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 376, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1805, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2126, 1, 2, 1, 1, 4, 3, 1, 4, 5, 377,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2127, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1697, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 37, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 76, 1, 2, 1, 1, 4, 3, 1, 4, 5, 715, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2131,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 38, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2132, 1, 2, 1, 1, 4, 3, 1, 4, 5, 910, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2133, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 235, 2134, 1, 588, 2135, 1, 265, 2135, 1, 265, 2134, 1, 590, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2136, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 0, 0, 948, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2138, 2139, 12, 3, 3,
    4, 2138, 1692, 1133, 2, 1, 1133, 4, 3, 1, 4, 5, 832, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 7, 4294967295, 2140, 513, 1, 2140, 2, 3, 4, 2141,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 259, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 235, 2149, 1, 1, 0, 265, 2149,
    1, 235, 2149, 1, 1, 0, 265, 2149, 1, 0, 405, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2155, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2033, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 5, 1136, 9, 3, 3, 4, 5, 2052,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 58, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 122, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2156, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2157, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2158, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2159, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2160, 1, 2, 1, 1, 4, 3, 1, 4, 5, 912,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2161, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2162, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2036, 1, 2, 1, 1, 4, 3, 1, 4,
    5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 46, 9, 0, 42, 0, 39, 40, 42, 43, 44, 45, 46,
    47, 48, 49, 50, 51, 0, 52, 40, 55, 0, 56, 57, 0, 59, 60, 62,
    43, 0, 64, 60, 62, 43, 65, 0, 59, 67, 62, 47, 0, 64, 67, 62,
    41, 43, 47, 44, 43, 78, 0, 80, 82, 85, 87, 89, 91, 0, 94, 95,
    97, 52, 0, 123, 47, 49, 125, 42, 43, 45, 0, 126, 127, 128, 0, 130,
    131, 0, 132, 133, 0, 40, 48, 136, 137, 0, 51, 0, 123, 49, 47, 0,
    127, 128, 139, 65, 0, 140, 50, 47, 48, 0, 141, 142, 126, 144, 123, 145,
    130, 146, 147, 40, 148, 149, 150, 151, 152, 153, 154, 155, 156, 157, 158, 137,
    159, 160, 0, 59, 142, 161, 53, 162, 164, 55, 166, 167, 168, 169, 170, 171,
    172, 173, 131, 47, 0, 127, 128, 0, 174, 167, 0, 175, 166, 0, 176, 177,
    56, 179, 180, 181, 182, 183, 184, 185, 186, 187, 60, 0, 127, 188, 0, 174,
    180, 0, 175, 179, 0, 176, 189, 57, 192, 194, 70, 0, 197, 58, 78, 0,
    125, 60, 198, 0, 49, 62, 200, 0, 203, 64, 78, 66, 200, 68, 204, 206,
    71, 211, 213, 75, 85, 95, 105, 0, 219, 176, 115, 224, 225, 0, 219, 226,
    119, 224, 225, 123, 133, 143, 153, 232, 0, 235, 238, 185, 0, 241, 181, 242,
    163, 172, 241, 184, 260, 261, 262, 263, 264, 91, 265, 186, 0, 268, 270, 196,
    206, 216, 0, 275, 226, 43, 47, 276, 278, 227, 282, 283, 228, 240, 0, 293,
    294, 97, 296, 297, 238, 0, 298, 97, 299, 239, 300, 301, 302, 0, 275, 172,
    303, 241, 305, 0, 308, 309, 311, 241, 250, 0, 313, 314, 245, 317, 335, 0,
    246, 219, 147, 309, 336, 248, 0, 247, 339, 340, 341, 161, 343, 0, 345, 249,
    346, 89, 268, 0, 275, 166, 268, 302, 0, 345, 89, 268, 0, 349, 345, 89,
    349, 0, 345, 309, 341, 89, 341, 0, 219, 251, 351, 309, 257, 254, 252, 0,
    352, 353, 0, 258, 275, 355, 309, 260, 341, 259, 378, 51, 379, 339, 340, 278,
    308, 51, 379, 381, 353, 0, 264, 278, 308, 382, 384, 0, 386, 351, 267, 265,
    0, 336, 381, 0, 308, 279, 0, 309, 311, 341, 0, 351, 311, 276, 274, 270,
    341, 0, 219, 387, 280, 349, 388, 0, 308, 311, 284, 349, 388, 0, 390, 0,
    386, 388, 0, 392, 393, 0, 278, 395, 397, 0, 349, 278, 285, 0, 401, 402,
    0, 403, 224, 404, 0, 349, 278, 406, 408, 0, 349, 278, 283, 0, 286, 219,
    308, 410, 341, 412, 412, 287, 414, 0, 160, 294, 0, 419, 294, 420, 0, 421,
    422, 294, 292, 0, 341, 290, 425, 426, 0, 428, 291, 339, 430, 0, 278, 431,
    433, 298, 0, 341, 0, 308, 431, 433, 300, 0, 341, 0, 275, 167, 268, 307,
    0, 345, 89, 268, 0, 278, 395, 397, 0, 346, 306, 303, 435, 308, 318, 327,
    337, 0, 442, 355, 0, 166, 268, 349, 443, 445, 353, 449, 0, 450, 351, 352,
    349, 347, 341, 0, 179, 0, 354, 91, 62, 276, 443, 445, 449, 356, 366, 375,
    385, 395, 405, 415, 425, 464, 465, 466, 467, 468, 469, 470, 471, 472, 473, 474,
    447, 427, 426, 97, 414, 475, 429, 438, 0, 450, 462, 460, 479, 0, 480, 483,
    379, 465, 474, 0, 346, 486, 496, 506, 516, 526, 536, 546, 556, 567, 577, 587,
    597, 607, 0, 498, 617, 627, 637, 647, 657, 668, 678, 691, 700, 317, 514, 515,
    204, 341, 414, 0, 379, 317, 87, 346, 701, 707, 712, 722, 732, 742, 752, 762,
    772, 531, 532, 533, 534, 535, 773, 780, 790, 800, 810, 820, 542, 0, 543, 543,
    544, 0, 545, 545, 830, 839, 849, 859, 548, 549, 860, 870, 550, 551, 871, 883,
    881, 317, 888, 886, 341, 891, 553, 894, 554, 897, 903, 557, 906, 916, 926, 936,
    0, 565, 566, 568, 97, 570, 946, 956, 966, 998, 976, 987, 999, 242, 0, 241,
    574, 346, 1002, 576, 1005, 0, 298, 577, 166, 268, 579, 0, 346, 576, 1015, 1025,
    1035, 1045, 1055, 1073, 0, 587, 588, 1065, 591, 593, 595, 597, 595, 595, 0, 587,
    414, 1069, 591, 593, 595, 597, 595, 595, 534, 87, 599, 1074, 1084, 1094, 1103, 1113,
    1123, 1133, 1143, 1153, 1163, 0, 1173, 1183, 1193, 1203, 1213, 1223, 1233, 1243, 1253, 1263,
    1273, 1283, 1293, 1303, 1313, 1323, 0, 622, 298, 1333, 1343, 1353, 0, 626, 294, 1364,
    0, 1363, 341, 97, 414, 627, 629, 262, 631, 634, 1366, 0, 636, 294, 161, 1376,
    1386, 1396, 1406, 1412, 0, 1422, 1432, 1444, 224, 1442, 241, 642, 643, 1445, 1455, 1465,
    1475, 1485, 1495, 1504, 1513, 1522, 652, 317, 204, 514, 0, 0, 410, 78, 1526, 0,
    382, 384, 0, 412, 1523, 414, 0, 658, 659, 0, 660, 661, 662, 663, 0, 664,
    661, 665, 0, 667, 668, 670, 0, 241, 379, 1527, 0, 1530, 671, 0, 341, 379,
    1534, 0, 1537, 671, 0, 673, 674, 346, 1542, 1541, 204, 514, 1545, 1555, 241, 642,
    643, 1557, 0, 676, 0, 0, 1567, 1577, 1586, 1596, 0, 378, 224, 0, 378, 224,
    1606, 1616, 1623, 1633, 1643, 1653, 1663, 314, 686, 687, 688, 689, 690, 691, 692, 693,
    0, 694, 695, 91, 0, 694, 382, 695, 91, 1664, 1671, 1681, 1691, 1697, 1703, 1713,
    0, 544, 544, 1723, 1733, 1743, 706, 1753, 707, 708, 709, 1754, 1764, 1774, 1784, 0,
    410, 78, 712, 1794, 414, 0, 164, 714, 1797, 0, 717, 158, 0, 308, 718, 1798,
    1808, 1818, 1828, 1838, 1848, 1858, 1868, 1878, 1888, 0, 730, 89, 731, 733, 0, 741,
    445, 349, 1898, 1908, 1918, 1928, 1938, 1948, 1958, 0, 123, 749, 1968, 379, 346, 379,
    750, 751, 1972, 171, 169, 170, 0, 1971, 756, 757, 758, 759, 80, 82, 0, 450,
    78, 0, 346, 760, 0, 123, 1973, 43, 47, 387, 276, 381, 283, 0, 750, 761,
    402, 0, 123, 762, 763, 169, 387, 765, 346, 224, 766, 0, 123, 762, 768, 769,
    387, 771, 164, 268, 0, 1976, 1974, 773, 1979, 0, 123, 762, 775, 387, 78, 0,
    777, 1997, 1995, 0, 313, 314, 1994, 317, 1991, 1989, 224, 346, 576, 0, 123, 49,
    387, 200, 0, 123, 47, 387, 778, 2002, 0, 293, 294, 97, 0, 781, 750, 97,
    296, 297, 2000, 0, 298, 97, 299, 2001, 300, 301, 302, 0, 123, 172, 303, 2003,
    782, 0, 123, 784, 0, 123, 156, 787, 2007, 790, 340, 0, 791, 792, 2011, 2010,
    146, 145, 0, 158, 792, 2008, 341, 794, 164, 794, 164, 164, 268, 2019, 0, 798,
    799, 718, 346, 313, 801, 718, 346, 313, 0, 91, 268, 2014, 0, 382, 803, 0,
    803, 2013, 0, 804, 805, 0, 804, 807, 0, 450, 809, 2015, 718, 812, 0, 813,
    2016, 414, 0, 130, 170, 765, 814, 0, 123, 125, 200, 0, 750, 47, 2023, 817,
    818, 0, 51, 819, 0, 123, 2024, 762, 820, 763, 821, 822, 174, 167, 387, 268,
    0, 2028, 2026, 2025, 0, 341, 379, 381, 823, 0, 825, 408, 579, 2031, 349, 826,
    381, 395, 828, 0, 346, 2035, 2032, 576, 829, 0, 123, 831, 2036, 0, 834, 835,
    836, 0, 837, 435, 0, 838, 835, 836, 2037, 2047, 2057, 2067, 2077, 2087, 2095, 0,
    2105, 845, 846, 2106, 847, 2109, 2119, 2129, 2139, 2149, 2159, 2169, 2179, 2189, 2199, 2209,
    2219, 2229, 2239, 2249, 858, 864, 78, 2250, 866, 867, 868, 869, 870, 731, 867, 871,
    838, 872, 2251, 0, 761, 402, 2261, 2280, 2271, 241, 335, 2284, 875, 877, 0, 879,
    2281, 0, 379, 87, 379, 880, 2285, 2301, 888, 0, 2313, 2311, 2310, 889, 916, 0,
    2316, 917, 918, 919, 920, 921, 922, 2317, 2328, 2338, 2374, 2383, 2393, 2403, 2413, 2423,
    2433, 2443, 2453, 2463, 2473, 2483, 2493, 2503, 2513, 2523, 2533, 2543, 2553, 0, 658, 938,
    2563, 2573, 2583, 2593, 2603, 2613, 2623, 2633, 2643, 0, 145, 945, 947, 2653, 2663, 952,
    2673, 2683, 2693, 2703, 2713, 2723, 2733, 2743, 2753, 0, 962, 963, 2763, 224, 733, 268,
    0, 962, 965, 2764, 224, 733, 268, 967, 2765, 2784, 0, 2775, 43, 47, 969, 276,
    0, 169, 969, 80, 0, 166, 969, 268, 349, 0, 241, 2776, 0, 971, 241, 0,
    987, 2782, 0, 166, 268, 0, 91, 268, 153, 2780, 341, 947, 988, 232, 967, 990,
    2783, 965, 963, 0, 2785, 962, 426, 992, 2786, 2796, 2806, 2816, 2826, 2836, 2846, 2856,
    2866, 2876, 2886, 2896, 2906, 2916, 2926, 2936, 0, 1007, 294, 2947, 0, 2946, 341, 97,
    414, 1008, 627, 629, 262, 631, 634, 2949, 2959, 1010, 2969, 2979, 2988, 2998, 3008, 3017,
    3027, 1014, 1015, 0, 308, 749, 3028, 379, 346, 379, 1016, 0, 308, 3031, 43, 47,
    311, 276, 1016, 0, 308, 763, 169, 311, 80, 0, 308, 769, 311, 771, 1016, 3032,
    0, 308, 775, 311, 78, 0, 308, 49, 311, 241, 0, 308, 47, 311, 778, 1016,
    0, 308, 172, 303, 0, 308, 763, 166, 311, 3042, 268, 1016, 0, 308, 156, 311,
    787, 0, 308, 51, 311, 78, 1016, 0, 308, 125, 311, 200, 0, 308, 167, 311,
    268, 1016, 3044, 3054, 3064, 3074, 3084, 3094, 3104, 3114, 3124, 0, 593, 414, 3134, 3144,
    3154, 0, 880, 1025, 3157, 3167, 3177, 3187, 3197, 3207, 3216, 3225, 3235, 3245, 1038, 3255,
    3264, 3273, 3283, 3293, 3303, 3313, 0, 1044, 3325, 3323, 341, 3328, 3338, 3348, 3358, 3368,
    3378, 3388, 3398, 3408, 3418, 3428, 3438, 3448, 3458, 3468, 0, 1055, 3478, 969, 1057, 1058,
    1059, 990, 3479, 1061, 0, 1062, 3490, 3489, 1063, 0, 599, 1062, 1068, 1069, 0, 694,
    382, 1070, 1071, 1072, 1073, 0, 382, 1074, 1075, 1076, 1062, 0, 1077, 1061, 3494, 1080,
    0, 1081, 1082, 1083, 1084, 0, 1085, 3497, 3496, 0, 3495, 531, 532, 534, 1085, 3501,
    1080, 0, 1081, 1082, 3510, 0, 1086, 3502, 576, 0, 3505, 1087, 1088, 3506, 1090, 652,
    87, 0, 3525, 514, 204, 3515, 3514, 1092, 414, 576, 3512, 3511, 317, 341, 204, 1093,
    414, 1095, 1096, 0, 3518, 880, 1097, 3521, 3519, 414, 0, 379, 3524, 224, 97, 1099,
    1100, 1101, 1093, 1102, 87, 1103, 3526, 435, 317, 515, 1104, 1106, 341, 1108, 3527, 0,
    3537, 3547, 3557, 3567, 3577, 3587, 0, 1116, 3598, 339, 1118, 3599, 97, 414, 718, 3600,
    718, 1119, 3601, 1121, 0, 750, 1122, 3602, 3612, 3622, 3632, 1125, 1126, 1127, 1128, 1129,
    0, 1130, 224, 1131, 224, 3635, 3670, 3646, 0, 1134, 3668, 3666, 3656, 971, 1134, 0,
    1125, 971, 1137, 0, 3671, 1138, 3679, 990, 3675, 1138, 3681, 1138, 3685, 3695, 0, 1141,
    3705, 0, 1142, 414, 3708, 3718, 3728, 0, 3738, 339, 0, 340, 341, 3739, 3749, 3759,
    3769, 3779, 3789, 3799, 3809, 0, 52, 1150, 3819, 3829, 3839, 3849, 3859, 3869, 0, 631,
    3882, 3881, 0, 1156, 718, 0, 3879, 97, 1157, 515, 3880, 1159, 1160, 0, 1074, 1076,
    3883, 1119, 1161, 3884, 3894, 3904, 3914, 0, 91, 3924, 1166, 3926, 1142, 627, 0, 197,
    294, 0, 1172, 294, 0, 626, 294, 0, 1007, 294, 0, 1008, 294, 629, 1173, 1174,
    829, 1176, 264, 0, 1178, 3930, 3927, 1179, 1181, 270, 3931, 1184, 1185, 1186, 3950, 3947,
    0, 3935, 1187, 1190, 3932, 1166, 0, 1172, 294, 0, 197, 294, 3936, 3940, 0, 3939,
    1193, 1185, 0, 1172, 294, 0, 197, 294, 3944, 0, 146, 3951, 568, 1198, 166, 268,
    3952, 0, 349, 161, 1199, 146, 0, 386, 947, 1199, 146, 0, 1178, 3956, 3953, 1179,
    1181, 270, 3957, 146, 1184, 1185, 1186, 3976, 3973, 0, 3961, 1187, 1200, 3958, 1201, 0,
    1172, 294, 0, 197, 294, 146, 3962, 3966, 0, 3965, 1193, 1185, 0, 1172, 294, 0,
    197, 294, 146, 3970, 0, 91, 3977, 1201, 0, 1202, 3979, 1204, 3983, 3993, 4003, 4013,
    414, 1207, 4020, 414, 0, 414, 346, 379, 0, 1210, 414, 91, 91, 414, 0, 4014,
    889, 414, 91, 414, 0, 695, 4017, 1207, 4015, 1211, 1212, 0, 634, 317, 0, 4018,
    224, 241, 341, 62, 4019, 224, 241, 341, 1214, 773, 1217, 0, 4024, 916, 0, 346,
    224, 0, 46, 335, 4027, 4037, 0, 4047, 0, 241, 971, 556, 4051, 765, 1218, 556,
    4054, 0, 313, 4052, 0, 1219, 51, 379, 4053, 0, 1219, 51, 379, 4057, 4055, 1220,
    4067, 4065, 0, 1220, 4061, 0, 658, 4060, 317, 204, 665, 4071, 0, 1221, 1222, 1223,
    0, 0, 80, 4070, 1221, 1224, 1225, 814, 1227, 4072, 4082, 4092, 4102, 4112, 4122, 0,
    1231, 4132, 450, 166, 4140, 224, 733, 268, 4141, 4151, 4161, 4172, 4182, 4192, 4202, 0,
    1238, 4212, 0, 1239, 164, 1241, 0, 49, 200, 0, 149, 164, 125, 198, 78, 89,
    1243, 4213, 0, 750, 1238, 1245, 0, 750, 1247, 1245, 0, 1248, 1156, 133, 0, 1250,
    294, 4214, 1251, 1252, 198, 4215, 4225, 0, 545, 1038, 545, 4235, 0, 197, 294, 4252,
    4246, 4245, 1255, 1257, 341, 97, 414, 0, 4249, 4248, 341, 97, 414, 4251, 1260, 1257,
    4254, 0, 4253, 1172, 1007, 626, 1008, 294, 627, 629, 262, 4255, 0, 4269, 4268, 341,
    97, 414, 4265, 1262, 4271, 0, 1263, 1264, 4284, 4282, 4281, 1255, 847, 4287, 0, 629,
    4300, 4297, 414, 4302, 0, 4301, 1172, 626, 1007, 1008, 294, 627, 1173, 262, 4303, 4313,
    4323, 4333, 4343, 0, 80, 814, 4353, 4363, 4373, 0, 4383, 4393, 4403, 4413, 4423, 4433,
    4443, 4453, 4471, 0, 4463, 1278, 1279, 1280, 917, 1281, 1282, 4464, 341, 0, 4467, 1283,
    157, 4468, 0, 97, 87, 341, 4472, 4482, 4492, 4499, 4509, 4519, 0, 1288, 1086, 4529,
    0, 1288, 382, 1086, 4539, 0, 4549, 1289, 1290, 428, 4550, 4560, 4570, 4580, 4590, 4600,
    4610, 4620, 0, 382, 62, 4636, 4633, 4632, 4630, 1061, 576, 204, 4637, 4647, 4657, 0,
    241, 4667, 425, 426, 4668, 4678, 4688, 4698, 4708, 4718, 4728, 4738, 4748, 4758, 4768, 4778,
    4788, 4798, 4808, 4818, 4828, 0, 146, 1198, 1309, 1311, 224, 1312, 1314, 4842, 4838, 1225,
    576, 0, 146, 1198, 1309, 1311, 224, 825, 408, 579, 4847, 4843, 1225, 576, 4854, 4848,
    1225, 576, 0, 166, 268, 0, 91, 268, 153, 4852, 341, 947, 988, 232, 967, 0,
    146, 4855, 568, 1198, 166, 268, 4857, 0, 349, 4856, 161, 0, 294, 1316, 1199, 0,
    386, 947, 1199, 0, 386, 825, 161, 1199, 4858, 4868, 4878, 4888, 4898, 4908, 4918, 4928,
    4938, 4948, 4958, 4968, 4978, 4988, 4998, 0, 920, 5012, 5008, 1327, 224, 5013, 0, 1080,
    5023, 97, 1329, 889, 89, 889, 5024, 5034, 5044, 1100, 1331, 1333, 1099, 1334, 0, 694,
    382, 695, 91, 5045, 0, 5055, 5065, 5075, 5085, 5095, 5107, 5118, 5146, 0, 261, 1340,
    1190, 5128, 5142, 1342, 0, 825, 5131, 5136, 5134, 241, 5139, 5143, 0, 1344, 1340, 1190,
    5147, 0, 1345, 0, 5157, 1346, 1348, 1350, 270, 5158, 1184, 1185, 0, 164, 5159, 5165,
    5162, 414, 5166, 5170, 706, 1353, 0, 5169, 1354, 1355, 1356, 1357, 0, 1355, 1359, 0,
    1355, 1360, 5172, 1353, 0, 5171, 1354, 1355, 1356, 1357, 0, 825, 5175, 5173, 241, 5178,
    5188, 5198, 5208, 5218, 5228, 5238, 5248, 5256, 5266, 5276, 5286, 5296, 5306, 0, 414, 270,
    5316, 0, 87, 260, 167, 1357, 204, 5330, 0, 241, 0, 346, 5326, 241, 0, 346,
    5328, 241, 5331, 5341, 5351, 5361, 5371, 5381, 5391, 0, 543, 1038, 543, 5401, 0, 0,
    382, 1372, 1061, 0, 1373, 1062, 5413, 0, 5411, 1374, 1375, 5412, 59, 1219, 1377, 1378,
    1379, 5414, 5424, 0, 627, 5435, 97, 59, 204, 5436, 5446, 5456, 0, 0, 1382, 176,
    5466, 224, 225, 0, 1382, 226, 5470, 224, 225, 5474, 5484, 5494, 224, 97, 1099, 1081,
    1100, 1101, 1384, 1386, 1388, 1390, 1391, 5495, 5504, 5511, 5521, 0, 1393, 56, 1309, 1394,
    224, 1198, 568, 166, 268, 349, 5531, 0, 5541, 5551, 5561, 5571, 0, 1397, 224, 5581,
    5591, 5601, 5611, 5621, 5631, 5641, 5651, 5661, 5671, 5681, 5691, 5701, 5711, 0, 1410, 971,
    241, 5724, 0, 775, 78, 0, 87, 5721, 576, 5727, 5737, 5747, 0, 5757, 0, 5767,
    5777, 5787, 5797, 5807, 5817, 5827, 5837, 5847, 5857, 5867, 145, 0, 146, 5877, 1421, 0,
    161, 1225, 0, 1423, 568, 5878, 5888, 1424, 1426, 1428, 0, 813, 1430, 0, 1076, 414,
    1431, 5892, 1433, 1435, 0, 813, 382, 1430, 294, 1437, 0, 1076, 414, 1431, 5893, 1433,
    1435, 0, 813, 382, 1430, 0, 294, 1439, 0, 1076, 414, 1431, 1441, 0, 1443, 5894,
    268, 1444, 825, 5898, 268, 1444, 0, 5895, 576, 270, 0, 204, 270, 5899, 1342, 5902,
    1445, 0, 158, 5905, 0, 278, 1421, 0, 1447, 5906, 5916, 5926, 5936, 5946, 5956, 5966,
    5976, 5986, 5996, 6005, 6015, 6025, 6035, 6045, 6055, 6065, 6075, 6085, 6095, 6105, 6115, 6125,
    6134, 6144, 6154, 6164, 6174, 6184, 6194, 0, 1465, 1466, 166, 268, 0, 6204, 219, 308,
    1469, 393, 0, 6207, 6205, 241, 450, 241, 62, 6217, 6215, 0, 6212, 6210, 341, 270,
    6220, 6229, 6239, 6249, 6259, 6269, 6278, 0, 241, 346, 6291, 241, 6288, 1475, 0, 262,
    6292, 1477, 6294, 6304, 6314, 6324, 6334, 6345, 0, 1484, 261, 6355, 6365, 6375, 6385, 0,
    6394, 6404, 6414, 6421, 6431, 6441, 6451, 6461, 6471, 6481, 6491, 6501, 6511, 6521, 6531, 6541,
    6560, 1225, 1500, 6551, 435, 6554, 1501, 6557, 1503, 1504, 0, 6561, 1505, 1507, 1509, 1511,
    1513, 6566, 1515, 6565, 6562, 435, 1503, 6567, 6577, 6587, 6597, 0, 6607, 6617, 6627, 0,
    6637, 0, 1521, 1038, 0, 543, 545, 6638, 0, 6666, 6648, 6658, 6667, 6677, 6687, 6697,
    6707, 6717, 6728, 6738, 6748, 6758, 6768, 6778, 6789, 0, 224, 1530, 479, 6810, 6799, 1531,
    6801, 6813, 971, 0, 971, 971, 6814, 164, 346, 825, 87, 1534, 1535, 1536, 599, 1138,
    1185, 6815, 6825, 6835, 6845, 6855, 6865, 0, 1543, 6875, 97, 414, 718, 6876, 718, 1119,
    6877, 6887, 6897, 6907, 6917, 6927, 6937, 6947, 6957, 6967, 6977, 6987, 6997, 7007, 7017, 7027,
    7037, 7047, 0, 777, 402, 7057, 7067, 7077, 0, 794, 1556, 0, 794, 386, 7086, 0,
    1172, 294, 7099, 0, 7096, 341, 97, 414, 7097, 425, 426, 0, 428, 7098, 339, 430,
    1557, 967, 631, 7101, 967, 629, 1173, 262, 631, 634, 1116, 7102, 7112, 1172, 1559, 7113,
    7123, 7133, 7143, 7153, 7163, 7173, 7183, 7193, 0, 1214, 1566, 7206, 241, 7203, 1475, 7207,
    0, 1568, 7220, 0, 7217, 1101, 87, 1101, 341, 7221, 7231, 7241, 7251, 7261, 7271, 7281,
    7291, 7301, 7311, 7320, 7329, 7339, 7349, 7359, 7369, 7379, 7389, 7399, 7409, 0, 420, 294,
    7424, 7421, 7419, 414, 0, 420, 1585, 7426, 7425, 336, 0, 341, 314, 317, 381, 1586,
    0, 750, 7429, 7428, 336, 0, 341, 314, 317, 381, 1586, 0, 346, 335, 7431, 0,
    7441, 420, 0, 1588, 294, 7445, 7443, 7442, 336, 0, 341, 314, 317, 381, 1586, 7448,
    7458, 7468, 7478, 7488, 7510, 0, 1134, 7508, 7498, 1134, 224, 0, 7511, 7521, 7531, 7541,
    7551, 7561, 7571, 7581, 0, 263, 7612, 7590, 0, 479, 270, 450, 7598, 7595, 7594, 241,
    7592, 241, 62, 7609, 7607, 0, 7606, 7601, 7599, 414, 7604, 414, 270, 7615, 7625, 7635,
    7645, 7655, 7664, 7674, 7684, 7693, 1603, 1605, 0, 7694, 7704, 0, 7714, 7724, 7734, 7744,
    7754, 7764, 7774, 0, 1614, 1150, 7784, 7796, 1615, 1616, 1617, 1618, 1619, 1620, 1621, 1622,
    1623, 1624, 917, 919, 921, 922, 1625, 0, 7794, 1626, 1627, 1628, 1629, 0, 7795, 1631,
    1632, 1633, 1629, 1634, 920, 1635, 7797, 7807, 7817, 7827, 7837, 7847, 7857, 7867, 7877, 7887,
    7897, 7907, 0, 1644, 314, 7924, 317, 7919, 7917, 317, 7922, 1646, 971, 7925, 1647, 7927,
    1644, 0, 7931, 7929, 1646, 971, 241, 7932, 7942, 0, 1080, 97, 0, 1173, 7955, 7952,
    414, 7956, 0, 534, 7966, 59, 1219, 1377, 7967, 576, 7970, 7980, 7990, 8000, 8033, 8011,
    8022, 8034, 8044, 8054, 8064, 8074, 8084, 8093, 8102, 8112, 8121, 8152, 8130, 8141, 8153, 8163,
    8173, 8183, 8193, 8203, 8213, 8223, 8233, 8243, 8253, 8263, 8273, 0, 152, 268, 161, 1665,
    8283, 0, 164, 145, 1666, 0, 164, 158, 1666, 0, 1667, 8287, 1354, 1669, 1670, 8288,
    8298, 8308, 8318, 1014, 1015, 0, 278, 384, 0, 741, 1671, 0, 278, 658, 8319, 0,
    1556, 8329, 224, 0, 166, 268, 0, 169, 80, 8330, 8340, 8350, 8360, 8370, 8380, 8390,
    8400, 8410, 8420, 8430, 8440, 8450, 8460, 8470, 0, 1685, 831, 835, 8480, 8490, 8500, 8510,
    8520, 0, 386, 784, 8530, 8540, 8550, 0, 1690, 8560, 241, 971, 8562, 8572, 226, 1693,
    176, 8573, 0, 825, 1694, 224, 8583, 8593, 8603, 0, 1696, 166, 8613, 224, 733, 268,
    89, 8614, 1698, 1699, 8615, 8625, 8635, 8645, 8655, 8665, 8675, 8685, 8695, 8705, 0, 1708,
    0, 1238, 1245, 450, 8715, 0, 1239, 164, 1241, 0, 49, 200, 0, 149, 164, 125,
    198, 198, 91, 1243, 1016, 8716, 8725, 8735, 8745, 241, 8755, 8765, 8775, 8783, 8793, 8803,
    8813, 8823, 8833, 0, 718, 177, 8843, 0, 1714, 0, 433, 1715, 0, 1717, 294, 224,
    0, 1718, 1719, 1715, 0, 879, 264, 1715, 0, 1720, 1715, 0, 384, 1721, 346, 224,
    0, 403, 224, 404, 8844, 8854, 8864, 8874, 8884, 8894, 8904, 8914, 8924, 8934, 0, 8944,
    8953, 0, 1733, 8969, 8963, 97, 8962, 1734, 1119, 8966, 1283, 97, 1735, 792, 97, 8970,
    8980, 8990, 9000, 9010, 9020, 9030, 9040, 9050, 9060, 9070, 9080, 9090, 9100, 9110, 9120, 9130,
    9140, 1421, 0, 479, 270, 0, 1751, 9141, 695, 59, 0, 153, 9144, 1753, 0, 1756,
    9142, 1758, 1760, 9155, 91, 9145, 1142, 0, 1172, 294, 967, 1568, 1174, 1116, 0, 626,
    294, 0, 1007, 294, 0, 1008, 294, 1173, 0, 0, 1761, 9160, 9156, 1762, 1764, 1764,
    9162, 0, 1765, 945, 947, 988, 1767, 1108, 1769, 1771, 232, 1773, 1775, 1777, 1779, 1781,
    967, 1769, 1174, 829, 1176, 9183, 9175, 9172, 1501, 9179, 9176, 1783, 1784, 9180, 576, 9184,
    9194, 0, 9204, 971, 1530, 9208, 1786, 9205, 1787, 211, 9214, 0, 9209, 971, 1530, 9213,
    1786, 9210, 1787, 211, 9218, 9227, 9237, 9247, 9257, 9267, 0, 1791, 97, 0, 741, 1791,
    9268, 0, 1792, 97, 0, 741, 1792, 9269, 9279, 9289, 0, 750, 401, 402, 9299, 9309,
    9319, 9329, 9339, 9349, 0, 278, 9359, 1796, 0, 341, 314, 9361, 317, 514, 204, 341,
    414, 1225, 658, 9369, 317, 9364, 9362, 317, 204, 514, 9367, 1646, 971, 0, 1138, 0,
    1503, 9370, 0, 1174, 1503, 232, 967, 1769, 9374, 9390, 0, 9384, 1044, 1798, 59, 0,
    9385, 1799, 1800, 9386, 695, 59, 0, 1044, 9387, 0, 0, 278, 47, 387, 778, 0,
    278, 1801, 9391, 0, 1644, 314, 1802, 1644, 0, 278, 9392, 1806, 1807, 9398, 9395, 9393,
    241, 314, 9403, 658, 9402, 9399, 414, 9404, 9414, 9424, 9434, 9444, 9454, 0, 9464, 1087,
    1088, 9465, 0, 1534, 379, 1090, 0, 9469, 1812, 60, 0, 1374, 224, 0, 9470, 125,
    1814, 59, 182, 9471, 0, 276, 971, 80, 80, 0, 1374, 224, 9472, 9483, 0, 123,
    166, 1179, 0, 346, 403, 0, 351, 62, 1179, 0, 62, 276, 0, 393, 268, 349,
    0, 166, 969, 0, 9482, 91, 62, 276, 1374, 224, 349, 0, 1815, 268, 1816, 1817,
    1818, 1819, 1820, 0, 1821, 1822, 0, 179, 0, 9484, 91, 62, 276, 0, 1374, 224,
    0, 180, 0, 9485, 91, 62, 276, 0, 1374, 224, 9486, 9496, 9528, 9506, 9517, 9529,
    1826, 1827, 9530, 9540, 9550, 335, 1218, 1829, 1137, 9551, 241, 0, 241, 642, 1207, 9553,
    0, 241, 450, 241, 62, 9566, 9564, 0, 341, 270, 9569, 0, 877, 211, 0, 1832,
    294, 161, 164, 9587, 9585, 9584, 317, 9581, 9579, 317, 0, 1833, 346, 1834, 9590, 9600,
    9609, 9618, 9628, 9638, 0, 1008, 294, 9651, 0, 9648, 341, 97, 414, 9649, 425, 426,
    0, 428, 9650, 339, 430, 627, 629, 1173, 262, 631, 634, 9653, 0, 1838, 294, 9666,
    9664, 0, 341, 9663, 425, 426, 9669, 9679, 9689, 9699, 9709, 9719, 9729, 9739, 9749, 9759,
    9769, 9779, 9789, 9799, 9809, 9819, 9829, 9839, 9849, 9859, 9869, 9879, 9889, 9899, 9909, 9919,
    9929, 9939, 9949, 9959, 9968, 9977, 9987, 9997, 10007, 10017, 10027, 576, 1511, 1507, 1868, 1870,
    1872, 1874, 1876, 1878, 1880, 1882, 1884, 1886, 1888, 1890, 1892, 1894, 1896, 1898, 1900, 1509,
    1505, 1902, 1904, 1906, 1908, 1910, 1912, 1914, 1916, 1918, 1920, 1922, 1924, 1926, 1928, 1886,
    1930, 1880, 1932, 1934, 1936, 1938, 1940, 1942, 1944, 1946, 1948, 1950, 1952, 1954, 1956, 1958,
    1513, 1960, 1962, 1777, 1779, 1964, 1966, 1968, 1970, 1972, 1974, 823, 1976, 1978, 1980, 1982,
    1138, 10028, 10038, 10048, 10058, 10068, 10078, 10088, 0, 1312, 1314, 1988, 10089, 0, 1833, 346,
    858, 0, 1833, 294, 224, 404, 10099, 10109, 10119, 10129, 10139, 10149, 10159, 10169, 1993, 10170,
    10180, 10192, 10190, 0, 479, 270, 10197, 10195, 0, 241, 1530, 379, 381, 0, 1994, 1995,
    10200, 10210, 10220, 10230, 10240, 10250, 10260, 10270, 10280, 10290, 10300, 10307, 10317, 10327, 10337, 10347,
    10357, 10367, 0, 626, 294, 10377, 161, 2012, 0, 0, 410, 78, 10378, 0, 768, 161,
    0, 660, 161, 0, 2013, 161, 664, 0, 10379, 762, 820, 763, 175, 821, 822, 166,
    387, 10380, 733, 268, 10389, 10385, 10383, 10382, 0, 10381, 352, 2014, 381, 823, 0, 1374,
    10388, 733, 268, 2016, 10390, 1312, 1314, 381, 579, 349, 2018, 283, 381, 395, 826, 0,
    346, 10397, 10394, 576, 0, 10404, 1225, 514, 204, 0, 10398, 733, 268, 10400, 2019, 0,
    10399, 1698, 1699, 10401, 576, 10405, 0, 1815, 402, 10415, 10425, 10435, 10445, 0, 10455, 2021,
    599, 2022, 0, 10459, 2023, 2022, 10463, 10473, 10483, 10493, 10503, 10513, 0, 0, 0, 2025,
    10523, 154, 155, 10524, 10534, 2027, 763, 10535, 0, 2029, 294, 224, 10545, 10555, 10565, 10575,
    10585, 10595, 10605, 10615, 10624, 0, 10634, 918, 919, 1629, 0, 10635, 750, 2034, 918, 2035,
    10636, 0, 2037, 918, 2035, 414, 0, 919, 346, 792, 10640, 224, 514, 204, 10641, 10651,
    10661, 10671, 10681, 10691, 10701, 10711, 10721, 10731, 10741, 10751, 10761, 10771, 10781, 10791, 10801, 0,
    10811, 781, 2049, 2050, 2051, 595, 10812, 2053, 2054, 0, 1316, 241, 0, 1076, 741, 2055,
    10813, 10823, 10833, 10843, 0, 2060, 10855, 10853, 241, 1312, 825, 224, 0, 346, 10864, 10862,
    10858, 241, 379, 1312, 10867, 10877, 10887, 10897, 10907, 10917, 10927, 10937, 10947, 10957, 10967, 10977,
    2069, 2070, 2071, 10978, 10988, 10999, 0, 157, 166, 268, 349, 11009, 11019, 11029, 11041, 11039,
    479, 11044, 0, 877, 1384, 0, 880, 2075, 11054, 11064, 11074, 11084, 11094, 0, 2079, 166,
    311, 268, 11104, 0, 11114, 1344, 2081, 2082, 11115, 11125, 11135, 0, 1799, 11145, 695, 59,
    11146, 0, 768, 11156, 11166, 0, 11176, 11186, 11196, 11206, 0, 1765, 945, 947, 988, 1767,
    1769, 1771, 1777, 1779, 1781, 1174, 829, 1176, 232, 967, 0, 1503, 11216, 0, 1174, 1503,
    0, 2091, 0, 11220, 2093, 2094, 428, 11222, 11221, 2095, 2097, 11225, 11235, 0, 2099, 1815,
    311, 1816, 11245, 11255, 11265, 0, 158, 11275, 733, 268, 270, 278, 1447, 947, 11276, 11286,
    11296, 11306, 0, 140, 276, 11316, 0, 140, 276, 11326, 11336, 241, 11346, 11356, 11366, 11376,
    11386, 0, 825, 408, 11396, 11406, 11416, 11426, 11436, 0, 2113, 11446, 224, 733, 268, 11447,
    0, 2114, 97, 889, 0, 2115, 2116, 11448, 11458, 11468, 11478, 0, 938, 11496, 11495, 11491,
    11489, 11488, 384, 414, 384, 414, 11494, 938, 270, 11498, 627, 1172, 232, 967, 11499, 11509,
    11519, 11529, 11539, 11549, 11559, 11569, 11579, 11589, 11599, 11609, 11619, 0, 2129, 346, 792, 97,
    11629, 11639, 11649, 11659, 11669, 11679, 11689, 11699, 11709, 11719, 0, 813, 0, 414, 11729, 1431,
    11732, 414, 11735, 11738, 11741, 11751, 0, 1142, 11764, 11761, 414, 11766, 627, 0, 11765, 626,
    1007, 197, 1172, 1008, 294, 0, 1172, 294, 0, 1007, 294, 629, 1173, 262, 1568, 126,
    11767, 11777, 11787, 11794, 0, 658, 224, 11804, 11814, 11823, 0, 85, 2142, 0, 11834, 11833,
    0, 241, 642, 0, 1207, 971, 1207, 11838, 0, 241, 11848, 420, 1172, 2144, 232, 2146,
    0, 750, 2148, 11849, 11852, 2150, 153, 11854, 2152, 0, 750, 2148, 11857, 11860, 2150, 153,
    11862, 1784, 0, 750, 11865, 2153, 2154, 0, 750, 0, 741, 56, 0, 11866, 0, 750,
    741, 47, 817, 11876, 11886, 11896, 11903, 11913, 11923, 11933, 11943, 11953, 11963, 11973, 11983, 11993,
    12003, 12013, 12023,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 66, 4294967295, 4294967295, 77, 4294967295, 4294967295, 4294967295, 4294967295, 77, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 81, 84, 86, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 66, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 163, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 193, 195, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295,
    4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 79, 4294967295, 201, 4294967295, 205, 4294967295,
    4294967295, 212, 214, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 222, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 222, 4294967295, 4294967295, 4294967295, 4294967295, 234, 4294967295, 235, 240, 4294967295, 4294967295, 4294967295, 4294967295, 243,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 269, 271, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    304, 4294967295, 306, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 337, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    337, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 337, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 409, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 269, 4294967295,
    4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 452, 452, 4294967295, 452, 452, 452,
    4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 516,
    205, 342, 415, 4294967295, 380, 4294967295, 4294967295, 4294967295, 517, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 452, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 555, 4294967295, 559, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295,
    575, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 592, 4294967295, 4294967295, 598, 4294967295, 4294967295, 4294967295, 4294967295,
    415, 4294967295, 592, 4294967295, 4294967295, 598, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 342, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 653, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 666, 4294967295, 4294967295, 669, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 672, 4294967295, 342, 380,
    4294967295, 4294967295, 4294967295, 672, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 680, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 316, 316, 316, 316, 316, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 452, 316, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 79, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 380,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 84, 4294967295, 4294967295,
    79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 772, 4294967295, 269, 4294967295, 4294967295, 4294967295, 774, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 779, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 304, 4294967295,
    783, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 788, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 815, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 380, 4294967295, 824, 4294967295, 4294967295, 409, 4294967295, 4294967295, 4294967295, 827,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 830, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 81, 4294967295, 848, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 876, 878, 4294967295, 4294967295,
    4294967295, 4294967295, 380, 4294967295, 380, 881, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 946, 949, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 269,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 269, 968, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 269, 4294967295, 4294967295, 342, 949, 989, 234, 968, 991,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295,
    415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 772, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 779, 4294967295,
    4294967295, 4294967295, 4294967295, 304, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    788, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295,
    269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 881, 1026, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 991, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 98, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 98, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1089, 4294967295, 1091, 653,
    4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 342, 205, 1094,
    415, 4294967295, 86, 4294967295, 4294967295, 881, 1098, 4294967295, 4294967295, 415, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1094, 4294967295, 4294967295, 4294967295, 4294967295, 436, 4294967295, 516, 1105, 1107, 342, 1110, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 991, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 516, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1175,
    830, 1177, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1180, 1183, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1189, 1191, 4294967295, 1167, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295,
    1196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 949, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1180,
    1183, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1189, 1191, 4294967295, 1167, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 1196, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 265, 4294967295, 4294967295, 4294967295, 1167, 4294967295, 1203, 4294967295, 1205, 4294967295, 4294967295, 4294967295, 4294967295,
    415, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295, 380, 4294967295, 4294967295, 415, 4294967295, 4294967295, 415, 4294967295, 4294967295,
    4294967295, 415, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1213, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 774, 1105, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 555, 4294967295, 4294967295, 4294967295, 555,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 666, 4294967295, 4294967295, 81, 815, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 81, 81, 1226, 815, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1240, 4294967295, 1242, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295, 199, 79, 4294967295,
    1244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1256, 1259, 342, 4294967295, 415, 4294967295, 4294967295, 4294967295, 342, 4294967295, 415, 4294967295, 1261, 1259,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342,
    4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1256, 848, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 815, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 504, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1313, 4294967295, 4294967295, 4294967295, 1226,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 409, 4294967295, 4294967295, 4294967295, 1226, 4294967295, 4294967295, 4294967295,
    1226, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 269, 4294967295, 4294967295, 342, 949, 989, 234, 968, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 949, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1328, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1191, 235, 4294967295, 1343, 4294967295, 4294967295, 235, 4294967295, 4294967295, 4294967295, 265, 265, 4294967295, 4294967295, 4294967295, 1191,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1347, 1349, 1351, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295,
    4294967295, 415, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 271,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 222, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 222, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 98,
    4294967295, 4294967295, 1385, 1387, 1389, 4294967295, 4294967295, 489, 489, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1422, 4294967295,
    4294967295, 1226, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1425, 1427, 1429, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415,
    4294967295, 4294967295, 1434, 1436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 1434,
    1436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 1442, 4294967295, 4294967295, 4294967295,
    269, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 205, 271, 235, 1343, 265,
    1446, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1422, 4294967295, 1448, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 271,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1476, 4294967295, 4294967295,
    4294967295, 1478, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1489, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1226, 436, 4294967295, 436, 4294967295, 1502, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1506, 1508, 1510, 1512,
    1514, 4294967295, 559, 4294967295, 4294967295, 436, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1532,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1558, 968, 4294967295, 4294967295, 968, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1476, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 337, 4294967295, 342, 316, 4294967295, 4294967295, 1587,
    4294967295, 4294967295, 4294967295, 4294967295, 337, 4294967295, 342, 316, 4294967295, 4294967295, 1587, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 337, 4294967295, 342, 316, 4294967295, 4294967295, 1587, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 271, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1604, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1630, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1630, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1645, 316, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1645, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 269,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1240, 4294967295, 1242, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295,
    199, 199, 4294967295, 1244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1422, 4294967295, 4294967295, 271, 4294967295, 1752, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1755, 4294967295, 1757,
    4294967295, 1759, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 968, 4294967295, 1175, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1763, 4294967295, 4294967295,
    4294967295, 4294967295, 1766, 946, 949, 989, 1768, 1110, 1770, 1772, 234, 1774, 1776, 1778, 1780, 1782,
    968, 1770, 1175, 830, 1177, 4294967295, 4294967295, 4294967295, 1502, 4294967295, 4294967295, 1502, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 212, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 212, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1797, 4294967295, 342, 316, 4294967295, 4294967295, 4294967295, 205, 342,
    415, 1226, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1175, 4294967295, 234, 968, 1770, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 779, 4294967295,
    4294967295, 1732, 4294967295, 4294967295, 1645, 316, 1803, 1645, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 316, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1089, 4294967295, 4294967295, 4294967295, 380, 1091, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 81, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1180, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1180, 4294967295, 4294967295, 277, 4294967295, 4294967295, 269, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 271, 4294967295, 4294967295, 878, 212, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 342, 4294967295, 415, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 342, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1512, 1508, 1869, 1871,
    1873, 1875, 1877, 1879, 1881, 1883, 1885, 1887, 1889, 1891, 1893, 1895, 1897, 1899, 1901, 1510,
    1506, 1903, 1905, 1907, 1909, 1911, 1913, 1915, 1917, 1919, 1921, 1923, 1925, 1927, 1929, 1887,
    1931, 1881, 1933, 1935, 1937, 1939, 1941, 1943, 1945, 1947, 1949, 1951, 1953, 1955, 1957, 1959,
    1514, 1961, 1963, 1778, 1780, 1965, 1967, 1969, 1971, 1973, 1975, 824, 1977, 1979, 1981, 1983,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 452, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 380, 4294967295, 4294967295, 4294967295, 1996,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 79, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 740, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 337, 2015, 4294967295, 824, 4294967295, 4294967295,
    4294967295, 740, 269, 2017, 4294967295, 1313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 827, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1226, 4294967295, 205, 4294967295, 4294967295, 740, 269, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1630, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 205, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 380, 1313, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 878, 1385, 4294967295, 881, 1026, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 269, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1766, 946, 949, 989, 1768,
    1770, 1772, 1778, 1780, 1782, 1175, 830, 1177, 234, 968, 4294967295, 4294967295, 4294967295, 4294967295, 1175, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2096, 2098, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 269, 271, 4294967295, 1448, 949, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 277, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 409, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 740, 269, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 415, 4294967295, 415, 4294967295, 4294967295, 271, 4294967295, 4294967295, 4294967295, 234, 968, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 235, 4294967295,
    235, 415, 265, 265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 415, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2138, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 86, 2143, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2145, 234, 2147,
    4294967295, 4294967295, 4294967295, 235, 4294967295, 2151, 4294967295, 265, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 2151, 4294967295,
    265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 67, 4294967295, 4294967295, 4294967295, 4294967295, 67, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 80, 82, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 65, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 162, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 192, 194, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 78, 4294967295, 200, 4294967295, 204, 4294967295,
    4294967295, 211, 213, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 232, 4294967295, 236, 238, 4294967295, 4294967295, 4294967295, 4294967295, 242,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 268, 270, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    303, 4294967295, 305, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 314, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 336, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    352, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 336, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 408, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 469, 470, 4294967295, 472, 473, 474,
    4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 515,
    204, 341, 414, 4294967295, 379, 4294967295, 4294967295, 4294967295, 518, 519, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 453, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 556, 4294967295, 557, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 242, 4294967295, 4294967295,
    574, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 589, 4294967295, 591, 4294967295, 4294967295, 597, 4294967295, 4294967295, 4294967295, 4294967295,
    414, 4294967295, 591, 4294967295, 4294967295, 597, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 341, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 513, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 652, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 665, 4294967295, 4294967295, 668, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 671, 4294967295, 341, 379,
    4294967295, 4294967295, 4294967295, 671, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 681, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 314, 4294967295, 4294967295, 688, 689, 690, 691, 692, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 697, 4294967295, 4294967295, 700, 701, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 78, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 379,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 82, 4294967295, 4294967295,
    78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 771, 4294967295, 268, 4294967295, 4294967295, 4294967295, 773, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 314, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 303, 4294967295,
    782, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 787, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 814, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 379, 4294967295, 823, 4294967295, 4294967295, 408, 4294967295, 4294967295, 4294967295, 826,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 829, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 845, 846, 4294967295, 847, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 875, 877, 4294967295, 4294967295,
    4294967295, 4294967295, 379, 4294967295, 379, 880, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 945, 947, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 268,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 268, 967, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276,
    4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 268, 4294967295, 4294967295, 341, 947, 988, 232, 967, 990,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295,
    414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 771, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295,
    4294967295, 4294967295, 4294967295, 303, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    787, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295,
    4294967295, 4294967295, 880, 1025, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 990, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1081, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1081, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1088, 4294967295, 1090, 652,
    4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 341, 204, 1093,
    414, 4294967295, 1096, 4294967295, 4294967295, 880, 1097, 4294967295, 4294967295, 414, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1093, 4294967295, 4294967295, 4294967295, 4294967295, 435, 4294967295, 515, 1104, 1106, 341, 1108, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 990, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 515, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1166, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1174,
    829, 1176, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1179, 1181, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1187, 1190, 4294967295, 1166, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295,
    1193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 947, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1179,
    1181, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1187, 1200, 4294967295, 1201, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 1193, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 266, 4294967295, 4294967295, 4294967295, 1201, 4294967295, 1202, 4294967295, 1204, 4294967295, 4294967295, 4294967295, 4294967295,
    414, 4294967295, 4294967295, 414, 4294967295, 414, 4294967295, 379, 4294967295, 4294967295, 414, 4294967295, 4294967295, 414, 4294967295, 4294967295,
    4294967295, 414, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1212, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 773, 1217, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 556, 4294967295, 4294967295, 4294967295, 556,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 665, 4294967295, 4294967295, 1221, 1222, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 1221, 1224, 1225, 814, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1239, 4294967295, 1241, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295, 198, 78, 4294967295,
    1243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 198, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1255, 1257, 341, 4294967295, 414, 4294967295, 4294967295, 4294967295, 341, 4294967295, 414, 4294967295, 1260, 1257,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341,
    4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1255, 847, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 80, 814, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 505, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1312, 4294967295, 4294967295, 4294967295, 1225,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 408, 4294967295, 4294967295, 4294967295, 1225, 4294967295, 4294967295, 4294967295,
    1225, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 268, 4294967295, 4294967295, 341, 947, 988, 232, 967, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 947, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1327, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1190, 236, 4294967295, 1342, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295, 266, 266, 4294967295, 4294967295, 4294967295, 1190,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1346, 1348, 1350, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 589, 4294967295,
    4294967295, 414, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 270,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1081,
    4294967295, 4294967295, 1384, 1386, 1388, 4294967295, 4294967295, 1114, 490, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1421, 4294967295,
    4294967295, 1225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1424, 1426, 1428, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414,
    4294967295, 4294967295, 1433, 1435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 1433,
    1435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 1441, 4294967295, 4294967295, 4294967295,
    268, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 204, 270, 236, 1342, 266,
    1445, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1421, 4294967295, 1447, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 270,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 4294967295, 4294967295,
    4294967295, 1477, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1490, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1225, 1500, 4294967295, 435, 4294967295, 1501, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1505, 1507, 1509, 1511,
    1513, 4294967295, 1515, 4294967295, 4294967295, 435, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1531,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1557, 967, 4294967295, 4294967295, 967, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1475, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 336, 4294967295, 341, 314, 4294967295, 4294967295, 1586,
    4294967295, 4294967295, 4294967295, 4294967295, 336, 4294967295, 341, 314, 4294967295, 4294967295, 1586, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 336, 4294967295, 341, 314, 4294967295, 4294967295, 1586, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 414, 270, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1603, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1629, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1644, 314, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1644, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 268,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1239, 4294967295, 1241, 4294967295, 4294967295, 200, 4294967295, 4294967295, 4294967295, 4294967295,
    198, 198, 4294967295, 1243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1421, 4294967295, 4294967295, 270, 4294967295, 1751, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1753, 4294967295, 1756,
    4294967295, 1758, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 967, 4294967295, 1174, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1762, 4294967295, 4294967295,
    4294967295, 4294967295, 1765, 945, 947, 988, 1767, 1108, 1769, 1771, 232, 1773, 1775, 1777, 1779, 1781,
    967, 1769, 1174, 829, 1176, 4294967295, 4294967295, 4294967295, 1501, 4294967295, 4294967295, 1783, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 211, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 211, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1796, 4294967295, 341, 314, 4294967295, 4294967295, 4294967295, 204, 341,
    414, 1225, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1174, 4294967295, 232, 967, 1769, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 778, 4294967295,
    4294967295, 1801, 4294967295, 4294967295, 1644, 314, 1802, 1644, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 314, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1088, 4294967295, 4294967295, 4294967295, 379, 1090, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 80, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1179, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1179, 4294967295, 4294967295, 276, 4294967295, 4294967295, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 270, 4294967295, 4294967295, 877, 211, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 341, 4294967295, 414, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 341, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1511, 1507, 1868, 1870,
    1872, 1874, 1876, 1878, 1880, 1882, 1884, 1886, 1888, 1890, 1892, 1894, 1896, 1898, 1900, 1509,
    1505, 1902, 1904, 1906, 1908, 1910, 1912, 1914, 1916, 1918, 1920, 1922, 1924, 1926, 1928, 1886,
    1930, 1880, 1932, 1934, 1936, 1938, 1940, 1942, 1944, 1946, 1948, 1950, 1952, 1954, 1956, 1958,
    1513, 1960, 1962, 1777, 1779, 1964, 1966, 1968, 1970, 1972, 1974, 823, 1976, 1978, 1980, 1982,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1312, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1993, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 379, 4294967295, 4294967295, 4294967295, 1995,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 245, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 78, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 733, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 352, 2014, 4294967295, 823, 4294967295, 4294967295,
    4294967295, 733, 268, 2016, 4294967295, 1312, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 826, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1225, 4294967295, 204, 4294967295, 4294967295, 733, 268, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1312, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 379, 1312, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 877, 1384, 4294967295, 880, 2075, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 268, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1765, 945, 947, 988, 1767,
    1769, 1771, 1777, 1779, 1781, 1174, 829, 1176, 232, 967, 4294967295, 4294967295, 4294967295, 4294967295, 1174, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2095, 2097, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 268, 270, 4294967295, 1447, 947, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 276, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 408, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 733, 268, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 414, 4294967295, 414, 4294967295, 4294967295, 270, 4294967295, 4294967295, 4294967295, 232, 967, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 236, 4294967295,
    589, 414, 266, 266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 414, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2139, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 85, 2142, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2144, 232, 2146,
    4294967295, 4294967295, 4294967295, 236, 4294967295, 2150, 4294967295, 266, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 2150, 4294967295,
    266, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1136, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (600, 1),
    (2804, 1),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
    (588, 0, 1),
    (600, 1, 1),
];

pub static TRIM_CHARS_DATA: &[u32] = &[
    491, 506,
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (38, 0, 2),
    (41, 2, 2),
    (46, 4, 2),
    (53, 6, 2),
    (55, 8, 2),
    (56, 10, 2),
    (57, 12, 3),
    (99, 15, 2),
    (135, 17, 2),
    (184, 19, 2),
    (185, 21, 2),
    (190, 23, 2),
    (194, 25, 3),
    (198, 28, 3),
    (202, 31, 2),
    (204, 33, 3),
    (206, 36, 2),
    (209, 38, 2),
    (210, 40, 2),
    (220, 42, 2),
    (226, 44, 2),
    (231, 46, 2),
    (233, 48, 4),
    (234, 52, 2),
    (239, 54, 2),
    (250, 56, 5),
    (253, 61, 3),
    (254, 64, 2),
    (263, 66, 3),
    (288, 69, 3),
    (290, 72, 2),
    (299, 74, 2),
    (308, 76, 2),
    (314, 78, 3),
    (322, 81, 3),
    (326, 84, 3),
    (331, 87, 3),
    (340, 90, 3),
    (342, 93, 3),
    (352, 96, 2),
    (360, 100, 3),
    (364, 98, 2),
    (370, 103, 2),
    (385, 105, 2),
    (393, 107, 3),
    (400, 110, 3),
    (440, 113, 2),
    (450, 115, 3),
    (454, 118, 2),
    (468, 120, 3),
    (483, 123, 3),
    (490, 126, 3),
    (494, 129, 3),
    (499, 132, 3),
    (508, 135, 2),
    (518, 137, 3),
    (530, 140, 3),
    (537, 143, 3),
    (554, 146, 2),
    (555, 148, 2),
    (557, 150, 2),
    (558, 152, 2),
    (559, 154, 3),
    (564, 157, 2),
    (576, 159, 2),
    (607, 161, 2),
    (608, 163, 2),
    (609, 165, 3),
    (610, 168, 2),
    (612, 170, 2),
    (616, 172, 2),
    (617, 174, 1),
    (630, 175, 3),
    (660, 178, 3),
    (665, 181, 2),
    (667, 183, 2),
    (685, 185, 2),
    (688, 187, 2),
    (697, 189, 3),
    (710, 192, 4),
    (712, 196, 2),
    (715, 198, 2),
    (720, 200, 2),
    (722, 202, 2),
    (725, 204, 2),
    (770, 206, 3),
    (772, 209, 2),
    (786, 211, 2),
    (806, 213, 2),
    (808, 215, 2),
    (813, 217, 2),
    (821, 219, 2),
    (833, 221, 2),
    (836, 223, 3),
    (840, 226, 2),
    (844, 228, 2),
    (846, 230, 3),
    (847, 233, 2),
    (851, 235, 2),
    (858, 237, 2),
    (881, 239, 3),
    (887, 242, 2),
    (890, 244, 2),
    (891, 246, 2),
    (892, 248, 2),
    (893, 250, 2),
    (894, 252, 2),
    (905, 254, 3),
    (908, 257, 2),
    (909, 259, 2),
    (929, 261, 2),
    (932, 263, 2),
    (957, 265, 2),
    (973, 267, 2),
    (975, 269, 2),
    (988, 271, 2),
    (989, 273, 2),
    (992, 275, 2),
    (1002, 277, 3),
    (1025, 280, 3),
    (1027, 283, 3),
    (1031, 286, 2),
    (1038, 288, 2),
    (1045, 290, 2),
    (1057, 292, 3),
    (1062, 295, 3),
    (1086, 298, 3),
    (1088, 301, 2),
    (1095, 303, 3),
    (1110, 306, 3),
    (1116, 309, 3),
    (1130, 312, 3),
    (1153, 315, 2),
    (1158, 317, 2),
    (1162, 319, 3),
    (1183, 322, 3),
    (1189, 325, 3),
    (1190, 328, 2),
    (1192, 330, 2),
    (1195, 332, 2),
    (1199, 334, 2),
    (1208, 336, 2),
    (1219, 338, 2),
    (1233, 340, 2),
    (1234, 342, 2),
    (1236, 344, 2),
    (1254, 346, 2),
    (1276, 348, 2),
    (1277, 350, 2),
    (1282, 352, 2),
    (1284, 354, 2),
    (1285, 356, 2),
    (1339, 358, 2),
    (1340, 360, 2),
    (1358, 362, 2),
    (1359, 364, 3),
    (1365, 367, 2),
    (1366, 369, 3),
    (1367, 372, 2),
    (1375, 374, 3),
    (1379, 377, 2),
    (1383, 379, 3),
    (1396, 382, 3),
    (1399, 385, 3),
    (1402, 388, 3),
    (1403, 391, 2),
    (1404, 393, 2),
    (1405, 395, 2),
    (1406, 397, 2),
    (1407, 399, 2),
    (1438, 401, 3),
    (1440, 404, 2),
    (1463, 406, 2),
    (1465, 408, 2),
    (1473, 410, 3),
    (1480, 413, 2),
    (1485, 415, 3),
    (1492, 418, 2),
    (1502, 420, 3),
    (1507, 423, 3),
    (1514, 426, 3),
    (1520, 429, 3),
    (1525, 432, 2),
    (1531, 434, 3),
    (1536, 437, 3),
    (1549, 440, 2),
    (1554, 442, 2),
    (1555, 444, 2),
    (1578, 446, 3),
    (1601, 449, 2),
    (1633, 451, 2),
    (1650, 453, 2),
    (1660, 455, 2),
    (1662, 457, 2),
    (1663, 459, 2),
    (1668, 461, 2),
    (1672, 463, 2),
    (1677, 465, 3),
    (1678, 468, 2),
    (1679, 470, 2),
    (1680, 472, 2),
    (1682, 474, 3),
    (1685, 477, 2),
    (1686, 479, 2),
    (1689, 481, 2),
    (1691, 483, 2),
    (1698, 485, 2),
    (1703, 487, 2),
    (1705, 489, 2),
    (1706, 491, 2),
    (1707, 493, 2),
    (1708, 495, 3),
    (1709, 498, 2),
    (1725, 500, 2),
    (1767, 502, 2),
    (1779, 504, 2),
    (1788, 506, 3),
    (1817, 509, 2),
    (1834, 511, 2),
    (1855, 513, 2),
    (1856, 515, 2),
    (1857, 517, 2),
    (1863, 519, 2),
    (1864, 521, 2),
    (1865, 523, 2),
    (1874, 525, 2),
    (1875, 527, 2),
    (1877, 529, 2),
    (1888, 531, 2),
    (1903, 533, 3),
    (1912, 536, 2),
    (1919, 538, 2),
    (1920, 540, 2),
    (1921, 542, 2),
    (1931, 544, 2),
    (1932, 546, 2),
    (1934, 548, 2),
    (1946, 550, 2),
    (1959, 552, 2),
    (1961, 554, 2),
    (1963, 556, 2),
    (1968, 558, 2),
    (1971, 560, 2),
    (1973, 562, 2),
    (1975, 564, 2),
    (1978, 566, 2),
    (1981, 568, 2),
    (1985, 570, 2),
    (1987, 572, 2),
    (1994, 574, 2),
    (2002, 576, 3),
    (2007, 579, 3),
    (2009, 582, 2),
    (2010, 584, 2),
    (2027, 586, 2),
    (2031, 588, 2),
    (2039, 590, 2),
    (2044, 592, 2),
    (2057, 594, 2),
    (2058, 596, 2),
    (2061, 598, 2),
    (2062, 600, 2),
    (2066, 608, 2),
    (2068, 602, 2),
    (2069, 604, 2),
    (2070, 606, 2),
    (2071, 610, 2),
    (2086, 612, 2),
    (2087, 614, 3),
    (2099, 617, 2),
    (2101, 619, 2),
    (2104, 621, 3),
    (2109, 624, 3),
    (2110, 627, 2),
    (2112, 629, 2),
    (2132, 631, 3),
    (2146, 634, 2),
    (2147, 636, 2),
    (2148, 638, 3),
    (2150, 641, 2),
    (2154, 643, 3),
    (2156, 646, 2),
    (2158, 648, 2),
    (2159, 650, 2),
    (2175, 652, 3),
    (2177, 655, 2),
    (2187, 657, 2),
    (2188, 659, 2),
    (2194, 661, 2),
    (2212, 663, 2),
    (2213, 665, 2),
    (2236, 667, 3),
    (2245, 670, 3),
    (2248, 673, 3),
    (2283, 676, 2),
    (2315, 678, 2),
    (2319, 680, 2),
    (2328, 682, 2),
    (2332, 684, 2),
    (2336, 686, 2),
    (2340, 688, 3),
    (2343, 691, 3),
    (2346, 694, 3),
    (2347, 697, 2),
    (2348, 699, 2),
    (2349, 701, 2),
    (2350, 703, 2),
    (2357, 705, 3),
    (2369, 708, 2),
    (2395, 710, 2),
    (2432, 712, 2),
    (2435, 714, 2),
    (2447, 716, 2),
    (2453, 718, 2),
    (2454, 720, 2),
    (2455, 722, 2),
    (2456, 724, 2),
    (2465, 726, 2),
    (2502, 728, 4),
    (2510, 732, 2),
    (2511, 734, 2),
    (2518, 736, 2),
    (2568, 738, 2),
    (2578, 740, 2),
    (2584, 742, 2),
    (2591, 744, 2),
    (2594, 746, 2),
    (2595, 748, 2),
    (2596, 750, 2),
    (2599, 752, 5),
    (2600, 757, 3),
    (2612, 760, 3),
    (2644, 763, 2),
    (2670, 765, 2),
    (2673, 767, 2),
    (2679, 769, 2),
    (2680, 771, 2),
    (2681, 773, 2),
    (2687, 775, 2),
    (2690, 777, 2),
    (2691, 779, 2),
    (2700, 781, 2),
    (2703, 783, 2),
    (2704, 785, 2),
    (2714, 787, 2),
    (2716, 789, 2),
    (2720, 791, 3),
    (2724, 794, 3),
    (2729, 797, 2),
    (2731, 799, 2),
    (2732, 801, 2),
    (2734, 803, 2),
    (2736, 805, 2),
    (2742, 807, 2),
    (2744, 809, 2),
    (2779, 811, 3),
    (2798, 814, 3),
    (2799, 817, 2),
    (2813, 819, 2),
    (2817, 821, 2),
    (2834, 823, 3),
    (2849, 826, 2),
    (2850, 828, 2),
    (2852, 830, 2),
    (2854, 832, 2),
    (2860, 834, 2),
    (2861, 836, 2),
    (2862, 838, 2),
    (2863, 840, 2),
    (2864, 842, 2),
    (2866, 844, 2),
    (2869, 846, 2),
    (2911, 848, 2),
    (2939, 850, 2),
    (2981, 852, 3),
    (2983, 855, 2),
    (2992, 857, 2),
    (2993, 859, 2),
    (2996, 861, 2),
    (3022, 863, 2),
    (3032, 865, 3),
    (3059, 868, 2),
    (3065, 870, 2),
    (3067, 872, 3),
    (3068, 875, 2),
    (3071, 877, 2),
    (3076, 879, 2),
    (3078, 881, 3),
    (3079, 884, 2),
    (3082, 886, 2),
    (3096, 888, 2),
    (3098, 890, 3),
    (3099, 893, 2),
    (3102, 895, 2),
    (3130, 897, 2),
    (3145, 899, 2),
    (3147, 901, 2),
    (3148, 903, 2),
    (3158, 905, 2),
    (3196, 907, 2),
    (3202, 909, 2),
    (3219, 911, 2),
    (3220, 913, 2),
    (3232, 915, 2),
    (3248, 917, 2),
    (3293, 919, 3),
    (3333, 922, 3),
    (3336, 925, 2),
    (3390, 927, 2),
    (3391, 929, 3),
    (3414, 932, 2),
    (3416, 934, 2),
    (3419, 936, 3),
    (3424, 939, 3),
    (3425, 942, 3),
    (3427, 945, 2),
    (3521, 947, 2),
    (3524, 949, 2),
    (3526, 951, 2),
    (3533, 953, 2),
    (3535, 955, 2),
    (3537, 957, 2),
    (3546, 959, 2),
    (3548, 961, 2),
    (3565, 963, 2),
    (3570, 965, 2),
    (3571, 967, 2),
    (3572, 969, 2),
    (3573, 971, 2),
    (3574, 973, 2),
    (3575, 975, 2),
    (3576, 977, 2),
    (3577, 979, 2),
    (3578, 981, 2),
    (3579, 983, 2),
    (3580, 985, 2),
    (3581, 987, 2),
    (3582, 989, 2),
    (3583, 991, 2),
    (3584, 993, 2),
    (3585, 995, 2),
    (3586, 997, 2),
    (3587, 999, 2),
    (3588, 1001, 2),
    (3592, 1003, 2),
    (3595, 1005, 2),
    (3609, 1007, 2),
    (3619, 1009, 2),
    (3655, 1011, 2),
    (3657, 1013, 3),
    (3658, 1016, 2),
    (3662, 1018, 2),
    (3663, 1020, 3),
    (3664, 1023, 2),
    (3665, 1025, 2),
    (3672, 1027, 2),
    (3683, 1029, 2),
    (3685, 1031, 2),
    (3686, 1033, 2),
    (3687, 1035, 2),
    (3710, 1037, 3),
    (3713, 1040, 2),
    (3716, 1042, 2),
    (3717, 1044, 2),
    (3718, 1046, 2),
    (3719, 1048, 2),
    (3729, 1050, 2),
    (3734, 1052, 2),
    (3744, 1054, 2),
    (3748, 1056, 2),
    (3749, 1058, 2),
    (3765, 1060, 3),
    (3767, 1063, 2),
    (3768, 1065, 2),
    (3777, 1067, 2),
    (3784, 1069, 2),
    (3787, 1071, 3),
    (3790, 1074, 3),
    (3799, 1077, 3),
    (3805, 1080, 3),
    (3820, 1083, 3),
    (3830, 1086, 3),
    (3864, 1089, 3),
    (3865, 1092, 2),
    (3868, 1094, 2),
    (3869, 1096, 2),
    (3898, 1098, 3),
    (3900, 1101, 2),
    (3922, 1103, 3),
    (3964, 1106, 2),
    (3965, 1108, 2),
    (3966, 1110, 2),
    (3967, 1112, 2),
    (3968, 1114, 2),
    (3969, 1116, 2),
    (3970, 1118, 2),
    (3971, 1120, 2),
    (3972, 1122, 2),
    (3973, 1124, 2),
    (3974, 1126, 2),
    (3975, 1128, 2),
    (3976, 1130, 2),
    (3977, 1132, 2),
    (3978, 1134, 2),
    (3979, 1136, 2),
    (3980, 1138, 2),
    (3981, 1140, 2),
    (3982, 1142, 2),
    (3983, 1144, 2),
    (3984, 1146, 2),
    (3985, 1148, 2),
    (3986, 1150, 2),
    (3987, 1152, 2),
    (3988, 1154, 2),
    (3989, 1156, 2),
    (3990, 1158, 2),
    (3991, 1160, 2),
    (3992, 1162, 2),
    (3993, 1164, 2),
    (3994, 1166, 2),
    (3995, 1168, 2),
    (3996, 1170, 2),
    (3997, 1172, 2),
    (3998, 1174, 2),
    (3999, 1176, 2),
    (4000, 1178, 2),
    (4001, 1180, 2),
    (4002, 1182, 2),
    (4003, 1184, 2),
    (4004, 1186, 2),
    (4005, 1188, 2),
    (4006, 1190, 2),
    (4007, 1192, 2),
    (4008, 1194, 2),
    (4009, 1196, 2),
    (4010, 1198, 2),
    (4011, 1200, 2),
    (4012, 1202, 2),
    (4013, 1204, 2),
    (4014, 1206, 2),
    (4015, 1208, 2),
    (4016, 1210, 2),
    (4017, 1212, 2),
    (4018, 1214, 2),
    (4019, 1216, 2),
    (4020, 1218, 2),
    (4021, 1220, 2),
    (4022, 1222, 2),
    (4023, 1224, 2),
    (4024, 1226, 2),
    (4025, 1228, 2),
    (4026, 1230, 2),
    (4027, 1232, 2),
    (4028, 1234, 2),
    (4029, 1236, 2),
    (4030, 1238, 2),
    (4031, 1240, 2),
    (4041, 1242, 2),
    (4062, 1244, 2),
    (4069, 1246, 2),
    (4075, 1248, 2),
    (4079, 1250, 2),
    (4090, 1252, 3),
    (4107, 1255, 2),
    (4130, 1257, 2),
    (4131, 1259, 3),
    (4138, 1262, 2),
    (4139, 1264, 2),
    (4141, 1266, 2),
    (4145, 1268, 2),
    (4146, 1270, 3),
    (4147, 1273, 2),
    (4149, 1275, 2),
    (4158, 1277, 2),
    (4166, 1279, 2),
    (4168, 1281, 2),
    (4171, 1283, 2),
    (4172, 1285, 3),
    (4233, 1288, 2),
    (4245, 1290, 2),
    (4253, 1292, 2),
    (4297, 1294, 2),
    (4306, 1296, 2),
    (4307, 1298, 2),
    (4329, 1300, 3),
    (4339, 1303, 2),
    (4340, 1305, 2),
    (4342, 1307, 2),
    (4343, 1309, 2),
    (4353, 1311, 3),
    (4379, 1314, 2),
    (4380, 1316, 2),
    (4381, 1318, 2),
    (4382, 1320, 2),
    (4383, 1322, 2),
    (4384, 1324, 2),
    (4385, 1326, 2),
    (4386, 1328, 2),
    (4387, 1330, 2),
    (4388, 1332, 2),
    (4389, 1334, 2),
    (4390, 1336, 2),
    (4391, 1338, 2),
    (4392, 1340, 2),
    (4393, 1342, 2),
    (4398, 1344, 2),
    (4409, 1346, 2),
    (4410, 1348, 2),
    (4424, 1350, 2),
    (4425, 1352, 3),
    (4426, 1355, 2),
    (4428, 1357, 2),
    (4429, 1359, 2),
    (4436, 1361, 3),
    (4440, 1364, 3),
    (4451, 1367, 2),
    (4461, 1369, 2),
    (4462, 1371, 3),
    (4483, 1374, 2),
    (4485, 1376, 2),
    (4488, 1378, 2),
    (4492, 1380, 2),
    (4493, 1382, 2),
    (4525, 1384, 2),
    (4529, 1386, 2),
    (4538, 1388, 2),
    (4562, 1390, 3),
    (4571, 1393, 3),
    (4572, 1396, 2),
    (4589, 1398, 2),
    (4590, 1400, 2),
    (4591, 1402, 2),
    (4597, 1404, 2),
    (4606, 1406, 2),
    (4629, 1408, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    66, 3, 77, 3, 77, 3, 3, 79, 3, 81, 3, 84, 3, 79, 86, 66,
    3, 163, 3, 3, 193, 3, 195, 3, 79, 3, 79, 199, 3, 79, 201, 3,
    79, 3, 79, 201, 3, 205, 212, 3, 3, 214, 3, 222, 3, 222, 3, 234,
    3, 235, 237, 4, 240, 3, 3, 243, 3, 265, 235, 237, 4, 3, 79, 269,
    271, 3, 3, 277, 79, 3, 79, 304, 306, 3, 3, 316, 3, 337, 3, 342,
    79, 3, 79, 269, 3, 79, 269, 3, 79, 269, 3, 342, 79, 3, 342, 79,
    3, 337, 3, 380, 3, 342, 79, 3, 380, 3, 337, 3, 342, 79, 3, 342,
    79, 3, 409, 3, 342, 79, 3, 415, 3, 342, 79, 3, 342, 79, 3, 342,
    79, 3, 79, 269, 3, 79, 269, 3, 436, 3, 79, 269, 3, 342, 79, 3,
    277, 79, 3, 452, 3, 452, 3, 452, 3, 452, 3, 452, 4, 3, 415, 3,
    380, 3, 516, 3, 205, 3, 342, 79, 3, 415, 3, 380, 3, 517, 3, 3,
    452, 4, 3, 342, 79, 3, 555, 3, 559, 3, 243, 3, 575, 3, 79, 269,
    3, 235, 237, 4, 3, 592, 3, 598, 3, 415, 3, 592, 3, 598, 3, 342,
    79, 3, 415, 3, 4, 3, 653, 3, 205, 3, 79, 3, 415, 3, 666, 3,
    669, 79, 3, 380, 3, 672, 3, 342, 79, 3, 380, 3, 672, 3, 205, 3,
    680, 4, 3, 316, 3, 316, 3, 316, 3, 316, 3, 316, 3, 316, 3, 316,
    4, 3, 452, 3, 316, 3, 79, 3, 415, 3, 740, 3, 380, 3, 380, 3,
    81, 3, 84, 3, 79, 3, 277, 79, 3, 772, 79, 3, 79, 269, 3, 774,
    3, 79, 3, 316, 3, 79, 201, 3, 79, 779, 3, 79, 304, 3, 783, 3,
    79, 788, 3, 342, 79, 3, 79, 269, 3, 79, 269, 3, 415, 3, 815, 3,
    79, 201, 3, 79, 269, 3, 342, 79, 3, 380, 3, 824, 3, 409, 3, 827,
    3, 830, 3, 436, 3, 81, 3, 81, 3, 848, 3, 79, 3, 876, 878, 3,
    3, 380, 3, 380, 3, 881, 3, 946, 3, 949, 3, 740, 3, 79, 269, 3,
    740, 3, 79, 269, 3, 968, 3, 277, 79, 3, 81, 3, 79, 269, 3, 79,
    269, 3, 79, 269, 3, 342, 79, 3, 949, 3, 989, 3, 234, 3, 968, 3,
    991, 3, 342, 79, 3, 415, 3, 380, 3, 380, 3, 277, 79, 3, 81, 3,
    772, 79, 3, 79, 3, 79, 779, 3, 79, 304, 3, 79, 269, 3, 79, 788,
    3, 79, 3, 79, 201, 3, 79, 269, 3, 415, 3, 881, 3, 1026, 3, 342,
    79, 3, 991, 3, 98, 3, 98, 3, 1089, 3, 1091, 3, 653, 3, 205, 3,
    415, 3, 342, 79, 3, 205, 3, 1094, 3, 415, 3, 79, 86, 3, 881, 3,
    1098, 3, 415, 3, 380, 3, 1094, 3, 436, 3, 516, 3, 1105, 1107, 3, 3,
    342, 79, 3, 1110, 3, 415, 3, 991, 3, 415, 3, 342, 79, 3, 516, 3,
    1167, 3, 1175, 3, 830, 3, 1177, 3, 1180, 3, 1183, 271, 3, 3, 1189, 3,
    1191, 3, 1167, 3, 1196, 3, 79, 269, 3, 949, 3, 1180, 3, 1183, 271, 3,
    3, 1189, 3, 1191, 3, 1167, 3, 1196, 3, 1167, 3, 1203, 3, 1205, 3, 415,
    3, 415, 3, 415, 3, 380, 3, 415, 3, 415, 3, 415, 3, 415, 1213, 3,
    3, 342, 79, 3, 342, 79, 3, 774, 3, 1105, 3, 555, 3, 555, 3, 380,
    3, 380, 3, 205, 3, 666, 3, 81, 3, 815, 3, 81, 3, 81, 3, 1226,
    3, 81, 3, 815, 3, 740, 3, 79, 269, 1240, 3, 1242, 3, 3, 79, 201,
    3, 79, 199, 3, 79, 1244, 3, 3, 79, 199, 3, 1256, 3, 1259, 3, 342,
    79, 3, 415, 3, 342, 79, 3, 415, 3, 1261, 3, 1259, 3, 342, 79, 3,
    415, 3, 1256, 3, 848, 3, 415, 3, 81, 3, 815, 3, 342, 79, 3, 342,
    79, 3, 504, 4, 3, 205, 3, 1313, 3, 1226, 3, 409, 3, 1226, 3, 1226,
    3, 79, 269, 3, 79, 269, 3, 342, 79, 3, 949, 3, 989, 3, 234, 3,
    968, 3, 79, 269, 3, 949, 3, 1328, 3, 1191, 3, 1343, 3, 1191, 3, 1347,
    3, 1349, 3, 1351, 271, 3, 3, 415, 3, 1, 4, 5, 3, 415, 271, 3,
    3, 205, 3, 205, 3, 222, 3, 222, 3, 98, 1385, 3, 3, 1387, 3, 1389,
    3, 1, 489, 4, 5, 3, 489, 4, 3, 79, 269, 3, 79, 3, 1422, 3,
    1226, 3, 1425, 3, 1427, 3, 1429, 3, 415, 3, 1434, 3, 1436, 3, 415, 3,
    1434, 3, 1436, 3, 415, 3, 1442, 3, 79, 269, 3, 79, 269, 271, 3, 3,
    205, 271, 3, 3, 1343, 3, 1446, 3, 1422, 3, 1448, 3, 79, 269, 3, 342,
    79, 271, 3, 3, 1476, 3, 1478, 3, 1489, 4, 3, 1226, 3, 436, 3, 436,
    3, 1502, 3, 1506, 3, 1508, 3, 1510, 3, 1512, 3, 1514, 3, 559, 3, 436,
    3, 1532, 3, 415, 3, 342, 79, 3, 415, 3, 1558, 3, 968, 3, 968, 3,
    1476, 3, 342, 79, 3, 415, 3, 337, 3, 342, 79, 3, 316, 3, 1587, 3,
    337, 3, 342, 79, 3, 316, 3, 1587, 3, 337, 3, 342, 79, 3, 316, 3,
    1587, 271, 3, 3, 415, 3, 415, 271, 3, 3, 1604, 3, 1630, 3, 1630, 3,
    1645, 3, 316, 3, 1645, 3, 415, 3, 79, 269, 3, 79, 269, 3, 81, 3,
    740, 3, 79, 269, 1240, 3, 1242, 3, 3, 79, 201, 3, 79, 199, 3, 79,
    199, 1244, 3, 3, 1422, 271, 3, 3, 1752, 3, 1755, 3, 1757, 3, 1759, 3,
    968, 3, 1175, 3, 1763, 3, 1766, 3, 946, 3, 949, 3, 989, 3, 1768, 3,
    1110, 3, 1770, 3, 1772, 3, 234, 3, 1774, 3, 1776, 3, 1778, 3, 1780, 3,
    1782, 3, 968, 3, 1770, 3, 1175, 3, 830, 3, 1177, 3, 1502, 3, 1502, 212,
    3, 212, 3, 3, 1797, 3, 342, 79, 3, 316, 3, 205, 3, 342, 79, 3,
    415, 3, 1226, 3, 205, 3, 1175, 3, 234, 3, 968, 3, 1770, 3, 79, 779,
    3, 1732, 3, 1645, 3, 316, 3, 1803, 3, 1645, 3, 316, 3, 415, 3, 1089,
    3, 380, 3, 1091, 3, 277, 79, 3, 81, 3, 81, 3, 1180, 3, 1180, 3,
    277, 79, 3, 79, 269, 3, 277, 79, 3, 79, 269, 3, 277, 79, 3, 277,
    79, 3, 342, 79, 271, 3, 878, 3, 212, 3, 3, 342, 79, 3, 415, 3,
    342, 79, 3, 1512, 3, 1508, 3, 1869, 3, 1871, 3, 1873, 3, 1875, 3, 1877,
    1879, 3, 3, 1881, 3, 1883, 3, 1885, 1887, 3, 3, 1889, 3, 1891, 3, 1893,
    3, 1895, 3, 1897, 3, 1899, 3, 1901, 3, 1510, 3, 1506, 3, 1903, 3, 1905,
    3, 1907, 3, 1909, 3, 1911, 3, 1913, 3, 1915, 3, 1917, 1919, 3, 3, 1921,
    3, 1923, 3, 1925, 3, 1927, 1929, 3, 1887, 3, 1931, 3, 3, 1881, 3, 1933,
    3, 1935, 1937, 3, 1939, 3, 1941, 3, 3, 1943, 3, 1945, 3, 1947, 3, 1949,
    3, 1951, 3, 1953, 3, 1955, 3, 1957, 3, 1959, 3, 1514, 3, 1961, 3, 1963,
    3, 1778, 3, 1780, 3, 1965, 3, 1967, 3, 1969, 3, 1971, 3, 1973, 3, 1975,
    3, 824, 3, 1977, 3, 1979, 3, 1981, 3, 1983, 3, 1313, 3, 452, 271, 3,
    3, 380, 3, 1996, 3, 4, 246, 3, 79, 3, 740, 3, 79, 269, 3, 337,
    3, 2015, 3, 824, 3, 740, 3, 79, 269, 3, 2017, 3, 1313, 3, 827, 3,
    1226, 3, 205, 3, 740, 3, 79, 269, 3, 1630, 3, 415, 3, 205, 3, 1313,
    3, 380, 3, 1313, 3, 79, 269, 878, 3, 1385, 3, 3, 881, 3, 1026, 3,
    79, 269, 3, 1766, 3, 946, 3, 949, 3, 989, 3, 1768, 3, 1770, 3, 1772,
    3, 1778, 3, 1780, 3, 1782, 3, 1175, 3, 830, 3, 1177, 3, 234, 3, 968,
    3, 1175, 3, 2096, 3, 2098, 3, 740, 3, 79, 269, 271, 3, 3, 1448, 3,
    949, 3, 277, 79, 3, 277, 79, 3, 409, 3, 740, 3, 79, 269, 3, 415,
    3, 415, 271, 3, 3, 234, 3, 968, 3, 415, 3, 415, 3, 415, 3, 4,
    2138, 3, 79, 86, 3, 2143, 3, 2145, 3, 234, 3, 2147, 3, 2151, 3, 2151,
    3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"[A-Z_][A-Z0-9_]*"#, // [0]
    r#"^(NOT)$"#, // [1]
    r#"[A-Z0-9_]*[A-Z][A-Z0-9_]*"#, // [2]
    r#"^(ALL|AND|ANY|AS|AUTHORIZATION|BOTH|CASE|CAST|CHECK|COLLATE|COLUMN|CONSTRAINT|CREATE|CROSS|CURRENT_DATE|CURRENT_TIME|CURRENT_TIMESTAMP|CURRENT_USER|ELSE|END|ESCAPE|EXCEPT|FALSE|FETCH|FILTER|FOR|FOREIGN|FROM|FULL|GRANT|GROUP|HAVING|IN|INNER|INTERSECT|INTO|IS|JOIN|LEADING|NATURAL|NOT|NULL|ON|ONLY|OR|ORDER|OUTER|OVERLAPS|PRIMARY|REFERENCES|SELECT|SESSION_USER|SOME|TABLE|THEN|TO|TRAILING|UNION|UNIQUE|UNKNOWN|USER|USING|WHEN|WHERE|WITH)$"#, // [3]
    r#"[A-Z0-9_]*"#, // [4]
    r#"\"?[A-Z][A-Z0-9_]*\"?"#, // [5]
    r#"[A-Z0-9]*[A-Z][A-Z0-9]*"#, // [6]
    r#"[A-Z][A-Z0-9_]*"#, // [7]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 10, 11, 12, 13, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 38, 26, 41, 38, 13, 14, 21, 22,
    29, 12, 27, 34, 36, 18, 53, 54, 54, 53,
    58, 11, 61, 63, 20, 15, 16, 17, 19, 23,
    24, 25, 28, 29, 30, 31, 32, 33, 35, 37,
    15, 16, 17, 68, 23, 24, 69, 70, 61, 71,
    72, 73, 74, 75, 76, 13, 29, 83, 88, 90,
    92, 93, 96, 98, 11, 99, 100, 101, 102, 103,
    104, 105, 106, 107, 41, 108, 109, 110, 111, 112,
    113, 114, 115, 116, 117, 118, 119, 120, 121, 122,
    101, 13, 21, 27, 124, 38, 124, 99, 23, 129,
    103, 35, 106, 134, 135, 119, 135, 119, 11, 99,
    100, 101, 102, 103, 104, 105, 107, 41, 108, 109,
    110, 111, 112, 113, 114, 115, 116, 117, 118, 119,
    120, 121, 122, 15, 16, 17, 19, 23, 24, 138,
    25, 28, 29, 30, 31, 32, 33, 35, 37, 138,
    120, 12, 29, 104, 143, 100, 102, 105, 107, 108,
    109, 110, 112, 111, 113, 114, 115, 116, 117, 118,
    121, 122, 165, 33, 37, 31, 19, 25, 28, 30,
    32, 24, 15, 16, 17, 178, 74, 76, 72, 68,
    69, 70, 71, 73, 75, 190, 191, 190, 191, 196,
    202, 207, 208, 209, 210, 210, 208, 209, 215, 216,
    217, 218, 220, 221, 222, 223, 5, 221, 223, 220,
    222, 5, 227, 228, 229, 230, 231, 233, 239, 247,
    249, 250, 92, 251, 252, 253, 254, 255, 256, 257,
    258, 259, 254, 249, 250, 251, 252, 255, 257, 258,
    259, 256, 253, 267, 272, 273, 274, 279, 280, 281,
    280, 281, 284, 285, 286, 287, 288, 289, 290, 291,
    292, 233, 286, 295, 288, 289, 287, 289, 284, 290,
    284, 290, 285, 291, 285, 291, 292, 233, 307, 310,
    312, 315, 318, 319, 320, 321, 322, 323, 324, 325,
    326, 327, 328, 329, 210, 330, 331, 332, 221, 98,
    333, 334, 223, 218, 107, 83, 230, 338, 230, 338,
    338, 344, 239, 90, 347, 348, 310, 350, 350, 274,
    354, 354, 230, 320, 356, 357, 358, 359, 360, 361,
    321, 362, 363, 364, 307, 338, 365, 366, 367, 323,
    368, 369, 370, 371, 279, 372, 373, 374, 326, 327,
    328, 375, 36, 376, 377, 361, 320, 356, 357, 358,
    359, 360, 321, 362, 363, 364, 365, 366, 367, 323,
    368, 369, 370, 371, 372, 373, 374, 326, 327, 328,
    375, 376, 377, 307, 279, 383, 324, 385, 347, 389,
    391, 394, 396, 398, 347, 348, 279, 399, 400, 400,
    399, 405, 407, 218, 307, 411, 413, 416, 417, 418,
    416, 417, 418, 417, 418, 423, 424, 423, 424, 427,
    338, 429, 429, 432, 434, 239, 344, 279, 398, 83,
    113, 437, 438, 440, 441, 33, 74, 444, 446, 447,
    448, 448, 447, 92, 63, 451, 249, 454, 455, 456,
    457, 438, 458, 318, 319, 220, 459, 460, 461, 462,
    463, 318, 319, 220, 458, 457, 463, 462, 459, 460,
    461, 209, 477, 320, 483, 484, 485, 486, 487, 488,
    492, 493, 494, 495, 496, 497, 499, 500, 501, 502,
    507, 508, 509, 510, 511, 323, 520, 521, 522, 523,
    356, 357, 524, 457, 438, 458, 318, 319, 525, 220,
    459, 526, 315, 527, 460, 451, 461, 528, 529, 462,
    463, 530, 451, 529, 524, 459, 526, 315, 527, 460,
    528, 530, 525, 536, 538, 539, 540, 541, 546, 547,
    529, 358, 322, 329, 329, 322, 552, 83, 113, 558,
    560, 561, 562, 563, 564, 567, 569, 571, 572, 331,
    332, 332, 331, 83, 113, 558, 405, 578, 580, 83,
    239, 113, 558, 405, 581, 582, 583, 584, 585, 586,
    590, 594, 596, 600, 601, 603, 604, 605, 606, 607,
    359, 608, 360, 609, 610, 611, 612, 613, 614, 615,
    616, 617, 618, 619, 620, 621, 623, 624, 625, 628,
    630, 632, 633, 635, 637, 638, 639, 640, 641, 208,
    644, 645, 646, 647, 648, 497, 413, 640, 361, 411,
    654, 655, 383, 324, 656, 111, 657, 497, 413, 640,
    361, 654, 655, 383, 324, 656, 111, 657, 383, 324,
    654, 656, 657, 655, 675, 678, 679, 682, 683, 684,
    685, 527, 526, 524, 459, 530, 528, 696, 698, 699,
    702, 703, 704, 705, 250, 251, 252, 255, 257, 710,
    711, 713, 715, 307, 716, 716, 719, 720, 721, 722,
    723, 724, 725, 726, 727, 728, 729, 732, 501, 536,
    734, 732, 735, 736, 737, 738, 739, 289, 742, 743,
    744, 745, 746, 747, 748, 752, 724, 19, 753, 754,
    25, 28, 755, 724, 753, 754, 755, 724, 753, 755,
    753, 755, 754, 764, 114, 115, 5, 767, 770, 776,
    284, 285, 286, 287, 288, 289, 290, 291, 292, 233,
    780, 780, 15, 764, 785, 529, 786, 33, 114, 115,
    230, 520, 789, 789, 793, 102, 105, 118, 795, 447,
    92, 796, 383, 797, 590, 797, 800, 802, 795, 796,
    383, 795, 795, 796, 796, 806, 808, 810, 719, 811,
    811, 538, 698, 816, 36, 816, 36, 816, 786, 785,
    625, 347, 348, 832, 833, 739, 833, 739, 839, 840,
    841, 734, 842, 842, 844, 844, 849, 850, 509, 851,
    852, 853, 854, 510, 855, 511, 856, 857, 501, 536,
    734, 732, 735, 859, 736, 860, 737, 738, 861, 862,
    739, 863, 865, 736, 738, 737, 735, 873, 356, 357,
    358, 359, 360, 321, 362, 363, 364, 365, 366, 367,
    323, 368, 370, 371, 372, 373, 326, 327, 328, 375,
    376, 377, 369, 374, 321, 882, 883, 884, 885, 886,
    887, 890, 891, 892, 893, 894, 895, 896, 897, 898,
    899, 900, 901, 902, 903, 904, 905, 906, 907, 908,
    909, 910, 911, 912, 913, 914, 915, 321, 323, 325,
    326, 327, 328, 325, 326, 327, 328, 223, 924, 925,
    927, 928, 890, 929, 930, 931, 932, 933, 934, 891,
    892, 935, 936, 362, 363, 937, 558, 939, 940, 941,
    942, 943, 944, 948, 950, 951, 953, 954, 955, 956,
    957, 958, 959, 960, 961, 964, 966, 970, 83, 218,
    274, 441, 284, 621, 625, 411, 729, 101, 102, 424,
    961, 972, 307, 973, 92, 974, 190, 105, 975, 976,
    977, 978, 979, 980, 981, 385, 982, 983, 191, 113,
    279, 984, 33, 117, 985, 118, 120, 986, 558, 405,
    979, 83, 218, 274, 441, 284, 621, 625, 411, 729,
    101, 102, 424, 961, 972, 307, 973, 92, 974, 190,
    105, 975, 976, 977, 978, 980, 981, 385, 982, 983,
    191, 113, 279, 984, 33, 117, 985, 118, 120, 986,
    558, 405, 83, 218, 274, 441, 284, 621, 625, 411,
    729, 101, 102, 424, 961, 972, 307, 973, 92, 974,
    190, 105, 975, 976, 977, 978, 980, 981, 385, 982,
    983, 191, 113, 279, 984, 117, 985, 118, 120, 986,
    558, 405, 964, 966, 424, 961, 993, 994, 995, 996,
    997, 998, 999, 1000, 1001, 1002, 1003, 1004, 1005, 1006,
    972, 1009, 1011, 364, 584, 1013, 1013, 1017, 1018, 1019,
    1020, 1021, 1022, 1023, 1024, 1027, 1028, 1029, 1030, 1031,
    1032, 1034, 1036, 1037, 1039, 1040, 1041, 1042, 1043, 1045,
    1046, 1047, 1048, 1049, 1050, 1051, 1052, 1053, 1054, 973,
    639, 745, 970, 1056, 1056, 1060, 1064, 1065, 383, 1066,
    1067, 63, 383, 523, 383, 524, 318, 319, 383, 1078,
    1079, 318, 319, 1079, 524, 383, 1078, 524, 457, 438,
    458, 318, 319, 220, 459, 526, 315, 527, 460, 461,
    528, 462, 463, 530, 318, 319, 1079, 321, 322, 323,
    324, 325, 326, 327, 328, 329, 221, 98, 223, 1109,
    1111, 1112, 1115, 338, 1117, 1117, 719, 633, 633, 1120,
    405, 1120, 1123, 1124, 862, 863, 861, 860, 859, 1132,
    222, 220, 5, 83, 953, 218, 274, 441, 284, 621,
    625, 411, 729, 101, 102, 424, 961, 972, 307, 973,
    92, 974, 190, 105, 975, 976, 977, 978, 980, 981,
    385, 982, 983, 191, 113, 279, 984, 117, 985, 118,
    120, 986, 558, 405, 1139, 1140, 1143, 1144, 1145, 365,
    1146, 1147, 1148, 1149, 1151, 1152, 1153, 1154, 523, 850,
    323, 1155, 98, 850, 323, 1155, 98, 323, 1155, 98,
    1155, 1148, 1158, 1158, 632, 1162, 1163, 1164, 1165, 625,
    972, 1043, 196, 630, 1168, 253, 628, 1169, 233, 1170,
    1009, 1171, 948, 405, 1170, 1043, 1168, 1169, 1171, 1182,
    1188, 249, 250, 251, 252, 1194, 255, 1195, 257, 258,
    569, 1197, 1197, 251, 1206, 552, 1208, 1209, 1215, 1216,
    221, 5, 221, 454, 320, 356, 357, 358, 359, 360,
    321, 362, 363, 364, 365, 366, 367, 323, 368, 369,
    370, 371, 372, 373, 374, 326, 327, 328, 375, 376,
    377, 361, 654, 1228, 1229, 1230, 974, 1232, 1234, 1235,
    764, 1236, 1237, 742, 405, 1246, 1249, 1253, 1254, 1258,
    1258, 405, 625, 972, 630, 628, 233, 1009, 259, 625,
    972, 233, 1009, 1265, 1266, 625, 972, 628, 233, 1170,
    1009, 259, 1267, 1268, 1269, 1270, 1271, 1272, 1273, 1274,
    894, 1275, 1276, 1277, 895, 563, 321, 924, 892, 895,
    905, 117, 913, 321, 924, 892, 895, 905, 913, 913,
    905, 563, 117, 1284, 1285, 1286, 1287, 1215, 1216, 1064,
    1291, 1292, 1293, 1294, 1295, 1296, 1297, 1298, 1299, 1300,
    1301, 1302, 1303, 1304, 252, 1305, 1306, 1307, 1308, 1310,
    1315, 83, 295, 1317, 1318, 1319, 1320, 1321, 1322, 1323,
    1324, 366, 367, 1325, 1168, 1326, 318, 319, 221, 98,
    223, 221, 98, 223, 1330, 1332, 1335, 1336, 1337, 1338,
    1339, 249, 250, 251, 252, 1194, 255, 257, 258, 1194,
    165, 767, 1195, 251, 255, 257, 255, 257, 1358, 255,
    258, 258, 249, 255, 1361, 1362, 1363, 1364, 1365, 1366,
    1208, 1367, 1368, 1369, 1370, 1371, 1064, 1065, 1066, 1067,
    1064, 1065, 1065, 11, 454, 1376, 1376, 1067, 1066, 1380,
    1381, 975, 1383, 330, 333, 334, 1392, 976, 1395, 1396,
    1398, 1399, 1400, 1401, 1402, 1403, 368, 1404, 1405, 1406,
    1407, 1408, 1409, 1411, 1412, 1413, 1414, 1415, 1416, 287,
    1417, 1418, 1419, 1420, 977, 1432, 102, 118, 1438, 1440,
    1449, 1450, 1451, 1452, 898, 1453, 1454, 1169, 900, 1455,
    1456, 288, 1457, 1458, 1459, 1460, 1461, 904, 1462, 1463,
    1464, 978, 1467, 218, 307, 1468, 1468, 1470, 1471, 1472,
    1479, 1480, 1481, 1483, 1485, 1486, 1488, 1491, 1492, 1493,
    1494, 1495, 1496, 1497, 1498, 1499, 102, 105, 977, 118,
    1516, 1517, 292, 1518, 1519, 1520, 524, 1523, 1524, 1525,
    1526, 1527, 1528, 1529, 370, 1537, 1538, 1539, 1540, 1541,
    1542, 1544, 1545, 1546, 1547, 1548, 1549, 1550, 1551, 1552,
    1553, 1554, 1555, 980, 1115, 630, 628, 1170, 632, 633,
    635, 259, 1560, 1561, 1562, 1563, 1564, 1565, 1567, 1569,
    1570, 1571, 1572, 1573, 1574, 1576, 1577, 1578, 1579, 1580,
    1581, 1582, 1583, 1584, 348, 1589, 1590, 1591, 220, 221,
    223, 1592, 1593, 1594, 1595, 1596, 1597, 1599, 1600, 1601,
    1602, 1606, 1607, 1608, 1609, 1610, 1611, 1612, 1613, 375,
    372, 371, 373, 359, 360, 376, 376, 362, 363, 370,
    377, 1636, 1637, 1638, 1639, 1640, 1641, 1642, 906, 1648,
    1650, 1651, 1652, 1654, 334, 333, 1656, 1657, 1658, 1659,
    1660, 1661, 1662, 1663, 1664, 584, 289, 1013, 279, 251,
    1584, 1668, 1668, 1672, 1673, 1674, 1675, 1676, 1677, 1678,
    1679, 1680, 1681, 1682, 1683, 1684, 981, 1686, 1687, 1688,
    1689, 982, 1691, 16, 17, 227, 228, 1692, 1692, 1695,
    983, 326, 1697, 1697, 1700, 1701, 1702, 1703, 1704, 1705,
    1706, 1707, 1709, 1710, 1711, 1712, 1713, 951, 399, 1716,
    1722, 1723, 1724, 1725, 1726, 1727, 1728, 1729, 1731, 1593,
    633, 1736, 1737, 1738, 1739, 1740, 1741, 1742, 1743, 1744,
    1745, 1746, 908, 1747, 1748, 1749, 1750, 11, 696, 1750,
    11, 696, 1754, 625, 972, 1043, 1115, 92, 1168, 628,
    1169, 233, 1109, 1170, 1009, 1171, 948, 83, 405, 1785,
    1789, 1790, 1793, 1794, 1795, 1043, 1169, 1168, 1171, 1171,
    1804, 1805, 1804, 1805, 1808, 1809, 1810, 1811, 873, 61,
    11, 68, 1813, 124, 11, 1813, 124, 1813, 984, 11,
    350, 101, 873, 68, 394, 61, 1813, 33, 74, 396,
    124, 76, 1823, 1824, 1828, 1830, 1831, 83, 318, 319,
    320, 321, 322, 323, 324, 325, 326, 327, 328, 329,
    210, 330, 331, 332, 221, 98, 333, 334, 223, 1837,
    1839, 1840, 1841, 1842, 1843, 1844, 1845, 1846, 1847, 1848,
    1849, 1850, 1851, 1852, 1853, 1854, 1855, 1856, 1857, 1858,
    1859, 1860, 1861, 1862, 1865, 1866, 1867, 985, 986, 1984,
    1985, 1986, 1987, 719, 1315, 1989, 1990, 1991, 1992, 1997,
    1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007,
    2008, 2009, 2010, 2011, 83, 1517, 411, 18, 656, 657,
    18, 656, 657, 83, 1065, 625, 564, 361, 281, 580,
    347, 348, 719, 1315, 396, 2020, 2024, 114, 115, 764,
    114, 115, 2026, 2026, 2028, 2030, 2031, 325, 326, 405,
    2033, 2033, 2036, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
    2045, 2046, 1209, 2047, 2048, 521, 682, 596, 1710, 780,
    2048, 2052, 2052, 2056, 2057, 2058, 2059, 2061, 2062, 2063,
    2064, 2065, 2066, 2067, 2068, 2072, 2073, 2074, 2076, 2077,
    2078, 2080, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090,
    2092, 1047, 1295, 2100, 2101, 2102, 2103, 2104, 2105, 2106,
    2107, 2108, 2109, 2110, 2111, 2112, 1017, 1701, 2117, 2118,
    2119, 2120, 628, 233, 2121, 2122, 2123, 2124, 2125, 2126,
    2127, 2128, 2130, 2131, 2132, 910, 2133, 2136, 99, 625,
    972, 196, 630, 628, 233, 1109, 1170, 1009, 259, 625,
    972, 196, 233, 1009, 2137, 2141, 233, 347, 2155, 2156,
    2157, 2158, 2159, 2160, 912, 2161, 2162,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 28, token_types_start: 33, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 33, raw_values_count: 1, token_types_start: 34, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 34, raw_values_count: 1, token_types_start: 35, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 35, raw_values_count: 1, token_types_start: 36, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 45, raw_values_count: 1, token_types_start: 46, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 46, raw_values_count: 2, token_types_start: 48, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 49, raw_values_count: 1, token_types_start: 50, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 50, raw_values_count: 1, token_types_start: 51, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 54, raw_values_count: 1, token_types_start: 55, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 55, raw_values_count: 15, token_types_start: 70, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 70, raw_values_count: 15, token_types_start: 85, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 85, raw_values_count: 2, token_types_start: 87, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 87, raw_values_count: 1, token_types_start: 88, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 90, raw_values_count: 1, token_types_start: 91, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 91, raw_values_count: 1, token_types_start: 92, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 92, raw_values_count: 1, token_types_start: 93, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 93, raw_values_count: 0, token_types_start: 93, token_types_count: 1 }, // [37]
    SimpleHintData { raw_values_start: 94, raw_values_count: 26, token_types_start: 120, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 120, raw_values_count: 1, token_types_start: 121, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 121, raw_values_count: 5, token_types_start: 126, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 126, raw_values_count: 1, token_types_start: 127, token_types_count: 0 }, // [41]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 131, raw_values_count: 1, token_types_start: 132, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 132, raw_values_count: 1, token_types_start: 133, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 134, raw_values_count: 2, token_types_start: 136, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 136, raw_values_count: 1, token_types_start: 137, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 137, raw_values_count: 1, token_types_start: 138, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 138, raw_values_count: 25, token_types_start: 163, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 163, raw_values_count: 16, token_types_start: 179, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 181, raw_values_count: 2, token_types_start: 183, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 213, raw_values_count: 2, token_types_start: 215, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 220, raw_values_count: 1, token_types_start: 221, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 221, raw_values_count: 1, token_types_start: 222, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 224, raw_values_count: 1, token_types_start: 225, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 225, raw_values_count: 2, token_types_start: 227, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 232, raw_values_count: 3, token_types_start: 235, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 236, raw_values_count: 2, token_types_start: 238, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 4 }, // [111]
    SimpleHintData { raw_values_start: 247, raw_values_count: 0, token_types_start: 247, token_types_count: 2 }, // [112]
    SimpleHintData { raw_values_start: 249, raw_values_count: 1, token_types_start: 250, token_types_count: 2 }, // [113]
    SimpleHintData { raw_values_start: 252, raw_values_count: 2, token_types_start: 254, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 257, raw_values_count: 1, token_types_start: 258, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 258, raw_values_count: 1, token_types_start: 259, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 259, raw_values_count: 1, token_types_start: 260, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 260, raw_values_count: 12, token_types_start: 272, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 272, raw_values_count: 1, token_types_start: 273, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 273, raw_values_count: 7, token_types_start: 280, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 280, raw_values_count: 1, token_types_start: 281, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 281, raw_values_count: 1, token_types_start: 282, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 282, raw_values_count: 1, token_types_start: 283, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 283, raw_values_count: 1, token_types_start: 284, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 284, raw_values_count: 1, token_types_start: 285, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 285, raw_values_count: 1, token_types_start: 286, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 286, raw_values_count: 1, token_types_start: 287, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 287, raw_values_count: 1, token_types_start: 288, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 288, raw_values_count: 2, token_types_start: 290, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 290, raw_values_count: 1, token_types_start: 291, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 291, raw_values_count: 1, token_types_start: 292, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 292, raw_values_count: 10, token_types_start: 302, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 302, raw_values_count: 1, token_types_start: 303, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 303, raw_values_count: 1, token_types_start: 304, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 304, raw_values_count: 2, token_types_start: 306, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 306, raw_values_count: 2, token_types_start: 308, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 308, raw_values_count: 2, token_types_start: 310, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 310, raw_values_count: 1, token_types_start: 311, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 311, raw_values_count: 1, token_types_start: 312, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 312, raw_values_count: 2, token_types_start: 314, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 314, raw_values_count: 1, token_types_start: 315, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 315, raw_values_count: 1, token_types_start: 316, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 316, raw_values_count: 2, token_types_start: 318, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 318, raw_values_count: 1, token_types_start: 319, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 319, raw_values_count: 1, token_types_start: 320, token_types_count: 0 }, // [148]
    SimpleHintData { raw_values_start: 320, raw_values_count: 1, token_types_start: 321, token_types_count: 0 }, // [149]
    SimpleHintData { raw_values_start: 321, raw_values_count: 1, token_types_start: 322, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 322, raw_values_count: 14, token_types_start: 336, token_types_count: 7 }, // [151]
    SimpleHintData { raw_values_start: 343, raw_values_count: 2, token_types_start: 345, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 345, raw_values_count: 3, token_types_start: 348, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 348, raw_values_count: 2, token_types_start: 350, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 350, raw_values_count: 1, token_types_start: 351, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 351, raw_values_count: 1, token_types_start: 352, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 352, raw_values_count: 2, token_types_start: 354, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 354, raw_values_count: 2, token_types_start: 356, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 356, raw_values_count: 2, token_types_start: 358, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 358, raw_values_count: 1, token_types_start: 359, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 359, raw_values_count: 2, token_types_start: 361, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 361, raw_values_count: 1, token_types_start: 362, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 362, raw_values_count: 33, token_types_start: 395, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 395, raw_values_count: 1, token_types_start: 396, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 396, raw_values_count: 27, token_types_start: 423, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 423, raw_values_count: 2, token_types_start: 425, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 425, raw_values_count: 1, token_types_start: 426, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 426, raw_values_count: 1, token_types_start: 427, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 427, raw_values_count: 1, token_types_start: 428, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 429, raw_values_count: 1, token_types_start: 430, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 430, raw_values_count: 1, token_types_start: 431, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 431, raw_values_count: 1, token_types_start: 432, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 433, raw_values_count: 1, token_types_start: 434, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 434, raw_values_count: 3, token_types_start: 437, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 437, raw_values_count: 2, token_types_start: 439, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 439, raw_values_count: 1, token_types_start: 440, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 440, raw_values_count: 1, token_types_start: 441, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 441, raw_values_count: 1, token_types_start: 442, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 442, raw_values_count: 1, token_types_start: 443, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 443, raw_values_count: 2, token_types_start: 445, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 445, raw_values_count: 1, token_types_start: 446, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 446, raw_values_count: 1, token_types_start: 447, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 447, raw_values_count: 3, token_types_start: 450, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 450, raw_values_count: 1, token_types_start: 451, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 451, raw_values_count: 2, token_types_start: 453, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 453, raw_values_count: 1, token_types_start: 454, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 454, raw_values_count: 1, token_types_start: 455, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 455, raw_values_count: 2, token_types_start: 457, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 458, raw_values_count: 1, token_types_start: 459, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 459, raw_values_count: 1, token_types_start: 460, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 460, raw_values_count: 2, token_types_start: 462, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 462, raw_values_count: 1, token_types_start: 463, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 463, raw_values_count: 1, token_types_start: 464, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 464, raw_values_count: 1, token_types_start: 465, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 465, raw_values_count: 4, token_types_start: 469, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 469, raw_values_count: 2, token_types_start: 471, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 471, raw_values_count: 1, token_types_start: 472, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 472, raw_values_count: 1, token_types_start: 473, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 473, raw_values_count: 1, token_types_start: 474, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 474, raw_values_count: 1, token_types_start: 475, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 475, raw_values_count: 2, token_types_start: 477, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 477, raw_values_count: 1, token_types_start: 478, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 478, raw_values_count: 1, token_types_start: 479, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 479, raw_values_count: 2, token_types_start: 481, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 481, raw_values_count: 1, token_types_start: 482, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 482, raw_values_count: 1, token_types_start: 483, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 483, raw_values_count: 2, token_types_start: 485, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 486, raw_values_count: 1, token_types_start: 487, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 487, raw_values_count: 1, token_types_start: 488, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 490, raw_values_count: 11, token_types_start: 501, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 501, raw_values_count: 1, token_types_start: 502, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 502, raw_values_count: 1, token_types_start: 503, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 503, raw_values_count: 1, token_types_start: 504, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 506, raw_values_count: 1, token_types_start: 507, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 507, raw_values_count: 1, token_types_start: 508, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 508, raw_values_count: 1, token_types_start: 509, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 509, raw_values_count: 1, token_types_start: 510, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 510, raw_values_count: 1, token_types_start: 511, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 511, raw_values_count: 1, token_types_start: 512, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 512, raw_values_count: 1, token_types_start: 513, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 513, raw_values_count: 1, token_types_start: 514, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 514, raw_values_count: 1, token_types_start: 515, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 515, raw_values_count: 1, token_types_start: 516, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 516, raw_values_count: 1, token_types_start: 517, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 517, raw_values_count: 1, token_types_start: 518, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 518, raw_values_count: 1, token_types_start: 519, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 519, raw_values_count: 0, token_types_start: 519, token_types_count: 1 }, // [235]
    SimpleHintData { raw_values_start: 520, raw_values_count: 1, token_types_start: 521, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 521, raw_values_count: 1, token_types_start: 522, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 522, raw_values_count: 1, token_types_start: 523, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 523, raw_values_count: 1, token_types_start: 524, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 528, raw_values_count: 1, token_types_start: 529, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 529, raw_values_count: 0, token_types_start: 529, token_types_count: 1 }, // [245]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 532, raw_values_count: 3, token_types_start: 535, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 536, raw_values_count: 1, token_types_start: 537, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 537, raw_values_count: 1, token_types_start: 538, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 538, raw_values_count: 1, token_types_start: 539, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 539, raw_values_count: 1, token_types_start: 540, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 542, raw_values_count: 19, token_types_start: 561, token_types_count: 1 }, // [256]
    SimpleHintData { raw_values_start: 562, raw_values_count: 2, token_types_start: 564, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 564, raw_values_count: 7, token_types_start: 571, token_types_count: 1 }, // [258]
    SimpleHintData { raw_values_start: 572, raw_values_count: 1, token_types_start: 573, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 573, raw_values_count: 1, token_types_start: 574, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 574, raw_values_count: 1, token_types_start: 575, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 575, raw_values_count: 1, token_types_start: 576, token_types_count: 0 }, // [262]
    SimpleHintData { raw_values_start: 576, raw_values_count: 1, token_types_start: 577, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 578, raw_values_count: 1, token_types_start: 579, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 579, raw_values_count: 1, token_types_start: 580, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 580, raw_values_count: 1, token_types_start: 581, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 581, raw_values_count: 1, token_types_start: 582, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 582, raw_values_count: 2, token_types_start: 584, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 584, raw_values_count: 1, token_types_start: 585, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 585, raw_values_count: 1, token_types_start: 586, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 586, raw_values_count: 1, token_types_start: 587, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 587, raw_values_count: 3, token_types_start: 590, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 590, raw_values_count: 1, token_types_start: 591, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 591, raw_values_count: 1, token_types_start: 592, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 592, raw_values_count: 1, token_types_start: 593, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 593, raw_values_count: 1, token_types_start: 594, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 594, raw_values_count: 1, token_types_start: 595, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 595, raw_values_count: 1, token_types_start: 596, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 596, raw_values_count: 1, token_types_start: 597, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 597, raw_values_count: 1, token_types_start: 598, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 598, raw_values_count: 1, token_types_start: 599, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 599, raw_values_count: 0, token_types_start: 599, token_types_count: 2 }, // [283]
    SimpleHintData { raw_values_start: 601, raw_values_count: 0, token_types_start: 601, token_types_count: 1 }, // [284]
    SimpleHintData { raw_values_start: 602, raw_values_count: 0, token_types_start: 602, token_types_count: 1 }, // [285]
    SimpleHintData { raw_values_start: 603, raw_values_count: 4, token_types_start: 607, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 607, raw_values_count: 1, token_types_start: 608, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 608, raw_values_count: 1, token_types_start: 609, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 609, raw_values_count: 5, token_types_start: 614, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 614, raw_values_count: 1, token_types_start: 615, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 617, raw_values_count: 1, token_types_start: 618, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 618, raw_values_count: 1, token_types_start: 619, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 619, raw_values_count: 1, token_types_start: 620, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 620, raw_values_count: 1, token_types_start: 621, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 621, raw_values_count: 1, token_types_start: 622, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 623, raw_values_count: 1, token_types_start: 624, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 625, raw_values_count: 1, token_types_start: 626, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 626, raw_values_count: 1, token_types_start: 627, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 628, raw_values_count: 1, token_types_start: 629, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 629, raw_values_count: 1, token_types_start: 630, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 630, raw_values_count: 1, token_types_start: 631, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 631, raw_values_count: 1, token_types_start: 632, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 635, raw_values_count: 1, token_types_start: 636, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 636, raw_values_count: 1, token_types_start: 637, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 637, raw_values_count: 1, token_types_start: 638, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 640, raw_values_count: 1, token_types_start: 641, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 641, raw_values_count: 1, token_types_start: 642, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 642, raw_values_count: 1, token_types_start: 643, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 644, raw_values_count: 1, token_types_start: 645, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 645, raw_values_count: 1, token_types_start: 646, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 646, raw_values_count: 1, token_types_start: 647, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 647, raw_values_count: 1, token_types_start: 648, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 648, raw_values_count: 1, token_types_start: 649, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 649, raw_values_count: 1, token_types_start: 650, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 650, raw_values_count: 1, token_types_start: 651, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 651, raw_values_count: 2, token_types_start: 653, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 653, raw_values_count: 1, token_types_start: 654, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 654, raw_values_count: 1, token_types_start: 655, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 655, raw_values_count: 1, token_types_start: 656, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 656, raw_values_count: 1, token_types_start: 657, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 657, raw_values_count: 1, token_types_start: 658, token_types_count: 0 }, // [332]
    SimpleHintData { raw_values_start: 658, raw_values_count: 1, token_types_start: 659, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 659, raw_values_count: 1, token_types_start: 660, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 660, raw_values_count: 1, token_types_start: 661, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 661, raw_values_count: 1, token_types_start: 662, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 662, raw_values_count: 1, token_types_start: 663, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 663, raw_values_count: 1, token_types_start: 664, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 664, raw_values_count: 1, token_types_start: 665, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 665, raw_values_count: 12, token_types_start: 677, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 677, raw_values_count: 11, token_types_start: 688, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 688, raw_values_count: 2, token_types_start: 690, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 690, raw_values_count: 1, token_types_start: 691, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 691, raw_values_count: 1, token_types_start: 692, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 692, raw_values_count: 1, token_types_start: 693, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 693, raw_values_count: 1, token_types_start: 694, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 694, raw_values_count: 1, token_types_start: 695, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 695, raw_values_count: 1, token_types_start: 696, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 696, raw_values_count: 1, token_types_start: 697, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 697, raw_values_count: 1, token_types_start: 698, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 698, raw_values_count: 1, token_types_start: 699, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 699, raw_values_count: 1, token_types_start: 700, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 700, raw_values_count: 1, token_types_start: 701, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 701, raw_values_count: 1, token_types_start: 702, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 702, raw_values_count: 1, token_types_start: 703, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 703, raw_values_count: 2, token_types_start: 705, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 705, raw_values_count: 0, token_types_start: 705, token_types_count: 1 }, // [357]
    SimpleHintData { raw_values_start: 706, raw_values_count: 1, token_types_start: 707, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 708, raw_values_count: 1, token_types_start: 709, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 709, raw_values_count: 1, token_types_start: 710, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 710, raw_values_count: 1, token_types_start: 711, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 711, raw_values_count: 1, token_types_start: 712, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 712, raw_values_count: 1, token_types_start: 713, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 713, raw_values_count: 1, token_types_start: 714, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 714, raw_values_count: 1, token_types_start: 715, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 715, raw_values_count: 4, token_types_start: 719, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 719, raw_values_count: 1, token_types_start: 720, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 720, raw_values_count: 1, token_types_start: 721, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 721, raw_values_count: 1, token_types_start: 722, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 722, raw_values_count: 1, token_types_start: 723, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 723, raw_values_count: 2, token_types_start: 725, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 725, raw_values_count: 1, token_types_start: 726, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 726, raw_values_count: 1, token_types_start: 727, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 727, raw_values_count: 1, token_types_start: 728, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 728, raw_values_count: 1, token_types_start: 729, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 729, raw_values_count: 1, token_types_start: 730, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 730, raw_values_count: 1, token_types_start: 731, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 731, raw_values_count: 1, token_types_start: 732, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 732, raw_values_count: 1, token_types_start: 733, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 733, raw_values_count: 1, token_types_start: 734, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 734, raw_values_count: 1, token_types_start: 735, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 737, raw_values_count: 1, token_types_start: 738, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 738, raw_values_count: 9, token_types_start: 747, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 748, raw_values_count: 1, token_types_start: 749, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 749, raw_values_count: 1, token_types_start: 750, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 752, raw_values_count: 1, token_types_start: 753, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 753, raw_values_count: 1, token_types_start: 754, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 756, raw_values_count: 7, token_types_start: 763, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 763, raw_values_count: 4, token_types_start: 767, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 767, raw_values_count: 3, token_types_start: 770, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 770, raw_values_count: 1, token_types_start: 771, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [400]
    SimpleHintData { raw_values_start: 772, raw_values_count: 1, token_types_start: 773, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 773, raw_values_count: 3, token_types_start: 776, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 776, raw_values_count: 0, token_types_start: 776, token_types_count: 1 }, // [403]
    SimpleHintData { raw_values_start: 777, raw_values_count: 1, token_types_start: 778, token_types_count: 0 }, // [404]
    SimpleHintData { raw_values_start: 778, raw_values_count: 1, token_types_start: 779, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 779, raw_values_count: 1, token_types_start: 780, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 780, raw_values_count: 11, token_types_start: 791, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 791, raw_values_count: 1, token_types_start: 792, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 792, raw_values_count: 8, token_types_start: 800, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 800, raw_values_count: 3, token_types_start: 803, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 803, raw_values_count: 1, token_types_start: 804, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 804, raw_values_count: 1, token_types_start: 805, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 805, raw_values_count: 3, token_types_start: 808, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 808, raw_values_count: 7, token_types_start: 815, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 815, raw_values_count: 1, token_types_start: 816, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 816, raw_values_count: 1, token_types_start: 817, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 817, raw_values_count: 1, token_types_start: 818, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 818, raw_values_count: 3, token_types_start: 821, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 822, raw_values_count: 2, token_types_start: 824, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 824, raw_values_count: 1, token_types_start: 825, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 825, raw_values_count: 1, token_types_start: 826, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 826, raw_values_count: 1, token_types_start: 827, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 827, raw_values_count: 1, token_types_start: 828, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 828, raw_values_count: 2, token_types_start: 830, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 830, raw_values_count: 1, token_types_start: 831, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 831, raw_values_count: 4, token_types_start: 835, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 835, raw_values_count: 2, token_types_start: 837, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 837, raw_values_count: 1, token_types_start: 838, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 838, raw_values_count: 1, token_types_start: 839, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 839, raw_values_count: 1, token_types_start: 840, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 840, raw_values_count: 3, token_types_start: 843, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 843, raw_values_count: 1, token_types_start: 844, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 844, raw_values_count: 2, token_types_start: 846, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 846, raw_values_count: 1, token_types_start: 847, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 847, raw_values_count: 1, token_types_start: 848, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 848, raw_values_count: 1, token_types_start: 849, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 849, raw_values_count: 1, token_types_start: 850, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 850, raw_values_count: 1, token_types_start: 851, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 851, raw_values_count: 1, token_types_start: 852, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 852, raw_values_count: 1, token_types_start: 853, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 853, raw_values_count: 2, token_types_start: 855, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 855, raw_values_count: 1, token_types_start: 856, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 856, raw_values_count: 1, token_types_start: 857, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 857, raw_values_count: 1, token_types_start: 858, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 858, raw_values_count: 1, token_types_start: 859, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 859, raw_values_count: 1, token_types_start: 860, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 860, raw_values_count: 1, token_types_start: 861, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 861, raw_values_count: 1, token_types_start: 862, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 862, raw_values_count: 1, token_types_start: 863, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 863, raw_values_count: 1, token_types_start: 864, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 864, raw_values_count: 1, token_types_start: 865, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 865, raw_values_count: 1, token_types_start: 866, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 866, raw_values_count: 1, token_types_start: 867, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 867, raw_values_count: 1, token_types_start: 868, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 868, raw_values_count: 14, token_types_start: 882, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 882, raw_values_count: 1, token_types_start: 883, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [458]
    SimpleHintData { raw_values_start: 884, raw_values_count: 1, token_types_start: 885, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 885, raw_values_count: 1, token_types_start: 886, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 886, raw_values_count: 1, token_types_start: 887, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 887, raw_values_count: 1, token_types_start: 888, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 888, raw_values_count: 24, token_types_start: 912, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 912, raw_values_count: 1, token_types_start: 913, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 913, raw_values_count: 1, token_types_start: 914, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 914, raw_values_count: 1, token_types_start: 915, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 915, raw_values_count: 6, token_types_start: 921, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 921, raw_values_count: 26, token_types_start: 947, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 947, raw_values_count: 6, token_types_start: 953, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 953, raw_values_count: 1, token_types_start: 954, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 954, raw_values_count: 1, token_types_start: 955, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 955, raw_values_count: 1, token_types_start: 956, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 956, raw_values_count: 1, token_types_start: 957, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 957, raw_values_count: 0, token_types_start: 957, token_types_count: 1 }, // [474]
    SimpleHintData { raw_values_start: 958, raw_values_count: 1, token_types_start: 959, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 959, raw_values_count: 1, token_types_start: 960, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 960, raw_values_count: 1, token_types_start: 961, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 961, raw_values_count: 1, token_types_start: 962, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 962, raw_values_count: 1, token_types_start: 963, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 963, raw_values_count: 1, token_types_start: 964, token_types_count: 0 }, // [480]
    SimpleHintData { raw_values_start: 964, raw_values_count: 1, token_types_start: 965, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 965, raw_values_count: 1, token_types_start: 966, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 966, raw_values_count: 1, token_types_start: 967, token_types_count: 0 }, // [483]
    SimpleHintData { raw_values_start: 967, raw_values_count: 1, token_types_start: 968, token_types_count: 0 }, // [484]
    SimpleHintData { raw_values_start: 968, raw_values_count: 1, token_types_start: 969, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 969, raw_values_count: 1, token_types_start: 970, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 970, raw_values_count: 1, token_types_start: 971, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 971, raw_values_count: 1, token_types_start: 972, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 973, raw_values_count: 1, token_types_start: 974, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 974, raw_values_count: 1, token_types_start: 975, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 975, raw_values_count: 1, token_types_start: 976, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 976, raw_values_count: 1, token_types_start: 977, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 977, raw_values_count: 1, token_types_start: 978, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 978, raw_values_count: 1, token_types_start: 979, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 979, raw_values_count: 1, token_types_start: 980, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 980, raw_values_count: 1, token_types_start: 981, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 985, raw_values_count: 1, token_types_start: 986, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 986, raw_values_count: 1, token_types_start: 987, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 989, raw_values_count: 1, token_types_start: 990, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 990, raw_values_count: 1, token_types_start: 991, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 993, raw_values_count: 1, token_types_start: 994, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 994, raw_values_count: 1, token_types_start: 995, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 996, raw_values_count: 1, token_types_start: 997, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 997, raw_values_count: 1, token_types_start: 998, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 998, raw_values_count: 42, token_types_start: 1040, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1040, raw_values_count: 1, token_types_start: 1041, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1041, raw_values_count: 41, token_types_start: 1082, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1082, raw_values_count: 40, token_types_start: 1122, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 2, token_types_start: 1124, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 2, token_types_start: 1126, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 1, token_types_start: 1133, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1133, raw_values_count: 1, token_types_start: 1134, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1134, raw_values_count: 1, token_types_start: 1135, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 1, token_types_start: 1141, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1141, raw_values_count: 1, token_types_start: 1142, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 1, token_types_start: 1143, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1143, raw_values_count: 1, token_types_start: 1144, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1144, raw_values_count: 2, token_types_start: 1146, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1146, raw_values_count: 1, token_types_start: 1147, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1147, raw_values_count: 1, token_types_start: 1148, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1148, raw_values_count: 1, token_types_start: 1149, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 1, token_types_start: 1150, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1150, raw_values_count: 1, token_types_start: 1151, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 1, token_types_start: 1152, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1152, raw_values_count: 1, token_types_start: 1153, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 1, token_types_start: 1155, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 1, token_types_start: 1156, token_types_count: 0 }, // [549]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 4, token_types_start: 1184, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 5, token_types_start: 1191, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1191, raw_values_count: 2, token_types_start: 1193, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 2, token_types_start: 1195, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1195, raw_values_count: 6, token_types_start: 1201, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1201, raw_values_count: 2, token_types_start: 1203, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1203, raw_values_count: 1, token_types_start: 1204, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1204, raw_values_count: 2, token_types_start: 1206, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1206, raw_values_count: 1, token_types_start: 1207, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1207, raw_values_count: 16, token_types_start: 1223, token_types_count: 1 }, // [585]
    SimpleHintData { raw_values_start: 1224, raw_values_count: 3, token_types_start: 1227, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 9, token_types_start: 1236, token_types_count: 3 }, // [587]
    SimpleHintData { raw_values_start: 1239, raw_values_count: 1, token_types_start: 1240, token_types_count: 0 }, // [588]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 1, token_types_start: 1241, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1241, raw_values_count: 1, token_types_start: 1242, token_types_count: 0 }, // [590]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [591]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 2, token_types_start: 1245, token_types_count: 0 }, // [592]
    SimpleHintData { raw_values_start: 1245, raw_values_count: 1, token_types_start: 1246, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1246, raw_values_count: 2, token_types_start: 1248, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1248, raw_values_count: 1, token_types_start: 1249, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1249, raw_values_count: 2, token_types_start: 1251, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 0, token_types_start: 1260, token_types_count: 1 }, // [606]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 1 }, // [607]
    SimpleHintData { raw_values_start: 1263, raw_values_count: 41, token_types_start: 1304, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 1, token_types_start: 1305, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1305, raw_values_count: 1, token_types_start: 1306, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1306, raw_values_count: 1, token_types_start: 1307, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1307, raw_values_count: 1, token_types_start: 1308, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1308, raw_values_count: 1, token_types_start: 1309, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1309, raw_values_count: 1, token_types_start: 1310, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1310, raw_values_count: 1, token_types_start: 1311, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1311, raw_values_count: 1, token_types_start: 1312, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1312, raw_values_count: 1, token_types_start: 1313, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1313, raw_values_count: 1, token_types_start: 1314, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1314, raw_values_count: 1, token_types_start: 1315, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1315, raw_values_count: 1, token_types_start: 1316, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1316, raw_values_count: 1, token_types_start: 1317, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1317, raw_values_count: 1, token_types_start: 1318, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1318, raw_values_count: 4, token_types_start: 1322, token_types_count: 1 }, // [623]
    SimpleHintData { raw_values_start: 1323, raw_values_count: 3, token_types_start: 1326, token_types_count: 1 }, // [624]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 2, token_types_start: 1329, token_types_count: 1 }, // [625]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 2, token_types_start: 1333, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 1, token_types_start: 1334, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 1, token_types_start: 1336, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 1, token_types_start: 1337, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 1, token_types_start: 1338, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 1, token_types_start: 1339, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1339, raw_values_count: 15, token_types_start: 1354, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1354, raw_values_count: 1, token_types_start: 1355, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 4, token_types_start: 1359, token_types_count: 0 }, // [636]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1360, raw_values_count: 1, token_types_start: 1361, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 9, token_types_start: 1370, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 2, token_types_start: 1372, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 1, token_types_start: 1374, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 3, token_types_start: 1378, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1378, raw_values_count: 2, token_types_start: 1380, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1380, raw_values_count: 0, token_types_start: 1380, token_types_count: 2 }, // [646]
    SimpleHintData { raw_values_start: 1382, raw_values_count: 0, token_types_start: 1382, token_types_count: 1 }, // [647]
    SimpleHintData { raw_values_start: 1383, raw_values_count: 28, token_types_start: 1411, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1411, raw_values_count: 2, token_types_start: 1413, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1413, raw_values_count: 1, token_types_start: 1414, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1414, raw_values_count: 1, token_types_start: 1415, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1415, raw_values_count: 1, token_types_start: 1416, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1416, raw_values_count: 1, token_types_start: 1417, token_types_count: 0 }, // [653]
    SimpleHintData { raw_values_start: 1417, raw_values_count: 1, token_types_start: 1418, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1418, raw_values_count: 1, token_types_start: 1419, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1419, raw_values_count: 0, token_types_start: 1419, token_types_count: 1 }, // [656]
    SimpleHintData { raw_values_start: 1420, raw_values_count: 1, token_types_start: 1421, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1421, raw_values_count: 1, token_types_start: 1422, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1422, raw_values_count: 1, token_types_start: 1423, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 2, token_types_start: 1425, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1425, raw_values_count: 1, token_types_start: 1426, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1426, raw_values_count: 1, token_types_start: 1427, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1427, raw_values_count: 1, token_types_start: 1428, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1428, raw_values_count: 1, token_types_start: 1429, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1429, raw_values_count: 1, token_types_start: 1430, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1430, raw_values_count: 2, token_types_start: 1432, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 7, token_types_start: 1439, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1439, raw_values_count: 4, token_types_start: 1443, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 7, token_types_start: 1452, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1453, raw_values_count: 1, token_types_start: 1454, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1454, raw_values_count: 1, token_types_start: 1455, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1455, raw_values_count: 1, token_types_start: 1456, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1456, raw_values_count: 1, token_types_start: 1457, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1457, raw_values_count: 1, token_types_start: 1458, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1458, raw_values_count: 1, token_types_start: 1459, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 1, token_types_start: 1462, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1462, raw_values_count: 1, token_types_start: 1463, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1463, raw_values_count: 1, token_types_start: 1464, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1464, raw_values_count: 1, token_types_start: 1465, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 8, token_types_start: 1473, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 6, token_types_start: 1479, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1479, raw_values_count: 1, token_types_start: 1480, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1480, raw_values_count: 1, token_types_start: 1481, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1481, raw_values_count: 2, token_types_start: 1483, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1483, raw_values_count: 1, token_types_start: 1484, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1484, raw_values_count: 1, token_types_start: 1485, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1485, raw_values_count: 1, token_types_start: 1486, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1486, raw_values_count: 1, token_types_start: 1487, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 1, token_types_start: 1500, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1500, raw_values_count: 1, token_types_start: 1501, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 1, token_types_start: 1503, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1503, raw_values_count: 1, token_types_start: 1504, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 1, token_types_start: 1507, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1507, raw_values_count: 1, token_types_start: 1508, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1508, raw_values_count: 1, token_types_start: 1509, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 1, token_types_start: 1510, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1510, raw_values_count: 1, token_types_start: 1511, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1511, raw_values_count: 2, token_types_start: 1513, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 1, token_types_start: 1514, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1514, raw_values_count: 1, token_types_start: 1515, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 1, token_types_start: 1517, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1517, raw_values_count: 1, token_types_start: 1518, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 1, token_types_start: 1519, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1519, raw_values_count: 1, token_types_start: 1520, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1520, raw_values_count: 1, token_types_start: 1521, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1521, raw_values_count: 1, token_types_start: 1522, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 1, token_types_start: 1523, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1523, raw_values_count: 1, token_types_start: 1524, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1524, raw_values_count: 1, token_types_start: 1525, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 2, token_types_start: 1528, token_types_count: 3 }, // [732]
    SimpleHintData { raw_values_start: 1531, raw_values_count: 0, token_types_start: 1531, token_types_count: 3 }, // [733]
    SimpleHintData { raw_values_start: 1534, raw_values_count: 1, token_types_start: 1535, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 8, token_types_start: 1549, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1549, raw_values_count: 1, token_types_start: 1550, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1550, raw_values_count: 2, token_types_start: 1552, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1552, raw_values_count: 1, token_types_start: 1553, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 3, token_types_start: 1556, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1556, raw_values_count: 1, token_types_start: 1557, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1557, raw_values_count: 1, token_types_start: 1558, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 2, token_types_start: 1561, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1561, raw_values_count: 1, token_types_start: 1562, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 2, token_types_start: 1564, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 1, token_types_start: 1565, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 1, token_types_start: 1566, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 1, token_types_start: 1567, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1567, raw_values_count: 1, token_types_start: 1568, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1568, raw_values_count: 1, token_types_start: 1569, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 1, token_types_start: 1570, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1570, raw_values_count: 1, token_types_start: 1571, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 1, token_types_start: 1575, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1575, raw_values_count: 1, token_types_start: 1576, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1576, raw_values_count: 4, token_types_start: 1580, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1580, raw_values_count: 2, token_types_start: 1582, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1582, raw_values_count: 1, token_types_start: 1583, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1583, raw_values_count: 3, token_types_start: 1586, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 1, token_types_start: 1588, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 1, token_types_start: 1593, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1593, raw_values_count: 1, token_types_start: 1594, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1594, raw_values_count: 0, token_types_start: 1594, token_types_count: 2 }, // [776]
    SimpleHintData { raw_values_start: 1596, raw_values_count: 1, token_types_start: 1597, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1597, raw_values_count: 1, token_types_start: 1598, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 1, token_types_start: 1599, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1599, raw_values_count: 1, token_types_start: 1600, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 1, token_types_start: 1602, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1602, raw_values_count: 1, token_types_start: 1603, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 1, token_types_start: 1605, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1605, raw_values_count: 1, token_types_start: 1606, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1606, raw_values_count: 1, token_types_start: 1607, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1607, raw_values_count: 1, token_types_start: 1608, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1608, raw_values_count: 1, token_types_start: 1609, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1609, raw_values_count: 1, token_types_start: 1610, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1610, raw_values_count: 1, token_types_start: 1611, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1611, raw_values_count: 1, token_types_start: 1612, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1612, raw_values_count: 1, token_types_start: 1613, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1613, raw_values_count: 1, token_types_start: 1614, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1614, raw_values_count: 1, token_types_start: 1615, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1615, raw_values_count: 1, token_types_start: 1616, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1616, raw_values_count: 1, token_types_start: 1617, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1617, raw_values_count: 1, token_types_start: 1618, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1618, raw_values_count: 1, token_types_start: 1619, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1619, raw_values_count: 1, token_types_start: 1620, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1620, raw_values_count: 1, token_types_start: 1621, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1621, raw_values_count: 1, token_types_start: 1622, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1622, raw_values_count: 1, token_types_start: 1623, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1623, raw_values_count: 1, token_types_start: 1624, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1624, raw_values_count: 1, token_types_start: 1625, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1625, raw_values_count: 1, token_types_start: 1626, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 2, token_types_start: 1628, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 1, token_types_start: 1633, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1633, raw_values_count: 1, token_types_start: 1634, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1634, raw_values_count: 1, token_types_start: 1635, token_types_count: 0 }, // [814]
    SimpleHintData { raw_values_start: 1635, raw_values_count: 1, token_types_start: 1636, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1636, raw_values_count: 1, token_types_start: 1637, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1637, raw_values_count: 1, token_types_start: 1638, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1638, raw_values_count: 1, token_types_start: 1639, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 1, token_types_start: 1640, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1640, raw_values_count: 1, token_types_start: 1641, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 1, token_types_start: 1643, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1643, raw_values_count: 1, token_types_start: 1644, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1644, raw_values_count: 1, token_types_start: 1645, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1645, raw_values_count: 1, token_types_start: 1646, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1646, raw_values_count: 1, token_types_start: 1647, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1647, raw_values_count: 1, token_types_start: 1648, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 1, token_types_start: 1649, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1649, raw_values_count: 1, token_types_start: 1650, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1650, raw_values_count: 1, token_types_start: 1651, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1651, raw_values_count: 1, token_types_start: 1652, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1652, raw_values_count: 1, token_types_start: 1653, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1653, raw_values_count: 3, token_types_start: 1656, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1656, raw_values_count: 1, token_types_start: 1657, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 1, token_types_start: 1658, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1658, raw_values_count: 1, token_types_start: 1659, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 1, token_types_start: 1660, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1660, raw_values_count: 1, token_types_start: 1661, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 1, token_types_start: 1664, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1664, raw_values_count: 1, token_types_start: 1665, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 1, token_types_start: 1667, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1667, raw_values_count: 1, token_types_start: 1668, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1668, raw_values_count: 1, token_types_start: 1669, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 1, token_types_start: 1674, token_types_count: 0 }, // [851]
    SimpleHintData { raw_values_start: 1674, raw_values_count: 1, token_types_start: 1675, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 1675, raw_values_count: 1, token_types_start: 1676, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 4, token_types_start: 1680, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 1, token_types_start: 1688, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 1688, raw_values_count: 1, token_types_start: 1689, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 1689, raw_values_count: 1, token_types_start: 1690, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 1690, raw_values_count: 1, token_types_start: 1691, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 1, token_types_start: 1693, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 1693, raw_values_count: 1, token_types_start: 1694, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 1, token_types_start: 1695, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 1695, raw_values_count: 1, token_types_start: 1696, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 1696, raw_values_count: 1, token_types_start: 1697, token_types_count: 0 }, // [871]
    SimpleHintData { raw_values_start: 1697, raw_values_count: 1, token_types_start: 1698, token_types_count: 0 }, // [872]
    SimpleHintData { raw_values_start: 1698, raw_values_count: 1, token_types_start: 1699, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 1, token_types_start: 1702, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 1702, raw_values_count: 1, token_types_start: 1703, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 1703, raw_values_count: 1, token_types_start: 1704, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 1704, raw_values_count: 1, token_types_start: 1705, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 1705, raw_values_count: 1, token_types_start: 1706, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 1706, raw_values_count: 1, token_types_start: 1707, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 1707, raw_values_count: 1, token_types_start: 1708, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 1708, raw_values_count: 1, token_types_start: 1709, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 1709, raw_values_count: 1, token_types_start: 1710, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 1710, raw_values_count: 1, token_types_start: 1711, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 1711, raw_values_count: 1, token_types_start: 1712, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 1712, raw_values_count: 1, token_types_start: 1713, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 1713, raw_values_count: 1, token_types_start: 1714, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 1714, raw_values_count: 8, token_types_start: 1722, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 1722, raw_values_count: 1, token_types_start: 1723, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 1723, raw_values_count: 1, token_types_start: 1724, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 1724, raw_values_count: 1, token_types_start: 1725, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 1, token_types_start: 1726, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 1726, raw_values_count: 1, token_types_start: 1727, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 1727, raw_values_count: 1, token_types_start: 1728, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 1728, raw_values_count: 1, token_types_start: 1729, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 1729, raw_values_count: 1, token_types_start: 1730, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 1730, raw_values_count: 1, token_types_start: 1731, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 1731, raw_values_count: 1, token_types_start: 1732, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 1732, raw_values_count: 1, token_types_start: 1733, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 1733, raw_values_count: 1, token_types_start: 1734, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 1734, raw_values_count: 1, token_types_start: 1735, token_types_count: 0 }, // [902]
    SimpleHintData { raw_values_start: 1735, raw_values_count: 1, token_types_start: 1736, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 1736, raw_values_count: 1, token_types_start: 1737, token_types_count: 0 }, // [904]
    SimpleHintData { raw_values_start: 1737, raw_values_count: 1, token_types_start: 1738, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 1738, raw_values_count: 1, token_types_start: 1739, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 1739, raw_values_count: 1, token_types_start: 1740, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 1740, raw_values_count: 1, token_types_start: 1741, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 1741, raw_values_count: 1, token_types_start: 1742, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 1742, raw_values_count: 1, token_types_start: 1743, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 1743, raw_values_count: 1, token_types_start: 1744, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 1744, raw_values_count: 1, token_types_start: 1745, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 1745, raw_values_count: 1, token_types_start: 1746, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 1746, raw_values_count: 1, token_types_start: 1747, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 1747, raw_values_count: 1, token_types_start: 1748, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 2 }, // [916]
    SimpleHintData { raw_values_start: 1751, raw_values_count: 1, token_types_start: 1752, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 1752, raw_values_count: 1, token_types_start: 1753, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 1753, raw_values_count: 1, token_types_start: 1754, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 1754, raw_values_count: 1, token_types_start: 1755, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 1755, raw_values_count: 1, token_types_start: 1756, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 1756, raw_values_count: 1, token_types_start: 1757, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 1757, raw_values_count: 1, token_types_start: 1758, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 1, token_types_start: 1759, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 1759, raw_values_count: 1, token_types_start: 1760, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 1760, raw_values_count: 1, token_types_start: 1761, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 1, token_types_start: 1763, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 1763, raw_values_count: 1, token_types_start: 1764, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 1764, raw_values_count: 1, token_types_start: 1765, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 1765, raw_values_count: 1, token_types_start: 1766, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 1766, raw_values_count: 1, token_types_start: 1767, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 1767, raw_values_count: 1, token_types_start: 1768, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 1768, raw_values_count: 1, token_types_start: 1769, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 1769, raw_values_count: 1, token_types_start: 1770, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 1770, raw_values_count: 1, token_types_start: 1771, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 1771, raw_values_count: 1, token_types_start: 1772, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 1772, raw_values_count: 1, token_types_start: 1773, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 1773, raw_values_count: 3, token_types_start: 1776, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 1776, raw_values_count: 1, token_types_start: 1777, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 3, token_types_start: 1780, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 1780, raw_values_count: 1, token_types_start: 1781, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 1781, raw_values_count: 1, token_types_start: 1782, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 1, token_types_start: 1784, token_types_count: 0 }, // [945]
    SimpleHintData { raw_values_start: 1784, raw_values_count: 1, token_types_start: 1785, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 1785, raw_values_count: 1, token_types_start: 1786, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 1786, raw_values_count: 1, token_types_start: 1787, token_types_count: 0 }, // [948]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 1, token_types_start: 1788, token_types_count: 0 }, // [949]
    SimpleHintData { raw_values_start: 1788, raw_values_count: 1, token_types_start: 1789, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 1789, raw_values_count: 1, token_types_start: 1790, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 1790, raw_values_count: 1, token_types_start: 1791, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 1791, raw_values_count: 1, token_types_start: 1792, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 1792, raw_values_count: 1, token_types_start: 1793, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 1793, raw_values_count: 1, token_types_start: 1794, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 0, token_types_start: 1794, token_types_count: 1 }, // [956]
    SimpleHintData { raw_values_start: 1795, raw_values_count: 0, token_types_start: 1795, token_types_count: 1 }, // [957]
    SimpleHintData { raw_values_start: 1796, raw_values_count: 1, token_types_start: 1797, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 1, token_types_start: 1798, token_types_count: 0 }, // [959]
    SimpleHintData { raw_values_start: 1798, raw_values_count: 1, token_types_start: 1799, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [961]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 1, token_types_start: 1801, token_types_count: 0 }, // [962]
    SimpleHintData { raw_values_start: 1801, raw_values_count: 1, token_types_start: 1802, token_types_count: 0 }, // [963]
    SimpleHintData { raw_values_start: 1802, raw_values_count: 1, token_types_start: 1803, token_types_count: 0 }, // [964]
    SimpleHintData { raw_values_start: 1803, raw_values_count: 1, token_types_start: 1804, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 1, token_types_start: 1805, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 1805, raw_values_count: 4, token_types_start: 1809, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 1809, raw_values_count: 3, token_types_start: 1812, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 1812, raw_values_count: 1, token_types_start: 1813, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 1813, raw_values_count: 1, token_types_start: 1814, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 1814, raw_values_count: 1, token_types_start: 1815, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 1815, raw_values_count: 1, token_types_start: 1816, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 1816, raw_values_count: 1, token_types_start: 1817, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 1817, raw_values_count: 1, token_types_start: 1818, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 1818, raw_values_count: 1, token_types_start: 1819, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 1819, raw_values_count: 1, token_types_start: 1820, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 1820, raw_values_count: 1, token_types_start: 1821, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 1821, raw_values_count: 1, token_types_start: 1822, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 1822, raw_values_count: 1, token_types_start: 1823, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 1823, raw_values_count: 1, token_types_start: 1824, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 1824, raw_values_count: 1, token_types_start: 1825, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 1, token_types_start: 1829, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 1829, raw_values_count: 1, token_types_start: 1830, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 5, token_types_start: 1838, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 1838, raw_values_count: 1, token_types_start: 1839, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 1839, raw_values_count: 1, token_types_start: 1840, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 1840, raw_values_count: 1, token_types_start: 1841, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 2, token_types_start: 1843, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 1, token_types_start: 1844, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 1844, raw_values_count: 1, token_types_start: 1845, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 1845, raw_values_count: 1, token_types_start: 1846, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 1846, raw_values_count: 1, token_types_start: 1847, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 1847, raw_values_count: 1, token_types_start: 1848, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 1848, raw_values_count: 1, token_types_start: 1849, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 1849, raw_values_count: 1, token_types_start: 1850, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 1850, raw_values_count: 1, token_types_start: 1851, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 1851, raw_values_count: 1, token_types_start: 1852, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 1852, raw_values_count: 1, token_types_start: 1853, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 1, token_types_start: 1855, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 1855, raw_values_count: 1, token_types_start: 1856, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 2, token_types_start: 1859, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 1859, raw_values_count: 1, token_types_start: 1860, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 1860, raw_values_count: 1, token_types_start: 1861, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 1861, raw_values_count: 1, token_types_start: 1862, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 1862, raw_values_count: 1, token_types_start: 1863, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 1863, raw_values_count: 1, token_types_start: 1864, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 1864, raw_values_count: 1, token_types_start: 1865, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 1865, raw_values_count: 1, token_types_start: 1866, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 1, token_types_start: 1867, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 1, token_types_start: 1868, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 1868, raw_values_count: 1, token_types_start: 1869, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 1869, raw_values_count: 2, token_types_start: 1871, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 1871, raw_values_count: 1, token_types_start: 1872, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 1872, raw_values_count: 1, token_types_start: 1873, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 1873, raw_values_count: 1, token_types_start: 1874, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 1, token_types_start: 1875, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 1875, raw_values_count: 1, token_types_start: 1876, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 1876, raw_values_count: 1, token_types_start: 1877, token_types_count: 0 }, // [1026]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 1, token_types_start: 1879, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 1879, raw_values_count: 1, token_types_start: 1880, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 1880, raw_values_count: 1, token_types_start: 1881, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 1, token_types_start: 1882, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 1882, raw_values_count: 1, token_types_start: 1883, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 1883, raw_values_count: 1, token_types_start: 1884, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 1884, raw_values_count: 1, token_types_start: 1885, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 1885, raw_values_count: 1, token_types_start: 1886, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 1886, raw_values_count: 3, token_types_start: 1889, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 1889, raw_values_count: 1, token_types_start: 1890, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 2, token_types_start: 1892, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 1892, raw_values_count: 1, token_types_start: 1893, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 1893, raw_values_count: 14, token_types_start: 1907, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 1907, raw_values_count: 2, token_types_start: 1909, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 1909, raw_values_count: 1, token_types_start: 1910, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 1910, raw_values_count: 1, token_types_start: 1911, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 1911, raw_values_count: 1, token_types_start: 1912, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 1912, raw_values_count: 1, token_types_start: 1913, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 1913, raw_values_count: 1, token_types_start: 1914, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 1914, raw_values_count: 1, token_types_start: 1915, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 2, token_types_start: 1917, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 2, token_types_start: 1919, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 1919, raw_values_count: 1, token_types_start: 1920, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 1920, raw_values_count: 2, token_types_start: 1922, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 1922, raw_values_count: 1, token_types_start: 1923, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 1923, raw_values_count: 1, token_types_start: 1924, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 1924, raw_values_count: 1, token_types_start: 1925, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 1925, raw_values_count: 1, token_types_start: 1926, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 1926, raw_values_count: 1, token_types_start: 1927, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 1927, raw_values_count: 1, token_types_start: 1928, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 1928, raw_values_count: 2, token_types_start: 1930, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 1930, raw_values_count: 4, token_types_start: 1934, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 1934, raw_values_count: 3, token_types_start: 1937, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 1937, raw_values_count: 1, token_types_start: 1938, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 1938, raw_values_count: 1, token_types_start: 1939, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 1939, raw_values_count: 13, token_types_start: 1952, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 1952, raw_values_count: 1, token_types_start: 1953, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 1953, raw_values_count: 1, token_types_start: 1954, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 1954, raw_values_count: 1, token_types_start: 1955, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 1955, raw_values_count: 1, token_types_start: 1956, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 1956, raw_values_count: 1, token_types_start: 1957, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 1957, raw_values_count: 15, token_types_start: 1972, token_types_count: 7 }, // [1069]
    SimpleHintData { raw_values_start: 1979, raw_values_count: 1, token_types_start: 1980, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 1980, raw_values_count: 1, token_types_start: 1981, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 1981, raw_values_count: 1, token_types_start: 1982, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 1982, raw_values_count: 1, token_types_start: 1983, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 1983, raw_values_count: 1, token_types_start: 1984, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 1984, raw_values_count: 1, token_types_start: 1985, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 1985, raw_values_count: 1, token_types_start: 1986, token_types_count: 0 }, // [1076]
    SimpleHintData { raw_values_start: 1986, raw_values_count: 1, token_types_start: 1987, token_types_count: 0 }, // [1077]
    SimpleHintData { raw_values_start: 1987, raw_values_count: 1, token_types_start: 1988, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 1988, raw_values_count: 1, token_types_start: 1989, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 1989, raw_values_count: 1, token_types_start: 1990, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 1990, raw_values_count: 1, token_types_start: 1991, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 1991, raw_values_count: 1, token_types_start: 1992, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 1992, raw_values_count: 1, token_types_start: 1993, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 1993, raw_values_count: 1, token_types_start: 1994, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 1994, raw_values_count: 1, token_types_start: 1995, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 1995, raw_values_count: 1, token_types_start: 1996, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 1996, raw_values_count: 1, token_types_start: 1997, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 1997, raw_values_count: 1, token_types_start: 1998, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 1998, raw_values_count: 1, token_types_start: 1999, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 1999, raw_values_count: 1, token_types_start: 2000, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 2000, raw_values_count: 1, token_types_start: 2001, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 2001, raw_values_count: 1, token_types_start: 2002, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 2002, raw_values_count: 1, token_types_start: 2003, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 2003, raw_values_count: 1, token_types_start: 2004, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 2004, raw_values_count: 1, token_types_start: 2005, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 2005, raw_values_count: 1, token_types_start: 2006, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 2006, raw_values_count: 1, token_types_start: 2007, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 2007, raw_values_count: 1, token_types_start: 2008, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 2008, raw_values_count: 1, token_types_start: 2009, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 2009, raw_values_count: 1, token_types_start: 2010, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 2010, raw_values_count: 1, token_types_start: 2011, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 2011, raw_values_count: 1, token_types_start: 2012, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 2012, raw_values_count: 1, token_types_start: 2013, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 2, token_types_start: 2015, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 2015, raw_values_count: 1, token_types_start: 2016, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 2016, raw_values_count: 1, token_types_start: 2017, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 2017, raw_values_count: 1, token_types_start: 2018, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 2018, raw_values_count: 1, token_types_start: 2019, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 2019, raw_values_count: 1, token_types_start: 2020, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 2020, raw_values_count: 1, token_types_start: 2021, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 2021, raw_values_count: 1, token_types_start: 2022, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 2022, raw_values_count: 1, token_types_start: 2023, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 2023, raw_values_count: 1, token_types_start: 2024, token_types_count: 0 }, // [1113]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 1, token_types_start: 2026, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 2026, raw_values_count: 1, token_types_start: 2027, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 1, token_types_start: 2028, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 2028, raw_values_count: 1, token_types_start: 2029, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 2029, raw_values_count: 1, token_types_start: 2030, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 2030, raw_values_count: 1, token_types_start: 2031, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 2031, raw_values_count: 1, token_types_start: 2032, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 2032, raw_values_count: 1, token_types_start: 2033, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 2033, raw_values_count: 1, token_types_start: 2034, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 2034, raw_values_count: 2, token_types_start: 2036, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 2036, raw_values_count: 4, token_types_start: 2040, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 2040, raw_values_count: 3, token_types_start: 2043, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 2043, raw_values_count: 2, token_types_start: 2045, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 2045, raw_values_count: 10, token_types_start: 2055, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 2055, raw_values_count: 1, token_types_start: 2056, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 2056, raw_values_count: 1, token_types_start: 2057, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 2057, raw_values_count: 2, token_types_start: 2059, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 2059, raw_values_count: 4, token_types_start: 2063, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 2063, raw_values_count: 1, token_types_start: 2064, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 2064, raw_values_count: 1, token_types_start: 2065, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 2065, raw_values_count: 1, token_types_start: 2066, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 2066, raw_values_count: 1, token_types_start: 2067, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 2067, raw_values_count: 2, token_types_start: 2069, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 2069, raw_values_count: 2, token_types_start: 2071, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 2071, raw_values_count: 1, token_types_start: 2072, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 2072, raw_values_count: 1, token_types_start: 2073, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 2073, raw_values_count: 1, token_types_start: 2074, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 2074, raw_values_count: 1, token_types_start: 2075, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 2075, raw_values_count: 1, token_types_start: 2076, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 2076, raw_values_count: 1, token_types_start: 2077, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 2077, raw_values_count: 1, token_types_start: 2078, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 2078, raw_values_count: 1, token_types_start: 2079, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 2079, raw_values_count: 1, token_types_start: 2080, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 2080, raw_values_count: 1, token_types_start: 2081, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 2081, raw_values_count: 1, token_types_start: 2082, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 2082, raw_values_count: 1, token_types_start: 2083, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 2083, raw_values_count: 1, token_types_start: 2084, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 2084, raw_values_count: 1, token_types_start: 2085, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 2085, raw_values_count: 5, token_types_start: 2090, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 2, token_types_start: 2092, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 1, token_types_start: 2093, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 1, token_types_start: 2094, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 2094, raw_values_count: 1, token_types_start: 2095, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 2095, raw_values_count: 1, token_types_start: 2096, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 2096, raw_values_count: 1, token_types_start: 2097, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 2097, raw_values_count: 1, token_types_start: 2098, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 2098, raw_values_count: 1, token_types_start: 2099, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 2099, raw_values_count: 1, token_types_start: 2100, token_types_count: 0 }, // [1162]
    SimpleHintData { raw_values_start: 2100, raw_values_count: 1, token_types_start: 2101, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 1, token_types_start: 2102, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 1, token_types_start: 2103, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 2103, raw_values_count: 1, token_types_start: 2104, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 1, token_types_start: 2105, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 1, token_types_start: 2106, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 1, token_types_start: 2107, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 2107, raw_values_count: 1, token_types_start: 2108, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 2108, raw_values_count: 1, token_types_start: 2109, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 2109, raw_values_count: 1, token_types_start: 2110, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 2110, raw_values_count: 1, token_types_start: 2111, token_types_count: 0 }, // [1173]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 1, token_types_start: 2112, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 2112, raw_values_count: 1, token_types_start: 2113, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 1, token_types_start: 2114, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 2114, raw_values_count: 1, token_types_start: 2115, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 2115, raw_values_count: 1, token_types_start: 2116, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 1, token_types_start: 2118, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 2118, raw_values_count: 1, token_types_start: 2119, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 1, token_types_start: 2120, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 2120, raw_values_count: 1, token_types_start: 2121, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 2121, raw_values_count: 2, token_types_start: 2123, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 2123, raw_values_count: 1, token_types_start: 2124, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 1, token_types_start: 2126, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 2126, raw_values_count: 1, token_types_start: 2127, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 1, token_types_start: 2129, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 2129, raw_values_count: 1, token_types_start: 2130, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 2130, raw_values_count: 1, token_types_start: 2131, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 2131, raw_values_count: 1, token_types_start: 2132, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 1, token_types_start: 2135, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 2135, raw_values_count: 1, token_types_start: 2136, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 2136, raw_values_count: 2, token_types_start: 2138, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 2138, raw_values_count: 1, token_types_start: 2139, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 2139, raw_values_count: 1, token_types_start: 2140, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2140, raw_values_count: 1, token_types_start: 2141, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 2, token_types_start: 2144, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2144, raw_values_count: 1, token_types_start: 2145, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2145, raw_values_count: 1, token_types_start: 2146, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2146, raw_values_count: 1, token_types_start: 2147, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2147, raw_values_count: 1, token_types_start: 2148, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 1, token_types_start: 2149, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 2149, raw_values_count: 1, token_types_start: 2150, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2150, raw_values_count: 1, token_types_start: 2151, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2151, raw_values_count: 1, token_types_start: 2152, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2152, raw_values_count: 1, token_types_start: 2153, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2153, raw_values_count: 1, token_types_start: 2154, token_types_count: 0 }, // [1213]
    SimpleHintData { raw_values_start: 2154, raw_values_count: 1, token_types_start: 2155, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2155, raw_values_count: 1, token_types_start: 2156, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2156, raw_values_count: 1, token_types_start: 2157, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2157, raw_values_count: 1, token_types_start: 2158, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2158, raw_values_count: 11, token_types_start: 2169, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2169, raw_values_count: 5, token_types_start: 2174, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 2174, raw_values_count: 1, token_types_start: 2175, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2175, raw_values_count: 1, token_types_start: 2176, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2176, raw_values_count: 2, token_types_start: 2178, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2178, raw_values_count: 1, token_types_start: 2179, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2179, raw_values_count: 1, token_types_start: 2180, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2180, raw_values_count: 1, token_types_start: 2181, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 2181, raw_values_count: 1, token_types_start: 2182, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 2182, raw_values_count: 1, token_types_start: 2183, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 2183, raw_values_count: 1, token_types_start: 2184, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 2184, raw_values_count: 1, token_types_start: 2185, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 2185, raw_values_count: 1, token_types_start: 2186, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 2186, raw_values_count: 1, token_types_start: 2187, token_types_count: 0 }, // [1231]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 0, 5, 0, 6, 7, 7, 8, 9, 10, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 19, 20, 21, 22, 22, 23, 24, 24, 25, 26,
    10, 27, 27, 25, 26, 10, 28, 24, 24, 29, 26, 14, 27, 27, 29, 26,
    30, 10, 14, 0, 0, 0, 0, 0, 31, 0, 32, 33, 34, 35, 35, 36,
    37, 38, 39, 39, 40, 16, 41, 9, 10, 12, 42, 42, 43, 44, 45, 45,
    46, 47, 47, 48, 8, 8, 49, 50, 51, 52, 52, 39, 39, 53, 14, 43,
    43, 44, 54, 28, 55, 55, 56, 14, 15, 57, 57, 58, 42, 59, 39, 60,
    45, 61, 62, 8, 63, 64, 65, 66, 67, 68, 69, 70, 71, 72, 73, 51,
    74, 75, 24, 24, 58, 31, 38, 38, 76, 28, 77, 78, 79, 80, 81, 82,
    83, 84, 46, 14, 43, 43, 44, 85, 85, 78, 86, 86, 77, 87, 87, 88,
    29, 89, 90, 91, 92, 93, 94, 95, 96, 97, 25, 43, 43, 0, 85, 85,
    90, 86, 86, 89, 87, 87, 0, 98, 99, 100, 0, 101, 101, 0, 0, 41,
    41, 0, 0, 16, 16, 0, 0, 102, 102, 0, 0, 0, 0, 0, 0, 103,
    104, 105, 106, 5, 107, 108, 109, 110, 110, 87, 111, 112, 113, 110, 110, 114,
    111, 112, 113, 110, 115, 116, 117, 118, 0, 0, 119, 0, 0, 0, 31, 0,
    31, 120, 0, 121, 122, 123, 124, 125, 126, 34, 0, 127, 0, 0, 0, 24,
    128, 129, 130, 130, 30, 10, 14, 0, 131, 132, 133, 134, 130, 135, 136, 136,
    137, 37, 138, 139, 140, 141, 141, 37, 142, 143, 144, 145, 146, 130, 130, 83,
    0, 135, 135, 147, 147, 148, 149, 0, 0, 0, 0, 150, 0, 151, 0, 152,
    152, 110, 62, 148, 0, 153, 154, 154, 155, 116, 0, 31, 147, 156, 156, 157,
    119, 33, 0, 130, 130, 77, 0, 0, 156, 156, 33, 0, 158, 158, 156, 33,
    158, 156, 156, 148, 0, 33, 0, 110, 110, 159, 160, 148, 0, 31, 0, 0,
    0, 154, 161, 161, 130, 162, 148, 0, 0, 163, 164, 18, 165, 155, 116, 131,
    147, 18, 165, 164, 154, 166, 166, 131, 147, 167, 168, 169, 169, 160, 31, 0,
    0, 0, 164, 147, 147, 159, 148, 148, 149, 0, 160, 160, 149, 31, 0, 0,
    0, 110, 110, 149, 158, 158, 170, 147, 147, 149, 158, 158, 170, 171, 171, 169,
    169, 170, 172, 172, 173, 131, 131, 174, 175, 176, 158, 131, 177, 178, 178, 31,
    179, 179, 112, 180, 176, 158, 131, 181, 0, 176, 158, 131, 134, 0, 182, 110,
    147, 183, 0, 184, 184, 31, 0, 75, 75, 185, 186, 186, 137, 170, 187, 188,
    189, 137, 0, 0, 0, 190, 191, 192, 193, 193, 194, 155, 195, 131, 131, 196,
    197, 0, 0, 0, 147, 147, 196, 197, 0, 0, 0, 130, 130, 78, 0, 198,
    156, 156, 33, 0, 131, 131, 174, 175, 119, 119, 199, 31, 68, 200, 201, 202,
    203, 203, 203, 204, 77, 77, 0, 158, 205, 206, 207, 208, 209, 209, 160, 0,
    31, 0, 0, 89, 89, 210, 210, 34, 26, 0, 205, 206, 208, 211, 211, 212,
    213, 42, 214, 215, 216, 217, 218, 219, 220, 221, 201, 222, 223, 224, 225, 226,
    105, 0, 0, 37, 0, 227, 105, 228, 0, 229, 105, 0, 0, 229, 229, 224,
    165, 224, 225, 119, 119, 119, 191, 230, 231, 232, 233, 234, 235, 236, 237, 238,
    239, 240, 241, 241, 241, 242, 243, 244, 245, 246, 247, 248, 0, 151, 248, 249,
    0, 0, 0, 165, 165, 151, 32, 119, 0, 0, 250, 251, 252, 253, 254, 255,
    256, 216, 222, 257, 258, 259, 0, 260, 261, 262, 263, 264, 201, 224, 224, 224,
    222, 225, 225, 225, 223, 265, 266, 257, 211, 267, 268, 269, 270, 271, 272, 31,
    151, 151, 31, 0, 0, 31, 0, 31, 0, 0, 31, 273, 274, 275, 276, 277,
    278, 278, 279, 280, 37, 281, 281, 282, 137, 283, 284, 285, 31, 0, 0, 0,
    31, 119, 31, 286, 141, 141, 141, 287, 77, 0, 288, 289, 119, 286, 290, 291,
    292, 293, 294, 295, 295, 295, 0, 296, 296, 297, 298, 297, 298, 298, 295, 295,
    0, 296, 296, 297, 298, 297, 298, 298, 258, 32, 256, 295, 299, 300, 15, 301,
    302, 303, 304, 162, 305, 0, 306, 307, 308, 309, 310, 311, 312, 313, 314, 184,
    315, 316, 317, 318, 319, 320, 321, 321, 141, 321, 322, 323, 324, 324, 137, 0,
    0, 0, 0, 37, 0, 325, 326, 124, 327, 328, 324, 278, 278, 137, 31, 278,
    329, 330, 0, 331, 0, 332, 333, 0, 112, 0, 0, 334, 0, 335, 336, 337,
    338, 339, 227, 227, 227, 0, 0, 151, 0, 248, 340, 183, 183, 0, 341, 342,
    167, 168, 184, 184, 31, 0, 343, 343, 0, 344, 344, 0, 345, 241, 67, 67,
    0, 164, 332, 332, 0, 346, 0, 0, 165, 31, 0, 340, 340, 0, 0, 165,
    31, 0, 340, 340, 346, 346, 200, 119, 31, 0, 0, 248, 148, 0, 0, 334,
    0, 347, 0, 0, 0, 0, 160, 32, 348, 349, 164, 164, 112, 164, 164, 112,
    164, 0, 350, 351, 352, 353, 258, 150, 354, 355, 225, 224, 225, 224, 356, 357,
    358, 358, 359, 34, 358, 358, 167, 359, 34, 0, 360, 361, 0, 0, 362, 205,
    222, 222, 222, 363, 364, 365, 366, 0, 367, 366, 0, 59, 368, 369, 183, 183,
    183, 0, 370, 31, 0, 76, 76, 371, 372, 373, 373, 73, 147, 147, 374, 375,
    376, 377, 378, 379, 380, 381, 382, 383, 384, 384, 384, 33, 385, 386, 387, 387,
    206, 158, 388, 389, 390, 391, 392, 393, 394, 39, 39, 299, 31, 165, 119, 165,
    180, 395, 396, 82, 80, 81, 397, 398, 399, 400, 379, 401, 0, 31, 209, 209,
    0, 119, 119, 232, 39, 39, 30, 10, 14, 149, 0, 164, 134, 180, 180, 133,
    31, 39, 39, 267, 402, 80, 149, 403, 119, 112, 404, 39, 39, 267, 345, 405,
    149, 0, 76, 0, 31, 31, 0, 0, 39, 39, 39, 267, 406, 149, 0, 288,
    288, 31, 0, 0, 0, 150, 151, 151, 105, 112, 112, 119, 286, 39, 39, 16,
    149, 0, 39, 39, 14, 149, 0, 407, 136, 136, 137, 37, 408, 408, 180, 37,
    138, 139, 140, 141, 141, 37, 142, 143, 144, 145, 146, 39, 39, 83, 0, 407,
    407, 39, 39, 409, 39, 39, 71, 0, 410, 250, 116, 411, 411, 412, 413, 413,
    61, 60, 73, 73, 412, 0, 0, 267, 76, 267, 76, 76, 0, 414, 415, 415,
    416, 374, 119, 0, 417, 374, 119, 0, 34, 34, 0, 418, 167, 167, 419, 420,
    419, 421, 421, 421, 422, 421, 421, 423, 209, 209, 424, 425, 374, 426, 296, 296,
    31, 0, 45, 45, 81, 403, 31, 39, 39, 41, 0, 180, 180, 14, 427, 261,
    360, 428, 18, 429, 39, 39, 267, 267, 267, 402, 430, 431, 85, 78, 149, 0,
    31, 31, 0, 0, 0, 0, 165, 164, 183, 404, 404, 0, 288, 432, 158, 324,
    164, 174, 180, 119, 119, 286, 31, 286, 180, 39, 39, 433, 434, 435, 435, 0,
    343, 318, 318, 68, 436, 436, 0, 343, 437, 438, 439, 366, 440, 441, 441, 442,
    442, 441, 443, 31, 0, 444, 445, 446, 447, 448, 449, 450, 451, 452, 453, 454,
    455, 144, 22, 0, 456, 457, 0, 386, 244, 440, 458, 459, 460, 385, 440, 461,
    436, 260, 10, 133, 133, 31, 462, 0, 0, 0, 0, 165, 463, 229, 464, 464,
    224, 165, 165, 32, 165, 465, 466, 467, 467, 31, 31, 0, 0, 468, 0, 469,
    469, 466, 470, 471, 249, 472, 473, 474, 475, 476, 468, 477, 478, 479, 480, 481,
    482, 483, 484, 485, 486, 487, 488, 133, 489, 490, 491, 492, 343, 343, 343, 493,
    494, 419, 423, 495, 496, 497, 498, 499, 60, 60, 60, 34, 500, 501, 502, 503,
    504, 505, 385, 506, 507, 508, 509, 510, 192, 511, 511, 512, 0, 112, 386, 0,
    511, 511, 513, 0, 112, 386, 0, 325, 511, 0, 30, 30, 10, 14, 514, 0,
    80, 80, 514, 0, 0, 77, 514, 0, 158, 0, 0, 334, 334, 334, 0, 515,
    516, 517, 77, 77, 0, 34, 34, 0, 68, 0, 0, 500, 101, 118, 325, 518,
    519, 513, 512, 520, 520, 511, 192, 0, 521, 522, 523, 512, 524, 525, 526, 527,
    528, 529, 530, 531, 532, 533, 359, 534, 535, 535, 137, 0, 0, 0, 0, 37,
    0, 536, 325, 326, 124, 327, 328, 535, 186, 226, 226, 219, 537, 11, 334, 538,
    539, 540, 293, 147, 147, 299, 31, 165, 119, 165, 539, 147, 147, 30, 10, 14,
    149, 0, 539, 147, 147, 402, 80, 149, 0, 147, 147, 405, 149, 0, 539, 147,
    147, 147, 406, 149, 0, 147, 147, 16, 149, 0, 147, 147, 14, 149, 0, 539,
    147, 147, 83, 0, 147, 147, 402, 77, 149, 0, 0, 539, 147, 147, 71, 149,
    0, 147, 147, 18, 149, 0, 539, 147, 147, 41, 149, 0, 147, 147, 78, 149,
    0, 539, 435, 541, 542, 543, 544, 545, 546, 424, 547, 297, 297, 0, 297, 548,
    31, 465, 465, 31, 549, 550, 551, 552, 553, 554, 555, 298, 556, 557, 150, 354,
    355, 558, 559, 560, 561, 429, 562, 562, 31, 0, 0, 562, 563, 564, 565, 566,
    567, 568, 45, 50, 569, 570, 571, 572, 370, 573, 573, 573, 574, 514, 331, 391,
    575, 518, 576, 0, 0, 0, 0, 0, 577, 256, 256, 0, 578, 258, 358, 358,
    167, 0, 0, 0, 0, 579, 167, 253, 0, 211, 0, 0, 0, 0, 580, 581,
    581, 581, 582, 583, 584, 0, 0, 585, 585, 585, 585, 216, 222, 258, 0, 586,
    581, 581, 581, 582, 0, 570, 570, 31, 286, 0, 0, 0, 295, 234, 234, 0,
    32, 0, 0, 248, 0, 31, 0, 0, 0, 286, 0, 0, 151, 0, 0, 0,
    0, 0, 0, 0, 0, 465, 0, 31, 0, 0, 165, 165, 587, 112, 37, 269,
    168, 469, 0, 0, 32, 104, 0, 68, 151, 249, 465, 0, 0, 588, 514, 0,
    589, 86, 590, 373, 271, 271, 591, 591, 592, 155, 593, 0, 37, 0, 374, 594,
    374, 595, 596, 597, 180, 180, 598, 591, 599, 197, 456, 386, 600, 601, 602, 603,
    604, 604, 112, 605, 112, 87, 113, 606, 607, 219, 403, 403, 403, 334, 219, 386,
    386, 334, 245, 608, 503, 503, 518, 518, 503, 503, 503, 503, 181, 609, 610, 610,
    31, 500, 500, 0, 610, 611, 155, 154, 154, 155, 116, 116, 0, 612, 613, 614,
    615, 616, 617, 209, 618, 19, 19, 619, 19, 88, 575, 620, 621, 622, 327, 327,
    623, 624, 445, 445, 374, 625, 625, 37, 626, 249, 627, 628, 617, 253, 253, 211,
    327, 595, 629, 630, 631, 632, 633, 34, 34, 0, 0, 634, 500, 325, 101, 101,
    137, 118, 118, 137, 324, 324, 137, 535, 535, 137, 536, 536, 137, 326, 635, 636,
    180, 180, 126, 0, 0, 0, 31, 0, 637, 0, 0, 634, 0, 0, 0, 31,
    0, 0, 638, 0, 31, 0, 118, 118, 137, 101, 101, 137, 0, 0, 0, 0,
    639, 0, 118, 118, 137, 101, 101, 137, 0, 61, 61, 640, 280, 641, 77, 0,
    0, 0, 158, 31, 0, 61, 169, 169, 500, 0, 61, 0, 0, 0, 31, 0,
    637, 0, 0, 61, 634, 0, 0, 0, 31, 0, 0, 638, 0, 31, 0, 118,
    118, 137, 101, 101, 137, 61, 0, 0, 0, 0, 639, 0, 118, 118, 137, 101,
    101, 137, 61, 0, 34, 34, 0, 0, 34, 34, 61, 61, 34, 642, 643, 0,
    0, 220, 0, 0, 0, 0, 119, 165, 644, 644, 0, 34, 34, 0, 0, 0,
    468, 0, 34, 0, 0, 359, 0, 220, 0, 0, 118, 328, 328, 151, 0, 0,
    112, 0, 0, 26, 0, 112, 0, 0, 645, 0, 465, 31, 31, 0, 119, 119,
    112, 13, 13, 0, 80, 403, 0, 0, 0, 0, 334, 31, 646, 403, 647, 31,
    0, 0, 0, 648, 213, 213, 18, 165, 648, 213, 213, 18, 165, 31, 0, 0,
    31, 0, 0, 0, 649, 343, 343, 0, 151, 0, 164, 0, 467, 467, 31, 0,
    0, 0, 0, 0, 467, 0, 493, 31, 0, 92, 650, 27, 651, 652, 653, 653,
    653, 654, 209, 77, 0, 112, 386, 0, 346, 655, 656, 657, 658, 659, 99, 99,
    99, 0, 38, 38, 76, 0, 16, 16, 0, 64, 64, 76, 41, 0, 0, 33,
    0, 660, 180, 180, 99, 661, 180, 180, 115, 661, 388, 388, 445, 48, 662, 662,
    137, 0, 453, 663, 0, 662, 48, 225, 225, 150, 225, 664, 101, 101, 137, 0,
    0, 0, 442, 665, 0, 37, 0, 0, 0, 0, 0, 37, 0, 666, 180, 665,
    667, 668, 668, 118, 535, 324, 536, 137, 325, 326, 124, 101, 0, 0, 0, 0,
    37, 0, 31, 667, 665, 665, 665, 669, 31, 0, 0, 442, 0, 670, 326, 326,
    0, 31, 0, 671, 668, 668, 118, 324, 535, 536, 137, 325, 635, 124, 326, 672,
    673, 674, 675, 0, 0, 31, 513, 676, 677, 0, 678, 679, 680, 681, 682, 683,
    684, 461, 685, 686, 686, 687, 688, 487, 466, 684, 475, 31, 0, 689, 689, 277,
    72, 31, 37, 37, 32, 0, 690, 196, 0, 691, 692, 693, 149, 149, 570, 149,
    149, 149, 167, 570, 694, 645, 645, 694, 695, 193, 696, 422, 697, 698, 699, 700,
    57, 26, 578, 167, 26, 0, 31, 0, 0, 0, 286, 0, 701, 702, 136, 0,
    0, 190, 191, 192, 405, 703, 704, 705, 706, 707, 708, 709, 710, 421, 711, 712,
    713, 714, 604, 715, 61, 61, 61, 641, 716, 529, 112, 374, 717, 286, 493, 493,
    286, 61, 61, 641, 716, 529, 112, 404, 0, 288, 286, 493, 493, 286, 0, 493,
    493, 286, 0, 77, 0, 34, 34, 0, 68, 0, 0, 500, 101, 118, 325, 61,
    61, 640, 280, 641, 77, 0, 0, 0, 158, 718, 31, 137, 137, 719, 0, 169,
    169, 500, 0, 169, 169, 404, 31, 0, 720, 399, 721, 411, 722, 723, 724, 725,
    726, 727, 728, 12, 729, 730, 731, 249, 249, 732, 732, 732, 112, 249, 732, 581,
    733, 37, 112, 468, 33, 468, 280, 734, 0, 168, 735, 0, 269, 0, 358, 358,
    167, 359, 34, 358, 0, 736, 737, 738, 739, 740, 114, 457, 639, 741, 123, 742,
    0, 0, 743, 76, 404, 404, 0, 31, 0, 0, 0, 0, 744, 744, 742, 0,
    742, 742, 742, 0, 0, 0, 0, 0, 0, 0, 634, 0, 76, 76, 0, 0,
    31, 0, 0, 123, 366, 711, 745, 745, 642, 746, 747, 748, 749, 746, 750, 751,
    746, 212, 367, 711, 745, 745, 642, 746, 747, 748, 404, 404, 31, 0, 0, 458,
    603, 752, 619, 753, 754, 126, 0, 755, 756, 13, 35, 195, 757, 0, 0, 0,
    122, 0, 32, 122, 78, 748, 0, 0, 0, 0, 0, 119, 0, 0, 0, 119,
    0, 0, 287, 758, 759, 760, 746, 761, 762, 224, 224, 150, 224, 763, 577, 577,
    167, 764, 0, 559, 559, 0, 764, 765, 765, 766, 696, 767, 24, 213, 768, 769,
    770, 766, 357, 325, 325, 0, 37, 24, 0, 325, 771, 772, 0, 773, 773, 87,
    111, 112, 113, 773, 773, 114, 111, 112, 113, 773, 774, 151, 112, 37, 269, 581,
    168, 469, 105, 229, 775, 776, 283, 0, 0, 431, 777, 778, 778, 22, 716, 713,
    112, 641, 280, 77, 0, 158, 778, 0, 716, 188, 779, 780, 134, 134, 112, 134,
    781, 782, 783, 784, 785, 786, 787, 788, 789, 790, 791, 792, 793, 638, 638, 334,
    0, 31, 406, 406, 0, 32, 32, 31, 286, 794, 47, 464, 0, 43, 0, 795,
    796, 85, 797, 798, 799, 800, 801, 802, 803, 804, 60, 61, 61, 0, 0, 0,
    31, 493, 805, 805, 280, 805, 296, 296, 296, 296, 296, 296, 796, 211, 211, 0,
    806, 807, 73, 60, 296, 296, 167, 796, 137, 808, 211, 211, 0, 806, 807, 73,
    60, 296, 296, 167, 796, 137, 137, 809, 211, 211, 0, 806, 61, 805, 805, 0,
    0, 0, 404, 0, 0, 0, 31, 31, 286, 0, 0, 0, 0, 0, 76, 0,
    296, 73, 73, 0, 131, 131, 0, 0, 131, 810, 811, 812, 401, 813, 814, 815,
    816, 817, 218, 818, 819, 820, 821, 638, 822, 823, 824, 406, 825, 62, 826, 221,
    8, 827, 828, 688, 829, 830, 831, 831, 831, 832, 77, 0, 833, 833, 110, 147,
    834, 173, 31, 31, 0, 0, 209, 0, 26, 31, 31, 31, 31, 0, 0, 0,
    220, 835, 836, 837, 0, 0, 719, 0, 0, 119, 0, 0, 31, 0, 124, 124,
    0, 0, 838, 839, 840, 735, 735, 841, 744, 744, 123, 744, 842, 843, 218, 0,
    844, 417, 0, 593, 387, 845, 846, 142, 847, 848, 849, 850, 851, 145, 852, 853,
    273, 493, 68, 31, 68, 31, 180, 31, 273, 31, 0, 854, 73, 61, 60, 805,
    61, 273, 273, 199, 31, 68, 273, 855, 856, 857, 858, 0, 859, 208, 860, 0,
    356, 861, 861, 150, 224, 224, 225, 167, 0, 583, 167, 861, 862, 863, 864, 865,
    168, 168, 866, 867, 193, 868, 869, 37, 36, 112, 112, 227, 0, 775, 112, 112,
    775, 334, 334, 334, 334, 334, 0, 76, 119, 404, 32, 300, 105, 31, 256, 503,
    0, 870, 871, 872, 412, 873, 874, 875, 875, 0, 37, 0, 374, 594, 374, 595,
    875, 876, 877, 416, 76, 878, 597, 879, 880, 881, 882, 883, 63, 884, 885, 886,
    661, 887, 288, 288, 31, 288, 267, 267, 267, 267, 888, 267, 267, 169, 460, 118,
    118, 137, 0, 0, 0, 0, 37, 0, 190, 191, 192, 193, 193, 194, 155, 195,
    0, 325, 327, 889, 325, 326, 635, 124, 327, 328, 591, 118, 146, 118, 857, 189,
    890, 891, 892, 893, 748, 894, 895, 605, 0, 645, 896, 0, 0, 31, 0, 896,
    588, 588, 0, 31, 31, 469, 32, 469, 0, 588, 897, 898, 641, 899, 64, 900,
    901, 902, 0, 903, 904, 905, 906, 907, 908, 909, 910, 459, 911, 170, 170, 137,
    0, 31, 0, 0, 170, 170, 599, 0, 0, 0, 0, 0, 150, 151, 164, 685,
    0, 180, 0, 0, 0, 0, 0, 150, 151, 164, 685, 119, 119, 0, 170, 158,
    158, 170, 912, 912, 137, 31, 0, 0, 0, 0, 0, 150, 151, 164, 685, 912,
    173, 913, 914, 915, 916, 219, 219, 403, 403, 219, 112, 0, 917, 918, 919, 920,
    921, 922, 54, 222, 0, 125, 31, 0, 0, 0, 0, 209, 0, 31, 0, 0,
    0, 0, 26, 31, 0, 0, 0, 31, 0, 0, 0, 0, 0, 125, 923, 924,
    925, 217, 44, 926, 217, 0, 0, 610, 0, 927, 928, 0, 628, 929, 930, 931,
    932, 933, 934, 344, 344, 619, 344, 463, 268, 935, 787, 936, 727, 728, 254, 614,
    937, 538, 466, 471, 472, 473, 938, 939, 939, 306, 308, 940, 31, 941, 941, 491,
    490, 869, 31, 255, 249, 942, 943, 944, 584, 58, 945, 946, 81, 93, 947, 948,
    949, 0, 0, 0, 150, 0, 151, 31, 151, 151, 0, 0, 334, 0, 0, 0,
    0, 0, 0, 0, 0, 334, 0, 103, 171, 581, 581, 37, 635, 635, 0, 31,
    0, 635, 258, 258, 767, 24, 213, 768, 31, 286, 950, 516, 951, 647, 112, 474,
    647, 474, 952, 953, 629, 954, 150, 225, 955, 224, 861, 776, 956, 957, 602, 65,
    958, 959, 937, 960, 961, 962, 963, 172, 964, 965, 966, 67, 67, 0, 31, 795,
    76, 76, 76, 60, 967, 76, 76, 73, 967, 795, 795, 968, 642, 911, 969, 66,
    67, 415, 967, 540, 293, 131, 131, 168, 387, 387, 108, 131, 131, 343, 888, 888,
    888, 0, 112, 0, 77, 0, 80, 80, 0, 770, 970, 971, 972, 973, 974, 975,
    976, 977, 978, 979, 980, 981, 982, 983, 983, 983, 433, 0, 156, 832, 984, 985,
    169, 169, 169, 409, 986, 987, 988, 988, 988, 0, 0, 334, 989, 990, 114, 991,
    87, 7, 404, 404, 990, 112, 695, 992, 993, 993, 993, 77, 0, 112, 386, 0,
    33, 994, 471, 995, 540, 996, 997, 998, 999, 1000, 1001, 1002, 1003, 100, 100, 100,
    99, 99, 661, 209, 0, 38, 38, 76, 0, 16, 16, 0, 64, 64, 76, 41,
    0, 0, 34, 0, 539, 259, 747, 769, 16, 0, 1004, 1005, 443, 443, 82, 1006,
    1007, 1008, 94, 374, 374, 88, 1009, 502, 502, 197, 197, 1010, 560, 560, 137, 112,
    339, 339, 739, 1010, 464, 464, 126, 1010, 772, 772, 1010, 168, 168, 495, 119, 112,
    179, 179, 112, 180, 374, 1011, 1012, 1013, 1014, 1015, 1016, 595, 1017, 1018, 218, 218,
    1019, 637, 637, 31, 31, 37, 1020, 918, 595, 31, 277, 37, 893, 412, 37, 1021,
    1022, 1023, 1024, 14, 1025, 25, 1026, 1027, 1028, 1029, 1030, 1031, 1032, 1033, 1034, 1035,
    0, 0, 0, 0, 0, 1036, 1037, 1038, 359, 24, 68, 68, 0, 1039, 0, 1036,
    0, 0, 1040, 1040, 34, 0, 500, 118, 118, 137, 325, 588, 636, 591, 324, 324,
    137, 535, 535, 137, 536, 536, 137, 635, 1037, 1037, 1037, 0, 0, 0, 555, 555,
    68, 68, 68, 34, 500, 101, 326, 588, 124, 635, 118, 875, 591, 324, 535, 536,
    325, 124, 636, 180, 180, 286, 1041, 31, 180, 1041, 31, 180, 273, 31, 286, 1042,
    750, 106, 106, 334, 227, 0, 0, 105, 474, 105, 104, 106, 106, 334, 227, 0,
    0, 105, 474, 105, 503, 1043, 1044, 328, 83, 139, 800, 800, 37, 387, 387, 800,
    138, 821, 821, 37, 387, 387, 821, 600, 95, 179, 180, 180, 178, 31, 178, 1045,
    21, 1046, 1047, 663, 131, 131, 0, 0, 0, 0, 150, 0, 151, 248, 0, 0,
    0, 493, 343, 0, 151, 31, 151, 151, 0, 248, 0, 0, 334, 0, 503, 273,
    273, 636, 636, 636, 273, 118, 325, 124, 131, 636, 1048, 1048, 562, 817, 24, 1049,
    1049, 1050, 730, 1038, 359, 24, 562, 562, 31, 0, 131, 131, 14, 149, 0, 131,
    131, 218, 0, 0, 0, 150, 0, 0, 131, 131, 1051, 1052, 1053, 0, 31, 0,
    0, 150, 0, 343, 0, 31, 0, 1054, 1055, 669, 1056, 102, 1057, 0, 0, 0,
    295, 300, 300, 300, 165, 234, 1058, 1058, 462, 25, 766, 766, 112, 1059, 1060, 41,
    1061, 24, 92, 0, 0, 0, 334, 0, 0, 766, 766, 112, 1062, 1063, 39, 39,
    77, 0, 119, 119, 179, 160, 160, 26, 0, 26, 26, 0, 173, 173, 0, 158,
    77, 77, 514, 210, 210, 34, 26, 0, 766, 112, 158, 174, 174, 0, 31, 1058,
    1059, 89, 90, 1062, 1062, 1063, 89, 89, 210, 210, 34, 26, 0, 766, 766, 112,
    90, 90, 210, 210, 34, 26, 0, 766, 766, 112, 1064, 1065, 112, 474, 647, 581,
    217, 218, 1066, 969, 0, 0, 647, 474, 245, 0, 0, 0, 0, 334, 220, 474,
    0, 0, 209, 0, 26, 31, 0, 0, 0, 0, 1067, 229, 229, 105, 1068, 1068,
    137, 31, 76, 31, 1069, 1069, 151, 31, 151, 151, 717, 717, 119, 528, 1068, 219,
    227, 936, 768, 1070, 536, 536, 137, 0, 0, 0, 0, 37, 0, 190, 191, 192,
    193, 193, 194, 155, 195, 325, 326, 635, 124, 327, 328, 536, 279, 279, 137, 31,
    0, 0, 0, 190, 191, 192, 279, 808, 1071, 1072, 395, 1073, 1074, 1075, 1076, 1077,
    1078, 1079, 1080, 1081, 1082, 1083, 1084, 1085, 1086, 1087, 1088, 1089, 1090, 1091, 1092, 1093,
    1094, 79, 91, 220, 1037, 408, 1095, 1096, 1097, 426, 518, 286, 805, 61, 147, 147,
    39, 147, 72, 98, 39, 39, 147, 130, 147, 147, 39, 147, 39, 147, 39, 60,
    73, 39, 147, 39, 147, 520, 55, 573, 39, 130, 147, 39, 147, 34, 130, 130,
    130, 39, 831, 55, 110, 110, 203, 141, 321, 773, 773, 888, 988, 131, 1062, 1098,
    61, 61, 778, 324, 535, 1099, 511, 511, 653, 384, 993, 183, 39, 983, 169, 131,
    503, 400, 206, 1100, 1101, 1102, 1103, 1104, 1104, 374, 717, 717, 1105, 717, 717, 119,
    456, 717, 717, 137, 112, 180, 717, 1106, 1107, 84, 430, 96, 1108, 222, 222, 938,
    465, 31, 0, 0, 0, 0, 224, 0, 0, 0, 227, 165, 164, 465, 465, 224,
    1109, 1110, 1111, 1112, 1113, 1114, 1115, 1116, 1117, 1118, 0, 1119, 834, 1120, 1121, 1122,
    1061, 1123, 324, 324, 137, 1124, 31, 856, 1125, 183, 183, 0, 1126, 345, 345, 31,
    344, 344, 31, 19, 19, 31, 67, 409, 267, 267, 267, 402, 86, 430, 431, 77,
    149, 0, 386, 0, 1127, 31, 0, 0, 0, 0, 0, 1125, 164, 183, 766, 766,
    0, 386, 0, 404, 1128, 374, 717, 164, 288, 158, 278, 134, 164, 174, 324, 289,
    119, 286, 31, 286, 0, 0, 493, 248, 0, 0, 0, 386, 0, 0, 235, 0,
    994, 471, 995, 31, 286, 77, 174, 174, 31, 1129, 89, 637, 17, 0, 580, 580,
    256, 0, 0, 586, 586, 0, 809, 46, 97, 174, 69, 1130, 0, 402, 657, 657,
    1131, 69, 70, 70, 1132, 1133, 402, 1134, 1010, 1010, 137, 112, 1010, 436, 601, 1135,
    1136, 806, 598, 582, 470, 1137, 1137, 470, 471, 31, 1138, 1138, 180, 1139, 470, 1140,
    234, 234, 234, 470, 1140, 0, 471, 471, 119, 412, 0, 112, 248, 0, 471, 472,
    473, 1141, 1142, 1143, 935, 1144, 33, 1145, 1146, 1147, 1148, 1149, 1150, 1151, 1152, 1153,
    1153, 408, 251, 350, 1005, 298, 1154, 1152, 1155, 719, 719, 0, 211, 211, 387, 304,
    1156, 1157, 1158, 1159, 1039, 1039, 31, 0, 0, 374, 404, 112, 119, 119, 31, 0,
    0, 0, 165, 374, 1039, 1160, 1133, 1161, 1162, 1163, 71, 1164, 1165, 1166, 1167, 644,
    272, 758, 1150, 270, 270, 72, 72, 72, 77, 0, 158, 1168, 1169, 1170, 31, 0,
    0, 18, 229, 229, 105, 465, 465, 31, 1171, 1172, 1173, 626, 1098, 1098, 1098, 77,
    149, 0, 1174, 0, 0, 744, 0, 0, 1175, 1176, 1177, 1050, 1050, 1038, 359, 24,
    1050, 345, 345, 345, 1178, 0, 1179, 1180, 1181, 1182, 68, 68, 34, 500, 101, 326,
    124, 635, 324, 535, 536, 636, 180, 180, 118, 325, 273, 273, 636, 636, 636, 273,
    0, 1183, 1184, 1184, 701, 565, 193, 31, 0, 0, 31, 1183, 175, 175, 175, 174,
    149, 31, 1185, 1186, 73, 73, 73, 0, 386, 0, 0, 131, 131, 500, 1187, 1188,
    1189, 51, 55, 55, 0, 55, 55, 55, 0, 74, 41, 0, 1190, 1191, 1192, 1193,
    1194, 404, 404, 0, 404, 1195, 1196, 1197, 1099, 1099, 1099, 0, 112, 386, 0, 1198,
    997, 997, 37, 468, 541, 541, 1018, 1199, 1200, 1201, 1202, 493, 493, 0, 0, 31,
    0, 0, 168, 0, 168, 0, 493, 493, 0, 1203, 325, 118, 118, 325, 493, 1052,
    1204, 1205, 1206, 1207, 940, 1208, 1053, 1209, 942, 1210, 1211, 995, 995, 119, 412, 37,
    995, 78, 90, 371, 1212, 1213, 9, 1214, 1215, 1216, 296, 296, 0, 0, 0, 806,
    0, 0, 0, 0, 296, 1217, 500, 500, 0, 31, 0, 1218, 325, 1219, 1219, 324,
    535, 101, 118, 536, 137, 118, 118, 137, 535, 535, 137, 326, 635, 124, 588, 42,
    500, 1220, 0, 991, 343, 343, 112, 433, 0, 1221, 0, 0, 562, 0, 0, 0,
    0, 0, 334, 220, 220, 334, 220, 124, 0, 0, 1222, 170, 118, 170, 118, 327,
    180, 180, 964, 0, 0, 0, 68, 0, 854, 180, 180, 964, 0, 0, 0, 68,
    0, 273, 180, 180, 442, 441, 443, 180, 180, 387, 387, 22, 0, 180, 180, 180,
    387, 14, 261, 1223, 1139, 0, 1155, 23, 75, 1224, 1225, 1226, 1227, 1228, 1229, 1230,
    687, 1231, 1140,
];

pub fn get_sparksql_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &SPARKSQL_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &SPARKSQL_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &SPARKSQL_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &SPARKSQL_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &SPARKSQL_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(65), tables: &SPARKSQL_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(134), tables: &SPARKSQL_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(137), tables: &SPARKSQL_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(160), tables: &SPARKSQL_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(183), tables: &SPARKSQL_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(186), tables: &SPARKSQL_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(208), tables: &SPARKSQL_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(211), tables: &SPARKSQL_TABLES }),
        "AccountsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &SPARKSQL_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &SPARKSQL_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(214), tables: &SPARKSQL_TABLES }),
        "AddFileSegment" => Some(RootGrammar { grammar_id: GrammarId(215), tables: &SPARKSQL_TABLES }),
        "AddJarSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &SPARKSQL_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &SPARKSQL_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &SPARKSQL_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(229), tables: &SPARKSQL_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(230), tables: &SPARKSQL_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &SPARKSQL_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &SPARKSQL_TABLES }),
        "AliasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(251), tables: &SPARKSQL_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(252), tables: &SPARKSQL_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(255), tables: &SPARKSQL_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(256), tables: &SPARKSQL_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(257), tables: &SPARKSQL_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(258), tables: &SPARKSQL_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(268), tables: &SPARKSQL_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(269), tables: &SPARKSQL_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(285), tables: &SPARKSQL_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(291), tables: &SPARKSQL_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(296), tables: &SPARKSQL_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(323), tables: &SPARKSQL_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(491), tables: &SPARKSQL_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(509), tables: &SPARKSQL_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(510), tables: &SPARKSQL_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(511), tables: &SPARKSQL_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(512), tables: &SPARKSQL_TABLES }),
        "AnalyzeTableSegment" => Some(RootGrammar { grammar_id: GrammarId(513), tables: &SPARKSQL_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(541), tables: &SPARKSQL_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(542), tables: &SPARKSQL_TABLES }),
        "AntiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(543), tables: &SPARKSQL_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(544), tables: &SPARKSQL_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(545), tables: &SPARKSQL_TABLES }),
        "ArchiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(546), tables: &SPARKSQL_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(547), tables: &SPARKSQL_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(548), tables: &SPARKSQL_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(560), tables: &SPARKSQL_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(568), tables: &SPARKSQL_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(569), tables: &SPARKSQL_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(570), tables: &SPARKSQL_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(573), tables: &SPARKSQL_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(579), tables: &SPARKSQL_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(581), tables: &SPARKSQL_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(582), tables: &SPARKSQL_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(583), tables: &SPARKSQL_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(584), tables: &SPARKSQL_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(585), tables: &SPARKSQL_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(586), tables: &SPARKSQL_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(587), tables: &SPARKSQL_TABLES }),
        "AtSignLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(588), tables: &SPARKSQL_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(589), tables: &SPARKSQL_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(590), tables: &SPARKSQL_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(591), tables: &SPARKSQL_TABLES }),
        "AuditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(592), tables: &SPARKSQL_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(593), tables: &SPARKSQL_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(594), tables: &SPARKSQL_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(596), tables: &SPARKSQL_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(597), tables: &SPARKSQL_TABLES }),
        "Avg_row_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(598), tables: &SPARKSQL_TABLES }),
        "AvroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(599), tables: &SPARKSQL_TABLES }),
        "BackQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(600), tables: &SPARKSQL_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(601), tables: &SPARKSQL_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(602), tables: &SPARKSQL_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(603), tables: &SPARKSQL_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(604), tables: &SPARKSQL_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(616), tables: &SPARKSQL_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(617), tables: &SPARKSQL_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(618), tables: &SPARKSQL_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(619), tables: &SPARKSQL_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(620), tables: &SPARKSQL_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(621), tables: &SPARKSQL_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(622), tables: &SPARKSQL_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(623), tables: &SPARKSQL_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(624), tables: &SPARKSQL_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(630), tables: &SPARKSQL_TABLES }),
        "BinaryfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(631), tables: &SPARKSQL_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(632), tables: &SPARKSQL_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(633), tables: &SPARKSQL_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(634), tables: &SPARKSQL_TABLES }),
        "BitvarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(635), tables: &SPARKSQL_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(636), tables: &SPARKSQL_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(637), tables: &SPARKSQL_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(640), tables: &SPARKSQL_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(641), tables: &SPARKSQL_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(644), tables: &SPARKSQL_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(645), tables: &SPARKSQL_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(646), tables: &SPARKSQL_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(647), tables: &SPARKSQL_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(650), tables: &SPARKSQL_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(651), tables: &SPARKSQL_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(654), tables: &SPARKSQL_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(655), tables: &SPARKSQL_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(658), tables: &SPARKSQL_TABLES }),
        "BracketedPropertyListGrammar" => Some(RootGrammar { grammar_id: GrammarId(661), tables: &SPARKSQL_TABLES }),
        "BracketedPropertyNameListGrammar" => Some(RootGrammar { grammar_id: GrammarId(663), tables: &SPARKSQL_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(665), tables: &SPARKSQL_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(666), tables: &SPARKSQL_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(668), tables: &SPARKSQL_TABLES }),
        "BreakKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(669), tables: &SPARKSQL_TABLES }),
        "BrowseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(670), tables: &SPARKSQL_TABLES }),
        "BucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(671), tables: &SPARKSQL_TABLES }),
        "BucketSpecGrammar" => Some(RootGrammar { grammar_id: GrammarId(672), tables: &SPARKSQL_TABLES }),
        "BucketsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(678), tables: &SPARKSQL_TABLES }),
        "BulkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(679), tables: &SPARKSQL_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(680), tables: &SPARKSQL_TABLES }),
        "BytesQuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(681), tables: &SPARKSQL_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(684), tables: &SPARKSQL_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(686), tables: &SPARKSQL_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(692), tables: &SPARKSQL_TABLES }),
        "CacheTableSegment" => Some(RootGrammar { grammar_id: GrammarId(693), tables: &SPARKSQL_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(702), tables: &SPARKSQL_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(703), tables: &SPARKSQL_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(704), tables: &SPARKSQL_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(705), tables: &SPARKSQL_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(706), tables: &SPARKSQL_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(707), tables: &SPARKSQL_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(731), tables: &SPARKSQL_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(732), tables: &SPARKSQL_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(733), tables: &SPARKSQL_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(734), tables: &SPARKSQL_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(735), tables: &SPARKSQL_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(736), tables: &SPARKSQL_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(737), tables: &SPARKSQL_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(738), tables: &SPARKSQL_TABLES }),
        "ChangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(739), tables: &SPARKSQL_TABLES }),
        "ChangesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(740), tables: &SPARKSQL_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(741), tables: &SPARKSQL_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(742), tables: &SPARKSQL_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(743), tables: &SPARKSQL_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(744), tables: &SPARKSQL_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(745), tables: &SPARKSQL_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(746), tables: &SPARKSQL_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(747), tables: &SPARKSQL_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(748), tables: &SPARKSQL_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(749), tables: &SPARKSQL_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(750), tables: &SPARKSQL_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(751), tables: &SPARKSQL_TABLES }),
        "CheckedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(752), tables: &SPARKSQL_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(753), tables: &SPARKSQL_TABLES }),
        "ChecksumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(754), tables: &SPARKSQL_TABLES }),
        "ChoicesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(755), tables: &SPARKSQL_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(756), tables: &SPARKSQL_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(757), tables: &SPARKSQL_TABLES }),
        "ClearCacheSegment" => Some(RootGrammar { grammar_id: GrammarId(758), tables: &SPARKSQL_TABLES }),
        "ClearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(761), tables: &SPARKSQL_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(762), tables: &SPARKSQL_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(763), tables: &SPARKSQL_TABLES }),
        "ClusterByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(764), tables: &SPARKSQL_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(778), tables: &SPARKSQL_TABLES }),
        "ClusteredBySpecGrammar" => Some(RootGrammar { grammar_id: GrammarId(779), tables: &SPARKSQL_TABLES }),
        "ClusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(783), tables: &SPARKSQL_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(784), tables: &SPARKSQL_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(785), tables: &SPARKSQL_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(786), tables: &SPARKSQL_TABLES }),
        "CodegenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(787), tables: &SPARKSQL_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(788), tables: &SPARKSQL_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(789), tables: &SPARKSQL_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(790), tables: &SPARKSQL_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(791), tables: &SPARKSQL_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(797), tables: &SPARKSQL_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(798), tables: &SPARKSQL_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(799), tables: &SPARKSQL_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(800), tables: &SPARKSQL_TABLES }),
        "CollectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(801), tables: &SPARKSQL_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(802), tables: &SPARKSQL_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(803), tables: &SPARKSQL_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(804), tables: &SPARKSQL_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(805), tables: &SPARKSQL_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(810), tables: &SPARKSQL_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(838), tables: &SPARKSQL_TABLES }),
        "ColumnFieldDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(845), tables: &SPARKSQL_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(852), tables: &SPARKSQL_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(860), tables: &SPARKSQL_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(865), tables: &SPARKSQL_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(866), tables: &SPARKSQL_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(867), tables: &SPARKSQL_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(868), tables: &SPARKSQL_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(869), tables: &SPARKSQL_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(870), tables: &SPARKSQL_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(871), tables: &SPARKSQL_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(872), tables: &SPARKSQL_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(873), tables: &SPARKSQL_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(874), tables: &SPARKSQL_TABLES }),
        "CommentGrammar" => Some(RootGrammar { grammar_id: GrammarId(877), tables: &SPARKSQL_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(880), tables: &SPARKSQL_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(881), tables: &SPARKSQL_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(882), tables: &SPARKSQL_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(883), tables: &SPARKSQL_TABLES }),
        "CompactKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(884), tables: &SPARKSQL_TABLES }),
        "CompactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(885), tables: &SPARKSQL_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(886), tables: &SPARKSQL_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(905), tables: &SPARKSQL_TABLES }),
        "CompensationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(906), tables: &SPARKSQL_TABLES }),
        "CompletionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(907), tables: &SPARKSQL_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(908), tables: &SPARKSQL_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(909), tables: &SPARKSQL_TABLES }),
        "CompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(910), tables: &SPARKSQL_TABLES }),
        "ComputeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(911), tables: &SPARKSQL_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(912), tables: &SPARKSQL_TABLES }),
        "ConcatenateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(915), tables: &SPARKSQL_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(916), tables: &SPARKSQL_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(917), tables: &SPARKSQL_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(918), tables: &SPARKSQL_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(919), tables: &SPARKSQL_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(923), tables: &SPARKSQL_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(924), tables: &SPARKSQL_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(925), tables: &SPARKSQL_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(926), tables: &SPARKSQL_TABLES }),
        "ConstraintStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(927), tables: &SPARKSQL_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(943), tables: &SPARKSQL_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(944), tables: &SPARKSQL_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(945), tables: &SPARKSQL_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(946), tables: &SPARKSQL_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(947), tables: &SPARKSQL_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(948), tables: &SPARKSQL_TABLES }),
        "ContainstableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(949), tables: &SPARKSQL_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(950), tables: &SPARKSQL_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(951), tables: &SPARKSQL_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(952), tables: &SPARKSQL_TABLES }),
        "ConvertToDeltaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(953), tables: &SPARKSQL_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(962), tables: &SPARKSQL_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(963), tables: &SPARKSQL_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(964), tables: &SPARKSQL_TABLES }),
        "CostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(965), tables: &SPARKSQL_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(966), tables: &SPARKSQL_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(967), tables: &SPARKSQL_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(968), tables: &SPARKSQL_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(969), tables: &SPARKSQL_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(996), tables: &SPARKSQL_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1009), tables: &SPARKSQL_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1019), tables: &SPARKSQL_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1032), tables: &SPARKSQL_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1033), tables: &SPARKSQL_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1053), tables: &SPARKSQL_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1058), tables: &SPARKSQL_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1063), tables: &SPARKSQL_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1083), tables: &SPARKSQL_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1089), tables: &SPARKSQL_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1092), tables: &SPARKSQL_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1159), tables: &SPARKSQL_TABLES }),
        "CreateViewClausesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1163), tables: &SPARKSQL_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1172), tables: &SPARKSQL_TABLES }),
        "CreateWidgetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1209), tables: &SPARKSQL_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1224), tables: &SPARKSQL_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1225), tables: &SPARKSQL_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1226), tables: &SPARKSQL_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1227), tables: &SPARKSQL_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1228), tables: &SPARKSQL_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1229), tables: &SPARKSQL_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1230), tables: &SPARKSQL_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1231), tables: &SPARKSQL_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1237), tables: &SPARKSQL_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1238), tables: &SPARKSQL_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1239), tables: &SPARKSQL_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1240), tables: &SPARKSQL_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1241), tables: &SPARKSQL_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1242), tables: &SPARKSQL_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1243), tables: &SPARKSQL_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1244), tables: &SPARKSQL_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1245), tables: &SPARKSQL_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1246), tables: &SPARKSQL_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1247), tables: &SPARKSQL_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1248), tables: &SPARKSQL_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1249), tables: &SPARKSQL_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1250), tables: &SPARKSQL_TABLES }),
        "DataSourceFormatSegment" => Some(RootGrammar { grammar_id: GrammarId(1251), tables: &SPARKSQL_TABLES }),
        "DataSourcesV2FileTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(1255), tables: &SPARKSQL_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1266), tables: &SPARKSQL_TABLES }),
        "DatabasePropertiesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1267), tables: &SPARKSQL_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "DatabasesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1270), tables: &SPARKSQL_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(1271), tables: &SPARKSQL_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(1275), tables: &SPARKSQL_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1286), tables: &SPARKSQL_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(1287), tables: &SPARKSQL_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(1288), tables: &SPARKSQL_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(1289), tables: &SPARKSQL_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(1295), tables: &SPARKSQL_TABLES }),
        "Date_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1304), tables: &SPARKSQL_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1305), tables: &SPARKSQL_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(1306), tables: &SPARKSQL_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1307), tables: &SPARKSQL_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1308), tables: &SPARKSQL_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1309), tables: &SPARKSQL_TABLES }),
        "Day_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1310), tables: &SPARKSQL_TABLES }),
        "Day_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1311), tables: &SPARKSQL_TABLES }),
        "Day_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1312), tables: &SPARKSQL_TABLES }),
        "Day_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1313), tables: &SPARKSQL_TABLES }),
        "DayofmonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1314), tables: &SPARKSQL_TABLES }),
        "DayofweekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1315), tables: &SPARKSQL_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1316), tables: &SPARKSQL_TABLES }),
        "DaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1317), tables: &SPARKSQL_TABLES }),
        "DbccKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1318), tables: &SPARKSQL_TABLES }),
        "DbpropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1319), tables: &SPARKSQL_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1320), tables: &SPARKSQL_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1321), tables: &SPARKSQL_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1322), tables: &SPARKSQL_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1323), tables: &SPARKSQL_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(250), tables: &SPARKSQL_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1324), tables: &SPARKSQL_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(1325), tables: &SPARKSQL_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1328), tables: &SPARKSQL_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1329), tables: &SPARKSQL_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1330), tables: &SPARKSQL_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1331), tables: &SPARKSQL_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1332), tables: &SPARKSQL_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1333), tables: &SPARKSQL_TABLES }),
        "Delay_key_writeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1334), tables: &SPARKSQL_TABLES }),
        "DelayedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1335), tables: &SPARKSQL_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1336), tables: &SPARKSQL_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1337), tables: &SPARKSQL_TABLES }),
        "DeletesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1341), tables: &SPARKSQL_TABLES }),
        "DelimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1342), tables: &SPARKSQL_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1343), tables: &SPARKSQL_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1344), tables: &SPARKSQL_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1345), tables: &SPARKSQL_TABLES }),
        "DeltaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1346), tables: &SPARKSQL_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1347), tables: &SPARKSQL_TABLES }),
        "DenyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1348), tables: &SPARKSQL_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1349), tables: &SPARKSQL_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1350), tables: &SPARKSQL_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1351), tables: &SPARKSQL_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1352), tables: &SPARKSQL_TABLES }),
        "DescribeDetailStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1353), tables: &SPARKSQL_TABLES }),
        "DescribeHistoryStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1360), tables: &SPARKSQL_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1368), tables: &SPARKSQL_TABLES }),
        "DescribeObjectGrammar" => Some(RootGrammar { grammar_id: GrammarId(1369), tables: &SPARKSQL_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1411), tables: &SPARKSQL_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1416), tables: &SPARKSQL_TABLES }),
        "DestroyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1417), tables: &SPARKSQL_TABLES }),
        "DestructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1418), tables: &SPARKSQL_TABLES }),
        "DetailKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1419), tables: &SPARKSQL_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1420), tables: &SPARKSQL_TABLES }),
        "DfsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1421), tables: &SPARKSQL_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1422), tables: &SPARKSQL_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1423), tables: &SPARKSQL_TABLES }),
        "DirectoriesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1424), tables: &SPARKSQL_TABLES }),
        "DirectoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1425), tables: &SPARKSQL_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1426), tables: &SPARKSQL_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1427), tables: &SPARKSQL_TABLES }),
        "DiskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1428), tables: &SPARKSQL_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1429), tables: &SPARKSQL_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1430), tables: &SPARKSQL_TABLES }),
        "DistinctrowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1431), tables: &SPARKSQL_TABLES }),
        "DistributeByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1432), tables: &SPARKSQL_TABLES }),
        "DistributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1447), tables: &SPARKSQL_TABLES }),
        "DistributedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1448), tables: &SPARKSQL_TABLES }),
        "DivBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(1449), tables: &SPARKSQL_TABLES }),
        "DivKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1450), tables: &SPARKSQL_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(1451), tables: &SPARKSQL_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1452), tables: &SPARKSQL_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1453), tables: &SPARKSQL_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(1454), tables: &SPARKSQL_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1455), tables: &SPARKSQL_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1456), tables: &SPARKSQL_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1459), tables: &SPARKSQL_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1467), tables: &SPARKSQL_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1475), tables: &SPARKSQL_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1481), tables: &SPARKSQL_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1487), tables: &SPARKSQL_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1488), tables: &SPARKSQL_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1493), tables: &SPARKSQL_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1498), tables: &SPARKSQL_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1504), tables: &SPARKSQL_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1508), tables: &SPARKSQL_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1516), tables: &SPARKSQL_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1521), tables: &SPARKSQL_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1527), tables: &SPARKSQL_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1532), tables: &SPARKSQL_TABLES }),
        "DropdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1538), tables: &SPARKSQL_TABLES }),
        "DryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1539), tables: &SPARKSQL_TABLES }),
        "DummyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1540), tables: &SPARKSQL_TABLES }),
        "DumpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1541), tables: &SPARKSQL_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1542), tables: &SPARKSQL_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1543), tables: &SPARKSQL_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1544), tables: &SPARKSQL_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1545), tables: &SPARKSQL_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1546), tables: &SPARKSQL_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1547), tables: &SPARKSQL_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1550), tables: &SPARKSQL_TABLES }),
        "ElseifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1551), tables: &SPARKSQL_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(1552), tables: &SPARKSQL_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1553), tables: &SPARKSQL_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1556), tables: &SPARKSQL_TABLES }),
        "EnclosedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1557), tables: &SPARKSQL_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1558), tables: &SPARKSQL_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1559), tables: &SPARKSQL_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1560), tables: &SPARKSQL_TABLES }),
        "EndAngleBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(578), tables: &SPARKSQL_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(241), tables: &SPARKSQL_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(1561), tables: &SPARKSQL_TABLES }),
        "EndHintSegment" => Some(RootGrammar { grammar_id: GrammarId(1562), tables: &SPARKSQL_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1563), tables: &SPARKSQL_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(567), tables: &SPARKSQL_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1564), tables: &SPARKSQL_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1565), tables: &SPARKSQL_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(1566), tables: &SPARKSQL_TABLES }),
        "EqualsSegment_a" => Some(RootGrammar { grammar_id: GrammarId(1567), tables: &SPARKSQL_TABLES }),
        "EqualsSegment_b" => Some(RootGrammar { grammar_id: GrammarId(1568), tables: &SPARKSQL_TABLES }),
        "ErrlvlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1569), tables: &SPARKSQL_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1570), tables: &SPARKSQL_TABLES }),
        "EscapedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1571), tables: &SPARKSQL_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1572), tables: &SPARKSQL_TABLES }),
        "EvolutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1573), tables: &SPARKSQL_TABLES }),
        "ExceptClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1574), tables: &SPARKSQL_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1579), tables: &SPARKSQL_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1580), tables: &SPARKSQL_TABLES }),
        "ExchangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1581), tables: &SPARKSQL_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1582), tables: &SPARKSQL_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1583), tables: &SPARKSQL_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1584), tables: &SPARKSQL_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1585), tables: &SPARKSQL_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1586), tables: &SPARKSQL_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1587), tables: &SPARKSQL_TABLES }),
        "ExistingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1588), tables: &SPARKSQL_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1589), tables: &SPARKSQL_TABLES }),
        "ExitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1590), tables: &SPARKSQL_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1591), tables: &SPARKSQL_TABLES }),
        "ExpectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1592), tables: &SPARKSQL_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1593), tables: &SPARKSQL_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1594), tables: &SPARKSQL_TABLES }),
        "ExportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1602), tables: &SPARKSQL_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1603), tables: &SPARKSQL_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1604), tables: &SPARKSQL_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1630), tables: &SPARKSQL_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1637), tables: &SPARKSQL_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1647), tables: &SPARKSQL_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1652), tables: &SPARKSQL_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(1665), tables: &SPARKSQL_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(1702), tables: &SPARKSQL_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1710), tables: &SPARKSQL_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1711), tables: &SPARKSQL_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1712), tables: &SPARKSQL_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1713), tables: &SPARKSQL_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1714), tables: &SPARKSQL_TABLES }),
        "FailKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1715), tables: &SPARKSQL_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1716), tables: &SPARKSQL_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(1717), tables: &SPARKSQL_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1718), tables: &SPARKSQL_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1735), tables: &SPARKSQL_TABLES }),
        "FieldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1736), tables: &SPARKSQL_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1737), tables: &SPARKSQL_TABLES }),
        "FileFormatGrammar" => Some(RootGrammar { grammar_id: GrammarId(1738), tables: &SPARKSQL_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1749), tables: &SPARKSQL_TABLES }),
        "FileLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(1750), tables: &SPARKSQL_TABLES }),
        "FileReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(1759), tables: &SPARKSQL_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(1763), tables: &SPARKSQL_TABLES }),
        "FileformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1772), tables: &SPARKSQL_TABLES }),
        "FillfactorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1773), tables: &SPARKSQL_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(1774), tables: &SPARKSQL_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1780), tables: &SPARKSQL_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1781), tables: &SPARKSQL_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1782), tables: &SPARKSQL_TABLES }),
        "FirstOrAfterGrammar" => Some(RootGrammar { grammar_id: GrammarId(1783), tables: &SPARKSQL_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1789), tables: &SPARKSQL_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1790), tables: &SPARKSQL_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1791), tables: &SPARKSQL_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1792), tables: &SPARKSQL_TABLES }),
        "FlushKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1793), tables: &SPARKSQL_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1794), tables: &SPARKSQL_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1795), tables: &SPARKSQL_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1796), tables: &SPARKSQL_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(1797), tables: &SPARKSQL_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1800), tables: &SPARKSQL_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1801), tables: &SPARKSQL_TABLES }),
        "FormattedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1802), tables: &SPARKSQL_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1803), tables: &SPARKSQL_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1804), tables: &SPARKSQL_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1805), tables: &SPARKSQL_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1806), tables: &SPARKSQL_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(1824), tables: &SPARKSQL_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1827), tables: &SPARKSQL_TABLES }),
        "FreetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1828), tables: &SPARKSQL_TABLES }),
        "FreetexttableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1829), tables: &SPARKSQL_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1830), tables: &SPARKSQL_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1831), tables: &SPARKSQL_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(1835), tables: &SPARKSQL_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1859), tables: &SPARKSQL_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1870), tables: &SPARKSQL_TABLES }),
        "FromInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1897), tables: &SPARKSQL_TABLES }),
        "FromInsertFromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(1915), tables: &SPARKSQL_TABLES }),
        "FromInsertFromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(1927), tables: &SPARKSQL_TABLES }),
        "FromInsertSourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(1956), tables: &SPARKSQL_TABLES }),
        "FromInsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1960), tables: &SPARKSQL_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1964), tables: &SPARKSQL_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1965), tables: &SPARKSQL_TABLES }),
        "FulltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1966), tables: &SPARKSQL_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(1967), tables: &SPARKSQL_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1970), tables: &SPARKSQL_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2011), tables: &SPARKSQL_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2014), tables: &SPARKSQL_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2020), tables: &SPARKSQL_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2021), tables: &SPARKSQL_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2022), tables: &SPARKSQL_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2032), tables: &SPARKSQL_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2045), tables: &SPARKSQL_TABLES }),
        "FunctionParameterListGrammarWithComments" => Some(RootGrammar { grammar_id: GrammarId(2048), tables: &SPARKSQL_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2059), tables: &SPARKSQL_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2073), tables: &SPARKSQL_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2074), tables: &SPARKSQL_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2075), tables: &SPARKSQL_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2076), tables: &SPARKSQL_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2077), tables: &SPARKSQL_TABLES }),
        "GenerateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2078), tables: &SPARKSQL_TABLES }),
        "GenerateManifestFileStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2079), tables: &SPARKSQL_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &SPARKSQL_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &SPARKSQL_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2090), tables: &SPARKSQL_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2091), tables: &SPARKSQL_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2092), tables: &SPARKSQL_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2093), tables: &SPARKSQL_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2094), tables: &SPARKSQL_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2095), tables: &SPARKSQL_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2133), tables: &SPARKSQL_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2134), tables: &SPARKSQL_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2135), tables: &SPARKSQL_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2138), tables: &SPARKSQL_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2139), tables: &SPARKSQL_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2140), tables: &SPARKSQL_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2160), tables: &SPARKSQL_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &SPARKSQL_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(2172), tables: &SPARKSQL_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2180), tables: &SPARKSQL_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2181), tables: &SPARKSQL_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2189), tables: &SPARKSQL_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2190), tables: &SPARKSQL_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2195), tables: &SPARKSQL_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2206), tables: &SPARKSQL_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2207), tables: &SPARKSQL_TABLES }),
        "HeapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2208), tables: &SPARKSQL_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2209), tables: &SPARKSQL_TABLES }),
        "High_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2210), tables: &SPARKSQL_TABLES }),
        "HintFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2211), tables: &SPARKSQL_TABLES }),
        "HistoryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2214), tables: &SPARKSQL_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2215), tables: &SPARKSQL_TABLES }),
        "HoldlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2216), tables: &SPARKSQL_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2217), tables: &SPARKSQL_TABLES }),
        "HostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2218), tables: &SPARKSQL_TABLES }),
        "HostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2219), tables: &SPARKSQL_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &SPARKSQL_TABLES }),
        "Hour_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2221), tables: &SPARKSQL_TABLES }),
        "Hour_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &SPARKSQL_TABLES }),
        "Hour_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2223), tables: &SPARKSQL_TABLES }),
        "HoursKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2224), tables: &SPARKSQL_TABLES }),
        "IcebergKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &SPARKSQL_TABLES }),
        "IcebergTransformationSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &SPARKSQL_TABLES }),
        "IdentifiedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &SPARKSQL_TABLES }),
        "IdentifierKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2247), tables: &SPARKSQL_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2248), tables: &SPARKSQL_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2249), tables: &SPARKSQL_TABLES }),
        "Identity_insertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2250), tables: &SPARKSQL_TABLES }),
        "IdentitycolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2251), tables: &SPARKSQL_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &SPARKSQL_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &SPARKSQL_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2256), tables: &SPARKSQL_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2260), tables: &SPARKSQL_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2261), tables: &SPARKSQL_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2266), tables: &SPARKSQL_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &SPARKSQL_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2268), tables: &SPARKSQL_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2269), tables: &SPARKSQL_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(710), tables: &SPARKSQL_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &SPARKSQL_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2271), tables: &SPARKSQL_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2272), tables: &SPARKSQL_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2273), tables: &SPARKSQL_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &SPARKSQL_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &SPARKSQL_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2285), tables: &SPARKSQL_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2286), tables: &SPARKSQL_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(233), tables: &SPARKSQL_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &SPARKSQL_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2292), tables: &SPARKSQL_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2293), tables: &SPARKSQL_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2294), tables: &SPARKSQL_TABLES }),
        "InfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &SPARKSQL_TABLES }),
        "InfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2296), tables: &SPARKSQL_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2297), tables: &SPARKSQL_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &SPARKSQL_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &SPARKSQL_TABLES }),
        "InitializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2300), tables: &SPARKSQL_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2301), tables: &SPARKSQL_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &SPARKSQL_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &SPARKSQL_TABLES }),
        "InpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2304), tables: &SPARKSQL_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2305), tables: &SPARKSQL_TABLES }),
        "InputformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &SPARKSQL_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &SPARKSQL_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2308), tables: &SPARKSQL_TABLES }),
        "InsertOverwriteDirectoryHiveFmtSegment" => Some(RootGrammar { grammar_id: GrammarId(2309), tables: &SPARKSQL_TABLES }),
        "InsertOverwriteDirectorySegment" => Some(RootGrammar { grammar_id: GrammarId(2321), tables: &SPARKSQL_TABLES }),
        "InsertSourceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2334), tables: &SPARKSQL_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2351), tables: &SPARKSQL_TABLES }),
        "Insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2376), tables: &SPARKSQL_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2377), tables: &SPARKSQL_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &SPARKSQL_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2379), tables: &SPARKSQL_TABLES }),
        "Int1KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2380), tables: &SPARKSQL_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2381), tables: &SPARKSQL_TABLES }),
        "Int3KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2382), tables: &SPARKSQL_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2383), tables: &SPARKSQL_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2384), tables: &SPARKSQL_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2385), tables: &SPARKSQL_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2386), tables: &SPARKSQL_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2387), tables: &SPARKSQL_TABLES }),
        "IntegrationsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2388), tables: &SPARKSQL_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2389), tables: &SPARKSQL_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2390), tables: &SPARKSQL_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2391), tables: &SPARKSQL_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &SPARKSQL_TABLES }),
        "IntervalLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &SPARKSQL_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &SPARKSQL_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &SPARKSQL_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2408), tables: &SPARKSQL_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(2414), tables: &SPARKSQL_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2419), tables: &SPARKSQL_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2420), tables: &SPARKSQL_TABLES }),
        "IsamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2421), tables: &SPARKSQL_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2422), tables: &SPARKSQL_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2423), tables: &SPARKSQL_TABLES }),
        "ItemsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &SPARKSQL_TABLES }),
        "IterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &SPARKSQL_TABLES }),
        "JarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2426), tables: &SPARKSQL_TABLES }),
        "JdbcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &SPARKSQL_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &SPARKSQL_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2448), tables: &SPARKSQL_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &SPARKSQL_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(2451), tables: &SPARKSQL_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2460), tables: &SPARKSQL_TABLES }),
        "JoinTypeKeywords" => Some(RootGrammar { grammar_id: GrammarId(2467), tables: &SPARKSQL_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2482), tables: &SPARKSQL_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2490), tables: &SPARKSQL_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2495), tables: &SPARKSQL_TABLES }),
        "JsonfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2496), tables: &SPARKSQL_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &SPARKSQL_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2498), tables: &SPARKSQL_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2499), tables: &SPARKSQL_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2500), tables: &SPARKSQL_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2501), tables: &SPARKSQL_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2502), tables: &SPARKSQL_TABLES }),
        "KillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2503), tables: &SPARKSQL_TABLES }),
        "LancompilerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2504), tables: &SPARKSQL_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &SPARKSQL_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2506), tables: &SPARKSQL_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2507), tables: &SPARKSQL_TABLES }),
        "Last_insert_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2508), tables: &SPARKSQL_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2509), tables: &SPARKSQL_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2512), tables: &SPARKSQL_TABLES }),
        "LateralViewClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2513), tables: &SPARKSQL_TABLES }),
        "LazyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2530), tables: &SPARKSQL_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2531), tables: &SPARKSQL_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2532), tables: &SPARKSQL_TABLES }),
        "LeaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2533), tables: &SPARKSQL_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2534), tables: &SPARKSQL_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2535), tables: &SPARKSQL_TABLES }),
        "LessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2536), tables: &SPARKSQL_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2537), tables: &SPARKSQL_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(2540), tables: &SPARKSQL_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2541), tables: &SPARKSQL_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2542), tables: &SPARKSQL_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2550), tables: &SPARKSQL_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2561), tables: &SPARKSQL_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2562), tables: &SPARKSQL_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2563), tables: &SPARKSQL_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2569), tables: &SPARKSQL_TABLES }),
        "LinenoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2570), tables: &SPARKSQL_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2571), tables: &SPARKSQL_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2572), tables: &SPARKSQL_TABLES }),
        "ListFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2573), tables: &SPARKSQL_TABLES }),
        "ListJarSegment" => Some(RootGrammar { grammar_id: GrammarId(2579), tables: &SPARKSQL_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2585), tables: &SPARKSQL_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2586), tables: &SPARKSQL_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2587), tables: &SPARKSQL_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2599), tables: &SPARKSQL_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2600), tables: &SPARKSQL_TABLES }),
        "LiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2601), tables: &SPARKSQL_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2602), tables: &SPARKSQL_TABLES }),
        "LoadDataSegment" => Some(RootGrammar { grammar_id: GrammarId(2603), tables: &SPARKSQL_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2614), tables: &SPARKSQL_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(2615), tables: &SPARKSQL_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2616), tables: &SPARKSQL_TABLES }),
        "LocallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2617), tables: &SPARKSQL_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2618), tables: &SPARKSQL_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2619), tables: &SPARKSQL_TABLES }),
        "LocationGrammar" => Some(RootGrammar { grammar_id: GrammarId(2620), tables: &SPARKSQL_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2623), tables: &SPARKSQL_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2624), tables: &SPARKSQL_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2625), tables: &SPARKSQL_TABLES }),
        "LocksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2626), tables: &SPARKSQL_TABLES }),
        "LogicalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2627), tables: &SPARKSQL_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2628), tables: &SPARKSQL_TABLES }),
        "LogsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2629), tables: &SPARKSQL_TABLES }),
        "LongKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2630), tables: &SPARKSQL_TABLES }),
        "LongblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2631), tables: &SPARKSQL_TABLES }),
        "LongtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2632), tables: &SPARKSQL_TABLES }),
        "LoopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2633), tables: &SPARKSQL_TABLES }),
        "Low_priorityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2634), tables: &SPARKSQL_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2635), tables: &SPARKSQL_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2636), tables: &SPARKSQL_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2637), tables: &SPARKSQL_TABLES }),
        "MacroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2649), tables: &SPARKSQL_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2650), tables: &SPARKSQL_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2651), tables: &SPARKSQL_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2652), tables: &SPARKSQL_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2653), tables: &SPARKSQL_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(2654), tables: &SPARKSQL_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2655), tables: &SPARKSQL_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2656), tables: &SPARKSQL_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2657), tables: &SPARKSQL_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2658), tables: &SPARKSQL_TABLES }),
        "Max_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2659), tables: &SPARKSQL_TABLES }),
        "MaxextentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2660), tables: &SPARKSQL_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2661), tables: &SPARKSQL_TABLES }),
        "MediumblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2662), tables: &SPARKSQL_TABLES }),
        "MediumintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2663), tables: &SPARKSQL_TABLES }),
        "MediumtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2664), tables: &SPARKSQL_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2665), tables: &SPARKSQL_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2666), tables: &SPARKSQL_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2667), tables: &SPARKSQL_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2674), tables: &SPARKSQL_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &SPARKSQL_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(2678), tables: &SPARKSQL_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2682), tables: &SPARKSQL_TABLES }),
        "MergeNotMatchedBySourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2692), tables: &SPARKSQL_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2705), tables: &SPARKSQL_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2717), tables: &SPARKSQL_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2737), tables: &SPARKSQL_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2745), tables: &SPARKSQL_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2746), tables: &SPARKSQL_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2747), tables: &SPARKSQL_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2748), tables: &SPARKSQL_TABLES }),
        "MiddleintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2749), tables: &SPARKSQL_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2750), tables: &SPARKSQL_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2751), tables: &SPARKSQL_TABLES }),
        "Min_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2752), tables: &SPARKSQL_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2753), tables: &SPARKSQL_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(2754), tables: &SPARKSQL_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2755), tables: &SPARKSQL_TABLES }),
        "Minute_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2756), tables: &SPARKSQL_TABLES }),
        "Minute_secondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2757), tables: &SPARKSQL_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2758), tables: &SPARKSQL_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2759), tables: &SPARKSQL_TABLES }),
        "MlslabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2760), tables: &SPARKSQL_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2761), tables: &SPARKSQL_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2762), tables: &SPARKSQL_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2763), tables: &SPARKSQL_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2764), tables: &SPARKSQL_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2765), tables: &SPARKSQL_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2766), tables: &SPARKSQL_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(2767), tables: &SPARKSQL_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2768), tables: &SPARKSQL_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2769), tables: &SPARKSQL_TABLES }),
        "MonthnameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2770), tables: &SPARKSQL_TABLES }),
        "MonthsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2771), tables: &SPARKSQL_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2772), tables: &SPARKSQL_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2773), tables: &SPARKSQL_TABLES }),
        "MsckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2774), tables: &SPARKSQL_TABLES }),
        "MsckRepairTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2775), tables: &SPARKSQL_TABLES }),
        "MultiValueColumnUnpivotSegment" => Some(RootGrammar { grammar_id: GrammarId(2786), tables: &SPARKSQL_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &SPARKSQL_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &SPARKSQL_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &SPARKSQL_TABLES }),
        "MyisamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &SPARKSQL_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(2804), tables: &SPARKSQL_TABLES }),
        "NakedSemiStructuredElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &SPARKSQL_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &SPARKSQL_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2807), tables: &SPARKSQL_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(2814), tables: &SPARKSQL_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2818), tables: &SPARKSQL_TABLES }),
        "NamespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2819), tables: &SPARKSQL_TABLES }),
        "NamespacesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2820), tables: &SPARKSQL_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2821), tables: &SPARKSQL_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2822), tables: &SPARKSQL_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2823), tables: &SPARKSQL_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2824), tables: &SPARKSQL_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2827), tables: &SPARKSQL_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2828), tables: &SPARKSQL_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2829), tables: &SPARKSQL_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(2830), tables: &SPARKSQL_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(2831), tables: &SPARKSQL_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2832), tables: &SPARKSQL_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2833), tables: &SPARKSQL_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(2834), tables: &SPARKSQL_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2835), tables: &SPARKSQL_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2836), tables: &SPARKSQL_TABLES }),
        "No_write_to_binlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2837), tables: &SPARKSQL_TABLES }),
        "NoauditKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2838), tables: &SPARKSQL_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2839), tables: &SPARKSQL_TABLES }),
        "NocheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2840), tables: &SPARKSQL_TABLES }),
        "NocompressKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2841), tables: &SPARKSQL_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2842), tables: &SPARKSQL_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2843), tables: &SPARKSQL_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2844), tables: &SPARKSQL_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2845), tables: &SPARKSQL_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2846), tables: &SPARKSQL_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2847), tables: &SPARKSQL_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2848), tables: &SPARKSQL_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2858), tables: &SPARKSQL_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2859), tables: &SPARKSQL_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(2865), tables: &SPARKSQL_TABLES }),
        "NonclusteredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2871), tables: &SPARKSQL_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2872), tables: &SPARKSQL_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2873), tables: &SPARKSQL_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2874), tables: &SPARKSQL_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2875), tables: &SPARKSQL_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2876), tables: &SPARKSQL_TABLES }),
        "NoscanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2877), tables: &SPARKSQL_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2878), tables: &SPARKSQL_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2879), tables: &SPARKSQL_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(2880), tables: &SPARKSQL_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2887), tables: &SPARKSQL_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(2888), tables: &SPARKSQL_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2889), tables: &SPARKSQL_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2892), tables: &SPARKSQL_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2893), tables: &SPARKSQL_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2894), tables: &SPARKSQL_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2895), tables: &SPARKSQL_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2896), tables: &SPARKSQL_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2897), tables: &SPARKSQL_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2898), tables: &SPARKSQL_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2899), tables: &SPARKSQL_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2900), tables: &SPARKSQL_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2901), tables: &SPARKSQL_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2902), tables: &SPARKSQL_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2903), tables: &SPARKSQL_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2904), tables: &SPARKSQL_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(2905), tables: &SPARKSQL_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2909), tables: &SPARKSQL_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(2913), tables: &SPARKSQL_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2918), tables: &SPARKSQL_TABLES }),
        "ObjectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &SPARKSQL_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2930), tables: &SPARKSQL_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2931), tables: &SPARKSQL_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2932), tables: &SPARKSQL_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2933), tables: &SPARKSQL_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2934), tables: &SPARKSQL_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2935), tables: &SPARKSQL_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2944), tables: &SPARKSQL_TABLES }),
        "OffsetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2945), tables: &SPARKSQL_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2946), tables: &SPARKSQL_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2947), tables: &SPARKSQL_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2948), tables: &SPARKSQL_TABLES }),
        "OnlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2949), tables: &SPARKSQL_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2950), tables: &SPARKSQL_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2951), tables: &SPARKSQL_TABLES }),
        "OpendatasourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2952), tables: &SPARKSQL_TABLES }),
        "OpenqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2953), tables: &SPARKSQL_TABLES }),
        "OpenrowsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2954), tables: &SPARKSQL_TABLES }),
        "OpenxmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2955), tables: &SPARKSQL_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2956), tables: &SPARKSQL_TABLES }),
        "OperationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2957), tables: &SPARKSQL_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2958), tables: &SPARKSQL_TABLES }),
        "OptimizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2959), tables: &SPARKSQL_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2960), tables: &SPARKSQL_TABLES }),
        "OptionallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2961), tables: &SPARKSQL_TABLES }),
        "OptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2962), tables: &SPARKSQL_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2965), tables: &SPARKSQL_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2966), tables: &SPARKSQL_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2967), tables: &SPARKSQL_TABLES }),
        "OrRefreshGrammar" => Some(RootGrammar { grammar_id: GrammarId(2968), tables: &SPARKSQL_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(2971), tables: &SPARKSQL_TABLES }),
        "OrcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2974), tables: &SPARKSQL_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2975), tables: &SPARKSQL_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(2995), tables: &SPARKSQL_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3003), tables: &SPARKSQL_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(3004), tables: &SPARKSQL_TABLES }),
        "OrderedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3007), tables: &SPARKSQL_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3008), tables: &SPARKSQL_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3009), tables: &SPARKSQL_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3010), tables: &SPARKSQL_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3011), tables: &SPARKSQL_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3012), tables: &SPARKSQL_TABLES }),
        "OutfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3013), tables: &SPARKSQL_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3014), tables: &SPARKSQL_TABLES }),
        "OutputformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3015), tables: &SPARKSQL_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3016), tables: &SPARKSQL_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3023), tables: &SPARKSQL_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3024), tables: &SPARKSQL_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3033), tables: &SPARKSQL_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3034), tables: &SPARKSQL_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3035), tables: &SPARKSQL_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3036), tables: &SPARKSQL_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3037), tables: &SPARKSQL_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3038), tables: &SPARKSQL_TABLES }),
        "Pack_keysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3039), tables: &SPARKSQL_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3040), tables: &SPARKSQL_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3041), tables: &SPARKSQL_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3042), tables: &SPARKSQL_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(3043), tables: &SPARKSQL_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3044), tables: &SPARKSQL_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3045), tables: &SPARKSQL_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3046), tables: &SPARKSQL_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3047), tables: &SPARKSQL_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3048), tables: &SPARKSQL_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3049), tables: &SPARKSQL_TABLES }),
        "ParametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3050), tables: &SPARKSQL_TABLES }),
        "ParquetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3051), tables: &SPARKSQL_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3052), tables: &SPARKSQL_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3053), tables: &SPARKSQL_TABLES }),
        "PartitionFieldGrammar" => Some(RootGrammar { grammar_id: GrammarId(3060), tables: &SPARKSQL_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3086), tables: &SPARKSQL_TABLES }),
        "PartitionSpecGrammar" => Some(RootGrammar { grammar_id: GrammarId(3087), tables: &SPARKSQL_TABLES }),
        "PartitionedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3103), tables: &SPARKSQL_TABLES }),
        "PartitionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3104), tables: &SPARKSQL_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3105), tables: &SPARKSQL_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3106), tables: &SPARKSQL_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3107), tables: &SPARKSQL_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(3108), tables: &SPARKSQL_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3115), tables: &SPARKSQL_TABLES }),
        "PctfreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3116), tables: &SPARKSQL_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3117), tables: &SPARKSQL_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3118), tables: &SPARKSQL_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3119), tables: &SPARKSQL_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3120), tables: &SPARKSQL_TABLES }),
        "PercentlitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3121), tables: &SPARKSQL_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3122), tables: &SPARKSQL_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(3123), tables: &SPARKSQL_TABLES }),
        "PivotClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3124), tables: &SPARKSQL_TABLES }),
        "PivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3149), tables: &SPARKSQL_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3150), tables: &SPARKSQL_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3151), tables: &SPARKSQL_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3152), tables: &SPARKSQL_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(3153), tables: &SPARKSQL_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3154), tables: &SPARKSQL_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3155), tables: &SPARKSQL_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(3156), tables: &SPARKSQL_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3157), tables: &SPARKSQL_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3160), tables: &SPARKSQL_TABLES }),
        "PostfixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3161), tables: &SPARKSQL_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3162), tables: &SPARKSQL_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3163), tables: &SPARKSQL_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3164), tables: &SPARKSQL_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3165), tables: &SPARKSQL_TABLES }),
        "PrefixKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3166), tables: &SPARKSQL_TABLES }),
        "PreorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3167), tables: &SPARKSQL_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3168), tables: &SPARKSQL_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3169), tables: &SPARKSQL_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3170), tables: &SPARKSQL_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3171), tables: &SPARKSQL_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3174), tables: &SPARKSQL_TABLES }),
        "PrimitiveTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3175), tables: &SPARKSQL_TABLES }),
        "PrincipalsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3206), tables: &SPARKSQL_TABLES }),
        "PrintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3207), tables: &SPARKSQL_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3208), tables: &SPARKSQL_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3209), tables: &SPARKSQL_TABLES }),
        "ProcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3210), tables: &SPARKSQL_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3211), tables: &SPARKSQL_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3212), tables: &SPARKSQL_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3213), tables: &SPARKSQL_TABLES }),
        "ProcessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3214), tables: &SPARKSQL_TABLES }),
        "ProcesslistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3215), tables: &SPARKSQL_TABLES }),
        "PropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3216), tables: &SPARKSQL_TABLES }),
        "PropertiesNakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3217), tables: &SPARKSQL_TABLES }),
        "PropertyGrammar" => Some(RootGrammar { grammar_id: GrammarId(3218), tables: &SPARKSQL_TABLES }),
        "PropertyListGrammar" => Some(RootGrammar { grammar_id: GrammarId(3229), tables: &SPARKSQL_TABLES }),
        "PropertyNameListGrammar" => Some(RootGrammar { grammar_id: GrammarId(3231), tables: &SPARKSQL_TABLES }),
        "PropertyNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3233), tables: &SPARKSQL_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3239), tables: &SPARKSQL_TABLES }),
        "PurgeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3240), tables: &SPARKSQL_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3241), tables: &SPARKSQL_TABLES }),
        "QualifyClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3244), tables: &SPARKSQL_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &SPARKSQL_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3250), tables: &SPARKSQL_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3258), tables: &SPARKSQL_TABLES }),
        "QueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3259), tables: &SPARKSQL_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3260), tables: &SPARKSQL_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3261), tables: &SPARKSQL_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3262), tables: &SPARKSQL_TABLES }),
        "QuotedSemiStructuredElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3265), tables: &SPARKSQL_TABLES }),
        "Raid0KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3266), tables: &SPARKSQL_TABLES }),
        "RaiserrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3267), tables: &SPARKSQL_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3268), tables: &SPARKSQL_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3269), tables: &SPARKSQL_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(3270), tables: &SPARKSQL_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3271), tables: &SPARKSQL_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3272), tables: &SPARKSQL_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(3273), tables: &SPARKSQL_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(3274), tables: &SPARKSQL_TABLES }),
        "RawQuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3275), tables: &SPARKSQL_TABLES }),
        "RcfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3278), tables: &SPARKSQL_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3279), tables: &SPARKSQL_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3280), tables: &SPARKSQL_TABLES }),
        "ReadtextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3281), tables: &SPARKSQL_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3282), tables: &SPARKSQL_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3283), tables: &SPARKSQL_TABLES }),
        "ReconfigureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3284), tables: &SPARKSQL_TABLES }),
        "RecordreaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3285), tables: &SPARKSQL_TABLES }),
        "RecordwriterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3286), tables: &SPARKSQL_TABLES }),
        "RecoverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3287), tables: &SPARKSQL_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3288), tables: &SPARKSQL_TABLES }),
        "ReduceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3289), tables: &SPARKSQL_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3290), tables: &SPARKSQL_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3291), tables: &SPARKSQL_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(3305), tables: &SPARKSQL_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3311), tables: &SPARKSQL_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3312), tables: &SPARKSQL_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3313), tables: &SPARKSQL_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(3314), tables: &SPARKSQL_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3326), tables: &SPARKSQL_TABLES }),
        "RefreshStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3327), tables: &SPARKSQL_TABLES }),
        "RegexpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3337), tables: &SPARKSQL_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3338), tables: &SPARKSQL_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3339), tables: &SPARKSQL_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3340), tables: &SPARKSQL_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3341), tables: &SPARKSQL_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3342), tables: &SPARKSQL_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3343), tables: &SPARKSQL_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3344), tables: &SPARKSQL_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3345), tables: &SPARKSQL_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3346), tables: &SPARKSQL_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3347), tables: &SPARKSQL_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3348), tables: &SPARKSQL_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3349), tables: &SPARKSQL_TABLES }),
        "ReloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3350), tables: &SPARKSQL_TABLES }),
        "RemoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3351), tables: &SPARKSQL_TABLES }),
        "RemoveWidgetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3352), tables: &SPARKSQL_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3356), tables: &SPARKSQL_TABLES }),
        "RepairKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3357), tables: &SPARKSQL_TABLES }),
        "RepeatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3358), tables: &SPARKSQL_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3359), tables: &SPARKSQL_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3360), tables: &SPARKSQL_TABLES }),
        "ReplaceTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3361), tables: &SPARKSQL_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3364), tables: &SPARKSQL_TABLES }),
        "RequireKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3365), tables: &SPARKSQL_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3366), tables: &SPARKSQL_TABLES }),
        "ResetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3367), tables: &SPARKSQL_TABLES }),
        "ResignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3372), tables: &SPARKSQL_TABLES }),
        "ResourceFileGrammar" => Some(RootGrammar { grammar_id: GrammarId(3373), tables: &SPARKSQL_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3377), tables: &SPARKSQL_TABLES }),
        "ResourceLocationGrammar" => Some(RootGrammar { grammar_id: GrammarId(3378), tables: &SPARKSQL_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3382), tables: &SPARKSQL_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3383), tables: &SPARKSQL_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3384), tables: &SPARKSQL_TABLES }),
        "RestoreTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3385), tables: &SPARKSQL_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3396), tables: &SPARKSQL_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3397), tables: &SPARKSQL_TABLES }),
        "RetainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3398), tables: &SPARKSQL_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3399), tables: &SPARKSQL_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3400), tables: &SPARKSQL_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3401), tables: &SPARKSQL_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3402), tables: &SPARKSQL_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3403), tables: &SPARKSQL_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3404), tables: &SPARKSQL_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3405), tables: &SPARKSQL_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3406), tables: &SPARKSQL_TABLES }),
        "RightArrowOperator" => Some(RootGrammar { grammar_id: GrammarId(3429), tables: &SPARKSQL_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3430), tables: &SPARKSQL_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3431), tables: &SPARKSQL_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3432), tables: &SPARKSQL_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(3433), tables: &SPARKSQL_TABLES }),
        "RolesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3434), tables: &SPARKSQL_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3435), tables: &SPARKSQL_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(3436), tables: &SPARKSQL_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3437), tables: &SPARKSQL_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3438), tables: &SPARKSQL_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3439), tables: &SPARKSQL_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3440), tables: &SPARKSQL_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3441), tables: &SPARKSQL_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3442), tables: &SPARKSQL_TABLES }),
        "RowFormatClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3443), tables: &SPARKSQL_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3476), tables: &SPARKSQL_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3477), tables: &SPARKSQL_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3478), tables: &SPARKSQL_TABLES }),
        "RowcountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3479), tables: &SPARKSQL_TABLES }),
        "RowguidcolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3480), tables: &SPARKSQL_TABLES }),
        "RowidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3481), tables: &SPARKSQL_TABLES }),
        "RownumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3482), tables: &SPARKSQL_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3483), tables: &SPARKSQL_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3484), tables: &SPARKSQL_TABLES }),
        "RunKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3485), tables: &SPARKSQL_TABLES }),
        "SQLConfPropertiesSegment" => Some(RootGrammar { grammar_id: GrammarId(3486), tables: &SPARKSQL_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3489), tables: &SPARKSQL_TABLES }),
        "SaveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3503), tables: &SPARKSQL_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3504), tables: &SPARKSQL_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3505), tables: &SPARKSQL_TABLES }),
        "ScdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3506), tables: &SPARKSQL_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3507), tables: &SPARKSQL_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3508), tables: &SPARKSQL_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3509), tables: &SPARKSQL_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3510), tables: &SPARKSQL_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3511), tables: &SPARKSQL_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3512), tables: &SPARKSQL_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3513), tables: &SPARKSQL_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3514), tables: &SPARKSQL_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3515), tables: &SPARKSQL_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3516), tables: &SPARKSQL_TABLES }),
        "Second_microsecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3517), tables: &SPARKSQL_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3518), tables: &SPARKSQL_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3519), tables: &SPARKSQL_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(3520), tables: &SPARKSQL_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3525), tables: &SPARKSQL_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3530), tables: &SPARKSQL_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(3539), tables: &SPARKSQL_TABLES }),
        "SelectHintSegment" => Some(RootGrammar { grammar_id: GrammarId(3560), tables: &SPARKSQL_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3568), tables: &SPARKSQL_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3569), tables: &SPARKSQL_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3589), tables: &SPARKSQL_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3599), tables: &SPARKSQL_TABLES }),
        "SemiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3600), tables: &SPARKSQL_TABLES }),
        "SemiStructuredAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(3601), tables: &SPARKSQL_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(3620), tables: &SPARKSQL_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3621), tables: &SPARKSQL_TABLES }),
        "SeparatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3622), tables: &SPARKSQL_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3623), tables: &SPARKSQL_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3624), tables: &SPARKSQL_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3625), tables: &SPARKSQL_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(3632), tables: &SPARKSQL_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "SequencefileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3639), tables: &SPARKSQL_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3640), tables: &SPARKSQL_TABLES }),
        "SerdeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3641), tables: &SPARKSQL_TABLES }),
        "SerdePropertiesGrammar" => Some(RootGrammar { grammar_id: GrammarId(3642), tables: &SPARKSQL_TABLES }),
        "SerdepropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3646), tables: &SPARKSQL_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3647), tables: &SPARKSQL_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3648), tables: &SPARKSQL_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3649), tables: &SPARKSQL_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3650), tables: &SPARKSQL_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3651), tables: &SPARKSQL_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(3652), tables: &SPARKSQL_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3656), tables: &SPARKSQL_TABLES }),
        "SetConfigValueSegment" => Some(RootGrammar { grammar_id: GrammarId(3667), tables: &SPARKSQL_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(3679), tables: &SPARKSQL_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3688), tables: &SPARKSQL_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(3689), tables: &SPARKSQL_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3706), tables: &SPARKSQL_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3711), tables: &SPARKSQL_TABLES }),
        "SetVariableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3720), tables: &SPARKSQL_TABLES }),
        "SetminusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3735), tables: &SPARKSQL_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3736), tables: &SPARKSQL_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3737), tables: &SPARKSQL_TABLES }),
        "SetuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3738), tables: &SPARKSQL_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3739), tables: &SPARKSQL_TABLES }),
        "SharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3740), tables: &SPARKSQL_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(3741), tables: &SPARKSQL_TABLES }),
        "ShowDatabasesSchemasGrammar" => Some(RootGrammar { grammar_id: GrammarId(3750), tables: &SPARKSQL_TABLES }),
        "ShowFunctionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3757), tables: &SPARKSQL_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3772), tables: &SPARKSQL_TABLES }),
        "ShowObjectGrammar" => Some(RootGrammar { grammar_id: GrammarId(3773), tables: &SPARKSQL_TABLES }),
        "ShowStatement" => Some(RootGrammar { grammar_id: GrammarId(3811), tables: &SPARKSQL_TABLES }),
        "ShowTablesGrammar" => Some(RootGrammar { grammar_id: GrammarId(3814), tables: &SPARKSQL_TABLES }),
        "ShowViewsGrammar" => Some(RootGrammar { grammar_id: GrammarId(3824), tables: &SPARKSQL_TABLES }),
        "ShutdownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3834), tables: &SPARKSQL_TABLES }),
        "SignalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3835), tables: &SPARKSQL_TABLES }),
        "SignedQuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(3836), tables: &SPARKSQL_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(3839), tables: &SPARKSQL_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3842), tables: &SPARKSQL_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3843), tables: &SPARKSQL_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(3844), tables: &SPARKSQL_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(3849), tables: &SPARKSQL_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(3851), tables: &SPARKSQL_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(3855), tables: &SPARKSQL_TABLES }),
        "SingleValueColumnUnpivotSegment" => Some(RootGrammar { grammar_id: GrammarId(3856), tables: &SPARKSQL_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3866), tables: &SPARKSQL_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(3867), tables: &SPARKSQL_TABLES }),
        "SkewedByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3870), tables: &SPARKSQL_TABLES }),
        "SkewedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3886), tables: &SPARKSQL_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(3887), tables: &SPARKSQL_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(3888), tables: &SPARKSQL_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3889), tables: &SPARKSQL_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3890), tables: &SPARKSQL_TABLES }),
        "SonameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3891), tables: &SPARKSQL_TABLES }),
        "SortByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(3892), tables: &SPARKSQL_TABLES }),
        "SortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3915), tables: &SPARKSQL_TABLES }),
        "SortedBySpecGrammar" => Some(RootGrammar { grammar_id: GrammarId(3916), tables: &SPARKSQL_TABLES }),
        "SortedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3926), tables: &SPARKSQL_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3927), tables: &SPARKSQL_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3928), tables: &SPARKSQL_TABLES }),
        "SpatialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3929), tables: &SPARKSQL_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3930), tables: &SPARKSQL_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3931), tables: &SPARKSQL_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3932), tables: &SPARKSQL_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3933), tables: &SPARKSQL_TABLES }),
        "Sql_big_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3934), tables: &SPARKSQL_TABLES }),
        "Sql_big_selectsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3935), tables: &SPARKSQL_TABLES }),
        "Sql_big_tablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3936), tables: &SPARKSQL_TABLES }),
        "Sql_calc_found_rowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3937), tables: &SPARKSQL_TABLES }),
        "Sql_log_offKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3938), tables: &SPARKSQL_TABLES }),
        "Sql_log_updateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3939), tables: &SPARKSQL_TABLES }),
        "Sql_low_priority_updatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3940), tables: &SPARKSQL_TABLES }),
        "Sql_select_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3941), tables: &SPARKSQL_TABLES }),
        "Sql_small_resultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3942), tables: &SPARKSQL_TABLES }),
        "Sql_warningsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3943), tables: &SPARKSQL_TABLES }),
        "SqlcaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3944), tables: &SPARKSQL_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3945), tables: &SPARKSQL_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3946), tables: &SPARKSQL_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3947), tables: &SPARKSQL_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3948), tables: &SPARKSQL_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3949), tables: &SPARKSQL_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3950), tables: &SPARKSQL_TABLES }),
        "SslKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3951), tables: &SPARKSQL_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3952), tables: &SPARKSQL_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3953), tables: &SPARKSQL_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3954), tables: &SPARKSQL_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(3955), tables: &SPARKSQL_TABLES }),
        "StartAngleBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(577), tables: &SPARKSQL_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(240), tables: &SPARKSQL_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2912), tables: &SPARKSQL_TABLES }),
        "StartHintSegment" => Some(RootGrammar { grammar_id: GrammarId(3956), tables: &SPARKSQL_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3957), tables: &SPARKSQL_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(566), tables: &SPARKSQL_TABLES }),
        "StartingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3958), tables: &SPARKSQL_TABLES }),
        "StartsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3959), tables: &SPARKSQL_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3960), tables: &SPARKSQL_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3961), tables: &SPARKSQL_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3962), tables: &SPARKSQL_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &SPARKSQL_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4034), tables: &SPARKSQL_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4035), tables: &SPARKSQL_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &SPARKSQL_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4037), tables: &SPARKSQL_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4038), tables: &SPARKSQL_TABLES }),
        "StorageFormatGrammar" => Some(RootGrammar { grammar_id: GrammarId(4039), tables: &SPARKSQL_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4044), tables: &SPARKSQL_TABLES }),
        "StoredAsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4045), tables: &SPARKSQL_TABLES }),
        "StoredByGrammar" => Some(RootGrammar { grammar_id: GrammarId(4049), tables: &SPARKSQL_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4054), tables: &SPARKSQL_TABLES }),
        "Straight_joinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4055), tables: &SPARKSQL_TABLES }),
        "StratifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4056), tables: &SPARKSQL_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4057), tables: &SPARKSQL_TABLES }),
        "StreamingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4058), tables: &SPARKSQL_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4059), tables: &SPARKSQL_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4060), tables: &SPARKSQL_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4061), tables: &SPARKSQL_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4063), tables: &SPARKSQL_TABLES }),
        "StructKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4064), tables: &SPARKSQL_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4065), tables: &SPARKSQL_TABLES }),
        "StructTypeSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(4070), tables: &SPARKSQL_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4077), tables: &SPARKSQL_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4080), tables: &SPARKSQL_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4081), tables: &SPARKSQL_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4082), tables: &SPARKSQL_TABLES }),
        "SublistKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4083), tables: &SPARKSQL_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4084), tables: &SPARKSQL_TABLES }),
        "SubstrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4085), tables: &SPARKSQL_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4086), tables: &SPARKSQL_TABLES }),
        "SuccessfulKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4087), tables: &SPARKSQL_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4088), tables: &SPARKSQL_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &SPARKSQL_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(4090), tables: &SPARKSQL_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4091), tables: &SPARKSQL_TABLES }),
        "SyncKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &SPARKSQL_TABLES }),
        "SynonymKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4093), tables: &SPARKSQL_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4094), tables: &SPARKSQL_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4095), tables: &SPARKSQL_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4096), tables: &SPARKSQL_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4097), tables: &SPARKSQL_TABLES }),
        "TableClusterByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4098), tables: &SPARKSQL_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4104), tables: &SPARKSQL_TABLES }),
        "TableDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(4119), tables: &SPARKSQL_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4164), tables: &SPARKSQL_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4165), tables: &SPARKSQL_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4181), tables: &SPARKSQL_TABLES }),
        "TablePropertiesGrammar" => Some(RootGrammar { grammar_id: GrammarId(4182), tables: &SPARKSQL_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4185), tables: &SPARKSQL_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4186), tables: &SPARKSQL_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4187), tables: &SPARKSQL_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4188), tables: &SPARKSQL_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4189), tables: &SPARKSQL_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4194), tables: &SPARKSQL_TABLES }),
        "TargetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4198), tables: &SPARKSQL_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4199), tables: &SPARKSQL_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4200), tables: &SPARKSQL_TABLES }),
        "TblpropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4201), tables: &SPARKSQL_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4202), tables: &SPARKSQL_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4203), tables: &SPARKSQL_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(4204), tables: &SPARKSQL_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(4205), tables: &SPARKSQL_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4211), tables: &SPARKSQL_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(4212), tables: &SPARKSQL_TABLES }),
        "TerminateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4215), tables: &SPARKSQL_TABLES }),
        "TerminatedByGrammar" => Some(RootGrammar { grammar_id: GrammarId(4216), tables: &SPARKSQL_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4220), tables: &SPARKSQL_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4221), tables: &SPARKSQL_TABLES }),
        "TextfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4222), tables: &SPARKSQL_TABLES }),
        "TextsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4223), tables: &SPARKSQL_TABLES }),
        "ThanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4224), tables: &SPARKSQL_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4225), tables: &SPARKSQL_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4226), tables: &SPARKSQL_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(4227), tables: &SPARKSQL_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4228), tables: &SPARKSQL_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(4229), tables: &SPARKSQL_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(4240), tables: &SPARKSQL_TABLES }),
        "TimestampAsOfGrammar" => Some(RootGrammar { grammar_id: GrammarId(4246), tables: &SPARKSQL_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4254), tables: &SPARKSQL_TABLES }),
        "Timestamp_ltzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4255), tables: &SPARKSQL_TABLES }),
        "Timestamp_ntzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4256), tables: &SPARKSQL_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4257), tables: &SPARKSQL_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4258), tables: &SPARKSQL_TABLES }),
        "TinyblobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4259), tables: &SPARKSQL_TABLES }),
        "TinyintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4260), tables: &SPARKSQL_TABLES }),
        "TinytextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4261), tables: &SPARKSQL_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4262), tables: &SPARKSQL_TABLES }),
        "ToastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4263), tables: &SPARKSQL_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4264), tables: &SPARKSQL_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4265), tables: &SPARKSQL_TABLES }),
        "TouchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4266), tables: &SPARKSQL_TABLES }),
        "TrackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4267), tables: &SPARKSQL_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4268), tables: &SPARKSQL_TABLES }),
        "TranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4269), tables: &SPARKSQL_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4270), tables: &SPARKSQL_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4271), tables: &SPARKSQL_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4288), tables: &SPARKSQL_TABLES }),
        "TransactionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4289), tables: &SPARKSQL_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4290), tables: &SPARKSQL_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4291), tables: &SPARKSQL_TABLES }),
        "TransformClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4292), tables: &SPARKSQL_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4308), tables: &SPARKSQL_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4309), tables: &SPARKSQL_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4310), tables: &SPARKSQL_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4311), tables: &SPARKSQL_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4312), tables: &SPARKSQL_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4313), tables: &SPARKSQL_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4314), tables: &SPARKSQL_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(861), tables: &SPARKSQL_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4315), tables: &SPARKSQL_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4316), tables: &SPARKSQL_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4317), tables: &SPARKSQL_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4318), tables: &SPARKSQL_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(4319), tables: &SPARKSQL_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4323), tables: &SPARKSQL_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(4324), tables: &SPARKSQL_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4325), tables: &SPARKSQL_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4326), tables: &SPARKSQL_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4331), tables: &SPARKSQL_TABLES }),
        "Try_castKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4332), tables: &SPARKSQL_TABLES }),
        "TsequalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4333), tables: &SPARKSQL_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(4334), tables: &SPARKSQL_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4337), tables: &SPARKSQL_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4338), tables: &SPARKSQL_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4341), tables: &SPARKSQL_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4344), tables: &SPARKSQL_TABLES }),
        "UidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4345), tables: &SPARKSQL_TABLES }),
        "UnarchiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4346), tables: &SPARKSQL_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4347), tables: &SPARKSQL_TABLES }),
        "UncacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4348), tables: &SPARKSQL_TABLES }),
        "UncacheTableSegment" => Some(RootGrammar { grammar_id: GrammarId(4349), tables: &SPARKSQL_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4354), tables: &SPARKSQL_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4355), tables: &SPARKSQL_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4356), tables: &SPARKSQL_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4360), tables: &SPARKSQL_TABLES }),
        "UndoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4361), tables: &SPARKSQL_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4362), tables: &SPARKSQL_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(4363), tables: &SPARKSQL_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4368), tables: &SPARKSQL_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4369), tables: &SPARKSQL_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4371), tables: &SPARKSQL_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4372), tables: &SPARKSQL_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4373), tables: &SPARKSQL_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4374), tables: &SPARKSQL_TABLES }),
        "UnlockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4375), tables: &SPARKSQL_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4376), tables: &SPARKSQL_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4377), tables: &SPARKSQL_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4378), tables: &SPARKSQL_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4394), tables: &SPARKSQL_TABLES }),
        "UnpivotClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4400), tables: &SPARKSQL_TABLES }),
        "UnpivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4411), tables: &SPARKSQL_TABLES }),
        "UnsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4412), tables: &SPARKSQL_TABLES }),
        "UnsetTablePropertiesGrammar" => Some(RootGrammar { grammar_id: GrammarId(4413), tables: &SPARKSQL_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4418), tables: &SPARKSQL_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4419), tables: &SPARKSQL_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4420), tables: &SPARKSQL_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4421), tables: &SPARKSQL_TABLES }),
        "UpdatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4430), tables: &SPARKSQL_TABLES }),
        "UpdatetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4431), tables: &SPARKSQL_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4432), tables: &SPARKSQL_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4433), tables: &SPARKSQL_TABLES }),
        "UseDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4434), tables: &SPARKSQL_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4437), tables: &SPARKSQL_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4438), tables: &SPARKSQL_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4441), tables: &SPARKSQL_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4442), tables: &SPARKSQL_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4443), tables: &SPARKSQL_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4444), tables: &SPARKSQL_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4445), tables: &SPARKSQL_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4446), tables: &SPARKSQL_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4447), tables: &SPARKSQL_TABLES }),
        "UsersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4448), tables: &SPARKSQL_TABLES }),
        "UsingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4449), tables: &SPARKSQL_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4452), tables: &SPARKSQL_TABLES }),
        "Utc_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4453), tables: &SPARKSQL_TABLES }),
        "Utc_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4454), tables: &SPARKSQL_TABLES }),
        "Utc_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4455), tables: &SPARKSQL_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4456), tables: &SPARKSQL_TABLES }),
        "VacuumStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4457), tables: &SPARKSQL_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4471), tables: &SPARKSQL_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4472), tables: &SPARKSQL_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4473), tables: &SPARKSQL_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4474), tables: &SPARKSQL_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4475), tables: &SPARKSQL_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4494), tables: &SPARKSQL_TABLES }),
        "VarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4495), tables: &SPARKSQL_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4496), tables: &SPARKSQL_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4497), tables: &SPARKSQL_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4498), tables: &SPARKSQL_TABLES }),
        "Varchar2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4499), tables: &SPARKSQL_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4500), tables: &SPARKSQL_TABLES }),
        "VarcharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4501), tables: &SPARKSQL_TABLES }),
        "VariableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4502), tables: &SPARKSQL_TABLES }),
        "VariablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4503), tables: &SPARKSQL_TABLES }),
        "VariantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4504), tables: &SPARKSQL_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4505), tables: &SPARKSQL_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4506), tables: &SPARKSQL_TABLES }),
        "VersionAsOfGrammar" => Some(RootGrammar { grammar_id: GrammarId(4507), tables: &SPARKSQL_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4512), tables: &SPARKSQL_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4513), tables: &SPARKSQL_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4514), tables: &SPARKSQL_TABLES }),
        "ViolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4515), tables: &SPARKSQL_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4516), tables: &SPARKSQL_TABLES }),
        "WaitforKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4517), tables: &SPARKSQL_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4518), tables: &SPARKSQL_TABLES }),
        "WarehousesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4519), tables: &SPARKSQL_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4520), tables: &SPARKSQL_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4521), tables: &SPARKSQL_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4522), tables: &SPARKSQL_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4532), tables: &SPARKSQL_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4533), tables: &SPARKSQL_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4534), tables: &SPARKSQL_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4539), tables: &SPARKSQL_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4560), tables: &SPARKSQL_TABLES }),
        "WhileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4561), tables: &SPARKSQL_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(4562), tables: &SPARKSQL_TABLES }),
        "WhlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4563), tables: &SPARKSQL_TABLES }),
        "WidgetDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(4564), tables: &SPARKSQL_TABLES }),
        "WidgetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4567), tables: &SPARKSQL_TABLES }),
        "WidgetNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4568), tables: &SPARKSQL_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4569), tables: &SPARKSQL_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4570), tables: &SPARKSQL_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4573), tables: &SPARKSQL_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4583), tables: &SPARKSQL_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(4584), tables: &SPARKSQL_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4592), tables: &SPARKSQL_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4601), tables: &SPARKSQL_TABLES }),
        "WithCubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4610), tables: &SPARKSQL_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4615), tables: &SPARKSQL_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(4620), tables: &SPARKSQL_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4621), tables: &SPARKSQL_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4622), tables: &SPARKSQL_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4627), tables: &SPARKSQL_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4628), tables: &SPARKSQL_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(4629), tables: &SPARKSQL_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4630), tables: &SPARKSQL_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4631), tables: &SPARKSQL_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4632), tables: &SPARKSQL_TABLES }),
        "WritetextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4633), tables: &SPARKSQL_TABLES }),
        "X509KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4634), tables: &SPARKSQL_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4635), tables: &SPARKSQL_TABLES }),
        "XorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4636), tables: &SPARKSQL_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4637), tables: &SPARKSQL_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4638), tables: &SPARKSQL_TABLES }),
        "Year_monthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4639), tables: &SPARKSQL_TABLES }),
        "YearsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4640), tables: &SPARKSQL_TABLES }),
        "ZerofillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4641), tables: &SPARKSQL_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4642), tables: &SPARKSQL_TABLES }),
        _ => None,
    }
}

pub fn get_sparksql_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AddFileSegment" => Some("add_file_statement"),
        "AddJarSegment" => Some("add_jar_statement"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AnalyzeTableSegment" => Some("analyze_table_statement"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CacheTableSegment" => Some("cache_table"),
        "CaseExpressionSegment" => Some("case_expression"),
        "ClearCacheSegment" => Some("clear_cache"),
        "ClusterByClauseSegment" => Some("cluster_by_clause"),
        "CodeSegment" => Some("raw"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnFieldDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "ConcatSegment" => Some("binary_operator"),
        "ConstraintStatementSegment" => Some("constraint_statement"),
        "ConvertToDeltaStatementSegment" => Some("convert_to_delta_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CreateWidgetStatementSegment" => Some("create_widget_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DataSourceFormatSegment" => Some("data_source_format"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "Dedent" => Some("dedent"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescribeDetailStatementSegment" => Some("describe_detail_statement"),
        "DescribeHistoryStatementSegment" => Some("describe_history_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DistributeByClauseSegment" => Some("distribute_by_clause"),
        "DivBinaryOperatorSegment" => Some("binary_operator"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExceptClauseSegment" => Some("select_except_clause"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FileLiteralSegment" => Some("file_literal"),
        "FileReferenceSegment" => Some("file_reference"),
        "FileSegment" => Some("file"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FromInsertClauseSegment" => Some("from_insert_clause"),
        "FromInsertFromExpressionElementSegment" => Some("from_expression_element"),
        "FromInsertFromExpressionSegment" => Some("from_expression"),
        "FromInsertSourceClauseSegment" => Some("from_insert_source_clause"),
        "FromInsertStatementSegment" => Some("from_insert_statement"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionParameterListGrammarWithComments" => Some("function_parameter_list_with_comments"),
        "FunctionSegment" => Some("function"),
        "GenerateManifestFileStatementSegment" => Some("generate_manifest_file_statement"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "HintFunctionSegment" => Some("hint_function"),
        "IcebergTransformationSegment" => Some("iceberg_transformation"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "Indent" => Some("indent"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertOverwriteDirectoryHiveFmtSegment" => Some("insert_overwrite_directory_hive_fmt_statement"),
        "InsertOverwriteDirectorySegment" => Some("insert_overwrite_directory_statement"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntervalLiteralSegment" => Some("interval_literal"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LateralViewClauseSegment" => Some("lateral_view_clause"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListFileSegment" => Some("list_file_statement"),
        "ListJarSegment" => Some("list_jar_statement"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadDataSegment" => Some("load_data_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedBySourceClauseSegment" => Some("merge_when_not_matched_by_source_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MsckRepairTableStatementSegment" => Some("msck_repair_table_statement"),
        "MultiValueColumnUnpivotSegment" => Some("unpivot_multi_column"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PathSegment" => Some("path_segment"),
        "PivotClauseSegment" => Some("pivot_clause"),
        "PrimitiveTypeSegment" => Some("primitive_type"),
        "PropertyNameSegment" => Some("property_name_identifier"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifyClauseSegment" => Some("qualify_clause"),
        "RefreshStatementSegment" => Some("refresh_statement"),
        "RemoveWidgetStatementSegment" => Some("remove_widget_statement"),
        "ReplaceTableStatementSegment" => Some("replace_table_statement"),
        "ResetStatementSegment" => Some("reset_statement"),
        "RestoreTableStatementSegment" => Some("restore_table_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RowFormatClauseSegment" => Some("row_format_clause"),
        "SQLConfPropertiesSegment" => Some("sql_conf_option"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectHintSegment" => Some("select_hint"),
        "SelectStatementSegment" => Some("select_statement"),
        "SemiStructuredAccessorSegment" => Some("semi_structured_expression"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetConfigValueSegment" => Some("set_config_value"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "SetVariableStatementSegment" => Some("set_variable_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "ShowStatement" => Some("show_statement"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SingleValueColumnUnpivotSegment" => Some("unpivot_single_column"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "SkewedByClauseSegment" => Some("skewed_by_clause"),
        "SortByClauseSegment" => Some("sort_by_clause"),
        "StatementSegment" => Some("statement"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSchemaSegment" => Some("struct_type_schema"),
        "StructTypeSegment" => Some("struct_type"),
        "SymbolSegment" => Some("symbol"),
        "TableClusterByClauseSegment" => Some("table_cluster_by_clause"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TransformClauseSegment" => Some("transform_clause"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UncacheTableSegment" => Some("uncache_table"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UnpivotClauseSegment" => Some("unpivot_clause"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseDatabaseStatementSegment" => Some("use_database_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "UsingClauseSegment" => Some("using_clause"),
        "VacuumStatementSegment" => Some("vacuum_statement"),
        "ValuesClauseSegment" => Some("values_clause"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithCubeRollupClauseSegment" => Some("with_cube_rollup_clause"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static SPARKSQL_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_sparksql_root_grammar_id() -> GrammarId {
    GrammarId(1763)
}
pub fn get_sparksql_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_sparksql_root_grammar_id(),
        tables: &SPARKSQL_TABLES,
    }
}
pub fn get_sparksql_root_grammar() -> RootGrammar {
    get_sparksql_root_grammar_table()
}
