/* This is a generated file! */
/* Generated by `utils/build_parsers.py` via `utils/rustify.py` */
/* This process can be run via tox: `tox -e generate-rs` */
#![cfg_attr(rustfmt, rustfmt_skip)]

use sqlfluffrs_types::{
    GrammarInst, GrammarVariant, GrammarFlags,
    ParseMode, GrammarId, GrammarTables,
    SimpleHintData, RootGrammar
};
// Flattening grammar tree...
// Validating tables...
// Validation passed!

// Table Statistics:
//   Instructions:      8251 × 20 bytes =   165020 bytes
//   Child IDs:         7133 ×  4 bytes =    28532 bytes
//   Terminators:         88 ×  4 bytes =      352 bytes
//   Strings:           2783 strings      =    43202 bytes
//   Aux Data:         13921 ×  4 bytes =    55684 bytes
//   Aux Offsets:       8251 ×  4 bytes =    33004 bytes
//   Regex Patterns:      11 patterns     =     1387 bytes
//   Simple Hints:      1499 hints        =    23984 bytes
//   Hint Indices:      3295 ×  4 bytes =    13180 bytes
//   TOTAL:                                    ≈   364345 bytes

pub static INSTRUCTIONS: &[GrammarInst] = &[
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [0] StringParser("A")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [1] StringParser("ABORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [2] StringParser("ABS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [3] StringParser("ABSENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [4] StringParser("ABSOLUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [5] StringParser("ACCEPTANYDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [6] StringParser("ACCEPTINVCHARS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 0, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [7] StringParser("ACCESS")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 0, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [8] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [9] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [10] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5, child_count: 17, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [11] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 22, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [12] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [13] Ref(ResourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [14] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [15] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [16] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [17] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [18] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [19] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [20] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [21] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [22] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [23] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [24] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 24, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [25] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [26] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 26, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [27] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [28] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 28, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [29] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [30] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [31] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 30, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [32] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [33] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [34] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [35] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [36] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 34, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [37] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [38] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [39] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [40] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [41] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [42] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 38, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [43] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [44] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [45] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [46] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [47] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 42, child_count: 4, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [48] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [49] Ref(FutureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [50] Ref(AccessSchemaPluralObjectSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [51] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 46, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [52] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [53] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [54] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 48, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 2, _padding: 0 }, // [55] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 50, child_count: 3, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [56] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [57] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 53, child_count: 2, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [58] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [59] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [60] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [61] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [62] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [63] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 0, min_times: 0, first_terminator_idx: 0, terminator_count: 0, _padding: 0 }, // [64] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 55, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [65] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [66] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [67] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [68] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 58, child_count: 6, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [69] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 64, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [70] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [71] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 66, child_count: 5, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [72] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [73] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [74] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [75] Ref(WarehouseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [76] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [77] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 71, child_count: 3, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [78] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [79] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [80] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [81] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 74, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [82] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [83] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [84] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 76, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [85] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [86] Ref(ManageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [87] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 78, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [88] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [89] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 80, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [90] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [91] Ref(ExecutionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [92] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 82, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [93] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 84, child_count: 26, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [94] OneOf(26 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 110, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [95] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [96] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 112, child_count: 4, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [97] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [98] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 116, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [99] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [100] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [101] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [102] Ref(PipeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [103] Ref(AccessSchemaObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 118, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [105] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 120, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [106] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [107] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [108] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 122, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [110] Ref(ImportedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [111] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [112] Ref(ApplyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [113] Ref(ConnectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [114] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [115] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [116] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [117] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [118] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [119] Ref(MonitorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [120] Ref(OperateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [121] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [122] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [123] Ref(Reference_usageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [124] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [125] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [126] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [127] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [128] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [129] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [130] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [131] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [132] Ref(Use_any_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [133] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 124, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [134] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [135] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [136] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 126, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [137] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 126, child_count: 2, min_times: 0, first_terminator_idx: 2, terminator_count: 1, _padding: 0 }, // [138] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [139] Ref(AccessPermissionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 0, min_times: 0, first_terminator_idx: 2, terminator_count: 0, _padding: 0 }, // [140] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 128, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [141] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [142] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [143] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [144] Ref(StageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [145] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [146] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [147] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [148] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [149] Ref(StreamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [150] Ref(TaskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [151] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 142, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [152] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [153] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [154] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 144, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [155] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [156] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [157] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 146, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [159] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [160] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 148, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [161] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [162] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [163] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 150, child_count: 14, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [164] OneOf(14 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [165] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [166] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [167] Ref(StagesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [168] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [169] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [170] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [171] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [172] Ref(StreamsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [173] Ref(TasksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [174] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 164, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [175] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [176] Ref(MaskingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [177] Ref(PoliciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 166, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [178] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [179] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [180] Ref(ViewsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 168, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [181] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [182] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [183] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 170, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [184] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [185] Ref(FileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [186] Ref(FormatsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 172, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [187] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [188] Ref(GrantStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [189] Ref(RevokeStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 174, child_count: 7, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [190] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 181, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [191] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [192] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 183, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [193] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [194] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 185, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [195] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [196] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 187, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [197] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [198] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 189, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [199] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [200] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 191, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [201] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [202] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 193, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [203] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [204] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 195, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [205] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [206] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 197, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [207] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [208] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 199, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [209] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [210] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [211] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 201, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [212] StringParser("ACCESS_KEY_ID")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 201, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [213] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [214] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [215] Ref(SemiStructuredAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [216] StringParser("ACCORDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [217] StringParser("ACCOUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [218] StringParser("ACOS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [219] StringParser("ACTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [220] StringParser("ADA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [221] StringParser("ADD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [222] StringParser("ADDQUOTES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 203, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [223] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [224] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [225] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [226] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [227] StringParser("ADMIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [228] StringParser("AES128")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [229] StringParser("AES256")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [230] StringParser("AFTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [231] StringParser("AGGREGATE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [232] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 206, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [233] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [234] Meta("indent")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 210, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [235] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 210, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [236] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 212, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [237] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 214, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [238] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 214, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [239] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [240] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [241] StringParser("(")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 217, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [242] StringParser(")")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 217, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [243] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 219, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [244] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 219, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [245] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 222, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [246] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 224, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [247] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [248] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [249] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 226, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [250] Meta("dedent")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 226, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [251] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [252] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [253] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [254] StringParser("ALL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [255] StringParser("ALLOCATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [256] StringParser("ALLOWOVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 228, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [257] StringParser("ALSO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 228, child_count: 5, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [258] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [259] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [260] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [261] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 233, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [262] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 236, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [263] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [264] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [265] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [266] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 239, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [267] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 242, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [268] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [269] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [270] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [271] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 245, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [273] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [274] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 248, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [275] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [276] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [277] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [278] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [279] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 252, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [280] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [281] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [282] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [283] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 255, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [284] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [285] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [286] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 259, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [287] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 259, child_count: 6, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [288] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 265, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [289] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 267, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [290] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 267, child_count: 3, min_times: 1, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [291] AnyNumberOf(3 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 270, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [292] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [293] Ref(Allow_connectionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [294] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 272, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [295] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [296] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [297] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [298] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 275, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [299] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [300] Ref(Is_templateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [301] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 277, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [303] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [304] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [305] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 280, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [306] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [307] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [308] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 283, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [309] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [310] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [311] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [312] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [313] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 287, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [314] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [315] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [316] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [317] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 290, child_count: 3, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [318] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [319] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [320] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 293, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [321] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 295, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [322] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 297, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [323] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [324] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [325] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 299, child_count: 4, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [326] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [327] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [328] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [329] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [330] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 303, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [332] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [333] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 305, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [335] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 307, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [336] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [337] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [338] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 309, child_count: 7, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [339] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [340] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [341] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [342] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [343] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [344] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 316, child_count: 2, min_times: 0, first_terminator_idx: 3, terminator_count: 1, _padding: 0 }, // [345] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [346] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 318, child_count: 0, min_times: 0, first_terminator_idx: 3, terminator_count: 0, _padding: 0 }, // [347] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 318, child_count: 3, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [348] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [349] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [350] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [351] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 321, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [352] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 323, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [353] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [354] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 325, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [355] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 325, child_count: 2, min_times: 0, first_terminator_idx: 4, terminator_count: 1, _padding: 0 }, // [356] Delimited(10 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 327, child_count: 10, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [357] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [358] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [359] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [360] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [361] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [362] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [363] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [364] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [365] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [366] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [367] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 0, min_times: 0, first_terminator_idx: 4, terminator_count: 0, _padding: 0 }, // [368] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 337, child_count: 8, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [369] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 345, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [370] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 345, child_count: 3, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [372] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [373] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [374] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [375] Ref(AlterDefaultPrivilegesObjectPrivilegesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [376] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [377] Ref(AlterDefaultPrivilegesSchemaObjectsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [378] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 348, child_count: 2, min_times: 0, first_terminator_idx: 5, terminator_count: 2, _padding: 0 }, // [379] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [380] Ref(AlterDefaultPrivilegesToFromRolesSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [381] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 5, terminator_count: 0, _padding: 0 }, // [382] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 350, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [383] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 350, child_count: 7, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [384] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [385] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [386] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [387] Ref(RoutinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [388] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [389] Ref(TypesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [390] Ref(SchemasKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 357, child_count: 1, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [391] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [392] Ref(ProceduresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 358, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [393] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [394] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [395] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 364, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [396] Ref(PrivilegesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 364, child_count: 3, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [397] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [398] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 367, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [399] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [400] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [401] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 369, child_count: 2, min_times: 0, first_terminator_idx: 7, terminator_count: 3, _padding: 0 }, // [402] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [403] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [404] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [405] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 371, child_count: 0, min_times: 0, first_terminator_idx: 7, terminator_count: 0, _padding: 0 }, // [406] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 371, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [407] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [408] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [409] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 374, child_count: 2, min_times: 0, first_terminator_idx: 10, terminator_count: 2, _padding: 0 }, // [410] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [411] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [412] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 0, min_times: 0, first_terminator_idx: 10, terminator_count: 0, _padding: 0 }, // [413] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 376, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [414] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [415] Ref(AlterDefaultPrivilegesGrantSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [416] Ref(AlterDefaultPrivilegesRevokeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 378, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [417] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 380, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [419] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [420] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [421] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 382, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [422] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [423] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [424] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [425] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 386, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [426] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 396, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [427] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [428] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [429] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [430] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 399, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [431] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [432] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [433] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 401, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [434] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 404, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [435] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [436] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [437] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [438] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [439] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 406, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [440] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [441] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 410, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [442] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [443] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [444] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 412, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [445] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 414, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [446] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [447] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [448] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 416, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [450] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 418, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [451] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 418, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [452] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [453] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [454] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 420, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [455] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [456] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [457] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [458] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [459] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 425, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [460] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [461] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [462] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 427, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [463] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [464] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [465] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [466] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [467] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [468] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 432, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [469] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [470] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [471] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [472] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 435, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [473] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [474] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [475] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 438, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [476] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [477] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [478] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [479] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [480] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 442, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [481] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [482] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [483] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [484] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 445, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [485] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [486] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [487] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [488] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 448, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [489] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [490] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [491] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [492] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 452, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [493] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 455, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [494] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [495] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 457, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [496] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [497] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [498] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 459, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [499] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [500] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [501] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 462, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [502] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [503] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [504] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 464, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [505] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 466, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [506] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [507] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [508] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 468, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [509] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 475, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [510] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 477, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [511] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 490, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [512] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [513] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [514] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [515] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [516] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [517] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 492, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [518] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [519] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [520] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 494, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [521] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [522] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [523] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [524] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 497, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [525] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [526] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [527] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 499, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [528] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [529] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [530] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [531] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [532] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [533] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 501, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [534] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [535] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [536] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 504, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [537] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [538] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [539] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [540] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [541] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [542] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [543] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 508, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 510, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [545] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 513, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [546] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [547] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [548] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [549] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [550] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [551] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 515, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [552] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [553] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [554] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 518, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [555] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 521, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [556] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 522, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [557] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 522, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [558] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [559] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 524, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [560] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 527, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [561] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [562] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [563] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [564] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 530, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [565] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 532, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [566] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 535, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [567] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 535, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [568] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 535, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [569] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 535, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [570] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 537, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [571] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [572] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 539, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [573] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 539, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [575] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 541, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [576] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 543, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [577] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [578] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 545, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [579] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 548, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [580] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 550, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [581] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [582] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [583] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [584] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 553, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [585] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 557, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [586] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [587] Ref(ClassKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [588] Ref(FamilyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [589] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [590] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [591] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 559, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [592] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 563, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [593] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 563, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [594] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 563, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [595] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 563, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [596] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 563, child_count: 24, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [597] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 587, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [598] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [599] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [600] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [601] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [602] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 594, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [603] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 594, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [604] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 596, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [605] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 596, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [606] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 596, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [607] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [608] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 597, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [609] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [610] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [611] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [612] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [613] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 602, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [614] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 602, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [615] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [616] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [617] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [618] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 606, child_count: 13, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [619] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 619, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [620] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 624, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [621] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [622] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [623] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [624] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 626, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [625] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 626, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [626] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [627] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 628, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [628] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 628, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [630] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 630, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [631] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [632] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 631, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [633] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [634] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [635] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 634, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [636] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 635, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [637] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [638] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [639] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [640] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [641] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 639, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [642] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [643] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [644] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 641, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [645] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 644, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [646] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [647] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [648] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [649] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [650] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 646, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [651] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [652] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [653] Ref(ExpressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [654] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 649, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [655] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [656] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [657] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 655, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [658] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [659] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 657, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [660] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [661] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [662] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [663] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 659, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [664] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 659, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [665] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 662, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [666] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [667] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 663, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [668] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 664, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [669] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 667, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [670] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [671] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [672] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 670, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [673] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [674] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 672, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [675] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [676] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [677] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 674, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [678] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [679] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [680] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 676, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [681] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [682] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 678, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [683] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [684] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [685] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 680, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [686] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [687] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [688] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 683, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [689] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 683, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [690] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [691] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [692] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [693] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 686, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [694] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [695] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [696] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 688, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [698] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [699] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 690, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [700] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [701] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [702] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 693, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [703] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [704] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [705] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [706] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [707] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 697, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [708] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [709] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [710] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [711] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 700, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [713] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [714] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 702, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [715] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [716] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [717] Ref(TableConstraintUsingIndexSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 704, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [718] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [719] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [720] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 709, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [721] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 709, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [722] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [723] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 711, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [724] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [725] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 713, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [726] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 713, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [727] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 715, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [728] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [729] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [730] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 717, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [731] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [732] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [733] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 719, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [734] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [735] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [736] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [737] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 722, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [738] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [739] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [740] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [741] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [742] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 727, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [743] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 727, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [744] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 730, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [745] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [746] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [747] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [748] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 732, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [749] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [750] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [751] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [752] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 735, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [753] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [754] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 739, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [755] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [756] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [757] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [758] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [759] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 741, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [760] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 744, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [761] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [762] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [763] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 748, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [764] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [765] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [766] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 750, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [768] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [769] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [770] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [771] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 752, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [772] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 756, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [773] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [774] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [775] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [776] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 760, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [777] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [778] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [779] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [780] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [781] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [782] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 762, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [783] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [784] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [785] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [786] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 765, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [787] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [788] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [789] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 768, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [790] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [791] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [792] Ref(OidsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 770, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [793] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [794] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [795] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [796] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 773, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [798] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 775, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [799] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [800] Ref(LoggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [801] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 777, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [802] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [803] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [804] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 779, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [805] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [806] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [807] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 781, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [808] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [809] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [810] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [811] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 784, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [812] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [813] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [814] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 786, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [815] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [816] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 788, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [817] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 788, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [818] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [819] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [820] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 791, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [821] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [822] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [823] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [824] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [825] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 795, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [826] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [827] Ref(ReplicaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [828] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 798, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [829] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [830] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 802, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [831] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [832] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [833] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [834] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [835] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [836] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 805, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 808, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [838] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [839] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [840] Ref(COLUMN)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [841] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [842] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 811, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [843] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 814, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [844] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 816, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [845] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 819, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [846] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [847] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [848] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [849] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [850] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 822, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [851] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 822, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [852] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [853] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [854] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [855] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 826, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [856] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [857] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [858] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [859] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 831, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [860] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 831, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [861] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 834, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [862] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [863] Ref(AlterForeignTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 836, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [864] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [865] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [866] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [867] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [868] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [869] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 841, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [870] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [871] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [872] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [873] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 844, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 846, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [875] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 856, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [876] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 859, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [877] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [878] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [879] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [880] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [881] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 863, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [882] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [883] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [884] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [885] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [886] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [887] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [888] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 868, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [889] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [890] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [891] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [892] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 871, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [893] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [894] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [895] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 873, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [896] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [897] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [898] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 876, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [899] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [900] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [901] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 878, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [902] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [903] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 880, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [904] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [905] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [906] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [907] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 883, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [909] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [910] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 885, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [912] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [913] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 887, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [914] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [915] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [916] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 889, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [917] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [918] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [919] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 892, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [920] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 894, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [921] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 896, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [922] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [923] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [924] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 898, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [925] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [926] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [927] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [928] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 901, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [929] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [930] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [931] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 903, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [932] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [933] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 905, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [934] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [935] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [936] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 907, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [937] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 907, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [938] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [939] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [940] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 911, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [941] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 913, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [943] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 915, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [944] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 915, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [945] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 920, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [946] Ref(AlterFunctionActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 920, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [947] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [948] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [949] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [950] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 923, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [951] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [952] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [953] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [954] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 926, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [955] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [956] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [957] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 929, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [958] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 933, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [959] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [960] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [961] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [962] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [963] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [964] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 935, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [965] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [966] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [967] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [968] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [969] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [970] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 940, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [971] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [972] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [973] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [974] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 944, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [975] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 946, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [976] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 949, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [977] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [978] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [979] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [980] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 951, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [981] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [982] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 953, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [983] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [984] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [985] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [986] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 956, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [987] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [988] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [989] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 959, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [990] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 961, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [991] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [992] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [993] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 964, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [994] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 971, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [995] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [996] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [997] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 974, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [998] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 974, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [999] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1000] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1001] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1002] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 977, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1003] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1004] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1005] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1006] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 980, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1007] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1008] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1009] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1010] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1011] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1012] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 985, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1013] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1014] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 987, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1015] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 990, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1016] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 992, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1017] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 994, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1018] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 994, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1020] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1021] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 996, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1023] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 998, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1024] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1001, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1025] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1026] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1003, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1027] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1028] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1029] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1030] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1031] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1032] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1033] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1009, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1034] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1018, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1035] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1018, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1036] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1018, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1037] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1018, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1038] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1018, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1039] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1040] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1041] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1021, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1042] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1043] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1044] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1045] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1046] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1047] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 1023, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1048] StringParser("ALTER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1023, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1049] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1029, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1050] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1051] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1052] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1053] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1033, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1054] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1038, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1055] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1056] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1057] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1058] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1041, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1043, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1060] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1043, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1061] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1046, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1062] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1048, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1063] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1064] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1065] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1066] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1051, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1067] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1068] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1053, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1069] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1056, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1070] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1071] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1058, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1072] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1073] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1074] Ref(StorageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1061, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1075] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1076] Ref(PlainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1077] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1078] Ref(ExtendedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1079] Ref(MainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1065, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1080] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1081] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1082] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1083] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1068, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1084] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1085] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1086] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1087] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1071, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1088] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1089] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1090] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1091] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1074, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1092] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1093] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1076, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1094] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1079, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1095] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1081, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1083, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1097] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1083, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1098] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1099] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1100] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1085, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1101] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1102] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1087, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1103] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1090, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1104] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1105] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1092, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1106] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1107] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1108] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1095, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1109] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1110] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1111] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1112] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1113] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1099, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1114] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1115] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1116] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1117] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1103, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1118] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1106, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1119] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1120] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1121] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1109, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1122] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1113, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1123] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1124] Ref(AlterMaterializedViewActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1115, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1125] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1120, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1126] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1120, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1127] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1128] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1129] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1130] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1131] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1121, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1132] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1133] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1134] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1135] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1124, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1136] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1137] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1138] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1139] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1127, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1140] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1141] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1142] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1143] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1144] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1145] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1146] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1133, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1147] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1148] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1149] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1150] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1151] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1142, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1152] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1153] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1154] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1145, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1155] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1156] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1157] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1158] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1147, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1159] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1147, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1160] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1161] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1148, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1162] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1163] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1164] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1165] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1166] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1167] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1154, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1168] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1156, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1169] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1170] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1171] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1172] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1159, child_count: 3, min_times: 1, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1173] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1162, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1174] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1175] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1164, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1176] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1166, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1177] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1178] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1179] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1180] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1181] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1182] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1171, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1183] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1173, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1184] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1173, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1185] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1186] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1176, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1187] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1188] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1189] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1179, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1190] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1191] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1182, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1192] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1184, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1193] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1187, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1194] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1195] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1196] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1190, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1197] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1198] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1199] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1192, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1200] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1195, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1201] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1195, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1202] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1195, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1203] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1197, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1204] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1199, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1205] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1206] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1207] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1201, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1208] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1209] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1210] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1211] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1204, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1212] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1213] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1214] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1206, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1215] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1216] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1208, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1217] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1218] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1219] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1210, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1220] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1210, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1221] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1222] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1223] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1224] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1215, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1225] Ref(ProcedureParameterListSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1215, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1226] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1217, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1227] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1228] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1229] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1230] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1220, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1231] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1232] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1233] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1223, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1234] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1226, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1235] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1236] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1237] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1238] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1239] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1228, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1240] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1241] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1242] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1243] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1232, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1244] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1238, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1245] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1240, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1246] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1240, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1247] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1240, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1248] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1249] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1242, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1250] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1251] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1244, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1252] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1253] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1246, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1254] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1255] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1248, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1256] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1257] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1250, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1258] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1259] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1252, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1260] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1261] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1262] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1263] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1255, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1264] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1265] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1266] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1258, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1267] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1268] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1269] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1270] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1271] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1262, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1272] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1273] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1265, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1274] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1275] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1276] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1267, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1277] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1270, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1278] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1273, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1279] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1280] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1281] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1282] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1283] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1277, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1284] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1277, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1285] AnySetOf(10 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1287, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1286] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1287] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1288] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1289, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1289] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1290] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1291] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1291, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1293] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1294] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1293, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1295] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1296] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1297] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1295, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1298] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1299] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1300] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1297, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1301] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1302] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1303] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1299, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1304] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1305] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1306] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1301, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1307] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1308] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1309] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1310] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1304, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1311] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1307, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1312] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1313] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1307, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1314] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1315] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1316] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1309, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1317] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1318] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1319] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1320] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1312, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1321] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1322] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1314, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1323] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1324] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1325] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1326] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1317, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1327] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1320, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1328] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1329] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1330] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1331] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1332] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1325, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1333] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1325, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1334] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1335] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1336] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1337] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1328, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1338] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1330, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1339] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1340] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1341] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1333, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1342] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1335, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1343] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1337, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1345] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1346] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1339, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1347] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1348] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1341, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1349] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1343, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1350] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1351] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1352] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1353] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1346, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1354] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1355] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1356] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1348, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1358] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1350, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1359] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1360] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1361] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1352, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1362] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1363] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1364] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1365] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1356, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1366] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1359, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1367] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1368] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1369] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1370] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1362, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1371] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1372] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1373] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1374] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1375] Ref(QuotaGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1365, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1376] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1375, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1377] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1378] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1379] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1377, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1380] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1381] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1382] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1383] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1384] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1381, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1385] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1383, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1386] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1387] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1388] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1389] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1386, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1390] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1391] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1392] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1388, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1393] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1390, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1394] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1395] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1396] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1397] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1393, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1398] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1399] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1400] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1395, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1401] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1402] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1403] Ref(NameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1404] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1398, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1405] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1406] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1407] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1408] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1409] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1402, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1410] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1411] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1412] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1413] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1414] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1406, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1416] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1417] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1408, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1418] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1419] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1420] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1410, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1421] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1422] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1423] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1413, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1424] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1425] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1426] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1415, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1427] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1428] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1429] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1430] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1431] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1420, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1432] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1424, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1433] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1425, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1434] Ref(AlterSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1425, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1435] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1436] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1437] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1428, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1438] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1439] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1440] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1441] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1431, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1442] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1443] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1444] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1445] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1434, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1446] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1447] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1448] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1449] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1437, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1450] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1451] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1452] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1453] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1454] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1442, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1455] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1445, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1456] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1457] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1458] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1448, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1459] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1452, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1460] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1461] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1462] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1463] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1464] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1465] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1454, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1466] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1467] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1468] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1469] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1457, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1470] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1471] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1459, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1472] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1461, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1473] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1474] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1475] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1463, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1476] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1477] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1478] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1465, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1479] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1480] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1481] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1482] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1469, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1483] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1478, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1484] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1485] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1486] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1480, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1487] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1484, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1488] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1489] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1490] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1491] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1492] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1487, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1493] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1489, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1494] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1489, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1496] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1497] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1491, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1498] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1499] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1494, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1500] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1494, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1501] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1502] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1503] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1504] Ref(EnableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1505] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1496, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1506] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1507] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1508] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1498, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1510] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1500, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1511] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1512] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1513] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1514] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1505, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1515] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1516] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1517] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1508, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1518] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1519] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1520] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1521] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1522] Ref(Current_sessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1512, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1523] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1524] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1525] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1526] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1515, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1527] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1528] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1529] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1518, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1530] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1520, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1531] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1532] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1533] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1524, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1534] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1535] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1536] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1526, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1537] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1528, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1538] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1528, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1539] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1530, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1540] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1541] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1542] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1543] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1544] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1534, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1545] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1546] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1536, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1547] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1548] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1549] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1538, child_count: 16, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1550] OneOf(16 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1554, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1551] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1552] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1557, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1553] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1557, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1554] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1555] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1556] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1559, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1557] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1558] Ref(ValidateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1559] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1560] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1562, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1561] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1562] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1563] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1564] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1566, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1565] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1566, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1566] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1569, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1567] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1569, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1568] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1569, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1569] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1570, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1570] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1571] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1572] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1572, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1573] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1575, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1574] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1575, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1575] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1575, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1576] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1576, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1577] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1578] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1579] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1578, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1580] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1581] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1582] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1583] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1582, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1584] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1585] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1583, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1586] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1587] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1588] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1589] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1587, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1589, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1592] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1593] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1591, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1594] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1595] Ref(EncodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1593, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1596] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1597] Ref(ColumnEncodingGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1595, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1598] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1599] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1600] Ref(DistkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1601] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1598, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1602] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1603] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1604] Ref(DiststyleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1601, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1605] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1606] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1607] Ref(EvenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1605, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1608] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1609] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1610] Ref(DistkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1611] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1612] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1608, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1613] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1614] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1615] Ref(CompoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1616] Ref(SortkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1612, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1617] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1615, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1618] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1619] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1617, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1620] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1621] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1622] Ref(SortkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1620, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1623] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1624] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1625] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1622, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1626] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1627] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1628] Ref(EncodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1629] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1625, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1630] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1631] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1632] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1633] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1633, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1634] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1633, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1635] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1636] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1635, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1637] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1635, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1639] Ref(EncodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1637, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1640] Ref(ColumnEncodingGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1637, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1642] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1639, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1643] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1639, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1644] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1645] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1640, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1646] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1647] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1648] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1649] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1644, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1650] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1644, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1651] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1652] Ref(AppendKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1653] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1654] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1655] Ref(IgnoreextraKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1649, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1656] Ref(FilltargetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1649, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1657] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1658] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1659] Ref(LocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1660] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1652, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1661] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1662] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1663] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1664] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1665] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1656, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1666] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1660, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1667] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1668] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1663, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1669] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1663, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1670] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1671] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1672] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1665, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1673] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1669, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1674] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1675] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1676] Ref(ModifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1677] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1671, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1678] Ref(ColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1671, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1679] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1673, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1680] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1675, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1681] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1682] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1683] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1684] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1685] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1686] Ref(AlterTableDropColumnGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1677, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1687] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1680, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1688] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1680, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1689] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1690] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1691] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1692] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1682, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1693] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1694] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1695] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1685, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1696] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1688, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1697] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1698] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1699] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1700] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1693, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1701] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1693, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1702] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1696, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1703] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1704] Ref(AlterTableActionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1698, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1705] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1706] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1707] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1708] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1709] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1710] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1703, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1711] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1712] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1713] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1714] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1715] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1716] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1708, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1717] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1718] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1719] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1711, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1720] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1715, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1721] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1722] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1723] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1724] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1718, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1725] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1726] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1727] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1728] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1721, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1729] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1730] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1731] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1732] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1725, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1733] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1727, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1734] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1735] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1736] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1737] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1738] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1730, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1739] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1740] Ref(DetachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1741] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1742] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1743] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1735, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1744] Ref(FinalizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1735, child_count: 9, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1745] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1746] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1747] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1748] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1744, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1749] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1744, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1750] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1751] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1752] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1747, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1753] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1754] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1755] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1756] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1757] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1749, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1758] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1749, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1759] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1760] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1761] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1762] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1763] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1764] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1755, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1765] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1763, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1766] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1767] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1768] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1769] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1769, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1770] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1771] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1772] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1771, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1773] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1774] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1773, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1775] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1776] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1777] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1778] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1779, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1779] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1780] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1781] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1781, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1782] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1783] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1783, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1784] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1785] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1786] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1787] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1788] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1789] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1790] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1789, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1791] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1792] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1793] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1794] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1797, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1795] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1796] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1797] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1798] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1799] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1800] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1799, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1801] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1802] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1803] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1804] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1805] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1804, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1806] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1807] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1806, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1808] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1809] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1810] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1811] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1809, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1812] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1813] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1814] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1812, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1815] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1816] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1817] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1818] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1819] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1816, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1820] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1821] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1822] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1823] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1819, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1824] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1825] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1826] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1827] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1828] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1829] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1825, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1830] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1827, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1831] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1832] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1833] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1834] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1830, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1835] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1835, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1836] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1837] Ref(DependsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1838] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1839] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1840] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1835, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1841] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1842] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1843] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1844] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1839, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1845] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1845, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1846] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1847] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1848] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1848, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1849] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1850] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1851] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1852] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1853] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1852, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1854] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1855] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1856] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1857] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1858] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1859] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1857, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1860] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1861] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1862] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1863] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1860, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1864] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1865] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1866] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1867] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1863, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1868] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1865, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1869] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1869, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1870] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1871] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1872] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1873] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1875, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1874] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1875, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1875] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1876] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1877] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1877, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1878] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1877, child_count: 8, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1879] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1880] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1881] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1885, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1882] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1885, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1883] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1884] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1885] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1886] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1887, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1887] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1887, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1888] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1889] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1890] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1889, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1891] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1889, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1892] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1893] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1894] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1895] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1896] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1894, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1897] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1894, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1898] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1899] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1900] Ref(AttributeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1901] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1902] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1903] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1900, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1904] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1900, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1905] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1906] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1907] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1908] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1905, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1909] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1905, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1910] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1907, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1911] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1912] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1913] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1914] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1909, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1915] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1916] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1917] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1918] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1914, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1919] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1914, child_count: 8, min_times: 1, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1920] AnySetOf(8 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1922, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1921] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1922] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1923] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1924, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1924] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1925] Ref(CreateuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1926] Ref(NocreateuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1926, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1927] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1928] Ref(SyslogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1929] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1929, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1930] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1931] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1932] Ref(UnrestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1931, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1933] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1934] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1934, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1935] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1936] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1936, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1937] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1936, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1938] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1939] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1940] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1941] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1939, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1942] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1943] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1944] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1945] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1946] Ref(ConnectionLimitSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1942, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1947] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1944, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1948] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1949] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1950] Ref(TimeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1951] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1947, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1952] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1953] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1954] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1955] Ref(TimeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1950, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1956] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1952, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1957] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1958] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1959] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1956, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1961] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1962] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1958, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1963] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1964] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1965] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1960, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1966] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1967] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1968] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1962, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1969] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1970] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1971] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1972] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1973] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1967, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1974] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1974, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1975] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1976] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1977] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1978] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1978, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1979] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1980, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1981] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1982] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1983, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1983] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1984] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1985] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1986] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1987] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1987, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1988] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1989] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1990] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1989, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1991] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1992] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1993] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1992, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1994] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1995] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1996] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1997] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1998] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 1996, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [1999] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2000] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2001] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2002] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2003] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2004] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2001, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2005] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2006] Ref(RenameKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2007] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2008] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2004, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2009] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2010] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2011] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2012] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2007, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2013] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2014] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2009, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2015] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2012, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2016] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2014, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2017] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2016, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2018] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2016, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2020] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2021] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2018, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2022] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2020, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2023] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2020, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2024] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2023, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2025] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2025, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2026] Ref(QualifiedParameterNameGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2025, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2027] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2028] Ref(AlterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2029] Ref(DatashareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2030] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2029, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2031] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2031, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2032] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2033, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2033] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2034] Ref(AddKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2035] Ref(RemoveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2035, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2036] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2039, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2037] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2041, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2038] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2041, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2039] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2040] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2043, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2041] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2045, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2042] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2045, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2043] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2044] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2047, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2045] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2046] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2049, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2047] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2048] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2051, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2049] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2050] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2056, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2051] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2052] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2053] Ref(FunctionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2054] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2055] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2058, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2056] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2057] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2060, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2058] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2059] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2062, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2060] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2064, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2061] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2062] Ref(PublicaccessibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2063] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2064] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2067, child_count: 6, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2065] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2066] Ref(IncludenewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2067] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2068] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2069] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2070] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2071] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2072] StringParser("ALWAYS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2073] StringParser("&")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2073, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2074] StringParser("ANALYSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2073, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2075] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2076, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2076] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2077] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2078] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2078, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2079] Ref(CompressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2078, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2080] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2081, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2081] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2081, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2082] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2084, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2083] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2086, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2084] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2086, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2085] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2086] Ref(ComprowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2087] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2088, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2088] StringParser("ANALYZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2088, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2089] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2091, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2090] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2091] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2093, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2092] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2093, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2093] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2095, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2094] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2098, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2095] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2100, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2102, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2097] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2098] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2099] Ref(Skip_lockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2100] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2104, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2101] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2104, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2102] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2106, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2103] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2108, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2104] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2108, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2105] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2111, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2106] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2107] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2108] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2109] StringParser("AND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2110] StringParser("ANY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2111] StringParser("ANYELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2112] StringParser("APPEND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2113] StringParser("APPLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2114] StringParser("APPROXIMATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2113, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2115] StringParser("ARE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2113, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2116] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2117] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2118] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2119] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2116, child_count: 10, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2120] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2121] Ref(PlusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2122] Ref(MinusSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2123] Ref(DivideSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2124] Ref(MultiplySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2125] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2126] Ref(BitwiseAndSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2127] Ref(BitwiseOrSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2128] Ref(BitwiseXorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2129] Ref(BitwiseLShiftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2130] Ref(BitwiseRShiftSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2126, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2131] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2132] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2133] Ref(ArnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2134] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2130, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2135] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2132, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2136] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2137] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2138] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2139] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2140] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2141] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2137, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2142] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2143] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2144] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2145] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2146] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2147] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2142, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2148] StringParser("ARN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2142, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2149] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2143, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2150] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2144, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2151] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2147, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2152] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2153] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2154] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2155] StringParser("[")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2156] StringParser("]")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2157] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2149, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2158] StringParser("ARRAY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2149, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2159] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2152, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2160] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2161] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2162] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2154, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2163] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2157, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2164] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2157, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2165] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2166] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2167] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2168] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2169] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2170] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2171] StringParser("ARRAY_AGG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2162, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2172] StringParser("ARRAY_MAX_CARDINALITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2162, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2173] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2174] Ref(AsAliasOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2175] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2166, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2176] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2177] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2178] StringParser("AS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2179] StringParser("ASC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2180] StringParser("ASENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2181] StringParser("ASIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2182] StringParser("ASSERTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2183] StringParser("ASSIGNMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2184] StringParser("ASYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2185] StringParser("AT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2186] StringParser("ATAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2187] StringParser("ATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2188] StringParser("ATTACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2189] StringParser("ATTRIBUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2190] StringParser("ATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2191] StringParser("AUTHENTICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2192] StringParser("AUTHENTICATION_ARN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2167, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2193] StringParser("AUTHORIZATION")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2167, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2194] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2170, child_count: 3, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2195] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2173, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2197] Ref(Iam_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2175, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2198] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2199] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2200] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2177, child_count: 4, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2201] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2202] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2203] Ref(CredentialsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2204] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2205] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2181, child_count: 5, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2206] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2207] Ref(Access_key_idKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2208] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2209] Ref(Secret_access_keyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2186, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2210] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2186, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2211] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2212] Ref(Session_tokenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2188, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2213] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2188, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2214] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2190, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2215] Ref(Kms_key_idKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2190, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2216] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2190, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2218] Ref(Master_symmetric_keyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2219] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2192, child_count: 1, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2220] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2221] Ref(Auto_incrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2222] StringParser("AUTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2223] StringParser("AUTO_INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2224] StringParser("AVG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2225] StringParser("AVRO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2226] StringParser("AZ64")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2227] StringParser("BACKUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2228] StringParser("BACKWARD")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2229] MultiStringParser(9 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2193, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2230] StringParser("BASE64")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2193, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2231] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2195, child_count: 7, min_times: 0, first_terminator_idx: 12, terminator_count: 2, _padding: 0 }, // [2232] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2233] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2234] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2235] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2236] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2237] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2238] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2202, child_count: 2, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2239] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2240] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2241] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2242] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 12, terminator_count: 0, _padding: 0 }, // [2243] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2244] Ref(CompositeValueExpansionSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2245] Token("file")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2246] Token("base")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2247] StringParser("BEFORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2248] StringParser("BEGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2249] StringParser("BEGIN_FRAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2250] StringParser("BEGIN_PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2251] StringParser("BERNOULLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2252] StringParser("BETWEEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2253] StringParser("BIGINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2204, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2254] StringParser("BINARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2204, child_count: 5, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2255] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2256] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2257] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2258] Ref(BooleanBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2259] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2260] Ref(JsonOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2261] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2262] StringParser("BINARY_CLASSIFICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2263] StringParser("BINDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2264] StringParser("BIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2265] StringParser("BIT_LENGTH")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2209, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2266] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2209, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2267] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2268] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2269] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2211, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2270] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2211, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2271] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2272] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2273] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2274] StringParser("^")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2275] StringParser("BLANKSASNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2276] StringParser("BLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2277] StringParser("BLOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2278] StringParser("BOM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2213, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2279] StringParser("BOOL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2213, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2280] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2281] Ref(AndOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2282] Ref(OrOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2215, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2283] StringParser("BOOLEAN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2215, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2284] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2285] Ref(TrueSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2286] Ref(FalseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2287] StringParser("BOOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2288] StringParser("BOTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2217, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2289] StringParser("BPCHAR")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2217, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2290] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2220, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2291] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2222, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2293] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2294] Ref(MaxKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2224, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2295] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2227, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2296] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2229, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2297] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2229, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2298] Token("bracketed")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2229, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2299] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2300] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2301] StringParser("BREADTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2302] StringParser("BUFFERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2303] StringParser("BY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2304] StringParser("BYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2305] StringParser("BYTEDICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2306] StringParser("BZIP2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2232, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2307] StringParser("C")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2232, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2308] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2235, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2309] Ref(SingleIdentifierListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2235, child_count: 6, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2310] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2311] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2312] Ref(CTEColumnList)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2241, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2313] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2241, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2314] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2315] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2316] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 2243, child_count: 3, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2317] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2246, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2318] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2246, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2319] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2248, child_count: 7, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2320] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2321] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2255, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2322] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2323] Ref(BreadthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2324] Ref(DepthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2325] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2326] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2327] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2328] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2329] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2257, child_count: 6, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2330] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2331] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2332] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2333] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2334] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2335] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2336] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2337] StringParser("CACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2263, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2338] StringParser("CALL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2263, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2339] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2340] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2341] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2265, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2342] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2343] Ref(CallKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2344] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2345] StringParser("CALLED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2346] StringParser("CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2267, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2347] StringParser("CASCADE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2267, child_count: 2, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2348] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2349] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2350] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2269, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2351] StringParser("CASCADED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2269, child_count: 2, min_times: 0, first_terminator_idx: 20, terminator_count: 3, _padding: 0 }, // [2352] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2271, child_count: 6, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2353] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2354] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2277, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2355] Meta("implicit_indent")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2277, child_count: 1, min_times: 0, first_terminator_idx: 14, terminator_count: 2, _padding: 0 }, // [2356] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2357] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2358] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 14, terminator_count: 0, _padding: 0 }, // [2359] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 1, _padding: 0 }, // [2360] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 16, terminator_count: 0, _padding: 0 }, // [2361] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2362] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2278, child_count: 7, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2363] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2364] Ref(CaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2285, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2365] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2285, child_count: 1, min_times: 0, first_terminator_idx: 17, terminator_count: 2, _padding: 0 }, // [2366] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2367] Ref(WhenClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2368] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 17, terminator_count: 0, _padding: 0 }, // [2369] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(7), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 1, _padding: 0 }, // [2370] Ref(ElseClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 19, terminator_count: 0, _padding: 0 }, // [2371] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2372] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2373] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2374] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 20, terminator_count: 0, _padding: 0 }, // [2375] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2376] StringParser("CASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2377] StringParser("CASE_INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2378] StringParser("CASE_SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2379] StringParser("CAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2380] StringParser("::")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2381] StringParser("CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2382] StringParser("CATALOG_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2383] StringParser("CATALOG_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2384] StringParser("CEIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2385] StringParser("CEILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2386] StringParser("CHAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2387] StringParser("CHAINING")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2388] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2389] StringParser("CHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2390] StringParser("CHAR_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2391] StringParser("CHARACTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2392] StringParser("CHARACTER_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2393] StringParser("CHARACTER_SET_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2394] StringParser("CHARACTER_SET_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2395] StringParser("CHARACTER_SET_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2396] StringParser("CHARACTERISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2397] StringParser("CHARACTERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2398] StringParser("CHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2399] StringParser("CHECKPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2400] StringParser("CI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2401] StringParser("CLASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2402] StringParser("CLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2403] StringParser("CLASSIFIER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2404] StringParser("CLEANPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2405] StringParser("CLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2286, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2406] StringParser("CLOSE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2286, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2407] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2408] Ref(CloseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2409] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2288, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2410] StringParser("CLUSTER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2288, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2411] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2291, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2412] Ref(ClusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2291, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2413] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2291, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2414] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2293, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2415] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2295, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2416] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2295, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2417] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2418] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2419] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2297, child_count: 3, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2420] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2421] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2422] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2423] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2424] StringParser("COALESCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2425] StringParser("COBOL")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2426] Token("raw")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2427] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2300, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2428] StringParser("COLLATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2300, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2429] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2430] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2302, child_count: 4, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2431] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2432] Ref(Case_sensitiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2433] Ref(CsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2434] Ref(Case_insensitiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2435] Ref(CiKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2306, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2436] StringParser("COLLATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2306, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2437] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2308, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2438] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2308, child_count: 2, min_times: 0, first_terminator_idx: 23, terminator_count: 1, _padding: 0 }, // [2439] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2440] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2441] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 23, terminator_count: 0, _padding: 0 }, // [2442] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2443] StringParser("COLLATION_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2444] StringParser("COLLATION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2445] StringParser("COLLATION_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2446] StringParser("COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2447] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2448] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2310, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2449] StringParser(":")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2310, child_count: 7, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2450] AnySetOf(7 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2317, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2451] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2452] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2453] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2319, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2454] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2321, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2455] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2321, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2456] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2324, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2457] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2458] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2326, child_count: 6, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2459] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2460] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2461] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2462] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2463] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2332, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2464] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2332, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2465] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2335, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2466] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2467] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2337, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2468] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2469] Ref(EncodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2470] Ref(ColumnEncodingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2471] Ref(DistkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2472] Ref(SortkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2339, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2473] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2474] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2341, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2475] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2476] Ref(Case_sensitiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2477] Ref(Case_insensitiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2343, child_count: 4, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2478] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2479] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2480] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2481] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2347, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2482] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2347, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2483] AnySetOf(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2350, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2484] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2352, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2485] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2486] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2487] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2488] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2354, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2489] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2490] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2491] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2356, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2492] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2493] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2359, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2494] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2359, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2495] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2496] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2362, child_count: 3, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2497] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2498] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2365, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2499] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2365, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2500] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2501] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2367, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2502] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2503] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2504] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2369, child_count: 13, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2505] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2506] Ref(RawKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2507] Ref(Az64KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2508] Ref(BytedictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2509] Ref(DeltaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2510] Ref(Delta32kKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2511] Ref(LzoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2512] Ref(Mostly8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2513] Ref(Mostly16KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2514] Ref(Mostly32KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2515] Ref(RunlengthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2516] Ref(Text255KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2517] Ref(Text32kKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2518] Ref(ZstdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2519] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2382, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2520] StringParser("COLUMN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2382, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2521] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2384, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2522] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 2384, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2523] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2386, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2524] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2525] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2388, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2527] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2390, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2528] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2390, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 12, _padding: 0 }, // [2529] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2530] Ref(SingleIdentifierFullGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2392, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2531] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2532] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2394, child_count: 2, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2533] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2534] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2535] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2536] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2537] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2538] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2539] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2540] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2541] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2542] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2543] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2544] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2545] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 0, min_times: 0, first_terminator_idx: 24, terminator_count: 0, _padding: 0 }, // [2546] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2396, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2547] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2548] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2549] Ref(ModuloSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2550] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2551] StringParser("COLUMN_NAME")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2552] Nothing
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2553] Ref(ColumnsExpressionNameGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2554] Nothing
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2555] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2556] StringParser("COLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2557] StringParser(",")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2558] StringParser("COMMAND_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2559] StringParser("COMMAND_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2399, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2560] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2561] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2562] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2401, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2563] StringParser("COMMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2401, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2564] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2565] Ref(CommentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2566] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2404, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2567] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2406, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2568] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2417, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2419, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2570] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2571] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2572] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2573] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2421, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2575] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2423, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2576] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2426, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2577] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2578] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2579] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2580] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2429, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2581] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2582] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2583] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2431, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2584] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2585] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2586] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2434, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2587] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2588] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2589] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2590] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2437, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2592] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2593] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2439, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2594] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2595] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2596] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2441, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2597] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2598] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2599] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2444, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2600] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2601] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2445, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2602] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2603] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2604] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2447, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2606] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2607] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2449, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2608] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2452, child_count: 24, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2609] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2610] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2611] Ref(ConversionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2612] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2613] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2614] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2615] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2616] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2617] Ref(RuleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2618] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2619] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2620] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2621] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2622] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2623] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2624] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2476, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2626] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2627] Ref(MethodKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2478, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2628] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2629] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2630] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2480, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2631] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2632] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2633] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2634] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2483, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2635] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2636] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2637] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2485, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2639] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2640] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2487, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2641] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2642] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2643] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2644] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2490, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2645] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2646] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2647] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2648] Ref(DictionaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2493, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2649] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2650] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2651] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2652] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2496, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2653] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2654] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2655] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2656] Ref(TemplateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2657] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2499, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2658] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2659] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2660] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2501, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2661] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2504, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2662] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2663] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2664] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2665] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2666] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2507, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2667] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2510, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2668] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2511, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2669] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2511, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2670] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2671] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2513, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2672] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2673] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2674] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2675] Token("comment")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2676] StringParser("COMMENTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2677] StringParser("COMMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2515, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2678] StringParser("COMMITTED")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2515, child_count: 17, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2679] OneOf(17 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2680] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2681] Ref(GreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2682] Ref(LessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2683] Ref(GreaterThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2684] Ref(LessThanOrEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2685] Ref(NotEqualToSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2686] Ref(LikeOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2532, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2687] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2688] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2689] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2690] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2535, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2691] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2692] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2693] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2694] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2695] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2696] Ref(OverlapSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2697] Ref(NotExtendRightSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2698] Ref(NotExtendLeftSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2699] Ref(AdjacentSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2700] Ref(PostgisOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2701] Ref(PgvectorOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2702] Ref(PgTrgmOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2703] Ref(QualifiedOperatorSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2704] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2705] Token("binary_operator")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2539, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2706] Token("comparison_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2539, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2707] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2542, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2708] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2709] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2710] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2711] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2712] StringParser("COMPOUND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2713] StringParser("COMPRESSION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2545, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2714] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2715] Ref(Bzip2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2716] Ref(GzipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2717] Ref(LzopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2718] Ref(ZstdKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2719] StringParser("COMPROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2549, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2720] StringParser("COMPUPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2549, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2721] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2722] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2723] Ref(PipeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2724] StringParser("CONCURRENTLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2725] StringParser("CONDITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2726] StringParser("CONDITION_NUMBER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2727] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2551, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2728] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2729] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2730] Ref(ConditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2731] Ref(NonStandardJoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2732] StringParser("CONDITIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2733] StringParser("CONFIGURATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2554, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2734] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2735] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2556, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2736] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2737] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2558, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2738] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2739] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2740] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2566, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2741] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2568, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2742] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2571, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2743] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2744] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2745] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2574, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2746] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2576, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2747] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2576, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2748] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2576, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2749] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2580, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2750] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2583, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2751] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2585, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2752] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2585, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2753] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2585, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2754] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2585, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2755] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2588, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2756] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2590, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2757] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2758] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2759] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2592, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2760] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2595, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2761] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2598, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2762] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2763] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2764] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2600, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2765] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2603, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2766] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2603, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2768] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2769] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2605, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2770] StringParser("CONFLICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2605, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2771] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2607, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2772] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2609, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2773] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2612, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2774] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2614, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2775] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2617, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2776] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2777] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2620, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2778] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2779] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2623, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2780] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2623, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2781] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2625, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2782] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2625, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2783] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2625, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2784] Ref(OperationClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2625, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2785] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2786] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2787] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2627, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2788] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2789] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2790] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2791] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2792] StringParser("CONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2630, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2793] StringParser("CONNECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2630, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2794] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2795] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2796] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2633, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2797] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2798] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2799] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2800] StringParser("CONNECTION_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2801] StringParser("CONNECTION_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2802] StringParser("CONSTRAINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2803] StringParser("CONSTRAINT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2804] StringParser("CONSTRAINT_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2805] StringParser("CONSTRAINT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2806] StringParser("CONSTRAINTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2807] StringParser("CONSTRUCTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2808] StringParser("CONTAINS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2809] StringParser("CONTENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2810] StringParser("CONTINUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2811] StringParser("CONTROL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2635, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2812] StringParser("CONVERSION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2635, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2813] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2636, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2814] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2815] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2816] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2817] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2641, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2818] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2819] Ref(ConvertKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2820] StringParser("CONVERT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2642, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2821] StringParser("COPY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2642, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2822] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2823] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2824] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2825] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2826] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2827] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2648, child_count: 30, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2828] AnySetOf(30 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2678, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2829] Ref(AuthorizationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2678, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2830] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2831] Ref(RegionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2832] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2833] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2834] Ref(CompressionTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2681, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2835] Ref(DataFormatSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2681, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2836] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2683, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2837] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2838] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2839] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2840] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2686, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2841] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2842] Ref(FixedwidthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2843] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2689, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2844] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2689, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2691, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2846] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2691, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2847] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2691, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2848] Ref(ManifestKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2691, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2849] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2693, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2850] Ref(ComprowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2693, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2851] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2693, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2852] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2853] Ref(MaxerrorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2696, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2854] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2696, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2855] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2696, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2856] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2698, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2857] Ref(CompupdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2698, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2858] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2859] Ref(PresetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2860] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2861] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2862] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2703, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2863] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2703, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2864] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2705, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2865] Ref(StatupdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2705, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2866] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2867] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2868] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2869] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2870] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2871] Ref(NoloadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2872] Ref(AcceptanydateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2709, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2873] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2874] Ref(AcceptinvcharsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2875] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2876] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2712, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2877] Ref(BlanksasnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2712, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2878] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2715, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2879] Ref(DateformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2715, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2880] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2715, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2881] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2882] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2883] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2717, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2884] Ref(EmptyasnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2717, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2885] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2886] Ref(EncodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2720, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2887] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2720, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2888] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2889] Ref(Utf8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2890] Ref(Utf16KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2891] Ref(Utf16beKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2892] Ref(Utf16leKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2893] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2894] Ref(Explicit_idsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2895] Ref(FillrecordKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2896] Ref(IgnoreblanklinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2724, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2897] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2898] Ref(IgnoreheaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2727, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2899] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2727, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2900] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2727, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2901] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2902] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2903] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2730, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2904] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2730, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2906] Ref(ReadratioKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2907] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2908] Ref(RemovequotesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2732, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2909] Ref(RoundecKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2732, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2910] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2911] Ref(TimeformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2735, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2912] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2735, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2913] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2914] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2915] Ref(EpochsecsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2916] Ref(EpochmillisecsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2917] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2918] Ref(TrimblanksKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2919] Ref(TruncatecolumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2920] StringParser("CORR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2921] StringParser("CORRESPONDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2922] StringParser("COS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2923] StringParser("COSH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2924] StringParser("COST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2925] StringParser("COSTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2926] StringParser("COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2927] StringParser("COVAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2739, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2928] StringParser("COVAR_SAMP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2739, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2929] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2930] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2931] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2932] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2933] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2745, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2934] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 2748, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2935] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2936] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2748, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2937] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2938] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2939] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2753, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2940] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2941] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2942] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2943] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2758, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2944] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2761, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2945] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2946] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2947] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2948] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2765, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2949] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2765, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2950] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2951] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2952] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2767, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2953] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2954] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2769, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2955] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2769, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2956] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2771, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2958] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2773, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2959] Ref(AssignmentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2773, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2960] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2775, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2961] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2775, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2962] Ref(ImplicitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2775, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2963] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2964] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2965] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2966] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2967] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2780, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2968] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2782, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2969] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2785, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2970] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2787, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2971] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2793, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2972] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2973] Ref(LocaleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2974] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2975] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2796, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2976] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2977] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2978] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2979] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2799, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2980] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2981] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2982] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2983] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2802, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2984] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2985] Ref(ProviderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2986] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2805, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2987] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2807, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2988] Ref(IcuKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2807, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2989] Ref(LibcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2807, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2990] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2991] Ref(DeterministicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2992] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2993] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2810, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2994] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2995] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2996] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2997] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2813, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2998] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [2999] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3000] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2815, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3001] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2819, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3002] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2819, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3003] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2819, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3004] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2819, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3005] AnySetOf(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3006] Ref(FromIntegrationClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2824, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3007] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3008] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3009] Ref(CreateDatabaseWithOptionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3010] Ref(ArnCatalogSchemaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3011] Ref(IamRoleClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2826, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3012] Ref(FromDatashareClauseSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2826, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3013] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2830, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3014] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3015] Ref(OwnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3016] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3017] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3018] Ref(ConnectionLimitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3019] Ref(CollationClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3020] Ref(IsolationLevelClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2833, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3021] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3022] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3023] Ref(DatashareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2837, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3024] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2837, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3025] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3026] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3027] Ref(PublicaccessibleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2841, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3028] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2841, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3029] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3030] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3031] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2843, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3032] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3033] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3034] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2851, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3035] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2851, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3036] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3037] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2852, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3038] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2852, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3039] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3040] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2854, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3041] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2854, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3043] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2856, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3044] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2856, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3045] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2857, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3046] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2859, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3047] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3048] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3049] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2861, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3050] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2863, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3051] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3052] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3053] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2865, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3054] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3055] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3056] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2867, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3057] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3058] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3059] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3060] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3061] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2876, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3062] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2876, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3064] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2878, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3065] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2878, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3067] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2880, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3068] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2880, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3069] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3070] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3071] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2882, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3072] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2882, child_count: 14, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3073] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3074] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3075] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3076] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3077] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3078] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2896, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3079] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2899, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3080] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3081] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3082] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3083] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2901, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3084] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3085] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3086] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3087] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2904, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3088] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3089] Ref(LambdaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3090] Ref(SagemakerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3091] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3092] Ref(Iam_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2906, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3093] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3094] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2908, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3095] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2908, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3097] Ref(Retry_timeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3098] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2910, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3099] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3100] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3101] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3102] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3103] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3104] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3105] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2918, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3106] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2926, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3108] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3109] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2928, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3110] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3111] Ref(HiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3112] Ref(MetastoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3113] Ref(PostgresKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3114] Ref(MysqlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3115] Ref(KinesisKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3116] Ref(MskKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3117] Ref(RedshiftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2930, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3118] Ref(KafkaKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2930, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3119] AnySetOf(9 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2939, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3120] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3121] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3122] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2941, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3123] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3124] Ref(RegionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3125] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2943, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3126] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3127] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3128] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2945, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3129] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3130] Ref(UriKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2948, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3131] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2948, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3132] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3133] Ref(PortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3134] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2950, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3135] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3136] Ref(Iam_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2952, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3137] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3138] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3139] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2954, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3140] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2956, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3141] Ref(AuthenticationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2956, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3142] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3143] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3144] Ref(IamKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3145] Ref(MtlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2959, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3146] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2961, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3148] Ref(Authentication_arnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3149] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2963, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3151] Ref(Secret_arnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3152] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2965, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2967, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3154] Ref(Catalog_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2967, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3155] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2967, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3156] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3157] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3158] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3159] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3160] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3161] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3162] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2973, child_count: 14, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3163] Sequence(14 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3164] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3165] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3166] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3167] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2987, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3168] Ref(PartitionedBySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2987, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3169] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3170] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3171] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3172] Ref(DelimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3173] Ref(RowFormatDelimitedSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3174] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3175] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2991, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2993, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3177] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2993, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3178] Ref(TextfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2993, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3179] Ref(LocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2993, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3180] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 2993, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3181] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3182] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3183] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2996, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3184] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 2999, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3185] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3001, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3186] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3004, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3187] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3004, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3188] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3004, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3189] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3004, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3190] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3004, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3191] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3006, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3192] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3193] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3009, child_count: 13, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3194] Sequence(13 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3195] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3196] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3197] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3198] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3022, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3199] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3025, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3200] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3027, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3201] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3202] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3203] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3029, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3204] Ref(PartitionedBySegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3029, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3205] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3206] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3207] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3032, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3208] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3034, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3209] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3210] Ref(DelimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3211] Ref(RowFormatDelimitedSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3036, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3212] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3213] Ref(SerdeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3039, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3214] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3039, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3215] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3216] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3217] Ref(SerdepropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3042, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3218] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3045, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3219] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3047, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3220] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3221] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3222] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3223] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3224] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3225] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3050, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3226] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3227] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3228] Ref(RcfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3229] Ref(SequencefileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3230] Ref(TextfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3231] Ref(OrcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3232] Ref(AvroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3057, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3233] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3234] Ref(InputformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3235] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3236] Ref(OutputformatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3237] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3238] Ref(LocationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3061, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3239] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3061, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3240] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3064, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3241] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3064, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3242] Ref(PropertiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3064, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3243] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3067, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3244] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3069, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3245] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3246] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3247] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3248] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3072, child_count: 11, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3249] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3250] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3251] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3252] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3253] Ref(HandlerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3254] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3255] Ref(ValidatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3256] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3083, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3257] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3258] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3259] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3260] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3086, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3261] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3088, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3262] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3263] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3264] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3095, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3265] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3095, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3266] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3098, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3267] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3100, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3268] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3102, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3269] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3270] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3271] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3107, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3272] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3107, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3273] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3274] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3109, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3275] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3109, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3276] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3110, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3277] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3110, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3278] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3110, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3279] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3112, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3280] Ref(InheritsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3112, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3281] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3115, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3282] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3283] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3117, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3284] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3285] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3286] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3119, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3287] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3119, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3288] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3125, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3289] Ref(CreateForeignTableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3125, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3290] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3125, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3291] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3125, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3292] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3293] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3294] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3130, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3295] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3130, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3296] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3133, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3297] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3135, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3298] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3137, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3299] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3140, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3300] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3140, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3301] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3142, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3302] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3142, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3303] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3142, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3304] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3305] Ref(ForeignTableColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3306] Ref(ForeignTableTableConstraintSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3143, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3307] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3145, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3308] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3309] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3310] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3311] Ref(PartitionBoundSpecSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3312] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3148, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3313] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3314] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3315] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3150, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3316] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3150, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3317] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3318] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3319] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3320] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3321] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3322] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3323] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3159, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3324] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3159, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3325] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3326] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3161, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3327] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3164, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3328] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3329] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3166, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3330] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3169, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3331] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3171, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3332] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3333] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3173, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3335] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3175, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3336] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3337] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3338] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3177, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3339] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3340] Ref(SetofKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3341] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3342] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3343] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3179, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3344] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3345] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3346] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3183, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3347] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3183, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3348] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3186, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3349] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3186, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3350] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3186, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3351] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3188, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3352] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3188, child_count: 15, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3353] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3203, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3354] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3203, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3355] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3203, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3356] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3203, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3357] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3203, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3358] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3205, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3359] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3205, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3360] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3205, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3361] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3205, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3362] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3205, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3363] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3205, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3364] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3207, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3365] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3207, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3366] Ref(IndexAccessMethodSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3207, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3367] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3210, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3368] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3212, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3369] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3212, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3370] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3214, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3371] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3214, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3372] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3217, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3373] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3219, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3374] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3219, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3375] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3376] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3377] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3222, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3378] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3222, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3379] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3380] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3224, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3381] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3224, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3382] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3226, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3383] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3226, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3384] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3226, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3385] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3386] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3387] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3228, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3388] StringParser("CREATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3228, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3389] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3390] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3391] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3392] Ref(LibraryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3393] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3394] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3395] Ref(PlpythonuKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3396] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3237, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3397] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3237, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3398] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3239, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3399] Ref(AuthorizationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3239, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3400] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3401] Ref(RegionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3402] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3403] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3242, child_count: 10, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3404] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3405] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3406] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3407] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3252, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3408] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3252, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3409] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3410] Ref(BackupKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3254, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3411] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3256, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3412] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3256, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3413] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3256, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3414] Ref(TableAttributeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3256, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3415] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3416] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3417] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3259, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3418] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3261, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3419] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3261, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3420] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3261, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3421] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3261, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3422] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3265, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3423] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3267, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3424] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3425] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3270, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3426] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3272, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3427] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3275, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3428] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3429] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3430] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3431] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3277, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3432] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3279, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3433] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3282, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3434] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3435] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3436] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3284, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3437] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3284, child_count: 16, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3438] Sequence(16 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3300, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3439] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3300, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3440] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3300, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3441] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3300, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3442] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3302, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3443] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3302, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3444] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3445] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3305, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3446] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3308, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3447] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3308, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3448] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3308, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3449] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3450] Ref(TargetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3451] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3310, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3452] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3453] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3313, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3454] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3313, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3455] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3316, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3456] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3318, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3457] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3318, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3458] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3459] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3320, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3460] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3320, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3461] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3322, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3462] Ref(SagemakerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3322, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3463] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3322, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3464] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3465] Ref(Iam_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3324, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3466] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3467] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3326, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3468] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3326, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3469] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3470] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3328, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3471] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3472] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3330, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3473] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3330, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3474] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3475] Ref(Model_typeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3332, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3476] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3477] Ref(XgboostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3478] Ref(MlpKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3335, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3479] Ref(KmeansKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3335, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3480] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3481] Ref(Problem_typeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3337, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3482] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3483] Ref(RegressionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3484] Ref(Binary_classificationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3340, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3485] Ref(Multiclass_classificationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3340, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3486] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3487] Ref(ObjectiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3342, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3488] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3342, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3489] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3490] Ref(PreprocessorsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3344, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3491] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3344, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3492] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3347, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3493] Ref(HyperparametersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3347, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3494] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3347, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3495] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3496] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3349, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3497] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3352, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3498] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3354, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3499] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3354, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3500] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3501] Ref(SettingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3356, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3502] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3359, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3503] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3365, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3504] Ref(S3_bucketKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3365, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3505] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3365, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3506] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3367, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3507] Ref(Kms_key_idKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3367, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3508] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3367, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3509] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3510] Ref(S3_garbage_collectKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3369, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3511] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3512] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3371, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3513] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3371, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3514] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3515] Ref(Max_cellsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3373, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3516] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3373, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3517] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3375, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3518] Ref(Max_runtimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3375, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3519] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3375, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3520] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3379, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3521] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3379, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3522] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3379, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3523] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3380, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3524] RegexParser
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3380, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3525] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3383, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3526] Delimited(6 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3385, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3527] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3391, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3528] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3394, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3529] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3530] Ref(LeftargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3531] Ref(RightargKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3532] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3396, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3533] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3396, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3534] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3535] Ref(CommutatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3399, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3536] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3399, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3537] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3400, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3538] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3400, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3539] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3403, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3540] Ref(NegatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3403, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3541] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3403, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3542] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3404, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3543] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3404, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3544] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3407, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3545] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3546] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3547] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3410, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3548] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3549] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3550] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3551] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3552] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3553] Ref(HashesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3412, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3554] Ref(MergesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3412, child_count: 10, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3555] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3556] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3557] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3558] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3559] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3422, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3560] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3422, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3561] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3424, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3562] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3424, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3563] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3564] Ref(PermissiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3426, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3565] Ref(RestrictiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3426, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3566] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3567] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3428, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3568] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3569] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3570] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3571] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3572] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3433, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3573] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3433, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3574] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3575] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3435, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3576] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3437, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3577] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3442, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3578] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3442, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3579] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3442, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3580] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3442, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3581] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3442, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3582] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3442, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3583] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3444, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3584] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3444, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3585] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3447, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3586] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3447, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3587] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3588] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3589] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3450, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3590] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3453, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3591] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3453, child_count: 7, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3592] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3593] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3594] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3595] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3596] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3597] Ref(ProcedureParameterListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3598] Ref(NonatomicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3460, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3599] Ref(FunctionDefinitionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3460, child_count: 5, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3600] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3465, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3601] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3465, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3602] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3465, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3603] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3465, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3604] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3467, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3605] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3606] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3607] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3608] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3470, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3609] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3610] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3472, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3611] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3474, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3612] Ref(PublicationObjectsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3474, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3613] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3614] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3476, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3615] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3476, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3616] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3617] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3618] Ref(RlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3619] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3482, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3620] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3482, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3621] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3485, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3622] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3485, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3623] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3488, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3624] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3490, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3492, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3626] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3492, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3627] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3492, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3628] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3494, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3629] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3494, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3630] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3494, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3631] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3496, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3632] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3496, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3633] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3634] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3499, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3635] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3636] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3503, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3637] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3638] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3639] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3505, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3640] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3505, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3507, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3642] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3507, child_count: 16, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3643] AnySetOf(16 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3523, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3644] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3645] Ref(SuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3646] Ref(NosuperuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3525, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3647] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3648] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3649] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3527, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3650] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3651] Ref(CreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3652] Ref(NocreateroleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3529, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3653] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3654] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3531, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3655] Ref(NoinheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3531, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3656] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3657] Ref(LoginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3658] Ref(NologinKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3533, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3659] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3660] Ref(ReplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3661] Ref(NoreplicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3535, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3663] Ref(BypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3537, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3664] Ref(NobypassrlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3537, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3665] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3666] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3667] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3668] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3540, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3669] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3543, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3670] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3671] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3543, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3672] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3673] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3545, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3674] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3545, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3675] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3676] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3677] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3678] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3548, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3679] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3680] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3681] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3682] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3551, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3683] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3684] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3685] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3554, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3686] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3554, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3687] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3556, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3688] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3556, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3689] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3556, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3690] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3558, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3691] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3558, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3692] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3558, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3693] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3694] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3695] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3560, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3697] Ref(SysidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3562, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3698] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3562, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3699] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3566, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3700] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3566, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3701] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3566, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3702] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3568, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3703] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3704] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3571, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3705] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3571, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3707] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3708] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3573, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3709] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3710] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3711] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3575, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3712] Ref(QuotaGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3575, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3713] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3583, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3714] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3715] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3585, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3716] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3585, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3717] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3718] Ref(IncrementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3719] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3720] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3721] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3589, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3722] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3591, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3723] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3724] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3594, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3725] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3726] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3594, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3727] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3596, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3728] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3596, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3729] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3596, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3730] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3598, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3731] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3732] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3733] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3734] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3601, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3735] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3603, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3736] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3603, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3737] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3603, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3738] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3739] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3740] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3741] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3742] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3607, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3743] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3744] Ref(CacheKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3609, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3745] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3609, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3746] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3611, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3747] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3611, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3748] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3613, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3749] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3613, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3750] Ref(CycleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3613, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3751] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3752] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3753] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3616, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3754] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3755] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3756] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3618, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3757] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3758] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3759] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3760] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3761] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3624, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3762] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3624, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3763] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3625, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3764] Ref(CreateSequenceOptionsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3625, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3765] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3766] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3767] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3768] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3634, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3769] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3634, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3771] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3636, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3772] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3636, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3773] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3774] Ref(VersionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3775] Ref(VersionIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3776] Ref(ForeignDataWrapperGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3777] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3638, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3778] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3638, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3779] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3780] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3646, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3781] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3646, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3782] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3783] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3648, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3784] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3648, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3785] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3651, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3786] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3653, child_count: 4, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3787] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3788] Ref(DependenciesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3789] Ref(McvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3790] Ref(NdistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3791] Ref(CorrelationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3792] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3657, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3793] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3659, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3794] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3795] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3796] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3797] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3798] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3661, child_count: 8, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3799] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3800] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3801] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3802] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3803] Ref(ConnectionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3804] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3805] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3669, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3806] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3671, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3807] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3671, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3808] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3809] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3810] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3673, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3811] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3682, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3812] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3682, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3813] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3684, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3814] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3684, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3815] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3816] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3817] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3818] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3686, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3819] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3686, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3820] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3689, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3821] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3691, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3822] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3691, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3823] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3693, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3824] Ref(BackupKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3693, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3825] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3826] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3827] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3828] Ref(TableAttributeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3829] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3695, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3830] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3697, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3831] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3700, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3832] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3700, child_count: 9, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3833] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3834] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3835] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3836] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3837] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3838] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3709, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3839] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3709, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3840] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3712, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3841] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3714, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3842] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3717, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3843] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3844] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3720, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3845] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3720, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3846] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3847] Ref(ColumnAttributeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3848] Ref(ColumnConstraintSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3849] Ref(TableConstraintSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3722, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3850] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3725, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3851] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3725, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3852] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3725, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3853] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3726, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3854] Ref(LikeOptionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3726, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3855] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3728, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3856] Ref(BackupKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3728, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3857] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3730, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3858] Ref(YesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3730, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3859] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3730, child_count: 1, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3860] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3731, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3861] Ref(TableAttributeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3731, child_count: 6, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3862] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3863] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3864] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3865] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3866] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3867] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3737, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3868] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3740, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3869] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3742, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3870] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3871] Ref(ParserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3872] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3745, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3873] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3745, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3874] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3875] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3876] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3877] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3748, child_count: 11, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3878] Sequence(11 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3879] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3880] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3881] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3882] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3883] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3759, child_count: 3, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3884] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3885] Ref(BeforeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3886] Ref(AfterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3762, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3887] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3888] Ref(InsteadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3889] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3764, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3890] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3766, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3891] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3892] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3893] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3894] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3770, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3895] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3772, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3896] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3772, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3897] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3774, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3898] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3774, child_count: 2, min_times: 0, first_terminator_idx: 36, terminator_count: 2, _padding: 0 }, // [3899] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3900] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3901] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 36, terminator_count: 0, _padding: 0 }, // [3902] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3903] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3904] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3776, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3905] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3776, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3906] AnyNumberOf(5 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3781, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3907] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3908] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3909] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3783, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3910] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3785, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3911] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3912] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3913] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3787, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3914] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3789, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3915] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3789, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3916] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3791, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3917] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3918] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3919] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3793, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3920] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3921] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3922] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3795, child_count: 6, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3923] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3801, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3924] Ref(ReferencingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3801, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3925] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3926] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3927] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3928] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3929] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3803, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3930] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3803, child_count: 4, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3931] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3807, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3932] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3933] Ref(OldKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3934] Ref(NewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3935] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3936] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3937] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3809, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3938] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3812, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3939] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3812, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3940] Ref(EachKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3812, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3941] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3942] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3943] Ref(StatementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3814, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3944] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3945] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3816, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3946] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3947] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3819, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3948] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3949] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3822, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3950] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3951] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3952] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3953] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3824, child_count: 5, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3954] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3955] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3956] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3829, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3957] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3829, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3958] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3831, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3959] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3831, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3833, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3961] Ref(EnumKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3833, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3962] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3833, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3963] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3836, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3964] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3838, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3965] Anything
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3838, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3966] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3841, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3967] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3841, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3968] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3841, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3969] Ref(MappingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3841, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3970] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3848, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3971] Ref(CreateUserMappingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3848, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3972] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3848, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3973] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3848, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3974] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3975] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3976] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3977] Ref(SessionInformationUserFunctionsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3978] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3979] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3851, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3980] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3851, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3981] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3982] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3983] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3984] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3985] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3986] Ref(PasswordKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3858, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3987] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3860, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3988] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3860, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3989] Ref(DisableKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3860, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3990] AnySetOf(7 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3867, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3991] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3869, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3992] Ref(CreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3869, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3993] Ref(NocreatedbKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3869, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3994] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3871, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3995] Ref(CreateuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3871, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3996] Ref(NocreateuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3871, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3997] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3998] Ref(SyslogKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [3999] Ref(AccessKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3874, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4000] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4001] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4002] Ref(UnrestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3876, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4003] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3879, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4004] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3879, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4005] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3879, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4006] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4007] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3881, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4008] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4009] Ref(ValidKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4010] Ref(UntilKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4011] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4012] Ref(ConnectionLimitSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3884, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4013] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4014] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4015] Ref(TimeoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4016] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3887, child_count: 9, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4017] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4018] Ref(CreateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4019] Ref(OrReplaceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4020] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4021] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4022] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4023] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4024] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3896, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4025] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3898, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4026] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4027] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4028] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4029] StringParser("CREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4030] StringParser("CREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4031] StringParser("CREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4032] StringParser("CREDENTIALS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4033] StringParser("CROSS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4034] StringParser("CS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4035] StringParser("CSV")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4036] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3901, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4037] StringParser("CUBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3901, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4038] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3903, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4040] Ref(CubeFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4041] Ref(RollupFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3905, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4042] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4043] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4044] StringParser("CUME_DIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4045] StringParser("CURRENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4046] StringParser("CURRENT_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4047] StringParser("CURRENT_DATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4048] StringParser("CURRENT_DEFAULT_TRANSFORM_GROUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4049] StringParser("CURRENT_PATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4050] StringParser("CURRENT_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4051] StringParser("CURRENT_ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4052] StringParser("CURRENT_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4053] StringParser("CURRENT_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4054] StringParser("CURRENT_TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4055] StringParser("CURRENT_TRANSFORM_GROUP_FOR_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4056] StringParser("CURRENT_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4057] StringParser("CURRENT_USER_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4058] StringParser("CURSOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4059] StringParser("CURSOR_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3908, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4060] StringParser("CYCLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3908, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4061] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3910, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4063] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3912, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4064] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3912, child_count: 7, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4065] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3919, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3921, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4067] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3921, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4068] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4069] Ref(QuoteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3924, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4070] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4071] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3924, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4072] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3926, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4073] Ref(ShapefileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3926, child_count: 3, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4074] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3929, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4075] Ref(SimplifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3929, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4076] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3929, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4077] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3929, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4078] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3931, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4079] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4080] Ref(AvroKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3933, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4081] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3933, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4082] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4083] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4084] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4085] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4086] Ref(OrcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4087] Ref(RcfileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4088] Ref(SequencefileKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4089] StringParser("DATA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4090] StringParser("DATA_CATALOG_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3935, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4091] StringParser("DATABASE")
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3935, child_count: 2, min_times: 0, first_terminator_idx: 38, terminator_count: 1, _padding: 0 }, // [4092] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4093] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4094] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 38, terminator_count: 0, _padding: 0 }, // [4095] Ref(ObjectReferenceTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4096] StringParser("DATALINK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4097] StringParser("DATASHARE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3937, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4098] StringParser("DATASHARES")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3937, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3939, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4100] RegexParser
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 3939, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4101] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4102] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3940, child_count: 24, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4103] OneOf(24 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4104] Ref(SmallintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4105] Ref(Int2KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4106] Ref(IntegerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4107] Ref(IntKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4108] Ref(Int4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4109] Ref(BigintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4110] Ref(Int8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4111] Ref(RealKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4112] Ref(Float4KeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3964, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4113] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4114] Ref(DoubleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4115] Ref(PrecisionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4116] Ref(Float8KeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4117] Ref(FloatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3966, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4118] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3968, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4119] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4120] Ref(DecimalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4121] Ref(NumericKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3970, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4122] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3970, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4123] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3973, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4124] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3975, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4125] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4126] Ref(CharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4127] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4128] Ref(NcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4129] Ref(VarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3981, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4130] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4131] Ref(CharacterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4132] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4133] Ref(NvarcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4134] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4135] Ref(BpcharKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4136] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4137] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4138] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3983, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4139] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4140] Ref(BooleanKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4141] Ref(BoolKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4142] Ref(HllsketchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4143] Ref(SuperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4144] Ref(GeometryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4145] Ref(GeographyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3985, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4146] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3987, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4147] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3990, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4148] Ref(VarbyteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3990, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4149] Ref(VarbinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3990, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3992, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4151] Ref(BinaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3992, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4152] Ref(VaryingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 3992, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4153] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3992, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4154] Ref(AnyelementKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3992, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4155] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4156] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4157] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4158] Ref(DatatypeIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3995, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4159] StringParser("DATE")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 3995, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4160] MultiStringParser(4 templates)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4161] Ref(DatePartFunctionName)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3995, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4162] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3996, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4163] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 3999, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4164] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4001, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4165] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4166] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4003, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4167] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4003, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4168] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4169] Ref(DateTimeTypeIdentifier)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4170] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4005, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4171] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4172] Ref(DateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4173] Ref(DatetimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4174] Ref(TimeWithTZGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4009, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4175] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4176] Ref(TimetzKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4177] Ref(TimestamptzKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4178] StringParser("DATEFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4179] StringParser("DATETIME")
    GrammarInst { variant: GrammarVariant::MultiStringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4180] MultiStringParser(80 templates)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4181] StringParser("DATETIME_INTERVAL_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4182] StringParser("DATETIME_INTERVAL_PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4183] StringParser("DAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4184] StringParser("DAYOFYEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4185] StringParser("DB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4186] StringParser("DBUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4011, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4187] StringParser("DEALLOCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4011, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4188] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4189] Ref(DeallocateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4190] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4191] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4192] StringParser("DEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4193] StringParser("DECFLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4194] StringParser("DECIMAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4014, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4195] StringParser("DECLARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4014, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4196] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4197] Ref(DeclareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4198] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4199] Ref(CursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4200] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4201] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4019, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4202] StringParser("DEFAULT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4019, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4203] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4204] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4205] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4206] StringParser("DEFAULTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4207] StringParser("DEFERRABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4208] StringParser("DEFERRED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4209] StringParser("DEFINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4210] StringParser("DEFINED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4021, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4211] StringParser("DEFINER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4021, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4212] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4213] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4214] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4023, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4215] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4025, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4216] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4025, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4217] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4218] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4219] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4027, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4220] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4030, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4221] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4222] Ref(DefinitionParameterSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4223] StringParser("DEFLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4224] StringParser("DEFRAG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4225] StringParser("DEGREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4032, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4226] StringParser("DELETE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4032, child_count: 10, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4227] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4228] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4229] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4230] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4231] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4232] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4042, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4233] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4042, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4234] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4046, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4235] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4046, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4236] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4048, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4237] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4238] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4239] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4050, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4240] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4050, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4241] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4052, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4242] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4243] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4244] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4245] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4246] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4056, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4247] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4056, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4248] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4249] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4060, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4250] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4251] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4062, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4252] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4064, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4253] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4254] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4255] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4256] StringParser("DELIMITED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4257] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4258] StringParser("DELIMITER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4259] StringParser("DELIMITERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4260] StringParser("DELTA32K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4261] StringParser("DELTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4262] StringParser("DENSE_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4263] StringParser("DEPENDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4264] StringParser("DEPTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4265] StringParser("DEREF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4066, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4266] StringParser("DERIVED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4066, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4267] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4268] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4269] Ref(DatashareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4070, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4270] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4070, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4271] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4074, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4272] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4074, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4273] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4274] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4275] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4276] Ref(NamespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4277] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4278] StringParser("DESC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4076, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4279] StringParser("DESCRIBE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4076, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4280] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4281] Ref(DescribeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4282] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4283] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4284] StringParser("DESCRIPTOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4285] StringParser("DETACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4286] StringParser("DETERMINISTIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4287] StringParser("DIAGNOSTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4288] StringParser("DICTIONARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4289] StringParser("DISABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4079, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4290] StringParser("DISCARD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4079, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4291] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4081, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4292] Ref(DiscardKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4081, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4293] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4294] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4295] Ref(PlansKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4296] Ref(SequencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4297] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4298] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4299] StringParser("DISCONNECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4300] StringParser("DISPATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4301] StringParser("DISTINCT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4302] StringParser("DISTKEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4303] StringParser("DISTSTYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4304] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4305] StringParser("DLNEWCOPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4306] StringParser("DLPREVIOUSCOPY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4307] StringParser("DLURLCOMPLETE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4308] StringParser("DLURLCOMPLETEONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4309] StringParser("DLURLCOMPLETEWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4310] StringParser("DLURLPATH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4311] StringParser("DLURLPATHONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4312] StringParser("DLURLPATHWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4313] StringParser("DLURLSCHEME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4314] StringParser("DLURLSERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4315] StringParser("DLVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4086, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4316] StringParser("DO")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4086, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4317] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4318] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4088, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4319] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4090, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4320] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4321] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4322] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4092, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4323] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4324] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4325] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4326] StringParser("DOCUMENT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4327] TypedParser("dollar_numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4328] StringParser("DOMAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4329] StringParser(".")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4094, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4330] StringParser("DOUBLE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4094, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4331] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4099, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4332] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4099, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4333] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4099, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4334] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4099, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4335] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4101, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4336] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4337] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4103, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4338] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4339] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Anything, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4340] Anything
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4341] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4106, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4342] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4106, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4343] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4108, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4344] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4108, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4345] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4108, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4346] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4347] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4348] Ref(CastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4113, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4349] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4113, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4350] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4118, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4351] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4118, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4352] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4118, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4353] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4118, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4354] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4118, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4355] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4123, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4356] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4123, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4357] Ref(CollationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4123, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4358] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4123, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4359] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4123, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4360] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4123, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4361] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4128, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4362] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4128, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4363] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4128, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4364] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4128, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4365] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4128, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4366] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4130, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4367] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4130, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4368] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4133, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4369] Ref(ForceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4133, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4370] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4371] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4372] Ref(DatashareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4373] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4136, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4374] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4141, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4375] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4141, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4376] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4141, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4377] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4141, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4378] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4143, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4379] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4143, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4380] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4143, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4381] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4148, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4382] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4148, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4383] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4148, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4384] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4148, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4385] Ref(ExtensionReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4148, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4386] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4148, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4387] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4388] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4389] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4390] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4154, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4391] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4154, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4392] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4156, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4393] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4156, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4394] Ref(CascadeRestrictGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4156, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4395] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4161, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4396] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4161, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4397] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4161, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4398] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4161, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4399] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4163, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4400] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4165, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4401] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4165, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4402] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4165, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4403] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4165, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4404] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4405] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4406] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4407] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4171, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4408] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4171, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4409] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4173, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4410] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4173, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4411] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4173, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4412] StringParser("DROP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4173, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4413] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4414] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4415] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4416] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4179, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4417] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4179, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4418] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4419] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4181, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4420] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4181, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4421] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4422] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4423] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4424] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4425] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4185, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4426] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4427] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4428] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4429] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4190, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4430] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4192, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4431] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4432] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4433] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4434] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4435] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4196, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4436] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4196, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4437] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4438] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4439] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4440] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4441] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4442] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4443] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4203, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4444] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4203, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4445] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4207, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4446] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4207, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4447] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4207, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4448] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4207, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4449] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4209, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4450] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4451] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4211, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4452] Ref(ProcedureParameterListSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4211, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4453] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4454] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4455] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4216, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4456] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4216, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4457] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4458] Ref(PublicationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4218, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4459] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4218, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4460] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4461] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4462] Ref(RlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4463] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4464] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4224, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4465] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4224, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4466] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4467] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4468] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4226, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4469] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4230, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4470] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4230, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4471] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4230, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4472] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4230, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4473] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4230, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4474] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4475] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4476] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4477] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4478] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4235, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4479] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4235, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4480] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4481] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4482] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4240, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4483] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4240, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4484] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4485] Ref(SequenceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4242, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4486] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4242, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4487] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4247, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4488] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4247, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4489] Ref(StatisticsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4247, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4490] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4247, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4491] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4249, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4492] Ref(StatisticsReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4249, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4493] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4494] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4251, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4495] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4251, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4496] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4497] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4498] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4499] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4256, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4500] Ref(SubscriptionReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4256, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4501] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4258, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4502] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4258, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4503] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4258, child_count: 6, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4504] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4505] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4506] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4507] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4264, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4508] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4264, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4509] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4510] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4266, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4511] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4266, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4512] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4513] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4514] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4515] Ref(SearchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4516] Ref(ConfigurationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4517] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4518] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4273, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4519] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4273, child_count: 7, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4520] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4521] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4522] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4523] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4524] Ref(TriggerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4525] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4526] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4280, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4527] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4280, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4528] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4285, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4529] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4285, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4530] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4285, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4531] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4285, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4532] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4533] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4287, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4534] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4287, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4535] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4536] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4537] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4291, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4538] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4539] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4291, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4540] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4541] Ref(DropKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4542] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4296, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4543] Ref(IfExistsGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4296, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4544] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4545] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4546] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4547] StringParser("DUPLICATES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4548] StringParser("DYNAMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4549] StringParser("DYNAMIC_FUNCTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4550] StringParser("DYNAMIC_FUNCTION_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4551] StringParser("EACH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4298, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4552] StringParser("ELEMENT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4298, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4553] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4302, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4554] Ref(ElseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4302, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4555] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4302, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4556] StringParser("ELSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4302, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4557] StringParser("EMPTY")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4302, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4558] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4304, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4559] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4560] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4561] Ref(EmptyStructLiteralBracketsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4562] StringParser("EMPTYASNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4563] StringParser("ENABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4564] StringParser("ENCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4565] StringParser("ENCODING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4566] StringParser("ENCRYPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4567] StringParser("ENCRYPTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4568] StringParser("ENCRYPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4569] StringParser("END-EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4570] StringParser("}")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4571] StringParser("END")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4572] StringParser("END_FRAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4573] StringParser("END_PARTITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4574] StringParser("ENFORCED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4575] StringParser("ENUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4576] StringParser("EPOCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4577] StringParser("EPOCHMILLISECS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4578] StringParser("EPOCHSECS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4579] StringParser("EQUALS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4580] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4581] StringParser("ERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4582] StringParser("ESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4583] StringParser("EVEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4584] StringParser("EVENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4585] StringParser("EVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4586] StringParser("EXCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4306, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4587] StringParser("EXCEPTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4306, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4588] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4308, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4589] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4308, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4590] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4310, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4591] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4313, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4592] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4593] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4594] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4595] StringParser("EXCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4315, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4596] StringParser("EXCLUDING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4315, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4597] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4318, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4598] Ref(IndexElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4318, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4599] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4318, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4600] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4318, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4601] StringParser("EXCLUSIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4318, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4602] StringParser("EXEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4318, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4603] StringParser("EXECUTE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4318, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4604] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4321, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4605] Ref(ExecuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4321, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4606] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4321, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4607] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4324, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4608] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4609] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4610] StringParser("EXECUTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4611] StringParser("EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4612] StringParser("EXP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4326, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4613] StringParser("EXPLAIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4326, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4614] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4328, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4615] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4330, child_count: 9, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4616] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4617] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4618] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4619] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4620] Ref(CostsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4621] Ref(SettingsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4622] Ref(BuffersKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4623] Ref(WalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4624] Ref(TimingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4625] Ref(SummaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4339, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4626] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4339, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4627] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4341, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4628] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4341, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4629] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4630] Ref(TextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4631] Ref(XmlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4632] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4633] Ref(YamlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4345, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4634] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4348, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4635] Ref(ExplainKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4348, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4636] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4350, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4637] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4352, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4638] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4639] Ref(AnalyzeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4640] Ref(AnalyseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4354, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4641] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4354, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4642] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4357, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4643] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4359, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4644] Ref(ExplainOptionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4359, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4645] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4646] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4647] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4648] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4649] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4650] StringParser("EXPLICIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4651] StringParser("EXPLICIT_IDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4652] StringParser("EXPRESSION")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4363, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4653] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4363, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4654] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4365, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4655] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4365, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4656] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4366, child_count: 10, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4657] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4376, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4658] Ref(LikeExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4376, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4659] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4660] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4661] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4662] Ref(InOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4663] Ref(QuantifiedComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4378, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4664] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4665] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4666] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4667] Ref(IsClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4668] Ref(IsNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4669] Ref(NotNullGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4670] Ref(CollateGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4381, child_count: 5, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4671] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4672] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4673] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4674] Ref(Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4675] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4676] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4386, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4677] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4388, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4678] Ref(PatternMatchingGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4388, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4679] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4388, child_count: 4, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4680] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4392, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4681] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4393, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4682] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4683] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4684] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4685] Ref(NotOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4686] Ref(PriorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4394, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4687] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4396, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4688] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4396, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4689] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4397, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4690] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4398, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4400, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4692] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4693] Ref(ArithmeticBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4694] Ref(StringBinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4695] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4696] Ref(Tail_Recurse_Expression_B_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4403, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4697] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4405, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4698] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4406, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4699] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4700] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4701] Ref(TildeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4407, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4702] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4409, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4703] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4409, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4704] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4411, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 1, _padding: 0 }, // [4705] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4414, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4706] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4416, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4707] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4416, child_count: 3, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4708] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4419, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4709] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4419, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4710] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4421, child_count: 2, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4711] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4423, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4712] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4423, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4713] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4423, child_count: 1, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4714] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4715] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4716] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 0, min_times: 0, first_terminator_idx: 39, terminator_count: 0, _padding: 0 }, // [4717] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4424, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4718] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4426, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4719] Ref(FullTextSearchOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4426, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4720] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4426, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4721] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4428, child_count: 9, min_times: 0, first_terminator_idx: 40, terminator_count: 1, _padding: 0 }, // [4722] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4437, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4723] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4437, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4724] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 4437, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4725] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4440, child_count: 4, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4726] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4444, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4727] Ref(LateralColumnAliasExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4444, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4728] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4444, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4729] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4444, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4730] Delimited(5 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4446, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4731] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4732] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4733] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4734] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4735] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4736] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4737] Ref(Expression_D_Potential_Select_Statement_Without_Brackets)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4738] Ref(SingleIdentifierWildcardSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4451, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4739] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4453, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4740] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4455, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4741] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4455, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4742] Ref(MapTypeSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4455, child_count: 3, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4743] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4458, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4744] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4745] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4460, child_count: 2, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4746] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4462, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4747] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4462, child_count: 5, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4748] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4749] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4750] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4751] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4752] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4753] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4754] Ref(LocalAliasSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4755] Ref(ListComprehensionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 40, terminator_count: 0, _padding: 0 }, // [4756] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4467, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4757] Ref(AccessorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4467, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4758] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4759] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4760] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4761] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4762] Ref(TypedStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4763] Ref(ArrayExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4764] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4765] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4766] StringParser("EXTENDED")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4474, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4767] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4474, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4768] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4769] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4477, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4770] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4771] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4479, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4772] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4481, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4773] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4484, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4774] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4484, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4775] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4776] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4777] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4778] StringParser("EXTENSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4779] StringParser("EXTERNAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4780] StringParser("EXTRACT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4781] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4782] StringParser("FALSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4486, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4783] StringParser("FAMILY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4486, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4784] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4491, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4785] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4491, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4786] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4493, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4787] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4493, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4788] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4493, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4789] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4495, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4790] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 4495, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4791] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4792] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4496, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4793] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4794] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4795] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4498, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4796] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4797] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4500, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4798] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4502, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4799] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4502, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4800] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4502, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4801] StringParser("FETCH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4502, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4802] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4803] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4506, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4804] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4805] Ref(NextKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4806] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4509, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4808] Ref(ForwardKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4511, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4809] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4810] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4811] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4812] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4813] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4814] StringParser("FIELDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4513, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4815] StringParser("FILE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4513, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4816] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4517, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4817] Ref(PostgresCopyStdinDataStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4517, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4818] Ref(PsqlCopyMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4517, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4819] Ref(PsqlSetMetaCommandStatementSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(12), first_child_idx: 4517, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4820] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4821] Ref(StatementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4519, child_count: 1, min_times: 1, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4822] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4823] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4824] StringParser("FILLRECORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4520, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4825] StringParser("FILLTARGET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4520, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4826] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4522, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4827] Ref(FilterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4522, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4828] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4525, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4829] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4830] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4831] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4832] StringParser("FILTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4833] StringParser("FINAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4834] StringParser("FINALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4835] StringParser("FINISH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4836] StringParser("FIRST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4837] StringParser("FIRST_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4838] StringParser("FIXEDWIDTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4839] StringParser("FLAG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4840] StringParser("FLOAT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4841] StringParser("FLOAT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4842] StringParser("FLOAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4843] StringParser("FLOOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4527, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4844] StringParser("FOLLOWING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4527, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4845] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4531, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4846] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4531, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4847] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4535, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4848] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4535, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4849] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4850] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4851] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4852] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4853] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4538, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4854] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4855] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4540, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4856] Ref(ShareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4540, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4857] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4542, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4858] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4542, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4859] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4544, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4860] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4544, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4861] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4546, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4862] Ref(NowaitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4546, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4864] Ref(SkipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4865] Ref(LockedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4866] StringParser("FOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4548, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4867] StringParser("FORCE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4548, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4868] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4869] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4870] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4871] Ref(WrapperKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4551, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4872] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4553, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4873] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4553, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4874] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4553, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4875] StringParser("FOREIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4553, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4876] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4555, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4877] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4878] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4879] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4557, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4880] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4561, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4881] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4563, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4882] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4883] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4563, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4884] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4566, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4885] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4566, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4886] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4569, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4887] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4569, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4888] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4571, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4889] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4571, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4890] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4571, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4891] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4573, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4892] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4573, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4893] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4894] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4895] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4896] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4897] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4898] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4578, child_count: 5, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4899] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4900] Ref(GeneratedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4901] Ref(AlwaysKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4902] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4903] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4904] Ref(StoredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4583, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4905] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4585, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4906] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4587, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4907] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4587, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4908] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4587, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4909] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4590, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4910] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4590, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4911] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4593, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4912] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4593, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4913] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4595, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4914] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4595, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4915] Ref(InheritKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4595, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4916] StringParser("FORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4595, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4917] StringParser("FORTRAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4595, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4918] StringParser("FORWARD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4595, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4919] StringParser("FOUND")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4595, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4920] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4598, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4921] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4598, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4922] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4600, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4923] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4602, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4924] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4604, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4925] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4604, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4926] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4604, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4927] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4606, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4928] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4929] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4930] Ref(IntervalExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4931] Ref(UnboundedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4932] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4610, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4933] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4934] Ref(PrecedingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4935] Ref(FollowingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4612, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4936] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4616, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4937] Ref(BetweenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4616, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4938] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4616, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4939] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4618, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4940] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4618, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4941] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4622, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4942] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4943] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4944] Ref(RowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4945] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4946] Ref(TiesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4624, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4947] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4626, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4948] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4626, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4949] Ref(OthersKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4626, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4950] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4951] Ref(RangeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4952] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4953] Ref(GroupsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4954] StringParser("FRAME_ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4955] StringParser("FREE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4629, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4956] StringParser("FREEZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4629, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4958] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4631, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4959] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4633, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4635, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4961] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4962] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4638, child_count: 13, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4963] OneOf(13 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4651, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4964] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4651, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4965] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4651, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4966] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4967] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4968] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4653, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4969] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4970] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4971] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4972] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4973] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4974] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4975] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4976] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4977] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4978] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4979] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4655, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4980] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4655, child_count: 8, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4981] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4663, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4982] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4983] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4984] Ref(PermissionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4985] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4986] Ref(DatashareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4987] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4665, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4988] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4665, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4989] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4667, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4990] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4667, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4991] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4670, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4992] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4670, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4993] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4670, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4994] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4670, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4995] Ref(NamespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4670, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4996] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4672, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4997] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4672, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4998] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4672, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [4999] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4674, child_count: 7, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5000] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4681, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5001] Ref(PreTableFunctionKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4681, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5002] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4683, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5003] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5004] Ref(TableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4686, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5005] Ref(TemporalQuerySegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4686, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5006] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4687, child_count: 4, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5007] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4691, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5008] Ref(FromClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4691, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5009] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4691, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5010] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4691, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5011] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4691, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5012] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5013] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5014] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5015] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5016] Ref(SamplingExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4694, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5017] Ref(PostTableExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4694, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5018] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4697, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5019] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4699, child_count: 1, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5020] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4700, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5021] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4700, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5022] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4702, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5023] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4705, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5024] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4711, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 2, _padding: 0 }, // [5025] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4714, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5026] Ref(MLTableExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4714, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5027] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4714, child_count: 3, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5028] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5029] Ref(FromExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4717, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5030] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4719, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5031] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4719, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5032] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4719, child_count: 2, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5033] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4721, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5034] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4721, child_count: 0, min_times: 0, first_terminator_idx: 41, terminator_count: 0, _padding: 0 }, // [5035] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4721, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5036] Meta(conditional: indented_joins=True -> indent)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4721, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 2, _padding: 0 }, // [5037] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4722, child_count: 1, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5038] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4723, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5039] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4725, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5040] Ref(JoinClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4725, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5041] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4725, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5042] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5043] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5044] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4727, child_count: 2, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5045] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5046] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 43, terminator_count: 0, _padding: 0 }, // [5047] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 4729, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5048] Meta(conditional: indented_joins=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4729, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5049] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5050] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5051] Ref(IntegrationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4734, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5052] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4734, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5053] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4736, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5054] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4736, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5055] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4736, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5056] AnySetOf(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4742, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5057] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4742, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5058] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4745, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5059] Ref(AcceptinvcharsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4745, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5060] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4745, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5061] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4745, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5062] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5063] Ref(Query_all_statesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5064] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5065] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4748, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4750, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5067] Ref(Refresh_intervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4750, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5068] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4750, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5069] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4753, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5070] Ref(TruncatecolumnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4753, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5071] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4753, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5072] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4753, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5073] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4756, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5074] Ref(History_modeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4756, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5075] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4756, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5076] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4756, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5077] StringParser("FROM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4756, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5078] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4758, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5079] Ref(PivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4758, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5080] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4761, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5081] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4767, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5082] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4769, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5083] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5084] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5085] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5086] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5087] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4772, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5088] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4772, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5089] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4775, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5090] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4777, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5091] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4779, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5092] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4779, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5093] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4779, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5094] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4782, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5095] Ref(UnpivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4782, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5096] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4784, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5097] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5098] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5099] Ref(ExcludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5100] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4786, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5101] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4789, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5102] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5103] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5104] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5105] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5106] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4794, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5107] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4797, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5108] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4799, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5109] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4801, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5110] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4801, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5111] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4801, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5112] StringParser("FS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4801, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5113] StringParser("FULFILL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4801, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5114] StringParser("FULL")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4801, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5115] TypedParser("full_text_search_operator")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4801, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5116] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5117] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4804, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5118] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4804, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5119] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4806, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5120] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4806, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5121] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4808, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5122] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4808, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5123] Ref(NamedArgumentSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4808, child_count: 14, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5124] AnyNumberOf(14 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4822, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5125] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4822, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5126] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4824, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5127] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4827, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5128] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4827, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5129] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4830, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5130] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4830, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5131] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4830, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5132] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4830, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5133] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4834, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5134] Ref(TrimParametersGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 4834, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5135] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5136] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5137] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5138] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4835, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5139] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4837, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5140] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5141] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4839, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5142] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4839, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5143] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4841, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5144] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4843, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5145] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4843, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5146] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4843, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5147] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5148] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5149] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4845, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5150] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5151] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4847, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5152] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4847, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4849, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5154] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4849, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5155] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5156] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4851, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5157] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5158] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4853, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5159] Ref(AggregateOrderByClause)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4853, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5160] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5161] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4855, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5162] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4855, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5163] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4858, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5164] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4862, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5165] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4862, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5166] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4862, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5167] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4862, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5168] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4862, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5169] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4862, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5170] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5171] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5172] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5173] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5174] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5175] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5176] Ref(IndexColumnDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5177] Ref(EmptyStructLiteralSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4866, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5178] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4868, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5179] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4871, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5180] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4871, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5181] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5182] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5183] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4873, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5184] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4873, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5185] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4874, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5186] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4877, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5187] Ref(FunctionContentsGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4877, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5188] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4879, child_count: 15, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5189] AnyNumberOf(15 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4894, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5190] Ref(LanguageClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4894, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5191] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4898, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5192] Ref(TransformKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4898, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5193] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4898, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5194] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4898, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5195] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4898, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5196] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4898, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5197] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4901, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5198] Ref(ImmutableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4901, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5199] Ref(StableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4901, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5200] Ref(VolatileKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4901, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5201] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5202] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5203] Ref(LeakproofKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4903, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5204] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4906, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5205] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5206] Ref(CalledKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5207] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5208] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4910, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5209] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4910, child_count: 5, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5210] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5211] Ref(ReturnsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5212] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5213] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5214] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5215] Ref(InputKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5216] Ref(StrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4915, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5217] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4918, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5218] Ref(ExternalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5219] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4918, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5220] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4920, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5221] Ref(InvokerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4920, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5222] Ref(DefinerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4920, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5223] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4922, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5224] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4922, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5225] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4925, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5226] Ref(UnsafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4925, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5227] Ref(RestrictedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4925, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5228] Ref(SafeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4925, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5229] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4927, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5230] Ref(CostKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4927, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5231] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4927, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5232] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4929, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5233] Ref(RowsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4929, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5234] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4929, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5235] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5236] Ref(SupportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5237] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4931, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5238] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4934, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5239] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4934, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5240] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4934, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5241] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4936, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5242] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4938, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5243] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5244] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5245] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4940, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5246] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4942, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5247] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4944, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5248] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4944, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5249] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4944, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5250] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5251] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5252] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4946, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5253] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5254] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4948, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5255] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4950, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5256] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4950, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5257] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5258] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5259] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5260] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4953, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5261] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4955, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5262] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4955, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5263] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4955, child_count: 6, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5264] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4961, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5265] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4961, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5266] Ref(AtomicKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4961, child_count: 4, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5267] AnyNumberOf(4 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4965, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5268] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4967, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5269] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4967, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5270] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4967, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5271] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4969, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5272] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4969, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5273] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4969, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5274] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4971, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5275] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4974, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5276] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4974, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5277] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4974, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5278] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4974, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5279] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4974, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5280] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5281] Ref(ReturnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5282] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5283] Ref(SemicolonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4977, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5284] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4977, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5285] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5286] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4979, child_count: 3, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5287] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4982, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5288] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5289] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5290] StringParser("FUNCTION")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4984, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5291] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 4984, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5292] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4986, child_count: 1, min_times: 0, first_terminator_idx: 45, terminator_count: 1, _padding: 0 }, // [5293] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4987, child_count: 2, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5294] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4989, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5295] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4989, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5296] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4989, child_count: 0, min_times: 0, first_terminator_idx: 45, terminator_count: 0, _padding: 0 }, // [5297] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4989, child_count: 2, min_times: 0, first_terminator_idx: 46, terminator_count: 1, _padding: 0 }, // [5298] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5299] Ref(FunctionNameIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5300] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 0, min_times: 0, first_terminator_idx: 46, terminator_count: 0, _padding: 0 }, // [5301] Ref(BracketedSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4991, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5302] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 4994, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5303] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5304] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5305] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5306] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5307] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 4998, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5308] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5309] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5000, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5310] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5003, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5311] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5003, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5312] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5007, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5313] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5007, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5314] Ref(OutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5007, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5315] Ref(InoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5007, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5316] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5007, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5317] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5009, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5318] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5009, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5319] Ref(ColumnTypeReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5009, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5320] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5011, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5321] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5322] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5323] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5324] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5325] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5014, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5326] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5017, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5327] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5328] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5019, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5329] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5022, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5330] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5023, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5331] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5332] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5025, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5333] Ref(DateTimeFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5025, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5027, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5335] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5029, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5336] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 5031, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5337] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5032, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5338] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5339] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5340] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5341] Ref(ConvertFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5035, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5342] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5037, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5343] Ref(ApproximateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 5037, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5344] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5038, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5345] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5346] Ref(DatePartFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5347] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5348] Ref(ConvertFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5349] Ref(FunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5041, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5350] Ref(PostFunctionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5041, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5351] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5352] Ref(ConvertFunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5353] Ref(ConvertFunctionContentsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5354] StringParser("FUNCTIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5355] StringParser("FUSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5356] StringParser("FUTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5357] StringParser("G")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5358] StringParser("GB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5359] StringParser("GENERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5360] StringParser("GENERATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5361] StringParser("GEOGRAPHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5362] StringParser("GEOMETRY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5363] StringParser("GET")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5364] TypedParser("glob_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5365] StringParser("GLOBAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5366] StringParser("GLOBALDICT256")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5367] StringParser("GLOBALDICT64K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5368] StringParser("GO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5369] StringParser("GOTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5043, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5370] StringParser("GRANT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5043, child_count: 5, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5371] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5372] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5048, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5373] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5052, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5374] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5375] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5376] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5377] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5055, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5378] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5379] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5380] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5057, child_count: 4, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5381] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5382] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5383] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5384] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5385] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5386] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5061, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5387] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5063, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5388] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5065, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5389] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5065, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5390] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5067, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5391] Ref(RedshiftGroupGrantTargetSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5067, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5392] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5070, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5393] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5070, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5394] Ref(RedshiftGrantTargetSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5070, child_count: 1, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5395] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5071, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5396] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5397] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5398] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5074, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5399] Ref(RedshiftGrantTargetSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5074, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5400] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5077, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5401] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5080, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5402] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5080, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5403] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5080, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5404] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5080, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5405] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5083, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5406] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5083, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5407] Ref(AdminKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5083, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5408] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5083, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5409] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5086, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5410] Ref(CopyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5086, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5411] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5086, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5412] Ref(GrantsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5086, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5413] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5414] Ref(GrantedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5089, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5415] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5089, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5416] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5417] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5418] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5419] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5092, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5420] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5099, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5421] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5422] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5423] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5424] Ref(UsageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5425] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5426] Ref(DatashareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5427] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5101, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5428] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5429] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5430] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5103, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5431] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5105, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5432] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5433] Ref(NamespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5434] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5107, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5435] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5109, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5436] Ref(AccountKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5109, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5437] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5111, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5438] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5111, child_count: 3, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5439] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5440] Ref(ViaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5441] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5442] Ref(CatalogKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5443] StringParser("GRANTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5114, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5444] StringParser("GRANTS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5114, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5445] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5446] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5447] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5448] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5116, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5449] StringParser("GREATEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5116, child_count: 5, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5450] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5121, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5451] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5121, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5452] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5121, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 6, _padding: 0 }, // [5453] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5123, child_count: 7, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5454] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5455] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5456] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5457] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5458] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5459] Ref(GroupingSetsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5460] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5130, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5461] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5132, child_count: 2, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5462] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5463] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5464] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5465] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5466] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5467] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5468] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 0, min_times: 0, first_terminator_idx: 47, terminator_count: 0, _padding: 0 }, // [5469] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5134, child_count: 6, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5470] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5140, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5471] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5472] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5473] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5474] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5475] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5476] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5477] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5478] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5142, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5479] StringParser("GROUP")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5142, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5480] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5143, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 1, _padding: 0 }, // [5481] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5145, child_count: 4, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5482] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5149, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5483] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5149, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5484] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5149, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5485] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5149, child_count: 2, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5486] Bracketed(0 elements -> 0 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5151, child_count: 0, min_times: 0, first_terminator_idx: 53, terminator_count: 0, _padding: 0 }, // [5487] Ref(GroupByClauseTerminatorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5151, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5488] StringParser("GROUPING")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5151, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5489] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5154, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5490] Ref(GroupingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5154, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5491] Ref(SetsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5154, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5492] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5157, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5493] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5159, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5494] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5495] Ref(CubeRollupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5496] Ref(GroupingExpressionList)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5497] StringParser("GROUPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5498] StringParser("GZIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5499] StringParser("HANDLER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5161, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5500] StringParser("HASH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5161, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5501] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5165, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5502] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5165, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5503] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5167, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5504] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5505] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5170, child_count: 5, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5506] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5175, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5507] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5508] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5509] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5510] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5511] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5512] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5513] Ref(FetchKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5514] StringParser("HAVING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5515] StringParser("HEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5516] StringParser("HEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5517] StringParser("HIERARCHY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5518] StringParser("HISTORY_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5519] StringParser("HIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5520] StringParser("HLLSKETCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5521] StringParser("HOLD")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5522] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5523] StringParser("HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5524] StringParser("HYPERPARAMETERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5177, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5525] StringParser("IAM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5177, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5526] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5179, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5527] Ref(Iam_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5179, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5528] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5181, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5529] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5181, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5530] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5181, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5531] StringParser("IAM_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5181, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5532] StringParser("ID")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5181, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5533] Token("identifier")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5181, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5534] StringParser("IDENTITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5181, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5535] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5536] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5537] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5183, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5538] StringParser("IF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5183, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5539] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5186, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5540] Ref(IfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5186, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5541] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5186, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5542] Ref(ExistsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5186, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5543] StringParser("IGNORE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5186, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5544] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5188, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5545] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5546] Ref(IgnoreKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5547] Ref(RespectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5548] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5549] StringParser("IGNOREBLANKLINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5550] StringParser("IGNOREEXTRA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5551] StringParser("IGNOREHEADER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5552] StringParser("ILIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5553] StringParser("IMMEDIATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5554] StringParser("IMMEDIATELY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5555] StringParser("IMMUTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5556] StringParser("IMPLEMENTATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5190, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5557] StringParser("IMPLICIT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5190, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5558] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5559] Ref(ImportKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5560] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5561] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5193, child_count: 9, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5562] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5202, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5563] Ref(ImportForeignSchemaGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5202, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5564] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5202, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5565] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5204, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5566] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5206, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5567] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5208, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5568] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5208, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5569] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5208, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5570] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5208, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5571] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5211, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5572] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5573] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5574] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5575] Ref(ServerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5576] Ref(ServerReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5577] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5578] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5579] Ref(OptionsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5580] StringParser("IMPORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5581] StringParser("IMPORTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5213, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5582] StringParser("IN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5213, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5583] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5216, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5584] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5216, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5585] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5216, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5586] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5218, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5587] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5221, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5588] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5223, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5589] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5590] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5591] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5592] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5593] StringParser("INCLUDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5594] StringParser("INCLUDENEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5595] StringParser("INCLUDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5596] StringParser("INCREMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5597] StringParser("INDENT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5598] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5225, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5599] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5227, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5600] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5227, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5601] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5229, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5602] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5229, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5603] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5229, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5604] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5233, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5605] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5235, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5606] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5235, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5607] Ref(CollationReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5235, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5608] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 5237, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5609] Ref(OperatorClassReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5238, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5610] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5240, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5611] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5240, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5612] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5613] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5614] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5242, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5615] Ref(RelationOptionsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5242, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5616] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5244, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5617] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5244, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5618] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5244, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5619] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5246, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5620] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5246, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5621] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5622] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5623] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5248, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5624] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5250, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5625] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5626] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5627] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5253, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5628] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5629] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5630] Ref(IndexElementOptionsSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5256, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5631] StringParser("INDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5256, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5632] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5259, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5633] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5634] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5261, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5635] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5261, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5637] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5263, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5638] Ref(DefinitionParametersSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5263, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5639] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5640] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5641] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5642] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5643] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5644] StringParser("INDEXES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5645] StringParser("INDICATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5646] StringParser("INHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5647] StringParser("INHERITS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5648] StringParser("INITIAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5649] StringParser("INITIALLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5650] StringParser("INLINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5651] StringParser("INNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5652] StringParser("INOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5653] StringParser("INPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5654] StringParser("INPUTFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5655] StringParser("INSENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5267, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5656] StringParser("INSERT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5267, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5657] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5271, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5658] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5271, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5659] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5271, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5660] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5271, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5661] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5274, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5662] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5276, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5663] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5279, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5664] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5279, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5665] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5281, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5666] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5281, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5667] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5281, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5668] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5283, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5669] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5283, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5670] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5285, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5671] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5285, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5672] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5287, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5673] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5674] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5675] StringParser("INSTANCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5676] StringParser("INSTANTIABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5677] StringParser("INSTEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5678] StringParser("INT2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5679] StringParser("INT4")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5680] StringParser("INT8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5681] StringParser("INT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5682] StringParser("INTEGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5683] StringParser("INTEGRATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5684] StringParser("INTEGRITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5685] StringParser("INTERLEAVED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5686] StringParser("INTERSECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5290, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5687] StringParser("INTERSECTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5290, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5688] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5689] Ref(IntervalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5292, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5690] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5295, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5692] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5297, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5693] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5694] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5695] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5696] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5299, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5697] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5301, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5698] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5301, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5699] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5303, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5700] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5303, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5701] Ref(DatetimeUnitSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5303, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5702] StringParser("INTERVAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5303, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5703] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5704] Ref(YearKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5705] Ref(MonthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5706] Ref(DayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5707] Ref(HourKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5708] Ref(MinuteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5709] Ref(SecondKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5309, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5710] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5313, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5711] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5313, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5712] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5713] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5714] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5715] Ref(UnloggedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5716] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5717] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5718] StringParser("INTO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5316, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5719] StringParser("INVOKER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5316, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5720] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5721] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5722] Ref(NanLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5723] Ref(UnknownLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5724] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5725] Ref(NormalizedGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5726] Ref(JsonTestGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5322, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5727] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5728] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5729] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5730] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5731] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5732] StringParser("IS")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5733] Ref(IsnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5734] StringParser("ISNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5326, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5735] StringParser("ISOLATION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5326, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5736] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5329, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5737] Ref(IsolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5329, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5738] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5329, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5739] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5331, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5740] Ref(SerializableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5331, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5741] Ref(SnapshotKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5331, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5742] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5334, child_count: 7, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5743] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5341, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5744] Ref(ConditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5341, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5745] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5341, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5746] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5341, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5747] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5342, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5748] Ref(NestedJoinGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5342, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5749] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5346, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5750] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5346, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5751] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5346, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5752] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5753] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5754] Ref(JoinUsingConditionGrammar)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5348, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5755] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5348, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5756] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5757] Ref(UnconditionalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5758] Ref(JoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5759] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5354, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5760] Ref(MatchConditionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5354, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5761] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5358, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5762] Ref(ExtendedNaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5358, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5763] Ref(FromExpressionElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5358, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5764] StringParser("JOIN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5358, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5765] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5359, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5766] Ref(JoinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5359, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5767] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5361, child_count: 2, min_times: 1, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5768] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5363, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5769] Ref(FromPivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5363, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5770] Ref(FromUnpivotExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5363, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5771] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5363, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5772] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5367, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5773] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5367, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5774] Meta(conditional: indented_on_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5367, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5775] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5369, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5776] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5372, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5777] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5372, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5778] Meta(conditional: indented_on_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5372, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5779] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5374, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5780] Ref(InnerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5374, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5781] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5376, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5782] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5783] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5784] Ref(LeftKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5785] Ref(RightKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5379, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5786] Ref(OuterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5379, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5787] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5383, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5788] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5383, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5789] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5386, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5790] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5388, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5791] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5388, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5792] StringParser("JSON")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5388, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5793] TypedParser("json_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5388, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5794] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5795] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5391, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5796] Ref(JsonTypeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5391, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5797] Ref(JsonUniqueKeysGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5391, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5798] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5395, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5799] Ref(ValueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5395, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5800] Ref(ScalarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5395, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5801] Ref(ArrayKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5395, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5802] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5395, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5803] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5398, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5804] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5400, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5805] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5400, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5806] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5400, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5807] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5400, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5808] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5809] Ref(KeysKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5810] StringParser("JSON_ARRAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5811] StringParser("JSON_ARRAYAGG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5812] StringParser("JSON_EXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5813] StringParser("JSON_OBJECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5814] StringParser("JSON_OBJECTAGG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5815] StringParser("JSON_QUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5816] StringParser("JSON_TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5817] StringParser("JSON_TABLE_PRIMITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5818] StringParser("JSON_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5819] StringParser("K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5820] StringParser("KAFKA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5821] StringParser("KEEP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5822] StringParser("KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5823] StringParser("KEY_MEMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5824] StringParser("KEY_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5825] StringParser("KEYS")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5826] Token("keyword")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5827] StringParser("KINESIS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5828] StringParser("KMEANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5829] StringParser("KMS_KEY_ID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5830] StringParser("LABEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5831] StringParser("LAG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5401, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5832] StringParser("LAMBDA")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5401, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5833] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5403, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5834] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5403, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5835] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5405, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5836] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5405, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5837] Ref(SingleQuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5405, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5838] StringParser("LANGUAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5405, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5839] StringParser("LARGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5405, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5840] StringParser("LAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5405, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5841] StringParser("LAST_VALUE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5405, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5842] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5843] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5844] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5845] StringParser("LATERAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5846] StringParser("LEAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5847] StringParser("LEADING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5848] StringParser("LEAKPROOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5849] StringParser("LEAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5850] StringParser("LEFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5851] StringParser("LENGTH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5407, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5852] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5409, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5853] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5409, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5854] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5409, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5855] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5409, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5856] StringParser("LEVEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5409, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5857] StringParser("LIBRARY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5409, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5858] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5412, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5859] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5414, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5860] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5414, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5861] Ref(LikeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5414, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5862] Ref(Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5414, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5863] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5416, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5864] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5416, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5865] Ref(Tail_Recurse_Expression_A_Grammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5416, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5866] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5867] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5868] Ref(IlikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5419, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5869] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5421, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5870] Ref(SimilarKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5421, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5871] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5421, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5872] StringParser("LIKE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5421, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5873] TypedParser("like_operator")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5421, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5423, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5875] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5876] Ref(IncludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5877] Ref(ExcludingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5878] Ref(DefaultsKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5425, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5879] StringParser("LIKE_REGEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5425, child_count: 5, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5880] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5430, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5881] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5430, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5882] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5432, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5883] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5435, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5884] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5438, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5885] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5438, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5886] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5438, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5887] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5438, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5888] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5440, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5889] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5442, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5890] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5442, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5891] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5444, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5892] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5444, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5893] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5444, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5894] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5895] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5896] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5897] StringParser("LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5898] StringParser("LINES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5899] StringParser("LINK")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5900] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5901] StringParser("LIST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5902] StringParser("LISTAGG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5446, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5903] StringParser("LISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5446, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5904] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5448, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5905] Ref(ListenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5448, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5906] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5448, child_count: 11, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5907] OneOf(11 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5908] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5909] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5910] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5911] Ref(QualifiedNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5912] Ref(NullLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5913] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5914] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5915] Ref(TypedArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5916] Ref(ObjectLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5917] Ref(MaxLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5918] Ref(DollarNumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5919] Token("literal")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5920] Token("literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5921] StringParser("LN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5459, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5922] StringParser("LOAD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5459, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5923] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5924] Ref(LoadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5925] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5926] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5927] StringParser("LOCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5928] StringParser("LOCALTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5929] StringParser("LOCALTIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5930] StringParser("LOCATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5931] StringParser("LOCATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5461, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5932] StringParser("LOCK")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5461, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5933] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5464, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5934] Ref(LockKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5464, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5935] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5464, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5936] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5937] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5938] StringParser("LOCKED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5939] StringParser("LOG10")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5940] StringParser("LOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5941] StringParser("LOGGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5942] StringParser("LOGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5943] StringParser("LOWER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5944] StringParser("LUN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5945] StringParser("LUNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5946] StringParser("LZO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5947] StringParser("LZOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5466, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5948] StringParser("M")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5466, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5949] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5470, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5950] Ref(MlKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5470, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5951] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5470, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5952] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5470, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5953] Bracketed(2 elements -> 2 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5474, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5954] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5476, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5955] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5476, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5956] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5476, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5957] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5478, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5958] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5478, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5959] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5960] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5961] StringParser("MAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5481, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5962] StringParser("MANAGE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5481, child_count: 9, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5963] Sequence(9 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5490, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5964] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5965] Ref(AttachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5966] Ref(DetachKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5967] Ref(RlsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5968] Ref(PolicyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5969] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5970] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5492, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5971] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5492, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5972] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5494, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5973] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5494, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5974] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5496, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5975] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5496, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5976] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5496, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5977] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5498, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5978] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5500, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5979] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5980] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5981] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5982] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5983] StringParser("MANIFEST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5984] StringParser("MAP")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5985] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5986] StringParser("MAPPING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5987] StringParser("MASKING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5988] StringParser("MASTER_SYMMETRIC_KEY")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5989] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5990] StringParser("MATCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5991] StringParser("MATCH_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5992] StringParser("MATCH_RECOGNIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5993] StringParser("MATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5994] StringParser("MATCHES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5995] StringParser("MATERIALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5996] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5997] StringParser("MAX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5998] StringParser("MAX_CELLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [5999] StringParser("MAX_RUNTIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6000] StringParser("MAXERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6001] StringParser("MAXFILESIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6002] StringParser("MAXVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6003] StringParser("MB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6004] StringParser("MEASURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6005] StringParser("MEMBER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5502, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6006] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5502, child_count: 5, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6007] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6008] Ref(InsertKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5507, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6009] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5507, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6010] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5507, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6011] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5509, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6012] Ref(MergeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5509, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6013] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5509, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6014] StringParser("MERGE")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5509, child_count: 2, min_times: 1, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6015] AnyNumberOf(2 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5511, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6016] Ref(MergeMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5511, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6017] Ref(MergeNotMatchedClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5511, child_count: 7, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6018] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5518, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6019] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5518, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6020] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5518, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6021] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5520, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6022] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5520, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6023] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5520, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6024] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5520, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6025] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6026] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6027] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5522, child_count: 10, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6028] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6029] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6030] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6031] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6032] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5532, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6033] Ref(SourceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5532, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6034] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6035] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6036] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6037] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5534, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6038] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6039] Ref(MergeUpdateClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6040] Ref(MergeDeleteClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5537, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6041] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5539, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6042] Ref(DoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5539, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6043] Ref(NothingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5539, child_count: 8, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6044] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5547, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6045] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5547, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6046] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5547, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6047] Ref(MatchedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5547, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6048] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6049] Ref(AndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6050] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6051] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6052] Ref(MergeInsertClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5549, child_count: 12, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6053] Sequence(12 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6054] Ref(MergeIntoLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5561, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6055] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6056] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6057] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6058] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5563, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6059] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5567, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6060] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5567, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6061] Ref(AliasedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5567, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5569, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6063] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5572, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6064] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5572, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6065] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5572, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6066] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6067] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6068] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6069] Meta(conditional: indented_using_on=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6070] Ref(JoinOnConditionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 5574, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6071] Meta(conditional: indented_using_on=True -> dedent)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5574, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6072] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5576, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6073] Ref(MergeMatchSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5576, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6074] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5578, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6075] Ref(RemoveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5578, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6076] Ref(DuplicatesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5578, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6077] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6078] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6079] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6080] StringParser("MESSAGE_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6081] StringParser("MESSAGE_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6082] StringParser("MESSAGE_TEXT")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5582, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6083] TypedParser("meta_command_query_buffer")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5582, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6084] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5583, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6085] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5584, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6086] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6087] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6088] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6089] StringParser("METASTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6090] StringParser("METHOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6091] StringParser("MILLISECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6092] StringParser("MIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6093] StringParser("MINUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6094] StringParser("-")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6095] StringParser("MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6096] StringParser("MINVALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6097] StringParser("ML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6098] StringParser("MLP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6099] StringParser("MOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6100] StringParser("MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6101] StringParser("MODEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6102] StringParser("MODEL_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6103] StringParser("MODIFIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6104] StringParser("MODIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6105] StringParser("MODULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6106] StringParser("%")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6107] StringParser("MODULUS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6108] StringParser("MONITOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6109] StringParser("MONTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6110] StringParser("MORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6111] StringParser("MOSTLY16")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6112] StringParser("MOSTLY32")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6113] StringParser("MOSTLY8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6114] StringParser("MOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6115] StringParser("MSK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6116] StringParser("MTLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6117] StringParser("MULTICLASS_CLASSIFICATION")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5586, child_count: 1, min_times: 1, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6118] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6119] Ref(MultilineConcatenateNewline)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6120] TypedParser("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6121] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6122] StringParser("MULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6123] StringParser("MUMPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6124] StringParser("MYSQL")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6125] TypedParser("word")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6126] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5587, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6127] StringParser("NAME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5587, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6128] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5590, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6129] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5590, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6130] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5593, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6131] Ref(RightArrowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5593, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6132] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5593, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6133] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5593, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6134] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5593, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6135] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5596, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6136] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5596, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6137] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5596, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6138] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5598, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6139] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5598, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6140] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5601, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6141] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5601, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6142] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5605, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6143] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5605, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6144] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5607, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6145] Ref(NamedWindowExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5607, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6146] StringParser("NAMES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5607, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6147] StringParser("NAMESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5607, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6148] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5607, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6149] StringParser("NAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5607, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6150] StringParser("NATIONAL")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5607, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6151] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6152] Ref(NaturalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6153] Ref(JoinTypeKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6154] StringParser("NATURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6155] StringParser("NCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6156] StringParser("NCLOB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6157] StringParser("-")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6158] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6159] StringParser("NESTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6160] StringParser("NESTING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6161] StringParser("NEW")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6162] Token("newline")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6163] StringParser("NEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6164] StringParser("NFC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6165] StringParser("NFD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6166] StringParser("NFKC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6167] StringParser("NFKD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6168] StringParser("NIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6169] StringParser("NO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6170] StringParser("NOBYPASSRLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6171] StringParser("NOCACHE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6172] StringParser("NOCREATEDB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6173] StringParser("NOCREATEROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6174] StringParser("NOCREATEUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6175] StringParser("NOCYCLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6176] StringParser("NOINHERIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6177] StringParser("NOLOAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5609, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6178] StringParser("NOLOGIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5609, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6179] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5615, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6180] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5615, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6181] Ref(UnorderedSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5615, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6182] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5618, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6183] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5618, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6184] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5621, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6185] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5621, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6186] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6187] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6188] Ref(BracketedSetExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6189] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6190] OneOf(0 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5624, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6191] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5630, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6192] Ref(SetExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5630, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6193] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5632, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6194] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6195] Ref(SelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6196] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6197] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6198] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6199] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6200] StringParser("NONATOMIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6201] StringParser("NONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6202] StringParser("NOORDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6203] StringParser("NOREPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5635, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6204] StringParser("NORMALIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5635, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6205] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5637, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6206] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6207] Ref(NfcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6208] Ref(NfdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6209] Ref(NfkcKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6210] Ref(NfkdKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6211] Ref(NormalizedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6212] StringParser("NORMALIZED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6213] StringParser("NOSUPERUSER")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5641, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6214] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5641, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6215] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5643, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6216] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6217] Ref(RawNotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5645, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6218] Ref(RawEqualsSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5645, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6219] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5647, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6220] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5647, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6221] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5647, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6222] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5649, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6223] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5649, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6224] Ref(RawLessThanSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5649, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6225] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6226] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6227] Ref(RawGreaterThanSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6228] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6229] Ref(NotnullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6230] StringParser("NOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6231] StringParser("NOTHING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5651, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6232] StringParser("NOTIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5651, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6233] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5654, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6234] Ref(NotifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5654, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6235] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5654, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6236] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6237] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6238] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6239] StringParser("NOTNULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6240] StringParser("NOWAIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6241] StringParser("NTH_VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6242] StringParser("NTILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6243] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6244] StringParser("NULL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6245] StringParser("NULLABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6246] StringParser("NULLIF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6247] StringParser("NULLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6248] StringParser("NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6249] StringParser("NUMERIC")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6250] TypedParser("numeric_literal")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6251] StringParser("NVARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5656, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6252] StringParser("OBJECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5656, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6253] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6254] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6255] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6256] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5659, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6257] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5662, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6258] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5664, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6259] Ref(ObjectLiteralElementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5664, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6260] StringParser("{")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5664, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6261] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5666, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6262] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5666, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6263] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5668, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6264] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5668, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6265] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5668, child_count: 12, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6266] OneOf(12 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6267] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6268] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6269] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6270] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6271] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6272] Ref(StartSquareBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6273] Ref(StartBracketSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6274] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6275] Ref(ColonSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6276] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6277] Ref(JoinLikeClauseGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5680, child_count: 6, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6278] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6279] Ref(UnpivotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6280] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6281] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6282] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6283] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6284] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6285] StringParser("OBJECTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6286] StringParser("OCCURRENCES_REGEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6287] StringParser("OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6288] StringParser("OCTETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6289] StringParser("OF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6290] StringParser("OFF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5686, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6291] StringParser("OFFLINE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5686, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6292] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5690, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6293] Ref(OffsetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5690, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6294] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6295] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6296] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6297] StringParser("OFFSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6298] StringParser("OID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6299] StringParser("OIDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6300] StringParser("OLD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6301] StringParser("OMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6302] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6303] StringParser("ON")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6304] StringParser("ONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6305] StringParser("ONLY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6306] StringParser("OPEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6307] StringParser("OPERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6308] StringParser("OPERATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5692, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6309] StringParser("OPTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5692, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6310] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5694, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6311] Ref(OptionsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5694, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6312] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5697, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6313] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5698, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6314] Ref(OptionsListGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5698, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6315] StringParser("OPTIONS")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5698, child_count: 1, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6316] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5699, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6317] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5701, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6318] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5703, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6319] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5703, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6320] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5703, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6321] StringParser("OR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5703, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6322] StringParser("OR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5703, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6323] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5705, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6324] Ref(OrKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5705, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6325] Ref(ReplaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5705, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6326] StringParser("ORC")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5705, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6327] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5710, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6328] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5710, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6329] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5710, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 2, _padding: 0 }, // [6330] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5712, child_count: 4, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6331] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5716, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6332] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5719, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6333] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5719, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6334] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5719, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6335] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5719, child_count: 3, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6336] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5722, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6337] Ref(AscKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5722, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6338] Ref(DescKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5722, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6339] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5724, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6340] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5724, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6341] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5724, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6342] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6343] Ref(NullsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5726, child_count: 2, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6344] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5728, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6345] Ref(FirstKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5728, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6346] Ref(LastKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5728, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6347] Ref(WithFillSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5728, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6348] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5728, child_count: 0, min_times: 0, first_terminator_idx: 54, terminator_count: 0, _padding: 0 }, // [6349] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5728, child_count: 8, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6350] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6351] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6352] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6353] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6354] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6355] Ref(FrameClauseUnitGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6356] Ref(SeparatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5736, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6357] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5738, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6358] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5738, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6359] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5738, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6360] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5738, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6361] StringParser("ORDER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5738, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6362] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6363] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6364] Ref(NoorderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6365] StringParser("ORDERING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6366] StringParser("ORDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6367] StringParser("OTHERS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6368] StringParser("OUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6369] StringParser("OUTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6370] StringParser("OUTPUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5740, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6371] StringParser("OUTPUTFORMAT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5740, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6372] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6373] Ref(IgnoreRespectNullsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6374] Ref(OverKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5745, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6375] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5747, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6376] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5747, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6377] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5750, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6378] Ref(WindowSpecificationSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5750, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6379] StringParser("OVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5750, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6380] StringParser("OVERFLOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5750, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6381] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6382] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6383] Ref(AmpersandSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5752, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6384] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5755, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6385] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5757, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6386] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5758, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6387] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5763, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6388] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5767, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6389] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5767, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6390] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5767, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6391] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5767, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6392] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5767, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6393] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5767, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6394] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6395] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6396] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6397] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6398] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6399] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6400] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5771, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6401] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5773, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6402] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5774, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6403] Bracketed(3 elements -> 3 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5779, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6404] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6405] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6406] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6407] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6408] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6409] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5783, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6410] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6411] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6412] Ref(DateTimeLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6413] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6414] Ref(FunctionContentsExpressionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6415] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6416] StringParser("OVERLAPS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6417] StringParser("OVERLAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6418] StringParser("OVERRIDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6419] StringParser("OVERWRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6420] StringParser("OWNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6421] StringParser("OWNER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6422] StringParser("OWNERSHIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6423] StringParser("P")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6424] StringParser("PAD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6425] StringParser("PARALLEL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6426] StringParser("PARAMETER")
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6427] RegexParser
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6428] StringParser("?")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6429] StringParser("PARAMETER_MODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6430] StringParser("PARAMETER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6431] StringParser("PARAMETER_ORDINAL_POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6432] StringParser("PARAMETER_SPECIFIC_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6433] StringParser("PARAMETER_SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6434] StringParser("PARAMETER_SPECIFIC_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6435] StringParser("PARQUET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6436] StringParser("PARSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5787, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6437] StringParser("PARTIAL")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5787, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6438] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5790, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6439] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5792, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6440] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5792, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6441] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5795, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6442] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6443] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5797, child_count: 4, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6444] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5801, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6445] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5801, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6446] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5804, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6447] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5806, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6448] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6449] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6450] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6451] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6452] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5809, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6453] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5812, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6454] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5814, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6455] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6456] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6457] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6458] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5817, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6459] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5819, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6460] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5819, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6461] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5822, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6462] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6463] Ref(ModulusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6464] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6465] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6466] Ref(RemainderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6467] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5827, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6468] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6469] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6470] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5832, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6471] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5834, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6472] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5837, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6473] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5839, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6474] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5839, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6475] StringParser("PARTITION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5839, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6476] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5842, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6477] Ref(PartitionedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5842, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6478] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5842, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6479] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5845, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6480] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5847, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6481] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6482] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6483] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6484] StringParser("PARTITIONED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6485] StringParser("PASCAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6486] StringParser("PASS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6487] StringParser("PASSING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6488] StringParser("PASSTHROUGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6489] StringParser("PASSWORD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6490] StringParser("PAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5849, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6491] StringParser("PATH")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5849, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6492] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5851, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6493] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5853, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6494] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5853, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6495] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6496] TypedParser("word")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6497] Ref(SlashSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6498] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6499] StringParser("PATTERN")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6500] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6501] StringParser("PER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6502] StringParser("PERCENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6503] StringParser("PERCENT_RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6504] StringParser("PERCENTILE_CONT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6505] StringParser("PERCENTILE_DISC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6506] StringParser("PERIOD")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6507] StringParser("PERMISSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6508] StringParser("PERMISSIONS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6509] StringParser("PERMUTE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6510] TypedParser("pg_trgm_operator")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6511] TypedParser("pgvector_operator")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6512] StringParser("PIPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6513] StringParser("|")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6514] StringParser("PIVOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6515] StringParser("PLACING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6516] StringParser("PLAIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6517] StringParser("PLAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6518] StringParser("PLANS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6519] StringParser("PLI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6520] StringParser("PLPYTHONU")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6521] StringParser("+")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6522] StringParser("POLICY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6523] StringParser("PORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6524] StringParser("PORTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6525] StringParser("POSITION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6526] StringParser("POSITION_REGEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5855, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6527] StringParser("+")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5855, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6528] AnyNumberOf(3 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6529] Ref(WithinGroupClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6530] Ref(OverClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6531] Ref(FilterClauseGrammar)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6532] Nothing
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6533] TypedParser("postgis_operator")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5858, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6534] TypedParser("postgres_copy_stdin_data_block")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5858, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6535] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5859, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6536] Ref(PostgresCopyStdinDataSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5859, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6537] StringParser("POSTGRES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5859, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6538] StringParser("POWER")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5859, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6539] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5860, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6540] Ref(LateralKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5860, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6541] StringParser("PRECEDES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5860, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6542] StringParser("PRECEDING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5860, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6543] StringParser("PRECISION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5860, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6544] StringParser("PREPARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5860, child_count: 5, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6545] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6546] Ref(PrepareKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5865, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6547] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5865, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6548] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5868, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6549] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6550] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6551] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6552] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6553] StringParser("PREPARED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6554] StringParser("PREPROCESSORS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6555] StringParser("PRESERVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5870, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6556] StringParser("PRESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5870, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6557] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6558] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6559] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6560] StringParser("PRIMARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6561] StringParser("PRIOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6562] StringParser("PRIVATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6563] StringParser("PRIVILEGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6564] StringParser("PROBLEM_TYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6565] StringParser("PROCEDURAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5872, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6566] StringParser("PROCEDURE")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5872, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6567] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5875, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6568] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5877, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6569] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5879, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6570] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 5881, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6571] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5882, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6572] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5884, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6573] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6574] Ref(RefcursorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6575] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6576] Ref(ArgModeGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6577] Ref(ArgModeGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6578] StringParser("PROCEDURES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6579] StringParser("PROGRAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6580] StringParser("PROPERTIES")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6581] TypedParser("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6582] StringParser("PRUNE")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5886, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6583] TypedParser("psql_copy_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5886, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6584] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5887, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6585] Ref(PsqlCopyMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5887, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6586] TypedParser("psql_set_command")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5887, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6587] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5888, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6588] Ref(PsqlSetMetaCommandSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5888, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6589] StringParser(":")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5888, child_count: 1, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6590] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5889, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6591] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5891, child_count: 3, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6592] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5894, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6593] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6594] Ref(PsqlVariableColonSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5896, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6595] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5898, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6596] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5898, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6597] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5898, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6598] StringParser("PTF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5898, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6599] StringParser("PUBLIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5898, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6600] StringParser("PUBLICACCESSIBLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5898, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6601] StringParser("PUBLICATION")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5898, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6602] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5900, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6603] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5902, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6604] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5902, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 1, _padding: 0 }, // [6605] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5904, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6606] Ref(PublicationTableSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5904, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6607] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6608] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5906, child_count: 2, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6609] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5908, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6610] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5908, child_count: 0, min_times: 0, first_terminator_idx: 56, terminator_count: 0, _padding: 0 }, // [6611] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5908, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6612] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6613] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6614] Ref(InKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6615] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5912, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 1, _padding: 0 }, // [6616] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5914, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6617] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6618] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6619] Ref(Current_schemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5916, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6620] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5918, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6621] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5918, child_count: 2, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6622] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6623] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 0, min_times: 0, first_terminator_idx: 57, terminator_count: 0, _padding: 0 }, // [6624] Ref(TablesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6625] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5920, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6626] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5923, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6627] Ref(ExtendedTableReferenceGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5923, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6628] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5923, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6629] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5925, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6630] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5925, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6631] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5928, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6632] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5928, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6633] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5930, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6634] Ref(SignedSegmentGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5930, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6635] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5930, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6636] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6637] Ref(OperatorKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5932, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6638] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5935, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6639] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5938, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6640] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5938, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6641] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5938, child_count: 1, min_times: 1, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6642] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5939, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6643] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5939, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6644] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5941, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6645] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(1), first_child_idx: 5941, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6646] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5943, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6647] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5943, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6648] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5943, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6649] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6650] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6651] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5947, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6652] StringParser("QUALIFY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5947, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6653] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5950, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6654] Ref(ComparisonOperatorGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5950, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6655] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6656] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6657] Ref(AnyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6658] Ref(SomeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 5953, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6659] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6660] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6661] StringParser("QUARTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5956, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6662] StringParser("QUERY_ALL_STATES")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5956, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6663] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5958, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6664] Ref(QuotaKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5958, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6665] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5960, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6666] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5962, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6667] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5962, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6668] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6669] Ref(MbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6670] Ref(GbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6671] Ref(TbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6672] Ref(UnlimitedKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6673] StringParser("QUOTA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5965, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6674] StringParser("QUOTE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5965, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6675] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6676] TypedParser("double_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5967, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6677] TypedParser("unicode_double_quote")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5967, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6678] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5971, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6679] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5973, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6680] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5975, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6681] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5975, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6682] TypedParser("escaped_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5975, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6683] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5977, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6684] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5977, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6685] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5977, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6686] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5979, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6687] TypedParser("bit_string_literal")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5979, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6688] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5981, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6689] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5981, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6690] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5981, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6691] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5984, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6692] TypedParser("unicode_single_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5984, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6693] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5986, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6694] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5986, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6695] RegexParser
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5986, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6696] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5988, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6697] Ref(UescapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::RegexParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5988, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6698] RegexParser
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5988, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6699] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5990, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6700] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5992, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6701] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 5992, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6702] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6703] Ref(MultilineConcatenateDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6704] TypedParser("dollar_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6705] StringParser("QUOTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6706] StringParser("RANGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6707] StringParser("RANK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6708] StringParser("=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6709] StringParser(">")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6710] StringParser("RAW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6711] StringParser("<")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6712] StringParser("!")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6713] StringParser("RCFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6714] StringParser("READ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6715] StringParser("READRATIO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6716] StringParser("READS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6717] StringParser("REAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 5994, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6718] StringParser("REASSIGN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 5994, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6719] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6000, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6720] Ref(ReassignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6000, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6721] Ref(OwnedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6000, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6722] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6000, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6723] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6002, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6724] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6725] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6726] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6727] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6728] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6729] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6006, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6730] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6731] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6732] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6733] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6734] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6735] StringParser("RECHECK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6736] StringParser("RECLUSTER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6737] StringParser("RECOVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6738] StringParser("RECOVERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6010, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6739] StringParser("RECURSIVE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6010, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6740] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6014, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6741] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6742] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6743] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6016, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6744] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6745] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6746] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6747] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6748] Ref(PublicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6018, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6749] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6750] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6751] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6752] StringParser("REDSHIFT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6753] StringParser("REF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6020, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6754] StringParser("REFCURSOR")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6020, child_count: 5, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6755] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6025, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6756] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6025, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6757] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6025, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6758] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6025, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6759] Ref(ReferenceMatchGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6025, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6760] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6027, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6761] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6030, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6762] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6030, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6763] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6030, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6764] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6030, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6765] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6766] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6767] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6768] Ref(ReferentialActionGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6033, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6769] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6770] Ref(MatchKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6035, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6771] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6038, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6772] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6038, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6773] Ref(PartialKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6038, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6774] Ref(SimpleKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6038, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6775] StringParser("REFERENCE_USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6038, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6776] StringParser("REFERENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6038, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6777] StringParser("REFERENCING")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6038, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6778] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6042, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6779] Ref(CascadeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6042, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6780] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6045, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6781] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6045, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6782] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6783] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6047, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6784] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6047, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6785] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6050, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6786] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6787] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6788] Ref(RestrictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6052, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6789] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6054, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6790] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6054, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6791] Ref(ActionKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6054, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6792] StringParser("REFRESH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6054, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6793] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6794] Ref(RefreshKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6795] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6796] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6797] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6798] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6799] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6800] StringParser("REFRESH_INTERVAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6801] StringParser("REGION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6802] StringParser("REGR_AVGX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6803] StringParser("REGR_AVGY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6804] StringParser("REGR_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6805] StringParser("REGR_INTERCEPT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6806] StringParser("REGR_R2")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6807] StringParser("REGR_SLOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6808] StringParser("REGR_SXX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6809] StringParser("REGR_SXY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6810] StringParser("REGR_SYY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6811] StringParser("REGRESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6060, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6812] StringParser("REINDEX")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6060, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6813] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6063, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6814] Ref(ReindexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6063, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6815] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6066, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6816] Delimited(3 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6068, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6817] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6071, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6818] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6073, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6819] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6073, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6820] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6073, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6821] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6075, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6822] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6075, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6823] Ref(TablespaceReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6075, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6824] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6077, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6825] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6077, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6826] Ref(BooleanLiteralGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6077, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6827] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6081, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6828] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6084, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6829] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6084, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6830] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6084, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6831] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6084, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6832] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6087, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6833] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6087, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6834] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6087, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6835] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6087, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6836] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6090, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6837] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6090, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6838] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6090, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6839] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6090, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6840] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6093, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6841] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6095, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6842] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6095, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6843] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6095, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6844] Ref(ConcurrentlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6095, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6845] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6095, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6846] StringParser("REJECTLOG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6095, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6847] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6098, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6848] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6098, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6849] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6100, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6850] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6100, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6851] Ref(PropertiesNakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6100, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6852] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6102, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6853] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6102, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6854] Ref(DefinitionArgumentValueGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6102, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6855] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6105, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6856] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6857] Ref(RelationOptionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6858] StringParser("RELATIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6859] StringParser("RELEASE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6860] StringParser("REMAINDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6861] StringParser("REMOVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6862] StringParser("REMOVEQUOTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6863] StringParser("RENAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6864] StringParser("REPEATABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6865] StringParser("REPLACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6866] StringParser("REPLICA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6867] StringParser("REPLICATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6868] StringParser("REQUIRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6107, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6869] StringParser("RESET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6107, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6870] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6871] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6872] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6873] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6110, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6874] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6875] Ref(ResetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6112, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6876] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6877] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6878] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6879] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6880] StringParser("RESORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6881] StringParser("RESOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6882] StringParser("RESPECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6883] StringParser("RESTART")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6884] StringParser("RESTORE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6885] StringParser("RESTRICT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6886] StringParser("RESTRICTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6887] StringParser("RESULT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6888] StringParser("RETRY_TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6889] StringParser("RETURN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6890] StringParser("RETURNED_CARDINALITY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6891] StringParser("RETURNED_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6892] StringParser("RETURNED_OCTET_LENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6893] StringParser("RETURNED_SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6894] StringParser("RETURNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6895] StringParser("RETURNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6115, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6896] StringParser("REVOKE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6115, child_count: 6, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6897] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6121, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6898] Ref(RevokeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6121, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6899] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6900] Ref(GrantKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6901] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6902] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6124, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6903] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6128, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6904] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6905] Ref(AccessPermissionsSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6906] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6907] Ref(AccessObjectSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6131, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6908] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6909] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6133, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6910] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6133, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6911] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6912] Ref(OwnershipKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6913] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6914] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6915] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6916] Ref(UserReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6917] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6918] Ref(AccessTargetSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6919] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6920] StringParser("=>")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6921] StringParser("RIGHT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6922] StringParser("RLIKE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6923] StringParser("RLS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6924] StringParser("ROLE")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6925] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6926] StringParser("ROLLBACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6927] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6928] StringParser("ROLLUP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6929] StringParser("ROUNDEC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6930] StringParser("ROUTINE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6931] StringParser("ROUTINE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6932] StringParser("ROUTINE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6933] StringParser("ROUTINE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6137, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6934] StringParser("ROUTINES")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6137, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6935] AnySetOf(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6139, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6936] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6937] Ref(FieldsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6938] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6939] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6940] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6143, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6941] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6942] Ref(LinesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6943] Ref(TerminatedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6944] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6945] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6946] StringParser("ROW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6947] StringParser("ROW_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6948] StringParser("ROW_NUMBER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6949] StringParser("ROWGROUPSIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6950] StringParser("ROWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6951] StringParser("RULE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6952] StringParser("RUNLENGTH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6953] StringParser("RUNNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6954] StringParser("S3_BUCKET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6955] StringParser("S3_GARBAGE_COLLECT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6956] StringParser("SAFE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6147, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6957] StringParser("SAGEMAKER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6147, child_count: 4, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6958] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6151, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6959] Ref(TablesampleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6151, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6960] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6153, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6961] Ref(BernoulliKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6153, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6962] Ref(SystemKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6153, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6963] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6156, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6964] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6156, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6965] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6158, child_count: 1, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6966] OneOf(1 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6159, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6967] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6159, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6968] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6969] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6970] StringParser("SAVEPOINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6971] StringParser("SCALAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6972] StringParser("SCALE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6973] StringParser("SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6974] StringParser("SCHEMA_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6975] StringParser("SCHEMAS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6976] StringParser("SCOPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6977] StringParser("SCOPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6978] StringParser("SCOPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6979] StringParser("SCOPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6980] StringParser("SCROLL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6981] StringParser("SEARCH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6982] StringParser("SECOND")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6983] StringParser("SECRET_ACCESS_KEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6984] StringParser("SECRET_ARN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6985] StringParser("SECTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6162, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6986] StringParser("SECURITY")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6162, child_count: 7, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6987] Sequence(7 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6988] Ref(SecurityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6169, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6989] Ref(LabelKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6169, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6990] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6171, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6991] Ref(ForKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6171, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6992] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6171, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6993] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6171, child_count: 21, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6994] OneOf(21 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6192, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6995] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6996] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6997] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6194, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6998] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6196, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [6999] Ref(ColumnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6196, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7000] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6196, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7001] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6199, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7002] Ref(AggregateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6199, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7003] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6199, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7004] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6202, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7005] Ref(FunctionParameterListGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6202, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7006] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7007] Ref(DatabaseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7008] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6204, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7009] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7010] Ref(DomainKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7011] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6206, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7012] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6209, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7013] Ref(EventKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6209, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7014] Ref(TriggerKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6209, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7015] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6209, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7016] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7017] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7018] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7019] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6212, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7020] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6215, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7021] Ref(FunctionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6215, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7022] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6215, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7023] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6217, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7024] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6220, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7025] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6220, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7026] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6223, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7027] Ref(LargeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6223, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7028] Ref(ObjectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6223, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7029] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6223, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7030] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7031] Ref(MaterializedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7032] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7033] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6226, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7034] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6229, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7035] Ref(ProceduralKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6229, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7036] Ref(LanguageKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6229, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7037] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6229, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7038] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7039] Ref(ProcedureKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7040] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6232, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7041] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6234, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7042] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6237, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7043] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6237, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7044] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6239, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7045] Ref(PublicationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6239, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7046] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6239, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7047] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6241, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7048] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6241, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7049] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6241, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7050] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6244, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7051] Ref(RoutineKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6244, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7052] Ref(FunctionNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6244, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7053] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6246, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7054] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6249, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7055] Ref(FunctionParameterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6249, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7056] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6251, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7057] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6251, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7058] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6251, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7059] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6253, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7060] Ref(SequenceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6253, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7061] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6253, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7062] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6255, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7063] Ref(SubscriptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6255, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7064] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6255, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7065] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6257, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7066] Ref(TablespaceKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6257, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7067] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6257, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7068] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6259, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7069] Ref(TypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6259, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7070] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6259, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7071] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6261, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7072] Ref(ViewKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6261, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7073] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6261, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7074] Ref(IsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6261, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7075] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7076] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7077] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6263, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7078] StringParser("SEEK")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6263, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7079] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6265, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7080] Ref(WildcardExpressionSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6265, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7081] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6267, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7082] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6267, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7083] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6267, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7084] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6270, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7085] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6272, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7086] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6272, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7087] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6274, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7088] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6274, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7089] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6277, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7090] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6279, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7091] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6279, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7092] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6279, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7093] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7094] Ref(TopKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7095] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6281, child_count: 5, min_times: 0, first_terminator_idx: 58, terminator_count: 12, _padding: 0 }, // [7096] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6286, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7097] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6286, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7098] Ref(SelectClauseModifierSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(13), first_child_idx: 6286, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7099] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6288, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7100] Ref(SelectClauseElementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6288, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7101] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6288, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7102] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6288, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7103] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6288, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7104] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6290, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7105] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6290, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7106] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6290, child_count: 2, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7107] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7108] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7109] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7110] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7111] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7112] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7113] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6292, child_count: 3, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7114] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7115] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6295, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7116] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7117] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7118] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 0, min_times: 0, first_terminator_idx: 58, terminator_count: 0, _padding: 0 }, // [7119] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6295, child_count: 9, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7120] OneOf(9 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6304, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7121] Ref(IntoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(68), first_child_idx: 6304, child_count: 1, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7122] Ref(FromKeywordSegment)
    GrammarInst { variant: GrammarVariant::PrecededBy, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6305, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7123] PrecededBy(preceding_sequences=(('IS', 'DISTINCT'), ('IS', 'NOT', 'DISTINCT')))
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6305, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7124] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6305, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7125] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7126] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7127] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7128] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7129] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7130] Ref(CommaSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7131] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7132] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6307, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7133] StringParser("SELECT")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6307, child_count: 15, min_times: 0, first_terminator_idx: 70, terminator_count: 8, _padding: 0 }, // [7134] Sequence(15 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7135] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7136] Ref(IntoClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7137] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7138] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7139] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7140] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7141] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7142] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7143] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7144] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7145] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7146] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7147] Ref(OffsetClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7148] Ref(FetchClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7149] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7150] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7151] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7152] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6322, child_count: 2, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7153] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7154] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7155] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7156] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7157] Ref(WithCheckOptionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7158] Ref(MetaCommandQueryBufferSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 0, min_times: 0, first_terminator_idx: 70, terminator_count: 0, _padding: 0 }, // [7159] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6324, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7160] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6328, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7161] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6330, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7162] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6333, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7163] Ref(WithCompoundStatementSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6333, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7164] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6335, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7165] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6338, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7166] Ref(WithCompoundNonSelectStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6338, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7167] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6338, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7168] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6341, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7169] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6341, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7170] StringParser("SELECTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6341, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7171] StringParser("SELF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6341, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7172] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6345, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7173] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6345, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7174] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6345, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7175] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6345, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7176] AnyNumberOf(2 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6347, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7177] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7178] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7179] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7180] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7181] StringParser(";")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7182] StringParser("SENSITIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7183] StringParser("SEPARATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6349, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7184] StringParser("SEQUENCE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6349, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7185] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6351, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7186] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6353, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7187] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6353, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7188] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6353, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7189] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6355, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7190] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6355, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7191] Ref(MaxvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6355, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7192] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6357, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7193] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7194] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7195] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6359, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7196] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7197] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7198] Ref(MinvalueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7199] StringParser("SEQUENCEFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7200] StringParser("SEQUENCES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7201] StringParser("SERDE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7202] StringParser("SERDEPROPERTIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7203] StringParser("SERIALIZABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7204] StringParser("SERVER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6361, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7205] StringParser("SERVER_NAME")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6361, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7206] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7207] Ref(UserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7208] Ref(Current_roleKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7209] Ref(Current_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7210] Ref(Session_userKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7211] StringParser("SESSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7212] StringParser("SESSION_TOKEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6365, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7213] StringParser("SESSION_USER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6365, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7214] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6369, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7215] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6369, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7216] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6371, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7217] Ref(SetClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6371, child_count: 1, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7218] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6372, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7219] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6374, child_count: 5, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7220] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6379, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7221] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6379, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7222] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6379, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7223] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6379, child_count: 6, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7224] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7225] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7226] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7227] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7228] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7229] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6385, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7230] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6385, child_count: 1, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7231] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6386, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7232] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6386, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7233] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6389, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7234] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6392, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7235] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6394, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7236] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6394, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7237] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6394, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7238] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6397, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7239] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6399, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7240] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6399, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7241] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6401, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7242] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6403, child_count: 6, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7243] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6409, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7244] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6409, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7245] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6409, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7246] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6409, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7247] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6409, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7248] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6409, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7249] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6409, child_count: 1, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7250] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7251] Ref(ShorthandCastSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6410, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7252] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7253] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7254] Ref(ConstraintsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6414, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7255] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7256] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6416, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7257] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6418, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7258] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6418, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7259] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6420, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7260] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6420, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7261] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6420, child_count: 5, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7262] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6425, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7263] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6425, child_count: 1, min_times: 1, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7264] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6426, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7265] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6428, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7266] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6428, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7267] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6428, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7268] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6428, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7269] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6428, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7270] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6428, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7271] StringParser("SET")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6428, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7272] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7273] Ref(UnionGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7274] Ref(IntersectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7275] Ref(ExceptKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7276] Ref(MinusKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6432, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7277] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7278] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7279] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7280] Ref(IfNotExistsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7281] Ref(SchemaReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6436, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7282] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7283] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6440, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7284] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6442, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7285] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6444, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7286] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6444, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7287] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6444, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7288] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6446, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7289] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7290] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7291] Ref(AuthorizationKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6446, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7292] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7293] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7294] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6448, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7295] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6451, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7296] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6451, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7297] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6453, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7298] Ref(SessionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6453, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7299] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6453, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7300] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6457, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7301] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7302] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6460, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7303] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6462, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7304] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6462, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7305] Ref(EqualsSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6462, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7306] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6464, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7307] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6464, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7308] Delimited(4 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6466, child_count: 4, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7309] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6470, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7310] Ref(LiteralGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6470, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7311] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6470, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7312] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6470, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7313] Ref(OnKeywordAsIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6470, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7314] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7315] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7316] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6473, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7317] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6476, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7318] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6476, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7319] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6476, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7320] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6476, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7321] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6478, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7322] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6478, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7323] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6478, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7324] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6480, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7325] Ref(RoleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6480, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7326] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7327] Ref(NoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7328] Ref(RoleReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7329] StringParser("SETOF")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7330] StringParser("SETS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7331] StringParser("SETTINGS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7332] StringParser("SHAPEFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6482, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7333] StringParser("SHARE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6482, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7334] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6484, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7335] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7336] Ref(Expression_D_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7337] Ref(CaseExpressionSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6486, child_count: 1, min_times: 1, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7338] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6487, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7339] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6490, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7340] Ref(CastOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6490, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7341] Ref(DatatypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6490, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7342] Ref(TimeZoneGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6490, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7343] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6493, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7344] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6493, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7345] Ref(DatasharesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6493, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7346] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6495, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7347] Ref(LikeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6495, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7348] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6495, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7349] StringParser("SHOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6495, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7350] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6498, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7351] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6498, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7352] Ref(ModelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6498, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7353] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6500, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7354] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6500, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7355] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6500, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7356] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6502, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7357] Ref(ShowKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6502, child_count: 7, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7358] OneOf(7 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7359] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7360] Ref(Is_superuserKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7361] Ref(Lc_collateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7362] Ref(Lc_ctypeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7363] Ref(Server_encodingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7364] Ref(Server_versionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7365] Ref(ParameterNameSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6509, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7366] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6511, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7367] Ref(PositiveSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6511, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7368] Ref(NegativeSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6511, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7369] StringParser("SIMILAR")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6511, child_count: 1, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7370] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6512, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7371] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6512, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7372] StringParser("SIMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6512, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7373] StringParser("SIMPLIFY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6512, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7374] StringParser("SIN")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6512, child_count: 3, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7375] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6515, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7376] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6515, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7377] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6515, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7378] Ref(NakedIdentifierFullSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6515, child_count: 2, min_times: 0, first_terminator_idx: 78, terminator_count: 1, _padding: 0 }, // [7379] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6517, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7380] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6517, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7381] Ref(QuotedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6517, child_count: 0, min_times: 0, first_terminator_idx: 78, terminator_count: 0, _padding: 0 }, // [7382] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6517, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7383] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6519, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7384] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6519, child_count: 3, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7385] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7386] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7387] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7388] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::TypedParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7389] TypedParser("single_quote")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7390] StringParser("SINH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6522, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7391] StringParser("SIZE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6522, child_count: 2, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7392] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7393] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7394] Ref(ArrayAccessorSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7395] StringParser("SKIP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7396] StringParser("/")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7397] StringParser(":")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7398] StringParser("SMALLINT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7399] StringParser("SNAPSHOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7400] StringParser("SOME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7401] StringParser("SORT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7402] StringParser("SORTKEY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7403] StringParser("SOURCE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7404] StringParser("SPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7405] StringParser("SPECIFIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7406] StringParser("SPECIFIC_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7407] StringParser("SPECIFICTYPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7408] StringParser("SQL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7409] StringParser("SQLCODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7410] StringParser("SQLERROR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7411] StringParser("SQLEXCEPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7412] StringParser("SQLSTATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7413] StringParser("SQLWARNING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7414] StringParser("SQRT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7415] StringParser("STABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7416] StringParser("STAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7417] StringParser("STAGES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7418] StringParser("STANDALONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7419] StringParser("*")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7420] StringParser("START")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7421] StringParser("STATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6524, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7422] StringParser("STATEMENT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6524, child_count: 147, min_times: 0, first_terminator_idx: 79, terminator_count: 1, _padding: 0 }, // [7423] OneOf(147 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7424] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7425] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7426] Ref(InsertStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7427] Ref(TransactionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7428] Ref(DropTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7429] Ref(DropViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7430] Ref(CreateUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7431] Ref(DropUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7432] Ref(TruncateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7433] Ref(AccessStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7434] Ref(CreateTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7435] Ref(CreateRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7436] Ref(DropRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7437] Ref(AlterTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7438] Ref(CreateSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7439] Ref(SetSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7440] Ref(DropSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7441] Ref(DropTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7442] Ref(CreateDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7443] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7444] Ref(CreateIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7445] Ref(DropIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7446] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7447] Ref(DeleteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7448] Ref(UpdateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7449] Ref(CreateCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7450] Ref(DropCastStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7451] Ref(CreateFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7452] Ref(DropFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7453] Ref(CreateModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7454] Ref(DropModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7455] Ref(DescribeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7456] Ref(UseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7457] Ref(ExplainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7458] Ref(CreateSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7459] Ref(AlterSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7460] Ref(DropSequenceStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7461] Ref(CreateTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7462] Ref(DropTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7463] Ref(AlterDefaultPrivilegesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7464] Ref(DropOwnedStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7465] Ref(ReassignOwnedStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7466] Ref(CommentOnStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7467] Ref(AnalyzeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7468] Ref(CreateTableAsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7469] Ref(AlterTriggerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7470] Ref(SetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7471] Ref(AlterSystemStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7472] Ref(AlterPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7473] Ref(CreatePolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7474] Ref(DropPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7475] Ref(CreateDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7476] Ref(AlterDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7477] Ref(DropDomainStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7478] Ref(CreateMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7479] Ref(AlterMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7480] Ref(DropMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7481] Ref(RefreshMaterializedViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7482] Ref(AlterDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7483] Ref(DropDatabaseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7484] Ref(VacuumStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7485] Ref(AlterFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7486] Ref(CreateViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7487] Ref(AlterViewStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7488] Ref(ListenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7489] Ref(NotifyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7490] Ref(UnlistenStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7491] Ref(LoadStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7492] Ref(ResetStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7493] Ref(DiscardStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7494] Ref(AlterProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7495] Ref(CreateProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7496] Ref(DropProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7497] Ref(CopyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7498] Ref(DoStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7499] Ref(AlterIndexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7500] Ref(ReindexStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7501] Ref(AlterRoleStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7502] Ref(CreateExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7503] Ref(DropExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7504] Ref(AlterExtensionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7505] Ref(CreateSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7506] Ref(AlterSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7507] Ref(DropSubscriptionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7508] Ref(CreatePublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7509] Ref(AlterPublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7510] Ref(DropPublicationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7511] Ref(CreateTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7512] Ref(AlterTypeStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7513] Ref(CreateTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7514] Ref(AlterTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7515] Ref(DropTextSearchConfigurationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7516] Ref(AlterSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7517] Ref(LockTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7518] Ref(ClusterStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7519] Ref(CreateCollationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7520] Ref(DropCollationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7521] Ref(CallStoredProcedureSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7522] Ref(CreateServerStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7523] Ref(CreateUserMappingStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7524] Ref(ImportForeignSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7525] Ref(CreateForeignTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7526] Ref(DropAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7527] Ref(CreateAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7528] Ref(AlterAggregateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7529] Ref(CreateStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7530] Ref(AlterStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7531] Ref(DropStatisticsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7532] Ref(SetConstraintsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7533] Ref(CreateForeignDataWrapperStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7534] Ref(MetaCommandQueryBufferStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7535] Ref(DropForeignTableStatement)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7536] Ref(CreateOperatorStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7537] Ref(AlterForeignTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7538] Ref(SecurityLabelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7539] Ref(PrepareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7540] Ref(ExecuteStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7541] Ref(DeallocateStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7542] Ref(SetSessionAuthorizationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7543] Ref(ResetSessionAuthorizationStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7544] Ref(CreateLibraryStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7545] Ref(CreateGroupStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7546] Ref(AlterUserStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7547] Ref(AlterGroupStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7548] Ref(CreateExternalTableAsStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7549] Ref(CreateExternalTableStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7550] Ref(CreateExternalSchemaStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7551] Ref(DataFormatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7552] Ref(UnloadStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7553] Ref(CopyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7554] Ref(ShowModelStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7555] Ref(CreateDatashareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7556] Ref(DescDatashareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7557] Ref(DropDatashareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7558] Ref(ShowDatasharesStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7559] Ref(AltereDatashareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7560] Ref(DeclareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7561] Ref(FetchStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7562] Ref(CloseStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7563] Ref(AnalyzeCompressionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7564] Ref(AlterProcedureStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7565] Ref(CallStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7566] Ref(CreateRlsPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7567] Ref(ManageRlsPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7568] Ref(DropRlsPolicyStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7569] Ref(CreateExternalFunctionStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7570] Ref(GrantUsageDatashareStatementSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 79, terminator_count: 0, _padding: 0 }, // [7571] Ref(DelimiterGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7572] StringParser("STATIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7573] StringParser("STATISTICS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7574] StringParser("STATUPDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7575] StringParser("STDDEV_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7576] StringParser("STDDEV_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7577] StringParser("STDIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7578] StringParser("STDOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7579] StringParser("STORAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7580] StringParser("STORED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7581] StringParser("STREAM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7582] StringParser("STREAMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6671, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7583] StringParser("STRICT")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6671, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7584] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6673, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7585] Ref(ConcatSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6673, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7586] Ref(CollateKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6673, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7587] StringParser("STRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6673, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7588] StringParser("STRIP")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6673, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7589] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6676, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7590] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6678, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7591] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7592] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7593] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7594] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7595] StringParser("STRUCTURE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7596] StringParser("STYLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7597] StringParser("SUBCLASS_ORIGIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7598] StringParser("SUBMULTISET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7599] StringParser("SUBSCRIPTION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7600] StringParser("SUBSET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7601] StringParser("SUBSTRING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7602] StringParser("SUBSTRING_REGEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7603] StringParser("SUCCEEDS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7604] StringParser("SUM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7605] StringParser("SUPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7606] StringParser("SUPERUSER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7607] StringParser("SUPPORT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7608] Token("symbol")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7609] StringParser("SYMMETRIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7610] StringParser("SYSDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7611] StringParser("SYSID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7612] StringParser("SYSLOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7613] StringParser("SYSTEM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7614] StringParser("SYSTEM_TIME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7615] StringParser("SYSTEM_USER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6680, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7616] StringParser("T")
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6680, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7617] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6684, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7618] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6686, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7619] Ref(DiststyleKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6686, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7620] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7621] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7622] Ref(EvenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7623] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6690, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7624] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6690, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7625] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6692, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7626] Ref(DistkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6692, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7627] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6695, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7628] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6695, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7629] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6697, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7630] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6700, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7631] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6702, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7632] Ref(CompoundKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6702, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7633] Ref(InterleavedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6702, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7634] Ref(SortkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6702, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7635] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6705, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7636] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6707, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7637] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6707, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7638] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6709, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7639] Ref(SortkeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6709, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7640] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6709, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7641] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6711, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7642] Ref(EncodeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6711, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7643] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6711, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7644] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6713, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7645] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6715, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7646] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6715, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7647] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6715, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7648] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6718, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7649] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6720, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7650] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6720, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7651] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6723, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7652] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6725, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7653] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6725, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7654] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6728, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7655] Ref(PrimaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6728, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7656] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6728, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7657] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6731, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7658] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6733, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7659] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6733, child_count: 6, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7660] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6739, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7661] Ref(ForeignKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6739, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7662] Ref(KeyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6739, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7663] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6742, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7664] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6744, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7665] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6744, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7666] Ref(ReferencesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6744, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7667] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6744, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7668] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6747, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7669] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6749, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7670] Ref(ColumnReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6749, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7671] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6753, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7672] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7673] Ref(ConstraintKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7674] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6755, child_count: 4, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7675] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6759, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7676] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6761, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7677] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6761, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7678] Ref(PrimaryKeyGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6761, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7679] Ref(UsingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6761, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7680] Ref(IndexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6761, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7681] Ref(IndexReferenceSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6761, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7682] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6763, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7683] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6763, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7684] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6765, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7685] Ref(NotKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6765, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7686] Ref(DeferrableKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6765, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7687] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6767, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7688] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6769, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7689] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6769, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7690] Ref(DeferredKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6769, child_count: 2, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7691] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6771, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7692] Ref(InitiallyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6771, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7693] Ref(ImmediateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6771, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7694] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6771, child_count: 8, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7695] OneOf(8 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6779, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7696] Ref(ValuesClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6779, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7697] Ref(BareFunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6779, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7698] Ref(FunctionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6779, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7699] Ref(ObjectUnpivotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6779, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7700] Ref(ArrayUnnestSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6779, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7701] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6779, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7702] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7703] Ref(SelectableGrammar)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6782, child_count: 3, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7704] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7705] Ref(MergeStatementSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7706] StringParser("TABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7707] StringParser("TABLE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7708] StringParser("TABLES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7709] StringParser("TABLESAMPLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7710] StringParser("TABLESPACE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6785, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7711] StringParser("TAG")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6785, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7712] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6787, child_count: 1, min_times: 0, first_terminator_idx: 80, terminator_count: 1, _padding: 0 }, // [7713] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7714] Ref(Expression_A_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 0, min_times: 0, first_terminator_idx: 80, terminator_count: 0, _padding: 0 }, // [7715] Ref(BinaryOperatorGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7716] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6788, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7717] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6790, child_count: 1, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7718] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7719] Ref(Expression_B_Unary_Operator_Grammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7720] Ref(Expression_C_Grammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7721] StringParser("TAN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7722] StringParser("TANH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7723] StringParser("TARGET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7724] StringParser("TASK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7725] StringParser("TASKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7726] StringParser("TB")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7727] StringParser("TDES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7728] StringParser("TEMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7729] StringParser("TEMPLATE")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6791, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7730] Nothing
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6791, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7731] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7732] Ref(TempKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7733] Ref(TemporaryKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6793, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7734] StringParser("TEMPORARY")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6793, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7735] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7736] Ref(TransientKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7737] Ref(TemporaryGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7738] StringParser("TERMINATED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7739] StringParser("TEXT255")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7740] StringParser("TEXT32K")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7741] StringParser("TEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7742] StringParser("TEXTFILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7743] StringParser("THEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7744] StringParser("THROUGH")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7745] StringParser("TIES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7746] StringParser("~")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6795, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7747] StringParser("TIME")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6795, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7748] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6798, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7749] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6800, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7750] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6800, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7751] Ref(TimestampKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6800, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7752] Ref(BracketedArguments)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6800, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7753] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6803, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7754] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7755] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7756] Ref(WithoutKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7757] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6805, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7758] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6805, child_count: 1, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7759] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6806, child_count: 4, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7760] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7761] Ref(AtKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7762] Ref(TimeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7763] Ref(ZoneKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7764] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7765] StringParser("TIMEFORMAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7766] StringParser("TIMEOUT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7767] StringParser("TIMESTAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7768] StringParser("TIMESTAMPTZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7769] StringParser("TIMETZ")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7770] StringParser("TIMEZONE_HOUR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7771] StringParser("TIMEZONE_MINUTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7772] StringParser("TO")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7773] StringParser("TOKEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7774] StringParser("TOP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7775] StringParser("TOP_LEVEL_COUNT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7776] StringParser("TRAILING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6810, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7777] StringParser("TRANSACTION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6810, child_count: 4, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7778] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6814, child_count: 6, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7779] OneOf(6 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6820, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7780] Ref(BeginKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6820, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7781] Ref(StartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6820, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7782] Ref(CommitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6820, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7783] Ref(EndKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6820, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7784] Ref(RollbackKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6820, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7785] Ref(AbortKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6820, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7786] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6822, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7787] Ref(TransactionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6822, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7788] Ref(WorkKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6822, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7789] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7790] Ref(IsolationKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7791] Ref(LevelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6825, child_count: 4, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7792] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6829, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7793] Ref(SerializableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6829, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7794] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7795] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7796] Ref(CommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6831, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7797] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7798] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7799] Ref(UncommittedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6833, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7800] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6835, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7801] Ref(RepeatableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6835, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7802] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6835, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7803] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6837, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7804] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6839, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7805] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6839, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7806] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6839, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7807] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7808] Ref(ReadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7809] Ref(WriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7810] StringParser("TRANSACTION_ACTIVE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7811] StringParser("TRANSACTIONS_COMMITTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7812] StringParser("TRANSACTIONS_ROLLED_BACK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7813] StringParser("TRANSFORM")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7814] StringParser("TRANSFORMS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7815] StringParser("TRANSIENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7816] StringParser("TRANSLATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7817] StringParser("TRANSLATE_REGEX")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7818] StringParser("TRANSLATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7819] StringParser("TREAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7820] StringParser("TRIGGER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7821] StringParser("TRIGGER_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7822] StringParser("TRIGGER_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7823] StringParser("TRIGGER_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6841, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7824] StringParser("TRIM")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6841, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7825] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7826] Ref(BothKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7827] Ref(LeadingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7828] Ref(TrailingKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7829] StringParser("TRIM_ARRAY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7830] StringParser("TRIMBLANKS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7831] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7832] StringParser("TRUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6844, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7833] StringParser("TRUNCATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6844, child_count: 5, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7834] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7835] Ref(TruncateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6849, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7836] Ref(TableKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6849, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7837] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6851, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7838] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6853, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7839] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7840] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6855, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7841] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6855, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7842] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6857, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7843] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6857, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7844] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6857, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7845] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6859, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7846] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6861, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7847] Ref(RestartKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6861, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7848] Ref(ContinueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6861, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7849] Ref(IdentityKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6861, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7850] Ref(DropBehaviorGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6861, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7851] StringParser("TRUNCATECOLUMNS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6861, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7852] StringParser("TRUSTED")
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6861, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7853] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6864, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7854] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7855] Ref(BaseExpressionElementGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6866, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7856] StringParser("TYPE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6866, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7857] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6869, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7858] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6871, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7859] Ref(VariadicKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6871, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7860] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6873, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7861] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6873, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7862] Ref(WalrusOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6873, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7863] Ref(ArrayTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6873, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7864] Ref(ArrayLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6873, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7865] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7866] Ref(StructTypeSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7867] Ref(StructLiteralSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7868] StringParser("TYPES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7869] StringParser("UESCAPE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7870] StringParser("UNBOUNDED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7871] StringParser("UNCOMMITTED")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6875, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7872] Ref(CrossKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6875, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7873] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6878, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7874] Ref(NaturalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6878, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7875] Ref(UnconditionalCrossJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6878, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7876] Ref(HorizontalJoinKeywordsGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6878, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7877] StringParser("UNCONDITIONAL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6878, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7878] StringParser("UNDER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6878, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7879] StringParser("UNENCRYPTED")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6878, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7880] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6880, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7881] Ref(UnionKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6880, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7882] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6882, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7883] Ref(DistinctKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6882, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7884] Ref(AllKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6882, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7885] StringParser("UNION")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6882, child_count: 1, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7886] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7887] Ref(UniqueKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7888] StringParser("UNIQUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7889] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7890] StringParser("UNKNOWN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7891] StringParser("UNLIMITED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7892] StringParser("UNLINK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6883, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7893] StringParser("UNLISTEN")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6883, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7894] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6885, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7895] Ref(UnlistenKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6885, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7896] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6887, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7897] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6887, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7898] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6887, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7899] StringParser("UNLOAD")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6887, child_count: 5, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7900] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6892, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7901] Ref(UnloadKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6892, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7902] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7903] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7904] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6895, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7905] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6895, child_count: 18, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7906] AnySetOf(18 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6913, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7907] Ref(AuthorizationSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6913, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7908] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7909] Ref(RegionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7910] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7911] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6916, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7912] Ref(CompressionTypeGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6916, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7913] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6918, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7914] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6920, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7915] Ref(FormatKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6920, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7916] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6920, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7917] OneOf(3 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6923, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7918] Ref(CsvKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6923, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7919] Ref(JsonKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6923, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7920] Ref(ParquetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6923, child_count: 4, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7921] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6927, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7922] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6927, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7923] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6927, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7924] Ref(BracketedColumnReferenceListGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6927, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7925] Ref(IncludeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6927, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7926] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6929, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7927] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6929, child_count: 5, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7928] OneOf(5 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6934, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7929] Ref(PresetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6934, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7930] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6934, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7931] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6934, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7932] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6934, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7933] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6934, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7934] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6937, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7935] Ref(ExtensionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6937, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7936] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6937, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7937] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6939, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7938] Ref(ParallelKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6939, child_count: 4, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7939] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6943, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7940] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6943, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7941] Ref(OffKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6943, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7942] Ref(TrueKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6943, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7943] Ref(FalseKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6943, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7944] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6945, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7945] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6948, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7946] Ref(DelimiterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6948, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7947] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6948, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7948] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6948, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7949] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6951, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7950] Ref(FixedwidthKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6951, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7951] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6951, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7952] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6951, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7953] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6953, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7954] Ref(ManifestKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6953, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7955] Ref(VerboseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6953, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7956] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6956, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7957] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6956, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7958] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6956, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7959] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6956, child_count: 3, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7960] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6959, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7961] Ref(NullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6959, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7962] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6959, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7963] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6959, child_count: 4, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7964] AnySetOf(4 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6963, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7965] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6965, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7966] Ref(MaxfilesizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6965, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7967] Ref(RowgroupsizeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6965, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7968] Ref(AsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6965, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7969] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6965, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7970] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6967, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7971] Ref(MbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6967, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7972] Ref(GbKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6967, child_count: 2, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7973] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7974] Ref(EncryptedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7975] Ref(AutoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7976] Ref(AllowoverwriteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7977] Ref(CleanpathKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7978] Ref(EscapeKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7979] Ref(AddquotesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7980] Ref(HeaderKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7981] StringParser("UNLOGGED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7982] StringParser("UNMATCHED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7983] StringParser("UNNAMED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6969, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7984] StringParser("UNNEST")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::GreedyOnceStarted, flags: GrammarFlags::from_bits(4), first_child_idx: 6969, child_count: 8, min_times: 0, first_terminator_idx: 81, terminator_count: 5, _padding: 0 }, // [7985] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7986] Ref(SelectClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7987] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7988] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7989] Ref(GroupByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7990] Ref(HavingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7991] Ref(QualifyClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7992] Ref(OverlapsClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7993] Ref(NamedWindowSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7994] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7995] Ref(WithNoSchemaBindingClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7996] Ref(WithDataClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7997] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 0, min_times: 0, first_terminator_idx: 81, terminator_count: 0, _padding: 0 }, // [7998] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6977, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [7999] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6979, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8000] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6979, child_count: 1, min_times: 1, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8001] AnyNumberOf(1 elements, min=1)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6980, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8002] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8003] Ref(SetOperatorSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8004] Ref(NonSetSelectableGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8005] StringParser("UNPIVOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8006] StringParser("UNRESTRICTED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8007] StringParser("UNSAFE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8008] StringParser("UNSIGNED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8009] StringParser("UNTIL")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8010] StringParser("UNTYPED")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 6982, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8011] StringParser("UPDATE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6982, child_count: 10, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8012] Sequence(10 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6992, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8013] Ref(UpdateKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6992, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8014] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6992, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8015] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 6992, child_count: 1, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8016] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6993, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8017] Ref(SetKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6993, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8018] Ref(SetClauseListSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6993, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8019] Ref(FromClauseSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6993, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8020] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6995, child_count: 4, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8021] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6999, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8022] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6999, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8023] Ref(CurrentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6999, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8024] Ref(OfKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6999, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8025] Ref(ObjectReferenceSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 6999, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8026] Ref(WhereClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 6999, child_count: 4, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8027] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7003, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8028] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7003, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8029] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7005, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8030] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7005, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8031] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7007, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8032] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7009, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8033] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7009, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8034] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7009, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8035] StringParser("UPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7009, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8036] StringParser("URI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7009, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8037] StringParser("USAGE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7009, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8038] StringParser("USE")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7009, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8039] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8040] Ref(UseKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8041] Ref(DatabaseReferenceSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8042] StringParser("USE_ANY_ROLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8043] StringParser("USER")
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8044] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8045] StringParser("USER_DEFINED_TYPE_CATALOG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8046] StringParser("USER_DEFINED_TYPE_CODE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8047] StringParser("USER_DEFINED_TYPE_NAME")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8048] StringParser("USER_DEFINED_TYPE_SCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8049] StringParser("USER_LIMIT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8050] StringParser("USING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8051] StringParser("UTF16")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8052] StringParser("UTF16BE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8053] StringParser("UTF16LE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8054] StringParser("UTF32")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8055] StringParser("UTF8")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7011, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8056] StringParser("VACUUM")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7011, child_count: 5, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8057] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7016, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8058] Ref(VacuumKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7016, child_count: 4, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8059] OneOf(4 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7020, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8060] Ref(FullKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7020, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8061] Ref(ReindexKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7020, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8062] Ref(ReclusterKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7020, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8063] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7022, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8064] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7024, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8065] Ref(SortKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7024, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8066] Ref(DeleteKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7024, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8067] Ref(OnlyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7024, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8068] Ref(TableReferenceSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7024, child_count: 3, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8069] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8070] Ref(ToKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8071] Ref(NumericLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8072] Ref(PercentKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8073] Ref(BoostKeywordSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8074] StringParser("VALID")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8075] StringParser("VALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8076] StringParser("VALIDATOR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8077] StringParser("VALUE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7027, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8078] StringParser("VALUE_OF")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7027, child_count: 5, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8079] Sequence(5 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7032, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8080] Ref(ValuesKeywordSegment)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7032, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8081] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Greedy, flags: GrammarFlags::from_bits(4), first_child_idx: 7034, child_count: 3, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8082] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7037, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8083] Delimited(2 elements via OneOf)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7039, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8084] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8085] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8086] Ref(DefaultKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8087] Ref(AliasExpressionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8088] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8089] Ref(LimitClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8090] StringParser("VALUES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8091] StringParser("VAR_POP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8092] StringParser("VAR_SAMP")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8093] StringParser("VARBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8094] StringParser("VARBYTE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8095] StringParser("VARCHAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8096] StringParser("VARIADIC")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8097] StringParser("VARYING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7041, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8098] StringParser("VERBOSE")
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7041, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8099] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8100] Ref(QuotedLiteralSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8101] Ref(NakedIdentifierSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8102] StringParser("VERSION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8103] StringParser("VERSIONING")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8104] StringParser("VIA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8105] StringParser("VIEW")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8106] StringParser("VIEWS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8107] StringParser("VOLATILE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8108] StringParser("WALLET")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8109] StringParser(":=")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8110] StringParser("WAREHOUSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8111] StringParser("WEEK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8112] StringParser("WEEKDAY")
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7043, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8113] Nothing
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7043, child_count: 8, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8114] Sequence(8 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7051, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8115] Ref(WhenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7051, child_count: 3, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8116] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8117] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8118] Meta(conditional: indented_then=True -> indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8119] Ref(ThenKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8120] Meta(conditional: indented_then_contents=True -> implicit_indent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8121] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8122] Meta(conditional: indented_then_contents=True -> dedent)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8123] Meta(conditional: indented_then=True -> dedent)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8124] StringParser("WHEN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7054, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8125] StringParser("WHENEVER")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7054, child_count: 4, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8126] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7058, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8127] Ref(WhereKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7058, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8128] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7060, child_count: 3, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8129] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7063, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8130] Ref(ExpressionSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7063, child_count: 10, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8131] OneOf(10 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7073, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8132] Ref(LimitKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7073, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8133] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7075, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8134] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7075, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8135] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7075, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8136] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8137] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8138] Ref(ByKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8139] Ref(HavingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8140] Ref(QualifyKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8141] Ref(WindowKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8142] Ref(OverlapsKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8143] Ref(ReturningKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7077, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8144] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8145] Ref(OnKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8146] Ref(ConflictKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8147] Ref(ForClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8148] StringParser("WHERE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8149] StringParser("WHITESPACE")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8150] Token("whitespace")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7079, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8151] StringParser("WIDTH_BUCKET")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7079, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8152] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7081, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8153] Ref(WildcardIdentifierSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7081, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8154] Ref(ExcludeClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7081, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8155] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::AnyNumberOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7083, child_count: 1, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8156] AnyNumberOf(1 elements, min=0)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7084, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8157] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7086, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8158] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7088, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8159] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7088, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8160] Ref(ObjectReferenceDelimiterGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7088, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8161] Sequence(2 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7090, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8162] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7090, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8163] Ref(DotSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7090, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8164] Ref(StarSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7090, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8165] StringParser("WINDOW")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7090, child_count: 4, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8166] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(69), first_child_idx: 7094, child_count: 1, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8167] Ref(SingleIdentifierGrammar)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7095, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8168] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8169] Ref(PartitionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8170] Ref(OrderKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8171] Ref(PartitionClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8172] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7097, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8173] Ref(FrameClauseSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7097, child_count: 4, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8174] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7101, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8175] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::OneOf, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7101, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8176] OneOf(2 options)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7103, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8177] Ref(CascadedKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7103, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8178] Ref(LocalKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7103, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8179] Ref(CheckKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7103, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8180] Ref(OptionKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7103, child_count: 6, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8181] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7109, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8182] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7109, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8183] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7109, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8184] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7109, child_count: 2, min_times: 0, first_terminator_idx: 86, terminator_count: 1, _padding: 0 }, // [8185] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8186] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 86, terminator_count: 0, _padding: 0 }, // [8187] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8188] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8189] Ref(NonWithNonSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7111, child_count: 6, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8190] Sequence(6 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7117, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8191] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7117, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8192] Ref(RecursiveKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7117, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8193] Meta(conditional: indented_ctes=True -> indent)
    GrammarInst { variant: GrammarVariant::Delimited, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7117, child_count: 2, min_times: 0, first_terminator_idx: 87, terminator_count: 1, _padding: 0 }, // [8194] Delimited(1 element)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7119, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8195] Ref(CTEDefinitionSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7119, child_count: 0, min_times: 0, first_terminator_idx: 87, terminator_count: 0, _padding: 0 }, // [8196] Ref(SelectKeywordSegment)
    GrammarInst { variant: GrammarVariant::Meta, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(256), first_child_idx: 7119, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8197] Meta(conditional: indented_ctes=True -> dedent)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7119, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8198] Ref(NonWithSelectableGrammar)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7119, child_count: 3, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8199] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7122, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8200] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7122, child_count: 1, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8201] Sequence(1 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7123, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8202] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7123, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8203] Ref(DataKeywordSegment)
    GrammarInst { variant: GrammarVariant::Nothing, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7123, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8204] Nothing
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7123, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8205] StringParser("WITH")
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7123, child_count: 4, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8206] Sequence(4 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7127, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8207] Ref(WithKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7127, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8208] Ref(NoKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7127, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8209] Ref(SchemaKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7127, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8210] Ref(BindingKeywordSegment)
    GrammarInst { variant: GrammarVariant::Sequence, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7127, child_count: 3, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8211] Sequence(3 elements)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7130, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8212] Ref(WithinKeywordSegment)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7130, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8213] Ref(GroupKeywordSegment)
    GrammarInst { variant: GrammarVariant::Bracketed, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(4), first_child_idx: 7130, child_count: 3, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8214] Bracketed(1 elements -> 1 child)
    GrammarInst { variant: GrammarVariant::Ref, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(5), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8215] Ref(OrderByClauseSegment)
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8216] StringParser("WITHIN")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8217] StringParser("WITHOUT")
    GrammarInst { variant: GrammarVariant::Token, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8218] Token("word")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8219] StringParser("WORK")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8220] StringParser("WRAPPER")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8221] StringParser("WRITE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8222] StringParser("XGBOOST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8223] StringParser("XML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8224] StringParser("XMLAGG")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8225] StringParser("XMLATTRIBUTES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8226] StringParser("XMLBINARY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8227] StringParser("XMLCAST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8228] StringParser("XMLCOMMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8229] StringParser("XMLCONCAT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8230] StringParser("XMLDECLARATION")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8231] StringParser("XMLDOCUMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8232] StringParser("XMLELEMENT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8233] StringParser("XMLEXISTS")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8234] StringParser("XMLFOREST")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8235] StringParser("XMLITERATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8236] StringParser("XMLNAMESPACES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8237] StringParser("XMLPARSE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8238] StringParser("XMLPI")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8239] StringParser("XMLQUERY")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8240] StringParser("XMLROOT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8241] StringParser("XMLSCHEMA")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8242] StringParser("XMLSERIALIZE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8243] StringParser("XMLTABLE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8244] StringParser("XMLTEXT")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8245] StringParser("XMLVALIDATE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8246] StringParser("YAML")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8247] StringParser("YEAR")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8248] StringParser("YES")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8249] StringParser("ZONE")
    GrammarInst { variant: GrammarVariant::StringParser, parse_mode: ParseMode::Strict, flags: GrammarFlags::from_bits(0), first_child_idx: 7133, child_count: 0, min_times: 0, first_terminator_idx: 88, terminator_count: 0, _padding: 0 }, // [8250] StringParser("ZSTD")
];

pub static CHILD_IDS: &[u32] = &[
    9, 10, 65, 11, 55, 12, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24,
    25, 32, 37, 42, 43, 48, 13, 14, 26, 27, 28, 29, 30, 31, 33, 34,
    35, 36, 38, 39, 40, 41, 44, 45, 46, 47, 49, 50, 51, 52, 53, 54,
    56, 62, 57, 58, 61, 59, 60, 66, 67, 68, 70, 78, 82, 85, 88, 93,
    71, 72, 73, 74, 75, 76, 77, 79, 80, 81, 83, 84, 86, 87, 89, 90,
    91, 92, 94, 137, 95, 104, 109, 112, 113, 114, 115, 116, 117, 118, 119, 120,
    121, 122, 123, 124, 125, 126, 127, 128, 129, 130, 131, 132, 133, 134, 96, 97,
    98, 99, 102, 103, 100, 101, 105, 106, 107, 108, 110, 111, 135, 136, 139, 62,
    142, 143, 144, 145, 146, 147, 148, 149, 150, 151, 152, 155, 158, 161, 153, 154,
    156, 157, 159, 160, 162, 163, 165, 166, 167, 168, 169, 170, 171, 172, 173, 174,
    175, 178, 181, 184, 176, 177, 179, 180, 182, 183, 185, 186, 188, 189, 191, 195,
    199, 203, 207, 209, 211, 192, 193, 194, 62, 196, 197, 198, 62, 200, 201, 202,
    62, 204, 205, 206, 62, 208, 62, 210, 62, 214, 215, 224, 225, 226, 234, 235,
    236, 250, 237, 243, 238, 239, 240, 241, 242, 244, 245, 246, 241, 242, 247, 62,
    248, 249, 252, 253, 259, 260, 261, 262, 267, 263, 241, 242, 264, 265, 266, 268,
    272, 280, 269, 270, 271, 273, 274, 275, 276, 277, 278, 279, 281, 282, 283, 285,
    286, 287, 288, 289, 302, 306, 314, 318, 334, 290, 291, 292, 295, 299, 293, 294,
    296, 297, 298, 300, 301, 303, 304, 305, 307, 308, 309, 310, 311, 312, 313, 315,
    316, 317, 319, 320, 321, 322, 331, 323, 326, 324, 325, 327, 328, 329, 330, 332,
    333, 335, 336, 337, 338, 340, 341, 342, 343, 344, 345, 348, 346, 62, 349, 350,
    351, 353, 356, 354, 355, 357, 62, 358, 359, 360, 361, 362, 363, 364, 365, 366,
    367, 370, 371, 375, 376, 377, 378, 379, 383, 372, 373, 374, 380, 62, 385, 386,
    387, 388, 389, 390, 391, 392, 394, 395, 396, 397, 407, 414, 398, 399, 402, 400,
    401, 403, 62, 408, 409, 410, 411, 62, 415, 416, 418, 421, 419, 420, 423, 424,
    425, 426, 427, 431, 434, 440, 455, 463, 469, 473, 481, 485, 428, 429, 430, 432,
    433, 435, 438, 439, 436, 437, 441, 442, 445, 452, 443, 444, 446, 449, 447, 448,
    450, 451, 453, 454, 456, 457, 458, 459, 460, 461, 462, 464, 465, 466, 467, 468,
    470, 471, 472, 474, 475, 476, 477, 478, 479, 480, 482, 483, 484, 486, 487, 488,
    490, 491, 492, 493, 494, 499, 505, 495, 496, 497, 498, 500, 501, 502, 503, 504,
    506, 509, 507, 508, 510, 544, 552, 558, 565, 574, 592, 511, 543, 512, 515, 516,
    517, 518, 521, 525, 528, 531, 532, 533, 534, 542, 513, 514, 519, 520, 522, 523,
    524, 526, 527, 529, 530, 535, 536, 537, 538, 539, 540, 541, 545, 551, 546, 549,
    550, 547, 548, 553, 554, 555, 556, 241, 242, 557, 559, 560, 561, 241, 242, 562,
    563, 564, 566, 570, 567, 568, 569, 571, 62, 572, 573, 575, 576, 577, 585, 578,
    579, 580, 241, 242, 581, 62, 582, 583, 584, 586, 589, 590, 591, 587, 588, 593,
    594, 595, 596, 598, 609, 615, 712, 715, 718, 734, 738, 744, 753, 760, 772, 783,
    787, 793, 797, 802, 805, 808, 812, 815, 818, 826, 837, 599, 600, 601, 602, 603,
    604, 607, 605, 606, 608, 610, 611, 612, 613, 614, 616, 617, 618, 619, 620, 633,
    642, 645, 651, 655, 668, 686, 690, 694, 697, 700, 708, 621, 624, 625, 626, 629,
    622, 623, 627, 628, 630, 631, 632, 634, 635, 636, 637, 638, 639, 640, 641, 643,
    644, 646, 649, 650, 647, 648, 652, 653, 654, 656, 657, 658, 663, 664, 665, 659,
    660, 661, 662, 666, 241, 242, 667, 669, 670, 678, 681, 671, 672, 673, 674, 675,
    676, 677, 679, 680, 682, 683, 684, 685, 687, 688, 689, 691, 692, 693, 695, 696,
    698, 699, 701, 702, 703, 704, 705, 706, 707, 709, 710, 711, 713, 714, 716, 717,
    719, 720, 721, 722, 727, 723, 724, 725, 726, 728, 731, 729, 730, 732, 733, 735,
    736, 737, 739, 740, 741, 742, 743, 745, 748, 749, 746, 747, 750, 751, 752, 754,
    755, 758, 759, 756, 757, 761, 770, 771, 762, 763, 764, 767, 765, 766, 768, 769,
    773, 780, 781, 782, 774, 775, 776, 777, 778, 779, 784, 785, 786, 788, 789, 790,
    791, 792, 794, 795, 796, 798, 799, 800, 801, 803, 804, 806, 807, 809, 810, 811,
    813, 814, 816, 817, 819, 820, 821, 822, 823, 824, 825, 827, 828, 829, 830, 831,
    835, 836, 832, 833, 834, 838, 842, 843, 839, 840, 841, 844, 241, 242, 845, 62,
    846, 850, 851, 847, 848, 849, 853, 854, 855, 856, 857, 858, 859, 860, 861, 862,
    864, 870, 863, 62, 865, 866, 867, 868, 869, 871, 872, 873, 875, 937, 876, 889,
    893, 896, 902, 908, 911, 914, 917, 932, 877, 882, 888, 878, 879, 880, 881, 883,
    884, 885, 886, 887, 890, 891, 892, 894, 895, 897, 898, 899, 900, 901, 903, 904,
    905, 906, 907, 909, 910, 912, 913, 915, 916, 918, 919, 920, 921, 929, 922, 925,
    923, 924, 926, 927, 928, 930, 931, 933, 934, 935, 936, 939, 940, 941, 945, 942,
    62, 943, 944, 946, 947, 951, 955, 965, 948, 949, 950, 952, 953, 954, 956, 957,
    958, 959, 962, 963, 964, 960, 961, 966, 967, 968, 969, 970, 972, 973, 974, 975,
    976, 983, 977, 980, 981, 978, 979, 982, 62, 984, 985, 986, 988, 989, 990, 991,
    1034, 992, 993, 994, 995, 999, 1003, 1007, 1013, 1022, 1027, 996, 997, 998, 1000, 1001,
    1002, 1004, 1005, 1006, 1008, 1009, 1010, 1011, 1012, 1014, 1015, 1016, 241, 242, 1017, 62,
    1018, 1019, 1020, 1021, 1023, 1024, 1025, 241, 242, 1026, 62, 1028, 1029, 1030, 1031, 1032,
    1033, 1035, 1036, 1037, 1038, 1039, 1044, 1045, 1046, 1047, 1040, 1041, 1042, 1043, 62, 1050,
    1084, 1088, 1092, 1101, 1106, 1051, 1052, 1053, 1054, 1055, 1059, 1067, 1072, 1080, 1056, 1057,
    1058, 1060, 1061, 1062, 241, 242, 1063, 62, 1064, 1065, 1066, 1068, 1069, 1070, 241, 242,
    1071, 62, 1073, 1074, 1075, 1076, 1077, 1078, 1079, 1081, 1082, 1083, 1085, 1086, 1087, 1089,
    1090, 1091, 1093, 1094, 1095, 241, 242, 1096, 62, 1097, 1098, 1099, 1100, 1102, 1103, 1104,
    241, 242, 1105, 62, 1107, 1108, 1109, 1110, 1111, 1112, 1113, 1115, 1116, 1117, 1118, 1119,
    1140, 1147, 1120, 1121, 1122, 1123, 1125, 1132, 1136, 1124, 62, 1126, 1127, 1129, 1130, 1131,
    1128, 1133, 1134, 1135, 1137, 1138, 1139, 1141, 1142, 1143, 1144, 1145, 1146, 1148, 1149, 1150,
    1151, 1152, 1157, 1158, 1159, 1160, 1153, 1154, 1155, 1156, 62, 1161, 1163, 1164, 1165, 1166,
    1167, 1168, 1169, 1173, 1170, 1171, 1172, 1174, 1183, 1187, 1175, 1176, 1177, 62, 1178, 1179,
    1180, 1181, 1182, 1184, 1185, 1186, 241, 242, 1188, 1189, 1190, 1191, 241, 242, 1193, 1220,
    1194, 1200, 1215, 1195, 1196, 1197, 1198, 1199, 1201, 1202, 1203, 1204, 1212, 1205, 1208, 1206,
    1207, 1209, 1210, 1211, 1213, 1214, 1216, 1217, 1218, 1219, 1222, 1223, 1224, 1225, 1226, 1227,
    1231, 1228, 1229, 1230, 1232, 1233, 1234, 1235, 1238, 1239, 1236, 1237, 1241, 1242, 1243, 1244,
    1245, 1248, 1252, 1256, 1260, 1264, 1246, 1247, 1249, 1250, 1251, 62, 1253, 1254, 1255, 62,
    1257, 1258, 1259, 62, 1261, 1262, 1263, 1265, 1266, 1267, 1268, 1269, 1270, 1271, 1273, 1274,
    1277, 1275, 1276, 1278, 1321, 1327, 1279, 1284, 1285, 1280, 1281, 1282, 1283, 1286, 1289, 1292,
    1295, 1298, 1301, 1304, 1307, 1311, 1317, 1287, 1288, 1290, 1291, 1293, 1294, 1296, 1297, 1299,
    1300, 1302, 1303, 1305, 1306, 1308, 1309, 1310, 1312, 1313, 1314, 1315, 1316, 1318, 1319, 1320,
    1322, 1323, 1324, 1325, 1326, 1328, 1334, 1338, 1329, 1330, 1331, 1332, 1333, 1335, 1336, 1337,
    1339, 1357, 1340, 1341, 1342, 1343, 1354, 1344, 1347, 1345, 1346, 1348, 1349, 1350, 62, 1351,
    1352, 1353, 1355, 1356, 1358, 1359, 1360, 1361, 1363, 1364, 1365, 1366, 1367, 1371, 1375, 1368,
    1369, 1370, 1372, 1373, 1374, 1377, 1380, 1385, 1393, 1401, 1405, 1410, 1415, 1418, 1421, 1378,
    1379, 1381, 1382, 1383, 1384, 1386, 1390, 1387, 1388, 1389, 1391, 1392, 1394, 1398, 1395, 1396,
    1397, 1399, 1400, 1402, 1403, 1404, 1406, 1407, 1408, 1409, 1411, 1412, 1413, 1414, 1416, 1417,
    1419, 1420, 1422, 1423, 1424, 1425, 1426, 1428, 1429, 1430, 1431, 1432, 1433, 1435, 1442, 1446,
    1434, 1436, 1437, 1438, 1439, 1440, 1441, 1443, 1444, 1445, 1447, 1448, 1449, 1451, 1452, 1453,
    1454, 1455, 1456, 1466, 1470, 1457, 1458, 1459, 1460, 1463, 1464, 1465, 1461, 1462, 1467, 1468,
    1469, 1471, 1472, 1473, 1476, 1474, 1475, 1477, 1478, 1480, 1481, 1482, 1483, 1484, 1487, 1498,
    1504, 1505, 1506, 1509, 1515, 1523, 1485, 1486, 1488, 1492, 1493, 1495, 1489, 1490, 1491, 1494,
    62, 1496, 1497, 1499, 1500, 1501, 1502, 1503, 1507, 1508, 1510, 1511, 1512, 1513, 1514, 241,
    242, 1516, 1517, 1518, 1519, 1520, 1521, 1522, 1524, 1525, 1526, 1528, 1529, 1530, 1531, 1545,
    1532, 1533, 1534, 1537, 1535, 1536, 1538, 1539, 1540, 62, 1541, 1542, 1543, 1544, 1546, 1547,
    1548, 1549, 1551, 1557, 1561, 1566, 1573, 1580, 1586, 1598, 1602, 1613, 1620, 1626, 1630, 1646,
    1651, 1657, 1552, 1553, 1554, 1555, 1556, 1558, 1559, 1560, 1562, 1563, 1564, 1565, 1567, 1568,
    1569, 1570, 1571, 1572, 1574, 1575, 1576, 1577, 1578, 1579, 1581, 1582, 1583, 1584, 1585, 1587,
    1588, 1589, 1590, 1591, 1594, 1592, 1593, 1595, 1596, 1597, 62, 1599, 1600, 1601, 1603, 1604,
    1605, 1606, 1607, 1608, 1612, 1609, 1610, 1611, 1614, 1615, 1616, 1617, 1618, 241, 242, 1619,
    62, 1621, 1622, 1623, 1624, 1625, 1627, 1628, 1629, 1631, 1632, 1633, 1634, 1635, 1638, 1641,
    1644, 1636, 1637, 1639, 1640, 1642, 1643, 1645, 1647, 1648, 1649, 1650, 1652, 1653, 1654, 1655,
    1656, 1658, 1659, 1660, 1662, 1663, 1664, 1665, 1667, 1673, 1686, 1687, 1668, 1669, 1670, 1671,
    1672, 1674, 1677, 1678, 1679, 1675, 1676, 1680, 1685, 1681, 1684, 1682, 1683, 1688, 1689, 1692,
    1690, 1691, 1694, 1695, 1696, 1697, 1717, 1745, 1698, 1699, 1700, 1701, 1702, 1703, 1705, 1711,
    1704, 62, 1706, 1707, 1708, 1709, 1710, 1712, 1713, 1714, 1715, 1716, 1718, 1719, 1720, 1721,
    1725, 1729, 1739, 1722, 1723, 1724, 1726, 1727, 1728, 1730, 1731, 1732, 1733, 1734, 1738, 1735,
    1736, 1737, 1740, 1741, 1742, 1743, 1744, 1746, 1747, 1748, 1749, 1750, 1755, 1756, 1757, 1758,
    1751, 1752, 1753, 1754, 62, 1760, 1761, 1762, 1763, 1764, 1765, 1766, 1775, 1784, 1791, 1801,
    1808, 1812, 1820, 1767, 1768, 1769, 1770, 1772, 1773, 1771, 62, 1774, 62, 1776, 1777, 1778,
    1779, 1781, 1782, 1780, 62, 1783, 62, 1785, 1786, 1787, 1788, 1789, 1790, 1792, 1793, 1794,
    1795, 1797, 1798, 1799, 1800, 1796, 62, 1802, 1803, 1804, 1805, 1806, 1807, 62, 1809, 1810,
    1811, 1813, 1814, 1815, 1816, 1817, 1818, 1819, 1821, 1822, 1823, 1825, 1826, 1827, 1828, 1829,
    1830, 1831, 1835, 1832, 1833, 1834, 1836, 1837, 1838, 1839, 1840, 1842, 1843, 1844, 1845, 1846,
    1854, 1860, 1864, 1868, 1905, 1847, 1848, 1849, 1850, 1851, 1852, 1853, 1855, 1856, 1857, 1858,
    1859, 1861, 1862, 1863, 1865, 1866, 1867, 1869, 62, 1870, 1879, 1892, 1898, 1871, 1872, 1873,
    1874, 1875, 1878, 1876, 1877, 1880, 1881, 1882, 1883, 1886, 1887, 1888, 1891, 1884, 1885, 1889,
    1890, 1893, 1894, 1895, 1896, 1897, 1899, 1900, 1901, 1902, 1903, 1904, 1906, 1907, 1908, 1909,
    1910, 1911, 1914, 1912, 1913, 1916, 1917, 1918, 1919, 1920, 1921, 1924, 1927, 1933, 1942, 1946,
    1947, 1956, 1922, 1923, 1925, 1926, 1928, 1929, 1930, 1931, 1932, 1934, 1935, 1938, 1936, 1937,
    1939, 1940, 1941, 1943, 1944, 1945, 1948, 1952, 1949, 1950, 1951, 1953, 1954, 1955, 1957, 1966,
    1958, 1959, 1960, 1963, 1961, 1962, 1964, 1965, 1967, 1968, 1970, 1971, 1972, 1973, 1974, 1975,
    1991, 1999, 2005, 2009, 2013, 2022, 1976, 1977, 1978, 1979, 1980, 1988, 1981, 1982, 1983, 1984,
    1985, 1986, 1987, 1989, 1990, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 2000, 2001, 2002, 2003,
    2004, 2006, 2007, 2008, 2010, 2011, 2012, 2014, 2015, 2016, 241, 242, 2017, 62, 2018, 2019,
    2020, 2021, 2023, 2024, 2025, 241, 242, 2026, 62, 2028, 2029, 2030, 2031, 2032, 2058, 2033,
    2036, 2034, 2035, 2037, 2041, 2045, 2049, 2038, 2039, 2040, 62, 2042, 2043, 2044, 62, 2046,
    2047, 2048, 62, 2050, 2051, 2054, 2055, 2056, 2052, 2053, 2057, 62, 2059, 2060, 2061, 2065,
    2062, 2063, 2064, 2066, 2067, 2068, 2069, 2070, 2071, 2076, 2079, 2080, 2077, 2078, 2081, 2082,
    2085, 2083, 241, 242, 2084, 62, 2086, 2087, 2090, 2093, 2102, 2091, 2092, 2094, 2101, 2095,
    241, 242, 2096, 62, 2097, 2100, 2098, 2099, 2103, 62, 2104, 2105, 2106, 241, 242, 2107,
    62, 2117, 2118, 2119, 2121, 2122, 2123, 2124, 2125, 2126, 2127, 2128, 2129, 2130, 2132, 2133,
    2134, 2135, 2136, 2142, 2137, 2138, 2139, 2140, 2141, 2143, 2144, 2145, 2146, 2147, 2150, 2151,
    2152, 2155, 2156, 2153, 2154, 2160, 2155, 2156, 2161, 62, 2164, 2155, 2156, 2166, 2167, 2168,
    2169, 2170, 234, 2174, 2175, 250, 2177, 2195, 2214, 2217, 2196, 2201, 2206, 2197, 2198, 2199,
    2200, 2202, 2203, 2204, 2205, 2207, 2208, 2209, 2210, 2211, 2212, 2213, 2215, 2216, 2218, 2219,
    2221, 2232, 2244, 2233, 2234, 2235, 2236, 2237, 2238, 2239, 2240, 2241, 2256, 2257, 2258, 2259,
    2260, 2268, 2269, 2272, 2273, 2281, 2282, 2285, 2286, 2291, 241, 242, 2292, 62, 2293, 2294,
    2296, 241, 242, 2297, 62, 2300, 241, 242, 2309, 241, 242, 2311, 2312, 2313, 2314, 2317,
    2319, 2315, 2316, 2318, 241, 242, 2320, 2330, 2321, 2322, 2325, 2326, 2327, 2328, 2329, 2323,
    2324, 2331, 2332, 2333, 2334, 2335, 2336, 2340, 2341, 2343, 2344, 2349, 2350, 2353, 2363, 2354,
    2355, 2356, 2360, 250, 2362, 2357, 2364, 2365, 2355, 2366, 2370, 250, 2372, 2367, 2408, 2409,
    2412, 2413, 2414, 2415, 2420, 2416, 2417, 2418, 2419, 2421, 2422, 2423, 2430, 2431, 2432, 2433,
    2434, 2435, 2438, 2439, 2440, 2441, 2451, 2454, 2459, 2468, 2471, 2472, 2473, 2452, 2453, 2455,
    2456, 2457, 241, 242, 2458, 62, 2460, 2461, 2462, 2463, 2464, 2465, 2466, 241, 242, 2467,
    62, 2469, 2470, 2474, 2475, 2476, 2477, 2479, 2480, 2481, 2482, 2484, 2489, 2492, 2485, 2488,
    2486, 2487, 2490, 2491, 2493, 2494, 2495, 2496, 241, 242, 2498, 2499, 2500, 2501, 2502, 2503,
    2504, 2506, 2507, 2508, 2509, 2510, 2511, 2512, 2513, 2514, 2515, 2516, 2517, 2518, 2522, 2523,
    2524, 2529, 2525, 2526, 2527, 2528, 2530, 2531, 2532, 2533, 2534, 2535, 2548, 2549, 2550, 2561,
    2562, 2565, 2566, 2567, 2568, 2670, 2569, 2574, 2581, 2584, 2591, 2594, 2597, 2602, 2605, 2608,
    2661, 2570, 2573, 2571, 2572, 2575, 2576, 2577, 241, 242, 2578, 2579, 2580, 2582, 2583, 2585,
    2586, 2587, 2588, 2589, 2590, 2592, 2593, 2595, 2596, 2598, 2599, 2600, 2601, 2603, 2604, 2606,
    2607, 2609, 2657, 2658, 2610, 2611, 2612, 2613, 2614, 2615, 2616, 2617, 2618, 2619, 2620, 2621,
    2622, 2623, 2624, 2625, 2628, 2631, 2635, 2638, 2641, 2645, 2649, 2653, 2626, 2627, 2629, 2630,
    2632, 2633, 2634, 2636, 2637, 2639, 2640, 2642, 2643, 2644, 2646, 2647, 2648, 2650, 2651, 2652,
    2654, 2655, 2656, 2659, 2660, 2662, 2666, 2667, 2663, 2664, 2665, 2668, 241, 242, 2669, 2671,
    2672, 2673, 2674, 2680, 2681, 2682, 2683, 2684, 2685, 2686, 2687, 2691, 2696, 2697, 2698, 2699,
    2700, 2701, 2702, 2703, 2688, 2689, 2690, 2692, 2693, 2694, 2695, 2708, 2710, 2711, 2709, 241,
    242, 2715, 2716, 2717, 2718, 2722, 2723, 2729, 2730, 2731, 2735, 2736, 2737, 2738, 2739, 234,
    2740, 2355, 2741, 250, 2767, 250, 2742, 62, 2743, 2749, 2760, 2744, 2745, 2746, 2747, 2748,
    2750, 2753, 2754, 2755, 2751, 241, 242, 2752, 62, 2756, 241, 242, 2757, 62, 2758, 2759,
    2761, 2764, 2765, 2762, 241, 242, 2763, 62, 2766, 241, 242, 2768, 2769, 2772, 2788, 2773,
    2785, 2774, 241, 242, 2775, 62, 2776, 2781, 2784, 2777, 2778, 2780, 2779, 241, 242, 2782,
    2783, 2786, 2787, 2789, 2790, 2791, 2795, 2796, 2797, 2798, 2799, 2814, 2815, 2816, 2817, 241,
    242, 2819, 2823, 2824, 2825, 2826, 2827, 2828, 2829, 2830, 2834, 2835, 2836, 2845, 2848, 2849,
    2852, 2856, 2864, 2871, 2872, 2873, 2877, 2878, 2884, 2885, 2893, 2894, 2895, 2896, 2897, 2901,
    2905, 2908, 2909, 2910, 2918, 2919, 2831, 2832, 2833, 2837, 2841, 2838, 2839, 2840, 2842, 2843,
    2844, 2846, 2847, 2850, 2851, 2853, 2854, 2855, 2857, 2858, 2859, 2860, 2861, 2862, 2863, 2865,
    2866, 2867, 2868, 2869, 2870, 2874, 2875, 2876, 2879, 2880, 2881, 2882, 2883, 2886, 2887, 2888,
    2889, 2890, 2891, 2892, 2898, 2899, 2900, 2902, 2903, 2904, 2906, 2907, 2911, 2912, 2913, 2914,
    2915, 2916, 2917, 2930, 2931, 2932, 2933, 2934, 2936, 2935, 241, 242, 2938, 2939, 2940, 2944,
    2956, 2941, 2942, 2943, 241, 242, 2945, 2950, 2953, 2946, 2947, 2948, 2949, 2951, 2952, 2954,
    2955, 2957, 2960, 2958, 2959, 2961, 2962, 2964, 2965, 2966, 2967, 2968, 2969, 2998, 2970, 241,
    242, 2971, 62, 2972, 2976, 2980, 2984, 2990, 2994, 2973, 2974, 2975, 2977, 2978, 2979, 2981,
    2982, 2983, 2985, 2986, 2987, 2988, 2989, 2991, 2992, 2993, 2995, 2996, 2997, 2999, 3000, 3002,
    3003, 3004, 3005, 3006, 3007, 3010, 3011, 3012, 3008, 3009, 3014, 3018, 3019, 3020, 3015, 3016,
    3017, 3022, 3023, 3024, 3025, 3026, 3027, 3028, 3029, 3030, 3031, 3033, 3034, 3035, 3036, 3038,
    3039, 3042, 3045, 3037, 3040, 3041, 3043, 3044, 3046, 3047, 3050, 3048, 3049, 3051, 3054, 3052,
    3053, 3055, 3056, 3058, 3059, 3060, 3061, 3062, 3063, 3066, 3069, 3072, 3064, 3065, 3067, 3068,
    3070, 3071, 3074, 3075, 3076, 3077, 3078, 3079, 3082, 3083, 3084, 3088, 3091, 3092, 3093, 3096,
    3080, 241, 242, 3081, 62, 3085, 3086, 3087, 3089, 3090, 3094, 3095, 3097, 3098, 3100, 3101,
    3102, 3103, 3104, 3105, 3106, 3119, 3107, 3110, 3113, 3114, 3115, 3116, 3117, 3118, 3108, 3109,
    3111, 3112, 3120, 3123, 3126, 3129, 3135, 3140, 3146, 3153, 3156, 3121, 3122, 3124, 3125, 3127,
    3128, 3130, 3131, 3132, 3133, 3134, 3136, 3137, 3138, 3139, 3141, 3142, 3143, 3144, 3145, 3147,
    3150, 3148, 3149, 3151, 3152, 3154, 3155, 3157, 3158, 3159, 3160, 3161, 3162, 3164, 3165, 3166,
    3167, 3168, 3169, 3174, 3175, 3176, 3179, 3180, 3181, 3190, 3191, 3170, 3171, 3172, 3173, 3177,
    3178, 3182, 3183, 3184, 3185, 241, 242, 3186, 62, 3187, 3188, 3189, 3192, 3193, 3193, 241,
    242, 3195, 3196, 3197, 3198, 3199, 3204, 3205, 3224, 3225, 3226, 3238, 3239, 3240, 3200, 241,
    242, 3201, 62, 3202, 3203, 3206, 3207, 3208, 3209, 3212, 3210, 3211, 3213, 3214, 3215, 3216,
    3217, 3218, 3219, 241, 242, 3220, 62, 3221, 3222, 3223, 3227, 3228, 3229, 3230, 3231, 3232,
    3233, 3234, 3235, 3236, 3237, 3241, 3242, 3243, 3244, 241, 242, 3245, 62, 3246, 3247, 3248,
    3250, 3251, 3252, 234, 3253, 3254, 250, 234, 3255, 3256, 250, 3258, 3259, 3260, 3262, 3288,
    3263, 3264, 3265, 3266, 3279, 3284, 3287, 3267, 241, 242, 3268, 62, 3269, 3278, 3270, 3271,
    3272, 3273, 3276, 3274, 3275, 3277, 3280, 3281, 3282, 241, 242, 3283, 62, 3285, 3286, 3289,
    3290, 3291, 3292, 3313, 3316, 3293, 3294, 3295, 3296, 3307, 3297, 241, 242, 3298, 62, 3299,
    3306, 3300, 3301, 3304, 3302, 3303, 3305, 3308, 3312, 3309, 3310, 3311, 3314, 3315, 3318, 3319,
    3320, 3321, 3322, 3323, 3324, 3325, 3343, 3326, 3327, 3328, 3339, 3342, 3329, 3330, 3331, 241,
    242, 3332, 62, 3333, 3334, 3335, 3336, 3337, 3338, 3340, 3341, 3345, 3346, 3347, 3348, 3349,
    3350, 3351, 3352, 62, 3354, 3355, 3356, 3357, 3358, 3361, 3362, 3363, 3364, 3367, 3370, 3375,
    3379, 3382, 3385, 3359, 3360, 3365, 3366, 3368, 241, 242, 3369, 62, 3371, 3372, 3373, 241,
    242, 3374, 62, 3376, 3377, 3378, 3380, 3381, 3383, 3384, 3386, 3387, 3390, 3391, 3392, 3393,
    3394, 3395, 3396, 3397, 3398, 3399, 3400, 3401, 3402, 3403, 3405, 3406, 3407, 3408, 3409, 3414,
    3415, 3421, 3422, 3437, 3410, 3411, 3412, 3413, 3416, 3417, 3418, 3419, 3420, 3423, 3426, 3431,
    3432, 3424, 3425, 3425, 241, 242, 3427, 3428, 3428, 241, 242, 3429, 3430, 3433, 3434, 3434,
    241, 242, 3435, 3436, 3439, 3440, 3441, 3442, 3449, 3452, 3458, 3461, 3464, 3469, 3474, 3480,
    3486, 3489, 3492, 3500, 3443, 3444, 3445, 3446, 3448, 3447, 241, 242, 3450, 3451, 3453, 3454,
    3455, 3456, 241, 242, 3457, 62, 3459, 3460, 3462, 3463, 3465, 3466, 3467, 3468, 3470, 3471,
    3472, 3473, 3475, 3476, 3477, 3478, 3479, 3481, 3482, 3483, 3484, 3485, 3487, 3488, 3490, 3491,
    3493, 3494, 3495, 3496, 3497, 3498, 241, 242, 3499, 62, 3501, 3502, 3503, 241, 242, 3504,
    3505, 3506, 3509, 3514, 3517, 3507, 3508, 3510, 3511, 3512, 3513, 3515, 3516, 3518, 3519, 3521,
    3522, 3523, 3525, 3524, 3526, 241, 242, 3527, 62, 3528, 3534, 3539, 3544, 3553, 3554, 3529,
    3532, 3533, 3530, 3531, 3535, 3536, 3537, 3538, 3540, 3541, 3542, 3543, 3545, 3551, 3552, 3546,
    3547, 3548, 3549, 3550, 3556, 3557, 3558, 3559, 3560, 3561, 3566, 3574, 3583, 3587, 3562, 3563,
    3564, 3565, 3567, 3568, 3569, 3570, 3571, 3572, 3573, 3575, 3576, 3577, 62, 3578, 3579, 3580,
    3581, 3582, 3584, 3585, 3586, 241, 242, 3588, 3589, 3590, 3591, 241, 242, 3593, 3594, 3595,
    3596, 3597, 3598, 3599, 3601, 3602, 3603, 3604, 3613, 3605, 3609, 3606, 3607, 3608, 3610, 3611,
    3612, 62, 3614, 3615, 3617, 3618, 3619, 3620, 3621, 3631, 3622, 3623, 3628, 3624, 241, 242,
    3625, 62, 3626, 3627, 3629, 3630, 3632, 3633, 3634, 241, 242, 3636, 3637, 3640, 3641, 3638,
    3639, 3642, 3643, 3644, 3647, 3650, 3653, 3656, 3659, 3662, 3665, 3669, 3675, 3679, 3683, 3687,
    3690, 3693, 3696, 3645, 3646, 3648, 3649, 3651, 3652, 3654, 3655, 3657, 3658, 3660, 3661, 3663,
    3664, 3666, 3667, 3668, 3670, 3671, 3672, 3673, 3674, 3676, 3677, 3678, 3680, 3681, 3682, 3684,
    3685, 3686, 3688, 3689, 3691, 3692, 3694, 3695, 3697, 3698, 3700, 3701, 3702, 3712, 3703, 3709,
    3704, 3705, 3706, 3707, 3708, 3710, 3711, 3714, 3717, 3722, 3730, 3738, 3743, 3746, 3751, 3715,
    3716, 3718, 3719, 3720, 3721, 3723, 3727, 3724, 3725, 3726, 3728, 3729, 3731, 3735, 3732, 3733,
    3734, 3736, 3737, 3739, 3740, 3741, 3742, 3744, 3745, 3747, 3748, 3749, 3750, 3752, 3753, 3754,
    3755, 3756, 3758, 3759, 3760, 3761, 3762, 3763, 3764, 3766, 3767, 3768, 3769, 3770, 3773, 3776,
    3777, 3778, 3771, 3772, 3774, 3775, 3780, 3781, 3782, 3785, 3792, 3793, 3797, 3798, 3783, 3784,
    3786, 241, 242, 3787, 62, 3788, 3789, 3790, 3791, 3794, 62, 3795, 3796, 3800, 3801, 3802,
    3803, 3804, 3805, 3806, 3808, 3807, 62, 3809, 3810, 3812, 3813, 3818, 3819, 3820, 3823, 3828,
    3829, 3830, 3814, 3815, 3816, 3817, 3821, 241, 242, 3822, 62, 3824, 3825, 3826, 3827, 3831,
    3832, 3832, 241, 242, 3834, 3835, 3836, 3837, 3838, 3839, 3840, 3855, 3860, 3841, 241, 242,
    3842, 62, 3843, 3849, 3850, 3844, 3845, 3846, 3847, 3848, 3851, 3852, 3853, 3854, 3856, 3857,
    3858, 3859, 3861, 3863, 3864, 3865, 3866, 3867, 3868, 3869, 241, 242, 3870, 3874, 3871, 3872,
    3873, 3875, 3876, 3877, 3879, 3880, 3881, 3882, 3883, 3884, 3890, 3904, 3905, 3906, 3948, 3885,
    3886, 3887, 3888, 3889, 3891, 3903, 3892, 3893, 3894, 3895, 3896, 3897, 3898, 3899, 3900, 62,
    3907, 3910, 3923, 3938, 3944, 3908, 3909, 3911, 3914, 3912, 3913, 3915, 3916, 3917, 3920, 3918,
    3919, 3921, 3922, 3924, 3925, 3928, 3929, 3930, 3931, 3926, 3927, 3932, 3935, 3936, 3937, 3933,
    3934, 3939, 3940, 3941, 3942, 3943, 3945, 3946, 3947, 241, 242, 3949, 3950, 3953, 3951, 3952,
    3955, 3956, 3957, 3958, 3963, 3959, 3960, 3961, 3962, 3964, 241, 242, 3965, 62, 3967, 3968,
    3969, 3971, 3972, 3973, 3974, 3978, 3979, 3980, 3975, 3976, 3977, 3982, 3983, 3984, 3985, 3986,
    3987, 3990, 3988, 3989, 3991, 3994, 3997, 4003, 4008, 4012, 4013, 3992, 3993, 3995, 3996, 3998,
    3999, 4000, 4001, 4002, 4004, 4005, 4006, 4007, 62, 4009, 4010, 4011, 4014, 4015, 4016, 4018,
    4019, 4020, 4021, 4022, 4023, 4024, 4025, 4028, 4026, 4027, 4027, 241, 242, 4039, 4042, 4040,
    4041, 4043, 241, 242, 4062, 4065, 4063, 4064, 4066, 4072, 4078, 4085, 4086, 4087, 4088, 4067,
    4068, 4069, 4070, 4071, 4073, 4074, 4075, 4076, 4077, 4079, 4082, 4080, 4081, 4083, 4084, 4093,
    4094, 4100, 4101, 4102, 4104, 4105, 4106, 4107, 4108, 4109, 4110, 4111, 4112, 4113, 4116, 4117,
    4118, 4123, 4137, 4138, 4139, 4142, 4143, 4144, 4145, 4146, 4154, 4155, 4114, 4115, 4119, 4122,
    4120, 4121, 4124, 4135, 4136, 4125, 4134, 4126, 4127, 4128, 4129, 4130, 4133, 4131, 4132, 4140,
    4141, 4147, 4153, 4148, 4149, 4150, 4151, 4152, 4156, 4157, 4158, 4163, 4164, 241, 242, 4165,
    62, 4166, 4167, 4169, 4170, 4172, 4173, 4174, 4175, 4176, 4177, 4189, 4190, 4191, 4197, 4198,
    4199, 4200, 4201, 4204, 4205, 4213, 4214, 4216, 4217, 4218, 4219, 4221, 241, 242, 4222, 62,
    4228, 4229, 4230, 4231, 4232, 4233, 4234, 4240, 4241, 4248, 4235, 234, 4236, 250, 4237, 62,
    4238, 4239, 4242, 4247, 4243, 4244, 4245, 4246, 4249, 234, 4250, 250, 4251, 4252, 4253, 62,
    4254, 4255, 4268, 4269, 4270, 4271, 4272, 4273, 4276, 4277, 4274, 4275, 4281, 4282, 4283, 4292,
    4293, 4294, 4295, 4296, 4297, 4298, 4318, 4319, 4320, 4323, 4321, 4322, 4324, 4325, 4332, 4333,
    4334, 4335, 4342, 4336, 62, 4337, 4338, 4339, 4340, 4341, 4344, 4345, 4347, 4348, 4349, 4350,
    4354, 4351, 4352, 4353, 241, 242, 4356, 4357, 4358, 4359, 4360, 4362, 4363, 4364, 4365, 4366,
    4367, 4368, 4369, 241, 242, 4371, 4372, 4373, 4375, 4376, 4377, 4378, 4380, 4379, 62, 4382,
    4383, 4384, 4385, 4386, 4388, 4389, 4390, 4391, 4392, 4394, 4393, 62, 4396, 4397, 4398, 4399,
    4403, 4400, 62, 4401, 4402, 4405, 4406, 4407, 4408, 4409, 4411, 4410, 62, 4414, 4415, 4416,
    4417, 4418, 4420, 4419, 62, 4422, 4423, 4424, 4425, 4427, 4428, 4429, 4430, 4436, 4431, 62,
    4432, 4433, 4434, 4435, 4438, 4439, 4440, 4441, 4442, 4443, 4444, 4446, 4447, 4448, 4449, 4450,
    62, 4451, 4452, 4454, 4455, 4456, 4457, 4459, 4458, 62, 4461, 4462, 4463, 4464, 4465, 4466,
    4467, 4468, 4470, 4471, 4472, 4473, 4475, 4476, 4477, 4478, 4479, 4481, 4482, 4483, 4484, 4486,
    4485, 62, 4488, 4489, 4490, 4491, 4493, 4492, 62, 4494, 4495, 4497, 4498, 4499, 4500, 4501,
    4502, 4503, 4505, 4506, 4507, 4508, 4509, 4511, 4510, 62, 4513, 4514, 4515, 4516, 4517, 4518,
    4519, 4521, 4522, 4523, 4524, 4525, 4526, 4527, 4529, 4530, 4531, 4532, 4534, 4533, 62, 4536,
    4537, 4538, 4539, 4541, 4542, 4543, 4544, 4546, 4545, 62, 4554, 2355, 4555, 250, 241, 242,
    4560, 4561, 4589, 4590, 4591, 4594, 4592, 241, 242, 4593, 62, 4598, 4599, 4600, 4605, 4606,
    4607, 4608, 241, 242, 4609, 62, 4615, 4627, 4616, 4626, 4617, 4618, 4619, 4620, 4621, 4622,
    4623, 4624, 4625, 4628, 4629, 4630, 4631, 4632, 4633, 4635, 4636, 4645, 4637, 4642, 4638, 4641,
    4639, 4640, 4643, 241, 242, 4644, 62, 4646, 4647, 4648, 4649, 4655, 4656, 4657, 4658, 4659,
    4662, 4663, 4664, 4668, 4669, 4670, 4671, 4677, 4660, 4661, 4665, 4666, 4667, 4672, 4673, 4674,
    4675, 4676, 4678, 4679, 4681, 4684, 4685, 4686, 4682, 4683, 4688, 4689, 4690, 4691, 4692, 4696,
    4693, 4694, 4695, 4698, 4701, 4699, 4700, 4703, 4704, 4705, 4718, 4706, 4710, 4716, 4707, 4708,
    4709, 241, 242, 4711, 4714, 4712, 4713, 4715, 4719, 4720, 4722, 4757, 4723, 4724, 4725, 4737,
    4738, 4739, 4746, 4754, 4755, 4726, 241, 242, 4727, 4728, 4729, 4730, 4731, 62, 4732, 4733,
    4734, 4735, 4736, 4740, 4743, 4741, 4742, 4744, 241, 242, 4745, 62, 4747, 4748, 4749, 4750,
    4751, 4752, 4753, 4759, 4760, 4761, 4762, 4763, 4764, 4765, 4769, 4770, 4775, 4771, 4772, 4773,
    4774, 4774, 241, 242, 4776, 4777, 4785, 4786, 4789, 4793, 4796, 4787, 4788, 4790, 4791, 4792,
    4794, 4795, 4797, 4798, 4799, 4800, 4803, 4804, 4812, 4813, 4805, 4806, 4807, 4808, 4809, 4810,
    4811, 4817, 4818, 4819, 4820, 4821, 4822, 4823, 4827, 4828, 4829, 241, 242, 4830, 4831, 4846,
    4847, 4857, 4861, 4848, 4849, 4853, 4854, 4850, 4851, 4852, 4855, 4856, 4858, 4859, 4860, 62,
    4862, 4863, 4864, 4865, 4869, 4870, 4871, 4873, 4874, 4877, 4880, 4878, 4879, 4881, 4884, 4891,
    4899, 4882, 4883, 4885, 4886, 4888, 4887, 241, 242, 4889, 4890, 4892, 4893, 4894, 4895, 4896,
    4897, 4898, 4900, 4901, 4902, 4903, 4904, 4906, 4909, 4907, 4908, 4910, 4911, 4913, 4912, 241,
    242, 4914, 4915, 4921, 4922, 4939, 4923, 4936, 4924, 4927, 4925, 4926, 4928, 4933, 4929, 4930,
    4931, 4932, 4934, 4935, 4937, 4923, 4938, 4923, 4940, 4941, 4942, 4945, 4946, 4947, 4943, 4944,
    4948, 4949, 4951, 4952, 4953, 4958, 4959, 4960, 62, 4961, 4962, 4962, 241, 242, 4964, 4965,
    4966, 4969, 4972, 4973, 4974, 4975, 4976, 4977, 4978, 4979, 4980, 4967, 4968, 4970, 4971, 4982,
    4985, 4986, 4987, 4988, 4989, 4995, 4996, 4983, 4984, 4990, 4991, 4992, 4993, 4994, 4997, 4998,
    5000, 5018, 5001, 5002, 5005, 5006, 5012, 5016, 5017, 5003, 5004, 5004, 241, 242, 5007, 5008,
    5009, 5010, 5011, 5013, 5014, 5015, 5019, 241, 242, 5000, 5020, 5021, 5023, 5024, 5024, 241,
    242, 2355, 5025, 250, 5036, 5037, 5048, 5026, 5027, 5028, 5029, 241, 242, 5031, 5032, 5034,
    5035, 5038, 5039, 5040, 5041, 5043, 5044, 5046, 5047, 5050, 5051, 5052, 5053, 5056, 5054, 5055,
    5057, 5058, 5062, 5066, 5069, 5073, 5059, 5060, 5061, 5063, 5064, 5065, 5067, 5068, 5070, 5071,
    5072, 5074, 5075, 5076, 5079, 5080, 5081, 241, 242, 5082, 5085, 5086, 5087, 5088, 5089, 5083,
    5084, 5084, 241, 242, 5090, 241, 242, 5091, 62, 5092, 5093, 5095, 5096, 5101, 5097, 5100,
    5098, 5099, 5102, 241, 242, 5103, 5104, 5105, 5106, 5107, 5108, 241, 242, 5109, 62, 5110,
    5111, 5117, 5118, 5119, 5120, 5121, 5122, 5123, 5125, 5126, 5129, 5133, 5139, 5150, 5153, 5159,
    5160, 5163, 5175, 5176, 5177, 5178, 5127, 5128, 5128, 241, 242, 5130, 5131, 5132, 5134, 5135,
    5137, 5138, 5136, 5140, 5143, 5141, 5142, 5144, 5147, 5145, 5146, 5148, 5149, 5151, 5152, 5154,
    5155, 5156, 5157, 5158, 62, 5161, 5162, 5164, 5169, 5170, 5165, 5166, 5167, 5168, 5171, 5172,
    5173, 5174, 5179, 62, 5180, 5181, 5184, 5182, 5183, 5186, 5187, 241, 242, 5189, 5285, 5190,
    5191, 5196, 5197, 5201, 5204, 5217, 5223, 5229, 5232, 5235, 5238, 5253, 5261, 5264, 5192, 5193,
    5194, 5195, 5198, 5199, 5200, 5202, 5203, 5205, 5210, 5216, 5206, 5207, 5208, 5209, 5211, 5212,
    5213, 5214, 5215, 5218, 5219, 5220, 5221, 5222, 5224, 5225, 5226, 5227, 5228, 5230, 5231, 5233,
    5234, 5236, 5237, 5239, 5240, 5241, 5242, 5250, 5243, 5246, 5244, 5245, 5247, 62, 5248, 5249,
    5251, 5252, 5254, 5255, 5256, 5257, 5258, 5259, 5260, 5262, 5263, 5265, 5266, 234, 5267, 250,
    5284, 5268, 5271, 5274, 5280, 5269, 5270, 5272, 5273, 5275, 5279, 5276, 5277, 5278, 5281, 5282,
    5283, 5286, 5287, 5288, 241, 242, 5289, 62, 5293, 5298, 5294, 5295, 5296, 5299, 5300, 5303,
    5308, 5320, 5304, 5305, 5306, 5307, 5309, 5310, 5311, 5312, 5317, 5313, 5314, 5315, 5316, 5318,
    5319, 5321, 5325, 5322, 5323, 5324, 5327, 241, 242, 5328, 62, 5330, 5334, 5351, 5331, 5332,
    5333, 5335, 5350, 5336, 5349, 5337, 5342, 5338, 5339, 5340, 5341, 5343, 5344, 5345, 5346, 5347,
    5348, 5352, 5353, 5372, 5373, 5387, 5400, 5413, 5374, 5378, 5381, 5386, 5375, 5376, 5377, 5379,
    5380, 5382, 5383, 5384, 5385, 5388, 5392, 5389, 5390, 5391, 62, 5393, 5394, 5395, 5396, 5397,
    5398, 5399, 5401, 5405, 5409, 5402, 5403, 5404, 5406, 5407, 5408, 5410, 5411, 5412, 5414, 5415,
    5416, 5417, 5418, 5419, 5421, 5424, 5425, 5426, 5427, 5428, 5431, 5422, 5423, 5429, 5430, 5432,
    5435, 5433, 5434, 5436, 5437, 5438, 5439, 5440, 5441, 5442, 5446, 5447, 5451, 5452, 234, 5453,
    250, 5454, 62, 5455, 5456, 5457, 5458, 5459, 5460, 5461, 241, 242, 5463, 5464, 5471, 5474,
    5475, 5476, 5477, 5478, 5472, 5473, 5481, 5482, 62, 5483, 5484, 5485, 5486, 241, 242, 5490,
    5491, 5492, 5493, 241, 242, 5494, 62, 5495, 5496, 5502, 2355, 5503, 250, 5504, 5505, 5505,
    241, 242, 5507, 5510, 5511, 5512, 5513, 5508, 5509, 5527, 5528, 5529, 5530, 5536, 5537, 5540,
    5541, 5542, 5545, 5548, 5546, 5547, 5559, 5560, 5561, 5563, 5564, 5565, 5574, 5575, 5576, 5577,
    5578, 5579, 5566, 5571, 5567, 5570, 5568, 5569, 5572, 241, 242, 5573, 62, 5584, 5585, 5586,
    5587, 5592, 5588, 241, 242, 5589, 5591, 5590, 62, 5600, 5601, 5602, 5603, 5605, 5608, 5616,
    5619, 5606, 5607, 5609, 5615, 5610, 5611, 5612, 5613, 5614, 5617, 5618, 5620, 5621, 5622, 5623,
    5625, 5630, 5626, 5627, 5628, 5629, 241, 242, 5633, 5636, 5639, 5634, 5635, 5637, 5638, 5640,
    5641, 5642, 5643, 5658, 5659, 5660, 5661, 5662, 5665, 5668, 5663, 5664, 5664, 241, 242, 5666,
    5667, 5669, 5670, 5671, 5672, 5673, 5674, 5674, 241, 242, 5689, 5690, 5691, 5696, 5697, 5692,
    5693, 5694, 5695, 5698, 5699, 5700, 5701, 5704, 5705, 5706, 5707, 5708, 5709, 5711, 5712, 5716,
    5717, 5713, 5714, 5715, 5721, 5722, 5723, 5724, 5725, 5726, 5728, 5729, 5730, 5731, 5737, 5738,
    5739, 5740, 5741, 5743, 5756, 5761, 5744, 5745, 234, 5746, 5747, 250, 5749, 5748, 5750, 5751,
    5752, 5755, 5753, 5754, 5757, 5758, 234, 5759, 5760, 250, 5762, 234, 5763, 250, 5766, 5768,
    5771, 5769, 5770, 5773, 5774, 5775, 5778, 5776, 5777, 5777, 241, 242, 5780, 5781, 5782, 5786,
    5783, 5784, 5785, 5788, 234, 5789, 250, 5790, 241, 242, 5791, 62, 5795, 5796, 5797, 5799,
    5800, 5801, 5802, 5804, 5807, 5808, 5805, 5806, 5809, 5834, 5835, 5836, 5837, 5843, 5844, 5853,
    5854, 5859, 5862, 5863, 5860, 5861, 5864, 5865, 5867, 5868, 5869, 5870, 5871, 5875, 5878, 5876,
    5877, 5881, 234, 5882, 5888, 250, 5883, 5884, 5884, 241, 242, 5885, 5886, 5887, 5889, 5894,
    5890, 5891, 5892, 5893, 5895, 5896, 5905, 5906, 5908, 5909, 5910, 5911, 5912, 5913, 5914, 5915,
    5916, 5917, 5918, 5924, 5925, 5934, 5935, 5936, 5937, 62, 5950, 5951, 5952, 5953, 5954, 5957,
    241, 242, 5955, 5956, 5958, 5959, 5960, 241, 242, 5964, 5967, 5968, 5969, 5970, 5971, 5972,
    5974, 5977, 5965, 5966, 5973, 62, 5975, 5976, 5978, 62, 5979, 5982, 5980, 5981, 6008, 234,
    6009, 250, 6010, 6012, 6013, 6016, 6017, 6019, 6020, 6021, 6024, 234, 6025, 250, 6022, 6023,
    6026, 6027, 6029, 6030, 6031, 6032, 6033, 6034, 6037, 234, 6038, 250, 6035, 6036, 6039, 6040,
    6041, 6042, 6043, 6045, 6046, 6047, 6048, 6051, 234, 6052, 250, 6049, 6050, 6054, 234, 6055,
    250, 6058, 234, 6059, 250, 6069, 6070, 6071, 6072, 6056, 6057, 6060, 6061, 6062, 6066, 6063,
    6065, 6064, 241, 242, 6067, 6068, 6073, 6074, 6075, 6076, 6078, 234, 6079, 250, 6085, 6086,
    6087, 6088, 6119, 6129, 6130, 6134, 6131, 6132, 6133, 6136, 6137, 6138, 6139, 6140, 6141, 241,
    242, 6143, 234, 6144, 250, 6145, 62, 6152, 6153, 6180, 6181, 6182, 6184, 6186, 6188, 6183,
    241, 242, 6185, 241, 242, 6187, 241, 242, 6192, 6193, 6196, 6197, 6198, 6199, 6194, 6195,
    6195, 241, 242, 6206, 6211, 6207, 6208, 6209, 6210, 6216, 6219, 6217, 6218, 6220, 6221, 6223,
    6224, 6226, 6227, 6234, 6235, 6236, 6237, 6238, 6254, 6255, 6256, 6258, 6260, 4570, 6259, 62,
    6262, 6263, 6264, 6265, 6267, 6268, 6269, 6270, 6271, 6272, 6273, 6274, 6275, 6276, 6277, 2298,
    6279, 6280, 6281, 6282, 6283, 6284, 6293, 234, 6294, 250, 6295, 6296, 6311, 6312, 6313, 241,
    242, 6314, 6317, 6318, 62, 6319, 6320, 6324, 6325, 6328, 6329, 234, 6330, 250, 6331, 62,
    6332, 6336, 6342, 6347, 6333, 6334, 6335, 6337, 6338, 6339, 6340, 6341, 6343, 6344, 6345, 6346,
    6351, 6352, 6353, 6354, 6355, 6356, 6357, 6360, 6358, 6359, 6363, 6364, 234, 6373, 6374, 6375,
    250, 6376, 6377, 6378, 241, 242, 6382, 6383, 6385, 6400, 6401, 6386, 6399, 6387, 6388, 6393,
    6394, 241, 242, 6389, 6390, 6391, 6392, 6395, 6396, 6397, 6398, 6402, 6415, 6403, 6404, 6409,
    6410, 241, 242, 6405, 6406, 6407, 6408, 6411, 6412, 6413, 6414, 6439, 6444, 6459, 6440, 6441,
    6442, 241, 242, 6443, 62, 6445, 6446, 6452, 6453, 6447, 241, 242, 6448, 62, 6449, 6450,
    6451, 6454, 241, 242, 6455, 62, 6456, 6457, 6458, 6460, 6461, 6462, 241, 242, 6463, 6464,
    6465, 6466, 6467, 6469, 6470, 234, 6471, 250, 6472, 6473, 6473, 241, 242, 6474, 62, 6477,
    6478, 6479, 6480, 241, 242, 6481, 62, 6482, 6483, 6493, 6498, 6494, 6495, 6496, 6497, 6529,
    6530, 6531, 6536, 6540, 6546, 6547, 6548, 6551, 6552, 6549, 241, 242, 6550, 62, 6558, 6559,
    6568, 241, 242, 6569, 62, 6570, 6573, 6571, 6577, 6572, 6573, 6576, 6574, 6575, 6585, 6588,
    6591, 6592, 6593, 6593, 241, 242, 6594, 6595, 6596, 6597, 6603, 6612, 6604, 6605, 6606, 62,
    6608, 6609, 6610, 6611, 6613, 6614, 6615, 6616, 6617, 62, 6618, 6619, 6621, 6622, 6623, 6624,
    6627, 6628, 6629, 6630, 6631, 6632, 241, 242, 6634, 6635, 6637, 6638, 6639, 241, 242, 6640,
    6641, 6642, 6643, 6645, 6646, 6647, 6648, 6650, 2355, 6651, 250, 6654, 6655, 6659, 6656, 6657,
    6658, 6660, 241, 242, 6664, 6665, 6666, 6672, 6667, 6668, 6669, 6670, 6671, 6676, 6677, 6679,
    6686, 6691, 6699, 6680, 6683, 6681, 6682, 6684, 6685, 6687, 6688, 6689, 6690, 6692, 6693, 6696,
    6694, 6695, 6697, 6698, 6700, 62, 6701, 6702, 6703, 6704, 6720, 6721, 6722, 6723, 6729, 6730,
    6724, 62, 6725, 6726, 6727, 6728, 6731, 6732, 6733, 6734, 6741, 6744, 6747, 6748, 6742, 6743,
    6745, 6746, 6750, 6751, 6756, 6757, 6758, 6759, 6760, 6761, 6765, 6762, 6763, 6764, 6766, 6767,
    6768, 6770, 6771, 6772, 6773, 6774, 6779, 6780, 6788, 6789, 6781, 6782, 6785, 6783, 6784, 6786,
    241, 242, 6787, 62, 6790, 6791, 6794, 6795, 6796, 6797, 6798, 6799, 6814, 6815, 6827, 6816,
    241, 242, 6817, 62, 6818, 6821, 6824, 6819, 6820, 6822, 6823, 6825, 6826, 6828, 6832, 6836,
    6840, 6829, 6830, 6831, 6833, 6834, 6835, 6837, 6838, 6839, 6841, 6844, 6845, 6842, 6843, 6848,
    6849, 6852, 6850, 6851, 6853, 6854, 6856, 241, 242, 6857, 62, 6871, 6872, 6873, 6875, 6876,
    6877, 6878, 6879, 6898, 6899, 6903, 6917, 6918, 6919, 6900, 6901, 6902, 6904, 6908, 6911, 6916,
    6905, 6906, 6907, 6909, 6910, 6912, 6913, 6914, 6915, 6936, 6941, 6937, 6938, 6939, 6940, 6942,
    6943, 6944, 6945, 6959, 6960, 6963, 6965, 6961, 6962, 6964, 241, 242, 6966, 6968, 6967, 6969,
    241, 242, 6988, 6989, 6990, 6993, 6994, 7074, 7075, 6991, 6992, 6995, 6998, 7001, 7006, 7009,
    7012, 7016, 7020, 7026, 7030, 7034, 7038, 7044, 7047, 7050, 7056, 7059, 7062, 7065, 7068, 7071,
    6996, 6997, 6999, 7000, 7002, 7003, 7004, 7005, 241, 242, 7007, 7008, 7010, 7011, 7013, 7014,
    7015, 7017, 7018, 7019, 7021, 7022, 7023, 7024, 7025, 7025, 241, 242, 7027, 7028, 7029, 7031,
    7032, 7033, 7035, 7036, 7037, 7039, 7040, 7041, 7042, 7043, 7043, 241, 242, 7045, 7046, 7048,
    7049, 7051, 7052, 7053, 7054, 7055, 7055, 241, 242, 7057, 7058, 7060, 7061, 7063, 7064, 7066,
    7067, 7069, 7070, 7072, 7073, 7076, 7077, 7080, 7081, 7082, 7083, 7085, 7092, 7093, 7086, 7087,
    7088, 7089, 7090, 241, 242, 7091, 62, 7094, 7095, 7097, 7098, 234, 7099, 250, 7100, 62,
    7105, 7106, 7108, 7109, 7115, 7116, 7117, 7121, 7122, 7124, 7125, 7128, 7129, 7130, 7131, 7132,
    7123, 7126, 7127, 7135, 7136, 7137, 7138, 7139, 7140, 7141, 7142, 7143, 7144, 7145, 7146, 7147,
    7148, 7149, 7154, 7155, 7161, 7164, 7167, 7168, 7162, 7163, 7163, 241, 242, 7165, 7166, 7166,
    241, 242, 7169, 241, 242, 7173, 7174, 7175, 7176, 7177, 7180, 7178, 7179, 7186, 7189, 7187,
    7188, 7190, 7191, 7193, 7196, 7194, 7195, 7197, 7198, 7207, 7208, 7209, 7210, 7215, 234, 7216,
    250, 7217, 62, 7219, 7220, 7233, 7221, 7222, 7223, 7224, 7231, 7225, 7226, 7227, 7228, 7229,
    7230, 7232, 7234, 7237, 7238, 7235, 241, 242, 7236, 62, 7239, 241, 242, 7240, 7241, 7242,
    62, 7243, 7250, 7244, 7245, 7246, 7247, 7248, 7249, 7251, 7253, 7254, 7255, 7259, 7256, 7257,
    7258, 62, 7260, 7261, 7263, 7264, 7268, 7269, 7270, 7265, 7266, 7267, 7273, 7274, 7275, 7276,
    7278, 7279, 7280, 7281, 7283, 7284, 7291, 7292, 7285, 7288, 7286, 7287, 7289, 7290, 7293, 7294,
    7296, 7297, 7300, 7298, 7299, 7301, 7314, 7321, 7324, 7302, 7303, 7306, 7304, 7305, 7307, 7308,
    7309, 62, 7310, 7311, 7312, 7313, 7315, 7316, 7317, 7318, 7319, 7320, 7322, 7323, 7325, 7326,
    7327, 7328, 7335, 7338, 7336, 7337, 7339, 7340, 7341, 7342, 7344, 7345, 7346, 7347, 7348, 7351,
    7352, 7353, 7354, 7355, 7357, 7358, 7359, 7360, 7361, 7362, 7363, 7364, 7365, 7367, 7368, 7371,
    7376, 7377, 7378, 7380, 7381, 7384, 62, 7386, 7387, 7388, 7393, 7394, 7424, 7425, 7426, 7427,
    7428, 7429, 7430, 7431, 7432, 7433, 7434, 7435, 7436, 7437, 7438, 7439, 7440, 7441, 7442, 7443,
    7444, 7445, 7446, 7447, 7448, 7449, 7450, 7451, 7452, 7453, 7454, 7455, 7456, 7457, 7458, 7459,
    7460, 7461, 7462, 7463, 7464, 7465, 7466, 7467, 7468, 7469, 7470, 7471, 7472, 7473, 7474, 7475,
    7476, 7477, 7478, 7479, 7480, 7481, 7482, 7483, 7484, 7485, 7486, 7487, 7488, 7489, 7490, 7491,
    7492, 7493, 7494, 7495, 7496, 7497, 7498, 7499, 7500, 7501, 7502, 7503, 7504, 7505, 7506, 7507,
    7508, 7509, 7510, 7511, 7512, 7513, 7514, 7515, 7516, 7517, 7518, 7519, 7520, 7521, 7522, 7523,
    7524, 7525, 7526, 7527, 7528, 7529, 7530, 7531, 7532, 7533, 7534, 7535, 7536, 7537, 7538, 7539,
    7540, 7541, 7542, 7543, 7544, 7545, 7546, 7547, 7548, 7549, 7550, 7551, 7552, 7553, 7554, 7555,
    7556, 7557, 7558, 7559, 7560, 7561, 7562, 7563, 7564, 7565, 7566, 7567, 7568, 7569, 7570, 7585,
    7586, 7590, 241, 242, 7591, 62, 7592, 7593, 7618, 7625, 7629, 7641, 7619, 7620, 7621, 7622,
    7623, 7624, 7626, 7627, 7628, 241, 242, 7630, 7638, 7631, 7634, 7635, 7632, 7633, 7636, 241,
    242, 7637, 62, 7639, 7640, 7642, 7643, 7645, 7648, 7646, 7647, 7649, 7654, 7660, 7650, 7651,
    7652, 241, 242, 7653, 62, 7655, 7656, 7657, 7658, 241, 242, 7659, 62, 7661, 7662, 7663,
    7666, 7667, 7668, 7664, 241, 242, 7665, 62, 7669, 241, 242, 7670, 62, 7672, 7675, 7682,
    7687, 7673, 7674, 7676, 7679, 7680, 7681, 7677, 7678, 7683, 7684, 7685, 7686, 7688, 7691, 7689,
    7690, 7692, 7693, 7696, 7697, 7698, 7699, 7700, 7701, 7702, 7704, 7703, 241, 242, 7705, 241,
    242, 7713, 7716, 7714, 7718, 7720, 7719, 7732, 7733, 7736, 7737, 7749, 7752, 7753, 7750, 7751,
    7754, 7757, 7758, 7755, 7756, 7760, 7761, 7762, 7763, 7764, 7779, 7786, 7789, 7803, 7780, 7781,
    7782, 7783, 7784, 7785, 7787, 7788, 7790, 7791, 7792, 7793, 7794, 7797, 7800, 7795, 7796, 7798,
    7799, 7801, 7802, 7804, 7807, 7805, 7806, 7808, 7809, 7826, 7827, 7828, 7835, 7836, 7837, 7845,
    7850, 7838, 62, 7839, 7842, 7840, 7841, 7843, 7844, 7846, 7849, 7847, 7848, 7854, 241, 242,
    7855, 62, 7858, 7863, 7864, 7859, 7860, 7861, 7862, 7866, 7867, 7874, 7875, 7876, 7881, 7882,
    7883, 7884, 7887, 7895, 7896, 7897, 7898, 7901, 7902, 7904, 7905, 7906, 7903, 241, 242, 7907,
    7908, 7912, 7913, 7921, 7926, 7934, 7944, 7953, 7956, 7960, 7964, 7973, 7976, 7977, 7978, 7979,
    7980, 7909, 7910, 7911, 7914, 7917, 7915, 7916, 7918, 7919, 7920, 7922, 7923, 7924, 7925, 7927,
    7928, 7929, 7930, 7931, 7932, 7933, 7935, 7936, 7937, 7938, 7939, 7940, 7941, 7942, 7943, 7945,
    7949, 7946, 7947, 7948, 7950, 7951, 7952, 7954, 7955, 7957, 7958, 7959, 7961, 7962, 7963, 7965,
    7968, 7969, 7970, 7966, 7967, 7971, 7972, 7974, 7975, 7986, 7987, 7988, 7989, 7990, 7991, 7992,
    7993, 8000, 8001, 8002, 8003, 8004, 8013, 8014, 234, 8015, 8016, 250, 8018, 8019, 8020, 8027,
    8017, 8021, 8026, 8022, 8023, 8024, 8025, 8028, 234, 8029, 250, 8030, 8031, 8032, 62, 8033,
    8034, 8040, 8041, 8058, 8059, 8068, 8069, 8073, 8060, 8061, 8062, 8063, 8064, 8067, 8065, 8066,
    8070, 8071, 8072, 8080, 8081, 8087, 8088, 8089, 8082, 62, 8083, 241, 242, 8084, 62, 8085,
    8086, 8100, 8101, 8115, 8116, 8118, 8119, 8120, 8121, 8122, 8123, 2355, 8117, 250, 8127, 2355,
    8128, 250, 8129, 8130, 8130, 241, 242, 8132, 8133, 8136, 8139, 8140, 8141, 8142, 8143, 8144,
    8147, 8134, 8135, 8137, 8138, 8145, 8146, 8153, 8154, 8156, 8164, 8157, 8158, 8161, 8159, 8160,
    8162, 8163, 8167, 8171, 8172, 8173, 8168, 8169, 8170, 8175, 8176, 8179, 8180, 8177, 8178, 8182,
    8183, 8184, 8185, 8188, 8189, 8186, 62, 8191, 8192, 8193, 8194, 8197, 8198, 8195, 62, 8200,
    8201, 8203, 8202, 8207, 8208, 8209, 8210, 8212, 8213, 8214, 8215, 241, 242,
];

pub static TERMINATORS: &[u32] = &[
    63, 64, 140, 347, 368, 381, 382, 404, 405, 406, 412, 413, 2242, 2243, 2358, 2359,
    2361, 2368, 2369, 2371, 2373, 2374, 2375, 2442, 2536, 2537, 2538, 2539, 2540, 2541, 2542, 2543,
    2544, 2545, 2546, 2298, 3901, 3902, 4095, 4717, 4756, 5030, 5033, 5042, 5045, 5297, 5301, 5462,
    5465, 5466, 5467, 5468, 5469, 5487, 6348, 6349, 6607, 6620, 7101, 7102, 7103, 7104, 7107, 7110,
    7111, 7112, 7113, 7114, 7118, 7119, 7150, 7151, 7152, 7153, 7156, 7157, 7158, 7159, 7382, 7571,
    7715, 7994, 7995, 7996, 7997, 7998, 8187, 8196,
];

pub static STRINGS: &[&str] = &[
    "A", // [0]
    "keyword", // [1]
    "KeywordSegment", // [2]
    "base", // [3]
    "raw", // [4]
    "word", // [5]
    "ABORT", // [6]
    "ABS", // [7]
    "ABSENT", // [8]
    "ABSOLUTE", // [9]
    "ACCEPTANYDATE", // [10]
    "ACCEPTINVCHARS", // [11]
    "ACCESS", // [12]
    "AccountKeywordSegment", // [13]
    "ACCOUNT", // [14]
    "ALL", // [15]
    "CATALOG", // [16]
    "DATABASE", // [17]
    "DOMAIN", // [18]
    "EXTERNAL", // [19]
    "FILE", // [20]
    "FOREIGN", // [21]
    "FUNCTION", // [22]
    "FUTURE", // [23]
    "INTEGRATION", // [24]
    "LANGUAGE", // [25]
    "MASKING", // [26]
    "MATERIALIZED", // [27]
    "PROCEDURE", // [28]
    "RESOURCE", // [29]
    "ROLE", // [30]
    "ROUTINE", // [31]
    "SCHEMA", // [32]
    "SEQUENCE", // [33]
    "STAGE", // [34]
    "STREAM", // [35]
    "TABLE", // [36]
    "TABLESPACE", // [37]
    "TASK", // [38]
    "TYPE", // [39]
    "VIEW", // [40]
    "WAREHOUSE", // [41]
    "ResourceKeywordSegment", // [42]
    "MonitorKeywordSegment", // [43]
    "MONITOR", // [44]
    "WarehouseKeywordSegment", // [45]
    "DatabaseKeywordSegment", // [46]
    "DomainKeywordSegment", // [47]
    "IntegrationKeywordSegment", // [48]
    "LanguageKeywordSegment", // [49]
    "SchemaKeywordSegment", // [50]
    "CatalogKeywordSegment", // [51]
    "RoleKeywordSegment", // [52]
    "TablespaceKeywordSegment", // [53]
    "TypeKeywordSegment", // [54]
    "ForeignKeywordSegment", // [55]
    "DATA", // [56]
    "SERVER", // [57]
    "ServerKeywordSegment", // [58]
    "DataKeywordSegment", // [59]
    "WrapperKeywordSegment", // [60]
    "WRAPPER", // [61]
    "AllKeywordSegment", // [62]
    "SchemasKeywordSegment", // [63]
    "SCHEMAS", // [64]
    "InKeywordSegment", // [65]
    "IN", // [66]
    "FutureKeywordSegment", // [67]
    "AccessSchemaObjectSegment", // [68]
    "access_schema_object", // [69]
    "AccessSchemaPluralObjectSegment", // [70]
    "FUNCTIONS", // [71]
    "PROCEDURES", // [72]
    "ROUTINES", // [73]
    "SEQUENCES", // [74]
    "STAGES", // [75]
    "STREAMS", // [76]
    "TABLES", // [77]
    "TASKS", // [78]
    "VIEWS", // [79]
    "access_schema_plural_object", // [80]
    "ObjectReferenceSegment", // [81]
    "object_reference", // [82]
    "FunctionNameSegment", // [83]
    "function_name", // [84]
    "FunctionParameterListGrammar", // [85]
    "(", // [86]
    "function_parameter_list", // [87]
    "WildcardIdentifierSegment", // [88]
    "wildcard_identifier", // [89]
    "CommaSegment", // [90]
    ",", // [91]
    "ToKeywordSegment", // [92]
    "TO", // [93]
    "FromKeywordSegment", // [94]
    "FROM", // [95]
    "LARGE", // [96]
    "LargeKeywordSegment", // [97]
    "ObjectKeywordSegment", // [98]
    "OBJECT", // [99]
    "NumericLiteralSegment", // [100]
    "numeric_literal", // [101]
    "APPLY", // [102]
    "CONNECT", // [103]
    "CREATE", // [104]
    "DELETE", // [105]
    "EXECUTE", // [106]
    "IMPORTED", // [107]
    "INSERT", // [108]
    "MANAGE", // [109]
    "MODIFY", // [110]
    "OPERATE", // [111]
    "OWNERSHIP", // [112]
    "READ", // [113]
    "REFERENCES", // [114]
    "REFERENCE_USAGE", // [115]
    "SELECT", // [116]
    "TEMP", // [117]
    "TEMPORARY", // [118]
    "TRIGGER", // [119]
    "TRUNCATE", // [120]
    "UPDATE", // [121]
    "USAGE", // [122]
    "USE", // [123]
    "USE_ANY_ROLE", // [124]
    "WRITE", // [125]
    "CreateKeywordSegment", // [126]
    "USER", // [127]
    "UserKeywordSegment", // [128]
    "ApplyKeywordSegment", // [129]
    "MaskingKeywordSegment", // [130]
    "PolicyKeywordSegment", // [131]
    "POLICY", // [132]
    "ExecuteKeywordSegment", // [133]
    "TaskKeywordSegment", // [134]
    "ManageKeywordSegment", // [135]
    "GrantsKeywordSegment", // [136]
    "GRANTS", // [137]
    "EXECUTION", // [138]
    "ExecutionKeywordSegment", // [139]
    "UsageKeywordSegment", // [140]
    "PIPE", // [141]
    "PipeKeywordSegment", // [142]
    "UseKeywordSegment", // [143]
    "ImportedKeywordSegment", // [144]
    "PrivilegesKeywordSegment", // [145]
    "PRIVILEGES", // [146]
    "ConnectKeywordSegment", // [147]
    "DeleteKeywordSegment", // [148]
    "InsertKeywordSegment", // [149]
    "ModifyKeywordSegment", // [150]
    "OperateKeywordSegment", // [151]
    "OwnershipKeywordSegment", // [152]
    "ReadKeywordSegment", // [153]
    "Reference_usageKeywordSegment", // [154]
    "ReferencesKeywordSegment", // [155]
    "SelectKeywordSegment", // [156]
    "TempKeywordSegment", // [157]
    "TemporaryKeywordSegment", // [158]
    "TriggerKeywordSegment", // [159]
    "TruncateKeywordSegment", // [160]
    "UpdateKeywordSegment", // [161]
    "Use_any_roleKeywordSegment", // [162]
    "WriteKeywordSegment", // [163]
    "BracketedColumnReferenceListGrammar", // [164]
    "AccessPermissionSegment", // [165]
    "access_permission", // [166]
    "OnKeywordSegment", // [167]
    "ON", // [168]
    "TableKeywordSegment", // [169]
    "ViewKeywordSegment", // [170]
    "StageKeywordSegment", // [171]
    "FunctionKeywordSegment", // [172]
    "ProcedureKeywordSegment", // [173]
    "RoutineKeywordSegment", // [174]
    "SequenceKeywordSegment", // [175]
    "StreamKeywordSegment", // [176]
    "MaterializedKeywordSegment", // [177]
    "ExternalKeywordSegment", // [178]
    "FileKeywordSegment", // [179]
    "FormatKeywordSegment", // [180]
    "FORMAT", // [181]
    "TablesKeywordSegment", // [182]
    "ViewsKeywordSegment", // [183]
    "StagesKeywordSegment", // [184]
    "FunctionsKeywordSegment", // [185]
    "ProceduresKeywordSegment", // [186]
    "RoutinesKeywordSegment", // [187]
    "SequencesKeywordSegment", // [188]
    "StreamsKeywordSegment", // [189]
    "TasksKeywordSegment", // [190]
    "PoliciesKeywordSegment", // [191]
    "FormatsKeywordSegment", // [192]
    "GRANT", // [193]
    "REVOKE", // [194]
    "GrantStatementSegment", // [195]
    "grant_statement", // [196]
    "RevokeStatementSegment", // [197]
    "revoke_statement", // [198]
    "GROUP", // [199]
    "GroupKeywordSegment", // [200]
    "UserReferenceSegment", // [201]
    "user_reference", // [202]
    "RoleReferenceSegment", // [203]
    "role_reference", // [204]
    "SHARE", // [205]
    "ShareKeywordSegment", // [206]
    "FunctionSegment", // [207]
    "function", // [208]
    "PublicKeywordSegment", // [209]
    "PUBLIC", // [210]
    "ACCESS_KEY_ID", // [211]
    ".", // [212]
    "[", // [213]
    "ArrayAccessorSegment", // [214]
    "array_accessor", // [215]
    "SemiStructuredAccessorSegment", // [216]
    "semi_structured_expression", // [217]
    "ACCORDING", // [218]
    "ACOS", // [219]
    "ACTION", // [220]
    "ADA", // [221]
    "ADD", // [222]
    "ADDQUOTES", // [223]
    "-", // [224]
    "MinusSegment", // [225]
    "PipeSegment", // [226]
    "|", // [227]
    "ADMIN", // [228]
    "AES128", // [229]
    "AES256", // [230]
    "AFTER", // [231]
    "AGGREGATE", // [232]
    "OrderByClauseSegment", // [233]
    "ORDER", // [234]
    "orderby_clause", // [235]
    "indent", // [236]
    "Indent", // [237]
    "meta", // [238]
    "AsAliasOperatorSegment", // [239]
    "AS", // [240]
    "alias_operator", // [241]
    "SingleIdentifierGrammar", // [242]
    "SingleIdentifierListSegment", // [243]
    "identifier_list", // [244]
    "start_bracket", // [245]
    "SymbolSegment", // [246]
    "symbol", // [247]
    ")", // [248]
    "end_bracket", // [249]
    "ParameterNameSegment", // [250]
    "DatatypeSegment", // [251]
    "data_type", // [252]
    "dedent", // [253]
    "Dedent", // [254]
    "TableReferenceSegment", // [255]
    "table_reference", // [256]
    "AliasExpressionSegment", // [257]
    "alias_expression", // [258]
    "ALLOCATE", // [259]
    "ALLOWOVERWRITE", // [260]
    "ALSO", // [261]
    "ALTER", // [262]
    "AlterKeywordSegment", // [263]
    "AggregateKeywordSegment", // [264]
    "StarSegment", // [265]
    "*", // [266]
    "OWNER", // [267]
    "RENAME", // [268]
    "SET", // [269]
    "RenameKeywordSegment", // [270]
    "OwnerKeywordSegment", // [271]
    "Current_roleKeywordSegment", // [272]
    "CURRENT_ROLE", // [273]
    "Current_userKeywordSegment", // [274]
    "CURRENT_USER", // [275]
    "Session_userKeywordSegment", // [276]
    "SESSION_USER", // [277]
    "SetKeywordSegment", // [278]
    "SchemaReferenceSegment", // [279]
    "schema_reference", // [280]
    "DatabaseReferenceSegment", // [281]
    "database_reference", // [282]
    "WithKeywordSegment", // [283]
    "WITH", // [284]
    "Allow_connectionsKeywordSegment", // [285]
    "BooleanLiteralGrammar", // [286]
    "FALSE", // [287]
    "TRUE", // [288]
    "CONNECTION", // [289]
    "ConnectionKeywordSegment", // [290]
    "LimitKeywordSegment", // [291]
    "LIMIT", // [292]
    "Is_templateKeywordSegment", // [293]
    "TablespaceReferenceSegment", // [294]
    "tablespace_reference", // [295]
    "=", // [296]
    "EqualsSegment", // [297]
    "comparison_operator", // [298]
    "DefaultKeywordSegment", // [299]
    "DEFAULT", // [300]
    "LiteralGrammar", // [301]
    "+", // [302]
    "ARRAY", // [303]
    "DATE", // [304]
    "DATETIME", // [305]
    "MAX", // [306]
    "NULL", // [307]
    "TIME", // [308]
    "TIMESTAMP", // [309]
    "TIMESTAMPTZ", // [310]
    "TIMETZ", // [311]
    "VARIADIC", // [312]
    "{", // [313]
    "bit_string_literal", // [314]
    "dollar_numeric_literal", // [315]
    "dollar_quote", // [316]
    "escaped_single_quote", // [317]
    "newline", // [318]
    "single_quote", // [319]
    "unicode_single_quote", // [320]
    "NakedIdentifierSegment", // [321]
    "QuotedIdentifierSegment", // [322]
    "double_quote", // [323]
    "unicode_double_quote", // [324]
    "CurrentKeywordSegment", // [325]
    "CURRENT", // [326]
    "RESET", // [327]
    "ResetKeywordSegment", // [328]
    "GrantKeywordSegment", // [329]
    "AlterDefaultPrivilegesObjectPrivilegesSegment", // [330]
    "alter_default_privileges_object_privilege", // [331]
    "AlterDefaultPrivilegesSchemaObjectsSegment", // [332]
    "TYPES", // [333]
    "alter_default_privileges_schema_object", // [334]
    "AlterDefaultPrivilegesToFromRolesSegment", // [335]
    "alter_default_privileges_to_from_roles", // [336]
    "OptionKeywordSegment", // [337]
    "OPTION", // [338]
    "RevokeKeywordSegment", // [339]
    "ForKeywordSegment", // [340]
    "FOR", // [341]
    "RestrictKeywordSegment", // [342]
    "RESTRICT", // [343]
    "CascadeKeywordSegment", // [344]
    "CASCADE", // [345]
    "DropBehaviorGrammar", // [346]
    "TypesKeywordSegment", // [347]
    "AlterDefaultPrivilegesGrantSegment", // [348]
    "alter_default_privileges_grant", // [349]
    "AlterDefaultPrivilegesRevokeSegment", // [350]
    "alter_default_privileges_revoke", // [351]
    "DROP", // [352]
    "VALIDATE", // [353]
    "ExpressionSegment", // [354]
    "expression", // [355]
    "DropKeywordSegment", // [356]
    "NotKeywordSegment", // [357]
    "NOT", // [358]
    "NullKeywordSegment", // [359]
    "AddKeywordSegment", // [360]
    "CONSTRAINT", // [361]
    "ConstraintKeywordSegment", // [362]
    "CHECK", // [363]
    "CheckKeywordSegment", // [364]
    "ValidKeywordSegment", // [365]
    "VALID", // [366]
    "IfExistsGrammar", // [367]
    "IF", // [368]
    "ValidateKeywordSegment", // [369]
    "ExtensionKeywordSegment", // [370]
    "EXTENSION", // [371]
    "ExtensionReferenceSegment", // [372]
    "extension_reference", // [373]
    "Current_schemaKeywordSegment", // [374]
    "CURRENT_SCHEMA", // [375]
    "CAST", // [376]
    "COLLATION", // [377]
    "CONVERSION", // [378]
    "EVENT", // [379]
    "OPERATOR", // [380]
    "PROCEDURAL", // [381]
    "TEXT", // [382]
    "TRANSFORM", // [383]
    "AccessKeywordSegment", // [384]
    "MethodKeywordSegment", // [385]
    "METHOD", // [386]
    "CollationKeywordSegment", // [387]
    "ConversionKeywordSegment", // [388]
    "EventKeywordSegment", // [389]
    "ProceduralKeywordSegment", // [390]
    "TextKeywordSegment", // [391]
    "SearchKeywordSegment", // [392]
    "SEARCH", // [393]
    "CONFIGURATION", // [394]
    "DICTIONARY", // [395]
    "PARSER", // [396]
    "TEMPLATE", // [397]
    "ConfigurationKeywordSegment", // [398]
    "DictionaryKeywordSegment", // [399]
    "ParserKeywordSegment", // [400]
    "TemplateKeywordSegment", // [401]
    "CastKeywordSegment", // [402]
    "AsKeywordSegment", // [403]
    "OperatorKeywordSegment", // [404]
    "CLASS", // [405]
    "FAMILY", // [406]
    "ClassKeywordSegment", // [407]
    "FamilyKeywordSegment", // [408]
    "UsingKeywordSegment", // [409]
    "USING", // [410]
    "IndexAccessMethodSegment", // [411]
    "index_access_method", // [412]
    "TransformKeywordSegment", // [413]
    "CLUSTER", // [414]
    "DISABLE", // [415]
    "ENABLE", // [416]
    "FORCE", // [417]
    "INHERIT", // [418]
    "NO", // [419]
    "OF", // [420]
    "OPTIONS", // [421]
    "REPLICA", // [422]
    "ColumnKeywordSegment", // [423]
    "COLUMN", // [424]
    "IfNotExistsGrammar", // [425]
    "ColumnReferenceSegment", // [426]
    "column_reference", // [427]
    "COLLATE", // [428]
    "CollateKeywordSegment", // [429]
    "CollationReferenceSegment", // [430]
    "collation_reference", // [431]
    "PRIMARY", // [432]
    "UNIQUE", // [433]
    "ColumnConstraintSegment", // [434]
    "column_constraint_segment", // [435]
    "RESTART", // [436]
    "BareFunctionSegment", // [437]
    "CURRENT_AWS_ACCOUNT", // [438]
    "CURRENT_DATE", // [439]
    "CURRENT_NAMESPACE", // [440]
    "CURRENT_TIME", // [441]
    "CURRENT_TIMESTAMP", // [442]
    "CURRENT_USER_ID", // [443]
    "SYSDATE", // [444]
    "ExpressionKeywordSegment", // [445]
    "EXPRESSION", // [446]
    "GeneratedKeywordSegment", // [447]
    "GENERATED", // [448]
    "ALWAYS", // [449]
    "BY", // [450]
    "AlwaysKeywordSegment", // [451]
    "ByKeywordSegment", // [452]
    "IdentityKeywordSegment", // [453]
    "IDENTITY", // [454]
    "CACHE", // [455]
    "CYCLE", // [456]
    "INCREMENT", // [457]
    "MAXVALUE", // [458]
    "MINVALUE", // [459]
    "OWNED", // [460]
    "START", // [461]
    "AlterSequenceOptionsSegment", // [462]
    "alter_sequence_options_segment", // [463]
    "RestartKeywordSegment", // [464]
    "StatisticsKeywordSegment", // [465]
    "STATISTICS", // [466]
    "RelationOptionsSegment", // [467]
    "relation_options", // [468]
    "StorageKeywordSegment", // [469]
    "STORAGE", // [470]
    "EXTENDED", // [471]
    "MAIN", // [472]
    "PLAIN", // [473]
    "PlainKeywordSegment", // [474]
    "ExtendedKeywordSegment", // [475]
    "MainKeywordSegment", // [476]
    "CompressionKeywordSegment", // [477]
    "COMPRESSION", // [478]
    "TableConstraintSegment", // [479]
    "table_constraint", // [480]
    "TableConstraintUsingIndexSegment", // [481]
    "DEFERRABLE", // [482]
    "DeferrableKeywordSegment", // [483]
    "INITIALLY", // [484]
    "InitiallyKeywordSegment", // [485]
    "DeferredKeywordSegment", // [486]
    "DEFERRED", // [487]
    "ImmediateKeywordSegment", // [488]
    "IMMEDIATE", // [489]
    "EnableKeywordSegment", // [490]
    "DisableKeywordSegment", // [491]
    "ReplicaKeywordSegment", // [492]
    "RuleKeywordSegment", // [493]
    "RULE", // [494]
    "ForceKeywordSegment", // [495]
    "NoKeywordSegment", // [496]
    "RowKeywordSegment", // [497]
    "ROW", // [498]
    "LevelKeywordSegment", // [499]
    "LEVEL", // [500]
    "SecurityKeywordSegment", // [501]
    "SECURITY", // [502]
    "ClusterKeywordSegment", // [503]
    "WithoutKeywordSegment", // [504]
    "WITHOUT", // [505]
    "OIDS", // [506]
    "OidsKeywordSegment", // [507]
    "LOGGED", // [508]
    "UNLOGGED", // [509]
    "LoggedKeywordSegment", // [510]
    "UnloggedKeywordSegment", // [511]
    "InheritKeywordSegment", // [512]
    "OfKeywordSegment", // [513]
    "FULL", // [514]
    "NOTHING", // [515]
    "IndexKeywordSegment", // [516]
    "INDEX", // [517]
    "IndexReferenceSegment", // [518]
    "index_reference", // [519]
    "FullKeywordSegment", // [520]
    "NothingKeywordSegment", // [521]
    "OptionsKeywordSegment", // [522]
    "QuotedLiteralSegment", // [523]
    "OnlyKeywordSegment", // [524]
    "ONLY", // [525]
    "AlterForeignTableActionSegment", // [526]
    "alter_foreign_table_action_segment", // [527]
    "alter_table_action_segment", // [528]
    "CALLED", // [529]
    "COST", // [530]
    "IMMUTABLE", // [531]
    "LEAKPROOF", // [532]
    "PARALLEL", // [533]
    "RETURNS", // [534]
    "ROWS", // [535]
    "STABLE", // [536]
    "STRICT", // [537]
    "SUPPORT", // [538]
    "VOLATILE", // [539]
    "CalledKeywordSegment", // [540]
    "InputKeywordSegment", // [541]
    "INPUT", // [542]
    "ReturnsKeywordSegment", // [543]
    "StrictKeywordSegment", // [544]
    "ImmutableKeywordSegment", // [545]
    "StableKeywordSegment", // [546]
    "VolatileKeywordSegment", // [547]
    "LeakproofKeywordSegment", // [548]
    "DEFINER", // [549]
    "INVOKER", // [550]
    "DefinerKeywordSegment", // [551]
    "InvokerKeywordSegment", // [552]
    "ParallelKeywordSegment", // [553]
    "RESTRICTED", // [554]
    "SAFE", // [555]
    "UNSAFE", // [556]
    "UnsafeKeywordSegment", // [557]
    "RestrictedKeywordSegment", // [558]
    "SafeKeywordSegment", // [559]
    "CostKeywordSegment", // [560]
    "RowsKeywordSegment", // [561]
    "SupportKeywordSegment", // [562]
    "DEPENDS", // [563]
    "AlterFunctionActionSegment", // [564]
    "alter_function_action_segment", // [565]
    "DependsKeywordSegment", // [566]
    "ATTACH", // [567]
    "AttachKeywordSegment", // [568]
    "PartitionKeywordSegment", // [569]
    "PARTITION", // [570]
    "OwnedKeywordSegment", // [571]
    "NowaitKeywordSegment", // [572]
    "NOWAIT", // [573]
    "QualifiedParameterNameGrammar", // [574]
    "AlterMaterializedViewActionSegment", // [575]
    "alter_materialized_view_action_segment", // [576]
    "ProcedureParameterListSegment", // [577]
    "procedure_parameter_list", // [578]
    "PublicationKeywordSegment", // [579]
    "PUBLICATION", // [580]
    "PublicationReferenceSegment", // [581]
    "publication_reference", // [582]
    "DefinitionParametersSegment", // [583]
    "definition_parameters", // [584]
    "PublicationObjectsSegment", // [585]
    "publication_objects", // [586]
    "BYPASSRLS", // [587]
    "CREATEDB", // [588]
    "CREATEROLE", // [589]
    "ENCRYPTED", // [590]
    "LOGIN", // [591]
    "NOBYPASSRLS", // [592]
    "NOCREATEDB", // [593]
    "NOCREATEROLE", // [594]
    "NOINHERIT", // [595]
    "NOLOGIN", // [596]
    "NOREPLICATION", // [597]
    "NOSUPERUSER", // [598]
    "PASSWORD", // [599]
    "REPLICATION", // [600]
    "SUPERUSER", // [601]
    "SuperuserKeywordSegment", // [602]
    "NosuperuserKeywordSegment", // [603]
    "CreatedbKeywordSegment", // [604]
    "NocreatedbKeywordSegment", // [605]
    "CreateroleKeywordSegment", // [606]
    "NocreateroleKeywordSegment", // [607]
    "NoinheritKeywordSegment", // [608]
    "LoginKeywordSegment", // [609]
    "NologinKeywordSegment", // [610]
    "ReplicationKeywordSegment", // [611]
    "NoreplicationKeywordSegment", // [612]
    "BypassrlsKeywordSegment", // [613]
    "NobypassrlsKeywordSegment", // [614]
    "EncryptedKeywordSegment", // [615]
    "PasswordKeywordSegment", // [616]
    "UntilKeywordSegment", // [617]
    "UNTIL", // [618]
    "OnKeywordAsIdentifierSegment", // [619]
    "QUOTA", // [620]
    "QuotaGrammar", // [621]
    "IncrementKeywordSegment", // [622]
    "SignedSegmentGrammar", // [623]
    "MinvalueKeywordSegment", // [624]
    "MaxvalueKeywordSegment", // [625]
    "NameKeywordSegment", // [626]
    "NAME", // [627]
    "SequenceReferenceSegment", // [628]
    "sequence_reference", // [629]
    "StartKeywordSegment", // [630]
    "CacheKeywordSegment", // [631]
    "CycleKeywordSegment", // [632]
    "NoneKeywordSegment", // [633]
    "NONE", // [634]
    "StatisticsReferenceSegment", // [635]
    "statistics_reference", // [636]
    "SubscriptionKeywordSegment", // [637]
    "SUBSCRIPTION", // [638]
    "SubscriptionReferenceSegment", // [639]
    "subscription_reference", // [640]
    "REFRESH", // [641]
    "SKIP", // [642]
    "RefreshKeywordSegment", // [643]
    "SkipKeywordSegment", // [644]
    "RawEqualsSegment", // [645]
    "Current_sessionKeywordSegment", // [646]
    "SystemKeywordSegment", // [647]
    "SYSTEM", // [648]
    "APPEND", // [649]
    "ENCODE", // [650]
    "EncodeKeywordSegment", // [651]
    "AZ64", // [652]
    "BYTEDICT", // [653]
    "DELTA", // [654]
    "DELTA32K", // [655]
    "LZO", // [656]
    "MOSTLY16", // [657]
    "MOSTLY32", // [658]
    "MOSTLY8", // [659]
    "RAW", // [660]
    "RUNLENGTH", // [661]
    "TEXT255", // [662]
    "TEXT32K", // [663]
    "ZSTD", // [664]
    "ColumnEncodingGrammar", // [665]
    "DistkeyKeywordSegment", // [666]
    "DISTKEY", // [667]
    "DiststyleKeywordSegment", // [668]
    "DISTSTYLE", // [669]
    "AUTO", // [670]
    "EVEN", // [671]
    "KEY", // [672]
    "EvenKeywordSegment", // [673]
    "KeyKeywordSegment", // [674]
    "AutoKeywordSegment", // [675]
    "CompoundKeywordSegment", // [676]
    "COMPOUND", // [677]
    "SortkeyKeywordSegment", // [678]
    "SORTKEY", // [679]
    "AppendKeywordSegment", // [680]
    "IgnoreextraKeywordSegment", // [681]
    "IGNOREEXTRA", // [682]
    "FilltargetKeywordSegment", // [683]
    "FILLTARGET", // [684]
    "LocationKeywordSegment", // [685]
    "LOCATION", // [686]
    "ColumnDefinitionSegment", // [687]
    "column_definition", // [688]
    "FIRST", // [689]
    "FirstKeywordSegment", // [690]
    "AfterKeywordSegment", // [691]
    "AlterTableDropColumnGrammar", // [692]
    "AlterTableActionSegment", // [693]
    "DETACH", // [694]
    "ValuesKeywordSegment", // [695]
    "VALUES", // [696]
    "PartitionBoundSpecSegment", // [697]
    "partition_bound_spec", // [698]
    "DetachKeywordSegment", // [699]
    "ConcurrentlyKeywordSegment", // [700]
    "CONCURRENTLY", // [701]
    "FinalizeKeywordSegment", // [702]
    "FINALIZE", // [703]
    "MappingKeywordSegment", // [704]
    "MAPPING", // [705]
    "ReplaceKeywordSegment", // [706]
    "REPLACE", // [707]
    "TriggerReferenceSegment", // [708]
    "trigger_reference", // [709]
    "ValueKeywordSegment", // [710]
    "VALUE", // [711]
    "AttributeKeywordSegment", // [712]
    "ATTRIBUTE", // [713]
    "CascadeRestrictGrammar", // [714]
    "BEFORE", // [715]
    "BeforeKeywordSegment", // [716]
    "CREATEUSER", // [717]
    "NOCREATEUSER", // [718]
    "SESSION", // [719]
    "SYSLOG", // [720]
    "CreateuserKeywordSegment", // [721]
    "NocreateuserKeywordSegment", // [722]
    "SyslogKeywordSegment", // [723]
    "UNRESTRICTED", // [724]
    "UnrestrictedKeywordSegment", // [725]
    "ConnectionLimitSegment", // [726]
    "connection_limit_segment", // [727]
    "SessionKeywordSegment", // [728]
    "TimeoutKeywordSegment", // [729]
    "TIMEOUT", // [730]
    "DatashareKeywordSegment", // [731]
    "DATASHARE", // [732]
    "REMOVE", // [733]
    "RemoveKeywordSegment", // [734]
    "INCLUDENEW", // [735]
    "PUBLICACCESSIBLE", // [736]
    "PublicaccessibleKeywordSegment", // [737]
    "IncludenewKeywordSegment", // [738]
    "&", // [739]
    "ampersand", // [740]
    "ANALYSE", // [741]
    "ANALYZE", // [742]
    "AnalyzeKeywordSegment", // [743]
    "AnalyseKeywordSegment", // [744]
    "COMPROWS", // [745]
    "ComprowsKeywordSegment", // [746]
    "VERBOSE", // [747]
    "VerboseKeywordSegment", // [748]
    "Skip_lockedKeywordSegment", // [749]
    "AND", // [750]
    "binary_operator", // [751]
    "BinaryOperatorSegment", // [752]
    "ANY", // [753]
    "ANYELEMENT", // [754]
    "APPROXIMATE", // [755]
    "ARE", // [756]
    "INOUT", // [757]
    "OUT", // [758]
    "OutKeywordSegment", // [759]
    "InoutKeywordSegment", // [760]
    "%", // [761]
    "/", // [762]
    "<", // [763]
    ">", // [764]
    "^", // [765]
    "PlusSegment", // [766]
    "DivideSegment", // [767]
    "MultiplySegment", // [768]
    "ModuloSegment", // [769]
    "BitwiseAndSegment", // [770]
    "BitwiseOrSegment", // [771]
    "BitwiseXorSegment", // [772]
    "BitwiseLShiftSegment", // [773]
    "BitwiseRShiftSegment", // [774]
    "ArnKeywordSegment", // [775]
    "ARN", // [776]
    "start_square_bracket", // [777]
    "]", // [778]
    "end_square_bracket", // [779]
    "BaseExpressionElementGrammar", // [780]
    "ArrayKeywordSegment", // [781]
    "AtKeywordSegment", // [782]
    "AT", // [783]
    "ARRAY_AGG", // [784]
    "ARRAY_MAX_CARDINALITY", // [785]
    "ASC", // [786]
    "ASENSITIVE", // [787]
    "ASIN", // [788]
    "ASSERTION", // [789]
    "ASSIGNMENT", // [790]
    "ASYMMETRIC", // [791]
    "ATAN", // [792]
    "ATOMIC", // [793]
    "ATTRIBUTES", // [794]
    "AUTHENTICATION", // [795]
    "AUTHENTICATION_ARN", // [796]
    "AUTHORIZATION", // [797]
    "CREDENTIALS", // [798]
    "IAM_ROLE", // [799]
    "KMS_KEY_ID", // [800]
    "MASTER_SYMMETRIC_KEY", // [801]
    "Iam_roleKeywordSegment", // [802]
    "CredentialsKeywordSegment", // [803]
    "Access_key_idKeywordSegment", // [804]
    "Secret_access_keyKeywordSegment", // [805]
    "SECRET_ACCESS_KEY", // [806]
    "SESSION_TOKEN", // [807]
    "Session_tokenKeywordSegment", // [808]
    "Kms_key_idKeywordSegment", // [809]
    "Master_symmetric_keyKeywordSegment", // [810]
    "AUTO_INCREMENT", // [811]
    "Auto_incrementKeywordSegment", // [812]
    "AVG", // [813]
    "AVRO", // [814]
    "BACKUP", // [815]
    "BACKWARD", // [816]
    "bare_function", // [817]
    "CodeSegment", // [818]
    "BASE64", // [819]
    "IntervalExpressionSegment", // [820]
    "INTERVAL", // [821]
    "interval_expression", // [822]
    "CompositeValueExpansionSegment", // [823]
    "composite_value_expansion", // [824]
    "file", // [825]
    "BaseFileSegment", // [826]
    "BaseSegment", // [827]
    "BEGIN", // [828]
    "BEGIN_FRAME", // [829]
    "BEGIN_PARTITION", // [830]
    "BERNOULLI", // [831]
    "BETWEEN", // [832]
    "BIGINT", // [833]
    "BINARY", // [834]
    "!", // [835]
    "IS", // [836]
    "OR", // [837]
    "json_operator", // [838]
    "like_operator", // [839]
    "pg_trgm_operator", // [840]
    "pgvector_operator", // [841]
    "postgis_operator", // [842]
    "ArithmeticBinaryOperatorGrammar", // [843]
    "StringBinaryOperatorGrammar", // [844]
    "BooleanBinaryOperatorGrammar", // [845]
    "ComparisonOperatorGrammar", // [846]
    "JsonOperatorSegment", // [847]
    "BINARY_CLASSIFICATION", // [848]
    "BINDING", // [849]
    "BIT", // [850]
    "BIT_LENGTH", // [851]
    "AmpersandSegment", // [852]
    "RawLessThanSegment", // [853]
    "RawGreaterThanSegment", // [854]
    "BLANKSASNULL", // [855]
    "BLOB", // [856]
    "BLOCKED", // [857]
    "BOM", // [858]
    "BOOL", // [859]
    "AndOperatorGrammar", // [860]
    "OrOperatorGrammar", // [861]
    "BOOLEAN", // [862]
    "TrueSegment", // [863]
    "FalseSegment", // [864]
    "BOOST", // [865]
    "BOTH", // [866]
    "BPCHAR", // [867]
    "MaxKeywordSegment", // [868]
    "bracketed", // [869]
    "BracketedSegment", // [870]
    "SetExpressionSegment", // [871]
    "set_expression", // [872]
    "BREADTH", // [873]
    "BUFFERS", // [874]
    "BZIP2", // [875]
    "C", // [876]
    "CTEColumnList", // [877]
    "cte_column_list", // [878]
    "SelectableGrammar", // [879]
    "DEPTH", // [880]
    "BreadthKeywordSegment", // [881]
    "DepthKeywordSegment", // [882]
    "CALL", // [883]
    "CallKeywordSegment", // [884]
    "CARDINALITY", // [885]
    "CASCADED", // [886]
    "CASE", // [887]
    "CaseKeywordSegment", // [888]
    "implicit_indent", // [889]
    "ImplicitIndent", // [890]
    "WHEN", // [891]
    "WhenClauseSegment", // [892]
    "when_clause", // [893]
    "ElseKeywordSegment", // [894]
    "ELSE", // [895]
    "EndKeywordSegment", // [896]
    "END", // [897]
    "ElseClauseSegment", // [898]
    "else_clause", // [899]
    "BinaryOperatorGrammar", // [900]
    "CASE_INSENSITIVE", // [901]
    "CASE_SENSITIVE", // [902]
    "::", // [903]
    "casting_operator", // [904]
    "CATALOG_NAME", // [905]
    "CATALOG_ROLE", // [906]
    "CEIL", // [907]
    "CEILING", // [908]
    "CHAIN", // [909]
    "CHAINING", // [910]
    "CHAR", // [911]
    "CHAR_LENGTH", // [912]
    "CHARACTER", // [913]
    "CHARACTER_LENGTH", // [914]
    "CHARACTER_SET_CATALOG", // [915]
    "CHARACTER_SET_NAME", // [916]
    "CHARACTER_SET_SCHEMA", // [917]
    "CHARACTERISTICS", // [918]
    "CHARACTERS", // [919]
    "CHECKPOINT", // [920]
    "CI", // [921]
    "CLASS_ORIGIN", // [922]
    "CLASSIFIER", // [923]
    "CLEANPATH", // [924]
    "CLOB", // [925]
    "CLOSE", // [926]
    "CloseKeywordSegment", // [927]
    "COALESCE", // [928]
    "COBOL", // [929]
    "CS", // [930]
    "Case_sensitiveKeywordSegment", // [931]
    "CsKeywordSegment", // [932]
    "Case_insensitiveKeywordSegment", // [933]
    "CiKeywordSegment", // [934]
    "ObjectReferenceDelimiterGrammar", // [935]
    "ObjectReferenceTerminatorGrammar", // [936]
    ":", // [937]
    ";", // [938]
    "PIVOT", // [939]
    "UNPIVOT", // [940]
    "COLLATION_CATALOG", // [941]
    "COLLATION_NAME", // [942]
    "COLLATION_SCHEMA", // [943]
    "COLLECT", // [944]
    "colon_delimiter", // [945]
    "colon_prefix", // [946]
    "colon", // [947]
    "ShorthandCastSegment", // [948]
    "cast_expression", // [949]
    "UniqueKeywordSegment", // [950]
    "PrimaryKeyGrammar", // [951]
    "RawKeywordSegment", // [952]
    "Az64KeywordSegment", // [953]
    "BytedictKeywordSegment", // [954]
    "DeltaKeywordSegment", // [955]
    "Delta32kKeywordSegment", // [956]
    "LzoKeywordSegment", // [957]
    "Mostly8KeywordSegment", // [958]
    "Mostly16KeywordSegment", // [959]
    "Mostly32KeywordSegment", // [960]
    "RunlengthKeywordSegment", // [961]
    "Text255KeywordSegment", // [962]
    "Text32kKeywordSegment", // [963]
    "ZstdKeywordSegment", // [964]
    "DotSegment", // [965]
    "SingleIdentifierFullGrammar", // [966]
    "CastOperatorSegment", // [967]
    "StartSquareBracketSegment", // [968]
    "StartBracketSegment", // [969]
    "ColonSegment", // [970]
    "DelimiterGrammar", // [971]
    "JoinLikeClauseGrammar", // [972]
    "COLUMN_NAME", // [973]
    "ColumnsExpressionNameGrammar", // [974]
    "COLUMNS", // [975]
    "comma", // [976]
    "COMMAND_FUNCTION", // [977]
    "COMMAND_FUNCTION_CODE", // [978]
    "COMMENT", // [979]
    "CommentKeywordSegment", // [980]
    "IsKeywordSegment", // [981]
    "comment", // [982]
    "CommentSegment", // [983]
    "COMMENTS", // [984]
    "COMMIT", // [985]
    "COMMITTED", // [986]
    "GreaterThanSegment", // [987]
    "LessThanSegment", // [988]
    "GreaterThanOrEqualToSegment", // [989]
    "LessThanOrEqualToSegment", // [990]
    "NotEqualToSegment", // [991]
    "LikeOperatorSegment", // [992]
    "DistinctKeywordSegment", // [993]
    "DISTINCT", // [994]
    "OverlapSegment", // [995]
    "NotExtendRightSegment", // [996]
    "NotExtendLeftSegment", // [997]
    "AdjacentSegment", // [998]
    "PostgisOperatorSegment", // [999]
    "PgvectorOperatorSegment", // [1000]
    "PgTrgmOperatorSegment", // [1001]
    "QualifiedOperatorSegment", // [1002]
    "qualified_operator", // [1003]
    "ComparisonOperatorSegment", // [1004]
    "CompositeBinaryOperatorSegment", // [1005]
    "CompositeComparisonOperatorSegment", // [1006]
    "GZIP", // [1007]
    "LZOP", // [1008]
    "Bzip2KeywordSegment", // [1009]
    "GzipKeywordSegment", // [1010]
    "LzopKeywordSegment", // [1011]
    "COMPUPDATE", // [1012]
    "CONDITION", // [1013]
    "CONDITION_NUMBER", // [1014]
    "JoinTypeKeywordsGrammar", // [1015]
    "INNER", // [1016]
    "LEFT", // [1017]
    "RIGHT", // [1018]
    "ConditionalCrossJoinKeywordsGrammar", // [1019]
    "NonStandardJoinTypeKeywordsGrammar", // [1020]
    "CONDITIONAL", // [1021]
    "DO", // [1022]
    "DoKeywordSegment", // [1023]
    "WHERE", // [1024]
    "WhereKeywordSegment", // [1025]
    "CONFLICT", // [1026]
    "OperationClassReferenceSegment", // [1027]
    "operation_class_reference", // [1028]
    "UNLIMITED", // [1029]
    "UnlimitedKeywordSegment", // [1030]
    "CONNECTION_LIMIT", // [1031]
    "CONNECTION_NAME", // [1032]
    "CONSTRAINT_CATALOG", // [1033]
    "CONSTRAINT_NAME", // [1034]
    "CONSTRAINT_SCHEMA", // [1035]
    "CONSTRAINTS", // [1036]
    "CONSTRUCTOR", // [1037]
    "CONTAINS", // [1038]
    "CONTENT", // [1039]
    "CONTINUE", // [1040]
    "CONTROL", // [1041]
    "CONVERT", // [1042]
    "ConvertKeywordSegment", // [1043]
    "COPY", // [1044]
    "CopyKeywordSegment", // [1045]
    "CSV", // [1046]
    "DATEFORMAT", // [1047]
    "DELIMITER", // [1048]
    "EMPTYASNULL", // [1049]
    "ENCODING", // [1050]
    "ESCAPE", // [1051]
    "EXPLICIT_IDS", // [1052]
    "FILLRECORD", // [1053]
    "FIXEDWIDTH", // [1054]
    "IGNOREBLANKLINES", // [1055]
    "IGNOREHEADER", // [1056]
    "JSON", // [1057]
    "MANIFEST", // [1058]
    "MAXERROR", // [1059]
    "NOLOAD", // [1060]
    "ORC", // [1061]
    "PARQUET", // [1062]
    "RCFILE", // [1063]
    "READRATIO", // [1064]
    "REGION", // [1065]
    "REMOVEQUOTES", // [1066]
    "ROUNDEC", // [1067]
    "SEQUENCEFILE", // [1068]
    "SHAPEFILE", // [1069]
    "STATUPDATE", // [1070]
    "TIMEFORMAT", // [1071]
    "TRIMBLANKS", // [1072]
    "TRUNCATECOLUMNS", // [1073]
    "AuthorizationSegment", // [1074]
    "authorization_segment", // [1075]
    "RegionKeywordSegment", // [1076]
    "CompressionTypeGrammar", // [1077]
    "DataFormatSegment", // [1078]
    "data_format_segment", // [1079]
    "DelimiterKeywordSegment", // [1080]
    "FixedwidthKeywordSegment", // [1081]
    "ManifestKeywordSegment", // [1082]
    "MaxerrorKeywordSegment", // [1083]
    "CompupdateKeywordSegment", // [1084]
    "OFF", // [1085]
    "PRESET", // [1086]
    "PresetKeywordSegment", // [1087]
    "OffKeywordSegment", // [1088]
    "TrueKeywordSegment", // [1089]
    "FalseKeywordSegment", // [1090]
    "StatupdateKeywordSegment", // [1091]
    "NoloadKeywordSegment", // [1092]
    "AcceptanydateKeywordSegment", // [1093]
    "AcceptinvcharsKeywordSegment", // [1094]
    "BlanksasnullKeywordSegment", // [1095]
    "DateformatKeywordSegment", // [1096]
    "EmptyasnullKeywordSegment", // [1097]
    "EncodingKeywordSegment", // [1098]
    "UTF16", // [1099]
    "UTF16BE", // [1100]
    "UTF16LE", // [1101]
    "UTF8", // [1102]
    "Utf8KeywordSegment", // [1103]
    "Utf16KeywordSegment", // [1104]
    "Utf16beKeywordSegment", // [1105]
    "Utf16leKeywordSegment", // [1106]
    "EscapeKeywordSegment", // [1107]
    "Explicit_idsKeywordSegment", // [1108]
    "FillrecordKeywordSegment", // [1109]
    "IgnoreblanklinesKeywordSegment", // [1110]
    "IgnoreheaderKeywordSegment", // [1111]
    "ReadratioKeywordSegment", // [1112]
    "RemovequotesKeywordSegment", // [1113]
    "RoundecKeywordSegment", // [1114]
    "TimeformatKeywordSegment", // [1115]
    "EPOCHMILLISECS", // [1116]
    "EPOCHSECS", // [1117]
    "EpochsecsKeywordSegment", // [1118]
    "EpochmillisecsKeywordSegment", // [1119]
    "TrimblanksKeywordSegment", // [1120]
    "TruncatecolumnsKeywordSegment", // [1121]
    "CORR", // [1122]
    "CORRESPONDING", // [1123]
    "COS", // [1124]
    "COSH", // [1125]
    "COSTS", // [1126]
    "COUNT", // [1127]
    "COVAR_POP", // [1128]
    "COVAR_SAMP", // [1129]
    "OrReplaceGrammar", // [1130]
    "AssignmentKeywordSegment", // [1131]
    "ImplicitKeywordSegment", // [1132]
    "IMPLICIT", // [1133]
    "LocaleKeywordSegment", // [1134]
    "Lc_collateKeywordSegment", // [1135]
    "Lc_ctypeKeywordSegment", // [1136]
    "ProviderKeywordSegment", // [1137]
    "IcuKeywordSegment", // [1138]
    "LibcKeywordSegment", // [1139]
    "DETERMINISTIC", // [1140]
    "DeterministicKeywordSegment", // [1141]
    "VERSION", // [1142]
    "VersionKeywordSegment", // [1143]
    "FromIntegrationClauseSegment", // [1144]
    "from_integration_clause", // [1145]
    "CreateDatabaseWithOptionsSegment", // [1146]
    "ISOLATION", // [1147]
    "create_database_with_options", // [1148]
    "ArnCatalogSchemaSegment", // [1149]
    "arn_catalog_schema_segment", // [1150]
    "IamRoleClauseSegment", // [1151]
    "iam_role_clause", // [1152]
    "FromDatashareClauseSegment", // [1153]
    "from_datashare_clause", // [1154]
    "CollationClauseSegment", // [1155]
    "collation_clause", // [1156]
    "IsolationLevelClauseSegment", // [1157]
    "isolation_level_clause", // [1158]
    "VersionIdentifierSegment", // [1159]
    "version_identifier", // [1160]
    "LAMBDA", // [1161]
    "SAGEMAKER", // [1162]
    "LambdaKeywordSegment", // [1163]
    "SagemakerKeywordSegment", // [1164]
    "RETRY_TIMEOUT", // [1165]
    "Retry_timeoutKeywordSegment", // [1166]
    "HIVE", // [1167]
    "KAFKA", // [1168]
    "KINESIS", // [1169]
    "MSK", // [1170]
    "MYSQL", // [1171]
    "POSTGRES", // [1172]
    "REDSHIFT", // [1173]
    "HiveKeywordSegment", // [1174]
    "MetastoreKeywordSegment", // [1175]
    "METASTORE", // [1176]
    "PostgresKeywordSegment", // [1177]
    "MysqlKeywordSegment", // [1178]
    "KinesisKeywordSegment", // [1179]
    "MskKeywordSegment", // [1180]
    "RedshiftKeywordSegment", // [1181]
    "KafkaKeywordSegment", // [1182]
    "SECRET_ARN", // [1183]
    "URI", // [1184]
    "UriKeywordSegment", // [1185]
    "PORT", // [1186]
    "PortKeywordSegment", // [1187]
    "AuthenticationKeywordSegment", // [1188]
    "IAM", // [1189]
    "MTLS", // [1190]
    "IamKeywordSegment", // [1191]
    "MtlsKeywordSegment", // [1192]
    "Authentication_arnKeywordSegment", // [1193]
    "Secret_arnKeywordSegment", // [1194]
    "Catalog_roleKeywordSegment", // [1195]
    "IfKeywordSegment", // [1196]
    "ExistsKeywordSegment", // [1197]
    "EXISTS", // [1198]
    "PartitionedBySegment", // [1199]
    "PARTITIONED", // [1200]
    "partitioned_by_segment", // [1201]
    "DelimitedKeywordSegment", // [1202]
    "DELIMITED", // [1203]
    "RowFormatDelimitedSegment", // [1204]
    "FIELDS", // [1205]
    "LINES", // [1206]
    "row_format_delimited_segment", // [1207]
    "StoredKeywordSegment", // [1208]
    "STORED", // [1209]
    "TEXTFILE", // [1210]
    "ParquetKeywordSegment", // [1211]
    "TextfileKeywordSegment", // [1212]
    "PropertiesKeywordSegment", // [1213]
    "PROPERTIES", // [1214]
    "SERDE", // [1215]
    "SerdeKeywordSegment", // [1216]
    "SerdepropertiesKeywordSegment", // [1217]
    "SERDEPROPERTIES", // [1218]
    "INPUTFORMAT", // [1219]
    "RcfileKeywordSegment", // [1220]
    "SequencefileKeywordSegment", // [1221]
    "OrcKeywordSegment", // [1222]
    "AvroKeywordSegment", // [1223]
    "InputformatKeywordSegment", // [1224]
    "OutputformatKeywordSegment", // [1225]
    "OUTPUTFORMAT", // [1226]
    "ForeignDataWrapperGrammar", // [1227]
    "HandlerKeywordSegment", // [1228]
    "HANDLER", // [1229]
    "ValidatorKeywordSegment", // [1230]
    "VALIDATOR", // [1231]
    "CreateForeignTableGrammar", // [1232]
    "OptionsGrammar", // [1233]
    "ForeignTableColumnConstraintSegment", // [1234]
    "ForeignTableTableConstraintSegment", // [1235]
    "INHERITS", // [1236]
    "InheritsKeywordSegment", // [1237]
    "ServerReferenceSegment", // [1238]
    "server_reference", // [1239]
    "TemporaryGrammar", // [1240]
    "ColumnTypeReferenceSegment", // [1241]
    "column_type_reference", // [1242]
    "SETOF", // [1243]
    "SetofKeywordSegment", // [1244]
    "FunctionDefinitionGrammar", // [1245]
    "RETURN", // [1246]
    "WINDOW", // [1247]
    "function_definition", // [1248]
    "IndexElementSegment", // [1249]
    "index_element", // [1250]
    "INCLUDE", // [1251]
    "IncludeKeywordSegment", // [1252]
    "NULLS", // [1253]
    "NullsKeywordSegment", // [1254]
    "LibraryKeywordSegment", // [1255]
    "LIBRARY", // [1256]
    "PlpythonuKeywordSegment", // [1257]
    "PLPYTHONU", // [1258]
    "BackupKeywordSegment", // [1259]
    "YES", // [1260]
    "YesKeywordSegment", // [1261]
    "TableAttributeSegment", // [1262]
    "INTERLEAVED", // [1263]
    "ValuesClauseSegment", // [1264]
    "values_clause", // [1265]
    "WithDataClauseSegment", // [1266]
    "with_data_clause", // [1267]
    "ModelKeywordSegment", // [1268]
    "MODEL", // [1269]
    "TARGET", // [1270]
    "TargetKeywordSegment", // [1271]
    "MODEL_TYPE", // [1272]
    "Model_typeKeywordSegment", // [1273]
    "KMEANS", // [1274]
    "MLP", // [1275]
    "XGBOOST", // [1276]
    "XgboostKeywordSegment", // [1277]
    "MlpKeywordSegment", // [1278]
    "KmeansKeywordSegment", // [1279]
    "PROBLEM_TYPE", // [1280]
    "Problem_typeKeywordSegment", // [1281]
    "MULTICLASS_CLASSIFICATION", // [1282]
    "REGRESSION", // [1283]
    "RegressionKeywordSegment", // [1284]
    "Binary_classificationKeywordSegment", // [1285]
    "Multiclass_classificationKeywordSegment", // [1286]
    "OBJECTIVE", // [1287]
    "ObjectiveKeywordSegment", // [1288]
    "PREPROCESSORS", // [1289]
    "PreprocessorsKeywordSegment", // [1290]
    "HYPERPARAMETERS", // [1291]
    "HyperparametersKeywordSegment", // [1292]
    "EXCEPT", // [1293]
    "ExceptKeywordSegment", // [1294]
    "SETTINGS", // [1295]
    "SettingsKeywordSegment", // [1296]
    "S3_BUCKET", // [1297]
    "S3_bucketKeywordSegment", // [1298]
    "S3_GARBAGE_COLLECT", // [1299]
    "S3_garbage_collectKeywordSegment", // [1300]
    "MAX_CELLS", // [1301]
    "Max_cellsKeywordSegment", // [1302]
    "MAX_RUNTIME", // [1303]
    "Max_runtimeKeywordSegment", // [1304]
    "commutator", // [1305]
    "LeftargKeywordSegment", // [1306]
    "RightargKeywordSegment", // [1307]
    "CommutatorKeywordSegment", // [1308]
    "NegatorKeywordSegment", // [1309]
    "negator", // [1310]
    "JOIN", // [1311]
    "JoinKeywordSegment", // [1312]
    "HashesKeywordSegment", // [1313]
    "MergesKeywordSegment", // [1314]
    "PermissiveKeywordSegment", // [1315]
    "RestrictiveKeywordSegment", // [1316]
    "NonatomicKeywordSegment", // [1317]
    "NONATOMIC", // [1318]
    "RlsKeywordSegment", // [1319]
    "RLS", // [1320]
    "SYSID", // [1321]
    "AdminKeywordSegment", // [1322]
    "SysidKeywordSegment", // [1323]
    "AuthorizationKeywordSegment", // [1324]
    "CreateSequenceOptionsSegment", // [1325]
    "create_sequence_options_segment", // [1326]
    "DependenciesKeywordSegment", // [1327]
    "McvKeywordSegment", // [1328]
    "NdistinctKeywordSegment", // [1329]
    "CorrelationKeywordSegment", // [1330]
    "LOCAL", // [1331]
    "LocalKeywordSegment", // [1332]
    "ColumnAttributeSegment", // [1333]
    "column_attribute_segment", // [1334]
    "LIKE", // [1335]
    "LikeKeywordSegment", // [1336]
    "EXCLUDING", // [1337]
    "INCLUDING", // [1338]
    "LikeOptionSegment", // [1339]
    "like_option_segment", // [1340]
    "INSTEAD", // [1341]
    "InsteadKeywordSegment", // [1342]
    "OrKeywordSegment", // [1343]
    "REFERENCING", // [1344]
    "ReferencingKeywordSegment", // [1345]
    "NEW", // [1346]
    "OLD", // [1347]
    "OldKeywordSegment", // [1348]
    "NewKeywordSegment", // [1349]
    "EachKeywordSegment", // [1350]
    "EACH", // [1351]
    "STATEMENT", // [1352]
    "StatementKeywordSegment", // [1353]
    "WhenKeywordSegment", // [1354]
    "ENUM", // [1355]
    "RANGE", // [1356]
    "EnumKeywordSegment", // [1357]
    "RangeKeywordSegment", // [1358]
    "CreateUserMappingGrammar", // [1359]
    "SessionInformationUserFunctionsGrammar", // [1360]
    "WithNoSchemaBindingClauseSegment", // [1361]
    "with_no_schema_binding_clause", // [1362]
    "CROSS", // [1363]
    "CUBE", // [1364]
    "function_name_identifier", // [1365]
    "ROLLUP", // [1366]
    "CubeFunctionNameSegment", // [1367]
    "RollupFunctionNameSegment", // [1368]
    "GroupingExpressionList", // [1369]
    "grouping_expression_list", // [1370]
    "CUME_DIST", // [1371]
    "CURRENT_CATALOG", // [1372]
    "CURRENT_DEFAULT_TRANSFORM_GROUP", // [1373]
    "CURRENT_PATH", // [1374]
    "CURRENT_ROW", // [1375]
    "CURRENT_TRANSFORM_GROUP_FOR_TYPE", // [1376]
    "CURSOR", // [1377]
    "CURSOR_NAME", // [1378]
    "CsvKeywordSegment", // [1379]
    "QUOTE", // [1380]
    "QuoteKeywordSegment", // [1381]
    "ShapefileKeywordSegment", // [1382]
    "SIMPLIFY", // [1383]
    "SimplifyKeywordSegment", // [1384]
    "JsonKeywordSegment", // [1385]
    "DATA_CATALOG_SCHEMA", // [1386]
    "DATALINK", // [1387]
    "DATASHARES", // [1388]
    "data_type_identifier", // [1389]
    "SmallintKeywordSegment", // [1390]
    "SMALLINT", // [1391]
    "Int2KeywordSegment", // [1392]
    "INT2", // [1393]
    "IntegerKeywordSegment", // [1394]
    "INTEGER", // [1395]
    "IntKeywordSegment", // [1396]
    "INT", // [1397]
    "Int4KeywordSegment", // [1398]
    "INT4", // [1399]
    "BigintKeywordSegment", // [1400]
    "Int8KeywordSegment", // [1401]
    "INT8", // [1402]
    "RealKeywordSegment", // [1403]
    "REAL", // [1404]
    "Float4KeywordSegment", // [1405]
    "FLOAT4", // [1406]
    "DOUBLE", // [1407]
    "DoubleKeywordSegment", // [1408]
    "PrecisionKeywordSegment", // [1409]
    "PRECISION", // [1410]
    "Float8KeywordSegment", // [1411]
    "FLOAT8", // [1412]
    "FloatKeywordSegment", // [1413]
    "FLOAT", // [1414]
    "DECIMAL", // [1415]
    "NUMERIC", // [1416]
    "DecimalKeywordSegment", // [1417]
    "NumericKeywordSegment", // [1418]
    "BracketedArguments", // [1419]
    "bracketed_arguments", // [1420]
    "NCHAR", // [1421]
    "NVARCHAR", // [1422]
    "VARCHAR", // [1423]
    "CharKeywordSegment", // [1424]
    "CharacterKeywordSegment", // [1425]
    "NcharKeywordSegment", // [1426]
    "VarcharKeywordSegment", // [1427]
    "VaryingKeywordSegment", // [1428]
    "VARYING", // [1429]
    "NvarcharKeywordSegment", // [1430]
    "BpcharKeywordSegment", // [1431]
    "DateTimeTypeIdentifier", // [1432]
    "datetime_type_identifier", // [1433]
    "IntervalKeywordSegment", // [1434]
    "BooleanKeywordSegment", // [1435]
    "BoolKeywordSegment", // [1436]
    "HllsketchKeywordSegment", // [1437]
    "HLLSKETCH", // [1438]
    "SuperKeywordSegment", // [1439]
    "SUPER", // [1440]
    "GeometryKeywordSegment", // [1441]
    "GEOMETRY", // [1442]
    "GeographyKeywordSegment", // [1443]
    "GEOGRAPHY", // [1444]
    "VARBINARY", // [1445]
    "VARBYTE", // [1446]
    "VarbyteKeywordSegment", // [1447]
    "VarbinaryKeywordSegment", // [1448]
    "BinaryKeywordSegment", // [1449]
    "AnyelementKeywordSegment", // [1450]
    "DatatypeIdentifierSegment", // [1451]
    "DATEADD", // [1452]
    "DATEDIFF", // [1453]
    "DATE_PART", // [1454]
    "EXTRACT", // [1455]
    "DatePartFunctionName", // [1456]
    "DatetimeUnitSegment", // [1457]
    "CENT", // [1458]
    "CENTS", // [1459]
    "CENTURIES", // [1460]
    "CENTURY", // [1461]
    "D", // [1462]
    "DAY", // [1463]
    "DAYOFWEEK", // [1464]
    "DAYOFYEAR", // [1465]
    "DAYS", // [1466]
    "DEC", // [1467]
    "DECADE", // [1468]
    "DECADES", // [1469]
    "DECS", // [1470]
    "DOW", // [1471]
    "DOY", // [1472]
    "DW", // [1473]
    "DY", // [1474]
    "EPOCH", // [1475]
    "H", // [1476]
    "HOUR", // [1477]
    "HOURS", // [1478]
    "HR", // [1479]
    "HRS", // [1480]
    "ISODOW", // [1481]
    "ISOYEAR", // [1482]
    "M", // [1483]
    "MICROSEC", // [1484]
    "MICROSECOND", // [1485]
    "MICROSECONDS", // [1486]
    "MICROSECS", // [1487]
    "MIL", // [1488]
    "MILLENNIA", // [1489]
    "MILLENNIUM", // [1490]
    "MILLISEC", // [1491]
    "MILLISECON", // [1492]
    "MILLISECOND", // [1493]
    "MILLISECONDS", // [1494]
    "MILLISECS", // [1495]
    "MILS", // [1496]
    "MIN", // [1497]
    "MINS", // [1498]
    "MINUTE", // [1499]
    "MINUTES", // [1500]
    "MON", // [1501]
    "MONS", // [1502]
    "MONTH", // [1503]
    "MONTHS", // [1504]
    "MS", // [1505]
    "MSEC", // [1506]
    "MSECOND", // [1507]
    "MSECONDS", // [1508]
    "MSECS", // [1509]
    "QTR", // [1510]
    "QTRS", // [1511]
    "QUARTER", // [1512]
    "QUARTERS", // [1513]
    "S", // [1514]
    "SEC", // [1515]
    "SECOND", // [1516]
    "SECONDS", // [1517]
    "SECS", // [1518]
    "TIMEZONE", // [1519]
    "TIMEZONE_HOUR", // [1520]
    "TIMEZONE_MINUTE", // [1521]
    "US", // [1522]
    "USEC", // [1523]
    "USECOND", // [1524]
    "USECONDS", // [1525]
    "USECS", // [1526]
    "W", // [1527]
    "WEEK", // [1528]
    "WEEKDAY", // [1529]
    "WEEKS", // [1530]
    "Y", // [1531]
    "YEAR", // [1532]
    "YEARDAY", // [1533]
    "YEARS", // [1534]
    "YR", // [1535]
    "YRS", // [1536]
    "FunctionContentsGrammar", // [1537]
    "DateKeywordSegment", // [1538]
    "DatetimeKeywordSegment", // [1539]
    "TimeWithTZGrammar", // [1540]
    "TimetzKeywordSegment", // [1541]
    "TimestamptzKeywordSegment", // [1542]
    "date_part", // [1543]
    "DATETIME_INTERVAL_CODE", // [1544]
    "DATETIME_INTERVAL_PRECISION", // [1545]
    "DB", // [1546]
    "DBUSER", // [1547]
    "DEALLOCATE", // [1548]
    "DeallocateKeywordSegment", // [1549]
    "PrepareKeywordSegment", // [1550]
    "PREPARE", // [1551]
    "DECFLOAT", // [1552]
    "DECLARE", // [1553]
    "DeclareKeywordSegment", // [1554]
    "CursorKeywordSegment", // [1555]
    "DEFAULTS", // [1556]
    "DEFINE", // [1557]
    "DEFINED", // [1558]
    "PropertiesNakedIdentifierSegment", // [1559]
    "DefinitionArgumentValueGrammar", // [1560]
    "DefinitionParameterSegment", // [1561]
    "definition_parameter", // [1562]
    "DEFLATE", // [1563]
    "DEFRAG", // [1564]
    "DEGREE", // [1565]
    "TableExpressionSegment", // [1566]
    "table_expression", // [1567]
    "JoinClauseSegment", // [1568]
    "join_clause", // [1569]
    "WhereClauseSegment", // [1570]
    "where_clause", // [1571]
    "RETURNING", // [1572]
    "ReturningKeywordSegment", // [1573]
    "SemicolonSegment", // [1574]
    "DELIMITERS", // [1575]
    "DENSE_RANK", // [1576]
    "DEREF", // [1577]
    "DERIVED", // [1578]
    "DESC", // [1579]
    "DescKeywordSegment", // [1580]
    "NamespaceKeywordSegment", // [1581]
    "NAMESPACE", // [1582]
    "DESCRIBE", // [1583]
    "DescribeKeywordSegment", // [1584]
    "DESCRIPTOR", // [1585]
    "DIAGNOSTICS", // [1586]
    "DISCARD", // [1587]
    "DiscardKeywordSegment", // [1588]
    "PLANS", // [1589]
    "PlansKeywordSegment", // [1590]
    "DISCONNECT", // [1591]
    "DISPATCH", // [1592]
    "DLNEWCOPY", // [1593]
    "DLPREVIOUSCOPY", // [1594]
    "DLURLCOMPLETE", // [1595]
    "DLURLCOMPLETEONLY", // [1596]
    "DLURLCOMPLETEWRITE", // [1597]
    "DLURLPATH", // [1598]
    "DLURLPATHONLY", // [1599]
    "DLURLPATHWRITE", // [1600]
    "DLURLSCHEME", // [1601]
    "DLURLSERVER", // [1602]
    "DLVALUE", // [1603]
    "LanguageClauseSegment", // [1604]
    "language_clause", // [1605]
    "DOCUMENT", // [1606]
    "literal", // [1607]
    "LiteralSegment", // [1608]
    "dot", // [1609]
    "DUPLICATES", // [1610]
    "DYNAMIC", // [1611]
    "DYNAMIC_FUNCTION", // [1612]
    "DYNAMIC_FUNCTION_CODE", // [1613]
    "ELEMENT", // [1614]
    "EMPTY", // [1615]
    "StructTypeSegment", // [1616]
    "struct_type", // [1617]
    "EmptyStructLiteralBracketsSegment", // [1618]
    "struct_literal", // [1619]
    "ENCRYPT", // [1620]
    "ENCRYPTION", // [1621]
    "END-EXEC", // [1622]
    "}", // [1623]
    "end_curly_bracket", // [1624]
    "END_FRAME", // [1625]
    "END_PARTITION", // [1626]
    "ENFORCED", // [1627]
    "EQUALS", // [1628]
    "ERROR", // [1629]
    "EVERY", // [1630]
    "EXCEPTION", // [1631]
    "EXCLUDE", // [1632]
    "ExcludeKeywordSegment", // [1633]
    "EXCLUSIVE", // [1634]
    "EXEC", // [1635]
    "EXP", // [1636]
    "EXPLAIN", // [1637]
    "CostsKeywordSegment", // [1638]
    "BuffersKeywordSegment", // [1639]
    "WalKeywordSegment", // [1640]
    "TimingKeywordSegment", // [1641]
    "SummaryKeywordSegment", // [1642]
    "XML", // [1643]
    "YAML", // [1644]
    "XmlKeywordSegment", // [1645]
    "YamlKeywordSegment", // [1646]
    "ExplainKeywordSegment", // [1647]
    "ExplainOptionSegment", // [1648]
    "explain_option", // [1649]
    "InsertStatementSegment", // [1650]
    "insert_statement", // [1651]
    "UpdateStatementSegment", // [1652]
    "update_statement", // [1653]
    "DeleteStatementSegment", // [1654]
    "delete_statement", // [1655]
    "EXPLICIT", // [1656]
    "Expression_A_Grammar", // [1657]
    "Tail_Recurse_Expression_A_Grammar", // [1658]
    "LikeExpressionGrammar", // [1659]
    "ILIKE", // [1660]
    "SIMILAR", // [1661]
    "InOperatorGrammar", // [1662]
    "QuantifiedComparisonOperatorGrammar", // [1663]
    "IsClauseGrammar", // [1664]
    "NAN", // [1665]
    "NFC", // [1666]
    "NFD", // [1667]
    "NFKC", // [1668]
    "NFKD", // [1669]
    "NORMALIZED", // [1670]
    "UNKNOWN", // [1671]
    "IsNullGrammar", // [1672]
    "ISNULL", // [1673]
    "NotNullGrammar", // [1674]
    "NOTNULL", // [1675]
    "CollateGrammar", // [1676]
    "BetweenKeywordSegment", // [1677]
    "Expression_B_Grammar", // [1678]
    "AndKeywordSegment", // [1679]
    "PatternMatchingGrammar", // [1680]
    "PRIOR", // [1681]
    "~", // [1682]
    "QualifiedNumericLiteralSegment", // [1683]
    "TildeSegment", // [1684]
    "NotOperatorGrammar", // [1685]
    "PriorKeywordSegment", // [1686]
    "Tail_Recurse_Expression_B_Grammar", // [1687]
    "WalrusOperatorSegment", // [1688]
    ":=", // [1689]
    "Expression_D_Grammar", // [1690]
    "CaseExpressionSegment", // [1691]
    "case_expression", // [1692]
    "TimeZoneGrammar", // [1693]
    "time_zone_grammar", // [1694]
    "FullTextSearchOperatorSegment", // [1695]
    "full_text_search_operator", // [1696]
    "LateralColumnAliasExpressionGrammar", // [1697]
    "LocalAliasSegment", // [1698]
    "local_alias_segment", // [1699]
    "Expression_D_Potential_Select_Statement_Without_Brackets", // [1700]
    "SingleIdentifierWildcardSegment", // [1701]
    "MapTypeSegment", // [1702]
    "map_type", // [1703]
    "NullLiteralSegment", // [1704]
    "DateTimeLiteralGrammar", // [1705]
    "datetime_literal", // [1706]
    "ListComprehensionGrammar", // [1707]
    "AccessorGrammar", // [1708]
    "SelectStatementSegment", // [1709]
    "select_statement", // [1710]
    "TypedStructLiteralSegment", // [1711]
    "typed_struct_literal", // [1712]
    "ArrayExpressionSegment", // [1713]
    "array_expression", // [1714]
    "OverlapsClauseSegment", // [1715]
    "overlaps_clause", // [1716]
    "boolean_literal", // [1717]
    "LiteralKeywordSegment", // [1718]
    "FETCH", // [1719]
    "FetchKeywordSegment", // [1720]
    "NEXT", // [1721]
    "NextKeywordSegment", // [1722]
    "TiesKeywordSegment", // [1723]
    "TIES", // [1724]
    "FORWARD", // [1725]
    "ForwardKeywordSegment", // [1726]
    "IMPORT", // [1727]
    "LISTEN", // [1728]
    "LOAD", // [1729]
    "LOCK", // [1730]
    "MERGE", // [1731]
    "NOTIFY", // [1732]
    "REASSIGN", // [1733]
    "REINDEX", // [1734]
    "ROLLBACK", // [1735]
    "SHOW", // [1736]
    "UNLISTEN", // [1737]
    "UNLOAD", // [1738]
    "VACUUM", // [1739]
    "postgres_copy_stdin_data_block", // [1740]
    "psql_copy_command", // [1741]
    "psql_set_command", // [1742]
    "PostgresCopyStdinDataStatementSegment", // [1743]
    "postgres_copy_stdin_data_statement", // [1744]
    "PsqlCopyMetaCommandStatementSegment", // [1745]
    "psql_copy_meta_command_statement", // [1746]
    "PsqlSetMetaCommandStatementSegment", // [1747]
    "psql_set_meta_command_statement", // [1748]
    "StatementSegment", // [1749]
    "statement", // [1750]
    "FILTER", // [1751]
    "FilterKeywordSegment", // [1752]
    "FINAL", // [1753]
    "FINISH", // [1754]
    "FIRST_VALUE", // [1755]
    "FLAG", // [1756]
    "FLOOR", // [1757]
    "FOLLOWING", // [1758]
    "LockedKeywordSegment", // [1759]
    "LOCKED", // [1760]
    "FORTRAN", // [1761]
    "FOUND", // [1762]
    "GROUPS", // [1763]
    "FrameClauseUnitGrammar", // [1764]
    "UnboundedKeywordSegment", // [1765]
    "UNBOUNDED", // [1766]
    "PRECEDING", // [1767]
    "PrecedingKeywordSegment", // [1768]
    "FollowingKeywordSegment", // [1769]
    "OthersKeywordSegment", // [1770]
    "OTHERS", // [1771]
    "GroupsKeywordSegment", // [1772]
    "FRAME_ROW", // [1773]
    "FREE", // [1774]
    "FREEZE", // [1775]
    "FromExpressionSegment", // [1776]
    "from_expression", // [1777]
    "HAVING", // [1778]
    "INTERSECT", // [1779]
    "MINUS", // [1780]
    "OFFSET", // [1781]
    "QUALIFY", // [1782]
    "UNION", // [1783]
    "LimitClauseSegment", // [1784]
    "limit_clause", // [1785]
    "OrderKeywordSegment", // [1786]
    "HavingKeywordSegment", // [1787]
    "QualifyKeywordSegment", // [1788]
    "WindowKeywordSegment", // [1789]
    "SetOperatorSegment", // [1790]
    "set_operator", // [1791]
    "OffsetKeywordSegment", // [1792]
    "ForClauseSegment", // [1793]
    "for_clause", // [1794]
    "PermissionsKeywordSegment", // [1795]
    "PERMISSIONS", // [1796]
    "PreTableFunctionKeywordsGrammar", // [1797]
    "LATERAL", // [1798]
    "TemporalQuerySegment", // [1799]
    "temporal_query", // [1800]
    "FromClauseTerminatorGrammar", // [1801]
    "SamplingExpressionSegment", // [1802]
    "TABLESAMPLE", // [1803]
    "sample_expression", // [1804]
    "PostTableExpressionGrammar", // [1805]
    "MLTableExpressionSegment", // [1806]
    "ML", // [1807]
    "ml_table_expression", // [1808]
    "FromExpressionElementSegment", // [1809]
    "from_expression_element", // [1810]
    "indented_joins", // [1811]
    "HISTORY_MODE", // [1812]
    "QUERY_ALL_STATES", // [1813]
    "REFRESH_INTERVAL", // [1814]
    "Query_all_statesKeywordSegment", // [1815]
    "Refresh_intervalKeywordSegment", // [1816]
    "History_modeKeywordSegment", // [1817]
    "PivotKeywordSegment", // [1818]
    "UnpivotKeywordSegment", // [1819]
    "FS", // [1820]
    "FULFILL", // [1821]
    "NamedArgumentSegment", // [1822]
    "named_argument", // [1823]
    "VariadicKeywordSegment", // [1824]
    "LEADING", // [1825]
    "TRAILING", // [1826]
    "TrimParametersGrammar", // [1827]
    "FunctionContentsExpressionGrammar", // [1828]
    "AggregateOrderByClause", // [1829]
    "aggregate_order_by", // [1830]
    "SEPARATOR", // [1831]
    "SeparatorKeywordSegment", // [1832]
    "IgnoreRespectNullsGrammar", // [1833]
    "IGNORE", // [1834]
    "RESPECT", // [1835]
    "IndexColumnDefinitionSegment", // [1836]
    "index_column_definition", // [1837]
    "EmptyStructLiteralSegment", // [1838]
    "ReturnKeywordSegment", // [1839]
    "BeginKeywordSegment", // [1840]
    "AtomicKeywordSegment", // [1841]
    "WithCompoundStatementSegment", // [1842]
    "with_compound_statement", // [1843]
    "FunctionNameIdentifierSegment", // [1844]
    "FunctionParameterGrammar", // [1845]
    "DatePartFunctionNameSegment", // [1846]
    "DateTimeFunctionContentsSegment", // [1847]
    "function_contents", // [1848]
    "ConvertFunctionNameSegment", // [1849]
    "ApproximateKeywordSegment", // [1850]
    "FunctionContentsSegment", // [1851]
    "PostFunctionGrammar", // [1852]
    "ConvertFunctionContentsSegment", // [1853]
    "FUSION", // [1854]
    "G", // [1855]
    "GB", // [1856]
    "GENERAL", // [1857]
    "GET", // [1858]
    "glob_operator", // [1859]
    "GLOBAL", // [1860]
    "GLOBALDICT256", // [1861]
    "GLOBALDICT64K", // [1862]
    "GO", // [1863]
    "GOTO", // [1864]
    "AccessPermissionsSegment", // [1865]
    "access_permissions", // [1866]
    "AccessObjectSegment", // [1867]
    "access_object", // [1868]
    "RedshiftGroupGrantTargetSegment", // [1869]
    "access_target", // [1870]
    "RedshiftGrantTargetSegment", // [1871]
    "GRANTED", // [1872]
    "GrantedKeywordSegment", // [1873]
    "VIA", // [1874]
    "ViaKeywordSegment", // [1875]
    "GREATEST", // [1876]
    "CubeRollupClauseSegment", // [1877]
    "cube_rollup_clause", // [1878]
    "GroupingSetsClauseSegment", // [1879]
    "GROUPING", // [1880]
    "grouping_sets_clause", // [1881]
    "GroupByClauseTerminatorGrammar", // [1882]
    "GroupingKeywordSegment", // [1883]
    "SetsKeywordSegment", // [1884]
    "SETS", // [1885]
    "HASH", // [1886]
    "HEADER", // [1887]
    "HEX", // [1888]
    "HIERARCHY", // [1889]
    "HOLD", // [1890]
    "ID", // [1891]
    "identifier", // [1892]
    "IdentifierSegment", // [1893]
    "IgnoreKeywordSegment", // [1894]
    "RespectKeywordSegment", // [1895]
    "IMMEDIATELY", // [1896]
    "IMPLEMENTATION", // [1897]
    "ImportKeywordSegment", // [1898]
    "ImportForeignSchemaGrammar", // [1899]
    "NakedIdentifierFullSegment", // [1900]
    "IntoKeywordSegment", // [1901]
    "INTO", // [1902]
    "INDENT", // [1903]
    "AscKeywordSegment", // [1904]
    "OperatorClassReferenceSegment", // [1905]
    "LAST", // [1906]
    "LastKeywordSegment", // [1907]
    "operator_class_reference", // [1908]
    "IndexElementOptionsSegment", // [1909]
    "index_element_options", // [1910]
    "INDEXES", // [1911]
    "INDICATOR", // [1912]
    "INITIAL", // [1913]
    "INLINE", // [1914]
    "INSENSITIVE", // [1915]
    "INSTANCE", // [1916]
    "INSTANTIABLE", // [1917]
    "INTEGRITY", // [1918]
    "INTERSECTION", // [1919]
    "YearKeywordSegment", // [1920]
    "MonthKeywordSegment", // [1921]
    "DayKeywordSegment", // [1922]
    "HourKeywordSegment", // [1923]
    "MinuteKeywordSegment", // [1924]
    "SecondKeywordSegment", // [1925]
    "NanLiteralSegment", // [1926]
    "UnknownLiteralSegment", // [1927]
    "NormalizedGrammar", // [1928]
    "JsonTestGrammar", // [1929]
    "IsnullKeywordSegment", // [1930]
    "IsolationKeywordSegment", // [1931]
    "SERIALIZABLE", // [1932]
    "SNAPSHOT", // [1933]
    "SerializableKeywordSegment", // [1934]
    "SnapshotKeywordSegment", // [1935]
    "ConditionalJoinKeywordsGrammar", // [1936]
    "JoinKeywordsGrammar", // [1937]
    "NestedJoinGrammar", // [1938]
    "indented_using_on", // [1939]
    "MatchConditionSegment", // [1940]
    "match_condition", // [1941]
    "JoinOnConditionSegment", // [1942]
    "join_on_condition", // [1943]
    "JoinUsingConditionGrammar", // [1944]
    "UnconditionalJoinKeywordsGrammar", // [1945]
    "ExtendedNaturalJoinKeywordsGrammar", // [1946]
    "FromPivotExpressionSegment", // [1947]
    "from_pivot_expression", // [1948]
    "FromUnpivotExpressionSegment", // [1949]
    "from_unpivot_expression", // [1950]
    "indented_on_contents", // [1951]
    "InnerKeywordSegment", // [1952]
    "LeftKeywordSegment", // [1953]
    "RightKeywordSegment", // [1954]
    "OuterKeywordSegment", // [1955]
    "OUTER", // [1956]
    "JsonTypeGrammar", // [1957]
    "SCALAR", // [1958]
    "JsonUniqueKeysGrammar", // [1959]
    "ScalarKeywordSegment", // [1960]
    "KEYS", // [1961]
    "KeysKeywordSegment", // [1962]
    "JSON_ARRAY", // [1963]
    "JSON_ARRAYAGG", // [1964]
    "JSON_EXISTS", // [1965]
    "JSON_OBJECT", // [1966]
    "JSON_OBJECTAGG", // [1967]
    "JSON_QUERY", // [1968]
    "JSON_TABLE", // [1969]
    "JSON_TABLE_PRIMITIVE", // [1970]
    "JSON_VALUE", // [1971]
    "K", // [1972]
    "KEEP", // [1973]
    "KEY_MEMBER", // [1974]
    "KEY_TYPE", // [1975]
    "LABEL", // [1976]
    "LAG", // [1977]
    "SingleQuotedIdentifierSegment", // [1978]
    "LAST_VALUE", // [1979]
    "LEAD", // [1980]
    "LEAST", // [1981]
    "LENGTH", // [1982]
    "LikeGrammar", // [1983]
    "IlikeKeywordSegment", // [1984]
    "SimilarKeywordSegment", // [1985]
    "IncludingKeywordSegment", // [1986]
    "ExcludingKeywordSegment", // [1987]
    "DefaultsKeywordSegment", // [1988]
    "LIKE_REGEX", // [1989]
    "LINK", // [1990]
    "LIST", // [1991]
    "LISTAGG", // [1992]
    "ListenKeywordSegment", // [1993]
    "ArrayLiteralSegment", // [1994]
    "array_literal", // [1995]
    "TypedArrayLiteralSegment", // [1996]
    "typed_array_literal", // [1997]
    "ObjectLiteralSegment", // [1998]
    "object_literal", // [1999]
    "MaxLiteralSegment", // [2000]
    "DollarNumericLiteralSegment", // [2001]
    "LN", // [2002]
    "LoadKeywordSegment", // [2003]
    "LOCALTIME", // [2004]
    "LOCALTIMESTAMP", // [2005]
    "LOCATOR", // [2006]
    "LockKeywordSegment", // [2007]
    "LOG10", // [2008]
    "LOG", // [2009]
    "LOWER", // [2010]
    "LUN", // [2011]
    "LUNS", // [2012]
    "MlKeywordSegment", // [2013]
    "MAP", // [2014]
    "MATCH", // [2015]
    "MATCH_NUMBER", // [2016]
    "MATCH_RECOGNIZE", // [2017]
    "MATCHED", // [2018]
    "MATCHES", // [2019]
    "max_literal", // [2020]
    "MAXFILESIZE", // [2021]
    "MB", // [2022]
    "MEASURES", // [2023]
    "MEMBER", // [2024]
    "MergeKeywordSegment", // [2025]
    "MergeMatchedClauseSegment", // [2026]
    "merge_when_matched_clause", // [2027]
    "MergeNotMatchedClauseSegment", // [2028]
    "merge_when_not_matched_clause", // [2029]
    "MatchedKeywordSegment", // [2030]
    "ThenKeywordSegment", // [2031]
    "THEN", // [2032]
    "MergeUpdateClauseSegment", // [2033]
    "merge_update_clause", // [2034]
    "MergeDeleteClauseSegment", // [2035]
    "merge_delete_clause", // [2036]
    "SourceKeywordSegment", // [2037]
    "SOURCE", // [2038]
    "MergeInsertClauseSegment", // [2039]
    "merge_insert_clause", // [2040]
    "MergeIntoLiteralGrammar", // [2041]
    "AliasedTableReferenceGrammar", // [2042]
    "MergeMatchSegment", // [2043]
    "merge_match", // [2044]
    "DuplicatesKeywordSegment", // [2045]
    "SetClauseListSegment", // [2046]
    "set_clause_list", // [2047]
    "MESSAGE_LENGTH", // [2048]
    "MESSAGE_OCTET_LENGTH", // [2049]
    "MESSAGE_TEXT", // [2050]
    "meta_command_query_buffer", // [2051]
    "meta_command", // [2052]
    "MetaCommandQueryBufferSegment", // [2053]
    "MOD", // [2054]
    "MODE", // [2055]
    "MODIFIES", // [2056]
    "MODULE", // [2057]
    "MODULUS", // [2058]
    "MORE", // [2059]
    "MOVE", // [2060]
    "MultilineConcatenateNewline", // [2061]
    "NewlineSegment", // [2062]
    "MULTISET", // [2063]
    "MUMPS", // [2064]
    "naked_identifier_all", // [2065]
    "naked_identifier", // [2066]
    "=>", // [2067]
    "RightArrowSegment", // [2068]
    "WindowSpecificationSegment", // [2069]
    "window_specification", // [2070]
    "NamedWindowExpressionSegment", // [2071]
    "named_window_expression", // [2072]
    "NAMES", // [2073]
    "null_literal", // [2074]
    "NATIONAL", // [2075]
    "NATURAL", // [2076]
    "NaturalKeywordSegment", // [2077]
    "NCLOB", // [2078]
    "sign_indicator", // [2079]
    "NESTED", // [2080]
    "NESTING", // [2081]
    "NIL", // [2082]
    "NOCACHE", // [2083]
    "NOCYCLE", // [2084]
    "UnorderedSelectStatementSegment", // [2085]
    "NonSetSelectableGrammar", // [2086]
    "BracketedSetExpressionGrammar", // [2087]
    "NOORDER", // [2088]
    "NORMALIZE", // [2089]
    "NfcKeywordSegment", // [2090]
    "NfdKeywordSegment", // [2091]
    "NfkcKeywordSegment", // [2092]
    "NfkdKeywordSegment", // [2093]
    "NormalizedKeywordSegment", // [2094]
    "RawNotSegment", // [2095]
    "NotnullKeywordSegment", // [2096]
    "NotifyKeywordSegment", // [2097]
    "NTH_VALUE", // [2098]
    "NTILE", // [2099]
    "NULLABLE", // [2100]
    "NULLIF", // [2101]
    "NUMBER", // [2102]
    "ObjectLiteralElementSegment", // [2103]
    "object_literal_element", // [2104]
    "start_curly_bracket", // [2105]
    "OCCURRENCES_REGEX", // [2106]
    "OCTET_LENGTH", // [2107]
    "OCTETS", // [2108]
    "OFFLINE", // [2109]
    "OID", // [2110]
    "OMIT", // [2111]
    "ONE", // [2112]
    "OPEN", // [2113]
    "OptionsListGrammar", // [2114]
    "WithFillSegment", // [2115]
    "with_fill", // [2116]
    "NoorderKeywordSegment", // [2117]
    "ORDERING", // [2118]
    "ORDINALITY", // [2119]
    "OUTPUT", // [2120]
    "OverKeywordSegment", // [2121]
    "OVER", // [2122]
    "OVERFLOW", // [2123]
    "OverlapsKeywordSegment", // [2124]
    "OVERLAPS", // [2125]
    "OVERLAY", // [2126]
    "OVERRIDING", // [2127]
    "OVERWRITE", // [2128]
    "P", // [2129]
    "PAD", // [2130]
    "PARAMETER", // [2131]
    "parameter", // [2132]
    "?", // [2133]
    "PARAMETER_MODE", // [2134]
    "PARAMETER_NAME", // [2135]
    "PARAMETER_ORDINAL_POSITION", // [2136]
    "PARAMETER_SPECIFIC_CATALOG", // [2137]
    "PARAMETER_SPECIFIC_NAME", // [2138]
    "PARAMETER_SPECIFIC_SCHEMA", // [2139]
    "PARTIAL", // [2140]
    "ModulusKeywordSegment", // [2141]
    "RemainderKeywordSegment", // [2142]
    "REMAINDER", // [2143]
    "PartitionedKeywordSegment", // [2144]
    "PASCAL", // [2145]
    "PASS", // [2146]
    "PASSING", // [2147]
    "PASSTHROUGH", // [2148]
    "PAST", // [2149]
    "PATH", // [2150]
    "SlashSegment", // [2151]
    "path_segment", // [2152]
    "WordSegment", // [2153]
    "PATTERN", // [2154]
    "PER", // [2155]
    "PERCENT", // [2156]
    "PERCENT_RANK", // [2157]
    "PERCENTILE_CONT", // [2158]
    "PERCENTILE_DISC", // [2159]
    "PERIOD", // [2160]
    "PERMISSION", // [2161]
    "PERMUTE", // [2162]
    "pipe", // [2163]
    "PLACING", // [2164]
    "PLAN", // [2165]
    "PLI", // [2166]
    "PORTION", // [2167]
    "POSITION", // [2168]
    "POSITION_REGEX", // [2169]
    "WithinGroupClauseSegment", // [2170]
    "WITHIN", // [2171]
    "withingroup_clause", // [2172]
    "OverClauseSegment", // [2173]
    "over_clause", // [2174]
    "FilterClauseGrammar", // [2175]
    "PostgresCopyStdinDataSegment", // [2176]
    "POWER", // [2177]
    "LateralKeywordSegment", // [2178]
    "PRECEDES", // [2179]
    "PREPARED", // [2180]
    "PRESERVE", // [2181]
    "PrimaryKeywordSegment", // [2182]
    "PRIVATE", // [2183]
    "RefcursorKeywordSegment", // [2184]
    "REFCURSOR", // [2185]
    "ArgModeGrammar", // [2186]
    "PROGRAM", // [2187]
    "properties_naked_identifier", // [2188]
    "PRUNE", // [2189]
    "PsqlCopyMetaCommandSegment", // [2190]
    "PsqlSetMetaCommandSegment", // [2191]
    "psql_variable_colon", // [2192]
    "PsqlVariableColonSegment", // [2193]
    "PTF", // [2194]
    "PublicationTableSegment", // [2195]
    "publication_table", // [2196]
    "ExtendedTableReferenceGrammar", // [2197]
    "operator", // [2198]
    "SOME", // [2199]
    "AnyKeywordSegment", // [2200]
    "SomeKeywordSegment", // [2201]
    "QuotaKeywordSegment", // [2202]
    "TB", // [2203]
    "MbKeywordSegment", // [2204]
    "GbKeywordSegment", // [2205]
    "TbKeywordSegment", // [2206]
    "quoted_identifier", // [2207]
    "quoted_literal", // [2208]
    "MultilineConcatenateDelimiterGrammar", // [2209]
    "UESCAPE", // [2210]
    "UescapeKeywordSegment", // [2211]
    "unicode_escape_value", // [2212]
    "QUOTES", // [2213]
    "RANK", // [2214]
    "raw_comparison_operator", // [2215]
    "READS", // [2216]
    "ReassignKeywordSegment", // [2217]
    "RECHECK", // [2218]
    "RECLUSTER", // [2219]
    "RECOVER", // [2220]
    "RECOVERY", // [2221]
    "RECURSIVE", // [2222]
    "REF", // [2223]
    "ReferenceMatchGrammar", // [2224]
    "ReferentialActionGrammar", // [2225]
    "MatchKeywordSegment", // [2226]
    "SIMPLE", // [2227]
    "PartialKeywordSegment", // [2228]
    "SimpleKeywordSegment", // [2229]
    "ActionKeywordSegment", // [2230]
    "REGR_AVGX", // [2231]
    "REGR_AVGY", // [2232]
    "REGR_COUNT", // [2233]
    "REGR_INTERCEPT", // [2234]
    "REGR_R2", // [2235]
    "REGR_SLOPE", // [2236]
    "REGR_SXX", // [2237]
    "REGR_SXY", // [2238]
    "REGR_SYY", // [2239]
    "ReindexKeywordSegment", // [2240]
    "REJECTLOG", // [2241]
    "RelationOptionSegment", // [2242]
    "relation_option", // [2243]
    "RELATIVE", // [2244]
    "RELEASE", // [2245]
    "REPEATABLE", // [2246]
    "REQUIRING", // [2247]
    "RESORT", // [2248]
    "RESTORE", // [2249]
    "RESULT", // [2250]
    "RETURNED_CARDINALITY", // [2251]
    "RETURNED_LENGTH", // [2252]
    "RETURNED_OCTET_LENGTH", // [2253]
    "RETURNED_SQLSTATE", // [2254]
    "AccessTargetSegment", // [2255]
    "right_arrow", // [2256]
    "RLIKE", // [2257]
    "ROUTINE_CATALOG", // [2258]
    "ROUTINE_NAME", // [2259]
    "ROUTINE_SCHEMA", // [2260]
    "FieldsKeywordSegment", // [2261]
    "TerminatedKeywordSegment", // [2262]
    "TERMINATED", // [2263]
    "LinesKeywordSegment", // [2264]
    "ROW_COUNT", // [2265]
    "ROW_NUMBER", // [2266]
    "ROWGROUPSIZE", // [2267]
    "RUNNING", // [2268]
    "TablesampleKeywordSegment", // [2269]
    "BernoulliKeywordSegment", // [2270]
    "RepeatableKeywordSegment", // [2271]
    "SAVEPOINT", // [2272]
    "SCALE", // [2273]
    "SCHEMA_NAME", // [2274]
    "SCOPE", // [2275]
    "SCOPE_CATALOG", // [2276]
    "SCOPE_NAME", // [2277]
    "SCOPE_SCHEMA", // [2278]
    "SCROLL", // [2279]
    "SECTION", // [2280]
    "LabelKeywordSegment", // [2281]
    "SEEK", // [2282]
    "WildcardExpressionSegment", // [2283]
    "wildcard_expression", // [2284]
    "TOP", // [2285]
    "TopKeywordSegment", // [2286]
    "SelectClauseModifierSegment", // [2287]
    "select_clause_modifier", // [2288]
    "SelectClauseElementSegment", // [2289]
    "select_clause_element", // [2290]
    "ConflictKeywordSegment", // [2291]
    "WithCheckOptionSegment", // [2292]
    "with_check_option", // [2293]
    "SelectClauseSegment", // [2294]
    "select_clause", // [2295]
    "IntoClauseSegment", // [2296]
    "into_clause", // [2297]
    "FromClauseSegment", // [2298]
    "from_clause", // [2299]
    "GroupByClauseSegment", // [2300]
    "groupby_clause", // [2301]
    "HavingClauseSegment", // [2302]
    "having_clause", // [2303]
    "NamedWindowSegment", // [2304]
    "named_window", // [2305]
    "QualifyClauseSegment", // [2306]
    "qualify_clause", // [2307]
    "OffsetClauseSegment", // [2308]
    "offset_clause", // [2309]
    "FetchClauseSegment", // [2310]
    "fetch_clause", // [2311]
    "WithCompoundNonSelectStatementSegment", // [2312]
    "NonWithSelectableGrammar", // [2313]
    "SELECTIVE", // [2314]
    "SELF", // [2315]
    "statement_terminator", // [2316]
    "SENSITIVE", // [2317]
    "SERVER_NAME", // [2318]
    "SetClauseSegment", // [2319]
    "set_clause", // [2320]
    "ConstraintsKeywordSegment", // [2321]
    "UnionGrammar", // [2322]
    "IntersectKeywordSegment", // [2323]
    "MinusKeywordSegment", // [2324]
    "TimeKeywordSegment", // [2325]
    "ZoneKeywordSegment", // [2326]
    "ZONE", // [2327]
    "ShowKeywordSegment", // [2328]
    "DatasharesKeywordSegment", // [2329]
    "Is_superuserKeywordSegment", // [2330]
    "Server_encodingKeywordSegment", // [2331]
    "Server_versionKeywordSegment", // [2332]
    "PositiveSegment", // [2333]
    "NegativeSegment", // [2334]
    "SIN", // [2335]
    "SINH", // [2336]
    "SIZE", // [2337]
    "ArrayTypeSegment", // [2338]
    "array_type", // [2339]
    "slash", // [2340]
    "slice", // [2341]
    "SORT", // [2342]
    "SPACE", // [2343]
    "SPECIFIC", // [2344]
    "SPECIFIC_NAME", // [2345]
    "SPECIFICTYPE", // [2346]
    "SQL", // [2347]
    "SQLCODE", // [2348]
    "SQLERROR", // [2349]
    "SQLEXCEPTION", // [2350]
    "SQLSTATE", // [2351]
    "SQLWARNING", // [2352]
    "SQRT", // [2353]
    "STANDALONE", // [2354]
    "star", // [2355]
    "STATE", // [2356]
    "MergeStatementSegment", // [2357]
    "merge_statement", // [2358]
    "TransactionStatementSegment", // [2359]
    "transaction_statement", // [2360]
    "DropTableStatementSegment", // [2361]
    "drop_table_statement", // [2362]
    "DropViewStatementSegment", // [2363]
    "drop_view_statement", // [2364]
    "CreateUserStatementSegment", // [2365]
    "create_user_statement", // [2366]
    "DropUserStatementSegment", // [2367]
    "drop_user_statement", // [2368]
    "TruncateStatementSegment", // [2369]
    "truncate_table", // [2370]
    "AccessStatementSegment", // [2371]
    "access_statement", // [2372]
    "CreateTableStatementSegment", // [2373]
    "create_table_statement", // [2374]
    "CreateRoleStatementSegment", // [2375]
    "create_role_statement", // [2376]
    "DropRoleStatementSegment", // [2377]
    "drop_role_statement", // [2378]
    "AlterTableStatementSegment", // [2379]
    "alter_table_statement", // [2380]
    "CreateSchemaStatementSegment", // [2381]
    "create_schema_statement", // [2382]
    "SetSchemaStatementSegment", // [2383]
    "set_schema_statement", // [2384]
    "DropSchemaStatementSegment", // [2385]
    "drop_schema_statement", // [2386]
    "DropTypeStatementSegment", // [2387]
    "drop_type_statement", // [2388]
    "CreateDatabaseStatementSegment", // [2389]
    "create_database_statement", // [2390]
    "DropDatabaseStatementSegment", // [2391]
    "drop_database_statement", // [2392]
    "CreateIndexStatementSegment", // [2393]
    "create_index_statement", // [2394]
    "DropIndexStatementSegment", // [2395]
    "drop_index_statement", // [2396]
    "CreateViewStatementSegment", // [2397]
    "create_view_statement", // [2398]
    "CreateCastStatementSegment", // [2399]
    "create_cast_statement", // [2400]
    "DropCastStatementSegment", // [2401]
    "drop_cast_statement", // [2402]
    "CreateFunctionStatementSegment", // [2403]
    "create_function_statement", // [2404]
    "DropFunctionStatementSegment", // [2405]
    "drop_function_statement", // [2406]
    "CreateModelStatementSegment", // [2407]
    "create_model_statement", // [2408]
    "DropModelStatementSegment", // [2409]
    "drop_MODELstatement", // [2410]
    "DescribeStatementSegment", // [2411]
    "describe_statement", // [2412]
    "UseStatementSegment", // [2413]
    "use_statement", // [2414]
    "ExplainStatementSegment", // [2415]
    "explain_statement", // [2416]
    "CreateSequenceStatementSegment", // [2417]
    "create_sequence_statement", // [2418]
    "AlterSequenceStatementSegment", // [2419]
    "alter_sequence_statement", // [2420]
    "DropSequenceStatementSegment", // [2421]
    "drop_sequence_statement", // [2422]
    "CreateTriggerStatementSegment", // [2423]
    "create_trigger", // [2424]
    "DropTriggerStatementSegment", // [2425]
    "drop_trigger", // [2426]
    "AlterDefaultPrivilegesStatementSegment", // [2427]
    "alter_default_privileges_statement", // [2428]
    "DropOwnedStatementSegment", // [2429]
    "drop_owned_statement", // [2430]
    "ReassignOwnedStatementSegment", // [2431]
    "reassign_owned_statement", // [2432]
    "CommentOnStatementSegment", // [2433]
    "comment_clause", // [2434]
    "AnalyzeStatementSegment", // [2435]
    "analyze_statement", // [2436]
    "CreateTableAsStatementSegment", // [2437]
    "create_table_as_statement", // [2438]
    "AlterTriggerStatementSegment", // [2439]
    "alter_trigger", // [2440]
    "SetStatementSegment", // [2441]
    "set_statement", // [2442]
    "AlterSystemStatementSegment", // [2443]
    "alter_system_statement", // [2444]
    "AlterPolicyStatementSegment", // [2445]
    "alter_policy_statement", // [2446]
    "CreatePolicyStatementSegment", // [2447]
    "create_policy_statement", // [2448]
    "DropPolicyStatementSegment", // [2449]
    "drop_policy_statement", // [2450]
    "CreateDomainStatementSegment", // [2451]
    "create_domain_statement", // [2452]
    "AlterDomainStatementSegment", // [2453]
    "alter_domain_statement", // [2454]
    "DropDomainStatementSegment", // [2455]
    "drop_domain_statement", // [2456]
    "CreateMaterializedViewStatementSegment", // [2457]
    "create_materialized_view_statement", // [2458]
    "AlterMaterializedViewStatementSegment", // [2459]
    "alter_materialized_view_statement", // [2460]
    "DropMaterializedViewStatementSegment", // [2461]
    "drop_materialized_view_statement", // [2462]
    "RefreshMaterializedViewStatementSegment", // [2463]
    "refresh_materialized_view_statement", // [2464]
    "AlterDatabaseStatementSegment", // [2465]
    "alter_database_statement", // [2466]
    "VacuumStatementSegment", // [2467]
    "vacuum_statement", // [2468]
    "AlterFunctionStatementSegment", // [2469]
    "alter_function_statement", // [2470]
    "AlterViewStatementSegment", // [2471]
    "alter_view_statement", // [2472]
    "ListenStatementSegment", // [2473]
    "listen_statement", // [2474]
    "NotifyStatementSegment", // [2475]
    "notify_statement", // [2476]
    "UnlistenStatementSegment", // [2477]
    "unlisten_statement", // [2478]
    "LoadStatementSegment", // [2479]
    "load_statement", // [2480]
    "ResetStatementSegment", // [2481]
    "reset_statement", // [2482]
    "DiscardStatementSegment", // [2483]
    "discard_statement", // [2484]
    "AlterProcedureStatementSegment", // [2485]
    "alter_procedure_statement", // [2486]
    "CreateProcedureStatementSegment", // [2487]
    "create_procedure_statement", // [2488]
    "DropProcedureStatementSegment", // [2489]
    "drop_procedure_statement", // [2490]
    "CopyStatementSegment", // [2491]
    "copy_statement", // [2492]
    "DoStatementSegment", // [2493]
    "do_statement", // [2494]
    "AlterIndexStatementSegment", // [2495]
    "alter_index_statement", // [2496]
    "ReindexStatementSegment", // [2497]
    "reindex_statement_segment", // [2498]
    "AlterRoleStatementSegment", // [2499]
    "alter_role_statement", // [2500]
    "CreateExtensionStatementSegment", // [2501]
    "create_extension_statement", // [2502]
    "DropExtensionStatementSegment", // [2503]
    "drop_extension_statement", // [2504]
    "AlterExtensionStatementSegment", // [2505]
    "alter_extension_statement", // [2506]
    "CreateSubscriptionStatementSegment", // [2507]
    "create_subscription", // [2508]
    "AlterSubscriptionStatementSegment", // [2509]
    "alter_subscription", // [2510]
    "DropSubscriptionStatementSegment", // [2511]
    "drop_subscription", // [2512]
    "CreatePublicationStatementSegment", // [2513]
    "create_publication_statement", // [2514]
    "AlterPublicationStatementSegment", // [2515]
    "alter_publication_statement", // [2516]
    "DropPublicationStatementSegment", // [2517]
    "drop_publication_statement", // [2518]
    "CreateTypeStatementSegment", // [2519]
    "create_type_statement", // [2520]
    "AlterTypeStatementSegment", // [2521]
    "alter_type_statement", // [2522]
    "CreateTextSearchConfigurationStatementSegment", // [2523]
    "create_text_search_configuration_statement", // [2524]
    "AlterTextSearchConfigurationStatementSegment", // [2525]
    "alter_text_search_configuration_statement", // [2526]
    "DropTextSearchConfigurationStatementSegment", // [2527]
    "drop_text_search_configuration_statement", // [2528]
    "AlterSchemaStatementSegment", // [2529]
    "alter_schema_statement", // [2530]
    "LockTableStatementSegment", // [2531]
    "lock_table_statement", // [2532]
    "ClusterStatementSegment", // [2533]
    "cluster_statement", // [2534]
    "CreateCollationStatementSegment", // [2535]
    "create_collation_statement", // [2536]
    "DropCollationStatementSegment", // [2537]
    "drop_collation_statement", // [2538]
    "CallStoredProcedureSegment", // [2539]
    "call_statement", // [2540]
    "CreateServerStatementSegment", // [2541]
    "create_server_statement", // [2542]
    "CreateUserMappingStatementSegment", // [2543]
    "create_user_mapping_statement", // [2544]
    "ImportForeignSchemaStatementSegment", // [2545]
    "import_foreign_schema_statement", // [2546]
    "CreateForeignTableStatementSegment", // [2547]
    "create_foreign_table_statement", // [2548]
    "DropAggregateStatementSegment", // [2549]
    "drop_aggregate_statement", // [2550]
    "CreateAggregateStatementSegment", // [2551]
    "create_aggregate_statement", // [2552]
    "AlterAggregateStatementSegment", // [2553]
    "alter_aggregate_statement", // [2554]
    "CreateStatisticsStatementSegment", // [2555]
    "create_statistics_statement", // [2556]
    "AlterStatisticsStatementSegment", // [2557]
    "alter_statistics_statement", // [2558]
    "DropStatisticsStatementSegment", // [2559]
    "drop_statistics_statement", // [2560]
    "SetConstraintsStatementSegment", // [2561]
    "set_constraint_statement", // [2562]
    "CreateForeignDataWrapperStatementSegment", // [2563]
    "create_foreign_data_wrapper", // [2564]
    "MetaCommandQueryBufferStatement", // [2565]
    "meta_command_statement", // [2566]
    "DropForeignTableStatement", // [2567]
    "drop_foreign_table_statement", // [2568]
    "CreateOperatorStatementSegment", // [2569]
    "create_operator_statement", // [2570]
    "AlterForeignTableStatementSegment", // [2571]
    "alter_foreign_table_statement", // [2572]
    "SecurityLabelStatementSegment", // [2573]
    "security_label_statement", // [2574]
    "PrepareStatementSegment", // [2575]
    "prepare_statement", // [2576]
    "ExecuteStatementSegment", // [2577]
    "execute_statement", // [2578]
    "DeallocateStatementSegment", // [2579]
    "deallocate_statement", // [2580]
    "SetSessionAuthorizationStatementSegment", // [2581]
    "set_session_authorization_statement", // [2582]
    "ResetSessionAuthorizationStatementSegment", // [2583]
    "reset_session_authorization_statement", // [2584]
    "CreateLibraryStatementSegment", // [2585]
    "create_library_statement", // [2586]
    "CreateGroupStatementSegment", // [2587]
    "create_group", // [2588]
    "AlterUserStatementSegment", // [2589]
    "alter_user_statement", // [2590]
    "AlterGroupStatementSegment", // [2591]
    "alter_group", // [2592]
    "CreateExternalTableAsStatementSegment", // [2593]
    "create_external_table_statement", // [2594]
    "CreateExternalTableStatementSegment", // [2595]
    "CreateExternalSchemaStatementSegment", // [2596]
    "create_external_schema_statement", // [2597]
    "UnloadStatementSegment", // [2598]
    "unload_statement", // [2599]
    "ShowModelStatementSegment", // [2600]
    "show_model_statement", // [2601]
    "CreateDatashareStatementSegment", // [2602]
    "create_datashare_statement", // [2603]
    "DescDatashareStatementSegment", // [2604]
    "desc_datashare_statement", // [2605]
    "DropDatashareStatementSegment", // [2606]
    "drop_datashare_statement", // [2607]
    "ShowDatasharesStatementSegment", // [2608]
    "show_datashares_statement", // [2609]
    "AltereDatashareStatementSegment", // [2610]
    "DeclareStatementSegment", // [2611]
    "declare_statement", // [2612]
    "FetchStatementSegment", // [2613]
    "fetch_statement", // [2614]
    "CloseStatementSegment", // [2615]
    "close_statement", // [2616]
    "AnalyzeCompressionStatementSegment", // [2617]
    "analyze_compression_statement", // [2618]
    "CallStatementSegment", // [2619]
    "CreateRlsPolicyStatementSegment", // [2620]
    "create_rls_policy_statement", // [2621]
    "ManageRlsPolicyStatementSegment", // [2622]
    "manage_rls_policy_statement", // [2623]
    "DropRlsPolicyStatementSegment", // [2624]
    "drop_rls_policy_statement", // [2625]
    "CreateExternalFunctionStatementSegment", // [2626]
    "create_external_function_statement", // [2627]
    "GrantUsageDatashareStatementSegment", // [2628]
    "grant_datashare_statement", // [2629]
    "STATIC", // [2630]
    "STDDEV_POP", // [2631]
    "STDDEV_SAMP", // [2632]
    "STDIN", // [2633]
    "STDOUT", // [2634]
    "ConcatSegment", // [2635]
    "STRING", // [2636]
    "STRIP", // [2637]
    "STRUCTURE", // [2638]
    "STYLE", // [2639]
    "SUBCLASS_ORIGIN", // [2640]
    "SUBMULTISET", // [2641]
    "SUBSET", // [2642]
    "SUBSTRING", // [2643]
    "SUBSTRING_REGEX", // [2644]
    "SUCCEEDS", // [2645]
    "SUM", // [2646]
    "SYMMETRIC", // [2647]
    "SYSTEM_TIME", // [2648]
    "SYSTEM_USER", // [2649]
    "T", // [2650]
    "InterleavedKeywordSegment", // [2651]
    "ObjectUnpivotSegment", // [2652]
    "object_unpivoting", // [2653]
    "ArrayUnnestSegment", // [2654]
    "array_unnesting", // [2655]
    "TABLE_NAME", // [2656]
    "TAG", // [2657]
    "Expression_A_Unary_Operator_Grammar", // [2658]
    "Expression_C_Grammar", // [2659]
    "Expression_B_Unary_Operator_Grammar", // [2660]
    "TAN", // [2661]
    "TANH", // [2662]
    "TDES", // [2663]
    "TRANSIENT", // [2664]
    "TransientKeywordSegment", // [2665]
    "THROUGH", // [2666]
    "tilde", // [2667]
    "TimestampKeywordSegment", // [2668]
    "TOKEN", // [2669]
    "TOP_LEVEL_COUNT", // [2670]
    "TRANSACTION", // [2671]
    "CommitKeywordSegment", // [2672]
    "RollbackKeywordSegment", // [2673]
    "AbortKeywordSegment", // [2674]
    "WORK", // [2675]
    "TransactionKeywordSegment", // [2676]
    "WorkKeywordSegment", // [2677]
    "CommittedKeywordSegment", // [2678]
    "UncommittedKeywordSegment", // [2679]
    "UNCOMMITTED", // [2680]
    "TRANSACTION_ACTIVE", // [2681]
    "TRANSACTIONS_COMMITTED", // [2682]
    "TRANSACTIONS_ROLLED_BACK", // [2683]
    "TRANSFORMS", // [2684]
    "TRANSLATE", // [2685]
    "TRANSLATE_REGEX", // [2686]
    "TRANSLATION", // [2687]
    "TREAT", // [2688]
    "TRIGGER_CATALOG", // [2689]
    "TRIGGER_NAME", // [2690]
    "TRIGGER_SCHEMA", // [2691]
    "TRIM", // [2692]
    "BothKeywordSegment", // [2693]
    "LeadingKeywordSegment", // [2694]
    "TrailingKeywordSegment", // [2695]
    "TRIM_ARRAY", // [2696]
    "ContinueKeywordSegment", // [2697]
    "TRUSTED", // [2698]
    "StructLiteralSegment", // [2699]
    "CrossKeywordSegment", // [2700]
    "NaturalJoinKeywordsGrammar", // [2701]
    "UnconditionalCrossJoinKeywordsGrammar", // [2702]
    "HorizontalJoinKeywordsGrammar", // [2703]
    "UNCONDITIONAL", // [2704]
    "UNDER", // [2705]
    "UNENCRYPTED", // [2706]
    "UnionKeywordSegment", // [2707]
    "UNLINK", // [2708]
    "UnlistenKeywordSegment", // [2709]
    "UnloadKeywordSegment", // [2710]
    "MaxfilesizeKeywordSegment", // [2711]
    "RowgroupsizeKeywordSegment", // [2712]
    "AllowoverwriteKeywordSegment", // [2713]
    "CleanpathKeywordSegment", // [2714]
    "AddquotesKeywordSegment", // [2715]
    "HeaderKeywordSegment", // [2716]
    "UNMATCHED", // [2717]
    "UNNAMED", // [2718]
    "UNNEST", // [2719]
    "UNSIGNED", // [2720]
    "UNTYPED", // [2721]
    "UPPER", // [2722]
    "USER_DEFINED_TYPE_CATALOG", // [2723]
    "USER_DEFINED_TYPE_CODE", // [2724]
    "USER_DEFINED_TYPE_NAME", // [2725]
    "USER_DEFINED_TYPE_SCHEMA", // [2726]
    "USER_LIMIT", // [2727]
    "UTF32", // [2728]
    "VacuumKeywordSegment", // [2729]
    "ReclusterKeywordSegment", // [2730]
    "SortKeywordSegment", // [2731]
    "PercentKeywordSegment", // [2732]
    "BoostKeywordSegment", // [2733]
    "VALUE_OF", // [2734]
    "VAR_POP", // [2735]
    "VAR_SAMP", // [2736]
    "VERSIONING", // [2737]
    "WALLET", // [2738]
    "assignment_operator", // [2739]
    "indented_then", // [2740]
    "indented_then_contents", // [2741]
    "WHENEVER", // [2742]
    "WHITESPACE", // [2743]
    "whitespace", // [2744]
    "WhitespaceSegment", // [2745]
    "WIDTH_BUCKET", // [2746]
    "ExcludeClauseSegment", // [2747]
    "select_exclude_clause", // [2748]
    "PartitionClauseSegment", // [2749]
    "partitionby_clause", // [2750]
    "FrameClauseSegment", // [2751]
    "frame_clause", // [2752]
    "CascadedKeywordSegment", // [2753]
    "RecursiveKeywordSegment", // [2754]
    "indented_ctes", // [2755]
    "CTEDefinitionSegment", // [2756]
    "common_table_expression", // [2757]
    "NonWithNonSelectableGrammar", // [2758]
    "BindingKeywordSegment", // [2759]
    "WithinKeywordSegment", // [2760]
    "XMLAGG", // [2761]
    "XMLATTRIBUTES", // [2762]
    "XMLBINARY", // [2763]
    "XMLCAST", // [2764]
    "XMLCOMMENT", // [2765]
    "XMLCONCAT", // [2766]
    "XMLDECLARATION", // [2767]
    "XMLDOCUMENT", // [2768]
    "XMLELEMENT", // [2769]
    "XMLEXISTS", // [2770]
    "XMLFOREST", // [2771]
    "XMLITERATE", // [2772]
    "XMLNAMESPACES", // [2773]
    "XMLPARSE", // [2774]
    "XMLPI", // [2775]
    "XMLQUERY", // [2776]
    "XMLROOT", // [2777]
    "XMLSCHEMA", // [2778]
    "XMLSERIALIZE", // [2779]
    "XMLTABLE", // [2780]
    "XMLTEXT", // [2781]
    "XMLVALIDATE", // [2782]
];

pub static AUX_DATA: &[u32] = &[
    0, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 7, 1, 2, 1, 1, 4, 3, 1, 4, 5, 8, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 9, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 10, 1, 2, 1, 1, 4, 3, 1, 4, 5, 11, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 12, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0,
    0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 211,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 218, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 14, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 219, 1, 2, 1, 1, 4, 3, 1, 4, 5, 220, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 221, 1, 2, 1, 1, 4, 3, 1, 4, 5, 222,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 223, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 228, 1, 2, 1, 1, 4, 3, 1, 4, 5, 229, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 230, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 232, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 86, 245, 246, 1, 245, 3, 3, 4, 247, 248, 249,
    246, 1, 249, 3, 3, 4, 247, 1, 2, 1, 1, 0, 1, 2, 1, 0,
    15, 1, 2, 1, 1, 4, 3, 1, 4, 5, 259, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 261, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0, 1, 4294967295,
    4294967295, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0,
    0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0, 0, 1, 0, 1, 2,
    1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 4294967295,
    4294967295, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1, 0, 0,
    1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 1, 0, 0, 262, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 1, 0, 1,
    2, 1, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0,
    1, 0, 0, 1, 0, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 1,
    4294967295, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 1,
    0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    4294967295, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 3,
    4, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    1, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 1, 0, 1,
    0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0,
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 4294967295, 1, 0, 0, 0,
    0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 1, 0, 1,
    0, 1, 0, 0, 1, 0, 0, 0, 0, 449, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 739, 740, 246, 1, 740, 3, 3, 4, 247, 741, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 742, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 1, 2, 1, 0, 1, 0,
    1, 2, 1, 1, 0, 750, 1, 2, 1, 1, 4, 3, 1, 4, 5, 750,
    751, 752, 1, 751, 3, 3, 751, 4, 753, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 754, 1, 2, 1, 1, 4, 3, 1, 4, 5, 649, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 102, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    755, 1, 2, 1, 1, 4, 3, 1, 4, 5, 756, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 776, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 213, 777, 246, 1, 777, 3, 3, 4, 247, 778, 779, 246, 1, 779,
    3, 3, 4, 247, 1, 2, 0, 0, 4294967295, 4294967295, 0, 303, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 0, 1, 2, 0, 1, 2, 0, 784, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 785, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 240, 1, 2, 1, 1, 4, 3, 1, 4, 5, 786, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 787, 1, 2, 1, 1, 4, 3, 1, 4, 5, 788,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 789, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 790, 1, 2, 1, 1, 4, 3, 1, 4, 5, 791, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 783, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 792, 1, 2, 1, 1, 4, 3, 1, 4, 5, 793, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 567, 1, 2, 1, 1, 4, 3, 1, 4, 5, 713,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 794, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 795, 1, 2, 1, 1, 4, 3, 1, 4, 5, 796, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 797, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 0, 4294967295, 1, 0, 670, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 811, 1, 2, 1, 1, 4, 3, 1, 4, 5, 813, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 652,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 815, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 816, 1, 2, 1, 1, 4, 3, 1, 4, 5, 438, 439, 440,
    441, 442, 275, 443, 444, 127, 1117, 9, 817, 818, 1, 817, 2, 3, 4, 819,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 825, 826, 0, 2, 3,
    825, 3, 827, 0, 1, 3, 715, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    828, 1, 2, 1, 1, 4, 3, 1, 4, 5, 829, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 830, 1, 2, 1, 1, 4, 3, 1, 4, 5, 831, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 832, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 833, 1, 2, 1, 1, 4, 3, 1, 4, 5, 834, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 751, 752, 9, 3, 3, 751, 4, 848, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 849, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 850, 1, 2, 1, 1, 4, 3, 1, 4, 5, 851, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 765, 751, 246, 1, 751, 3, 3, 4, 247, 855,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 856, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 857, 1, 2, 1, 1, 4, 3, 1, 4, 5, 858, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 859, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 862, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 865, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 866, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 867, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 869, 870, 0, 2, 3, 869, 1, 2, 1, 873,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 874, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 450, 1, 2, 1, 1, 4, 3, 1, 4, 5, 587, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 653, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 875, 1, 2, 1, 1, 4, 3, 1, 4, 5, 876, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 0, 0, 455, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 883, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 885, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 345, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    886, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295,
    4294967295, 0, 0, 887, 1, 2, 1, 1, 4, 3, 1, 4, 5, 901, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 902, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 903, 904, 246, 1, 904,
    3, 3, 4, 247, 16, 1, 2, 1, 1, 4, 3, 1, 4, 5, 905, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 906, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 907, 1, 2, 1, 1, 4, 3, 1, 4, 5, 908, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 909, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    910, 1, 2, 1, 1, 4, 3, 1, 4, 5, 911, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 912, 1, 2, 1, 1, 4, 3, 1, 4, 5, 913, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 914, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 915, 1, 2, 1, 1, 4, 3, 1, 4, 5, 916, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 917, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    918, 1, 2, 1, 1, 4, 3, 1, 4, 5, 919, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 363, 1, 2, 1, 1, 4, 3, 1, 4, 5, 920, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 921, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 405, 1, 2, 1, 1, 4, 3, 1, 4, 5, 922, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 923, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    924, 1, 2, 1, 1, 4, 3, 1, 4, 5, 925, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 926, 1, 2, 1, 1, 4, 3, 1, 4, 5, 414, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 928, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 929, 1, 2, 1, 1, 4, 3, 1, 4, 5, 4, 818, 9,
    2, 3, 4, 428, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 377, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 941, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 942, 1, 2, 1, 1, 4, 3, 1, 4, 5, 943,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 944, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 937, 945, 246, 1, 945, 3, 3, 4, 247, 937, 946, 246, 1,
    946, 3, 3, 4, 247, 937, 947, 246, 1, 947, 3, 3, 4, 247, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 4294967295, 1, 0, 0, 0, 0,
    1, 2, 1, 0, 4294967295, 1, 0, 0, 4294967295, 4294967295, 0, 0, 424, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 973, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 975, 1, 2, 1, 1, 4, 3, 1, 4, 5, 91, 976,
    246, 1, 976, 3, 3, 4, 247, 977, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 978, 1, 2, 1, 1, 4, 3, 1, 4, 5, 979, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0,
    982, 983, 10, 3, 3, 982, 4, 984, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 985, 1, 2, 1, 1, 4, 3, 1, 4, 5, 986, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 298, 1004, 9, 3, 3, 298, 4, 751, 1005, 0,
    2, 3, 751, 298, 1006, 0, 2, 3, 298, 1, 2, 1, 677, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 478, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 745, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1012, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 701, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1013,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1014, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1021, 1, 2, 1, 1, 4, 3, 1, 4, 5, 394, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0, 0,
    1026, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0,
    1, 2, 1, 0, 103, 1, 2, 1, 1, 4, 3, 1, 4, 5, 289, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1031, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1032, 1, 2, 1, 1, 4, 3, 1, 4, 5, 361, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1033, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1034, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1035, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1036, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1037,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1038, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1039, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1040, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1041, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 378, 1, 2, 1, 1, 4, 3, 1, 4, 5, 3, 4, 1, 1042, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1044, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0, 1122, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1123, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1124, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1125, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 530, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1126, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1127, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1128, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1129, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 3, 4, 1, 0, 0, 0, 0,
    1, 0, 1, 2, 1, 0, 0, 4294967295, 1, 0, 0, 4294967295, 1, 0, 0, 0,
    0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 0, 0,
    0, 4294967295, 1, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 1, 0,
    1, 2, 1, 1, 0, 1, 2, 1, 0, 0, 1, 0, 1, 2, 1, 0,
    4294967295, 4294967295, 0, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 0, 4294967295,
    4294967295, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 1, 0, 1, 2,
    1, 0, 1, 0, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 104, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 0, 0, 1, 2,
    1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 0, 1, 2, 1, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 1, 2,
    1, 0, 4294967295, 1305, 246, 1, 1305, 3, 3, 4, 247, 0, 4294967295, 4294967295, 0, 0,
    0, 4294967295, 1305, 246, 1, 1305, 3, 3, 4, 247, 0, 4294967295, 4294967295, 0, 0, 4294967295,
    1310, 246, 1, 1310, 3, 3, 4, 247, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1,
    0, 1, 2, 1, 0, 0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 1,
    0, 0, 1, 0, 1, 2, 1, 1, 2, 1, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 4294967295, 1, 0, 0, 0, 0, 0, 0, 0, 0, 4294967295, 4294967295,
    0, 0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 0,
    1, 0, 1, 2, 1, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2, 1, 0, 1,
    0, 0, 1, 0, 0, 0, 0, 0, 0, 1, 2, 1, 0, 4294967295, 4294967295, 0,
    0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 0, 1, 0, 0, 4294967295,
    1, 0, 1, 2, 1, 0, 588, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    589, 1, 2, 1, 1, 4, 3, 1, 4, 5, 717, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 798, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1363, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 930, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1046, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1364, 1365, 818, 1,
    1365, 2, 3, 4, 1364, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    2, 1, 1371, 1, 2, 1, 1, 4, 3, 1, 4, 5, 326, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1372, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    439, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1373, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1374, 1, 2, 1, 1, 4, 3, 1, 4, 5, 273, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1375, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 375, 1, 2, 1, 1, 4, 3, 1, 4, 5, 441, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 442, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1376, 1, 2, 1, 1, 4, 3, 1, 4, 5, 275, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 443, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1377, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1378, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 456, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 56, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1386, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 17, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1387, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 732, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1388, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1389, 818,
    1, 1389, 2, 3, 4, 0, 0, 0, 0, 0, 0, 0, 304, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1452, 1453, 1454, 1455, 3270, 4, 1365, 818, 1, 1365,
    2, 3, 4, 0, 1, 0, 1, 2, 1, 0, 0, 1047, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 305, 1, 2, 1, 1, 4, 3, 1, 4, 5, 876,
    1458, 1459, 1460, 1461, 1462, 1463, 1464, 1465, 1466, 1467, 1468, 1469, 1470, 1471, 1472, 1473,
    1474, 1475, 1476, 1477, 1478, 1479, 1480, 1481, 1482, 1483, 1484, 1485, 1486, 1487, 1488, 1489,
    1490, 1491, 1492, 1493, 1494, 1495, 1496, 1497, 1498, 1499, 1500, 1501, 1502, 1503, 1504, 1505,
    1506, 1507, 1508, 1509, 1510, 1511, 1512, 1513, 1514, 1515, 1516, 1517, 1518, 1519, 1520, 1521,
    1522, 1523, 1524, 1525, 1526, 1527, 1528, 1529, 1530, 1531, 1532, 1533, 1534, 1535, 1536, 3311,
    80, 1543, 818, 1, 1543, 2, 3, 4, 1544, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1545, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1463, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1465, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1546, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1547, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1548, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1467, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1552, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1415, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1553, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 300, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1556, 1, 2, 1, 1, 4, 3, 1, 4, 5, 482, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 487, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1557, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1558, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 549, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1,
    2, 1, 1563, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1564, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1565, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    105, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 1, 0, 0,
    1203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1048, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1575, 1, 2, 1, 1, 4, 3, 1, 4, 5, 655, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 654, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1576, 1, 2, 1, 1, 4, 3, 1, 4, 5, 563, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 880, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1577, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1578, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1579, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1583, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1585, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 694, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1140, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1586, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    395, 1, 2, 1, 1, 4, 3, 1, 4, 5, 415, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1587, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1591,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1592, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 994, 1, 2, 1, 1, 4, 3, 1, 4, 5, 667, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 669, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 762, 751, 246, 1, 751, 3, 3, 4, 247, 1593, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1595, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1596, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1597, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1598, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1599, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1600, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1601, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1602, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1603, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1022, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1606, 1, 2, 1, 1, 4, 3, 1, 4, 5, 315, 315, 1608,
    3, 315, 1607, 315, 3, 3, 1607, 4, 18, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 212, 1609, 246, 1, 1609, 3, 3, 4, 247, 1407, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 0, 3, 4, 1, 1, 2, 1, 1,
    0, 1, 0, 1, 0, 1, 0, 352, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 0, 1, 0, 0,
    0, 1, 0, 1, 0, 1, 0, 1610, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1611, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1612, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1613, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1351,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1614, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 895, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1615, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 1, 1, 1049, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 650, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1050, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1620, 1, 2, 1, 1, 4, 3, 1, 4, 5, 590, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1621, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1622, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1623, 1624, 246, 1, 1624, 3,
    3, 4, 247, 897, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1625, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1626, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1627, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1355, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1475, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1116,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1117, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1628, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1629, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1051, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 671, 1, 2, 1, 1, 4, 3, 1, 4, 5, 379, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1630, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1293,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1631, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 1, 2, 1, 0, 1632, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1634, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1635, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 106, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    138, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1198, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1636, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1637, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 0, 1, 2,
    1, 0, 0, 1656, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1052, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 446, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0, 1, 2,
    1, 0, 0, 4294967295, 4294967295, 0, 0, 0, 0, 1, 0, 0, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 0, 0, 0, 471, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1, 2, 1, 0, 0, 371, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 19, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1455, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 287, 1, 2, 1, 1, 4, 3, 1, 4, 5, 287,
    1717, 1718, 1, 1717, 5, 3, 1, 1607, 4, 5, 406, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 0, 0, 0, 1719, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 1205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 20, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 1, 0, 0, 4294967295,
    4294967295, 0, 1053, 1, 2, 1, 1, 4, 3, 1, 4, 5, 684, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 2, 1, 1751, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1753, 1, 2, 1, 1, 4, 3, 1, 4, 5, 703, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1754, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1755, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1054, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1756,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1406, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1412, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1414, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1757, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 341,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 417, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 21, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 0, 1, 2, 1, 181, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1761,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1725, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1762, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    0, 0, 0, 1773, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1774, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1775, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 1, 0, 0, 0, 0, 1, 2, 1, 0, 0, 0,
    4294967295, 4294967295, 0, 1, 2, 1, 0, 1, 2, 1, 0, 236, 1811, 1, 0, 0,
    4294967295, 4294967295, 0, 253, 1811, 1, 1, 2, 1, 0, 0, 4294967295, 1, 0, 95, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0, 1, 0, 1, 2,
    1, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 2, 1, 1820, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1821, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 514, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1696, 1696, 1608, 3, 1696,
    1607, 1696, 3, 3, 1607, 4, 0, 0, 1, 2, 1, 0, 0, 0, 4294967295, 1,
    0, 1, 0, 0, 0, 0, 0, 1, 0, 0, 4294967295, 4294967295, 0, 1, 2, 1,
    0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 4294967295, 4294967295, 0, 0,
    4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 22, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 3, 4294967295, 1365, 818, 1, 1365, 2, 3, 4, 0, 4294967295, 4294967295, 0, 0,
    0, 0, 0, 0, 0, 1, 0, 1, 2, 1, 0, 0, 0, 0, 71, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1854, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 23, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1855, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1856, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1857, 1, 2, 1, 1, 4, 3, 1, 4, 5, 448, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1444, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1442, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1858, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1859, 1859, 1004, 3, 1859, 298, 1859, 3, 3, 298, 4, 1860, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1861, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1862, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1863, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1864, 1, 2, 1, 1, 4, 3, 1, 4, 5, 193,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 4294967295, 4294967295, 0,
    0, 0, 0, 0, 0, 0, 1872, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    137, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1876, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 0, 199, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 1, 0, 1, 0, 1880, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 1763, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1007, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1229,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1886, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 0, 0, 1778, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1887, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1888, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1889, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1812, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1167, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1438, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1890, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1477, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1291, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1189, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 799, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1891, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1892, 1893, 9, 3, 3,
    1892, 4, 454, 1, 2, 1, 1, 4, 3, 1, 4, 5, 368, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1834, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1055, 1, 2, 1, 1, 4, 3, 1, 4, 5, 682, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1056, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1660,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 489, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1896, 1, 2, 1, 1, 4, 3, 1, 4, 5, 531, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1897, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1133, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2,
    1, 1727, 1, 2, 1, 1, 4, 3, 1, 4, 5, 107, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 66, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 0, 1, 2, 1, 0, 1251, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1338, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 457, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1903, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 0, 1, 2, 1, 0,
    517, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1911, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1912, 1, 2, 1, 1, 4, 3, 1, 4, 5, 418, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1236, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1913, 1, 2, 1, 1, 4, 3, 1, 4, 5, 484, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1914, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1016, 1, 2, 1, 1, 4, 3, 1, 4, 5, 757, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 542, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1219, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1915, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 108, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 0,
    1, 2, 1, 0, 0, 0, 1916, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1917, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1341, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1393, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1399, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1402, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1397, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1395, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 24, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1918, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1263, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1779, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1919, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 821, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 1902, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 550, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 836, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1673, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 236,
    1939, 1, 0, 253, 1939, 1, 0, 1311, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 4294967295, 1, 0, 889, 1951, 1, 1, 2, 1, 0, 253, 1951, 1, 0,
    0, 1, 0, 1, 2, 1, 1057, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    838, 751, 246, 3, 751, 247, 838, 3, 3, 4, 247, 0, 0, 1963, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1964, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1965, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1966, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1967, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1968,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1969, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1970, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1971, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1972, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1168, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1973, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 672, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1974,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1975, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1961, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 9,
    4, 3, 1, 4, 5, 1169, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1274,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 800, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1976, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1977, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1161, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 25, 1, 2, 1, 1, 4, 3, 1, 4, 5, 96, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1906, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1979, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1798, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1980, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1825, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 532, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1981, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1017, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1982, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    500, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1256, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1335, 1, 2, 1, 1, 4, 3, 1, 4, 5, 839,
    839, 1004, 3, 839, 298, 839, 3, 3, 298, 4, 0, 1989, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 0, 0, 292, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1206, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1990, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1991, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1992, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1728, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 1607, 1718, 9, 5, 3, 1, 1607,
    4, 5, 1607, 1608, 9, 3, 3, 1607, 4, 2002, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1729, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1331, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2004, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2005, 1, 2, 1, 1, 4, 3, 1, 4, 5, 686, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2006, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1730,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1760, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2008, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2009,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 508, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 591, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2010, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2011, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2012, 1, 2, 1, 1, 4, 3, 1, 4, 5, 656, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1008, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1483,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 2, 3, 1, 472,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 109, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 0, 0, 1, 0, 1058, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2014, 1, 2, 1, 1, 4, 3, 1, 4, 5, 705, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 26, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 801, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2015, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2016, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2017, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2018, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2019, 1, 2, 1, 1, 4, 3, 1, 4, 5, 27, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 306, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 306, 2020, 1718, 1, 2020, 5, 3, 1, 1607, 4, 5, 1301, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1303, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1059, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2021, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 458, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2022,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2023, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2024, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1731, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 0, 0, 0, 1, 2,
    1, 0, 236, 1939, 1, 253, 1939, 1, 0, 2048, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2049, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2050, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2051, 2052, 246, 3, 2052, 247, 2051, 3, 3,
    4, 247, 0, 4294967295, 4294967295, 0, 1176, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    386, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1493, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1497, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1780, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 224, 751, 246, 1, 751, 3, 3, 4,
    247, 1499, 1, 2, 1, 1, 4, 3, 1, 4, 5, 459, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1807, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1275,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2054, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2055, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1269, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1272, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2056, 1, 2, 1, 1, 4, 3, 1, 4, 5, 110, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2057, 1, 2, 1, 1, 4, 3, 1, 4, 5, 761,
    751, 246, 1, 751, 3, 3, 4, 247, 2058, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 44, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1503, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2059, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    657, 1, 2, 1, 1, 4, 3, 1, 4, 5, 658, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 659, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2060, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1170, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1190, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1282, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 318, 318, 2062, 1, 318, 3,
    3, 318, 4, 266, 751, 246, 1, 751, 3, 3, 4, 247, 2063, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2064, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1171, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 2065, 1893, 3, 2065, 1892,
    5, 3, 3, 1892, 4, 4, 5, 2066, 1893, 1, 2066, 3, 3, 1892, 4, 627,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 0,
    2073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1582, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1665, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1665, 2074,
    1718, 1, 2074, 5, 3, 1, 1607, 4, 5, 2075, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2076, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1421, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2078, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 224, 2079, 246, 1, 2079, 3, 3, 4, 247, 2080, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2081, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1346, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 318, 2062, 12, 3, 3, 318, 4, 1721,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1666, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1667, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1668, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1669, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2082, 1, 2, 1, 1, 4, 3, 1, 4, 5, 419, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 592, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2083,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 593, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 594, 1, 2, 1, 1, 4, 3, 1, 4, 5, 718, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2084, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 595, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1060, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 596, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 1, 2, 1, 1, 2, 1, 0, 0, 1, 2, 1, 0, 0, 1318,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 634, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2088, 1, 2, 1, 1, 4, 3, 1, 4, 5, 597, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2089, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1670, 1, 2, 1, 1, 4, 3, 1, 4, 5, 598, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 358, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 358, 1, 2, 1, 1, 4, 3, 1, 4, 5, 515, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1732, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1675,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 573, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2098, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2099, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 307, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 307, 2074, 1718, 1, 2074, 5, 3, 1, 1607, 4, 5, 2100, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2101, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1253, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2102, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1416, 1, 2, 1, 1, 4, 3, 1, 4, 5, 101, 101,
    1608, 3, 101, 1607, 101, 3, 3, 1607, 4, 1422, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 99, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 313,
    2105, 246, 1, 2105, 3, 3, 4, 247, 1, 2, 0, 0, 0, 1287, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2106, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2107, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2108, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 420, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1085,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2109, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1781, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2110, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 506, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1347, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2111, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 168, 2066, 1893, 1, 2066, 3, 3, 1892, 4, 168,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2112, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 525, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2113, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 111, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 338, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 1, 2, 1, 421, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 837, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 837, 751, 752, 1, 751, 3, 3, 751, 4, 1061, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 0, 0, 1, 0, 0, 234, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 2118, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2119, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1771, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 758, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1956,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2120, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1226, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1,
    0, 2122, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2123, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 0, 3, 4, 1, 0, 0, 0, 3, 4, 1,
    0, 2125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2126, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2127, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2128,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 460, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 267, 1, 2, 1, 1, 4, 3, 1, 4, 5, 112, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2129, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2130, 1, 2, 1, 1, 4, 3, 1, 4, 5, 533, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2131, 1, 2, 1, 1, 4, 3, 1, 4, 5, 6,
    4294967295, 2132, 818, 1, 2132, 2, 3, 4, 2133, 2132, 246, 1, 2132, 3, 3, 4,
    247, 2134, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2135, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2136, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2137,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2138, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2139, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1062, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 396, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2140, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1,
    0, 1, 0, 1, 2, 1, 0, 1, 0, 1, 2, 1, 1, 2, 1, 0,
    1, 0, 1, 2, 1, 0, 570, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1, 0, 1, 2, 1, 1200, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2145,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2146, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2147, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2148, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 599, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2149, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2150, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 5, 2152, 2153, 2, 2152, 5, 3, 3, 4, 5, 1,
    0, 0, 2154, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2155, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2156, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2157, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2158, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2159, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2160, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2161, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1796, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2162, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 840, 751, 246, 3, 751, 247, 840, 3, 3, 4,
    247, 841, 751, 246, 3, 751, 247, 841, 3, 3, 4, 247, 141, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 227, 2163, 246, 1, 2163, 3, 3, 4, 247, 939,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2164, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 473, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2165, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1589, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2166, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1258, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 302, 751, 246, 1, 751, 3, 3, 4, 247, 132, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1186, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2167, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2168, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2169, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    302, 2079, 246, 1, 2079, 3, 3, 4, 247, 0, 4294967295, 4294967295, 0, 842, 751, 246,
    3, 751, 247, 842, 3, 3, 4, 247, 1740, 1744, 818, 3, 1744, 4, 1740, 2,
    3, 4, 1172, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2177, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 2179, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1767, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1410, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1551, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 2180, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1289, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2181, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1086, 1, 2, 1, 1, 4, 3, 1, 4, 5, 432, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1681, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2183, 1, 2, 1, 1, 4, 3, 1, 4, 5, 146, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 381, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 28, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 0, 0, 4294967295, 1, 0, 1, 0, 1, 2, 1, 72, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2187, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1214, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 2188, 818, 3, 2188,
    4, 5, 2, 3, 4, 2189, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1741,
    1741, 818, 3, 1741, 4, 1741, 2, 3, 4, 1742, 1742, 818, 3, 1742, 4, 1742,
    2, 3, 4, 937, 2192, 246, 1, 2192, 3, 3, 4, 247, 0, 1, 2, 1,
    0, 2194, 1, 2, 1, 1, 4, 3, 1, 4, 5, 210, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 736, 1, 2, 1, 1, 4, 3, 1, 4, 5, 580,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 0, 0, 1, 0, 0,
    0, 1, 2, 1, 7, 4294967295, 2198, 246, 1, 2198, 3, 3, 4, 247, 1, 4294967295,
    4294967295, 0, 1, 2, 1, 1782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0,
    1, 2, 1, 1512, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1813, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 620, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1380, 1, 2, 1, 1, 4, 3, 1, 4, 5, 323, 2207, 1893,
    3, 2207, 1892, 323, 3, 3, 1892, 4, 324, 2207, 1893, 3, 2207, 1892, 324, 3,
    3, 1892, 4, 0, 319, 2208, 1608, 3, 2208, 1607, 319, 3, 3, 1607, 4, 317,
    2208, 1608, 3, 2208, 1607, 317, 3, 3, 1607, 4, 0, 319, 2208, 1608, 3, 2208,
    1607, 319, 3, 3, 1607, 4, 0, 4294967295, 4294967295, 0, 314, 2208, 1608, 3, 2208, 1607,
    314, 3, 3, 1607, 4, 8, 4294967295, 2208, 1608, 1, 2208, 3, 3, 1607, 4, 0,
    4294967295, 4294967295, 0, 320, 2208, 1608, 3, 2208, 1607, 320, 3, 3, 1607, 4, 9, 4294967295,
    2208, 1608, 1, 2208, 3, 3, 1607, 4, 0, 4294967295, 4294967295, 0, 10, 4294967295, 2212, 818,
    1, 2212, 2, 3, 4, 316, 2208, 1608, 3, 2208, 1607, 316, 3, 3, 1607, 4,
    316, 2208, 1608, 3, 2208, 1607, 316, 3, 3, 1607, 4, 0, 4294967295, 4294967295, 0, 0,
    1, 0, 0, 2213, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1356, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2214, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 296, 2215, 246, 1, 2215, 3, 3, 4, 247, 764, 2215, 246, 1, 2215, 3,
    3, 4, 247, 660, 1, 2, 1, 1, 4, 3, 1, 4, 5, 763, 2215, 246,
    1, 2215, 3, 3, 4, 247, 835, 2215, 246, 1, 2215, 3, 3, 4, 247, 1063,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 113, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1064, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2216, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1404, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1733, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 0, 0, 2218,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2219, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2220, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2221, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2222, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 1173, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2223, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2185, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 4294967295, 1, 0, 0, 115, 1, 2, 1, 1, 4, 3, 1, 4, 5, 114,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1344, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 641, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1814, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1065, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2231, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2232, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2233, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2234, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2235, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2236, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2237, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2238, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2239, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1283, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1734, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 0, 2241, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 2244, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2245, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2143, 1, 2, 1, 1, 4, 3, 1, 4, 5, 733, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1066, 1, 2, 1, 1, 4, 3, 1, 4, 5, 268,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2246, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 707, 1, 2, 1, 1, 4, 3, 1, 4, 5, 422, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 600, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2247, 1, 2, 1, 1, 4, 3, 1, 4, 5, 327, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 0, 2248, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    29, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1835, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 436, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2249, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 343, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 554, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2250, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1165, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1246, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2251, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2252, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2253, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2254, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1572, 1, 2, 1, 1, 4, 3, 1, 4, 5, 534, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 194, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 2067, 2256, 246, 1, 2256, 3, 3, 4, 247, 1018, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2257, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1320, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 30, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1366, 1365, 818, 1,
    1365, 2, 3, 4, 1366, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1067, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 31, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2258, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2259, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2260, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    73, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 1, 0, 498, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2265, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2266, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2267, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 535, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    494, 1, 2, 1, 1, 4, 3, 1, 4, 5, 661, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2268, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1297, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1299, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 555, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1162, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 0, 1, 2, 1, 0, 1, 2, 1, 2272, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1958, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2273, 1, 2, 1, 1, 4, 3, 1, 4, 5, 32, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2274, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    64, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2275, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2276, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2277, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2278, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2279, 1, 2, 1, 1, 4, 3, 1, 4, 5, 393, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1516, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    806, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1183, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2280, 1, 2, 1, 1, 4, 3, 1, 4, 5, 502, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 2, 1, 1, 2, 1, 0, 1,
    2, 1, 0, 1, 2, 1, 0, 0, 0, 2282, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 0, 1, 0, 1, 2, 1, 0, 1, 0, 2, 10993, 2, 10995,
    3, 836, 994, 836, 358, 994, 0, 116, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1, 2, 1, 0, 1, 2, 1, 0, 1, 2, 1, 0, 2314, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2315, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 4294967295, 4294967295, 0, 938, 2316, 246, 1, 2316, 3, 3, 4, 247, 2317, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1831, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 33, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1068, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 74, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1215, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1218, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1932, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    57, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2318, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 719, 1, 2, 1, 1, 4, 3, 1, 4, 5, 807,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 277, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 0, 0, 0, 4294967295, 4294967295, 0, 1, 0, 1, 2, 1, 0,
    0, 4294967295, 4294967295, 0, 1, 0, 0, 1, 2, 1, 0, 1, 0, 0, 0, 1,
    4294967295, 4294967295, 0, 269, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0,
    0, 0, 0, 1, 0, 0, 0, 0, 0, 1243, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1885, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1295, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1069, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 205, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 4294967295, 4294967295, 0,
    1736, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 1661, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 2227, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1383, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2335,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 1, 0, 319, 2207, 1893,
    3, 2207, 1892, 319, 3, 3, 1892, 4, 2336, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2337, 1, 2, 1, 1, 4, 3, 1, 4, 5, 642, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 762, 2340, 246, 1, 2340, 3, 3, 4, 247, 937,
    2341, 246, 1, 2341, 3, 3, 4, 247, 1391, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1933, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2199, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2342, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    679, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2038, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2343, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2344, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2345, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2346, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2347, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2348, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2349, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2350, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2351, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2352, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2353, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 536, 1, 2, 1, 1, 4, 3, 1, 4, 5, 34, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 75, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2354, 1, 2, 1, 1, 4, 3, 1, 4, 5, 266, 2355, 246, 1, 2355, 3,
    3, 4, 247, 461, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2356, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1352, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 0, 2630, 1, 2, 1, 1, 4, 3, 1, 4, 5, 466, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1070, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2631, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2632, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2633, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2634, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 470, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1209, 1, 2, 1, 1, 4, 3, 1, 4, 5, 35, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 76, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    537, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2636, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2637, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    0, 1, 2, 1, 2638, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2639, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2640, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2641, 1, 2, 1, 1, 4, 3, 1, 4, 5, 638, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2642, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2643, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2644, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2645, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2646, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1440, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 601, 1, 2, 1, 1, 4, 3, 1, 4, 5, 538, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 247, 246, 9, 3, 3, 4, 247, 2647, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 444, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1321, 1, 2, 1, 1, 4, 3, 1, 4, 5, 720, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 648, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2648,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2649, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2650, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1, 2,
    1, 0, 1, 0, 1, 2, 1, 0, 0, 4294967295, 1, 0, 1, 0, 1, 2,
    1, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1, 1, 0, 1, 2, 1,
    0, 0, 0, 0, 1, 2, 1, 1, 2, 1, 0, 36, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2656, 1, 2, 1, 1, 4, 3, 1, 4, 5, 77,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1803, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 37, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2657, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 4294967295, 4294967295, 0, 0, 4294967295, 4294967295, 0, 2661,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2662, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1270, 1, 2, 1, 1, 4, 3, 1, 4, 5, 38, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 78, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2203, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2663, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 117, 1, 2, 1, 1, 4, 3, 1, 4, 5, 397,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 118, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 2263, 1, 2, 1, 1, 4, 3, 1, 4, 5, 662,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 663, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 382, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1210, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2032, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2666, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1724, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1682, 2667, 246, 1, 2667, 3, 3, 4, 247, 308, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 0, 4294967295, 4294967295, 0, 1071, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 730, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 309, 1, 2, 1, 1, 4, 3, 1, 4, 5, 310, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 311, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1520, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1521, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 93, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2669, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2285, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2670, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1826, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2671, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 0, 0, 0, 2681, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2682, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2683, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 383, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2684, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2664, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2685, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2686, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2687, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2688, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 119, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2689, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2690, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2691, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2692, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2696, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1072, 1, 2, 1, 1, 4, 3, 1, 4, 5, 288,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 288, 1717, 1718, 1, 1717, 5, 3,
    1, 1607, 4, 5, 120, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 0, 1073, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2698, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1, 0, 1, 2, 1, 39, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 333, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2210,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1766, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2680, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 2704, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 2705, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2706, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1783, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 433, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 1671, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1671, 2074, 1718, 1, 2074,
    5, 3, 1, 1607, 4, 5, 1029, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2708, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1737, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 0, 1738, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1,
    2, 1, 0, 0, 0, 0, 0, 0, 0, 4294967295, 1, 0, 0, 4294967295, 1, 0,
    509, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2717, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2718, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2719, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1, 4294967295, 4294967295, 0, 940, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 724, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    556, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2720, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 618, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2721, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 121, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 0, 1, 0, 0, 2722, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1184, 1, 2, 1, 1, 4, 3, 1, 4, 5, 122, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 123, 1, 2, 1, 1, 4, 3, 1, 4, 5, 124, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 127, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 2723, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2724, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 2725, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2726, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2727, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 410, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1099, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1100, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 1101, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2728, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 1102, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1739, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 366, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 353, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    1231, 1, 2, 1, 1, 4, 3, 1, 4, 5, 711, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2734, 1, 2, 1, 1, 4, 3, 1, 4, 5, 0, 1,
    0, 1, 2, 1, 1, 0, 696, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    2735, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2736, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1445, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1446, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 1423, 1, 2, 1, 1, 4, 3, 1,
    4, 5, 312, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1429, 1, 2, 1,
    1, 4, 3, 1, 4, 5, 747, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    0, 1142, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2737, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 1874, 1, 2, 1, 1, 4, 3, 1, 4, 5, 40,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 79, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 539, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2738, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 1689, 2739, 246, 1, 2739, 3, 3, 4, 247,
    41, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1528, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 1529, 1, 2, 1, 1, 4, 3, 1, 4, 5, 236, 2740,
    1, 889, 2741, 1, 253, 2741, 1, 253, 2740, 1, 891, 1, 2, 1, 1, 4,
    3, 1, 4, 5, 2742, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1, 2,
    1, 0, 0, 1024, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2743, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2744, 2745, 12, 3, 3, 4, 2744, 2746, 1,
    2, 1, 1, 4, 3, 1, 4, 5, 0, 0, 4294967295, 4294967295, 0, 1247, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 0, 0, 236, 2755, 1, 1, 0, 253, 2755,
    1, 236, 2755, 1, 1, 0, 253, 2755, 1, 284, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1, 2, 1, 2171, 1, 2, 1, 1, 4, 3, 1, 4, 5,
    505, 1, 2, 1, 1, 4, 3, 1, 4, 5, 5, 2153, 9, 3, 3, 4,
    5, 2675, 1, 2, 1, 1, 4, 3, 1, 4, 5, 61, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 125, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1276,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1643, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2761, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2762, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2763, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2764, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2765, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2766, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2767,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2768, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2769, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2770, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2771, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2772, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2773, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2774, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2775,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 2776, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 2777, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2778, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 2779, 1, 2, 1, 1, 4, 3, 1, 4,
    5, 2780, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2781, 1, 2, 1, 1,
    4, 3, 1, 4, 5, 2782, 1, 2, 1, 1, 4, 3, 1, 4, 5, 1644,
    1, 2, 1, 1, 4, 3, 1, 4, 5, 1532, 1, 2, 1, 1, 4, 3,
    1, 4, 5, 1260, 1, 2, 1, 1, 4, 3, 1, 4, 5, 2327, 1, 2,
    1, 1, 4, 3, 1, 4, 5, 664, 1, 2, 1, 1, 4, 3, 1, 4,
    5,
];

pub static AUX_DATA_OFFSETS: &[u32] = &[
    0, 10, 20, 30, 40, 50, 60, 70, 86, 13, 0, 82, 0, 42, 43, 45,
    46, 47, 48, 49, 50, 51, 52, 53, 54, 0, 55, 80, 58, 0, 59, 60,
    0, 62, 63, 65, 46, 0, 67, 63, 65, 46, 68, 0, 62, 70, 65, 50,
    0, 67, 70, 65, 81, 46, 50, 84, 83, 81, 0, 83, 85, 88, 90, 92,
    94, 0, 97, 98, 100, 92, 0, 126, 87, 52, 128, 45, 46, 48, 0, 129,
    130, 131, 0, 133, 134, 0, 135, 136, 0, 43, 88, 139, 140, 0, 91, 0,
    126, 89, 50, 0, 130, 131, 142, 68, 0, 143, 90, 50, 51, 0, 144, 145,
    129, 147, 126, 148, 133, 149, 150, 43, 151, 152, 153, 154, 155, 156, 157, 158,
    159, 160, 161, 140, 162, 163, 0, 62, 145, 164, 93, 165, 167, 95, 169, 170,
    171, 172, 173, 174, 175, 176, 134, 50, 0, 130, 131, 0, 177, 170, 0, 178,
    169, 0, 179, 180, 96, 182, 183, 184, 185, 186, 187, 188, 189, 190, 63, 0,
    130, 191, 0, 177, 183, 0, 178, 182, 0, 179, 192, 97, 195, 197, 110, 0,
    200, 98, 81, 0, 128, 100, 201, 0, 52, 102, 203, 0, 206, 104, 81, 106,
    203, 108, 207, 209, 111, 121, 214, 216, 125, 135, 145, 155, 165, 175, 185, 0,
    225, 226, 225, 195, 205, 215, 225, 235, 233, 0, 236, 239, 271, 0, 242, 263,
    243, 245, 254, 0, 242, 268, 266, 0, 250, 251, 253, 0, 255, 257, 272, 282,
    292, 302, 0, 263, 264, 81, 313, 312, 85, 0, 265, 317, 0, 270, 92, 83,
    0, 271, 92, 316, 272, 274, 276, 203, 0, 278, 50, 279, 0, 263, 46, 281,
    327, 0, 283, 318, 0, 285, 286, 0, 290, 291, 100, 0, 293, 286, 0, 270,
    92, 281, 0, 271, 92, 322, 81, 272, 274, 276, 0, 278, 53, 294, 0, 278,
    250, 325, 0, 323, 92, 297, 324, 299, 301, 321, 322, 0, 94, 325, 0, 328,
    326, 62, 250, 0, 329, 330, 167, 332, 92, 328, 335, 283, 0, 283, 329, 337,
    333, 0, 62, 145, 331, 330, 126, 148, 133, 149, 155, 156, 159, 160, 161, 140,
    167, 0, 339, 0, 329, 337, 340, 330, 167, 332, 94, 334, 335, 342, 344, 346,
    336, 182, 185, 187, 188, 347, 63, 0, 186, 0, 263, 299, 145, 0, 340, 337,
    52, 128, 338, 81, 65, 329, 339, 0, 65, 50, 340, 279, 329, 339, 342, 348,
    350, 343, 0, 200, 203, 209, 0, 263, 47, 81, 348, 0, 278, 299, 354, 0,
    356, 299, 0, 344, 278, 356, 357, 359, 0, 360, 0, 362, 81, 345, 0, 357,
    359, 0, 364, 354, 0, 357, 365, 0, 356, 362, 367, 81, 346, 342, 344, 0,
    270, 362, 81, 92, 81, 0, 369, 362, 81, 0, 271, 92, 347, 81, 272, 274,
    276, 0, 270, 92, 81, 0, 278, 50, 81, 0, 263, 370, 372, 372, 0, 161,
    0, 92, 301, 0, 278, 50, 349, 279, 374, 0, 350, 360, 356, 371, 0, 352,
    0, 384, 385, 387, 388, 47, 0, 389, 159, 0, 55, 59, 60, 0, 55, 169,
    0, 390, 49, 50, 175, 58, 0, 391, 392, 351, 398, 399, 400, 401, 54, 81,
    0, 353, 0, 177, 170, 169, 170, 255, 0, 264, 81, 354, 0, 0, 0, 402,
    357, 0, 81, 403, 81, 0, 360, 172, 173, 174, 361, 0, 83, 85, 0, 404,
    370, 0, 81, 366, 364, 363, 251, 90, 251, 0, 369, 407, 408, 81, 409, 411,
    0, 413, 340, 54, 250, 410, 0, 360, 423, 425, 426, 251, 0, 429, 430, 373,
    434, 0, 356, 423, 367, 426, 346, 0, 263, 423, 426, 392, 0, 0, 278, 59,
    54, 251, 0, 429, 430, 0, 409, 377, 354, 0, 278, 299, 379, 378, 301, 207,
    437, 354, 0, 356, 299, 0, 380, 278, 356, 357, 359, 0, 356, 445, 367, 0,
    360, 447, 381, 451, 0, 452, 299, 403, 453, 386, 382, 462, 0, 390, 0, 278,
    447, 389, 451, 0, 452, 299, 0, 278, 462, 0, 464, 0, 283, 100, 0, 356,
    453, 367, 0, 278, 465, 100, 0, 278, 467, 0, 328, 467, 0, 278, 469, 391,
    474, 178, 475, 476, 0, 278, 477, 250, 0, 360, 479, 0, 360, 481, 0, 263,
    362, 250, 393, 483, 0, 357, 483, 394, 0, 485, 486, 0, 485, 488, 0, 369,
    362, 250, 0, 356, 362, 367, 250, 346, 0, 395, 490, 491, 159, 396, 250, 62,
    128, 0, 490, 397, 492, 451, 159, 250, 0, 398, 490, 491, 0, 490, 492, 0,
    490, 493, 493, 250, 0, 399, 491, 490, 495, 0, 496, 495, 497, 499, 501, 0,
    503, 167, 250, 0, 278, 504, 400, 503, 507, 0, 278, 53, 294, 0, 278, 401,
    510, 511, 0, 278, 467, 0, 328, 467, 0, 496, 512, 255, 0, 513, 250, 0,
    357, 513, 0, 271, 92, 402, 250, 272, 274, 276, 0, 492, 453, 403, 299, 0,
    409, 516, 518, 520, 521, 0, 0, 263, 424, 426, 522, 407, 405, 0, 404, 360,
    278, 356, 242, 523, 0, 263, 55, 169, 0, 367, 524, 255, 265, 413, 411, 526,
    0, 270, 423, 426, 92, 426, 0, 270, 92, 255, 0, 422, 414, 0, 540, 167,
    359, 541, 0, 543, 359, 167, 359, 541, 544, 415, 545, 546, 547, 0, 357, 548,
    0, 178, 501, 416, 551, 552, 0, 553, 417, 557, 558, 559, 0, 560, 100, 0,
    561, 100, 0, 562, 250, 0, 278, 250, 420, 0, 418, 92, 297, 419, 301, 321,
    299, 0, 94, 325, 0, 328, 421, 62, 250, 342, 0, 263, 172, 423, 0, 83,
    85, 427, 564, 0, 270, 92, 83, 0, 278, 50, 279, 0, 271, 92, 426, 425,
    250, 322, 272, 274, 276, 0, 496, 566, 167, 370, 372, 0, 263, 200, 81, 431,
    0, 428, 360, 356, 128, 429, 81, 0, 270, 92, 81, 0, 263, 516, 445, 0,
    367, 518, 442, 0, 270, 92, 518, 0, 278, 53, 294, 0, 568, 569, 518, 0,
    496, 566, 167, 370, 372, 0, 278, 434, 432, 0, 250, 0, 297, 301, 0, 328,
    439, 437, 250, 0, 263, 423, 100, 278, 465, 100, 0, 62, 65, 53, 294, 0,
    571, 452, 443, 203, 278, 53, 294, 572, 446, 479, 0, 263, 423, 426, 467, 0,
    278, 465, 100, 0, 278, 458, 456, 0, 574, 297, 301, 0, 328, 463, 461, 574,
    0, 278, 469, 466, 474, 178, 475, 476, 0, 278, 477, 250, 0, 503, 167, 250,
    0, 278, 504, 503, 0, 278, 470, 468, 0, 574, 0, 297, 301, 0, 328, 475,
    473, 574, 0, 271, 92, 478, 81, 272, 274, 276, 0, 263, 177, 170, 485, 0,
    367, 255, 482, 480, 575, 0, 270, 0, 423, 426, 92, 426, 0, 270, 92, 255,
    0, 278, 50, 279, 0, 255, 496, 566, 167, 370, 372, 0, 62, 65, 53, 294,
    0, 571, 452, 483, 81, 278, 53, 294, 0, 572, 0, 263, 131, 81, 167, 255,
    499, 0, 270, 92, 81, 495, 0, 92, 487, 486, 203, 209, 272, 274, 276, 0,
    409, 489, 354, 0, 283, 364, 492, 354, 0, 505, 0, 178, 501, 500, 551, 552,
    0, 278, 250, 503, 0, 501, 92, 297, 502, 301, 321, 299, 0, 94, 325, 0,
    328, 504, 62, 250, 342, 0, 263, 173, 83, 577, 508, 0, 270, 92, 83, 0,
    271, 92, 507, 506, 250, 322, 274, 276, 0, 263, 579, 581, 516, 0, 278, 583,
    0, 360, 509, 585, 0, 278, 511, 585, 0, 356, 513, 585, 0, 270, 92, 581,
    0, 271, 92, 515, 272, 274, 276, 203, 0, 263, 517, 52, 128, 540, 0, 518,
    272, 274, 276, 203, 283, 527, 519, 602, 603, 520, 604, 605, 521, 606, 607, 522,
    512, 608, 523, 609, 610, 524, 611, 612, 525, 613, 614, 0, 290, 291, 100, 0,
    615, 616, 526, 523, 359, 0, 365, 617, 523, 0, 203, 0, 270, 92, 203, 0,
    531, 272, 274, 276, 62, 203, 0, 65, 46, 281, 539, 0, 278, 250, 537, 0,
    532, 92, 297, 536, 299, 534, 533, 301, 321, 619, 0, 94, 325, 0, 328, 538,
    250, 62, 0, 263, 50, 279, 541, 0, 270, 92, 279, 0, 271, 92, 203, 621,
    545, 0, 403, 251, 0, 622, 452, 623, 100, 542, 0, 624, 623, 100, 0, 496,
    624, 543, 0, 625, 623, 100, 0, 496, 625, 0, 175, 626, 628, 0, 630, 283,
    623, 100, 0, 464, 283, 623, 100, 0, 631, 100, 0, 496, 632, 0, 571, 452,
    544, 633, 426, 0, 263, 175, 367, 628, 551, 546, 462, 0, 271, 92, 550, 250,
    274, 276, 0, 270, 92, 628, 0, 278, 50, 279, 0, 263, 465, 367, 635, 555,
    0, 271, 92, 553, 552, 250, 322, 272, 274, 276, 0, 270, 92, 635, 0, 278,
    554, 0, 50, 279, 0, 465, 100, 0, 263, 637, 639, 563, 0, 290, 523, 0,
    556, 278, 360, 356, 579, 557, 581, 0, 283, 583, 0, 643, 579, 0, 283, 583,
    490, 491, 0, 278, 583, 0, 644, 559, 250, 645, 354, 0, 271, 92, 562, 81,
    272, 274, 646, 0, 270, 92, 639, 0, 263, 647, 570, 0, 278, 250, 564, 92,
    297, 568, 299, 566, 565, 301, 321, 322, 619, 0, 328, 569, 62, 250, 590, 0,
    360, 479, 0, 357, 365, 0, 369, 362, 250, 0, 356, 362, 250, 346, 0, 271,
    92, 572, 571, 250, 322, 0, 270, 92, 574, 573, 250, 322, 0, 270, 423, 92,
    575, 426, 0, 263, 423, 426, 578, 0, 54, 251, 0, 651, 576, 665, 0, 263,
    666, 426, 0, 263, 668, 579, 62, 673, 0, 674, 666, 426, 675, 0, 263, 676,
    678, 582, 580, 426, 0, 263, 678, 585, 675, 633, 0, 263, 651, 675, 0, 360,
    423, 426, 251, 0, 299, 354, 0, 651, 665, 0, 429, 430, 586, 434, 0, 356,
    423, 426, 346, 0, 680, 94, 255, 681, 683, 0, 278, 685, 523, 0, 356, 423,
    367, 242, 596, 0, 250, 297, 591, 301, 321, 0, 592, 360, 150, 423, 687, 594,
    0, 593, 690, 691, 426, 164, 692, 0, 270, 595, 403, 92, 255, 0, 263, 169,
    604, 0, 367, 524, 255, 265, 599, 597, 693, 0, 270, 423, 426, 92, 426, 0,
    270, 362, 250, 92, 250, 0, 367, 255, 601, 0, 270, 92, 255, 0, 278, 50,
    279, 0, 568, 569, 250, 600, 0, 340, 695, 697, 299, 0, 699, 569, 250, 700,
    702, 0, 62, 65, 53, 294, 0, 571, 452, 602, 81, 278, 53, 294, 572, 0,
    263, 391, 392, 398, 81, 618, 0, 360, 704, 340, 605, 81, 283, 607, 81, 0,
    263, 704, 340, 609, 81, 283, 611, 81, 0, 263, 704, 706, 81, 283, 81, 0,
    263, 704, 340, 613, 81, 706, 81, 283, 81, 0, 356, 704, 367, 340, 615, 81,
    0, 270, 92, 81, 0, 271, 92, 617, 81, 272, 274, 276, 0, 278, 50, 81,
    0, 263, 159, 708, 167, 255, 619, 0, 270, 92, 708, 0, 496, 566, 167, 370,
    372, 0, 263, 54, 251, 625, 0, 271, 92, 620, 274, 276, 272, 81, 0, 270,
    710, 523, 92, 523, 0, 270, 92, 81, 0, 278, 50, 279, 622, 621, 0, 360,
    712, 426, 251, 0, 429, 430, 714, 0, 263, 712, 426, 0, 278, 59, 54, 251,
    0, 429, 430, 714, 0, 356, 712, 367, 426, 714, 0, 270, 712, 426, 92, 426,
    714, 0, 360, 710, 425, 523, 0, 624, 716, 691, 523, 0, 263, 128, 203, 283,
    634, 626, 604, 605, 627, 721, 722, 0, 723, 384, 628, 558, 725, 0, 616, 629,
    523, 491, 0, 365, 617, 523, 0, 270, 92, 81, 726, 630, 0, 728, 729, 100,
    0, 328, 728, 729, 633, 0, 278, 81, 631, 92, 297, 632, 299, 301, 0, 328,
    81, 0, 263, 170, 367, 255, 651, 0, 263, 423, 426, 639, 0, 278, 299, 638,
    301, 207, 437, 354, 0, 356, 299, 0, 271, 92, 640, 81, 272, 274, 276, 0,
    270, 423, 426, 92, 426, 0, 270, 92, 255, 0, 278, 50, 279, 0, 278, 643,
    641, 0, 574, 0, 297, 301, 0, 328, 648, 646, 574, 0, 263, 731, 81, 664,
    0, 652, 360, 734, 662, 0, 169, 653, 255, 0, 50, 655, 279, 0, 172, 657,
    83, 0, 62, 659, 182, 185, 65, 50, 660, 279, 0, 278, 663, 0, 737, 297,
    286, 0, 738, 297, 286, 340, 50, 279, 665, 675, 684, 0, 694, 743, 744, 477,
    0, 255, 697, 695, 426, 0, 746, 100, 700, 0, 710, 743, 744, 717, 714, 712,
    0, 711, 748, 749, 286, 748, 723, 0, 255, 720, 718, 426, 725, 735, 744, 754,
    764, 774, 784, 794, 804, 65, 759, 760, 805, 766, 225, 767, 768, 769, 770, 771,
    772, 773, 774, 0, 94, 775, 523, 806, 0, 283, 59, 51, 50, 523, 0, 283,
    496, 59, 51, 50, 807, 0, 839, 836, 817, 100, 354, 818, 827, 0, 843, 855,
    853, 780, 781, 858, 354, 0, 81, 403, 242, 782, 242, 861, 871, 0, 239, 242,
    0, 403, 881, 891, 901, 911, 921, 931, 941, 951, 961, 971, 981, 991, 1001, 1011,
    1021, 1031, 1043, 1042, 0, 802, 1041, 299, 523, 0, 283, 803, 403, 523, 0, 804,
    523, 805, 523, 0, 808, 523, 0, 809, 523, 0, 810, 523, 0, 812, 1047, 1057,
    1067, 1077, 1087, 1097, 1107, 1126, 1135, 1146, 1145, 301, 437, 820, 207, 426, 354, 0,
    251, 301, 90, 403, 823, 1147, 1153, 1158, 1168, 1178, 1188, 1198, 1208, 1218, 1228, 1238,
    843, 844, 845, 846, 847, 1239, 1246, 1256, 1266, 1276, 852, 0, 853, 853, 226, 0,
    854, 854, 1286, 1295, 1305, 1315, 1325, 1335, 1345, 860, 861, 1346, 1356, 863, 864, 1357,
    1367, 1377, 1390, 1388, 1387, 301, 868, 1395, 1393, 426, 1398, 1404, 871, 1407, 1417, 1427,
    1437, 1447, 1457, 1467, 1477, 243, 0, 242, 877, 403, 0, 357, 177, 1480, 879, 1484,
    0, 392, 1483, 881, 882, 690, 452, 426, 278, 426, 0, 632, 426, 278, 426, 409,
    426, 1485, 1495, 0, 884, 207, 0, 884, 207, 1505, 1515, 1525, 1535, 344, 342, 1536,
    1554, 0, 888, 889, 1546, 892, 894, 896, 898, 896, 896, 0, 888, 354, 1550, 892,
    894, 896, 898, 896, 896, 846, 90, 900, 1555, 1565, 1575, 1585, 1595, 1604, 1614, 1624,
    1634, 1644, 1654, 1664, 0, 1674, 1684, 1694, 1704, 1714, 1724, 1734, 1744, 1754, 1764, 1774,
    1784, 1794, 1804, 1814, 1824, 1834, 1844, 0, 927, 81, 1854, 0, 503, 748, 1864, 0,
    255, 0, 409, 518, 0, 518, 167, 255, 1865, 1875, 1885, 0, 1891, 0, 429, 1901,
    931, 932, 933, 934, 1902, 1914, 523, 1912, 242, 935, 936, 1915, 1925, 1935, 1945, 1955,
    1964, 1973, 1993, 0, 299, 354, 0, 453, 1984, 1982, 100, 0, 447, 452, 299, 403,
    453, 1989, 1987, 100, 0, 651, 665, 666, 678, 0, 429, 1992, 931, 933, 1997, 948,
    301, 207, 437, 2003, 1998, 0, 357, 359, 359, 1999, 950, 951, 0, 155, 255, 2000,
    426, 0, 242, 251, 2007, 434, 0, 429, 430, 2011, 952, 953, 954, 955, 956, 957,
    958, 959, 960, 961, 962, 963, 964, 0, 2012, 0, 242, 0, 2022, 965, 0, 965,
    965, 2024, 966, 2023, 965, 0, 965, 965, 167, 403, 409, 90, 967, 968, 969, 900,
    970, 971, 972, 0, 426, 769, 54, 2026, 0, 974, 0, 0, 2036, 2046, 2055, 2065,
    0, 980, 523, 2075, 0, 980, 167, 0, 2094, 0, 2085, 169, 170, 255, 0, 402,
    2086, 0, 81, 403, 81, 0, 423, 426, 0, 362, 81, 0, 167, 47, 81, 0,
    46, 281, 0, 370, 372, 0, 172, 83, 0, 85, 0, 516, 518, 0, 50, 279,
    0, 2089, 387, 388, 47, 49, 131, 579, 52, 493, 175, 58, 465, 637, 53, 159,
    54, 0, 384, 385, 0, 389, 159, 0, 55, 59, 60, 0, 55, 169, 0, 177,
    170, 0, 391, 392, 398, 0, 391, 392, 399, 0, 391, 392, 400, 0, 391, 392,
    401, 81, 0, 167, 81, 0, 2090, 264, 173, 174, 81, 2091, 0, 0, 0, 981,
    2095, 523, 359, 2096, 2103, 2113, 2123, 2133, 297, 987, 988, 989, 990, 991, 992, 0,
    981, 993, 94, 0, 981, 357, 993, 94, 995, 996, 997, 998, 999, 1000, 1001, 1002,
    2134, 2141, 2147, 0, 2153, 354, 965, 265, 2156, 2166, 2176, 1009, 1010, 1011, 964, 2177,
    2187, 0, 226, 226, 2197, 2207, 2217, 0, 2227, 1015, 1019, 1020, 2228, 2238, 0, 1023,
    2271, 521, 0, 161, 278, 2269, 2268, 0, 426, 297, 2248, 354, 299, 0, 2251, 2249,
    426, 297, 497, 2257, 2255, 2254, 354, 299, 0, 2262, 2260, 426, 297, 2265, 879, 0,
    1025, 354, 2272, 2291, 0, 2288, 2286, 0, 2285, 426, 2282, 354, 207, 0, 429, 430,
    1027, 0, 1025, 354, 0, 167, 362, 250, 2292, 2302, 0, 290, 291, 2312, 100, 1030,
    2313, 2323, 2333, 2343, 2353, 2363, 2373, 2383, 2393, 2403, 2413, 2423, 2433, 0, 2443, 251,
    90, 354, 0, 1043, 2446, 2456, 0, 1045, 255, 164, 94, 523, 2472, 1074, 0, 1076,
    403, 523, 1077, 1078, 2466, 0, 1080, 403, 523, 0, 1081, 403, 523, 0, 615, 675,
    1082, 0, 746, 100, 0, 1083, 403, 100, 0, 1084, 2467, 1087, 167, 1088, 1089, 1090,
    0, 1091, 2468, 167, 1088, 1089, 1090, 1092, 1093, 0, 1094, 403, 523, 1095, 0, 1096,
    403, 2469, 675, 523, 1097, 0, 1098, 403, 2470, 1103, 1104, 1105, 1106, 1107, 1108, 1109,
    1110, 0, 1111, 403, 301, 0, 359, 403, 523, 0, 1112, 100, 1113, 1114, 0, 1115,
    403, 2471, 675, 1118, 1119, 523, 1120, 1121, 2476, 2486, 2496, 2506, 2516, 2526, 2536, 2546,
    2556, 0, 126, 1130, 264, 81, 2566, 0, 85, 0, 126, 402, 2569, 251, 403, 251,
    2572, 0, 283, 172, 83, 85, 0, 504, 172, 0, 283, 760, 2573, 0, 403, 1131,
    0, 403, 1132, 0, 126, 387, 425, 81, 2581, 2578, 2576, 2575, 0, 1134, 297, 523,
    0, 1135, 297, 523, 0, 1136, 297, 523, 0, 1137, 297, 2574, 1138, 1139, 0, 1141,
    297, 286, 0, 1143, 297, 523, 0, 94, 81, 0, 126, 46, 281, 2582, 1144, 0,
    283, 1146, 1149, 1151, 1153, 2586, 0, 271, 297, 203, 726, 1155, 1157, 0, 126, 731,
    81, 0, 278, 737, 297, 2590, 1089, 1090, 0, 126, 47, 81, 0, 403, 251, 0,
    429, 430, 0, 299, 354, 2592, 0, 0, 362, 81, 2591, 0, 357, 359, 0, 364,
    354, 0, 126, 370, 425, 372, 283, 0, 50, 279, 0, 1143, 1159, 0, 94, 1159,
    344, 0, 126, 1130, 178, 172, 83, 2598, 2596, 251, 543, 251, 2601, 547, 546, 545,
    2602, 1163, 1164, 523, 802, 2603, 299, 523, 0, 1166, 100, 0, 126, 178, 50, 425,
    279, 94, 2604, 0, 59, 51, 0, 1174, 1175, 1177, 1178, 1179, 1180, 1181, 1182, 2608,
    0, 46, 523, 0, 1076, 523, 0, 50, 523, 0, 1185, 523, 0, 1187, 100, 0,
    802, 2605, 299, 523, 0, 1188, 2606, 633, 1191, 1192, 2607, 0, 1193, 523, 0, 1194,
    523, 0, 1195, 523, 0, 126, 178, 46, 1196, 357, 1197, 0, 126, 178, 169, 255,
    1199, 0, 497, 180, 1202, 1204, 1208, 403, 2612, 1211, 1212, 685, 523, 0, 169, 1213,
    2615, 2613, 0, 523, 297, 523, 403, 2621, 2618, 879, 0, 126, 178, 169, 255, 2624,
    2622, 0, 426, 251, 1199, 0, 497, 180, 2632, 0, 1202, 1204, 0, 1216, 523, 0,
    283, 1217, 2629, 2627, 0, 523, 297, 523, 1208, 403, 2633, 1211, 1220, 1221, 1212, 1222,
    1223, 0, 1224, 523, 1225, 523, 685, 523, 0, 169, 1213, 2636, 2634, 0, 523, 297,
    523, 0, 126, 1227, 242, 1228, 242, 1230, 242, 0, 126, 55, 169, 2665, 0, 1232,
    425, 255, 2646, 2644, 2643, 0, 426, 251, 1233, 0, 429, 430, 2639, 1234, 1235, 0,
    1237, 2651, 2649, 255, 0, 58, 1238, 1233, 0, 1232, 425, 255, 0, 569, 513, 255,
    2661, 2659, 2658, 0, 426, 0, 283, 522, 2654, 1234, 1235, 2664, 0, 340, 695, 697,
    299, 0, 58, 1238, 1233, 0, 126, 1130, 1240, 172, 425, 83, 85, 0, 543, 2673,
    0, 169, 2670, 2668, 2667, 251, 0, 426, 2666, 251, 1241, 0, 1244, 251, 251, 1245,
    0, 126, 200, 81, 0, 283, 128, 2674, 81, 0, 126, 950, 516, 700, 0, 425,
    518, 167, 524, 255, 0, 409, 411, 2678, 2676, 1249, 0, 1252, 2683, 2681, 1249, 0,
    1254, 357, 993, 0, 283, 467, 0, 53, 294, 0, 1025, 354, 2686, 0, 126, 1130,
    1255, 81, 49, 1257, 94, 523, 2696, 1074, 0, 1076, 403, 523, 0, 126, 177, 170,
    255, 0, 1259, 2700, 1261, 496, 1262, 0, 675, 643, 2701, 1261, 496, 403, 2714, 2705,
    2702, 879, 2709, 2706, 0, 169, 255, 1264, 2713, 2710, 0, 133, 207, 1266, 0, 126,
    1268, 81, 0, 94, 2718, 523, 2715, 879, 81, 0, 1271, 426, 0, 172, 81, 2721,
    2719, 251, 0, 543, 251, 0, 1164, 523, 0, 802, 2724, 299, 523, 0, 675, 2725,
    167, 1088, 0, 1273, 2726, 1277, 1278, 1279, 0, 1281, 2727, 1284, 1285, 1286, 0, 1288,
    523, 0, 1290, 523, 0, 1292, 299, 0, 1294, 2730, 2728, 0, 0, 1296, 2734, 0,
    1298, 523, 0, 809, 523, 0, 1300, 2733, 167, 1088, 0, 1302, 100, 0, 1304, 100,
    0, 126, 404, 2747, 2737, 2785, 2783, 2782, 0, 2751, 1306, 1307, 297, 81, 0, 1308,
    297, 2762, 2752, 0, 1309, 297, 2776, 2766, 0, 2781, 342, 1312, 2780, 173, 172, 297,
    83, 1313, 1314, 0, 126, 131, 81, 167, 255, 0, 403, 2788, 1315, 1316, 0, 340,
    2789, 62, 156, 149, 161, 148, 0, 92, 2791, 2790, 81, 209, 272, 274, 276, 0,
    409, 2793, 354, 0, 283, 364, 2796, 354, 0, 126, 1130, 173, 83, 577, 1317, 1245,
    0, 126, 579, 581, 2801, 0, 340, 62, 182, 0, 340, 2799, 585, 0, 283, 583,
    0, 126, 1319, 131, 81, 0, 283, 2804, 2802, 0, 426, 251, 0, 403, 257, 0,
    409, 2807, 354, 0, 126, 2810, 52, 128, 203, 0, 283, 2819, 2811, 602, 603, 2812,
    604, 605, 2813, 606, 607, 2814, 512, 608, 2815, 609, 610, 2816, 611, 612, 2817, 613,
    614, 0, 290, 291, 100, 0, 615, 616, 2818, 523, 359, 0, 365, 617, 523, 0,
    65, 52, 203, 0, 65, 200, 203, 0, 52, 203, 0, 1322, 203, 0, 128, 203,
    0, 1323, 100, 0, 126, 50, 2823, 0, 425, 279, 0, 1324, 203, 0, 1324, 203,
    621, 2828, 0, 403, 251, 0, 622, 452, 623, 100, 2824, 0, 624, 623, 100, 0,
    496, 624, 2825, 0, 625, 623, 100, 0, 496, 625, 0, 630, 283, 623, 100, 0,
    631, 100, 2826, 632, 0, 496, 632, 0, 571, 452, 2827, 633, 426, 0, 126, 1240,
    175, 425, 628, 2829, 1325, 0, 126, 58, 425, 1238, 0, 54, 523, 0, 1143, 1159,
    1227, 81, 1233, 0, 126, 465, 0, 425, 635, 2836, 2834, 2833, 1327, 1328, 1329, 1330,
    167, 2840, 2839, 426, 354, 94, 255, 0, 126, 637, 639, 290, 523, 579, 2842, 581,
    0, 283, 583, 0, 126, 0, 1332, 2844, 158, 157, 169, 81, 2847, 2845, 426, 0,
    1259, 2850, 1261, 496, 1262, 403, 2854, 2851, 879, 0, 126, 1332, 1240, 169, 425, 255,
    2866, 2864, 2863, 0, 426, 251, 2855, 1333, 434, 479, 0, 1336, 255, 2859, 1339, 0,
    1259, 2869, 1261, 496, 2870, 1262, 0, 126, 391, 392, 398, 81, 2875, 2874, 0, 400,
    297, 81, 0, 1045, 297, 81, 0, 126, 1130, 362, 159, 708, 2878, 716, 691, 0,
    1342, 513, 2882, 2881, 149, 148, 160, 0, 161, 0, 513, 2879, 426, 1343, 167, 1343,
    167, 255, 2892, 0, 94, 255, 2885, 0, 357, 483, 0, 483, 2884, 0, 485, 488,
    0, 485, 486, 0, 1345, 2886, 1348, 1349, 169, 403, 255, 0, 2887, 1348, 1349, 169,
    403, 255, 0, 340, 1350, 2888, 497, 1353, 0, 1354, 2889, 354, 0, 133, 2896, 172,
    173, 207, 0, 126, 54, 251, 0, 403, 2897, 1357, 1358, 2900, 2898, 0, 0, 126,
    128, 704, 0, 1359, 425, 340, 2903, 209, 242, 1360, 58, 1238, 1233, 0, 126, 128,
    203, 283, 616, 2904, 523, 491, 2910, 2905, 604, 605, 2906, 721, 722, 0, 723, 384,
    2907, 558, 725, 0, 65, 200, 2908, 81, 0, 365, 617, 523, 726, 0, 728, 729,
    100, 0, 126, 1130, 170, 425, 255, 164, 403, 2917, 2914, 879, 1361, 2918, 2928, 2938,
    2948, 2958, 2968, 2978, 2988, 2996, 0, 3006, 1367, 1368, 3007, 1369, 3010, 3020, 3030, 3040,
    3050, 3060, 3070, 3080, 3090, 3100, 3110, 3120, 3130, 3140, 3150, 3160, 3170, 0, 0, 180,
    403, 3181, 0, 1379, 0, 1381, 403, 523, 0, 1382, 0, 1384, 675, 100, 0, 3180,
    1223, 1385, 0, 403, 523, 1211, 1222, 1220, 1221, 3182, 3192, 3202, 3212, 242, 935, 936,
    3214, 3224, 3234, 3253, 3244, 242, 321, 3259, 1390, 1392, 1394, 1396, 1398, 1400, 1401, 1403,
    1405, 0, 1408, 1409, 1411, 1413, 0, 3254, 1417, 1418, 1419, 3256, 0, 3255, 1424, 1425,
    1426, 1427, 0, 1425, 1428, 1430, 1419, 1431, 391, 1432, 1434, 3257, 1435, 1436, 1437, 1439,
    1441, 1443, 0, 3258, 1447, 1448, 0, 1449, 1428, 1419, 1450, 0, 242, 965, 1451, 3260,
    3274, 1456, 0, 3286, 3284, 3283, 1457, 1537, 0, 1432, 523, 3290, 1538, 1539, 1540, 3289,
    1541, 1542, 3291, 3301, 3391, 3400, 3410, 3420, 3430, 3440, 3450, 3460, 0, 1549, 1550, 81,
    3470, 3480, 3490, 3500, 0, 1554, 81, 1555, 340, 879, 3510, 0, 299, 695, 3520, 3530,
    3540, 3550, 3560, 3570, 3580, 301, 1559, 0, 1559, 0, 297, 1560, 3583, 3581, 1561, 3586,
    3596, 3606, 3616, 0, 148, 94, 524, 255, 265, 257, 0, 409, 3626, 0, 1566, 257,
    1568, 3628, 0, 1025, 325, 513, 81, 1570, 0, 1573, 3631, 265, 3629, 0, 354, 257,
    3632, 1574, 3642, 3652, 3662, 3672, 3682, 3692, 3702, 3712, 3722, 0, 1580, 731, 81, 0,
    513, 0, 13, 523, 1581, 523, 3732, 3742, 0, 1584, 321, 81, 3752, 3762, 3772, 3782,
    3792, 3802, 3812, 0, 1588, 3822, 62, 1590, 188, 158, 157, 3823, 3833, 3843, 3853, 3863,
    3873, 3882, 3892, 3902, 3912, 3922, 3932, 3942, 3952, 3962, 3972, 3982, 3992, 0, 1023, 4002,
    0, 1604, 523, 0, 523, 1604, 4003, 4013, 4024, 4034, 4043, 0, 356, 264, 367, 4054,
    0, 81, 4053, 85, 0, 265, 346, 4056, 342, 344, 0, 356, 402, 367, 4057, 251,
    403, 251, 346, 0, 356, 387, 367, 81, 346, 0, 356, 46, 367, 281, 0, 283,
    4060, 495, 0, 356, 731, 81, 0, 356, 47, 367, 4063, 81, 346, 0, 356, 370,
    367, 372, 346, 0, 356, 55, 169, 367, 4065, 255, 714, 0, 356, 172, 367, 4067,
    0, 81, 85, 346, 0, 356, 516, 700, 367, 4069, 518, 346, 4071, 0, 356, 177,
    170, 367, 4081, 255, 346, 0, 356, 1268, 367, 81, 0, 356, 571, 452, 4084, 4083,
    272, 274, 276, 203, 346, 0, 356, 131, 367, 81, 167, 255, 346, 0, 356, 173,
    367, 4086, 0, 83, 577, 0, 356, 579, 367, 4088, 581, 346, 0, 356, 1319, 131,
    367, 81, 4090, 344, 342, 0, 356, 52, 367, 242, 0, 356, 50, 367, 279, 346,
    0, 356, 175, 367, 4091, 628, 346, 0, 356, 465, 367, 4093, 635, 4095, 344, 342,
    0, 356, 637, 367, 639, 4096, 344, 342, 0, 356, 1240, 169, 367, 4097, 255, 346,
    0, 356, 391, 392, 398, 367, 81, 346, 0, 356, 159, 367, 708, 167, 255, 346,
    0, 356, 54, 367, 4099, 251, 346, 0, 356, 128, 367, 203, 0, 356, 170, 367,
    4101, 255, 346, 4103, 4113, 4123, 4133, 4143, 4153, 0, 894, 354, 4163, 4173, 4183, 0,
    1616, 1618, 4186, 4196, 4206, 4216, 4226, 4236, 4246, 4256, 4266, 4275, 4285, 4295, 4305, 4315,
    4325, 4335, 4345, 4355, 645, 4365, 4375, 4385, 4395, 4405, 4415, 4425, 0, 1633, 4440, 4437,
    4435, 242, 242, 4441, 4451, 0, 1249, 283, 846, 4461, 4471, 4481, 0, 133, 81, 4493,
    4491, 354, 4496, 4506, 4516, 4526, 4538, 0, 4536, 743, 744, 748, 1638, 1296, 1639, 1640,
    1641, 1642, 286, 0, 180, 4537, 391, 1645, 1385, 1646, 0, 1647, 4545, 0, 4539, 743,
    744, 748, 4542, 4540, 1648, 4546, 879, 1650, 1652, 1654, 4547, 4557, 4567, 1657, 0, 1658,
    4578, 4577, 1659, 0, 900, 1658, 1662, 1663, 0, 981, 357, 1664, 1672, 1674, 1676, 0,
    357, 1677, 1678, 1679, 1658, 0, 1680, 1657, 4582, 623, 0, 1683, 1684, 1685, 1686, 0,
    1687, 4585, 4584, 0, 4583, 843, 844, 846, 1687, 4589, 623, 0, 1683, 1684, 0, 1688,
    4599, 4598, 0, 1197, 4590, 879, 0, 4593, 1690, 1691, 4594, 1693, 948, 90, 0, 1695,
    948, 0, 4614, 437, 207, 4604, 4603, 1697, 354, 879, 4601, 4600, 301, 426, 207, 1698,
    354, 1700, 1701, 0, 4607, 1616, 1702, 4610, 4608, 354, 0, 251, 4613, 523, 100, 286,
    1704, 1705, 1698, 1707, 90, 1708, 4615, 1709, 301, 820, 1711, 1713, 426, 1715, 4616, 0,
    4630, 255, 0, 524, 4629, 4626, 255, 0, 255, 265, 4631, 4641, 4651, 4661, 4671, 4682,
    0, 1720, 4692, 690, 1722, 4693, 100, 354, 497, 4694, 497, 561, 4695, 524, 0, 283,
    1723, 4696, 0, 1720, 4707, 1722, 62, 0, 1726, 4706, 62, 100, 94, 81, 4708, 4718,
    4734, 1743, 1745, 1747, 4732, 1749, 4728, 971, 4738, 4748, 0, 1752, 4758, 0, 1025, 354,
    4761, 4771, 4781, 4791, 4801, 4811, 4821, 4831, 4841, 4851, 4861, 4871, 4881, 0, 340, 4891,
    161, 0, 496, 674, 161, 206, 0, 674, 206, 0, 513, 4892, 255, 4894, 572, 0,
    644, 1759, 4895, 4905, 0, 55, 59, 60, 0, 55, 674, 4915, 0, 0, 362, 81,
    4929, 0, 357, 359, 0, 364, 4925, 354, 0, 496, 512, 0, 299, 4928, 948, 301,
    207, 437, 354, 0, 447, 451, 403, 354, 1208, 0, 0, 362, 81, 0, 364, 4930,
    354, 0, 496, 512, 4933, 4943, 4953, 4963, 0, 1764, 4976, 4975, 0, 325, 497, 0,
    4973, 100, 820, 1765, 426, 4974, 1768, 1769, 0, 1677, 1679, 0, 1633, 4977, 0, 325,
    497, 200, 1723, 0, 496, 1770, 4978, 1358, 561, 1772, 4979, 4989, 4999, 0, 94, 5013,
    5012, 5009, 1776, 5015, 1025, 1784, 0, 200, 452, 0, 1786, 452, 1787, 1788, 1789, 1790,
    1361, 1266, 1720, 1792, 1793, 0, 0, 283, 1795, 94, 731, 81, 513, 0, 13, 5016,
    523, 100, 81, 1581, 5017, 523, 81, 5030, 0, 1797, 5021, 5018, 1566, 1799, 257, 5022,
    1801, 1802, 972, 1568, 0, 283, 1792, 257, 1802, 1805, 5027, 0, 5023, 1568, 5049, 5046,
    0, 5034, 1806, 1809, 5031, 1776, 0, 1786, 452, 0, 200, 452, 5035, 5039, 0, 5038,
    1568, 972, 0, 1786, 452, 0, 200, 452, 5043, 0, 94, 48, 523, 0, 46, 523,
    5050, 278, 0, 1094, 297, 286, 0, 1815, 297, 286, 0, 1816, 100, 0, 1121, 297,
    286, 0, 1817, 297, 286, 5054, 0, 1818, 5073, 0, 5067, 5064, 207, 257, 340, 426,
    65, 5070, 5068, 0, 354, 257, 0, 1819, 0, 5076, 1252, 1633, 1254, 5082, 0, 426,
    340, 426, 65, 5079, 5077, 0, 426, 257, 5085, 5095, 5105, 5115, 5127, 354, 1822, 0,
    1824, 5126, 354, 1822, 5145, 354, 5131, 5128, 871, 0, 354, 403, 251, 0, 1827, 354,
    94, 94, 354, 0, 5132, 1457, 354, 5133, 0, 94, 354, 0, 340, 354, 0, 1573,
    251, 0, 993, 5139, 265, 5137, 1828, 1829, 0, 1832, 301, 0, 5140, 523, 242, 426,
    354, 65, 5141, 523, 242, 426, 354, 1833, 1836, 1838, 5143, 0, 354, 5142, 710, 970,
    354, 0, 5149, 1537, 0, 5167, 1604, 0, 413, 340, 54, 250, 1789, 5152, 545, 546,
    547, 0, 357, 548, 5153, 0, 540, 167, 359, 541, 0, 543, 359, 167, 359, 541,
    544, 0, 178, 501, 5154, 552, 551, 0, 553, 5155, 557, 558, 559, 0, 560, 100,
    0, 561, 100, 0, 562, 250, 0, 278, 250, 5160, 0, 5156, 92, 297, 5158, 5157,
    250, 301, 0, 94, 325, 0, 403, 5161, 523, 0, 523, 90, 523, 0, 1839, 354,
    0, 1840, 1841, 5163, 0, 1650, 1574, 0, 1652, 1574, 0, 5162, 1842, 871, 1709, 1574,
    0, 1839, 354, 1574, 896, 0, 283, 5173, 5171, 250, 5176, 5186, 0, 5195, 0, 242,
    965, 870, 5199, 1844, 322, 870, 0, 5200, 65, 759, 760, 1824, 5203, 251, 0, 250,
    5201, 65, 759, 760, 1824, 5202, 251, 1241, 0, 5204, 299, 297, 1688, 354, 5207, 5205,
    1845, 5213, 0, 0, 1846, 1847, 0, 0, 5212, 83, 5210, 1846, 1264, 1849, 0, 1850,
    83, 5211, 1846, 1264, 1849, 1851, 1852, 0, 1849, 1853, 5214, 5224, 5234, 5244, 5254, 5264,
    5274, 5284, 5294, 5304, 5314, 5325, 5335, 5345, 5355, 5365, 5375, 0, 329, 5385, 0, 1865,
    167, 1867, 0, 52, 203, 0, 152, 167, 128, 201, 81, 5392, 0, 92, 5386, 1869,
    0, 92, 1871, 5388, 0, 90, 92, 1871, 5393, 0, 283, 329, 337, 0, 283, 1322,
    337, 0, 1045, 325, 136, 0, 1873, 452, 5394, 274, 276, 201, 0, 5395, 329, 339,
    140, 167, 731, 81, 5396, 92, 94, 5397, 0, 1581, 523, 0, 13, 0, 523, 0,
    1875, 59, 51, 5398, 5408, 0, 854, 645, 854, 5418, 0, 200, 452, 5432, 5431, 62,
    426, 100, 1877, 1879, 354, 5428, 0, 1786, 452, 291, 1787, 1788, 1789, 1790, 5434, 0,
    1786, 452, 1784, 1787, 1788, 1789, 1720, 5435, 0, 5449, 5448, 426, 100, 354, 5445, 1882,
    5451, 0, 1883, 1884, 5464, 5462, 5461, 1877, 1369, 5467, 5477, 5487, 5497, 0, 1787, 5510,
    5507, 354, 5511, 0, 1786, 452, 1784, 1788, 1789, 1720, 5512, 5522, 5532, 5542, 5552, 5562,
    5572, 5582, 0, 5592, 5602, 5612, 0, 802, 5622, 299, 523, 5623, 5633, 5643, 5650, 0,
    1196, 1197, 5660, 0, 1196, 357, 1197, 5670, 0, 5680, 1894, 1895, 1254, 5681, 5691, 5701,
    5711, 5721, 5731, 5741, 5751, 5761, 0, 1898, 55, 50, 0, 1899, 279, 0, 5771, 0,
    291, 92, 1294, 5774, 5772, 1900, 94, 58, 1238, 1901, 279, 1233, 5777, 5787, 5797, 0,
    357, 65, 5813, 5810, 5809, 5807, 1657, 879, 207, 5814, 5824, 5834, 5844, 5854, 242, 0,
    242, 5864, 1904, 1580, 0, 0, 429, 430, 0, 1905, 0, 1254, 5865, 690, 1907, 467,
    5866, 1904, 1580, 0, 1254, 5867, 690, 1907, 0, 5871, 426, 207, 5868, 354, 1909, 5872,
    0, 0, 1252, 164, 0, 283, 583, 0, 409, 516, 53, 294, 5882, 5892, 5902, 5912,
    5922, 5932, 5942, 5952, 5962, 5972, 5982, 5992, 6002, 0, 149, 1901, 255, 6021, 6015, 6012,
    879, 0, 299, 695, 0, 164, 6020, 1264, 6019, 6016, 879, 6022, 6032, 6042, 6052, 6062,
    6072, 6082, 6092, 6102, 6112, 6122, 6132, 6142, 0, 1434, 6154, 0, 100, 6152, 523, 1457,
    523, 0, 523, 6153, 523, 1457, 6155, 6165, 1920, 1921, 1922, 1923, 1924, 1925, 0, 1901,
    6166, 158, 157, 511, 169, 255, 6167, 6177, 6187, 1704, 1926, 1927, 286, 1928, 1929, 0,
    981, 357, 993, 94, 6188, 1930, 6198, 6208, 0, 1931, 499, 6218, 1934, 1935, 6230, 0,
    1936, 1937, 1809, 6219, 1938, 0, 6223, 1940, 6226, 1942, 1944, 6227, 0, 1945, 1937, 1809,
    1940, 0, 1946, 1809, 6231, 0, 1312, 0, 6241, 1947, 1949, 257, 0, 167, 6245, 6251,
    6248, 354, 6252, 6256, 1952, 0, 6255, 520, 1953, 1954, 1955, 0, 409, 6259, 6257, 242,
    6262, 6272, 0, 1385, 1957, 1959, 6283, 710, 1960, 781, 98, 0, 6284, 283, 504, 950,
    0, 1962, 6285, 6295, 6305, 6315, 6325, 6335, 6345, 6355, 6365, 6375, 6385, 6395, 6405, 6415,
    6425, 6435, 6445, 6453, 6463, 6473, 6483, 6493, 6503, 0, 49, 6513, 321, 1978, 6514, 6524,
    6534, 6544, 0, 354, 257, 6554, 6564, 6574, 6584, 6594, 6604, 6614, 0, 853, 645, 853,
    6624, 6634, 0, 0, 357, 1983, 1657, 0, 1107, 1658, 6644, 1336, 1984, 0, 1985, 92,
    6645, 6655, 0, 6666, 1986, 1987, 1988, 6667, 0, 291, 6681, 6678, 6677, 100, 354, 62,
    6683, 0, 1792, 6682, 100, 354, 0, 90, 100, 6684, 6694, 6704, 0, 6714, 6724, 6734,
    0, 1993, 242, 6744, 523, 100, 286, 1683, 1704, 1705, 1994, 1996, 1998, 2000, 2001, 6745,
    6754, 6761, 6771, 0, 2003, 523, 0, 6781, 6791, 6801, 6811, 6821, 6831, 0, 2007, 169,
    6841, 255, 6843, 6853, 6863, 6873, 6883, 6893, 6903, 6913, 6923, 6933, 6943, 0, 2013, 965,
    242, 6956, 0, 1268, 81, 0, 90, 6953, 879, 6959, 6969, 0, 6979, 568, 699, 1319,
    131, 81, 167, 169, 6980, 255, 6982, 92, 94, 6984, 6983, 0, 52, 203, 209, 6986,
    6996, 0, 7006, 7016, 7026, 0, 7036, 7046, 7056, 7066, 7076, 7086, 7096, 7106, 7117, 7127,
    7137, 7147, 7157, 7167, 7177, 7187, 148, 0, 149, 164, 1264, 0, 2025, 1901, 7197, 7207,
    2026, 2028, 0, 1354, 2030, 0, 1679, 354, 2031, 7211, 2033, 2035, 0, 1354, 357, 2030,
    452, 2037, 0, 1679, 354, 2031, 7212, 2033, 2035, 0, 1023, 521, 0, 1354, 357, 2030,
    0, 1679, 354, 2031, 2039, 0, 2041, 7213, 255, 2042, 409, 7217, 255, 2042, 0, 7214,
    879, 257, 0, 207, 257, 7218, 1942, 7221, 7224, 2043, 0, 734, 2045, 0, 161, 2046,
    7225, 7235, 7245, 7255, 0, 7266, 0, 879, 2053, 7270, 7280, 7290, 7300, 7310, 7320, 7329,
    7339, 7349, 7359, 7369, 7379, 7389, 7399, 7409, 7419, 7429, 7439, 7448, 7458, 7468, 7478, 7488,
    7498, 7508, 7518, 7528, 7538, 7548, 7558, 2061, 7562, 7571, 7580, 7590, 7600, 7610, 7621, 7631,
    0, 321, 7641, 2068, 1688, 297, 354, 0, 242, 403, 7645, 242, 7642, 2069, 0, 1789,
    7646, 2071, 7648, 7658, 7668, 7678, 7689, 0, 2077, 1015, 7699, 7709, 7719, 7729, 0, 7738,
    7748, 7758, 7768, 7775, 7785, 7795, 7805, 7815, 7825, 7835, 7845, 7855, 7865, 7875, 7885, 7895,
    7905, 7915, 7925, 7944, 1264, 2085, 7935, 1709, 7938, 1842, 7941, 2086, 2087, 0, 7945, 7950,
    871, 7949, 7946, 1709, 2086, 1652, 1650, 1654, 7951, 7961, 7971, 7981, 7991, 0, 8001, 2090,
    2091, 2092, 2093, 2094, 8002, 8012, 0, 8022, 0, 2095, 645, 0, 853, 854, 0, 852,
    853, 0, 852, 854, 8023, 2096, 8033, 8043, 8053, 0, 2097, 242, 0, 90, 523, 8063,
    8073, 8083, 8093, 8103, 8113, 8124, 8134, 8144, 8154, 8164, 8174, 8185, 8195, 0, 523, 970,
    780, 8216, 8205, 2103, 8207, 8219, 965, 0, 965, 965, 8220, 167, 403, 409, 90, 967,
    968, 969, 900, 970, 971, 972, 0, 1819, 81, 403, 242, 782, 242, 8221, 8231, 8241,
    8251, 8261, 8271, 8281, 0, 1792, 8291, 100, 354, 8292, 8302, 8312, 8322, 8332, 8342, 8351,
    8361, 8371, 8381, 8391, 8401, 8411, 0, 522, 8425, 8421, 2114, 8428, 0, 8439, 8438, 1900,
    523, 8441, 8451, 0, 1343, 706, 8460, 0, 1786, 452, 8473, 0, 8470, 426, 100, 354,
    8471, 1904, 1580, 0, 409, 846, 0, 1254, 8472, 690, 1907, 2115, 1784, 1764, 8475, 291,
    1787, 1788, 1789, 1764, 1832, 0, 283, 59, 1793, 8476, 8486, 1786, 2117, 8487, 8497, 8507,
    8517, 8527, 8537, 8547, 0, 1833, 2121, 8560, 242, 8557, 2069, 8561, 8571, 0, 852, 852,
    0, 8586, 0, 8583, 8581, 426, 1705, 948, 1828, 90, 8582, 426, 1705, 948, 1828, 426,
    2124, 8592, 0, 8589, 8587, 426, 1705, 948, 1828, 90, 8588, 426, 1705, 948, 1828, 426,
    8593, 8603, 8613, 8623, 8633, 8643, 8653, 8663, 8673, 8683, 8693, 8703, 8712, 8721, 8731, 8741,
    8751, 8761, 8771, 8781, 8791, 8801, 8831, 0, 65, 8813, 8811, 354, 0, 94, 8819, 8817,
    8816, 354, 624, 625, 92, 8825, 8823, 8822, 354, 624, 625, 0, 283, 8828, 0, 2141,
    100, 90, 2142, 100, 0, 569, 452, 8837, 8834, 8832, 354, 8838, 0, 2144, 452, 8850,
    8848, 0, 426, 251, 8853, 8863, 8873, 8883, 8893, 8903, 8913, 8923, 8945, 0, 2151, 8943,
    8933, 2151, 523, 8946, 0, 8956, 8966, 8976, 8986, 8996, 9006, 9016, 9026, 9036, 9046, 9057,
    9068, 9078, 9087, 9097, 9107, 9117, 9127, 9137, 9147, 9157, 9166, 9176, 9186, 9196, 9206, 9216,
    9225, 2170, 2173, 2175, 0, 9229, 9240, 0, 2176, 9250, 9260, 9270, 2178, 9271, 9281, 9291,
    9301, 0, 1550, 81, 9313, 9311, 251, 403, 879, 9316, 9326, 9336, 9346, 0, 2182, 674,
    9356, 9366, 9376, 9386, 9396, 9406, 9416, 9434, 9432, 0, 9428, 250, 9427, 9426, 2184, 251,
    2186, 2186, 9437, 9447, 9457, 9467, 9477, 9487, 0, 2190, 9497, 0, 2191, 9507, 0, 9520,
    9517, 0, 2193, 9516, 250, 523, 9521, 9531, 9541, 9551, 9568, 0, 169, 9561, 2195, 0,
    90, 9563, 169, 182, 0, 182, 65, 50, 9565, 9564, 279, 374, 0, 90, 9567, 169,
    182, 242, 0, 2197, 164, 0, 1025, 9569, 354, 0, 623, 100, 0, 404, 9586, 0,
    321, 965, 9582, 9572, 0, 250, 0, 965, 250, 0, 1788, 354, 9589, 0, 846, 9599,
    62, 2200, 2201, 9600, 879, 9603, 9613, 0, 2202, 9624, 0, 100, 9623, 2204, 2205, 2206,
    1030, 9625, 9635, 9667, 9645, 9656, 9794, 0, 9690, 9668, 9679, 9702, 2209, 9691, 0, 9706,
    9727, 2209, 9717, 0, 9731, 9752, 2209, 9742, 0, 2211, 9756, 9792, 9791, 9765, 9787, 2209,
    9776, 9795, 9805, 9815, 9825, 9834, 9843, 9853, 9862, 9871, 9881, 9891, 9901, 9911, 9921, 0,
    2217, 571, 452, 9932, 9931, 272, 274, 276, 203, 92, 9934, 272, 274, 276, 203, 9935,
    9945, 9955, 9965, 9975, 9985, 0, 128, 201, 0, 52, 203, 203, 209, 0, 200, 81,
    9986, 9996, 10006, 0, 155, 255, 164, 2224, 10016, 0, 167, 148, 2225, 0, 167, 161,
    2225, 0, 2226, 10020, 520, 2228, 2229, 10021, 10031, 10041, 10057, 344, 0, 278, 10051, 299,
    359, 10054, 10052, 426, 342, 0, 496, 2230, 10058, 0, 643, 177, 170, 700, 255, 1266,
    10068, 10078, 10088, 10098, 10108, 10118, 10128, 10138, 10148, 10158, 10168, 10178, 10188, 0, 2240, 10201,
    10199, 10198, 0, 700, 286, 0, 53, 294, 0, 748, 286, 10205, 0, 516, 700, 518,
    0, 169, 700, 255, 0, 50, 700, 279, 0, 10204, 46, 647, 700, 281, 10206, 0,
    1559, 0, 965, 1559, 0, 297, 1560, 10218, 10216, 2242, 10221, 10231, 10241, 10251, 10261, 10271,
    10281, 10291, 10301, 10311, 10321, 10331, 0, 328, 728, 1324, 0, 328, 10341, 62, 52, 250,
    10342, 10352, 10362, 10372, 10382, 10392, 10402, 10412, 10422, 10432, 10442, 10452, 10462, 10472, 10482, 10492,
    10502, 0, 339, 0, 329, 337, 340, 10512, 0, 1865, 167, 1867, 0, 52, 203, 0,
    152, 167, 128, 201, 201, 94, 2255, 346, 10513, 10522, 10532, 10542, 10552, 242, 10562, 10572,
    10580, 10590, 10600, 10610, 10620, 10630, 10640, 10650, 0, 2261, 2262, 452, 523, 0, 2264, 2262,
    452, 523, 10654, 10664, 10674, 10684, 10694, 10704, 10714, 10724, 10734, 10744, 10754, 10764, 0, 2269,
    10774, 2270, 647, 10775, 100, 0, 10778, 2271, 10779, 100, 10782, 10792, 10802, 10812, 10822, 10832,
    10842, 10852, 10862, 10872, 10882, 10892, 10902, 10912, 10922, 10932, 10942, 0, 501, 2281, 0, 340,
    81, 167, 10967, 0, 169, 255, 0, 423, 426, 0, 264, 83, 10952, 85, 0, 46,
    281, 0, 47, 81, 0, 389, 159, 81, 0, 55, 169, 255, 0, 172, 83, 10958,
    10955, 1845, 0, 97, 98, 100, 0, 177, 170, 255, 0, 390, 49, 81, 0, 173,
    83, 10962, 10959, 1845, 0, 579, 81, 0, 52, 203, 0, 174, 83, 10966, 10963, 1845,
    0, 50, 279, 0, 175, 81, 0, 637, 81, 0, 53, 81, 0, 54, 81, 0,
    170, 255, 981, 10968, 523, 359, 10969, 10979, 2283, 0, 780, 257, 10985, 0, 993, 0,
    167, 10982, 10980, 354, 62, 0, 2286, 100, 0, 156, 2287, 10986, 2289, 1901, 94, 1025,
    0, 1786, 452, 0, 167, 2291, 291, 1573, 2124, 1790, 0, 283, 496, 59, 2292, 2053,
    10998, 1901, 94, 10988, 1025, 0, 1786, 452, 291, 1573, 90, 1790, 2053, 10999, 0, 2294,
    2296, 2298, 1570, 2300, 2302, 1715, 2304, 2304, 2306, 233, 1784, 2308, 2310, 1793, 1790, 1361,
    1266, 0, 167, 2291, 1573, 2292, 2053, 1790, 11020, 11012, 11009, 1842, 11016, 11013, 2312, 2313,
    11017, 879, 11021, 11031, 0, 965, 242, 214, 11041, 0, 965, 242, 214, 11045, 11054, 11064,
    11074, 11084, 0, 625, 100, 0, 496, 625, 11085, 0, 624, 100, 0, 496, 624, 11086,
    11096, 11106, 11116, 11126, 11136, 11146, 11156, 128, 272, 274, 276, 11157, 11167, 11177, 0, 278,
    11187, 2319, 0, 11210, 0, 426, 214, 297, 11189, 301, 437, 207, 426, 354, 299, 11190,
    948, 0, 11196, 11194, 426, 297, 11207, 11206, 879, 11204, 0, 11199, 301, 437, 207, 426,
    354, 299, 11200, 948, 0, 278, 2321, 11213, 62, 11211, 81, 11214, 486, 488, 0, 2086,
    11215, 0, 1790, 2086, 233, 1784, 2304, 11219, 11229, 2322, 2323, 1294, 2324, 0, 278, 50,
    425, 279, 0, 278, 11230, 0, 1332, 728, 0, 728, 728, 1324, 11231, 203, 299, 0,
    278, 11232, 728, 1332, 11240, 0, 250, 11233, 92, 297, 11237, 299, 11235, 11234, 301, 321,
    322, 619, 0, 2325, 2326, 11238, 523, 1332, 299, 0, 50, 523, 0, 52, 11239, 633,
    203, 11241, 11251, 11261, 11271, 11281, 0, 11291, 1690, 1691, 11292, 0, 967, 251, 1693, 0,
    2328, 2329, 0, 1336, 523, 11296, 0, 2328, 1268, 11306, 62, 81, 0, 2328, 11307, 62,
    2330, 1135, 1136, 2331, 2332, 250, 11308, 2333, 2334, 11309, 11319, 100, 11323, 11333, 11343, 11353,
    321, 322, 1900, 11354, 321, 322, 965, 11355, 242, 0, 242, 935, 265, 11357, 11368, 11378,
    0, 2338, 214, 11388, 11398, 11407, 11416, 11426, 11436, 11446, 11456, 11466, 11476, 11486, 11496, 11506,
    11516, 11526, 11536, 11546, 11556, 11566, 11576, 11586, 11596, 11606, 11616, 11626, 11635, 11645, 11655, 11665,
    879, 2357, 1650, 2359, 2361, 2363, 2365, 2367, 2369, 2371, 2373, 2375, 2377, 2379, 2381, 2383,
    2385, 2387, 2389, 2391, 2393, 2395, 2397, 1654, 1652, 2399, 2401, 2403, 2405, 2407, 2409, 2411,
    2413, 2415, 2417, 2419, 2421, 2423, 2425, 2427, 2429, 2431, 2433, 2435, 2437, 2439, 2441, 2443,
    2445, 2447, 2449, 2451, 2453, 2455, 2457, 2459, 2461, 2463, 2465, 2391, 2467, 2469, 2397, 2471,
    2473, 2475, 2477, 2479, 2481, 2483, 2485, 2487, 2489, 2491, 2493, 2495, 2497, 2499, 2501, 2503,
    2505, 2507, 2509, 2511, 2513, 2515, 2517, 2519, 2521, 2523, 2525, 2527, 2529, 2531, 2533, 2535,
    2537, 2539, 2541, 2543, 2545, 2547, 2549, 2551, 2553, 2555, 2557, 2559, 2561, 2563, 2565, 2567,
    2569, 2571, 2573, 2575, 2577, 2579, 2581, 2583, 2585, 2587, 2589, 2591, 2593, 2595, 2596, 1078,
    2598, 2491, 2600, 2602, 2604, 2606, 2608, 2610, 2611, 2613, 2615, 2617, 2485, 2619, 2620, 2622,
    2624, 2626, 2628, 971, 11666, 11676, 11686, 11696, 11706, 11716, 11726, 11736, 11746, 11756, 11766, 11776,
    11786, 2635, 429, 11787, 11797, 11809, 11807, 0, 780, 257, 0, 11812, 11822, 11832, 11842, 11852,
    11862, 11872, 11882, 11892, 11902, 11912, 11922, 11932, 11942, 11949, 11959, 11969, 11979, 11989, 11999, 12009,
    12019, 12040, 0, 668, 12029, 675, 673, 674, 62, 0, 666, 12030, 426, 12039, 0, 12033,
    676, 2651, 678, 12036, 12034, 426, 0, 678, 675, 0, 651, 675, 0, 0, 362, 81,
    12064, 0, 950, 12046, 12044, 426, 0, 2182, 674, 12051, 12049, 426, 0, 55, 674, 12056,
    12054, 426, 155, 255, 12061, 12059, 426, 0, 0, 362, 81, 0, 12065, 950, 951, 409,
    516, 518, 12066, 483, 0, 357, 483, 12067, 0, 485, 486, 0, 485, 488, 0, 12074,
    1264, 437, 207, 2652, 2654, 255, 12068, 879, 12071, 2357, 12075, 12085, 12095, 12105, 12115, 12125,
    0, 12135, 2658, 900, 2659, 0, 12139, 2660, 2659, 12143, 12153, 12163, 12173, 12183, 12193, 12203,
    12213, 12223, 0, 12233, 157, 158, 12234, 12244, 2665, 1240, 12245, 12255, 12265, 12275, 12285, 12295,
    12305, 12315, 12325, 12334, 0, 12344, 2325, 2668, 1419, 0, 12345, 283, 504, 2325, 2326, 12346,
    0, 782, 2325, 2326, 354, 12350, 12360, 12370, 12380, 12390, 12400, 12410, 12420, 12430, 12440, 12450,
    12460, 12470, 0, 12480, 1840, 630, 2672, 896, 2673, 2674, 12481, 2676, 2677, 0, 1931, 499,
    12482, 1934, 0, 153, 2678, 0, 153, 2679, 0, 2271, 153, 12483, 0, 153, 524, 0,
    153, 163, 12484, 12494, 12504, 12514, 12524, 12534, 12544, 12554, 12564, 12574, 12584, 12594, 12604, 12614,
    12624, 12634, 2693, 2694, 2695, 12635, 12645, 12655, 12665, 12676, 0, 160, 169, 12687, 12686, 0,
    524, 255, 0, 255, 265, 0, 12689, 464, 2697, 453, 346, 12690, 12700, 12712, 12710, 780,
    12715, 0, 0, 1824, 0, 321, 1688, 2338, 1994, 0, 1616, 2699, 12725, 12735, 12745, 12755,
    2700, 12765, 2701, 2702, 2703, 12766, 12776, 12786, 0, 2707, 12796, 993, 62, 12797, 0, 950,
    12807, 12817, 12827, 12838, 12848, 12858, 0, 2709, 12868, 242, 265, 12869, 0, 2710, 12879, 523,
    92, 523, 12892, 1074, 0, 1076, 403, 523, 1077, 0, 0, 180, 403, 12882, 1379, 1385,
    1211, 0, 569, 452, 164, 1252, 0, 553, 12883, 1087, 167, 1088, 1089, 1090, 0, 370,
    523, 0, 553, 12884, 167, 1088, 1089, 1090, 12885, 0, 1080, 403, 523, 0, 1081, 403,
    523, 0, 1082, 748, 0, 359, 403, 523, 0, 359, 403, 523, 12888, 12886, 2711, 2712,
    403, 100, 12887, 2204, 2205, 0, 615, 675, 2713, 2714, 1107, 2715, 2716, 12896, 12906, 12916,
    12926, 0, 2294, 2298, 1570, 2300, 2302, 2306, 1715, 2304, 1790, 1361, 1266, 233, 1784, 0,
    2086, 12936, 0, 1790, 2086, 12940, 12950, 12960, 12970, 12980, 12990, 13000, 0, 161, 524, 255,
    257, 278, 2046, 2298, 13010, 0, 1025, 325, 513, 81, 1570, 0, 1573, 13013, 265, 13011,
    0, 354, 257, 13014, 13024, 13034, 13044, 0, 143, 281, 13054, 13064, 242, 13074, 13084, 13094,
    13104, 13114, 13124, 13134, 13144, 13154, 13164, 13174, 13184, 0, 2729, 13195, 520, 2240, 2730, 0,
    13194, 2731, 148, 524, 255, 0, 92, 100, 2732, 2733, 13196, 13206, 13216, 13226, 13236, 0,
    695, 13252, 13249, 13247, 13246, 354, 299, 257, 233, 1784, 13254, 13264, 13274, 13284, 13294, 13304,
    13314, 13324, 13334, 13344, 523, 321, 13345, 13355, 13365, 13375, 13385, 13395, 13405, 13415, 13424, 13434,
    13444, 0, 0, 1354, 0, 354, 13454, 2031, 13457, 354, 13460, 13463, 13466, 13476, 0, 1025,
    13489, 13486, 354, 13490, 291, 0, 200, 452, 0, 1786, 452, 1787, 1788, 1789, 2124, 1573,
    0, 167, 2291, 1793, 13491, 13501, 13511, 13518, 0, 88, 2747, 0, 13529, 13528, 0, 242,
    935, 0, 265, 965, 265, 13533, 0, 242, 13543, 569, 1786, 2749, 233, 2751, 0, 283,
    13544, 2753, 1332, 364, 337, 0, 283, 2754, 13545, 13548, 2756, 156, 13550, 2758, 0, 283,
    2754, 13553, 13556, 2756, 156, 13558, 2313, 0, 283, 0, 496, 59, 0, 13561, 0, 283,
    496, 50, 2759, 0, 2760, 200, 13571, 233, 13574, 13584, 13594, 13601, 13611, 13621, 13631, 13641,
    13651, 13661, 13671, 13681, 13691, 13701, 13711, 13721, 13731, 13741, 13751, 13761, 13771, 13781, 13791, 13801,
    13811, 13821, 13831, 13841, 13851, 13861, 13871, 13881, 13891, 13901, 13911,
];

pub static SEGMENT_TYPE_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 69, 4294967295, 4294967295, 80, 4294967295, 4294967295,
    4294967295, 4294967295, 80, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 84, 87, 89, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 69, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 166, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 196, 198, 4294967295, 4294967295,
    4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 82, 4294967295,
    204, 4294967295, 208, 4294967295, 4294967295, 4294967295, 215, 217, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 236, 241, 4294967295, 4294967295, 4294967295, 4294967295,
    244, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252, 253, 4294967295, 256, 258, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 87, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 282,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 282, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 334, 4294967295, 4294967295, 336, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 331, 4294967295, 334, 4294967295, 4294967295, 336, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 349,
    351, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 82, 4294967295, 82, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 82, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84, 87, 4294967295, 4294967295,
    4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 252, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 412,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 252, 4294967295, 4294967295, 431, 4294967295,
    435, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 252, 4294967295, 4294967295, 431, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208,
    4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 463, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 463, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 468, 4294967295, 4294967295, 468, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 480, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 468, 4294967295, 4294967295, 468, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 519, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 527,
    4294967295, 4294967295, 4294967295, 427, 4294967295, 427, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84,
    87, 4294967295, 565, 4294967295, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 519, 4294967295, 4294967295, 4294967295, 4294967295, 519, 4294967295, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295, 519, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 295, 4294967295,
    4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 576, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 427, 4294967295, 4294967295, 4294967295, 256,
    4294967295, 4294967295, 4294967295, 280, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 256,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84, 578, 4294967295, 4294967295, 4294967295, 4294967295, 84, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 582, 4294967295, 4294967295, 4294967295, 584,
    4294967295, 4294967295, 4294967295, 586, 4294967295, 4294967295, 4294967295, 586, 4294967295, 4294967295, 4294967295, 586, 4294967295, 4294967295, 4294967295, 582,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 463, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 636, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 636, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 640, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 582, 4294967295, 4294967295, 584, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 584,
    4294967295, 4294967295, 4294967295, 4294967295, 584, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 640, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 427, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 427, 252, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 431, 4294967295, 435, 4294967295, 4294967295,
    4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 688, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 528, 4294967295, 4294967295, 4294967295, 427, 4294967295, 427, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295,
    280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 698, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 82, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 82, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 709, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 709, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    373, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 427, 252, 4294967295, 4294967295, 431, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 252,
    4294967295, 4294967295, 431, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 427,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 727, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    82, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 208, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 427, 4294967295, 427, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295,
    84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298,
    4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 751, 751,
    4294967295, 751, 751, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 241, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 822, 208, 427, 355, 4294967295,
    252, 4294967295, 4294967295, 4294967295, 824, 825, 3, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 751, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 869, 4294967295, 872, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 244, 4294967295, 4294967295, 878, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 427, 4294967295, 4294967295, 427, 4294967295, 427, 4294967295,
    427, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 236, 4294967295, 893, 4294967295, 4294967295, 899, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 893,
    4294967295, 4294967295, 899, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    256, 4294967295, 4294967295, 519, 4294967295, 519, 4294967295, 256, 4294967295, 4294967295, 4, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 949,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295,
    427, 4294967295, 4294967295, 252, 4294967295, 435, 4294967295, 4294967295, 431, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295,
    4294967295, 4294967295, 82, 4294967295, 82, 4294967295, 4294967295, 427, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 282, 4294967295, 4294967295, 373, 4294967295, 4294967295, 84, 4294967295, 87, 4294967295, 4294967295, 519, 4294967295, 4294967295, 280,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 982, 4294967295, 4294967295, 4294967295, 4294967295, 298, 298, 298, 298, 298, 298, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 298, 298, 298, 4294967295, 4294967295, 4294967295, 1003,
    298, 751, 298, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 298, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295,
    427, 298, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 427, 298, 4294967295, 4294967295, 4294967295,
    4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 355, 208, 4294967295, 4294967295, 431,
    1028, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252,
    4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 1075, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1079, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 87, 4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 252,
    4294967295, 4294967295, 4294967295, 4294967295, 84, 87, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295,
    4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    298, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 282, 4294967295, 1145, 4294967295,
    4294967295, 1148, 1150, 1152, 1154, 4294967295, 4294967295, 4294967295, 298, 204, 727, 1156, 1158, 4294967295, 4294967295, 4294967295,
    82, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 252, 4294967295,
    4294967295, 431, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    355, 4294967295, 4294967295, 4294967295, 4294967295, 373, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 1160, 4294967295, 4294967295, 1160,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84, 4294967295, 4294967295, 252, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256,
    1201, 4294967295, 4294967295, 4294967295, 4294967295, 1207, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295,
    4294967295, 4294967295, 427, 252, 1201, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1207, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 427, 252, 4294967295, 4294967295, 4294967295, 431, 4294967295, 435, 480, 4294967295,
    4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 1239, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 256,
    4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 435, 480, 4294967295, 4294967295, 4294967295, 4294967295, 698,
    4294967295, 4294967295, 4294967295, 1239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84, 87, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 427, 4294967295, 252, 1242, 4294967295, 4294967295, 252, 252, 1248,
    4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    519, 4294967295, 4294967295, 256, 4294967295, 4294967295, 412, 4294967295, 4294967295, 1250, 4294967295, 4294967295, 4294967295, 4294967295, 1250, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 468, 4294967295, 4294967295, 295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1075, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 1265, 4294967295, 4294967295, 4294967295, 4294967295, 208, 1267, 4294967295, 4294967295,
    4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 427, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 252, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 82, 4294967295, 4294967295,
    298, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298,
    84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 84, 578, 4294967295, 1248,
    4294967295, 4294967295, 4294967295, 582, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 586, 4294967295, 4294967295, 584,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 252, 4294967295, 4294967295, 258, 4294967295,
    4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 204, 4294967295, 4294967295, 204, 4294967295, 4294967295, 204,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 204, 4294967295, 4294967295, 204,
    4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 629, 4294967295, 1326, 4294967295, 4294967295, 4294967295, 4294967295, 1239, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1160,
    4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 636, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 427, 355, 4294967295, 256, 4294967295, 4294967295, 4294967295, 640, 4294967295, 4294967295, 4294967295, 4294967295, 582,
    4294967295, 4294967295, 584, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 427, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256,
    4294967295, 4294967295, 4294967295, 4294967295, 427, 252, 4294967295, 1334, 435, 480, 4294967295, 4294967295, 256, 4294967295, 1340, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 480, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    298, 82, 4294967295, 4294967295, 298, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 709, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 208, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1239, 4294967295, 4294967295, 4294967295, 4294967295,
    204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 727, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1362, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 84, 84, 4294967295, 1370, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1420, 4294967295, 4294967295, 1433, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1433, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 1562, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 258, 4294967295, 4294967295, 4294967295, 4294967295, 1567, 258,
    1569, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 1571, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 258,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1605, 4294967295, 4294967295, 4294967295, 1605, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 4294967295, 87, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252,
    4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 373, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 87, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 519, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 84, 578, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 582, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 636, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 640, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 709, 4294967295, 256, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295,
    1617, 1619, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1250, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1649, 4294967295, 4294967295, 1651, 1653, 1655, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 101, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 101, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1692, 4294967295, 1694, 949, 4294967295, 4294967295, 4294967295,
    949, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 427, 208, 1699,
    355, 4294967295, 89, 4294967295, 4294967295, 1617, 1703, 4294967295, 4294967295, 355, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1706, 1699, 4294967295, 4294967295, 4294967295, 4294967295, 1710, 4294967295, 822, 1712, 1714, 427, 1716, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295,
    4294967295, 1744, 1746, 1748, 4294967295, 1750, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 949, 4294967295,
    208, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295,
    355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 822, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1777, 4294967295, 4294967295, 1785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1791,
    1362, 1267, 4294967295, 4294967295, 1794, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1567, 1800, 258, 4294967295,
    4294967295, 1804, 4294967295, 1569, 4294967295, 4294967295, 4294967295, 258, 1804, 4294967295, 4294967295, 4294967295, 4294967295, 1569, 4294967295, 4294967295,
    4294967295, 4294967295, 1808, 1810, 4294967295, 1777, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 4294967295, 4294967295,
    1569, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 253, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298,
    4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 258, 4294967295, 427,
    4294967295, 4294967295, 4294967295, 4294967295, 355, 258, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427,
    4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 427, 258, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 1823, 4294967295,
    4294967295, 4294967295, 355, 1823, 4294967295, 355, 4294967295, 4294967295, 872, 4294967295, 355, 4294967295, 252, 4294967295, 4294967295, 355,
    4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 355, 4294967295, 4294967295,
    252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1830, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427,
    355, 4294967295, 4294967295, 4294967295, 4294967295, 427, 355, 4294967295, 1837, 1712, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295,
    355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1605, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1651, 4294967295, 4294967295, 1653, 4294967295, 4294967295, 4294967295, 1843, 872, 1710, 4294967295,
    4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 869, 4294967295, 4294967295, 4294967295, 869, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252, 1242, 4294967295, 4294967295, 4294967295, 298, 4294967295, 355, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 84, 1848, 4294967295, 4294967295, 4294967295, 84, 4294967295, 84, 1265, 84, 4294967295, 4294967295,
    84, 4294967295, 84, 1265, 84, 1848, 4294967295, 4294967295, 84, 1848, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1866,
    4294967295, 1868, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 202, 82, 4294967295, 4294967295, 4294967295, 4294967295, 1870,
    4294967295, 4294967295, 1870, 4294967295, 4294967295, 4294967295, 4294967295, 1870, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    427, 4294967295, 1878, 1881, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1791, 4294967295, 4294967295,
    4294967295, 4294967295, 1785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 355, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1878, 1370, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 1785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1892, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1239, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 431, 4294967295, 1908, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 468,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 208, 4294967295, 355, 1910, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 584, 4294967295, 4294967295, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1810, 4294967295, 4294967295, 4294967295, 236, 1941, 4294967295, 1943, 4294967295, 253, 4294967295, 4294967295, 4294967295, 1810,
    1941, 4294967295, 4294967295, 1810, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1948, 1950, 258, 4294967295, 4294967295, 236, 4294967295,
    4294967295, 355, 253, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 355, 258, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 101, 4294967295, 1706, 1995, 1997, 1999, 4294967295, 4294967295, 1607,
    1607, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 82, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1265, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    2027, 2029, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 2034, 2036, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 2034, 2036, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 355, 4294967295, 2040, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295,
    4294967295, 258, 4294967295, 208, 258, 236, 1943, 253, 4294967295, 2044, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2047,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2070, 4294967295, 4294967295,
    4294967295, 2072, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 318, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1265, 1710, 4294967295, 1710, 4294967295, 1843, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    872, 4294967295, 4294967295, 1710, 4294967295, 1653, 1651, 1655, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 2104, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 355,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2116, 1785, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1794, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2070, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 1706, 949, 4294967295, 4294967295, 4294967295, 427, 1706, 949, 4294967295, 427,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 1706, 949, 4294967295, 4294967295, 4294967295, 427, 1706, 949, 4294967295, 427,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 427, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2172, 2174, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 252, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 252,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2196, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 202, 4294967295, 4294967295, 204, 204, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 1267,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 519,
    4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 2243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1866, 4294967295, 1868, 4294967295, 4294967295, 204, 4294967295,
    4294967295, 4294967295, 4294967295, 202, 202, 4294967295, 1870, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    82, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 427, 4294967295, 4294967295, 84, 4294967295, 87, 4294967295, 4294967295,
    282, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 84, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295,
    84, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 204, 4294967295, 4294967295, 84, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 280, 4294967295, 4294967295, 82, 4294967295, 4294967295, 82, 4294967295, 4294967295, 82, 4294967295, 4294967295, 82, 4294967295,
    4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2284, 4294967295, 4294967295, 258, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2288, 4294967295, 2290, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1791, 4294967295, 4294967295, 4294967295, 4294967295, 2293, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1791, 4294967295, 4294967295, 4294967295, 2295,
    2297, 2299, 1571, 2301, 2303, 1716, 2305, 2305, 2307, 235, 1785, 2309, 2311, 1794, 1791, 1362,
    1267, 4294967295, 4294967295, 4294967295, 4294967295, 2293, 4294967295, 1791, 4294967295, 4294967295, 4294967295, 1843, 4294967295, 4294967295, 1843, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 215, 4294967295, 4294967295, 4294967295, 4294967295, 215, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2320, 4294967295, 4294967295, 4294967295, 427, 215, 298, 4294967295, 4294967295, 4294967295, 208, 427, 355, 4294967295, 4294967295,
    949, 4294967295, 4294967295, 4294967295, 427, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 208, 427,
    355, 4294967295, 4294967295, 949, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1791, 4294967295, 235, 1785, 2305, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 280, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 204, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1692, 4294967295, 4294967295, 4294967295, 252, 1694, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2339, 215, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2358, 1651, 2360, 2362, 2364, 2366, 2368, 2370, 2372, 2374, 2376, 2378, 2380, 2382, 2384,
    2386, 2388, 2390, 2392, 2394, 2396, 2398, 1655, 1653, 2400, 2402, 2404, 2406, 2408, 2410, 2412,
    2414, 2416, 2418, 2420, 2422, 2424, 2426, 2428, 2430, 2432, 2434, 2436, 2438, 2440, 2442, 2444,
    2446, 2448, 2450, 2452, 2454, 2456, 2458, 2460, 2462, 2464, 2466, 2392, 2468, 2470, 2398, 2472,
    2474, 2476, 2478, 2480, 2482, 2484, 2486, 2488, 2490, 2492, 2494, 2496, 2498, 2500, 2502, 2504,
    2506, 2508, 2510, 2512, 2514, 2516, 2518, 2520, 2522, 2524, 2526, 2528, 2530, 2532, 2534, 2536,
    2538, 2540, 2542, 2544, 2546, 2548, 2550, 2552, 2554, 2556, 2558, 2560, 2562, 2564, 2566, 2568,
    2570, 2572, 2574, 2576, 2578, 2580, 2582, 2584, 2586, 2588, 2590, 2592, 2594, 2594, 2597, 1079,
    2599, 2492, 2601, 2603, 2605, 2607, 2609, 2603, 2612, 2614, 2616, 2618, 2486, 2540, 2621, 2623,
    2625, 2627, 2629, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 751, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 258, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 247, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 427, 4294967295, 256, 4294967295, 4294967295, 427, 4294967295, 4294967295, 4294967295, 82, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 519, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1265, 4294967295, 208, 2653, 2655, 256, 4294967295, 4294967295, 4294967295, 2358, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1420, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 256, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2339, 1995, 4294967295, 1617, 1619, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1075, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2295, 2299, 1571, 2301, 2303, 2307, 1716, 2305, 1791, 1362, 1267, 235, 1785, 4294967295,
    4294967295, 4294967295, 4294967295, 1791, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 256,
    258, 4294967295, 2047, 2299, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 82, 1571, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 355, 258, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 282, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 256, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 4294967295, 258, 235, 1785, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 355, 236, 4294967295, 236, 355, 253, 253, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 355, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1794, 4294967295, 4294967295, 2744, 4294967295, 4294967295, 89, 2748, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2750, 235, 2752, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 236, 4294967295, 2757, 4294967295, 253, 4294967295, 4294967295, 4294967295,
    4294967295, 236, 4294967295, 2757, 4294967295, 253, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 235, 4294967295, 4294967295, 5, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static SEGMENT_CLASS_OFFSETS: &[u32] = &[
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 68, 4294967295, 4294967295, 70, 4294967295, 4294967295,
    4294967295, 4294967295, 70, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 83, 85, 88, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 68, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 165, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 195, 197, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    203, 4294967295, 207, 4294967295, 4294967295, 4294967295, 214, 216, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 233, 4294967295, 237, 239, 4294967295, 4294967295, 4294967295, 4294967295,
    243, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251, 254, 4294967295, 255, 257, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 281,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 332, 4294967295, 4294967295, 335, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 330, 4294967295, 332, 4294967295, 4294967295, 335, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 348,
    350, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 372, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 85, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 251, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 411,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 251, 4294967295, 4294967295, 430, 4294967295,
    434, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 251, 4294967295, 4294967295, 430, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 207,
    4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 462, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 462, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 467, 4294967295, 4294967295, 467, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 479, 4294967295, 4294967295, 481, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 467, 4294967295, 4294967295, 467, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 518, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 526,
    4294967295, 4294967295, 4294967295, 426, 4294967295, 426, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83,
    85, 4294967295, 564, 4294967295, 4294967295, 4294967295, 83, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 372, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 518, 4294967295, 4294967295, 4294967295, 4294967295, 518, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 518, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 372, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295,
    4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 575, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 426, 4294967295, 4294967295, 4294967295, 255,
    4294967295, 4294967295, 4294967295, 279, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 372, 4294967295, 4294967295, 4294967295, 4294967295, 294,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 255,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 577, 4294967295, 4294967295, 4294967295, 4294967295, 83, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 581, 4294967295, 4294967295, 4294967295, 583,
    4294967295, 4294967295, 4294967295, 585, 4294967295, 4294967295, 4294967295, 585, 4294967295, 4294967295, 4294967295, 585, 4294967295, 4294967295, 4294967295, 581,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 462, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 635, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 635, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 581, 4294967295, 4294967295, 583, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 583,
    4294967295, 4294967295, 4294967295, 4294967295, 583, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 479, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 426, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 426, 251, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 430, 4294967295, 434, 4294967295, 4294967295,
    4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 687, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 693, 4294967295, 4294967295, 4294967295, 426, 4294967295, 426, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295,
    279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 697, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 294, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 708, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 708, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    372, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 426, 251, 4294967295, 4294967295, 430, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 251,
    4294967295, 4294967295, 430, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 426,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 726, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 207, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 426, 4294967295, 426, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295,
    83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297,
    4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 770, 771,
    4294967295, 773, 774, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 239, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 820, 207, 426, 354, 4294967295,
    251, 4294967295, 4294967295, 4294967295, 823, 826, 827, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 752, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 870, 4294967295, 871, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 243, 4294967295, 4294967295, 877, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 426, 4294967295, 4294967295, 426, 4294967295, 426, 4294967295,
    426, 4294967295, 4294967295, 4294967295, 4294967295, 207, 4294967295, 4294967295, 207, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 890, 4294967295, 892, 4294967295, 4294967295, 898, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 892,
    4294967295, 4294967295, 898, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    255, 4294967295, 4294967295, 518, 4294967295, 518, 4294967295, 255, 4294967295, 4294967295, 818, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 948,
    4294967295, 207, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295,
    426, 4294967295, 4294967295, 251, 4294967295, 434, 4294967295, 4294967295, 430, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 81, 4294967295, 4294967295, 426, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 281, 4294967295, 4294967295, 372, 4294967295, 4294967295, 83, 4294967295, 85, 4294967295, 4294967295, 518, 4294967295, 4294967295, 279,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 983, 4294967295, 4294967295, 4294967295, 4294967295, 297, 987, 988, 989, 990, 991, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 995, 996, 997, 998, 4294967295, 4294967295, 4294967295, 1002,
    1004, 1005, 1006, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 297, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295,
    426, 297, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 426, 297, 4294967295, 4294967295, 4294967295,
    4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 354, 207, 4294967295, 4294967295, 430,
    1027, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251,
    4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 1074, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1078, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 251,
    4294967295, 4294967295, 4294967295, 4294967295, 83, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295,
    4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    297, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 281, 4294967295, 1144, 4294967295,
    4294967295, 1146, 1149, 1151, 1153, 4294967295, 4294967295, 4294967295, 297, 203, 726, 1155, 1157, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 251, 4294967295,
    4294967295, 430, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    354, 4294967295, 4294967295, 4294967295, 4294967295, 372, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 1159, 4294967295, 4294967295, 1159,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 4294967295, 4294967295, 251, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255,
    1199, 4294967295, 4294967295, 4294967295, 4294967295, 1204, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295,
    4294967295, 4294967295, 426, 251, 1199, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1204, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 426, 251, 4294967295, 4294967295, 4294967295, 430, 4294967295, 1234, 1235, 4294967295,
    4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 1238, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 255,
    4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 1234, 1235, 4294967295, 4294967295, 4294967295, 4294967295, 697,
    4294967295, 4294967295, 4294967295, 1238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 83, 85, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 426, 4294967295, 251, 1241, 4294967295, 4294967295, 251, 251, 1245,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    518, 4294967295, 4294967295, 255, 4294967295, 4294967295, 411, 4294967295, 4294967295, 1249, 4294967295, 4294967295, 4294967295, 4294967295, 1249, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 467, 4294967295, 4294967295, 294, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1074, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1262, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 1264, 4294967295, 4294967295, 4294967295, 4294967295, 207, 1266, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 426, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 251, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 81, 4294967295, 4294967295,
    297, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297,
    83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 83, 577, 4294967295, 1245,
    4294967295, 4294967295, 4294967295, 581, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 585, 4294967295, 4294967295, 583,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 251, 4294967295, 4294967295, 257, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 203, 4294967295, 4294967295, 203, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 203, 4294967295, 4294967295, 203,
    4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 628, 4294967295, 1325, 4294967295, 4294967295, 4294967295, 4294967295, 1238, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1159,
    4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 635, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 426, 354, 4294967295, 255, 4294967295, 4294967295, 4294967295, 639, 4294967295, 4294967295, 4294967295, 4294967295, 581,
    4294967295, 4294967295, 583, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 426, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1262, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255,
    4294967295, 4294967295, 4294967295, 4294967295, 426, 251, 4294967295, 1333, 434, 479, 4294967295, 4294967295, 255, 4294967295, 1339, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1262, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    297, 81, 4294967295, 4294967295, 297, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 708, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 207, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1238, 4294967295, 4294967295, 4294967295, 4294967295,
    203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 726, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1361, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1367, 1368, 4294967295, 1369, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1419, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1419, 4294967295, 4294967295, 1432, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1419, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1432, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 1561, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 1566, 257,
    1568, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 1570, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 257,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1604, 4294967295, 4294967295, 4294967295, 1604, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251,
    4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 372, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 85, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 518, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 83, 577, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 581, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 628, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 635, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 639, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 708, 4294967295, 255, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295,
    1616, 1618, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1249, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1648, 4294967295, 4294967295, 1650, 1652, 1654, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1683, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1683, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1691, 4294967295, 1693, 948, 4294967295, 4294967295, 4294967295,
    948, 4294967295, 4294967295, 4294967295, 207, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 426, 207, 1698,
    354, 4294967295, 1701, 4294967295, 4294967295, 1616, 1702, 4294967295, 4294967295, 354, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 1705, 1698, 4294967295, 4294967295, 4294967295, 4294967295, 1709, 4294967295, 820, 1711, 1713, 426, 1715, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    4294967295, 1743, 1745, 1747, 4294967295, 1749, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 948, 4294967295,
    207, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295,
    354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 820, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1776, 4294967295, 4294967295, 1784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1790,
    1361, 1266, 4294967295, 4294967295, 1793, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1566, 1799, 257, 4294967295,
    4294967295, 1802, 4294967295, 1568, 4294967295, 4294967295, 4294967295, 257, 1802, 4294967295, 4294967295, 4294967295, 4294967295, 1568, 4294967295, 4294967295,
    4294967295, 4294967295, 1806, 1809, 4294967295, 1776, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 4294967295, 4294967295,
    1568, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 254, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297,
    4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 207, 257, 4294967295, 426,
    4294967295, 4294967295, 4294967295, 4294967295, 354, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426,
    4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 426, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 1822, 4294967295,
    4294967295, 4294967295, 354, 1822, 4294967295, 354, 4294967295, 4294967295, 871, 4294967295, 354, 4294967295, 251, 4294967295, 4294967295, 354,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 354, 4294967295, 4294967295,
    251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1829, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426,
    354, 4294967295, 4294967295, 4294967295, 4294967295, 426, 354, 4294967295, 1836, 1838, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295,
    354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1604, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1650, 4294967295, 4294967295, 1652, 4294967295, 4294967295, 4294967295, 1842, 871, 1709, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 870, 4294967295, 4294967295, 4294967295, 870, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251, 1241, 4294967295, 4294967295, 4294967295, 297, 4294967295, 354, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1846, 1847, 4294967295, 4294967295, 4294967295, 83, 4294967295, 1846, 1264, 1849, 4294967295, 4294967295,
    83, 4294967295, 1846, 1264, 1849, 1851, 4294967295, 4294967295, 1849, 1853, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1865,
    4294967295, 1867, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 201, 81, 4294967295, 4294967295, 4294967295, 4294967295, 1869,
    4294967295, 4294967295, 1871, 4294967295, 4294967295, 4294967295, 4294967295, 1871, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    426, 4294967295, 1877, 1879, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1790, 4294967295, 4294967295,
    4294967295, 4294967295, 1784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 354, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1877, 1369, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 1784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1893, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1238, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 207, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 430, 4294967295, 1905, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 467,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 207, 4294967295, 354, 1909, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 583, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1809, 4294967295, 4294967295, 4294967295, 237, 1940, 4294967295, 1942, 4294967295, 254, 4294967295, 4294967295, 4294967295, 1809,
    1940, 4294967295, 4294967295, 1809, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1947, 1949, 257, 4294967295, 4294967295, 890, 4294967295,
    4294967295, 354, 254, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 354, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1683, 4294967295, 1705, 1994, 1996, 1998, 4294967295, 4294967295, 1718,
    1608, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 81, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1264, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    2026, 2028, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 2033, 2035, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 2033, 2035, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 2039, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295,
    4294967295, 257, 4294967295, 207, 257, 237, 1942, 254, 4294967295, 2043, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2046,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2069, 4294967295, 4294967295,
    4294967295, 2071, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2062, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 1264, 2085, 4294967295, 1709, 4294967295, 1842, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    871, 4294967295, 4294967295, 1709, 4294967295, 1652, 1650, 1654, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 2103, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 354,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2115, 1784, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1793, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2069, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 1705, 948, 4294967295, 4294967295, 4294967295, 426, 1705, 948, 4294967295, 426,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 1705, 948, 4294967295, 4294967295, 4294967295, 426, 1705, 948, 4294967295, 426,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 426, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2170, 2173, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 251, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 251,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2195, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 201, 4294967295, 4294967295, 203, 203, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 1266,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 294, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 518,
    4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 2242, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1865, 4294967295, 1867, 4294967295, 4294967295, 203, 4294967295,
    4294967295, 4294967295, 4294967295, 201, 201, 4294967295, 2255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    81, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 426, 4294967295, 4294967295, 83, 4294967295, 85, 4294967295, 4294967295,
    281, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 83, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295,
    83, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 203, 4294967295, 4294967295, 83, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 279, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295, 4294967295, 81, 4294967295,
    4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2283, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2287, 4294967295, 2289, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1790, 4294967295, 4294967295, 4294967295, 4294967295, 2292, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1790, 4294967295, 4294967295, 4294967295, 2294,
    2296, 2298, 1570, 2300, 2302, 1715, 2304, 2304, 2306, 233, 1784, 2308, 2310, 1793, 1790, 1361,
    1266, 4294967295, 4294967295, 4294967295, 4294967295, 2292, 4294967295, 1790, 4294967295, 4294967295, 4294967295, 1842, 4294967295, 4294967295, 2312, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 214, 4294967295, 4294967295, 4294967295, 4294967295, 214, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2319, 4294967295, 4294967295, 4294967295, 426, 214, 297, 4294967295, 4294967295, 4294967295, 207, 426, 354, 4294967295, 4294967295,
    948, 4294967295, 4294967295, 4294967295, 426, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 207, 426,
    354, 4294967295, 4294967295, 948, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 1790, 4294967295, 233, 1784, 2304, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 279, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 203, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 297, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    203, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1691, 4294967295, 4294967295, 4294967295, 251, 1693, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2338, 214, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2357, 1650, 2359, 2361, 2363, 2365, 2367, 2369, 2371, 2373, 2375, 2377, 2379, 2381, 2383,
    2385, 2387, 2389, 2391, 2393, 2395, 2397, 1654, 1652, 2399, 2401, 2403, 2405, 2407, 2409, 2411,
    2413, 2415, 2417, 2419, 2421, 2423, 2425, 2427, 2429, 2431, 2433, 2435, 2437, 2439, 2441, 2443,
    2445, 2447, 2449, 2451, 2453, 2455, 2457, 2459, 2461, 2463, 2465, 2391, 2467, 2469, 2397, 2471,
    2473, 2475, 2477, 2479, 2481, 2483, 2485, 2487, 2489, 2491, 2493, 2495, 2497, 2499, 2501, 2503,
    2505, 2507, 2509, 2511, 2513, 2515, 2517, 2519, 2521, 2523, 2525, 2527, 2529, 2531, 2533, 2535,
    2537, 2539, 2541, 2543, 2545, 2547, 2549, 2551, 2553, 2555, 2557, 2559, 2561, 2563, 2565, 2567,
    2569, 2571, 2573, 2575, 2577, 2579, 2581, 2583, 2585, 2587, 2589, 2591, 2593, 2595, 2596, 1078,
    2598, 2491, 2600, 2602, 2604, 2606, 2608, 2610, 2611, 2613, 2615, 2617, 2485, 2619, 2620, 2622,
    2624, 2626, 2628, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 2635, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 246, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 426, 4294967295, 255, 4294967295, 4294967295, 426, 4294967295, 4294967295, 4294967295, 81, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 518, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    1264, 4294967295, 207, 2652, 2654, 255, 4294967295, 4294967295, 4294967295, 2357, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 1419, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 255, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2338, 1994, 4294967295, 1616, 2699, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1074, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 2294, 2298, 1570, 2300, 2302, 2306, 1715, 2304, 1790, 1361, 1266, 233, 1784, 4294967295,
    4294967295, 4294967295, 4294967295, 1790, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 255,
    257, 4294967295, 2046, 2298, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 81, 1570, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 354, 257, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 281, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 255, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 4294967295, 257, 233, 1784, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 354, 237, 4294967295, 890, 354, 254, 254, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 354, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 1793, 4294967295, 4294967295, 2745, 4294967295, 4294967295, 88, 2747, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 2749, 233, 2751, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 237, 4294967295, 2756, 4294967295, 254, 4294967295, 4294967295, 4294967295,
    4294967295, 237, 4294967295, 2756, 4294967295, 254, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 233, 4294967295, 4294967295, 2153, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
    4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295, 4294967295,
];

pub static CASEFOLD_SPARSE: &[(u32, u8)] = &[
    (6126, 2),
];

pub static TRIM_CHARS_SPARSE: &[(u32, u32, u8)] = &[
];

pub static TRIM_CHARS_DATA: &[u32] = &[
];

pub static SEGMENT_CLASS_TYPES_SPARSE: &[(u32, u32, u8)] = &[
    (42, 0, 2),
    (45, 2, 2),
    (50, 4, 2),
    (57, 6, 2),
    (59, 8, 2),
    (60, 10, 2),
    (61, 12, 3),
    (103, 15, 2),
    (139, 17, 2),
    (188, 19, 2),
    (189, 21, 2),
    (194, 23, 2),
    (198, 25, 3),
    (202, 28, 3),
    (206, 31, 2),
    (208, 33, 3),
    (210, 36, 2),
    (214, 38, 2),
    (215, 40, 2),
    (232, 42, 2),
    (234, 44, 4),
    (235, 48, 2),
    (240, 50, 2),
    (249, 52, 2),
    (250, 54, 5),
    (252, 59, 3),
    (253, 62, 2),
    (261, 64, 2),
    (264, 66, 2),
    (271, 68, 2),
    (279, 70, 3),
    (283, 73, 3),
    (287, 76, 3),
    (305, 79, 3),
    (310, 82, 2),
    (317, 84, 3),
    (325, 87, 2),
    (341, 89, 2),
    (343, 91, 2),
    (346, 93, 2),
    (375, 95, 2),
    (377, 97, 2),
    (380, 99, 2),
    (403, 101, 2),
    (411, 103, 3),
    (415, 106, 2),
    (416, 108, 2),
    (420, 110, 3),
    (425, 113, 2),
    (430, 115, 2),
    (444, 117, 2),
    (451, 119, 2),
    (459, 121, 2),
    (466, 123, 2),
    (468, 125, 2),
    (472, 127, 2),
    (477, 129, 2),
    (484, 131, 2),
    (488, 133, 2),
    (492, 135, 3),
    (503, 138, 3),
    (543, 141, 2),
    (551, 143, 3),
    (554, 146, 2),
    (562, 148, 2),
    (564, 150, 2),
    (572, 152, 2),
    (573, 154, 2),
    (578, 156, 2),
    (582, 158, 2),
    (584, 160, 2),
    (589, 162, 2),
    (591, 164, 2),
    (602, 166, 3),
    (603, 169, 2),
    (606, 171, 3),
    (608, 174, 2),
    (613, 176, 3),
    (618, 179, 3),
    (625, 182, 2),
    (628, 184, 3),
    (632, 187, 2),
    (639, 189, 2),
    (641, 191, 2),
    (667, 193, 2),
    (680, 195, 2),
    (696, 197, 2),
    (699, 199, 2),
    (714, 201, 2),
    (717, 203, 2),
    (796, 205, 3),
    (804, 208, 2),
    (807, 210, 2),
    (811, 212, 3),
    (834, 215, 3),
    (841, 218, 3),
    (859, 221, 3),
    (863, 224, 3),
    (867, 227, 3),
    (869, 230, 3),
    (873, 233, 3),
    (924, 236, 2),
    (943, 238, 2),
    (944, 240, 2),
    (946, 242, 2),
    (950, 244, 2),
    (954, 246, 3),
    (970, 249, 3),
    (974, 252, 2),
    (982, 254, 2),
    (986, 256, 2),
    (993, 258, 3),
    (998, 261, 3),
    (1002, 264, 3),
    (1006, 267, 3),
    (1012, 270, 3),
    (1020, 273, 2),
    (1038, 275, 3),
    (1043, 278, 3),
    (1046, 281, 3),
    (1053, 284, 3),
    (1065, 287, 2),
    (1099, 289, 2),
    (1110, 291, 2),
    (1121, 293, 3),
    (1124, 296, 2),
    (1129, 298, 3),
    (1131, 301, 3),
    (1135, 304, 3),
    (1139, 307, 3),
    (1141, 310, 3),
    (1146, 313, 3),
    (1151, 316, 3),
    (1156, 319, 2),
    (1159, 321, 3),
    (1165, 324, 2),
    (1167, 326, 3),
    (1172, 329, 2),
    (1178, 331, 3),
    (1186, 334, 2),
    (1191, 336, 2),
    (1207, 338, 2),
    (1224, 340, 2),
    (1225, 342, 2),
    (1230, 344, 2),
    (1243, 346, 3),
    (1247, 349, 2),
    (1251, 351, 2),
    (1255, 353, 2),
    (1259, 355, 2),
    (1263, 357, 3),
    (1271, 360, 3),
    (1283, 363, 3),
    (1322, 366, 3),
    (1326, 369, 3),
    (1333, 372, 3),
    (1337, 375, 3),
    (1346, 378, 2),
    (1365, 380, 3),
    (1370, 383, 3),
    (1374, 386, 3),
    (1379, 389, 2),
    (1404, 391, 3),
    (1426, 394, 3),
    (1431, 397, 3),
    (1434, 400, 2),
    (1445, 402, 3),
    (1449, 405, 3),
    (1454, 408, 3),
    (1469, 411, 3),
    (1475, 414, 3),
    (1482, 417, 3),
    (1494, 420, 3),
    (1497, 423, 2),
    (1503, 425, 2),
    (1508, 427, 2),
    (1514, 429, 2),
    (1519, 431, 2),
    (1526, 433, 3),
    (1536, 436, 2),
    (1553, 438, 2),
    (1585, 440, 3),
    (1589, 443, 3),
    (1593, 446, 2),
    (1601, 448, 3),
    (1611, 451, 3),
    (1619, 454, 3),
    (1633, 457, 3),
    (1634, 460, 2),
    (1637, 462, 2),
    (1643, 464, 3),
    (1645, 467, 2),
    (1649, 469, 3),
    (1654, 472, 3),
    (1669, 475, 2),
    (1678, 477, 2),
    (1684, 479, 3),
    (1692, 482, 3),
    (1700, 485, 3),
    (1704, 488, 2),
    (1708, 490, 3),
    (1710, 493, 3),
    (1719, 496, 3),
    (1724, 499, 3),
    (1728, 502, 3),
    (1737, 505, 2),
    (1749, 507, 3),
    (1754, 510, 2),
    (1757, 512, 3),
    (1764, 515, 2),
    (1771, 517, 2),
    (1774, 519, 2),
    (1780, 521, 2),
    (1783, 523, 2),
    (1788, 525, 2),
    (1790, 527, 2),
    (1796, 529, 2),
    (1798, 531, 2),
    (1800, 533, 2),
    (1807, 535, 2),
    (1811, 537, 2),
    (1816, 539, 2),
    (1823, 541, 2),
    (1827, 543, 3),
    (1829, 546, 3),
    (1834, 549, 3),
    (1840, 552, 3),
    (1844, 555, 2),
    (1853, 557, 2),
    (1863, 559, 2),
    (1867, 561, 3),
    (1873, 564, 3),
    (1874, 567, 2),
    (1877, 569, 3),
    (1882, 572, 3),
    (1887, 575, 2),
    (1890, 577, 3),
    (1896, 580, 3),
    (1901, 583, 3),
    (1903, 586, 3),
    (1918, 589, 3),
    (1945, 592, 2),
    (1946, 594, 2),
    (1959, 596, 2),
    (1962, 598, 2),
    (1968, 600, 2),
    (1973, 602, 3),
    (1978, 605, 3),
    (1985, 608, 2),
    (1987, 610, 2),
    (1995, 612, 2),
    (2002, 614, 3),
    (2004, 617, 3),
    (2008, 620, 3),
    (2012, 623, 3),
    (2020, 626, 2),
    (2030, 628, 2),
    (2040, 630, 3),
    (2044, 633, 3),
    (2048, 636, 2),
    (2057, 638, 3),
    (2063, 641, 2),
    (2067, 643, 2),
    (2071, 645, 3),
    (2081, 648, 3),
    (2084, 651, 3),
    (2104, 654, 3),
    (2107, 657, 3),
    (2126, 660, 2),
    (2127, 662, 2),
    (2129, 664, 2),
    (2130, 666, 2),
    (2154, 668, 2),
    (2164, 670, 2),
    (2166, 672, 2),
    (2174, 674, 2),
    (2235, 676, 2),
    (2236, 678, 2),
    (2237, 680, 3),
    (2238, 683, 2),
    (2240, 685, 2),
    (2244, 687, 2),
    (2245, 689, 2),
    (2246, 691, 1),
    (2261, 692, 3),
    (2297, 695, 3),
    (2298, 698, 2),
    (2300, 700, 2),
    (2309, 702, 2),
    (2312, 704, 2),
    (2327, 706, 3),
    (2329, 709, 3),
    (2332, 712, 3),
    (2334, 715, 3),
    (2336, 718, 3),
    (2341, 721, 2),
    (2344, 723, 2),
    (2355, 725, 4),
    (2357, 729, 2),
    (2360, 731, 2),
    (2365, 733, 2),
    (2367, 735, 2),
    (2370, 737, 2),
    (2409, 739, 2),
    (2416, 741, 3),
    (2419, 744, 3),
    (2421, 747, 3),
    (2423, 750, 3),
    (2426, 753, 2),
    (2453, 755, 2),
    (2479, 757, 2),
    (2481, 759, 2),
    (2494, 761, 3),
    (2496, 764, 3),
    (2499, 767, 2),
    (2501, 769, 2),
    (2504, 771, 3),
    (2548, 774, 3),
    (2573, 777, 3),
    (2578, 780, 2),
    (2580, 782, 2),
    (2583, 784, 3),
    (2586, 787, 2),
    (2590, 789, 2),
    (2593, 791, 3),
    (2596, 794, 3),
    (2599, 797, 2),
    (2601, 799, 2),
    (2604, 801, 3),
    (2607, 804, 3),
    (2657, 807, 2),
    (2660, 809, 2),
    (2666, 811, 2),
    (2675, 813, 3),
    (2680, 816, 2),
    (2681, 818, 2),
    (2682, 820, 2),
    (2683, 822, 2),
    (2684, 824, 2),
    (2685, 826, 2),
    (2696, 828, 2),
    (2697, 830, 2),
    (2698, 832, 2),
    (2699, 834, 2),
    (2703, 836, 2),
    (2704, 838, 3),
    (2705, 841, 2),
    (2706, 843, 2),
    (2709, 845, 2),
    (2744, 847, 3),
    (2745, 850, 2),
    (2747, 852, 2),
    (2752, 854, 3),
    (2753, 857, 2),
    (2758, 859, 2),
    (2763, 861, 3),
    (2764, 864, 2),
    (2769, 866, 2),
    (2777, 868, 3),
    (2779, 871, 2),
    (2780, 873, 2),
    (2783, 875, 3),
    (2784, 878, 3),
    (2787, 881, 2),
    (2815, 883, 2),
    (2817, 885, 2),
    (2824, 887, 3),
    (2829, 890, 2),
    (2835, 892, 2),
    (2933, 894, 2),
    (2936, 896, 2),
    (2941, 898, 2),
    (2943, 900, 2),
    (2948, 902, 2),
    (2949, 904, 2),
    (2967, 906, 2),
    (2974, 908, 2),
    (2978, 910, 2),
    (2982, 912, 2),
    (2986, 914, 2),
    (2992, 916, 2),
    (2996, 918, 2),
    (3000, 920, 2),
    (3004, 922, 3),
    (3006, 925, 2),
    (3009, 927, 2),
    (3010, 929, 2),
    (3011, 931, 2),
    (3012, 933, 2),
    (3016, 935, 2),
    (3017, 937, 3),
    (3018, 940, 2),
    (3019, 942, 2),
    (3020, 944, 2),
    (3024, 946, 2),
    (3028, 948, 2),
    (3035, 950, 2),
    (3038, 952, 2),
    (3041, 954, 3),
    (3044, 957, 2),
    (3049, 959, 2),
    (3056, 961, 2),
    (3061, 963, 3),
    (3065, 966, 3),
    (3068, 969, 2),
    (3071, 971, 2),
    (3078, 973, 2),
    (3081, 975, 2),
    (3083, 977, 2),
    (3104, 979, 3),
    (3167, 982, 3),
    (3168, 985, 2),
    (3173, 987, 2),
    (3188, 989, 2),
    (3198, 991, 3),
    (3202, 994, 3),
    (3203, 997, 2),
    (3204, 999, 2),
    (3211, 1001, 2),
    (3222, 1003, 2),
    (3247, 1005, 2),
    (3265, 1007, 3),
    (3270, 1010, 3),
    (3271, 1013, 2),
    (3275, 1015, 3),
    (3277, 1018, 2),
    (3278, 1020, 2),
    (3283, 1022, 3),
    (3286, 1025, 3),
    (3291, 1028, 3),
    (3295, 1031, 3),
    (3300, 1034, 3),
    (3305, 1037, 2),
    (3306, 1039, 2),
    (3311, 1041, 2),
    (3315, 1043, 3),
    (3323, 1046, 2),
    (3324, 1048, 2),
    (3333, 1050, 2),
    (3335, 1052, 3),
    (3337, 1055, 2),
    (3338, 1057, 2),
    (3341, 1059, 2),
    (3342, 1061, 2),
    (3343, 1063, 2),
    (3347, 1065, 2),
    (3352, 1067, 2),
    (3360, 1069, 3),
    (3363, 1072, 3),
    (3366, 1075, 2),
    (3369, 1077, 2),
    (3374, 1079, 2),
    (3381, 1081, 2),
    (3384, 1083, 3),
    (3387, 1086, 2),
    (3393, 1088, 2),
    (3399, 1090, 2),
    (3408, 1092, 3),
    (3414, 1095, 2),
    (3430, 1097, 3),
    (3431, 1100, 2),
    (3436, 1102, 2),
    (3437, 1104, 2),
    (3441, 1106, 2),
    (3448, 1108, 2),
    (3451, 1110, 3),
    (3454, 1113, 2),
    (3457, 1115, 2),
    (3460, 1117, 2),
    (3532, 1119, 2),
    (3533, 1121, 2),
    (3536, 1123, 2),
    (3541, 1125, 2),
    (3551, 1127, 2),
    (3552, 1129, 2),
    (3558, 1131, 2),
    (3560, 1133, 3),
    (3578, 1136, 2),
    (3586, 1138, 2),
    (3591, 1140, 2),
    (3596, 1142, 2),
    (3597, 1144, 2),
    (3599, 1146, 2),
    (3603, 1148, 3),
    (3612, 1151, 2),
    (3615, 1153, 2),
    (3620, 1155, 2),
    (3626, 1157, 3),
    (3627, 1160, 2),
    (3630, 1162, 2),
    (3634, 1164, 2),
    (3640, 1166, 3),
    (3682, 1169, 3),
    (3686, 1172, 3),
    (3689, 1175, 3),
    (3692, 1178, 3),
    (3695, 1181, 3),
    (3705, 1184, 3),
    (3708, 1187, 3),
    (3711, 1190, 3),
    (3716, 1193, 2),
    (3756, 1195, 3),
    (3762, 1198, 3),
    (3764, 1201, 2),
    (3769, 1203, 3),
    (3775, 1206, 2),
    (3777, 1208, 2),
    (3784, 1210, 3),
    (3795, 1213, 3),
    (3796, 1216, 2),
    (3798, 1218, 3),
    (3802, 1221, 3),
    (3807, 1224, 3),
    (3810, 1227, 2),
    (3819, 1229, 2),
    (3822, 1231, 3),
    (3828, 1234, 2),
    (3839, 1236, 3),
    (3844, 1239, 3),
    (3845, 1242, 2),
    (3847, 1244, 2),
    (3848, 1246, 2),
    (3849, 1248, 2),
    (3852, 1250, 3),
    (3854, 1253, 2),
    (3861, 1255, 2),
    (3867, 1257, 2),
    (3872, 1259, 2),
    (3873, 1261, 2),
    (3876, 1263, 2),
    (3877, 1265, 2),
    (3883, 1267, 3),
    (3900, 1270, 3),
    (3905, 1273, 3),
    (3909, 1276, 3),
    (3930, 1279, 3),
    (3937, 1282, 3),
    (3947, 1285, 2),
    (3953, 1287, 2),
    (3957, 1289, 2),
    (3979, 1291, 3),
    (3984, 1294, 3),
    (4007, 1297, 2),
    (4012, 1299, 2),
    (4022, 1301, 3),
    (4028, 1304, 2),
    (4040, 1306, 2),
    (4041, 1308, 2),
    (4043, 1310, 2),
    (4122, 1312, 2),
    (4134, 1314, 2),
    (4137, 1316, 2),
    (4153, 1318, 2),
    (4169, 1320, 2),
    (4191, 1322, 2),
    (4198, 1324, 2),
    (4218, 1326, 2),
    (4222, 1328, 2),
    (4231, 1330, 3),
    (4233, 1333, 2),
    (4238, 1335, 2),
    (4239, 1337, 2),
    (4240, 1339, 2),
    (4246, 1341, 2),
    (4247, 1343, 2),
    (4254, 1345, 2),
    (4255, 1347, 2),
    (4270, 1349, 2),
    (4283, 1351, 2),
    (4321, 1353, 2),
    (4325, 1355, 2),
    (4337, 1357, 2),
    (4339, 1359, 2),
    (4351, 1361, 2),
    (4353, 1363, 2),
    (4359, 1365, 2),
    (4365, 1367, 3),
    (4373, 1370, 2),
    (4379, 1372, 2),
    (4385, 1374, 3),
    (4393, 1377, 3),
    (4401, 1380, 2),
    (4402, 1382, 2),
    (4410, 1384, 3),
    (4419, 1387, 3),
    (4425, 1390, 2),
    (4435, 1392, 3),
    (4441, 1395, 2),
    (4443, 1397, 3),
    (4451, 1400, 2),
    (4452, 1402, 2),
    (4458, 1404, 3),
    (4465, 1407, 2),
    (4478, 1409, 3),
    (4485, 1412, 3),
    (4492, 1415, 3),
    (4500, 1418, 3),
    (4510, 1421, 3),
    (4518, 1424, 2),
    (4524, 1426, 3),
    (4526, 1429, 3),
    (4533, 1432, 2),
    (4539, 1434, 3),
    (4545, 1437, 3),
    (4555, 1440, 2),
    (4560, 1442, 2),
    (4561, 1444, 2),
    (4598, 1446, 2),
    (4606, 1448, 2),
    (4609, 1450, 2),
    (4644, 1452, 2),
    (4647, 1454, 2),
    (4648, 1456, 2),
    (4649, 1458, 2),
    (4683, 1460, 2),
    (4700, 1462, 2),
    (4713, 1464, 2),
    (4715, 1466, 2),
    (4716, 1468, 2),
    (4720, 1470, 2),
    (4724, 1472, 2),
    (4728, 1474, 2),
    (4733, 1476, 3),
    (4734, 1479, 2),
    (4735, 1481, 2),
    (4736, 1483, 2),
    (4738, 1485, 3),
    (4741, 1488, 2),
    (4742, 1490, 2),
    (4745, 1492, 2),
    (4747, 1494, 2),
    (4753, 1496, 2),
    (4754, 1498, 2),
    (4759, 1500, 2),
    (4761, 1502, 2),
    (4762, 1504, 2),
    (4763, 1506, 2),
    (4764, 1508, 3),
    (4765, 1511, 2),
    (4769, 1513, 3),
    (4774, 1516, 3),
    (4776, 1519, 3),
    (4791, 1522, 2),
    (4813, 1524, 2),
    (4817, 1526, 2),
    (4818, 1528, 2),
    (4819, 1530, 2),
    (4821, 1532, 2),
    (4831, 1534, 2),
    (4860, 1536, 3),
    (4879, 1539, 2),
    (4887, 1541, 2),
    (4894, 1543, 2),
    (4896, 1545, 2),
    (4898, 1547, 2),
    (4903, 1549, 2),
    (4908, 1551, 2),
    (4912, 1553, 2),
    (4930, 1555, 2),
    (4932, 1557, 3),
    (4962, 1560, 2),
    (4965, 1562, 2),
    (4975, 1564, 2),
    (4976, 1566, 2),
    (4977, 1568, 2),
    (4980, 1570, 2),
    (4987, 1572, 2),
    (4994, 1574, 2),
    (4998, 1576, 2),
    (5004, 1578, 2),
    (5005, 1580, 2),
    (5006, 1586, 2),
    (5009, 1582, 2),
    (5011, 1584, 2),
    (5015, 1588, 2),
    (5016, 1590, 2),
    (5021, 1592, 2),
    (5026, 1594, 2),
    (5027, 1596, 2),
    (5029, 1598, 2),
    (5040, 1600, 2),
    (5060, 1602, 2),
    (5064, 1604, 2),
    (5071, 1606, 2),
    (5075, 1608, 2),
    (5084, 1610, 2),
    (5085, 1612, 2),
    (5087, 1614, 3),
    (5092, 1617, 2),
    (5093, 1619, 2),
    (5103, 1621, 3),
    (5105, 1624, 3),
    (5110, 1627, 3),
    (5111, 1630, 2),
    (5117, 1632, 2),
    (5118, 1634, 2),
    (5122, 1636, 2),
    (5123, 1638, 2),
    (5125, 1640, 2),
    (5128, 1642, 2),
    (5130, 1644, 2),
    (5132, 1646, 2),
    (5135, 1648, 2),
    (5138, 1650, 2),
    (5142, 1652, 2),
    (5146, 1654, 2),
    (5149, 1656, 2),
    (5152, 1658, 2),
    (5159, 1660, 2),
    (5167, 1662, 3),
    (5168, 1665, 2),
    (5173, 1667, 3),
    (5174, 1670, 2),
    (5176, 1672, 2),
    (5177, 1674, 2),
    (5180, 1676, 2),
    (5184, 1678, 2),
    (5190, 1680, 2),
    (5245, 1682, 2),
    (5263, 1684, 2),
    (5269, 1686, 2),
    (5272, 1688, 2),
    (5276, 1690, 2),
    (5277, 1692, 2),
    (5278, 1694, 2),
    (5282, 1696, 2),
    (5297, 1698, 2),
    (5301, 1700, 2),
    (5309, 1702, 2),
    (5318, 1704, 2),
    (5319, 1706, 2),
    (5323, 1708, 2),
    (5325, 1710, 2),
    (5332, 1712, 2),
    (5333, 1714, 2),
    (5337, 1722, 2),
    (5339, 1716, 2),
    (5340, 1718, 2),
    (5341, 1720, 2),
    (5344, 1730, 2),
    (5346, 1724, 2),
    (5347, 1726, 2),
    (5348, 1728, 2),
    (5349, 1732, 2),
    (5352, 1734, 2),
    (5353, 1736, 2),
    (5375, 1738, 2),
    (5377, 1740, 2),
    (5380, 1742, 3),
    (5385, 1745, 3),
    (5386, 1748, 2),
    (5391, 1750, 2),
    (5394, 1752, 2),
    (5399, 1754, 2),
    (5419, 1756, 3),
    (5427, 1759, 2),
    (5456, 1761, 3),
    (5458, 1764, 2),
    (5459, 1766, 2),
    (5460, 1768, 2),
    (5469, 1770, 2),
    (5474, 1772, 2),
    (5483, 1774, 3),
    (5485, 1777, 2),
    (5495, 1779, 2),
    (5496, 1781, 2),
    (5505, 1783, 2),
    (5510, 1785, 2),
    (5533, 1787, 3),
    (5564, 1790, 3),
    (5576, 1793, 3),
    (5578, 1796, 3),
    (5592, 1799, 2),
    (5607, 1801, 3),
    (5609, 1804, 3),
    (5615, 1807, 2),
    (5626, 1809, 3),
    (5627, 1812, 2),
    (5629, 1814, 2),
    (5630, 1816, 2),
    (5638, 1818, 2),
    (5643, 1820, 3),
    (5660, 1823, 3),
    (5671, 1826, 2),
    (5717, 1828, 3),
    (5746, 1831, 2),
    (5751, 1833, 2),
    (5753, 1835, 2),
    (5759, 1837, 2),
    (5760, 1839, 2),
    (5763, 1841, 2),
    (5769, 1843, 2),
    (5770, 1845, 2),
    (5771, 1847, 2),
    (5777, 1849, 2),
    (5826, 1851, 4),
    (5843, 1855, 2),
    (5844, 1857, 2),
    (5886, 1859, 2),
    (5893, 1861, 2),
    (5911, 1863, 2),
    (5913, 1865, 2),
    (5914, 1867, 2),
    (5915, 1869, 2),
    (5916, 1871, 2),
    (5919, 1873, 5),
    (5920, 1878, 3),
    (5937, 1881, 3),
    (5956, 1884, 2),
    (5969, 1886, 2),
    (5973, 1888, 3),
    (5981, 1891, 3),
    (6010, 1894, 2),
    (6016, 1896, 2),
    (6017, 1898, 2),
    (6023, 1900, 2),
    (6026, 1902, 2),
    (6027, 1904, 2),
    (6036, 1906, 2),
    (6039, 1908, 2),
    (6040, 1910, 2),
    (6050, 1912, 2),
    (6052, 1914, 2),
    (6056, 1916, 3),
    (6060, 1919, 3),
    (6065, 1922, 2),
    (6067, 1924, 2),
    (6068, 1926, 2),
    (6070, 1928, 2),
    (6073, 1930, 2),
    (6079, 1932, 2),
    (6133, 1934, 2),
    (6134, 1936, 2),
    (6141, 1938, 2),
    (6145, 1940, 2),
    (6162, 1942, 3),
    (6180, 1945, 2),
    (6181, 1947, 2),
    (6183, 1949, 2),
    (6185, 1951, 2),
    (6192, 1953, 2),
    (6195, 1955, 2),
    (6197, 1957, 2),
    (6198, 1959, 2),
    (6199, 1961, 2),
    (6259, 1963, 2),
    (6280, 1965, 2),
    (6296, 1967, 2),
    (6333, 1969, 3),
    (6335, 1972, 2),
    (6347, 1974, 2),
    (6348, 1976, 2),
    (6360, 1978, 2),
    (6378, 1980, 2),
    (6389, 1982, 3),
    (6390, 1985, 2),
    (6391, 1987, 2),
    (6395, 1989, 3),
    (6396, 1992, 2),
    (6397, 1994, 2),
    (6399, 1996, 3),
    (6405, 1999, 3),
    (6406, 2002, 2),
    (6407, 2004, 2),
    (6411, 2006, 3),
    (6412, 2009, 2),
    (6413, 2011, 2),
    (6415, 2013, 3),
    (6443, 2016, 2),
    (6449, 2018, 2),
    (6456, 2020, 2),
    (6474, 2022, 2),
    (6482, 2024, 3),
    (6483, 2027, 2),
    (6529, 2029, 2),
    (6530, 2031, 2),
    (6547, 2033, 2),
    (6550, 2035, 2),
    (6575, 2037, 2),
    (6606, 2039, 2),
    (6618, 2041, 3),
    (6632, 2044, 2),
    (6651, 2046, 2),
    (6728, 2048, 3),
    (6734, 2051, 3),
    (6743, 2054, 3),
    (6746, 2057, 3),
    (6747, 2060, 3),
    (6751, 2063, 2),
    (6757, 2065, 3),
    (6787, 2068, 3),
    (6798, 2071, 3),
    (6799, 2074, 2),
    (6823, 2076, 3),
    (6831, 2079, 3),
    (6835, 2082, 3),
    (6839, 2085, 3),
    (6845, 2088, 3),
    (6853, 2091, 2),
    (6857, 2093, 2),
    (6905, 2095, 2),
    (6907, 2097, 2),
    (6910, 2099, 3),
    (6915, 2102, 3),
    (6916, 2105, 3),
    (6918, 2108, 2),
    (6992, 2110, 2),
    (6997, 2112, 3),
    (7000, 2115, 3),
    (7003, 2118, 2),
    (7005, 2120, 2),
    (7008, 2122, 3),
    (7011, 2125, 2),
    (7015, 2127, 2),
    (7019, 2129, 3),
    (7022, 2132, 2),
    (7033, 2134, 3),
    (7037, 2137, 2),
    (7040, 2139, 2),
    (7046, 2141, 2),
    (7049, 2143, 3),
    (7052, 2146, 2),
    (7058, 2148, 3),
    (7061, 2151, 2),
    (7064, 2153, 2),
    (7067, 2155, 2),
    (7070, 2157, 2),
    (7073, 2159, 3),
    (7080, 2162, 2),
    (7083, 2164, 2),
    (7091, 2166, 2),
    (7098, 2168, 2),
    (7100, 2170, 2),
    (7113, 2172, 2),
    (7118, 2174, 2),
    (7131, 2176, 2),
    (7135, 2178, 2),
    (7136, 2180, 2),
    (7137, 2182, 2),
    (7138, 2184, 2),
    (7139, 2186, 2),
    (7140, 2188, 2),
    (7141, 2190, 2),
    (7142, 2192, 2),
    (7143, 2194, 2),
    (7144, 2196, 2),
    (7145, 2198, 2),
    (7146, 2200, 2),
    (7147, 2202, 2),
    (7148, 2204, 2),
    (7149, 2206, 2),
    (7150, 2208, 2),
    (7151, 2210, 2),
    (7152, 2212, 2),
    (7157, 2214, 2),
    (7159, 2216, 2),
    (7163, 2218, 2),
    (7166, 2220, 2),
    (7175, 2222, 2),
    (7180, 2224, 2),
    (7217, 2226, 2),
    (7221, 2228, 3),
    (7222, 2231, 2),
    (7223, 2233, 2),
    (7227, 2235, 2),
    (7228, 2237, 3),
    (7229, 2240, 2),
    (7232, 2242, 2),
    (7236, 2244, 3),
    (7237, 2247, 2),
    (7246, 2249, 2),
    (7247, 2251, 3),
    (7248, 2254, 2),
    (7251, 2256, 2),
    (7258, 2258, 2),
    (7266, 2260, 2),
    (7268, 2262, 2),
    (7269, 2264, 2),
    (7270, 2266, 2),
    (7281, 2268, 3),
    (7293, 2271, 3),
    (7305, 2274, 2),
    (7328, 2276, 3),
    (7337, 2279, 2),
    (7341, 2281, 2),
    (7342, 2283, 2),
    (7355, 2285, 2),
    (7393, 2287, 2),
    (7394, 2289, 2),
    (7425, 2291, 2),
    (7426, 2293, 2),
    (7427, 2295, 2),
    (7428, 2297, 2),
    (7429, 2299, 2),
    (7430, 2301, 2),
    (7431, 2303, 2),
    (7432, 2305, 2),
    (7433, 2307, 2),
    (7434, 2309, 2),
    (7435, 2311, 2),
    (7436, 2313, 2),
    (7437, 2315, 2),
    (7438, 2317, 2),
    (7439, 2319, 2),
    (7440, 2321, 2),
    (7441, 2323, 2),
    (7442, 2325, 2),
    (7443, 2327, 2),
    (7444, 2329, 2),
    (7445, 2331, 2),
    (7446, 2333, 2),
    (7447, 2335, 2),
    (7448, 2337, 2),
    (7449, 2339, 2),
    (7450, 2341, 2),
    (7451, 2343, 2),
    (7452, 2345, 2),
    (7453, 2347, 2),
    (7454, 2349, 2),
    (7455, 2351, 2),
    (7456, 2353, 2),
    (7457, 2355, 2),
    (7458, 2357, 2),
    (7459, 2359, 2),
    (7460, 2361, 2),
    (7461, 2363, 2),
    (7462, 2365, 2),
    (7463, 2367, 2),
    (7464, 2369, 2),
    (7465, 2371, 2),
    (7466, 2373, 2),
    (7467, 2375, 2),
    (7468, 2377, 2),
    (7469, 2379, 2),
    (7470, 2381, 2),
    (7471, 2383, 2),
    (7472, 2385, 2),
    (7473, 2387, 2),
    (7474, 2389, 2),
    (7475, 2391, 2),
    (7476, 2393, 2),
    (7477, 2395, 2),
    (7478, 2397, 2),
    (7479, 2399, 2),
    (7480, 2401, 2),
    (7481, 2403, 2),
    (7482, 2405, 2),
    (7483, 2407, 2),
    (7484, 2409, 2),
    (7485, 2411, 2),
    (7486, 2413, 2),
    (7487, 2415, 2),
    (7488, 2417, 2),
    (7489, 2419, 2),
    (7490, 2421, 2),
    (7491, 2423, 2),
    (7492, 2425, 2),
    (7493, 2427, 2),
    (7494, 2429, 2),
    (7495, 2431, 2),
    (7496, 2433, 2),
    (7497, 2435, 2),
    (7498, 2437, 2),
    (7499, 2439, 2),
    (7500, 2441, 2),
    (7501, 2443, 2),
    (7502, 2445, 2),
    (7503, 2447, 2),
    (7504, 2449, 2),
    (7505, 2451, 2),
    (7506, 2453, 2),
    (7507, 2455, 2),
    (7508, 2457, 2),
    (7509, 2459, 2),
    (7510, 2461, 2),
    (7511, 2463, 2),
    (7512, 2465, 2),
    (7513, 2467, 2),
    (7514, 2469, 2),
    (7515, 2471, 2),
    (7516, 2473, 2),
    (7517, 2475, 2),
    (7518, 2477, 2),
    (7519, 2479, 2),
    (7520, 2481, 2),
    (7521, 2483, 2),
    (7522, 2485, 2),
    (7523, 2487, 2),
    (7524, 2489, 2),
    (7525, 2491, 2),
    (7526, 2493, 2),
    (7527, 2495, 2),
    (7528, 2497, 2),
    (7529, 2499, 2),
    (7530, 2501, 2),
    (7531, 2503, 2),
    (7532, 2505, 2),
    (7533, 2507, 2),
    (7534, 2509, 2),
    (7535, 2511, 2),
    (7536, 2513, 2),
    (7537, 2515, 2),
    (7538, 2517, 2),
    (7539, 2519, 2),
    (7540, 2521, 2),
    (7541, 2523, 2),
    (7542, 2525, 2),
    (7543, 2527, 2),
    (7544, 2529, 2),
    (7545, 2531, 2),
    (7546, 2533, 2),
    (7547, 2535, 2),
    (7548, 2537, 2),
    (7549, 2539, 2),
    (7550, 2541, 2),
    (7551, 2543, 2),
    (7552, 2545, 2),
    (7553, 2547, 2),
    (7554, 2549, 2),
    (7555, 2551, 2),
    (7556, 2553, 2),
    (7557, 2555, 2),
    (7558, 2557, 2),
    (7559, 2559, 2),
    (7560, 2561, 2),
    (7561, 2563, 2),
    (7562, 2565, 2),
    (7563, 2567, 2),
    (7564, 2569, 2),
    (7565, 2571, 2),
    (7566, 2573, 2),
    (7567, 2575, 2),
    (7568, 2577, 2),
    (7569, 2579, 2),
    (7570, 2581, 2),
    (7585, 2583, 2),
    (7593, 2585, 2),
    (7608, 2587, 3),
    (7628, 2590, 3),
    (7637, 2593, 3),
    (7647, 2596, 2),
    (7653, 2598, 3),
    (7659, 2601, 3),
    (7665, 2604, 3),
    (7667, 2607, 3),
    (7670, 2610, 3),
    (7674, 2613, 2),
    (7681, 2615, 3),
    (7696, 2618, 2),
    (7698, 2620, 2),
    (7699, 2622, 2),
    (7700, 2624, 2),
    (7701, 2626, 3),
    (7705, 2629, 2),
    (7752, 2631, 2),
    (7764, 2633, 2),
    (7841, 2635, 3),
    (7843, 2638, 3),
    (7863, 2641, 2),
    (7864, 2643, 2),
    (7866, 2645, 2),
    (7867, 2647, 2),
    (7907, 2649, 2),
    (7986, 2651, 2),
    (7987, 2653, 2),
    (7988, 2655, 2),
    (7989, 2657, 2),
    (7990, 2659, 2),
    (7991, 2661, 2),
    (7992, 2663, 2),
    (7993, 2665, 2),
    (7994, 2667, 2),
    (7995, 2669, 2),
    (7996, 2671, 2),
    (7997, 2673, 2),
    (7998, 2675, 2),
    (8003, 2677, 2),
    (8015, 2679, 3),
    (8016, 2682, 2),
    (8018, 2684, 2),
    (8019, 2686, 2),
    (8025, 2688, 2),
    (8026, 2690, 2),
    (8033, 2692, 2),
    (8034, 2694, 2),
    (8041, 2696, 3),
    (8068, 2699, 3),
    (8085, 2702, 2),
    (8087, 2704, 2),
    (8088, 2706, 2),
    (8089, 2708, 2),
    (8117, 2710, 2),
    (8121, 2712, 2),
    (8130, 2714, 2),
    (8147, 2716, 2),
    (8150, 2718, 3),
    (8153, 2721, 3),
    (8154, 2724, 2),
    (8171, 2726, 2),
    (8172, 2728, 2),
    (8173, 2730, 2),
    (8186, 2732, 2),
    (8195, 2734, 2),
    (8215, 2736, 2),
    (8218, 2738, 3),
];

pub static SEGMENT_CLASS_TYPES_DATA: &[u32] = &[
    69, 3, 80, 3, 80, 3, 3, 82, 3, 84, 3, 87, 3, 82, 89, 69,
    3, 166, 3, 3, 196, 3, 198, 3, 82, 3, 82, 202, 3, 82, 204, 3,
    82, 3, 82, 204, 3, 208, 215, 3, 3, 217, 3, 235, 3, 236, 238, 4,
    241, 3, 3, 244, 3, 252, 3, 253, 236, 238, 4, 3, 82, 256, 258, 3,
    3, 82, 3, 87, 3, 84, 3, 82, 204, 3, 82, 280, 3, 282, 82, 3,
    282, 82, 3, 82, 3, 82, 295, 3, 298, 331, 3, 334, 3, 336, 3, 331,
    3, 334, 3, 336, 3, 3, 82, 3, 82, 280, 349, 3, 351, 3, 3, 82,
    204, 3, 82, 3, 355, 3, 82, 3, 355, 3, 82, 3, 82, 3, 82, 3,
    82, 3, 82, 3, 82, 3, 82, 3, 373, 82, 3, 82, 280, 3, 82, 3,
    82, 256, 3, 82, 3, 82, 3, 82, 3, 84, 3, 87, 3, 82, 3, 252,
    3, 252, 3, 82, 3, 412, 3, 427, 82, 3, 252, 3, 431, 82, 3, 435,
    3, 427, 82, 3, 427, 82, 3, 252, 3, 431, 82, 3, 355, 3, 208, 3,
    355, 463, 3, 463, 3, 3, 468, 3, 468, 3, 480, 3, 480, 3, 82, 295,
    3, 468, 3, 468, 3, 82, 256, 3, 519, 82, 3, 427, 82, 3, 82, 256,
    527, 528, 3, 3, 427, 82, 3, 427, 82, 3, 82, 256, 3, 298, 3, 84,
    3, 87, 565, 3, 3, 84, 3, 82, 280, 3, 373, 82, 3, 82, 3, 82,
    3, 82, 3, 519, 82, 3, 519, 82, 3, 82, 295, 3, 519, 82, 3, 373,
    82, 3, 298, 3, 82, 295, 3, 82, 204, 3, 82, 295, 3, 427, 82, 3,
    298, 3, 298, 3, 82, 3, 82, 256, 576, 3, 3, 427, 82, 3, 427, 82,
    3, 82, 256, 3, 82, 280, 3, 82, 256, 3, 373, 82, 3, 82, 295, 3,
    82, 3, 82, 295, 3, 82, 3, 82, 256, 3, 82, 3, 82, 204, 3, 355,
    3, 355, 3, 298, 3, 84, 3, 578, 3, 84, 3, 82, 582, 3, 584, 3,
    586, 3, 586, 3, 586, 3, 82, 582, 3, 82, 204, 3, 82, 204, 3, 82,
    204, 3, 82, 204, 3, 82, 204, 3, 282, 82, 3, 298, 3, 82, 280, 3,
    82, 280, 3, 82, 204, 3, 252, 3, 82, 629, 3, 427, 82, 3, 82, 629,
    463, 3, 3, 82, 629, 3, 82, 280, 3, 82, 636, 3, 82, 636, 3, 82,
    280, 3, 82, 640, 3, 82, 582, 3, 584, 3, 584, 3, 584, 3, 355, 3,
    82, 3, 82, 640, 3, 298, 3, 480, 3, 427, 82, 3, 427, 82, 3, 252,
    3, 427, 82, 3, 427, 82, 3, 427, 82, 3, 427, 82, 3, 252, 3, 355,
    3, 431, 82, 3, 435, 3, 427, 82, 3, 82, 256, 3, 298, 3, 688, 3,
    427, 82, 3, 82, 256, 3, 82, 256, 528, 3, 3, 427, 82, 3, 427, 82,
    3, 82, 256, 3, 82, 256, 3, 82, 280, 3, 698, 3, 82, 295, 3, 82,
    3, 82, 295, 3, 82, 3, 82, 3, 82, 3, 82, 3, 82, 3, 82, 3,
    82, 3, 82, 3, 82, 3, 82, 3, 82, 3, 82, 3, 82, 3, 82, 3,
    82, 709, 3, 82, 256, 3, 82, 709, 3, 373, 82, 3, 252, 3, 82, 3,
    82, 3, 82, 280, 3, 427, 82, 3, 252, 3, 431, 82, 3, 427, 82, 3,
    252, 3, 431, 82, 3, 427, 82, 3, 427, 82, 3, 427, 82, 3, 82, 204,
    3, 82, 3, 727, 3, 82, 3, 298, 3, 82, 3, 82, 256, 3, 427, 82,
    3, 208, 3, 355, 3, 82, 3, 427, 82, 3, 427, 82, 3, 82, 256, 3,
    82, 280, 3, 298, 3, 82, 3, 82, 256, 3, 82, 280, 3, 84, 3, 82,
    280, 3, 298, 3, 298, 3, 82, 280, 3, 82, 256, 3, 427, 82, 3, 82,
    256, 3, 427, 82, 3, 751, 3, 751, 3, 751, 3, 751, 3, 355, 3, 355,
    3, 82, 241, 3, 3, 822, 3, 208, 3, 427, 82, 3, 355, 3, 252, 3,
    824, 3, 825, 3, 3, 751, 4, 3, 427, 82, 3, 869, 3, 872, 3, 244,
    3, 878, 3, 427, 82, 3, 427, 82, 3, 427, 82, 3, 427, 82, 3, 427,
    82, 3, 208, 3, 208, 3, 236, 238, 4, 3, 893, 3, 899, 3, 355, 3,
    893, 3, 899, 3, 82, 3, 82, 256, 3, 519, 82, 3, 519, 82, 3, 82,
    256, 3, 4, 3, 355, 3, 949, 3, 208, 3, 82, 256, 3, 427, 82, 3,
    252, 3, 435, 3, 431, 82, 3, 427, 82, 3, 82, 256, 3, 82, 3, 82,
    3, 427, 82, 3, 82, 3, 82, 3, 282, 82, 3, 373, 82, 3, 84, 3,
    87, 3, 519, 82, 3, 82, 280, 3, 82, 3, 82, 3, 82, 3, 982, 4,
    3, 298, 3, 298, 3, 298, 3, 298, 3, 298, 3, 298, 3, 298, 3, 298,
    3, 298, 3, 298, 3, 1003, 3, 298, 4, 3, 751, 3, 298, 3, 355, 3,
    427, 82, 3, 298, 3, 355, 3, 427, 82, 3, 298, 3, 355, 3, 427, 82,
    3, 298, 3, 355, 3, 427, 82, 3, 355, 3, 208, 3, 431, 82, 3, 82,
    1028, 3, 355, 3, 252, 3, 355, 3, 82, 256, 1075, 3, 3, 1079, 3, 82,
    3, 87, 3, 252, 3, 252, 3, 84, 3, 87, 3, 82, 3, 298, 3, 298,
    3, 298, 3, 298, 3, 298, 3, 298, 3, 82, 3, 282, 82, 3, 1145, 3,
    1148, 1150, 3, 3, 1152, 3, 1154, 3, 298, 3, 82, 204, 3, 727, 3, 1156,
    3, 1158, 3, 82, 3, 298, 3, 82, 3, 252, 3, 431, 82, 3, 355, 3,
    82, 3, 355, 3, 373, 82, 3, 82, 280, 3, 1160, 3, 1160, 3, 84, 3,
    252, 3, 252, 3, 82, 280, 3, 82, 256, 3, 1201, 3, 1207, 3, 298, 3,
    82, 256, 3, 427, 82, 3, 252, 3, 1201, 3, 1207, 3, 298, 3, 298, 3,
    82, 256, 3, 427, 82, 3, 252, 3, 431, 82, 3, 435, 3, 480, 3, 82,
    256, 3, 82, 1239, 3, 82, 256, 3, 82, 256, 3, 427, 82, 3, 435, 3,
    480, 3, 698, 3, 82, 1239, 3, 84, 3, 87, 3, 252, 3, 427, 82, 3,
    252, 3, 1242, 3, 252, 3, 252, 3, 1248, 3, 82, 3, 82, 3, 519, 82,
    3, 82, 256, 3, 412, 3, 1250, 3, 1250, 3, 468, 3, 82, 295, 3, 355,
    3, 82, 1075, 3, 3, 82, 256, 3, 480, 3, 82, 256, 3, 1265, 3, 208,
    3, 1267, 3, 82, 3, 82, 3, 427, 82, 3, 82, 3, 252, 3, 252, 3,
    298, 3, 82, 3, 298, 3, 298, 3, 298, 3, 84, 3, 82, 3, 82, 256,
    3, 82, 3, 355, 3, 355, 3, 84, 3, 578, 3, 1248, 3, 82, 582, 3,
    586, 3, 584, 3, 82, 3, 427, 82, 3, 252, 258, 3, 3, 355, 3, 82,
    204, 3, 82, 204, 3, 82, 204, 3, 82, 204, 3, 82, 204, 3, 82, 204,
    3, 82, 280, 3, 82, 204, 3, 82, 204, 3, 252, 3, 427, 82, 3, 82,
    629, 3, 1326, 3, 82, 1239, 3, 1160, 3, 82, 3, 82, 636, 3, 427, 82,
    3, 355, 3, 82, 256, 3, 82, 640, 3, 82, 582, 3, 584, 3, 82, 3,
    427, 82, 3, 480, 3, 82, 256, 3, 427, 82, 3, 252, 3, 1334, 3, 435,
    3, 480, 3, 82, 256, 3, 1340, 3, 480, 3, 82, 3, 298, 3, 82, 3,
    298, 3, 82, 3, 82, 709, 3, 427, 82, 3, 82, 256, 3, 82, 256, 3,
    82, 256, 3, 82, 256, 3, 355, 3, 208, 3, 252, 3, 82, 1239, 3, 82,
    204, 3, 82, 3, 727, 3, 82, 256, 3, 1362, 3, 84, 3, 84, 3, 1370,
    3, 1420, 3, 1420, 3, 1433, 3, 1420, 3, 1433, 3, 82, 3, 82, 3, 298,
    3, 1562, 3, 82, 256, 258, 3, 3, 1567, 258, 3, 3, 1569, 3, 82, 3,
    1571, 3, 355, 258, 3, 3, 82, 3, 82, 3, 1605, 3, 1605, 3, 82, 3,
    87, 3, 252, 3, 252, 3, 82, 3, 282, 82, 3, 82, 3, 82, 3, 373,
    82, 3, 82, 256, 3, 82, 3, 87, 3, 519, 82, 3, 82, 256, 3, 82,
    3, 82, 204, 3, 82, 3, 82, 256, 3, 84, 3, 578, 3, 82, 582, 3,
    82, 3, 82, 280, 3, 82, 629, 3, 82, 636, 3, 82, 640, 3, 82, 256,
    3, 82, 3, 82, 709, 3, 82, 256, 3, 252, 3, 82, 204, 3, 82, 256,
    3, 355, 3, 1617, 3, 1619, 3, 1250, 3, 82, 3, 355, 3, 1649, 3, 1651,
    3, 1653, 3, 1655, 3, 101, 3, 101, 3, 1692, 3, 1694, 3, 949, 3, 949,
    3, 208, 3, 355, 3, 427, 82, 3, 208, 3, 1699, 3, 355, 3, 82, 89,
    3, 1617, 3, 1703, 3, 355, 3, 252, 3, 1706, 3, 1699, 3, 1710, 3, 822,
    3, 1712, 1714, 3, 3, 427, 82, 3, 1716, 3, 82, 256, 3, 82, 256, 3,
    82, 256, 3, 355, 3, 82, 3, 1744, 3, 1746, 3, 1748, 3, 1750, 3, 355,
    3, 82, 256, 3, 82, 3, 355, 3, 949, 3, 208, 3, 355, 3, 355, 3,
    82, 3, 355, 3, 822, 3, 427, 82, 3, 1777, 3, 1785, 3, 1791, 3, 1362,
    3, 1267, 3, 1794, 3, 82, 3, 82, 3, 82, 3, 1567, 3, 1800, 3, 1804,
    3, 1569, 258, 3, 258, 3, 3, 1804, 3, 1569, 3, 1808, 3, 1810, 3, 1777,
    3, 1569, 3, 298, 3, 298, 3, 298, 3, 298, 3, 208, 258, 3, 3, 427,
    82, 3, 355, 258, 3, 3, 427, 82, 3, 427, 82, 3, 427, 82, 258, 3,
    3, 355, 3, 1823, 3, 355, 3, 1823, 3, 355, 3, 872, 3, 355, 3, 252,
    3, 355, 3, 355, 3, 355, 3, 355, 3, 355, 3, 252, 1830, 3, 3, 427,
    82, 3, 355, 3, 427, 82, 3, 355, 3, 1837, 3, 1712, 3, 355, 3, 355,
    3, 1605, 3, 298, 3, 355, 3, 1651, 3, 1653, 3, 1843, 3, 872, 3, 1710,
    3, 355, 3, 869, 3, 869, 3, 252, 3, 252, 3, 1242, 3, 298, 3, 355,
    3, 84, 3, 1848, 3, 84, 3, 1265, 3, 84, 3, 84, 3, 84, 3, 1265,
    3, 84, 3, 84, 3, 1848, 3, 84, 3, 1848, 1866, 3, 1868, 3, 3, 82,
    204, 3, 82, 202, 3, 82, 1870, 3, 1870, 3, 1870, 3, 3, 82, 202, 3,
    82, 3, 427, 82, 3, 1878, 3, 1881, 3, 355, 3, 1791, 3, 1785, 3, 427,
    82, 3, 355, 3, 1878, 3, 1370, 3, 355, 3, 1785, 3, 1892, 4, 3, 82,
    280, 3, 82, 1239, 3, 82, 280, 3, 208, 3, 431, 82, 3, 82, 1908, 3,
    468, 3, 427, 82, 3, 208, 3, 355, 3, 1910, 3, 584, 3, 82, 295, 3,
    82, 256, 3, 1265, 3, 82, 256, 3, 1810, 3, 1941, 3, 1943, 3, 1810, 3,
    1941, 3, 1810, 3, 1948, 3, 1950, 258, 3, 3, 355, 3, 1, 4, 5, 3,
    355, 258, 3, 3, 355, 3, 355, 3, 101, 3, 1706, 1995, 3, 3, 1997, 3,
    1999, 3, 1, 1607, 4, 5, 3, 1607, 4, 3, 82, 256, 3, 82, 3, 82,
    3, 82, 256, 3, 82, 204, 3, 1265, 3, 2027, 3, 2029, 3, 355, 3, 2034,
    3, 2036, 3, 355, 3, 2034, 3, 2036, 3, 355, 3, 2040, 3, 82, 256, 3,
    82, 256, 258, 3, 3, 208, 258, 3, 3, 1943, 3, 2044, 3, 2047, 3, 298,
    3, 355, 3, 2070, 3, 2072, 3, 318, 4, 3, 1265, 3, 1710, 3, 1710, 3,
    1843, 3, 872, 3, 1710, 3, 1653, 3, 1651, 3, 1655, 3, 2104, 3, 82, 3,
    355, 3, 427, 82, 3, 355, 3, 2116, 3, 1785, 3, 1794, 3, 2070, 3, 427,
    82, 3, 1706, 3, 949, 3, 427, 82, 3, 1706, 3, 949, 3, 427, 82, 3,
    427, 82, 3, 1706, 3, 949, 3, 427, 82, 3, 1706, 3, 949, 3, 427, 82,
    3, 355, 3, 355, 3, 355, 3, 355, 3, 427, 82, 3, 252, 3, 2172, 3,
    2174, 3, 82, 3, 252, 3, 252, 3, 2196, 3, 82, 280, 3, 355, 3, 355,
    3, 82, 204, 3, 82, 204, 3, 82, 202, 3, 82, 204, 3, 82, 204, 3,
    82, 3, 82, 256, 3, 427, 82, 3, 82, 256, 3, 1267, 3, 82, 295, 3,
    519, 82, 3, 82, 256, 3, 82, 280, 3, 282, 82, 3, 298, 3, 2243, 1866,
    3, 1868, 3, 3, 82, 204, 3, 82, 202, 3, 82, 202, 1870, 3, 3, 82,
    3, 82, 256, 3, 427, 82, 3, 84, 3, 87, 3, 282, 82, 3, 82, 3,
    82, 3, 82, 256, 3, 84, 3, 82, 256, 3, 82, 3, 84, 3, 82, 3,
    82, 204, 3, 84, 3, 82, 280, 3, 82, 3, 82, 3, 82, 3, 82, 3,
    82, 256, 3, 2284, 258, 3, 3, 355, 3, 2288, 3, 2290, 3, 1791, 3, 2293,
    3, 1791, 3, 2295, 3, 2297, 3, 2299, 3, 1571, 3, 2301, 3, 2303, 3, 1716,
    3, 2305, 3, 2305, 3, 2307, 3, 235, 3, 1785, 3, 2309, 3, 2311, 3, 1794,
    3, 1791, 3, 1362, 3, 1267, 3, 2293, 3, 1791, 3, 1843, 3, 1843, 215, 3,
    215, 3, 3, 2320, 3, 427, 82, 215, 3, 3, 298, 3, 208, 3, 427, 82,
    3, 355, 3, 949, 3, 427, 82, 3, 298, 3, 208, 3, 427, 82, 3, 355,
    3, 949, 3, 82, 3, 1791, 3, 235, 3, 1785, 3, 2305, 3, 82, 280, 3,
    82, 204, 3, 298, 3, 82, 204, 3, 1692, 3, 252, 3, 1694, 3, 82, 2339,
    3, 215, 3, 3, 2358, 3, 1651, 3, 2360, 3, 2362, 3, 2364, 3, 2366, 3,
    2368, 3, 2370, 2372, 3, 3, 2374, 3, 2376, 3, 2378, 2380, 3, 3, 2382, 3,
    2384, 3, 2386, 3, 2388, 3, 2390, 3, 2392, 3, 2394, 3, 2396, 3, 2398, 3,
    1655, 3, 1653, 3, 2400, 3, 2402, 3, 2404, 3, 2406, 3, 2408, 3, 2410, 3,
    2412, 3, 2414, 3, 2416, 3, 2418, 2420, 3, 3, 2422, 3, 2424, 3, 2426, 2428,
    3, 3, 2430, 3, 2432, 3, 2434, 2436, 3, 3, 2438, 2440, 3, 3, 2442, 2444,
    3, 2446, 3, 3, 2448, 3, 2450, 3, 2452, 2454, 3, 3, 2456, 3, 2458, 2460,
    3, 3, 2462, 3, 2464, 2466, 3, 3, 2392, 3, 2468, 2470, 3, 3, 2398, 2472,
    3, 3, 2474, 3, 2476, 3, 2478, 3, 2480, 3, 2482, 3, 2484, 2486, 3, 3,
    2488, 3, 2490, 3, 2492, 3, 2494, 2496, 3, 3, 2498, 2500, 3, 3, 2502, 3,
    2504, 2506, 3, 3, 2508, 2510, 3, 3, 2512, 3, 2514, 2516, 3, 3, 2518, 3,
    2520, 2522, 3, 3, 2524, 2526, 3, 3, 2528, 2530, 3, 3, 2532, 3, 2534, 3,
    2536, 3, 2538, 3, 2540, 3, 2542, 3, 2544, 3, 2546, 3, 2548, 3, 2550, 3,
    2552, 2554, 3, 3, 2556, 2558, 3, 3, 2560, 3, 2562, 3, 2564, 3, 2566, 3,
    2568, 3, 2570, 2572, 3, 3, 2574, 3, 2576, 3, 2578, 3, 2580, 3, 2582, 3,
    2584, 3, 2586, 3, 2588, 2590, 3, 2592, 3, 3, 2594, 3, 2594, 3, 2597, 3,
    1079, 3, 2599, 3, 2492, 3, 2601, 3, 2603, 3, 2605, 3, 2607, 3, 2609, 3,
    2603, 3, 2612, 3, 2614, 3, 2616, 2618, 3, 2486, 3, 3, 2540, 3, 2621, 3,
    2623, 3, 2625, 3, 2627, 3, 2629, 3, 751, 258, 3, 3, 4, 247, 3, 427,
    82, 3, 427, 82, 3, 82, 3, 427, 82, 3, 427, 82, 3, 427, 82, 3,
    82, 256, 3, 427, 82, 3, 82, 3, 519, 82, 3, 1265, 3, 208, 3, 2653,
    2655, 3, 3, 82, 256, 3, 2358, 3, 1420, 3, 355, 3, 82, 256, 3, 82,
    256, 2339, 3, 1995, 3, 3, 1617, 3, 1619, 1075, 3, 3, 2295, 3, 2299, 3,
    1571, 3, 2301, 3, 2303, 3, 2307, 3, 1716, 3, 2305, 3, 1791, 3, 1362, 3,
    1267, 3, 235, 3, 1785, 3, 1791, 3, 82, 256, 258, 3, 3, 2047, 3, 2299,
    3, 82, 3, 1571, 3, 355, 258, 3, 3, 282, 82, 3, 82, 256, 3, 355,
    258, 3, 3, 235, 3, 1785, 3, 355, 3, 355, 3, 355, 3, 1794, 3, 4,
    2744, 3, 82, 89, 3, 2748, 3, 2750, 3, 235, 3, 2752, 3, 2757, 3, 2757,
    3, 235, 3, 4, 5,
];

pub static REGEX_PATTERNS: &[&str] = &[
    r#"^[+\-*/<>=~!@#%^&|`?]+$"#, // [0]
    r#"[A-Z_][A-Z0-9_]*"#, // [1]
    r#"^(NOT)$"#, // [2]
    r#"[A-Z_][A-Z0-9_$]*"#, // [3]
    r#"#?([A-Z_]+|[0-9]+[A-Z_$])[A-Z0-9_$]*"#, // [4]
    r#"^(AES128|AES256|ALL|ALLOWOVERWRITE|ANALYSE|ANALYZE|AND|ANY|APPEND|ARRAY|AS|ASC|AUTHORIZATION|AZ64|BETWEEN|BINARY|BLANKSASNULL|BOTH|BYTEDICT|CASE|CAST|CHECK|COLLATE|COLUMN|COMPROWS|COMPUPDATE|CONSTRAINT|CREATE|CREDENTIALS|CROSS|CURRENT_DATE|CURRENT_TIME|CURRENT_TIMESTAMP|CURRENT_USER|CURRENT_USER_ID|DATETIME|DEFAULT|DEFERRABLE|DEFRAG|DELIMITERS|DELTA|DELTA32K|DESC|DISABLE|DISTINCT|DO|ELSE|EMPTYASNULL|ENABLE|ENCRYPT|ENCRYPTION|END|EXCEPT|EXPLICIT_IDS|FALSE|FILLRECORD|FOR|FOREIGN|FREEZE|FROM|FULL|GLOBALDICT256|GLOBALDICT64K|GRANT|GROUP|HAVING|IDENTITY|IGNORE|IGNOREBLANKLINES|IGNOREHEADER|ILIKE|IN|INITIALLY|INNER|INTERSECT|INTO|IS|ISNULL|JOIN|LEADING|LEFT|LIKE|LIMIT|LOCALTIME|LOCALTIMESTAMP|LUN|LUNS|LZO|MINUS|MOSTLY16|MOSTLY32|MOSTLY8|NATURAL|NEW|NOT|NOTNULL|NULL|NULLS|OFF|OFFSET|OID|OLD|ON|ONLY|OPEN|OR|ORDER|OUTER|OVERLAPS|PARALLEL|PARTITION|PERCENT|PERMISSIONS|PIVOT|PLACING|PRIMARY|RAW|READRATIO|RECOVER|REFERENCES|REJECTLOG|RESORT|RESPECT|RESTORE|RIGHT|RUNLENGTH|SELECT|SESSION_USER|SIMILAR|SNAPSHOT|SOME|SYSDATE|SYSTEM|TABLE|TAG|TDES|TEXT255|TEXT32K|THEN|TIMESTAMP|TO|TOP|TRAILING|TRUE|TRUNCATECOLUMNS|UNION|UNIQUE|UNNEST|UNPIVOT|USER|USING|VERBOSE|WHEN|WHERE|WITH|WITHIN|WITHOUT)$"#, // [5]
    r#"[A-Z_][A-Z0-9_$]*|"[^"]*""#, // [6]
    r#"[!<>=~@#%^&|`?+\-*/]+"#, // [7]
    r#"(?i)'[0-9a-f]*'"#, // [8]
    r#"'([^']|'')*'"#, // [9]
    r#"'[^0-9A-Fa-f'+\-\s)]'"#, // [10]
];

pub static HINT_STRING_INDICES: &[u32] = &[
    0, 6, 7, 8, 9, 10, 11, 12, 14, 15,
    16, 17, 18, 19, 20, 21, 22, 23, 24, 25,
    26, 27, 28, 29, 30, 31, 32, 33, 34, 35,
    36, 37, 38, 39, 40, 41, 29, 44, 41, 17,
    18, 24, 25, 32, 16, 30, 37, 39, 21, 56,
    57, 57, 56, 61, 15, 64, 66, 23, 19, 20,
    22, 26, 27, 28, 31, 32, 33, 34, 35, 36,
    38, 40, 19, 20, 71, 26, 27, 72, 73, 64,
    74, 75, 76, 77, 78, 79, 17, 32, 86, 91,
    93, 95, 96, 99, 101, 15, 102, 103, 104, 105,
    106, 107, 108, 109, 110, 44, 111, 112, 113, 114,
    115, 116, 117, 118, 119, 120, 121, 122, 123, 124,
    125, 104, 17, 24, 30, 127, 41, 127, 102, 26,
    132, 106, 38, 109, 137, 138, 122, 138, 122, 15,
    102, 103, 104, 105, 106, 107, 108, 110, 44, 111,
    112, 113, 114, 115, 116, 117, 118, 119, 120, 121,
    122, 123, 124, 125, 19, 20, 22, 26, 27, 141,
    28, 31, 32, 33, 34, 35, 36, 38, 40, 141,
    123, 16, 32, 107, 146, 103, 105, 108, 110, 111,
    112, 113, 115, 114, 116, 117, 118, 119, 120, 121,
    124, 125, 168, 36, 40, 34, 22, 28, 31, 33,
    35, 27, 19, 20, 181, 77, 79, 75, 71, 72,
    73, 74, 76, 78, 193, 194, 193, 194, 199, 205,
    210, 211, 212, 213, 213, 212, 218, 219, 220, 221,
    222, 223, 224, 227, 228, 229, 230, 231, 232, 234,
    240, 248, 259, 260, 261, 262, 266, 267, 268, 269,
    268, 267, 273, 275, 277, 269, 284, 287, 288, 289,
    292, 296, 95, 93, 296, 93, 296, 300, 302, 224,
    303, 304, 305, 287, 306, 307, 308, 309, 310, 311,
    288, 312, 213, 313, 314, 315, 316, 317, 318, 101,
    319, 320, 323, 324, 326, 327, 15, 104, 105, 106,
    108, 114, 116, 119, 120, 121, 122, 71, 72, 73,
    64, 74, 77, 333, 338, 104, 105, 106, 108, 114,
    116, 119, 120, 121, 122, 341, 343, 345, 345, 343,
    333, 30, 127, 222, 352, 267, 268, 269, 353, 352,
    352, 269, 358, 307, 361, 363, 358, 307, 358, 307,
    363, 366, 368, 353, 371, 222, 352, 269, 121, 375,
    222, 352, 12, 232, 376, 377, 378, 18, 379, 21,
    22, 25, 27, 380, 381, 28, 31, 32, 33, 57,
    36, 382, 383, 39, 40, 12, 377, 378, 18, 379,
    21, 25, 381, 32, 33, 57, 382, 39, 386, 377,
    378, 379, 25, 381, 381, 382, 393, 394, 395, 396,
    397, 394, 395, 396, 397, 27, 36, 40, 376, 22,
    28, 31, 380, 405, 406, 405, 406, 410, 383, 222,
    262, 414, 415, 352, 416, 417, 418, 419, 358, 420,
    421, 267, 422, 327, 269, 353, 424, 428, 358, 307,
    432, 114, 433, 222, 352, 358, 327, 436, 269, 39,
    269, 39, 438, 439, 440, 441, 442, 275, 443, 444,
    127, 352, 358, 269, 446, 448, 449, 450, 449, 450,
    454, 240, 455, 456, 457, 458, 459, 419, 460, 436,
    33, 461, 436, 269, 436, 466, 470, 471, 19, 472,
    473, 473, 471, 472, 478, 361, 21, 432, 433, 361,
    432, 433, 482, 358, 482, 484, 487, 489, 415, 416,
    416, 415, 449, 422, 422, 494, 415, 416, 417, 419,
    417, 419, 498, 500, 502, 414, 505, 414, 506, 506,
    508, 509, 508, 509, 418, 419, 418, 420, 300, 514,
    515, 410, 517, 514, 515, 262, 421, 421, 222, 352,
    269, 314, 316, 317, 318, 319, 320, 525, 222, 262,
    414, 415, 352, 416, 417, 418, 419, 358, 420, 421,
    267, 268, 422, 327, 269, 353, 529, 530, 19, 531,
    532, 358, 533, 327, 534, 535, 502, 269, 536, 537,
    538, 539, 529, 534, 537, 529, 542, 534, 537, 531,
    536, 539, 531, 536, 539, 532, 358, 532, 19, 502,
    549, 550, 549, 550, 533, 554, 555, 556, 556, 554,
    555, 530, 535, 538, 529, 530, 563, 19, 531, 532,
    419, 358, 267, 533, 268, 327, 534, 535, 502, 269,
    536, 537, 538, 539, 563, 419, 563, 222, 352, 268,
    262, 567, 563, 419, 268, 327, 269, 567, 570, 460,
    573, 262, 414, 267, 327, 269, 327, 269, 262, 414,
    267, 268, 327, 269, 268, 93, 410, 284, 93, 410,
    284, 19, 327, 502, 269, 267, 268, 580, 222, 352,
    267, 268, 269, 36, 77, 587, 289, 588, 589, 590,
    418, 591, 592, 593, 594, 595, 596, 597, 598, 599,
    600, 601, 366, 598, 601, 601, 598, 588, 593, 588,
    593, 589, 594, 589, 594, 418, 595, 595, 591, 596,
    591, 596, 597, 600, 600, 597, 587, 592, 587, 592,
    590, 599, 590, 599, 307, 314, 316, 317, 318, 319,
    320, 618, 267, 620, 268, 620, 457, 302, 224, 459,
    419, 459, 458, 419, 458, 627, 461, 455, 456, 419,
    456, 634, 240, 455, 456, 457, 458, 459, 419, 460,
    267, 268, 436, 33, 269, 461, 32, 466, 638, 222,
    289, 415, 352, 416, 267, 641, 268, 269, 642, 641,
    642, 648, 222, 262, 649, 352, 267, 268, 269, 353,
    650, 39, 650, 652, 653, 654, 655, 656, 657, 658,
    659, 660, 661, 662, 663, 664, 667, 669, 15, 670,
    671, 672, 671, 672, 670, 677, 679, 670, 634, 649,
    682, 684, 686, 222, 110, 86, 231, 689, 231, 689,
    689, 240, 93, 567, 694, 268, 269, 300, 341, 696,
    95, 66, 284, 694, 701, 703, 222, 262, 352, 267,
    268, 269, 705, 707, 563, 419, 268, 711, 222, 262,
    352, 268, 713, 231, 715, 715, 289, 588, 717, 593,
    718, 599, 268, 327, 719, 269, 720, 717, 718, 717,
    718, 720, 554, 724, 724, 415, 314, 316, 317, 318,
    319, 320, 327, 719, 719, 730, 302, 224, 303, 304,
    305, 300, 287, 306, 307, 308, 309, 310, 311, 288,
    312, 213, 313, 314, 315, 316, 317, 318, 101, 319,
    320, 262, 267, 268, 327, 269, 732, 222, 733, 269,
    222, 733, 733, 15, 22, 32, 36, 71, 77, 735,
    736, 736, 735, 739, 741, 741, 742, 742, 745, 86,
    747, 747, 750, 753, 754, 755, 756, 66, 757, 758,
    758, 757, 761, 739, 266, 302, 224, 762, 763, 764,
    765, 227, 302, 762, 761, 765, 763, 764, 776, 778,
    303, 783, 784, 785, 786, 787, 788, 789, 790, 791,
    792, 793, 794, 795, 796, 797, 211, 798, 799, 800,
    801, 284, 211, 798, 799, 284, 799, 300, 314, 316,
    317, 318, 319, 320, 798, 284, 798, 806, 807, 800,
    801, 811, 813, 814, 652, 815, 816, 819, 821, 828,
    829, 830, 831, 832, 833, 834, 835, 761, 739, 266,
    302, 224, 762, 763, 296, 764, 750, 428, 836, 380,
    837, 765, 227, 838, 839, 840, 841, 842, 428, 227,
    750, 837, 835, 739, 224, 763, 296, 764, 836, 380,
    839, 840, 841, 842, 838, 848, 849, 850, 851, 855,
    856, 857, 858, 859, 837, 862, 288, 287, 865, 866,
    867, 306, 86, 116, 696, 873, 874, 653, 875, 876,
    27, 358, 86, 105, 108, 116, 121, 696, 284, 456,
    393, 873, 880, 880, 883, 885, 886, 887, 891, 895,
    897, 901, 902, 903, 905, 906, 907, 908, 909, 910,
    911, 912, 913, 914, 915, 916, 917, 918, 919, 920,
    921, 922, 923, 924, 925, 926, 928, 929, 901, 902,
    921, 930, 930, 835, 761, 739, 86, 266, 302, 91,
    224, 762, 937, 903, 938, 763, 296, 764, 750, 240,
    428, 836, 168, 380, 837, 939, 940, 410, 213, 765,
    227, 838, 839, 840, 841, 842, 941, 942, 943, 944,
    937, 428, 300, 667, 650, 448, 454, 679, 901, 902,
    432, 433, 433, 432, 428, 358, 307, 432, 114, 433,
    660, 654, 655, 656, 659, 657, 658, 661, 662, 663,
    664, 938, 939, 940, 973, 975, 977, 978, 979, 12,
    232, 376, 377, 424, 361, 378, 17, 18, 379, 371,
    21, 22, 517, 25, 27, 132, 28, 580, 30, 31,
    494, 32, 33, 57, 466, 638, 36, 37, 382, 119,
    39, 40, 36, 40, 12, 377, 378, 18, 379, 21,
    25, 27, 132, 580, 30, 494, 33, 57, 466, 638,
    37, 382, 119, 39, 232, 28, 31, 836, 984, 985,
    986, 835, 763, 839, 994, 842, 841, 840, 875, 1007,
    1008, 664, 1007, 1008, 1012, 1013, 1014, 514, 1016, 1017,
    1018, 1021, 1022, 515, 121, 1024, 1026, 86, 168, 1029,
    101, 1029, 1031, 1032, 1033, 1034, 1035, 1036, 1037, 1038,
    1039, 1040, 1041, 1042, 1044, 10, 11, 211, 814, 855,
    875, 745, 1012, 798, 1046, 1047, 1048, 1049, 1050, 590,
    1051, 1052, 1053, 1054, 181, 1007, 799, 1055, 1056, 1057,
    800, 1008, 1058, 801, 1059, 1060, 307, 1061, 1062, 1063,
    1064, 1065, 1066, 1067, 1068, 1069, 1070, 1071, 1072, 1073,
    284, 664, 1065, 814, 1046, 181, 1057, 1061, 1062, 1063,
    1068, 1069, 1048, 1054, 1048, 1054, 1058, 1059, 287, 1085,
    168, 1086, 288, 1086, 1085, 1070, 287, 1085, 168, 288,
    1060, 1047, 670, 314, 316, 317, 318, 319, 320, 1049,
    1050, 1099, 1100, 1101, 1102, 1102, 1099, 1100, 1101, 1051,
    1052, 1053, 1055, 1056, 1064, 1066, 1067, 1071, 670, 1116,
    1117, 314, 316, 317, 318, 319, 320, 1117, 1116, 1072,
    1073, 1122, 1123, 1124, 1125, 1126, 1127, 1128, 1129, 284,
    505, 1133, 86, 95, 1140, 1142, 95, 799, 284, 428,
    289, 1147, 267, 95, 284, 1147, 736, 269, 363, 361,
    358, 307, 1161, 1162, 1161, 1162, 1165, 56, 1167, 1168,
    1169, 1170, 1171, 1172, 1173, 1167, 1176, 1172, 1171, 1169,
    1170, 1173, 1168, 795, 796, 906, 104, 17, 799, 1065,
    32, 1183, 1184, 1184, 1186, 1189, 1190, 634, 1189, 1190,
    796, 1183, 1183, 1198, 1200, 1203, 1205, 1206, 1209, 1062,
    1210, 1062, 1210, 1214, 1203, 1215, 1215, 1218, 814, 1219,
    1061, 1062, 1063, 1068, 1210, 1063, 1068, 1061, 1219, 1226,
    1229, 1231, 363, 361, 300, 448, 358, 307, 363, 361,
    1236, 117, 118, 1243, 240, 828, 529, 530, 19, 531,
    25, 532, 358, 533, 1246, 534, 535, 502, 269, 536,
    537, 538, 383, 539, 1247, 284, 127, 284, 1251, 1253,
    1256, 1258, 211, 798, 799, 800, 801, 1065, 284, 419,
    1260, 1260, 677, 667, 669, 650, 1263, 679, 86, 105,
    106, 108, 116, 36, 121, 696, 284, 86, 36, 86,
    106, 1269, 1270, 1085, 168, 1272, 1274, 1275, 1276, 1276,
    1275, 1274, 1280, 848, 1282, 1283, 1283, 1282, 1287, 1289,
    1291, 1293, 1295, 1297, 1299, 1301, 1303, 22, 1311, 28,
    343, 1311, 22, 28, 15, 105, 108, 116, 121, 1318,
    1320, 228, 587, 289, 588, 589, 590, 66, 418, 591,
    592, 593, 594, 595, 596, 597, 598, 599, 600, 30,
    601, 1321, 127, 366, 284, 228, 587, 289, 588, 589,
    590, 66, 418, 591, 592, 593, 594, 595, 596, 597,
    598, 599, 600, 30, 601, 1321, 127, 366, 1321, 240,
    455, 456, 457, 458, 459, 419, 460, 461, 1331, 117,
    118, 1331, 428, 300, 667, 650, 448, 454, 358, 307,
    432, 114, 679, 433, 1335, 1337, 1338, 1044, 396, 231,
    715, 1341, 1341, 105, 108, 120, 121, 482, 341, 95,
    484, 358, 1344, 891, 482, 484, 358, 482, 484, 1344,
    1346, 1347, 1347, 1346, 1351, 498, 1352, 1352, 1355, 1356,
    1355, 1356, 273, 275, 277, 127, 289, 588, 717, 66,
    593, 718, 719, 720, 366, 1363, 1046, 1364, 1364, 1366,
    1366, 1371, 1372, 439, 1373, 1374, 1375, 441, 442, 1376,
    443, 1377, 1378, 814, 1046, 1057, 1061, 1062, 1063, 1068,
    1069, 1380, 1069, 1383, 814, 1057, 1057, 240, 314, 316,
    317, 318, 319, 320, 1386, 1387, 1388, 1391, 1393, 1395,
    1397, 1399, 1402, 1404, 1406, 1407, 1410, 1412, 1414, 1415,
    1416, 1415, 1416, 867, 911, 913, 1421, 1422, 382, 1423,
    911, 913, 1421, 1422, 1423, 1421, 1423, 1429, 1422, 304,
    305, 308, 309, 310, 311, 859, 862, 1438, 1440, 1442,
    1444, 834, 1445, 1446, 1446, 1445, 304, 1452, 1453, 1454,
    1455, 876, 1458, 1459, 1460, 1461, 1462, 1463, 1464, 1465,
    1466, 1467, 1468, 1469, 1470, 1471, 1472, 1473, 1474, 1475,
    1476, 1477, 1478, 1479, 1480, 1481, 1482, 1483, 1484, 1485,
    1486, 1487, 1488, 1489, 1490, 1491, 1492, 1493, 1494, 1495,
    1496, 1497, 1498, 1499, 1500, 1501, 1502, 1503, 1504, 1505,
    1506, 1507, 1508, 1509, 1510, 1511, 1512, 1513, 1514, 1515,
    1516, 1517, 1518, 1519, 1520, 1521, 1522, 1523, 1524, 1525,
    1526, 1527, 1528, 1529, 1530, 1531, 1532, 1533, 1534, 1535,
    1536, 304, 305, 308, 309, 310, 311, 314, 316, 317,
    318, 319, 320, 305, 308, 309, 310, 311, 311, 310,
    1544, 1545, 1463, 1465, 1546, 1547, 1548, 1551, 1467, 1552,
    1553, 1556, 1557, 1558, 302, 224, 303, 304, 305, 287,
    306, 307, 308, 309, 310, 311, 288, 312, 213, 313,
    314, 315, 316, 317, 318, 101, 319, 320, 5, 5,
    1563, 1564, 1565, 1572, 1575, 1576, 1577, 1578, 1579, 1582,
    1583, 1585, 1586, 1587, 15, 1589, 74, 117, 118, 1589,
    1591, 1592, 1593, 1594, 1595, 1596, 1597, 1598, 1599, 1600,
    1601, 1602, 1603, 25, 314, 316, 317, 318, 319, 320,
    1606, 315, 86, 284, 1610, 1611, 1612, 1613, 1614, 1615,
    1620, 1621, 1622, 1623, 1625, 1626, 1627, 1475, 1628, 1629,
    1630, 1631, 1632, 1337, 1634, 1635, 1636, 1637, 1057, 382,
    1643, 1644, 1643, 1644, 86, 741, 742, 747, 741, 742,
    747, 1656, 1660, 1335, 358, 1661, 66, 358, 287, 1057,
    1665, 1666, 1667, 1668, 1669, 1670, 307, 288, 1671, 1673,
    1675, 832, 358, 302, 224, 358, 1681, 1682, 1682, 1681,
    835, 761, 739, 266, 302, 224, 762, 763, 296, 764,
    428, 836, 380, 765, 227, 839, 840, 841, 842, 302,
    224, 1682, 1689, 1696, 304, 305, 287, 307, 308, 309,
    310, 311, 288, 314, 316, 317, 318, 101, 319, 320,
    1455, 1719, 689, 1721, 1721, 498, 535, 525, 284, 1724,
    15, 1725, 1721, 1725, 15, 101, 1205, 86, 6, 262,
    741, 742, 567, 814, 828, 883, 926, 414, 979, 985,
    1044, 104, 1046, 1548, 1553, 105, 1579, 1583, 694, 1587,
    1022, 352, 897, 106, 1637, 1719, 181, 193, 1727, 108,
    1057, 1728, 1729, 1730, 1731, 1732, 1061, 1062, 1551, 1063,
    1733, 641, 1734, 327, 194, 1735, 502, 116, 1068, 269,
    1069, 1736, 461, 120, 1737, 1738, 121, 123, 1739, 696,
    284, 1740, 1741, 1742, 1740, 1741, 1742, 86, 6, 262,
    741, 742, 567, 814, 828, 883, 926, 414, 979, 985,
    1044, 104, 1046, 1548, 1553, 105, 1579, 1583, 694, 1587,
    1022, 352, 897, 106, 1637, 1719, 181, 193, 1727, 108,
    1057, 1728, 1729, 1730, 1731, 1732, 1061, 1062, 1551, 1063,
    1733, 641, 1734, 327, 194, 1735, 502, 116, 1068, 269,
    1069, 1736, 461, 120, 1737, 1738, 121, 123, 1739, 696,
    284, 1751, 1753, 1754, 1755, 1756, 1757, 1758, 672, 419,
    205, 121, 573, 642, 1760, 363, 300, 448, 358, 307,
    1761, 1762, 1763, 1356, 535, 1766, 1758, 1767, 1767, 326,
    199, 419, 1724, 1771, 1763, 1773, 1774, 1775, 1293, 1719,
    341, 199, 1778, 1779, 292, 1780, 1781, 234, 1782, 1783,
    1024, 1247, 284, 1778, 1782, 1247, 1293, 1779, 1780, 1783,
    1781, 1796, 1798, 1803, 1807, 11, 1812, 1813, 1814, 269,
    1073, 1813, 1814, 1812, 939, 940, 1632, 1251, 1820, 1821,
    312, 866, 1825, 1826, 341, 95, 1831, 1834, 1835, 937,
    711, 240, 828, 529, 530, 19, 531, 25, 532, 358,
    533, 1246, 534, 535, 502, 269, 536, 537, 538, 383,
    539, 1247, 1246, 86, 108, 1246, 116, 121, 696, 284,
    86, 116, 696, 284, 66, 757, 758, 312, 1689, 296,
    300, 1042, 1452, 1453, 1454, 1455, 696, 1854, 1855, 1856,
    1857, 1858, 1859, 1860, 1861, 1862, 1863, 1864, 1044, 284,
    1872, 95, 93, 14, 1582, 1874, 1876, 1880, 1719, 1778,
    292, 234, 1782, 1247, 1885, 1886, 1719, 292, 234, 1782,
    1247, 1887, 1888, 1889, 1890, 1477, 1891, 1834, 1835, 1660,
    1896, 1897, 1727, 1293, 292, 1902, 1338, 1903, 786, 1579,
    689, 1906, 1906, 1251, 410, 284, 1911, 1912, 1913, 1914,
    1016, 1915, 86, 300, 105, 108, 116, 121, 696, 284,
    1916, 1917, 1918, 1263, 1779, 1919, 314, 316, 317, 318,
    101, 319, 320, 876, 1458, 1459, 1460, 1461, 1462, 1463,
    1464, 1465, 1466, 1467, 1468, 1469, 1470, 1471, 1472, 1473,
    1474, 1475, 1476, 1477, 1478, 1479, 1480, 1481, 1482, 1483,
    1484, 1485, 1486, 1487, 1488, 1489, 1490, 1491, 1492, 1493,
    1494, 1495, 1496, 1497, 1498, 1499, 1500, 1501, 1502, 1503,
    1504, 1505, 1506, 1507, 1508, 1509, 1510, 1511, 1512, 1513,
    1514, 1515, 1516, 1517, 1518, 1519, 1520, 1521, 1522, 1523,
    1524, 1525, 1526, 1527, 1528, 1529, 1530, 1531, 1532, 1533,
    1534, 1535, 1536, 314, 316, 317, 318, 319, 320, 1463,
    1477, 1499, 1503, 1516, 1532, 1532, 1503, 1499, 1516, 117,
    118, 509, 1665, 1671, 1666, 1667, 1668, 1669, 1670, 1932,
    1933, 1932, 1933, 168, 410, 514, 1017, 1018, 1017, 1018,
    1956, 303, 99, 1958, 711, 1958, 1961, 1963, 1964, 1965,
    1966, 1967, 1968, 1969, 1970, 1971, 1972, 1973, 1974, 1975,
    1976, 1977, 319, 1979, 1980, 1825, 1981, 1982, 1660, 1335,
    1661, 1661, 1989, 91, 1781, 1206, 1990, 1991, 1992, 1728,
    303, 312, 313, 2002, 1729, 2004, 2005, 2006, 1730, 2008,
    2009, 2010, 2011, 2012, 1483, 567, 694, 2014, 2015, 2016,
    2017, 2018, 2019, 2021, 2022, 2023, 2024, 1731, 2032, 105,
    121, 2038, 105, 1022, 121, 733, 891, 2048, 2049, 2050,
    2051, 1493, 1497, 1780, 2054, 2055, 2056, 2057, 2058, 2059,
    2060, 318, 2063, 2064, 1689, 296, 2067, 2067, 2073, 2075,
    2076, 2078, 2080, 2081, 1666, 1667, 1668, 1669, 2082, 2083,
    2084, 86, 105, 108, 116, 121, 696, 86, 116, 2088,
    2089, 1666, 1667, 1668, 1669, 1670, 835, 1732, 2098, 2099,
    2100, 2101, 2102, 2106, 2107, 2108, 2109, 2110, 2111, 2112,
    2113, 314, 316, 317, 318, 319, 320, 5, 786, 1579,
    410, 341, 1763, 1778, 292, 1782, 1356, 535, 1831, 1247,
    284, 2088, 234, 2118, 2119, 2120, 2122, 2123, 2125, 2126,
    2127, 2128, 2129, 2130, 2131, 2133, 2134, 2135, 2136, 2137,
    2138, 2139, 2140, 2143, 2145, 2146, 2147, 2148, 2149, 2150,
    762, 314, 316, 317, 318, 319, 320, 2154, 2155, 2156,
    2157, 2158, 2159, 2160, 2161, 2162, 2164, 2165, 2166, 2167,
    2168, 2169, 2171, 2177, 2179, 2180, 2181, 2183, 2185, 2187,
    2189, 86, 937, 2194, 15, 753, 2199, 2199, 1512, 1856,
    2022, 2203, 2203, 323, 324, 317, 319, 317, 318, 319,
    314, 320, 2210, 316, 318, 316, 2213, 2214, 2216, 1733,
    2218, 2219, 2220, 2221, 2222, 2223, 345, 419, 343, 269,
    514, 2140, 2227, 2227, 300, 307, 2231, 2232, 2233, 2234,
    2235, 2236, 2237, 2238, 2239, 1734, 701, 37, 747, 17,
    517, 32, 648, 36, 17, 648, 2241, 2244, 2245, 2246,
    2247, 2248, 2249, 2250, 2251, 2252, 2253, 2254, 2257, 1735,
    2258, 2259, 2260, 2263, 2265, 2266, 2267, 2268, 831, 648,
    2272, 2273, 2274, 2275, 2276, 2277, 2278, 2279, 2280, 232,
    424, 17, 18, 379, 21, 22, 25, 96, 27, 381,
    28, 580, 30, 31, 32, 33, 638, 36, 37, 39,
    40, 2282, 15, 994, 2285, 2285, 91, 1293, 95, 1779,
    1902, 292, 1780, 234, 1572, 1783, 1024, 2051, 2314, 2315,
    2317, 2318, 487, 489, 1783, 1331, 719, 308, 2327, 300,
    1331, 314, 316, 317, 318, 319, 320, 1736, 2335, 2336,
    2337, 2342, 2343, 2344, 2345, 2346, 2347, 2348, 2349, 2350,
    2351, 2352, 2353, 2354, 2356, 6, 828, 985, 897, 1735,
    461, 1739, 1737, 1738, 2630, 2631, 2632, 2633, 2634, 2636,
    2637, 2638, 2639, 2640, 2641, 2642, 2643, 2644, 2645, 2646,
    2647, 444, 2648, 2649, 2650, 677, 1263, 679, 677, 1263,
    21, 432, 433, 2656, 2657, 2661, 2662, 2663, 117, 118,
    2664, 2664, 2666, 309, 1520, 1521, 2669, 2670, 1826, 2671,
    2671, 2675, 2675, 113, 2246, 1932, 2680, 2681, 2682, 2683,
    2684, 2685, 2686, 2687, 2688, 2689, 2690, 2691, 2692, 2696,
    1040, 436, 2698, 2704, 2705, 2706, 15, 994, 2708, 211,
    223, 260, 240, 875, 924, 798, 1046, 1048, 590, 1051,
    371, 1054, 181, 1856, 1007, 1887, 799, 1057, 800, 1008,
    1058, 801, 2021, 2022, 307, 533, 1062, 570, 1065, 2267,
    284, 664, 101, 1046, 181, 1057, 1062, 1046, 1057, 1062,
    240, 1856, 2021, 2022, 2267, 101, 2021, 2267, 1856, 2022,
    2717, 2718, 2719, 2720, 2721, 2722, 2723, 2724, 2725, 2726,
    2727, 2728, 105, 514, 2219, 1734, 2342, 105, 2342, 2734,
    2735, 2736, 2737, 2738, 1528, 1529, 2742, 341, 199, 1778,
    292, 168, 234, 2125, 1782, 1572, 1247, 2743, 2746, 234,
    570, 886, 1331, 2761, 2762, 2763, 2764, 2765, 2766, 2767,
    2768, 2769, 2770, 2771, 2772, 2773, 2774, 2775, 2776, 2777,
    2778, 2779, 2780, 2781, 2782,
];

pub static SIMPLE_HINTS: &[SimpleHintData] = &[
    SimpleHintData::empty(), // [0] reserved for no hint
    SimpleHintData { raw_values_start: 0, raw_values_count: 1, token_types_start: 1, token_types_count: 0 }, // [1]
    SimpleHintData { raw_values_start: 1, raw_values_count: 1, token_types_start: 2, token_types_count: 0 }, // [2]
    SimpleHintData { raw_values_start: 2, raw_values_count: 1, token_types_start: 3, token_types_count: 0 }, // [3]
    SimpleHintData { raw_values_start: 3, raw_values_count: 1, token_types_start: 4, token_types_count: 0 }, // [4]
    SimpleHintData { raw_values_start: 4, raw_values_count: 1, token_types_start: 5, token_types_count: 0 }, // [5]
    SimpleHintData { raw_values_start: 5, raw_values_count: 1, token_types_start: 6, token_types_count: 0 }, // [6]
    SimpleHintData { raw_values_start: 6, raw_values_count: 1, token_types_start: 7, token_types_count: 0 }, // [7]
    SimpleHintData { raw_values_start: 7, raw_values_count: 1, token_types_start: 8, token_types_count: 0 }, // [8]
    SimpleHintData { raw_values_start: 8, raw_values_count: 1, token_types_start: 9, token_types_count: 0 }, // [9]
    SimpleHintData { raw_values_start: 9, raw_values_count: 27, token_types_start: 36, token_types_count: 0 }, // [10]
    SimpleHintData { raw_values_start: 36, raw_values_count: 1, token_types_start: 37, token_types_count: 0 }, // [11]
    SimpleHintData { raw_values_start: 37, raw_values_count: 1, token_types_start: 38, token_types_count: 0 }, // [12]
    SimpleHintData { raw_values_start: 38, raw_values_count: 1, token_types_start: 39, token_types_count: 0 }, // [13]
    SimpleHintData { raw_values_start: 39, raw_values_count: 1, token_types_start: 40, token_types_count: 0 }, // [14]
    SimpleHintData { raw_values_start: 40, raw_values_count: 1, token_types_start: 41, token_types_count: 0 }, // [15]
    SimpleHintData { raw_values_start: 41, raw_values_count: 1, token_types_start: 42, token_types_count: 0 }, // [16]
    SimpleHintData { raw_values_start: 42, raw_values_count: 1, token_types_start: 43, token_types_count: 0 }, // [17]
    SimpleHintData { raw_values_start: 43, raw_values_count: 1, token_types_start: 44, token_types_count: 0 }, // [18]
    SimpleHintData { raw_values_start: 44, raw_values_count: 1, token_types_start: 45, token_types_count: 0 }, // [19]
    SimpleHintData { raw_values_start: 45, raw_values_count: 1, token_types_start: 46, token_types_count: 0 }, // [20]
    SimpleHintData { raw_values_start: 46, raw_values_count: 1, token_types_start: 47, token_types_count: 0 }, // [21]
    SimpleHintData { raw_values_start: 47, raw_values_count: 1, token_types_start: 48, token_types_count: 0 }, // [22]
    SimpleHintData { raw_values_start: 48, raw_values_count: 1, token_types_start: 49, token_types_count: 0 }, // [23]
    SimpleHintData { raw_values_start: 49, raw_values_count: 2, token_types_start: 51, token_types_count: 0 }, // [24]
    SimpleHintData { raw_values_start: 51, raw_values_count: 1, token_types_start: 52, token_types_count: 0 }, // [25]
    SimpleHintData { raw_values_start: 52, raw_values_count: 1, token_types_start: 53, token_types_count: 0 }, // [26]
    SimpleHintData { raw_values_start: 53, raw_values_count: 1, token_types_start: 54, token_types_count: 0 }, // [27]
    SimpleHintData { raw_values_start: 54, raw_values_count: 1, token_types_start: 55, token_types_count: 0 }, // [28]
    SimpleHintData { raw_values_start: 55, raw_values_count: 1, token_types_start: 56, token_types_count: 0 }, // [29]
    SimpleHintData { raw_values_start: 56, raw_values_count: 1, token_types_start: 57, token_types_count: 0 }, // [30]
    SimpleHintData { raw_values_start: 57, raw_values_count: 1, token_types_start: 58, token_types_count: 0 }, // [31]
    SimpleHintData { raw_values_start: 58, raw_values_count: 14, token_types_start: 72, token_types_count: 0 }, // [32]
    SimpleHintData { raw_values_start: 72, raw_values_count: 14, token_types_start: 86, token_types_count: 0 }, // [33]
    SimpleHintData { raw_values_start: 86, raw_values_count: 2, token_types_start: 88, token_types_count: 0 }, // [34]
    SimpleHintData { raw_values_start: 88, raw_values_count: 1, token_types_start: 89, token_types_count: 0 }, // [35]
    SimpleHintData { raw_values_start: 89, raw_values_count: 1, token_types_start: 90, token_types_count: 0 }, // [36]
    SimpleHintData { raw_values_start: 90, raw_values_count: 1, token_types_start: 91, token_types_count: 0 }, // [37]
    SimpleHintData { raw_values_start: 91, raw_values_count: 1, token_types_start: 92, token_types_count: 0 }, // [38]
    SimpleHintData { raw_values_start: 92, raw_values_count: 1, token_types_start: 93, token_types_count: 0 }, // [39]
    SimpleHintData { raw_values_start: 93, raw_values_count: 1, token_types_start: 94, token_types_count: 0 }, // [40]
    SimpleHintData { raw_values_start: 94, raw_values_count: 0, token_types_start: 94, token_types_count: 1 }, // [41]
    SimpleHintData { raw_values_start: 95, raw_values_count: 26, token_types_start: 121, token_types_count: 0 }, // [42]
    SimpleHintData { raw_values_start: 121, raw_values_count: 1, token_types_start: 122, token_types_count: 0 }, // [43]
    SimpleHintData { raw_values_start: 122, raw_values_count: 5, token_types_start: 127, token_types_count: 0 }, // [44]
    SimpleHintData { raw_values_start: 127, raw_values_count: 1, token_types_start: 128, token_types_count: 0 }, // [45]
    SimpleHintData { raw_values_start: 128, raw_values_count: 1, token_types_start: 129, token_types_count: 0 }, // [46]
    SimpleHintData { raw_values_start: 129, raw_values_count: 1, token_types_start: 130, token_types_count: 0 }, // [47]
    SimpleHintData { raw_values_start: 130, raw_values_count: 1, token_types_start: 131, token_types_count: 0 }, // [48]
    SimpleHintData { raw_values_start: 131, raw_values_count: 1, token_types_start: 132, token_types_count: 0 }, // [49]
    SimpleHintData { raw_values_start: 132, raw_values_count: 1, token_types_start: 133, token_types_count: 0 }, // [50]
    SimpleHintData { raw_values_start: 133, raw_values_count: 1, token_types_start: 134, token_types_count: 0 }, // [51]
    SimpleHintData { raw_values_start: 134, raw_values_count: 1, token_types_start: 135, token_types_count: 0 }, // [52]
    SimpleHintData { raw_values_start: 135, raw_values_count: 2, token_types_start: 137, token_types_count: 0 }, // [53]
    SimpleHintData { raw_values_start: 137, raw_values_count: 1, token_types_start: 138, token_types_count: 0 }, // [54]
    SimpleHintData { raw_values_start: 138, raw_values_count: 1, token_types_start: 139, token_types_count: 0 }, // [55]
    SimpleHintData { raw_values_start: 139, raw_values_count: 25, token_types_start: 164, token_types_count: 0 }, // [56]
    SimpleHintData { raw_values_start: 164, raw_values_count: 15, token_types_start: 179, token_types_count: 0 }, // [57]
    SimpleHintData { raw_values_start: 179, raw_values_count: 1, token_types_start: 180, token_types_count: 0 }, // [58]
    SimpleHintData { raw_values_start: 180, raw_values_count: 1, token_types_start: 181, token_types_count: 0 }, // [59]
    SimpleHintData { raw_values_start: 181, raw_values_count: 2, token_types_start: 183, token_types_count: 0 }, // [60]
    SimpleHintData { raw_values_start: 183, raw_values_count: 1, token_types_start: 184, token_types_count: 0 }, // [61]
    SimpleHintData { raw_values_start: 184, raw_values_count: 1, token_types_start: 185, token_types_count: 0 }, // [62]
    SimpleHintData { raw_values_start: 185, raw_values_count: 1, token_types_start: 186, token_types_count: 0 }, // [63]
    SimpleHintData { raw_values_start: 186, raw_values_count: 1, token_types_start: 187, token_types_count: 0 }, // [64]
    SimpleHintData { raw_values_start: 187, raw_values_count: 1, token_types_start: 188, token_types_count: 0 }, // [65]
    SimpleHintData { raw_values_start: 188, raw_values_count: 1, token_types_start: 189, token_types_count: 0 }, // [66]
    SimpleHintData { raw_values_start: 189, raw_values_count: 1, token_types_start: 190, token_types_count: 0 }, // [67]
    SimpleHintData { raw_values_start: 190, raw_values_count: 1, token_types_start: 191, token_types_count: 0 }, // [68]
    SimpleHintData { raw_values_start: 191, raw_values_count: 1, token_types_start: 192, token_types_count: 0 }, // [69]
    SimpleHintData { raw_values_start: 192, raw_values_count: 1, token_types_start: 193, token_types_count: 0 }, // [70]
    SimpleHintData { raw_values_start: 193, raw_values_count: 1, token_types_start: 194, token_types_count: 0 }, // [71]
    SimpleHintData { raw_values_start: 194, raw_values_count: 1, token_types_start: 195, token_types_count: 0 }, // [72]
    SimpleHintData { raw_values_start: 195, raw_values_count: 1, token_types_start: 196, token_types_count: 0 }, // [73]
    SimpleHintData { raw_values_start: 196, raw_values_count: 1, token_types_start: 197, token_types_count: 0 }, // [74]
    SimpleHintData { raw_values_start: 197, raw_values_count: 1, token_types_start: 198, token_types_count: 0 }, // [75]
    SimpleHintData { raw_values_start: 198, raw_values_count: 1, token_types_start: 199, token_types_count: 0 }, // [76]
    SimpleHintData { raw_values_start: 199, raw_values_count: 1, token_types_start: 200, token_types_count: 0 }, // [77]
    SimpleHintData { raw_values_start: 200, raw_values_count: 1, token_types_start: 201, token_types_count: 0 }, // [78]
    SimpleHintData { raw_values_start: 201, raw_values_count: 1, token_types_start: 202, token_types_count: 0 }, // [79]
    SimpleHintData { raw_values_start: 202, raw_values_count: 1, token_types_start: 203, token_types_count: 0 }, // [80]
    SimpleHintData { raw_values_start: 203, raw_values_count: 1, token_types_start: 204, token_types_count: 0 }, // [81]
    SimpleHintData { raw_values_start: 204, raw_values_count: 1, token_types_start: 205, token_types_count: 0 }, // [82]
    SimpleHintData { raw_values_start: 205, raw_values_count: 1, token_types_start: 206, token_types_count: 0 }, // [83]
    SimpleHintData { raw_values_start: 206, raw_values_count: 1, token_types_start: 207, token_types_count: 0 }, // [84]
    SimpleHintData { raw_values_start: 207, raw_values_count: 1, token_types_start: 208, token_types_count: 0 }, // [85]
    SimpleHintData { raw_values_start: 208, raw_values_count: 1, token_types_start: 209, token_types_count: 0 }, // [86]
    SimpleHintData { raw_values_start: 209, raw_values_count: 1, token_types_start: 210, token_types_count: 0 }, // [87]
    SimpleHintData { raw_values_start: 210, raw_values_count: 1, token_types_start: 211, token_types_count: 0 }, // [88]
    SimpleHintData { raw_values_start: 211, raw_values_count: 1, token_types_start: 212, token_types_count: 0 }, // [89]
    SimpleHintData { raw_values_start: 212, raw_values_count: 1, token_types_start: 213, token_types_count: 0 }, // [90]
    SimpleHintData { raw_values_start: 213, raw_values_count: 1, token_types_start: 214, token_types_count: 0 }, // [91]
    SimpleHintData { raw_values_start: 214, raw_values_count: 1, token_types_start: 215, token_types_count: 0 }, // [92]
    SimpleHintData { raw_values_start: 215, raw_values_count: 1, token_types_start: 216, token_types_count: 0 }, // [93]
    SimpleHintData { raw_values_start: 216, raw_values_count: 1, token_types_start: 217, token_types_count: 0 }, // [94]
    SimpleHintData { raw_values_start: 217, raw_values_count: 1, token_types_start: 218, token_types_count: 0 }, // [95]
    SimpleHintData { raw_values_start: 218, raw_values_count: 1, token_types_start: 219, token_types_count: 0 }, // [96]
    SimpleHintData { raw_values_start: 219, raw_values_count: 1, token_types_start: 220, token_types_count: 0 }, // [97]
    SimpleHintData { raw_values_start: 220, raw_values_count: 1, token_types_start: 221, token_types_count: 0 }, // [98]
    SimpleHintData { raw_values_start: 221, raw_values_count: 1, token_types_start: 222, token_types_count: 0 }, // [99]
    SimpleHintData { raw_values_start: 222, raw_values_count: 1, token_types_start: 223, token_types_count: 0 }, // [100]
    SimpleHintData { raw_values_start: 223, raw_values_count: 1, token_types_start: 224, token_types_count: 0 }, // [101]
    SimpleHintData { raw_values_start: 224, raw_values_count: 2, token_types_start: 226, token_types_count: 0 }, // [102]
    SimpleHintData { raw_values_start: 226, raw_values_count: 1, token_types_start: 227, token_types_count: 0 }, // [103]
    SimpleHintData { raw_values_start: 227, raw_values_count: 1, token_types_start: 228, token_types_count: 0 }, // [104]
    SimpleHintData { raw_values_start: 228, raw_values_count: 1, token_types_start: 229, token_types_count: 0 }, // [105]
    SimpleHintData { raw_values_start: 229, raw_values_count: 1, token_types_start: 230, token_types_count: 0 }, // [106]
    SimpleHintData { raw_values_start: 230, raw_values_count: 1, token_types_start: 231, token_types_count: 0 }, // [107]
    SimpleHintData { raw_values_start: 231, raw_values_count: 1, token_types_start: 232, token_types_count: 0 }, // [108]
    SimpleHintData { raw_values_start: 232, raw_values_count: 2, token_types_start: 234, token_types_count: 0 }, // [109]
    SimpleHintData { raw_values_start: 234, raw_values_count: 1, token_types_start: 235, token_types_count: 0 }, // [110]
    SimpleHintData { raw_values_start: 235, raw_values_count: 1, token_types_start: 236, token_types_count: 0 }, // [111]
    SimpleHintData { raw_values_start: 236, raw_values_count: 1, token_types_start: 237, token_types_count: 0 }, // [112]
    SimpleHintData { raw_values_start: 237, raw_values_count: 1, token_types_start: 238, token_types_count: 0 }, // [113]
    SimpleHintData { raw_values_start: 238, raw_values_count: 1, token_types_start: 239, token_types_count: 0 }, // [114]
    SimpleHintData { raw_values_start: 239, raw_values_count: 1, token_types_start: 240, token_types_count: 0 }, // [115]
    SimpleHintData { raw_values_start: 240, raw_values_count: 1, token_types_start: 241, token_types_count: 0 }, // [116]
    SimpleHintData { raw_values_start: 241, raw_values_count: 1, token_types_start: 242, token_types_count: 0 }, // [117]
    SimpleHintData { raw_values_start: 242, raw_values_count: 1, token_types_start: 243, token_types_count: 0 }, // [118]
    SimpleHintData { raw_values_start: 243, raw_values_count: 1, token_types_start: 244, token_types_count: 0 }, // [119]
    SimpleHintData { raw_values_start: 244, raw_values_count: 1, token_types_start: 245, token_types_count: 0 }, // [120]
    SimpleHintData { raw_values_start: 245, raw_values_count: 1, token_types_start: 246, token_types_count: 0 }, // [121]
    SimpleHintData { raw_values_start: 246, raw_values_count: 1, token_types_start: 247, token_types_count: 0 }, // [122]
    SimpleHintData { raw_values_start: 247, raw_values_count: 1, token_types_start: 248, token_types_count: 0 }, // [123]
    SimpleHintData { raw_values_start: 248, raw_values_count: 1, token_types_start: 249, token_types_count: 0 }, // [124]
    SimpleHintData { raw_values_start: 249, raw_values_count: 1, token_types_start: 250, token_types_count: 0 }, // [125]
    SimpleHintData { raw_values_start: 250, raw_values_count: 1, token_types_start: 251, token_types_count: 0 }, // [126]
    SimpleHintData { raw_values_start: 251, raw_values_count: 1, token_types_start: 252, token_types_count: 0 }, // [127]
    SimpleHintData { raw_values_start: 252, raw_values_count: 1, token_types_start: 253, token_types_count: 0 }, // [128]
    SimpleHintData { raw_values_start: 253, raw_values_count: 1, token_types_start: 254, token_types_count: 0 }, // [129]
    SimpleHintData { raw_values_start: 254, raw_values_count: 1, token_types_start: 255, token_types_count: 0 }, // [130]
    SimpleHintData { raw_values_start: 255, raw_values_count: 1, token_types_start: 256, token_types_count: 0 }, // [131]
    SimpleHintData { raw_values_start: 256, raw_values_count: 1, token_types_start: 257, token_types_count: 0 }, // [132]
    SimpleHintData { raw_values_start: 257, raw_values_count: 3, token_types_start: 260, token_types_count: 0 }, // [133]
    SimpleHintData { raw_values_start: 260, raw_values_count: 1, token_types_start: 261, token_types_count: 0 }, // [134]
    SimpleHintData { raw_values_start: 261, raw_values_count: 1, token_types_start: 262, token_types_count: 0 }, // [135]
    SimpleHintData { raw_values_start: 262, raw_values_count: 1, token_types_start: 263, token_types_count: 0 }, // [136]
    SimpleHintData { raw_values_start: 263, raw_values_count: 1, token_types_start: 264, token_types_count: 0 }, // [137]
    SimpleHintData { raw_values_start: 264, raw_values_count: 1, token_types_start: 265, token_types_count: 0 }, // [138]
    SimpleHintData { raw_values_start: 265, raw_values_count: 1, token_types_start: 266, token_types_count: 0 }, // [139]
    SimpleHintData { raw_values_start: 266, raw_values_count: 1, token_types_start: 267, token_types_count: 0 }, // [140]
    SimpleHintData { raw_values_start: 267, raw_values_count: 2, token_types_start: 269, token_types_count: 0 }, // [141]
    SimpleHintData { raw_values_start: 269, raw_values_count: 1, token_types_start: 270, token_types_count: 0 }, // [142]
    SimpleHintData { raw_values_start: 270, raw_values_count: 1, token_types_start: 271, token_types_count: 0 }, // [143]
    SimpleHintData { raw_values_start: 271, raw_values_count: 3, token_types_start: 274, token_types_count: 0 }, // [144]
    SimpleHintData { raw_values_start: 274, raw_values_count: 2, token_types_start: 276, token_types_count: 0 }, // [145]
    SimpleHintData { raw_values_start: 276, raw_values_count: 1, token_types_start: 277, token_types_count: 0 }, // [146]
    SimpleHintData { raw_values_start: 277, raw_values_count: 1, token_types_start: 278, token_types_count: 0 }, // [147]
    SimpleHintData { raw_values_start: 278, raw_values_count: 16, token_types_start: 294, token_types_count: 8 }, // [148]
    SimpleHintData { raw_values_start: 302, raw_values_count: 0, token_types_start: 302, token_types_count: 2 }, // [149]
    SimpleHintData { raw_values_start: 304, raw_values_count: 1, token_types_start: 305, token_types_count: 0 }, // [150]
    SimpleHintData { raw_values_start: 305, raw_values_count: 1, token_types_start: 306, token_types_count: 0 }, // [151]
    SimpleHintData { raw_values_start: 306, raw_values_count: 11, token_types_start: 317, token_types_count: 0 }, // [152]
    SimpleHintData { raw_values_start: 317, raw_values_count: 7, token_types_start: 324, token_types_count: 0 }, // [153]
    SimpleHintData { raw_values_start: 324, raw_values_count: 1, token_types_start: 325, token_types_count: 0 }, // [154]
    SimpleHintData { raw_values_start: 325, raw_values_count: 10, token_types_start: 335, token_types_count: 0 }, // [155]
    SimpleHintData { raw_values_start: 335, raw_values_count: 1, token_types_start: 336, token_types_count: 0 }, // [156]
    SimpleHintData { raw_values_start: 336, raw_values_count: 1, token_types_start: 337, token_types_count: 0 }, // [157]
    SimpleHintData { raw_values_start: 337, raw_values_count: 1, token_types_start: 338, token_types_count: 0 }, // [158]
    SimpleHintData { raw_values_start: 338, raw_values_count: 2, token_types_start: 340, token_types_count: 0 }, // [159]
    SimpleHintData { raw_values_start: 340, raw_values_count: 1, token_types_start: 341, token_types_count: 0 }, // [160]
    SimpleHintData { raw_values_start: 341, raw_values_count: 2, token_types_start: 343, token_types_count: 0 }, // [161]
    SimpleHintData { raw_values_start: 343, raw_values_count: 6, token_types_start: 349, token_types_count: 0 }, // [162]
    SimpleHintData { raw_values_start: 349, raw_values_count: 1, token_types_start: 350, token_types_count: 0 }, // [163]
    SimpleHintData { raw_values_start: 350, raw_values_count: 2, token_types_start: 352, token_types_count: 0 }, // [164]
    SimpleHintData { raw_values_start: 352, raw_values_count: 1, token_types_start: 353, token_types_count: 0 }, // [165]
    SimpleHintData { raw_values_start: 353, raw_values_count: 1, token_types_start: 354, token_types_count: 0 }, // [166]
    SimpleHintData { raw_values_start: 354, raw_values_count: 1, token_types_start: 355, token_types_count: 0 }, // [167]
    SimpleHintData { raw_values_start: 355, raw_values_count: 3, token_types_start: 358, token_types_count: 0 }, // [168]
    SimpleHintData { raw_values_start: 358, raw_values_count: 2, token_types_start: 360, token_types_count: 0 }, // [169]
    SimpleHintData { raw_values_start: 360, raw_values_count: 1, token_types_start: 361, token_types_count: 0 }, // [170]
    SimpleHintData { raw_values_start: 361, raw_values_count: 1, token_types_start: 362, token_types_count: 0 }, // [171]
    SimpleHintData { raw_values_start: 362, raw_values_count: 1, token_types_start: 363, token_types_count: 0 }, // [172]
    SimpleHintData { raw_values_start: 363, raw_values_count: 1, token_types_start: 364, token_types_count: 0 }, // [173]
    SimpleHintData { raw_values_start: 364, raw_values_count: 1, token_types_start: 365, token_types_count: 0 }, // [174]
    SimpleHintData { raw_values_start: 365, raw_values_count: 4, token_types_start: 369, token_types_count: 0 }, // [175]
    SimpleHintData { raw_values_start: 369, raw_values_count: 1, token_types_start: 370, token_types_count: 0 }, // [176]
    SimpleHintData { raw_values_start: 370, raw_values_count: 2, token_types_start: 372, token_types_count: 0 }, // [177]
    SimpleHintData { raw_values_start: 372, raw_values_count: 23, token_types_start: 395, token_types_count: 0 }, // [178]
    SimpleHintData { raw_values_start: 395, raw_values_count: 13, token_types_start: 408, token_types_count: 0 }, // [179]
    SimpleHintData { raw_values_start: 408, raw_values_count: 1, token_types_start: 409, token_types_count: 0 }, // [180]
    SimpleHintData { raw_values_start: 409, raw_values_count: 1, token_types_start: 410, token_types_count: 0 }, // [181]
    SimpleHintData { raw_values_start: 410, raw_values_count: 1, token_types_start: 411, token_types_count: 0 }, // [182]
    SimpleHintData { raw_values_start: 411, raw_values_count: 1, token_types_start: 412, token_types_count: 0 }, // [183]
    SimpleHintData { raw_values_start: 412, raw_values_count: 2, token_types_start: 414, token_types_count: 0 }, // [184]
    SimpleHintData { raw_values_start: 414, raw_values_count: 1, token_types_start: 415, token_types_count: 0 }, // [185]
    SimpleHintData { raw_values_start: 415, raw_values_count: 1, token_types_start: 416, token_types_count: 0 }, // [186]
    SimpleHintData { raw_values_start: 416, raw_values_count: 1, token_types_start: 417, token_types_count: 0 }, // [187]
    SimpleHintData { raw_values_start: 417, raw_values_count: 4, token_types_start: 421, token_types_count: 0 }, // [188]
    SimpleHintData { raw_values_start: 421, raw_values_count: 1, token_types_start: 422, token_types_count: 0 }, // [189]
    SimpleHintData { raw_values_start: 422, raw_values_count: 1, token_types_start: 423, token_types_count: 0 }, // [190]
    SimpleHintData { raw_values_start: 423, raw_values_count: 1, token_types_start: 424, token_types_count: 0 }, // [191]
    SimpleHintData { raw_values_start: 424, raw_values_count: 1, token_types_start: 425, token_types_count: 0 }, // [192]
    SimpleHintData { raw_values_start: 425, raw_values_count: 3, token_types_start: 428, token_types_count: 0 }, // [193]
    SimpleHintData { raw_values_start: 428, raw_values_count: 1, token_types_start: 429, token_types_count: 0 }, // [194]
    SimpleHintData { raw_values_start: 429, raw_values_count: 3, token_types_start: 432, token_types_count: 0 }, // [195]
    SimpleHintData { raw_values_start: 432, raw_values_count: 1, token_types_start: 433, token_types_count: 0 }, // [196]
    SimpleHintData { raw_values_start: 433, raw_values_count: 2, token_types_start: 435, token_types_count: 0 }, // [197]
    SimpleHintData { raw_values_start: 435, raw_values_count: 1, token_types_start: 436, token_types_count: 0 }, // [198]
    SimpleHintData { raw_values_start: 436, raw_values_count: 1, token_types_start: 437, token_types_count: 0 }, // [199]
    SimpleHintData { raw_values_start: 437, raw_values_count: 1, token_types_start: 438, token_types_count: 0 }, // [200]
    SimpleHintData { raw_values_start: 438, raw_values_count: 1, token_types_start: 439, token_types_count: 0 }, // [201]
    SimpleHintData { raw_values_start: 439, raw_values_count: 17, token_types_start: 456, token_types_count: 0 }, // [202]
    SimpleHintData { raw_values_start: 456, raw_values_count: 1, token_types_start: 457, token_types_count: 0 }, // [203]
    SimpleHintData { raw_values_start: 457, raw_values_count: 1, token_types_start: 458, token_types_count: 0 }, // [204]
    SimpleHintData { raw_values_start: 458, raw_values_count: 5, token_types_start: 463, token_types_count: 0 }, // [205]
    SimpleHintData { raw_values_start: 463, raw_values_count: 7, token_types_start: 470, token_types_count: 0 }, // [206]
    SimpleHintData { raw_values_start: 470, raw_values_count: 2, token_types_start: 472, token_types_count: 0 }, // [207]
    SimpleHintData { raw_values_start: 472, raw_values_count: 9, token_types_start: 481, token_types_count: 0 }, // [208]
    SimpleHintData { raw_values_start: 481, raw_values_count: 3, token_types_start: 484, token_types_count: 0 }, // [209]
    SimpleHintData { raw_values_start: 484, raw_values_count: 1, token_types_start: 485, token_types_count: 0 }, // [210]
    SimpleHintData { raw_values_start: 485, raw_values_count: 1, token_types_start: 486, token_types_count: 0 }, // [211]
    SimpleHintData { raw_values_start: 486, raw_values_count: 2, token_types_start: 488, token_types_count: 0 }, // [212]
    SimpleHintData { raw_values_start: 488, raw_values_count: 1, token_types_start: 489, token_types_count: 0 }, // [213]
    SimpleHintData { raw_values_start: 489, raw_values_count: 1, token_types_start: 490, token_types_count: 0 }, // [214]
    SimpleHintData { raw_values_start: 490, raw_values_count: 1, token_types_start: 491, token_types_count: 0 }, // [215]
    SimpleHintData { raw_values_start: 491, raw_values_count: 11, token_types_start: 502, token_types_count: 0 }, // [216]
    SimpleHintData { raw_values_start: 502, raw_values_count: 2, token_types_start: 504, token_types_count: 0 }, // [217]
    SimpleHintData { raw_values_start: 504, raw_values_count: 1, token_types_start: 505, token_types_count: 0 }, // [218]
    SimpleHintData { raw_values_start: 505, raw_values_count: 1, token_types_start: 506, token_types_count: 0 }, // [219]
    SimpleHintData { raw_values_start: 506, raw_values_count: 1, token_types_start: 507, token_types_count: 0 }, // [220]
    SimpleHintData { raw_values_start: 507, raw_values_count: 4, token_types_start: 511, token_types_count: 0 }, // [221]
    SimpleHintData { raw_values_start: 511, raw_values_count: 1, token_types_start: 512, token_types_count: 0 }, // [222]
    SimpleHintData { raw_values_start: 512, raw_values_count: 1, token_types_start: 513, token_types_count: 0 }, // [223]
    SimpleHintData { raw_values_start: 513, raw_values_count: 1, token_types_start: 514, token_types_count: 0 }, // [224]
    SimpleHintData { raw_values_start: 514, raw_values_count: 1, token_types_start: 515, token_types_count: 0 }, // [225]
    SimpleHintData { raw_values_start: 515, raw_values_count: 4, token_types_start: 519, token_types_count: 0 }, // [226]
    SimpleHintData { raw_values_start: 519, raw_values_count: 3, token_types_start: 522, token_types_count: 0 }, // [227]
    SimpleHintData { raw_values_start: 522, raw_values_count: 2, token_types_start: 524, token_types_count: 0 }, // [228]
    SimpleHintData { raw_values_start: 524, raw_values_count: 1, token_types_start: 525, token_types_count: 0 }, // [229]
    SimpleHintData { raw_values_start: 525, raw_values_count: 1, token_types_start: 526, token_types_count: 0 }, // [230]
    SimpleHintData { raw_values_start: 526, raw_values_count: 1, token_types_start: 527, token_types_count: 0 }, // [231]
    SimpleHintData { raw_values_start: 527, raw_values_count: 1, token_types_start: 528, token_types_count: 0 }, // [232]
    SimpleHintData { raw_values_start: 528, raw_values_count: 2, token_types_start: 530, token_types_count: 0 }, // [233]
    SimpleHintData { raw_values_start: 530, raw_values_count: 1, token_types_start: 531, token_types_count: 0 }, // [234]
    SimpleHintData { raw_values_start: 531, raw_values_count: 1, token_types_start: 532, token_types_count: 0 }, // [235]
    SimpleHintData { raw_values_start: 532, raw_values_count: 2, token_types_start: 534, token_types_count: 0 }, // [236]
    SimpleHintData { raw_values_start: 534, raw_values_count: 1, token_types_start: 535, token_types_count: 0 }, // [237]
    SimpleHintData { raw_values_start: 535, raw_values_count: 1, token_types_start: 536, token_types_count: 0 }, // [238]
    SimpleHintData { raw_values_start: 536, raw_values_count: 4, token_types_start: 540, token_types_count: 0 }, // [239]
    SimpleHintData { raw_values_start: 540, raw_values_count: 1, token_types_start: 541, token_types_count: 0 }, // [240]
    SimpleHintData { raw_values_start: 541, raw_values_count: 1, token_types_start: 542, token_types_count: 0 }, // [241]
    SimpleHintData { raw_values_start: 542, raw_values_count: 1, token_types_start: 543, token_types_count: 0 }, // [242]
    SimpleHintData { raw_values_start: 543, raw_values_count: 1, token_types_start: 544, token_types_count: 0 }, // [243]
    SimpleHintData { raw_values_start: 544, raw_values_count: 1, token_types_start: 545, token_types_count: 0 }, // [244]
    SimpleHintData { raw_values_start: 545, raw_values_count: 1, token_types_start: 546, token_types_count: 0 }, // [245]
    SimpleHintData { raw_values_start: 546, raw_values_count: 1, token_types_start: 547, token_types_count: 0 }, // [246]
    SimpleHintData { raw_values_start: 547, raw_values_count: 2, token_types_start: 549, token_types_count: 0 }, // [247]
    SimpleHintData { raw_values_start: 549, raw_values_count: 1, token_types_start: 550, token_types_count: 0 }, // [248]
    SimpleHintData { raw_values_start: 550, raw_values_count: 2, token_types_start: 552, token_types_count: 0 }, // [249]
    SimpleHintData { raw_values_start: 552, raw_values_count: 1, token_types_start: 553, token_types_count: 0 }, // [250]
    SimpleHintData { raw_values_start: 553, raw_values_count: 1, token_types_start: 554, token_types_count: 0 }, // [251]
    SimpleHintData { raw_values_start: 554, raw_values_count: 2, token_types_start: 556, token_types_count: 0 }, // [252]
    SimpleHintData { raw_values_start: 556, raw_values_count: 1, token_types_start: 557, token_types_count: 0 }, // [253]
    SimpleHintData { raw_values_start: 557, raw_values_count: 1, token_types_start: 558, token_types_count: 0 }, // [254]
    SimpleHintData { raw_values_start: 558, raw_values_count: 4, token_types_start: 562, token_types_count: 0 }, // [255]
    SimpleHintData { raw_values_start: 562, raw_values_count: 1, token_types_start: 563, token_types_count: 0 }, // [256]
    SimpleHintData { raw_values_start: 563, raw_values_count: 1, token_types_start: 564, token_types_count: 0 }, // [257]
    SimpleHintData { raw_values_start: 564, raw_values_count: 1, token_types_start: 565, token_types_count: 0 }, // [258]
    SimpleHintData { raw_values_start: 565, raw_values_count: 2, token_types_start: 567, token_types_count: 0 }, // [259]
    SimpleHintData { raw_values_start: 567, raw_values_count: 1, token_types_start: 568, token_types_count: 0 }, // [260]
    SimpleHintData { raw_values_start: 568, raw_values_count: 3, token_types_start: 571, token_types_count: 0 }, // [261]
    SimpleHintData { raw_values_start: 571, raw_values_count: 0, token_types_start: 571, token_types_count: 6 }, // [262]
    SimpleHintData { raw_values_start: 577, raw_values_count: 1, token_types_start: 578, token_types_count: 0 }, // [263]
    SimpleHintData { raw_values_start: 578, raw_values_count: 18, token_types_start: 596, token_types_count: 0 }, // [264]
    SimpleHintData { raw_values_start: 596, raw_values_count: 16, token_types_start: 612, token_types_count: 0 }, // [265]
    SimpleHintData { raw_values_start: 612, raw_values_count: 3, token_types_start: 615, token_types_count: 0 }, // [266]
    SimpleHintData { raw_values_start: 615, raw_values_count: 1, token_types_start: 616, token_types_count: 0 }, // [267]
    SimpleHintData { raw_values_start: 616, raw_values_count: 1, token_types_start: 617, token_types_count: 0 }, // [268]
    SimpleHintData { raw_values_start: 617, raw_values_count: 1, token_types_start: 618, token_types_count: 0 }, // [269]
    SimpleHintData { raw_values_start: 618, raw_values_count: 1, token_types_start: 619, token_types_count: 0 }, // [270]
    SimpleHintData { raw_values_start: 619, raw_values_count: 3, token_types_start: 622, token_types_count: 0 }, // [271]
    SimpleHintData { raw_values_start: 622, raw_values_count: 1, token_types_start: 623, token_types_count: 0 }, // [272]
    SimpleHintData { raw_values_start: 623, raw_values_count: 1, token_types_start: 624, token_types_count: 0 }, // [273]
    SimpleHintData { raw_values_start: 624, raw_values_count: 1, token_types_start: 625, token_types_count: 0 }, // [274]
    SimpleHintData { raw_values_start: 625, raw_values_count: 2, token_types_start: 627, token_types_count: 0 }, // [275]
    SimpleHintData { raw_values_start: 627, raw_values_count: 1, token_types_start: 628, token_types_count: 0 }, // [276]
    SimpleHintData { raw_values_start: 628, raw_values_count: 2, token_types_start: 630, token_types_count: 0 }, // [277]
    SimpleHintData { raw_values_start: 630, raw_values_count: 2, token_types_start: 632, token_types_count: 0 }, // [278]
    SimpleHintData { raw_values_start: 632, raw_values_count: 1, token_types_start: 633, token_types_count: 0 }, // [279]
    SimpleHintData { raw_values_start: 633, raw_values_count: 1, token_types_start: 634, token_types_count: 0 }, // [280]
    SimpleHintData { raw_values_start: 634, raw_values_count: 1, token_types_start: 635, token_types_count: 0 }, // [281]
    SimpleHintData { raw_values_start: 635, raw_values_count: 3, token_types_start: 638, token_types_count: 0 }, // [282]
    SimpleHintData { raw_values_start: 638, raw_values_count: 1, token_types_start: 639, token_types_count: 0 }, // [283]
    SimpleHintData { raw_values_start: 639, raw_values_count: 1, token_types_start: 640, token_types_count: 0 }, // [284]
    SimpleHintData { raw_values_start: 640, raw_values_count: 1, token_types_start: 641, token_types_count: 0 }, // [285]
    SimpleHintData { raw_values_start: 641, raw_values_count: 1, token_types_start: 642, token_types_count: 0 }, // [286]
    SimpleHintData { raw_values_start: 642, raw_values_count: 1, token_types_start: 643, token_types_count: 0 }, // [287]
    SimpleHintData { raw_values_start: 643, raw_values_count: 1, token_types_start: 644, token_types_count: 0 }, // [288]
    SimpleHintData { raw_values_start: 644, raw_values_count: 20, token_types_start: 664, token_types_count: 0 }, // [289]
    SimpleHintData { raw_values_start: 664, raw_values_count: 2, token_types_start: 666, token_types_count: 0 }, // [290]
    SimpleHintData { raw_values_start: 666, raw_values_count: 1, token_types_start: 667, token_types_count: 0 }, // [291]
    SimpleHintData { raw_values_start: 667, raw_values_count: 3, token_types_start: 670, token_types_count: 0 }, // [292]
    SimpleHintData { raw_values_start: 670, raw_values_count: 7, token_types_start: 677, token_types_count: 0 }, // [293]
    SimpleHintData { raw_values_start: 677, raw_values_count: 1, token_types_start: 678, token_types_count: 0 }, // [294]
    SimpleHintData { raw_values_start: 678, raw_values_count: 1, token_types_start: 679, token_types_count: 0 }, // [295]
    SimpleHintData { raw_values_start: 679, raw_values_count: 1, token_types_start: 680, token_types_count: 0 }, // [296]
    SimpleHintData { raw_values_start: 680, raw_values_count: 1, token_types_start: 681, token_types_count: 0 }, // [297]
    SimpleHintData { raw_values_start: 681, raw_values_count: 5, token_types_start: 686, token_types_count: 0 }, // [298]
    SimpleHintData { raw_values_start: 686, raw_values_count: 2, token_types_start: 688, token_types_count: 0 }, // [299]
    SimpleHintData { raw_values_start: 688, raw_values_count: 6, token_types_start: 694, token_types_count: 0 }, // [300]
    SimpleHintData { raw_values_start: 694, raw_values_count: 4, token_types_start: 698, token_types_count: 0 }, // [301]
    SimpleHintData { raw_values_start: 698, raw_values_count: 3, token_types_start: 701, token_types_count: 0 }, // [302]
    SimpleHintData { raw_values_start: 701, raw_values_count: 4, token_types_start: 705, token_types_count: 0 }, // [303]
    SimpleHintData { raw_values_start: 705, raw_values_count: 2, token_types_start: 707, token_types_count: 0 }, // [304]
    SimpleHintData { raw_values_start: 707, raw_values_count: 1, token_types_start: 708, token_types_count: 0 }, // [305]
    SimpleHintData { raw_values_start: 708, raw_values_count: 5, token_types_start: 713, token_types_count: 0 }, // [306]
    SimpleHintData { raw_values_start: 713, raw_values_count: 2, token_types_start: 715, token_types_count: 0 }, // [307]
    SimpleHintData { raw_values_start: 715, raw_values_count: 18, token_types_start: 733, token_types_count: 0 }, // [308]
    SimpleHintData { raw_values_start: 733, raw_values_count: 2, token_types_start: 735, token_types_count: 0 }, // [309]
    SimpleHintData { raw_values_start: 735, raw_values_count: 1, token_types_start: 736, token_types_count: 0 }, // [310]
    SimpleHintData { raw_values_start: 736, raw_values_count: 1, token_types_start: 737, token_types_count: 0 }, // [311]
    SimpleHintData { raw_values_start: 737, raw_values_count: 2, token_types_start: 739, token_types_count: 0 }, // [312]
    SimpleHintData { raw_values_start: 739, raw_values_count: 1, token_types_start: 740, token_types_count: 0 }, // [313]
    SimpleHintData { raw_values_start: 740, raw_values_count: 1, token_types_start: 741, token_types_count: 0 }, // [314]
    SimpleHintData { raw_values_start: 741, raw_values_count: 2, token_types_start: 743, token_types_count: 0 }, // [315]
    SimpleHintData { raw_values_start: 743, raw_values_count: 1, token_types_start: 744, token_types_count: 0 }, // [316]
    SimpleHintData { raw_values_start: 744, raw_values_count: 1, token_types_start: 745, token_types_count: 0 }, // [317]
    SimpleHintData { raw_values_start: 745, raw_values_count: 2, token_types_start: 747, token_types_count: 0 }, // [318]
    SimpleHintData { raw_values_start: 747, raw_values_count: 1, token_types_start: 748, token_types_count: 0 }, // [319]
    SimpleHintData { raw_values_start: 748, raw_values_count: 2, token_types_start: 750, token_types_count: 0 }, // [320]
    SimpleHintData { raw_values_start: 750, raw_values_count: 1, token_types_start: 751, token_types_count: 0 }, // [321]
    SimpleHintData { raw_values_start: 751, raw_values_count: 1, token_types_start: 752, token_types_count: 0 }, // [322]
    SimpleHintData { raw_values_start: 752, raw_values_count: 2, token_types_start: 754, token_types_count: 0 }, // [323]
    SimpleHintData { raw_values_start: 754, raw_values_count: 1, token_types_start: 755, token_types_count: 0 }, // [324]
    SimpleHintData { raw_values_start: 755, raw_values_count: 1, token_types_start: 756, token_types_count: 0 }, // [325]
    SimpleHintData { raw_values_start: 756, raw_values_count: 2, token_types_start: 758, token_types_count: 0 }, // [326]
    SimpleHintData { raw_values_start: 758, raw_values_count: 1, token_types_start: 759, token_types_count: 0 }, // [327]
    SimpleHintData { raw_values_start: 759, raw_values_count: 1, token_types_start: 760, token_types_count: 0 }, // [328]
    SimpleHintData { raw_values_start: 760, raw_values_count: 2, token_types_start: 762, token_types_count: 0 }, // [329]
    SimpleHintData { raw_values_start: 762, raw_values_count: 1, token_types_start: 763, token_types_count: 0 }, // [330]
    SimpleHintData { raw_values_start: 763, raw_values_count: 1, token_types_start: 764, token_types_count: 0 }, // [331]
    SimpleHintData { raw_values_start: 764, raw_values_count: 1, token_types_start: 765, token_types_count: 6 }, // [332]
    SimpleHintData { raw_values_start: 771, raw_values_count: 1, token_types_start: 772, token_types_count: 0 }, // [333]
    SimpleHintData { raw_values_start: 772, raw_values_count: 3, token_types_start: 775, token_types_count: 0 }, // [334]
    SimpleHintData { raw_values_start: 775, raw_values_count: 1, token_types_start: 776, token_types_count: 0 }, // [335]
    SimpleHintData { raw_values_start: 776, raw_values_count: 1, token_types_start: 777, token_types_count: 0 }, // [336]
    SimpleHintData { raw_values_start: 777, raw_values_count: 2, token_types_start: 779, token_types_count: 0 }, // [337]
    SimpleHintData { raw_values_start: 779, raw_values_count: 2, token_types_start: 781, token_types_count: 0 }, // [338]
    SimpleHintData { raw_values_start: 781, raw_values_count: 1, token_types_start: 782, token_types_count: 0 }, // [339]
    SimpleHintData { raw_values_start: 782, raw_values_count: 2, token_types_start: 784, token_types_count: 0 }, // [340]
    SimpleHintData { raw_values_start: 784, raw_values_count: 1, token_types_start: 785, token_types_count: 0 }, // [341]
    SimpleHintData { raw_values_start: 785, raw_values_count: 1, token_types_start: 786, token_types_count: 0 }, // [342]
    SimpleHintData { raw_values_start: 786, raw_values_count: 1, token_types_start: 787, token_types_count: 0 }, // [343]
    SimpleHintData { raw_values_start: 787, raw_values_count: 1, token_types_start: 788, token_types_count: 0 }, // [344]
    SimpleHintData { raw_values_start: 788, raw_values_count: 2, token_types_start: 790, token_types_count: 0 }, // [345]
    SimpleHintData { raw_values_start: 790, raw_values_count: 1, token_types_start: 791, token_types_count: 0 }, // [346]
    SimpleHintData { raw_values_start: 791, raw_values_count: 1, token_types_start: 792, token_types_count: 0 }, // [347]
    SimpleHintData { raw_values_start: 792, raw_values_count: 14, token_types_start: 806, token_types_count: 0 }, // [348]
    SimpleHintData { raw_values_start: 806, raw_values_count: 2, token_types_start: 808, token_types_count: 0 }, // [349]
    SimpleHintData { raw_values_start: 808, raw_values_count: 1, token_types_start: 809, token_types_count: 0 }, // [350]
    SimpleHintData { raw_values_start: 809, raw_values_count: 10, token_types_start: 819, token_types_count: 0 }, // [351]
    SimpleHintData { raw_values_start: 819, raw_values_count: 1, token_types_start: 820, token_types_count: 0 }, // [352]
    SimpleHintData { raw_values_start: 820, raw_values_count: 1, token_types_start: 821, token_types_count: 0 }, // [353]
    SimpleHintData { raw_values_start: 821, raw_values_count: 1, token_types_start: 822, token_types_count: 0 }, // [354]
    SimpleHintData { raw_values_start: 822, raw_values_count: 8, token_types_start: 830, token_types_count: 0 }, // [355]
    SimpleHintData { raw_values_start: 830, raw_values_count: 2, token_types_start: 832, token_types_count: 0 }, // [356]
    SimpleHintData { raw_values_start: 832, raw_values_count: 1, token_types_start: 833, token_types_count: 0 }, // [357]
    SimpleHintData { raw_values_start: 833, raw_values_count: 13, token_types_start: 846, token_types_count: 0 }, // [358]
    SimpleHintData { raw_values_start: 846, raw_values_count: 1, token_types_start: 847, token_types_count: 0 }, // [359]
    SimpleHintData { raw_values_start: 847, raw_values_count: 1, token_types_start: 848, token_types_count: 0 }, // [360]
    SimpleHintData { raw_values_start: 848, raw_values_count: 4, token_types_start: 852, token_types_count: 0 }, // [361]
    SimpleHintData { raw_values_start: 852, raw_values_count: 1, token_types_start: 853, token_types_count: 0 }, // [362]
    SimpleHintData { raw_values_start: 853, raw_values_count: 1, token_types_start: 854, token_types_count: 0 }, // [363]
    SimpleHintData { raw_values_start: 854, raw_values_count: 1, token_types_start: 855, token_types_count: 0 }, // [364]
    SimpleHintData { raw_values_start: 855, raw_values_count: 1, token_types_start: 856, token_types_count: 0 }, // [365]
    SimpleHintData { raw_values_start: 856, raw_values_count: 1, token_types_start: 857, token_types_count: 0 }, // [366]
    SimpleHintData { raw_values_start: 857, raw_values_count: 2, token_types_start: 859, token_types_count: 0 }, // [367]
    SimpleHintData { raw_values_start: 859, raw_values_count: 1, token_types_start: 860, token_types_count: 0 }, // [368]
    SimpleHintData { raw_values_start: 860, raw_values_count: 1, token_types_start: 861, token_types_count: 0 }, // [369]
    SimpleHintData { raw_values_start: 861, raw_values_count: 1, token_types_start: 862, token_types_count: 0 }, // [370]
    SimpleHintData { raw_values_start: 862, raw_values_count: 1, token_types_start: 863, token_types_count: 0 }, // [371]
    SimpleHintData { raw_values_start: 863, raw_values_count: 2, token_types_start: 865, token_types_count: 0 }, // [372]
    SimpleHintData { raw_values_start: 865, raw_values_count: 3, token_types_start: 868, token_types_count: 0 }, // [373]
    SimpleHintData { raw_values_start: 868, raw_values_count: 2, token_types_start: 870, token_types_count: 0 }, // [374]
    SimpleHintData { raw_values_start: 870, raw_values_count: 1, token_types_start: 871, token_types_count: 0 }, // [375]
    SimpleHintData { raw_values_start: 871, raw_values_count: 2, token_types_start: 873, token_types_count: 0 }, // [376]
    SimpleHintData { raw_values_start: 873, raw_values_count: 4, token_types_start: 877, token_types_count: 0 }, // [377]
    SimpleHintData { raw_values_start: 877, raw_values_count: 2, token_types_start: 879, token_types_count: 0 }, // [378]
    SimpleHintData { raw_values_start: 879, raw_values_count: 1, token_types_start: 880, token_types_count: 0 }, // [379]
    SimpleHintData { raw_values_start: 880, raw_values_count: 3, token_types_start: 883, token_types_count: 0 }, // [380]
    SimpleHintData { raw_values_start: 883, raw_values_count: 1, token_types_start: 884, token_types_count: 0 }, // [381]
    SimpleHintData { raw_values_start: 884, raw_values_count: 1, token_types_start: 885, token_types_count: 0 }, // [382]
    SimpleHintData { raw_values_start: 885, raw_values_count: 1, token_types_start: 886, token_types_count: 0 }, // [383]
    SimpleHintData { raw_values_start: 886, raw_values_count: 6, token_types_start: 892, token_types_count: 0 }, // [384]
    SimpleHintData { raw_values_start: 892, raw_values_count: 1, token_types_start: 893, token_types_count: 0 }, // [385]
    SimpleHintData { raw_values_start: 893, raw_values_count: 1, token_types_start: 894, token_types_count: 0 }, // [386]
    SimpleHintData { raw_values_start: 894, raw_values_count: 3, token_types_start: 897, token_types_count: 0 }, // [387]
    SimpleHintData { raw_values_start: 897, raw_values_count: 1, token_types_start: 898, token_types_count: 0 }, // [388]
    SimpleHintData { raw_values_start: 898, raw_values_count: 4, token_types_start: 902, token_types_count: 0 }, // [389]
    SimpleHintData { raw_values_start: 902, raw_values_count: 1, token_types_start: 903, token_types_count: 0 }, // [390]
    SimpleHintData { raw_values_start: 903, raw_values_count: 2, token_types_start: 905, token_types_count: 0 }, // [391]
    SimpleHintData { raw_values_start: 905, raw_values_count: 1, token_types_start: 906, token_types_count: 0 }, // [392]
    SimpleHintData { raw_values_start: 906, raw_values_count: 11, token_types_start: 917, token_types_count: 0 }, // [393]
    SimpleHintData { raw_values_start: 917, raw_values_count: 2, token_types_start: 919, token_types_count: 0 }, // [394]
    SimpleHintData { raw_values_start: 919, raw_values_count: 1, token_types_start: 920, token_types_count: 0 }, // [395]
    SimpleHintData { raw_values_start: 920, raw_values_count: 1, token_types_start: 921, token_types_count: 0 }, // [396]
    SimpleHintData { raw_values_start: 921, raw_values_count: 1, token_types_start: 922, token_types_count: 0 }, // [397]
    SimpleHintData { raw_values_start: 922, raw_values_count: 2, token_types_start: 924, token_types_count: 0 }, // [398]
    SimpleHintData { raw_values_start: 924, raw_values_count: 1, token_types_start: 925, token_types_count: 0 }, // [399]
    SimpleHintData { raw_values_start: 925, raw_values_count: 1, token_types_start: 926, token_types_count: 6 }, // [400]
    SimpleHintData { raw_values_start: 932, raw_values_count: 2, token_types_start: 934, token_types_count: 0 }, // [401]
    SimpleHintData { raw_values_start: 934, raw_values_count: 1, token_types_start: 935, token_types_count: 0 }, // [402]
    SimpleHintData { raw_values_start: 935, raw_values_count: 1, token_types_start: 936, token_types_count: 0 }, // [403]
    SimpleHintData { raw_values_start: 936, raw_values_count: 17, token_types_start: 953, token_types_count: 8 }, // [404]
    SimpleHintData { raw_values_start: 961, raw_values_count: 5, token_types_start: 966, token_types_count: 0 }, // [405]
    SimpleHintData { raw_values_start: 966, raw_values_count: 1, token_types_start: 967, token_types_count: 0 }, // [406]
    SimpleHintData { raw_values_start: 967, raw_values_count: 3, token_types_start: 970, token_types_count: 0 }, // [407]
    SimpleHintData { raw_values_start: 970, raw_values_count: 2, token_types_start: 972, token_types_count: 0 }, // [408]
    SimpleHintData { raw_values_start: 972, raw_values_count: 1, token_types_start: 973, token_types_count: 0 }, // [409]
    SimpleHintData { raw_values_start: 973, raw_values_count: 4, token_types_start: 977, token_types_count: 0 }, // [410]
    SimpleHintData { raw_values_start: 977, raw_values_count: 2, token_types_start: 979, token_types_count: 0 }, // [411]
    SimpleHintData { raw_values_start: 979, raw_values_count: 2, token_types_start: 981, token_types_count: 0 }, // [412]
    SimpleHintData { raw_values_start: 981, raw_values_count: 1, token_types_start: 982, token_types_count: 0 }, // [413]
    SimpleHintData { raw_values_start: 982, raw_values_count: 1, token_types_start: 983, token_types_count: 0 }, // [414]
    SimpleHintData { raw_values_start: 983, raw_values_count: 1, token_types_start: 984, token_types_count: 0 }, // [415]
    SimpleHintData { raw_values_start: 984, raw_values_count: 1, token_types_start: 985, token_types_count: 0 }, // [416]
    SimpleHintData { raw_values_start: 985, raw_values_count: 2, token_types_start: 987, token_types_count: 0 }, // [417]
    SimpleHintData { raw_values_start: 987, raw_values_count: 1, token_types_start: 988, token_types_count: 0 }, // [418]
    SimpleHintData { raw_values_start: 988, raw_values_count: 1, token_types_start: 989, token_types_count: 0 }, // [419]
    SimpleHintData { raw_values_start: 989, raw_values_count: 2, token_types_start: 991, token_types_count: 0 }, // [420]
    SimpleHintData { raw_values_start: 991, raw_values_count: 1, token_types_start: 992, token_types_count: 0 }, // [421]
    SimpleHintData { raw_values_start: 992, raw_values_count: 1, token_types_start: 993, token_types_count: 0 }, // [422]
    SimpleHintData { raw_values_start: 993, raw_values_count: 1, token_types_start: 994, token_types_count: 0 }, // [423]
    SimpleHintData { raw_values_start: 994, raw_values_count: 1, token_types_start: 995, token_types_count: 0 }, // [424]
    SimpleHintData { raw_values_start: 995, raw_values_count: 1, token_types_start: 996, token_types_count: 0 }, // [425]
    SimpleHintData { raw_values_start: 996, raw_values_count: 1, token_types_start: 997, token_types_count: 0 }, // [426]
    SimpleHintData { raw_values_start: 997, raw_values_count: 3, token_types_start: 1000, token_types_count: 0 }, // [427]
    SimpleHintData { raw_values_start: 1000, raw_values_count: 1, token_types_start: 1001, token_types_count: 0 }, // [428]
    SimpleHintData { raw_values_start: 1001, raw_values_count: 1, token_types_start: 1002, token_types_count: 0 }, // [429]
    SimpleHintData { raw_values_start: 1002, raw_values_count: 10, token_types_start: 1012, token_types_count: 0 }, // [430]
    SimpleHintData { raw_values_start: 1012, raw_values_count: 1, token_types_start: 1013, token_types_count: 0 }, // [431]
    SimpleHintData { raw_values_start: 1013, raw_values_count: 1, token_types_start: 1014, token_types_count: 0 }, // [432]
    SimpleHintData { raw_values_start: 1014, raw_values_count: 1, token_types_start: 1015, token_types_count: 0 }, // [433]
    SimpleHintData { raw_values_start: 1015, raw_values_count: 1, token_types_start: 1016, token_types_count: 0 }, // [434]
    SimpleHintData { raw_values_start: 1016, raw_values_count: 1, token_types_start: 1017, token_types_count: 0 }, // [435]
    SimpleHintData { raw_values_start: 1017, raw_values_count: 1, token_types_start: 1018, token_types_count: 0 }, // [436]
    SimpleHintData { raw_values_start: 1018, raw_values_count: 1, token_types_start: 1019, token_types_count: 0 }, // [437]
    SimpleHintData { raw_values_start: 1019, raw_values_count: 1, token_types_start: 1020, token_types_count: 0 }, // [438]
    SimpleHintData { raw_values_start: 1020, raw_values_count: 1, token_types_start: 1021, token_types_count: 0 }, // [439]
    SimpleHintData { raw_values_start: 1021, raw_values_count: 1, token_types_start: 1022, token_types_count: 0 }, // [440]
    SimpleHintData { raw_values_start: 1022, raw_values_count: 1, token_types_start: 1023, token_types_count: 0 }, // [441]
    SimpleHintData { raw_values_start: 1023, raw_values_count: 1, token_types_start: 1024, token_types_count: 0 }, // [442]
    SimpleHintData { raw_values_start: 1024, raw_values_count: 1, token_types_start: 1025, token_types_count: 0 }, // [443]
    SimpleHintData { raw_values_start: 1025, raw_values_count: 1, token_types_start: 1026, token_types_count: 0 }, // [444]
    SimpleHintData { raw_values_start: 1026, raw_values_count: 1, token_types_start: 1027, token_types_count: 0 }, // [445]
    SimpleHintData { raw_values_start: 1027, raw_values_count: 1, token_types_start: 1028, token_types_count: 0 }, // [446]
    SimpleHintData { raw_values_start: 1028, raw_values_count: 1, token_types_start: 1029, token_types_count: 0 }, // [447]
    SimpleHintData { raw_values_start: 1029, raw_values_count: 1, token_types_start: 1030, token_types_count: 0 }, // [448]
    SimpleHintData { raw_values_start: 1030, raw_values_count: 1, token_types_start: 1031, token_types_count: 0 }, // [449]
    SimpleHintData { raw_values_start: 1031, raw_values_count: 1, token_types_start: 1032, token_types_count: 0 }, // [450]
    SimpleHintData { raw_values_start: 1032, raw_values_count: 1, token_types_start: 1033, token_types_count: 0 }, // [451]
    SimpleHintData { raw_values_start: 1033, raw_values_count: 1, token_types_start: 1034, token_types_count: 0 }, // [452]
    SimpleHintData { raw_values_start: 1034, raw_values_count: 1, token_types_start: 1035, token_types_count: 0 }, // [453]
    SimpleHintData { raw_values_start: 1035, raw_values_count: 1, token_types_start: 1036, token_types_count: 0 }, // [454]
    SimpleHintData { raw_values_start: 1036, raw_values_count: 6, token_types_start: 1042, token_types_count: 0 }, // [455]
    SimpleHintData { raw_values_start: 1042, raw_values_count: 4, token_types_start: 1046, token_types_count: 0 }, // [456]
    SimpleHintData { raw_values_start: 1046, raw_values_count: 1, token_types_start: 1047, token_types_count: 0 }, // [457]
    SimpleHintData { raw_values_start: 1047, raw_values_count: 1, token_types_start: 1048, token_types_count: 6 }, // [458]
    SimpleHintData { raw_values_start: 1054, raw_values_count: 2, token_types_start: 1056, token_types_count: 0 }, // [459]
    SimpleHintData { raw_values_start: 1056, raw_values_count: 1, token_types_start: 1057, token_types_count: 0 }, // [460]
    SimpleHintData { raw_values_start: 1057, raw_values_count: 1, token_types_start: 1058, token_types_count: 0 }, // [461]
    SimpleHintData { raw_values_start: 1058, raw_values_count: 1, token_types_start: 1059, token_types_count: 0 }, // [462]
    SimpleHintData { raw_values_start: 1059, raw_values_count: 1, token_types_start: 1060, token_types_count: 0 }, // [463]
    SimpleHintData { raw_values_start: 1060, raw_values_count: 1, token_types_start: 1061, token_types_count: 0 }, // [464]
    SimpleHintData { raw_values_start: 1061, raw_values_count: 1, token_types_start: 1062, token_types_count: 0 }, // [465]
    SimpleHintData { raw_values_start: 1062, raw_values_count: 1, token_types_start: 1063, token_types_count: 0 }, // [466]
    SimpleHintData { raw_values_start: 1063, raw_values_count: 1, token_types_start: 1064, token_types_count: 0 }, // [467]
    SimpleHintData { raw_values_start: 1064, raw_values_count: 1, token_types_start: 1065, token_types_count: 0 }, // [468]
    SimpleHintData { raw_values_start: 1065, raw_values_count: 1, token_types_start: 1066, token_types_count: 0 }, // [469]
    SimpleHintData { raw_values_start: 1066, raw_values_count: 1, token_types_start: 1067, token_types_count: 0 }, // [470]
    SimpleHintData { raw_values_start: 1067, raw_values_count: 1, token_types_start: 1068, token_types_count: 0 }, // [471]
    SimpleHintData { raw_values_start: 1068, raw_values_count: 1, token_types_start: 1069, token_types_count: 0 }, // [472]
    SimpleHintData { raw_values_start: 1069, raw_values_count: 1, token_types_start: 1070, token_types_count: 0 }, // [473]
    SimpleHintData { raw_values_start: 1070, raw_values_count: 1, token_types_start: 1071, token_types_count: 0 }, // [474]
    SimpleHintData { raw_values_start: 1071, raw_values_count: 1, token_types_start: 1072, token_types_count: 0 }, // [475]
    SimpleHintData { raw_values_start: 1072, raw_values_count: 1, token_types_start: 1073, token_types_count: 0 }, // [476]
    SimpleHintData { raw_values_start: 1073, raw_values_count: 1, token_types_start: 1074, token_types_count: 0 }, // [477]
    SimpleHintData { raw_values_start: 1074, raw_values_count: 1, token_types_start: 1075, token_types_count: 0 }, // [478]
    SimpleHintData { raw_values_start: 1075, raw_values_count: 1, token_types_start: 1076, token_types_count: 0 }, // [479]
    SimpleHintData { raw_values_start: 1076, raw_values_count: 17, token_types_start: 1093, token_types_count: 5 }, // [480]
    SimpleHintData { raw_values_start: 1098, raw_values_count: 2, token_types_start: 1100, token_types_count: 0 }, // [481]
    SimpleHintData { raw_values_start: 1100, raw_values_count: 2, token_types_start: 1102, token_types_count: 0 }, // [482]
    SimpleHintData { raw_values_start: 1102, raw_values_count: 8, token_types_start: 1110, token_types_count: 4 }, // [483]
    SimpleHintData { raw_values_start: 1114, raw_values_count: 0, token_types_start: 1114, token_types_count: 1 }, // [484]
    SimpleHintData { raw_values_start: 1115, raw_values_count: 1, token_types_start: 1116, token_types_count: 0 }, // [485]
    SimpleHintData { raw_values_start: 1116, raw_values_count: 1, token_types_start: 1117, token_types_count: 0 }, // [486]
    SimpleHintData { raw_values_start: 1117, raw_values_count: 1, token_types_start: 1118, token_types_count: 0 }, // [487]
    SimpleHintData { raw_values_start: 1118, raw_values_count: 1, token_types_start: 1119, token_types_count: 0 }, // [488]
    SimpleHintData { raw_values_start: 1119, raw_values_count: 1, token_types_start: 1120, token_types_count: 0 }, // [489]
    SimpleHintData { raw_values_start: 1120, raw_values_count: 1, token_types_start: 1121, token_types_count: 0 }, // [490]
    SimpleHintData { raw_values_start: 1121, raw_values_count: 1, token_types_start: 1122, token_types_count: 0 }, // [491]
    SimpleHintData { raw_values_start: 1122, raw_values_count: 1, token_types_start: 1123, token_types_count: 0 }, // [492]
    SimpleHintData { raw_values_start: 1123, raw_values_count: 1, token_types_start: 1124, token_types_count: 0 }, // [493]
    SimpleHintData { raw_values_start: 1124, raw_values_count: 1, token_types_start: 1125, token_types_count: 0 }, // [494]
    SimpleHintData { raw_values_start: 1125, raw_values_count: 1, token_types_start: 1126, token_types_count: 0 }, // [495]
    SimpleHintData { raw_values_start: 1126, raw_values_count: 1, token_types_start: 1127, token_types_count: 0 }, // [496]
    SimpleHintData { raw_values_start: 1127, raw_values_count: 1, token_types_start: 1128, token_types_count: 0 }, // [497]
    SimpleHintData { raw_values_start: 1128, raw_values_count: 1, token_types_start: 1129, token_types_count: 0 }, // [498]
    SimpleHintData { raw_values_start: 1129, raw_values_count: 1, token_types_start: 1130, token_types_count: 0 }, // [499]
    SimpleHintData { raw_values_start: 1130, raw_values_count: 1, token_types_start: 1131, token_types_count: 0 }, // [500]
    SimpleHintData { raw_values_start: 1131, raw_values_count: 1, token_types_start: 1132, token_types_count: 0 }, // [501]
    SimpleHintData { raw_values_start: 1132, raw_values_count: 3, token_types_start: 1135, token_types_count: 0 }, // [502]
    SimpleHintData { raw_values_start: 1135, raw_values_count: 1, token_types_start: 1136, token_types_count: 0 }, // [503]
    SimpleHintData { raw_values_start: 1136, raw_values_count: 1, token_types_start: 1137, token_types_count: 0 }, // [504]
    SimpleHintData { raw_values_start: 1137, raw_values_count: 1, token_types_start: 1138, token_types_count: 0 }, // [505]
    SimpleHintData { raw_values_start: 1138, raw_values_count: 1, token_types_start: 1139, token_types_count: 0 }, // [506]
    SimpleHintData { raw_values_start: 1139, raw_values_count: 1, token_types_start: 1140, token_types_count: 0 }, // [507]
    SimpleHintData { raw_values_start: 1140, raw_values_count: 2, token_types_start: 1142, token_types_count: 0 }, // [508]
    SimpleHintData { raw_values_start: 1142, raw_values_count: 7, token_types_start: 1149, token_types_count: 0 }, // [509]
    SimpleHintData { raw_values_start: 1149, raw_values_count: 2, token_types_start: 1151, token_types_count: 0 }, // [510]
    SimpleHintData { raw_values_start: 1151, raw_values_count: 2, token_types_start: 1153, token_types_count: 0 }, // [511]
    SimpleHintData { raw_values_start: 1153, raw_values_count: 1, token_types_start: 1154, token_types_count: 0 }, // [512]
    SimpleHintData { raw_values_start: 1154, raw_values_count: 1, token_types_start: 1155, token_types_count: 0 }, // [513]
    SimpleHintData { raw_values_start: 1155, raw_values_count: 1, token_types_start: 1156, token_types_count: 0 }, // [514]
    SimpleHintData { raw_values_start: 1156, raw_values_count: 1, token_types_start: 1157, token_types_count: 0 }, // [515]
    SimpleHintData { raw_values_start: 1157, raw_values_count: 1, token_types_start: 1158, token_types_count: 0 }, // [516]
    SimpleHintData { raw_values_start: 1158, raw_values_count: 1, token_types_start: 1159, token_types_count: 0 }, // [517]
    SimpleHintData { raw_values_start: 1159, raw_values_count: 1, token_types_start: 1160, token_types_count: 0 }, // [518]
    SimpleHintData { raw_values_start: 1160, raw_values_count: 1, token_types_start: 1161, token_types_count: 0 }, // [519]
    SimpleHintData { raw_values_start: 1161, raw_values_count: 1, token_types_start: 1162, token_types_count: 0 }, // [520]
    SimpleHintData { raw_values_start: 1162, raw_values_count: 1, token_types_start: 1163, token_types_count: 0 }, // [521]
    SimpleHintData { raw_values_start: 1163, raw_values_count: 1, token_types_start: 1164, token_types_count: 0 }, // [522]
    SimpleHintData { raw_values_start: 1164, raw_values_count: 1, token_types_start: 1165, token_types_count: 0 }, // [523]
    SimpleHintData { raw_values_start: 1165, raw_values_count: 1, token_types_start: 1166, token_types_count: 0 }, // [524]
    SimpleHintData { raw_values_start: 1166, raw_values_count: 1, token_types_start: 1167, token_types_count: 0 }, // [525]
    SimpleHintData { raw_values_start: 1167, raw_values_count: 1, token_types_start: 1168, token_types_count: 0 }, // [526]
    SimpleHintData { raw_values_start: 1168, raw_values_count: 1, token_types_start: 1169, token_types_count: 0 }, // [527]
    SimpleHintData { raw_values_start: 1169, raw_values_count: 1, token_types_start: 1170, token_types_count: 0 }, // [528]
    SimpleHintData { raw_values_start: 1170, raw_values_count: 1, token_types_start: 1171, token_types_count: 0 }, // [529]
    SimpleHintData { raw_values_start: 1171, raw_values_count: 1, token_types_start: 1172, token_types_count: 0 }, // [530]
    SimpleHintData { raw_values_start: 1172, raw_values_count: 1, token_types_start: 1173, token_types_count: 0 }, // [531]
    SimpleHintData { raw_values_start: 1173, raw_values_count: 1, token_types_start: 1174, token_types_count: 0 }, // [532]
    SimpleHintData { raw_values_start: 1174, raw_values_count: 1, token_types_start: 1175, token_types_count: 0 }, // [533]
    SimpleHintData { raw_values_start: 1175, raw_values_count: 1, token_types_start: 1176, token_types_count: 0 }, // [534]
    SimpleHintData { raw_values_start: 1176, raw_values_count: 1, token_types_start: 1177, token_types_count: 0 }, // [535]
    SimpleHintData { raw_values_start: 1177, raw_values_count: 1, token_types_start: 1178, token_types_count: 0 }, // [536]
    SimpleHintData { raw_values_start: 1178, raw_values_count: 1, token_types_start: 1179, token_types_count: 0 }, // [537]
    SimpleHintData { raw_values_start: 1179, raw_values_count: 1, token_types_start: 1180, token_types_count: 0 }, // [538]
    SimpleHintData { raw_values_start: 1180, raw_values_count: 1, token_types_start: 1181, token_types_count: 0 }, // [539]
    SimpleHintData { raw_values_start: 1181, raw_values_count: 1, token_types_start: 1182, token_types_count: 0 }, // [540]
    SimpleHintData { raw_values_start: 1182, raw_values_count: 1, token_types_start: 1183, token_types_count: 0 }, // [541]
    SimpleHintData { raw_values_start: 1183, raw_values_count: 1, token_types_start: 1184, token_types_count: 0 }, // [542]
    SimpleHintData { raw_values_start: 1184, raw_values_count: 1, token_types_start: 1185, token_types_count: 0 }, // [543]
    SimpleHintData { raw_values_start: 1185, raw_values_count: 1, token_types_start: 1186, token_types_count: 0 }, // [544]
    SimpleHintData { raw_values_start: 1186, raw_values_count: 1, token_types_start: 1187, token_types_count: 0 }, // [545]
    SimpleHintData { raw_values_start: 1187, raw_values_count: 1, token_types_start: 1188, token_types_count: 0 }, // [546]
    SimpleHintData { raw_values_start: 1188, raw_values_count: 4, token_types_start: 1192, token_types_count: 0 }, // [547]
    SimpleHintData { raw_values_start: 1192, raw_values_count: 1, token_types_start: 1193, token_types_count: 0 }, // [548]
    SimpleHintData { raw_values_start: 1193, raw_values_count: 28, token_types_start: 1221, token_types_count: 5 }, // [549]
    SimpleHintData { raw_values_start: 1226, raw_values_count: 1, token_types_start: 1227, token_types_count: 0 }, // [550]
    SimpleHintData { raw_values_start: 1227, raw_values_count: 1, token_types_start: 1228, token_types_count: 0 }, // [551]
    SimpleHintData { raw_values_start: 1228, raw_values_count: 1, token_types_start: 1229, token_types_count: 0 }, // [552]
    SimpleHintData { raw_values_start: 1229, raw_values_count: 1, token_types_start: 1230, token_types_count: 0 }, // [553]
    SimpleHintData { raw_values_start: 1230, raw_values_count: 1, token_types_start: 1231, token_types_count: 0 }, // [554]
    SimpleHintData { raw_values_start: 1231, raw_values_count: 7, token_types_start: 1238, token_types_count: 0 }, // [555]
    SimpleHintData { raw_values_start: 1238, raw_values_count: 2, token_types_start: 1240, token_types_count: 0 }, // [556]
    SimpleHintData { raw_values_start: 1240, raw_values_count: 2, token_types_start: 1242, token_types_count: 0 }, // [557]
    SimpleHintData { raw_values_start: 1242, raw_values_count: 1, token_types_start: 1243, token_types_count: 0 }, // [558]
    SimpleHintData { raw_values_start: 1243, raw_values_count: 1, token_types_start: 1244, token_types_count: 0 }, // [559]
    SimpleHintData { raw_values_start: 1244, raw_values_count: 6, token_types_start: 1250, token_types_count: 0 }, // [560]
    SimpleHintData { raw_values_start: 1250, raw_values_count: 1, token_types_start: 1251, token_types_count: 0 }, // [561]
    SimpleHintData { raw_values_start: 1251, raw_values_count: 1, token_types_start: 1252, token_types_count: 0 }, // [562]
    SimpleHintData { raw_values_start: 1252, raw_values_count: 1, token_types_start: 1253, token_types_count: 0 }, // [563]
    SimpleHintData { raw_values_start: 1253, raw_values_count: 1, token_types_start: 1254, token_types_count: 0 }, // [564]
    SimpleHintData { raw_values_start: 1254, raw_values_count: 1, token_types_start: 1255, token_types_count: 0 }, // [565]
    SimpleHintData { raw_values_start: 1255, raw_values_count: 1, token_types_start: 1256, token_types_count: 0 }, // [566]
    SimpleHintData { raw_values_start: 1256, raw_values_count: 1, token_types_start: 1257, token_types_count: 0 }, // [567]
    SimpleHintData { raw_values_start: 1257, raw_values_count: 1, token_types_start: 1258, token_types_count: 0 }, // [568]
    SimpleHintData { raw_values_start: 1258, raw_values_count: 1, token_types_start: 1259, token_types_count: 0 }, // [569]
    SimpleHintData { raw_values_start: 1259, raw_values_count: 1, token_types_start: 1260, token_types_count: 0 }, // [570]
    SimpleHintData { raw_values_start: 1260, raw_values_count: 1, token_types_start: 1261, token_types_count: 0 }, // [571]
    SimpleHintData { raw_values_start: 1261, raw_values_count: 1, token_types_start: 1262, token_types_count: 0 }, // [572]
    SimpleHintData { raw_values_start: 1262, raw_values_count: 2, token_types_start: 1264, token_types_count: 0 }, // [573]
    SimpleHintData { raw_values_start: 1264, raw_values_count: 1, token_types_start: 1265, token_types_count: 0 }, // [574]
    SimpleHintData { raw_values_start: 1265, raw_values_count: 1, token_types_start: 1266, token_types_count: 0 }, // [575]
    SimpleHintData { raw_values_start: 1266, raw_values_count: 1, token_types_start: 1267, token_types_count: 0 }, // [576]
    SimpleHintData { raw_values_start: 1267, raw_values_count: 1, token_types_start: 1268, token_types_count: 0 }, // [577]
    SimpleHintData { raw_values_start: 1268, raw_values_count: 1, token_types_start: 1269, token_types_count: 0 }, // [578]
    SimpleHintData { raw_values_start: 1269, raw_values_count: 33, token_types_start: 1302, token_types_count: 0 }, // [579]
    SimpleHintData { raw_values_start: 1302, raw_values_count: 2, token_types_start: 1304, token_types_count: 0 }, // [580]
    SimpleHintData { raw_values_start: 1304, raw_values_count: 20, token_types_start: 1324, token_types_count: 0 }, // [581]
    SimpleHintData { raw_values_start: 1324, raw_values_count: 3, token_types_start: 1327, token_types_count: 0 }, // [582]
    SimpleHintData { raw_values_start: 1327, raw_values_count: 1, token_types_start: 1328, token_types_count: 0 }, // [583]
    SimpleHintData { raw_values_start: 1328, raw_values_count: 1, token_types_start: 1329, token_types_count: 0 }, // [584]
    SimpleHintData { raw_values_start: 1329, raw_values_count: 1, token_types_start: 1330, token_types_count: 0 }, // [585]
    SimpleHintData { raw_values_start: 1330, raw_values_count: 1, token_types_start: 1331, token_types_count: 0 }, // [586]
    SimpleHintData { raw_values_start: 1331, raw_values_count: 2, token_types_start: 1333, token_types_count: 0 }, // [587]
    SimpleHintData { raw_values_start: 1333, raw_values_count: 0, token_types_start: 1333, token_types_count: 1 }, // [588]
    SimpleHintData { raw_values_start: 1334, raw_values_count: 1, token_types_start: 1335, token_types_count: 0 }, // [589]
    SimpleHintData { raw_values_start: 1335, raw_values_count: 0, token_types_start: 1335, token_types_count: 1 }, // [590]
    SimpleHintData { raw_values_start: 1336, raw_values_count: 0, token_types_start: 1336, token_types_count: 1 }, // [591]
    SimpleHintData { raw_values_start: 1337, raw_values_count: 0, token_types_start: 1337, token_types_count: 1 }, // [592]
    SimpleHintData { raw_values_start: 1338, raw_values_count: 4, token_types_start: 1342, token_types_count: 0 }, // [593]
    SimpleHintData { raw_values_start: 1342, raw_values_count: 1, token_types_start: 1343, token_types_count: 0 }, // [594]
    SimpleHintData { raw_values_start: 1343, raw_values_count: 1, token_types_start: 1344, token_types_count: 0 }, // [595]
    SimpleHintData { raw_values_start: 1344, raw_values_count: 1, token_types_start: 1345, token_types_count: 0 }, // [596]
    SimpleHintData { raw_values_start: 1345, raw_values_count: 1, token_types_start: 1346, token_types_count: 0 }, // [597]
    SimpleHintData { raw_values_start: 1346, raw_values_count: 1, token_types_start: 1347, token_types_count: 0 }, // [598]
    SimpleHintData { raw_values_start: 1347, raw_values_count: 4, token_types_start: 1351, token_types_count: 0 }, // [599]
    SimpleHintData { raw_values_start: 1351, raw_values_count: 1, token_types_start: 1352, token_types_count: 0 }, // [600]
    SimpleHintData { raw_values_start: 1352, raw_values_count: 1, token_types_start: 1353, token_types_count: 0 }, // [601]
    SimpleHintData { raw_values_start: 1353, raw_values_count: 2, token_types_start: 1355, token_types_count: 0 }, // [602]
    SimpleHintData { raw_values_start: 1355, raw_values_count: 1, token_types_start: 1356, token_types_count: 0 }, // [603]
    SimpleHintData { raw_values_start: 1356, raw_values_count: 1, token_types_start: 1357, token_types_count: 0 }, // [604]
    SimpleHintData { raw_values_start: 1357, raw_values_count: 2, token_types_start: 1359, token_types_count: 0 }, // [605]
    SimpleHintData { raw_values_start: 1359, raw_values_count: 1, token_types_start: 1360, token_types_count: 1 }, // [606]
    SimpleHintData { raw_values_start: 1361, raw_values_count: 1, token_types_start: 1362, token_types_count: 0 }, // [607]
    SimpleHintData { raw_values_start: 1362, raw_values_count: 1, token_types_start: 1363, token_types_count: 0 }, // [608]
    SimpleHintData { raw_values_start: 1363, raw_values_count: 1, token_types_start: 1364, token_types_count: 0 }, // [609]
    SimpleHintData { raw_values_start: 1364, raw_values_count: 1, token_types_start: 1365, token_types_count: 0 }, // [610]
    SimpleHintData { raw_values_start: 1365, raw_values_count: 1, token_types_start: 1366, token_types_count: 0 }, // [611]
    SimpleHintData { raw_values_start: 1366, raw_values_count: 1, token_types_start: 1367, token_types_count: 0 }, // [612]
    SimpleHintData { raw_values_start: 1367, raw_values_count: 1, token_types_start: 1368, token_types_count: 0 }, // [613]
    SimpleHintData { raw_values_start: 1368, raw_values_count: 1, token_types_start: 1369, token_types_count: 0 }, // [614]
    SimpleHintData { raw_values_start: 1369, raw_values_count: 1, token_types_start: 1370, token_types_count: 0 }, // [615]
    SimpleHintData { raw_values_start: 1370, raw_values_count: 1, token_types_start: 1371, token_types_count: 0 }, // [616]
    SimpleHintData { raw_values_start: 1371, raw_values_count: 1, token_types_start: 1372, token_types_count: 0 }, // [617]
    SimpleHintData { raw_values_start: 1372, raw_values_count: 1, token_types_start: 1373, token_types_count: 0 }, // [618]
    SimpleHintData { raw_values_start: 1373, raw_values_count: 1, token_types_start: 1374, token_types_count: 0 }, // [619]
    SimpleHintData { raw_values_start: 1374, raw_values_count: 1, token_types_start: 1375, token_types_count: 0 }, // [620]
    SimpleHintData { raw_values_start: 1375, raw_values_count: 47, token_types_start: 1422, token_types_count: 0 }, // [621]
    SimpleHintData { raw_values_start: 1422, raw_values_count: 1, token_types_start: 1423, token_types_count: 0 }, // [622]
    SimpleHintData { raw_values_start: 1423, raw_values_count: 9, token_types_start: 1432, token_types_count: 0 }, // [623]
    SimpleHintData { raw_values_start: 1432, raw_values_count: 2, token_types_start: 1434, token_types_count: 0 }, // [624]
    SimpleHintData { raw_values_start: 1434, raw_values_count: 1, token_types_start: 1435, token_types_count: 0 }, // [625]
    SimpleHintData { raw_values_start: 1435, raw_values_count: 1, token_types_start: 1436, token_types_count: 0 }, // [626]
    SimpleHintData { raw_values_start: 1436, raw_values_count: 1, token_types_start: 1437, token_types_count: 0 }, // [627]
    SimpleHintData { raw_values_start: 1437, raw_values_count: 1, token_types_start: 1438, token_types_count: 0 }, // [628]
    SimpleHintData { raw_values_start: 1438, raw_values_count: 5, token_types_start: 1443, token_types_count: 0 }, // [629]
    SimpleHintData { raw_values_start: 1443, raw_values_count: 1, token_types_start: 1444, token_types_count: 0 }, // [630]
    SimpleHintData { raw_values_start: 1444, raw_values_count: 1, token_types_start: 1445, token_types_count: 0 }, // [631]
    SimpleHintData { raw_values_start: 1445, raw_values_count: 1, token_types_start: 1446, token_types_count: 0 }, // [632]
    SimpleHintData { raw_values_start: 1446, raw_values_count: 4, token_types_start: 1450, token_types_count: 0 }, // [633]
    SimpleHintData { raw_values_start: 1450, raw_values_count: 1, token_types_start: 1451, token_types_count: 0 }, // [634]
    SimpleHintData { raw_values_start: 1451, raw_values_count: 1, token_types_start: 1452, token_types_count: 0 }, // [635]
    SimpleHintData { raw_values_start: 1452, raw_values_count: 1, token_types_start: 1453, token_types_count: 6 }, // [636]
    SimpleHintData { raw_values_start: 1459, raw_values_count: 1, token_types_start: 1460, token_types_count: 0 }, // [637]
    SimpleHintData { raw_values_start: 1460, raw_values_count: 1, token_types_start: 1461, token_types_count: 0 }, // [638]
    SimpleHintData { raw_values_start: 1461, raw_values_count: 4, token_types_start: 1465, token_types_count: 0 }, // [639]
    SimpleHintData { raw_values_start: 1465, raw_values_count: 1, token_types_start: 1466, token_types_count: 0 }, // [640]
    SimpleHintData { raw_values_start: 1466, raw_values_count: 1, token_types_start: 1467, token_types_count: 0 }, // [641]
    SimpleHintData { raw_values_start: 1467, raw_values_count: 1, token_types_start: 1468, token_types_count: 0 }, // [642]
    SimpleHintData { raw_values_start: 1468, raw_values_count: 1, token_types_start: 1469, token_types_count: 0 }, // [643]
    SimpleHintData { raw_values_start: 1469, raw_values_count: 1, token_types_start: 1470, token_types_count: 0 }, // [644]
    SimpleHintData { raw_values_start: 1470, raw_values_count: 1, token_types_start: 1471, token_types_count: 0 }, // [645]
    SimpleHintData { raw_values_start: 1471, raw_values_count: 1, token_types_start: 1472, token_types_count: 0 }, // [646]
    SimpleHintData { raw_values_start: 1472, raw_values_count: 1, token_types_start: 1473, token_types_count: 0 }, // [647]
    SimpleHintData { raw_values_start: 1473, raw_values_count: 1, token_types_start: 1474, token_types_count: 0 }, // [648]
    SimpleHintData { raw_values_start: 1474, raw_values_count: 1, token_types_start: 1475, token_types_count: 0 }, // [649]
    SimpleHintData { raw_values_start: 1475, raw_values_count: 1, token_types_start: 1476, token_types_count: 0 }, // [650]
    SimpleHintData { raw_values_start: 1476, raw_values_count: 1, token_types_start: 1477, token_types_count: 0 }, // [651]
    SimpleHintData { raw_values_start: 1477, raw_values_count: 1, token_types_start: 1478, token_types_count: 0 }, // [652]
    SimpleHintData { raw_values_start: 1478, raw_values_count: 3, token_types_start: 1481, token_types_count: 6 }, // [653]
    SimpleHintData { raw_values_start: 1487, raw_values_count: 1, token_types_start: 1488, token_types_count: 0 }, // [654]
    SimpleHintData { raw_values_start: 1488, raw_values_count: 1, token_types_start: 1489, token_types_count: 0 }, // [655]
    SimpleHintData { raw_values_start: 1489, raw_values_count: 1, token_types_start: 1490, token_types_count: 0 }, // [656]
    SimpleHintData { raw_values_start: 1490, raw_values_count: 1, token_types_start: 1491, token_types_count: 0 }, // [657]
    SimpleHintData { raw_values_start: 1491, raw_values_count: 1, token_types_start: 1492, token_types_count: 0 }, // [658]
    SimpleHintData { raw_values_start: 1492, raw_values_count: 1, token_types_start: 1493, token_types_count: 0 }, // [659]
    SimpleHintData { raw_values_start: 1493, raw_values_count: 1, token_types_start: 1494, token_types_count: 0 }, // [660]
    SimpleHintData { raw_values_start: 1494, raw_values_count: 1, token_types_start: 1495, token_types_count: 0 }, // [661]
    SimpleHintData { raw_values_start: 1495, raw_values_count: 1, token_types_start: 1496, token_types_count: 0 }, // [662]
    SimpleHintData { raw_values_start: 1496, raw_values_count: 1, token_types_start: 1497, token_types_count: 0 }, // [663]
    SimpleHintData { raw_values_start: 1497, raw_values_count: 1, token_types_start: 1498, token_types_count: 0 }, // [664]
    SimpleHintData { raw_values_start: 1498, raw_values_count: 1, token_types_start: 1499, token_types_count: 0 }, // [665]
    SimpleHintData { raw_values_start: 1499, raw_values_count: 2, token_types_start: 1501, token_types_count: 0 }, // [666]
    SimpleHintData { raw_values_start: 1501, raw_values_count: 1, token_types_start: 1502, token_types_count: 0 }, // [667]
    SimpleHintData { raw_values_start: 1502, raw_values_count: 2, token_types_start: 1504, token_types_count: 0 }, // [668]
    SimpleHintData { raw_values_start: 1504, raw_values_count: 1, token_types_start: 1505, token_types_count: 0 }, // [669]
    SimpleHintData { raw_values_start: 1505, raw_values_count: 1, token_types_start: 1506, token_types_count: 0 }, // [670]
    SimpleHintData { raw_values_start: 1506, raw_values_count: 3, token_types_start: 1509, token_types_count: 0 }, // [671]
    SimpleHintData { raw_values_start: 1509, raw_values_count: 4, token_types_start: 1513, token_types_count: 0 }, // [672]
    SimpleHintData { raw_values_start: 1513, raw_values_count: 2, token_types_start: 1515, token_types_count: 0 }, // [673]
    SimpleHintData { raw_values_start: 1515, raw_values_count: 1, token_types_start: 1516, token_types_count: 0 }, // [674]
    SimpleHintData { raw_values_start: 1516, raw_values_count: 2, token_types_start: 1518, token_types_count: 0 }, // [675]
    SimpleHintData { raw_values_start: 1518, raw_values_count: 4, token_types_start: 1522, token_types_count: 0 }, // [676]
    SimpleHintData { raw_values_start: 1522, raw_values_count: 2, token_types_start: 1524, token_types_count: 0 }, // [677]
    SimpleHintData { raw_values_start: 1524, raw_values_count: 1, token_types_start: 1525, token_types_count: 0 }, // [678]
    SimpleHintData { raw_values_start: 1525, raw_values_count: 1, token_types_start: 1526, token_types_count: 0 }, // [679]
    SimpleHintData { raw_values_start: 1526, raw_values_count: 1, token_types_start: 1527, token_types_count: 0 }, // [680]
    SimpleHintData { raw_values_start: 1527, raw_values_count: 8, token_types_start: 1535, token_types_count: 0 }, // [681]
    SimpleHintData { raw_values_start: 1535, raw_values_count: 1, token_types_start: 1536, token_types_count: 0 }, // [682]
    SimpleHintData { raw_values_start: 1536, raw_values_count: 1, token_types_start: 1537, token_types_count: 0 }, // [683]
    SimpleHintData { raw_values_start: 1537, raw_values_count: 1, token_types_start: 1538, token_types_count: 0 }, // [684]
    SimpleHintData { raw_values_start: 1538, raw_values_count: 1, token_types_start: 1539, token_types_count: 0 }, // [685]
    SimpleHintData { raw_values_start: 1539, raw_values_count: 1, token_types_start: 1540, token_types_count: 0 }, // [686]
    SimpleHintData { raw_values_start: 1540, raw_values_count: 1, token_types_start: 1541, token_types_count: 0 }, // [687]
    SimpleHintData { raw_values_start: 1541, raw_values_count: 1, token_types_start: 1542, token_types_count: 0 }, // [688]
    SimpleHintData { raw_values_start: 1542, raw_values_count: 1, token_types_start: 1543, token_types_count: 0 }, // [689]
    SimpleHintData { raw_values_start: 1543, raw_values_count: 10, token_types_start: 1553, token_types_count: 0 }, // [690]
    SimpleHintData { raw_values_start: 1553, raw_values_count: 1, token_types_start: 1554, token_types_count: 0 }, // [691]
    SimpleHintData { raw_values_start: 1554, raw_values_count: 1, token_types_start: 1555, token_types_count: 0 }, // [692]
    SimpleHintData { raw_values_start: 1555, raw_values_count: 3, token_types_start: 1558, token_types_count: 0 }, // [693]
    SimpleHintData { raw_values_start: 1558, raw_values_count: 1, token_types_start: 1559, token_types_count: 0 }, // [694]
    SimpleHintData { raw_values_start: 1559, raw_values_count: 1, token_types_start: 1560, token_types_count: 0 }, // [695]
    SimpleHintData { raw_values_start: 1560, raw_values_count: 2, token_types_start: 1562, token_types_count: 0 }, // [696]
    SimpleHintData { raw_values_start: 1562, raw_values_count: 1, token_types_start: 1563, token_types_count: 0 }, // [697]
    SimpleHintData { raw_values_start: 1563, raw_values_count: 1, token_types_start: 1564, token_types_count: 0 }, // [698]
    SimpleHintData { raw_values_start: 1564, raw_values_count: 1, token_types_start: 1565, token_types_count: 0 }, // [699]
    SimpleHintData { raw_values_start: 1565, raw_values_count: 1, token_types_start: 1566, token_types_count: 0 }, // [700]
    SimpleHintData { raw_values_start: 1566, raw_values_count: 2, token_types_start: 1568, token_types_count: 0 }, // [701]
    SimpleHintData { raw_values_start: 1568, raw_values_count: 1, token_types_start: 1569, token_types_count: 0 }, // [702]
    SimpleHintData { raw_values_start: 1569, raw_values_count: 2, token_types_start: 1571, token_types_count: 0 }, // [703]
    SimpleHintData { raw_values_start: 1571, raw_values_count: 1, token_types_start: 1572, token_types_count: 0 }, // [704]
    SimpleHintData { raw_values_start: 1572, raw_values_count: 1, token_types_start: 1573, token_types_count: 0 }, // [705]
    SimpleHintData { raw_values_start: 1573, raw_values_count: 1, token_types_start: 1574, token_types_count: 0 }, // [706]
    SimpleHintData { raw_values_start: 1574, raw_values_count: 2, token_types_start: 1576, token_types_count: 0 }, // [707]
    SimpleHintData { raw_values_start: 1576, raw_values_count: 1, token_types_start: 1577, token_types_count: 0 }, // [708]
    SimpleHintData { raw_values_start: 1577, raw_values_count: 1, token_types_start: 1578, token_types_count: 0 }, // [709]
    SimpleHintData { raw_values_start: 1578, raw_values_count: 7, token_types_start: 1585, token_types_count: 0 }, // [710]
    SimpleHintData { raw_values_start: 1585, raw_values_count: 1, token_types_start: 1586, token_types_count: 0 }, // [711]
    SimpleHintData { raw_values_start: 1586, raw_values_count: 1, token_types_start: 1587, token_types_count: 0 }, // [712]
    SimpleHintData { raw_values_start: 1587, raw_values_count: 1, token_types_start: 1588, token_types_count: 0 }, // [713]
    SimpleHintData { raw_values_start: 1588, raw_values_count: 1, token_types_start: 1589, token_types_count: 0 }, // [714]
    SimpleHintData { raw_values_start: 1589, raw_values_count: 1, token_types_start: 1590, token_types_count: 0 }, // [715]
    SimpleHintData { raw_values_start: 1590, raw_values_count: 1, token_types_start: 1591, token_types_count: 0 }, // [716]
    SimpleHintData { raw_values_start: 1591, raw_values_count: 1, token_types_start: 1592, token_types_count: 0 }, // [717]
    SimpleHintData { raw_values_start: 1592, raw_values_count: 6, token_types_start: 1598, token_types_count: 0 }, // [718]
    SimpleHintData { raw_values_start: 1598, raw_values_count: 2, token_types_start: 1600, token_types_count: 0 }, // [719]
    SimpleHintData { raw_values_start: 1600, raw_values_count: 1, token_types_start: 1601, token_types_count: 0 }, // [720]
    SimpleHintData { raw_values_start: 1601, raw_values_count: 2, token_types_start: 1603, token_types_count: 0 }, // [721]
    SimpleHintData { raw_values_start: 1603, raw_values_count: 1, token_types_start: 1604, token_types_count: 0 }, // [722]
    SimpleHintData { raw_values_start: 1604, raw_values_count: 22, token_types_start: 1626, token_types_count: 0 }, // [723]
    SimpleHintData { raw_values_start: 1626, raw_values_count: 2, token_types_start: 1628, token_types_count: 0 }, // [724]
    SimpleHintData { raw_values_start: 1628, raw_values_count: 1, token_types_start: 1629, token_types_count: 0 }, // [725]
    SimpleHintData { raw_values_start: 1629, raw_values_count: 1, token_types_start: 1630, token_types_count: 0 }, // [726]
    SimpleHintData { raw_values_start: 1630, raw_values_count: 1, token_types_start: 1631, token_types_count: 0 }, // [727]
    SimpleHintData { raw_values_start: 1631, raw_values_count: 1, token_types_start: 1632, token_types_count: 0 }, // [728]
    SimpleHintData { raw_values_start: 1632, raw_values_count: 7, token_types_start: 1639, token_types_count: 0 }, // [729]
    SimpleHintData { raw_values_start: 1639, raw_values_count: 2, token_types_start: 1641, token_types_count: 0 }, // [730]
    SimpleHintData { raw_values_start: 1641, raw_values_count: 1, token_types_start: 1642, token_types_count: 0 }, // [731]
    SimpleHintData { raw_values_start: 1642, raw_values_count: 6, token_types_start: 1648, token_types_count: 0 }, // [732]
    SimpleHintData { raw_values_start: 1648, raw_values_count: 9, token_types_start: 1657, token_types_count: 0 }, // [733]
    SimpleHintData { raw_values_start: 1657, raw_values_count: 2, token_types_start: 1659, token_types_count: 0 }, // [734]
    SimpleHintData { raw_values_start: 1659, raw_values_count: 2, token_types_start: 1661, token_types_count: 0 }, // [735]
    SimpleHintData { raw_values_start: 1661, raw_values_count: 1, token_types_start: 1662, token_types_count: 0 }, // [736]
    SimpleHintData { raw_values_start: 1662, raw_values_count: 1, token_types_start: 1663, token_types_count: 0 }, // [737]
    SimpleHintData { raw_values_start: 1663, raw_values_count: 2, token_types_start: 1665, token_types_count: 0 }, // [738]
    SimpleHintData { raw_values_start: 1665, raw_values_count: 1, token_types_start: 1666, token_types_count: 0 }, // [739]
    SimpleHintData { raw_values_start: 1666, raw_values_count: 3, token_types_start: 1669, token_types_count: 0 }, // [740]
    SimpleHintData { raw_values_start: 1669, raw_values_count: 1, token_types_start: 1670, token_types_count: 0 }, // [741]
    SimpleHintData { raw_values_start: 1670, raw_values_count: 1, token_types_start: 1671, token_types_count: 0 }, // [742]
    SimpleHintData { raw_values_start: 1671, raw_values_count: 1, token_types_start: 1672, token_types_count: 0 }, // [743]
    SimpleHintData { raw_values_start: 1672, raw_values_count: 1, token_types_start: 1673, token_types_count: 0 }, // [744]
    SimpleHintData { raw_values_start: 1673, raw_values_count: 3, token_types_start: 1676, token_types_count: 0 }, // [745]
    SimpleHintData { raw_values_start: 1676, raw_values_count: 1, token_types_start: 1677, token_types_count: 0 }, // [746]
    SimpleHintData { raw_values_start: 1677, raw_values_count: 1, token_types_start: 1678, token_types_count: 0 }, // [747]
    SimpleHintData { raw_values_start: 1678, raw_values_count: 1, token_types_start: 1679, token_types_count: 0 }, // [748]
    SimpleHintData { raw_values_start: 1679, raw_values_count: 1, token_types_start: 1680, token_types_count: 0 }, // [749]
    SimpleHintData { raw_values_start: 1680, raw_values_count: 1, token_types_start: 1681, token_types_count: 0 }, // [750]
    SimpleHintData { raw_values_start: 1681, raw_values_count: 1, token_types_start: 1682, token_types_count: 0 }, // [751]
    SimpleHintData { raw_values_start: 1682, raw_values_count: 1, token_types_start: 1683, token_types_count: 0 }, // [752]
    SimpleHintData { raw_values_start: 1683, raw_values_count: 1, token_types_start: 1684, token_types_count: 0 }, // [753]
    SimpleHintData { raw_values_start: 1684, raw_values_count: 1, token_types_start: 1685, token_types_count: 0 }, // [754]
    SimpleHintData { raw_values_start: 1685, raw_values_count: 1, token_types_start: 1686, token_types_count: 0 }, // [755]
    SimpleHintData { raw_values_start: 1686, raw_values_count: 1, token_types_start: 1687, token_types_count: 0 }, // [756]
    SimpleHintData { raw_values_start: 1687, raw_values_count: 4, token_types_start: 1691, token_types_count: 0 }, // [757]
    SimpleHintData { raw_values_start: 1691, raw_values_count: 1, token_types_start: 1692, token_types_count: 0 }, // [758]
    SimpleHintData { raw_values_start: 1692, raw_values_count: 2, token_types_start: 1694, token_types_count: 0 }, // [759]
    SimpleHintData { raw_values_start: 1694, raw_values_count: 5, token_types_start: 1699, token_types_count: 0 }, // [760]
    SimpleHintData { raw_values_start: 1699, raw_values_count: 1, token_types_start: 1700, token_types_count: 0 }, // [761]
    SimpleHintData { raw_values_start: 1700, raw_values_count: 1, token_types_start: 1701, token_types_count: 0 }, // [762]
    SimpleHintData { raw_values_start: 1701, raw_values_count: 24, token_types_start: 1725, token_types_count: 0 }, // [763]
    SimpleHintData { raw_values_start: 1725, raw_values_count: 23, token_types_start: 1748, token_types_count: 0 }, // [764]
    SimpleHintData { raw_values_start: 1748, raw_values_count: 1, token_types_start: 1749, token_types_count: 0 }, // [765]
    SimpleHintData { raw_values_start: 1749, raw_values_count: 9, token_types_start: 1758, token_types_count: 0 }, // [766]
    SimpleHintData { raw_values_start: 1758, raw_values_count: 3, token_types_start: 1761, token_types_count: 0 }, // [767]
    SimpleHintData { raw_values_start: 1761, raw_values_count: 1, token_types_start: 1762, token_types_count: 0 }, // [768]
    SimpleHintData { raw_values_start: 1762, raw_values_count: 12, token_types_start: 1774, token_types_count: 0 }, // [769]
    SimpleHintData { raw_values_start: 1774, raw_values_count: 1, token_types_start: 1775, token_types_count: 0 }, // [770]
    SimpleHintData { raw_values_start: 1775, raw_values_count: 2, token_types_start: 1777, token_types_count: 0 }, // [771]
    SimpleHintData { raw_values_start: 1777, raw_values_count: 2, token_types_start: 1779, token_types_count: 0 }, // [772]
    SimpleHintData { raw_values_start: 1779, raw_values_count: 3, token_types_start: 1782, token_types_count: 0 }, // [773]
    SimpleHintData { raw_values_start: 1782, raw_values_count: 1, token_types_start: 1783, token_types_count: 0 }, // [774]
    SimpleHintData { raw_values_start: 1783, raw_values_count: 4, token_types_start: 1787, token_types_count: 0 }, // [775]
    SimpleHintData { raw_values_start: 1787, raw_values_count: 7, token_types_start: 1794, token_types_count: 0 }, // [776]
    SimpleHintData { raw_values_start: 1794, raw_values_count: 3, token_types_start: 1797, token_types_count: 0 }, // [777]
    SimpleHintData { raw_values_start: 1797, raw_values_count: 2, token_types_start: 1799, token_types_count: 0 }, // [778]
    SimpleHintData { raw_values_start: 1799, raw_values_count: 1, token_types_start: 1800, token_types_count: 0 }, // [779]
    SimpleHintData { raw_values_start: 1800, raw_values_count: 2, token_types_start: 1802, token_types_count: 0 }, // [780]
    SimpleHintData { raw_values_start: 1802, raw_values_count: 1, token_types_start: 1803, token_types_count: 0 }, // [781]
    SimpleHintData { raw_values_start: 1803, raw_values_count: 1, token_types_start: 1804, token_types_count: 0 }, // [782]
    SimpleHintData { raw_values_start: 1804, raw_values_count: 1, token_types_start: 1805, token_types_count: 0 }, // [783]
    SimpleHintData { raw_values_start: 1805, raw_values_count: 2, token_types_start: 1807, token_types_count: 0 }, // [784]
    SimpleHintData { raw_values_start: 1807, raw_values_count: 1, token_types_start: 1808, token_types_count: 0 }, // [785]
    SimpleHintData { raw_values_start: 1808, raw_values_count: 2, token_types_start: 1810, token_types_count: 0 }, // [786]
    SimpleHintData { raw_values_start: 1810, raw_values_count: 1, token_types_start: 1811, token_types_count: 0 }, // [787]
    SimpleHintData { raw_values_start: 1811, raw_values_count: 1, token_types_start: 1812, token_types_count: 0 }, // [788]
    SimpleHintData { raw_values_start: 1812, raw_values_count: 4, token_types_start: 1816, token_types_count: 0 }, // [789]
    SimpleHintData { raw_values_start: 1816, raw_values_count: 9, token_types_start: 1825, token_types_count: 0 }, // [790]
    SimpleHintData { raw_values_start: 1825, raw_values_count: 1, token_types_start: 1826, token_types_count: 0 }, // [791]
    SimpleHintData { raw_values_start: 1826, raw_values_count: 1, token_types_start: 1827, token_types_count: 0 }, // [792]
    SimpleHintData { raw_values_start: 1827, raw_values_count: 1, token_types_start: 1828, token_types_count: 0 }, // [793]
    SimpleHintData { raw_values_start: 1828, raw_values_count: 2, token_types_start: 1830, token_types_count: 0 }, // [794]
    SimpleHintData { raw_values_start: 1830, raw_values_count: 1, token_types_start: 1831, token_types_count: 0 }, // [795]
    SimpleHintData { raw_values_start: 1831, raw_values_count: 1, token_types_start: 1832, token_types_count: 0 }, // [796]
    SimpleHintData { raw_values_start: 1832, raw_values_count: 1, token_types_start: 1833, token_types_count: 0 }, // [797]
    SimpleHintData { raw_values_start: 1833, raw_values_count: 1, token_types_start: 1834, token_types_count: 0 }, // [798]
    SimpleHintData { raw_values_start: 1834, raw_values_count: 1, token_types_start: 1835, token_types_count: 0 }, // [799]
    SimpleHintData { raw_values_start: 1835, raw_values_count: 1, token_types_start: 1836, token_types_count: 0 }, // [800]
    SimpleHintData { raw_values_start: 1836, raw_values_count: 1, token_types_start: 1837, token_types_count: 0 }, // [801]
    SimpleHintData { raw_values_start: 1837, raw_values_count: 1, token_types_start: 1838, token_types_count: 0 }, // [802]
    SimpleHintData { raw_values_start: 1838, raw_values_count: 1, token_types_start: 1839, token_types_count: 0 }, // [803]
    SimpleHintData { raw_values_start: 1839, raw_values_count: 1, token_types_start: 1840, token_types_count: 0 }, // [804]
    SimpleHintData { raw_values_start: 1840, raw_values_count: 1, token_types_start: 1841, token_types_count: 0 }, // [805]
    SimpleHintData { raw_values_start: 1841, raw_values_count: 1, token_types_start: 1842, token_types_count: 0 }, // [806]
    SimpleHintData { raw_values_start: 1842, raw_values_count: 1, token_types_start: 1843, token_types_count: 0 }, // [807]
    SimpleHintData { raw_values_start: 1843, raw_values_count: 8, token_types_start: 1851, token_types_count: 0 }, // [808]
    SimpleHintData { raw_values_start: 1851, raw_values_count: 1, token_types_start: 1852, token_types_count: 0 }, // [809]
    SimpleHintData { raw_values_start: 1852, raw_values_count: 1, token_types_start: 1853, token_types_count: 0 }, // [810]
    SimpleHintData { raw_values_start: 1853, raw_values_count: 1, token_types_start: 1854, token_types_count: 0 }, // [811]
    SimpleHintData { raw_values_start: 1854, raw_values_count: 2, token_types_start: 1856, token_types_count: 0 }, // [812]
    SimpleHintData { raw_values_start: 1856, raw_values_count: 1, token_types_start: 1857, token_types_count: 0 }, // [813]
    SimpleHintData { raw_values_start: 1857, raw_values_count: 1, token_types_start: 1858, token_types_count: 6 }, // [814]
    SimpleHintData { raw_values_start: 1864, raw_values_count: 1, token_types_start: 1865, token_types_count: 0 }, // [815]
    SimpleHintData { raw_values_start: 1865, raw_values_count: 1, token_types_start: 1866, token_types_count: 0 }, // [816]
    SimpleHintData { raw_values_start: 1866, raw_values_count: 1, token_types_start: 1867, token_types_count: 0 }, // [817]
    SimpleHintData { raw_values_start: 1867, raw_values_count: 1, token_types_start: 1868, token_types_count: 0 }, // [818]
    SimpleHintData { raw_values_start: 1868, raw_values_count: 1, token_types_start: 1869, token_types_count: 0 }, // [819]
    SimpleHintData { raw_values_start: 1869, raw_values_count: 1, token_types_start: 1870, token_types_count: 0 }, // [820]
    SimpleHintData { raw_values_start: 1870, raw_values_count: 1, token_types_start: 1871, token_types_count: 0 }, // [821]
    SimpleHintData { raw_values_start: 1871, raw_values_count: 1, token_types_start: 1872, token_types_count: 0 }, // [822]
    SimpleHintData { raw_values_start: 1872, raw_values_count: 1, token_types_start: 1873, token_types_count: 0 }, // [823]
    SimpleHintData { raw_values_start: 1873, raw_values_count: 1, token_types_start: 1874, token_types_count: 0 }, // [824]
    SimpleHintData { raw_values_start: 1874, raw_values_count: 1, token_types_start: 1875, token_types_count: 0 }, // [825]
    SimpleHintData { raw_values_start: 1875, raw_values_count: 1, token_types_start: 1876, token_types_count: 0 }, // [826]
    SimpleHintData { raw_values_start: 1876, raw_values_count: 1, token_types_start: 1877, token_types_count: 0 }, // [827]
    SimpleHintData { raw_values_start: 1877, raw_values_count: 1, token_types_start: 1878, token_types_count: 0 }, // [828]
    SimpleHintData { raw_values_start: 1878, raw_values_count: 1, token_types_start: 1879, token_types_count: 0 }, // [829]
    SimpleHintData { raw_values_start: 1879, raw_values_count: 2, token_types_start: 1881, token_types_count: 0 }, // [830]
    SimpleHintData { raw_values_start: 1881, raw_values_count: 1, token_types_start: 1882, token_types_count: 0 }, // [831]
    SimpleHintData { raw_values_start: 1882, raw_values_count: 1, token_types_start: 1883, token_types_count: 0 }, // [832]
    SimpleHintData { raw_values_start: 1883, raw_values_count: 7, token_types_start: 1890, token_types_count: 0 }, // [833]
    SimpleHintData { raw_values_start: 1890, raw_values_count: 5, token_types_start: 1895, token_types_count: 0 }, // [834]
    SimpleHintData { raw_values_start: 1895, raw_values_count: 1, token_types_start: 1896, token_types_count: 0 }, // [835]
    SimpleHintData { raw_values_start: 1896, raw_values_count: 1, token_types_start: 1897, token_types_count: 0 }, // [836]
    SimpleHintData { raw_values_start: 1897, raw_values_count: 1, token_types_start: 1898, token_types_count: 0 }, // [837]
    SimpleHintData { raw_values_start: 1898, raw_values_count: 1, token_types_start: 1899, token_types_count: 0 }, // [838]
    SimpleHintData { raw_values_start: 1899, raw_values_count: 6, token_types_start: 1905, token_types_count: 0 }, // [839]
    SimpleHintData { raw_values_start: 1905, raw_values_count: 2, token_types_start: 1907, token_types_count: 0 }, // [840]
    SimpleHintData { raw_values_start: 1907, raw_values_count: 1, token_types_start: 1908, token_types_count: 0 }, // [841]
    SimpleHintData { raw_values_start: 1908, raw_values_count: 1, token_types_start: 1909, token_types_count: 0 }, // [842]
    SimpleHintData { raw_values_start: 1909, raw_values_count: 1, token_types_start: 1910, token_types_count: 0 }, // [843]
    SimpleHintData { raw_values_start: 1910, raw_values_count: 1, token_types_start: 1911, token_types_count: 0 }, // [844]
    SimpleHintData { raw_values_start: 1911, raw_values_count: 3, token_types_start: 1914, token_types_count: 0 }, // [845]
    SimpleHintData { raw_values_start: 1914, raw_values_count: 1, token_types_start: 1915, token_types_count: 0 }, // [846]
    SimpleHintData { raw_values_start: 1915, raw_values_count: 1, token_types_start: 1916, token_types_count: 0 }, // [847]
    SimpleHintData { raw_values_start: 1916, raw_values_count: 1, token_types_start: 1917, token_types_count: 0 }, // [848]
    SimpleHintData { raw_values_start: 1917, raw_values_count: 4, token_types_start: 1921, token_types_count: 0 }, // [849]
    SimpleHintData { raw_values_start: 1921, raw_values_count: 80, token_types_start: 2001, token_types_count: 0 }, // [850]
    SimpleHintData { raw_values_start: 2001, raw_values_count: 6, token_types_start: 2007, token_types_count: 6 }, // [851]
    SimpleHintData { raw_values_start: 2013, raw_values_count: 1, token_types_start: 2014, token_types_count: 0 }, // [852]
    SimpleHintData { raw_values_start: 2014, raw_values_count: 2, token_types_start: 2016, token_types_count: 0 }, // [853]
    SimpleHintData { raw_values_start: 2016, raw_values_count: 2, token_types_start: 2018, token_types_count: 0 }, // [854]
    SimpleHintData { raw_values_start: 2018, raw_values_count: 1, token_types_start: 2019, token_types_count: 0 }, // [855]
    SimpleHintData { raw_values_start: 2019, raw_values_count: 1, token_types_start: 2020, token_types_count: 0 }, // [856]
    SimpleHintData { raw_values_start: 2020, raw_values_count: 1, token_types_start: 2021, token_types_count: 0 }, // [857]
    SimpleHintData { raw_values_start: 2021, raw_values_count: 1, token_types_start: 2022, token_types_count: 0 }, // [858]
    SimpleHintData { raw_values_start: 2022, raw_values_count: 1, token_types_start: 2023, token_types_count: 0 }, // [859]
    SimpleHintData { raw_values_start: 2023, raw_values_count: 1, token_types_start: 2024, token_types_count: 0 }, // [860]
    SimpleHintData { raw_values_start: 2024, raw_values_count: 1, token_types_start: 2025, token_types_count: 0 }, // [861]
    SimpleHintData { raw_values_start: 2025, raw_values_count: 1, token_types_start: 2026, token_types_count: 0 }, // [862]
    SimpleHintData { raw_values_start: 2026, raw_values_count: 1, token_types_start: 2027, token_types_count: 0 }, // [863]
    SimpleHintData { raw_values_start: 2027, raw_values_count: 1, token_types_start: 2028, token_types_count: 0 }, // [864]
    SimpleHintData { raw_values_start: 2028, raw_values_count: 1, token_types_start: 2029, token_types_count: 0 }, // [865]
    SimpleHintData { raw_values_start: 2029, raw_values_count: 1, token_types_start: 2030, token_types_count: 0 }, // [866]
    SimpleHintData { raw_values_start: 2030, raw_values_count: 1, token_types_start: 2031, token_types_count: 0 }, // [867]
    SimpleHintData { raw_values_start: 2031, raw_values_count: 1, token_types_start: 2032, token_types_count: 0 }, // [868]
    SimpleHintData { raw_values_start: 2032, raw_values_count: 1, token_types_start: 2033, token_types_count: 0 }, // [869]
    SimpleHintData { raw_values_start: 2033, raw_values_count: 1, token_types_start: 2034, token_types_count: 0 }, // [870]
    SimpleHintData { raw_values_start: 2034, raw_values_count: 16, token_types_start: 2050, token_types_count: 9 }, // [871]
    SimpleHintData { raw_values_start: 2059, raw_values_count: 0, token_types_start: 2059, token_types_count: 1 }, // [872]
    SimpleHintData { raw_values_start: 2060, raw_values_count: 1, token_types_start: 2061, token_types_count: 0 }, // [873]
    SimpleHintData { raw_values_start: 2061, raw_values_count: 1, token_types_start: 2062, token_types_count: 0 }, // [874]
    SimpleHintData { raw_values_start: 2062, raw_values_count: 1, token_types_start: 2063, token_types_count: 0 }, // [875]
    SimpleHintData { raw_values_start: 2063, raw_values_count: 1, token_types_start: 2064, token_types_count: 0 }, // [876]
    SimpleHintData { raw_values_start: 2064, raw_values_count: 1, token_types_start: 2065, token_types_count: 0 }, // [877]
    SimpleHintData { raw_values_start: 2065, raw_values_count: 1, token_types_start: 2066, token_types_count: 0 }, // [878]
    SimpleHintData { raw_values_start: 2066, raw_values_count: 1, token_types_start: 2067, token_types_count: 0 }, // [879]
    SimpleHintData { raw_values_start: 2067, raw_values_count: 1, token_types_start: 2068, token_types_count: 0 }, // [880]
    SimpleHintData { raw_values_start: 2068, raw_values_count: 1, token_types_start: 2069, token_types_count: 0 }, // [881]
    SimpleHintData { raw_values_start: 2069, raw_values_count: 1, token_types_start: 2070, token_types_count: 0 }, // [882]
    SimpleHintData { raw_values_start: 2070, raw_values_count: 1, token_types_start: 2071, token_types_count: 0 }, // [883]
    SimpleHintData { raw_values_start: 2071, raw_values_count: 1, token_types_start: 2072, token_types_count: 0 }, // [884]
    SimpleHintData { raw_values_start: 2072, raw_values_count: 1, token_types_start: 2073, token_types_count: 0 }, // [885]
    SimpleHintData { raw_values_start: 2073, raw_values_count: 1, token_types_start: 2074, token_types_count: 0 }, // [886]
    SimpleHintData { raw_values_start: 2074, raw_values_count: 5, token_types_start: 2079, token_types_count: 0 }, // [887]
    SimpleHintData { raw_values_start: 2079, raw_values_count: 1, token_types_start: 2080, token_types_count: 0 }, // [888]
    SimpleHintData { raw_values_start: 2080, raw_values_count: 1, token_types_start: 2081, token_types_count: 0 }, // [889]
    SimpleHintData { raw_values_start: 2081, raw_values_count: 1, token_types_start: 2082, token_types_count: 0 }, // [890]
    SimpleHintData { raw_values_start: 2082, raw_values_count: 1, token_types_start: 2083, token_types_count: 0 }, // [891]
    SimpleHintData { raw_values_start: 2083, raw_values_count: 1, token_types_start: 2084, token_types_count: 0 }, // [892]
    SimpleHintData { raw_values_start: 2084, raw_values_count: 1, token_types_start: 2085, token_types_count: 0 }, // [893]
    SimpleHintData { raw_values_start: 2085, raw_values_count: 1, token_types_start: 2086, token_types_count: 0 }, // [894]
    SimpleHintData { raw_values_start: 2086, raw_values_count: 1, token_types_start: 2087, token_types_count: 0 }, // [895]
    SimpleHintData { raw_values_start: 2087, raw_values_count: 1, token_types_start: 2088, token_types_count: 0 }, // [896]
    SimpleHintData { raw_values_start: 2088, raw_values_count: 1, token_types_start: 2089, token_types_count: 0 }, // [897]
    SimpleHintData { raw_values_start: 2089, raw_values_count: 1, token_types_start: 2090, token_types_count: 0 }, // [898]
    SimpleHintData { raw_values_start: 2090, raw_values_count: 1, token_types_start: 2091, token_types_count: 0 }, // [899]
    SimpleHintData { raw_values_start: 2091, raw_values_count: 1, token_types_start: 2092, token_types_count: 0 }, // [900]
    SimpleHintData { raw_values_start: 2092, raw_values_count: 1, token_types_start: 2093, token_types_count: 0 }, // [901]
    SimpleHintData { raw_values_start: 2093, raw_values_count: 1, token_types_start: 2094, token_types_count: 6 }, // [902]
    SimpleHintData { raw_values_start: 2100, raw_values_count: 1, token_types_start: 2101, token_types_count: 0 }, // [903]
    SimpleHintData { raw_values_start: 2101, raw_values_count: 0, token_types_start: 2101, token_types_count: 1 }, // [904]
    SimpleHintData { raw_values_start: 2102, raw_values_count: 2, token_types_start: 2104, token_types_count: 0 }, // [905]
    SimpleHintData { raw_values_start: 2104, raw_values_count: 1, token_types_start: 2105, token_types_count: 0 }, // [906]
    SimpleHintData { raw_values_start: 2105, raw_values_count: 1, token_types_start: 2106, token_types_count: 0 }, // [907]
    SimpleHintData { raw_values_start: 2106, raw_values_count: 1, token_types_start: 2107, token_types_count: 0 }, // [908]
    SimpleHintData { raw_values_start: 2107, raw_values_count: 1, token_types_start: 2108, token_types_count: 0 }, // [909]
    SimpleHintData { raw_values_start: 2108, raw_values_count: 1, token_types_start: 2109, token_types_count: 0 }, // [910]
    SimpleHintData { raw_values_start: 2109, raw_values_count: 1, token_types_start: 2110, token_types_count: 0 }, // [911]
    SimpleHintData { raw_values_start: 2110, raw_values_count: 1, token_types_start: 2111, token_types_count: 0 }, // [912]
    SimpleHintData { raw_values_start: 2111, raw_values_count: 1, token_types_start: 2112, token_types_count: 0 }, // [913]
    SimpleHintData { raw_values_start: 2112, raw_values_count: 1, token_types_start: 2113, token_types_count: 0 }, // [914]
    SimpleHintData { raw_values_start: 2113, raw_values_count: 1, token_types_start: 2114, token_types_count: 0 }, // [915]
    SimpleHintData { raw_values_start: 2114, raw_values_count: 1, token_types_start: 2115, token_types_count: 0 }, // [916]
    SimpleHintData { raw_values_start: 2115, raw_values_count: 1, token_types_start: 2116, token_types_count: 0 }, // [917]
    SimpleHintData { raw_values_start: 2116, raw_values_count: 1, token_types_start: 2117, token_types_count: 0 }, // [918]
    SimpleHintData { raw_values_start: 2117, raw_values_count: 1, token_types_start: 2118, token_types_count: 0 }, // [919]
    SimpleHintData { raw_values_start: 2118, raw_values_count: 1, token_types_start: 2119, token_types_count: 0 }, // [920]
    SimpleHintData { raw_values_start: 2119, raw_values_count: 1, token_types_start: 2120, token_types_count: 0 }, // [921]
    SimpleHintData { raw_values_start: 2120, raw_values_count: 1, token_types_start: 2121, token_types_count: 0 }, // [922]
    SimpleHintData { raw_values_start: 2121, raw_values_count: 1, token_types_start: 2122, token_types_count: 0 }, // [923]
    SimpleHintData { raw_values_start: 2122, raw_values_count: 1, token_types_start: 2123, token_types_count: 0 }, // [924]
    SimpleHintData { raw_values_start: 2123, raw_values_count: 1, token_types_start: 2124, token_types_count: 0 }, // [925]
    SimpleHintData { raw_values_start: 2124, raw_values_count: 1, token_types_start: 2125, token_types_count: 0 }, // [926]
    SimpleHintData { raw_values_start: 2125, raw_values_count: 1, token_types_start: 2126, token_types_count: 0 }, // [927]
    SimpleHintData { raw_values_start: 2126, raw_values_count: 1, token_types_start: 2127, token_types_count: 0 }, // [928]
    SimpleHintData { raw_values_start: 2127, raw_values_count: 1, token_types_start: 2128, token_types_count: 0 }, // [929]
    SimpleHintData { raw_values_start: 2128, raw_values_count: 4, token_types_start: 2132, token_types_count: 0 }, // [930]
    SimpleHintData { raw_values_start: 2132, raw_values_count: 1, token_types_start: 2133, token_types_count: 0 }, // [931]
    SimpleHintData { raw_values_start: 2133, raw_values_count: 1, token_types_start: 2134, token_types_count: 0 }, // [932]
    SimpleHintData { raw_values_start: 2134, raw_values_count: 4, token_types_start: 2138, token_types_count: 0 }, // [933]
    SimpleHintData { raw_values_start: 2138, raw_values_count: 3, token_types_start: 2141, token_types_count: 0 }, // [934]
    SimpleHintData { raw_values_start: 2141, raw_values_count: 1, token_types_start: 2142, token_types_count: 0 }, // [935]
    SimpleHintData { raw_values_start: 2142, raw_values_count: 4, token_types_start: 2146, token_types_count: 0 }, // [936]
    SimpleHintData { raw_values_start: 2146, raw_values_count: 2, token_types_start: 2148, token_types_count: 0 }, // [937]
    SimpleHintData { raw_values_start: 2148, raw_values_count: 11, token_types_start: 2159, token_types_count: 0 }, // [938]
    SimpleHintData { raw_values_start: 2159, raw_values_count: 1, token_types_start: 2160, token_types_count: 0 }, // [939]
    SimpleHintData { raw_values_start: 2160, raw_values_count: 1, token_types_start: 2161, token_types_count: 0 }, // [940]
    SimpleHintData { raw_values_start: 2161, raw_values_count: 2, token_types_start: 2163, token_types_count: 0 }, // [941]
    SimpleHintData { raw_values_start: 2163, raw_values_count: 5, token_types_start: 2168, token_types_count: 0 }, // [942]
    SimpleHintData { raw_values_start: 2168, raw_values_count: 1, token_types_start: 2169, token_types_count: 0 }, // [943]
    SimpleHintData { raw_values_start: 2169, raw_values_count: 1, token_types_start: 2170, token_types_count: 0 }, // [944]
    SimpleHintData { raw_values_start: 2170, raw_values_count: 15, token_types_start: 2185, token_types_count: 4 }, // [945]
    SimpleHintData { raw_values_start: 2189, raw_values_count: 3, token_types_start: 2192, token_types_count: 0 }, // [946]
    SimpleHintData { raw_values_start: 2192, raw_values_count: 1, token_types_start: 2193, token_types_count: 0 }, // [947]
    SimpleHintData { raw_values_start: 2193, raw_values_count: 0, token_types_start: 2193, token_types_count: 1 }, // [948]
    SimpleHintData { raw_values_start: 2194, raw_values_count: 9, token_types_start: 2203, token_types_count: 7 }, // [949]
    SimpleHintData { raw_values_start: 2210, raw_values_count: 1, token_types_start: 2211, token_types_count: 0 }, // [950]
    SimpleHintData { raw_values_start: 2211, raw_values_count: 1, token_types_start: 2212, token_types_count: 0 }, // [951]
    SimpleHintData { raw_values_start: 2212, raw_values_count: 2, token_types_start: 2214, token_types_count: 0 }, // [952]
    SimpleHintData { raw_values_start: 2214, raw_values_count: 1, token_types_start: 2215, token_types_count: 0 }, // [953]
    SimpleHintData { raw_values_start: 2215, raw_values_count: 2, token_types_start: 2217, token_types_count: 0 }, // [954]
    SimpleHintData { raw_values_start: 2217, raw_values_count: 2, token_types_start: 2219, token_types_count: 0 }, // [955]
    SimpleHintData { raw_values_start: 2219, raw_values_count: 1, token_types_start: 2220, token_types_count: 0 }, // [956]
    SimpleHintData { raw_values_start: 2220, raw_values_count: 3, token_types_start: 2223, token_types_count: 0 }, // [957]
    SimpleHintData { raw_values_start: 2223, raw_values_count: 1, token_types_start: 2224, token_types_count: 0 }, // [958]
    SimpleHintData { raw_values_start: 2224, raw_values_count: 1, token_types_start: 2225, token_types_count: 1 }, // [959]
    SimpleHintData { raw_values_start: 2226, raw_values_count: 1, token_types_start: 2227, token_types_count: 0 }, // [960]
    SimpleHintData { raw_values_start: 2227, raw_values_count: 64, token_types_start: 2291, token_types_count: 3 }, // [961]
    SimpleHintData { raw_values_start: 2294, raw_values_count: 0, token_types_start: 2294, token_types_count: 1 }, // [962]
    SimpleHintData { raw_values_start: 2295, raw_values_count: 0, token_types_start: 2295, token_types_count: 1 }, // [963]
    SimpleHintData { raw_values_start: 2296, raw_values_count: 0, token_types_start: 2296, token_types_count: 1 }, // [964]
    SimpleHintData { raw_values_start: 2297, raw_values_count: 64, token_types_start: 2361, token_types_count: 0 }, // [965]
    SimpleHintData { raw_values_start: 2361, raw_values_count: 1, token_types_start: 2362, token_types_count: 0 }, // [966]
    SimpleHintData { raw_values_start: 2362, raw_values_count: 1, token_types_start: 2363, token_types_count: 0 }, // [967]
    SimpleHintData { raw_values_start: 2363, raw_values_count: 1, token_types_start: 2364, token_types_count: 0 }, // [968]
    SimpleHintData { raw_values_start: 2364, raw_values_count: 1, token_types_start: 2365, token_types_count: 0 }, // [969]
    SimpleHintData { raw_values_start: 2365, raw_values_count: 1, token_types_start: 2366, token_types_count: 0 }, // [970]
    SimpleHintData { raw_values_start: 2366, raw_values_count: 1, token_types_start: 2367, token_types_count: 0 }, // [971]
    SimpleHintData { raw_values_start: 2367, raw_values_count: 1, token_types_start: 2368, token_types_count: 0 }, // [972]
    SimpleHintData { raw_values_start: 2368, raw_values_count: 4, token_types_start: 2372, token_types_count: 0 }, // [973]
    SimpleHintData { raw_values_start: 2372, raw_values_count: 2, token_types_start: 2374, token_types_count: 0 }, // [974]
    SimpleHintData { raw_values_start: 2374, raw_values_count: 1, token_types_start: 2375, token_types_count: 0 }, // [975]
    SimpleHintData { raw_values_start: 2375, raw_values_count: 5, token_types_start: 2380, token_types_count: 0 }, // [976]
    SimpleHintData { raw_values_start: 2380, raw_values_count: 1, token_types_start: 2381, token_types_count: 0 }, // [977]
    SimpleHintData { raw_values_start: 2381, raw_values_count: 1, token_types_start: 2382, token_types_count: 0 }, // [978]
    SimpleHintData { raw_values_start: 2382, raw_values_count: 3, token_types_start: 2385, token_types_count: 0 }, // [979]
    SimpleHintData { raw_values_start: 2385, raw_values_count: 1, token_types_start: 2386, token_types_count: 0 }, // [980]
    SimpleHintData { raw_values_start: 2386, raw_values_count: 2, token_types_start: 2388, token_types_count: 0 }, // [981]
    SimpleHintData { raw_values_start: 2388, raw_values_count: 1, token_types_start: 2389, token_types_count: 0 }, // [982]
    SimpleHintData { raw_values_start: 2389, raw_values_count: 4, token_types_start: 2393, token_types_count: 0 }, // [983]
    SimpleHintData { raw_values_start: 2393, raw_values_count: 1, token_types_start: 2394, token_types_count: 0 }, // [984]
    SimpleHintData { raw_values_start: 2394, raw_values_count: 1, token_types_start: 2395, token_types_count: 0 }, // [985]
    SimpleHintData { raw_values_start: 2395, raw_values_count: 1, token_types_start: 2396, token_types_count: 0 }, // [986]
    SimpleHintData { raw_values_start: 2396, raw_values_count: 1, token_types_start: 2397, token_types_count: 0 }, // [987]
    SimpleHintData { raw_values_start: 2397, raw_values_count: 1, token_types_start: 2398, token_types_count: 0 }, // [988]
    SimpleHintData { raw_values_start: 2398, raw_values_count: 15, token_types_start: 2413, token_types_count: 0 }, // [989]
    SimpleHintData { raw_values_start: 2413, raw_values_count: 1, token_types_start: 2414, token_types_count: 0 }, // [990]
    SimpleHintData { raw_values_start: 2414, raw_values_count: 1, token_types_start: 2415, token_types_count: 0 }, // [991]
    SimpleHintData { raw_values_start: 2415, raw_values_count: 1, token_types_start: 2416, token_types_count: 0 }, // [992]
    SimpleHintData { raw_values_start: 2416, raw_values_count: 4, token_types_start: 2420, token_types_count: 0 }, // [993]
    SimpleHintData { raw_values_start: 2420, raw_values_count: 1, token_types_start: 2421, token_types_count: 0 }, // [994]
    SimpleHintData { raw_values_start: 2421, raw_values_count: 1, token_types_start: 2422, token_types_count: 0 }, // [995]
    SimpleHintData { raw_values_start: 2422, raw_values_count: 1, token_types_start: 2423, token_types_count: 0 }, // [996]
    SimpleHintData { raw_values_start: 2423, raw_values_count: 1, token_types_start: 2424, token_types_count: 0 }, // [997]
    SimpleHintData { raw_values_start: 2424, raw_values_count: 1, token_types_start: 2425, token_types_count: 0 }, // [998]
    SimpleHintData { raw_values_start: 2425, raw_values_count: 6, token_types_start: 2431, token_types_count: 0 }, // [999]
    SimpleHintData { raw_values_start: 2431, raw_values_count: 1, token_types_start: 2432, token_types_count: 0 }, // [1000]
    SimpleHintData { raw_values_start: 2432, raw_values_count: 1, token_types_start: 2433, token_types_count: 0 }, // [1001]
    SimpleHintData { raw_values_start: 2433, raw_values_count: 1, token_types_start: 2434, token_types_count: 0 }, // [1002]
    SimpleHintData { raw_values_start: 2434, raw_values_count: 1, token_types_start: 2435, token_types_count: 0 }, // [1003]
    SimpleHintData { raw_values_start: 2435, raw_values_count: 1, token_types_start: 2436, token_types_count: 0 }, // [1004]
    SimpleHintData { raw_values_start: 2436, raw_values_count: 2, token_types_start: 2438, token_types_count: 0 }, // [1005]
    SimpleHintData { raw_values_start: 2438, raw_values_count: 1, token_types_start: 2439, token_types_count: 0 }, // [1006]
    SimpleHintData { raw_values_start: 2439, raw_values_count: 1, token_types_start: 2440, token_types_count: 0 }, // [1007]
    SimpleHintData { raw_values_start: 2440, raw_values_count: 1, token_types_start: 2441, token_types_count: 0 }, // [1008]
    SimpleHintData { raw_values_start: 2441, raw_values_count: 3, token_types_start: 2444, token_types_count: 0 }, // [1009]
    SimpleHintData { raw_values_start: 2444, raw_values_count: 2, token_types_start: 2446, token_types_count: 0 }, // [1010]
    SimpleHintData { raw_values_start: 2446, raw_values_count: 1, token_types_start: 2447, token_types_count: 0 }, // [1011]
    SimpleHintData { raw_values_start: 2447, raw_values_count: 2, token_types_start: 2449, token_types_count: 0 }, // [1012]
    SimpleHintData { raw_values_start: 2449, raw_values_count: 2, token_types_start: 2451, token_types_count: 0 }, // [1013]
    SimpleHintData { raw_values_start: 2451, raw_values_count: 21, token_types_start: 2472, token_types_count: 0 }, // [1014]
    SimpleHintData { raw_values_start: 2472, raw_values_count: 1, token_types_start: 2473, token_types_count: 0 }, // [1015]
    SimpleHintData { raw_values_start: 2473, raw_values_count: 7, token_types_start: 2480, token_types_count: 0 }, // [1016]
    SimpleHintData { raw_values_start: 2480, raw_values_count: 4, token_types_start: 2484, token_types_count: 0 }, // [1017]
    SimpleHintData { raw_values_start: 2484, raw_values_count: 4, token_types_start: 2488, token_types_count: 0 }, // [1018]
    SimpleHintData { raw_values_start: 2488, raw_values_count: 3, token_types_start: 2491, token_types_count: 0 }, // [1019]
    SimpleHintData { raw_values_start: 2491, raw_values_count: 6, token_types_start: 2497, token_types_count: 0 }, // [1020]
    SimpleHintData { raw_values_start: 2497, raw_values_count: 1, token_types_start: 2498, token_types_count: 0 }, // [1021]
    SimpleHintData { raw_values_start: 2498, raw_values_count: 1, token_types_start: 2499, token_types_count: 0 }, // [1022]
    SimpleHintData { raw_values_start: 2499, raw_values_count: 1, token_types_start: 2500, token_types_count: 0 }, // [1023]
    SimpleHintData { raw_values_start: 2500, raw_values_count: 1, token_types_start: 2501, token_types_count: 0 }, // [1024]
    SimpleHintData { raw_values_start: 2501, raw_values_count: 1, token_types_start: 2502, token_types_count: 0 }, // [1025]
    SimpleHintData { raw_values_start: 2502, raw_values_count: 0, token_types_start: 2502, token_types_count: 1 }, // [1026]
    SimpleHintData { raw_values_start: 2503, raw_values_count: 1, token_types_start: 2504, token_types_count: 0 }, // [1027]
    SimpleHintData { raw_values_start: 2504, raw_values_count: 1, token_types_start: 2505, token_types_count: 0 }, // [1028]
    SimpleHintData { raw_values_start: 2505, raw_values_count: 1, token_types_start: 2506, token_types_count: 0 }, // [1029]
    SimpleHintData { raw_values_start: 2506, raw_values_count: 1, token_types_start: 2507, token_types_count: 0 }, // [1030]
    SimpleHintData { raw_values_start: 2507, raw_values_count: 1, token_types_start: 2508, token_types_count: 0 }, // [1031]
    SimpleHintData { raw_values_start: 2508, raw_values_count: 2, token_types_start: 2510, token_types_count: 0 }, // [1032]
    SimpleHintData { raw_values_start: 2510, raw_values_count: 1, token_types_start: 2511, token_types_count: 0 }, // [1033]
    SimpleHintData { raw_values_start: 2511, raw_values_count: 2, token_types_start: 2513, token_types_count: 0 }, // [1034]
    SimpleHintData { raw_values_start: 2513, raw_values_count: 2, token_types_start: 2515, token_types_count: 0 }, // [1035]
    SimpleHintData { raw_values_start: 2515, raw_values_count: 1, token_types_start: 2516, token_types_count: 0 }, // [1036]
    SimpleHintData { raw_values_start: 2516, raw_values_count: 1, token_types_start: 2517, token_types_count: 0 }, // [1037]
    SimpleHintData { raw_values_start: 2517, raw_values_count: 1, token_types_start: 2518, token_types_count: 0 }, // [1038]
    SimpleHintData { raw_values_start: 2518, raw_values_count: 6, token_types_start: 2524, token_types_count: 0 }, // [1039]
    SimpleHintData { raw_values_start: 2524, raw_values_count: 1, token_types_start: 2525, token_types_count: 0 }, // [1040]
    SimpleHintData { raw_values_start: 2525, raw_values_count: 1, token_types_start: 2526, token_types_count: 0 }, // [1041]
    SimpleHintData { raw_values_start: 2526, raw_values_count: 5, token_types_start: 2531, token_types_count: 0 }, // [1042]
    SimpleHintData { raw_values_start: 2531, raw_values_count: 1, token_types_start: 2532, token_types_count: 0 }, // [1043]
    SimpleHintData { raw_values_start: 2532, raw_values_count: 1, token_types_start: 2533, token_types_count: 0 }, // [1044]
    SimpleHintData { raw_values_start: 2533, raw_values_count: 1, token_types_start: 2534, token_types_count: 0 }, // [1045]
    SimpleHintData { raw_values_start: 2534, raw_values_count: 1, token_types_start: 2535, token_types_count: 0 }, // [1046]
    SimpleHintData { raw_values_start: 2535, raw_values_count: 1, token_types_start: 2536, token_types_count: 0 }, // [1047]
    SimpleHintData { raw_values_start: 2536, raw_values_count: 1, token_types_start: 2537, token_types_count: 0 }, // [1048]
    SimpleHintData { raw_values_start: 2537, raw_values_count: 1, token_types_start: 2538, token_types_count: 0 }, // [1049]
    SimpleHintData { raw_values_start: 2538, raw_values_count: 1, token_types_start: 2539, token_types_count: 0 }, // [1050]
    SimpleHintData { raw_values_start: 2539, raw_values_count: 1, token_types_start: 2540, token_types_count: 0 }, // [1051]
    SimpleHintData { raw_values_start: 2540, raw_values_count: 1, token_types_start: 2541, token_types_count: 0 }, // [1052]
    SimpleHintData { raw_values_start: 2541, raw_values_count: 1, token_types_start: 2542, token_types_count: 0 }, // [1053]
    SimpleHintData { raw_values_start: 2542, raw_values_count: 1, token_types_start: 2543, token_types_count: 0 }, // [1054]
    SimpleHintData { raw_values_start: 2543, raw_values_count: 2, token_types_start: 2545, token_types_count: 0 }, // [1055]
    SimpleHintData { raw_values_start: 2545, raw_values_count: 1, token_types_start: 2546, token_types_count: 0 }, // [1056]
    SimpleHintData { raw_values_start: 2546, raw_values_count: 1, token_types_start: 2547, token_types_count: 0 }, // [1057]
    SimpleHintData { raw_values_start: 2547, raw_values_count: 1, token_types_start: 2548, token_types_count: 0 }, // [1058]
    SimpleHintData { raw_values_start: 2548, raw_values_count: 2, token_types_start: 2550, token_types_count: 0 }, // [1059]
    SimpleHintData { raw_values_start: 2550, raw_values_count: 2, token_types_start: 2552, token_types_count: 0 }, // [1060]
    SimpleHintData { raw_values_start: 2552, raw_values_count: 1, token_types_start: 2553, token_types_count: 0 }, // [1061]
    SimpleHintData { raw_values_start: 2553, raw_values_count: 3, token_types_start: 2556, token_types_count: 0 }, // [1062]
    SimpleHintData { raw_values_start: 2556, raw_values_count: 1, token_types_start: 2557, token_types_count: 0 }, // [1063]
    SimpleHintData { raw_values_start: 2557, raw_values_count: 1, token_types_start: 2558, token_types_count: 0 }, // [1064]
    SimpleHintData { raw_values_start: 2558, raw_values_count: 1, token_types_start: 2559, token_types_count: 0 }, // [1065]
    SimpleHintData { raw_values_start: 2559, raw_values_count: 1, token_types_start: 2560, token_types_count: 0 }, // [1066]
    SimpleHintData { raw_values_start: 2560, raw_values_count: 1, token_types_start: 2561, token_types_count: 0 }, // [1067]
    SimpleHintData { raw_values_start: 2561, raw_values_count: 1, token_types_start: 2562, token_types_count: 0 }, // [1068]
    SimpleHintData { raw_values_start: 2562, raw_values_count: 8, token_types_start: 2570, token_types_count: 0 }, // [1069]
    SimpleHintData { raw_values_start: 2570, raw_values_count: 1, token_types_start: 2571, token_types_count: 0 }, // [1070]
    SimpleHintData { raw_values_start: 2571, raw_values_count: 1, token_types_start: 2572, token_types_count: 0 }, // [1071]
    SimpleHintData { raw_values_start: 2572, raw_values_count: 1, token_types_start: 2573, token_types_count: 0 }, // [1072]
    SimpleHintData { raw_values_start: 2573, raw_values_count: 1, token_types_start: 2574, token_types_count: 0 }, // [1073]
    SimpleHintData { raw_values_start: 2574, raw_values_count: 1, token_types_start: 2575, token_types_count: 0 }, // [1074]
    SimpleHintData { raw_values_start: 2575, raw_values_count: 1, token_types_start: 2576, token_types_count: 0 }, // [1075]
    SimpleHintData { raw_values_start: 2576, raw_values_count: 0, token_types_start: 2576, token_types_count: 7 }, // [1076]
    SimpleHintData { raw_values_start: 2583, raw_values_count: 80, token_types_start: 2663, token_types_count: 6 }, // [1077]
    SimpleHintData { raw_values_start: 2669, raw_values_count: 6, token_types_start: 2675, token_types_count: 0 }, // [1078]
    SimpleHintData { raw_values_start: 2675, raw_values_count: 1, token_types_start: 2676, token_types_count: 0 }, // [1079]
    SimpleHintData { raw_values_start: 2676, raw_values_count: 1, token_types_start: 2677, token_types_count: 0 }, // [1080]
    SimpleHintData { raw_values_start: 2677, raw_values_count: 1, token_types_start: 2678, token_types_count: 0 }, // [1081]
    SimpleHintData { raw_values_start: 2678, raw_values_count: 1, token_types_start: 2679, token_types_count: 0 }, // [1082]
    SimpleHintData { raw_values_start: 2679, raw_values_count: 3, token_types_start: 2682, token_types_count: 0 }, // [1083]
    SimpleHintData { raw_values_start: 2682, raw_values_count: 1, token_types_start: 2683, token_types_count: 0 }, // [1084]
    SimpleHintData { raw_values_start: 2683, raw_values_count: 1, token_types_start: 2684, token_types_count: 0 }, // [1085]
    SimpleHintData { raw_values_start: 2684, raw_values_count: 5, token_types_start: 2689, token_types_count: 0 }, // [1086]
    SimpleHintData { raw_values_start: 2689, raw_values_count: 2, token_types_start: 2691, token_types_count: 0 }, // [1087]
    SimpleHintData { raw_values_start: 2691, raw_values_count: 1, token_types_start: 2692, token_types_count: 0 }, // [1088]
    SimpleHintData { raw_values_start: 2692, raw_values_count: 1, token_types_start: 2693, token_types_count: 0 }, // [1089]
    SimpleHintData { raw_values_start: 2693, raw_values_count: 2, token_types_start: 2695, token_types_count: 0 }, // [1090]
    SimpleHintData { raw_values_start: 2695, raw_values_count: 3, token_types_start: 2698, token_types_count: 0 }, // [1091]
    SimpleHintData { raw_values_start: 2698, raw_values_count: 1, token_types_start: 2699, token_types_count: 0 }, // [1092]
    SimpleHintData { raw_values_start: 2699, raw_values_count: 1, token_types_start: 2700, token_types_count: 0 }, // [1093]
    SimpleHintData { raw_values_start: 2700, raw_values_count: 1, token_types_start: 2701, token_types_count: 0 }, // [1094]
    SimpleHintData { raw_values_start: 2701, raw_values_count: 4, token_types_start: 2705, token_types_count: 0 }, // [1095]
    SimpleHintData { raw_values_start: 2705, raw_values_count: 1, token_types_start: 2706, token_types_count: 0 }, // [1096]
    SimpleHintData { raw_values_start: 2706, raw_values_count: 1, token_types_start: 2707, token_types_count: 0 }, // [1097]
    SimpleHintData { raw_values_start: 2707, raw_values_count: 1, token_types_start: 2708, token_types_count: 0 }, // [1098]
    SimpleHintData { raw_values_start: 2708, raw_values_count: 1, token_types_start: 2709, token_types_count: 0 }, // [1099]
    SimpleHintData { raw_values_start: 2709, raw_values_count: 1, token_types_start: 2710, token_types_count: 0 }, // [1100]
    SimpleHintData { raw_values_start: 2710, raw_values_count: 1, token_types_start: 2711, token_types_count: 0 }, // [1101]
    SimpleHintData { raw_values_start: 2711, raw_values_count: 1, token_types_start: 2712, token_types_count: 0 }, // [1102]
    SimpleHintData { raw_values_start: 2712, raw_values_count: 1, token_types_start: 2713, token_types_count: 0 }, // [1103]
    SimpleHintData { raw_values_start: 2713, raw_values_count: 1, token_types_start: 2714, token_types_count: 0 }, // [1104]
    SimpleHintData { raw_values_start: 2714, raw_values_count: 1, token_types_start: 2715, token_types_count: 0 }, // [1105]
    SimpleHintData { raw_values_start: 2715, raw_values_count: 1, token_types_start: 2716, token_types_count: 0 }, // [1106]
    SimpleHintData { raw_values_start: 2716, raw_values_count: 1, token_types_start: 2717, token_types_count: 0 }, // [1107]
    SimpleHintData { raw_values_start: 2717, raw_values_count: 1, token_types_start: 2718, token_types_count: 0 }, // [1108]
    SimpleHintData { raw_values_start: 2718, raw_values_count: 1, token_types_start: 2719, token_types_count: 0 }, // [1109]
    SimpleHintData { raw_values_start: 2719, raw_values_count: 1, token_types_start: 2720, token_types_count: 0 }, // [1110]
    SimpleHintData { raw_values_start: 2720, raw_values_count: 1, token_types_start: 2721, token_types_count: 0 }, // [1111]
    SimpleHintData { raw_values_start: 2721, raw_values_count: 1, token_types_start: 2722, token_types_count: 0 }, // [1112]
    SimpleHintData { raw_values_start: 2722, raw_values_count: 0, token_types_start: 2722, token_types_count: 1 }, // [1113]
    SimpleHintData { raw_values_start: 2723, raw_values_count: 1, token_types_start: 2724, token_types_count: 0 }, // [1114]
    SimpleHintData { raw_values_start: 2724, raw_values_count: 1, token_types_start: 2725, token_types_count: 0 }, // [1115]
    SimpleHintData { raw_values_start: 2725, raw_values_count: 1, token_types_start: 2726, token_types_count: 0 }, // [1116]
    SimpleHintData { raw_values_start: 2726, raw_values_count: 1, token_types_start: 2727, token_types_count: 0 }, // [1117]
    SimpleHintData { raw_values_start: 2727, raw_values_count: 1, token_types_start: 2728, token_types_count: 0 }, // [1118]
    SimpleHintData { raw_values_start: 2728, raw_values_count: 3, token_types_start: 2731, token_types_count: 0 }, // [1119]
    SimpleHintData { raw_values_start: 2731, raw_values_count: 1, token_types_start: 2732, token_types_count: 0 }, // [1120]
    SimpleHintData { raw_values_start: 2732, raw_values_count: 1, token_types_start: 2733, token_types_count: 0 }, // [1121]
    SimpleHintData { raw_values_start: 2733, raw_values_count: 2, token_types_start: 2735, token_types_count: 0 }, // [1122]
    SimpleHintData { raw_values_start: 2735, raw_values_count: 1, token_types_start: 2736, token_types_count: 0 }, // [1123]
    SimpleHintData { raw_values_start: 2736, raw_values_count: 1, token_types_start: 2737, token_types_count: 0 }, // [1124]
    SimpleHintData { raw_values_start: 2737, raw_values_count: 1, token_types_start: 2738, token_types_count: 0 }, // [1125]
    SimpleHintData { raw_values_start: 2738, raw_values_count: 1, token_types_start: 2739, token_types_count: 0 }, // [1126]
    SimpleHintData { raw_values_start: 2739, raw_values_count: 1, token_types_start: 2740, token_types_count: 0 }, // [1127]
    SimpleHintData { raw_values_start: 2740, raw_values_count: 2, token_types_start: 2742, token_types_count: 0 }, // [1128]
    SimpleHintData { raw_values_start: 2742, raw_values_count: 1, token_types_start: 2743, token_types_count: 0 }, // [1129]
    SimpleHintData { raw_values_start: 2743, raw_values_count: 1, token_types_start: 2744, token_types_count: 0 }, // [1130]
    SimpleHintData { raw_values_start: 2744, raw_values_count: 1, token_types_start: 2745, token_types_count: 0 }, // [1131]
    SimpleHintData { raw_values_start: 2745, raw_values_count: 1, token_types_start: 2746, token_types_count: 0 }, // [1132]
    SimpleHintData { raw_values_start: 2746, raw_values_count: 1, token_types_start: 2747, token_types_count: 0 }, // [1133]
    SimpleHintData { raw_values_start: 2747, raw_values_count: 1, token_types_start: 2748, token_types_count: 0 }, // [1134]
    SimpleHintData { raw_values_start: 2748, raw_values_count: 1, token_types_start: 2749, token_types_count: 0 }, // [1135]
    SimpleHintData { raw_values_start: 2749, raw_values_count: 1, token_types_start: 2750, token_types_count: 0 }, // [1136]
    SimpleHintData { raw_values_start: 2750, raw_values_count: 1, token_types_start: 2751, token_types_count: 0 }, // [1137]
    SimpleHintData { raw_values_start: 2751, raw_values_count: 1, token_types_start: 2752, token_types_count: 0 }, // [1138]
    SimpleHintData { raw_values_start: 2752, raw_values_count: 1, token_types_start: 2753, token_types_count: 0 }, // [1139]
    SimpleHintData { raw_values_start: 2753, raw_values_count: 1, token_types_start: 2754, token_types_count: 0 }, // [1140]
    SimpleHintData { raw_values_start: 2754, raw_values_count: 1, token_types_start: 2755, token_types_count: 0 }, // [1141]
    SimpleHintData { raw_values_start: 2755, raw_values_count: 2, token_types_start: 2757, token_types_count: 0 }, // [1142]
    SimpleHintData { raw_values_start: 2757, raw_values_count: 1, token_types_start: 2758, token_types_count: 0 }, // [1143]
    SimpleHintData { raw_values_start: 2758, raw_values_count: 1, token_types_start: 2759, token_types_count: 0 }, // [1144]
    SimpleHintData { raw_values_start: 2759, raw_values_count: 1, token_types_start: 2760, token_types_count: 0 }, // [1145]
    SimpleHintData { raw_values_start: 2760, raw_values_count: 1, token_types_start: 2761, token_types_count: 0 }, // [1146]
    SimpleHintData { raw_values_start: 2761, raw_values_count: 1, token_types_start: 2762, token_types_count: 0 }, // [1147]
    SimpleHintData { raw_values_start: 2762, raw_values_count: 1, token_types_start: 2763, token_types_count: 0 }, // [1148]
    SimpleHintData { raw_values_start: 2763, raw_values_count: 1, token_types_start: 2764, token_types_count: 0 }, // [1149]
    SimpleHintData { raw_values_start: 2764, raw_values_count: 1, token_types_start: 2765, token_types_count: 0 }, // [1150]
    SimpleHintData { raw_values_start: 2765, raw_values_count: 1, token_types_start: 2766, token_types_count: 0 }, // [1151]
    SimpleHintData { raw_values_start: 2766, raw_values_count: 1, token_types_start: 2767, token_types_count: 0 }, // [1152]
    SimpleHintData { raw_values_start: 2767, raw_values_count: 1, token_types_start: 2768, token_types_count: 0 }, // [1153]
    SimpleHintData { raw_values_start: 2768, raw_values_count: 1, token_types_start: 2769, token_types_count: 0 }, // [1154]
    SimpleHintData { raw_values_start: 2769, raw_values_count: 2, token_types_start: 2771, token_types_count: 0 }, // [1155]
    SimpleHintData { raw_values_start: 2771, raw_values_count: 1, token_types_start: 2772, token_types_count: 0 }, // [1156]
    SimpleHintData { raw_values_start: 2772, raw_values_count: 3, token_types_start: 2775, token_types_count: 0 }, // [1157]
    SimpleHintData { raw_values_start: 2775, raw_values_count: 2, token_types_start: 2777, token_types_count: 0 }, // [1158]
    SimpleHintData { raw_values_start: 2777, raw_values_count: 1, token_types_start: 2778, token_types_count: 0 }, // [1159]
    SimpleHintData { raw_values_start: 2778, raw_values_count: 1, token_types_start: 2779, token_types_count: 0 }, // [1160]
    SimpleHintData { raw_values_start: 2779, raw_values_count: 1, token_types_start: 2780, token_types_count: 0 }, // [1161]
    SimpleHintData { raw_values_start: 2780, raw_values_count: 0, token_types_start: 2780, token_types_count: 1 }, // [1162]
    SimpleHintData { raw_values_start: 2781, raw_values_count: 1, token_types_start: 2782, token_types_count: 0 }, // [1163]
    SimpleHintData { raw_values_start: 2782, raw_values_count: 1, token_types_start: 2783, token_types_count: 0 }, // [1164]
    SimpleHintData { raw_values_start: 2783, raw_values_count: 1, token_types_start: 2784, token_types_count: 0 }, // [1165]
    SimpleHintData { raw_values_start: 2784, raw_values_count: 1, token_types_start: 2785, token_types_count: 0 }, // [1166]
    SimpleHintData { raw_values_start: 2785, raw_values_count: 1, token_types_start: 2786, token_types_count: 0 }, // [1167]
    SimpleHintData { raw_values_start: 2786, raw_values_count: 1, token_types_start: 2787, token_types_count: 0 }, // [1168]
    SimpleHintData { raw_values_start: 2787, raw_values_count: 1, token_types_start: 2788, token_types_count: 0 }, // [1169]
    SimpleHintData { raw_values_start: 2788, raw_values_count: 1, token_types_start: 2789, token_types_count: 0 }, // [1170]
    SimpleHintData { raw_values_start: 2789, raw_values_count: 1, token_types_start: 2790, token_types_count: 0 }, // [1171]
    SimpleHintData { raw_values_start: 2790, raw_values_count: 1, token_types_start: 2791, token_types_count: 0 }, // [1172]
    SimpleHintData { raw_values_start: 2791, raw_values_count: 0, token_types_start: 2791, token_types_count: 1 }, // [1173]
    SimpleHintData { raw_values_start: 2792, raw_values_count: 1, token_types_start: 2793, token_types_count: 0 }, // [1174]
    SimpleHintData { raw_values_start: 2793, raw_values_count: 1, token_types_start: 2794, token_types_count: 0 }, // [1175]
    SimpleHintData { raw_values_start: 2794, raw_values_count: 3, token_types_start: 2797, token_types_count: 0 }, // [1176]
    SimpleHintData { raw_values_start: 2797, raw_values_count: 1, token_types_start: 2798, token_types_count: 0 }, // [1177]
    SimpleHintData { raw_values_start: 2798, raw_values_count: 1, token_types_start: 2799, token_types_count: 0 }, // [1178]
    SimpleHintData { raw_values_start: 2799, raw_values_count: 1, token_types_start: 2800, token_types_count: 0 }, // [1179]
    SimpleHintData { raw_values_start: 2800, raw_values_count: 1, token_types_start: 2801, token_types_count: 0 }, // [1180]
    SimpleHintData { raw_values_start: 2801, raw_values_count: 1, token_types_start: 2802, token_types_count: 0 }, // [1181]
    SimpleHintData { raw_values_start: 2802, raw_values_count: 1, token_types_start: 2803, token_types_count: 0 }, // [1182]
    SimpleHintData { raw_values_start: 2803, raw_values_count: 1, token_types_start: 2804, token_types_count: 0 }, // [1183]
    SimpleHintData { raw_values_start: 2804, raw_values_count: 1, token_types_start: 2805, token_types_count: 0 }, // [1184]
    SimpleHintData { raw_values_start: 2805, raw_values_count: 1, token_types_start: 2806, token_types_count: 0 }, // [1185]
    SimpleHintData { raw_values_start: 2806, raw_values_count: 1, token_types_start: 2807, token_types_count: 0 }, // [1186]
    SimpleHintData { raw_values_start: 2807, raw_values_count: 1, token_types_start: 2808, token_types_count: 0 }, // [1187]
    SimpleHintData { raw_values_start: 2808, raw_values_count: 1, token_types_start: 2809, token_types_count: 0 }, // [1188]
    SimpleHintData { raw_values_start: 2809, raw_values_count: 1, token_types_start: 2810, token_types_count: 0 }, // [1189]
    SimpleHintData { raw_values_start: 2810, raw_values_count: 1, token_types_start: 2811, token_types_count: 0 }, // [1190]
    SimpleHintData { raw_values_start: 2811, raw_values_count: 0, token_types_start: 2811, token_types_count: 0 }, // [1191]
    SimpleHintData { raw_values_start: 2811, raw_values_count: 6, token_types_start: 2817, token_types_count: 0 }, // [1192]
    SimpleHintData { raw_values_start: 2817, raw_values_count: 2, token_types_start: 2819, token_types_count: 0 }, // [1193]
    SimpleHintData { raw_values_start: 2819, raw_values_count: 1, token_types_start: 2820, token_types_count: 0 }, // [1194]
    SimpleHintData { raw_values_start: 2820, raw_values_count: 1, token_types_start: 2821, token_types_count: 0 }, // [1195]
    SimpleHintData { raw_values_start: 2821, raw_values_count: 4, token_types_start: 2825, token_types_count: 0 }, // [1196]
    SimpleHintData { raw_values_start: 2825, raw_values_count: 1, token_types_start: 2826, token_types_count: 0 }, // [1197]
    SimpleHintData { raw_values_start: 2826, raw_values_count: 1, token_types_start: 2827, token_types_count: 0 }, // [1198]
    SimpleHintData { raw_values_start: 2827, raw_values_count: 1, token_types_start: 2828, token_types_count: 0 }, // [1199]
    SimpleHintData { raw_values_start: 2828, raw_values_count: 1, token_types_start: 2829, token_types_count: 0 }, // [1200]
    SimpleHintData { raw_values_start: 2829, raw_values_count: 1, token_types_start: 2830, token_types_count: 0 }, // [1201]
    SimpleHintData { raw_values_start: 2830, raw_values_count: 1, token_types_start: 2831, token_types_count: 0 }, // [1202]
    SimpleHintData { raw_values_start: 2831, raw_values_count: 1, token_types_start: 2832, token_types_count: 0 }, // [1203]
    SimpleHintData { raw_values_start: 2832, raw_values_count: 1, token_types_start: 2833, token_types_count: 0 }, // [1204]
    SimpleHintData { raw_values_start: 2833, raw_values_count: 1, token_types_start: 2834, token_types_count: 0 }, // [1205]
    SimpleHintData { raw_values_start: 2834, raw_values_count: 1, token_types_start: 2835, token_types_count: 0 }, // [1206]
    SimpleHintData { raw_values_start: 2835, raw_values_count: 1, token_types_start: 2836, token_types_count: 0 }, // [1207]
    SimpleHintData { raw_values_start: 2836, raw_values_count: 1, token_types_start: 2837, token_types_count: 0 }, // [1208]
    SimpleHintData { raw_values_start: 2837, raw_values_count: 1, token_types_start: 2838, token_types_count: 0 }, // [1209]
    SimpleHintData { raw_values_start: 2838, raw_values_count: 1, token_types_start: 2839, token_types_count: 0 }, // [1210]
    SimpleHintData { raw_values_start: 2839, raw_values_count: 1, token_types_start: 2840, token_types_count: 0 }, // [1211]
    SimpleHintData { raw_values_start: 2840, raw_values_count: 1, token_types_start: 2841, token_types_count: 0 }, // [1212]
    SimpleHintData { raw_values_start: 2841, raw_values_count: 0, token_types_start: 2841, token_types_count: 7 }, // [1213]
    SimpleHintData { raw_values_start: 2848, raw_values_count: 3, token_types_start: 2851, token_types_count: 0 }, // [1214]
    SimpleHintData { raw_values_start: 2851, raw_values_count: 10, token_types_start: 2861, token_types_count: 0 }, // [1215]
    SimpleHintData { raw_values_start: 2861, raw_values_count: 2, token_types_start: 2863, token_types_count: 0 }, // [1216]
    SimpleHintData { raw_values_start: 2863, raw_values_count: 1, token_types_start: 2864, token_types_count: 0 }, // [1217]
    SimpleHintData { raw_values_start: 2864, raw_values_count: 1, token_types_start: 2865, token_types_count: 0 }, // [1218]
    SimpleHintData { raw_values_start: 2865, raw_values_count: 1, token_types_start: 2866, token_types_count: 0 }, // [1219]
    SimpleHintData { raw_values_start: 2866, raw_values_count: 1, token_types_start: 2867, token_types_count: 0 }, // [1220]
    SimpleHintData { raw_values_start: 2867, raw_values_count: 1, token_types_start: 2868, token_types_count: 0 }, // [1221]
    SimpleHintData { raw_values_start: 2868, raw_values_count: 1, token_types_start: 2869, token_types_count: 0 }, // [1222]
    SimpleHintData { raw_values_start: 2869, raw_values_count: 1, token_types_start: 2870, token_types_count: 0 }, // [1223]
    SimpleHintData { raw_values_start: 2870, raw_values_count: 1, token_types_start: 2871, token_types_count: 0 }, // [1224]
    SimpleHintData { raw_values_start: 2871, raw_values_count: 1, token_types_start: 2872, token_types_count: 0 }, // [1225]
    SimpleHintData { raw_values_start: 2872, raw_values_count: 1, token_types_start: 2873, token_types_count: 0 }, // [1226]
    SimpleHintData { raw_values_start: 2873, raw_values_count: 1, token_types_start: 2874, token_types_count: 0 }, // [1227]
    SimpleHintData { raw_values_start: 2874, raw_values_count: 1, token_types_start: 2875, token_types_count: 0 }, // [1228]
    SimpleHintData { raw_values_start: 2875, raw_values_count: 1, token_types_start: 2876, token_types_count: 0 }, // [1229]
    SimpleHintData { raw_values_start: 2876, raw_values_count: 1, token_types_start: 2877, token_types_count: 0 }, // [1230]
    SimpleHintData { raw_values_start: 2877, raw_values_count: 1, token_types_start: 2878, token_types_count: 0 }, // [1231]
    SimpleHintData { raw_values_start: 2878, raw_values_count: 1, token_types_start: 2879, token_types_count: 0 }, // [1232]
    SimpleHintData { raw_values_start: 2879, raw_values_count: 1, token_types_start: 2880, token_types_count: 0 }, // [1233]
    SimpleHintData { raw_values_start: 2880, raw_values_count: 1, token_types_start: 2881, token_types_count: 0 }, // [1234]
    SimpleHintData { raw_values_start: 2881, raw_values_count: 1, token_types_start: 2882, token_types_count: 0 }, // [1235]
    SimpleHintData { raw_values_start: 2882, raw_values_count: 1, token_types_start: 2883, token_types_count: 0 }, // [1236]
    SimpleHintData { raw_values_start: 2883, raw_values_count: 1, token_types_start: 2884, token_types_count: 0 }, // [1237]
    SimpleHintData { raw_values_start: 2884, raw_values_count: 1, token_types_start: 2885, token_types_count: 0 }, // [1238]
    SimpleHintData { raw_values_start: 2885, raw_values_count: 1, token_types_start: 2886, token_types_count: 0 }, // [1239]
    SimpleHintData { raw_values_start: 2886, raw_values_count: 1, token_types_start: 2887, token_types_count: 0 }, // [1240]
    SimpleHintData { raw_values_start: 2887, raw_values_count: 1, token_types_start: 2888, token_types_count: 0 }, // [1241]
    SimpleHintData { raw_values_start: 2888, raw_values_count: 1, token_types_start: 2889, token_types_count: 0 }, // [1242]
    SimpleHintData { raw_values_start: 2889, raw_values_count: 1, token_types_start: 2890, token_types_count: 0 }, // [1243]
    SimpleHintData { raw_values_start: 2890, raw_values_count: 1, token_types_start: 2891, token_types_count: 6 }, // [1244]
    SimpleHintData { raw_values_start: 2897, raw_values_count: 1, token_types_start: 2898, token_types_count: 0 }, // [1245]
    SimpleHintData { raw_values_start: 2898, raw_values_count: 1, token_types_start: 2899, token_types_count: 0 }, // [1246]
    SimpleHintData { raw_values_start: 2899, raw_values_count: 1, token_types_start: 2900, token_types_count: 0 }, // [1247]
    SimpleHintData { raw_values_start: 2900, raw_values_count: 1, token_types_start: 2901, token_types_count: 0 }, // [1248]
    SimpleHintData { raw_values_start: 2901, raw_values_count: 1, token_types_start: 2902, token_types_count: 0 }, // [1249]
    SimpleHintData { raw_values_start: 2902, raw_values_count: 1, token_types_start: 2903, token_types_count: 0 }, // [1250]
    SimpleHintData { raw_values_start: 2903, raw_values_count: 1, token_types_start: 2904, token_types_count: 0 }, // [1251]
    SimpleHintData { raw_values_start: 2904, raw_values_count: 1, token_types_start: 2905, token_types_count: 0 }, // [1252]
    SimpleHintData { raw_values_start: 2905, raw_values_count: 1, token_types_start: 2906, token_types_count: 0 }, // [1253]
    SimpleHintData { raw_values_start: 2906, raw_values_count: 1, token_types_start: 2907, token_types_count: 0 }, // [1254]
    SimpleHintData { raw_values_start: 2907, raw_values_count: 1, token_types_start: 2908, token_types_count: 0 }, // [1255]
    SimpleHintData { raw_values_start: 2908, raw_values_count: 1, token_types_start: 2909, token_types_count: 0 }, // [1256]
    SimpleHintData { raw_values_start: 2909, raw_values_count: 1, token_types_start: 2910, token_types_count: 0 }, // [1257]
    SimpleHintData { raw_values_start: 2910, raw_values_count: 1, token_types_start: 2911, token_types_count: 0 }, // [1258]
    SimpleHintData { raw_values_start: 2911, raw_values_count: 1, token_types_start: 2912, token_types_count: 0 }, // [1259]
    SimpleHintData { raw_values_start: 2912, raw_values_count: 1, token_types_start: 2913, token_types_count: 0 }, // [1260]
    SimpleHintData { raw_values_start: 2913, raw_values_count: 1, token_types_start: 2914, token_types_count: 0 }, // [1261]
    SimpleHintData { raw_values_start: 2914, raw_values_count: 1, token_types_start: 2915, token_types_count: 0 }, // [1262]
    SimpleHintData { raw_values_start: 2915, raw_values_count: 1, token_types_start: 2916, token_types_count: 0 }, // [1263]
    SimpleHintData { raw_values_start: 2916, raw_values_count: 1, token_types_start: 2917, token_types_count: 0 }, // [1264]
    SimpleHintData { raw_values_start: 2917, raw_values_count: 1, token_types_start: 2918, token_types_count: 0 }, // [1265]
    SimpleHintData { raw_values_start: 2918, raw_values_count: 1, token_types_start: 2919, token_types_count: 0 }, // [1266]
    SimpleHintData { raw_values_start: 2919, raw_values_count: 1, token_types_start: 2920, token_types_count: 0 }, // [1267]
    SimpleHintData { raw_values_start: 2920, raw_values_count: 1, token_types_start: 2921, token_types_count: 0 }, // [1268]
    SimpleHintData { raw_values_start: 2921, raw_values_count: 2, token_types_start: 2923, token_types_count: 0 }, // [1269]
    SimpleHintData { raw_values_start: 2923, raw_values_count: 1, token_types_start: 2924, token_types_count: 0 }, // [1270]
    SimpleHintData { raw_values_start: 2924, raw_values_count: 3, token_types_start: 2927, token_types_count: 0 }, // [1271]
    SimpleHintData { raw_values_start: 2927, raw_values_count: 1, token_types_start: 2928, token_types_count: 0 }, // [1272]
    SimpleHintData { raw_values_start: 2928, raw_values_count: 1, token_types_start: 2929, token_types_count: 0 }, // [1273]
    SimpleHintData { raw_values_start: 2929, raw_values_count: 3, token_types_start: 2932, token_types_count: 0 }, // [1274]
    SimpleHintData { raw_values_start: 2932, raw_values_count: 1, token_types_start: 2933, token_types_count: 0 }, // [1275]
    SimpleHintData { raw_values_start: 2933, raw_values_count: 0, token_types_start: 2933, token_types_count: 1 }, // [1276]
    SimpleHintData { raw_values_start: 2934, raw_values_count: 0, token_types_start: 2934, token_types_count: 1 }, // [1277]
    SimpleHintData { raw_values_start: 2935, raw_values_count: 0, token_types_start: 2935, token_types_count: 2 }, // [1278]
    SimpleHintData { raw_values_start: 2937, raw_values_count: 0, token_types_start: 2937, token_types_count: 1 }, // [1279]
    SimpleHintData { raw_values_start: 2938, raw_values_count: 0, token_types_start: 2938, token_types_count: 2 }, // [1280]
    SimpleHintData { raw_values_start: 2940, raw_values_count: 0, token_types_start: 2940, token_types_count: 1 }, // [1281]
    SimpleHintData { raw_values_start: 2941, raw_values_count: 0, token_types_start: 2941, token_types_count: 1 }, // [1282]
    SimpleHintData { raw_values_start: 2942, raw_values_count: 1, token_types_start: 2943, token_types_count: 0 }, // [1283]
    SimpleHintData { raw_values_start: 2943, raw_values_count: 0, token_types_start: 2943, token_types_count: 2 }, // [1284]
    SimpleHintData { raw_values_start: 2945, raw_values_count: 0, token_types_start: 2945, token_types_count: 1 }, // [1285]
    SimpleHintData { raw_values_start: 2946, raw_values_count: 1, token_types_start: 2947, token_types_count: 0 }, // [1286]
    SimpleHintData { raw_values_start: 2947, raw_values_count: 1, token_types_start: 2948, token_types_count: 0 }, // [1287]
    SimpleHintData { raw_values_start: 2948, raw_values_count: 1, token_types_start: 2949, token_types_count: 0 }, // [1288]
    SimpleHintData { raw_values_start: 2949, raw_values_count: 1, token_types_start: 2950, token_types_count: 0 }, // [1289]
    SimpleHintData { raw_values_start: 2950, raw_values_count: 1, token_types_start: 2951, token_types_count: 0 }, // [1290]
    SimpleHintData { raw_values_start: 2951, raw_values_count: 1, token_types_start: 2952, token_types_count: 0 }, // [1291]
    SimpleHintData { raw_values_start: 2952, raw_values_count: 1, token_types_start: 2953, token_types_count: 0 }, // [1292]
    SimpleHintData { raw_values_start: 2953, raw_values_count: 1, token_types_start: 2954, token_types_count: 0 }, // [1293]
    SimpleHintData { raw_values_start: 2954, raw_values_count: 1, token_types_start: 2955, token_types_count: 0 }, // [1294]
    SimpleHintData { raw_values_start: 2955, raw_values_count: 1, token_types_start: 2956, token_types_count: 0 }, // [1295]
    SimpleHintData { raw_values_start: 2956, raw_values_count: 4, token_types_start: 2960, token_types_count: 0 }, // [1296]
    SimpleHintData { raw_values_start: 2960, raw_values_count: 3, token_types_start: 2963, token_types_count: 0 }, // [1297]
    SimpleHintData { raw_values_start: 2963, raw_values_count: 1, token_types_start: 2964, token_types_count: 0 }, // [1298]
    SimpleHintData { raw_values_start: 2964, raw_values_count: 2, token_types_start: 2966, token_types_count: 0 }, // [1299]
    SimpleHintData { raw_values_start: 2966, raw_values_count: 1, token_types_start: 2967, token_types_count: 0 }, // [1300]
    SimpleHintData { raw_values_start: 2967, raw_values_count: 1, token_types_start: 2968, token_types_count: 0 }, // [1301]
    SimpleHintData { raw_values_start: 2968, raw_values_count: 1, token_types_start: 2969, token_types_count: 0 }, // [1302]
    SimpleHintData { raw_values_start: 2969, raw_values_count: 1, token_types_start: 2970, token_types_count: 0 }, // [1303]
    SimpleHintData { raw_values_start: 2970, raw_values_count: 1, token_types_start: 2971, token_types_count: 0 }, // [1304]
    SimpleHintData { raw_values_start: 2971, raw_values_count: 1, token_types_start: 2972, token_types_count: 0 }, // [1305]
    SimpleHintData { raw_values_start: 2972, raw_values_count: 1, token_types_start: 2973, token_types_count: 0 }, // [1306]
    SimpleHintData { raw_values_start: 2973, raw_values_count: 1, token_types_start: 2974, token_types_count: 0 }, // [1307]
    SimpleHintData { raw_values_start: 2974, raw_values_count: 1, token_types_start: 2975, token_types_count: 0 }, // [1308]
    SimpleHintData { raw_values_start: 2975, raw_values_count: 1, token_types_start: 2976, token_types_count: 0 }, // [1309]
    SimpleHintData { raw_values_start: 2976, raw_values_count: 3, token_types_start: 2979, token_types_count: 0 }, // [1310]
    SimpleHintData { raw_values_start: 2979, raw_values_count: 5, token_types_start: 2984, token_types_count: 0 }, // [1311]
    SimpleHintData { raw_values_start: 2984, raw_values_count: 2, token_types_start: 2986, token_types_count: 0 }, // [1312]
    SimpleHintData { raw_values_start: 2986, raw_values_count: 1, token_types_start: 2987, token_types_count: 0 }, // [1313]
    SimpleHintData { raw_values_start: 2987, raw_values_count: 1, token_types_start: 2988, token_types_count: 0 }, // [1314]
    SimpleHintData { raw_values_start: 2988, raw_values_count: 1, token_types_start: 2989, token_types_count: 0 }, // [1315]
    SimpleHintData { raw_values_start: 2989, raw_values_count: 1, token_types_start: 2990, token_types_count: 0 }, // [1316]
    SimpleHintData { raw_values_start: 2990, raw_values_count: 1, token_types_start: 2991, token_types_count: 0 }, // [1317]
    SimpleHintData { raw_values_start: 2991, raw_values_count: 1, token_types_start: 2992, token_types_count: 0 }, // [1318]
    SimpleHintData { raw_values_start: 2992, raw_values_count: 1, token_types_start: 2993, token_types_count: 0 }, // [1319]
    SimpleHintData { raw_values_start: 2993, raw_values_count: 1, token_types_start: 2994, token_types_count: 0 }, // [1320]
    SimpleHintData { raw_values_start: 2994, raw_values_count: 1, token_types_start: 2995, token_types_count: 0 }, // [1321]
    SimpleHintData { raw_values_start: 2995, raw_values_count: 1, token_types_start: 2996, token_types_count: 0 }, // [1322]
    SimpleHintData { raw_values_start: 2996, raw_values_count: 1, token_types_start: 2997, token_types_count: 0 }, // [1323]
    SimpleHintData { raw_values_start: 2997, raw_values_count: 1, token_types_start: 2998, token_types_count: 0 }, // [1324]
    SimpleHintData { raw_values_start: 2998, raw_values_count: 1, token_types_start: 2999, token_types_count: 0 }, // [1325]
    SimpleHintData { raw_values_start: 2999, raw_values_count: 1, token_types_start: 3000, token_types_count: 0 }, // [1326]
    SimpleHintData { raw_values_start: 3000, raw_values_count: 1, token_types_start: 3001, token_types_count: 0 }, // [1327]
    SimpleHintData { raw_values_start: 3001, raw_values_count: 1, token_types_start: 3002, token_types_count: 0 }, // [1328]
    SimpleHintData { raw_values_start: 3002, raw_values_count: 1, token_types_start: 3003, token_types_count: 0 }, // [1329]
    SimpleHintData { raw_values_start: 3003, raw_values_count: 1, token_types_start: 3004, token_types_count: 0 }, // [1330]
    SimpleHintData { raw_values_start: 3004, raw_values_count: 1, token_types_start: 3005, token_types_count: 0 }, // [1331]
    SimpleHintData { raw_values_start: 3005, raw_values_count: 1, token_types_start: 3006, token_types_count: 0 }, // [1332]
    SimpleHintData { raw_values_start: 3006, raw_values_count: 1, token_types_start: 3007, token_types_count: 0 }, // [1333]
    SimpleHintData { raw_values_start: 3007, raw_values_count: 1, token_types_start: 3008, token_types_count: 0 }, // [1334]
    SimpleHintData { raw_values_start: 3008, raw_values_count: 2, token_types_start: 3010, token_types_count: 0 }, // [1335]
    SimpleHintData { raw_values_start: 3010, raw_values_count: 1, token_types_start: 3011, token_types_count: 0 }, // [1336]
    SimpleHintData { raw_values_start: 3011, raw_values_count: 1, token_types_start: 3012, token_types_count: 0 }, // [1337]
    SimpleHintData { raw_values_start: 3012, raw_values_count: 1, token_types_start: 3013, token_types_count: 0 }, // [1338]
    SimpleHintData { raw_values_start: 3013, raw_values_count: 1, token_types_start: 3014, token_types_count: 0 }, // [1339]
    SimpleHintData { raw_values_start: 3014, raw_values_count: 1, token_types_start: 3015, token_types_count: 0 }, // [1340]
    SimpleHintData { raw_values_start: 3015, raw_values_count: 1, token_types_start: 3016, token_types_count: 0 }, // [1341]
    SimpleHintData { raw_values_start: 3016, raw_values_count: 1, token_types_start: 3017, token_types_count: 0 }, // [1342]
    SimpleHintData { raw_values_start: 3017, raw_values_count: 1, token_types_start: 3018, token_types_count: 0 }, // [1343]
    SimpleHintData { raw_values_start: 3018, raw_values_count: 1, token_types_start: 3019, token_types_count: 0 }, // [1344]
    SimpleHintData { raw_values_start: 3019, raw_values_count: 22, token_types_start: 3041, token_types_count: 0 }, // [1345]
    SimpleHintData { raw_values_start: 3041, raw_values_count: 1, token_types_start: 3042, token_types_count: 0 }, // [1346]
    SimpleHintData { raw_values_start: 3042, raw_values_count: 3, token_types_start: 3045, token_types_count: 0 }, // [1347]
    SimpleHintData { raw_values_start: 3045, raw_values_count: 1, token_types_start: 3046, token_types_count: 0 }, // [1348]
    SimpleHintData { raw_values_start: 3046, raw_values_count: 11, token_types_start: 3057, token_types_count: 1 }, // [1349]
    SimpleHintData { raw_values_start: 3058, raw_values_count: 1, token_types_start: 3059, token_types_count: 0 }, // [1350]
    SimpleHintData { raw_values_start: 3059, raw_values_count: 1, token_types_start: 3060, token_types_count: 0 }, // [1351]
    SimpleHintData { raw_values_start: 3060, raw_values_count: 1, token_types_start: 3061, token_types_count: 0 }, // [1352]
    SimpleHintData { raw_values_start: 3061, raw_values_count: 1, token_types_start: 3062, token_types_count: 0 }, // [1353]
    SimpleHintData { raw_values_start: 3062, raw_values_count: 2, token_types_start: 3064, token_types_count: 0 }, // [1354]
    SimpleHintData { raw_values_start: 3064, raw_values_count: 1, token_types_start: 3065, token_types_count: 0 }, // [1355]
    SimpleHintData { raw_values_start: 3065, raw_values_count: 2, token_types_start: 3067, token_types_count: 0 }, // [1356]
    SimpleHintData { raw_values_start: 3067, raw_values_count: 1, token_types_start: 3068, token_types_count: 0 }, // [1357]
    SimpleHintData { raw_values_start: 3068, raw_values_count: 1, token_types_start: 3069, token_types_count: 0 }, // [1358]
    SimpleHintData { raw_values_start: 3069, raw_values_count: 2, token_types_start: 3071, token_types_count: 6 }, // [1359]
    SimpleHintData { raw_values_start: 3077, raw_values_count: 1, token_types_start: 3078, token_types_count: 0 }, // [1360]
    SimpleHintData { raw_values_start: 3078, raw_values_count: 1, token_types_start: 3079, token_types_count: 0 }, // [1361]
    SimpleHintData { raw_values_start: 3079, raw_values_count: 1, token_types_start: 3080, token_types_count: 0 }, // [1362]
    SimpleHintData { raw_values_start: 3080, raw_values_count: 1, token_types_start: 3081, token_types_count: 0 }, // [1363]
    SimpleHintData { raw_values_start: 3081, raw_values_count: 1, token_types_start: 3082, token_types_count: 0 }, // [1364]
    SimpleHintData { raw_values_start: 3082, raw_values_count: 1, token_types_start: 3083, token_types_count: 0 }, // [1365]
    SimpleHintData { raw_values_start: 3083, raw_values_count: 1, token_types_start: 3084, token_types_count: 0 }, // [1366]
    SimpleHintData { raw_values_start: 3084, raw_values_count: 1, token_types_start: 3085, token_types_count: 0 }, // [1367]
    SimpleHintData { raw_values_start: 3085, raw_values_count: 1, token_types_start: 3086, token_types_count: 0 }, // [1368]
    SimpleHintData { raw_values_start: 3086, raw_values_count: 1, token_types_start: 3087, token_types_count: 0 }, // [1369]
    SimpleHintData { raw_values_start: 3087, raw_values_count: 1, token_types_start: 3088, token_types_count: 0 }, // [1370]
    SimpleHintData { raw_values_start: 3088, raw_values_count: 1, token_types_start: 3089, token_types_count: 0 }, // [1371]
    SimpleHintData { raw_values_start: 3089, raw_values_count: 1, token_types_start: 3090, token_types_count: 0 }, // [1372]
    SimpleHintData { raw_values_start: 3090, raw_values_count: 1, token_types_start: 3091, token_types_count: 0 }, // [1373]
    SimpleHintData { raw_values_start: 3091, raw_values_count: 1, token_types_start: 3092, token_types_count: 0 }, // [1374]
    SimpleHintData { raw_values_start: 3092, raw_values_count: 1, token_types_start: 3093, token_types_count: 0 }, // [1375]
    SimpleHintData { raw_values_start: 3093, raw_values_count: 1, token_types_start: 3094, token_types_count: 0 }, // [1376]
    SimpleHintData { raw_values_start: 3094, raw_values_count: 1, token_types_start: 3095, token_types_count: 0 }, // [1377]
    SimpleHintData { raw_values_start: 3095, raw_values_count: 6, token_types_start: 3101, token_types_count: 0 }, // [1378]
    SimpleHintData { raw_values_start: 3101, raw_values_count: 1, token_types_start: 3102, token_types_count: 0 }, // [1379]
    SimpleHintData { raw_values_start: 3102, raw_values_count: 1, token_types_start: 3103, token_types_count: 0 }, // [1380]
    SimpleHintData { raw_values_start: 3103, raw_values_count: 1, token_types_start: 3104, token_types_count: 0 }, // [1381]
    SimpleHintData { raw_values_start: 3104, raw_values_count: 1, token_types_start: 3105, token_types_count: 0 }, // [1382]
    SimpleHintData { raw_values_start: 3105, raw_values_count: 1, token_types_start: 3106, token_types_count: 0 }, // [1383]
    SimpleHintData { raw_values_start: 3106, raw_values_count: 1, token_types_start: 3107, token_types_count: 0 }, // [1384]
    SimpleHintData { raw_values_start: 3107, raw_values_count: 1, token_types_start: 3108, token_types_count: 0 }, // [1385]
    SimpleHintData { raw_values_start: 3108, raw_values_count: 1, token_types_start: 3109, token_types_count: 0 }, // [1386]
    SimpleHintData { raw_values_start: 3109, raw_values_count: 1, token_types_start: 3110, token_types_count: 0 }, // [1387]
    SimpleHintData { raw_values_start: 3110, raw_values_count: 1, token_types_start: 3111, token_types_count: 0 }, // [1388]
    SimpleHintData { raw_values_start: 3111, raw_values_count: 1, token_types_start: 3112, token_types_count: 0 }, // [1389]
    SimpleHintData { raw_values_start: 3112, raw_values_count: 1, token_types_start: 3113, token_types_count: 0 }, // [1390]
    SimpleHintData { raw_values_start: 3113, raw_values_count: 1, token_types_start: 3114, token_types_count: 0 }, // [1391]
    SimpleHintData { raw_values_start: 3114, raw_values_count: 1, token_types_start: 3115, token_types_count: 0 }, // [1392]
    SimpleHintData { raw_values_start: 3115, raw_values_count: 1, token_types_start: 3116, token_types_count: 0 }, // [1393]
    SimpleHintData { raw_values_start: 3116, raw_values_count: 1, token_types_start: 3117, token_types_count: 0 }, // [1394]
    SimpleHintData { raw_values_start: 3117, raw_values_count: 1, token_types_start: 3118, token_types_count: 0 }, // [1395]
    SimpleHintData { raw_values_start: 3118, raw_values_count: 1, token_types_start: 3119, token_types_count: 0 }, // [1396]
    SimpleHintData { raw_values_start: 3119, raw_values_count: 1, token_types_start: 3120, token_types_count: 0 }, // [1397]
    SimpleHintData { raw_values_start: 3120, raw_values_count: 1, token_types_start: 3121, token_types_count: 0 }, // [1398]
    SimpleHintData { raw_values_start: 3121, raw_values_count: 1, token_types_start: 3122, token_types_count: 0 }, // [1399]
    SimpleHintData { raw_values_start: 3122, raw_values_count: 1, token_types_start: 3123, token_types_count: 0 }, // [1400]
    SimpleHintData { raw_values_start: 3123, raw_values_count: 1, token_types_start: 3124, token_types_count: 0 }, // [1401]
    SimpleHintData { raw_values_start: 3124, raw_values_count: 1, token_types_start: 3125, token_types_count: 0 }, // [1402]
    SimpleHintData { raw_values_start: 3125, raw_values_count: 3, token_types_start: 3128, token_types_count: 0 }, // [1403]
    SimpleHintData { raw_values_start: 3128, raw_values_count: 2, token_types_start: 3130, token_types_count: 0 }, // [1404]
    SimpleHintData { raw_values_start: 3130, raw_values_count: 3, token_types_start: 3133, token_types_count: 0 }, // [1405]
    SimpleHintData { raw_values_start: 3133, raw_values_count: 1, token_types_start: 3134, token_types_count: 0 }, // [1406]
    SimpleHintData { raw_values_start: 3134, raw_values_count: 1, token_types_start: 3135, token_types_count: 0 }, // [1407]
    SimpleHintData { raw_values_start: 3135, raw_values_count: 1, token_types_start: 3136, token_types_count: 0 }, // [1408]
    SimpleHintData { raw_values_start: 3136, raw_values_count: 1, token_types_start: 3137, token_types_count: 0 }, // [1409]
    SimpleHintData { raw_values_start: 3137, raw_values_count: 1, token_types_start: 3138, token_types_count: 0 }, // [1410]
    SimpleHintData { raw_values_start: 3138, raw_values_count: 3, token_types_start: 3141, token_types_count: 0 }, // [1411]
    SimpleHintData { raw_values_start: 3141, raw_values_count: 1, token_types_start: 3142, token_types_count: 0 }, // [1412]
    SimpleHintData { raw_values_start: 3142, raw_values_count: 1, token_types_start: 3143, token_types_count: 0 }, // [1413]
    SimpleHintData { raw_values_start: 3143, raw_values_count: 1, token_types_start: 3144, token_types_count: 0 }, // [1414]
    SimpleHintData { raw_values_start: 3144, raw_values_count: 1, token_types_start: 3145, token_types_count: 0 }, // [1415]
    SimpleHintData { raw_values_start: 3145, raw_values_count: 1, token_types_start: 3146, token_types_count: 0 }, // [1416]
    SimpleHintData { raw_values_start: 3146, raw_values_count: 1, token_types_start: 3147, token_types_count: 0 }, // [1417]
    SimpleHintData { raw_values_start: 3147, raw_values_count: 1, token_types_start: 3148, token_types_count: 0 }, // [1418]
    SimpleHintData { raw_values_start: 3148, raw_values_count: 1, token_types_start: 3149, token_types_count: 0 }, // [1419]
    SimpleHintData { raw_values_start: 3149, raw_values_count: 1, token_types_start: 3150, token_types_count: 0 }, // [1420]
    SimpleHintData { raw_values_start: 3150, raw_values_count: 2, token_types_start: 3152, token_types_count: 0 }, // [1421]
    SimpleHintData { raw_values_start: 3152, raw_values_count: 1, token_types_start: 3153, token_types_count: 0 }, // [1422]
    SimpleHintData { raw_values_start: 3153, raw_values_count: 3, token_types_start: 3156, token_types_count: 0 }, // [1423]
    SimpleHintData { raw_values_start: 3156, raw_values_count: 1, token_types_start: 3157, token_types_count: 0 }, // [1424]
    SimpleHintData { raw_values_start: 3157, raw_values_count: 1, token_types_start: 3158, token_types_count: 0 }, // [1425]
    SimpleHintData { raw_values_start: 3158, raw_values_count: 1, token_types_start: 3159, token_types_count: 0 }, // [1426]
    SimpleHintData { raw_values_start: 3159, raw_values_count: 1, token_types_start: 3160, token_types_count: 0 }, // [1427]
    SimpleHintData { raw_values_start: 3160, raw_values_count: 1, token_types_start: 3161, token_types_count: 0 }, // [1428]
    SimpleHintData { raw_values_start: 3161, raw_values_count: 1, token_types_start: 3162, token_types_count: 0 }, // [1429]
    SimpleHintData { raw_values_start: 3162, raw_values_count: 1, token_types_start: 3163, token_types_count: 0 }, // [1430]
    SimpleHintData { raw_values_start: 3163, raw_values_count: 1, token_types_start: 3164, token_types_count: 0 }, // [1431]
    SimpleHintData { raw_values_start: 3164, raw_values_count: 1, token_types_start: 3165, token_types_count: 0 }, // [1432]
    SimpleHintData { raw_values_start: 3165, raw_values_count: 1, token_types_start: 3166, token_types_count: 0 }, // [1433]
    SimpleHintData { raw_values_start: 3166, raw_values_count: 1, token_types_start: 3167, token_types_count: 0 }, // [1434]
    SimpleHintData { raw_values_start: 3167, raw_values_count: 1, token_types_start: 3168, token_types_count: 0 }, // [1435]
    SimpleHintData { raw_values_start: 3168, raw_values_count: 1, token_types_start: 3169, token_types_count: 0 }, // [1436]
    SimpleHintData { raw_values_start: 3169, raw_values_count: 1, token_types_start: 3170, token_types_count: 0 }, // [1437]
    SimpleHintData { raw_values_start: 3170, raw_values_count: 2, token_types_start: 3172, token_types_count: 0 }, // [1438]
    SimpleHintData { raw_values_start: 3172, raw_values_count: 1, token_types_start: 3173, token_types_count: 0 }, // [1439]
    SimpleHintData { raw_values_start: 3173, raw_values_count: 1, token_types_start: 3174, token_types_count: 0 }, // [1440]
    SimpleHintData { raw_values_start: 3174, raw_values_count: 1, token_types_start: 3175, token_types_count: 0 }, // [1441]
    SimpleHintData { raw_values_start: 3175, raw_values_count: 1, token_types_start: 3176, token_types_count: 0 }, // [1442]
    SimpleHintData { raw_values_start: 3176, raw_values_count: 2, token_types_start: 3178, token_types_count: 0 }, // [1443]
    SimpleHintData { raw_values_start: 3178, raw_values_count: 1, token_types_start: 3179, token_types_count: 0 }, // [1444]
    SimpleHintData { raw_values_start: 3179, raw_values_count: 33, token_types_start: 3212, token_types_count: 1 }, // [1445]
    SimpleHintData { raw_values_start: 3213, raw_values_count: 4, token_types_start: 3217, token_types_count: 0 }, // [1446]
    SimpleHintData { raw_values_start: 3217, raw_values_count: 3, token_types_start: 3220, token_types_count: 0 }, // [1447]
    SimpleHintData { raw_values_start: 3220, raw_values_count: 5, token_types_start: 3225, token_types_count: 1 }, // [1448]
    SimpleHintData { raw_values_start: 3226, raw_values_count: 2, token_types_start: 3228, token_types_count: 0 }, // [1449]
    SimpleHintData { raw_values_start: 3228, raw_values_count: 2, token_types_start: 3230, token_types_count: 0 }, // [1450]
    SimpleHintData { raw_values_start: 3230, raw_values_count: 1, token_types_start: 3231, token_types_count: 0 }, // [1451]
    SimpleHintData { raw_values_start: 3231, raw_values_count: 1, token_types_start: 3232, token_types_count: 0 }, // [1452]
    SimpleHintData { raw_values_start: 3232, raw_values_count: 1, token_types_start: 3233, token_types_count: 0 }, // [1453]
    SimpleHintData { raw_values_start: 3233, raw_values_count: 1, token_types_start: 3234, token_types_count: 0 }, // [1454]
    SimpleHintData { raw_values_start: 3234, raw_values_count: 1, token_types_start: 3235, token_types_count: 0 }, // [1455]
    SimpleHintData { raw_values_start: 3235, raw_values_count: 1, token_types_start: 3236, token_types_count: 0 }, // [1456]
    SimpleHintData { raw_values_start: 3236, raw_values_count: 1, token_types_start: 3237, token_types_count: 0 }, // [1457]
    SimpleHintData { raw_values_start: 3237, raw_values_count: 1, token_types_start: 3238, token_types_count: 0 }, // [1458]
    SimpleHintData { raw_values_start: 3238, raw_values_count: 1, token_types_start: 3239, token_types_count: 0 }, // [1459]
    SimpleHintData { raw_values_start: 3239, raw_values_count: 1, token_types_start: 3240, token_types_count: 0 }, // [1460]
    SimpleHintData { raw_values_start: 3240, raw_values_count: 1, token_types_start: 3241, token_types_count: 0 }, // [1461]
    SimpleHintData { raw_values_start: 3241, raw_values_count: 1, token_types_start: 3242, token_types_count: 0 }, // [1462]
    SimpleHintData { raw_values_start: 3242, raw_values_count: 5, token_types_start: 3247, token_types_count: 0 }, // [1463]
    SimpleHintData { raw_values_start: 3247, raw_values_count: 2, token_types_start: 3249, token_types_count: 0 }, // [1464]
    SimpleHintData { raw_values_start: 3249, raw_values_count: 1, token_types_start: 3250, token_types_count: 0 }, // [1465]
    SimpleHintData { raw_values_start: 3250, raw_values_count: 1, token_types_start: 3251, token_types_count: 0 }, // [1466]
    SimpleHintData { raw_values_start: 3251, raw_values_count: 1, token_types_start: 3252, token_types_count: 0 }, // [1467]
    SimpleHintData { raw_values_start: 3252, raw_values_count: 1, token_types_start: 3253, token_types_count: 0 }, // [1468]
    SimpleHintData { raw_values_start: 3253, raw_values_count: 1, token_types_start: 3254, token_types_count: 0 }, // [1469]
    SimpleHintData { raw_values_start: 3254, raw_values_count: 1, token_types_start: 3255, token_types_count: 0 }, // [1470]
    SimpleHintData { raw_values_start: 3255, raw_values_count: 1, token_types_start: 3256, token_types_count: 0 }, // [1471]
    SimpleHintData { raw_values_start: 3256, raw_values_count: 1, token_types_start: 3257, token_types_count: 0 }, // [1472]
    SimpleHintData { raw_values_start: 3257, raw_values_count: 10, token_types_start: 3267, token_types_count: 0 }, // [1473]
    SimpleHintData { raw_values_start: 3267, raw_values_count: 1, token_types_start: 3268, token_types_count: 0 }, // [1474]
    SimpleHintData { raw_values_start: 3268, raw_values_count: 1, token_types_start: 3269, token_types_count: 0 }, // [1475]
    SimpleHintData { raw_values_start: 3269, raw_values_count: 2, token_types_start: 3271, token_types_count: 0 }, // [1476]
    SimpleHintData { raw_values_start: 3271, raw_values_count: 2, token_types_start: 3273, token_types_count: 0 }, // [1477]
    SimpleHintData { raw_values_start: 3273, raw_values_count: 1, token_types_start: 3274, token_types_count: 0 }, // [1478]
    SimpleHintData { raw_values_start: 3274, raw_values_count: 1, token_types_start: 3275, token_types_count: 0 }, // [1479]
    SimpleHintData { raw_values_start: 3275, raw_values_count: 1, token_types_start: 3276, token_types_count: 0 }, // [1480]
    SimpleHintData { raw_values_start: 3276, raw_values_count: 1, token_types_start: 3277, token_types_count: 0 }, // [1481]
    SimpleHintData { raw_values_start: 3277, raw_values_count: 1, token_types_start: 3278, token_types_count: 0 }, // [1482]
    SimpleHintData { raw_values_start: 3278, raw_values_count: 1, token_types_start: 3279, token_types_count: 0 }, // [1483]
    SimpleHintData { raw_values_start: 3279, raw_values_count: 1, token_types_start: 3280, token_types_count: 0 }, // [1484]
    SimpleHintData { raw_values_start: 3280, raw_values_count: 1, token_types_start: 3281, token_types_count: 0 }, // [1485]
    SimpleHintData { raw_values_start: 3281, raw_values_count: 1, token_types_start: 3282, token_types_count: 0 }, // [1486]
    SimpleHintData { raw_values_start: 3282, raw_values_count: 1, token_types_start: 3283, token_types_count: 0 }, // [1487]
    SimpleHintData { raw_values_start: 3283, raw_values_count: 1, token_types_start: 3284, token_types_count: 0 }, // [1488]
    SimpleHintData { raw_values_start: 3284, raw_values_count: 1, token_types_start: 3285, token_types_count: 0 }, // [1489]
    SimpleHintData { raw_values_start: 3285, raw_values_count: 1, token_types_start: 3286, token_types_count: 0 }, // [1490]
    SimpleHintData { raw_values_start: 3286, raw_values_count: 1, token_types_start: 3287, token_types_count: 0 }, // [1491]
    SimpleHintData { raw_values_start: 3287, raw_values_count: 1, token_types_start: 3288, token_types_count: 0 }, // [1492]
    SimpleHintData { raw_values_start: 3288, raw_values_count: 1, token_types_start: 3289, token_types_count: 0 }, // [1493]
    SimpleHintData { raw_values_start: 3289, raw_values_count: 1, token_types_start: 3290, token_types_count: 0 }, // [1494]
    SimpleHintData { raw_values_start: 3290, raw_values_count: 1, token_types_start: 3291, token_types_count: 0 }, // [1495]
    SimpleHintData { raw_values_start: 3291, raw_values_count: 1, token_types_start: 3292, token_types_count: 0 }, // [1496]
    SimpleHintData { raw_values_start: 3292, raw_values_count: 1, token_types_start: 3293, token_types_count: 0 }, // [1497]
    SimpleHintData { raw_values_start: 3293, raw_values_count: 1, token_types_start: 3294, token_types_count: 0 }, // [1498]
    SimpleHintData { raw_values_start: 3294, raw_values_count: 1, token_types_start: 3295, token_types_count: 0 }, // [1499]
];

pub static SIMPLE_HINT_INDICES: &[u32] = &[
    1, 2, 3, 4, 5, 6, 7, 8, 0, 9, 0, 10, 11, 11, 12, 13,
    14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 23, 24, 25, 26, 26, 27,
    28, 28, 29, 30, 14, 31, 31, 29, 30, 14, 32, 28, 28, 33, 30, 18,
    31, 31, 33, 30, 34, 14, 18, 0, 0, 0, 0, 0, 35, 0, 36, 37,
    38, 39, 39, 40, 41, 42, 43, 43, 44, 20, 45, 13, 14, 16, 46, 46,
    47, 48, 49, 49, 50, 51, 51, 52, 12, 12, 53, 54, 55, 56, 56, 43,
    43, 57, 18, 47, 47, 48, 58, 32, 59, 59, 60, 18, 19, 61, 61, 62,
    46, 63, 43, 64, 49, 65, 66, 12, 67, 68, 69, 70, 71, 72, 73, 74,
    75, 76, 77, 55, 78, 79, 28, 28, 62, 35, 42, 42, 80, 32, 81, 82,
    83, 84, 85, 86, 87, 88, 50, 18, 47, 47, 48, 89, 89, 82, 90, 90,
    81, 91, 91, 92, 33, 93, 94, 95, 96, 97, 98, 99, 100, 101, 29, 47,
    47, 0, 89, 89, 94, 90, 90, 93, 91, 91, 0, 102, 103, 104, 0, 105,
    105, 0, 0, 45, 45, 0, 0, 20, 20, 0, 0, 106, 106, 0, 0, 0,
    0, 0, 0, 107, 108, 109, 110, 111, 112, 9, 113, 114, 115, 116, 117, 118,
    118, 119, 118, 120, 121, 122, 123, 124, 125, 0, 0, 126, 0, 0, 0, 35,
    0, 35, 127, 0, 0, 35, 0, 0, 0, 0, 0, 0, 0, 0, 28, 128,
    129, 130, 131, 131, 124, 0, 35, 0, 35, 0, 132, 133, 134, 134, 37, 0,
    135, 135, 37, 0, 136, 137, 138, 0, 139, 139, 18, 0, 131, 131, 14, 0,
    0, 0, 140, 0, 0, 0, 141, 142, 142, 143, 41, 0, 0, 141, 134, 134,
    37, 0, 135, 135, 37, 0, 0, 136, 137, 138, 139, 139, 21, 0, 139, 139,
    0, 144, 145, 145, 37, 146, 0, 147, 148, 0, 149, 38, 38, 150, 151, 151,
    0, 28, 0, 103, 103, 152, 80, 153, 37, 0, 0, 140, 140, 140, 103, 154,
    152, 28, 28, 62, 155, 155, 43, 64, 49, 65, 71, 72, 75, 76, 77, 55,
    80, 104, 104, 103, 103, 154, 156, 152, 80, 153, 38, 0, 0, 157, 158, 159,
    153, 93, 96, 98, 99, 160, 29, 97, 97, 131, 131, 147, 62, 156, 156, 161,
    20, 45, 0, 0, 30, 103, 104, 30, 30, 18, 0, 0, 103, 104, 102, 103,
    104, 0, 0, 105, 0, 107, 131, 131, 15, 0, 162, 139, 139, 147, 0, 163,
    163, 147, 164, 164, 139, 163, 165, 166, 116, 116, 167, 167, 0, 168, 169, 165,
    166, 170, 170, 0, 165, 165, 171, 163, 163, 167, 172, 0, 159, 157, 158, 134,
    134, 167, 0, 37, 0, 173, 173, 167, 0, 135, 135, 37, 0, 0, 136, 137,
    138, 134, 134, 37, 0, 139, 139, 18, 0, 131, 131, 174, 0, 175, 77, 77,
    37, 37, 148, 139, 139, 18, 0, 0, 176, 177, 177, 116, 163, 178, 179, 179,
    8, 8, 180, 181, 182, 15, 183, 183, 75, 23, 23, 26, 27, 23, 23, 81,
    184, 185, 17, 18, 87, 25, 186, 186, 187, 188, 189, 190, 191, 192, 22, 0,
    193, 193, 89, 89, 82, 81, 82, 0, 124, 124, 0, 35, 0, 0, 194, 194,
    35, 0, 0, 126, 0, 195, 195, 84, 85, 86, 0, 0, 0, 35, 196, 196,
    0, 0, 0, 35, 0, 0, 0, 36, 0, 197, 197, 198, 199, 0, 200, 0,
    201, 201, 156, 22, 0, 202, 116, 116, 203, 172, 0, 0, 204, 204, 0, 205,
    205, 163, 163, 203, 172, 0, 159, 131, 131, 203, 0, 206, 207, 139, 139, 26,
    22, 0, 204, 204, 0, 200, 200, 0, 0, 139, 139, 147, 0, 0, 148, 0,
    208, 0, 163, 163, 147, 209, 164, 139, 163, 165, 166, 163, 163, 210, 172, 116,
    116, 211, 212, 213, 214, 214, 147, 126, 215, 35, 216, 216, 217, 217, 139, 139,
    211, 212, 213, 214, 214, 147, 139, 139, 216, 218, 218, 140, 140, 41, 163, 163,
    215, 172, 139, 139, 219, 41, 139, 139, 35, 151, 151, 35, 139, 139, 220, 221,
    222, 90, 223, 224, 139, 139, 225, 0, 116, 116, 226, 116, 116, 227, 131, 131,
    167, 0, 228, 229, 165, 165, 229, 230, 230, 230, 231, 230, 230, 232, 173, 173,
    167, 0, 163, 163, 167, 172, 0, 159, 233, 233, 234, 235, 75, 0, 0, 28,
    45, 234, 234, 236, 237, 213, 75, 0, 233, 233, 234, 235, 234, 234, 237, 234,
    234, 238, 238, 0, 239, 239, 235, 234, 240, 241, 241, 240, 242, 243, 244, 245,
    245, 80, 0, 139, 139, 246, 247, 245, 248, 139, 139, 21, 0, 139, 139, 249,
    250, 251, 139, 139, 35, 151, 151, 35, 252, 241, 253, 0, 254, 254, 0, 165,
    165, 254, 135, 135, 37, 0, 0, 136, 137, 138, 237, 237, 215, 255, 147, 200,
    200, 256, 0, 257, 258, 259, 131, 131, 0, 0, 260, 35, 0, 0, 261, 116,
    139, 163, 0, 262, 131, 131, 23, 81, 0, 172, 263, 0, 132, 264, 202, 202,
    134, 134, 203, 0, 37, 0, 134, 134, 37, 0, 265, 265, 266, 267, 267, 80,
    166, 268, 269, 269, 166, 80, 166, 268, 270, 271, 272, 273, 274, 275, 165, 276,
    277, 90, 244, 278, 279, 280, 281, 281, 282, 283, 284, 285, 286, 286, 41, 287,
    287, 41, 288, 288, 0, 139, 139, 0, 144, 145, 145, 37, 146, 0, 148, 0,
    147, 38, 38, 150, 151, 151, 0, 28, 0, 157, 131, 131, 84, 0, 0, 0,
    35, 289, 265, 134, 134, 37, 0, 139, 139, 18, 0, 135, 135, 37, 0, 0,
    0, 149, 136, 137, 138, 290, 241, 291, 80, 174, 0, 131, 131, 105, 0, 292,
    177, 177, 116, 163, 45, 0, 0, 134, 134, 37, 0, 131, 131, 256, 0, 0,
    172, 0, 293, 134, 134, 37, 0, 139, 139, 21, 0, 294, 294, 295, 0, 290,
    241, 291, 80, 174, 0, 139, 139, 35, 0, 0, 0, 146, 146, 148, 151, 151,
    35, 0, 0, 131, 131, 203, 41, 139, 219, 41, 28, 28, 30, 21, 0, 296,
    296, 214, 0, 0, 139, 21, 0, 297, 131, 298, 131, 131, 203, 0, 299, 139,
    139, 219, 41, 139, 139, 35, 0, 0, 0, 146, 148, 151, 151, 35, 0, 0,
    139, 139, 220, 221, 222, 90, 223, 224, 139, 139, 225, 0, 245, 245, 80, 0,
    139, 139, 246, 245, 139, 139, 35, 0, 0, 0, 146, 146, 148, 151, 151, 35,
    0, 0, 135, 135, 37, 0, 0, 136, 137, 138, 131, 131, 89, 82, 0, 0,
    172, 0, 300, 298, 298, 134, 134, 203, 203, 0, 37, 0, 134, 134, 37, 0,
    139, 139, 18, 0, 0, 0, 241, 291, 80, 174, 0, 28, 28, 30, 21, 0,
    296, 296, 214, 0, 0, 139, 21, 0, 297, 297, 131, 131, 48, 0, 80, 0,
    301, 134, 134, 37, 0, 302, 37, 37, 0, 0, 0, 107, 136, 137, 138, 200,
    200, 35, 0, 140, 140, 170, 35, 0, 303, 303, 277, 90, 244, 278, 279, 280,
    139, 139, 0, 144, 145, 145, 37, 146, 0, 148, 0, 147, 38, 38, 150, 151,
    151, 0, 28, 0, 157, 131, 131, 85, 0, 35, 304, 134, 134, 37, 0, 135,
    135, 37, 0, 0, 0, 149, 137, 138, 131, 131, 305, 0, 306, 139, 139, 35,
    116, 116, 307, 307, 139, 139, 307, 307, 163, 163, 307, 307, 134, 134, 37, 0,
    135, 135, 37, 0, 136, 137, 138, 0, 131, 131, 161, 20, 45, 0, 0, 0,
    136, 137, 138, 0, 140, 308, 309, 310, 311, 312, 313, 314, 315, 316, 317, 318,
    253, 319, 320, 321, 322, 323, 324, 325, 326, 327, 328, 142, 142, 143, 41, 329,
    330, 331, 332, 262, 166, 171, 171, 333, 262, 0, 0, 134, 134, 37, 0, 0,
    0, 136, 137, 138, 28, 0, 30, 30, 14, 0, 299, 139, 139, 0, 144, 145,
    145, 37, 146, 0, 147, 0, 0, 148, 0, 80, 38, 38, 150, 151, 151, 0,
    0, 28, 131, 131, 18, 0, 334, 134, 134, 37, 0, 135, 135, 37, 0, 335,
    216, 126, 126, 0, 336, 336, 214, 337, 41, 338, 339, 339, 337, 41, 241, 241,
    339, 340, 341, 341, 337, 41, 241, 241, 341, 87, 87, 342, 0, 343, 343, 140,
    337, 41, 218, 218, 140, 337, 41, 344, 344, 41, 345, 241, 346, 296, 296, 214,
    0, 347, 0, 131, 131, 87, 172, 0, 348, 216, 216, 135, 135, 37, 0, 0,
    137, 138, 134, 134, 37, 0, 139, 139, 18, 0, 131, 131, 219, 172, 0, 133,
    135, 135, 37, 0, 0, 0, 149, 136, 137, 138, 134, 134, 37, 0, 139, 139,
    349, 18, 18, 0, 219, 219, 41, 131, 131, 350, 0, 351, 142, 142, 262, 261,
    261, 139, 116, 163, 305, 0, 0, 140, 140, 35, 352, 352, 305, 140, 140, 35,
    234, 235, 139, 139, 35, 353, 353, 35, 0, 146, 0, 135, 135, 37, 0, 0,
    136, 137, 0, 134, 134, 37, 0, 131, 131, 354, 299, 139, 139, 0, 145, 37,
    146, 0, 147, 0, 0, 148, 0, 149, 80, 151, 151, 0, 28, 0, 355, 116,
    116, 226, 165, 165, 171, 173, 173, 167, 0, 163, 163, 167, 0, 159, 135, 135,
    37, 0, 0, 0, 149, 134, 134, 37, 0, 0, 0, 149, 134, 134, 203, 37,
    0, 0, 131, 131, 203, 0, 356, 22, 22, 0, 357, 357, 358, 358, 131, 131,
    359, 0, 131, 131, 360, 361, 28, 362, 363, 363, 359, 0, 364, 131, 131, 365,
    366, 35, 0, 0, 131, 131, 366, 367, 364, 347, 131, 131, 357, 364, 116, 116,
    203, 0, 0, 147, 147, 0, 357, 357, 358, 204, 204, 0, 205, 205, 163, 163,
    203, 0, 159, 368, 368, 38, 0, 369, 370, 139, 139, 371, 262, 163, 163, 203,
    172, 0, 0, 0, 0, 146, 0, 148, 0, 372, 372, 116, 66, 203, 0, 373,
    374, 374, 375, 123, 0, 35, 163, 134, 134, 376, 126, 37, 0, 131, 131, 81,
    0, 0, 172, 263, 0, 132, 355, 355, 355, 134, 134, 203, 0, 37, 0, 134,
    134, 167, 0, 37, 0, 0, 172, 0, 377, 134, 134, 37, 0, 139, 139, 18,
    0, 294, 294, 295, 0, 378, 156, 156, 379, 380, 147, 381, 381, 295, 0, 382,
    383, 28, 28, 30, 21, 0, 296, 296, 214, 0, 0, 139, 21, 0, 297, 131,
    131, 186, 187, 189, 0, 384, 116, 116, 385, 156, 0, 0, 140, 0, 0, 131,
    131, 385, 156, 0, 0, 140, 0, 0, 131, 131, 385, 386, 0, 140, 0, 131,
    131, 385, 156, 0, 0, 386, 0, 140, 0, 163, 163, 385, 172, 156, 0, 0,
    134, 134, 37, 0, 135, 135, 37, 0, 0, 136, 137, 138, 139, 139, 18, 0,
    131, 131, 75, 0, 80, 0, 387, 134, 134, 37, 0, 290, 241, 291, 80, 174,
    0, 131, 131, 22, 0, 384, 135, 135, 37, 0, 137, 138, 136, 0, 134, 134,
    388, 262, 37, 262, 134, 134, 37, 0, 139, 139, 18, 0, 389, 389, 116, 116,
    390, 0, 0, 204, 204, 0, 159, 131, 131, 390, 0, 139, 139, 26, 22, 0,
    204, 204, 0, 159, 163, 163, 390, 172, 0, 159, 134, 134, 390, 0, 37, 0,
    159, 116, 116, 388, 172, 262, 391, 391, 392, 123, 262, 131, 131, 45, 0, 140,
    393, 312, 313, 314, 394, 395, 396, 397, 397, 8, 398, 284, 399, 331, 331, 400,
    262, 235, 171, 171, 333, 262, 134, 134, 37, 0, 142, 401, 402, 402, 403, 41,
    151, 151, 402, 403, 299, 139, 139, 0, 145, 37, 146, 404, 147, 148, 151, 151,
    0, 131, 131, 82, 172, 0, 405, 131, 131, 203, 0, 164, 139, 139, 147, 0,
    148, 0, 208, 0, 163, 163, 147, 135, 135, 37, 0, 0, 136, 137, 138, 134,
    134, 203, 0, 37, 0, 134, 134, 37, 0, 139, 139, 18, 0, 139, 139, 35,
    0, 0, 0, 146, 146, 148, 151, 151, 35, 0, 0, 131, 131, 406, 0, 407,
    408, 408, 116, 409, 410, 81, 81, 0, 0, 18, 18, 0, 0, 84, 84, 0,
    0, 28, 28, 411, 93, 96, 30, 18, 0, 0, 139, 139, 412, 413, 413, 146,
    141, 414, 414, 146, 141, 156, 18, 0, 213, 415, 416, 417, 417, 418, 416, 225,
    0, 0, 35, 0, 0, 419, 419, 41, 418, 417, 417, 418, 416, 420, 35, 0,
    0, 0, 421, 0, 141, 421, 0, 0, 0, 35, 0, 0, 422, 422, 423, 424,
    368, 46, 425, 426, 427, 30, 428, 429, 430, 431, 118, 432, 132, 433, 415, 119,
    434, 435, 436, 38, 38, 437, 262, 140, 140, 140, 26, 19, 18, 262, 140, 140,
    241, 26, 19, 18, 437, 110, 110, 110, 0, 41, 0, 110, 438, 0, 439, 110,
    0, 0, 439, 110, 0, 0, 0, 126, 0, 440, 0, 441, 442, 0, 126, 0,
    126, 126, 126, 443, 444, 445, 446, 447, 448, 440, 449, 450, 294, 390, 451, 452,
    453, 454, 455, 456, 457, 457, 458, 147, 262, 459, 140, 460, 126, 262, 108, 108,
    262, 461, 262, 462, 462, 262, 463, 463, 262, 464, 464, 262, 465, 465, 364, 465,
    466, 467, 468, 469, 470, 208, 471, 0, 0, 148, 208, 472, 0, 0, 0, 0,
    0, 148, 36, 126, 35, 0, 0, 392, 473, 474, 475, 476, 477, 478, 479, 480,
    430, 481, 482, 483, 484, 0, 485, 486, 487, 488, 415, 435, 435, 435, 119, 436,
    436, 436, 434, 489, 490, 491, 492, 493, 482, 422, 494, 495, 141, 496, 497, 498,
    499, 500, 35, 148, 148, 148, 501, 35, 0, 0, 0, 35, 502, 503, 504, 214,
    327, 505, 506, 507, 35, 0, 0, 0, 35, 126, 508, 165, 89, 35, 509, 510,
    187, 187, 511, 503, 512, 375, 214, 0, 139, 0, 346, 346, 0, 139, 0, 200,
    0, 344, 513, 513, 513, 0, 513, 513, 0, 267, 514, 158, 159, 158, 157, 515,
    516, 516, 516, 0, 517, 517, 518, 519, 518, 519, 519, 516, 516, 0, 517, 517,
    518, 519, 518, 519, 519, 483, 36, 480, 516, 520, 521, 194, 522, 19, 523, 524,
    525, 526, 527, 528, 0, 529, 530, 531, 532, 533, 534, 535, 536, 537, 170, 538,
    539, 198, 540, 541, 542, 543, 544, 544, 544, 0, 245, 245, 245, 421, 0, 0,
    0, 200, 200, 0, 0, 0, 80, 0, 545, 546, 0, 0, 204, 204, 204, 547,
    521, 548, 520, 539, 181, 0, 262, 0, 0, 111, 549, 550, 551, 552, 553, 554,
    554, 554, 555, 147, 147, 0, 215, 215, 35, 41, 41, 211, 211, 214, 147, 126,
    215, 35, 41, 41, 357, 357, 358, 359, 366, 204, 204, 556, 521, 520, 0, 0,
    148, 0, 208, 205, 169, 165, 165, 166, 166, 557, 558, 559, 71, 71, 0, 35,
    0, 0, 0, 0, 560, 205, 204, 204, 0, 358, 561, 468, 505, 562, 563, 564,
    565, 566, 567, 568, 569, 570, 571, 0, 203, 0, 0, 111, 111, 111, 111, 111,
    111, 0, 0, 111, 111, 111, 111, 111, 80, 126, 200, 36, 522, 110, 35, 480,
    554, 572, 573, 0, 0, 433, 22, 574, 0, 0, 0, 0, 575, 36, 576, 577,
    578, 578, 262, 578, 578, 578, 80, 579, 579, 580, 580, 81, 82, 0, 194, 194,
    35, 0, 0, 126, 0, 203, 203, 0, 167, 167, 0, 80, 80, 15, 0, 14,
    14, 0, 174, 174, 0, 84, 84, 0, 35, 35, 256, 256, 0, 18, 18, 0,
    581, 581, 181, 182, 15, 17, 48, 305, 20, 238, 87, 25, 219, 350, 21, 75,
    22, 8, 8, 180, 183, 183, 75, 23, 23, 26, 27, 23, 23, 81, 89, 89,
    82, 186, 186, 187, 189, 186, 186, 187, 190, 186, 186, 187, 191, 186, 186, 187,
    192, 0, 80, 80, 0, 582, 582, 124, 85, 86, 0, 35, 0, 0, 583, 583,
    332, 262, 166, 0, 584, 585, 586, 483, 146, 436, 435, 436, 435, 587, 588, 583,
    583, 589, 38, 583, 583, 165, 589, 38, 415, 415, 415, 118, 590, 591, 592, 196,
    0, 0, 0, 35, 35, 0, 111, 132, 365, 225, 593, 506, 594, 595, 571, 419,
    596, 119, 119, 119, 382, 597, 598, 0, 0, 599, 0, 0, 600, 189, 601, 601,
    602, 258, 77, 77, 139, 0, 0, 0, 0, 146, 0, 0, 147, 35, 35, 0,
    0, 146, 242, 35, 0, 0, 0, 147, 35, 35, 0, 0, 146, 35, 509, 603,
    603, 0, 604, 605, 35, 35, 0, 0, 0, 0, 35, 0, 0, 204, 204, 0,
    0, 603, 603, 0, 80, 80, 167, 0, 63, 142, 142, 142, 143, 606, 41, 607,
    608, 609, 167, 610, 611, 612, 613, 614, 615, 616, 617, 618, 182, 35, 35, 0,
    36, 0, 619, 619, 619, 620, 620, 620, 0, 35, 38, 262, 621, 455, 622, 622,
    126, 262, 593, 623, 624, 625, 625, 126, 262, 626, 626, 126, 262, 330, 330, 364,
    627, 419, 419, 41, 628, 628, 126, 41, 596, 596, 629, 630, 80, 631, 496, 497,
    632, 632, 633, 80, 631, 496, 497, 634, 6, 7, 7, 126, 262, 489, 635, 635,
    126, 636, 364, 262, 637, 638, 638, 126, 639, 640, 641, 642, 643, 644, 645, 646,
    647, 648, 648, 126, 148, 166, 166, 126, 262, 649, 649, 41, 650, 651, 652, 652,
    126, 653, 364, 654, 655, 262, 656, 657, 658, 659, 660, 661, 286, 662, 663, 664,
    665, 43, 43, 494, 124, 0, 35, 0, 35, 43, 43, 194, 35, 0, 126, 0,
    666, 140, 140, 84, 0, 35, 246, 246, 84, 140, 140, 429, 126, 126, 126, 447,
    126, 126, 667, 43, 43, 181, 172, 0, 668, 35, 0, 0, 0, 0, 146, 262,
    0, 0, 146, 262, 0, 0, 146, 262, 0, 0, 146, 0, 0, 0, 669, 669,
    146, 141, 670, 670, 146, 262, 38, 38, 0, 43, 43, 14, 0, 671, 38, 140,
    140, 672, 38, 457, 673, 672, 135, 135, 146, 0, 142, 204, 674, 43, 43, 406,
    0, 675, 139, 413, 146, 141, 496, 497, 43, 43, 15, 0, 126, 126, 0, 204,
    204, 0, 147, 147, 0, 676, 676, 167, 167, 0, 168, 169, 165, 166, 170, 170,
    0, 43, 43, 174, 172, 0, 140, 18, 18, 0, 670, 670, 0, 38, 38, 0,
    158, 43, 43, 494, 90, 84, 0, 35, 0, 0, 269, 0, 271, 274, 273, 272,
    677, 678, 679, 262, 457, 458, 147, 262, 680, 680, 41, 43, 43, 90, 18, 172,
    0, 38, 681, 26, 26, 19, 682, 682, 683, 684, 685, 686, 687, 688, 689, 690,
    14, 14, 262, 622, 622, 262, 18, 18, 262, 691, 691, 262, 692, 692, 41, 457,
    457, 458, 147, 262, 452, 452, 693, 347, 694, 695, 696, 453, 453, 262, 697, 697,
    262, 524, 524, 262, 43, 43, 90, 14, 172, 165, 698, 43, 43, 90, 81, 0,
    699, 242, 242, 92, 700, 701, 702, 126, 703, 704, 705, 371, 262, 81, 81, 706,
    35, 262, 262, 262, 146, 262, 126, 509, 35, 509, 43, 43, 90, 81, 0, 35,
    0, 0, 0, 0, 699, 242, 242, 92, 707, 700, 700, 701, 708, 708, 262, 140,
    140, 709, 35, 262, 262, 262, 146, 262, 702, 126, 710, 704, 711, 712, 705, 713,
    467, 714, 714, 262, 715, 262, 371, 262, 81, 81, 706, 35, 262, 262, 262, 146,
    262, 43, 43, 23, 0, 716, 0, 717, 0, 43, 43, 23, 81, 43, 43, 43,
    172, 0, 35, 0, 0, 0, 0, 0, 260, 204, 204, 0, 718, 718, 719, 720,
    720, 35, 0, 0, 25, 25, 0, 260, 43, 43, 172, 0, 295, 295, 254, 0,
    35, 0, 0, 0, 0, 140, 140, 260, 718, 718, 719, 378, 156, 156, 379, 380,
    147, 25, 25, 0, 260, 43, 43, 494, 721, 84, 172, 0, 35, 269, 269, 0,
    81, 81, 35, 0, 0, 0, 0, 0, 0, 0, 0, 722, 722, 0, 0, 723,
    43, 43, 105, 0, 724, 140, 45, 0, 0, 43, 43, 558, 256, 382, 0, 172,
    0, 80, 263, 0, 200, 200, 0, 35, 0, 0, 725, 725, 35, 0, 0, 726,
    726, 165, 589, 140, 140, 35, 21, 21, 0, 603, 603, 0, 43, 43, 43, 494,
    727, 0, 17, 728, 38, 262, 729, 455, 622, 622, 126, 262, 43, 43, 89, 82,
    0, 469, 469, 730, 731, 241, 732, 364, 364, 352, 730, 731, 241, 126, 733, 509,
    35, 509, 734, 35, 81, 81, 0, 379, 735, 35, 49, 49, 0, 140, 43, 43,
    736, 0, 38, 38, 0, 262, 35, 509, 0, 737, 737, 0, 84, 84, 0, 35,
    0, 0, 269, 269, 0, 679, 679, 262, 457, 457, 458, 147, 262, 364, 364, 738,
    80, 631, 739, 739, 740, 741, 742, 743, 744, 744, 745, 746, 485, 747, 748, 748,
    262, 749, 749, 262, 750, 750, 147, 751, 751, 35, 0, 0, 752, 752, 35, 753,
    753, 262, 463, 463, 262, 754, 754, 738, 80, 631, 755, 755, 41, 756, 756, 41,
    43, 43, 196, 0, 0, 35, 0, 0, 0, 0, 0, 0, 146, 0, 0, 0,
    146, 0, 0, 0, 0, 146, 0, 0, 757, 757, 157, 758, 759, 85, 84, 146,
    0, 0, 0, 43, 43, 48, 0, 80, 0, 126, 126, 0, 0, 0, 156, 156,
    760, 28, 72, 65, 77, 64, 37, 37, 0, 0, 0, 107, 136, 137, 138, 200,
    200, 35, 0, 140, 140, 170, 35, 0, 43, 43, 494, 85, 0, 35, 761, 723,
    43, 43, 305, 0, 156, 156, 156, 28, 93, 156, 156, 307, 307, 140, 140, 35,
    43, 43, 762, 48, 0, 140, 140, 35, 0, 0, 0, 0, 0, 126, 0, 200,
    200, 35, 0, 43, 43, 161, 20, 45, 0, 763, 140, 764, 309, 310, 311, 312,
    313, 314, 315, 316, 317, 318, 253, 319, 320, 321, 322, 323, 324, 325, 326, 327,
    328, 142, 142, 143, 41, 329, 330, 331, 332, 262, 166, 171, 171, 333, 262, 30,
    30, 20, 0, 30, 30, 105, 0, 20, 20, 0, 120, 120, 0, 45, 45, 0,
    765, 765, 41, 43, 43, 18, 0, 0, 172, 0, 454, 454, 0, 454, 454, 0,
    335, 766, 126, 126, 0, 336, 336, 214, 337, 41, 338, 339, 339, 337, 41, 241,
    241, 339, 340, 341, 341, 337, 41, 241, 241, 341, 343, 343, 140, 337, 41, 344,
    344, 41, 345, 346, 241, 241, 346, 296, 296, 214, 0, 347, 0, 43, 43, 721,
    87, 172, 0, 766, 766, 43, 43, 25, 172, 0, 22, 22, 262, 670, 670, 0,
    23, 0, 260, 43, 43, 219, 0, 172, 0, 35, 0, 0, 0, 0, 0, 0,
    80, 0, 0, 0, 0, 38, 0, 43, 43, 350, 0, 142, 262, 305, 0, 0,
    140, 140, 35, 43, 43, 767, 768, 721, 74, 73, 81, 0, 35, 0, 0, 469,
    469, 730, 731, 241, 732, 126, 509, 35, 509, 43, 43, 768, 721, 81, 172, 0,
    35, 0, 0, 0, 0, 0, 769, 555, 205, 226, 770, 770, 0, 771, 771, 469,
    469, 730, 731, 241, 732, 732, 43, 43, 186, 187, 189, 0, 35, 772, 191, 191,
    146, 0, 620, 620, 146, 0, 43, 43, 494, 167, 75, 0, 773, 392, 123, 774,
    774, 254, 775, 775, 65, 64, 76, 77, 77, 254, 254, 0, 0, 494, 80, 494,
    80, 0, 776, 38, 38, 0, 777, 165, 165, 229, 778, 229, 230, 230, 230, 232,
    230, 230, 231, 779, 779, 780, 781, 782, 81, 126, 0, 780, 780, 781, 782, 81,
    126, 0, 156, 156, 783, 784, 242, 785, 517, 517, 35, 0, 49, 49, 759, 84,
    85, 0, 43, 43, 22, 0, 126, 126, 786, 787, 788, 35, 0, 0, 43, 43,
    45, 385, 43, 43, 172, 156, 0, 107, 0, 789, 25, 0, 260, 43, 43, 45,
    0, 140, 331, 400, 262, 235, 790, 312, 313, 314, 394, 395, 396, 397, 397, 8,
    398, 284, 399, 30, 30, 105, 0, 0, 171, 171, 333, 262, 142, 402, 402, 403,
    41, 43, 43, 494, 82, 172, 0, 35, 126, 509, 35, 509, 140, 313, 316, 395,
    460, 791, 548, 792, 793, 793, 794, 794, 793, 795, 35, 0, 796, 150, 797, 798,
    799, 800, 136, 801, 176, 802, 803, 804, 137, 805, 806, 807, 346, 623, 92, 92,
    126, 808, 792, 792, 809, 809, 126, 262, 810, 810, 811, 811, 364, 41, 812, 812,
    467, 813, 814, 126, 262, 704, 713, 711, 712, 26, 815, 14, 0, 0, 111, 549,
    816, 406, 817, 0, 0, 0, 0, 0, 818, 819, 820, 821, 822, 478, 823, 824,
    825, 826, 826, 827, 828, 829, 830, 830, 831, 832, 35, 833, 834, 834, 529, 531,
    835, 836, 531, 531, 837, 838, 35, 500, 186, 839, 472, 840, 495, 493, 841, 842,
    843, 844, 845, 845, 846, 847, 479, 479, 837, 35, 424, 0, 0, 111, 0, 848,
    849, 849, 35, 35, 0, 0, 850, 0, 851, 839, 262, 839, 848, 852, 853, 854,
    855, 856, 635, 852, 850, 857, 858, 859, 860, 861, 862, 863, 863, 863, 864, 0,
    865, 866, 831, 867, 867, 867, 0, 806, 156, 509, 147, 147, 147, 379, 868, 229,
    231, 869, 870, 279, 871, 148, 872, 872, 872, 146, 146, 871, 35, 872, 872, 873,
    874, 875, 64, 64, 64, 38, 263, 0, 132, 0, 200, 200, 0, 0, 0, 0,
    0, 603, 603, 603, 150, 254, 0, 603, 876, 876, 0, 132, 0, 0, 0, 0,
    700, 572, 625, 877, 563, 562, 878, 291, 512, 879, 880, 881, 881, 406, 0, 254,
    254, 9, 9, 262, 882, 262, 881, 883, 883, 883, 0, 0, 884, 381, 669, 885,
    190, 235, 886, 886, 886, 887, 28, 888, 99, 74, 73, 889, 890, 589, 359, 360,
    432, 891, 892, 893, 894, 895, 896, 897, 898, 899, 900, 901, 601, 601, 601, 902,
    902, 17, 262, 262, 262, 17, 903, 904, 15, 111, 826, 163, 163, 124, 172, 0,
    0, 0, 0, 35, 0, 132, 159, 159, 157, 158, 163, 163, 194, 172, 35, 0,
    126, 0, 159, 163, 163, 181, 172, 0, 159, 163, 163, 14, 172, 0, 905, 140,
    35, 240, 163, 163, 406, 0, 163, 163, 15, 172, 0, 0, 159, 163, 163, 174,
    172, 0, 159, 163, 163, 23, 81, 172, 0, 0, 159, 163, 163, 84, 172, 0,
    0, 0, 35, 159, 163, 163, 256, 382, 172, 0, 0, 159, 163, 163, 163, 89,
    82, 172, 0, 0, 159, 163, 163, 736, 172, 0, 163, 163, 296, 214, 0, 0,
    136, 137, 138, 0, 159, 163, 163, 48, 172, 0, 80, 0, 159, 163, 163, 85,
    172, 0, 0, 0, 35, 163, 163, 305, 172, 0, 0, 159, 163, 163, 762, 48,
    172, 0, 159, 158, 157, 163, 163, 20, 172, 0, 163, 163, 18, 172, 0, 159,
    163, 163, 87, 172, 0, 0, 159, 163, 163, 219, 172, 0, 0, 159, 158, 157,
    163, 163, 350, 172, 0, 159, 158, 157, 163, 163, 721, 81, 172, 0, 0, 159,
    163, 163, 186, 187, 189, 172, 0, 159, 163, 163, 75, 172, 0, 80, 0, 159,
    163, 163, 22, 172, 0, 0, 159, 163, 163, 45, 172, 0, 163, 163, 82, 172,
    0, 0, 159, 906, 907, 908, 909, 783, 910, 518, 518, 0, 518, 911, 35, 0,
    0, 35, 637, 234, 357, 638, 912, 330, 913, 914, 915, 519, 916, 917, 918, 787,
    919, 655, 654, 920, 146, 921, 644, 362, 183, 922, 751, 923, 924, 924, 0, 35,
    0, 0, 0, 924, 925, 0, 0, 140, 483, 926, 927, 49, 49, 49, 0, 35,
    0, 0, 54, 698, 928, 929, 0, 0, 0, 418, 416, 421, 662, 752, 504, 0,
    0, 0, 141, 92, 92, 930, 186, 931, 813, 932, 929, 929, 933, 934, 417, 418,
    416, 421, 35, 0, 0, 509, 509, 65, 77, 64, 935, 645, 210, 0, 0, 0,
    0, 0, 936, 480, 480, 0, 937, 483, 583, 583, 165, 938, 939, 940, 0, 941,
    165, 477, 0, 422, 0, 0, 0, 0, 942, 337, 337, 337, 943, 165, 944, 0,
    0, 945, 945, 945, 945, 430, 481, 483, 0, 946, 337, 337, 337, 943, 0, 947,
    0, 0, 698, 698, 35, 509, 0, 0, 0, 516, 440, 440, 0, 36, 0, 948,
    0, 0, 0, 208, 0, 35, 0, 0, 0, 509, 0, 0, 148, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 35, 0, 0, 0, 0, 949, 262, 41, 141,
    166, 851, 0, 0, 36, 109, 0, 72, 148, 472, 0, 0, 0, 0, 223, 0,
    0, 0, 263, 263, 0, 35, 0, 0, 0, 132, 174, 90, 950, 497, 497, 199,
    951, 951, 952, 375, 953, 0, 41, 0, 242, 954, 242, 287, 955, 263, 140, 140,
    956, 951, 951, 951, 957, 953, 28, 958, 958, 959, 28, 41, 38, 0, 960, 91,
    961, 962, 963, 964, 965, 965, 572, 572, 646, 370, 966, 966, 35, 603, 603, 0,
    966, 967, 383, 968, 375, 969, 626, 970, 825, 828, 829, 971, 972, 156, 156, 973,
    77, 241, 241, 363, 77, 106, 363, 363, 106, 254, 254, 0, 0, 974, 297, 353,
    353, 975, 156, 240, 23, 23, 26, 27, 23, 23, 363, 23, 718, 167, 167, 0,
    976, 169, 165, 166, 170, 170, 35, 0, 241, 241, 253, 147, 147, 0, 0, 148,
    0, 208, 0, 211, 211, 213, 126, 0, 702, 719, 167, 167, 0, 170, 170, 35,
    0, 241, 241, 253, 92, 977, 958, 978, 979, 979, 0, 0, 150, 150, 242, 0,
    0, 41, 472, 980, 0, 981, 982, 972, 477, 477, 422, 924, 924, 983, 150, 150,
    242, 105, 956, 241, 241, 984, 979, 788, 287, 985, 986, 987, 988, 38, 38, 0,
    0, 35, 0, 989, 603, 143, 105, 105, 214, 125, 125, 214, 990, 991, 992, 993,
    140, 140, 951, 994, 156, 673, 140, 140, 995, 38, 406, 0, 254, 9, 9, 0,
    262, 41, 0, 882, 0, 262, 0, 0, 0, 996, 0, 35, 0, 0, 0, 0,
    989, 997, 573, 0, 140, 140, 994, 0, 997, 0, 35, 0, 0, 0, 0, 35,
    0, 0, 998, 0, 35, 0, 125, 125, 214, 105, 105, 214, 0, 0, 0, 0,
    0, 573, 125, 125, 214, 105, 105, 214, 0, 38, 38, 16, 262, 14, 14, 262,
    999, 139, 7, 7, 146, 141, 1000, 1000, 146, 141, 1001, 1001, 41, 657, 657, 146,
    141, 1002, 1002, 146, 141, 38, 1003, 1003, 35, 0, 0, 35, 0, 0, 156, 0,
    30, 35, 0, 0, 0, 0, 1004, 1004, 1005, 1005, 725, 924, 726, 35, 0, 0,
    156, 0, 30, 35, 0, 0, 0, 0, 1006, 1007, 257, 948, 0, 0, 0, 1008,
    1008, 0, 0, 0, 0, 0, 502, 35, 502, 0, 0, 126, 0, 1009, 1009, 0,
    38, 38, 0, 0, 0, 850, 0, 1010, 38, 38, 0, 156, 156, 0, 876, 876,
    0, 0, 589, 0, 132, 0, 0, 125, 1011, 1011, 148, 0, 0, 262, 0, 0,
    0, 30, 0, 262, 0, 0, 0, 1012, 0, 0, 0, 0, 0, 1013, 388, 554,
    0, 35, 35, 0, 723, 1014, 17, 201, 201, 156, 22, 0, 992, 271, 272, 273,
    274, 275, 165, 276, 266, 267, 267, 80, 166, 268, 269, 269, 166, 80, 166, 268,
    270, 277, 90, 244, 278, 280, 279, 281, 281, 282, 283, 284, 285, 286, 286, 41,
    287, 287, 41, 288, 288, 0, 139, 139, 0, 144, 145, 145, 37, 146, 0, 0,
    0, 148, 38, 38, 150, 126, 126, 262, 262, 262, 262, 36, 262, 1015, 1015, 0,
    473, 473, 450, 1016, 65, 65, 572, 77, 77, 572, 1017, 1017, 140, 502, 72, 572,
    1015, 1015, 0, 572, 519, 140, 140, 35, 0, 0, 84, 0, 0, 0, 0, 0,
    111, 35, 0, 0, 149, 35, 0, 1018, 30, 428, 429, 1008, 0, 0, 0, 0,
    1018, 30, 428, 429, 1008, 0, 0, 0, 1019, 1019, 147, 146, 947, 0, 35, 0,
    0, 0, 849, 849, 849, 35, 0, 0, 0, 0, 1020, 849, 379, 619, 425, 425,
    0, 1020, 849, 379, 619, 35, 0, 619, 619, 35, 96, 1021, 31, 1022, 1023, 1024,
    211, 844, 843, 1025, 1026, 1027, 1028, 1029, 1030, 1031, 103, 103, 103, 0, 42, 42,
    80, 0, 20, 20, 0, 68, 68, 80, 45, 0, 0, 37, 37, 37, 105, 105,
    37, 37, 0, 36, 36, 36, 37, 0, 1032, 140, 140, 103, 154, 140, 140, 120,
    154, 620, 620, 150, 52, 1033, 1033, 214, 0, 137, 138, 0, 102, 102, 103, 104,
    55, 80, 406, 0, 1034, 37, 38, 1035, 882, 882, 262, 9, 9, 262, 262, 1036,
    1036, 26, 19, 1033, 52, 436, 436, 146, 436, 1037, 105, 105, 214, 0, 0, 28,
    0, 41, 794, 1038, 0, 35, 125, 125, 214, 143, 990, 991, 992, 993, 1039, 125,
    125, 214, 143, 990, 991, 992, 951, 105, 0, 0, 0, 0, 41, 0, 35, 1039,
    1038, 1038, 1038, 1040, 35, 0, 0, 794, 0, 985, 594, 716, 1041, 990, 990, 0,
    35, 0, 1042, 125, 125, 214, 143, 991, 992, 951, 990, 1043, 1044, 1045, 1002, 682,
    841, 1046, 0, 1047, 750, 694, 457, 457, 458, 147, 262, 457, 1048, 0, 215, 172,
    172, 698, 172, 172, 172, 165, 698, 1049, 1012, 1012, 1049, 1050, 726, 647, 369, 648,
    1051, 232, 1052, 272, 1053, 667, 1054, 1054, 23, 18, 1054, 1054, 0, 1055, 1055, 143,
    143, 37, 751, 35, 872, 872, 38, 25, 0, 1056, 0, 260, 1054, 61, 30, 937,
    165, 30, 0, 35, 0, 0, 0, 509, 0, 725, 414, 1057, 336, 1058, 0, 0,
    0, 1059, 443, 881, 0, 204, 204, 0, 0, 0, 726, 726, 1060, 375, 1061, 35,
    1059, 443, 881, 726, 726, 1060, 375, 1061, 0, 0, 0, 0, 35, 0, 0, 256,
    1062, 725, 725, 35, 140, 140, 35, 200, 200, 256, 21, 0, 1063, 1064, 253, 720,
    1065, 230, 1066, 1067, 429, 268, 714, 1068, 65, 65, 65, 1056, 0, 1069, 509, 35,
    509, 147, 147, 379, 509, 35, 509, 379, 509, 35, 509, 1070, 1071, 774, 819, 822,
    823, 821, 820, 16, 1072, 1073, 1074, 1075, 472, 472, 1076, 41, 41, 1077, 262, 850,
    262, 262, 262, 1077, 262, 850, 472, 1078, 1079, 1080, 859, 1047, 1081, 1082, 1056, 1056,
    1083, 74, 73, 251, 81, 0, 1056, 280, 938, 166, 1084, 1085, 141, 1086, 813, 583,
    583, 165, 589, 38, 583, 939, 939, 674, 674, 674, 243, 1087, 1088, 1089, 0, 0,
    0, 758, 0, 0, 0, 0, 0, 0, 1090, 80, 200, 0, 0, 0, 758, 0,
    0, 0, 0, 0, 758, 758, 758, 573, 573, 1003, 1004, 0, 80, 80, 0, 0,
    35, 0, 0, 599, 1067, 1091, 1091, 257, 1092, 1093, 1094, 200, 200, 35, 0, 0,
    813, 484, 813, 813, 1095, 666, 1095, 388, 1096, 439, 40, 666, 666, 140, 246, 558,
    1097, 1097, 1098, 1099, 1100, 1101, 1102, 1103, 1104, 1105, 1106, 1107, 689, 1108, 363, 1109,
    1110, 1097, 0, 686, 743, 463, 1111, 1112, 678, 17, 17, 0, 0, 1113, 17, 39,
    1061, 1114, 0, 0, 0, 996, 1115, 1116, 276, 1117, 1092, 1118, 435, 435, 146, 435,
    243, 727, 936, 936, 165, 1119, 0, 644, 644, 0, 1119, 770, 1051, 1120, 1120, 37,
    770, 588, 771, 771, 1057, 925, 868, 1121, 143, 143, 0, 35, 0, 41, 0, 28,
    1122, 994, 994, 0, 41, 0, 36, 36, 41, 143, 1123, 1124, 0, 1125, 1126, 1127,
    1127, 1127, 0, 148, 262, 41, 141, 337, 166, 851, 110, 1128, 1129, 501, 904, 0,
    0, 1130, 1131, 1131, 1131, 262, 0, 768, 1132, 1133, 371, 1134, 1135, 1135, 1135, 81,
    0, 0, 975, 1136, 1137, 250, 321, 1138, 1139, 1140, 564, 595, 1141, 998, 998, 111,
    0, 35, 736, 736, 0, 36, 36, 35, 509, 224, 51, 1142, 1142, 294, 381, 762,
    48, 0, 80, 81, 0, 0, 1034, 37, 38, 0, 0, 0, 20, 0, 107, 627,
    1143, 0, 385, 47, 464, 0, 1144, 1145, 1146, 1147, 1148, 89, 501, 501, 755, 756,
    628, 1149, 341, 1150, 1151, 1152, 64, 65, 65, 35, 379, 1153, 1153, 1056, 1153, 517,
    517, 517, 517, 517, 1147, 422, 422, 0, 1154, 1155, 77, 64, 517, 517, 165, 1147,
    214, 1156, 422, 422, 0, 1154, 1157, 77, 64, 601, 601, 258, 517, 517, 165, 1147,
    422, 422, 0, 1154, 65, 1153, 1153, 0, 0, 0, 200, 0, 0, 0, 35, 35,
    509, 0, 0, 0, 0, 0, 80, 0, 1158, 517, 409, 409, 906, 77, 77, 139,
    1159, 1160, 1161, 1162, 509, 509, 509, 509, 1162, 683, 180, 1163, 1164, 1165, 118, 1081,
    339, 998, 742, 1166, 1167, 736, 739, 1168, 66, 1169, 433, 1170, 12, 1080, 1171, 566,
    567, 565, 1172, 687, 695, 747, 1173, 1173, 1173, 132, 1174, 1175, 685, 872, 0, 342,
    0, 0, 1176, 1177, 947, 146, 0, 0, 0, 126, 0, 0, 35, 0, 992, 992,
    0, 0, 1178, 882, 1084, 1084, 1179, 1180, 1180, 599, 1180, 835, 1181, 118, 0, 1182,
    1183, 782, 0, 953, 1184, 1185, 1186, 1187, 1188, 241, 328, 1189, 314, 317, 396, 1190,
    319, 634, 322, 502, 379, 72, 35, 72, 35, 140, 35, 502, 35, 0, 1191, 1192,
    502, 1193, 35, 72, 502, 77, 65, 64, 761, 347, 1194, 325, 1195, 1086, 1196, 1184,
    1185, 1186, 1187, 1197, 1197, 311, 0, 587, 1198, 1198, 146, 435, 435, 436, 415, 415,
    435, 415, 415, 436, 165, 940, 165, 258, 1199, 1199, 1199, 0, 36, 36, 262, 940,
    297, 1200, 1201, 166, 166, 1202, 1203, 726, 1204, 832, 41, 838, 40, 262, 262, 554,
    0, 1129, 262, 262, 1129, 111, 111, 111, 111, 111, 549, 80, 126, 200, 36, 522,
    110, 35, 480, 554, 572, 573, 1004, 1004, 0, 126, 0, 440, 0, 748, 1205, 1206,
    1207, 254, 631, 1208, 994, 994, 0, 41, 0, 994, 1209, 248, 781, 1210, 80, 80,
    1211, 263, 1212, 67, 196, 154, 260, 260, 35, 1213, 1213, 260, 1213, 1213, 1213, 872,
    262, 494, 494, 494, 494, 386, 713, 125, 125, 214, 0, 0, 0, 0, 41, 0,
    1214, 443, 881, 200, 200, 483, 726, 726, 1060, 375, 1061, 0, 143, 979, 1215, 143,
    990, 991, 992, 979, 1011, 140, 140, 26, 156, 125, 1216, 125, 1194, 1217, 1218, 984,
    428, 1094, 1219, 715, 0, 1012, 1220, 0, 0, 35, 0, 1220, 1221, 415, 415, 415,
    0, 0, 35, 35, 0, 0, 851, 0, 0, 36, 0, 0, 851, 0, 0, 0,
    1222, 0, 35, 35, 0, 0, 851, 0, 0, 36, 0, 0, 851, 0, 0, 0,
    1222, 1223, 1224, 1225, 296, 135, 68, 1226, 1227, 281, 1228, 0, 1229, 1230, 1231, 1232,
    1233, 1234, 1235, 704, 191, 1236, 380, 30, 30, 35, 0, 0, 38, 38, 35, 0,
    0, 0, 339, 341, 37, 35, 0, 0, 0, 339, 341, 140, 140, 35, 1170, 1170,
    41, 36, 1237, 41, 295, 295, 214, 0, 35, 0, 0, 295, 699, 699, 214, 35,
    0, 0, 0, 0, 699, 1238, 1239, 1240, 1241, 331, 1242, 1243, 1244, 432, 432, 872,
    872, 432, 262, 1245, 0, 1246, 1247, 1248, 1249, 1250, 1251, 1252, 995, 1253, 592, 591,
    58, 119, 1003, 1254, 222, 1255, 888, 1256, 728, 431, 48, 692, 1257, 1258, 1259, 431,
    0, 1260, 0, 966, 0, 590, 962, 962, 962, 684, 1261, 996, 996, 1262, 982, 827,
    864, 864, 864, 0, 35, 0, 0, 126, 509, 1263, 749, 1264, 630, 559, 559, 363,
    559, 944, 1265, 62, 744, 185, 85, 35, 0, 0, 0, 0, 0, 0, 1266, 0,
    427, 427, 97, 1267, 706, 872, 1268, 963, 963, 963, 964, 964, 964, 554, 1269, 1269,
    35, 554, 554, 0, 0, 262, 1270, 107, 413, 305, 307, 81, 81, 0, 0, 36,
    36, 307, 81, 93, 93, 93, 30, 18, 0, 0, 0, 176, 36, 36, 307, 81,
    93, 0, 0, 0, 35, 603, 603, 35, 0, 337, 337, 41, 196, 196, 35, 0,
    0, 111, 0, 0, 0, 0, 111, 111, 0, 991, 991, 0, 991, 483, 483, 1271,
    28, 423, 1272, 35, 509, 1273, 1000, 335, 335, 606, 41, 41, 1274, 1150, 1023, 1275,
    607, 335, 809, 149, 1276, 1277, 262, 1278, 1278, 1113, 1279, 1280, 1173, 1113, 1281, 1281,
    0, 1173, 0, 1282, 1282, 0, 1173, 0, 1283, 1283, 0, 1284, 1284, 1285, 1284, 1173,
    1285, 1286, 788, 1287, 146, 436, 561, 435, 1198, 711, 69, 649, 1288, 824, 1289, 1289,
    1289, 296, 214, 0, 0, 136, 137, 138, 0, 37, 0, 136, 137, 138, 0, 1290,
    1291, 1292, 1293, 1294, 0, 45, 45, 0, 20, 20, 0, 0, 107, 105, 105, 0,
    688, 1295, 1266, 71, 71, 0, 35, 1144, 80, 80, 80, 64, 1296, 80, 80, 77,
    1296, 1144, 1144, 1297, 257, 1236, 1298, 70, 71, 779, 1296, 158, 139, 139, 1299, 147,
    166, 35, 0, 0, 157, 241, 241, 114, 352, 352, 352, 89, 82, 382, 0, 140,
    1001, 622, 1300, 1301, 1302, 1303, 1304, 1305, 1306, 1307, 1308, 746, 1309, 1309, 1309, 35,
    1310, 1310, 382, 382, 141, 21, 21, 0, 421, 421, 141, 1311, 256, 256, 382, 0,
    81, 81, 382, 0, 18, 18, 382, 0, 1312, 1312, 14, 354, 382, 0, 1313, 872,
    872, 111, 111, 872, 146, 146, 871, 35, 872, 872, 1314, 1315, 1237, 409, 650, 134,
    1316, 386, 237, 324, 1317, 151, 151, 151, 402, 454, 151, 151, 0, 28, 20, 0,
    1318, 11, 1050, 218, 1319, 157, 284, 1320, 680, 1015, 1321, 1322, 1323, 1324, 876, 269,
    104, 104, 104, 103, 103, 154, 156, 0, 42, 42, 80, 0, 20, 20, 0, 68,
    68, 80, 45, 0, 0, 38, 0, 159, 1177, 1093, 1325, 762, 20, 0, 1326, 795,
    795, 651, 86, 1327, 1328, 1329, 98, 701, 960, 960, 1330, 214, 262, 1123, 1123, 1330,
    214, 262, 242, 1331, 1332, 1333, 287, 238, 568, 1334, 753, 754, 285, 679, 997, 997,
    1335, 476, 354, 35, 41, 1316, 1316, 1316, 35, 41, 1336, 1096, 1337, 18, 1338, 29,
    1339, 1340, 1341, 1342, 1343, 187, 1082, 461, 697, 1344, 244, 244, 244, 1111, 156, 156,
    0, 80, 1345, 81, 81, 0, 203, 203, 0, 124, 124, 0, 35, 35, 14, 14,
    0, 15, 15, 0, 183, 183, 75, 0, 23, 23, 81, 0, 84, 84, 0, 0,
    35, 0, 39, 39, 40, 41, 89, 89, 82, 0, 184, 185, 17, 0, 85, 85,
    0, 0, 35, 0, 305, 305, 0, 20, 20, 0, 86, 86, 0, 0, 35, 0,
    18, 18, 0, 87, 87, 0, 350, 350, 0, 21, 21, 0, 22, 22, 0, 82,
    82, 0, 583, 332, 262, 166, 1346, 0, 0, 0, 0, 0, 1347, 589, 589, 80,
    80, 35, 0, 0, 28, 1348, 1348, 41, 72, 72, 1347, 0, 0, 1056, 38, 603,
    125, 125, 214, 80, 80, 604, 143, 876, 1222, 993, 140, 140, 241, 26, 140, 1162,
    1349, 1056, 38, 0, 603, 125, 125, 214, 143, 876, 36, 993, 1162, 72, 72, 72,
    1056, 38, 603, 105, 990, 0, 992, 992, 991, 125, 143, 994, 951, 156, 993, 140,
    140, 80, 80, 604, 876, 140, 1162, 993, 509, 905, 35, 140, 905, 35, 140, 1192,
    35, 509, 1350, 1351, 111, 111, 0, 110, 109, 111, 111, 0, 110, 572, 1352, 1011,
    87, 340, 341, 341, 41, 241, 241, 341, 338, 339, 339, 41, 241, 241, 339, 712,
    99, 708, 709, 1088, 25, 1353, 789, 45, 136, 137, 138, 402, 462, 138, 139, 139,
    0, 0, 0, 0, 0, 0, 110, 146, 0, 148, 208, 0, 0, 0, 147, 0,
    0, 35, 35, 0, 0, 146, 35, 0, 509, 0, 0, 0, 148, 208, 0, 0,
    0, 147, 0, 0, 139, 139, 613, 0, 28, 0, 0, 1354, 231, 232, 502, 502,
    993, 993, 993, 502, 125, 143, 992, 139, 993, 1355, 1074, 751, 1165, 139, 139, 18,
    172, 0, 139, 139, 1356, 1356, 768, 402, 402, 402, 402, 454, 0, 0, 147, 139,
    139, 1356, 402, 768, 0, 0, 0, 145, 37, 146, 0, 147, 0, 0, 148, 0,
    149, 80, 1357, 1357, 1358, 1359, 262, 768, 147, 18, 18, 262, 20, 20, 0, 347,
    0, 722, 1040, 752, 810, 106, 0, 0, 0, 516, 522, 522, 522, 0, 440, 1360,
    1360, 817, 770, 770, 262, 1360, 1360, 1360, 736, 0, 28, 0, 1360, 1360, 0, 28,
    0, 0, 0, 0, 0, 0, 337, 431, 118, 1120, 41, 41, 1298, 811, 1361, 0,
    0, 149, 872, 0, 0, 149, 111, 0, 0, 0, 0, 111, 132, 1113, 1362, 1363,
    439, 439, 110, 353, 432, 554, 818, 1089, 1272, 1364, 366, 1156, 1365, 1366, 1367, 1368,
    1369, 1370, 1371, 1372, 1373, 1374, 1375, 273, 83, 95, 1376, 132, 343, 1377, 785, 965,
    509, 1153, 65, 1378, 163, 163, 43, 163, 76, 102, 43, 43, 163, 131, 43, 139,
    163, 163, 43, 163, 43, 163, 43, 64, 77, 43, 163, 43, 163, 43, 163, 883,
    59, 929, 43, 131, 163, 43, 163, 131, 163, 1289, 578, 417, 43, 131, 139, 131,
    131, 43, 163, 43, 131, 163, 43, 131, 163, 352, 131, 163, 1379, 131, 43, 131,
    1127, 1199, 1380, 1131, 151, 886, 131, 43, 163, 620, 601, 131, 1309, 131, 43, 163,
    131, 43, 131, 163, 43, 131, 163, 43, 131, 43, 131, 163, 131, 1135, 245, 43,
    163, 513, 43, 43, 1054, 43, 163, 43, 131, 43, 131, 163, 139, 43, 509, 163,
    43, 131, 244, 864, 49, 863, 139, 151, 43, 43, 131, 131, 43, 43, 43, 623,
    1381, 620, 1360, 43, 881, 163, 1360, 131, 867, 951, 544, 417, 131, 513, 43, 1142,
    163, 43, 102, 572, 1382, 219, 632, 1383, 1384, 1385, 1386, 220, 702, 88, 100, 270,
    481, 119, 204, 1387, 1388, 35, 0, 0, 0, 0, 0, 1389, 1390, 1391, 1392, 350,
    1393, 1394, 1395, 1396, 1397, 842, 310, 288, 0, 1398, 1399, 765, 397, 354, 1400, 1401,
    1402, 732, 360, 360, 361, 364, 362, 363, 28, 359, 359, 35, 0, 1403, 1403, 1404,
    365, 1073, 366, 35, 0, 0, 366, 366, 364, 357, 357, 364, 226, 167, 167, 0,
    1405, 558, 558, 35, 0, 0, 559, 559, 363, 35, 0, 0, 23, 23, 363, 35,
    0, 0, 71, 0, 35, 0, 0, 227, 167, 167, 0, 557, 557, 558, 559, 200,
    256, 0, 228, 229, 165, 165, 229, 230, 230, 230, 231, 230, 230, 232, 0, 0,
    379, 208, 0, 1004, 0, 0, 35, 509, 35, 1153, 81, 1406, 93, 997, 21, 1407,
    0, 942, 942, 480, 0, 0, 946, 946, 0, 1408, 1409, 737, 50, 101, 1275, 1410,
    73, 192, 0, 721, 73, 74, 74, 1411, 1412, 721, 1330, 569, 570, 186, 705, 1154,
    1413, 956, 943, 1357, 853, 853, 1357, 1414, 35, 666, 666, 140, 246, 1357, 1358, 440,
    440, 440, 1357, 1358, 0, 652, 403, 1414, 856, 855, 1415, 1416, 37, 1417, 1348, 1418,
    1419, 1420, 1378, 1378, 473, 343, 585, 519, 1326, 2, 1421, 1420, 1422, 674, 674, 243,
    1423, 1088, 69, 69, 586, 69, 69, 1424, 1316, 1316, 69, 69, 69, 69, 263, 69,
    69, 79, 1425, 1426, 1427, 201, 1428, 1412, 1429, 1430, 1431, 1432, 75, 1433, 1434, 1435,
    1436, 1009, 499, 1116, 1419, 1437, 656, 496, 496, 76, 76, 76, 81, 0, 0, 0,
    263, 0, 0, 0, 132, 1438, 1438, 218, 617, 215, 159, 657, 1439, 35, 0, 0,
    22, 1128, 1008, 1008, 0, 0, 947, 439, 110, 0, 0, 35, 160, 1283, 980, 1424,
    791, 0, 1180, 791, 0, 1440, 1441, 1442, 1355, 1355, 1443, 589, 28, 1355, 558, 558,
    558, 1085, 1085, 607, 1444, 1380, 1380, 1380, 0, 0, 132, 1381, 1381, 1381, 35, 262,
    37, 262, 1445, 455, 622, 622, 126, 262, 593, 1446, 92, 92, 126, 1447, 792, 813,
    704, 295, 295, 214, 35, 725, 281, 281, 629, 630, 80, 631, 496, 497, 174, 174,
    262, 281, 281, 633, 80, 631, 496, 497, 624, 625, 625, 126, 262, 626, 626, 126,
    262, 627, 627, 421, 166, 166, 126, 262, 166, 166, 126, 262, 1448, 1449, 1149, 1333,
    126, 41, 1450, 1150, 1023, 330, 330, 364, 129, 542, 644, 117, 1043, 251, 1451, 1452,
    1453, 72, 72, 38, 603, 105, 990, 991, 0, 992, 993, 140, 140, 125, 143, 502,
    502, 993, 993, 993, 502, 1004, 399, 283, 1454, 333, 1455, 77, 77, 77, 263, 0,
    0, 139, 139, 38, 603, 603, 603, 150, 254, 0, 603, 876, 876, 0, 132, 0,
    0, 0, 0, 1456, 691, 55, 59, 59, 59, 0, 78, 45, 0, 1457, 1458, 1459,
    1460, 1461, 200, 641, 642, 643, 1462, 640, 1379, 1379, 1379, 1463, 257, 1309, 1291, 1464,
    1464, 1364, 64, 263, 0, 37, 37, 41, 1247, 498, 171, 173, 717, 388, 1465, 379,
    379, 35, 35, 0, 0, 0, 147, 0, 125, 143, 379, 1466, 1467, 847, 846, 836,
    1008, 837, 421, 0, 262, 0, 670, 1468, 1036, 82, 94, 274, 1469, 947, 13, 1470,
    1471, 0, 517, 517, 0, 0, 0, 1154, 0, 0, 0, 0, 517, 1472, 603, 603,
    0, 35, 0, 1473, 143, 105, 105, 214, 125, 125, 214, 990, 991, 992, 1222, 876,
    80, 80, 604, 156, 603, 1474, 0, 1475, 0, 0, 924, 0, 0, 0, 0, 0,
    111, 132, 132, 111, 132, 992, 0, 0, 1476, 295, 125, 295, 125, 979, 140, 140,
    1477, 515, 768, 170, 154, 140, 140, 1294, 0, 0, 0, 72, 0, 1191, 140, 140,
    1294, 0, 0, 0, 72, 0, 1192, 140, 140, 241, 241, 26, 0, 140, 140, 140,
    241, 18, 486, 1260, 1260, 105, 35, 125, 1260, 246, 0, 1422, 27, 79, 741, 931,
    1478, 1479, 1480, 1481, 1482, 1483, 1484, 1485, 1486, 1487, 1488, 1489, 1490, 1491, 1492, 1493,
    1494, 1495, 1496, 1497, 1498, 1499, 932, 1079, 731, 1358, 571,
];

pub fn get_redshift_segment_grammar(name: &str) -> Option<RootGrammar> {
    match name {
        "AKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(0), tables: &REDSHIFT_TABLES }),
        "AbortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1), tables: &REDSHIFT_TABLES }),
        "AbsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2), tables: &REDSHIFT_TABLES }),
        "AbsentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3), tables: &REDSHIFT_TABLES }),
        "AbsoluteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4), tables: &REDSHIFT_TABLES }),
        "AcceptanydateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5), tables: &REDSHIFT_TABLES }),
        "AcceptinvcharsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6), tables: &REDSHIFT_TABLES }),
        "AccessKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7), tables: &REDSHIFT_TABLES }),
        "AccessObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(8), tables: &REDSHIFT_TABLES }),
        "AccessPermissionSegment" => Some(RootGrammar { grammar_id: GrammarId(69), tables: &REDSHIFT_TABLES }),
        "AccessPermissionsSegment" => Some(RootGrammar { grammar_id: GrammarId(138), tables: &REDSHIFT_TABLES }),
        "AccessSchemaObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(141), tables: &REDSHIFT_TABLES }),
        "AccessSchemaPluralObjectSegment" => Some(RootGrammar { grammar_id: GrammarId(164), tables: &REDSHIFT_TABLES }),
        "AccessStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(187), tables: &REDSHIFT_TABLES }),
        "AccessTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(190), tables: &REDSHIFT_TABLES }),
        "Access_key_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(212), tables: &REDSHIFT_TABLES }),
        "AccessorGrammar" => Some(RootGrammar { grammar_id: GrammarId(213), tables: &REDSHIFT_TABLES }),
        "AccordingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(216), tables: &REDSHIFT_TABLES }),
        "AccountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(217), tables: &REDSHIFT_TABLES }),
        "AcosKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(218), tables: &REDSHIFT_TABLES }),
        "ActionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(219), tables: &REDSHIFT_TABLES }),
        "AdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(220), tables: &REDSHIFT_TABLES }),
        "AddKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(221), tables: &REDSHIFT_TABLES }),
        "AddquotesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(222), tables: &REDSHIFT_TABLES }),
        "AdjacentSegment" => Some(RootGrammar { grammar_id: GrammarId(223), tables: &REDSHIFT_TABLES }),
        "AdminKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(227), tables: &REDSHIFT_TABLES }),
        "Aes128KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(228), tables: &REDSHIFT_TABLES }),
        "Aes256KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(229), tables: &REDSHIFT_TABLES }),
        "AfterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(230), tables: &REDSHIFT_TABLES }),
        "AggregateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(231), tables: &REDSHIFT_TABLES }),
        "AggregateOrderByClause" => Some(RootGrammar { grammar_id: GrammarId(232), tables: &REDSHIFT_TABLES }),
        "AliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(233), tables: &REDSHIFT_TABLES }),
        "AliasedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(251), tables: &REDSHIFT_TABLES }),
        "AllKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(254), tables: &REDSHIFT_TABLES }),
        "AllocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(255), tables: &REDSHIFT_TABLES }),
        "AllowoverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(256), tables: &REDSHIFT_TABLES }),
        "AlsoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(257), tables: &REDSHIFT_TABLES }),
        "AlterAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(258), tables: &REDSHIFT_TABLES }),
        "AlterDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(284), tables: &REDSHIFT_TABLES }),
        "AlterDefaultPrivilegesGrantSegment" => Some(RootGrammar { grammar_id: GrammarId(339), tables: &REDSHIFT_TABLES }),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some(RootGrammar { grammar_id: GrammarId(352), tables: &REDSHIFT_TABLES }),
        "AlterDefaultPrivilegesRevokeSegment" => Some(RootGrammar { grammar_id: GrammarId(369), tables: &REDSHIFT_TABLES }),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(384), tables: &REDSHIFT_TABLES }),
        "AlterDefaultPrivilegesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(393), tables: &REDSHIFT_TABLES }),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some(RootGrammar { grammar_id: GrammarId(417), tables: &REDSHIFT_TABLES }),
        "AlterDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(422), tables: &REDSHIFT_TABLES }),
        "AlterExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(489), tables: &REDSHIFT_TABLES }),
        "AlterForeignTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(597), tables: &REDSHIFT_TABLES }),
        "AlterForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(852), tables: &REDSHIFT_TABLES }),
        "AlterFunctionActionSegment" => Some(RootGrammar { grammar_id: GrammarId(874), tables: &REDSHIFT_TABLES }),
        "AlterFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(938), tables: &REDSHIFT_TABLES }),
        "AlterGroupStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(971), tables: &REDSHIFT_TABLES }),
        "AlterIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(987), tables: &REDSHIFT_TABLES }),
        "AlterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(1048), tables: &REDSHIFT_TABLES }),
        "AlterMaterializedViewActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1049), tables: &REDSHIFT_TABLES }),
        "AlterMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1114), tables: &REDSHIFT_TABLES }),
        "AlterPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1162), tables: &REDSHIFT_TABLES }),
        "AlterProcedureActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1192), tables: &REDSHIFT_TABLES }),
        "AlterProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1221), tables: &REDSHIFT_TABLES }),
        "AlterPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1240), tables: &REDSHIFT_TABLES }),
        "AlterRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1272), tables: &REDSHIFT_TABLES }),
        "AlterSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1362), tables: &REDSHIFT_TABLES }),
        "AlterSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(1376), tables: &REDSHIFT_TABLES }),
        "AlterSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1427), tables: &REDSHIFT_TABLES }),
        "AlterStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1450), tables: &REDSHIFT_TABLES }),
        "AlterSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1479), tables: &REDSHIFT_TABLES }),
        "AlterSystemStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1527), tables: &REDSHIFT_TABLES }),
        "AlterTableActionSegment" => Some(RootGrammar { grammar_id: GrammarId(1550), tables: &REDSHIFT_TABLES }),
        "AlterTableDropColumnGrammar" => Some(RootGrammar { grammar_id: GrammarId(1661), tables: &REDSHIFT_TABLES }),
        "AlterTableOptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(1666), tables: &REDSHIFT_TABLES }),
        "AlterTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1693), tables: &REDSHIFT_TABLES }),
        "AlterTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1759), tables: &REDSHIFT_TABLES }),
        "AlterTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1824), tables: &REDSHIFT_TABLES }),
        "AlterTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1841), tables: &REDSHIFT_TABLES }),
        "AlterUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1915), tables: &REDSHIFT_TABLES }),
        "AlterViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(1969), tables: &REDSHIFT_TABLES }),
        "AltereDatashareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2027), tables: &REDSHIFT_TABLES }),
        "AlwaysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2072), tables: &REDSHIFT_TABLES }),
        "AmpersandSegment" => Some(RootGrammar { grammar_id: GrammarId(2073), tables: &REDSHIFT_TABLES }),
        "AnalyseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2074), tables: &REDSHIFT_TABLES }),
        "AnalyzeCompressionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2075), tables: &REDSHIFT_TABLES }),
        "AnalyzeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2088), tables: &REDSHIFT_TABLES }),
        "AnalyzeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2089), tables: &REDSHIFT_TABLES }),
        "AndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2108), tables: &REDSHIFT_TABLES }),
        "AndOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2109), tables: &REDSHIFT_TABLES }),
        "AnyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2110), tables: &REDSHIFT_TABLES }),
        "AnyelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2111), tables: &REDSHIFT_TABLES }),
        "AppendKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2112), tables: &REDSHIFT_TABLES }),
        "ApplyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2113), tables: &REDSHIFT_TABLES }),
        "ApproximateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2114), tables: &REDSHIFT_TABLES }),
        "AreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2115), tables: &REDSHIFT_TABLES }),
        "ArgModeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2116), tables: &REDSHIFT_TABLES }),
        "ArithmeticBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2120), tables: &REDSHIFT_TABLES }),
        "ArnCatalogSchemaSegment" => Some(RootGrammar { grammar_id: GrammarId(2131), tables: &REDSHIFT_TABLES }),
        "ArnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2148), tables: &REDSHIFT_TABLES }),
        "ArrayAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(2149), tables: &REDSHIFT_TABLES }),
        "ArrayExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2157), tables: &REDSHIFT_TABLES }),
        "ArrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2158), tables: &REDSHIFT_TABLES }),
        "ArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(2159), tables: &REDSHIFT_TABLES }),
        "ArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(2162), tables: &REDSHIFT_TABLES }),
        "ArrayTypeSuffixSegment" => Some(RootGrammar { grammar_id: GrammarId(2163), tables: &REDSHIFT_TABLES }),
        "ArrayUnnestSegment" => Some(RootGrammar { grammar_id: GrammarId(2165), tables: &REDSHIFT_TABLES }),
        "Array_aggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2171), tables: &REDSHIFT_TABLES }),
        "Array_max_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2172), tables: &REDSHIFT_TABLES }),
        "AsAliasExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2173), tables: &REDSHIFT_TABLES }),
        "AsAliasOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2176), tables: &REDSHIFT_TABLES }),
        "AsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2178), tables: &REDSHIFT_TABLES }),
        "AscKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2179), tables: &REDSHIFT_TABLES }),
        "AsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2180), tables: &REDSHIFT_TABLES }),
        "AsinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2181), tables: &REDSHIFT_TABLES }),
        "AssertionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2182), tables: &REDSHIFT_TABLES }),
        "AssignmentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2183), tables: &REDSHIFT_TABLES }),
        "AsymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2184), tables: &REDSHIFT_TABLES }),
        "AtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2185), tables: &REDSHIFT_TABLES }),
        "AtanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2186), tables: &REDSHIFT_TABLES }),
        "AtomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2187), tables: &REDSHIFT_TABLES }),
        "AttachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2188), tables: &REDSHIFT_TABLES }),
        "AttributeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2189), tables: &REDSHIFT_TABLES }),
        "AttributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2190), tables: &REDSHIFT_TABLES }),
        "AuthenticationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2191), tables: &REDSHIFT_TABLES }),
        "Authentication_arnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2192), tables: &REDSHIFT_TABLES }),
        "AuthorizationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2193), tables: &REDSHIFT_TABLES }),
        "AuthorizationSegment" => Some(RootGrammar { grammar_id: GrammarId(2194), tables: &REDSHIFT_TABLES }),
        "AutoIncrementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2220), tables: &REDSHIFT_TABLES }),
        "AutoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2222), tables: &REDSHIFT_TABLES }),
        "Auto_incrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2223), tables: &REDSHIFT_TABLES }),
        "AvgKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2224), tables: &REDSHIFT_TABLES }),
        "AvroKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2225), tables: &REDSHIFT_TABLES }),
        "Az64KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2226), tables: &REDSHIFT_TABLES }),
        "BackupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2227), tables: &REDSHIFT_TABLES }),
        "BackwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2228), tables: &REDSHIFT_TABLES }),
        "BareFunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(2229), tables: &REDSHIFT_TABLES }),
        "Base64KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2230), tables: &REDSHIFT_TABLES }),
        "BaseExpressionElementGrammar" => Some(RootGrammar { grammar_id: GrammarId(2231), tables: &REDSHIFT_TABLES }),
        "BaseFileSegment" => Some(RootGrammar { grammar_id: GrammarId(2245), tables: &REDSHIFT_TABLES }),
        "BaseSegment" => Some(RootGrammar { grammar_id: GrammarId(2246), tables: &REDSHIFT_TABLES }),
        "BeforeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2247), tables: &REDSHIFT_TABLES }),
        "BeginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2248), tables: &REDSHIFT_TABLES }),
        "Begin_frameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2249), tables: &REDSHIFT_TABLES }),
        "Begin_partitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2250), tables: &REDSHIFT_TABLES }),
        "BernoulliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2251), tables: &REDSHIFT_TABLES }),
        "BetweenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2252), tables: &REDSHIFT_TABLES }),
        "BigintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2253), tables: &REDSHIFT_TABLES }),
        "BinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2254), tables: &REDSHIFT_TABLES }),
        "BinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2255), tables: &REDSHIFT_TABLES }),
        "BinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2261), tables: &REDSHIFT_TABLES }),
        "Binary_classificationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2262), tables: &REDSHIFT_TABLES }),
        "BindingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2263), tables: &REDSHIFT_TABLES }),
        "BitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2264), tables: &REDSHIFT_TABLES }),
        "Bit_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2265), tables: &REDSHIFT_TABLES }),
        "BitwiseAndSegment" => Some(RootGrammar { grammar_id: GrammarId(2266), tables: &REDSHIFT_TABLES }),
        "BitwiseLShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2267), tables: &REDSHIFT_TABLES }),
        "BitwiseOrSegment" => Some(RootGrammar { grammar_id: GrammarId(2270), tables: &REDSHIFT_TABLES }),
        "BitwiseRShiftSegment" => Some(RootGrammar { grammar_id: GrammarId(2271), tables: &REDSHIFT_TABLES }),
        "BitwiseXorSegment" => Some(RootGrammar { grammar_id: GrammarId(2274), tables: &REDSHIFT_TABLES }),
        "BlanksasnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2275), tables: &REDSHIFT_TABLES }),
        "BlobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2276), tables: &REDSHIFT_TABLES }),
        "BlockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2277), tables: &REDSHIFT_TABLES }),
        "BomKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2278), tables: &REDSHIFT_TABLES }),
        "BoolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2279), tables: &REDSHIFT_TABLES }),
        "BooleanBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2280), tables: &REDSHIFT_TABLES }),
        "BooleanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2283), tables: &REDSHIFT_TABLES }),
        "BooleanLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(2284), tables: &REDSHIFT_TABLES }),
        "BoostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2287), tables: &REDSHIFT_TABLES }),
        "BothKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2288), tables: &REDSHIFT_TABLES }),
        "BpcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2289), tables: &REDSHIFT_TABLES }),
        "BracketedArguments" => Some(RootGrammar { grammar_id: GrammarId(2290), tables: &REDSHIFT_TABLES }),
        "BracketedColumnReferenceListGrammar" => Some(RootGrammar { grammar_id: GrammarId(2295), tables: &REDSHIFT_TABLES }),
        "BracketedSegment" => Some(RootGrammar { grammar_id: GrammarId(2298), tables: &REDSHIFT_TABLES }),
        "BracketedSetExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2299), tables: &REDSHIFT_TABLES }),
        "BreadthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2301), tables: &REDSHIFT_TABLES }),
        "BuffersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2302), tables: &REDSHIFT_TABLES }),
        "ByKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2303), tables: &REDSHIFT_TABLES }),
        "BypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2304), tables: &REDSHIFT_TABLES }),
        "BytedictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2305), tables: &REDSHIFT_TABLES }),
        "Bzip2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2306), tables: &REDSHIFT_TABLES }),
        "CKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2307), tables: &REDSHIFT_TABLES }),
        "CTEColumnList" => Some(RootGrammar { grammar_id: GrammarId(2308), tables: &REDSHIFT_TABLES }),
        "CTEDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2310), tables: &REDSHIFT_TABLES }),
        "CacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2337), tables: &REDSHIFT_TABLES }),
        "CallKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2338), tables: &REDSHIFT_TABLES }),
        "CallStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2339), tables: &REDSHIFT_TABLES }),
        "CallStoredProcedureSegment" => Some(RootGrammar { grammar_id: GrammarId(2342), tables: &REDSHIFT_TABLES }),
        "CalledKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2345), tables: &REDSHIFT_TABLES }),
        "CardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2346), tables: &REDSHIFT_TABLES }),
        "CascadeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2347), tables: &REDSHIFT_TABLES }),
        "CascadeRestrictGrammar" => Some(RootGrammar { grammar_id: GrammarId(2348), tables: &REDSHIFT_TABLES }),
        "CascadedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2351), tables: &REDSHIFT_TABLES }),
        "CaseExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(2352), tables: &REDSHIFT_TABLES }),
        "CaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2376), tables: &REDSHIFT_TABLES }),
        "Case_insensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2377), tables: &REDSHIFT_TABLES }),
        "Case_sensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2378), tables: &REDSHIFT_TABLES }),
        "CastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2379), tables: &REDSHIFT_TABLES }),
        "CastOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2380), tables: &REDSHIFT_TABLES }),
        "CatalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2381), tables: &REDSHIFT_TABLES }),
        "Catalog_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2382), tables: &REDSHIFT_TABLES }),
        "Catalog_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2383), tables: &REDSHIFT_TABLES }),
        "CeilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2384), tables: &REDSHIFT_TABLES }),
        "CeilingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2385), tables: &REDSHIFT_TABLES }),
        "ChainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2386), tables: &REDSHIFT_TABLES }),
        "ChainingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2387), tables: &REDSHIFT_TABLES }),
        "CharCharacterSetGrammar" => Some(RootGrammar { grammar_id: GrammarId(2388), tables: &REDSHIFT_TABLES }),
        "CharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2389), tables: &REDSHIFT_TABLES }),
        "Char_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2390), tables: &REDSHIFT_TABLES }),
        "CharacterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2391), tables: &REDSHIFT_TABLES }),
        "Character_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2392), tables: &REDSHIFT_TABLES }),
        "Character_set_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2393), tables: &REDSHIFT_TABLES }),
        "Character_set_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2394), tables: &REDSHIFT_TABLES }),
        "Character_set_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2395), tables: &REDSHIFT_TABLES }),
        "CharacteristicsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2396), tables: &REDSHIFT_TABLES }),
        "CharactersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2397), tables: &REDSHIFT_TABLES }),
        "CheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2398), tables: &REDSHIFT_TABLES }),
        "CheckpointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2399), tables: &REDSHIFT_TABLES }),
        "CiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2400), tables: &REDSHIFT_TABLES }),
        "ClassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2401), tables: &REDSHIFT_TABLES }),
        "Class_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2402), tables: &REDSHIFT_TABLES }),
        "ClassifierKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2403), tables: &REDSHIFT_TABLES }),
        "CleanpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2404), tables: &REDSHIFT_TABLES }),
        "ClobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2405), tables: &REDSHIFT_TABLES }),
        "CloseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2406), tables: &REDSHIFT_TABLES }),
        "CloseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2407), tables: &REDSHIFT_TABLES }),
        "ClusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2410), tables: &REDSHIFT_TABLES }),
        "ClusterStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2411), tables: &REDSHIFT_TABLES }),
        "CoalesceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2424), tables: &REDSHIFT_TABLES }),
        "CobolKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2425), tables: &REDSHIFT_TABLES }),
        "CodeSegment" => Some(RootGrammar { grammar_id: GrammarId(2426), tables: &REDSHIFT_TABLES }),
        "CollateGrammar" => Some(RootGrammar { grammar_id: GrammarId(2427), tables: &REDSHIFT_TABLES }),
        "CollateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2428), tables: &REDSHIFT_TABLES }),
        "CollationClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2429), tables: &REDSHIFT_TABLES }),
        "CollationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2436), tables: &REDSHIFT_TABLES }),
        "CollationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2437), tables: &REDSHIFT_TABLES }),
        "Collation_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2443), tables: &REDSHIFT_TABLES }),
        "Collation_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2444), tables: &REDSHIFT_TABLES }),
        "Collation_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2445), tables: &REDSHIFT_TABLES }),
        "CollectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2446), tables: &REDSHIFT_TABLES }),
        "ColonDelimiterSegment" => Some(RootGrammar { grammar_id: GrammarId(2447), tables: &REDSHIFT_TABLES }),
        "ColonPrefixSegment" => Some(RootGrammar { grammar_id: GrammarId(2448), tables: &REDSHIFT_TABLES }),
        "ColonSegment" => Some(RootGrammar { grammar_id: GrammarId(2449), tables: &REDSHIFT_TABLES }),
        "ColumnAttributeSegment" => Some(RootGrammar { grammar_id: GrammarId(2450), tables: &REDSHIFT_TABLES }),
        "ColumnConstraintDefaultGrammar" => Some(RootGrammar { grammar_id: GrammarId(2478), tables: &REDSHIFT_TABLES }),
        "ColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(2483), tables: &REDSHIFT_TABLES }),
        "ColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(2497), tables: &REDSHIFT_TABLES }),
        "ColumnEncodingGrammar" => Some(RootGrammar { grammar_id: GrammarId(2505), tables: &REDSHIFT_TABLES }),
        "ColumnGeneratedGrammar" => Some(RootGrammar { grammar_id: GrammarId(2519), tables: &REDSHIFT_TABLES }),
        "ColumnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2520), tables: &REDSHIFT_TABLES }),
        "ColumnReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2521), tables: &REDSHIFT_TABLES }),
        "ColumnTypeReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(2547), tables: &REDSHIFT_TABLES }),
        "Column_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2551), tables: &REDSHIFT_TABLES }),
        "ColumnsExpressionFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2552), tables: &REDSHIFT_TABLES }),
        "ColumnsExpressionFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2553), tables: &REDSHIFT_TABLES }),
        "ColumnsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(2554), tables: &REDSHIFT_TABLES }),
        "ColumnsExpressionNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(2555), tables: &REDSHIFT_TABLES }),
        "ColumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2556), tables: &REDSHIFT_TABLES }),
        "CommaSegment" => Some(RootGrammar { grammar_id: GrammarId(2557), tables: &REDSHIFT_TABLES }),
        "Command_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2558), tables: &REDSHIFT_TABLES }),
        "Command_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2559), tables: &REDSHIFT_TABLES }),
        "CommentClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(2560), tables: &REDSHIFT_TABLES }),
        "CommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2563), tables: &REDSHIFT_TABLES }),
        "CommentOnStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2564), tables: &REDSHIFT_TABLES }),
        "CommentSegment" => Some(RootGrammar { grammar_id: GrammarId(2675), tables: &REDSHIFT_TABLES }),
        "CommentsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2676), tables: &REDSHIFT_TABLES }),
        "CommitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2677), tables: &REDSHIFT_TABLES }),
        "CommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2678), tables: &REDSHIFT_TABLES }),
        "ComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(2679), tables: &REDSHIFT_TABLES }),
        "ComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2704), tables: &REDSHIFT_TABLES }),
        "CompositeBinaryOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2705), tables: &REDSHIFT_TABLES }),
        "CompositeComparisonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(2706), tables: &REDSHIFT_TABLES }),
        "CompositeValueExpansionSegment" => Some(RootGrammar { grammar_id: GrammarId(2707), tables: &REDSHIFT_TABLES }),
        "CompoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2712), tables: &REDSHIFT_TABLES }),
        "CompressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2713), tables: &REDSHIFT_TABLES }),
        "CompressionTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(2714), tables: &REDSHIFT_TABLES }),
        "ComprowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2719), tables: &REDSHIFT_TABLES }),
        "CompupdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2720), tables: &REDSHIFT_TABLES }),
        "ConcatSegment" => Some(RootGrammar { grammar_id: GrammarId(2721), tables: &REDSHIFT_TABLES }),
        "ConcurrentlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2724), tables: &REDSHIFT_TABLES }),
        "ConditionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2725), tables: &REDSHIFT_TABLES }),
        "Condition_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2726), tables: &REDSHIFT_TABLES }),
        "ConditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2727), tables: &REDSHIFT_TABLES }),
        "ConditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(2728), tables: &REDSHIFT_TABLES }),
        "ConditionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2732), tables: &REDSHIFT_TABLES }),
        "ConfigurationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2733), tables: &REDSHIFT_TABLES }),
        "ConflictActionSegment" => Some(RootGrammar { grammar_id: GrammarId(2734), tables: &REDSHIFT_TABLES }),
        "ConflictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2770), tables: &REDSHIFT_TABLES }),
        "ConflictTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(2771), tables: &REDSHIFT_TABLES }),
        "ConnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2792), tables: &REDSHIFT_TABLES }),
        "ConnectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2793), tables: &REDSHIFT_TABLES }),
        "ConnectionLimitSegment" => Some(RootGrammar { grammar_id: GrammarId(2794), tables: &REDSHIFT_TABLES }),
        "Connection_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2800), tables: &REDSHIFT_TABLES }),
        "Connection_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2801), tables: &REDSHIFT_TABLES }),
        "ConstraintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2802), tables: &REDSHIFT_TABLES }),
        "Constraint_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2803), tables: &REDSHIFT_TABLES }),
        "Constraint_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2804), tables: &REDSHIFT_TABLES }),
        "Constraint_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2805), tables: &REDSHIFT_TABLES }),
        "ConstraintsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2806), tables: &REDSHIFT_TABLES }),
        "ConstructorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2807), tables: &REDSHIFT_TABLES }),
        "ContainsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2808), tables: &REDSHIFT_TABLES }),
        "ContentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2809), tables: &REDSHIFT_TABLES }),
        "ContinueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2810), tables: &REDSHIFT_TABLES }),
        "ControlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2811), tables: &REDSHIFT_TABLES }),
        "ConversionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2812), tables: &REDSHIFT_TABLES }),
        "ConvertFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(2813), tables: &REDSHIFT_TABLES }),
        "ConvertFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(2818), tables: &REDSHIFT_TABLES }),
        "ConvertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2820), tables: &REDSHIFT_TABLES }),
        "CopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2821), tables: &REDSHIFT_TABLES }),
        "CopyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2822), tables: &REDSHIFT_TABLES }),
        "CorrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2920), tables: &REDSHIFT_TABLES }),
        "CorrespondingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2921), tables: &REDSHIFT_TABLES }),
        "CosKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2922), tables: &REDSHIFT_TABLES }),
        "CoshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2923), tables: &REDSHIFT_TABLES }),
        "CostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2924), tables: &REDSHIFT_TABLES }),
        "CostsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2925), tables: &REDSHIFT_TABLES }),
        "CountKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2926), tables: &REDSHIFT_TABLES }),
        "Covar_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2927), tables: &REDSHIFT_TABLES }),
        "Covar_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(2928), tables: &REDSHIFT_TABLES }),
        "CreateAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2929), tables: &REDSHIFT_TABLES }),
        "CreateCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2937), tables: &REDSHIFT_TABLES }),
        "CreateCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(2963), tables: &REDSHIFT_TABLES }),
        "CreateDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3001), tables: &REDSHIFT_TABLES }),
        "CreateDatabaseWithOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(3013), tables: &REDSHIFT_TABLES }),
        "CreateDatashareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3021), tables: &REDSHIFT_TABLES }),
        "CreateDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3032), tables: &REDSHIFT_TABLES }),
        "CreateExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3057), tables: &REDSHIFT_TABLES }),
        "CreateExternalFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3073), tables: &REDSHIFT_TABLES }),
        "CreateExternalSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3099), tables: &REDSHIFT_TABLES }),
        "CreateExternalTableAsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3163), tables: &REDSHIFT_TABLES }),
        "CreateExternalTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3194), tables: &REDSHIFT_TABLES }),
        "CreateForeignDataWrapperStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3249), tables: &REDSHIFT_TABLES }),
        "CreateForeignTableGrammar" => Some(RootGrammar { grammar_id: GrammarId(3257), tables: &REDSHIFT_TABLES }),
        "CreateForeignTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3261), tables: &REDSHIFT_TABLES }),
        "CreateFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3317), tables: &REDSHIFT_TABLES }),
        "CreateGroupStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3344), tables: &REDSHIFT_TABLES }),
        "CreateIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3353), tables: &REDSHIFT_TABLES }),
        "CreateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(3388), tables: &REDSHIFT_TABLES }),
        "CreateLibraryStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3389), tables: &REDSHIFT_TABLES }),
        "CreateMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3404), tables: &REDSHIFT_TABLES }),
        "CreateModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3438), tables: &REDSHIFT_TABLES }),
        "CreateOperatorStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3520), tables: &REDSHIFT_TABLES }),
        "CreatePolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3555), tables: &REDSHIFT_TABLES }),
        "CreateProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3592), tables: &REDSHIFT_TABLES }),
        "CreatePublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3600), tables: &REDSHIFT_TABLES }),
        "CreateRlsPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3616), tables: &REDSHIFT_TABLES }),
        "CreateRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3635), tables: &REDSHIFT_TABLES }),
        "CreateSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3699), tables: &REDSHIFT_TABLES }),
        "CreateSequenceOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(3713), tables: &REDSHIFT_TABLES }),
        "CreateSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3757), tables: &REDSHIFT_TABLES }),
        "CreateServerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3765), tables: &REDSHIFT_TABLES }),
        "CreateStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3779), tables: &REDSHIFT_TABLES }),
        "CreateSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3799), tables: &REDSHIFT_TABLES }),
        "CreateTableAsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3811), tables: &REDSHIFT_TABLES }),
        "CreateTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3833), tables: &REDSHIFT_TABLES }),
        "CreateTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3862), tables: &REDSHIFT_TABLES }),
        "CreateTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3878), tables: &REDSHIFT_TABLES }),
        "CreateTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3954), tables: &REDSHIFT_TABLES }),
        "CreateUserMappingGrammar" => Some(RootGrammar { grammar_id: GrammarId(3966), tables: &REDSHIFT_TABLES }),
        "CreateUserMappingStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3970), tables: &REDSHIFT_TABLES }),
        "CreateUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(3981), tables: &REDSHIFT_TABLES }),
        "CreateViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4017), tables: &REDSHIFT_TABLES }),
        "CreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4029), tables: &REDSHIFT_TABLES }),
        "CreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4030), tables: &REDSHIFT_TABLES }),
        "CreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4031), tables: &REDSHIFT_TABLES }),
        "CredentialsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4032), tables: &REDSHIFT_TABLES }),
        "CrossKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4033), tables: &REDSHIFT_TABLES }),
        "CsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4034), tables: &REDSHIFT_TABLES }),
        "CsvKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4035), tables: &REDSHIFT_TABLES }),
        "CubeFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4036), tables: &REDSHIFT_TABLES }),
        "CubeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4037), tables: &REDSHIFT_TABLES }),
        "CubeRollupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4038), tables: &REDSHIFT_TABLES }),
        "Cume_distKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4044), tables: &REDSHIFT_TABLES }),
        "CurrentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4045), tables: &REDSHIFT_TABLES }),
        "Current_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4046), tables: &REDSHIFT_TABLES }),
        "Current_dateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4047), tables: &REDSHIFT_TABLES }),
        "Current_default_transform_groupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4048), tables: &REDSHIFT_TABLES }),
        "Current_pathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4049), tables: &REDSHIFT_TABLES }),
        "Current_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4050), tables: &REDSHIFT_TABLES }),
        "Current_rowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4051), tables: &REDSHIFT_TABLES }),
        "Current_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4052), tables: &REDSHIFT_TABLES }),
        "Current_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4053), tables: &REDSHIFT_TABLES }),
        "Current_timestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4054), tables: &REDSHIFT_TABLES }),
        "Current_transform_group_for_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4055), tables: &REDSHIFT_TABLES }),
        "Current_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4056), tables: &REDSHIFT_TABLES }),
        "Current_user_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4057), tables: &REDSHIFT_TABLES }),
        "CursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4058), tables: &REDSHIFT_TABLES }),
        "Cursor_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4059), tables: &REDSHIFT_TABLES }),
        "CycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4060), tables: &REDSHIFT_TABLES }),
        "DataFormatSegment" => Some(RootGrammar { grammar_id: GrammarId(4061), tables: &REDSHIFT_TABLES }),
        "DataKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4089), tables: &REDSHIFT_TABLES }),
        "Data_catalog_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4090), tables: &REDSHIFT_TABLES }),
        "DatabaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4091), tables: &REDSHIFT_TABLES }),
        "DatabaseReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "DatalinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4096), tables: &REDSHIFT_TABLES }),
        "DatashareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4097), tables: &REDSHIFT_TABLES }),
        "DatasharesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4098), tables: &REDSHIFT_TABLES }),
        "DatatypeIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(4099), tables: &REDSHIFT_TABLES }),
        "DatatypeSegment" => Some(RootGrammar { grammar_id: GrammarId(4103), tables: &REDSHIFT_TABLES }),
        "DateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4159), tables: &REDSHIFT_TABLES }),
        "DatePartFunctionName" => Some(RootGrammar { grammar_id: GrammarId(4160), tables: &REDSHIFT_TABLES }),
        "DatePartFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(4161), tables: &REDSHIFT_TABLES }),
        "DateTimeFunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(4162), tables: &REDSHIFT_TABLES }),
        "DateTimeLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(4168), tables: &REDSHIFT_TABLES }),
        "DateTimeTypeIdentifier" => Some(RootGrammar { grammar_id: GrammarId(4171), tables: &REDSHIFT_TABLES }),
        "DateformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4178), tables: &REDSHIFT_TABLES }),
        "DatetimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4179), tables: &REDSHIFT_TABLES }),
        "DatetimeUnitSegment" => Some(RootGrammar { grammar_id: GrammarId(4180), tables: &REDSHIFT_TABLES }),
        "Datetime_interval_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4181), tables: &REDSHIFT_TABLES }),
        "Datetime_interval_precisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4182), tables: &REDSHIFT_TABLES }),
        "DayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4183), tables: &REDSHIFT_TABLES }),
        "DayofyearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4184), tables: &REDSHIFT_TABLES }),
        "DbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4185), tables: &REDSHIFT_TABLES }),
        "DbuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4186), tables: &REDSHIFT_TABLES }),
        "DeallocateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4187), tables: &REDSHIFT_TABLES }),
        "DeallocateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4188), tables: &REDSHIFT_TABLES }),
        "DecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4192), tables: &REDSHIFT_TABLES }),
        "DecfloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4193), tables: &REDSHIFT_TABLES }),
        "DecimalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4194), tables: &REDSHIFT_TABLES }),
        "DeclareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4195), tables: &REDSHIFT_TABLES }),
        "DeclareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4196), tables: &REDSHIFT_TABLES }),
        "Dedent" => Some(RootGrammar { grammar_id: GrammarId(250), tables: &REDSHIFT_TABLES }),
        "DefaultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4202), tables: &REDSHIFT_TABLES }),
        "DefaultValuesGrammar" => Some(RootGrammar { grammar_id: GrammarId(4203), tables: &REDSHIFT_TABLES }),
        "DefaultsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4206), tables: &REDSHIFT_TABLES }),
        "DeferrableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4207), tables: &REDSHIFT_TABLES }),
        "DeferredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4208), tables: &REDSHIFT_TABLES }),
        "DefineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4209), tables: &REDSHIFT_TABLES }),
        "DefinedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4210), tables: &REDSHIFT_TABLES }),
        "DefinerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4211), tables: &REDSHIFT_TABLES }),
        "DefinitionArgumentValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(4212), tables: &REDSHIFT_TABLES }),
        "DefinitionParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(4215), tables: &REDSHIFT_TABLES }),
        "DefinitionParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(4220), tables: &REDSHIFT_TABLES }),
        "DeflateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4223), tables: &REDSHIFT_TABLES }),
        "DefragKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4224), tables: &REDSHIFT_TABLES }),
        "DegreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4225), tables: &REDSHIFT_TABLES }),
        "DeleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4226), tables: &REDSHIFT_TABLES }),
        "DeleteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4227), tables: &REDSHIFT_TABLES }),
        "DelimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4256), tables: &REDSHIFT_TABLES }),
        "DelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(4257), tables: &REDSHIFT_TABLES }),
        "DelimiterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4258), tables: &REDSHIFT_TABLES }),
        "DelimitersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4259), tables: &REDSHIFT_TABLES }),
        "Delta32kKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4260), tables: &REDSHIFT_TABLES }),
        "DeltaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4261), tables: &REDSHIFT_TABLES }),
        "Dense_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4262), tables: &REDSHIFT_TABLES }),
        "DependsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4263), tables: &REDSHIFT_TABLES }),
        "DepthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4264), tables: &REDSHIFT_TABLES }),
        "DerefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4265), tables: &REDSHIFT_TABLES }),
        "DerivedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4266), tables: &REDSHIFT_TABLES }),
        "DescDatashareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4267), tables: &REDSHIFT_TABLES }),
        "DescKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4278), tables: &REDSHIFT_TABLES }),
        "DescribeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4279), tables: &REDSHIFT_TABLES }),
        "DescribeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4280), tables: &REDSHIFT_TABLES }),
        "DescriptorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4284), tables: &REDSHIFT_TABLES }),
        "DetachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4285), tables: &REDSHIFT_TABLES }),
        "DeterministicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4286), tables: &REDSHIFT_TABLES }),
        "DiagnosticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4287), tables: &REDSHIFT_TABLES }),
        "DictionaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4288), tables: &REDSHIFT_TABLES }),
        "DisableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4289), tables: &REDSHIFT_TABLES }),
        "DiscardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4290), tables: &REDSHIFT_TABLES }),
        "DiscardStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4291), tables: &REDSHIFT_TABLES }),
        "DisconnectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4299), tables: &REDSHIFT_TABLES }),
        "DispatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4300), tables: &REDSHIFT_TABLES }),
        "DistinctKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4301), tables: &REDSHIFT_TABLES }),
        "DistkeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4302), tables: &REDSHIFT_TABLES }),
        "DiststyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4303), tables: &REDSHIFT_TABLES }),
        "DivideSegment" => Some(RootGrammar { grammar_id: GrammarId(4304), tables: &REDSHIFT_TABLES }),
        "DlnewcopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4305), tables: &REDSHIFT_TABLES }),
        "DlpreviouscopyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4306), tables: &REDSHIFT_TABLES }),
        "DlurlcompleteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4307), tables: &REDSHIFT_TABLES }),
        "DlurlcompleteonlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4308), tables: &REDSHIFT_TABLES }),
        "DlurlcompletewriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4309), tables: &REDSHIFT_TABLES }),
        "DlurlpathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4310), tables: &REDSHIFT_TABLES }),
        "DlurlpathonlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4311), tables: &REDSHIFT_TABLES }),
        "DlurlpathwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4312), tables: &REDSHIFT_TABLES }),
        "DlurlschemeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4313), tables: &REDSHIFT_TABLES }),
        "DlurlserverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4314), tables: &REDSHIFT_TABLES }),
        "DlvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4315), tables: &REDSHIFT_TABLES }),
        "DoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4316), tables: &REDSHIFT_TABLES }),
        "DoStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4317), tables: &REDSHIFT_TABLES }),
        "DocumentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4326), tables: &REDSHIFT_TABLES }),
        "DollarNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4327), tables: &REDSHIFT_TABLES }),
        "DomainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4328), tables: &REDSHIFT_TABLES }),
        "DotSegment" => Some(RootGrammar { grammar_id: GrammarId(4329), tables: &REDSHIFT_TABLES }),
        "DoubleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4330), tables: &REDSHIFT_TABLES }),
        "DropAggregateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4331), tables: &REDSHIFT_TABLES }),
        "DropBehaviorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4343), tables: &REDSHIFT_TABLES }),
        "DropCastStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4346), tables: &REDSHIFT_TABLES }),
        "DropCollationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4355), tables: &REDSHIFT_TABLES }),
        "DropDatabaseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4361), tables: &REDSHIFT_TABLES }),
        "DropDatashareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4370), tables: &REDSHIFT_TABLES }),
        "DropDomainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4374), tables: &REDSHIFT_TABLES }),
        "DropExtensionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4381), tables: &REDSHIFT_TABLES }),
        "DropForeignTableStatement" => Some(RootGrammar { grammar_id: GrammarId(4387), tables: &REDSHIFT_TABLES }),
        "DropFunctionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4395), tables: &REDSHIFT_TABLES }),
        "DropIndexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4404), tables: &REDSHIFT_TABLES }),
        "DropKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4412), tables: &REDSHIFT_TABLES }),
        "DropMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4413), tables: &REDSHIFT_TABLES }),
        "DropModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4421), tables: &REDSHIFT_TABLES }),
        "DropOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4426), tables: &REDSHIFT_TABLES }),
        "DropPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4437), tables: &REDSHIFT_TABLES }),
        "DropProcedureStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4445), tables: &REDSHIFT_TABLES }),
        "DropPublicationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4453), tables: &REDSHIFT_TABLES }),
        "DropRlsPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4460), tables: &REDSHIFT_TABLES }),
        "DropRoleStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4469), tables: &REDSHIFT_TABLES }),
        "DropSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4474), tables: &REDSHIFT_TABLES }),
        "DropSequenceStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4480), tables: &REDSHIFT_TABLES }),
        "DropStatisticsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4487), tables: &REDSHIFT_TABLES }),
        "DropSubscriptionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4496), tables: &REDSHIFT_TABLES }),
        "DropTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4504), tables: &REDSHIFT_TABLES }),
        "DropTextSearchConfigurationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4512), tables: &REDSHIFT_TABLES }),
        "DropTriggerStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4520), tables: &REDSHIFT_TABLES }),
        "DropTypeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4528), tables: &REDSHIFT_TABLES }),
        "DropUserStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4535), tables: &REDSHIFT_TABLES }),
        "DropViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4540), tables: &REDSHIFT_TABLES }),
        "DuplicatesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4547), tables: &REDSHIFT_TABLES }),
        "DynamicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4548), tables: &REDSHIFT_TABLES }),
        "Dynamic_functionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4549), tables: &REDSHIFT_TABLES }),
        "Dynamic_function_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4550), tables: &REDSHIFT_TABLES }),
        "EachKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4551), tables: &REDSHIFT_TABLES }),
        "ElementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4552), tables: &REDSHIFT_TABLES }),
        "ElseClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4553), tables: &REDSHIFT_TABLES }),
        "ElseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4556), tables: &REDSHIFT_TABLES }),
        "EmptyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4557), tables: &REDSHIFT_TABLES }),
        "EmptyStructLiteralBracketsSegment" => Some(RootGrammar { grammar_id: GrammarId(4558), tables: &REDSHIFT_TABLES }),
        "EmptyStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(4559), tables: &REDSHIFT_TABLES }),
        "EmptyasnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4562), tables: &REDSHIFT_TABLES }),
        "EnableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4563), tables: &REDSHIFT_TABLES }),
        "EncodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4564), tables: &REDSHIFT_TABLES }),
        "EncodingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4565), tables: &REDSHIFT_TABLES }),
        "EncryptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4566), tables: &REDSHIFT_TABLES }),
        "EncryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4567), tables: &REDSHIFT_TABLES }),
        "EncryptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4568), tables: &REDSHIFT_TABLES }),
        "End-execKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4569), tables: &REDSHIFT_TABLES }),
        "EndBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(242), tables: &REDSHIFT_TABLES }),
        "EndCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(4570), tables: &REDSHIFT_TABLES }),
        "EndKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4571), tables: &REDSHIFT_TABLES }),
        "EndSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2156), tables: &REDSHIFT_TABLES }),
        "End_frameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4572), tables: &REDSHIFT_TABLES }),
        "End_partitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4573), tables: &REDSHIFT_TABLES }),
        "EnforcedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4574), tables: &REDSHIFT_TABLES }),
        "EnumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4575), tables: &REDSHIFT_TABLES }),
        "EpochKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4576), tables: &REDSHIFT_TABLES }),
        "EpochmillisecsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4577), tables: &REDSHIFT_TABLES }),
        "EpochsecsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4578), tables: &REDSHIFT_TABLES }),
        "EqualsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4579), tables: &REDSHIFT_TABLES }),
        "EqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(4580), tables: &REDSHIFT_TABLES }),
        "ErrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4581), tables: &REDSHIFT_TABLES }),
        "EscapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4582), tables: &REDSHIFT_TABLES }),
        "EvenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4583), tables: &REDSHIFT_TABLES }),
        "EventKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4584), tables: &REDSHIFT_TABLES }),
        "EveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4585), tables: &REDSHIFT_TABLES }),
        "ExceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4586), tables: &REDSHIFT_TABLES }),
        "ExceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4587), tables: &REDSHIFT_TABLES }),
        "ExcludeClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4588), tables: &REDSHIFT_TABLES }),
        "ExcludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4595), tables: &REDSHIFT_TABLES }),
        "ExcludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4596), tables: &REDSHIFT_TABLES }),
        "ExclusionConstraintElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4597), tables: &REDSHIFT_TABLES }),
        "ExclusiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4601), tables: &REDSHIFT_TABLES }),
        "ExecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4602), tables: &REDSHIFT_TABLES }),
        "ExecuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4603), tables: &REDSHIFT_TABLES }),
        "ExecuteStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4604), tables: &REDSHIFT_TABLES }),
        "ExecutionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4610), tables: &REDSHIFT_TABLES }),
        "ExistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4611), tables: &REDSHIFT_TABLES }),
        "ExpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4612), tables: &REDSHIFT_TABLES }),
        "ExplainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4613), tables: &REDSHIFT_TABLES }),
        "ExplainOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(4614), tables: &REDSHIFT_TABLES }),
        "ExplainStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4634), tables: &REDSHIFT_TABLES }),
        "ExplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4650), tables: &REDSHIFT_TABLES }),
        "Explicit_idsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4651), tables: &REDSHIFT_TABLES }),
        "ExpressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4652), tables: &REDSHIFT_TABLES }),
        "ExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(4653), tables: &REDSHIFT_TABLES }),
        "Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4654), tables: &REDSHIFT_TABLES }),
        "Expression_A_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4680), tables: &REDSHIFT_TABLES }),
        "Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4687), tables: &REDSHIFT_TABLES }),
        "Expression_B_Unary_Operator_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4697), tables: &REDSHIFT_TABLES }),
        "Expression_C_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4702), tables: &REDSHIFT_TABLES }),
        "Expression_D_Grammar" => Some(RootGrammar { grammar_id: GrammarId(4721), tables: &REDSHIFT_TABLES }),
        "Expression_D_Potential_Select_Statement_Without_Brackets" => Some(RootGrammar { grammar_id: GrammarId(4758), tables: &REDSHIFT_TABLES }),
        "ExtendedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4766), tables: &REDSHIFT_TABLES }),
        "ExtendedNaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(4767), tables: &REDSHIFT_TABLES }),
        "ExtendedTableReferenceGrammar" => Some(RootGrammar { grammar_id: GrammarId(4768), tables: &REDSHIFT_TABLES }),
        "ExtensionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4778), tables: &REDSHIFT_TABLES }),
        "ExtensionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "ExternalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4779), tables: &REDSHIFT_TABLES }),
        "ExtractKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4780), tables: &REDSHIFT_TABLES }),
        "FalseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4781), tables: &REDSHIFT_TABLES }),
        "FalseSegment" => Some(RootGrammar { grammar_id: GrammarId(4782), tables: &REDSHIFT_TABLES }),
        "FamilyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4783), tables: &REDSHIFT_TABLES }),
        "FetchClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4784), tables: &REDSHIFT_TABLES }),
        "FetchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4801), tables: &REDSHIFT_TABLES }),
        "FetchStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(4802), tables: &REDSHIFT_TABLES }),
        "FieldsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4814), tables: &REDSHIFT_TABLES }),
        "FileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4815), tables: &REDSHIFT_TABLES }),
        "FileSegment" => Some(RootGrammar { grammar_id: GrammarId(4816), tables: &REDSHIFT_TABLES }),
        "FillrecordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4824), tables: &REDSHIFT_TABLES }),
        "FilltargetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4825), tables: &REDSHIFT_TABLES }),
        "FilterClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(4826), tables: &REDSHIFT_TABLES }),
        "FilterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4832), tables: &REDSHIFT_TABLES }),
        "FinalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4833), tables: &REDSHIFT_TABLES }),
        "FinalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4834), tables: &REDSHIFT_TABLES }),
        "FinishKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4835), tables: &REDSHIFT_TABLES }),
        "FirstKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4836), tables: &REDSHIFT_TABLES }),
        "First_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4837), tables: &REDSHIFT_TABLES }),
        "FixedwidthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4838), tables: &REDSHIFT_TABLES }),
        "FlagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4839), tables: &REDSHIFT_TABLES }),
        "Float4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4840), tables: &REDSHIFT_TABLES }),
        "Float8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4841), tables: &REDSHIFT_TABLES }),
        "FloatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4842), tables: &REDSHIFT_TABLES }),
        "FloorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4843), tables: &REDSHIFT_TABLES }),
        "FollowingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4844), tables: &REDSHIFT_TABLES }),
        "ForClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4845), tables: &REDSHIFT_TABLES }),
        "ForKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4866), tables: &REDSHIFT_TABLES }),
        "ForceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4867), tables: &REDSHIFT_TABLES }),
        "ForeignDataWrapperGrammar" => Some(RootGrammar { grammar_id: GrammarId(4868), tables: &REDSHIFT_TABLES }),
        "ForeignKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(4872), tables: &REDSHIFT_TABLES }),
        "ForeignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4875), tables: &REDSHIFT_TABLES }),
        "ForeignTableColumnConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4876), tables: &REDSHIFT_TABLES }),
        "ForeignTableTableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(4905), tables: &REDSHIFT_TABLES }),
        "FormatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4916), tables: &REDSHIFT_TABLES }),
        "FortranKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4917), tables: &REDSHIFT_TABLES }),
        "ForwardKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4918), tables: &REDSHIFT_TABLES }),
        "FoundKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4919), tables: &REDSHIFT_TABLES }),
        "FrameClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4920), tables: &REDSHIFT_TABLES }),
        "FrameClauseUnitGrammar" => Some(RootGrammar { grammar_id: GrammarId(4950), tables: &REDSHIFT_TABLES }),
        "Frame_rowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4954), tables: &REDSHIFT_TABLES }),
        "FreeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4955), tables: &REDSHIFT_TABLES }),
        "FreezeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(4956), tables: &REDSHIFT_TABLES }),
        "FromClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4957), tables: &REDSHIFT_TABLES }),
        "FromClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(4963), tables: &REDSHIFT_TABLES }),
        "FromDatashareClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(4981), tables: &REDSHIFT_TABLES }),
        "FromExpressionElementSegment" => Some(RootGrammar { grammar_id: GrammarId(4999), tables: &REDSHIFT_TABLES }),
        "FromExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5022), tables: &REDSHIFT_TABLES }),
        "FromIntegrationClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5049), tables: &REDSHIFT_TABLES }),
        "FromKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5077), tables: &REDSHIFT_TABLES }),
        "FromPivotExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5078), tables: &REDSHIFT_TABLES }),
        "FromUnpivotExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5094), tables: &REDSHIFT_TABLES }),
        "FsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5112), tables: &REDSHIFT_TABLES }),
        "FulfillKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5113), tables: &REDSHIFT_TABLES }),
        "FullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5114), tables: &REDSHIFT_TABLES }),
        "FullTextSearchOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5115), tables: &REDSHIFT_TABLES }),
        "FunctionContentsExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5116), tables: &REDSHIFT_TABLES }),
        "FunctionContentsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5124), tables: &REDSHIFT_TABLES }),
        "FunctionContentsSegment" => Some(RootGrammar { grammar_id: GrammarId(5185), tables: &REDSHIFT_TABLES }),
        "FunctionDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5188), tables: &REDSHIFT_TABLES }),
        "FunctionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5290), tables: &REDSHIFT_TABLES }),
        "FunctionNameIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5291), tables: &REDSHIFT_TABLES }),
        "FunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(5292), tables: &REDSHIFT_TABLES }),
        "FunctionParameterGrammar" => Some(RootGrammar { grammar_id: GrammarId(5302), tables: &REDSHIFT_TABLES }),
        "FunctionParameterListGrammar" => Some(RootGrammar { grammar_id: GrammarId(5326), tables: &REDSHIFT_TABLES }),
        "FunctionSegment" => Some(RootGrammar { grammar_id: GrammarId(5329), tables: &REDSHIFT_TABLES }),
        "FunctionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5354), tables: &REDSHIFT_TABLES }),
        "FusionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5355), tables: &REDSHIFT_TABLES }),
        "FutureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5356), tables: &REDSHIFT_TABLES }),
        "GKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5357), tables: &REDSHIFT_TABLES }),
        "GbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5358), tables: &REDSHIFT_TABLES }),
        "GeneralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5359), tables: &REDSHIFT_TABLES }),
        "GeneratedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5360), tables: &REDSHIFT_TABLES }),
        "GeographyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5361), tables: &REDSHIFT_TABLES }),
        "GeometryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5362), tables: &REDSHIFT_TABLES }),
        "GetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5363), tables: &REDSHIFT_TABLES }),
        "GlobOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5364), tables: &REDSHIFT_TABLES }),
        "GlobalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5365), tables: &REDSHIFT_TABLES }),
        "Globaldict256KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5366), tables: &REDSHIFT_TABLES }),
        "Globaldict64kKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5367), tables: &REDSHIFT_TABLES }),
        "GoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5368), tables: &REDSHIFT_TABLES }),
        "GotoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5369), tables: &REDSHIFT_TABLES }),
        "GrantKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5370), tables: &REDSHIFT_TABLES }),
        "GrantStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5371), tables: &REDSHIFT_TABLES }),
        "GrantUsageDatashareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5420), tables: &REDSHIFT_TABLES }),
        "GrantedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5443), tables: &REDSHIFT_TABLES }),
        "GrantsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5444), tables: &REDSHIFT_TABLES }),
        "GreaterThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5445), tables: &REDSHIFT_TABLES }),
        "GreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5448), tables: &REDSHIFT_TABLES }),
        "GreatestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5449), tables: &REDSHIFT_TABLES }),
        "GroupByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5450), tables: &REDSHIFT_TABLES }),
        "GroupByClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5470), tables: &REDSHIFT_TABLES }),
        "GroupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5479), tables: &REDSHIFT_TABLES }),
        "GroupingExpressionList" => Some(RootGrammar { grammar_id: GrammarId(5480), tables: &REDSHIFT_TABLES }),
        "GroupingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5488), tables: &REDSHIFT_TABLES }),
        "GroupingSetsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5489), tables: &REDSHIFT_TABLES }),
        "GroupsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5497), tables: &REDSHIFT_TABLES }),
        "GzipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5498), tables: &REDSHIFT_TABLES }),
        "HandlerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5499), tables: &REDSHIFT_TABLES }),
        "HashKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5500), tables: &REDSHIFT_TABLES }),
        "HavingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5501), tables: &REDSHIFT_TABLES }),
        "HavingClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5506), tables: &REDSHIFT_TABLES }),
        "HavingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5514), tables: &REDSHIFT_TABLES }),
        "HeaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5515), tables: &REDSHIFT_TABLES }),
        "HexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5516), tables: &REDSHIFT_TABLES }),
        "HierarchyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5517), tables: &REDSHIFT_TABLES }),
        "History_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5518), tables: &REDSHIFT_TABLES }),
        "HiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5519), tables: &REDSHIFT_TABLES }),
        "HllsketchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5520), tables: &REDSHIFT_TABLES }),
        "HoldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5521), tables: &REDSHIFT_TABLES }),
        "HorizontalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5522), tables: &REDSHIFT_TABLES }),
        "HourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5523), tables: &REDSHIFT_TABLES }),
        "HyperparametersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5524), tables: &REDSHIFT_TABLES }),
        "IamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5525), tables: &REDSHIFT_TABLES }),
        "IamRoleClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5526), tables: &REDSHIFT_TABLES }),
        "Iam_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5531), tables: &REDSHIFT_TABLES }),
        "IdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5532), tables: &REDSHIFT_TABLES }),
        "IdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(5533), tables: &REDSHIFT_TABLES }),
        "IdentityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5534), tables: &REDSHIFT_TABLES }),
        "IfExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5535), tables: &REDSHIFT_TABLES }),
        "IfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5538), tables: &REDSHIFT_TABLES }),
        "IfNotExistsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5539), tables: &REDSHIFT_TABLES }),
        "IgnoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5543), tables: &REDSHIFT_TABLES }),
        "IgnoreRespectNullsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5544), tables: &REDSHIFT_TABLES }),
        "IgnoreblanklinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5549), tables: &REDSHIFT_TABLES }),
        "IgnoreextraKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5550), tables: &REDSHIFT_TABLES }),
        "IgnoreheaderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5551), tables: &REDSHIFT_TABLES }),
        "IlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5552), tables: &REDSHIFT_TABLES }),
        "ImmediateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5553), tables: &REDSHIFT_TABLES }),
        "ImmediatelyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5554), tables: &REDSHIFT_TABLES }),
        "ImmutableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5555), tables: &REDSHIFT_TABLES }),
        "ImplementationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5556), tables: &REDSHIFT_TABLES }),
        "ImplicitIndent" => Some(RootGrammar { grammar_id: GrammarId(2355), tables: &REDSHIFT_TABLES }),
        "ImplicitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5557), tables: &REDSHIFT_TABLES }),
        "ImportForeignSchemaGrammar" => Some(RootGrammar { grammar_id: GrammarId(5558), tables: &REDSHIFT_TABLES }),
        "ImportForeignSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5562), tables: &REDSHIFT_TABLES }),
        "ImportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5580), tables: &REDSHIFT_TABLES }),
        "ImportedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5581), tables: &REDSHIFT_TABLES }),
        "InKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5582), tables: &REDSHIFT_TABLES }),
        "InOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(5583), tables: &REDSHIFT_TABLES }),
        "IncludeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5593), tables: &REDSHIFT_TABLES }),
        "IncludenewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5594), tables: &REDSHIFT_TABLES }),
        "IncludingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5595), tables: &REDSHIFT_TABLES }),
        "IncrementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5596), tables: &REDSHIFT_TABLES }),
        "Indent" => Some(RootGrammar { grammar_id: GrammarId(234), tables: &REDSHIFT_TABLES }),
        "IndentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5597), tables: &REDSHIFT_TABLES }),
        "IndexAccessMethodSegment" => Some(RootGrammar { grammar_id: GrammarId(5598), tables: &REDSHIFT_TABLES }),
        "IndexColumnDefinitionSegment" => Some(RootGrammar { grammar_id: GrammarId(5599), tables: &REDSHIFT_TABLES }),
        "IndexElementOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(5604), tables: &REDSHIFT_TABLES }),
        "IndexElementSegment" => Some(RootGrammar { grammar_id: GrammarId(5624), tables: &REDSHIFT_TABLES }),
        "IndexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5631), tables: &REDSHIFT_TABLES }),
        "IndexParametersSegment" => Some(RootGrammar { grammar_id: GrammarId(5632), tables: &REDSHIFT_TABLES }),
        "IndexReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "IndexesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5644), tables: &REDSHIFT_TABLES }),
        "IndicatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5645), tables: &REDSHIFT_TABLES }),
        "InheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5646), tables: &REDSHIFT_TABLES }),
        "InheritsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5647), tables: &REDSHIFT_TABLES }),
        "InitialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5648), tables: &REDSHIFT_TABLES }),
        "InitiallyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5649), tables: &REDSHIFT_TABLES }),
        "InlineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5650), tables: &REDSHIFT_TABLES }),
        "InnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5651), tables: &REDSHIFT_TABLES }),
        "InoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5652), tables: &REDSHIFT_TABLES }),
        "InputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5653), tables: &REDSHIFT_TABLES }),
        "InputformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5654), tables: &REDSHIFT_TABLES }),
        "InsensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5655), tables: &REDSHIFT_TABLES }),
        "InsertKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5656), tables: &REDSHIFT_TABLES }),
        "InsertStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5657), tables: &REDSHIFT_TABLES }),
        "InstanceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5675), tables: &REDSHIFT_TABLES }),
        "InstantiableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5676), tables: &REDSHIFT_TABLES }),
        "InsteadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5677), tables: &REDSHIFT_TABLES }),
        "Int2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5678), tables: &REDSHIFT_TABLES }),
        "Int4KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5679), tables: &REDSHIFT_TABLES }),
        "Int8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5680), tables: &REDSHIFT_TABLES }),
        "IntKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5681), tables: &REDSHIFT_TABLES }),
        "IntegerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5682), tables: &REDSHIFT_TABLES }),
        "IntegrationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5683), tables: &REDSHIFT_TABLES }),
        "IntegrityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5684), tables: &REDSHIFT_TABLES }),
        "InterleavedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5685), tables: &REDSHIFT_TABLES }),
        "IntersectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5686), tables: &REDSHIFT_TABLES }),
        "IntersectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5687), tables: &REDSHIFT_TABLES }),
        "IntervalExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5688), tables: &REDSHIFT_TABLES }),
        "IntervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5702), tables: &REDSHIFT_TABLES }),
        "IntervalUnitsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5703), tables: &REDSHIFT_TABLES }),
        "IntoClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5710), tables: &REDSHIFT_TABLES }),
        "IntoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5718), tables: &REDSHIFT_TABLES }),
        "InvokerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5719), tables: &REDSHIFT_TABLES }),
        "IsClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5720), tables: &REDSHIFT_TABLES }),
        "IsDistinctFromGrammar" => Some(RootGrammar { grammar_id: GrammarId(5727), tables: &REDSHIFT_TABLES }),
        "IsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5732), tables: &REDSHIFT_TABLES }),
        "IsNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(5733), tables: &REDSHIFT_TABLES }),
        "IsnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5734), tables: &REDSHIFT_TABLES }),
        "IsolationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5735), tables: &REDSHIFT_TABLES }),
        "IsolationLevelClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5736), tables: &REDSHIFT_TABLES }),
        "JoinClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5742), tables: &REDSHIFT_TABLES }),
        "JoinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5764), tables: &REDSHIFT_TABLES }),
        "JoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5765), tables: &REDSHIFT_TABLES }),
        "JoinLikeClauseGrammar" => Some(RootGrammar { grammar_id: GrammarId(5767), tables: &REDSHIFT_TABLES }),
        "JoinOnConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5772), tables: &REDSHIFT_TABLES }),
        "JoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(5779), tables: &REDSHIFT_TABLES }),
        "JoinUsingConditionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5787), tables: &REDSHIFT_TABLES }),
        "JsonKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5792), tables: &REDSHIFT_TABLES }),
        "JsonOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5793), tables: &REDSHIFT_TABLES }),
        "JsonTestGrammar" => Some(RootGrammar { grammar_id: GrammarId(5794), tables: &REDSHIFT_TABLES }),
        "JsonTypeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5798), tables: &REDSHIFT_TABLES }),
        "JsonUniqueKeysGrammar" => Some(RootGrammar { grammar_id: GrammarId(5803), tables: &REDSHIFT_TABLES }),
        "Json_arrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5810), tables: &REDSHIFT_TABLES }),
        "Json_arrayaggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5811), tables: &REDSHIFT_TABLES }),
        "Json_existsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5812), tables: &REDSHIFT_TABLES }),
        "Json_objectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5813), tables: &REDSHIFT_TABLES }),
        "Json_objectaggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5814), tables: &REDSHIFT_TABLES }),
        "Json_queryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5815), tables: &REDSHIFT_TABLES }),
        "Json_tableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5816), tables: &REDSHIFT_TABLES }),
        "Json_table_primitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5817), tables: &REDSHIFT_TABLES }),
        "Json_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5818), tables: &REDSHIFT_TABLES }),
        "KKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5819), tables: &REDSHIFT_TABLES }),
        "KafkaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5820), tables: &REDSHIFT_TABLES }),
        "KeepKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5821), tables: &REDSHIFT_TABLES }),
        "KeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5822), tables: &REDSHIFT_TABLES }),
        "Key_memberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5823), tables: &REDSHIFT_TABLES }),
        "Key_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5824), tables: &REDSHIFT_TABLES }),
        "KeysKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5825), tables: &REDSHIFT_TABLES }),
        "KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5826), tables: &REDSHIFT_TABLES }),
        "KinesisKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5827), tables: &REDSHIFT_TABLES }),
        "KmeansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5828), tables: &REDSHIFT_TABLES }),
        "Kms_key_idKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5829), tables: &REDSHIFT_TABLES }),
        "LabelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5830), tables: &REDSHIFT_TABLES }),
        "LagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5831), tables: &REDSHIFT_TABLES }),
        "LambdaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5832), tables: &REDSHIFT_TABLES }),
        "LanguageClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5833), tables: &REDSHIFT_TABLES }),
        "LanguageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5838), tables: &REDSHIFT_TABLES }),
        "LargeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5839), tables: &REDSHIFT_TABLES }),
        "LastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5840), tables: &REDSHIFT_TABLES }),
        "Last_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5841), tables: &REDSHIFT_TABLES }),
        "LateralColumnAliasExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5842), tables: &REDSHIFT_TABLES }),
        "LateralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5845), tables: &REDSHIFT_TABLES }),
        "LeadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5846), tables: &REDSHIFT_TABLES }),
        "LeadingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5847), tables: &REDSHIFT_TABLES }),
        "LeakproofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5848), tables: &REDSHIFT_TABLES }),
        "LeastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5849), tables: &REDSHIFT_TABLES }),
        "LeftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5850), tables: &REDSHIFT_TABLES }),
        "LengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5851), tables: &REDSHIFT_TABLES }),
        "LessThanOrEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(5852), tables: &REDSHIFT_TABLES }),
        "LessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(5855), tables: &REDSHIFT_TABLES }),
        "LevelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5856), tables: &REDSHIFT_TABLES }),
        "LibraryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5857), tables: &REDSHIFT_TABLES }),
        "LikeExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5858), tables: &REDSHIFT_TABLES }),
        "LikeGrammar" => Some(RootGrammar { grammar_id: GrammarId(5866), tables: &REDSHIFT_TABLES }),
        "LikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5872), tables: &REDSHIFT_TABLES }),
        "LikeOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(5873), tables: &REDSHIFT_TABLES }),
        "LikeOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(5874), tables: &REDSHIFT_TABLES }),
        "Like_regexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5879), tables: &REDSHIFT_TABLES }),
        "LimitClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(5880), tables: &REDSHIFT_TABLES }),
        "LimitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5897), tables: &REDSHIFT_TABLES }),
        "LinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5898), tables: &REDSHIFT_TABLES }),
        "LinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5899), tables: &REDSHIFT_TABLES }),
        "ListComprehensionGrammar" => Some(RootGrammar { grammar_id: GrammarId(5900), tables: &REDSHIFT_TABLES }),
        "ListKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5901), tables: &REDSHIFT_TABLES }),
        "ListaggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5902), tables: &REDSHIFT_TABLES }),
        "ListenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5903), tables: &REDSHIFT_TABLES }),
        "ListenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5904), tables: &REDSHIFT_TABLES }),
        "LiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(5907), tables: &REDSHIFT_TABLES }),
        "LiteralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5919), tables: &REDSHIFT_TABLES }),
        "LiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5920), tables: &REDSHIFT_TABLES }),
        "LnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5921), tables: &REDSHIFT_TABLES }),
        "LoadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5922), tables: &REDSHIFT_TABLES }),
        "LoadStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5923), tables: &REDSHIFT_TABLES }),
        "LocalAliasSegment" => Some(RootGrammar { grammar_id: GrammarId(5926), tables: &REDSHIFT_TABLES }),
        "LocalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5927), tables: &REDSHIFT_TABLES }),
        "LocaltimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5928), tables: &REDSHIFT_TABLES }),
        "LocaltimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5929), tables: &REDSHIFT_TABLES }),
        "LocationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5930), tables: &REDSHIFT_TABLES }),
        "LocatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5931), tables: &REDSHIFT_TABLES }),
        "LockKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5932), tables: &REDSHIFT_TABLES }),
        "LockTableStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5933), tables: &REDSHIFT_TABLES }),
        "LockedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5938), tables: &REDSHIFT_TABLES }),
        "Log10KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5939), tables: &REDSHIFT_TABLES }),
        "LogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5940), tables: &REDSHIFT_TABLES }),
        "LoggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5941), tables: &REDSHIFT_TABLES }),
        "LoginKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5942), tables: &REDSHIFT_TABLES }),
        "LowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5943), tables: &REDSHIFT_TABLES }),
        "LunKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5944), tables: &REDSHIFT_TABLES }),
        "LunsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5945), tables: &REDSHIFT_TABLES }),
        "LzoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5946), tables: &REDSHIFT_TABLES }),
        "LzopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5947), tables: &REDSHIFT_TABLES }),
        "MKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5948), tables: &REDSHIFT_TABLES }),
        "MLTableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(5949), tables: &REDSHIFT_TABLES }),
        "MainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5961), tables: &REDSHIFT_TABLES }),
        "ManageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5962), tables: &REDSHIFT_TABLES }),
        "ManageRlsPolicyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(5963), tables: &REDSHIFT_TABLES }),
        "ManifestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5983), tables: &REDSHIFT_TABLES }),
        "MapKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5984), tables: &REDSHIFT_TABLES }),
        "MapTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(5985), tables: &REDSHIFT_TABLES }),
        "MappingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5986), tables: &REDSHIFT_TABLES }),
        "MaskingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5987), tables: &REDSHIFT_TABLES }),
        "Master_symmetric_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5988), tables: &REDSHIFT_TABLES }),
        "MatchConditionSegment" => Some(RootGrammar { grammar_id: GrammarId(5989), tables: &REDSHIFT_TABLES }),
        "MatchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5990), tables: &REDSHIFT_TABLES }),
        "Match_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5991), tables: &REDSHIFT_TABLES }),
        "Match_recognizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5992), tables: &REDSHIFT_TABLES }),
        "MatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5993), tables: &REDSHIFT_TABLES }),
        "MatchesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5994), tables: &REDSHIFT_TABLES }),
        "MaterializedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5995), tables: &REDSHIFT_TABLES }),
        "MaxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5996), tables: &REDSHIFT_TABLES }),
        "MaxLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(5997), tables: &REDSHIFT_TABLES }),
        "Max_cellsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5998), tables: &REDSHIFT_TABLES }),
        "Max_runtimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(5999), tables: &REDSHIFT_TABLES }),
        "MaxerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6000), tables: &REDSHIFT_TABLES }),
        "MaxfilesizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6001), tables: &REDSHIFT_TABLES }),
        "MaxvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6002), tables: &REDSHIFT_TABLES }),
        "MbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6003), tables: &REDSHIFT_TABLES }),
        "MeasuresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6004), tables: &REDSHIFT_TABLES }),
        "MemberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6005), tables: &REDSHIFT_TABLES }),
        "MergeDeleteClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6006), tables: &REDSHIFT_TABLES }),
        "MergeInsertClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6007), tables: &REDSHIFT_TABLES }),
        "MergeIntoLiteralGrammar" => Some(RootGrammar { grammar_id: GrammarId(6011), tables: &REDSHIFT_TABLES }),
        "MergeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6014), tables: &REDSHIFT_TABLES }),
        "MergeMatchSegment" => Some(RootGrammar { grammar_id: GrammarId(6015), tables: &REDSHIFT_TABLES }),
        "MergeMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6018), tables: &REDSHIFT_TABLES }),
        "MergeNotMatchedBySourceClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6028), tables: &REDSHIFT_TABLES }),
        "MergeNotMatchedClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6044), tables: &REDSHIFT_TABLES }),
        "MergeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6053), tables: &REDSHIFT_TABLES }),
        "MergeUpdateClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6077), tables: &REDSHIFT_TABLES }),
        "Message_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6080), tables: &REDSHIFT_TABLES }),
        "Message_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6081), tables: &REDSHIFT_TABLES }),
        "Message_textKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6082), tables: &REDSHIFT_TABLES }),
        "MetaCommandQueryBufferSegment" => Some(RootGrammar { grammar_id: GrammarId(6083), tables: &REDSHIFT_TABLES }),
        "MetaCommandQueryBufferStatement" => Some(RootGrammar { grammar_id: GrammarId(6084), tables: &REDSHIFT_TABLES }),
        "MetastoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6089), tables: &REDSHIFT_TABLES }),
        "MethodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6090), tables: &REDSHIFT_TABLES }),
        "MillisecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6091), tables: &REDSHIFT_TABLES }),
        "MinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6092), tables: &REDSHIFT_TABLES }),
        "MinusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6093), tables: &REDSHIFT_TABLES }),
        "MinusSegment" => Some(RootGrammar { grammar_id: GrammarId(6094), tables: &REDSHIFT_TABLES }),
        "MinuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6095), tables: &REDSHIFT_TABLES }),
        "MinvalueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6096), tables: &REDSHIFT_TABLES }),
        "MlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6097), tables: &REDSHIFT_TABLES }),
        "MlpKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6098), tables: &REDSHIFT_TABLES }),
        "ModKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6099), tables: &REDSHIFT_TABLES }),
        "ModeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6100), tables: &REDSHIFT_TABLES }),
        "ModelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6101), tables: &REDSHIFT_TABLES }),
        "Model_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6102), tables: &REDSHIFT_TABLES }),
        "ModifiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6103), tables: &REDSHIFT_TABLES }),
        "ModifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6104), tables: &REDSHIFT_TABLES }),
        "ModuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6105), tables: &REDSHIFT_TABLES }),
        "ModuloSegment" => Some(RootGrammar { grammar_id: GrammarId(6106), tables: &REDSHIFT_TABLES }),
        "ModulusKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6107), tables: &REDSHIFT_TABLES }),
        "MonitorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6108), tables: &REDSHIFT_TABLES }),
        "MonthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6109), tables: &REDSHIFT_TABLES }),
        "MoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6110), tables: &REDSHIFT_TABLES }),
        "Mostly16KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6111), tables: &REDSHIFT_TABLES }),
        "Mostly32KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6112), tables: &REDSHIFT_TABLES }),
        "Mostly8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6113), tables: &REDSHIFT_TABLES }),
        "MoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6114), tables: &REDSHIFT_TABLES }),
        "MskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6115), tables: &REDSHIFT_TABLES }),
        "MtlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6116), tables: &REDSHIFT_TABLES }),
        "Multiclass_classificationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6117), tables: &REDSHIFT_TABLES }),
        "MultilineConcatenateDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(6118), tables: &REDSHIFT_TABLES }),
        "MultilineConcatenateNewline" => Some(RootGrammar { grammar_id: GrammarId(6120), tables: &REDSHIFT_TABLES }),
        "MultiplySegment" => Some(RootGrammar { grammar_id: GrammarId(6121), tables: &REDSHIFT_TABLES }),
        "MultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6122), tables: &REDSHIFT_TABLES }),
        "MumpsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6123), tables: &REDSHIFT_TABLES }),
        "MysqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6124), tables: &REDSHIFT_TABLES }),
        "NakedIdentifierFullSegment" => Some(RootGrammar { grammar_id: GrammarId(6125), tables: &REDSHIFT_TABLES }),
        "NakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6126), tables: &REDSHIFT_TABLES }),
        "NameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6127), tables: &REDSHIFT_TABLES }),
        "NamedArgumentSegment" => Some(RootGrammar { grammar_id: GrammarId(6128), tables: &REDSHIFT_TABLES }),
        "NamedWindowExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6135), tables: &REDSHIFT_TABLES }),
        "NamedWindowSegment" => Some(RootGrammar { grammar_id: GrammarId(6142), tables: &REDSHIFT_TABLES }),
        "NamesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6146), tables: &REDSHIFT_TABLES }),
        "NamespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6147), tables: &REDSHIFT_TABLES }),
        "NanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6148), tables: &REDSHIFT_TABLES }),
        "NanLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6149), tables: &REDSHIFT_TABLES }),
        "NationalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6150), tables: &REDSHIFT_TABLES }),
        "NaturalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6151), tables: &REDSHIFT_TABLES }),
        "NaturalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6154), tables: &REDSHIFT_TABLES }),
        "NcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6155), tables: &REDSHIFT_TABLES }),
        "NclobKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6156), tables: &REDSHIFT_TABLES }),
        "NegativeSegment" => Some(RootGrammar { grammar_id: GrammarId(6157), tables: &REDSHIFT_TABLES }),
        "NestedJoinGrammar" => Some(RootGrammar { grammar_id: GrammarId(6158), tables: &REDSHIFT_TABLES }),
        "NestedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6159), tables: &REDSHIFT_TABLES }),
        "NestingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6160), tables: &REDSHIFT_TABLES }),
        "NewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6161), tables: &REDSHIFT_TABLES }),
        "NewlineSegment" => Some(RootGrammar { grammar_id: GrammarId(6162), tables: &REDSHIFT_TABLES }),
        "NextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6163), tables: &REDSHIFT_TABLES }),
        "NfcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6164), tables: &REDSHIFT_TABLES }),
        "NfdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6165), tables: &REDSHIFT_TABLES }),
        "NfkcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6166), tables: &REDSHIFT_TABLES }),
        "NfkdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6167), tables: &REDSHIFT_TABLES }),
        "NilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6168), tables: &REDSHIFT_TABLES }),
        "NoKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6169), tables: &REDSHIFT_TABLES }),
        "NobypassrlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6170), tables: &REDSHIFT_TABLES }),
        "NocacheKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6171), tables: &REDSHIFT_TABLES }),
        "NocreatedbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6172), tables: &REDSHIFT_TABLES }),
        "NocreateroleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6173), tables: &REDSHIFT_TABLES }),
        "NocreateuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6174), tables: &REDSHIFT_TABLES }),
        "NocycleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6175), tables: &REDSHIFT_TABLES }),
        "NoinheritKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6176), tables: &REDSHIFT_TABLES }),
        "NoloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6177), tables: &REDSHIFT_TABLES }),
        "NologinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6178), tables: &REDSHIFT_TABLES }),
        "NonSetSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6179), tables: &REDSHIFT_TABLES }),
        "NonStandardJoinTypeKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6189), tables: &REDSHIFT_TABLES }),
        "NonWithNonSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6190), tables: &REDSHIFT_TABLES }),
        "NonWithSelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6191), tables: &REDSHIFT_TABLES }),
        "NonatomicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6200), tables: &REDSHIFT_TABLES }),
        "NoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6201), tables: &REDSHIFT_TABLES }),
        "NoorderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6202), tables: &REDSHIFT_TABLES }),
        "NoreplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6203), tables: &REDSHIFT_TABLES }),
        "NormalizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6204), tables: &REDSHIFT_TABLES }),
        "NormalizedGrammar" => Some(RootGrammar { grammar_id: GrammarId(6205), tables: &REDSHIFT_TABLES }),
        "NormalizedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6212), tables: &REDSHIFT_TABLES }),
        "NosuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6213), tables: &REDSHIFT_TABLES }),
        "NotEnforcedGrammar" => Some(RootGrammar { grammar_id: GrammarId(6214), tables: &REDSHIFT_TABLES }),
        "NotEqualToSegment" => Some(RootGrammar { grammar_id: GrammarId(6215), tables: &REDSHIFT_TABLES }),
        "NotExtendLeftSegment" => Some(RootGrammar { grammar_id: GrammarId(6222), tables: &REDSHIFT_TABLES }),
        "NotExtendRightSegment" => Some(RootGrammar { grammar_id: GrammarId(6225), tables: &REDSHIFT_TABLES }),
        "NotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6228), tables: &REDSHIFT_TABLES }),
        "NotNullGrammar" => Some(RootGrammar { grammar_id: GrammarId(6229), tables: &REDSHIFT_TABLES }),
        "NotOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6230), tables: &REDSHIFT_TABLES }),
        "NothingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6231), tables: &REDSHIFT_TABLES }),
        "NotifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6232), tables: &REDSHIFT_TABLES }),
        "NotifyStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6233), tables: &REDSHIFT_TABLES }),
        "NotnullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6239), tables: &REDSHIFT_TABLES }),
        "NowaitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6240), tables: &REDSHIFT_TABLES }),
        "Nth_valueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6241), tables: &REDSHIFT_TABLES }),
        "NtileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6242), tables: &REDSHIFT_TABLES }),
        "NullKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6243), tables: &REDSHIFT_TABLES }),
        "NullLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6244), tables: &REDSHIFT_TABLES }),
        "NullableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6245), tables: &REDSHIFT_TABLES }),
        "NullifKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6246), tables: &REDSHIFT_TABLES }),
        "NullsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6247), tables: &REDSHIFT_TABLES }),
        "NumberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6248), tables: &REDSHIFT_TABLES }),
        "NumericKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6249), tables: &REDSHIFT_TABLES }),
        "NumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6250), tables: &REDSHIFT_TABLES }),
        "NvarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6251), tables: &REDSHIFT_TABLES }),
        "ObjectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6252), tables: &REDSHIFT_TABLES }),
        "ObjectLiteralElementSegment" => Some(RootGrammar { grammar_id: GrammarId(6253), tables: &REDSHIFT_TABLES }),
        "ObjectLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6257), tables: &REDSHIFT_TABLES }),
        "ObjectReferenceDelimiterGrammar" => Some(RootGrammar { grammar_id: GrammarId(6261), tables: &REDSHIFT_TABLES }),
        "ObjectReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "ObjectReferenceTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6266), tables: &REDSHIFT_TABLES }),
        "ObjectUnpivotSegment" => Some(RootGrammar { grammar_id: GrammarId(6278), tables: &REDSHIFT_TABLES }),
        "ObjectiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6285), tables: &REDSHIFT_TABLES }),
        "Occurrences_regexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6286), tables: &REDSHIFT_TABLES }),
        "Octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6287), tables: &REDSHIFT_TABLES }),
        "OctetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6288), tables: &REDSHIFT_TABLES }),
        "OfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6289), tables: &REDSHIFT_TABLES }),
        "OffKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6290), tables: &REDSHIFT_TABLES }),
        "OfflineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6291), tables: &REDSHIFT_TABLES }),
        "OffsetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6292), tables: &REDSHIFT_TABLES }),
        "OffsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6297), tables: &REDSHIFT_TABLES }),
        "OidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6298), tables: &REDSHIFT_TABLES }),
        "OidsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6299), tables: &REDSHIFT_TABLES }),
        "OldKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6300), tables: &REDSHIFT_TABLES }),
        "OmitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6301), tables: &REDSHIFT_TABLES }),
        "OnKeywordAsIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6302), tables: &REDSHIFT_TABLES }),
        "OnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6303), tables: &REDSHIFT_TABLES }),
        "OneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6304), tables: &REDSHIFT_TABLES }),
        "OnlyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6305), tables: &REDSHIFT_TABLES }),
        "OpenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6306), tables: &REDSHIFT_TABLES }),
        "OperateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6307), tables: &REDSHIFT_TABLES }),
        "OperationClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "OperatorClassReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "OperatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6308), tables: &REDSHIFT_TABLES }),
        "OptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6309), tables: &REDSHIFT_TABLES }),
        "OptionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6310), tables: &REDSHIFT_TABLES }),
        "OptionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6315), tables: &REDSHIFT_TABLES }),
        "OptionsListGrammar" => Some(RootGrammar { grammar_id: GrammarId(6316), tables: &REDSHIFT_TABLES }),
        "OrKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6321), tables: &REDSHIFT_TABLES }),
        "OrOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6322), tables: &REDSHIFT_TABLES }),
        "OrReplaceGrammar" => Some(RootGrammar { grammar_id: GrammarId(6323), tables: &REDSHIFT_TABLES }),
        "OrcKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6326), tables: &REDSHIFT_TABLES }),
        "OrderByClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6327), tables: &REDSHIFT_TABLES }),
        "OrderByClauseTerminators" => Some(RootGrammar { grammar_id: GrammarId(6350), tables: &REDSHIFT_TABLES }),
        "OrderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6361), tables: &REDSHIFT_TABLES }),
        "OrderNoOrderGrammar" => Some(RootGrammar { grammar_id: GrammarId(6362), tables: &REDSHIFT_TABLES }),
        "OrderingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6365), tables: &REDSHIFT_TABLES }),
        "OrdinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6366), tables: &REDSHIFT_TABLES }),
        "OthersKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6367), tables: &REDSHIFT_TABLES }),
        "OutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6368), tables: &REDSHIFT_TABLES }),
        "OuterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6369), tables: &REDSHIFT_TABLES }),
        "OutputKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6370), tables: &REDSHIFT_TABLES }),
        "OutputformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6371), tables: &REDSHIFT_TABLES }),
        "OverClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6372), tables: &REDSHIFT_TABLES }),
        "OverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6379), tables: &REDSHIFT_TABLES }),
        "OverflowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6380), tables: &REDSHIFT_TABLES }),
        "OverlapSegment" => Some(RootGrammar { grammar_id: GrammarId(6381), tables: &REDSHIFT_TABLES }),
        "OverlapsClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6384), tables: &REDSHIFT_TABLES }),
        "OverlapsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6416), tables: &REDSHIFT_TABLES }),
        "OverlayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6417), tables: &REDSHIFT_TABLES }),
        "OverridingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6418), tables: &REDSHIFT_TABLES }),
        "OverwriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6419), tables: &REDSHIFT_TABLES }),
        "OwnedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6420), tables: &REDSHIFT_TABLES }),
        "OwnerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6421), tables: &REDSHIFT_TABLES }),
        "OwnershipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6422), tables: &REDSHIFT_TABLES }),
        "PKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6423), tables: &REDSHIFT_TABLES }),
        "PadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6424), tables: &REDSHIFT_TABLES }),
        "ParallelKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6425), tables: &REDSHIFT_TABLES }),
        "ParameterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6426), tables: &REDSHIFT_TABLES }),
        "ParameterNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6427), tables: &REDSHIFT_TABLES }),
        "ParameterSegment" => Some(RootGrammar { grammar_id: GrammarId(6428), tables: &REDSHIFT_TABLES }),
        "Parameter_modeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6429), tables: &REDSHIFT_TABLES }),
        "Parameter_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6430), tables: &REDSHIFT_TABLES }),
        "Parameter_ordinal_positionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6431), tables: &REDSHIFT_TABLES }),
        "Parameter_specific_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6432), tables: &REDSHIFT_TABLES }),
        "Parameter_specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6433), tables: &REDSHIFT_TABLES }),
        "Parameter_specific_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6434), tables: &REDSHIFT_TABLES }),
        "ParquetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6435), tables: &REDSHIFT_TABLES }),
        "ParserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6436), tables: &REDSHIFT_TABLES }),
        "PartialKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6437), tables: &REDSHIFT_TABLES }),
        "PartitionBoundSpecSegment" => Some(RootGrammar { grammar_id: GrammarId(6438), tables: &REDSHIFT_TABLES }),
        "PartitionClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6468), tables: &REDSHIFT_TABLES }),
        "PartitionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6475), tables: &REDSHIFT_TABLES }),
        "PartitionedBySegment" => Some(RootGrammar { grammar_id: GrammarId(6476), tables: &REDSHIFT_TABLES }),
        "PartitionedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6484), tables: &REDSHIFT_TABLES }),
        "PascalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6485), tables: &REDSHIFT_TABLES }),
        "PassKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6486), tables: &REDSHIFT_TABLES }),
        "PassingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6487), tables: &REDSHIFT_TABLES }),
        "PassthroughKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6488), tables: &REDSHIFT_TABLES }),
        "PasswordKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6489), tables: &REDSHIFT_TABLES }),
        "PastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6490), tables: &REDSHIFT_TABLES }),
        "PathKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6491), tables: &REDSHIFT_TABLES }),
        "PathSegment" => Some(RootGrammar { grammar_id: GrammarId(6492), tables: &REDSHIFT_TABLES }),
        "PatternKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6499), tables: &REDSHIFT_TABLES }),
        "PatternMatchingGrammar" => Some(RootGrammar { grammar_id: GrammarId(6500), tables: &REDSHIFT_TABLES }),
        "PerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6501), tables: &REDSHIFT_TABLES }),
        "PercentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6502), tables: &REDSHIFT_TABLES }),
        "Percent_rankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6503), tables: &REDSHIFT_TABLES }),
        "Percentile_contKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6504), tables: &REDSHIFT_TABLES }),
        "Percentile_discKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6505), tables: &REDSHIFT_TABLES }),
        "PeriodKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6506), tables: &REDSHIFT_TABLES }),
        "PermissionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6507), tables: &REDSHIFT_TABLES }),
        "PermissionsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6508), tables: &REDSHIFT_TABLES }),
        "PermuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6509), tables: &REDSHIFT_TABLES }),
        "PgTrgmOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6510), tables: &REDSHIFT_TABLES }),
        "PgvectorOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6511), tables: &REDSHIFT_TABLES }),
        "PipeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6512), tables: &REDSHIFT_TABLES }),
        "PipeSegment" => Some(RootGrammar { grammar_id: GrammarId(6513), tables: &REDSHIFT_TABLES }),
        "PivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6514), tables: &REDSHIFT_TABLES }),
        "PlacingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6515), tables: &REDSHIFT_TABLES }),
        "PlainKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6516), tables: &REDSHIFT_TABLES }),
        "PlanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6517), tables: &REDSHIFT_TABLES }),
        "PlansKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6518), tables: &REDSHIFT_TABLES }),
        "PliKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6519), tables: &REDSHIFT_TABLES }),
        "PlpythonuKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6520), tables: &REDSHIFT_TABLES }),
        "PlusSegment" => Some(RootGrammar { grammar_id: GrammarId(6521), tables: &REDSHIFT_TABLES }),
        "PolicyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6522), tables: &REDSHIFT_TABLES }),
        "PortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6523), tables: &REDSHIFT_TABLES }),
        "PortionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6524), tables: &REDSHIFT_TABLES }),
        "PositionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6525), tables: &REDSHIFT_TABLES }),
        "Position_regexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6526), tables: &REDSHIFT_TABLES }),
        "PositiveSegment" => Some(RootGrammar { grammar_id: GrammarId(6527), tables: &REDSHIFT_TABLES }),
        "PostFunctionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6528), tables: &REDSHIFT_TABLES }),
        "PostTableExpressionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6532), tables: &REDSHIFT_TABLES }),
        "PostgisOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6533), tables: &REDSHIFT_TABLES }),
        "PostgresCopyStdinDataSegment" => Some(RootGrammar { grammar_id: GrammarId(6534), tables: &REDSHIFT_TABLES }),
        "PostgresCopyStdinDataStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6535), tables: &REDSHIFT_TABLES }),
        "PostgresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6537), tables: &REDSHIFT_TABLES }),
        "PowerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6538), tables: &REDSHIFT_TABLES }),
        "PreTableFunctionKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(6539), tables: &REDSHIFT_TABLES }),
        "PrecedesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6541), tables: &REDSHIFT_TABLES }),
        "PrecedingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6542), tables: &REDSHIFT_TABLES }),
        "PrecisionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6543), tables: &REDSHIFT_TABLES }),
        "PrepareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6544), tables: &REDSHIFT_TABLES }),
        "PrepareStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6545), tables: &REDSHIFT_TABLES }),
        "PreparedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6553), tables: &REDSHIFT_TABLES }),
        "PreprocessorsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6554), tables: &REDSHIFT_TABLES }),
        "PreserveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6555), tables: &REDSHIFT_TABLES }),
        "PresetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6556), tables: &REDSHIFT_TABLES }),
        "PrimaryKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(6557), tables: &REDSHIFT_TABLES }),
        "PrimaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6560), tables: &REDSHIFT_TABLES }),
        "PriorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6561), tables: &REDSHIFT_TABLES }),
        "PrivateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6562), tables: &REDSHIFT_TABLES }),
        "PrivilegesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6563), tables: &REDSHIFT_TABLES }),
        "Problem_typeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6564), tables: &REDSHIFT_TABLES }),
        "ProceduralKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6565), tables: &REDSHIFT_TABLES }),
        "ProcedureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6566), tables: &REDSHIFT_TABLES }),
        "ProcedureParameterListSegment" => Some(RootGrammar { grammar_id: GrammarId(6567), tables: &REDSHIFT_TABLES }),
        "ProceduresKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6578), tables: &REDSHIFT_TABLES }),
        "ProgramKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6579), tables: &REDSHIFT_TABLES }),
        "PropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6580), tables: &REDSHIFT_TABLES }),
        "PropertiesNakedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6581), tables: &REDSHIFT_TABLES }),
        "PruneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6582), tables: &REDSHIFT_TABLES }),
        "PsqlCopyMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6583), tables: &REDSHIFT_TABLES }),
        "PsqlCopyMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6584), tables: &REDSHIFT_TABLES }),
        "PsqlSetMetaCommandSegment" => Some(RootGrammar { grammar_id: GrammarId(6586), tables: &REDSHIFT_TABLES }),
        "PsqlSetMetaCommandStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6587), tables: &REDSHIFT_TABLES }),
        "PsqlVariableColonSegment" => Some(RootGrammar { grammar_id: GrammarId(6589), tables: &REDSHIFT_TABLES }),
        "PsqlVariableGrammar" => Some(RootGrammar { grammar_id: GrammarId(6590), tables: &REDSHIFT_TABLES }),
        "PtfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6598), tables: &REDSHIFT_TABLES }),
        "PublicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6599), tables: &REDSHIFT_TABLES }),
        "PublicaccessibleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6600), tables: &REDSHIFT_TABLES }),
        "PublicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6601), tables: &REDSHIFT_TABLES }),
        "PublicationObjectsSegment" => Some(RootGrammar { grammar_id: GrammarId(6602), tables: &REDSHIFT_TABLES }),
        "PublicationReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6625), tables: &REDSHIFT_TABLES }),
        "PublicationTableSegment" => Some(RootGrammar { grammar_id: GrammarId(6626), tables: &REDSHIFT_TABLES }),
        "QualifiedNumericLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6633), tables: &REDSHIFT_TABLES }),
        "QualifiedOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(6636), tables: &REDSHIFT_TABLES }),
        "QualifiedParameterNameGrammar" => Some(RootGrammar { grammar_id: GrammarId(6644), tables: &REDSHIFT_TABLES }),
        "QualifyClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(6649), tables: &REDSHIFT_TABLES }),
        "QualifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6652), tables: &REDSHIFT_TABLES }),
        "QuantifiedComparisonOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(6653), tables: &REDSHIFT_TABLES }),
        "QuarterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6661), tables: &REDSHIFT_TABLES }),
        "Query_all_statesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6662), tables: &REDSHIFT_TABLES }),
        "QuotaGrammar" => Some(RootGrammar { grammar_id: GrammarId(6663), tables: &REDSHIFT_TABLES }),
        "QuotaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6673), tables: &REDSHIFT_TABLES }),
        "QuoteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6674), tables: &REDSHIFT_TABLES }),
        "QuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(6675), tables: &REDSHIFT_TABLES }),
        "QuotedLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(6678), tables: &REDSHIFT_TABLES }),
        "QuotesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6705), tables: &REDSHIFT_TABLES }),
        "RangeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6706), tables: &REDSHIFT_TABLES }),
        "RankKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6707), tables: &REDSHIFT_TABLES }),
        "RawEqualsSegment" => Some(RootGrammar { grammar_id: GrammarId(6708), tables: &REDSHIFT_TABLES }),
        "RawGreaterThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6709), tables: &REDSHIFT_TABLES }),
        "RawKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6710), tables: &REDSHIFT_TABLES }),
        "RawLessThanSegment" => Some(RootGrammar { grammar_id: GrammarId(6711), tables: &REDSHIFT_TABLES }),
        "RawNotSegment" => Some(RootGrammar { grammar_id: GrammarId(6712), tables: &REDSHIFT_TABLES }),
        "RcfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6713), tables: &REDSHIFT_TABLES }),
        "ReadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6714), tables: &REDSHIFT_TABLES }),
        "ReadratioKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6715), tables: &REDSHIFT_TABLES }),
        "ReadsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6716), tables: &REDSHIFT_TABLES }),
        "RealKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6717), tables: &REDSHIFT_TABLES }),
        "ReassignKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6718), tables: &REDSHIFT_TABLES }),
        "ReassignOwnedStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6719), tables: &REDSHIFT_TABLES }),
        "RecheckKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6735), tables: &REDSHIFT_TABLES }),
        "ReclusterKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6736), tables: &REDSHIFT_TABLES }),
        "RecoverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6737), tables: &REDSHIFT_TABLES }),
        "RecoveryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6738), tables: &REDSHIFT_TABLES }),
        "RecursiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6739), tables: &REDSHIFT_TABLES }),
        "RedshiftGrantTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(6740), tables: &REDSHIFT_TABLES }),
        "RedshiftGroupGrantTargetSegment" => Some(RootGrammar { grammar_id: GrammarId(6749), tables: &REDSHIFT_TABLES }),
        "RedshiftKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6752), tables: &REDSHIFT_TABLES }),
        "RefKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6753), tables: &REDSHIFT_TABLES }),
        "RefcursorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6754), tables: &REDSHIFT_TABLES }),
        "ReferenceDefinitionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6755), tables: &REDSHIFT_TABLES }),
        "ReferenceMatchGrammar" => Some(RootGrammar { grammar_id: GrammarId(6769), tables: &REDSHIFT_TABLES }),
        "Reference_usageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6775), tables: &REDSHIFT_TABLES }),
        "ReferencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6776), tables: &REDSHIFT_TABLES }),
        "ReferencingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6777), tables: &REDSHIFT_TABLES }),
        "ReferentialActionGrammar" => Some(RootGrammar { grammar_id: GrammarId(6778), tables: &REDSHIFT_TABLES }),
        "RefreshKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6792), tables: &REDSHIFT_TABLES }),
        "RefreshMaterializedViewStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6793), tables: &REDSHIFT_TABLES }),
        "Refresh_intervalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6800), tables: &REDSHIFT_TABLES }),
        "RegionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6801), tables: &REDSHIFT_TABLES }),
        "Regr_avgxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6802), tables: &REDSHIFT_TABLES }),
        "Regr_avgyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6803), tables: &REDSHIFT_TABLES }),
        "Regr_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6804), tables: &REDSHIFT_TABLES }),
        "Regr_interceptKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6805), tables: &REDSHIFT_TABLES }),
        "Regr_r2KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6806), tables: &REDSHIFT_TABLES }),
        "Regr_slopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6807), tables: &REDSHIFT_TABLES }),
        "Regr_sxxKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6808), tables: &REDSHIFT_TABLES }),
        "Regr_sxyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6809), tables: &REDSHIFT_TABLES }),
        "Regr_syyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6810), tables: &REDSHIFT_TABLES }),
        "RegressionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6811), tables: &REDSHIFT_TABLES }),
        "ReindexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6812), tables: &REDSHIFT_TABLES }),
        "ReindexStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6813), tables: &REDSHIFT_TABLES }),
        "RejectlogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6846), tables: &REDSHIFT_TABLES }),
        "RelationOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(6847), tables: &REDSHIFT_TABLES }),
        "RelationOptionsSegment" => Some(RootGrammar { grammar_id: GrammarId(6855), tables: &REDSHIFT_TABLES }),
        "RelativeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6858), tables: &REDSHIFT_TABLES }),
        "ReleaseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6859), tables: &REDSHIFT_TABLES }),
        "RemainderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6860), tables: &REDSHIFT_TABLES }),
        "RemoveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6861), tables: &REDSHIFT_TABLES }),
        "RemovequotesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6862), tables: &REDSHIFT_TABLES }),
        "RenameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6863), tables: &REDSHIFT_TABLES }),
        "RepeatableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6864), tables: &REDSHIFT_TABLES }),
        "ReplaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6865), tables: &REDSHIFT_TABLES }),
        "ReplicaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6866), tables: &REDSHIFT_TABLES }),
        "ReplicationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6867), tables: &REDSHIFT_TABLES }),
        "RequiringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6868), tables: &REDSHIFT_TABLES }),
        "ResetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6869), tables: &REDSHIFT_TABLES }),
        "ResetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6870), tables: &REDSHIFT_TABLES }),
        "ResetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6874), tables: &REDSHIFT_TABLES }),
        "ResortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6880), tables: &REDSHIFT_TABLES }),
        "ResourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6881), tables: &REDSHIFT_TABLES }),
        "RespectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6882), tables: &REDSHIFT_TABLES }),
        "RestartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6883), tables: &REDSHIFT_TABLES }),
        "RestoreKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6884), tables: &REDSHIFT_TABLES }),
        "RestrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6885), tables: &REDSHIFT_TABLES }),
        "RestrictedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6886), tables: &REDSHIFT_TABLES }),
        "ResultKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6887), tables: &REDSHIFT_TABLES }),
        "Retry_timeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6888), tables: &REDSHIFT_TABLES }),
        "ReturnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6889), tables: &REDSHIFT_TABLES }),
        "Returned_cardinalityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6890), tables: &REDSHIFT_TABLES }),
        "Returned_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6891), tables: &REDSHIFT_TABLES }),
        "Returned_octet_lengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6892), tables: &REDSHIFT_TABLES }),
        "Returned_sqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6893), tables: &REDSHIFT_TABLES }),
        "ReturningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6894), tables: &REDSHIFT_TABLES }),
        "ReturnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6895), tables: &REDSHIFT_TABLES }),
        "RevokeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6896), tables: &REDSHIFT_TABLES }),
        "RevokeStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6897), tables: &REDSHIFT_TABLES }),
        "RightArrowSegment" => Some(RootGrammar { grammar_id: GrammarId(6920), tables: &REDSHIFT_TABLES }),
        "RightKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6921), tables: &REDSHIFT_TABLES }),
        "RlikeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6922), tables: &REDSHIFT_TABLES }),
        "RlsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6923), tables: &REDSHIFT_TABLES }),
        "RoleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6924), tables: &REDSHIFT_TABLES }),
        "RoleReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(6925), tables: &REDSHIFT_TABLES }),
        "RollbackKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6926), tables: &REDSHIFT_TABLES }),
        "RollupFunctionNameSegment" => Some(RootGrammar { grammar_id: GrammarId(6927), tables: &REDSHIFT_TABLES }),
        "RollupKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6928), tables: &REDSHIFT_TABLES }),
        "RoundecKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6929), tables: &REDSHIFT_TABLES }),
        "RoutineKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6930), tables: &REDSHIFT_TABLES }),
        "Routine_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6931), tables: &REDSHIFT_TABLES }),
        "Routine_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6932), tables: &REDSHIFT_TABLES }),
        "Routine_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6933), tables: &REDSHIFT_TABLES }),
        "RoutinesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6934), tables: &REDSHIFT_TABLES }),
        "RowFormatDelimitedSegment" => Some(RootGrammar { grammar_id: GrammarId(6935), tables: &REDSHIFT_TABLES }),
        "RowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6946), tables: &REDSHIFT_TABLES }),
        "Row_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6947), tables: &REDSHIFT_TABLES }),
        "Row_numberKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6948), tables: &REDSHIFT_TABLES }),
        "RowgroupsizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6949), tables: &REDSHIFT_TABLES }),
        "RowsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6950), tables: &REDSHIFT_TABLES }),
        "RuleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6951), tables: &REDSHIFT_TABLES }),
        "RunlengthKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6952), tables: &REDSHIFT_TABLES }),
        "RunningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6953), tables: &REDSHIFT_TABLES }),
        "S3_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6954), tables: &REDSHIFT_TABLES }),
        "S3_garbage_collectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6955), tables: &REDSHIFT_TABLES }),
        "SafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6956), tables: &REDSHIFT_TABLES }),
        "SagemakerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6957), tables: &REDSHIFT_TABLES }),
        "SamplingExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(6958), tables: &REDSHIFT_TABLES }),
        "SavepointKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6970), tables: &REDSHIFT_TABLES }),
        "ScalarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6971), tables: &REDSHIFT_TABLES }),
        "ScaleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6972), tables: &REDSHIFT_TABLES }),
        "SchemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6973), tables: &REDSHIFT_TABLES }),
        "SchemaReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "Schema_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6974), tables: &REDSHIFT_TABLES }),
        "SchemasKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6975), tables: &REDSHIFT_TABLES }),
        "ScopeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6976), tables: &REDSHIFT_TABLES }),
        "Scope_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6977), tables: &REDSHIFT_TABLES }),
        "Scope_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6978), tables: &REDSHIFT_TABLES }),
        "Scope_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6979), tables: &REDSHIFT_TABLES }),
        "ScrollKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6980), tables: &REDSHIFT_TABLES }),
        "SearchKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6981), tables: &REDSHIFT_TABLES }),
        "SecondKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6982), tables: &REDSHIFT_TABLES }),
        "Secret_access_keyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6983), tables: &REDSHIFT_TABLES }),
        "Secret_arnKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6984), tables: &REDSHIFT_TABLES }),
        "SectionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6985), tables: &REDSHIFT_TABLES }),
        "SecurityKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(6986), tables: &REDSHIFT_TABLES }),
        "SecurityLabelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(6987), tables: &REDSHIFT_TABLES }),
        "SeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7078), tables: &REDSHIFT_TABLES }),
        "SelectClauseElementSegment" => Some(RootGrammar { grammar_id: GrammarId(7079), tables: &REDSHIFT_TABLES }),
        "SelectClauseModifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7084), tables: &REDSHIFT_TABLES }),
        "SelectClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7096), tables: &REDSHIFT_TABLES }),
        "SelectClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7120), tables: &REDSHIFT_TABLES }),
        "SelectKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7133), tables: &REDSHIFT_TABLES }),
        "SelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7134), tables: &REDSHIFT_TABLES }),
        "SelectableGrammar" => Some(RootGrammar { grammar_id: GrammarId(7160), tables: &REDSHIFT_TABLES }),
        "SelectiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7170), tables: &REDSHIFT_TABLES }),
        "SelfKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7171), tables: &REDSHIFT_TABLES }),
        "SemiStructuredAccessorSegment" => Some(RootGrammar { grammar_id: GrammarId(7172), tables: &REDSHIFT_TABLES }),
        "SemicolonSegment" => Some(RootGrammar { grammar_id: GrammarId(7181), tables: &REDSHIFT_TABLES }),
        "SensitiveKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7182), tables: &REDSHIFT_TABLES }),
        "SeparatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7183), tables: &REDSHIFT_TABLES }),
        "SequenceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7184), tables: &REDSHIFT_TABLES }),
        "SequenceMaxValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(7185), tables: &REDSHIFT_TABLES }),
        "SequenceMinValueGrammar" => Some(RootGrammar { grammar_id: GrammarId(7192), tables: &REDSHIFT_TABLES }),
        "SequenceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "SequencefileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7199), tables: &REDSHIFT_TABLES }),
        "SequencesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7200), tables: &REDSHIFT_TABLES }),
        "SerdeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7201), tables: &REDSHIFT_TABLES }),
        "SerdepropertiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7202), tables: &REDSHIFT_TABLES }),
        "SerializableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7203), tables: &REDSHIFT_TABLES }),
        "ServerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7204), tables: &REDSHIFT_TABLES }),
        "ServerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "Server_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7205), tables: &REDSHIFT_TABLES }),
        "SessionInformationUserFunctionsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7206), tables: &REDSHIFT_TABLES }),
        "SessionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7211), tables: &REDSHIFT_TABLES }),
        "Session_tokenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7212), tables: &REDSHIFT_TABLES }),
        "Session_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7213), tables: &REDSHIFT_TABLES }),
        "SetClauseListSegment" => Some(RootGrammar { grammar_id: GrammarId(7214), tables: &REDSHIFT_TABLES }),
        "SetClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7218), tables: &REDSHIFT_TABLES }),
        "SetConstraintsStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7252), tables: &REDSHIFT_TABLES }),
        "SetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7262), tables: &REDSHIFT_TABLES }),
        "SetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7271), tables: &REDSHIFT_TABLES }),
        "SetOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(7272), tables: &REDSHIFT_TABLES }),
        "SetSchemaStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7277), tables: &REDSHIFT_TABLES }),
        "SetSessionAuthorizationStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7282), tables: &REDSHIFT_TABLES }),
        "SetStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7295), tables: &REDSHIFT_TABLES }),
        "SetofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7329), tables: &REDSHIFT_TABLES }),
        "SetsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7330), tables: &REDSHIFT_TABLES }),
        "SettingsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7331), tables: &REDSHIFT_TABLES }),
        "ShapefileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7332), tables: &REDSHIFT_TABLES }),
        "ShareKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7333), tables: &REDSHIFT_TABLES }),
        "ShorthandCastSegment" => Some(RootGrammar { grammar_id: GrammarId(7334), tables: &REDSHIFT_TABLES }),
        "ShowDatasharesStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7343), tables: &REDSHIFT_TABLES }),
        "ShowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7349), tables: &REDSHIFT_TABLES }),
        "ShowModelStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7350), tables: &REDSHIFT_TABLES }),
        "ShowStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7356), tables: &REDSHIFT_TABLES }),
        "SignedSegmentGrammar" => Some(RootGrammar { grammar_id: GrammarId(7366), tables: &REDSHIFT_TABLES }),
        "SimilarKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7369), tables: &REDSHIFT_TABLES }),
        "SimpleGeometryGrammar" => Some(RootGrammar { grammar_id: GrammarId(7370), tables: &REDSHIFT_TABLES }),
        "SimpleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7372), tables: &REDSHIFT_TABLES }),
        "SimplifyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7373), tables: &REDSHIFT_TABLES }),
        "SinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7374), tables: &REDSHIFT_TABLES }),
        "SingleIdentifierFullGrammar" => Some(RootGrammar { grammar_id: GrammarId(7375), tables: &REDSHIFT_TABLES }),
        "SingleIdentifierGrammar" => Some(RootGrammar { grammar_id: GrammarId(7379), tables: &REDSHIFT_TABLES }),
        "SingleIdentifierListSegment" => Some(RootGrammar { grammar_id: GrammarId(7383), tables: &REDSHIFT_TABLES }),
        "SingleIdentifierWildcardSegment" => Some(RootGrammar { grammar_id: GrammarId(7385), tables: &REDSHIFT_TABLES }),
        "SingleQuotedIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(7389), tables: &REDSHIFT_TABLES }),
        "SinhKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7390), tables: &REDSHIFT_TABLES }),
        "SizeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7391), tables: &REDSHIFT_TABLES }),
        "SizedArrayTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(7392), tables: &REDSHIFT_TABLES }),
        "SkipKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7395), tables: &REDSHIFT_TABLES }),
        "SlashSegment" => Some(RootGrammar { grammar_id: GrammarId(7396), tables: &REDSHIFT_TABLES }),
        "SliceSegment" => Some(RootGrammar { grammar_id: GrammarId(7397), tables: &REDSHIFT_TABLES }),
        "SmallintKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7398), tables: &REDSHIFT_TABLES }),
        "SnapshotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7399), tables: &REDSHIFT_TABLES }),
        "SomeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7400), tables: &REDSHIFT_TABLES }),
        "SortKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7401), tables: &REDSHIFT_TABLES }),
        "SortkeyKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7402), tables: &REDSHIFT_TABLES }),
        "SourceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7403), tables: &REDSHIFT_TABLES }),
        "SpaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7404), tables: &REDSHIFT_TABLES }),
        "SpecificKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7405), tables: &REDSHIFT_TABLES }),
        "Specific_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7406), tables: &REDSHIFT_TABLES }),
        "SpecifictypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7407), tables: &REDSHIFT_TABLES }),
        "SqlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7408), tables: &REDSHIFT_TABLES }),
        "SqlcodeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7409), tables: &REDSHIFT_TABLES }),
        "SqlerrorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7410), tables: &REDSHIFT_TABLES }),
        "SqlexceptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7411), tables: &REDSHIFT_TABLES }),
        "SqlstateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7412), tables: &REDSHIFT_TABLES }),
        "SqlwarningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7413), tables: &REDSHIFT_TABLES }),
        "SqrtKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7414), tables: &REDSHIFT_TABLES }),
        "StableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7415), tables: &REDSHIFT_TABLES }),
        "StageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7416), tables: &REDSHIFT_TABLES }),
        "StagesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7417), tables: &REDSHIFT_TABLES }),
        "StandaloneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7418), tables: &REDSHIFT_TABLES }),
        "StarSegment" => Some(RootGrammar { grammar_id: GrammarId(7419), tables: &REDSHIFT_TABLES }),
        "StartBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(241), tables: &REDSHIFT_TABLES }),
        "StartCurlyBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(6260), tables: &REDSHIFT_TABLES }),
        "StartKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7420), tables: &REDSHIFT_TABLES }),
        "StartSquareBracketSegment" => Some(RootGrammar { grammar_id: GrammarId(2155), tables: &REDSHIFT_TABLES }),
        "StateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7421), tables: &REDSHIFT_TABLES }),
        "StatementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7422), tables: &REDSHIFT_TABLES }),
        "StatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7423), tables: &REDSHIFT_TABLES }),
        "StaticKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7572), tables: &REDSHIFT_TABLES }),
        "StatisticsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7573), tables: &REDSHIFT_TABLES }),
        "StatisticsReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "StatupdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7574), tables: &REDSHIFT_TABLES }),
        "Stddev_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7575), tables: &REDSHIFT_TABLES }),
        "Stddev_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7576), tables: &REDSHIFT_TABLES }),
        "StdinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7577), tables: &REDSHIFT_TABLES }),
        "StdoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7578), tables: &REDSHIFT_TABLES }),
        "StorageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7579), tables: &REDSHIFT_TABLES }),
        "StoredKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7580), tables: &REDSHIFT_TABLES }),
        "StreamKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7581), tables: &REDSHIFT_TABLES }),
        "StreamsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7582), tables: &REDSHIFT_TABLES }),
        "StrictKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7583), tables: &REDSHIFT_TABLES }),
        "StringBinaryOperatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(7584), tables: &REDSHIFT_TABLES }),
        "StringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7587), tables: &REDSHIFT_TABLES }),
        "StripKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7588), tables: &REDSHIFT_TABLES }),
        "StructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7589), tables: &REDSHIFT_TABLES }),
        "StructTypeSegment" => Some(RootGrammar { grammar_id: GrammarId(7594), tables: &REDSHIFT_TABLES }),
        "StructureKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7595), tables: &REDSHIFT_TABLES }),
        "StyleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7596), tables: &REDSHIFT_TABLES }),
        "Subclass_originKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7597), tables: &REDSHIFT_TABLES }),
        "SubmultisetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7598), tables: &REDSHIFT_TABLES }),
        "SubscriptionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7599), tables: &REDSHIFT_TABLES }),
        "SubscriptionReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "SubsetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7600), tables: &REDSHIFT_TABLES }),
        "SubstringKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7601), tables: &REDSHIFT_TABLES }),
        "Substring_regexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7602), tables: &REDSHIFT_TABLES }),
        "SucceedsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7603), tables: &REDSHIFT_TABLES }),
        "SumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7604), tables: &REDSHIFT_TABLES }),
        "SuperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7605), tables: &REDSHIFT_TABLES }),
        "SuperuserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7606), tables: &REDSHIFT_TABLES }),
        "SupportKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7607), tables: &REDSHIFT_TABLES }),
        "SymbolSegment" => Some(RootGrammar { grammar_id: GrammarId(7608), tables: &REDSHIFT_TABLES }),
        "SymmetricKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7609), tables: &REDSHIFT_TABLES }),
        "SysdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7610), tables: &REDSHIFT_TABLES }),
        "SysidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7611), tables: &REDSHIFT_TABLES }),
        "SyslogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7612), tables: &REDSHIFT_TABLES }),
        "SystemKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7613), tables: &REDSHIFT_TABLES }),
        "System_timeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7614), tables: &REDSHIFT_TABLES }),
        "System_userKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7615), tables: &REDSHIFT_TABLES }),
        "TKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7616), tables: &REDSHIFT_TABLES }),
        "TableAttributeSegment" => Some(RootGrammar { grammar_id: GrammarId(7617), tables: &REDSHIFT_TABLES }),
        "TableConstraintSegment" => Some(RootGrammar { grammar_id: GrammarId(7644), tables: &REDSHIFT_TABLES }),
        "TableConstraintUsingIndexSegment" => Some(RootGrammar { grammar_id: GrammarId(7671), tables: &REDSHIFT_TABLES }),
        "TableEndClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(7694), tables: &REDSHIFT_TABLES }),
        "TableExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7695), tables: &REDSHIFT_TABLES }),
        "TableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7706), tables: &REDSHIFT_TABLES }),
        "TableReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "Table_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7707), tables: &REDSHIFT_TABLES }),
        "TablesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7708), tables: &REDSHIFT_TABLES }),
        "TablesampleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7709), tables: &REDSHIFT_TABLES }),
        "TablespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7710), tables: &REDSHIFT_TABLES }),
        "TablespaceReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "TagKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7711), tables: &REDSHIFT_TABLES }),
        "TagReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "Tail_Recurse_Expression_A_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7712), tables: &REDSHIFT_TABLES }),
        "Tail_Recurse_Expression_B_Grammar" => Some(RootGrammar { grammar_id: GrammarId(7717), tables: &REDSHIFT_TABLES }),
        "TanKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7721), tables: &REDSHIFT_TABLES }),
        "TanhKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7722), tables: &REDSHIFT_TABLES }),
        "TargetKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7723), tables: &REDSHIFT_TABLES }),
        "TaskKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7724), tables: &REDSHIFT_TABLES }),
        "TasksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7725), tables: &REDSHIFT_TABLES }),
        "TbKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7726), tables: &REDSHIFT_TABLES }),
        "TdesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7727), tables: &REDSHIFT_TABLES }),
        "TempKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7728), tables: &REDSHIFT_TABLES }),
        "TemplateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7729), tables: &REDSHIFT_TABLES }),
        "TemporalQuerySegment" => Some(RootGrammar { grammar_id: GrammarId(7730), tables: &REDSHIFT_TABLES }),
        "TemporaryGrammar" => Some(RootGrammar { grammar_id: GrammarId(7731), tables: &REDSHIFT_TABLES }),
        "TemporaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7734), tables: &REDSHIFT_TABLES }),
        "TemporaryTransientGrammar" => Some(RootGrammar { grammar_id: GrammarId(7735), tables: &REDSHIFT_TABLES }),
        "TerminatedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7738), tables: &REDSHIFT_TABLES }),
        "Text255KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7739), tables: &REDSHIFT_TABLES }),
        "Text32kKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7740), tables: &REDSHIFT_TABLES }),
        "TextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7741), tables: &REDSHIFT_TABLES }),
        "TextfileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7742), tables: &REDSHIFT_TABLES }),
        "ThenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7743), tables: &REDSHIFT_TABLES }),
        "ThroughKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7744), tables: &REDSHIFT_TABLES }),
        "TiesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7745), tables: &REDSHIFT_TABLES }),
        "TildeSegment" => Some(RootGrammar { grammar_id: GrammarId(7746), tables: &REDSHIFT_TABLES }),
        "TimeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7747), tables: &REDSHIFT_TABLES }),
        "TimeWithTZGrammar" => Some(RootGrammar { grammar_id: GrammarId(7748), tables: &REDSHIFT_TABLES }),
        "TimeZoneGrammar" => Some(RootGrammar { grammar_id: GrammarId(7759), tables: &REDSHIFT_TABLES }),
        "TimeformatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7765), tables: &REDSHIFT_TABLES }),
        "TimeoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7766), tables: &REDSHIFT_TABLES }),
        "TimestampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7767), tables: &REDSHIFT_TABLES }),
        "TimestamptzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7768), tables: &REDSHIFT_TABLES }),
        "TimetzKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7769), tables: &REDSHIFT_TABLES }),
        "Timezone_hourKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7770), tables: &REDSHIFT_TABLES }),
        "Timezone_minuteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7771), tables: &REDSHIFT_TABLES }),
        "ToKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7772), tables: &REDSHIFT_TABLES }),
        "TokenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7773), tables: &REDSHIFT_TABLES }),
        "TopKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7774), tables: &REDSHIFT_TABLES }),
        "Top_level_countKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7775), tables: &REDSHIFT_TABLES }),
        "TrailingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7776), tables: &REDSHIFT_TABLES }),
        "TransactionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7777), tables: &REDSHIFT_TABLES }),
        "TransactionStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7778), tables: &REDSHIFT_TABLES }),
        "Transaction_activeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7810), tables: &REDSHIFT_TABLES }),
        "Transactions_committedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7811), tables: &REDSHIFT_TABLES }),
        "Transactions_rolled_backKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7812), tables: &REDSHIFT_TABLES }),
        "TransformKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7813), tables: &REDSHIFT_TABLES }),
        "TransformsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7814), tables: &REDSHIFT_TABLES }),
        "TransientKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7815), tables: &REDSHIFT_TABLES }),
        "TranslateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7816), tables: &REDSHIFT_TABLES }),
        "Translate_regexKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7817), tables: &REDSHIFT_TABLES }),
        "TranslationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7818), tables: &REDSHIFT_TABLES }),
        "TreatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7819), tables: &REDSHIFT_TABLES }),
        "TriggerKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7820), tables: &REDSHIFT_TABLES }),
        "TriggerReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(4092), tables: &REDSHIFT_TABLES }),
        "Trigger_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7821), tables: &REDSHIFT_TABLES }),
        "Trigger_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7822), tables: &REDSHIFT_TABLES }),
        "Trigger_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7823), tables: &REDSHIFT_TABLES }),
        "TrimKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7824), tables: &REDSHIFT_TABLES }),
        "TrimParametersGrammar" => Some(RootGrammar { grammar_id: GrammarId(7825), tables: &REDSHIFT_TABLES }),
        "Trim_arrayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7829), tables: &REDSHIFT_TABLES }),
        "TrimblanksKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7830), tables: &REDSHIFT_TABLES }),
        "TrueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7831), tables: &REDSHIFT_TABLES }),
        "TrueSegment" => Some(RootGrammar { grammar_id: GrammarId(7832), tables: &REDSHIFT_TABLES }),
        "TruncateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7833), tables: &REDSHIFT_TABLES }),
        "TruncateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7834), tables: &REDSHIFT_TABLES }),
        "TruncatecolumnsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7851), tables: &REDSHIFT_TABLES }),
        "TrustedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7852), tables: &REDSHIFT_TABLES }),
        "TupleSegment" => Some(RootGrammar { grammar_id: GrammarId(7853), tables: &REDSHIFT_TABLES }),
        "TypeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7856), tables: &REDSHIFT_TABLES }),
        "TypedArrayLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7857), tables: &REDSHIFT_TABLES }),
        "TypedStructLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7865), tables: &REDSHIFT_TABLES }),
        "TypesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7868), tables: &REDSHIFT_TABLES }),
        "UescapeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7869), tables: &REDSHIFT_TABLES }),
        "UnboundedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7870), tables: &REDSHIFT_TABLES }),
        "UncommittedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7871), tables: &REDSHIFT_TABLES }),
        "UnconditionalCrossJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7872), tables: &REDSHIFT_TABLES }),
        "UnconditionalJoinKeywordsGrammar" => Some(RootGrammar { grammar_id: GrammarId(7873), tables: &REDSHIFT_TABLES }),
        "UnconditionalKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7877), tables: &REDSHIFT_TABLES }),
        "UnderKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7878), tables: &REDSHIFT_TABLES }),
        "UnencryptedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7879), tables: &REDSHIFT_TABLES }),
        "UnionGrammar" => Some(RootGrammar { grammar_id: GrammarId(7880), tables: &REDSHIFT_TABLES }),
        "UnionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7885), tables: &REDSHIFT_TABLES }),
        "UniqueKeyGrammar" => Some(RootGrammar { grammar_id: GrammarId(7886), tables: &REDSHIFT_TABLES }),
        "UniqueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7888), tables: &REDSHIFT_TABLES }),
        "UnknownKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7889), tables: &REDSHIFT_TABLES }),
        "UnknownLiteralSegment" => Some(RootGrammar { grammar_id: GrammarId(7890), tables: &REDSHIFT_TABLES }),
        "UnlimitedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7891), tables: &REDSHIFT_TABLES }),
        "UnlinkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7892), tables: &REDSHIFT_TABLES }),
        "UnlistenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7893), tables: &REDSHIFT_TABLES }),
        "UnlistenStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7894), tables: &REDSHIFT_TABLES }),
        "UnloadKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7899), tables: &REDSHIFT_TABLES }),
        "UnloadStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7900), tables: &REDSHIFT_TABLES }),
        "UnloggedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7981), tables: &REDSHIFT_TABLES }),
        "UnmatchedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7982), tables: &REDSHIFT_TABLES }),
        "UnnamedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7983), tables: &REDSHIFT_TABLES }),
        "UnnestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(7984), tables: &REDSHIFT_TABLES }),
        "UnorderedSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(7985), tables: &REDSHIFT_TABLES }),
        "UnorderedSetExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(7999), tables: &REDSHIFT_TABLES }),
        "UnpivotKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8005), tables: &REDSHIFT_TABLES }),
        "UnrestrictedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8006), tables: &REDSHIFT_TABLES }),
        "UnsafeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8007), tables: &REDSHIFT_TABLES }),
        "UnsignedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8008), tables: &REDSHIFT_TABLES }),
        "UntilKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8009), tables: &REDSHIFT_TABLES }),
        "UntypedKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8010), tables: &REDSHIFT_TABLES }),
        "UpdateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8011), tables: &REDSHIFT_TABLES }),
        "UpdateStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(8012), tables: &REDSHIFT_TABLES }),
        "UpperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8035), tables: &REDSHIFT_TABLES }),
        "UriKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8036), tables: &REDSHIFT_TABLES }),
        "UsageKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8037), tables: &REDSHIFT_TABLES }),
        "UseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8038), tables: &REDSHIFT_TABLES }),
        "UseStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(8039), tables: &REDSHIFT_TABLES }),
        "Use_any_roleKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8042), tables: &REDSHIFT_TABLES }),
        "UserKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8043), tables: &REDSHIFT_TABLES }),
        "UserReferenceSegment" => Some(RootGrammar { grammar_id: GrammarId(8044), tables: &REDSHIFT_TABLES }),
        "User_defined_type_catalogKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8045), tables: &REDSHIFT_TABLES }),
        "User_defined_type_codeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8046), tables: &REDSHIFT_TABLES }),
        "User_defined_type_nameKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8047), tables: &REDSHIFT_TABLES }),
        "User_defined_type_schemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8048), tables: &REDSHIFT_TABLES }),
        "User_limitKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8049), tables: &REDSHIFT_TABLES }),
        "UsingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8050), tables: &REDSHIFT_TABLES }),
        "Utf16KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8051), tables: &REDSHIFT_TABLES }),
        "Utf16beKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8052), tables: &REDSHIFT_TABLES }),
        "Utf16leKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8053), tables: &REDSHIFT_TABLES }),
        "Utf32KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8054), tables: &REDSHIFT_TABLES }),
        "Utf8KeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8055), tables: &REDSHIFT_TABLES }),
        "VacuumKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8056), tables: &REDSHIFT_TABLES }),
        "VacuumStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(8057), tables: &REDSHIFT_TABLES }),
        "ValidKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8074), tables: &REDSHIFT_TABLES }),
        "ValidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8075), tables: &REDSHIFT_TABLES }),
        "ValidatorKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8076), tables: &REDSHIFT_TABLES }),
        "ValueKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8077), tables: &REDSHIFT_TABLES }),
        "Value_ofKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8078), tables: &REDSHIFT_TABLES }),
        "ValuesClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8079), tables: &REDSHIFT_TABLES }),
        "ValuesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8090), tables: &REDSHIFT_TABLES }),
        "Var_popKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8091), tables: &REDSHIFT_TABLES }),
        "Var_sampKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8092), tables: &REDSHIFT_TABLES }),
        "VarbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8093), tables: &REDSHIFT_TABLES }),
        "VarbyteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8094), tables: &REDSHIFT_TABLES }),
        "VarcharKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8095), tables: &REDSHIFT_TABLES }),
        "VariadicKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8096), tables: &REDSHIFT_TABLES }),
        "VaryingKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8097), tables: &REDSHIFT_TABLES }),
        "VerboseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8098), tables: &REDSHIFT_TABLES }),
        "VersionIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(8099), tables: &REDSHIFT_TABLES }),
        "VersionKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8102), tables: &REDSHIFT_TABLES }),
        "VersioningKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8103), tables: &REDSHIFT_TABLES }),
        "ViaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8104), tables: &REDSHIFT_TABLES }),
        "ViewKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8105), tables: &REDSHIFT_TABLES }),
        "ViewsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8106), tables: &REDSHIFT_TABLES }),
        "VolatileKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8107), tables: &REDSHIFT_TABLES }),
        "WalletKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8108), tables: &REDSHIFT_TABLES }),
        "WalrusOperatorSegment" => Some(RootGrammar { grammar_id: GrammarId(8109), tables: &REDSHIFT_TABLES }),
        "WarehouseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8110), tables: &REDSHIFT_TABLES }),
        "WeekKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8111), tables: &REDSHIFT_TABLES }),
        "WeekdayKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8112), tables: &REDSHIFT_TABLES }),
        "WellKnownTextGeometrySegment" => Some(RootGrammar { grammar_id: GrammarId(8113), tables: &REDSHIFT_TABLES }),
        "WhenClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8114), tables: &REDSHIFT_TABLES }),
        "WhenKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8124), tables: &REDSHIFT_TABLES }),
        "WheneverKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8125), tables: &REDSHIFT_TABLES }),
        "WhereClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8126), tables: &REDSHIFT_TABLES }),
        "WhereClauseTerminatorGrammar" => Some(RootGrammar { grammar_id: GrammarId(8131), tables: &REDSHIFT_TABLES }),
        "WhereKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8148), tables: &REDSHIFT_TABLES }),
        "WhitespaceKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8149), tables: &REDSHIFT_TABLES }),
        "WhitespaceSegment" => Some(RootGrammar { grammar_id: GrammarId(8150), tables: &REDSHIFT_TABLES }),
        "Width_bucketKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8151), tables: &REDSHIFT_TABLES }),
        "WildcardExpressionSegment" => Some(RootGrammar { grammar_id: GrammarId(8152), tables: &REDSHIFT_TABLES }),
        "WildcardIdentifierSegment" => Some(RootGrammar { grammar_id: GrammarId(8155), tables: &REDSHIFT_TABLES }),
        "WindowKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8165), tables: &REDSHIFT_TABLES }),
        "WindowSpecificationSegment" => Some(RootGrammar { grammar_id: GrammarId(8166), tables: &REDSHIFT_TABLES }),
        "WithCheckOptionSegment" => Some(RootGrammar { grammar_id: GrammarId(8174), tables: &REDSHIFT_TABLES }),
        "WithCompoundNonSelectStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(8181), tables: &REDSHIFT_TABLES }),
        "WithCompoundStatementSegment" => Some(RootGrammar { grammar_id: GrammarId(8190), tables: &REDSHIFT_TABLES }),
        "WithDataClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8199), tables: &REDSHIFT_TABLES }),
        "WithFillSegment" => Some(RootGrammar { grammar_id: GrammarId(8204), tables: &REDSHIFT_TABLES }),
        "WithKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8205), tables: &REDSHIFT_TABLES }),
        "WithNoSchemaBindingClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8206), tables: &REDSHIFT_TABLES }),
        "WithinGroupClauseSegment" => Some(RootGrammar { grammar_id: GrammarId(8211), tables: &REDSHIFT_TABLES }),
        "WithinKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8216), tables: &REDSHIFT_TABLES }),
        "WithoutKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8217), tables: &REDSHIFT_TABLES }),
        "WordSegment" => Some(RootGrammar { grammar_id: GrammarId(8218), tables: &REDSHIFT_TABLES }),
        "WorkKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8219), tables: &REDSHIFT_TABLES }),
        "WrapperKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8220), tables: &REDSHIFT_TABLES }),
        "WriteKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8221), tables: &REDSHIFT_TABLES }),
        "XgboostKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8222), tables: &REDSHIFT_TABLES }),
        "XmlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8223), tables: &REDSHIFT_TABLES }),
        "XmlaggKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8224), tables: &REDSHIFT_TABLES }),
        "XmlattributesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8225), tables: &REDSHIFT_TABLES }),
        "XmlbinaryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8226), tables: &REDSHIFT_TABLES }),
        "XmlcastKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8227), tables: &REDSHIFT_TABLES }),
        "XmlcommentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8228), tables: &REDSHIFT_TABLES }),
        "XmlconcatKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8229), tables: &REDSHIFT_TABLES }),
        "XmldeclarationKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8230), tables: &REDSHIFT_TABLES }),
        "XmldocumentKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8231), tables: &REDSHIFT_TABLES }),
        "XmlelementKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8232), tables: &REDSHIFT_TABLES }),
        "XmlexistsKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8233), tables: &REDSHIFT_TABLES }),
        "XmlforestKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8234), tables: &REDSHIFT_TABLES }),
        "XmliterateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8235), tables: &REDSHIFT_TABLES }),
        "XmlnamespacesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8236), tables: &REDSHIFT_TABLES }),
        "XmlparseKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8237), tables: &REDSHIFT_TABLES }),
        "XmlpiKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8238), tables: &REDSHIFT_TABLES }),
        "XmlqueryKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8239), tables: &REDSHIFT_TABLES }),
        "XmlrootKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8240), tables: &REDSHIFT_TABLES }),
        "XmlschemaKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8241), tables: &REDSHIFT_TABLES }),
        "XmlserializeKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8242), tables: &REDSHIFT_TABLES }),
        "XmltableKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8243), tables: &REDSHIFT_TABLES }),
        "XmltextKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8244), tables: &REDSHIFT_TABLES }),
        "XmlvalidateKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8245), tables: &REDSHIFT_TABLES }),
        "YamlKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8246), tables: &REDSHIFT_TABLES }),
        "YearKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8247), tables: &REDSHIFT_TABLES }),
        "YesKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8248), tables: &REDSHIFT_TABLES }),
        "ZoneKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8249), tables: &REDSHIFT_TABLES }),
        "ZstdKeywordSegment" => Some(RootGrammar { grammar_id: GrammarId(8250), tables: &REDSHIFT_TABLES }),
        _ => None,
    }
}

pub fn get_redshift_segment_type(name: &str) -> Option<&'static str> {
    match name {
        "AccessObjectSegment" => Some("access_object"),
        "AccessPermissionSegment" => Some("access_permission"),
        "AccessPermissionsSegment" => Some("access_permissions"),
        "AccessSchemaObjectSegment" => Some("access_schema_object"),
        "AccessSchemaPluralObjectSegment" => Some("access_schema_plural_object"),
        "AccessStatementSegment" => Some("access_statement"),
        "AccessTargetSegment" => Some("access_target"),
        "AdjacentSegment" => Some("comparison_operator"),
        "AggregateOrderByClause" => Some("aggregate_order_by"),
        "AliasExpressionSegment" => Some("alias_expression"),
        "AlterAggregateStatementSegment" => Some("alter_aggregate_statement"),
        "AlterDatabaseStatementSegment" => Some("alter_database_statement"),
        "AlterDefaultPrivilegesGrantSegment" => Some("alter_default_privileges_grant"),
        "AlterDefaultPrivilegesObjectPrivilegesSegment" => Some("alter_default_privileges_object_privilege"),
        "AlterDefaultPrivilegesRevokeSegment" => Some("alter_default_privileges_revoke"),
        "AlterDefaultPrivilegesSchemaObjectsSegment" => Some("alter_default_privileges_schema_object"),
        "AlterDefaultPrivilegesStatementSegment" => Some("alter_default_privileges_statement"),
        "AlterDefaultPrivilegesToFromRolesSegment" => Some("alter_default_privileges_to_from_roles"),
        "AlterDomainStatementSegment" => Some("alter_domain_statement"),
        "AlterExtensionStatementSegment" => Some("alter_extension_statement"),
        "AlterForeignTableActionSegment" => Some("alter_foreign_table_action_segment"),
        "AlterForeignTableStatementSegment" => Some("alter_foreign_table_statement"),
        "AlterFunctionActionSegment" => Some("alter_function_action_segment"),
        "AlterFunctionStatementSegment" => Some("alter_function_statement"),
        "AlterGroupStatementSegment" => Some("alter_group"),
        "AlterIndexStatementSegment" => Some("alter_index_statement"),
        "AlterMaterializedViewActionSegment" => Some("alter_materialized_view_action_segment"),
        "AlterMaterializedViewStatementSegment" => Some("alter_materialized_view_statement"),
        "AlterPolicyStatementSegment" => Some("alter_policy_statement"),
        "AlterProcedureActionSegment" => Some("alter_procedure_action_segment"),
        "AlterProcedureStatementSegment" => Some("alter_procedure_statement"),
        "AlterPublicationStatementSegment" => Some("alter_publication_statement"),
        "AlterRoleStatementSegment" => Some("alter_role_statement"),
        "AlterSchemaStatementSegment" => Some("alter_schema_statement"),
        "AlterSequenceOptionsSegment" => Some("alter_sequence_options_segment"),
        "AlterSequenceStatementSegment" => Some("alter_sequence_statement"),
        "AlterStatisticsStatementSegment" => Some("alter_statistics_statement"),
        "AlterSubscriptionStatementSegment" => Some("alter_subscription"),
        "AlterSystemStatementSegment" => Some("alter_system_statement"),
        "AlterTableActionSegment" => Some("alter_table_action_segment"),
        "AlterTableStatementSegment" => Some("alter_table_statement"),
        "AlterTextSearchConfigurationStatementSegment" => Some("alter_text_search_configuration_statement"),
        "AlterTriggerStatementSegment" => Some("alter_trigger"),
        "AlterTypeStatementSegment" => Some("alter_type_statement"),
        "AlterUserStatementSegment" => Some("alter_user_statement"),
        "AlterViewStatementSegment" => Some("alter_view_statement"),
        "AltereDatashareStatementSegment" => Some("create_datashare_statement"),
        "AnalyzeCompressionStatementSegment" => Some("analyze_compression_statement"),
        "AnalyzeStatementSegment" => Some("analyze_statement"),
        "ArnCatalogSchemaSegment" => Some("arn_catalog_schema_segment"),
        "ArrayAccessorSegment" => Some("array_accessor"),
        "ArrayExpressionSegment" => Some("array_expression"),
        "ArrayLiteralSegment" => Some("array_literal"),
        "ArrayTypeSegment" => Some("array_type"),
        "ArrayTypeSuffixSegment" => Some("array_type_suffix"),
        "ArrayUnnestSegment" => Some("array_unnesting"),
        "AsAliasExpressionSegment" => Some("alias_expression"),
        "AsAliasOperatorSegment" => Some("alias_operator"),
        "AuthorizationSegment" => Some("authorization_segment"),
        "BaseFileSegment" => Some("file"),
        "BaseSegment" => Some("base"),
        "BinaryOperatorSegment" => Some("binary_operator"),
        "BitwiseAndSegment" => Some("binary_operator"),
        "BitwiseLShiftSegment" => Some("binary_operator"),
        "BitwiseOrSegment" => Some("binary_operator"),
        "BitwiseRShiftSegment" => Some("binary_operator"),
        "BracketedArguments" => Some("bracketed_arguments"),
        "BracketedSegment" => Some("bracketed"),
        "CTEColumnList" => Some("cte_column_list"),
        "CTEDefinitionSegment" => Some("common_table_expression"),
        "CallStatementSegment" => Some("call_statement"),
        "CallStoredProcedureSegment" => Some("call_statement"),
        "CaseExpressionSegment" => Some("case_expression"),
        "CloseStatementSegment" => Some("close_statement"),
        "ClusterStatementSegment" => Some("cluster_statement"),
        "CodeSegment" => Some("raw"),
        "CollationClauseSegment" => Some("collation_clause"),
        "CollationReferenceSegment" => Some("collation_reference"),
        "ColumnAttributeSegment" => Some("column_attribute_segment"),
        "ColumnConstraintSegment" => Some("column_constraint_segment"),
        "ColumnDefinitionSegment" => Some("column_definition"),
        "ColumnReferenceSegment" => Some("column_reference"),
        "ColumnTypeReferenceSegment" => Some("column_type_reference"),
        "ColumnsExpressionFunctionContentsSegment" => Some("columns_expression"),
        "ColumnsExpressionFunctionNameSegment" => Some("function_name"),
        "CommentClauseSegment" => Some("comment_clause"),
        "CommentOnStatementSegment" => Some("comment_clause"),
        "CommentSegment" => Some("comment"),
        "ComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeBinaryOperatorSegment" => Some("binary_operator"),
        "CompositeComparisonOperatorSegment" => Some("comparison_operator"),
        "CompositeValueExpansionSegment" => Some("composite_value_expansion"),
        "ConcatSegment" => Some("binary_operator"),
        "ConflictActionSegment" => Some("conflict_action"),
        "ConflictTargetSegment" => Some("conflict_target"),
        "ConnectionLimitSegment" => Some("connection_limit_segment"),
        "ConvertFunctionContentsSegment" => Some("function_contents"),
        "ConvertFunctionNameSegment" => Some("function_name"),
        "CopyStatementSegment" => Some("copy_statement"),
        "CreateAggregateStatementSegment" => Some("create_aggregate_statement"),
        "CreateCastStatementSegment" => Some("create_cast_statement"),
        "CreateCollationStatementSegment" => Some("create_collation_statement"),
        "CreateDatabaseStatementSegment" => Some("create_database_statement"),
        "CreateDatabaseWithOptionsSegment" => Some("create_database_with_options"),
        "CreateDatashareStatementSegment" => Some("create_datashare_statement"),
        "CreateDomainStatementSegment" => Some("create_domain_statement"),
        "CreateExtensionStatementSegment" => Some("create_extension_statement"),
        "CreateExternalFunctionStatementSegment" => Some("create_external_function_statement"),
        "CreateExternalSchemaStatementSegment" => Some("create_external_schema_statement"),
        "CreateExternalTableAsStatementSegment" => Some("create_external_table_statement"),
        "CreateExternalTableStatementSegment" => Some("create_external_table_statement"),
        "CreateForeignDataWrapperStatementSegment" => Some("create_foreign_data_wrapper"),
        "CreateForeignTableStatementSegment" => Some("create_foreign_table_statement"),
        "CreateFunctionStatementSegment" => Some("create_function_statement"),
        "CreateGroupStatementSegment" => Some("create_group"),
        "CreateIndexStatementSegment" => Some("create_index_statement"),
        "CreateLibraryStatementSegment" => Some("create_library_statement"),
        "CreateMaterializedViewStatementSegment" => Some("create_materialized_view_statement"),
        "CreateModelStatementSegment" => Some("create_model_statement"),
        "CreateOperatorStatementSegment" => Some("create_operator_statement"),
        "CreatePolicyStatementSegment" => Some("create_policy_statement"),
        "CreateProcedureStatementSegment" => Some("create_procedure_statement"),
        "CreatePublicationStatementSegment" => Some("create_publication_statement"),
        "CreateRlsPolicyStatementSegment" => Some("create_rls_policy_statement"),
        "CreateRoleStatementSegment" => Some("create_role_statement"),
        "CreateSchemaStatementSegment" => Some("create_schema_statement"),
        "CreateSequenceOptionsSegment" => Some("create_sequence_options_segment"),
        "CreateSequenceStatementSegment" => Some("create_sequence_statement"),
        "CreateServerStatementSegment" => Some("create_server_statement"),
        "CreateStatisticsStatementSegment" => Some("create_statistics_statement"),
        "CreateSubscriptionStatementSegment" => Some("create_subscription"),
        "CreateTableAsStatementSegment" => Some("create_table_as_statement"),
        "CreateTableStatementSegment" => Some("create_table_statement"),
        "CreateTextSearchConfigurationStatementSegment" => Some("create_text_search_configuration_statement"),
        "CreateTriggerStatementSegment" => Some("create_trigger"),
        "CreateTypeStatementSegment" => Some("create_type_statement"),
        "CreateUserMappingStatementSegment" => Some("create_user_mapping_statement"),
        "CreateUserStatementSegment" => Some("create_user_statement"),
        "CreateViewStatementSegment" => Some("create_view_statement"),
        "CubeFunctionNameSegment" => Some("function_name"),
        "CubeRollupClauseSegment" => Some("cube_rollup_clause"),
        "DataFormatSegment" => Some("data_format_segment"),
        "DatabaseReferenceSegment" => Some("database_reference"),
        "DatatypeSegment" => Some("data_type"),
        "DatePartFunctionNameSegment" => Some("function_name"),
        "DateTimeFunctionContentsSegment" => Some("function_contents"),
        "DateTimeLiteralGrammar" => Some("datetime_literal"),
        "DateTimeTypeIdentifier" => Some("datetime_type_identifier"),
        "DeallocateStatementSegment" => Some("deallocate_statement"),
        "DeclareStatementSegment" => Some("declare_statement"),
        "Dedent" => Some("dedent"),
        "DefinitionParameterSegment" => Some("definition_parameter"),
        "DefinitionParametersSegment" => Some("definition_parameters"),
        "DeleteStatementSegment" => Some("delete_statement"),
        "DescDatashareStatementSegment" => Some("desc_datashare_statement"),
        "DescribeStatementSegment" => Some("describe_statement"),
        "DiscardStatementSegment" => Some("discard_statement"),
        "DoStatementSegment" => Some("do_statement"),
        "DropAggregateStatementSegment" => Some("drop_aggregate_statement"),
        "DropCastStatementSegment" => Some("drop_cast_statement"),
        "DropCollationStatementSegment" => Some("drop_collation_statement"),
        "DropDatabaseStatementSegment" => Some("drop_database_statement"),
        "DropDatashareStatementSegment" => Some("drop_datashare_statement"),
        "DropDomainStatementSegment" => Some("drop_domain_statement"),
        "DropExtensionStatementSegment" => Some("drop_extension_statement"),
        "DropForeignTableStatement" => Some("drop_foreign_table_statement"),
        "DropFunctionStatementSegment" => Some("drop_function_statement"),
        "DropIndexStatementSegment" => Some("drop_index_statement"),
        "DropMaterializedViewStatementSegment" => Some("drop_materialized_view_statement"),
        "DropModelStatementSegment" => Some("drop_MODELstatement"),
        "DropOwnedStatementSegment" => Some("drop_owned_statement"),
        "DropPolicyStatementSegment" => Some("drop_policy_statement"),
        "DropProcedureStatementSegment" => Some("drop_procedure_statement"),
        "DropPublicationStatementSegment" => Some("drop_publication_statement"),
        "DropRlsPolicyStatementSegment" => Some("drop_rls_policy_statement"),
        "DropRoleStatementSegment" => Some("drop_role_statement"),
        "DropSchemaStatementSegment" => Some("drop_schema_statement"),
        "DropSequenceStatementSegment" => Some("drop_sequence_statement"),
        "DropStatisticsStatementSegment" => Some("drop_statistics_statement"),
        "DropSubscriptionStatementSegment" => Some("drop_subscription"),
        "DropTableStatementSegment" => Some("drop_table_statement"),
        "DropTextSearchConfigurationStatementSegment" => Some("drop_text_search_configuration_statement"),
        "DropTriggerStatementSegment" => Some("drop_trigger"),
        "DropTypeStatementSegment" => Some("drop_type_statement"),
        "DropUserStatementSegment" => Some("drop_user_statement"),
        "DropViewStatementSegment" => Some("drop_view_statement"),
        "ElseClauseSegment" => Some("else_clause"),
        "EmptyStructLiteralBracketsSegment" => Some("struct_literal"),
        "EmptyStructLiteralSegment" => Some("typed_struct_literal"),
        "EqualsSegment" => Some("comparison_operator"),
        "ExcludeClauseSegment" => Some("select_exclude_clause"),
        "ExclusionConstraintElementSegment" => Some("exclusion_constraint_element"),
        "ExecuteStatementSegment" => Some("execute_statement"),
        "ExplainOptionSegment" => Some("explain_option"),
        "ExplainStatementSegment" => Some("explain_statement"),
        "ExpressionSegment" => Some("expression"),
        "ExtensionReferenceSegment" => Some("extension_reference"),
        "FetchClauseSegment" => Some("fetch_clause"),
        "FetchStatementSegment" => Some("fetch_statement"),
        "FileSegment" => Some("file"),
        "ForClauseSegment" => Some("for_clause"),
        "ForeignTableColumnConstraintSegment" => Some("column_constraint_segment"),
        "ForeignTableTableConstraintSegment" => Some("table_constraint"),
        "FrameClauseSegment" => Some("frame_clause"),
        "FromClauseSegment" => Some("from_clause"),
        "FromDatashareClauseSegment" => Some("from_datashare_clause"),
        "FromExpressionElementSegment" => Some("from_expression_element"),
        "FromExpressionSegment" => Some("from_expression"),
        "FromIntegrationClauseSegment" => Some("from_integration_clause"),
        "FromPivotExpressionSegment" => Some("from_pivot_expression"),
        "FromUnpivotExpressionSegment" => Some("from_unpivot_expression"),
        "FunctionContentsSegment" => Some("function_contents"),
        "FunctionDefinitionGrammar" => Some("function_definition"),
        "FunctionNameSegment" => Some("function_name"),
        "FunctionParameterListGrammar" => Some("function_parameter_list"),
        "FunctionSegment" => Some("function"),
        "GrantStatementSegment" => Some("grant_statement"),
        "GrantUsageDatashareStatementSegment" => Some("grant_datashare_statement"),
        "GreaterThanOrEqualToSegment" => Some("comparison_operator"),
        "GreaterThanSegment" => Some("comparison_operator"),
        "GroupByClauseSegment" => Some("groupby_clause"),
        "GroupingExpressionList" => Some("grouping_expression_list"),
        "GroupingSetsClauseSegment" => Some("grouping_sets_clause"),
        "HavingClauseSegment" => Some("having_clause"),
        "IamRoleClauseSegment" => Some("iam_role_clause"),
        "IdentifierSegment" => Some("identifier"),
        "ImplicitIndent" => Some("indent"),
        "ImportForeignSchemaStatementSegment" => Some("import_foreign_schema_statement"),
        "Indent" => Some("indent"),
        "IndexAccessMethodSegment" => Some("index_access_method"),
        "IndexColumnDefinitionSegment" => Some("index_column_definition"),
        "IndexElementOptionsSegment" => Some("index_element_options"),
        "IndexElementSegment" => Some("index_element"),
        "IndexParametersSegment" => Some("index_parameters"),
        "IndexReferenceSegment" => Some("index_reference"),
        "InsertStatementSegment" => Some("insert_statement"),
        "IntervalExpressionSegment" => Some("interval_expression"),
        "IntoClauseSegment" => Some("into_clause"),
        "IsolationLevelClauseSegment" => Some("isolation_level_clause"),
        "JoinClauseSegment" => Some("join_clause"),
        "JoinOnConditionSegment" => Some("join_on_condition"),
        "KeywordSegment" => Some("keyword"),
        "LanguageClauseSegment" => Some("language_clause"),
        "LessThanOrEqualToSegment" => Some("comparison_operator"),
        "LessThanSegment" => Some("comparison_operator"),
        "LikeOptionSegment" => Some("like_option_segment"),
        "LimitClauseSegment" => Some("limit_clause"),
        "ListenStatementSegment" => Some("listen_statement"),
        "LiteralKeywordSegment" => Some("literal"),
        "LiteralSegment" => Some("literal"),
        "LoadStatementSegment" => Some("load_statement"),
        "LocalAliasSegment" => Some("local_alias_segment"),
        "LockTableStatementSegment" => Some("lock_table_statement"),
        "MLTableExpressionSegment" => Some("ml_table_expression"),
        "ManageRlsPolicyStatementSegment" => Some("manage_rls_policy_statement"),
        "MapTypeSegment" => Some("map_type"),
        "MatchConditionSegment" => Some("match_condition"),
        "MergeDeleteClauseSegment" => Some("merge_delete_clause"),
        "MergeInsertClauseSegment" => Some("merge_insert_clause"),
        "MergeMatchSegment" => Some("merge_match"),
        "MergeMatchedClauseSegment" => Some("merge_when_matched_clause"),
        "MergeNotMatchedBySourceClauseSegment" => Some("merge_when_not_matched_by_source_clause"),
        "MergeNotMatchedClauseSegment" => Some("merge_when_not_matched_clause"),
        "MergeStatementSegment" => Some("merge_statement"),
        "MergeUpdateClauseSegment" => Some("merge_update_clause"),
        "MetaCommandQueryBufferStatement" => Some("meta_command_statement"),
        "NamedArgumentSegment" => Some("named_argument"),
        "NamedWindowExpressionSegment" => Some("named_window_expression"),
        "NamedWindowSegment" => Some("named_window"),
        "NewlineSegment" => Some("newline"),
        "NotEqualToSegment" => Some("comparison_operator"),
        "NotExtendLeftSegment" => Some("comparison_operator"),
        "NotExtendRightSegment" => Some("comparison_operator"),
        "NotifyStatementSegment" => Some("notify_statement"),
        "ObjectLiteralElementSegment" => Some("object_literal_element"),
        "ObjectLiteralSegment" => Some("object_literal"),
        "ObjectReferenceSegment" => Some("object_reference"),
        "ObjectUnpivotSegment" => Some("object_unpivoting"),
        "OffsetClauseSegment" => Some("offset_clause"),
        "OperationClassReferenceSegment" => Some("operation_class_reference"),
        "OperatorClassReferenceSegment" => Some("operator_class_reference"),
        "OrderByClauseSegment" => Some("orderby_clause"),
        "OverClauseSegment" => Some("over_clause"),
        "OverlapSegment" => Some("comparison_operator"),
        "OverlapsClauseSegment" => Some("overlaps_clause"),
        "PartitionBoundSpecSegment" => Some("partition_bound_spec"),
        "PartitionClauseSegment" => Some("partitionby_clause"),
        "PartitionedBySegment" => Some("partitioned_by_segment"),
        "PathSegment" => Some("path_segment"),
        "PostgresCopyStdinDataStatementSegment" => Some("postgres_copy_stdin_data_statement"),
        "PrepareStatementSegment" => Some("prepare_statement"),
        "ProcedureParameterListSegment" => Some("procedure_parameter_list"),
        "PsqlCopyMetaCommandStatementSegment" => Some("psql_copy_meta_command_statement"),
        "PsqlSetMetaCommandStatementSegment" => Some("psql_set_meta_command_statement"),
        "PsqlVariableGrammar" => Some("psql_variable"),
        "PublicationObjectsSegment" => Some("publication_objects"),
        "PublicationReferenceSegment" => Some("publication_reference"),
        "PublicationTableSegment" => Some("publication_table"),
        "QualifiedNumericLiteralSegment" => Some("numeric_literal"),
        "QualifiedOperatorSegment" => Some("qualified_operator"),
        "QualifyClauseSegment" => Some("qualify_clause"),
        "ReassignOwnedStatementSegment" => Some("reassign_owned_statement"),
        "RedshiftGrantTargetSegment" => Some("access_target"),
        "RedshiftGroupGrantTargetSegment" => Some("access_target"),
        "RefreshMaterializedViewStatementSegment" => Some("refresh_materialized_view_statement"),
        "ReindexStatementSegment" => Some("reindex_statement_segment"),
        "RelationOptionSegment" => Some("relation_option"),
        "RelationOptionsSegment" => Some("relation_options"),
        "ResetSessionAuthorizationStatementSegment" => Some("reset_session_authorization_statement"),
        "ResetStatementSegment" => Some("reset_statement"),
        "RevokeStatementSegment" => Some("revoke_statement"),
        "RoleReferenceSegment" => Some("role_reference"),
        "RollupFunctionNameSegment" => Some("function_name"),
        "RowFormatDelimitedSegment" => Some("row_format_delimited_segment"),
        "SamplingExpressionSegment" => Some("sample_expression"),
        "SchemaReferenceSegment" => Some("schema_reference"),
        "SecurityLabelStatementSegment" => Some("security_label_statement"),
        "SelectClauseElementSegment" => Some("select_clause_element"),
        "SelectClauseModifierSegment" => Some("select_clause_modifier"),
        "SelectClauseSegment" => Some("select_clause"),
        "SelectStatementSegment" => Some("select_statement"),
        "SemiStructuredAccessorSegment" => Some("semi_structured_expression"),
        "SequenceReferenceSegment" => Some("sequence_reference"),
        "ServerReferenceSegment" => Some("server_reference"),
        "SetClauseListSegment" => Some("set_clause_list"),
        "SetClauseSegment" => Some("set_clause"),
        "SetConstraintsStatementSegment" => Some("set_constraint_statement"),
        "SetExpressionSegment" => Some("set_expression"),
        "SetOperatorSegment" => Some("set_operator"),
        "SetSchemaStatementSegment" => Some("set_schema_statement"),
        "SetSessionAuthorizationStatementSegment" => Some("set_session_authorization_statement"),
        "SetStatementSegment" => Some("set_statement"),
        "ShorthandCastSegment" => Some("cast_expression"),
        "ShowDatasharesStatementSegment" => Some("show_datashares_statement"),
        "ShowModelStatementSegment" => Some("show_model_statement"),
        "ShowStatementSegment" => Some("show_statement"),
        "SingleIdentifierListSegment" => Some("identifier_list"),
        "SingleIdentifierWildcardSegment" => Some("wildcard_identifier"),
        "SizedArrayTypeSegment" => Some("sized_array_type"),
        "StatementSegment" => Some("statement"),
        "StatisticsReferenceSegment" => Some("statistics_reference"),
        "StructLiteralSegment" => Some("struct_literal"),
        "StructTypeSegment" => Some("struct_type"),
        "SubscriptionReferenceSegment" => Some("subscription_reference"),
        "SymbolSegment" => Some("symbol"),
        "TableAttributeSegment" => Some("table_constraint"),
        "TableConstraintSegment" => Some("table_constraint"),
        "TableConstraintUsingIndexSegment" => Some("table_constraint"),
        "TableEndClauseSegment" => Some("table_end_clause_segment"),
        "TableExpressionSegment" => Some("table_expression"),
        "TableReferenceSegment" => Some("table_reference"),
        "TablespaceReferenceSegment" => Some("tablespace_reference"),
        "TagReferenceSegment" => Some("tag_reference"),
        "TemporalQuerySegment" => Some("temporal_query"),
        "TimeZoneGrammar" => Some("time_zone_grammar"),
        "TransactionStatementSegment" => Some("transaction_statement"),
        "TriggerReferenceSegment" => Some("trigger_reference"),
        "TruncateStatementSegment" => Some("truncate_table"),
        "TupleSegment" => Some("tuple"),
        "TypedArrayLiteralSegment" => Some("typed_array_literal"),
        "TypedStructLiteralSegment" => Some("typed_struct_literal"),
        "UnlistenStatementSegment" => Some("unlisten_statement"),
        "UnloadStatementSegment" => Some("unload_statement"),
        "UnorderedSelectStatementSegment" => Some("select_statement"),
        "UnorderedSetExpressionSegment" => Some("set_expression"),
        "UpdateStatementSegment" => Some("update_statement"),
        "UseStatementSegment" => Some("use_statement"),
        "UserReferenceSegment" => Some("user_reference"),
        "VacuumStatementSegment" => Some("vacuum_statement"),
        "ValuesClauseSegment" => Some("values_clause"),
        "VersionIdentifierSegment" => Some("version_identifier"),
        "WhenClauseSegment" => Some("when_clause"),
        "WhereClauseSegment" => Some("where_clause"),
        "WhitespaceSegment" => Some("whitespace"),
        "WildcardExpressionSegment" => Some("wildcard_expression"),
        "WildcardIdentifierSegment" => Some("wildcard_identifier"),
        "WindowSpecificationSegment" => Some("window_specification"),
        "WithCheckOptionSegment" => Some("with_check_option"),
        "WithCompoundNonSelectStatementSegment" => Some("with_compound_statement"),
        "WithCompoundStatementSegment" => Some("with_compound_statement"),
        "WithDataClauseSegment" => Some("with_data_clause"),
        "WithFillSegment" => Some("with_fill"),
        "WithNoSchemaBindingClauseSegment" => Some("with_no_schema_binding_clause"),
        "WithinGroupClauseSegment" => Some("withingroup_clause"),
        "WordSegment" => Some("word"),
        _ => None,
    }
}

pub static REDSHIFT_TABLES: GrammarTables = GrammarTables {
    instructions: INSTRUCTIONS,
    child_ids: CHILD_IDS,
    terminators: TERMINATORS,
    strings: STRINGS,
    aux_data: AUX_DATA,
    aux_data_offsets: AUX_DATA_OFFSETS,
    regex_patterns: REGEX_PATTERNS,
    simple_hints: SIMPLE_HINTS,
    hint_string_indices: HINT_STRING_INDICES,
    simple_hint_indices: SIMPLE_HINT_INDICES,
    segment_type_offsets: SEGMENT_TYPE_OFFSETS,
    segment_class_offsets: SEGMENT_CLASS_OFFSETS,
    casefold_sparse: CASEFOLD_SPARSE,
    trim_chars_sparse: TRIM_CHARS_SPARSE,
    trim_chars_data: TRIM_CHARS_DATA,
    segment_class_types_sparse: SEGMENT_CLASS_TYPES_SPARSE,
    segment_class_types_data: SEGMENT_CLASS_TYPES_DATA,
};

pub fn get_redshift_root_grammar_id() -> GrammarId {
    GrammarId(4816)
}
pub fn get_redshift_root_grammar_table() -> RootGrammar {
    RootGrammar {
        grammar_id: get_redshift_root_grammar_id(),
        tables: &REDSHIFT_TABLES,
    }
}
pub fn get_redshift_root_grammar() -> RootGrammar {
    get_redshift_root_grammar_table()
}
